<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scUserPageAddresses extends miDefaultView implements ICMSBlock {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'customer/addresses/addresses.tmpl',
			self::TEMPLATE_ID_EDIT => 'customer/addresses/address_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'customer/addresses/address_create.tmpl',
		);
		
		protected $_dataFields = array(
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressID'
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressFirstName',
				'validator' => 'scValidatorNoEmpty',
			),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressLastName',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressCompany',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressLine1',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressLine2',
        	),
       		array(
				'field' => 'scCityNameWidget',
				'data' => 'CustomerAddressCity',
       			'validator' => 'scValidatorNoEmpty',
       			'properties' => array(
        			'template' => 'customer/widgets/city_name_widget.tmpl',
       				'CityIDFieldId' => 'CityID', 
       				'StateIDFieldId' => 'CustomerBillingState',
       				'CountryIDFieldId' => 'CustomerBillingCountry',
       				'tabindex' => 9
       				),
        		),
        	array(
        		'field' => 'miBaseTextWidget',
        		'data' => 'CityID',
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressZip',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressPhone',
				'validator' => 'scValidatorNoEmpty',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerAddressFax',
        	),
       		array(
				'field' => 'scVatNumberWidget',
				'data' => 'CustomerAddressVatNumber',
        	),
       		array(
				'field' => 'miBaseSelectWidget',
				'data' => 'CountryID',
				'properties' => array(
					'id' => 'CustomerBillingCountry'
       			),
       		),
       		array(
				'field' => 'scStateSelectWidget',
				'data' => 'StateID',
				'properties' => array(
					'id' => 'CustomerBillingState',
					'originalValueField' => 'OriginalCustomerBillingState',
       				'tabindex' => 8
       			),
        	),
		);
		
		public function getControllerCommands()
		{
			if (!scCustomerAuth::singleton()->isLoggedIn()) {
				$loginUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN);
				$addressesUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_ADDRESSES);
				$loginUrl = scUrlUtil::addQueryParams($loginUrl, 'Referer=' . urlencode($addressesUrl));
				return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loginUrl));
			}
			
			return parent::getControllerCommands();
		}
		
		/**
		 * @param miTable $table
		 */
		public function initTable($table)
		{
			$urlCreate = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_ADDRESSES);
			$urlCreate = scUrlUtil::addQueryParams($urlCreate, 'action=dmCreate');
			$table->addMainPageElements(array(
				'%%URL_ADDRESS_CREATE%%' => $urlCreate,
			));
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$field = $form->getWidget('CountryID');
			$field->setOptions(scCountry::getCountryNames(), scSettings::getSettingValue(scSettings::SETTING_DEFAULT_COUNTRY));
			
			$field = $form->getWidget('CustomerAddressVatNumber');
			$field->setCountryField('CountryID');
			
			$stateNames = array('' => '') + scCountry::getStateNames();
			$field = $form->getWidget('StateID');
			$field->setOptions($stateNames);
			
			$form->setErrorsHandler(new scByFieldErrorsHandler);
			
			// Assign Addresses Count and Orders Count
			$form->addMainPageElements(array(
				'%%CUSTOMER_ADDRESSES_COUNT%%' => scCustomerAddress::getAddressesCountByCustomerId($_SESSION['CustomerID']),
				'%%CUSTOMER_ORDERS_COUNT%%' => scCustomerOrdersUI::getOrdersCountByCustomerId($_SESSION['CustomerID'])
			));
			
			$message = new miMessage();
			$form->addMainPageElements($message->getMessageTemplateVars());
			$this->addMainPageElements(array('%%STATES_JS_DATA%%' => scUIUtil::getStatesJsData()));
			$form->addValidator('CustomerAddressCity', new scAddressValidator($form, 'CityID', 'CountryID'));
		}
		
		public function preProcess($actionObj)
		{
			parent::preProcess($actionObj);
			
			// No need to do these checks if the user is not logged in
			if (!scCustomerAuth::singleton()->isLoggedIn())
				return;
			
			// Check if the user is trying to access one of their addresses.
			$customerAddressId = miGetParamDefault('CustomerAddressID', 0);
			if ($customerAddressId > 0) {
				$sql = 'SELECT COUNT(CustomerAddressID) AS N FROM CustomerAddresses WHERE CustomerAddressID = ? AND CustomerID = ?';
				$count = miStaticDBUtil::execSelect($sql, array($customerAddressId, scCustomerAuth::singleton()->getCustomerId()));
				if (!isset($count[0]) || ($count[0]['N'] != 1)) {
					$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode(_t('Access denied.'));
					header('Location: ' . $url);
					exit;
				}
			}
		}
		
		public function addRedirectToListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_ADDRESSES);
			$loc = scUrlUtil::addQueryParams($loc, $this->getRedirectParams($msg, $msgType));
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
		
		protected function createDomainObject()
		{
			return new scCustomerAddress();
		}
	}
	
	class scUserCustomerAddressesTable extends scTable {
		
		protected function addUiElements(&$rows)
		{
			parent::addUiElements($rows);
			
			$baseUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_ADDRESSES);
			foreach ($rows as &$row) {
				$row['HTML_FULLADDRESS'] = scCustomerAddress::getFormattedAddress($row);
				$row['BILLINGADDRESS_CHECKED'] = ($row['CustomerAddressIsDefaultBilling'] ? ' checked="checked" ' : '');
				$row['SHIPPINGADDRESS_CHECKED'] = ($row['CustomerAddressIsDefaultShipping'] ? ' checked="checked" ' : '');
				
				$row['URL_ADDRESS_EDIT'] = scUrlUtil::addQueryParams($baseUrl, 'action=dmEdit&CustomerAddressID=' . $row['CustomerAddressID']);
				$row['URL_ADDRESS_DELETE'] = scUrlUtil::addQueryParams($baseUrl, 'action=dmExecDelete&CustomerAddressID=' . $row['CustomerAddressID']);
			}
		}
		
		protected function addMainUiElements(&$t)
		{
			parent::addMainUiElements($t);
			
			// Assign Addresses Count and Orders Count
			$t->assignArray(array(
				'%%CUSTOMER_ADDRESSES_COUNT%%' => scCustomerAddress::getAddressesCountByCustomerId($_SESSION['CustomerID']),
				'%%CUSTOMER_ORDERS_COUNT%%' => scCustomerOrdersUI::getOrdersCountByCustomerId($_SESSION['CustomerID'])
			));
		}
	}
	
	class scUserExecCreateAddressAction extends miExecCreateAction {
		
		protected function onSuccess($viewMapperControllerResult = null)
		{
			$msg = _t('Address successfully created.');
			$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	class scUserExecEditAddressAction extends miExecEditAction {
		
		protected function onSuccess($viewMapperControllerResult = null)
		{
			$msg = _t('Address successfully updated.');
			$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	class scUserExecDeleteAddressAction extends miExecDeleteAction {
		
		protected function onSuccess($viewMapperControllerResult = null)
		{
			$msg = _t('Address successfully deleted.');
			$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	class scUserSetDefaultAddressesAction extends miAction {
		
		public function doAction()
		{
			try {
				miStaticDBUtil::execSQL('START TRANSACTION');
				$customerId = scCustomerAuth::singleton()->getCustomerId();
				
				$addressIds = array(
					(int)miGetParam('CustomerAddressIsDefaultBilling'),
					(int)miGetParam('CustomerAddressIsDefaultShipping'),
				);
				$addressIds = array_unique($addressIds);
				
				// First check if both address id's are valid
				$this->checkValidity($customerId, $addressIds);
				
				// Clear the default address flag from all addresses
				$this->resetAddresses($customerId);
				
				// Set default billing and shipping address
				if (count($addressIds) == 1) {
					// The same address is being set as billing and shipping
					$sql = 'UPDATE CustomerAddresses SET' .
						' CustomerAddressIsDefaultBilling = 1, CustomerAddressIsDefaultShipping = 1' .
						' WHERE CustomerAddressID=?';
					miStaticDBUtil::execSQL($sql, array($addressIds[0]));
				} else {
					$sql = 'UPDATE CustomerAddresses SET CustomerAddressIsDefaultBilling = 1 WHERE CustomerAddressID=?';
					miStaticDBUtil::execSQL($sql, array($addressIds[0]));
					
					$sql = 'UPDATE CustomerAddresses SET CustomerAddressIsDefaultShipping = 1 WHERE CustomerAddressID=?';
					miStaticDBUtil::execSQL($sql, array($addressIds[1]));
				}
				miStaticDBUtil::execSQL('COMMIT');
				$msg = _t('Default addresses successfully changed.');
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
				return true;
				
			} catch (miException $e) {
				miStaticDBUtil::execSQL('ROLLBACK');
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($e->getMessage());
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
				return false;
			}
		}
		
		/**
		 * Checks if the addresses belong to this customer
		 * 
		 * @param int $customerId
		 * @param array $addressIds
		 * @throw miException if the address do exists, or not belong to this customer
		 */
		protected function checkValidity($customerId, $addressIds)
		{
			$customerId = (int)$customerId;
			$addressIds = array_map('intval', $addressIds);
			$sql = 'SELECT CustomerID, COUNT(DISTINCT(CustomerAddressID)) AS N' .
				' FROM CustomerAddresses' .
				' WHERE CustomerID=' . $customerId .
				' AND CustomerAddressID IN (' . implode(', ', $addressIds) . ')' .
				' GROUP BY CustomerID';
			$addressesCount = miUtil::getDBArray($sql, 'CustomerID', 'N');
			if ($addressesCount[$customerId] != count($addressIds)) {
				throw new miException(_i('Invalid addresses passed.'));
			}
		}
		
		protected function resetAddresses($customerId)
		{
			$sql = 'UPDATE CustomerAddresses SET' .
				' CustomerAddressIsDefaultBilling = 0, CustomerAddressIsDefaultShipping = 0' .
				' WHERE CustomerID=?';
			miStaticDBUtil::execSQL($sql, array($customerId));
		}
	}
	
	miSetParam('CustomerID', scCustomerAuth::singleton()->getCustomerId());
	
	$recordset = new miSqlRecordset('CustomerAddresses');
	$recordset->addFilter(new miSqlFilterEqual('CustomerID', scCustomerAuth::singleton()->getCustomerId()));
	
	$table = new scUserCustomerAddressesTable($recordset);
	
	$view = new scUserPageAddresses();
	$view->setTable($table);
	
	$page = new scUserDisplayController(array($view));
	$page->setSeo(new scDynamicPageSeo('addresses.php', _t('My Addresses')));
	$page->getDispatcher()->setActionHandler('dmExecCreate', 'scUserExecCreateAddressAction');
	$page->getDispatcher()->setActionHandler('dmExecEdit', 'scUserExecEditAddressAction');
	$page->getDispatcher()->setActionHandler('dmExecDelete', 'scUserExecDeleteAddressAction');
	$page->getDispatcher()->setActionHandler('dmSetDefaultAddresses', 'scUserSetDefaultAddressesAction');
	$page->dispatchAndShow();
?>