<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_ADMINISTRATORS);

	class scAdministratorsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/administrators/administrator_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/administrators/administrator_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/administrators/administrator_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'AdministratorID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'AdministratorUsername',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Username'
        	),
       		array(
				'field' => 'scHashedPasswordWidget',
				'data' => 'AdministratorPassword',
				'displayName' => 'Password'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'AdministratorPasswordHashMode',
        	),
        	array(
				'field' => 'miTextWidget',
				'data' => 'AdministratorEmail',
				'displayName' => 'Email'
        	),
			array(
				'field' => 'miSelectWidget',
				'data' => 'AdministratorAccessLevel',
				'properties' => array('id' => 'accessLevel')
        	),
			array(
				'field' => 'scAdministratorPriviligesWizard',
				'modelStrategy' => 'scViewMapperAdministratorPrivileges',
				'data' => 'AdministratorPrivileges'
        	)
		);
        
		protected $_domainObjectDisplayName = 'Administrator';	// _t('Administrator');
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$accessLevels = scAdminAuth::getAdministratorAccessLevels();
			$form->getWidget('AdministratorAccessLevel')->setOptions($accessLevels, scAdminAuth::ADMINISTRATOR_ACCESS_LEVEL_FULL_ACCESS);
			
			$record = new scDomainToRecordAdapter($this->getDomainObject());
			$form->addValidator('AdministratorUsername', new scValidatorUniqueFields($form, array('AdministratorUsername'), $record));
			
			$form->addMainPageElements(array('%%ACCESSLEVELLIMITED%%' => scAdminAuth::ADMINISTRATOR_ACCESS_LEVEL_LIMITED));
			
			if ($this->getController()->getDispatcher()->getActionObj() instanceof miCreateAction) {
				$field = $form->getWidget('AdministratorPasswordHashMode');
				$field->setDefaultValue(scSettings::getSettingValue(scSettings::SETTING_PASSWORD_HASH_MODE));
			}
		}
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
			
			// Create dropdown filter for billing country
			$accessLevels = array('' => _t('All access levels')) + scAdminAuth::getAdministratorAccessLevels();
			scUIUtil::createDropdownFilter($this, $table, 'AdministratorAccessLevel', $accessLevels, '=', '');
		}
		
		/**
		 * @return scAdministratorDomainObject
		 */
		protected function createDomainObject()
		{
			return new scAdministratorDomainObject();
		}
	}
	
	class scAdministratorPriviligesWizard extends miBaseCheckboxWidget {
		function getData()
		{
			return array($this->_fieldName => array_keys(miGetParamDefault($this->_fieldName, array())));
		}
		
		function getEditableControl()
		{
			$sections = scSections::getSections();
			
			$values = $this->_webForm->getFormData($this->_fieldName);
			
			// By default all sections are checked
			if ($values == null)
				$values = array_keys($sections);
			
			$fields = array();
			$sectionGroups = scSections::getSectionGroups();
			foreach ($sectionGroups as $sectionGroupId => $sectionGroupName) {
				$sectionsInGroup = scSections::getSectionsInGroup($sectionGroupId);
				
				$html = '<div>'.$sectionGroupName.'</div>';
				foreach ($sectionsInGroup as $sectionId) {
					$sectionName = $sections[$sectionId];
					$checked = in_array($sectionId, $values) ? ' checked="checked"' : '';
					$html .= '<label><input type="checkbox" name="' . $this->_fieldName .
						'[' . miI18N::htmlEscape($sectionId) . ']"' . $checked . '/>' . miI18N::htmlEscape($sectionName) . '</label><br/>';
				}
				$fields[$this->_fieldName . $sectionGroupId] = $html;
			}
			return $fields;
		}
	}
	
	class scViewMapperAdministratorPrivileges extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'AdministratorPrivs';
			$this->_tableKeyName = 'AdministratorID';
			$this->_tableValueName = 'AdministratorPrivSection';
			parent::__construct($model);
		}
	}
	
	class scAdministratorsTable extends scTable {
		protected function addUiElements(&$rows)
		{
			$accessLevels = scAdminAuth::getAdministratorAccessLevels();
			foreach ($rows as &$row) {
				$row['AdministratorAccessLevel'] = $accessLevels[$row['AdministratorAccessLevel']];
			}
		}
	}
	
	if (miGetParamDefault(miTableSorter::PARAM_SORTBY, '') == '')
		miSetParam(miTableSorter::PARAM_SORTBY, 'AdministratorUsername');
	
	$administrators = new miSqlRecordset('Administrators');
	$table = new scAdministratorsTable($administrators);
	
	$view = new scAdministratorsView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>