<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_API_ACCOUNTS);
	
	class scApiAccountsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/apiaccounts/apiaccount_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/apiaccounts/apiaccount_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/apiaccounts/apiaccount_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ApiAccountID'
        		),
       		array(
				'field' => 'miTextWidget',
				'data' => 'ApiAccountUsername',
       			'properties' => array('minLength' => 1),
				'displayName' => 'Username'
        		),
       		array(
				'field' => 'miTextWidget',
				'data' => 'ApiAccountPassword',
				'displayName' => 'Password'
        		),
        	array(
				'field' => 'scApiAccountPrivilegesWidget',
				'modelStrategy' => 'scViewMapperApiAccountPrivileges',
				'data' => 'ApiAccountPrivs'
        	)
        );
        
        protected $_domainObjectDisplayName = 'API Account';	// _t('API Account');
        
        public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$record = new scDomainToRecordAdapter($this->getDomainObject());
			$form->addValidator('ApiAccountUsername', new scValidatorUniqueFields($form, array('ApiAccountUsername'), $record));
		}
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
	}
	
	class scApiAccountPrivilegesWidget extends miBaseCheckboxWidget {
		function getData()
		{
			$entityOperations = miGetParamDefault($this->_fieldName, array());
			
			$privOperations = array();
			foreach ($entityOperations as $entityCode => $privs) {
				$privOperations[$entityCode] = 0;
				foreach ($privs as $priv)
					$privOperations[$entityCode] |= $priv;
			}

			return array($this->_fieldName => $privOperations);
		}
		
		function getEditableControl()
		{
			$entityCodes = $this->_webForm->getFormData($this->_fieldName);
			
			$entityNames = scApiPrivs::getTranslatedEntityNames();
			$entityMethods = scApiPrivs::getEntityMethods();
			
			$entities = array();
			foreach ($entityNames as $entityCode => $entityName)
				$entities[] = array('EntityCode' => $entityCode, 'EntityName' => $entityName);
			
			$t = new scTemplateParser();
			$sectionEntities = new miTemplateParserSectionInfo('api_priv_entity', count($entities), miUIUtil::templetizeRows(miUIUtil::transposeRows($entities)));
			$sectionsEntityMethods = array();
			
			foreach ($entityMethods as $entityCode => $methodCodes) {
				$entityOperations = (isset($entityCodes[$entityCode]) ? $entityCodes[$entityCode] : 0);
				$sectionsEntityMethodProperties = array();
				foreach ($methodCodes as $methodCode) {
					$viewMethod = array();
					$viewMethod['MethodCode'] = $methodCode;
					$viewMethod['MethodName'] = scApiPrivs::getMethodNameByCode($methodCode);
					$viewMethod['checked'] = ($entityOperations && scApiPrivs::hasAccess($entityOperations, $methodCode) ? 'checked="checked"' : '');
					
					$sectionsEntityMethodProperties[] = $viewMethod;
				}
				$sectionsEntityMethods[] = new miTemplateParserSectionInfo('api_priv_methods', count($methodCodes), miUIUtil::templetizeRows(miUIUtil::transposeRows($sectionsEntityMethodProperties)));
			}
			$sectionEntities->addSubsectionsArray('api_priv_methods', $sectionsEntityMethods);
			
			$t->addSectionInfo($sectionEntities);
			$t->readTemplate('admin/apiaccounts/apiaccount_privs.tmpl');
			
			return array($this->_fieldName => $t->templateParse());
		}
	}
	
	class scViewMapperApiAccountPrivileges extends scViewMapperMultiValues {
		
		protected $_tableValueKeyName;
		
		public function __construct($model)
		{
			$this->_tableName = 'ApiAccountsPrivs';
			$this->_tableKeyName = 'ApiAccountID';
			$this->_tableValueKeyName = 'ApiAccountPrivEntityCode';
			$this->_tableValueName = 'ApiAccountPrivOperations';
			parent::__construct($model);
		}
		
		public function read()
		{
			// set array with keys EntityCode and values Operations
			$sql = 'SELECT ' . $this->_tableValueName . ', ' . $this->_tableValueKeyName . ' FROM ' . $this->_tableName . ' WHERE ' .
				$this->_tableKeyName . '="' . sql_escape_string($this->getItemId()) . '"';
			$this->_subItemIds = miUtil::getDBArray($sql, $this->_tableValueKeyName, $this->_tableValueName);
		}
		
		public function update()
		{
			$itemId = $this->getItemId();
			
			miStaticDBUtil::startTransaction();
			
			// Remove old subitems
			miStaticDBUtil::execDelete($this->_tableName, $this->_tableKeyName, $itemId);

			if (count($this->_subItemIds) == 0)
				return;

			// Save subitems
			$records = array();
			foreach ($this->_subItemIds as $key => $subItemId) {
				$values = array(
					$this->_tableKeyName => $itemId,
					$this->_tableValueKeyName => $key,
					$this->_tableValueName => $subItemId
				);
				
				$records[] = "('" . implode("', '", $values) . "')";
			}

			$sql = 'INSERT INTO ' . $this->_tableName . '(' . $this->_tableKeyName . ',' . $this->_tableValueKeyName . ',' . $this->_tableValueName . ') VALUES ' . implode(', ', $records);
			miStaticDBUtil::execSQL($sql);
			
			miStaticDBUtil::commit();
		}
	}
	
	$apiAccounts = new miSqlRecordset('ApiAccounts');
	$table = new scTable($apiAccounts);
	
	$view = new scApiAccountsView();
	$view->setRecord(new scSqlRecord('ApiAccounts', 'ApiAccountID'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>