<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_BONUS_POINTS);
	scDateTime::setTimezone();
	
	class scPointsView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/bonus_points/bonus_points_mgmt.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/bonus_points/bonus_points_create.tmpl',
		);
		
		protected $_dataFields = array(
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerEmail',
				'validator' => 'scCustomerEmailValidator',
			),
			array(
				'field' => 'miSelectWidget',
				'data' => 'BonusPointsTransactionType',
			),
			array(
				'field' => 'miTextWidget',
				'data' => 'BonusPointsTransactionDescription',
			),
			array(
				'field' => 'miTextWidget',
				'data' => 'BonusPointsAmount',
				'validator' => array(
					'scValidatorInt',
				),
			),
		);
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
			
			$statusesArray = array('' => _t('All statuses')) + scBonusPoints::getAllActionStatuses();
			scUIUtil::createDropdownFilter($this, $table, 'BonusPointsTransactionStatus', $statusesArray, '=', '');
			
			$table->assign('%%CUSTOMEREMAILFILTER%%', miGetParamDefault('CustomerEmailFilter', ''));
		}
		
		/**
		 * @param miWebForm $form
		 */
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$customerEmail = miGetParamDefault('CustomerEmailFilter', '');
			$form->addMainPageElements(array(
				'%%CUSTOMEREMAIL%%' => $customerEmail,
			));
			
			$widget = $form->getWidget('BonusPointsTransactionType');
			$widget->setOptions(scBonusPoints::getAllActionTypes());
		}
		
		protected function createDomainObject()
		{
			return new scBonusPointsAdminTransaction();
		}
	}
	
	class scBonusPointsAdminTransaction extends scBonusPointsTransaction {
		
		public function insert()
		{
			// Get the CustomerID from the CustomerEmail
			$customerId = scUtil::getDbValue('Customers', 'CustomerID', 'CustomerEmail', $this->get('CustomerEmail'), 0);
			if (empty($customerId)) {
				throw new miException(_t('There is no customer with this email address.'));
			}
			$this->set('CustomerID', $customerId);
			$this->set('CustomerEmail', null);
			
			// Unset the email field
			$row = $this->getRow();
			unset($row['CustomerEmail']);
			$this->setRow($row);
			
			if (!$this->has('BonusPointsTransactionReason')) {
				$this->set('BonusPointsTransactionReason', scBonusPoints::ASSIGNMENT_REASON_ADMIN_ACTION);
			}
			
			$this->set('BonusPointsTransactionStatus', scBonusPoints::TRANSACTION_STATUS_APPROVED);
			return parent::insert();
		}
	}
	
	class scApproveBonusPointsAction extends miAction {
		
		public function doAction()
		{
			try {
				$bonusPointsAction = $this->_view->getDomainObject();
				$bonusPointsAction->read(miGetParam('BonusPointsTransactionID'));
				$bonusPointsAction->approve();
				
				$msg = $this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG');
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
				return true;
			} catch (miException $e) {
				$msg = $this->_view->getMessage('MI_RECORD_UPDATE_FAILED_MSG') . $e->getMessage();
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scVoidBonusPointsAction extends miAction {
		
		public function doAction()
		{
			try {
				$bonusPointsAction = $this->_view->getDomainObject();
				$bonusPointsAction->read(miGetParam('BonusPointsTransactionID'));
				$bonusPointsAction->void();
				
				$msg = $this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG');
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
				return true;
			} catch (miException $e) {
				$msg = $this->_view->getMessage('MI_RECORD_UPDATE_FAILED_MSG') . $e->getMessage();
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scCustomerEmailValidator extends miValidator
	{
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			$customerId = scUtil::getDbValue('Customers', 'CustomerID', 'CustomerEmail', $fieldValue, 0);
			if (empty($customerId)) {
				$errors->addError($this->_fieldName, _t('There is no customer with this email address.'));
			}
		}
	}
	
	if (miGetParamDefault(miTableSorter::PARAM_SORTBY, '') == '') {
		miSetParam(miTableSorter::PARAM_SORTBY, 'BonusPointsTransactionTime');
		miSetParam(miTableSorter::PARAM_SORTDIR, 'DESC');
	}
	
	$recordset = new miSqlRecordset('BonusPointsTransactions');
	$recordset->addJoinCondition('LEFT', 'Customers', 'USING(CustomerID)');
	$recordset->addJoinCondition('LEFT', 'CustomerAddresses', 'ON Customers.CustomerID = CustomerAddresses.CustomerID AND CustomerAddressIsDefaultBilling');
	$recordset->setSelectFields(array(
		'*', 
		'CONCAT_WS(" ", CustomerAddressFirstName, CustomerAddressLastName) AS CustomerName'
	));
	$table = new scBonusPointsTransactionsTable($recordset);
	
	$view = new scPointsView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('', 'scCreateAndListAction');
	$page->getDispatcher()->setActionHandler('dmExecCreate', 'scExecCreateAndListAction');
	$page->getDispatcher()->setActionHandler('dmApprove', 'scApproveBonusPointsAction');
	$page->getDispatcher()->setActionHandler('dmVoid', 'scVoidBonusPointsAction');
	$page->dispatchAndShow();
?>