<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_BOXES);

	class scBoxesView extends scView {
		
		protected static $_module = null;
		protected static $_moduleId = null;
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/boxes/boxes_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/boxes/boxes_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/boxes/boxes_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSBoxID'
        		),
        	array(
        		'field' => 'miSelectWidget',
        		'data' => 'CMSBoxType',
        		),
       		array(
				'field' => 'miTextWidget',
				'data' => 'CMSBoxName',
        		),
       		array(
				'field' => 'miCheckboxWidget',
				'data' => 'CMSBoxIsActive'
        		),
		);
		
		protected $_domainObjectDisplayName = 'Box options';	// _t('Box options');

		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$action = $this->getController()->getDispatcher()->getAction();
			
			$typeWidget = $form->getWidget('CMSBoxType');
			$typeWidget->setOptions(scCmsBoxTypes::getAllEditableTypes());
			
			if (('dmEdit' == $action) || ('dmExecEdit' == $action)) {
				$currentBoxType = $this->getDomainObject()->get('CMSBoxType');
				// The CMSBoxType should be selectable only in create actions.
				$typeWidget->setProperty('readOnly', true);
				$typeWidget->setOptions(array($currentBoxType => scCmsBoxTypes::getBoxType($currentBoxType)));
				
				// The CMSBoxName should be editable only if the box type is editable.
				$nameWidget = $form->getWidget('CMSBoxName');
				if (scCmsBoxTypes::isNameEditableForType($currentBoxType)) {
					$nameWidget->setProperty('readOnly', false);
				} else {
					$nameWidget->setProperty('readOnly', true);
				}
				
				list($moduleId, $module) = $this->getModule();
				$form->getWidget('ModuleOptionFields')->setupModule($moduleId, $module);
				
				$form->addMainPageElements(array(
					'%%CKEDITOR_CONTENTCSS%%' => scSkins::singleton()->getCurrentSkin()->getCssFilesForWysiwygEditor(),
					'%%CKEDITOR_LANGUAGE%%' => scCMS::getCkEditorLanguage()
				));
			}
		}
		
		public function initMainPageElements()
		{
			$this->addMLMainPageElements(scView::getCurrentDataLanguageId(), false);
			
			parent::initMainPageElements();
		}
		
		public function initViewMapperController($viewMapperController)
		{
			$action = $this->getController()->getDispatcher()->getAction();
			
			if ($action != 'dmCreate' && $action != 'dmExecCreate') {
				$viewMapper = $viewMapperController->getViewMapper('scViewMapperCmsBoxOptions');
				$viewMapper->setModule($this->getModule());
				$viewMapper->setCmsBoxId((int)miGetParam('CMSBoxID'));
			}
		}
		
		public function preProcess($actionObj)
		{
			if ($actionObj instanceof miEditAction) {
				$this->_dataFields[] = array(
					'field' => 'scModuleOptionsWidget',
					'data' => 'ModuleOptionFields',
					'modelStrategy' => 'scViewMapperCmsBoxOptions',
	       		);
			} elseif ($actionObj instanceof scExecChangeIsActiveAction) {
				$actionObj->setFieldName('CMSBoxIsActive');
			}
		}
		
		/**
		 * @return array(ModuleID, ModuleClass Implementation)
		 */
		protected function getModule()
		{
			$domainObject = $this->getDomainObject();
			$domainObject->read((int)miGetParam('CMSBoxID'));
			list($className, $moduleId) = scModulesFactory::loadClass($domainObject->get('ModuleClassID'));
			
			return array(
				$moduleId,
				new $className($moduleId, $domainObject->get('CMSBoxID'))
			);
		}
		
		/**
		 * @return scCmsBoxDomainObject
		 */
		protected function createDomainObject()
		{
			return new scCmsBoxDomainObject();
		}
	}
	
	class scCmsBoxesTable extends scTable {
		public function addUiElements(&$rows)
		{
			$areas = scLayout::getAreas();
			foreach ($rows as &$row) {
				$row['CMSBoxArea'] = isset($areas[$row['CMSBoxArea']]) ? $areas[$row['CMSBoxArea']] : '';
				
				try {
					$row['CMSBoxType'] = scCmsBoxTypes::getBoxType($row['CMSBoxType']);
				} catch (miException $e) {
					$row['CMSBoxType'] = _t('Unknown');
				}
			}
		}
	}
	
	class scExecDeleteCmsBoxAction extends miExecDeleteAction  {
		
		public function doAction()
		{
			try {
				$cmsBoxId = (int)miGetParam('CMSBoxID');
			 	$cmsBoxRecord = new scSqlRecord('CMSBoxes', 'CMSBoxID');
			 	$cmsBoxRecord->readPK($cmsBoxId);
			 	$cmsBoxType = $cmsBoxRecord->get('CMSBoxType');
				
			 	if (!scCmsBoxTypes::isDeletableType($cmsBoxType)) {
			 		throw new miException(_t('Can\'t delete this type of box.'));
			 	}
			 	
		 		$recordId = $cmsBoxRecord->delete();
		 		
			} catch (miException $exception) {
			 	$msg = $this->_view->getMessage('MI_RECORD_DELETE_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			$msg = sprintf($this->_view->getMessage('MI_RECORD_DELETED_SUCCESSFULLY_MSG'), $recordId);
			$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	$cmsBoxes = new miSqlRecordset('CMSBoxes');
	$table = new scCmsBoxesTable($cmsBoxes);
	
	$view = new scBoxesView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmExecDelete', 'scExecDeleteCmsBoxAction');
	$page->dispatchAndShow();
?>