<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_CATEGORIES);

	class scCategoriesView extends scView {
		const TEMPLATE_ID_DELETE = 'delete';
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/categories/categories_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/categories/categories_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/categories/categories_create.tmpl',
			self::TEMPLATE_ID_DELETE => 'admin/categories/categories_delete.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CategoryID'
        	),
       		array(
				'field' => 'scCategorySelect',
				'data' => 'CategoryParentID'
        	),
        	array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CategoryName',
				'displayName' => 'Name',
				'validator' => 'scValidatorNoEmpty'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'CategoryDescription',
				'displayName' => 'Description'
        	),
        	array(
				'field' => 'scCMSLayoutSelect',
				'data' => 'CMSLayoutID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CategorySEOTitle'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'CategorySEOKeywords'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'CategorySEODescription'
        	),
        	array(
				'field' => 'scImageUploadWidget',
				'data' => 'CategoryImage',
				'modelStrategy' => 'scViewMapperUploadImage',
				'properties' => array(
					'resource' => scPaths::RESOURCE_CATEGORY_IMAGE,
					'imageMaxWidthSetting' => scSettings::SETTING_CATEGORY_IMAGE_MAX_WIDTH,
					'imageMaxHeightSetting' => scSettings::SETTING_CATEGORY_IMAGE_MAX_HEIGHT,
					'imageWidthField' => 'CategoryImageWidth',
					'imageHeightField' => 'CategoryImageHeight',
					'noThumb' => true
				),
			),
			array(
				'field' => 'scCustomUrlsWidget',
				'data' => 'CustomUrls',
				'modelStrategy' => 'scViewMapperCategoryCustomUrls',
				'properties' => array(
					'template' => 'admin/custom_urls/custom_url.tmpl',
					
					'table' => 'CustomUrls',
					'foreignKey' => 'CustomUrlEntityID',
					'foreignTablePrimaryKey' => 'CustomUrlID',
					
					'fields' => array(
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CustomUrlText',
			       			'properties' => array('minLength' => 1),
			       			'validator' => 'scValidatorUniqueCustomUrl'
			        	),
			       		array(
							'field' => 'miCheckboxWidget',
							'data' => 'CustomUrlIsCurrent'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomUrlID'
			        	)
			        ),
			        'mlFields' => array('CustomUrlText')
				),
			)
        );
		
        protected $_domainObjectDisplayName = 'Category';	// _t('Category');
        
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$action = $this->_controller->getDispatcher()->getAction();
			
			$currentCategoryId = (($action == 'dmEdit') or ($action == 'dmExecEdit')) ? $this->getDomainObject()->get('CategoryID') : 0;
			$field = $form->getWidget('CategoryParentID');
			$field->setCategoryTree($currentCategoryId);
			
			$form->addMainPageElements(array(
				'%%CKEDITOR_CONTENTCSS%%' => scSkins::singleton()->getCurrentSkin()->getCssFilesForWysiwygEditor(),
				'%%CKEDITOR_LANGUAGE%%' => scCMS::getCkEditorLanguage()
			));
		}
		
		protected function createDomainObject()
		{
			return new scCategoryDomainObject();
		}
		
		/**
		 * Post processing - update the nested set
		 */
		public function postProcess($actionObj, $actionResult)
		{
			parent::postProcess($actionObj, $actionResult);
			
			if ($actionResult === false)
				return;
			
			if (($actionObj instanceof miExecCreateAction) or ($actionObj instanceof miExecEditAction) or ($actionObj instanceof miExecDeleteAction)) {
				scNestedSet::buildNestedSet('Categories', 'CategoryID', 'CategoryParentID', 'CategoryLeft', 'CategoryRight');
			}
			
			if ($actionObj instanceof miExecDeleteAction) {
				$discounts = $this->getDomainObject()->getDeactivatedDiscounts();
				$coupons = $this->getDomainObject()->getDeactivatedCoupons();
				
				$message = array($this->getDomainObjectDisplayName() . _t(' successfully deleted.'));
				$messageType = miMessage::MSG_TYPE_INFO;
				if (count($discounts) > 0) {
					$changedDiscounts =  scDiscountService::getDiscountsNames('Discounts', 'DiscountID', 'DiscountName', $discounts);
					$message[] = _t('Discounts: ') .
						scDiscountService::formatDiscountNames($changedDiscounts) .
						_t(' were automatically disabled to prevent unpredicted behaviour.');
						
					$messageType = miMessage::MSG_TYPE_WARNING;
				}
				if (count($coupons) > 0) {
					$changedCoupons = scDiscountService::getDiscountsNames('Coupons', 'CouponID', 'CouponCode', $coupons);
					$message[] = _t('Coupons: ') .
						scDiscountService::formatDiscountNames($changedCoupons) .
						_t(' were automatically disabled to prevent unpredicted behaviour.');
						
					$messageType = miMessage::MSG_TYPE_WARNING;
				}
				
				$message = implode("\r\n", $message);
				$this->addRedirectToListControllerCommand($message, $messageType);
			}
		}
		
		public function setUpSortColumn($sortColumnName)
		{
			parent::setUpSortColumn($sortColumnName);
			$categoryId = miGetParamDefault('CategoryID', false);
			if ($categoryId !== false) {
				$categoryParentId = scUtil::getDbValue('Categories', 'CategoryParentID', 'CategoryID', $categoryId);
				if ($categoryParentId == null)
					scSortAction::setForeignKeyClause('CategoryParentID IS NULL');
				else
					scSortAction::setForeignKeyClause('CategoryParentID=' . $categoryParentId);
			}
		}
	}
	
	class scViewMapperCategoryCustomUrls extends scViewMapperCustomUrls {
		protected $_entityCode = scFriendlyUrl::TYPE_CATEGORY;
		protected $_tableKeyName = 'CategoryID';
	}
	
	class scCategoriesTable extends scTable {
		
		function addUiElements(&$rows)
		{
			$this->assignTreeElements($rows);			
		}
		
		function assignTreeElements(&$rows)
		{
			foreach ($rows as $key => $row) {
				$rows[$key]['CATEGORYDISPLAYSTYLE'] = ($row['NodeLevel'] == 0) ? '' : 'none'; 
				$rows[$key]['HTML_INDENT'] = str_repeat('&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;', $row['NodeLevel']);
				
				$rows[$key]['SPACERDISPLAYSTYLE'] = ($row['SubNodesCount'] == 0) ? '' : 'none';
				$rows[$key]['OPENERDISPLAYSTYLE'] = ($row['SubNodesCount'] == 0) ? 'none' : '';
			}
		}
		
		/**
		 * @return scMessage
		 */
		protected function getMessageObj()
		{
			if ($this->_messageObj == null)
				$this->_messageObj = new scMessage;
			return $this->_messageObj;
		}
	}
	
	class scCategoryDeleteAction extends miAction {
		public function doAction()
		{
			// Basic initialization
			$this->_view->initMainPageElements();
			$parser = new scTemplateParser();
			$parser->assignArray($this->_view->getMainPageElements());
			
			$categoryId = miGetParam('CategoryID');
			$languageId = scMultiLanguage::getCurrentLanguageId();
			
			// Assign the category id
			$parser->assign('%%CATEGORYID%%', $categoryId);
			
			// Get the name of the category marked for deletion
			$record = new scSqlRecordML('Categories', 'CategoryID', array('CategoryName'), $languageId);
			$record->readPK($categoryId);
			$categoryName = $record->get('CategoryName');
			$categoryName = $categoryName[$languageId];
			// Assign the category name
			$parser->assign('%%CATEGORYNAME%%', $categoryName);

			// Get all subcategories that will be deleted
			$subcategories = array();
			
			$sql = 	'SELECT Categories.CategoryID as CategoryID, CategoryName FROM Categories ' .
					'LEFT JOIN CategoriesML ON (Categories.CategoryID = CategoriesML.CategoryID) AND LanguageID = ' . $languageId . ' ' .
					'WHERE CategoryLeft > ' . $record->get('CategoryLeft') . ' AND CategoryRight < '. $record->get('CategoryRight');
			$subcategories = miUtil::getDBArray($sql, 'CategoryID', 'CategoryName');
			
			// Sort the categories by name
			asort($subcategories);

			// Assign the subcategories count
			$subcategoriesCount = count($subcategories);
			$parser->assign('%%SUBCATEGORIESCOUNT%%', $subcategoriesCount);
			
			// Prepare the subcategories table sections
			if ($subcategoriesCount > 0)
				$parser->addSectionInfo(new miTemplateParserSectionInfo('subcategories', $subcategoriesCount, array('%%CATEGORYNAME%%' => array_values($subcategories))));
			else
				$parser->addSectionInfo(new miTemplateParserSectionInfo('nosubcategories'));

			// Get products that will be deleted
			$categoryIds = implode(', ', array_merge(array_keys($subcategories), array($categoryId)));
			$sql =	'SELECT * FROM Products INNER JOIN Categories USING (CategoryID) ' .
					'LEFT JOIN ProductsML ON Products.ProductID = ProductsML.ProductID AND ProductsML.LanguageID = ' . $languageId . ' ' .
					'LEFT JOIN CategoriesML ON (Categories.CategoryID = CategoriesML.CategoryID AND CategoriesML.LanguageID = ' . $languageId . ') ' .
					'WHERE Categories.CategoryID IN (' . $categoryIds . ') ORDER BY ProductName ASC';
			$products = miStaticDBUtil::execSelect($sql);
			
			// Assign the products count
			$productsCount = count($products);
			$parser->assign('%%PRODUCTSCOUNT%%', $productsCount);
			
			// Prepare the products table sections
			if ($productsCount > 0) {
				$products = miUIUtil::templetizeRows(miUIUtil::transposeRows($products));
				$parser->addSectionInfo(new miTemplateParserSectionInfo('products', $productsCount, $products));
			}
			else
				$parser->addSectionInfo(new miTemplateParserSectionInfo('noproducts'));
			
			// Parse and show the template				
			$parser->readTemplate($this->_view->getTemplateFileName(scCategoriesView::TEMPLATE_ID_DELETE));
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $parser->templateParse()));

			// Operation successfully completed
			return true;
		}
	}
	
	$categories = new scSqlTreeRecordsetML('Categories', 'CategoryID', 'CategoryParentID');
	$categories->setMode(scSqlTreeRecordset::MODE_PREORDER);
	$table = new scCategoriesTable($categories);
	
	$view = new scCategoriesView();
	$view->setUpSortColumn('CategorySort');
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	
	$page->getDispatcher()->setActionHandler('dmDelete', 'scCategoryDeleteAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>