<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_COUNTRIES);
	
	class scCountriesView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/countries/country_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/countries/country_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/countries/country_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CountryID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CountryCode2',
				'properties' => array('minLength' => 2, 'maxLength' => 2),
				'displayName' => 'Code 2'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CountryCode3',
				'properties' => array('minLength' => 3, 'maxLength' => 3),
				'displayName' => 'Code 3'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CountryName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Country'
        	),
       		array(
				'field' => 'miCheckboxWidget',
				'data' => 'CountryRequireValidCity',
				'displayName' => 'Require Valid City'
        	),
        );
        
        protected $_domainObjectDisplayName = 'Country';	// _t('Country');
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Add a special validator - the unique key
			$record = new scDomainToRecordAdapter($this->getDomainObject());
			$form->addValidator('CountryCode2', new scValidatorUniqueFields($form, array('CountryCode2'), $record));
			$form->addValidator('CountryCode3', new scValidatorUniqueFields($form, array('CountryCode3'), $record));
			if ($record->hasPK() and (scCountry::getStatesCount($record->getPK()) > 0)) {
				$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('RequireValidAddress', 1));
			} else {
				$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('RequireValidAddress', 0));
			}
		}
		
		/**
		 * @return scCountryDomainObject
		 */
		public function createDomainObject()
		{
			return new scCountryDomainObject();
		}
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
	}
	
	/**
	 * Delete the country
	 * Check that the country is not default country
	 */
	class scCountryExecDeleteAction extends miExecDeleteAction  {
		public function doAction()
		{
			$countryId = (int)miGetParam('CountryID');
			if (scSettings::getSettingValue(scSettings::SETTING_DEFAULT_COUNTRY) == $countryId) {
				$this->_view->addRedirectToListControllerCommand(_t('Default country cannot be deleted.'), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			
			$result = parent::doAction();
			
			if ($result == true) {
				// Delete county states
				$query = 'DELETE FROM Cities WHERE StateID IN (SELECT StateID FROM States WHERE CountryID = ?)';
				miStaticDBUtil::execSQL($query, array($countryId));
				miStaticDBUtil::execDelete('States', 'CountryID', $countryId);
			}
			
			return $result;
		}
	}
	
	/**
	 * Set the default country
	 */
	class scSetDefaultCountryAction extends miAction {
		public function doAction()
		{
			$countryId = (int)miGetParam('CountryID');
			scSettings::setSettingValue(scSettings::SETTING_DEFAULT_COUNTRY, $countryId);
			
			$this->_view->addRedirectToListControllerCommand(_t('Default country successfully set.'), miMessage::MSG_TYPE_INFO);
			return true;
		}	
	}
	
	class scCountriesTable extends scTable {
		protected function addUiElements(&$rows)
		{
			$defaultCountryId = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_COUNTRY);
			foreach ($rows as &$row) {
				$row['DefaultCountry'] = ($row['CountryID'] == $defaultCountryId) ? 1 : 0;
				$row['StatesCount'] = scCountry::getStatesCount($row['CountryID']);
			}
		}
	}
	
	if (!miHasParam('sortBy')) {
		miSetParam('sortBy', 'CountryName');
	}
	
	$countries = new scSqlRecordsetML('Countries', 'CountryID');
	$table = new scCountriesTable($countries);
	
	$view = new scCountriesView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmExecDelete', 'scCountryExecDeleteAction');
	$page->getDispatcher()->setActionHandler('setDefaultCountry', 'scSetDefaultCountryAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>