<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_COUPONS);

	class scCouponsView extends scView {
			
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/coupons/coupon_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/coupons/coupon_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/coupons/coupon_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CouponID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CouponCode',
				'displayName' => 'Code',
				'validator' => 'scValidatorNoEmpty',
        	),
        	array(
				'field' => 'miRadioWidget',
				'data' => 'CouponType'
        	),
        	array(
				'field' => 'miRadioWidget',
				'data' => 'CouponApplyMode'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CouponValue',
				'validator' => array('scValidatorDecimal'),
				'displayName' => 'Value'
        	),
       		array(
				'field' => 'scGroupableSelectWidget',
				'data' => 'CouponFreeShippingMethodID',
				'displayName' => 'Shipping Method',
       			'properties' => array('id' => 'CouponFreeShippingMethodID'),
        	),
       		array(
				'field' => 'scDateWidget',
				'data' => 'CouponValidFrom',
				'displayName' => 'Valid From'
        	),
       		array(
				'field' => 'scDateWidget',
				'data' => 'CouponValidTo',
				'displayName' => 'Valid To'
        	),
       		array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'CouponIsActive',
				'displayName' => 'Active'
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CouponAllowedUsageCount',
				'validator' => 'scValidatorInt'
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CouponUsageCount',
				'properties' => array('readOnly' => true)
        	),
			// Tab 1
       		array(
				'field' => 'scCouponOrderSubtotalExceedsText',
				'data' => 'CouponConditionOrderExceeds',
				'validator' => 'scValidatorDecimal'
        	),
			// Tab 2
			array(
				'field' => 'scDoubleListBoxWidget',			
				'data' => 'Categories',
				// _t('Available Categories'), _t('Selected Categories')
				'properties' => array('leftCaption' => 'Available Categories', 'rightCaption' => 'Selected Categories'),
				'modelStrategy' => 'scViewMapperCouponCategories',
			),
			// Tab 3
        	array(
				'field' => 'scProductSelectWidget',
				'data' => 'Products',
				'modelStrategy' => 'scViewMapperCouponProducts',
				'properties' => array('id' => 'Products')
        	),
			// Tab 4
        	array(
				'field' => 'scCustomerSelectWidget',
				'data' => 'Customers',
				'modelStrategy' => 'scViewMapperCouponCustomers',
				'properties' => array('id' => 'Customers')
        	),
			// Tab 5
        	array(
				'field' => 'scDoubleListBoxWidget',
				'data' => 'CustomerGroups',
				// _t('Available Customer Groups'), _t('Selected Customer Groups')
				'properties' => array('leftCaption' => 'Available Customer Groups', 'rightCaption' => 'Selected Customer Groups'),
				'modelStrategy' => 'scViewMapperCouponCustomerGroups',
        	),
        	// Tab 6
        	array(
				'field' => 'scManufacturerSelectWidget',
				'data' => 'Manufacturers',
				'modelStrategy' => 'scViewMapperCouponManufacturers',
				'properties' => array('id' => 'Manufacturers')
        	),
        	
        );
        
        protected $_domainObjectDisplayName = 'Coupon';	// _t('Coupon');
        
        public function preProcess($actionObj)
		{
			parent::preProcess($actionObj);
			
			// Change Is Active in list
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('CouponIsActive');
				
			if ($actionObj instanceof miCreateAction) {
				$form = $actionObj->getWebForm();
				// Add a default value for 'CouponAllowedUsageCount' field
				$form->setFormData('CouponAllowedUsageCount', 1);
			}
			
			if (($actionObj instanceof miExecCreateAction) || ($actionObj instanceof miExecEditAction)) {
				miStaticDBUtil::startTransaction();
			}
		}
        	
		public function postProcess($actionObj, $actionResult)
		{
			parent::postProcess($actionObj, $actionResult);
			
			if (($actionObj instanceof miExecCreateAction) || ($actionObj instanceof miExecEditAction)) {
				if (($actionResult !== false)) {
					miStaticDBUtil::commit();
				} else {
					miStaticDBUtil::rollback();
				}
			}
		}
        
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Set coupon type
			$field = $form->getWidget('CouponType');
			$field->setRadioButtons(scDiscount::getDiscountTypes(), scDiscount::TYPE_AMOUNT);
			
			//Set coupon applies to
			$field = $form->getWidget('CouponApplyMode');
			$field->setRadioButtons(scDiscount::getAppliesToTypes(), scDiscount::APPLY_MODE_PER_QUANTITY);
			
			// Create scDiscount object
			$domainObject = $this->getDomainObject();
			$coupon = new scDiscount('CouponID', @$domainObject->get($domainObject->getPKName()));
			
			// Initialize additional categories multi select
			$field = $form->getWidget('Categories');
			$coupon->setDiscountCategoriesFieldSettings($field, 'CouponCategories');
			
			// Initialize customer groups multi select
			$field = $form->getWidget('CustomerGroups');
			$coupon->setDiscountCustomerGroupsFieldSettings($field, 'CouponCustomerGroups');
			
			// Ensure that coupon with that code does not exist
			$record = new scDomainToRecordAdapter($this->getDomainObject());
			$form->addValidator('CouponCode', new scValidatorUniqueFields($form, array('CouponCode'), $record));
			
			$field = $form->getWidget('CouponFreeShippingMethodID');
			$field->setOptions(array('' => '') + scDiscount::getShippingMethodsForDropdown());
			
			$validator = new scDiscountValueValidator($form, 'CouponType', 'CouponValue', 'CouponFreeShippingMethodID');
			$form->addValidator('CouponType', $validator);
		}
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
			
			// Create dropdown filter types
			scUIUtil::createDropdownFilter($this, $table, 'CouponType', array('' => _t('All coupon types')) + scDiscount::getDiscountTypes(), '=', '');
		}
		
		protected function createDomainObject()
		{
			return new scCouponDomainObject();
		}
	}
	
	class scCouponOrderSubtotalExceedsText extends miBaseTextWidget {
		function getEditableControl()
		{
			$data = $this->_webForm->getFormData($this->_fieldName);
			if ($data == 0)
				$data = '';
			return array($this->_fieldName => miI18N::htmlEscape($data));
		}
		
		function getData()
		{
			if (isset($this->_properties['readOnly']) && $this->_properties['readOnly']) {
				return array();	
			}
			return array($this->_fieldName => intval(miGetParamDefault($this->_fieldName, '')));
		}
	}
	
	class scViewMapperCouponCategories extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'CouponCategories';
			$this->_tableKeyName = 'CouponID';
			$this->_tableValueName = 'CategoryID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperCouponCustomers extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'CouponCustomers';
			$this->_tableKeyName = 'CouponID';
			$this->_tableValueName = 'CustomerID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperCouponCustomerGroups extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'CouponCustomerGroups';
			$this->_tableKeyName = 'CouponID';
			$this->_tableValueName = 'CustomerGroupID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperCouponProducts extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'CouponProducts';
			$this->_tableKeyName = 'CouponID';
			$this->_tableValueName = 'ProductID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperCouponManufacturers extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'CouponManufacturers';
			$this->_tableKeyName = 'CouponID';
			$this->_tableValueName = 'ManufacturerID';
			parent::__construct($model);
		}
	}
	
	class scCouponDomainObject extends miDefaultDomainObject {
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('Coupons', 'CouponID'));
		}
		
		public function getTableName()
		{
			return $this->_record->getTableName();
		}
		
		public function update()
		{
			$this->prepareData();
			
			return parent::update();
		}
		
		public function insert()
		{
			$this->prepareData();
			
			return parent::insert();
		}
		
		protected function prepareData()
		{
			$this->prepareDataForType();
			$this->prepareDataForApplyMode();
		}
		
		protected function prepareDataForType()
		{
			$discountType = $this->get('CouponType');
			if ($discountType == scDiscount::TYPE_FREE_SHIPPING) {
				$this->set('CouponValue', 0);
				$this->set('CouponApplyMode', scDiscount::APPLY_MODE_TO_SUBTOTAL);
				$this->set('CouponImplementation', scDiscount::IMPLEMENTATION_ORDER_TOTAL);
			} else {
				$this->set('CouponFreeShippingMethodID', null);
			}
		}
		
		protected function prepareDataForApplyMode()
		{
			$applyMode = $this->get('CouponApplyMode');
			if ($applyMode == scDiscount::APPLY_MODE_TO_SUBTOTAL) {
				$this->set('CouponImplementation', scDiscount::IMPLEMENTATION_ORDER_TOTAL);
			} else {
				$this->set('CouponImplementation', scDiscount::IMPLEMENTATION_PRODUCT_PRICE_MODIFIER);
			}
		}
	}
	
	$coupons = new miSqlRecordset('Coupons');
	
	$table = new scDiscountsTable($coupons);
	$table->setDateFromFieldName('CouponValidFrom');
	$table->setDateToFieldName('CouponValidTo');
	$table->setTypeFieldName('CouponType');
	
	$view = new scCouponsView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>