<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_CURRENCIES);

	class scCurrenciesView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/currencies/currency_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/currencies/currency_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/currencies/currency_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyCode',
       			'validator' => 'scValidatorCurrencyCode',
				'displayName' => 'Code'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CurrencyName',
				'displayName' => 'Name',
				'validator' => 'scValidatorNoEmpty',
        	),
        	array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CurrencySymbolLeft',
				'displayName' => 'Symbol Left'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CurrencySymbolRight',
				'displayName' => 'Symbol Right'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyDecimalSymbol',
				'displayName' => 'Decimal Symbol'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyThousandsSeparator',
				'displayName' => 'Thousands Separator'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyNumDecimals',
				'validator' => 'scValidatorInt',
				'displayName' => 'Number Decimals'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyRate',
				'validator' => 'scCurrencyValidatorDecimal',
				'displayName' => 'Rate'
        	),
       		array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'CurrencyIsActive'
        	),
       		array(
				'field' => 'miCheckboxWidget',
				'data' => 'CurrencyAllowFixedPrices',
       			'modelStrategy' => 'scViewMapperCurrencyAllowFixedPrices',
        	),
        );
        
		protected $_domainObjectDisplayName = 'Currency';	// _t('Currency');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function preProcess($actionObj)
		{
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('CurrencyIsActive');
				
			parent::preProcess($actionObj);
		}
		
		public function createDomainObject()
		{
			return new scCurrencyDomainObject($this->_record);
		}
		
		public function postProcess($actionObj, $actionResult)
		{
			parent::postProcess($actionObj, $actionResult);
			
			if (($actionResult != false) && ($actionObj instanceof miExecEditAction)) {
				$domainObj = $this->getDomainObject();
				if ($domainObj->get('CurrencyAllowFixedPrices')) {
					scCurrencyUtil::updateProductPrices($domainObj->get('CurrencyCode'), $domainObj->get('CurrencyRate'));
				}
			}
		}
	}
	
	class scValidatorCurrencyCode extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (preg_match('#^[A-Z0-9]{3}$#', $fieldValue))
				return;
			
			$errors->addError($this->_fieldName, _t('The currency code must be exactly 3 uppercase letter or digit characters.'));
		}
	}
	
	/**
	 * Validates decimal
	 */
	class scCurrencyValidatorDecimal extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (preg_match('#^[0-9]{1,10}\.?([0-9]){0,4}$#', $fieldValue))
				return;
			
			$errors->addError($this->_fieldName, _t('Invalid decimal.'));
		}
	}
	
	/**
	 * Set the default currency
	 */
	class scSetDefaultCurrencyAction extends miAction {
		public function doAction()
		{
			$currencyId = miGetParam('CurrencyID');
			scCurrencyUtil::setBaseCurrency($currencyId);
			scCurrencyUtil::updateBaseRates();
			
			$this->_view->addRedirectToListControllerCommand(_t('Default currency successfully set. Currency rates updated.'), miMessage::MSG_TYPE_INFO);
			return true;
		}	
	}
	
	
	/**
	 * 
	 */
	class scUpdateCurrencyRatesAction extends miAction {
		public function doAction()
		{
			scCurrencyUtil::updateBaseRates();
			
			$this->_view->addRedirectToListControllerCommand(_t('Currency rates updated.'), miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	/**
	 * Delete the currency
	 * Check that the currency is not default currency
	 */
	class scCurrencyExecDeleteAction extends miExecDeleteAction  {
		public function doAction()
		{
			$currencyId = miGetParam('CurrencyID');
			if (scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CURRENCY) == $currencyId) {
				$this->_view->addRedirectToListControllerCommand(_t('Default currency cannot be deleted.'), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			
			return parent::doAction();
		}
	}
	
	class scCurrencyChangeIsActiveAction extends scExecChangeIsActiveAction {
		
		public function doAction()
		{
			$currencyId = (int)miGetParam('CurrencyID');
			if (scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CURRENCY) == $currencyId) {
				$domainObject = $this->_view->getDomainObject();
				$domainObject->read($currencyId);
				$becomeActive = $domainObject->get($this->_fieldName) ? false : true;
				if (!$becomeActive) {
					$this->_view->addRedirectToListControllerCommand(_t('Default currency cannot be deactivated.'), miMessage::MSG_TYPE_ERROR);
					return false;
				}
			}
			
			return parent::doAction();
		}
	}
	
	class scCurrenciesTable extends scTable {
		protected function addUiElements(&$rows)
		{
			scDateTime::setTimezone();
			
			$defaultCurrencyId = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CURRENCY);
			foreach ($rows as &$row) {
				$row['DefaultCurrency'] = ($row['CurrencyID'] == $defaultCurrencyId) ? 1 : 0;
				if ($row['CurrencyRateLastUpdated'] < (time() - (scSettings::getSettingValue(scSettings::SETTING_CURRENCY_RATE_EXPIRE_DAYS) * 24*60*60))) {
					$row['CurrencyRate'] = _t('Expired');
				}
				$row['CurrencyRateLastUpdated'] = scDateTime::getLocalFormattedTimestamp($row['CurrencyRateLastUpdated']);
			}
		}
	}
	
	class scViewMapperCurrencyAllowFixedPrices extends miViewMapper {
		
		protected $_domainObj;
		protected $_fieldValue = false;
		
		public function __construct(miViewMapperController $viewMapperController)
		{
			parent::__construct($viewMapperController);
			
			$this->_domainObj = $viewMapperController->getDomainObject();
		}
		
		public function get($fieldName)
		{
			return $this->_domainObj->get($fieldName);
		}
		
		public function set($fieldName, $fieldValue)
		{
			$this->_domainObj->set($fieldName, $fieldValue);
			$this->_fieldValue = $fieldValue;
		}
		
		public function insert()
		{
			$this->alterProductsTable();
		}
	
		public function update()
		{
			$this->alterProductsTable();
		}
		
		protected function alterProductsTable()
		{
			if (!$this->_fieldValue) {
				return;
			}
		
			$currencyCode = strtoupper($this->_domainObj->get('CurrencyCode'));
			scCurrencyUtil::validateCurrencyCode($currencyCode);
			
			$productFields = miStaticDBUtil::getTableFields('Products');
			if (in_array('ProductPrice' . $currencyCode, $productFields)) {
				return;	
			}
			
			// The price field does not exist, we should create it
			try {
				miStaticDBUtil::execSQL(sprintf(
					'ALTER TABLE Products 
						ADD COLUMN ProductPrice%s decimal(10,2) NOT NULL default 0.00 AFTER ProductPrice,
						ADD COLUMN ProductPrice%sAuto tinyint(1) unsigned NOT NULL default 1 AFTER ProductPrice%s;',
						$currencyCode, $currencyCode, $currencyCode
				));
			} catch (miException $exception) {
				// The user has no rights to alter the Products table
				$this->_domainObj->set('CurrencyAllowFixedPrices', 0);
				$this->_domainObj->update();
				throw $exception;
			}
			
			scCurrencyUtil::updateProductPrices($currencyCode, $this->_domainObj->get('CurrencyRate'));
		}
	}
	
	$currencies = new scSqlRecordsetML('Currencies', 'CurrencyID');
	$table = new scCurrenciesTable($currencies);
	
	$view = new scCurrenciesView();
	$view->setRecord(new scSqlRecordML('Currencies', 'CurrencyID',
		array('CurrencyName', 'CurrencySymbolLeft', 'CurrencySymbolRight'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
	$view->setUpSortColumn('CurrencySort');
	$view->setTable($table);
	$view->addMainPageElements(array('%%BASECURRENCYCODE%%' => miI18N::htmlEscape(scCurrencyUtil::getBaseCurrencyCode())));
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmExecDelete', 'scCurrencyExecDeleteAction');
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scCurrencyChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('setDefaultCurrency', 'scSetDefaultCurrencyAction');
	$page->getDispatcher()->setActionHandler('updateRates', 'scUpdateCurrencyRatesAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>