<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_MODULES_CURRENCY_RATE);

	class scCurrencyRateModulesView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_EDIT => 'admin/currency_rate_modules/currency_rate_modules_edit.tmpl',
			self::TEMPLATE_ID_LIST => 'admin/currency_rate_modules/currency_rate_modules_mgmt.tmpl',
			self::TEMPLATE_ID_VIEW => 'admin/currency_rate_modules/currency_rate_modules_view.tmpl',
		);
		
        protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyRateModuleID'
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyRateModuleName',
				'properties' => array('readOnly' => true)
        		),
        	array(
				'field' => 'miCheckboxWidget',
				'data' => 'CurrencyRateModuleIsActive'
        		),
        	array(
				'field' => 'miBaseTextWidget',
				'data' => 'CurrencyRateModuleImage',
        		'properties' => array('readOnly' => true)
        	),
		);
		
		protected $_domainObjectDisplayName = 'Currency rate module';	// _t('Currency rate module');
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function preProcess($actionObj)
		{
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('CurrencyRateModuleIsActive');
		}
	}
	
	class scViewAction extends miEditAction{
		public function doAction()
		{
			try{
				scDateTime::setTimezone();
				
				$vmController = $this->_view->getViewMapperController();
				$vmController->read();
				$moduleId = $vmController->getDomainObject()->get('ModuleID');
				
				$factory = new scModulesFactory();
				$currencyRate = $factory->getImplementation($moduleId, 'ICurrencyRates');
				$currencyRates = $currencyRate->getCurrencyRates();
				
				$rows = array();

				foreach ($currencyRates as $rate) {
					$rows[] = array(
						'baseCurrency' => $rate->getBaseCurrency(),
						'quoteCurrency' => $rate->getQuoteCurrency(),
						'rate' => $rate->getRate(),
						'timestamp' => scDateTime::getLocalFormattedTimestamp($rate->getTimestamp()),
					);		
									
				}
				$this->_view->initMainPageElements();
				$recordset = new scPreparedRecordset($rows);
				$table = new scTable($recordset);
				$table->assign('%%BANKNAME%%', $vmController->getDomainObject()->get('CurrencyRateModuleName'));
				
				$table->addMainPageElements($this->_view->getMainPageElements());
				
				$html = $table->parse($this->_view->getTemplateFileName(miDefaultView::TEMPLATE_ID_VIEW));
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	$currencyRateModules = new miSqlRecordset('CurrencyRateModules');
	$table = new scTable($currencyRateModules);
	
	$view = new scCurrencyRateModulesView();
	$view->setRecord(new miSqlRecord('CurrencyRateModules', 'CurrencyRateModuleID'));
	$view->setUpSortColumn('CurrencyRateModuleSort');
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmView', 'scViewAction');
	$page->dispatchAndShow();
?>