<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_CUSTOM_URLS);
	
	class scCustomUrlView extends scView {
			
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/custom_urls/custom_urls_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/custom_urls/custom_urls_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/custom_urls/custom_urls_create.tmpl'
		);
		
		protected $_dataFields = array(
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomUrlID'
        	),
			array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CustomUrlText',
				'validator' => 'scValidatorCustomUrlTextUnique'
        	),
        	array(
				'field' => 'miBaseRadioWidget',
				'data' => 'CustomUrlEntityCode',
        		'validator' => 'scValidatorNoEmpty'
        	),
        	array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomUrlEntityID',
        		'validator' => 'scValidatorCustomUrlEntityIDNoEmpty'
        	),
        	array(
				'field' => 'miCheckboxWidget',
				'data' => 'CustomUrlIsCurrent',
        	)
        );
        
        protected $_domainObjectDisplayName = 'Custom Urls';	// _t('Custom Urls');
        
        /**
         * @param miWebForm $form
         */
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$languageId = scMultiLanguage::getCurrentLanguageId();
			
			// Set EntityCode radio buttons
			$field = $form->getWidget('CustomUrlEntityCode');
			$field->setRadioButtons(scCustomUrlDomainObject::getAllEntities());
			
			// Set ProductID options
			$sql = 'SELECT Products.ProductID, ProductName FROM Products ' . 
				'LEFT JOIN ProductsML ON Products.ProductID = ProductsML.ProductID ' .
				'AND LanguageID = ' . $languageId . ' ORDER BY ProductName';
			$products = miUtil::getDBArray($sql, 'ProductID', 'ProductName');
			$widget = new miBaseSelectWidget($form);
			$widget->setFieldName(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_PRODUCT]);
			$widget->setOptions(array('' => '') + $products);
			$form->addWidget(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_PRODUCT], $widget);
			
			// Set CategoryID options
			$widget = new scCategorySelect($form, array('class' => 'customUrlEntityID', 'id' => 'customUrlEntityID_1'));
			$widget->setFieldName(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_CATEGORY]);
			$widget->setCategoryTree(0);
			$form->addWidget(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_CATEGORY], $widget);
			
			// Set ManufacturerID options
			$sql = 'SELECT Manufacturers.ManufacturerID, ManufacturerName FROM Manufacturers ' . 
				'LEFT JOIN ManufacturersML ON Manufacturers.ManufacturerID = ManufacturersML.ManufacturerID ' .
				'AND LanguageID = ' . $languageId . ' ORDER BY ManufacturerName';
			$manufacturers = miUtil::getDBArray($sql, 'ManufacturerID', 'ManufacturerName');
			$widget = new miBaseSelectWidget($form);
			$widget->setFieldName(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_MANUFACTURER]);
			$widget->setOptions(array('' => '') + $manufacturers);
			$form->addWidget(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_MANUFACTURER], $widget);
			
			// Set CMSFormID options
			$sql = 'SELECT CMSFormID,CMSFormName FROM CMSForms ORDER BY CMSFormName';
			$cmsForms = miUtil::getDBArray($sql, 'CMSFormID', 'CMSFormName');
			$widget = new miBaseSelectWidget($form);
			$widget->setFieldName(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_DYNAMIC_FORM]);
			$widget->setOptions(array('' => '') + $cmsForms);
			$form->addWidget(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_DYNAMIC_FORM], $widget);
			
			// Set CMSRichTextPageID options
			$sql = 'SELECT CMSRichTextPages.CMSRichTextPageID, CMSRichTextPageTitle FROM CMSRichTextPages ' . 
				'LEFT JOIN CMSRichTextPagesML ON CMSRichTextPages.CMSRichTextPageID = CMSRichTextPagesML.CMSRichTextPageID ' .
				'AND LanguageID = ' . $languageId . ' ORDER BY CMSRichTextPageTitle';
			$richTextPages = miUtil::getDBArray($sql, 'CMSRichTextPageID', 'CMSRichTextPageTitle');
			$widget = new miBaseSelectWidget($form);
			$widget->setFieldName(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_RICH_TEXT_PAGE]);
			$widget->setOptions(array('' => '') + $richTextPages);
			$form->addWidget(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_RICH_TEXT_PAGE], $widget);
			
			// Set GalleryID options
			$sql = 'SELECT GalleryID, GalleryName FROM Galleries ORDER BY GalleryName';
			$galleries = miUtil::getDBArray($sql, 'GalleryID', 'GalleryName');
			$widget = new miBaseSelectWidget($form);
			$widget->setFieldName(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_GALLERY]);
			$widget->setOptions(array('' => '') + $galleries);
			$form->addWidget(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_GALLERY], $widget);
			
			// Set CMSDynamicPageID options
			$dynamicPages = scFriendlyUrl::getLinkableDynamicPages();
			$widget = new miBaseSelectWidget($form);
			$widget->setFieldName(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_DYNAMIC_PAGE]);
			$widget->setOptions(array('' => '') + $dynamicPages);
			$form->addWidget(scCustomUrlDomainObject::$fieldNameByEntityCode[scFriendlyUrl::TYPE_DYNAMIC_PAGE], $widget);
		}
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
			
			$entities = scCustomUrlDomainObject::getAllEntities();
			scUIUtil::createDropdownFilter($this, $table, 'CustomUrlEntityCode', array('' => _t('All page types')) + $entities, '=', '');
			
			scUIUtil::createDropdownFilter($this, $table, 'CustomUrlIsCurrent', array('' => _t('All statuses'), '0' => _t('Not current'), '1' => _t('Current')), '=', '');
		}
		
		/**
		 * @return scCustomUrlDomainObject
		 */
		protected function createDomainObject()
		{
			return new scCustomUrlDomainObject;
		}
	}
	
	class scCustomUrlsTable extends scTable {
		public function addUiElements(&$rows)
		{
			$entities = scCustomUrlDomainObject::getAllEntities();
			$languageNames = scMultiLanguage::getActiveLanguageNames();
			
			$languageId = scView::getCurrentDataLanguageId();
			
			foreach ($rows as &$row) {
				$row['HTML_CustomUrlText'] = '';
				$customUrlTexts = miUtil::getDBArray('SELECT LanguageID, CustomUrlText FROM CustomUrlsML WHERE CustomUrlID = ' . $row['CustomUrlID'], 'LanguageID', 'CustomUrlText');
				foreach ($customUrlTexts as $langId => $url)
					$row['HTML_CustomUrlText'] .= '<a href="../' . $url . '">' . $languageNames[$langId] . ' - ' . $url . '</a><br />';
				
				$row['CustomUrlIsCurrent'] = $row['CustomUrlIsCurrent'] ? _t('Yes') : '-';
				
				switch ($row['CustomUrlEntityCode']) {
					case scFriendlyUrl::TYPE_PRODUCT:
						$row['CustomUrlEntityLink'] = 'products.php?action=dmEdit&ProductID=' . $row['CustomUrlEntityID'];
						$row['CustomUrlEntityName'] = scUtil::getMLDbValue('ProductsML', 'ProductName', 'ProductID', $row['CustomUrlEntityID'], $languageId);
						break;
					case scFriendlyUrl::TYPE_CATEGORY:
						$row['CustomUrlEntityLink'] = 'categories.php?action=dmEdit&CategoryID=' . $row['CustomUrlEntityID'];
						$row['CustomUrlEntityName'] = scUtil::getMLDbValue('CategoriesML', 'CategoryName', 'CategoryID', $row['CustomUrlEntityID'], $languageId);
						break;
					case scFriendlyUrl::TYPE_MANUFACTURER:
						$row['CustomUrlEntityLink'] = 'manufacturers.php?action=dmEdit&ManufacturerID=' . $row['CustomUrlEntityID'];
						$row['CustomUrlEntityName'] = scUtil::getMLDbValue('ManufacturersML', 'ManufacturerName', 'ManufacturerID', $row['CustomUrlEntityID'], $languageId);
						break;
					case scFriendlyUrl::TYPE_DYNAMIC_FORM:
						$row['CustomUrlEntityLink'] = 'forms.php?action=dmEdit&CMSFormID=' . $row['CustomUrlEntityID'];
						$row['CustomUrlEntityName'] = scUtil::getMLDbValue('CMSFormsML', 'CMSFormTitle', 'CMSFormID', $row['CustomUrlEntityID'], $languageId);
						break;
					case scFriendlyUrl::TYPE_RICH_TEXT_PAGE:
						$row['CustomUrlEntityLink'] = 'rich_text_pages.php?action=dmEdit&CMSRichTextPageID=' . $row['CustomUrlEntityID'];
						$row['CustomUrlEntityName'] = scUtil::getMLDbValue('CMSRichTextPagesML', 'CMSRichTextPageTitle', 'CMSRichTextPageID', $row['CustomUrlEntityID'], $languageId);
						break;
					case scFriendlyUrl::TYPE_GALLERY:
						$row['CustomUrlEntityLink'] = 'galleries.php?action=dmEdit&GalleryID=' . $row['CustomUrlEntityID'];
						$row['CustomUrlEntityName'] = scUtil::getDbValue('Galleries', 'GalleryName', 'GalleryID', $row['CustomUrlEntityID']);
						break;
					case scFriendlyUrl::TYPE_NEWS:
						$row['CustomUrlEntityLink'] = 'module.php?ModuleName=com.summercart.news&action=dmEdit&ModuleName=com.summercart.news&NewsID=' . $row['CustomUrlEntityID'];
						$row['CustomUrlEntityName'] = scUtil::getMLDbValue('NewsML', 'NewsTitle', 'NewsID', $row['CustomUrlEntityID'], $languageId);
						break;
					case scFriendlyUrl::TYPE_DYNAMIC_PAGE:
						$pages = scFriendlyUrl::getLinkableDynamicPages();
						$row['CustomUrlEntityLink'] = '?action=editDynPage&DynPage=' . $row['CustomUrlEntityID'];
						$row['CustomUrlEntityName'] = $pages[$row['CustomUrlEntityID']];
						break;
				}
				
				$row['CustomUrlEntityCode'] = $entities[$row['CustomUrlEntityCode']];
			}
		}
	}
	
	class scValidatorCustomUrlTextUnique extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			$customUrlId = $this->_webForm->getSubmittedData('CustomUrlID');
			
			foreach ($fieldValue as $langId => $url) {
				if ($url == '') {
					$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Url cannot be empty.'));
					continue;
				}
				
				$parsed = @scUrlUtil::parse($url);
				if ($parsed === false) {
					$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Invalid url.'));
					continue;
				}
				if (isset($parsed[scUrlUtil::SCHEME]) || isset($parsed[scUrlUtil::HOST])) {
					$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Url cannot be outside store root.'));
					continue;
				}
				
				// Check if there is another record with the value
				$checkRecord = miStaticDBUtil::execSelect('SELECT CustomUrlID FROM CustomUrlsML' .
					' WHERE (CustomUrlID != ? || LanguageID != ?) AND CustomUrlText = ?',
					array($customUrlId, $langId, $url));
				if (!empty($checkRecord))
					$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Url already exists.'));
				
				// Check if there is another post with the value
				foreach ($fieldValue as $languageId => $postUrl)
					if (($url == $postUrl) && ($langId != $languageId) && !$errors->hasFieldErrors($this->_fieldName . '[' . $langId . ']'))
						$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Url already exists.'));
			}
		}
	}
	
	class scValidatorCustomUrlEntityIDNoEmpty extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			
			// Non-multilanguage field
			if (!empty($fieldValue))
				return;
			
			$entityCode = $this->_webForm->getSubmittedData('CustomUrlEntityCode');
			$errors->addError(scCustomUrlDomainObject::$fieldNameByEntityCode[$entityCode], _t('Field is empty.'));
		}
	}
	
	class scRedirectToDynamicPageEditAction extends miAction {
		
		public function doAction()
		{
			$pages = scFriendlyUrl::getLinkableDynamicPages();
			$page = (int)miGetParam('DynPage');
			if (!isset($pages[$page])) {
				throw new miException(sprintf(_i('Unknown page type "%s".'), $page));
			}
			
			$pageId = scUtil::getDbValue('CMSDynamicPages', 'CMSDynamicPageID', 'CMSDynamicPagePath', $pages[$page]);
			if (empty($pageId)) {
				throw new miException(sprintf(_i('Unknown page "%s".'), $pages[$page]));
			}
			
			scUtil::redirect('dynamic_pages.php?action=dmEdit&CMSDynamicPageID=' . $pageId);
		}
	}

	$customUrls = new scSqlRecordsetML('CustomUrls', 'CustomUrlID');
	$table = new scCustomUrlsTable($customUrls);
	
	$view = new scCustomUrlView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('editDynPage', 'scRedirectToDynamicPageEditAction');
	$page->dispatchAndShow();
?>