<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_CUSTOMER_GROUPS);

	class scCustomerGroupsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/customer_groups/customer_group_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/customer_groups/customer_group_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/customer_groups/customer_group_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerGroupID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CustomerGroupName',
				'displayName' => 'Customer Group Name',
				'validator' => 'scValidatorNoEmpty',
        	),
        );
        
		protected $_domainObjectDisplayName = 'Customer Group';	// _t('Customer Group');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		protected function createDomainObject()
		{
			return new scCustomerGroupDomainObject();
		}
	}
	
	/**
	 * Set the default customer group
	 */
	class scSetDefaultCustomerGroupAction extends miAction {
		public function doAction()
		{
			$customerGroupId = miGetParam('CustomerGroupID');
			scSettings::setSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP, $customerGroupId);
			
			$this->_view->addRedirectToListControllerCommand(_t('Default customer group successfully set.'), miMessage::MSG_TYPE_INFO);
			return true;
		}	
	}
	
	/**
	 * Delete the customer group
	 * Check that the customer group is not default customer group
	 */
	class scCustomerGroupExecDeleteAction extends miExecDeleteAction  {
		public function doAction()
		{
			$customerGroupId = miGetParam('CustomerGroupID');
			if (scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP) == $customerGroupId) {
				$this->_view->addRedirectToListControllerCommand(_t('Default customer group cannot be deleted.'), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			
			return parent::doAction();
		}
	}
	
	class scCustomerGroupsTable extends scTable {
		protected function addUiElements(&$rows)
		{
			$defaultCustomerGroupId = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP);
			foreach ($rows as &$row) {
				$row['DefaultCustomerGroup'] = ($row['CustomerGroupID'] == $defaultCustomerGroupId) ? 1 : 0;
			}
		}
	}
	
	$customerGroups = new scSqlRecordsetML('CustomerGroups', 'CustomerGroupID');
	$table = new scCustomerGroupsTable($customerGroups);
	
	$view = new scCustomerGroupsView();
	$view->setUpSortColumn('CustomerGroupSort');
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmExecDelete', 'scCustomerGroupExecDeleteAction');
	$page->getDispatcher()->setActionHandler('acSetDefaultCustomerGroup', 'scSetDefaultCustomerGroupAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>