<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_CUSTOMERS);

	class scCustomersView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/customers/customer_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/customers/customer_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/customers/customer_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerEmail',
				'validator' => array('scValidatorEmail', 'scValidatorNoEmpty'),
				'displayName' => 'Email'
        	),
       		array(
				'field' => 'scCustomerHashedPasswordWidget',
				'data' => 'CustomerPassword',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerPasswordHashMode',
        	),
        	array(
				'field' => 'miSelectWidget',
				'data' => 'CustomerStatus',
			),
			array(
				'field' => 'miSelectWidget',
				'data' => 'CustomerGroupID',
			),
			array(
				'field' => 'miTextWidget',
				'data' => 'CustomerBonusPoints',
				'validator' => 'scValidatorPhpInt',
			),
			array(
				'field' => 'scNewslettersWidget',
				'data' => 'Newsletters',
				'modelStrategy' => 'scViewMapperNewsletters',
				'properties' => array(
					'template' => 'admin/customers/newsletters_widget.tmpl'
				),
			),
        	
        	array(
        		'field' => 'scCustomerAddressesWidget',
        		'data' => 'CustomerAddresses',
        		'modelStrategy' => 'scViewMapperCustomerAddresses',
        		'properties' => array(
					'template' => 'admin/customers/customer_addresses_rs.tmpl',
					'table' => 'CustomerAddresses',
					'foreignKey' => 'CustomerID',
					'foreignTablePrimaryKey' => 'CustomerAddressID',
					'fields' => array(
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressID'
		        		),
			       		array(
							'field' => 'scDefaultDynRowWidget',
							'data' => 'CustomerAddressIsDefaultBilling'
		        		),
			       		array(
							'field' => 'scDefaultDynRowWidget',
							'data' => 'CustomerAddressIsDefaultShipping'
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressFirstName',
							'displayName' => 'First Name',
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressLastName',
							'displayName' => 'Last Name'
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressCompany',
							'displayName' => 'Company'
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressLine1',
							'displayName' => 'Address 1',
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressLine2',
							'displayName' => 'Address 2'
		        		),
		        		array(
							'field' => 'scCityNameWidget',
							'data' => 'CustomerAddressCity',
		        			'displayName' => 'City',
			       			'properties' => array(
		        				'template' => 'admin/widgets/city_name_widget.tmpl',
			       				'CityIDFieldId' => 'CityID', 
			       				'StateIDFieldId' => 'StateID',
			       				'CountryIDFieldId' => 'CountryID',
		        				'tabindex' => 9
			       				),
			        		),
			        	array(
			        		'field' => 'miBaseTextWidget',
			        		'data' => 'CityID',
			        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressZip',
							'displayName' => 'State'
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressPhone',
							'displayName' => 'Phone'
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressFax',
							'displayName' => 'Fax'
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomerAddressVatNumber',
		        		),
			       		array(
							'field' => 'miBaseSelectWidget',
							'data' => 'CountryID',
							'displayName' => 'Country ID'
		        		),
			       		array(
							'field' => 'scOptionalBaseSelectWidget',
							'data' => 'StateID',
			       			'properties' => array(
			       				'noneValue' => '',
			       			),
			       			'displayName' => 'State ID'
		        		),
		        	),	
        		),
        	),
		);
       
		protected $_domainObjectDisplayName = 'Customer';	// _t('Customer');
		
		protected $_defaultFilterValues = array(
			'CustomerFullName' => '',
		);
		
		protected function createDomainObject()
		{
			return new scCustomerDomainObject();
		}
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
			
			// Create dropdown filter for billing country
			$countriesArray = array('' => _t('All countries')) + scCountry::getCountryNames();
			scUIUtil::createDropdownFilter($this, $table, 'CountryID', $countriesArray, '=', '');
			
			$customerGroupsArray = array('' => _t('All customer groups')) + scCustomerGroups::getCustomerGroupNames();
			scUIUtil::createDropdownFilter($this, $table, 'CustomerGroupID', $customerGroupsArray, '=', '');
		}
		
		/**
		 * 
		 * @param miWebForm $form
		 */
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$field = $form->getWidget('CustomerStatus');
			$field->setOptions(scCustomerStatuses::singleton()->getStatuses());
			
			$field = $form->getWidget('CustomerGroupID');
			$field->setOptions(scCustomerGroups::getCustomerGroupNames());
			
			$record = new scDomainToRecordAdapter($this->getDomainObject());
			$form->addValidator('CustomerEmail', new scValidatorUniqueFields($form, array('CustomerEmail'), $record, _t('This email is already registered.')));
			
			if ($this->getController()->getDispatcher()->getActionObj() instanceof miExecCreateAction) {
				$form->removeValidators('CustomerBonusPoints');
			}
			if ($this->getController()->getDispatcher()->getActionObj() instanceof miCreateAction) {
				$field = $form->getWidget('CustomerPasswordHashMode');
				$field->setDefaultValue(scSettings::getSettingValue(scSettings::SETTING_PASSWORD_HASH_MODE));
			}
		}
		
		public function postProcess($actionObj, $actionResult)
		{
			parent::postProcess($actionObj, $actionResult);
			
			if ($actionResult == false)
				return;
			
			if ($actionObj instanceof miExecDeleteAction) {
				// Check for coupons or dicounts referring to this customer and make them inactive if no customers are left
				$discounts = $this->getDomainObject()->getDeactivatedDiscounts();
				$coupons = $this->getDomainObject()->getDeactivatedCoupons();
				
				$message = array($this->getDomainObjectDisplayName() . _t(' successfully deleted.'));
				$messageType = miMessage::MSG_TYPE_INFO;
				if (count($discounts) > 0) {
					$changedDiscounts =  scDiscountService::getDiscountsNames('Discounts', 'DiscountID', 'DiscountName', $discounts);
					$message[] = _t('Discounts: ') .
						scDiscountService::formatDiscountNames($changedDiscounts) .
						_t(' were automatically disabled to prevent unpredicted behaviour.');
						
					$messageType = miMessage::MSG_TYPE_WARNING;
				}
				if (count($coupons) > 0) {
					$changedCoupons = scDiscountService::getDiscountsNames('Coupons', 'CouponID', 'CouponCode', $coupons);
					$message[] = _t('Coupons: ') .
						scDiscountService::formatDiscountNames($changedCoupons) .
						_t(' were automatically disabled to prevent unpredicted behaviour.');
						
					$messageType = miMessage::MSG_TYPE_WARNING;
				}
				
				$message = implode("\r\n", $message);
				$this->addRedirectToListControllerCommand($message, $messageType);
			}
		}
	}
	
	class scCustomerHashedPasswordWidget extends scHashedPasswordWidget {
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			$data = $this->_webForm->getSubmittedDataRow();
			if ($data['CustomerID'] == ICheckoutModule::GUEST_USER_ID)
				return;
			
			parent::validateData($errors);
		}
	}
	
	class scCustomerAddressesWidget extends scDynRowsWidget {
		
		/**
		 * @var array
		 */
		protected $_countryNames = null;
		
		/**
		 * @var array
		 */
		protected $_stateNames = null;
		
		public function getData()
		{
			return parent::getData();
		}
		
		protected function modifyFieldProperties(&$properties, $rowNum, $suffix, $fieldName)
		{
			$fieldName = str_replace($suffix, '', $fieldName);
			if (($fieldName == 'CustomerAddressCity') && ($rowNum > 0)) {
				$properties['CityIDFieldId'] = $properties['CityIDFieldId'] . $suffix;
				$properties['StateIDFieldId'] = $properties['StateIDFieldId'] . $suffix;
				$properties['CountryIDFieldId'] = $properties['CountryIDFieldId'] .$suffix;
			}
		}
		
		protected function initField(miBaseWidget $field, $fieldName)
		{
			if (!($field instanceof miBaseSelectWidget)) {
				return;
			}
			
			switch ($fieldName) {
				case 'CountryID':
					$field->setOptions($this->getCountryNames());
					break;
				
				case 'StateID':
					$field->setOptions(array(scOptionalBaseSelectWidget::NONE_VALUE => '') + $this->getStateNames());
					break;
			}
		}
		
		/**
		 * @return array
		 */
		protected function getCountryNames()
		{
			if (!is_null($this->_countryNames)) {
				return $this->_countryNames;
			}
			
			$this->_countryNames = scCountry::getCountryNames();
			return $this->_countryNames;
		}
		
		/**
		 * @return array
		 */
		protected function getStateNames()
		{
			if (!is_null($this->_stateNames)) {
				return $this->_stateNames;
			}
			
			$this->_stateNames = scCountry::getStateNames();
			return $this->_stateNames;
		}
	}
	
	class scViewMapperCustomerAddresses extends scViewMapperDynRows {
		
		protected function createRecord($rsProperties)
		{
			return new scDomainToRecordAdapter(new scCustomerAddress());
		}
	}
	
	class scCustomerExecEditAction extends miExecEditAction {
		
		public function doAction()
		{
			$form = $this->getWebForm();
			
			// Remove e-mail and password validators for gueast customer.
			if (miGetParam('CustomerID') == ICheckoutModule::GUEST_USER_ID) {
				$form->removeValidators('CustomerEmail');
				$form->removeValidators('CustomerPassword');
			}

			// Generate autoapproved points transaction
			$customerId = miGetParam('CustomerID');
			$customer = new scCustomerDomainObject();
			$customer->read($customerId);
			$existingPoints = $customer->get('CustomerBonusPoints');
			
			if (parent::doAction()) {
				$newPoints = $form->getSubmittedData('CustomerBonusPoints');
				if ($existingPoints != $newPoints) {
					$customer->set('CustomerBonusPoints', $existingPoints);
					$customer->update();

					$resultPoints = $newPoints - $existingPoints;
					$actionType = ($resultPoints > 0 ? scBonusPoints::ACTION_ASSIGN : scBonusPoints::ACTION_SPEND);
					$form->setFormData('CustomerBonusPoints', $existingPoints);
					scBonusPointsTransaction::create($actionType, abs($resultPoints), '', $customerId, _t('Admin updated number of points.'), scBonusPoints::ASSIGNMENT_REASON_ADMIN_ACTION, true);
				}
				
				return true;
			}
			
			return false;
		}
	}
	
	class scCustomerExecDeleteAction extends miExecDeleteAction {
		public function doAction()
		{
			if (miGetParam('CustomerID') == ICheckoutModule::GUEST_USER_ID) {
				$msg = _t('The guest customer cannot be deleted.');
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_ERROR);
				return false;
			}
				
			return parent::doAction();
		}
	}
	
	class scCustomersPrintExportUtil extends scPrintExportUtil
	{
		protected function addFields($fields)
		{
			foreach ($fields as $dataField) {
					if (array_key_exists('displayName', $dataField))
						$this->_fields[$dataField['data']] = scInterfaceTexts::getTranslatedText($dataField['displayName']);
						
					if (isset($dataField['properties']) && isset($dataField['properties']['fields']))
						$this->addFields($dataField['properties']['fields']);
			}
		}
		
		public function getFields()
		{
			if (is_null($this->_fields)) {
				$fieldSubstitues = $this->getFieldSubstitues();
				
				// Find the export fields
				$this->_fields = array();
				$this->addFields($this->_view->getDataFields());
			}
			
			return $this->_fields;
		}
		
		public function getRecordset()
		{
			$recordset = new miSqlRecordset('Customers');
			$recordset->addJoinCondition('LEFT', 'CustomerAddresses', 'USING(CustomerID)');
			$recordset->setSelectFields(array(
					'*',
					'CONCAT(CustomerAddressFirstName, " ", CustomerAddressLastName) AS CustomerFullName'
				));
			$recordset->setHavingFilterNames(array('CustomerFullName'));
			
			return $recordset;
		}
	}
	
	class scCustomersPrintAction extends scPrintAction {
			
		protected function createPrintExportUtil()
		{
			return new scCustomersPrintExportUtil($this->_view);
		}
	}
	
	class scCustomersExportCSVAction extends scExportCSVAction
	{
		protected function createPrintExportUtil()
		{
			return new scCustomersPrintExportUtil($this->_view);
		}
	}
	
	class scValidatorPhpInt extends miValidator {
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if ((string)(int)$fieldValue === (string)$fieldValue)
				return;
			
			$errors->addError($this->_fieldName, _t('Invalid integer.'));
		}
	}
	
	$showUnfinishedOrders = scSettings::getSettingValue(scSettings::SETTING_SHOW_UNFINISHED_ORDERS);
	$hideUnfinishedOrdersSql = $showUnfinishedOrders ? '' : ' AND OrderStatus != ' . scOrderStatus::ORDER_STATUS_UNFINISHED;
	
	// Get customers and allow filtering by all their addresses by joining the CustomerAddresses table.
	$customers = new miSqlRecordset('Customers');
	$customers->addJoinCondition('LEFT', 'CustomerAddresses', 'USING(CustomerID)');
	$customers->setSelectFields(array(
		'Customers.*',
		'CONCAT(CustomerAddressFirstName, " ", CustomerAddressLastName) AS CustomerFullName',
		'CustomerAddressIsDefaultBilling',
		'CustomerAddressIsDefaultShipping',
		'CustomerAddressFirstName',
		'CustomerAddressLastName',
		'CustomerAddressCompany',
		'CustomerAddressLine1',
		'CustomerAddressLine2',
		'CustomerAddressCity',
		'StateID',
		'CountryID',
		'CustomerAddressZip',
		'CustomerAddressPhone',
		'CustomerAddressFax',
		'(SELECT COUNT(OrderID) FROM Orders WHERE Orders.CustomerID = Customers.CustomerID' . $hideUnfinishedOrdersSql . ') AS OrdersCount'
	));
	$customers->setHavingFilterNames(array('CustomerFullName'));
	$customers->setGroupBy('Customers.CustomerID');
	
	$table = new scCustomersTable($customers);
	
	$view = new scCustomersView();
	$view->setTable($table);
	$view->addMainPageElements(array(
		'%%STATES_JS_DATA%%' => scUIUtil::getStatesJsData(),
	));
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmExecEdit', 'scCustomerExecEditAction');
	$page->getDispatcher()->setActionHandler('dmExecDelete', 'scCustomerExecDeleteAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scCustomersPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scCustomersExportCSVAction');
	
	$page->dispatchAndShow();
?>