<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	ini_set('max_execution_time', 0);
	set_time_limit(0);
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_IMPORT_EXPORT);

	class scCustomersImportView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_VIEW => 'admin/customers_import/import.tmpl',
			'importReport' => 'admin/customers_import/import_success.tmpl'
		);
	}
	
	class scCustomersImportAction extends miViewAction {
		
		/**
		 * @return miWebForm
		 */
		public function getWebForm()
		{
			if ($this->_webForm == null) {
				$this->_webForm = new miWebForm(array());
				$this->_view->initWebForm($this->_webForm);
			}
			return $this->_webForm;
		}
		
		public function doAction()
		{
			$form = $this->getWebForm();
			
			scModulesFactory::loadClassByName('scNewsletterCmsBox', 'system/newsletter/newsletter.php');
			$newsletters = scNewsletterCmsBox::getNewslettersArray();
			
			$form = $this->getWebForm();
			$widget = new miSelectWidget($form);
			$widget->setFieldName('NewsletterID');
			$widget->setOptions(array(0 => _t('Not selected')) + $newsletters);
			$widgetHtml = $widget->getEditableControl();
			
			$form->addMainPageElements(array('%%HTML_NEWSLETTERID%%' => $widgetHtml['NewsletterID']));
			
			$this->showForm($form, $this->_view->getTemplateFileName(miDefaultView::TEMPLATE_ID_VIEW), false);
			return true;
		}
	}
	
	class scExecCustomersImportAction extends scCustomersImportAction {
		
		const MAX_LINE_LENGTH = 1000000;
		
		const CUSTOMER_FIRST_NAME = 0;
		const CUSTOMER_LAST_NAME = 1;
		const CUSTOMER_EMAIL = 2;
		const CUSTOMER_GENDER = 3; // not present in Summer Cart
		const CUSTOMER_BIRTHDATE = 4; // not present in Summer Cart
		const CUSTOMER_ADDRESS_PHONE = 5;
		const CUSTOMER_CELL_PHONE = 6; // not present in Summer Cart
		const CUSTOMER_ADDRESS_LINE2 = 7;
		const CUSTOMER_USERNAME = 8; // not present in Summer Cart
		const CUSTOMER_STATUS = 9; // not present in Summer Cart
		const CUSTOMER_NEWSLETTER = 10;
		const CUSTOMER_REGISTRATION_DATE = 11; // not present in Summer Cart
		const CUSTOMER_ADDRESS_LINE1 = 12;
		const CUSTOMER_COUNTRY = 13;
		const CUSTOMER_CITY = 14;
		const CUSTOMER_GROUP = 15;
		const CUSTOMER_PASSWORD = 16;
		
		protected static $_headerMap = array(
			'Adı' => self::CUSTOMER_FIRST_NAME,			
			'Soyadı' => self::CUSTOMER_LAST_NAME,
			'Email' => self::CUSTOMER_EMAIL,
			'Cinsiyet' => self::CUSTOMER_GENDER,
			'Doğum Tarihi' => self::CUSTOMER_BIRTHDATE,
			'Telefon' => self::CUSTOMER_ADDRESS_PHONE,
			'Cep Telefonu' => self::CUSTOMER_CELL_PHONE,
			'Semt' => self::CUSTOMER_ADDRESS_LINE2,
			'Üye Adı' => self::CUSTOMER_USERNAME,
			'Durumu' => self::CUSTOMER_STATUS,
			'Mail Listesi' => self::CUSTOMER_NEWSLETTER,
			'Kayıt Tarihi' => self::CUSTOMER_REGISTRATION_DATE,
			'Adres' => self::CUSTOMER_ADDRESS_LINE1,
			'Ülke' => self::CUSTOMER_COUNTRY,
			'Şehir' => self::CUSTOMER_CITY,
			'Üye/Bayi grubu' => self::CUSTOMER_GROUP,
			'Üye Şifre(md5)' => self::CUSTOMER_PASSWORD,
		);
		
		protected static $_customerGroupNamesMap = array(
			'Üyeler' => 'Retail',
			'Ana Bayi' => 'Distributor',
			'Bayiler' => 'Distributor',
		);
		
		protected $_linesProcessed = 0;
		protected $_importedCustomers = 0;
		protected $_updatedCustomers = 0;
		
		/**
		 * The columns mapping
		 * The key is the column number in the file
		 * The value is one of the column constants
		 */
		protected $_columns = array();
		
		protected $_newsletterId;
		protected $_allowPartialSuccess = false;
		
		public function doAction()
		{
			$this->_newsletterId = (int)miGetParamDefault('NewsletterID', 0);
			$this->_allowPartialSuccess = miGetParamDefault('AllowPartialSuccess', '') != '' ? true : false;
			
			if (!isset($_FILES['CSVFile']) or empty($_FILES['CSVFile']['tmp_name'])) {
				miSetParam(miMessage::PARAM_MESSAGE, _t('No file uploaded.'));
				return parent::doAction();
			}
			
			if ($_FILES['CSVFile']['error']) {
				miSetParam(miMessage::PARAM_MESSAGE, sprintf(_t('Error during upload (error code %d).'), $_FILES['CSVFile']['error']));
				return parent::doAction();
			}
			
			$f = @fopen($_FILES['CSVFile']['tmp_name'], 'rb');
			if ($f === false) {
				miSetParam(miMessage::PARAM_MESSAGE, sprintf(_t('Cannot open file %s.'), $_FILES['CSVFile']['tmp_name']));
				return parent::doAction();
			}
			
			// if allowPartialSuccess we use per product transactions not a global one
			if (!$this->_allowPartialSuccess) {
				miStaticDBUtil::startTransaction();
			}
			try {
				$this->importCustomers($f);
				
			} catch(miException $exception) {
				
				// if allowPartialSuccess we don't have a transaction to rollback
				if (!$this->_allowPartialSuccess) {
					miStaticDBUtil::rollback();
					$msg = _t('Import failed! Reason: ');
				} else {
					// we need to build a report here
					$msg = sprintf(_t('Partial success. %d customers imported, %d customers updated. Import stopped on line #%d with message: '), $this->_importedCustomers, $this->_updatedCustomers, $this->_linesProcessed+1);
				}
				
				miSetParam(miMessage::PARAM_MESSAGE, $msg . $exception->getMessage());
				return parent::doAction();
			}
			if (!$this->_allowPartialSuccess) {
				miStaticDBUtil::commit();
			}
			
			fclose($f);
			
			// Success
			miSetParam(miMessage::PARAM_MESSAGE, _t('Import successful.'));
			miSetParam(miMessage::PARAM_MESSAGE_TYPE, miMessage::MSG_TYPE_INFO);
			$form = $this->getWebForm();
			$form->addMainPageElements(array(
				'%%IMPORTEDCUSTOMERS%%' => $this->_importedCustomers,
				'%%UPDATEDCUSTOMERS%%' => $this->_updatedCustomers,
			));
			$this->showForm($form, $this->_view->getTemplateFileName('importReport'), false);
			return true;
		}
		
		protected function importCustomers($f)
		{
			// Read the header
			$this->readHeader($f);
			
			// Read the data
			while (($data = fgetcsv($f, self::MAX_LINE_LENGTH)) !== false) {
				
				$this->_linesProcessed++;
				
				if ($this->_allowPartialSuccess) {
					miStaticDBUtil::startTransaction();
				}
				
				try {
					// check for empty line in the csv and skip that row
					if ((count($data) == 1) && is_null($data[0])) {
						if ($this->_allowPartialSuccess) {
							miStaticDBUtil::rollback();
						}
						continue;
					}
					
					$customerEmailColumn = array_search(self::CUSTOMER_EMAIL, $this->_columns);
					$customerEmail = $data[$customerEmailColumn];
					if (empty($customerEmail)) {
						throw new miException(_t('Customer email cannot be empty.'));
					}
					
					$customer = new scCustomerImportDomainObject();
					$customerId = $this->getCustomerId($customerEmail);
					if ($customerId) {
						
						$customer->read($customerId);
						
						$this->setCustomerData($customer, $data, 'update');
						
						$customer->update();
						$this->_updatedCustomers++;
						
					} else {
						
						$this->setCustomerData($customer, $data, 'insert');
						
						try {
							$customerId = $customer->insert();
							
						} catch (miException $exception) {
							// Ignore the exception only when we already have a customer with this email address in the same file
							if ($exception->getMessage() == _t('This email is already registered.')) {
								if ($this->_allowPartialSuccess) {
									miStaticDBUtil::rollback();
								}
								continue;
							}
							throw $exception;
						}
						$this->_importedCustomers++;
					}
					
					$customerAddress = new scCustomerAddress();
					$customerAddressId = $this->getCustomerAddressId($customerId);
					if ($customerAddressId) {
						
						$customerAddress->read($customerAddressId);
						$this->setCustomerAddressData($customerAddress, $data);
						
						$customerAddress->update();
						
					} else {
						
						$customerAddress->set('CustomerID', $customerId);
						$customerAddress->set('CustomerAddressIsDefaultBilling', 1);
						$customerAddress->set('CustomerAddressIsDefaultShipping', 1);
						$this->setCustomerAddressData($customerAddress, $data);
						
						$customerAddress->insert();
					}
					
					if ($this->_newsletterId) {
						// Subscribe the customer to the selected newsletter
						$newsletterColumn = array_search(self::CUSTOMER_NEWSLETTER, $this->_columns);
						if (($newsletterColumn !== false) && ($data[$newsletterColumn] == 'Aktif') && (!$this->isSubscribed($customerEmail))) {
							miStaticDBUtil::execInsert('NLSubscribers', array(
								'NewsletterID' => $this->_newsletterId,
								'CustomerID' => $customerId,
								'LanguageID' => scMultiLanguage::singleton()->getDefaultLanguageId(),
								'SubscriberEmail' => $customerEmail,
								'SubscriberIsActive' => $customer->get('CustomerStatus'),
							));
						}
					}
					
				} catch (miException $e) {
					if ($this->_allowPartialSuccess) {
						miStaticDBUtil::rollback();
					}
					throw $e;
				}
				
				if ($this->_allowPartialSuccess) {
					miStaticDBUtil::commit();
				}
			}
		}
		
		/**
		 * Read the header line of the CSV file
		 * Setups the $_columns mapping 
		 * 
		 * @param resource $f the file handler
		 * @throws miException if there is unknown header
		 */
		protected function readHeader($f)
		{
			// Read the first two empty lines
			$emptyLine = fgetcsv($f, self::MAX_LINE_LENGTH);
			$emptyLine = fgetcsv($f, self::MAX_LINE_LENGTH);
			$this->_linesProcessed = 2;
			
			// Read header
			$data = fgetcsv($f, self::MAX_LINE_LENGTH);
			foreach ($data as $key => $headerName) {
				if (!isset(self::$_headerMap[$headerName]))
					throw new miException(sprintf(_t('Unknown header "%s".'), $headerName));
				
				$this->_columns[$key] = self::$_headerMap[$headerName];
			}
			
			// Validation
			$customerEmailColumn = array_search(self::CUSTOMER_EMAIL, $this->_columns);
			if ($customerEmailColumn === false) {
				throw new miException(_t('Customer email column not found.'));
			}
		}
		
		protected function setCustomerData($customer, $data, $type)
		{
			foreach ($data as $key => $value) {
				$column = $this->_columns[$key];
				switch ($column) {
					
					case self::CUSTOMER_EMAIL:
						if ($type == 'insert') {
							$customer->set('CustomerEmail', $value);
						}
						break;
					
					case self::CUSTOMER_PASSWORD:
						$customer->set('CustomerPassword', $value);
						$customer->set('CustomerPasswordHashMode', scCustomerAuth::HASH_MODE_MD5);
						break;
					
					case self::CUSTOMER_STATUS:
						$customer->set('CustomerStatus', ($value == 'Aktif') ? scCustomerStatuses::CUSTOMER_STATUS_ACTIVE : scCustomerStatuses::CUSTOMER_STATUS_DISABLED);
						break;
					
					case self::CUSTOMER_GROUP:
						$customer->set('CustomerGroupID', $this->getCustomerGroupId($value));
						break;
				}
			}
		}
		
		protected function setCustomerAddressData($customerAddress, $data)
		{
			foreach ($data as $key => $value) {
				$column = $this->_columns[$key];
				switch ($column) {
					case self::CUSTOMER_FIRST_NAME:
						$customerAddress->set('CustomerAddressFirstName', $value);
						break;
					
					case self::CUSTOMER_LAST_NAME:
						$customerAddress->set('CustomerAddressLastName', $value);
						break;
					
					case self::CUSTOMER_ADDRESS_LINE1:
						$customerAddress->set('CustomerAddressLine1', $value);
						break;
					
					case self::CUSTOMER_ADDRESS_LINE2:
						$customerAddress->set('CustomerAddressLine2', $value);
						break;
					
					case self::CUSTOMER_CITY:
						$customerAddress->set('CustomerAddressCity', $value);
						break;
					
					case self::CUSTOMER_COUNTRY:
						$customerAddress->set('CountryID', $this->getCountryId($value));
						break;
					
					case self::CUSTOMER_ADDRESS_PHONE:
						if (!empty($value)) {
							$customerAddress->set('CustomerAddressPhone', $value);
						} else {
							// If the phone number is empty then set the cell phone (if not empty) as phone number
							$customerCellPhoneColumn = array_search(self::CUSTOMER_CELL_PHONE, $this->_columns);
							if (($customerCellPhoneColumn !== false) && (!empty($data[$customerCellPhoneColumn]))) {
								$customerAddress->set('CustomerAddressPhone', $data[$customerCellPhoneColumn]);
							}
						}
						break;
				}
			}
		}
		
		protected function getCustomerId($customerEmail)
		{
			static $customerIds;
			
			if (is_null($customerIds)) {
				$customerIds = miUtil::getDBArray('SELECT CustomerID, CustomerEmail FROM Customers', 'CustomerID', 'CustomerEmail');
			}
			
			return array_search($customerEmail, $customerIds);
		}
		
		protected function getCustomerAddressId($customerId)
		{
			static $customerAddresses;
			
			if (!$customerId) {
				return false;
			}
			
			if (is_null($customerAddresses)) {
				$addresses = miStaticDBUtil::execSelect('SELECT * FROM CustomerAddresses');
				foreach ($addresses as $address) {
					$customerAddresses[$address['CustomerID']][] = $address;
				}
			}
			
			if (empty($customerAddresses[$customerId])) {
				return false;
			}
			else if (count($customerAddresses[$customerId]) == 1) {
				return $customerAddresses[$customerId][0]['CustomerAddressID'];
			}
			else { // delete all addresses
				miStaticDBUtil::execDelete('CustomerAddresses', 'CustomerID', $customerId);
				return false;
			}
		}
		
		protected function getCustomerGroupId($importCustomerGroupName)
		{
			static $customerGroups;
			if (is_null($customerGroups)) {
				$customerGroups = scCustomerGroups::getCustomerGroupNames();
			}
			
			if (isset(self::$_customerGroupNamesMap[$importCustomerGroupName])) {
				$customerGroupName = self::$_customerGroupNamesMap[$importCustomerGroupName];
				
				$customerGroupId = array_search($customerGroupName, $customerGroups);
				if ($customerGroupId) {
					return $customerGroupId;
				}
			}
			
			throw new miException(sprintf(_t('Unknown customer group "%s".'), $importCustomerGroupName));
		}
		
		protected function getCountryId($countryName)
		{
			static $countryIds;
			
			if (!isset($countryIds[$countryName])) {
				$countryId = scUtil::getDbValue('CountriesML', 'CountryID', 'CountryName', $countryName, null);
				if (is_null($countryId)) {
					throw new miException(sprintf(_t('Country "%s" not found.'), $countryName));
				}
				$countryIds[$countryName] = $countryId;
			}
			
			return $countryIds[$countryName];
		}
		
		protected function isSubscribed($customerEmail)
		{
			static $newsletterSubscribers;
			
			if (is_null($newsletterSubscribers)) {
				$newsletterSubscribers = miUtil::getDBArray(
					'SELECT SubscriberID, SubscriberEmail FROM NLSubscribers WHERE NewsletterID = ' . $this->_newsletterId, 'SubscriberID', 'SubscriberEmail' 
				);
			}
			
			return array_search($customerEmail, $newsletterSubscribers);
		}
	}
	
	class scCustomerImportDomainObject extends scCustomerDomainObject {
		
		public function set($name, $value)
		{
			$this->_record->set($name, $value);
		}
	}
	
	$view = new scCustomersImportView();
	
	$page = new scAdminDisplayController(array($view));
	
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scCustomersImportAction');
	$page->getDispatcher()->setActionHandler('dmBack', 'scCustomersImportAction');
	$page->getDispatcher()->setActionHandler('dmExecImport', 'scExecCustomersImportAction');
	
	$page->dispatchAndShow();
?>