<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization();
	
	class scDashboardReportGenerate extends miView {
		
		public function getControllerCommands()
		{
			try {
				$reportGenerator = new scDashboardReportGenerator();
				$reportHtml = $reportGenerator->generate();
				return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $reportHtml));
			} catch (miException $e) {
				scLog::singleton()->log(scLog::SEVERITY_ERROR, $e->getMessage());
				return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $e->getMessage()));
			}
		}
	}
	
	class scDashboardReportGenerator extends scReportGenerator {
		
		public function generate()
		{
			$report = miGetParam('report');
			if ($report == 'ViewsByCategory') {
				miSetParam('NumberOfRecords', 0);
			}
			return parent::generate();
		}
		
		protected function getSettingsTemplateName()
		{
			$report = miGetParam('report');
			if (scReportUnitOfTime::isUnitOfTimeSummary()) {
				if ($report == 'ViewsByCategory') {
					return 'admin/dashboard/ampie_settings.xml';
				}
				return 'admin/dashboard/amcolumn_settings.xml';
			}
			return 'admin/dashboard/amline_settings.xml';
		}
		
		protected function displayXml($rows, $formattedRows, $columnNames)
		{
			header('Content-Type: text/xml');
			// Remove possible header and do gzip encoding for speeding up the CSV file download
			if (ob_get_level()) {
				ob_end_clean();
				header('Content-Encoding: none');
			}
			ob_start('ob_gzhandler');
			
			$report = miGetParam('report');
			
			// Find the display column name
			$reportInfo = scReportDefinitions::getReportInfo($this->_reportDefinition);
			
			// Find the value column name
			$chartValue = miGetParamDefault('ChartValue', $reportInfo[scReportDefinitions::REPORT_INFO_FIELDS][0]);
			$valueColumn = scReportDefinitions::getFieldSqlName($chartValue);
			
			if (isset($reportInfo[scReportDefinitions::REPORT_INFO_GROUP_BY_FIELD]))
				$displayColumn = scReportDefinitions::getFieldSqlName($reportInfo[scReportDefinitions::REPORT_INFO_GROUP_BY_FIELD]);
			else
				$displayColumn = $valueColumn;
			$isOneLine = !isset($reportInfo[scReportDefinitions::REPORT_INFO_GROUP_BY_FIELD]);
			
			if (scReportUnitOfTime::isUnitOfTimeSummary()) {
				if ($report == 'ViewsByCategory') {
					$this->displayXmlPie($displayColumn, $valueColumn, $formattedRows, $isOneLine);
				} else {
					$this->displayXmlColumn($displayColumn, $valueColumn, $formattedRows, $isOneLine);
				}
			} else {
				$this->displayXmlLine($displayColumn, $valueColumn, $rows, $formattedRows, $isOneLine);
			}
		}
		
		protected function displayXmlColumn($displayColumn, $valueColumn, $rows, $isOneLine)
		{
			if (empty($rows)) {
				echo '<NODATA/>';
				return;
			}
			
			return parent::displayXmlColumn($displayColumn, $valueColumn, $rows, $isOneLine);
		}
		
		protected function displayXmlPie($displayColumn, $valueColumn, $rows, $isOneLine)
		{
			if (empty($rows)) {
				// output default xml
				echo '<NODATA/>';
				return;
			}
			
			$colors = array(
				'99BCDC', 'DDB09B', 'A1C387', '9BDDDC', 'DDDC9B',
				'BD9BDD', 'CBA684', 'FEBB46', 'D6B8DA', 'FD9BD6',
			);
			
			$formattedRows = array();
			foreach ($rows as $rowKey => &$row) {
				if ($rowKey > 9) { // zero-based
					$formattedRows[9][$displayColumn] = _t('Other');
					$formattedRows[9][$valueColumn] += $row[$valueColumn];
					continue;
				}
				$row['color'] = array_shift($colors);
				$formattedRows[] = $row;
			}
			unset($row);
			
			echo '<pie>';
			foreach ($formattedRows as $rowKey => $row) {
				$attrPullOut = '';
				if ($rowKey == 0) {
					$attrPullOut = ' pull_out="true"';
				}
				
				$attrColor = ' color="#' . $row['color'] . '"';
				
				echo '<slice title="' . htmlspecialchars($row[$displayColumn]) . '"' . $attrPullOut . $attrColor . '>';
				echo htmlspecialchars($row[$valueColumn]);
				echo '</slice>';
			}
			echo '</pie>';
		}
		
		protected function displayXmlLine($displayColumn, $valueColumn, $rows, $formattedRows, $isOneLine)
		{
			if (empty($rows)) {
				// output default xml
				echo '<NODATA/>';
				return;
			}
			
			list($min, $max) = $this->getMinMaxTimes($rows);
			$timesArray = scReportUnitOfTime::getTimesArray($min, $max, scReportUnitOfTime::getUnitOfTime());
			$timesMapping = $this->reduceTimesCount($timesArray);
						
			$graphs = array();
			
			if ($isOneLine)
				$graphs[] = scReportDefinitions::getFieldName(substr($displayColumn, 1));
			else {
				foreach ($formattedRows as $row) {
					$graphs[] = $row[$displayColumn];
				}
				$graphs = array_unique($graphs);
			}
			
			$values = array();
			// fill all dates with zeroes
			$zeroValues = array_combine(array_values($timesArray), array_fill(0, count($timesArray), 0));
			foreach ($graphs as $graph) {
			 	$values[$graph] = $zeroValues;
			}
			
			// save non-zero values for each graph so we can decide which graph to remove later
			$graphMap = array_flip($graphs);
			$graphValues = array_combine(array_values($graphMap), array_fill(0, count($graphs), 0));			
			
			foreach ($formattedRows as $rowKey => $row) {
				$graph = $isOneLine ? $graphs[0] : $row[$displayColumn];
				$graphValues[$graphMap[$graph]] += $row[$valueColumn];
				// find original time in the timesArray
				$originalTime = $timesArray[$timesMapping[$row['T']]];				
				if (!is_array($values[$graph][$originalTime])) {
					$values[$graph][$originalTime] = array();
				}					
				$values[$graph][$originalTime][] = $row[$valueColumn];
			}
			
			if (count($graphs) > self::MAX_LINES_IN_CHART) {
				arsort($graphValues);
				$i = 0;
				foreach ($graphValues as $k => $v) {
					if ($i >= self::MAX_LINES_IN_CHART) {
						unset($graphs[$k]);
					}
					$i++;
				}
			}	
			
			// now arrange and calculate averages for values by time array
			$this->calcAvgValues($values);
			
			echo '<chart>';
			echo '<xaxis>';
			foreach ($timesArray as $timeValue) {
				echo '<value xid="' . htmlspecialchars($timeValue) . '">';
				echo htmlspecialchars($timeValue);
				echo '</value>';
			}
			echo '</xaxis>';

			if (scReportColumnsFormatter::isMoneyColumn(substr($valueColumn, 1)))
				list($currencyPrefix, $currencySuffix) = $this->getReportCurrency();
			else
				$currencyPrefix = $currencySuffix = '';
			
			echo '<graphs>';
			foreach ($graphs as $key => $graph) {
				$graphName = scUtf8::truncate($graph, 0, self::MAX_LEGEND_CHARS);
				echo '<graph gid="' . htmlspecialchars($key) . '" type="line" line_width="3" title="' . htmlspecialchars($graphName) .
					'" balloon_text="' . htmlspecialchars($graphName . ' (' . $currencyPrefix . '{value}' . $currencySuffix . ')') . '" bullet="round_outlined" bullet_size="10">';
				 
				foreach ($values[$graph] as $time => $value) {
					echo '<value xid="' . htmlspecialchars($time) . '">';
					echo htmlspecialchars($value);
					echo '</value>';
				}
				echo '</graph>';
			}
			echo '</graphs>';
			
			echo '</chart>';
		}
	}
	
	$page = new scAdminDisplayController(new scDashboardReportGenerate());
	$page->showPage();
?>