<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_DISCOUNTS);

	class scDiscountsView extends scView {
			
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/discounts/discount_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/discounts/discount_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/discounts/discount_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'DiscountID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'DiscountName',
				'displayName' => 'Name'
        	),
        	array(
				'field' => 'miRadioWidget',
				'data' => 'DiscountType'
        	),
        	array(
				'field' => 'miRadioWidget',
				'data' => 'DiscountApplyMode'
        	),
        	array(
				'field' => 'miSelectWidget',
				'data' => 'PaymentModuleID',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'DiscountValue',
				'validator' => array('scValidatorDecimal'),
				'displayName' => 'Value'
        	),
       		array(
				'field' => 'scGroupableSelectWidget',
				'data' => 'DiscountFreeShippingMethodID',
				'displayName' => 'Shipping Method',
       			'properties' => array('id' => 'DiscountFreeShippingMethodID'),
        	),
       		array(
				'field' => 'scDateWidget',
				'data' => 'DiscountValidFrom',
				'displayName' => 'Valid From'
        	),
       		array(
				'field' => 'scDateWidget',
				'data' => 'DiscountValidTo',
				'displayName' => 'Valid To'
        	),
       		array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'DiscountIsActive',
				'displayName' => 'Active'
        	),
			// Tab 1
       		array(
				'field' => 'scDiscountOrderSubtotalExceedsText',
				'data' => 'DiscountConditionOrderExceeds',
				'validator' => 'scValidatorDecimal'
        	),
			// Tab 2
			array(
				'field' => 'scDoubleListBoxWidget',			
				'data' => 'Categories',
				// _t('Available Categories'), _t('Selected Categories')
				'properties' => array('leftCaption' => 'Available Categories', 'rightCaption' => 'Selected Categories'),
				'modelStrategy' => 'scViewMapperDiscountCategories',
			),
			// Tab 3
        	array(
				'field' => 'scProductSelectWidget',
				'data' => 'Products',
				'modelStrategy' => 'scViewMapperDiscountProducts',
				'properties' => array('id' => 'Products')
        	),
			// Tab 4
        	array(
				'field' => 'scCustomerSelectWidget',
				'data' => 'Customers',
				'modelStrategy' => 'scViewMapperDiscountCustomers',
				'properties' => array('id' => 'Customers')
        	),
			// Tab 5
        	array(
				'field' => 'scDoubleListBoxWidget',
				'data' => 'CustomerGroups',
				// _t('Available Customer Groups'), _t('Selected Customer Groups')
				'properties' => array('leftCaption' => 'Available Customer Groups', 'rightCaption' => 'Selected Customer Groups'),
				'modelStrategy' => 'scViewMapperDiscountCustomerGroups',
        	),
        	// Tab 6
        	array(
				'field' => 'scManufacturerSelectWidget',
				'data' => 'Manufacturers',
				'modelStrategy' => 'scViewMapperDiscountManufacturers',
				'properties' => array('id' => 'Manufacturers')
        	),
        );
        
        protected $_domainObjectDisplayName = 'Discount';	// _t('Discount');
        	
		public function preProcess($actionObj)
		{
			parent::preProcess($actionObj);
			
			// Change Is Active in list
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('DiscountIsActive');
			
			if (($actionObj instanceof miExecCreateAction) || ($actionObj instanceof miExecEditAction)) {
				miStaticDBUtil::startTransaction();
			}
		}
        	
		public function postProcess($actionObj, $actionResult)
		{
			parent::postProcess($actionObj, $actionResult);
			
			if (($actionObj instanceof miExecCreateAction) || ($actionObj instanceof miExecEditAction)) {
				if (($actionResult !== false)) {
					miStaticDBUtil::commit();
				} else {
					miStaticDBUtil::rollback();
				}
			}
		}
		
        public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
			
			// Create dropdown filter types
			scUIUtil::createDropdownFilter($this, $table, 'DiscountType', array('' => _t('All discount types')) + scDiscount::getDiscountTypes(), '=', '');
		}
		
		/**
		 * @param miWebForm $form
		 */
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Set discount type
			$field = $form->getWidget('DiscountType');
			$field->setRadioButtons(scDiscount::getDiscountTypes(), scDiscount::TYPE_AMOUNT);
			
			//Set discount applies to
			$field = $form->getWidget('DiscountApplyMode');
			$field->setRadioButtons(scDiscount::getAppliesToTypes(), scDiscount::APPLY_MODE_PER_QUANTITY);
			
			// Create scDiscount object
			$discount = new scDiscount('DiscountID', @$this->getDomainObject()->get($this->getDomainObject()->getPKName()));
			
			// Initialize additional categories multi select
			$field = $form->getWidget('Categories');
			$discount->setDiscountCategoriesFieldSettings($field, 'DiscountCategories');
			
			// Initialize customer groups multi select
			$field = $form->getWidget('CustomerGroups');
			$discount->setDiscountCustomerGroupsFieldSettings($field, 'DiscountCustomerGroups');
			
			$field = $form->getWidget('DiscountFreeShippingMethodID');
			$field->setOptions(array('' => '') + scDiscount::getShippingMethodsForDropdown());
			
			$validator = new scDiscountValueValidator($form, 'DiscountType', 'DiscountValue', 'DiscountFreeShippingMethodID');
			$form->addValidator('DiscountType', $validator);
			
			$paymentModules = miUtil::getKeyValueArray(scCheckoutPaymentMethodWidget::getActivePaymentModules(), 'PaymentModuleID', 'PaymentModuleName');
			$field = $form->getWidget('PaymentModuleID');
			$field->setOptions(array('' => _t('All')) + $paymentModules);
		}
		
		/**
		 * @return scDiscountDomainObject
		 */
		protected function createDomainObject()
		{
			return new scDiscountDomainObject();
		}
	}
	
	class scDiscountOrderSubtotalExceedsText extends miBaseTextWidget {
		function getEditableControl()
		{
			$data = $this->_webForm->getFormData($this->_fieldName);
			if ($data == 0)
				$data = '';
			return array($this->_fieldName => miI18N::htmlEscape($data));
		}
		
		function getData()
		{
			if (isset($this->_properties['readOnly']) && $this->_properties['readOnly']) {
				return array();	
			}
			return array($this->_fieldName => intval(miGetParamDefault($this->_fieldName, '')));
		}
	}
	
	class scViewMapperDiscountCategories extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'DiscountCategories';
			$this->_tableKeyName = 'DiscountID';
			$this->_tableValueName = 'CategoryID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperDiscountCustomers extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'DiscountCustomers';
			$this->_tableKeyName = 'DiscountID';
			$this->_tableValueName = 'CustomerID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperDiscountCustomerGroups extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'DiscountCustomerGroups';
			$this->_tableKeyName = 'DiscountID';
			$this->_tableValueName = 'CustomerGroupID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperDiscountManufacturers extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'DiscountManufacturers';
			$this->_tableKeyName = 'DiscountID';
			$this->_tableValueName = 'ManufacturerID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperDiscountProducts extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'DiscountProducts';
			$this->_tableKeyName = 'DiscountID';
			$this->_tableValueName = 'ProductID';
			parent::__construct($model);
		}
	}
	
	class scDiscountDomainObject extends miDefaultDomainObject {
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('Discounts', 'DiscountID'));
		}
		
		public function update()
		{
			$this->prepareData();
			
			return parent::update();
		}
		
		public function insert()
		{
			$this->prepareData();
			
			return parent::insert();
		}
		
		protected function prepareData()
		{
			if ($this->has('PaymentModuleID') && ($this->get('PaymentModuleID') == 0)) {
				$this->set('PaymentModuleID', null);
			}
			
			$this->prepareDataForType();
			$this->prepareDataForApplyMode();
		}
		
		protected function prepareDataForType()
		{
			$discountType = $this->get('DiscountType');
			if ($discountType == scDiscount::TYPE_FREE_SHIPPING) {
				$this->set('DiscountValue', 0);
				$this->set('DiscountApplyMode', scDiscount::APPLY_MODE_TO_SUBTOTAL);
				$this->set('DiscountImplementation', scDiscount::IMPLEMENTATION_ORDER_TOTAL);
				$this->set('PaymentModuleID', null);
			} else {
				$this->set('DiscountFreeShippingMethodID', null);
			}
		}
		
		protected function prepareDataForApplyMode()
		{
			$applyMode = $this->get('DiscountApplyMode');
			if ($applyMode == scDiscount::APPLY_MODE_TO_SUBTOTAL) {
				$this->set('DiscountImplementation', scDiscount::IMPLEMENTATION_ORDER_TOTAL);
				$this->set('PaymentModuleID', null);
			} else {
				$this->set('DiscountImplementation', scDiscount::IMPLEMENTATION_PRODUCT_PRICE_MODIFIER);
			}
		}
	}
	
	$discounts = new miSqlRecordset('Discounts');
	
	$table = new scDiscountsTable($discounts);
	$table->setDateFromFieldName('DiscountValidFrom');
	$table->setDateToFieldName('DiscountValidTo');
	$table->setTypeFieldName('DiscountType');
	
	$view = new scDiscountsView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>