<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_DYNAMIC_PAGES);

	class scDynamicPagesView extends scView {
		var $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/dynamic_pages/dynamic_page_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/dynamic_pages/dynamic_page_edit.tmpl',
		);
		
		var $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSDynamicPageID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ModuleID',
 				'properties' => array('readOnly' => true)
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSDynamicPageName',
 				'properties' => array('readOnly' => true)
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CMSDynamicPageTitle',
				'validator' => 'scValidatorNoEmpty' 
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSDynamicPagePath',
 				'properties' => array('readOnly' => true),
        	),
        	array(
				'field' => 'scMultipleCMSTextareaWidget',
				'data' => 'CMSDynamicPageAreas',
				'modelStrategy' => 'scViewMapperDynamicPageAreas',
        	),
        	array(
				'field' => 'scCMSLayoutSelect',
				'data' => 'CMSLayoutID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CMSDynamicPageSEOTitle'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'CMSDynamicPageSEOKeywords'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'CMSDynamicPageSEODescription'
        	),
        	array(
				'field' => 'scDynamicPageCustomUrlsWidget',
				'data' => 'CustomUrls',
				'modelStrategy' => 'scViewMapperDynamicPageCustomUrls',
				'properties' => array(
					'template' => 'admin/custom_urls/custom_url.tmpl',
					
					'table' => 'CustomUrls',
					'foreignKey' => 'CustomUrlEntityID',
					'foreignTablePrimaryKey' => 'CustomUrlID',
					
					'fields' => array(
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CustomUrlText',
			       			'properties' => array('minLength' => 1),
			       			'validator' => 'scValidatorUniqueCustomUrl'
			        	),
			       		array(
							'field' => 'miCheckboxWidget',
							'data' => 'CustomUrlIsCurrent'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomUrlID'
			        	)
			        ),
			        'mlFields' => array('CustomUrlText')
				),
			)
        );
        
        protected $_domainObjectDisplayName = 'Dynamic page';	// _t('Dynamic page');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$form->addMainPageElements(array(
				'%%CKEDITOR_CONTENTCSS%%' => scSkins::singleton()->getCurrentSkin()->getCssFilesForWysiwygEditor(),
				'%%CKEDITOR_LANGUAGE%%' => scCMS::getCkEditorLanguage()
			));
		}
		
		protected function createDomainObject()
		{
			return new scDynamicPageDomainObject();
		}
	}
	
	class scDynamicPageDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new scSqlRecordML('CMSDynamicPages', 'CMSDynamicPageID',
				array('CMSDynamicPageTitle', 'CMSDynamicPageSEOTitle', 'CMSDynamicPageSEOKeywords', 'CMSDynamicPageSEODescription'),
				array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
	}
	
	class scDynamicPageCustomUrlsWidget extends scCustomUrlsWidget {
		
		public function getEditableControl()
		{
			// XXX: This way we will have access to the data even on exec edit with errors
			global $view;
			$page = $view->getDomainObject()->get('CMSDynamicPagePath');
//			$page = $this->_webForm->getFormData('CMSDynamicPagePath');

			if (!in_array($page, scFriendlyUrl::getLinkableDynamicPages())) {
				return array($this->_fieldName => '');
			}
			return parent::getEditableControl();
		}
	}
	
	class scViewMapperDynamicPageCustomUrls extends scViewMapperCustomUrls {
		protected $_entityCode = scFriendlyUrl::TYPE_DYNAMIC_PAGE;
		protected $_tableKeyName = 'CMSDynamicPagePath';
		
		public function read()
		{
			$page = $this->_domainObject->get('CMSDynamicPagePath');
			if (!in_array($page, scFriendlyUrl::getLinkableDynamicPages())) {
				return;
			}
			return parent::read();
		}
		
		public function update()
		{
			$page = $this->_domainObject->get('CMSDynamicPagePath');
			if (!in_array($page, scFriendlyUrl::getLinkableDynamicPages())) {
				return;
			}
			
			return parent::update();
		}
		
		protected function getEntityId()
		{
			$page = $this->_domainObject->get('CMSDynamicPagePath');
			$pages = scFriendlyUrl::getLinkableDynamicPages();
			return array_search($page, $pages);
		}
	}
	
	$dynamicPages = new scSqlRecordsetML('CMSDynamicPages', 'CMSDynamicPageID');
	$table = new scTable($dynamicPages);
	
	$view = new scDynamicPagesView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->dispatchAndShow();
?>