<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_EMAILS);

	class scEmailsView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_EDIT => 'admin/emails/emails_edit.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_ADMINISTRATOR
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_ORDER_DEPARTMENT
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_HELP_SUPPORT
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_REPLY_TO_NOTIFICATIONS
        		),
			array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_REPLY_TO_NEWSLETTER
        		),
       		array(
				'field' => 'miBaseSelectWidget',
				'data' => scSettings::SETTING_EMAIL_METHOD_OF_SENDING
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_SMTP_HOST
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_SMTP_PORT,
       			'properties' => array('defaultValue' => 25),
        		),
       		array(
				'field' => 'miRadioWidget',
				'data' => scSettings::SETTING_EMAIL_SMTP_SECURITY,
        		),
       		array(
				'field' => 'miBaseCheckboxWidget',
				'data' => scSettings::SETTING_EMAIL_USE_SMTP_AUTHENTICATION
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_SMTP_USERNAME
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => scSettings::SETTING_EMAIL_SMTP_PASSWORD
        		),
        );
        
        protected $_domainObjectDisplayName = 'Emails';	// _t('Emails');
                
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Set options Method Of Sending Emails
			$optionsMethodOfSendingEmails[scEmail::METHOD_SENDING_EMAILS_VIA_PHP] = _t('via php mail() fuction');
			$optionsMethodOfSendingEmails[scEmail::METHOD_SENDING_EMAILS_VIA_SMTP_SERVER] = _t('via SMTP server');
			
			$field = $form->getWidget(scSettings::SETTING_EMAIL_METHOD_OF_SENDING);
			$field->setOptions($optionsMethodOfSendingEmails, scEmail::METHOD_SENDING_EMAILS_VIA_PHP);
			
			$optionsMethodOfSmtpSecurity[scEmail::SMTP_SECURITY_NONE] = _t('None');
			$optionsMethodOfSmtpSecurity[scEmail::SMTP_SECURITY_SSL] = _t('SSL');
			$optionsMethodOfSmtpSecurity[scEmail::SMTP_SECURITY_TLS] = _t('TLS');
			
			$field = $form->getWidget(scSettings::SETTING_EMAIL_SMTP_SECURITY);
			$field->setRadioButtons($optionsMethodOfSmtpSecurity, scEmail::SMTP_SECURITY_NONE);
			
			$msgTypeCondition = (miGetParamDefault(miMessage::PARAM_MESSAGE_TYPE, '')  == miMessage::MSG_TYPE_WARNING);
			$sendViaCondition = (scSettings::getSettingValue(scSettings::SETTING_EMAIL_METHOD_OF_SENDING) == scEmail::METHOD_SENDING_EMAILS_VIA_SMTP_SERVER);
			$showLogLink = ($msgTypeCondition && $sendViaCondition);
			
			$form->addMainPageElements(array(
				'%%LINK_TO_LOG_CLASS%%' => $showLogLink ? '' : 'hidden',
				'%%SMTPMETHOD%%' => scEmail::METHOD_SENDING_EMAILS_VIA_SMTP_SERVER
			));
		}
	}

	class scExecEditAndSendSampleEmail extends miExecEditAction {
		public function doAction()
		{
			// save the data
			$result = parent::doAction();
			if ($result === false) {
				return $false;
			}
			
			scSettings::setSettingValue(scSettings::SETTING_EMAIL_METHOD_OF_SENDING, miGetParam(scSettings::SETTING_EMAIL_METHOD_OF_SENDING));
			scSettings::setSettingValue(scSettings::SETTING_EMAIL_SMTP_HOST, miGetParam(scSettings::SETTING_EMAIL_SMTP_HOST));
			scSettings::setSettingValue(scSettings::SETTING_EMAIL_SMTP_PORT, miGetParam(scSettings::SETTING_EMAIL_SMTP_PORT));
			scSettings::setSettingValue(scSettings::SETTING_EMAIL_USE_SMTP_AUTHENTICATION, miGetParamDefault(scSettings::SETTING_EMAIL_USE_SMTP_AUTHENTICATION, 0));
			scSettings::setSettingValue(scSettings::SETTING_EMAIL_SMTP_SECURITY, miGetParam(scSettings::SETTING_EMAIL_SMTP_SECURITY));
			
			// These fields might be disabled in the form so use miGetParamDefault 
			scSettings::setSettingValue(scSettings::SETTING_EMAIL_SMTP_PASSWORD, miGetParamDefault(scSettings::SETTING_EMAIL_SMTP_PASSWORD, ''));
			scSettings::setSettingValue(scSettings::SETTING_EMAIL_SMTP_USERNAME, miGetParamDefault(scSettings::SETTING_EMAIL_SMTP_USERNAME, ''));
			
			scSettings::setSettingValue(scSettings::SETTING_EMAIL_ADMINISTRATOR, miGetParam(scSettings::SETTING_EMAIL_ADMINISTRATOR));
			
			// prepare the sample email
			$to = new scEmailAddress(scSettings::getSettingValue(scSettings::SETTING_EMAIL_ADMINISTRATOR));
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/emails/test_email.tmpl');
			$t->assignArray(array(
				'%%COMPANYNAME%%' => scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME),
				'%%STORE_ROOT_URL%%' => 'http://' . scPaths::getHttpHost() . scPaths::getStoreWebPath(),
			));
			
			// try to send the email
			$email = new scEmail();
			$email->setTo($to);
			$email->setSubject(_t('Summer Cart test email'));
			$result = $email->sendHtmlEmail($t->templateParse());

			// report
			if ($result === false) {
				$msg = _t('Data updated but couldn\'t send email to %s. Please check the setup.');
				$msgtype = miMessage::MSG_TYPE_WARNING;
			} else {
				$msg = _t('Data updated and email was successfully sent to %s.');
				$msgtype = miMessage::MSG_TYPE_INFO;
			}
			
			$url = 'emails.php?' . miMessage::PARAM_MESSAGE_TYPE . '=' . $msgtype . '&' . miMessage::PARAM_MESSAGE . '=' . urlencode(sprintf($msg, $to));
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url);
			$this->_view->addControllerCommand($cmd);
			return $result;
		}
	}
	
	miSetParam('', 0);
	
	$emails = new miSqlRecordset('Settings');
	$table = new miTable($emails);
	
	$view = new scEmailsView();
	$view->setRecord(new miPropertiesRecord('Settings', 'SettingName', 'SettingValue'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'miEditAction');
	$page->getDispatcher()->setActionHandler('dmExecEdit', 'miExecEditAction');
	$page->getDispatcher()->setActionHandler('dmExecEditAndSendSampleEmail', 'scExecEditAndSendSampleEmail');
	
	$page->dispatchAndShow();
?>