<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_FEATURED_PRODUCTS);

	class scFeaturedProductsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/featured_products/featured_product_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/featured_products/featured_product_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/featured_products/featured_product_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'FeaturedProductID',
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductID',
				'validator' => array('scValidatorNoEmpty', 'scValidatorValueExists'),
				'properties' => array('existsIn' => array('Products', 'ProductID')),
				'displayName' => 'Product Name'
        	),
       		array(
				'field' => 'miCheckboxWidget',
				'data' => 'FeaturedProductOnHome'
        	),
			array(
				'field' => 'scDoubleListBoxWidget',			
				'data' => 'Categories',
				// _t('Available Categories'), _t('Selected Categories')
				'properties' => array('leftCaption' => 'Available Categories', 'rightCaption' => 'Selected Categories'),
				'modelStrategy' => 'scViewMapperFeaturedProductsCategories',
			),
        );
        
        protected $_domainObjectDisplayName = 'Featured product';	// _t('Featured product');
        
        var $_defaultFilterValues = array(
        	'ProductName' => '',
        );
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());

			// Create dropdown filter for category
			$dummyForm = new miWebForm(array());
			$categorySelectWidget = new scCategorySelect($dummyForm, array());
			$categoriesArray = $categorySelectWidget->getCategoriesTree(0, true);
			$categoriesArray = array('' => _t('Featured in'), '-1' => _t('HOME')) + $categoriesArray;
			scUIUtil::createDropdownFilter($this, $table, 'CategoryID', $categoriesArray, 'featuredIn', '');
			$this->getTableFilterObj()->setConditionHandler('featuredIn', 'scFeaturedInCategoryFilter');
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);

			// Initialize categories multi select
			$field = $form->getWidget('Categories');
			$field->setMode(scDoubleListBoxWidget::MODE_COPY_VALUES);
			$field->setLeftToRightJsHandler('SC.DMS.productCategoryLeftToRight');
			
			$dummyForm = new miWebForm(array());
			$categorySelectWidget = new scCategorySelect($dummyForm, array());
			$categoriesArray = $categorySelectWidget->getCategoriesTree(0, true);
			$field->setOptions($categoriesArray);
			$field->setSelectedItemsTextFilter(create_function('$name', 'return preg_replace("/.*▢/", "", $name);'));
			
			if (miGetParamDefault('FeaturedProductID', false) !== false) {
				$productName = scUtil::getMLDbValue('ProductsML', 'ProductName', 'ProductID', $this->getRecord()->get('ProductID'));
				$form->addMainPageElements(array('%%PRODUCTNAME%%' => miI18N::htmlEscape($productName)));
			}
			
			$form->addValidator('ProductID', new scValidatorUniqueFields($form, array('ProductID'), $this->getRecord(), _t('This product is already featured.')));
		}
	}
	
	class scViewMapperFeaturedProductsCategories extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'FeaturedProductCategories';
			$this->_tableKeyName = 'FeaturedProductID';
			$this->_tableValueName = 'CategoryID';
			parent::__construct($model);
		}
	}

	/**
	 * Custom recordset to get featured products
	 */
	class scFeaturedProductsRecordset extends miSqlRecordset {
		public function __construct()
		{
			parent::__construct('');
		}
		
		public function getSelectQuery()
		{
			$filterClause = $this->getSqlFilterClause();
			if ($filterClause != '')
				$filterClause = ' AND ' . $filterClause;

			// Get all products with records in FeaturedProducts or those that have ProductHomePageFeatured set
			$query = 'SELECT FeaturedProducts.*, ProductsML.ProductName, GROUP_CONCAT(FeaturedProductCategories.CategoryID) as Categories' .
						' FROM FeaturedProducts LEFT JOIN FeaturedProductCategories USING(FeaturedProductID)' .
						' JOIN Products ON FeaturedProducts.ProductID=Products.ProductID' .
						' JOIN ProductsML ON Products.ProductID=ProductsML.ProductID' .
						' WHERE ProductsML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId() .
						$filterClause . ' GROUP BY FeaturedProductID ' . $this->getSqlOrderClause();
			return $query;
		}
		
		public function getRecordsCount()
		{
			$query = 'SELECT COUNT(*) AS N FROM ('.$this->getSelectQuery().') SubQuery';
			$rows = miStaticDBUtil::execSelect($query);
			return (int)$rows[0]['N'];
		}
		
		public function getAllFields($includeTablePrefixed = false)
		{
			$this->_allFields = array('CategoryID', 'ProductName', 'FeaturedProductSort');
			return $this->_allFields;
		}
	}

	/**
	 * Category filter
	 */
	class scFeaturedInCategoryFilter extends miSqlFilterSimple {
		public function getSql()
		{
			if ($this->_value == '') {
				return '';	
			}
			// Show only products featured on home page
			if ($this->_value == -1) {
				return 'FeaturedProductOnHome=1';
			}
			
			return 'FeaturedProductCategories.CategoryID="' . sql_escape_string($this->_value) . '"';
		}
	}
	
	class scCreateFeaturedProductAction extends miCreateAction {
		public function doAction()
		{
			$rows = miStaticDBUtil::execSelect('SELECT COUNT(*) AS ProductsCount FROM Products');
			if (($rows[0]['ProductsCount']) == 0) {
				$msg = _t('Create a product first.');
				$this->_view->addRedirectToListControllerCommand($msg);
				return false;
			}
			parent::doAction();
		}
	}
	
	$featuredProducts = new scFeaturedProductsRecordset();
	$table = new scFeaturedProductsTable($featuredProducts);
	
	$view = new scFeaturedProductsView();
	$view->setUpSortColumn('FeaturedProductSort');
	$view->setRecord(new miSqlRecord('FeaturedProducts', 'FeaturedProductID'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmCreate', 'scCreateFeaturedProductAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>