<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_FORMS);

	class scCMSFormsView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/forms/form_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/forms/form_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/forms/form_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSFormID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSFormName',
				'validator' => 'scValidatorNoEmpty'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CMSFormTitle',
				'displayName' => 'Title'
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSFormEmail',
				'validator' => array('scValidatorNoEmpty', 'scValidatorEmail'),
				'displayName' => 'Email'
        	),
       		array(
				'field' => 'scWebFormWidgetTextareaML',
				'data' => 'CMSFormDescription',
				'displayName' => 'Description'
        	),
       		array(
				'field' => 'scWebFormWidgetTextareaML',
				'data' => 'CMSFormConfirmText',
				'displayName' => 'Confirm text'
        	),
        	array(
				'field' => 'miCheckboxWidget',
				'data' => 'CMSFormEnableCaptcha',
				'displayName' => 'Enable captcha'
        	),
        	array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CMSFormSEOTitle',
				'displayName' => 'SEO Title'
        	),
        	array(
				'field' => 'scFormItemsWidget',
				'data' => 'CMSFormItems',
				'modelStrategy' => 'scViewMapperFormItems',
				'properties' => array(
					'foreignKey' => 'CMSFormID',
					
					'table' => 'CMSFormItems',
					'foreignTablePrimaryKey' => 'CMSFormItemID',
					
					'valueTable' => 'CMSFormItemValues',
					'valueKey' => 'CMSFormItemValueID',
					'valueType' => 'CMSFormItemType',
					'valueField' => 'CMSFormItemValue',
        			
        			'sortField' => 'CMSFormItemSort',
        			'sortBy' => 'CMSFormItemSort',
        			
					'template' => 'admin/forms/form_item.tmpl',
					'fields' => array( 
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CMSFormItemID'
			        	),
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CMSFormItemName'
			        	),
			       		array(
							'field' => 'scWebFormFieldSelect',
							'data' => 'CMSFormItemType'
			        	),
			       		array(
							'field' => 'miBaseCheckboxWidget',
							'data' => 'CMSFormItemIsActive',
							'properties' => array('defaultValue' => true)
			        	),
			       		array(
							'field' => 'scWebFormWidgetTextareaML',
							'data' => 'CMSFormItemCustomHtml',
			        	),
			       		array(
							'field' => 'miBaseCheckboxWidget',
							'data' => 'CMSFormItemIsRequired'
			        	),
			        	array(
			        		'field' => 'miBaseTextWidget',
			        		'data' => 'CMSFormItemValues',
			        		'properties' => array('defaultValue' => '')
			        	)
			        ),
			        'mlFields' => array('CMSFormItemName', 'CMSFormItemCustomHtml') // Multi-language fields
				)
        	),
        	array(
				'field' => 'scCustomUrlsWidget',
				'data' => 'CustomUrls',
				'modelStrategy' => 'scViewMapperFormCustomUrls',
				'properties' => array(
					'template' => 'admin/custom_urls/custom_url.tmpl',
					
					'table' => 'CustomUrls',
					'foreignKey' => 'CustomUrlEntityID',
					'foreignTablePrimaryKey' => 'CustomUrlID',
					
					'fields' => array(
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CustomUrlText',
			       			'properties' => array('minLength' => 1),
			       			'validator' => 'scValidatorUniqueCustomUrl'
			        	),
			       		array(
							'field' => 'miCheckboxWidget',
							'data' => 'CustomUrlIsCurrent'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomUrlID'
			        	)
			        ),
			        'mlFields' => array('CustomUrlText')
				),
			)
        );
		
        protected $_domainObjectDisplayName = 'Form';	// _t('Form');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);

			$itemTypesWithValues = scFormItem::singleton()->getItemTypesWithValues();
			$this->addMainPageElements(array(
				'%%ITEMSWITHVALUES%%' => miI18N::htmlEscape(implode(',', $itemTypesWithValues)),
				'%%ITEMCUSTOMHTML%%' => scFormItem::ITEM_TYPE_CUSTOM_HTML,
			));
			
			// Add ML
			$this->addMainPageElements(array(
				'%%HTML_DATA_LANGUAGES%%' => $this->getDataLanguagesHtml(false),
				'%%LANGUAGEID%%' => scView::getCurrentDataLanguageId()
			));
			
			$form->addMainPageElements(array(
				'%%CKEDITOR_CONTENTCSS%%' => scSkins::singleton()->getCurrentSkin()->getCssFilesForWysiwygEditor(),
				'%%CKEDITOR_LANGUAGE%%' => scCMS::getCkEditorLanguage()
			));
		}
		
		protected function createDomainObject()
		{
			return new scFormDomainObject();
		}
	}
	
	class scFormDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new scSqlRecordML('CMSForms', 'CMSFormID',
				array('CMSFormTitle', 'CMSFormDescription', 'CMSFormConfirmText', 'CMSFormSEOTitle'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
	}
	
	class scViewMapperFormItems extends scViewMapperItemsWithValues {
		
		protected function getTypesWithValues()
		{
			return scFormItem::singleton()->getItemTypesWithValues();
		}
		
		protected function updateRecordValues($record, $rsValues, $rsProperties, $rowIndex)
		{
			$itemId = $record->getPK();
			// Save form item values
			if (!in_array($record->get($rsProperties['valueType']), $this->getTypesWithValues()))
				return;
			
			$itemValueIds = array();
			foreach ($rsValues[$rowIndex]['CMSFormItemsValues'] as $key => $itemValues) {
				if (trim(implode($itemValues)) == '')
					continue;
				$valueId = '';
				if (isset($rsValues[$rowIndex][$rsProperties['valueKey']][$key])) {
					$valueId = $rsValues[$rowIndex][$rsProperties['valueKey']][$key];
				}
				if (!empty($valueId)) {
					$itemValueIds[] = $this->updateItemValues($rsProperties, $itemId, $itemValues, $key, $valueId);
				} else {
					$itemValueIds[] = $this->insertItemValues($rsProperties, $itemId, $itemValues, $key);
				}
			}
			
			$deleteQuery = 'DELETE FROM ' . $rsProperties['valueTable'] . ' WHERE ' . $rsProperties['foreignTablePrimaryKey'] . ' = ' . sql_escape_string($itemId);
			if (!empty($itemValueIds)) {
				$itemValueIds = array_map('intval', $itemValueIds);
				$deleteQuery .= ' AND ' . $rsProperties['valueKey'] . ' NOT IN (' . implode(', ', $itemValueIds) . ')';
			}
			miStaticDBUtil::execSQL($deleteQuery);
		}
		
		protected function addValuesToRecords($rsProperties, $records, $values)
		{
			foreach ($records as &$record)
				$record['CMSFormItemsValues'] = array();
			foreach ($values as $row) {
				$key = $this->findRecord($rsProperties, $records, $row['CMSFormItemID']);
				if ($key === null)
					continue;
				if (isset($records[$key]['CMSFormItemsValues'][$row['CMSFormItemValueID']]))
					$records[$key]['CMSFormItemsValues'][$row['CMSFormItemValueID']][$row['LanguageID']] = $row['CMSFormItemValue'];
				else
					$records[$key]['CMSFormItemsValues'][$row['CMSFormItemValueID']] = array($row['LanguageID'] => $row['CMSFormItemValue']);
			}
			return $records;
		}
		
		protected function insertRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			$itemId = parent::insertRecord($record, $rsValues, $rsProperties, $rowIndex);
			
			// Save form item values
			if (!in_array($record->get('CMSFormItemType'), scFormItem::singleton()->getItemTypesWithValues()))
				return $itemId;
			
			foreach ($rsValues[$rowIndex]['CMSFormItemsValues'] as $cmsFormItemValues) {
				if (trim(implode($cmsFormItemValues)) == '')
					continue;
				
				$itemValueId = miStaticDBUtil::execInsert('CMSFormItemValues', array('CMSFormItemID' => $itemId));
				
				foreach ($cmsFormItemValues as $languageId => $cmsFormItemValue) {
					$values = array (
						'CMSFormItemValueID' => $itemValueId,
						'LanguageID' => $languageId,
						'CMSFormItemValue' => $cmsFormItemValue
					);
					miStaticDBUtil::execInsert('CMSFormItemValuesML', $values);
				}
			}
			
			return $itemId;
		}
	}
	
	class scViewMapperFormCustomUrls extends scViewMapperCustomUrls {
		protected $_entityCode = scFriendlyUrl::TYPE_DYNAMIC_FORM;
		protected $_tableKeyName = 'CMSFormID';
	}
	
	$CMSForms = new scSqlRecordsetML('CMSForms', 'CMSFormID');
	$table = new scTable($CMSForms);
	
	$view = new scCMSFormsView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>