<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_GALLERIES);
	scDateTime::setTimezone();
	
	class scGalleryView extends scView {
		
		const TEMPLATE_ID_LIST_ITEMS = 'listItem';
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/galleries/galleries_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/galleries/galleries_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/galleries/galleries_create.tmpl',
			
			self::TEMPLATE_ID_LIST_ITEMS => 'admin/gallery_items/gallery_item_mgmt.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'GalleryID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'GalleryName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name'
        	),
			array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'GalleryTitle',
				'displayName' => 'Title'
        	),
        	array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'GalleryDescription',
				'displayName' => 'Title'
        	),
        	array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'GallerySEOTitle',
				'displayName' => 'SEO Title'
        	),
			array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'GalleryIsActive'
        	),
        	array(
				'field' => 'scCustomUrlsWidget',
				'data' => 'CustomUrls',
				'modelStrategy' => 'scViewMapperGalleryCustomUrls',
				'properties' => array(
					'template' => 'admin/custom_urls/custom_url.tmpl',
					
					'table' => 'CustomUrls',
					'foreignKey' => 'CustomUrlEntityID',
					'foreignTablePrimaryKey' => 'CustomUrlID',
					
					'fields' => array(
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CustomUrlText',
			       			'properties' => array('minLength' => 1),
			       			'validator' => 'scValidatorUniqueCustomUrl'
			        	),
			       		array(
							'field' => 'miCheckboxWidget',
							'data' => 'CustomUrlIsCurrent'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomUrlID'
			        	)
			        ),
			        'mlFields' => array('CustomUrlText')
				),
			)
        );
        
        protected $_domainObjectDisplayName = 'Gallery';	// _t('Gallery');
        
		public function preProcess($actionObj)
		{
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('GalleryIsActive');
		}
		
        public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
        
		protected function createDomainObject()
		{
			return new scGalleryDomainObject();
		}
	}
	
	class scViewMapperGalleryCustomUrls extends scViewMapperCustomUrls {
		protected $_entityCode = scFriendlyUrl::TYPE_GALLERY;
		protected $_tableKeyName = 'GalleryID';
	}
	
	/**
	 * Show the page for updating a record in the database
	 */
	class scGalleryEditAction extends miEditAction {
		public function doAction()
		{
			try {
				$galleryId = (int)miGetParam('GalleryID');
				$galleryItems = new scSqlRecordsetML('GalleryItems', 'GalleryItemID', 
					array('GalleryItemName', 'GalleryItemDescription'), array_keys(scMultiLanguage::getActiveLanguageCodes()));
				$galleryItems->addFilter(new miSqlFilterEqual('GalleryID', $galleryId));
				$galleryItems->setLanguageId(scMultiLanguage::getCurrentLanguageId());
				$galleryItems->setOrder('GalleryItemSort', 'ASC');
				
				$table = new scGalleryItemsTable($galleryItems);
				$table->setSortingState(1);
				$table->assign('%%GALLERYID%%', $galleryId);
				$table->assign('%%LANGUAGEID%%', scView::getCurrentDataLanguageId());
				$galleryItemsHtml = $table->parse($this->_view->getTemplateFileName(scGalleryView::TEMPLATE_ID_LIST_ITEMS));
				
				$this->_view->addMainPageElements(array('%%HTML_GALLERYITEMS%%' => $galleryItemsHtml));
				
				return parent::doAction();
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_LIST_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	$galleries = new scSqlRecordsetML('Galleries', 'GalleryID');
	$galleries->setSelectFields(array('GalleriesML.*', 'Galleries.*'));
	$table = new scTable($galleries);
	
	$view = new scGalleryView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmEdit', 'scGalleryEditAction');
	
	$page->dispatchAndShow();
?>