<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_GALLERIES);
	scDateTime::setTimezone();
	
	class scGalleryImportView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_VIEW => 'admin/galleries/gallery_import.tmpl',
		);
		
		public function addRedirectToGalleryControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = 'galleries.php?action=dmEdit&GalleryID=' . miGetParam('GalleryID') . '&' . $this->getRedirectParams($msg, $msgType);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
		
		public function addRedirectToListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = 'gallery_import.php?action=&GalleryID=' . miGetParam('GalleryID') . '&' . $this->getRedirectParams($msg, $msgType);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
	}
	
	class scGalleryImportViewAction extends miAction {

		public function doAction()
		{
			$t = new scTemplateParser();
			
			$template = $this->_view->getTemplateFileName(scGalleryImportView::TEMPLATE_ID_VIEW);
			$t->readTemplate($template);
			
			$galleryId = miGetParamDefault('GalleryID','');
			$galleryInfo = self::getGalleryInfo($galleryId);
			// Add breadcrumb
			$breadcrumb = new scBreadcrumb();
			$breadcrumbHtml = $breadcrumb->getCurrentBreadcrumbHtml();
			
			// Remove link to current page in the breadcrumb
			$breadcrumbHtml = preg_replace('/href=\"[.\/]*\/admin\/gallery_import\.php\?action\=dmBack/', '', $breadcrumbHtml);
			
			$t->assignArray(array('%%HTML_BREADCRUMB%%' => $breadcrumbHtml));
			
			$t->assignArray(
				array(
					'%%DIRECTORY%%' => scPaths::getPath(scPaths::RESOURCE_GALLERY_IMAGE) . $galleryInfo['GalleryDirName'],
					'%%GALLERYNAME%%' => $galleryInfo['GalleryName'],
					'%%GALLERYID%%' => $galleryId,
				)
			); 
			
			$msg = new miMessage();
			$t->assignArray($msg->getMessageTemplateVars());
			
			$images = $this->readImageDirectory($galleryInfo['GalleryDirName']);
			$t->addSectionInfo(new miTemplateParserSectionInfo('imageRows', count($images), miUIUtil::templetizeRows(miUIUtil::transposeRows($images))));
			
			$html = $t->templateParse();
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;
		}
		
		public static function getGalleryInfo($galleryId)
		{
			$result = miStaticDBUtil::execSelect('SELECT * FROM Galleries WHERE GalleryID = ?', array($galleryId));
			if (empty($result))
				throw new miException(sprintf(_i('Cannot find gallery id "%s".'), $galleryId));
			return $result[0];
		}
		
		protected function readImageDirectory($galleryDirName)
		{
			$arrImages = Array();
			$rootDir = scPaths::getStoreRootDir();
			$imageDir = $rootDir . scPaths::getPath(scPaths::RESOURCE_GALLERY_IMAGE) . $galleryDirName . '/';
			$imageWebDir = scPaths::getStoreWebPath() . scPaths::getPath(scPaths::RESOURCE_GALLERY_IMAGE) . $galleryDirName . '/';
			try {
				if (!($handle = @opendir($imageDir)))
					throw new miException(sprintf(_i('Error opening directory "%s".'), $imageDir));
				$counter = 0;
				while (($file = readdir($handle)) !== false) {
					if (is_dir($imageDir . $file))
						continue;
					$arrImages[$counter]['GalleryItemFileName'] = $imageDir . $file;
					$arrImages[$counter]['imageWebFileName'] = $imageWebDir . $file;
					$arrImages[$counter]['imageName'] = preg_replace('/\.(jpg|png|gif)$/i','', $file);
					$arrImages[$counter]['imageNum'] = $counter; 
					$counter++;
				}
				closedir($handle);
			} catch (miException $exception) {
				if (miGetParamDefault('msg', '') == '') {
					$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
					return false;
				}
			}
			
			return $arrImages;
		}
	}
	
	class scGalleryExecImportAction extends miExecEditAction{
		
		private $_currentMaxSort = false;
		
		public function doAction()
		{		
			try {
				miStaticDBUtil::startTransaction();
				
				$selectedImages = miGetParamDefault('checkboxImage', '');
				$galleryId = (int)miGetParamDefault('GalleryID', 0);
				$galleryInfo = scGalleryImportViewAction::getGalleryInfo($galleryId);
				$deleteBeforeImport = (bool)miGetParamDefault('deleteBeforeImport', false);
				
				if (!is_array($selectedImages) || empty($selectedImages))
					throw new miException(_t('No images selected.'));
				
				$imageDir = scPaths::getStoreRootDir() . scPaths::getPath(scPaths::RESOURCE_GALLERY_IMAGE) . $galleryInfo['GalleryDirName'] . '/';
				if (!($handle = @opendir($imageDir)))
					throw new miException(sprintf(_i('Error opening directory "%s".'), $imageDir));
				
				if ($deleteBeforeImport) {
					$gallery = new scGalleryDomainObject();
					$gallery->emptyGalleries(array($galleryId));
					unset($gallery);
				}
				
				// Create the destination dirs, if needed
				$this->createImageDir(scPaths::getFilenamePath($galleryInfo['GalleryDirName'] . '/' . scPaths::GALLERY_LARGE_IMAGES, scPaths::RESOURCE_GALLERY_IMAGE, scPaths::PATH_FILE));
				$this->createImageDir(scPaths::getFilenamePath($galleryInfo['GalleryDirName'] . '/' . scPaths::GALLERY_THUMB_IMAGES, scPaths::RESOURCE_GALLERY_IMAGE_THUMB, scPaths::PATH_FILE));
				
				$counter = 0;
				while (($file = readdir($handle)) !== false) {
					if (is_dir($imageDir . $file))
						continue;
					
					if (in_array($counter, $selectedImages)) {
						$imageName = miGetParamDefault('image' . $counter, '');
						
						$galleryItemId = $this->getGalleryItemIdByName($galleryId, $imageName);
						
						$galleryItem = new scGalleryItemDomainObject();
						if ($galleryItemId > 0) {
							$galleryItem->read($galleryItemId);
						} else {
							$galleryItem->set('GalleryID', $galleryId);
							$galleryItem->set('GalleryItemName', $imageName);
							$galleryItem->set('GalleryItemSort', $this->getNextSort($galleryId));
							$galleryItemId = $galleryItem->insert();
						}
						
						$this->processImage($galleryItem, $imageDir, $file, $galleryInfo['GalleryDirName']);	
					}
					$counter++;
				}
				
			    miStaticDBUtil::commit();
			    closedir($handle);
			    $this->_view->addRedirectToGalleryControllerCommand(sprintf(_t('Successfully imported %d images.'), $counter), miMessage::MSG_TYPE_INFO);
				return true;
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		protected function createImageDir($dirName)
		{
			if (!is_dir($dirName))
				@mkdir($dirName);
			
			if (!is_dir($dirName))
				throw new miException(sprintf(_i('Cannot create directory "%s".'), $dirName));
		}
		
		protected function getGalleryItemIdByName($galleryId, $galleryItemName)
		{
			$findGalleryItem = 'SELECT GalleryItemID FROM GalleryItems WHERE GalleryID = ? AND GalleryItemName = ?';
			$items = miStaticDBUtil::execSelect($findGalleryItem, array($galleryId, $galleryItemName));
			if (count($items) == 0) {
				return false;
			}
			
			return (int)$items[0]['GalleryItemID'];
		}
		
		protected function processImage(scGalleryItemDomainObject $galleryItem, $imageDir, $imageFileName, $galleryDirName)
		{
			$oldPath = $imageDir . $imageFileName;
			
			$galleryItemId = $galleryItem->get($galleryItem->getPKName());
			
			// Process image
			$fileName = $galleryDirName . '/' . scPaths::GALLERY_LARGE_IMAGES . scPaths::getItemFilename($galleryItemId, scPaths::RESOURCE_GALLERY_IMAGE);
			$filePath = scPaths::getFilenamePath($fileName, scPaths::RESOURCE_GALLERY_IMAGE, scPaths::PATH_FILE);
			$maxWidth = (int) scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_MAX_WIDTH);
			$maxHeight = (int) scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_MAX_HEIGHT);
			
			scImageHandler::resizeImage($oldPath, $filePath, $maxWidth, $maxHeight);
			list($resultWidth, $resultHeight) = scImageHandler::getImageSize($filePath);
			
			$galleryItem->set('GalleryItemImage', scPaths::getFilenamePath($fileName, scPaths::RESOURCE_GALLERY_IMAGE, scPaths::PATH_WEB));
			$galleryItem->set('GalleryItemImageWidth', $resultWidth);
			$galleryItem->set('GalleryItemImageHeight', $resultHeight);
			
			// Process thumb
			$fileName = $galleryDirName . '/' . scPaths::GALLERY_THUMB_IMAGES . scPaths::getItemFilename($galleryItemId, scPaths::RESOURCE_GALLERY_IMAGE_THUMB);
			$filePath = scPaths::getFilenamePath($fileName, scPaths::RESOURCE_GALLERY_IMAGE_THUMB, scPaths::PATH_FILE);
			$maxWidth = (int) scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_THUMB_MAX_WIDTH);
			$maxHeight = (int) scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_THUMB_MAX_HEIGHT);
			
			scImageHandler::resizeImage($oldPath, $filePath, $maxWidth, $maxHeight);
			list($resultWidth, $resultHeight) = scImageHandler::getImageSize($filePath);
			
			$galleryItem->set('GalleryItemImageThumb', scPaths::getFilenamePath($fileName, scPaths::RESOURCE_GALLERY_IMAGE_THUMB, scPaths::PATH_WEB));
			$galleryItem->set('GalleryItemImageThumbWidth', $resultWidth);
			$galleryItem->set('GalleryItemImageThumbHeight', $resultHeight);
			
			$galleryItem->update();
		}
		
		protected function getNextSort($galleryId)
		{
			if ($this->_currentMaxSort === false) {
				$result = miStaticDBUtil::execSelect('SELECT MAX(GalleryItemSort) as MaxSort FROM GalleryItems WHERE GalleryID = ?', array($galleryId));
				$this->_currentMaxSort = $result[0]['MaxSort'];	
			}
			$this->_currentMaxSort += 1;
			return $this->_currentMaxSort;
		}
	}

	$view = new scGalleryImportView();
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('', 'scGalleryImportViewAction');
	$page->getDispatcher()->setActionHandler('dmExecImport', 'scGalleryExecImportAction');
	$page->dispatchAndShow();
?>