<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_GALLERIES);
	scDateTime::setTimezone();
	
	class scGalleryItemsView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/gallery_items/gallery_item_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/gallery_items/gallery_item_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/gallery_items/gallery_item_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'GalleryItemID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'GalleryItemName',
				'validator' => 'scValidatorNoEmpty'
        	),
        	array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'GalleryItemDescription'
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'GalleryItemURL',
        	),
			array(
				'field' => 'scImageUploadWidget',
				'data' => 'GalleryItemImage',
				'modelStrategy' => 'scViewMapperGalleryItemImage',
				'properties' => array(
					'resource' => scPaths::RESOURCE_GALLERY_IMAGE,
					'imageMaxWidthSetting' => scSettings::SETTING_GALLERY_ITEM_IMAGE_MAX_WIDTH,
					'imageMaxHeightSetting' => scSettings::SETTING_GALLERY_ITEM_IMAGE_MAX_HEIGHT,
					'imageThumbMaxHeightSetting' => scSettings::SETTING_GALLERY_ITEM_IMAGE_THUMB_MAX_HEIGHT,
					'imageThumbMaxHeightSetting' => scSettings::SETTING_GALLERY_ITEM_IMAGE_THUMB_MAX_HEIGHT,
					'imageWidthField' => 'GalleryItemImageWidth',
					'imageHeightField' => 'GalleryItemImageHeight',
					'required' => true,
					'deleteButton' => false,
				),
				'validator' => 'scImageUploadedValidator'
        	),
			array(
				'field' => 'scWebFormFieldHidden',
				'data' => 'GalleryID'
        	)
        );
        
        protected $_domainObjectDisplayName = 'Gallery item';	// _t('Gallery item');
		
		protected function createDomainObject()
		{
			return new scGalleryItemDomainObject();
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$record = new scDomainToRecordAdapter($this->getDomainObject());
			$form->addValidator('GalleryItemName', new scValidatorUniqueFields($form, array('GalleryItemName'), $record));
			
			$form->addMainPageElements(
				array(
					'%%GALLERYITEMIMAGEMAXWIDTH%%' => miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_MAX_WIDTH)),
					'%%GALLERYITEMIMAGEMAXHEIGHT%%' => miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_MAX_HEIGHT)),
				)
			);
		}
		
		public function preProcess($action)
		{
			if (!miHasParam('GalleryID')) {
				$location = 'galleries.php?' . $this->getRedirectParams($msg, $msgType);
				$this->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $location));
			}
			parent::preProcess($action);
		}
		
		public function addRedirectToListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = 'galleries.php?action=dmEdit&GalleryID=' . miGetParam('GalleryID') . '&' . $this->getRedirectParams($msg, $msgType);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
		
		public function addAjaxRefreshListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = 'galleries.php?action=dmEdit&GalleryID=' . miGetParam('GalleryID') . '&' . $this->getRedirectParams($msg, $msgType);
			$html = '<script>window.location.href="' . $loc . '";</script>';
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->addControllerCommand($cmd);
		}
	}
	
	$galleryId = (int)miGetParamDefault('GalleryID', 0);
	
	$galleryItems = new scSqlRecordsetML('GalleryItems', 'GalleryItemID', 
					array('GalleryItemName', 'GalleryItemDescription'), array_keys(scMultiLanguage::getActiveLanguageCodes()));
	$galleryItems->addFilter(new miSqlFilterEqual('GalleryID', $galleryId));
	$galleryItems->setOrder('GalleryItemSort', 'ASC');
	
	$table = new scGalleryItemsTable($galleryItems);
	
	$view = new scGalleryItemsView();
	$view->setUpSortColumn('GalleryItemSort');
	
	scSortAction::setForeignKeyClause('GalleryID=' . intval($galleryId));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->dispatchAndShow();
?>