<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_IMPORT_EXPORT);

	class scImportView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_VIEW => 'admin/import/import.tmpl',
			'importReport' => 'admin/import/import_report.tmpl'
		);
	}
	
	class scImportAction extends miViewAction {
		
		/**
		 * @return miWebForm
		 */
		public function getWebForm()
		{
			if ($this->_webForm == null) {
				$this->_webForm = new miWebForm(array());
				$this->_view->initWebForm($this->_webForm);
			}
			return $this->_webForm;
		}
		
		public function doAction()
		{
			$form = $this->getWebForm();
			
			$languages = scMultiLanguage::singleton()->getActiveLanguages();
			$sectionData = miUIUtil::templetizeRows(miUIUtil::transposeRows($languages));
			$importLanguages = new miTemplateParserSectionInfo('importLanguages', count($languages), $sectionData);
			
			$form->addTemplateSectionInfo($importLanguages);
			
			$this->showForm($form, $this->_view->getTemplateFileName(miDefaultView::TEMPLATE_ID_VIEW), false);
			return true;
		}
	}
	
	class scExecImportAction extends scImportAction {
		
		const CATEGORY_SEPARATOR = '|';
		
		const MAX_LINE_LENGTH = 1000000;
		
		const PRODUCTCODE = 0;
		const CATEGORYNAME = 1;
		const MANUFACTURERNAME = 2;
		const PRODUCTNAME = 3;
		const PRODUCTDESCRIPTION = 4;
		const PRODUCTDETAILEDDESCRIPTION = 5;
		const PRODUCTISACTIVE = 6;
		const PRODUCTPRICE = 7;
		const PRODUCTWEIGHT = 8;
		const PRODUCTQUANTITY = 9;
		const PRODUCTISEGOOD = 10;
		const PRODUCTCLASSNAME = 11;
		const CATEGORYBRANCH = 12;
		const PRODUCTDESCRIPTIONXHTML = 13;
		const PRODUCTDETAILEDDESCRIPTIONXHTML = 14;
		
		static protected $_headerMap = array(
			'ProductCode' => self::PRODUCTCODE,			
			'CategoryName' => self::CATEGORYNAME,
			'CategoryBranch' => self::CATEGORYBRANCH,
			'ManufacturerName' => self::MANUFACTURERNAME,
			'ProductName' => self::PRODUCTNAME,
			'ProductDescription' => self::PRODUCTDESCRIPTION,
			'ProductDetailedDescription' => self::PRODUCTDETAILEDDESCRIPTION,
			'ProductIsActive' => self::PRODUCTISACTIVE,
			'ProductPrice' => self::PRODUCTPRICE,
			'ProductWeight' => self::PRODUCTWEIGHT,
			'ProductQuantity' => self::PRODUCTQUANTITY,
			'ProductIsEgood' => self::PRODUCTISEGOOD,
			'ProductClassName' => self::PRODUCTCLASSNAME,
			'ProductDescriptionXHTML' => self::PRODUCTDESCRIPTIONXHTML,
			'ProductDetailedDescriptionXHTML' => self::PRODUCTDETAILEDDESCRIPTIONXHTML,
		);
		
		protected $_linesProcessed = 0;
		protected $_importedProducts = 0;
		protected $_updatedProducts = 0;
		protected $_importedCategories = 0;
		protected $_importedManufacturers = 0;
		protected $_importedProductClasses = 0;
		
		protected $_imagesArray = array();
		protected $_imagesDir = 'csvimport/';
		static protected $_imageTypes = array('jpg', 'jpeg', 'gif', 'png');
		protected $failedImages = array();
		
		protected $_taxesArray = array();
		
		protected $_multilanguage = ''; // '' = All
		
		/**
		 * The columns mapping
		 * The key is the column number in the file
		 * The value is one of the column constants
		 */
		protected $_columns = array();
		
		protected $_defaultLayoutId = 0;
		protected $_productImageIds = array();
		
		protected $_categoryBranches = array();
		protected $_categoryNames = array();
		protected $_manufacturerNames = array();
		
		protected $_allowPartialSuccess = false;
		
		protected $_fixedPricesColumns = array();
		
		public function doAction()
		{
			$this->_allowPartialSuccess = miGetParamDefault('AllowPartialSuccess', '') != '' ? true : false;
			
			if (!isset($_FILES['CSVFile']) or empty($_FILES['CSVFile']['tmp_name'])) {
				miSetParam(miMessage::PARAM_MESSAGE, _t('No file uploaded.'));
				return parent::doAction();
			}
			
			if ($_FILES['CSVFile']['error']) {
				miSetParam(miMessage::PARAM_MESSAGE, sprintf(_t('Error during upload (error code %d).'), $_FILES['CSVFile']['error']));
				return parent::doAction();
			}
			
			$f = @fopen($_FILES['CSVFile']['tmp_name'], 'rb');
			if ($f === false) {
				miSetParam(miMessage::PARAM_MESSAGE, sprintf(_t('Cannot open file %s.'), $_FILES['CSVFile']['tmp_name']));
				return parent::doAction();
			}
			
			$this->_multilanguage = (int)miGetParamDefault('ImportLanguage', 0);
			
			$this->setImagesArray();
			$this->setTaxesArray();
			
			// if allowPartialSuccess we use per product transactions not a global one
			if (!$this->_allowPartialSuccess) {
				miStaticDBUtil::startTransaction();
			}
			try {
				$deleteAllProducts = miGetParamDefault('DeleteAllProducts', '') != '' ? true : false;
				if ($deleteAllProducts)
					$this->deleteAllProducts();
				
				$this->importProducts($f);
				
				$this->deleteMarkedProductImages();
			} catch(miException $exception) {
				
				// if allowPartialSuccess we don't have a transaction to rollback
				if (!$this->_allowPartialSuccess) {
					miStaticDBUtil::rollback();
					$msg = _t('Import failed! Reason: ');
				} else {
					// we need to build a report here
					$msg = sprintf(_t('Partial success. %d products imported, %d products updated. Import stopped on line #%d with message: '), $this->_importedProducts, $this->_updatedProducts, $this->_linesProcessed+1);
				}
				
				miSetParam(miMessage::PARAM_MESSAGE, $msg . $exception->getMessage());
				return parent::doAction();
			}
			if (!$this->_allowPartialSuccess) {
				miStaticDBUtil::commit();
			}
			
			fclose($f);
			
			// Success
			miSetParam(miMessage::PARAM_MESSAGE, _t('Import successful.'));
			miSetParam(miMessage::PARAM_MESSAGE_TYPE, miMessage::MSG_TYPE_INFO);
			$form = $this->getWebForm();
			$form->addMainPageElements(array(
				'%%IMPORTEDPRODUCTS%%' => $this->_importedProducts,
				'%%UPDATEDPRODUCTS%%' => $this->_updatedProducts,
				'%%IMPORTEDCATEGORIES%%' => $this->_importedCategories,
				'%%IMPORTEDMANUFACTURERS%%' => $this->_importedManufacturers,
				'%%IMPORTEDPRODUCTCLASSES%%' => $this->_importedProductClasses,
				'%%FAILEDIMAGES%%' => count($this->failedImages) > 0 ? join(', ', $this->failedImages) : _t('All found images imported successfully.'),
			));
			$this->showForm($form, $this->_view->getTemplateFileName('importReport'), false);
			return true;
		}
		
		protected function deleteAllProducts()
		{
			// Delete all product images
			$this->_productImageIds = miUtil::getDBArray('SELECT ProductImageID FROM ProductImages', 'ProductImageID', 'ProductImageID');
			
			miStaticDBUtil::execSQL('DELETE FROM ProductImages');
			miStaticDBUtil::execSQL('DELETE FROM Products');
		}
		
		protected function deleteMarkedProductImages()
		{
			foreach ($this->_productImageIds as $productImageId) {
				$image = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_IMAGE);
				@unlink($image);
				
				$image = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE);
				@unlink($image);
				
				$image = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_THUMB);
				@unlink($image);
				
				$image = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE);
				@unlink($image);
				
				$image = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE);
				@unlink($image);
			}
		}
		
		/**
		 * Read the header line of the CSV file
		 * Setups the $_columns mapping 
		 * 
		 * @param resource $f the file handler
		 * @throws miException if there is unknown header
		 */
		protected function readHeaderLine($f)
		{
			$data = fgetcsv($f, self::MAX_LINE_LENGTH);
			
			// Add the product fixed price columns (if there are any) to the header map
			$currenciesWithFixedPrices = scCurrencyUtil::getCurrenciesWithFixedPrices();
			foreach ($currenciesWithFixedPrices as $currency) {
				$priceFieldName = 'ProductPrice' . $currency['CurrencyCode'];
				if (array_search($priceFieldName, $data) !== false) {
					$columnNum = max(self::$_headerMap) + 1;
					self::$_headerMap[$priceFieldName] = $columnNum;
					$this->_fixedPricesColumns[$columnNum] = $priceFieldName;
				}
			}
			
			foreach ($data as $key => $headerName) {
				if (!isset(self::$_headerMap[$headerName]))
					throw new miException(sprintf(_t('Unknown header "%s".'), $headerName));
				
				$this->_columns[$key] = self::$_headerMap[$headerName];
			}
		}
		
		/**
		 * Sets default image to product by product code
		 * 
		 * @param $productcode
		 * @return boolean
		 */
		protected function setImages($productCode)
		{
			$csvImagesPath = scPaths::getStoreRootDir() . scPaths::USER_FILES_PATH . $this->_imagesDir;
			
			try {
				$productId = $this->getProductID($productCode);
				if (!$productId)
					return false;
				$productId = (int)$productId;
				
				// Check if It's an image or a folder of images
				if (!is_array($this->_imagesArray[$productCode])) {
					$sql = 'SELECT ProductImageID FROM ProductImages WHERE ProductID = ' . $productId . ' AND ProductImageIsDefault';
					$currentDefaultImage = miStaticDBUtil::execSelect($sql);
					if (!empty($currentDefaultImage)) {
						$this->_productImageIds[] = $currentDefaultImage[0]['ProductImageID'];
					}
					
					$sql = 'DELETE FROM ProductImages WHERE ProductImageIsDefault = 1 AND ProductID = ' . $productId;
					miStaticDBUtil::execSQL($sql);
					$imageNames = array(scFilesUtil::getFilenameFromString($this->_imagesArray[$productCode]));
					$this->_imagesArray[$productCode] = array($this->_imagesArray[$productCode]);
					$replaceImages = array();
				} else {
					foreach ($this->_imagesArray[$productCode] as $key => $image) {
						// Get the filename (with extension)
						$imageName = basename($image);
						$imageName = scFilesUtil::getFilenameFromString($imageName);
						
						// The names will be used in a SQL query
						$imageNames[$key] = sql_escape_string($imageName);
					}
					
					// Find images that have to be replaced (based on imageName)
					$sql = 'SELECT ProductImageID, ProductImageOriginalName FROM ProductImages' .
						' WHERE ProductID = ' . $productId .
						' AND ProductImageOriginalName IN ("' . implode('", "', $imageNames) . '")';
					$replaceImages = miStaticDBUtil::execSelect($sql);
					$replaceImages = miUtil::getKeyValueArray($replaceImages, 'ProductImageID', 'ProductImageOriginalName');
				}
				
				
				// Check if there alredy is a default image
				$sql = 'SELECT ProductImageID FROM ProductImages WHERE ProductID = ' . $productId . ' AND ProductImageIsDefault LIMIT 1';
				$defaultImageSet = miStaticDBUtil::execSelect($sql);
				if (count($defaultImageSet) > 0) {
					$defaultImageSet = true;
				} else {
					$defaultImageSet = false;
				}
				
				// Get the image sort
				$sql = 'SELECT MAX(ProductImageSort) AS ProductImageSort FROM ProductImages WHERE ProductID = ' . $productId . ' GROUP BY ProductID';
				$maxProductImageSort = miStaticDBUtil::execSelect($sql);
				$productImageSort = 0;
				if (!empty($maxProductImageSort)) {
					$productImageSort = $maxProductImageSort[0]['ProductImageSort'] + 1;
				}
				
				foreach ($this->_imagesArray[$productCode] as $key => $image) {
					$sourceFileName = $csvImagesPath . $image;
					
					$originalImageName = $imageNames[$key];
					$productImageId = array_search($originalImageName, $replaceImages);
					if ($productImageId === false) {
						$values = array();
						$values['ProductID'] = $productId;
						$values['ProductImageOriginalName'] = scFilesUtil::generateFilenameFromString($originalImageName, array('scViewMapperProductImages', 'isProductImageNameAvailable'));
						$values['ProductImageSort'] = $productImageSort;
						$productImageSort++;
						
						if (!$defaultImageSet) {
							// If there is no default image, set the first one as such
							$values['ProductImageIsDefault'] = 1;
							$defaultImageSet = true;
						}
						$productImageId = miStaticDBUtil::execInsert('ProductImages', $values);
					} else {
						// The image was found so its name doesn't need updating.
						$originalImageName = '';
					}
					
					if ($this->isImageSame($productImageId, $sourceFileName))
						continue;
					
					list($originalImageDest, $imageDest, $largeImageDest, $boxImageDest, $thumbDest) = $this->setImage($productId, $productImageId, $sourceFileName);
					
					scViewMapperProductImages::setProductImageDimensions($productImageId, $originalImageDest, $imageDest, $largeImageDest, $boxImageDest, $thumbDest);
				}
			} catch(miException $exception) {
				return false;
			}
			return true;
		}
		
		protected function isImageSame($productImageId, $sourceFileName)
		{
			$originalImageDest = scPaths::getStoreRootDir() . scUtil::getDbValue('ProductImages', 'ProductImageOriginal', 'ProductImageID', $productImageId);
			if (!is_file($originalImageDest))
				return false;
			
			return md5_file($sourceFileName) === md5_file($originalImageDest);
		}
		
		protected function setImage($productId, $productImageId, $sourceFileName)
		{
			$originalImageDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE);
			if (!@copy($sourceFileName, $originalImageDest)) {
				throw new miException(sprintf(_t('Unable to copy "%s" to "%s".'), $sourceFileName, $originalImageDest));
			}
			
			// Normal image
			$imageDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_IMAGE);
			scImageHandler::resizeImage(
				$originalImageDest, $imageDest, 
				scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_MAX_WIDTH),
				scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_MAX_HEIGHT)
			);
			
			// Large image
			$largeImageDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE);
			scImageHandler::resizeImage(
				$originalImageDest, $largeImageDest, 
				scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_WIDTH),
				scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_HEIGHT)
			);
			
			// Box image
			$boxImageDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE);
			scImageHandler::resizeImage(
				$originalImageDest, $boxImageDest, 
				scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_WIDTH),
				scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_HEIGHT)
			);
			
			// Thumbnail
			$thumbDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_THUMB);
			scImageHandler::resizeImage(
				$originalImageDest, $thumbDest, 
				scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH),
				scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_HEIGHT)
			);
			
			return array($originalImageDest, $imageDest, $largeImageDest, $boxImageDest, $thumbDest);
		}
		
		protected function importProducts($f)
		{
			// Read the header
			$this->readHeaderLine($f);
			
			// Read the data
			while (($data = fgetcsv($f, self::MAX_LINE_LENGTH)) !== false) {
				
				$this->_linesProcessed++;
				
				if ($this->_allowPartialSuccess) {
					miStaticDBUtil::startTransaction();
				}
				
				try {
					// check for empty line in the csv and skip that row
					if ((count($data) == 1) && is_null($data[0])) 
						continue;
					
					$product = new scSqlRecordML('Products', 'ProductID',
						array('ProductName', 'ProductDescription', 'ProductDetailedDescription',
						'ProductPageTitle', 'ProductMetaKeywords', 'ProductMetaDescription'), array_keys(scMultiLanguage::getActiveLanguageCodes()));
					
					$productCodeColumn = array_search(self::PRODUCTCODE, $this->_columns);
					$productCode = $data[$productCodeColumn];
					$productId = $this->getProductID($productCode);
					
					if ($productId) {
						
						$product->read('ProductID', $productId);
						
						$this->setProductData($product, $data, 'update');
						
						$product->update();
						$this->_updatedProducts++;
						
					} else {
						
						$this->setProductData($product, $data, 'insert');
						
						$product->set('ProductCreatedTimestamp', time());
						if ($this->productCodeExists($product->get('ProductCode')))
							throw new miException(sprintf(_t('Product with SKU "%s" already exists!'), $product->get('ProductCode')));
						
						if (!$product->has('CategoryID')) {
							throw new miException(sprintf(_i('Category not set for product with SKU "%s".'), $product->get('ProductCode')));
						}
						
						$productId = $product->insert();
						
						if (!empty($this->_taxesArray)) {
							$multiValues = array();
							foreach ($this->_taxesArray as $taxClassId) {
								$multiValues[] = array(
									'ProductID' => $productId,
									'TaxClassID' => $taxClassId
								);
							}
							miStaticDBUtil::execMultiInsert('ProductTaxClasses', $multiValues);
						}
						
						$this->_importedProducts++;
					}
					// if the current product code is in the images array copy the image to the product
					if (isset($this->_imagesArray[$productCode])) {
						if (!$this->setImages($productCode)) {
							$this->failedImages[] = $productCode;
						}
					}
				} catch (miException $e) {
					if ($this->_allowPartialSuccess) {
						miStaticDBUtil::rollback();
					}
					throw $e;
				}
				
				if ($this->_allowPartialSuccess) {
					miStaticDBUtil::commit();
				}
			}
			scNestedSet::buildNestedSet('Categories', 'CategoryID', 'CategoryParentID', 'CategoryLeft', 'CategoryRight');
			scCategories::updateProductCounts();
		}
		
		protected function setProductData(&$product, $data, $type)
		{
			// By default the product is active
			$product->set('ProductIsActive', 1);
			
			// Manage CategoryName and CategoryBranch with highest priority on the branch.
			$branchKey = array_search(self::CATEGORYBRANCH, $this->_columns);
			$nameKey = array_search(self::CATEGORYNAME, $this->_columns);
			if (($branchKey !== false) && isset($data[$branchKey])) {
				$categoryId = $this->getCategoryIdByBranch($data[$branchKey]);
			} else if (($nameKey !== false) && isset($data[$nameKey])) {
				$categoryId = $this->getCategoryId($data[$nameKey]);
			}
			if (isset($categoryId)) {
				$product->set('CategoryID', $categoryId);
				$product->set('ProductSortInCategory', scProductSortService::getMaxSortInCategory($categoryId) + 1);
			}
			
			$variantFields = array(
				self::PRODUCTPRICE,
				self::PRODUCTWEIGHT,
				self::PRODUCTQUANTITY
			);
			foreach ($data as $key => $value) {
				$column = $this->_columns[$key];
				if (($type == 'update') && in_array($column, $variantFields) && $this->hasProductVariants($product->get('ProductID'))) {
					// Do not set to products having variants.
					continue;
				}
				
				if (array_key_exists($column, $this->_fixedPricesColumns)) {
					$product->set($this->_fixedPricesColumns[$column], str_replace(',', '.', $value));
					$product->set($this->_fixedPricesColumns[$column] . 'Auto', 0);
					continue;
				}
				
				switch ($column) {
					
					case self::PRODUCTCODE:
						if ($type == 'insert') {
							$product->set('ProductCode', $value);
						}
						break;
						
					case self::PRODUCTNAME:
						$this->setMLValue($product, 'ProductName', $value);
						break;
					
					case self::PRODUCTDESCRIPTION:
						$this->setMLValue($product, 'ProductDescription', $value);
						break;
					
					case self::PRODUCTDETAILEDDESCRIPTION:
						$this->setMLValue($product, 'ProductDetailedDescription', $value);
						break;
					
					case self::PRODUCTISACTIVE:
						$product->set('ProductIsActive', $value);
						break;
					
					case self::PRODUCTISEGOOD:
						$product->set('ProductIsEgood', $value);
						break;
					
					case self::PRODUCTDESCRIPTIONXHTML:
						$product->set('ProductDescriptionXHTML', $value);
						break;
					
					case self::PRODUCTDETAILEDDESCRIPTIONXHTML:
						$product->set('ProductDetailedDescriptionXHTML', $value);
						break;
					
					case self::PRODUCTPRICE:
						$product->set('ProductPrice', str_replace(',', '.', $value));
						break;
						
					case self::PRODUCTWEIGHT:
						$product->set('ProductWeight', str_replace(',', '.', $value));
						break;
						
					case self::PRODUCTQUANTITY:
						$product->set('ProductQuantity', $value);
						break;
					
					case self::MANUFACTURERNAME:
						$product->set('ManufacturerID', $this->getManufacturerId($value));
						break;
						
					case self::PRODUCTCLASSNAME:
						$product->set('ProductClassID', $this->getProductClassId($value));
						break;
				}
			}
			
			$baseCurrencyCode = scCurrencyUtil::getBaseCurrencyCode();
			if (in_array('ProductPrice' . $baseCurrencyCode, $this->_fixedPricesColumns)) {
				$product->set('ProductPrice' . $baseCurrencyCode, $product->get('ProductPrice'));
			}
		}
		
		/**
		 * @return bool
		 */
		protected function hasProductVariants($productId)
		{
			$query = 'SELECT COUNT(ProductVariantID) AS N FROM ProductVariants WHERE ProductID = ?';
			$result = miStaticDBUtil::execSelect($query, array($productId));
			if (isset($result[0]) && isset($result[0]['N']) && $result[0]['N'] > 1) {
				return true;
			}
			return false;
		}
		
		protected function getProductId($productCode)
		{
			$sql = 'SELECT ProductID FROM Products WHERE ProductCode=?';
			$result = miStaticDBUtil::execSelect($sql, array($productCode));
			
			return !empty($result[0]['ProductID']) ? $result[0]['ProductID'] : false;
		}
		
		protected function productCodeExists($productCode)
		{
			$sql = 'SELECT COUNT(*) AS N FROM Products WHERE ProductCode=?';
			$result = miStaticDBUtil::execSelect($sql, array($productCode));
			return $result[0]['N'] > 0;
		}
		
		protected function getLanguageId()
		{
			if ($this->_multilanguage != 0) {
				return $this->_multilanguage;
			}
			return scMultiLanguage::getCurrentLanguageId();
		}
		
		/**
		 * Find's the ID of the category, create new if does not exists and user checked the option  
		 * 
		 * If $categoryName starts with CATEGORY_SEPARATOR then 
		 * a whole branch of the category tree is provided
		 * 
		 * else it is just one categoryName
		 * 
		 * @param $categoryName
		 * @return integer
		 */
		protected function getCategoryId($categoryName)
		{
			if (empty($categoryName)) 
				return;
			
			$categoryName = trim($categoryName);
			
			$createCategories = miGetParamDefault('AutoCreateCategories', '') != '' ? true : false;
			
			// manage the single category name
			// first search that create or throw exception 
			if (strpos($categoryName, self::CATEGORY_SEPARATOR) !== 0) {
				if (isset($this->_categoryNames[$categoryName])) {
					return $this->_categoryNames[$categoryName];
				}
				
				$sql = 'SELECT CategoryID FROM CategoriesML WHERE CategoryName=? AND LanguageID=?';
				$res = miStaticDBUtil::execSelect($sql, array($categoryName, $this->getLanguageId()));
				if (isset($res[0]['CategoryID'])) { 
					$this->_categoryNames[$categoryName] = $res[0]['CategoryID'];
					return $res[0]['CategoryID'];
				} elseif ($createCategories) {
					$this->_categoryNames[$categoryName] = $this->saveNewCategory($categoryName);
					return $this->_categoryNames[$categoryName];
				} else {
					throw new miException(sprintf(_t('Category "%s" not found.'), $categoryName));
				}
			}
			
			return $this->getCategoryIdByBranch($categoryName);
		}
		
		protected function getCategoryIdByBranch($categoryFullName)
		{
			$createCategories = miGetParamDefault('AutoCreateCategories', '') != '' ? true : false;
			
			// manage category branch
			// remove the leading separator
			$categoryFullName = ltrim($categoryFullName, self::CATEGORY_SEPARATOR);
			if (isset($this->_categoryBranches[$categoryFullName])) {
				return $this->_categoryBranches[$categoryFullName];
			}
			
			$categoryBranch = explode(self::CATEGORY_SEPARATOR, $categoryFullName);
			$categoryParentsIds = array();
			
			$newBranch = false; // used to prevent one query to the db
			
			foreach ($categoryBranch as $level => $categoryName) {
				$categoryName = trim($categoryName);
				// skip this if a new branch is imported
				if (!$newBranch) {
					$parentCondition = ($level == 0) ? 'CategoryParentID IS NULL' : 'CategoryParentID = ' . (int)$categoryParentsIds[$level-1];
					$sql = 'SELECT Categories.CategoryID FROM CategoriesML LEFT JOIN Categories USING(CategoryID) WHERE CategoryName=? AND LanguageID=? AND ' . $parentCondition;
					$result = miStaticDBUtil::execSelect($sql, array($categoryName, $this->getLanguageId()));
					
					if (count($result) > 0) {
						$categoryParentsIds[$level] = $result[0]['CategoryID'];
						continue;
					}
				}
				
				if (!$createCategories && !isset($categoryParentsIds[$level])) {
					throw new miException(sprintf(_t('Category "%s" not found.'), $categoryName));	
				}
				
				if ($createCategories) {
					if (isset($categoryParentsIds[$level-1]))
						$parentId = $categoryParentsIds[$level-1];
					else 
						$parentId = null;

					$categoryParentsIds[$level] = $this->saveNewCategory($categoryName, $parentId);
					$newBranch = true; //don't even try to find categories in the branch down
				}
			}
			
			$this->_categoryBranches[$categoryFullName] = $categoryParentsIds[count($categoryParentsIds)-1];
			
			return $this->_categoryBranches[$categoryFullName];
		}
		
		/**
		 * Saves new category and returns the new ID 
		 * @param $categoryName
		 * @param $parentId
		 * @return integer
		 */
		protected function saveNewCategory($categoryName, $parentId = null)
		{
			$category = new scSqlRecordML('Categories', 'CategoryID',
				array('CategoryName', 'CategoryDescription', 'CategorySEOKeywords', 'CategorySEODescription'), array_keys(scMultiLanguage::getActiveLanguageCodes()));
			$category->set('CategoryParentID', $parentId);
			$category->set('CategorySort', $this->getNextCategorySort($category->get('CategoryParentID')));
			$category->set('CMSLayoutID', $this->getDefaultCmsLayoutId());
			$category->set('CategoryName', $this->getMLValue($categoryName));
			$this->_importedCategories++;
			return $category->insert();
		}
		
		protected function getNextCategorySort($parentId = null)
		{
			if (is_null($parentId)) {
				$sql = 'SELECT MAX(CategorySort) AS N FROM Categories WHERE CategoryParentID IS NULL';
			} else {
				$sql = 'SELECT MAX(CategorySort) AS N FROM Categories WHERE CategoryParentID = ' . intval($parentId);
			}
			$rows = miStaticDBUtil::execSelect($sql);
			return $rows[0]['N'] + 1;
		}
		
		protected function getNextManufacturerSort()
		{
			$sql = 'SELECT MAX(ManufacturerSort) AS N FROM Manufacturers';
			$rows = miStaticDBUtil::execSelect($sql);
			return $rows[0]['N'] + 1;
		}
		
		protected function getDefaultCmsLayoutId()
		{
			if ($this->_defaultLayoutId)
				return $this->_defaultLayoutId;
			
			$sql = 'SELECT CMSLayoutID FROM CMSLayouts WHERE CMSLayoutIsDefault<>0';
			$result = miStaticDBUtil::execSelect($sql);
			if (count($result) == 0)
				throw new miException(_t('Default Layout not set.'));
				
			$this->_defaultLayoutId = $result[0]['CMSLayoutID'];
			return $this->_defaultLayoutId;
		}
		
		protected function getManufacturerId($manufacturerName)
		{
			if ($manufacturerName == '')
				return null;
			
			if (isset($this->_manufacturerNames[$manufacturerName])) {
				return $this->_manufacturerNames[$manufacturerName];
			}
			
			$sql = 'SELECT ManufacturerID FROM ManufacturersML WHERE ManufacturerName=? AND LanguageID=?';
			$result = miStaticDBUtil::execSelect($sql, array($manufacturerName, $this->getLanguageId()));
			if (count($result) > 0) {
				$this->_manufacturerNames[$manufacturerName] = $result[0]['ManufacturerID'];
				return $result[0]['ManufacturerID'];
			}
			
			$createManufacturers = miGetParamDefault('AutoCreateManufacturers', '') != '' ? true : false;
			if ($createManufacturers) {
				$manufacturer = new scSqlRecordML('Manufacturers', 'ManufacturerID',
					array('ManufacturerName', 'ManufacturerDescription'), array_keys(scMultiLanguage::getActiveLanguageCodes()));
				$manufacturer->set('ManufacturerName', $this->getMLValue($manufacturerName));
				$manufacturer->set('ManufacturerSort', $this->getNextManufacturerSort());
				$this->_importedManufacturers++;
				$this->_manufacturerNames[$manufacturerName] = $manufacturer->insert();
				return $this->_manufacturerNames[$manufacturerName];
			}
			
			throw new miException(sprintf(_t('Manufacturer "%s" not found.'), $manufacturerName));
		}
		
		protected function getProductClassId($productClassName)
		{
			if ($productClassName == '')
				return null;
			
			$sql = 'SELECT ProductClassID FROM ProductClassesML WHERE ProductClassName=? AND LanguageID=?';
			$result = miStaticDBUtil::execSelect($sql, array($productClassName, $this->getLanguageId()));
			if (count($result) > 0)
				return $result[0]['ProductClassID'];
			
			$createProductClasses = miGetParamDefault('AutoCreateProductClasses', '') != '' ? true : false;
			if ($createProductClasses) {
				$productClass = new scSqlRecordML('ProductClasses', 'ProductClassID',
					array('ProductClassName'), array_keys(scMultiLanguage::getActiveLanguageCodes()));
				$productClass->set('ProductClassName', $this->getMLValue($productClassName));
				$this->_importedProductClasses++;
				return $productClass->insert();
			}
			
			throw new miException(sprintf(_t('Product class "%s" not found.'), $productClassName));
		}
		
		/**
		 * @param scSqlRecordML $record
		 * @param string $key
		 * @param mixed $value
		 */
		protected function setMLValue(scSqlRecordML $record, $key, $value)
		{
			if ($record->has($key)) {
				$value = $this->getMLValue($value, $record->get($key));
			} else {
				$value = $this->getMLValue($value);
			}
			$record->set($key, $value);
		}
		
		protected function getMLValue($value, $result = array())
		{
			$codes = scMultiLanguage::getActiveLanguageCodes();
			
			foreach ($codes as $key => $code) {
				if (($this->_multilanguage == 0) || ($this->_multilanguage == $key)) {
					$result[$key] = $value;
				} else if (!isset($result[$key])) { // Do not set empty value if it is not necessary
					$result[$key] = '';
				}
			}
			return $result;
		}
		
		/**
		 * Sets the _imagesArray 
		 * 
		 * @return void
		 */
		protected function setImagesArray()
		{
			// get all images from the import dir and assign each to a productcode
			$csvImagesPath = scPaths::getStoreRootDir() . scPaths::getPath(scPaths::RESOURCE_IMPORT_IMAGE);
			
			if (!file_exists($csvImagesPath))
				return;
			
			$images = scandir($csvImagesPath);
			if (!is_array($images))
				continue;
			
			foreach ($images as $image) {
				if (($image == '.') || ($image == '..')) {
					continue;
				}
				
				if (is_dir($csvImagesPath . '/' . $image)) {
					$inDirImages = scandir($csvImagesPath . '/' . $image);
					foreach ($inDirImages as $inDirImage) {
						if (($inDirImage == '.') || ($inDirImage == '..')) {
							continue;
						}
						
						$ext = pathinfo($inDirImage, PATHINFO_EXTENSION);
						if (in_array(strtolower($ext), self::$_imageTypes)) {
							if (!isset($this->_imagesArray[$image])) {
								$this->_imagesArray[$image] = array();
							}
							$this->_imagesArray[$image][] = $image . '/' . $inDirImage;
						}
					}
					
					continue;
				}
				
				$ext = pathinfo($image, PATHINFO_EXTENSION);
				if (in_array(strtolower($ext), self::$_imageTypes)) {
					$productCode = basename($image, '.' . $ext);
					$this->_imagesArray[$productCode] = $image;
				}
			}
		}
		
		protected function setTaxesArray()
		{
			$sql = 'SELECT TaxClasses.TaxID, TaxClassID' .
				' FROM Taxes' .
				' INNER JOIN TaxClasses USING(TaxID)' .
				' WHERE TaxApplyByDefault' .
				' ORDER BY TaxClassSort';
			$result = miStaticDBUtil::execSelect($sql);
			
			$this->_taxesArray = array();
			foreach ($result as $row) {
				if (isset($this->_taxesArray[$row['TaxID']])) {
					continue;
				}
				$this->_taxesArray[$row['TaxID']] = $row['TaxClassID'];
			}
		}
	}
	
	$view = new scImportView();
	
	$page = new scAdminDisplayController(array($view));
	
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scImportAction');
	$page->getDispatcher()->setActionHandler('dmBack', 'scImportAction');
	$page->getDispatcher()->setActionHandler('dmExecImport', 'scExecImportAction');
	
	$page->dispatchAndShow();
?>