<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_LANGUAGES);

	class scLanguageTextsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/languages/language_texts.tmpl',
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'TextID'
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'TextOriginal'
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'TextTranslated'
        		),
        );
		
		protected $_defaultFilterValues = array(
			'Texts_TextID' => '',
			'TextIsTranslated' => '0',
		);
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
			
			$languageNames = scMultiLanguage::getActiveLanguageNames();
			$languageId = miGetParam('LanguageID');
			$table->addMainPageElements(array('%%LANGUAGENAME%%' => $languageNames[$languageId]));
			$table->addMainPageElements(array('%%LANGUAGEID%%' => $languageId));
			
			$this->getTableFilterObj()->setConditionHandler('translated', 'scLanguageTextIsTranslatedFilter');
			$options = array('' => _t('All statuses'), 0 => _t('Not approved'), 1 => _t('Approved'));
			scUIUtil::createDropdownFilter($this, $table, 'TextIsTranslated', $options, 'translated', 0);
		}
		
		function redirectToCurrentPage($action, $msg, $msgType)
        {
        	$languageId = (int)miGetParam('LanguageID');
        	header('Location: ?action=' . urlencode($action) . '&LanguageID=' . $languageId . '&' . $this->getRedirectParams($msg, $msgType));
        	exit;
        }
	}
	
	class scUpdateLanguageTranslations extends miAction {
		public function doAction()
		{
			try {
				$languageId = (int)miGetParam('LanguageID');
				$textTranslated = miGetParam('TextTranslated');
				$textIsTranslated = miGetParamDefault('TextIsTranslated', array());
				
				$textIds = array();
				foreach ($textTranslated as $textId => $textTranslatedContents)
					$textIds[] = (int)$textId;
				$sql = 'SELECT TextID, TextIsTranslated FROM TextsTranslated WHERE TextID IN(' . implode(',', $textIds) . ') AND LanguageID=' . $languageId;
				$existingTextIds = miUtil::getDBArray($sql, 'TextID', 'TextIsTranslated');
				
				// get text hashes
				$sql = 'SELECT TextID, TextOriginalHash FROM Texts WHERE TextID IN ('.implode(',', $textIds).')';
				$hashes = miUtil::getDBArray($sql, 'TextID', 'TextOriginalHash');
				
				foreach ($textTranslated as $textId => $textTranslatedContents) {
					if (isset($existingTextIds[$textId])) {
						if ($textTranslatedContents == '') {
							$sql = 'DELETE FROM TextsTranslated';
						} else {
							$sql = 'UPDATE TextsTranslated SET' .
								' TextTranslated="' . sql_escape_string($textTranslatedContents) . '", ' .
								' TextIsTranslated=' . (isset($textIsTranslated[$textId]) ? 1 : 0);
						}
						$sql .= ' WHERE TextID="' . sql_escape_string($textId) . '" AND LanguageID=' . $languageId;
						miStaticDBUtil::execSql($sql);
					} else {
						if ($textTranslatedContents == '')
							continue;						
						
						$values = array(
							'TextID' => $textId,
							'LanguageID' => $languageId,
							'TextTranslated' => $textTranslatedContents,
							'TextOriginalHash' => $hashes[$textId],
							'TextIsTranslated' => (isset($textIsTranslated[$textId]) ? 1 : 0),
						);
						miStaticDBUtil::execInsert('TextsTranslated', $values);
					}
				}
				
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_UPDATE_FAILED_MSG');
				$this->_view->redirectToCurrentPage('', $msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			$msg = _t('Successfully updated language texts.');
			$this->_view->redirectToCurrentPage('', $msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	class scLanguageTextsRecordset extends miSqlRecordset {
		protected function getTablesClause()
		{
			return ' FROM Texts LEFT JOIN TextsTranslated ON Texts.TextID = TextsTranslated.TextID AND TextsTranslated.LanguageID="' . sql_escape_string(intval(miGetParam('LanguageID'))) . '"';
		}
	}
	
	class scLanguageTextsTable extends scTable {
		
		protected function addUiElements(&$rows)
		{
			parent::addUiElements($rows);
			
			foreach ($rows as &$row) {
				$row['TextIsTranslated_CHECKED'] = '';
				if ($row['TextIsTranslated']) {
					$row['TextIsTranslated_CHECKED'] = ' checked="checked" ';
				}
			}
		}
	}
	
	class scLanguageTextIsTranslatedFilter extends miSqlFilterOneValue {
		
		public function getSql()
		{
			if ($this->_value == '')
				return '';
			
			$field = sql_escape_string($this->_name);
			if ($this->_value == 1)
				return $field . ' = 1';
			
			$sql = '(%s = 0 OR %s IS NULL)';
			return sprintf($sql, $field, $field);
		}
	}
	
	$texts = new scLanguageTextsRecordset('Texts');
	$texts->setSelectFields(array('Texts.*', 'Texts.TextID as Texts_TextID', 'TextsTranslated.TextTranslated', 'TextsTranslated.TextIsTranslated'));
	$texts->setHavingFilterNames(array('Texts_TextID', 'TextIsTranslated', 'TextTranslated'));
	$table = new scLanguageTextsTable($texts);
	
	$view = new scLanguageTextsView();
	$view->setRecord(new miSqlRecord('Texts', 'TextID'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('acUpdateLanguageTranslations', 'scUpdateLanguageTranslations');
	$page->dispatchAndShow();
?>