<?php
	/**
	 * Copyright (c) 2008-2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_LANGUAGES);

	class scLanguagesView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/languages/languages_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/languages/language_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/languages/language_create.tmpl',
			'InstallLanguagePack' => 'admin/languages/language_install_langpack.tmpl',
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'LanguageID'
        	),
       		array(
				'field' => 'miTextWidget',
				'data' => 'LanguageName',
				'displayName' => 'Name'
        	),
       		array(
				'field' => 'miTextWidget',
				'data' => 'LanguageCode',
				'properties' => array('minLength' => 2, 'maxLength' => 2),
				'displayName' => 'Code',
        	),
       		array(
				'field' => 'miSelectWidget',
				'data' => 'LanguageDirection',
				'displayName' => 'Direction'
        	),
        	array(
				'field' => 'scImageUploadWidget',
				'data' => 'LanguageImage',
				'modelStrategy' => 'scViewMapperUploadImage',
				'properties' => array(
					'resource' => scPaths::RESOURCE_LANGUAGE_IMAGE,
					'imageMaxWidthSetting' => scSettings::SETTING_LANGUAGE_IMAGE_MAX_WIDTH,
					'imageMaxHeightSetting' => scSettings::SETTING_LANGUAGE_IMAGE_MAX_HEIGHT,
					'imageWidthField' => 'LanguageImageWidth',
					'imageHeightField' => 'LanguageImageHeight',
					'noThumb' => true
				)
			),
			array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'LanguageIsActive',
				'properties' => array('defaultValue' => true)
        	),
        	
       		array(
				'field' => 'miTextWidget',
				'data' => 'LanguagePackAuthor',
				'displayName' => 'Language Pack Author'
        	),
       		array(
				'field' => 'miTextWidget',
				'data' => 'LanguagePackDescription',
				'displayName' => 'Language Pack Description'
        	),
       		array(
				'field' => 'scLanguagePackLicenseWidget',
				'data' => 'LanguagePackLicense',
				'displayName' => 'Language Pack License'
        	),
       		array(
				'field' => 'scLanguagePackPublishDate',
				'data' => 'LanguagePackPublishDate',
				'displayName' => 'Language Pack Publish Date'
        	),
		);
        
		protected $_domainObjectDisplayName = 'Language';	// _t('Language');
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$languageNames = scMultiLanguage::getActiveLanguageNames();
			$basedOn = miUIUtil::getConstDropdown('BasedOn', null, $languageNames);
			$this->addMainPageElements(array('%%BASEDON%%' => $basedOn));
			
			if ($this->_controller->getDispatcher()->getAction() == 'dmEdit')
				$form->getWidget('LanguageImage')->setApplyButton();
			
			// Get the max width and height for the image
			$form->addMainPageElements(
				array(
					'%%LANGUAGEIMAGEMAXWIDTH%%' => miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_LANGUAGE_IMAGE_MAX_WIDTH)),
					'%%LANGUAGEIMAGEMAXHEIGHT%%' => miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_LANGUAGE_IMAGE_MAX_HEIGHT))
				)
			);
			
			$form->getWidget('LanguageDirection')->setOptions(scLanguageDomainObject::getLanguageDirections());
			
			$maxUploadSize = scFilesUtil::getMaxUploadSize();
			$this->addMainPageElements(array(
				'%%MAXUPLOADSIZEBYTES%%' => $maxUploadSize,
				'%%MAXUPLOADSIZE%%' => scFilesUtil::bytesToShorthand($maxUploadSize),
			));
			
			if (miGetParamDefault('action', '') == 'dmEdit')
				$this->assignTextsCounts($form);
		}
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function createDomainObject()
		{
			return new scLanguageDomainObject();
		}
		
		public function preProcess($actionObj)
		{
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('LanguageIsActive');
				
			if ($actionObj instanceof miExecCreateAction) {
				miSetParam('LanguagePackPublishDate', date('d/m/Y'));
			}
				
			if (($actionObj instanceof miExecCreateAction) or ($actionObj instanceof miExecEditAction)) {
				// Add language code unique validator
				$form = $actionObj->getWebForm();
				
				$record = new scDomainToRecordAdapter($this->getDomainObject());
				$form->addValidator('LanguageCode', new scValidatorUniqueFields($form, array('LanguageCode'), $record));
			}
			
			parent::preProcess($actionObj);
		}
		
		protected function assignTextsCounts(&$form)
		{
			$interfaceTextsCount = scUtil::getDbFirstRow('SELECT COUNT(1) AS N FROM Texts');
			$helpsTextsCount = scUtil::getDbFirstRow('SELECT COUNT(1) AS N FROM Helps');
			$quickLinksTextsCount = scUtil::getDbFirstRow('SELECT COUNT(1) AS N FROM AdminQuickLinks');
			$countriesTextsCount = scUtil::getDbFirstRow('SELECT COUNT(1) AS N FROM Countries');
			$statesTextsCount = scUtil::getDbFirstRow('SELECT COUNT(1) AS N FROM States');
			$orderStatusesTextsCount = scUtil::getDbFirstRow('SELECT COUNT(1) AS N FROM OrderStatuses');
			
			$this->addMainPageElements(array(
				'%%COUNT_INTERFACE_TEXTS%%' => $interfaceTextsCount['N'],
				'%%COUNT_HELPS_TEXTS%%' => $helpsTextsCount['N'],
				'%%COUNT_QUICK_LINKS_TEXTS%%' => $quickLinksTextsCount['N'],
				'%%COUNT_COUNTRIES_TEXTS%%' => $countriesTextsCount['N'],
				'%%COUNT_STATES_TEXTS%%' => $statesTextsCount['N'],
				'%%COUNT_ORDER_STATUSES_TEXTS%%' => $orderStatusesTextsCount['N'],
			));
		}
		
		protected function getMultiLangDbTables()
		{
			$sql = "SHOW TABLES LIKE '%ML'";
			$rows = miStaticDBUtil::execSelect($sql);
			
			$tables = array();
			foreach ($rows as $row) {
				$tableName = array_pop($row);
				if ($tableName == 'CustomUrlsML')
					continue;
				
				$tables[] = $tableName;
			}
			return $tables;
		}
		
		protected function copyLanguage($fromLanguageId, $toLanguageId)
		{
			$tables = $this->getMultiLangDbTables();
			$tables[] = 'TextsTranslated';
			
			foreach ($tables as $table) {
				$query = 'SELECT * FROM ' . $table . ' WHERE LanguageID=?';
				$rows = miStaticDBUtil::execSelect($query, array($fromLanguageId));
				
				foreach ($rows as $row) {
					$row['LanguageID'] = $toLanguageId;
					if ($table == 'TextsTranslated') {
						$row['TextIsTranslated'] = false;
					}
					miStaticDBUtil::execInsert($table, $row);
				}
			}
		}
		
		public function postProcess($actionObj, $actionResult)
		{
			if ($actionResult === false)
				return;
			
			if ($actionObj instanceof miExecCreateAction) {
				$basedOn = miGetParam('BasedOn');
				
				$domainObject = $this->getDomainObject();
				$pkValue = $domainObject->get($domainObject->getPKName());
				$this->copyLanguage($basedOn, $pkValue);
			}
			
			if ($actionObj instanceof miExecDeleteAction) {
				$tables = $this->getMultiLangDbTables();
				$tables[] = 'TextsTranslated';
				
				foreach ($tables as $table) {
					$query = 'DELETE FROM ' . $table . ' WHERE LanguageID = "' . sql_escape_string($pkValue) . '"';
					miStaticDBUtil::execSQL($query);
				}
			}
		}
	}
	
	class scLanguagePackLicenseWidget extends miBaseTextWidget {
		protected function getValue()
		{
			$value = trim($this->_webForm->getFormData($this->_fieldName));
			if (empty($value)) {
				return _i('This translation and language pack are free; you can use, redistribute and/or modify them according to the license published at http://www.summercart.com/licenses/translation');
			}
			return $value;
		}
	}
	
	/**
	 * Set the default language
	 */
	class scSetDefaultLanguageAction extends miAction {
		public function doAction()
		{
			$languageId = (int)miGetParam('LanguageID');
			scSettings::setSettingValue(scSettings::SETTING_DEFAULT_LANGUAGE, $languageId);
			
			$domainObject = $this->_view->getDomainObject();
			$domainObject->read($languageId);
			$domainObject->set('LanguageIsActive', true);
			$domainObject->update();
			
			$this->_view->addRedirectToListControllerCommand(_t('Default language successfully set.'), miMessage::MSG_TYPE_INFO);
			return true;
		}	
	}
	
	/**
	 * Delete the language
	 * Check that the language is not default language
	 */
	class scLanguageExecDeleteAction extends miExecDeleteAction  {
		public function doAction()
		{
			$languageId = (int)miGetParam('LanguageID');
			if (scSettings::getSettingValue(scSettings::SETTING_DEFAULT_LANGUAGE) == $languageId) {
				$this->_view->addRedirectToListControllerCommand(_t('Default language cannot be deleted.'), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			
			return parent::doAction();
		}
	}
	
	class scLanguageChangeIsActiveAction extends scExecChangeIsActiveAction {
		
		public function doAction()
		{
			$languageId = (int)miGetParam('LanguageID');
			if (scSettings::getSettingValue(scSettings::SETTING_DEFAULT_LANGUAGE) == $languageId) {
				$domainObject = $this->_view->getDomainObject();
				$domainObject->read($languageId);
				$becomeActive = $domainObject->get($this->_fieldName) ? false : true;
				if (!$becomeActive) {
					$this->_view->addRedirectToListControllerCommand(_t('Default language cannot be deactivated.'), miMessage::MSG_TYPE_ERROR);
					return false;
				}
			}
			
			return parent::doAction();
		}
	}
	
	class scUpdateUserInterfaceTextsAction extends miActionWithWebForm {
		const TEMPLATE_FILE = 'admin/languages/find_texts_stats.tmpl';
		
		public function doAction()
		{
			try {
				$findTexts = new scFindTexts;
				$stats = $findTexts->process();
				
				$t = new scTemplateParser();
				$t->readTemplate(self::TEMPLATE_FILE);
				
				$this->_view->initMainPageElements();
				miTable::escapeAndAssignArray($t, $this->_view->getMainPageElements());
				$t->assignArray(miUIUtil::templetizeRows($stats));
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $t->templateParse());
				$this->_view->addControllerCommand($cmd);
			} catch (Exception $e) {
				return $this->onFailure($e);
			}
		}
	}
	
	class scLanguagePackAction extends miExecEditAction {
		protected function getXliffFilesMap()
		{
			return array(
				'UserInterface' => 'user_interface.xlf',
				'Helps' => 'helps.xlf',
				'QuickLinks' => 'admin_quick_links.xlf',
				'Countries' => 'countries.xlf',
				'States' => 'states.xlf',
				'Currencies' => 'currencies.xlf',
				'OrderStatuses' => 'order_statuses.xlf',
			);
		}
		
		protected function getXliffExportClasses()
		{
			return array(
				'UserInterface' => 'scXliffExportUserInterfaceTexts',
				'Helps' => 'scXliffExportHelps',
				'QuickLinks' => 'scXliffExportQuickLinks',
				'Countries' => 'scXliffExportCountries',
				'States' => 'scXliffExportStates',
				'Currencies' => 'scXliffExportCurrencies',
				'OrderStatuses' => 'scXliffExportOrderStatuses',
			);
		}
		
		protected function getXliffImportClasses()
		{
			return array(
				'UserInterface' => 'scXliffImportUserInterfaceTexts',
				'Helps' => 'scXliffImportHelps',
				'QuickLinks' => 'scXliffImportQuickLinks',
				'Countries' => 'scXliffImportCountries',
				'States' => 'scXliffImportStates',
				'Currencies' => 'scXliffImportCurrencies',
				'OrderStatuses' => 'scXliffImportOrderStatuses',
			);
		}
		
		protected function downloadXliffFile($filename, $contents)
		{
			// Make sure there is no buffering
			while (ob_get_level())
				ob_end_clean();
			header('Content-Encoding: none');
			
			ob_start('ob_gzhandler');
			
			header('Pragma: public');
			header('Expires: 0');
			header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
//			header('Content-Type: application/force-download');
//			header('Content-Type: application/octet-stream');
			header('Content-Type: application/download');
			header('Content-Disposition: attachment; filename=' . $filename);
			header('Content-Transfer-Encoding: binary');
			
			echo $contents;
		}
	}
	
	class scExportLanguagePackAction extends scLanguagePackAction {
		public function doAction()
		{
			if (parent::doAction() == false)
				return;
			
			$entitiesMap = $this->getXliffExportClasses();
			$xliffFilesMap = $this->getXliffFilesMap();
			
			$languageId = miGetParam('LanguageID');
			$language = new scLanguageDomainObject;
			$language->read($languageId);
			
			// Create the language pack archive
			$tempFilename = tempnam('/tmp', 'scziplangpack' . uniqid());
			
			$zip = new ZipArchive;
			$result = $zip->open($tempFilename, ZIPARCHIVE::CREATE | ZipArchive::OVERWRITE);
			if (!$result) {
				throw new miException(_t('Could not create language pack.'));
			}
			
			foreach ($entitiesMap as $entity => $entityClass) {
				$xliffExport = new $entityClass();
				$fileContents = $xliffExport->export($languageId);
				$zip->addFromString($xliffFilesMap[$entity], $fileContents);
			}
			
			$zip->addFromString('langpack.xml', $this->getLangPackXml($zip, $language));
			
			// Create a zip file
			$zip->close();
			
			$zipContents = file_get_contents($tempFilename);
			unlink($tempFilename);
			
			$this->downloadXliffFile('summer_cart_lang_pack_' . strtolower($language->get('LanguageCode')) . '.zip', $zipContents);
			die;
		}
		
		protected function getLangPackXml($zip, scLanguageDomainObject $language)
		{
			$imageName = $language->get('LanguageImage');
			$imagePath = scPaths::getStoreRootDir() . $imageName;
			$localPath = '';

			try {
				if (file_exists($imagePath)) {
					$localPath = strtolower($language->get('LanguageCode')) . scImageHandler::getImageExtension($imagePath);
					$zip->addFile($imagePath, $localPath);
				}
			} catch (miException $e) {
				// Ignore exception - most likely the icon file is not an image
			}
			
			
			$publishDate = $language->get('LanguagePackPublishDate');
			if (($publishDate == '') or ($publishDate == '0000-00-00')) {
				$language->set('LanguagePackPublishDate', date('Y-m-d'));
				$language->update();
			}
			
			$definitions = array(
				array(
					scXml::TAG => 'LanguagePack',
					scXml::CHILDREN => array(
						array(
							scXml::TAG => 'Name',
							scXml::DATA => $language->get('LanguageName')
						),
						array(
							scXml::TAG => 'Alpha2Code',
							scXml::DATA => $language->get('LanguageCode')
						),
						array(
							scXml::TAG => 'Direction',
							scXml::DATA => ($language->get('LanguageDirection') == scLanguageDomainObject::LANGUAGE_DIRECTION_LTR) ? 'LTR' : 'RTL'
						),
						array(
							scXml::TAG => 'Icon',
							scXml::DATA => $localPath
						),
						array(
							scXml::TAG => 'Author',
							scXml::DATA => $language->get('LanguagePackAuthor')
						),
						array(
							scXml::TAG => 'Description',
							scXml::DATA => $language->get('LanguagePackDescription')
						),
						array(
							scXml::TAG => 'License',
							scXml::DATA => $language->get('LanguagePackLicense')
						),
						array(
							scXml::TAG => 'PublishDate',
							scXml::DATA => $language->get('LanguagePackPublishDate')
						),
					)
				)
			);
			return "<?xml version='1.0' encoding='utf-8'?>" . "\n" . scXml::xmlWriter($definitions);
		}
	}
	
	class scDownloadXliffAction extends scLanguagePackAction {
		public function doAction()
		{
			$xliffFilesMap = $this->getXliffFilesMap();
			
			$filename = miGetParam('filename');
			$languageId = miGetParam('LanguageID');
			
			$name = array_search($filename, $xliffFilesMap);
			if ($name === false) {
				throw new miException(_i('Invalid XLIFF filename.'));
			}
			
			$entitiesMap = $this->getXliffExportClasses();
			$className = $entitiesMap[$name];
			
			new scXliffExport; // fixme: autoload
			
			$xliffExport = new $className;
			
			$xliffContents = $xliffExport->export($languageId);
			
			$this->downloadXliffFile($filename, $xliffContents);
			die;
		}
	}
	
	class scInstallLanguagePackAction extends miCreateAction {
		public function doAction()
		{
			try {
				$this->_view->callPlugin($this, 'preShowForm');
			
				$form = $this->getWebForm();
				$this->showForm($form, $this->_view->getTemplateFileName('InstallLanguagePack'), true);
				return $this->onSuccess();
			} catch (miException $exception) {
				return $this->onFailure($exception);
			}
		}
	}
	
	class scExecInstallLanguagePackAction extends scLanguagePackAction {
		public function doAction()
		{
			try {
				miStaticDBUtil::startTransaction();
				
				if (($_SERVER['REQUEST_METHOD'] == 'POST') && empty($_POST) && ($_SERVER['CONTENT_LENGTH'] > 0)) {
					throw new miException(sprintf(_t('The uploaded file is too large (%s bytes).'), number_format($_SERVER['CONTENT_LENGTH'])));
				}
				
				$error = $_FILES['LanguagePackFile']['error'];
				if ($error == UPLOAD_ERR_NO_FILE) {
					throw new miException(_t('Please upload file.'));
				}
				
				$this->checkUpload('LanguagePackFile');
				
				$archiveZip = new scArchiveZip;
				if ($archiveZip->isZipArchive($_FILES['LanguagePackFile']['tmp_name'])) {
					$this->importLanguagePackZip($_FILES['LanguagePackFile']['tmp_name']);
					miStaticDBUtil::commit();
					return true;
				}
				
//				$filename = $_FILES['LanguagePackFile']['name'];
//				if (substr($filename, -4) == '.xlf') {
//					$this->importXliff($filename, $_FILES['LanguagePackFile']['tmp_name']);
//					return true;
//				}
				
				throw new miException(_t('You should upload language pack file (.zip).'));
				
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				
				$params = array(
					'action=acInstallLanguagePack',
					$this->_view->getRedirectParams($exception->getMessage(), miMessage::MSG_TYPE_ERROR)
				);
				$this->_view->addRedirectControllerCommand($params);
				return;
			}
		}
		
		protected function importLanguagePackZip($zipFilename)
		{
			$archiveZip = new scArchiveZip;
			$zipFiles = $archiveZip->getZipFiles($zipFilename);
			
			$language = $this->createLanguage($zipFiles);
			
			$status = array();
			foreach ($zipFiles as $zipFilename => $contents) {
				if (substr($zipFilename, -4) != '.xlf')
					continue;
				
				$xliffImport = $this->getXliffImportClass($zipFilename);
				try {
					$xliffImport->importLangFromXliffString($contents, $language->get('LanguageCode'));
				} catch (Exception $e) {
					throw new miException(sprintf(_t('Error processing "%s": %s'), $zipFilename, $e->getMessage()));
				}
				$status[] = $xliffImport;
			}
			
			$this->printInstallResult($language, $status);
		}
		
		/**
		 * Creates language based on language description in langpack.xml
		 * @param array $zipFiles
		 * @return scLanguageDomainObject $language
		 */
		protected function createLanguage($zipFiles)
		{
			if (!isset($zipFiles['langpack.xml']))
				throw new miException(_t('Cound not find langpack.xml in the language pack archive.'));
			
			$xml = simplexml_load_string($zipFiles['langpack.xml']);
			
			$langCode = (string)$xml->Alpha2Code;
			
			if (scLanguageDomainObject::checkLanguageExists($langCode))
				throw new miException(sprintf(_t('Language with code "%s" already exists.'), $langCode));
			
			// Check language image for correctness
			if (isset($xml->Icon) and !empty($xml->Icon)) {
				$languageImageInZip = (string)$xml->Icon;
				if (!isset($zipFiles[$languageImageInZip])) {
					throw new miException(sprintf(_t('Cannot find image file "%s" in the language pack archive.'), $languageImageInZip));
				}
			}
			
			$language = new scLanguageDomainObject;
			$language->set('LanguageName', (string)$xml->Name);
			$language->set('LanguageCode', $langCode);
			$language->set('LanguageDirection', ($xml->Direction == 'RTL') ? scLanguageDomainObject::LANGUAGE_DIRECTION_RTL : scLanguageDomainObject::LANGUAGE_DIRECTION_LTR);
			$language->set('LanguageIsActive', 0);
			$language->set('LanguagePackAuthor', (string)$xml->Author);
			$language->set('LanguagePackDescription', (string)$xml->Description);
			$language->set('LanguagePackLicense', (string)$xml->License);
			$language->set('LanguagePackPublishDate', (string)$xml->PublishDate);
			$languageId = $language->insert();
			
			// Add the language image
			if (isset($xml->Icon) and !empty($xml->Icon)) {
				$tempFilePath = tempnam('/tmp', 'language_image');
				file_put_contents($tempFilePath, $zipFiles[$languageImageInZip]);
				$imageFilePath = scPaths::getItemPath($languageId, scPaths::RESOURCE_LANGUAGE_IMAGE);
				
				$maxWidth = (int) scSettings::getSettingValue(scSettings::SETTING_LANGUAGE_IMAGE_MAX_WIDTH);
				$maxHeight = (int) scSettings::getSettingValue(scSettings::SETTING_LANGUAGE_IMAGE_MAX_HEIGHT);
				scImageHandler::resizeImage($tempFilePath, $imageFilePath, $maxWidth, $maxHeight);
				unlink($tempFilePath);
				
				$language->set('LanguageImage', scPaths::getItemPath($languageId, scPaths::RESOURCE_LANGUAGE_IMAGE, scPaths::PATH_WEB));
				list($width, $height) = scImageHandler::getImageSize($imageFilePath);
				$language->set('LanguageImageWidth', $width);
				$language->set('LanguageImageHeight', $height);
				$language->update();
			}
			
			return $language;
		}
		
		/**
		 * Import Xliff translation file
		 * @param string $filename the original name of the file (as received from the $_FILES info)
		 * @param string $filepath the path to the actual file
		 * @return unknown_type
		 */
		protected function importXliff($language, $filename, $filepath)
		{
			$xliffImport = $this->getXliffImportClass($filename);
			$xliffImport->importLangFromXliffFile($filepath, $language->get('LanguageCode'));
			
			// Print result
			$this->printInstallResult($language, array($xliffImport));
		}
		
		/**
		 * Generates status for the import of Xliffs
		 * @param scLanguageDomainObject $language
		 * @param scXliffImport[] $xliffImports
		 * @return array
		 */
		protected function printInstallResult(scLanguageDomainObject $language, $xliffImports)
		{
			$t = new scTemplateParser();
			$t->readTemplate('admin/languages/language_install_status.tmpl');
			
			$this->_view->initMainPageElements();
			miTable::escapeAndAssignArray($t, $this->_view->getMainPageElements());
			
			$textsToBeTranslatedCountsSections = array();
			$translatedCountsSections = array();
			$missingTextsSections = array();
			$unusedTextsSections = array();
			$rows = array();
			foreach ($xliffImports as $key => $xliffImport) {
				// Display translated counts
				$textsToBeTranslatedCounts = $xliffImport->getTextsToBeTranslatedCount();
				$counts = array();
				foreach ($textsToBeTranslatedCounts as $langCode => $textToBeTranslatedCount) {
					$counts[] = array('%%LANGCODE%%' => $langCode, '%%TEXTS_TO_BE_TRANSLATED_COUNT%%' => $textToBeTranslatedCount);
				}
				$textsToBeTranslatedCountsSections[] = new miTemplateParserSectionInfo('textToBeTranslatedCounts', count($counts), miUIUtil::transposeRows($counts));
				
				$translatedCounts = $xliffImport->getTranslatedCount();
				$counts = array();
				foreach ($translatedCounts as $langCode => $translatedCount) {
					$counts[] = array('%%LANGCODE%%' => $langCode, '%%TRANSLATED_COUNT%%' => $translatedCount);
				}
				$translatedCountsSections[] = new miTemplateParserSectionInfo('translatedCounts', count($counts), miUIUtil::transposeRows($counts));
				
				// Display missing translations
				$missingTranslations = $xliffImport->getMissingTranslations();
				$counts = array();
				foreach ($missingTranslations as $langCode => $missingTranslation) {
					$missingTranslationHtml = $this->getMissingTranslationsHtml($missingTranslation);
					$counts[] = array('LANGCODE' => $langCode, 'MISSING_COUNT' => count($missingTranslation), 'HTML_MISSING_TEXTS' => $missingTranslationHtml);
				}
				$missingTextsSections[] = new miTemplateParserSectionInfo('missingTranslations', count($counts), miUIUtil::templetizeRows(miUIUtil::transposeRows($counts)));
				
				// Display unused translations
				$unusedTranslations = $xliffImport->getUnusedTranslations();
				$counts = array();
				foreach ($unusedTranslations as $langCode => $unusedTranslation) {
					$unusedTranslationsHtml = $this->getUnusedTranslationsHtml($unusedTranslation);
					$counts[] = array('LANGCODE' => $langCode, 'UNUSED_COUNT' => count($unusedTranslation), 'HTML_UNUSED_TEXTS' => $unusedTranslationsHtml);
				}
				$unusedTextsSections[] = new miTemplateParserSectionInfo('unusedTranslations', count($counts), miUIUtil::templetizeRows(miUIUtil::transposeRows($counts)));
				
				$rows[] = array(
					'%%ENTITYNAME%%' => $xliffImport->getXliffBasename(),
					'%%KEY%%' => $key,
				);
			}
			
			$sectionInfo = new miTemplateParserSectionInfo('rows', count($xliffImports), miUIUtil::transposeRows($rows));
			$sectionInfo->addSubsectionsArray('textToBeTranslatedCounts', $textsToBeTranslatedCountsSections);
			$sectionInfo->addSubsectionsArray('translatedCounts', $translatedCountsSections);
			$sectionInfo->addSubsectionsArray('missingTranslations', $missingTextsSections);
			$sectionInfo->addSubsectionsArray('unusedTranslations', $unusedTextsSections);
			$t->addSectionInfo($sectionInfo);
			
			$t->assignArray(miUIUtil::templetizeAndEscapeRows($language->getRow()));
			$t->assign('%%LICENSE%%', scHtmlUtil::convertUrlsToLinks(miI18N::htmlEscape($language->get('LanguagePackLicense'))));
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $t->templateParse());
			$this->_view->addControllerCommand($cmd);
		}
		
		protected function getMissingTranslationsHtml($missingTranslation)
		{
			if (empty($missingTranslation))
				return '';
			
			$missingTranslation = array_keys($missingTranslation);
			sort($missingTranslation);
			$missingTranslation = array_map(array('miI18N', 'htmlEscape'), $missingTranslation);
			foreach ($missingTranslation as &$data)
				$data = '<li>' . $data . '</li>';
			
			return '<ul>' . implode('', $missingTranslation) . '</ul>';
		}
		
		protected function getUnusedTranslationsHtml($unusedTranslation)
		{
			if (empty($unusedTranslation))
				return '';
			
			sort($unusedTranslation);
			
			foreach ($unusedTranslation as &$data)
				$data = '<li>' . $data . '</li>';
			
			return '<ul>' . implode('', $unusedTranslation) . '</ul>';
		}
		
		/**
		 * 
		 * @param string $filename
		 * @return scXliffImport
		 */
		protected function getXliffImportClass($filename)
		{
			$filesMap = array_flip($this->getXliffFilesMap());
			if (!isset($filesMap[$filename]))
				throw new miException(sprintf(_t('Uploaded XLIFF filename "%s" is invalid.'), $filename));
			
			$entityName = $filesMap[$filename];
			
			$importClasses = $this->getXliffImportClasses();
			$importClass = $importClasses[$entityName];
			
			return new $importClass;
		}
		
		protected function checkUpload($name)
		{
			if ($_FILES[$name]['error'])
				throw new miException(sprintf(_t('Error uploading the file: %s (error code %s).'),
					scFilesUtil::getUploadErrorDescription($_FILES[$name]['error']), $_FILES[$name]['error']));
			
			if (!is_uploaded_file($_FILES[$name]['tmp_name']))
				throw new miException(_t('Internal error.'));
			
			if ($_FILES[$name]['size'] == 0)
				throw new miException(_t('The uploaded file is empty.'));
		}
	}
	
	class scUploadXliffAction extends scExecInstallLanguagePackAction {
		public function doAction()
		{
			try {
				miStaticDBUtil::startTransaction();
				
				if (($_SERVER['REQUEST_METHOD'] == 'POST') && empty($_POST) && ($_SERVER['CONTENT_LENGTH'] > 0)) {
					throw new miException(sprintf(_t('The uploaded file is too large (%s bytes).'), number_format($_SERVER['CONTENT_LENGTH'])));
				}
				
				$error = $_FILES['XliffFile']['error'];
				if ($error == UPLOAD_ERR_NO_FILE) {
					throw new miException(_t('Please upload file.'));
				}
				
				$this->checkUpload('XliffFile');
				
				$languageId = intval(miGetParam('LanguageID'));
				$language = new scLanguageDomainObject();
				$language->read($languageId);
				
				$filename = $_FILES['XliffFile']['name'];
				if (substr($filename, -4) != '.xlf') {
					throw new miException(_t('You should upload translation file (.xlf).'));
				}
				
				$this->importXliff($language, $filename, $_FILES['XliffFile']['tmp_name']);
				
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				
				$params = array(
					'action=dmEdit&LanguageID=' . intval(miGetParam('LanguageID')),
					$this->_view->getRedirectParams($exception->getMessage(), miMessage::MSG_TYPE_ERROR)
				);
				$this->_view->addRedirectControllerCommand($params);
				return false;
			}
			
			miStaticDBUtil::commit();
			return true;
		}
	}
	
	class scLanguagePackPublishDate extends scDateWidget {
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = miGetParamDefault($this->_fieldName, '');
			if (($fieldValue == '') or ($fieldValue == '00/00/0000'))
				return;
			
			parent::validateData($errors);
		}
	} 
	
	class scLanguagesTable extends scTable {
		protected function addUiElements(&$rows)
		{
			$countTranslatedTexts = $this->getTranslatedInterfaceTextsCounts();
			$allInterfaceTexts = $this->getAllInterfaceTextsCounts();
			
			$defaultLanguageId = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_LANGUAGE);
			foreach ($rows as &$row) {
				$row['DefaultLanguage'] = ($row['LanguageID'] == $defaultLanguageId) ? 1 : 0;
				
				$numTranslated = isset($countTranslatedTexts[$row['LanguageID']]) ? $countTranslatedTexts[$row['LanguageID']] : 0;
				if ($allInterfaceTexts > 0)
					$row['InterfaceTexts'] = sprintf(_t('%d%% translated'), (100 * $numTranslated) / $allInterfaceTexts);
				else
					$row['InterfaceTexts'] = '';
				
				$langImageFile = $row['LanguageImage'];
				if (!empty($langImageFile) and file_exists(dirname(__FILE__) . '/../' . $langImageFile))
					$row['LanguageImage'] = '../' . $langImageFile;
				else
					$row['LanguageImage'] = '../' . scSkins::singleton()->getCurrentSkin()->getSkinPath(false) . '/admin/images/spacer.gif';
			}
		}
		
		protected function getAllInterfaceTextsCounts()
		{
			$result = scUtil::getDbFirstRow('SELECT COUNT(1) AS N FROM Texts');
			return $result['N'];
		}
		
		protected function getTranslatedInterfaceTextsCounts()
		{
			$sql = 'SELECT LanguageID,COUNT(*) AS N FROM TextsTranslated WHERE TextIsTranslated GROUP BY LanguageID';
			return miUtil::getDBArray($sql, 'LanguageID', 'N');
		}
	}
	
	$languages = new miSqlRecordset('Languages');
	$table = new scLanguagesTable($languages);
	
	$view = new scLanguagesView();
	$view->setUpSortColumn('LanguageSort');
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmExecDelete', 'scLanguageExecDeleteAction');
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scLanguageChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('setDefaultLanguage', 'scSetDefaultLanguageAction');
	$page->getDispatcher()->setActionHandler('updateUserInterfaceTexts', 'scUpdateUserInterfaceTextsAction');
	$page->getDispatcher()->setActionHandler('acExportLanguagePack', 'scExportLanguagePackAction');
	$page->getDispatcher()->setActionHandler('acDownloadXliff', 'scDownloadXliffAction');
	$page->getDispatcher()->setActionHandler('acInstallLanguagePack', 'scInstallLanguagePackAction');
	$page->getDispatcher()->setActionHandler('acExecInstallLanguagePack', 'scExecInstallLanguagePackAction');
	$page->getDispatcher()->setActionHandler('acUploadXliff', 'scUploadXliffAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>