<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_LAYOUTS);

	class scCMSLayoutsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/layouts/layout_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/layouts/layout_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/layouts/layout_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSLayoutID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSLayoutName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name'
        	),
        	array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSLayoutCssClassNames'
        	),
        	array(
				'field' => 'miBaseSelectWidget',
				'data' => 'CMSLayoutArea'
        	)
        );
        
        protected $_domainObjectDisplayName = 'Layout';	// _t('Layout');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		/**
		 * Creates the custom domain object 
		 */
		protected function createDomainObject()
		{
			return new scLayoutDomainObject(new miSqlRecord('CMSLayouts', 'CMSLayoutID'));
		}
		
		/**
		 * Read boxes and their layouts from DB
		 * 
		 * @return array the boxes rows
		 */
		protected function getBoxes()
		{
			$boxesRecordset = new miSqlRecordset('CMSBoxes');
			$boxesRecordset->addFilter(new miSqlFilterEqual('CMSBoxArea', scLayout::AREA_CUSTOMER));
			$boxesRecordset->setOrder('CMSBoxID', 'ASC');
			$layoutId = miGetParamDefault('CMSLayoutID', 0);
			if ($layoutId) {
				// read boxes layout too
				$boxesRecordset->addJoinCondition('LEFT', 'CMSLayoutBoxes', 'ON CMSLayoutBoxes.CMSBoxID=CMSBoxes.CMSBoxID AND CMSLayoutBoxes.CMSLayoutID=' . intval($layoutId));
				$selectFields[] = 'CMSBoxes.CMSBoxName';
				$selectFields[] = 'CMSBoxes.CMSBoxID';
				$selectFields[] = 'CMSLayoutBoxes.CMSLayoutBoxColumn';
				$boxesRecordset->setOrder('CMSLayoutBoxSort', 'ASC');
				$boxesRecordset->setSelectFields($selectFields);
				
				$boxesRecordset->addFilter(new miSqlFilterNotEqual('CMSBoxIsActive', 0));
			}
			return $boxesRecordset->getAllRecords();
		}
		
		/**
		 * Add boxes template section info to the form
		 * 
		 * @param miWebForm $form
		 * @param array $boxes the boxes rows
		 */
		protected function addBoxesSection(&$form, $boxes)
		{
			$topBoxes = $bottomBoxes = $leftBoxes = $rightBoxes = $availableBoxes = array();
			
			foreach ($boxes as $box) {
				$column = isset($box['CMSLayoutBoxColumn']) ? $box['CMSLayoutBoxColumn'] : 0;
				switch ($column) {
					case scLayout::COLUMN_TOP:
						$topBoxes[] = $box;
						break;
					case scLayout::COLUMN_BOTTOM:
						$bottomBoxes[] = $box;
						break;
					case scLayout::COLUMN_LEFT:
						$leftBoxes[] = $box;
						break;
					case scLayout::COLUMN_RIGHT:
						$rightBoxes[] = $box;
						break;
					default:
						$availableBoxes[] = $box;
						break;
				}
			}
			
			$columnVars = array(
				'%%COLUMNNAME%%' => array(
					_t('Available'), 
					_t('Top'), 
					_t('Left'), 
					_t('Right'), 
					_t('Bottom')
				), 
				'%%COLUMNID%%' => array(
					'AvailableColumn', 
					'TopColumn', 
					'LeftColumn', 
					'RightColumn', 
					'BottomColumn'
				)
			);
			$columnSection = new miTemplateParserSectionInfo('columns', 5, $columnVars);
			
			// Set boxes sections
			$boxesTopSection = new miTemplateParserSectionInfo('boxes', count($topBoxes), miUIUtil::templetizeRows(miUIUtil::transposeRows($topBoxes)));
			$boxesBottomSection = new miTemplateParserSectionInfo('boxes', count($bottomBoxes), miUIUtil::templetizeRows(miUIUtil::transposeRows($bottomBoxes)));
			$boxesLeftSection = new miTemplateParserSectionInfo('boxes', count($leftBoxes), miUIUtil::templetizeRows(miUIUtil::transposeRows($leftBoxes)));
			$boxesRightSection = new miTemplateParserSectionInfo('boxes', count($rightBoxes), miUIUtil::templetizeRows(miUIUtil::transposeRows($rightBoxes)));
			$boxesAvailableSection = new miTemplateParserSectionInfo('boxes', count($availableBoxes), miUIUtil::templetizeRows(miUIUtil::transposeRows($availableBoxes)));
			$columnSection->addSubsectionsArray('boxes', array($boxesAvailableSection, $boxesTopSection, $boxesLeftSection, $boxesRightSection, $boxesBottomSection));
			
			$form->addTemplateSectionInfo($columnSection);
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);	
			
			// Set area field options
			$areas = scLayout::getAreas();
			unset($areas[scLayout::AREA_ADMIN]);
			$form->getWidget('CMSLayoutArea')->setOptions($areas);
			
			// Setup boxes
			$this->addBoxesSection($form, $this->getBoxes());
		}
		
		/**
		 * Insert the boxes layout records
		 * 
		 * @param array $submittedColumn array(sort => cmsBoxId)
		 * @param int $column
		 * @param int $layoutId
		 */
		protected function insertBoxesLayout($submittedColumn, $column, $layoutId)
		{
			foreach ($submittedColumn as $sort => $cmsBoxId) {
				$values = array(
					'CMSLayoutID' => $layoutId,
					'CMSBoxID' => $cmsBoxId,
					'CMSLayoutBoxColumn' => $column,
					'CMSLayoutBoxSort' => $sort
				);
				miStaticDBUtil::execInsert('CMSLayoutBoxes', $values);
			}
		}
		
		/**
		 * Save the boxes layout
		 * 
		 * @param int $layoutId
		 */
		protected function saveBoxesLayout($layoutId)
		{
			try {
				miStaticDBUtil::startTransaction();
				
				// Delete the current box layout records
				miStaticDBUtil::execDelete('CMSLayoutBoxes', 'CMSLayoutID', $layoutId);
				
				// Insert
				$boxLayouts = scUrlUtil::parseStr(miGetParam('BoxLayouts'));
				if (isset($boxLayouts['TopColumn']))
					$this->insertBoxesLayout($boxLayouts['TopColumn'], scLayout::COLUMN_TOP, $layoutId);
				if (isset($boxLayouts['BottomColumn']))
					$this->insertBoxesLayout($boxLayouts['BottomColumn'], scLayout::COLUMN_BOTTOM, $layoutId);
				if (isset($boxLayouts['LeftColumn']))
					$this->insertBoxesLayout($boxLayouts['LeftColumn'], scLayout::COLUMN_LEFT, $layoutId);
				if (isset($boxLayouts['RightColumn']))
					$this->insertBoxesLayout($boxLayouts['RightColumn'], scLayout::COLUMN_RIGHT, $layoutId);

				miStaticDBUtil::commit();
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				
				$msg = _t('Failed to save boxes layout: ');
				$this->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
			}
		}
		
		public function postProcess($actionObj, $actionResult)
		{			
			if ($actionResult === false)
				return;
				
			// Save Layout Boxs
			if (($actionObj instanceof miExecEditAction) or ($actionObj instanceof miExecCreateAction)) {
				$this->saveBoxesLayout($this->getDomainObject()->get('CMSLayoutID'));
			}
		}
	}
	
	class scMakeDefaultAction extends miAction {
		public function doAction()
		{
			$layoutId = miGetParam('CMSLayoutID');
			
			miStaticDBUtil::execSQL('UPDATE CMSLayouts SET CMSLayoutIsDefault = 0');
			miStaticDBUtil::execSQL('UPDATE CMSLayouts SET CMSLayoutIsDefault = 1 WHERE CMSLayoutID = ' . $layoutId);
			
			$this->_view->addRedirectToListControllerCommand(_t('Default layout successfully changed.'), miMessage::MSG_TYPE_INFO);
			
			return true;
		}
	}
	
	class scLayoutDomainObject extends miDefaultDomainObject {
		/**
		 * Delete the domain object
		 */
		public function delete($pkValue = null)
		{
			$this->checkForeignKeys($pkValue);
			
			if ($pkValue !== null)
				$this->_record->set($this->_record->getPrimaryKeyColumn(), $pkValue);
			
			// Before deleting the domain object manage the default flag
			$record = clone $this->_record;
			$record->readPK($record->getPK());
			if ($record->get('CMSLayoutIsDefault'))
				miStaticDBUtil::execSQL('UPDATE CMSLayouts SET CMSLayoutIsDefault = 1 LIMIT 1');

			return $this->_record->delete();
		}
		
		protected function checkForeignKeys($pkValue)
		{
			$rows = miStaticDBUtil::execSelect('SELECT COUNT(CategoryID) AS NumberOfRows FROM Categories WHERE CMSLayoutID = ?', array($pkValue));
			$rows = reset($rows);
			$total = intval($rows['NumberOfRows']);
			
			$rows = miStaticDBUtil::execSelect('SELECT COUNT(CMSDynamicPageID) AS NumberOfRows FROM CMSDynamicPages WHERE CMSLayoutID = ?', array($pkValue));
			$rows = reset($rows);
			$total += intval($rows['NumberOfRows']);
			
			$rows = miStaticDBUtil::execSelect('SELECT COUNT(CMSRichTextPageID) AS NumberOfRows FROM CMSRichTextPages WHERE CMSLayoutID = ?', array($pkValue));
			$rows = reset($rows);
			$total += intval($rows['NumberOfRows']);
			
			if ($total > 0) {
				throw new miException(_t('To delete this layout, no dynamic pages, categories or rich text pages should refer to it.'));
			}
		}
	}
	
	$CMSLayouts = new miSqlRecordset('CMSLayouts');
	$table = new scTable($CMSLayouts);
	
	$view = new scCMSLayoutsView();
	$view->setRecord(new miSqlRecord('CMSLayouts', 'CMSLayoutID'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmMakeDefault', 'scMakeDefaultAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>