<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_MANUFACTURERS);

	class scManufacturersView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/manufacturers/manufacturer_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/manufacturers/manufacturer_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/manufacturers/manufacturer_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ManufacturerID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'ManufacturerName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name',
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'ManufacturerDescription',
				'displayName' => 'Description'
        	),
       		array(
				'field' => 'scImageUploadWidget',
				'data' => 'ManufacturerImage',
				'modelStrategy' => 'scViewMapperUploadImage',
				'properties' => array(
					'resource' => scPaths::RESOURCE_MANUFACTURER_IMAGE,
					'imageMaxWidthSetting' => scSettings::SETTING_MANUFACTURER_IMAGE_MAX_WIDTH,
					'imageMaxHeightSetting' => scSettings::SETTING_MANUFACTURER_IMAGE_MAX_HEIGHT,
					'imageWidthField' => 'ManufacturerImageWidth',
					'imageHeightField' => 'ManufacturerImageHeight',
					'noThumb' => true
				),
			),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ManufacturerWebsite',
				'validator' => 'scValidatorHttp',
				'displayName' => 'Website'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'ManufacturerSEOTitle',
				'displayName' => 'SEO Title'
        	),
        	array(
				'field' => 'scCustomUrlsWidget',
				'data' => 'CustomUrls',
				'modelStrategy' => 'scViewMapperManufacturerCustomUrls',
				'properties' => array(
					'template' => 'admin/custom_urls/custom_url.tmpl',
					
					'table' => 'CustomUrls',
					'foreignKey' => 'CustomUrlEntityID',
					'foreignTablePrimaryKey' => 'CustomUrlID',
					
					'fields' => array(
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CustomUrlText',
			       			'properties' => array('minLength' => 1),
			       			'validator' => 'scValidatorUniqueCustomUrl'
			        	),
			       		array(
							'field' => 'miCheckboxWidget',
							'data' => 'CustomUrlIsCurrent'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomUrlID'
			        	)
			        ),
			        'mlFields' => array('CustomUrlText')
				),
			)
        );
		
        protected $_domainObjectDisplayName = 'Manufacturer';	// _t('Manufacturer');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			if ($this->_controller->getDispatcher()->getAction() == 'dmEdit')
				$form->getWidget('ManufacturerImage')->setApplyButton();
		}
		
		protected function createDomainObject()
		{
			return new scManufacturerDomainObject();
		}
	}
	
	class scViewMapperManufacturerCustomUrls extends scViewMapperCustomUrls {
		protected $_entityCode = scFriendlyUrl::TYPE_MANUFACTURER;
		protected $_tableKeyName = 'ManufacturerID';
	}
	
	$manufacturers = new scSqlRecordsetML('Manufacturers', 'ManufacturerID');
	$table = new scTable($manufacturers);
	
	$view = new scManufacturersView();
	$view->setUpSortColumn('ManufacturerSort');
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>