<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_MENUS);

	class scCMSMenuItemsView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/menu_items/menu_item_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/menu_items/menu_item_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/menu_items/menu_item_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSMenuItemID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CMSMenuItemName',
				'validator' => 'scValidatorNoEmpty'
        	),
       		array(
				'field' => 'miBaseRadioWidget',
				'data' => 'CMSMenuItemLinkType',
				'validator' => 'scValidatorMenuItemLinkType',
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSMenuItemURL',
        	),
       		array(
				'field' => 'miBaseSelectWidget',
				'data' => 'CMSRichTextPageID'
        	),
       		array(
				'field' => 'miBaseSelectWidget',
				'data' => 'CMSFormID'
        	),
        	array(
				'field' => 'miBaseSelectWidget',
				'data' => 'GalleryID'
        	),
        	array(
				'field' => 'miBaseSelectWidget',
				'data' => 'CMSDynamicPageID'
        	),
			array(
				'field' => 'scMenuItemImageWidget',
				'data' => 'CMSMenuItemImage',
				'modelStrategy' => 'scViewMapperCmsMenuItemImage'
        	),
			array(
				'field' => 'scWebFormFieldHidden',
				'data' => 'CMSMenuID'
        	)
        );
        
        protected $_domainObjectDisplayName = 'Menu item';	// _t('Menu item');

		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Set CMSRichTextPageID options
			$languageId = scMultiLanguage::getCurrentLanguageId();
			$sql = 	'SELECT CMSRichTextPages.CMSRichTextPageID, CMSRichTextPageTitle FROM CMSRichTextPages ' . 
					'LEFT JOIN CMSRichTextPagesML ON CMSRichTextPages.CMSRichTextPageID = CMSRichTextPagesML.CMSRichTextPageID ' .
					'AND LanguageID = ' . $languageId . ' ORDER BY CMSRichTextPageTitle';
			$richTextPage = miUtil::getDBArray($sql, 'CMSRichTextPageID', 'CMSRichTextPageTitle');
			$field = $form->getWidget('CMSRichTextPageID');
			$field->setOptions(array('' => '') + $richTextPage);
			
			// Set CMSFormID options
			$sql = 'SELECT CMSFormID,CMSFormName FROM CMSForms ORDER BY CMSFormName';
			$cmsForms = miUtil::getDBArray($sql, 'CMSFormID', 'CMSFormName');
			$field = $form->getWidget('CMSFormID');
			$field->setOptions(array('' => '') + $cmsForms);
			
			// Set GalleryID options
			$sql = 'SELECT GalleryID, GalleryName FROM Galleries ORDER BY GalleryName';
			$galleries = miUtil::getDBArray($sql, 'GalleryID', 'GalleryName');
			$field = $form->getWidget('GalleryID');
			$field->setOptions(array('' => '') + $galleries);
			
			// Set CMSDynamicPageID options
			$field = $form->getWidget('CMSDynamicPageID');
			$field->setOptions(array('' => '') + scDynamicPages::getLinkablePageNames());
			
			$field = $form->getWidget('CMSMenuItemLinkType');
			$field->setRadioButtons(scUserMenuItem::getLinkTypes(), scUserMenuItem::LINK_TYPE_URL);

			$form->addMainPageElements(
				array(
					'%%MENUITEMIMAGEMAXWIDTH%%' => miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_MENU_ITEM_IMAGE_MAX_WIDTH)),
					'%%MENUITEMIMAGEMAXHEIGHT%%' => miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_MENU_ITEM_IMAGE_MAX_HEIGHT))
				)
			);
		}
		
		protected function createDomainObject()
		{
			return new scMenuItemDomainObject($this->_record);
		}
		
		public function preProcess($action)
		{
			if (!miHasParam('CMSMenuID')) {
				$location = 'menus.php?' . $this->getRedirectParams($msg, $msgType);
				$this->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $location));
			}
				
			parent::preProcess($action);
		}
		
		public function addRedirectToListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = 'menus.php?action=dmEdit&CMSMenuID=' . miGetParam('CMSMenuID') . '&' . $this->getRedirectParams($msg, $msgType);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
		
		public function addAjaxErrorRedirect($msg)
		{
			$loc = 'menus.php?action=dmEdit&CMSMenuID=' . miGetParam('CMSMenuID') . '&' . $this->getRedirectParams($msg, miMessage::MSG_TYPE_ERROR);
			$html = '<script>window.location.href="' . $loc . '";</script>';
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->addControllerCommand($cmd);
		}
		
		public function addAjaxRefreshListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = 'menus.php?action=dmEdit&CMSMenuID=' . miGetParam('CMSMenuID') . '&' . $this->getRedirectParams($msg, $msgType);
			$html = '<script>window.location.href="' . $loc . '";</script>';
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->addControllerCommand($cmd);
		}
	}
	
	class scValidatorMenuItemLinkType extends miValidator {
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			parent::validate($errors);
			if ($errors->hasFieldErrors($this->_fieldName)) {
				return;
			}
			
			$type = $this->_webForm->getSubmittedData($this->_fieldName);
			switch ($type) {
				case scUserMenuItem::LINK_TYPE_URL:
					$field = 'CMSMenuItemURL';
					$value = $this->_webForm->getSubmittedData($field);
					if (empty($value)) {
						$errors->addError($field, _t('Field cannot be empty.'));
					}
					return;
				
				case scUserMenuItem::LINK_TYPE_RICH_TEXT_PAGE:
					$field = 'CMSRichTextPageID';
					break;
				
				case scUserMenuItem::LINK_TYPE_FORM:
					$field = 'CMSFormID';
					break;
				
				case scUserMenuItem::LINK_TYPE_GALLERY:
					$field = 'GalleryID';
					break;
				
				case scUserMenuItem::LINK_TYPE_DYN_PAGE:
					$field = 'CMSDynamicPageID';
					break;
			}
			
			$value = $this->_webForm->getSubmittedData($field);
			if (empty($value)) {
				$errors->addError($field, _t('A value has to be selected.'));
			}
		}
	}
	
	$menuId = (int)miGetParamDefault('CMSMenuID', 0);
	
	miSetParam(miTablePager::PARAM_RECORDS_PER_PAGE, PHP_INT_MAX);
	
	$cmsMenuItems = scCMS::getCMSMenuItemsRecordset();
	$cmsMenuItems->addFilter(new miSqlFilterEqual('CMSMenuID', $menuId));
	
	$table = new scMenuItemsTable($cmsMenuItems);
	$customParams = new miTableCustomParams($table);
	$customParams->setRecordLocatorParams(array('CMSMenuID' => $menuId));
	
	$view = new scCMSMenuItemsView();
	$view->setRecord(new scSqlRecordML('CMSMenuItems', 'CMSMenuItemID',
		array('CMSMenuItemName'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
	$view->setUpSortColumn('CMSMenuItemSort');
	scSortAction::setForeignKeyClause('CMSMenuID=' . intval($menuId));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->dispatchAndShow();
?>