<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_MENUS);

	class scCMSMenusView extends scView {
		
		const TEMPLATE_ID_LIST_ITEMS = 'listItem';
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/menus/menu_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/menus/menu_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/menus/menu_create.tmpl',
			
			self::TEMPLATE_ID_LIST_ITEMS => 'admin/menu_items/menu_item_mgmt.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSMenuID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSMenuName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name'
        	),
       		array(
				'field' => 'miSelectWidget',
				'data' => 'CMSMenuLocation'
        	),
			array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CMSMenuTitle',
				'displayName' => 'Title'
        	),
			array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'CMSMenuIsActive'
        	)
        );
        
        protected $_domainObjectDisplayName = 'Menu';	// _t('Menu');

		protected function createDomainObject()
		{
			return new scMenuDomainObject();
		}
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
//			$form->addValidator('CMSMenuName', new scValidatorUniqueFields($form, array('CMSMenuName'), $this->getRecord()));
			
			$field = $form->getWidget('CMSMenuLocation');
			$field->setOptions(scUserMenus::getMenuLocations());
		}
		
		public function preProcess($actionObj)
		{
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('CMSMenuIsActive');
		}
	}
	
	/**
	 * Show the page for updating a record in the database
	 */
	class scEditAction extends miEditAction {
		public function doAction()
		{
			try {
				$CMSMenuID = (int)miGetParam('CMSMenuID');
				$cmsMenuItems = scCMS::getCMSMenuItemsRecordset();
				$cmsMenuItems->addFilter(new miSqlFilterEqual('CMSMenuID', $CMSMenuID));
				
				$table = new scMenuItemsTable($cmsMenuItems);
				$table->setSortingState(1);
				$table->assign('%%CMSMENUID%%', $CMSMenuID);
				$table->assign('%%LANGUAGEID%%', scView::getCurrentDataLanguageId());
				$menuItemsHtml = $table->parse($this->_view->getTemplateFileName(scCMSMenusView::TEMPLATE_ID_LIST_ITEMS));
				
				$this->_view->addMainPageElements(array('%%HTML_MENUITEMS%%' => $menuItemsHtml));
				
				parent::doAction();
				return true;
				
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_LIST_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	/**
	 * 
	 */
	class scMenusTable extends scTable {
		public function addUiElements(&$rows)
		{
			parent::addUiElements($rows);
			
			$menuLocations = scUserMenus::getMenuLocations();
			foreach ($rows as &$row)
				$row['CMSMenuLocation'] = isset($menuLocations[$row['CMSMenuLocation']]) ? $menuLocations[$row['CMSMenuLocation']] : '';
		}
	}
	
	$CMSMenus = new scSqlRecordsetML('CMSMenus', 'CMSMenuID');
	$table = new scMenusTable($CMSMenus);
	
	$view = new scCMSMenusView();
	$view->setRecord(new scSqlRecordML('CMSMenus', 'CMSMenuID',
		array('CMSMenuTitle'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmEdit', 'scEditAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>