<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_OPTION_GROUPS);

	class scOptionGroupsView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/option_groups/option_group_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/option_groups/option_group_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/option_groups/option_group_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'OptionGroupID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'OptionGroupName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name'
        	),
       		array(
				'field' => 'miSelectWidget',
				'data' => 'OptionGroupType',
				'displayName' => 'Type'
        	),
       		array(
				'field' => 'scOptionGroupsDynRowsWidget',
				'data' => 'RSOptions',
				'modelStrategy' => 'scViewMapperOptionGroupsDynRows',
				'properties' => array(
					'template' => 'admin/option_groups/options_rs.tmpl',
       				
					'table' => 'Options',
					'foreignKey' => 'OptionGroupID',
					'foreignTablePrimaryKey' => 'OptionID',
       		
       				'sortField' => 'OptionSort',
       				'sortBy' => 'OptionSort',
       				
					'fields' => array( 
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'OptionID'
		        		),
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'OptionName',
			       			'properties' => array('minLength' => 1)
		        		),
			       		array(
							'field' => 'miBaseRadioWidget',
							'data' => 'OptionPriceModType'
		        		),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'OptionPriceModValue'
		        		),
			       		array(
							'field' => 'miBaseCheckboxWidget',
							'data' => 'OptionRequired'
		        		)
		        	),
					'mlFields' => array('OptionName'), // Multi-language fields
				)
        	),
        );
        
        protected $_domainObjectDisplayName = 'Option group';	// _t('Option group');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$widget = $form->getWidget('OptionGroupType');
			$widget->setOptions(scProductOptions::getGroupTypes());
		}
		
		protected function createDomainObject()
		{
			return new scOptionGroupDomainObject();
		}
	}
	
	class scOptionGroupsDynRowsWidget extends scDynRowsWidget {
		protected function createFieldObjects()
		{
			parent::createFieldObjects();
			
			// Initialize PriceModType radio buttons
			$rsFields = $this->getRsFieldObjs('OptionPriceModType');
			foreach ($rsFields as &$field)
				$field->setRadioButtons(scProductOptions::getOptionTypes(), scProductOptions::PRODUCT_OPTION_TYPE_AMOUNT);
		}
	}
	
	class scViewMapperOptionGroupsDynRows extends scViewMapperDynRows {
		
		protected $_newOptionIds = array();
		
		public function update()
		{
			$return = parent::update();
			
			// Regenerate variants that are using this global option group
			$productIds = $this->getProductsToRegenerate();
			if (empty($productIds)) {
				return;
			}
			
			$globals = $this->getGlobalOptionsForProducts($productIds);
			$locals = $this->getLocalOptionsForProducts($productIds);
			
			foreach ($productIds as $productId) {
				$product = new scProductDomainObject();
				$product->read($productId);
				
				$variantGroupsArray = array();
				if (isset($globals[$productId])) {
					$variantGroupsArray = array_merge($variantGroupsArray, $globals[$productId]);
				}
				if (isset($locals[$productId])) {
					$variantGroupsArray = array_merge($variantGroupsArray, $locals[$productId]);
				}
				
				$variantGroups = new scSet();
				foreach ($variantGroupsArray as $group) {
					$variantGroups = $variantGroups->cartesianProduct($group);
				}
				scProductVariantsUtil::generateVariants($product, $variantGroups);
			}
			
			return $return;
		}
		
		protected function insertRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			$optionId = $record->insert();
			$this->_newOptionIds[] = $optionId;
			return $optionId;
		}
		
		protected function updateRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			$optionId = parent::updateRecord($record, $rsValues, $rsProperties, $rowIndex);
			if ($this->getViewMapperController()->getDomainObject()->getJustBecameVariant()) {
				$this->_newOptionIds[] = $optionId;
			}
			return $optionId;
		}
		
		protected function setValuesToRecord($record, $values, $rsProperties, $rowIndex)
		{
			if ($this->getViewMapperController()->getDomainObject()->get('OptionGroupType') != scProductOptions::GROUP_TYPE_PRICE_MODIFIER) {
				$values['OptionPriceModType'] = scProductOptions::PRODUCT_OPTION_TYPE_NONE;
				$values['OptionPriceModValue'] = 0;
			}
			
			return parent::setValuesToRecord($record, $values, $rsProperties, $rowIndex);
		}
		
		protected function getProductsToRegenerate()
		{
			$optionGroup = $this->_viewMapperController->getView()->getDomainObject();
			$optionGroupId = $optionGroup->get($optionGroup->getPKName());
			
			$recordset = new miSqlRecordset('ProductOptionGroups');
			$recordset->addFilter(new miSqlFilterEqual('OptionGroupID', $optionGroupId));
			$recordset->setSelectFields(array('ProductID'));
			$productIds = $recordset->getAllRecords();
			if (empty($productIds)) {
				return array();
			}
			return miUtil::getKeyValueArray($productIds, 'ProductID', 'ProductID');
		}
		
		protected function getGlobalOptionsForProducts($productIds)
		{
			// The group that is currently being modified.
			$currentGroup = $this->_viewMapperController->getView()->getDomainObject();
			
			$recordset = new miSqlRecordset('ProductOptionGroups');
			$recordset->addFilter(new miSqlFilterIn('ProductID', $productIds));
			$records = $recordset->getAllRecords();
			
			// One global option may be assigned to more than one product.
			$optionGroups = array();
			foreach ($records as $row) {
				if (!empty($this->_newOptionIds) && ($row['OptionGroupID'] == $currentGroup->get('OptionGroupID'))) {
					if ($currentGroup->get('OptionGroupType') != scProductOptions::GROUP_TYPE_VARIANT) {
						continue;
					}
					// Get the group's options.
					// Will set which ones are new so variant generation can be done with respect to that.
					$options = $currentGroup->getValues();
					foreach ($options as &$option) {
						if (in_array($option[0]->getOptionId(), $this->_newOptionIds)) {
							$option[0]->setIsNew(true);
						}
					}
					unset($option);
					// Set them in the back in the group.
					$currentGroup->setValues($options);
					$optionGroups[$row['OptionGroupID']] = $currentGroup;
					continue;
				}
				$object = new scOptionGroupDomainObject();
				$object->read($row['OptionGroupID']);
				if ($object->get('OptionGroupType') != scProductOptions::GROUP_TYPE_VARIANT) {
					continue;
				}
				$optionGroups[$row['OptionGroupID']] = $object;
			}
			
			// Group by product
			$result = array();
			foreach ($records as $row) {
				if (isset($optionGroups[$row['OptionGroupID']])) {
					$result[$row['ProductID']][] = $optionGroups[$row['OptionGroupID']];
				}
			}
			return $result;
		}
		
		protected function getLocalOptionsForProducts($productIds)
		{
			$recordset = new miSqlRecordset('LocalOptionGroups');
			$recordset->addFilter(new miSqlFilterIn('ProductID', $productIds));
			$records = $recordset->getAllRecords();
			
			// Group by product
			$result = array();
			foreach ($records as $row) {
				$object = new scLocalOptionGroupDomainObject();
				$object->read($row['LocalOptionGroupID']);
				$result[$row['ProductID']][] = $object;
			}
			return $result;
		}
	}
	
	$optionGroups = new scSqlRecordsetML('OptionGroups', 'OptionGroupID');
	$table = new scOptionGroupsTable($optionGroups);
	
	$view = new scOptionGroupsView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>