<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_ORDER_STATUSES);
	
	class scOrderStatusesView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/order_statuses/order_status_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/order_statuses/order_status_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/order_statuses/order_status_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'OrderStatusID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'OrderStatusName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name'
        	),
       		array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'OrderStatusIsActive'
        	),
       		array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'OrderStatusSendNotification'
        	),
       		array(
				'field' => 'miSelectWidget',
				'data' => 'OrderStatusInventoryAction'
        	),
        );
        
        protected $_domainObjectDisplayName = 'Order status';	// _t('Order status');
        
		public function initTable($table)
		{
			parent::initTable($table);
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$widget = $form->getWidget('OrderStatusInventoryAction');
			$widget->setOptions(scOrderStatusInventoryAction::singleton()->getAllActions(), scOrderStatusInventoryAction::DO_NOTHING);
		}
		
		public function preProcess($actionObj)
		{
			parent::preProcess($actionObj);
			
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('OrderStatusIsActive');
			
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeSendNotification')
				$actionObj->setFieldName('OrderStatusSendNotification');
		}
		
		protected function createDomainObject()
		{
			return new scOrderStatusDomainObject();
		}
	}
	
	class scOrderStatusDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new scSqlRecordML('OrderStatuses', 'OrderStatusID',
				array('OrderStatusName'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
		
		public function update()
		{
			if (!$this->get('OrderStatusIsActive')) {
				$activeStatuses = array(
					scOrderStatus::ORDER_STATUS_NEW,
					scOrderStatus::ORDER_STATUS_UNFINISHED,
					scOrderStatus::ORDER_STATUS_PAYMENT_FAILED,
					scOrderStatus::ORDER_STATUS_PAYMENT_PENDING,
				);
				
				if (in_array($this->get('OrderStatusID'), $activeStatuses)) {
					throw new miException(_t('This status cannot be disabled.'));
				}
			}
			if (!$this->get('OrderStatusSendNotification')) {
				if ($this->get('OrderStatusID') == scOrderStatus::ORDER_STATUS_NEW) {
					throw new miException(_t('Sending notifications for this status cannot be disabled.'));
				}
			}
			
			parent::update();
		}
		
		protected function onDelete($pkValueArray)
		{
			$predefined = scOrderStatuses::singleton()->getPredefinedOrderStatusIds();
			$sql = 'SELECT OrderStatus, COUNT(OrderID) AS OrdersCount FROM Orders WHERE OrderStatus IN (' . implode(', ', $pkValueArray) . ') GROUP BY OrderStatus';
			$ordersCount = miUtil::getDBArray($sql, 'OrderStatus', 'OrdersCount');
			foreach ($pkValueArray as $pkValue) {
				if (in_array($pkValue, $predefined)) {
					throw new miException(sprintf(_t('Cannot delete predefined order status "%s".'),
						scOrderStatuses::singleton()->getOrderStatusName($pkValue)));
				}
				if ($ordersCount[$pkValue] > 0) {
					throw new miException(sprintf(_t('There are orders with status "%s". You cannot delete it.'),
						scOrderStatuses::singleton()->getOrderStatusName($pkValue)));
				}
			}
		}
	}
	
	class scOrderStatusTable extends scTable {
		protected function addUiElements(&$rows)
		{
			$predefined = scOrderStatuses::singleton()->getPredefinedOrderStatusIds();
			foreach ($rows as &$row) {
				$row['Predefined'] = in_array($row['OrderStatusID'], $predefined) ? _t('Predefined') : '';
				$row['FMT_OrderStatusInventoryAction'] = scOrderStatusInventoryAction::singleton()->getActionName($row['OrderStatusInventoryAction']);
			}
		}
	}
	
	$orderStatuses = new scSqlRecordsetML('OrderStatuses', 'OrderStatusID');
	$table = new scOrderStatusTable($orderStatuses);
	
	$view = new scOrderStatusesView();
	$view->setUpSortColumn('OrderStatusSort');
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmChangeSendNotification', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>