<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_MODULES_ORDER_TOTAL);

	class scOrderTotalModulesView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_EDIT => 'admin/order_total_modules/order_total_modules_edit.tmpl',
			self::TEMPLATE_ID_LIST => 'admin/order_total_modules/order_total_modules_mgmt.tmpl',
		);
		
        protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'OrderTotalModuleID'
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'OrderTotalModuleName',
				'properties' => array('readOnly' => true)
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'OrderTotalModuleType',
				'properties' => array('readOnly' => true)
        		),
        	array(
				'field' => 'miCheckboxWidget',
				'data' => 'OrderTotalModuleIsActive'
        		),
       		array(
				'field' => 'scModuleOptionsWidget',
				'data' => 'ModuleOptionFields',
				'modelStrategy' => 'scViewMapperModuleOptions',
				),
		);
		
		protected $_domainObjectDisplayName = 'Order total module';	// _t('Order total module');
		
		public function initWebForm(&$form)
        {
			parent::initWebForm($form);
			
        	$domainObject = $this->getDomainObject();
			$domainObject->read(miGetParam('OrderTotalModuleID'));
        	$form->getWidget('ModuleOptionFields')->setupModuleOptions($domainObject->get('ModuleID'), 'IOrderTotalModule');
        }
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function preProcess($actionObj)
		{
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('OrderTotalModuleIsActive');
		}
		
		protected function createDomainObject()
		{
			return new scOrderTotalModuleDomainObject();
		}
	}
	
	class scExecMoveOrderTotalDownAction extends scSortAction {
		
		protected $_direction = 'ASC';
		
		public function doAction()
		{
			try {
				miStaticDBUtil::startTransaction();
				// do the original action
				$record = new scDomainToRecordAdapter($this->_view->getDomainObject());
				$this->_tableName = $record->getTableName();
				$this->_primaryKeyColumn = $record->getPrimaryKeyColumn();
				$record->readPK(miGetParam($this->_primaryKeyColumn));
				
				$this->doSortAction($record->getPK(), $record->get(self::$_sortColumn));
				
				// check if the MIN(OrderTotalModuleSort) is the SUBTOTAL one
				$recordset = new miSqlRecordset('OrderTotalModules');
				$recordset->setSelectFields(array(
					'MIN(OrderTotalModuleSort) AS MINSORT'
				));
				$result = $recordset->getAllRecords();
				
				$recordset->addFilter(new miSqlFilterEqual('OrderTotalModuleSort', $result[0]['MINSORT']));
				$recordset->setSelectFields(array(
					'OrderTotalModuleType'
				));
				$result = $recordset->getAllRecords();
				if ($result[0]['OrderTotalModuleType'] != IOrderTotalModule::ORDER_TOTAL_TYPE_SUBTOTAL) {
					throw new miException(_t('Subtotal must always precede other order total modules.'));
				}
				// everything's OK
				miStaticDBUtil::commit();
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				
				$msg = $this->_view->getMessage('MI_RECORD_UPDATE_FAILED_MSG');
				$this->_view->addAjaxErrorRedirect($msg . $exception->getMessage());
				return false;
			}
			$msg = sprintf($this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG'), $record->getPK());
			$this->_view->addAjaxRefreshListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	class scExecMoveOrderTotalUpAction extends scExecMoveOrderTotalDownAction {
		
		protected $_direction = 'DESC';
	}
	
	class scOrderTotalModuleDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('OrderTotalModules', 'OrderTotalModuleID'));
		}
	}
	
	$orderTotalModules = new miSqlRecordset('OrderTotalModules');
	$table = new scTable($orderTotalModules);
	
	$view = new scOrderTotalModulesView();
	$view->setUpSortColumn('OrderTotalModuleSort');
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmMoveUp', 'scExecMoveOrderTotalUpAction');
	$page->getDispatcher()->setActionHandler('dmMoveDown', 'scExecMoveOrderTotalDownAction');
	$page->dispatchAndShow();
?>