<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');

	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_PLUGINS);
	
	class scPluginsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_CREATE => 'admin/plugins/plugins_install.tmpl',
			self::TEMPLATE_ID_LIST => 'admin/plugins/plugins_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/plugins/plugins_edit.tmpl',
		);
		
		protected $_dataFields = array(
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'PluginID',
			),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'PluginName',
			),
		);
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$maxUploadSize = scFilesUtil::getMaxUploadSize();
			$this->addMainPageElements(array(
				'%%MAXUPLOADSIZEBYTES%%' => $maxUploadSize,
				'%%MAXUPLOADSIZE%%' => scFilesUtil::bytesToShorthand($maxUploadSize),
			));
			
			$action = $this->getController()->getDispatcher()->getActionObj();
			if ($action instanceof miEditAction) {
	        	$domainObject = $this->getDomainObject();
				$domainObject->read(miGetParam('PluginID'));
	        	$form->getWidget('ModuleOptionFields')->setupModuleOptions($domainObject->get('ModuleID'), 'IOrderTotalModule');
			}
		}
	}
	
	class scInstallPluginAction extends miCreateAction {
		
	}
	
	class scExecInstallPluginAction extends miActionWithWebForm {
		
		public function doAction()
		{
			try {
				miStaticDBUtil::startTransaction();
				if (($_SERVER['REQUEST_METHOD'] == 'POST') && empty($_POST) && ($_SERVER['CONTENT_LENGTH'] > 0)) {
					throw new miException(sprintf(_t('The uploaded file is too large (%s bytes).'), number_format($_SERVER['CONTENT_LENGTH'])));
				}
				
				$error = $_FILES['PluginPackageFile']['error'];
				if ($error == UPLOAD_ERR_NO_FILE) {
					throw new miException(_t('Please upload file.'));
				}
				
				$this->checkUpload('PluginPackageFile');
				
				$installer = new scPluginZipInstaller();
				$installer->processZip($_FILES['PluginPackageFile']['tmp_name']);
				
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				$params = array(
					'action=acInstallPlugin',
					$this->_view->getRedirectParams($exception->getMessage(), miMessage::MSG_TYPE_ERROR)
				);
				$this->_view->addRedirectControllerCommand($params);
				return;
			}
			miStaticDBUtil::commit();
			
			$this->_view->addRedirectToListControllerCommand(_t('Plugin successfully installed.'), miMessage::MSG_TYPE_INFO);
			return true;
		}
		
		protected function checkUpload($name)
		{
			if ($_FILES[$name]['error'])
				throw new miException(sprintf(_t('Error uploading the file: %s (error code %s).'),
					scFilesUtil::getUploadErrorDescription($_FILES[$name]['error']), $_FILES[$name]['error']));
			
			if (!is_uploaded_file($_FILES[$name]['tmp_name']))
				throw new miException(_t('Internal error.'));
			
			if ($_FILES[$name]['size'] == 0)
				throw new miException(_t('The uploaded file is empty.'));
		}
	}
	
	class scPluginZipInstaller {
		
		protected $_tempPath = '';
		
		protected $_systemFiles = array(
			'plugin.xml',
			'install.php',
			'uninstall.php',
		);
		
		protected $_extractedFiles = array();
		
		public function __construct()
		{
			// FIXME: temp_images:
			$this->_tempPath = realpath(dirname(__FILE__) . '/../') . '/userfiles/temp_images/plugin_temp_' . uniqid() . '/';
		}
		
		public function processZip($pluginZip)
		{
			try {
				$this->extractZip($pluginZip);
				
				$this->checkExtractedFiles();
				$pluginInfo = $this->getPluginInfo();
				
				$pluginId = $this->doPluginInstall($pluginInfo);
				
				$file = $this->_tempPath . 'install.php';
				if (unlink($file)) {
					unset($this->_extractedFiles[array_search($file, $this->_extractedFiles)]);
				}
				$file = $this->_tempPath . 'plugin.xml';
				if (unlink($file)) {
					unset($this->_extractedFiles[array_search($file, $this->_extractedFiles)]);
				}
				// TODO: Move the uninstall script so it is callable later
			} catch (miException $e) {
				$this->revertExtract();
				throw $e;
			}
		}
		
		protected function extractZip($filename)
		{
			if (!function_exists('zip_open'))
				throw new miException(_t('PHP extension ZIP is not installed. Please contact your hosting provider to enable it.'));
			
			$zip = zip_open($filename);
			if (!is_resource($zip)) {
				throw new miException(sprintf(_t('Cannot read ZIP archive. Error %d: %s'), $zip, scArchiveZip::getZipErrorMessage($zip)));
			}
			
			
			while ($zipEntry = zip_read($zip)) {
				$extractTo = scPaths::getStoreRootDir();
				
				$entryName = trim(zip_entry_name($zipEntry));
				if (strstr($entryName, '../'))	// Security check
					continue;
				$entrySize = zip_entry_filesize($zipEntry);
				
				if (in_array($entryName, $this->_systemFiles)) {
					$extractTo = $this->_tempPath;
				}
				
				// If directory create it
				if ($entryName[strlen($entryName)-1] == '/') {
					if (!is_dir($extractTo . $entryName)) {
						if (!@mkdir($extractTo . $entryName, 0777, true))
							throw new miException(sprintf(_t('Could not create folder "%s"'), $extractTo . $entryName));
						}
					continue;
				}
				
				// It's a file
				if (file_exists($extractTo . $entryName)) {
					throw new miException(sprintf(_t('File "%s" already exists.'), $entryName));
				}
				
				// First check that its directory exists
				$dirname = dirname($entryName);
				if ($dirname == '.')
					$dirname = '';
				if (!is_dir($extractTo . $dirname)) {
					if (!@mkdir($extractTo . $dirname, 0777, true))
						throw new miException(sprintf(_t('Could not create folder "%s"'), $extractTo . $dirname));
				}
				
				if (!zip_entry_open($zip, $zipEntry)) {
					throw new miException(_t('Cannot open ZIP archive entry.'));
				}
				
				$entryContents = zip_entry_read($zipEntry, $entrySize);
				file_put_contents($extractTo . $entryName, $entryContents);
				$this->_extractedFiles[] = $extractTo . $entryName;
				
				zip_entry_close($zipEntry);
			}
			
			zip_close($zip);
		}
		
		protected function revertExtract()
		{
			foreach ($this->_extractedFiles as $file) {
				unlink($file);
				@rmdir(dirname($file));
			}
		}
		
		protected function checkExtractedFiles()
		{
			if (empty($this->_extractedFiles)) {
				throw new miException(_i('No extracted files.'));
			}
			
			foreach ($this->_systemFiles as $file) {
				if (!file_exists($this->_tempPath . $file)) {
					throw new miException(sprintf(_i('The plugin must contain a "%s" file.'), $file));
				}
			}
		}
		
		/**
		 * @return scPluginInstallationInfo
		 */
		protected function getPluginInfo()
		{
			$xml = simplexml_load_file($this->_tempPath . 'plugin.xml');
			if (!isset($xml->Name) || empty($xml->Name)) {
				throw new miException(_i('"Name" is a required property of the "plugin.xml".'));
			}
			$info = new scPluginInstallationInfo();
			$info->setName($xml->Name);
			return $info;
		}
		
		protected function doPluginInstall(scPluginInstallationInfo $installationInfo)
		{
			$plugin = new scPluginDomainObject();
			$plugin->setPluginInstallationInfo($installationInfo);
			$pluginId = $plugin->insert();
			scPluginInstall::$_pluginId = $pluginId;
			
			include($this->_tempPath . 'install.php');
			
			return $pluginId;
		}
	}
	
	class scPluginInstallationInfo {
		
		protected $_name = null;
		protected $_version = '';
		protected $_author = '';
		
		public function setName($name)
		{
			$this->_name = $name;
		}
		
		public function getName()
		{
			return $this->_name;
		}
		
		public function getAuthor()
		{
			return $this->_author;
		}
		
		public function getVersion()
		{
			return $this->_version;
		}
		
		public function getInstallationPath()
		{
			return 'com.summercart.' . scFilesUtil::escapeStringForFilename($this->getName());
		}
		
		// TODO: Setters
	}
	
	class scPluginDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('Plugins', 'PluginID'));
		}
		
		public function setPluginInstallationInfo(scPluginInstallationInfo $info)
		{
			$this->set('PluginName', $info->getName());
			$this->set('PluginTitle', $info->getName());
			$this->set('PluginAuthor', $info->getAuthor());
			$this->set('PluginVersion', $info->getVersion());
		}
	}
	
	class scModuleDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('Modules', 'ModuleID'));
		}
	}
	
	class scModuleClassDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('ModuleClasses', 'ModuleClassID'));
		}
	}
	
	final class scPluginInstall {
		
		public static $_pluginId;
		
		public static function registerModule($moduleName, $moduleTitle, $moduleVersion = '')
		{
			$module = new scModuleDomainObject();
			$module->set('PluginID', self::$_pluginId);
			$module->set('ModuleName', $moduleName);
			$module->set('ModuleTitle', $moduleTitle);
			$module->set('ModuleVersion', $moduleVersion);
			$module->set('ModuleIsActive', true);
			return $module->insert();
		}
		
		public static function registerInterface($moduleId, $interface, $className, $classPath)
		{
			if (!file_exists(scPaths::getStoreRootDir() . $classPath)) {
				throw new miException(sprintf(_i('File "%s" does not exist.'), $classPath));
			}
			
			$moduleClass = new scModuleClassDomainObject();
			$moduleClass->set('ModuleID', $moduleId);
			$moduleClass->set('ModuleClassInterface', $interface);
			$moduleClass->set('ModuleClassLocation', self::getClassPath($classPath));
			$moduleClass->set('ModuleClassName', $className);
			$moduleClass->insert();
		}
		
		protected static function getClassPath($classPath)
		{
			if (substr($classPath, 0, 8) == 'modules/') {
				return substr($classPath, 8);
			}
			return '../' . $classPath;
		}
	}
	
	class scPluginModulesTable extends scTable {
		
		protected function addUiElements(&$rows)
		{
			parent::addUiElements($rows);
			
			$hasIAdminModuleSections = array();
			foreach ($rows as $row) {
				$hasIAdminModuleSections[] = new miTemplateParserSectionInfo('hasIAdminModule', $row['HasIAdminModule']);
			}
			$this->getSection()->addSubsectionsArray('hasIAdminModule', $hasIAdminModuleSections);
		}
	}
	
	$recordset = new miSqlRecordset('Plugins');
	$recordset->addJoinCondition('LEFT', 'Modules', 'USING(PluginID)');
	$recordset->addJoinCondition('LEFT', 'ModuleClasses', 'ON Modules.ModuleID = HasIAdminTable.ModuleID AND HasIAdminTable.ModuleClassInterface = "IAdminModule"', 'HasIAdminTable');
	$recordset->setSelectFields(array(
		'Plugins.*',
		'Modules.*',
		'COUNT(HasIAdminTable.ModuleClassID) AS HasIAdminModule'
	));
	$recordset->setGroupBy('Modules.ModuleID');
	
	$table = new scPluginModulesTable($recordset);
	
	$view = new scPluginsView();
	$view->setRecord(new miSqlRecord('Plugins', 'PluginID'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('acInstallPlugin', 'scInstallPluginAction');
	$page->getDispatcher()->setActionHandler('acExecInstallPlugin', 'scExecInstallPluginAction');
	$page->dispatchAndShow();
?>