<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../../include/miphpf/Init.php');
	require_once('../../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization();
	
	scSkins::singleton()->setLayout(scLayout::AREA_ADMIN);
	
	/**
	 * The view for this page
	 */
	class scImageUploadView extends miDefaultView {
		
		public function __construct()
		{
			parent::__construct(null);
		}
		
		protected $_templates = array(
			miDefaultView::TEMPLATE_ID_VIEW => 'admin/popup/image_upload.tmpl',
		);
	}
	
	/**
	 * Uploads the images and performs basic template parsing
	 */
	class scImageUploadAction extends miAction {
		const SPACER = 'admin/images/spacer.gif';
		
		protected $_onOk = false;
		protected $_directApply = false;
		protected $_tempImageFilename = '';
		protected $_tempImagePath = '';
		protected $_tempThumbFilename = '';
		protected $_tempThumbPath = '';
		protected $_exceptionMessage = '';
		protected $_tempImageFilenameMd5 = '';
		protected $_hasApplyButton = false;
		protected $_hasThumb = true;
		
		public function doAction()
		{
			$t = new scTemplateParser();
			
			$template = $this->_view->getTemplateFileName(scImageUploadView::TEMPLATE_ID_VIEW);
			$t->readTemplate($template);
			
			$spacer = scUrlUtil::convertUrl(scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl() . self::SPACER, scUrlUtil::URL_TYPE_DOCROOT);
			
			// Set Main Page Elements
			$maxUploadSize = scFilesUtil::getMaxUploadSize();		
			$t->assignArray(
				array(
					'%%MAXUPLOADSIZEBYTES%%' => miI18N::htmlEscape($maxUploadSize),
					'%%MAXUPLOADSIZE%%' => miI18N::htmlEscape(scFilesUtil::bytesToShorthand($maxUploadSize)),
					'%%TEMPIMAGEFILENAME%%' => miI18N::htmlEscape($this->_tempImageFilename),
					'%%TEMPIMAGEWEBPATH%%' => miI18N::htmlEscape($this->_tempImagePath),
					'%%STORETEMPIMAGEWEBPATH%%' => empty($this->_tempImagePath) ? $spacer : '../../' . miI18N::htmlEscape($this->_tempImagePath),
					
					'%%TEMPTHUMBFILENAME%%' => miI18N::htmlEscape($this->_tempThumbFilename),
					'%%TEMPTHUMBWEBPATH%%' => miI18N::htmlEscape($this->_tempThumbPath),
					'%%STORETEMPTHUMBWEBPATH%%' => empty($this->_tempThumbPath) ? $spacer : '../../' . miI18N::htmlEscape($this->_tempThumbPath),
					
					'%%HASAPPLYBUTTON%%' => $this->_hasApplyButton ? 'true' : 'false',
					'%%HASTHUMB%%' => $this->_hasThumb ? 'true' : 'false',
					'%%ONOK%%' => $this->_onOk ? 'true' : 'false',
					'%%DIRECTAPPLY%%' => $this->_directApply ? 'true' : 'false',
					'%%MSG%%' => miI18N::htmlEscape($this->_exceptionMessage),
					'%%MSG_TYPE_STYLE%%' => empty($this->_exceptionMessage) ? '' : 'error',
					
					'%%RESOURCE%%' => miI18N::htmlEscape(miGetParam('Resource')),
					'%%IMAGENAME%%' => miI18N::htmlEscape(miGetParam('ImageName')),
				)
			);
			
			// If not closing display the upload form
			$showUploadSection = new miTemplateParserSectionInfo('showUpload', 0);
			if (!$this->_onOk)
				$showUploadSection->setIterationsCount(1);
				
			if ($this->_hasApplyButton)
				$showUploadSection->addSubsection(new miTemplateParserSectionInfo('hasApplyButton', 1));
			
			if ($this->_hasThumb)
				$showUploadSection->addSubsection(new miTemplateParserSectionInfo('hasThumb', 1));
			
			$t->addSectionInfo($showUploadSection);
			
			$html = $t->templateParse();
			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;
		}
		
		protected function saveTempFiles()
		{
			$this->checkUpload('uploadedFile');
			$this->saveImageAsTemp('uploadedFile');

			if ($this->_hasThumb) {
				if (miGetParamDefault('generateThumb', 0)) {
					// If image is uploaded but thumbnail is not
					$this->copyUploadedImageAsTempThumb();
				} else {
					// If both thumbnail and image is uploaded
					$this->checkUpload('uploadedThumb');
					$this->saveThumbAsTemp('uploadedThumb');
				}
			}
		}
		
		protected function checkUpload($name)
		{
			if ($_FILES[$name]['error'])
				throw new miException(sprintf(_t('Error uploading the file: %s (error code %s).'),
					scFilesUtil::getUploadErrorDescription($_FILES[$name]['error']), $_FILES[$name]['error']));
			
			if (!is_uploaded_file($_FILES[$name]['tmp_name']))
				throw new miException(_t('Internal error.'));
			
			if ($_FILES[$name]['size'] == 0)
				throw new miException(_t('The uploaded file is empty.'));
			
			if (@getimagesize($_FILES[$name]['tmp_name']) === false)
				throw new miException(_t('The uploaded file is not an image.'));
		}
		
		/**
		 * Copies the uploaded image to a temp location so it can later be used.
		 *
		 * @param string $name The input field's name
		 */
		protected function saveImageAsTemp($name)
		{
			$this->_tempImageFilenameMd5 = md5($_FILES[$name]['tmp_name'] . microtime(true));
			$this->_tempImageFilename = scPaths::getItemFilename($this->_tempImageFilenameMd5, scPaths::RESOURCE_TEMP_IMAGE);
			$filename = scPaths::getFilenamePath($this->_tempImageFilename, scPaths::RESOURCE_TEMP_IMAGE);
			if (@move_uploaded_file($_FILES[$name]['tmp_name'], $filename) === false) {
				scPaths::checkFileWritable($filename, true);
				throw new miException(_t('Cannot save the uploaded file!'));
			}
			@chmod($filename, 0644);
			
			$this->_tempImagePath = scPaths::getFilenamePath($this->_tempImageFilename, scPaths::RESOURCE_TEMP_IMAGE, scPaths::PATH_WEB);
		}
		
		/**
		 * If a thumb was uploaded, copy it to a temp location so it can later be used.
		 *
		 * @param string $name The input field's name
		 */
		protected function saveThumbAsTemp($name)
		{
			$this->_tempThumbFilename = scPaths::getItemFilename($this->_tempImageFilenameMd5, scPaths::RESOURCE_TEMP_THUMB);
			$filename = scPaths::getFilenamePath($this->_tempThumbFilename, scPaths::RESOURCE_TEMP_THUMB);
			if (@move_uploaded_file($_FILES[$name]['tmp_name'], $filename) === false) {
				scPaths::checkFileWritable($filename, true);
				throw new miException(_t('Cannot save the uploaded file!'));
			}
			@chmod($filename, 0644);
			
			$this->_tempThumbPath = scPaths::getFilenamePath($this->_tempThumbFilename, scPaths::RESOURCE_TEMP_THUMB, scPaths::PATH_WEB);
		}
		
		/**
		 * If a thumb has not been uploaded and a thumb is required
		 * copy the original size image to a TEMP THUMB location so it can later be used and scaled to thumb size.
		 */
		protected function copyUploadedImageAsTempThumb()
		{
			$imageFile = scPaths::getFilenamePath($this->_tempImageFilename, scPaths::RESOURCE_TEMP_IMAGE);
			
			$this->_tempThumbFilename = scPaths::getItemFilename($this->_tempImageFilenameMd5, scPaths::RESOURCE_TEMP_THUMB);
			$thumbFile = scPaths::getFilenamePath($this->_tempThumbFilename, scPaths::RESOURCE_TEMP_THUMB);
			
			if (@copy($imageFile, $thumbFile) === false) {
				scPaths::checkFileWritable($thumbFile, true);
				throw new miException(_t('Cannot save the uploaded file!'));
			}
			@chmod($thumbFile, 0644);
			
			$this->_tempThumbPath = scPaths::getFilenamePath($this->_tempThumbFilename, scPaths::RESOURCE_TEMP_THUMB, scPaths::PATH_WEB);
		}
		
		/**
		 * Initializes the flags that indicate if there should be available thumb uploading,
		 * and if there should be available direct apply of the image
		 * 
		 * @return void
		 */
		protected function initFlags()
		{
			$this->_hasApplyButton = miGetParamDefault('hasApplyButton', 'false');
			if ($this->_hasApplyButton == 'true')
				$this->_hasApplyButton = true;
			else
				$this->_hasApplyButton = false;
			
			$this->_hasThumb = miGetParamDefault('hasThumb', 'true');
			if ($this->_hasThumb == 'true')
				$this->_hasThumb = true;
			else
				$this->_hasThumb = false;
		}
		
		/**
		 * If the images are already uploaded restore them
		 * 
		 * @return void
		 */
		protected function restoreTempFiles()
		{
			// Check if we have already uploaded the images
			if (miGetParamDefault('tempImageFilename', '') != '') {
				$this->_tempImageFilename = miGetParam('tempImageFilename');
				$this->_tempImagePath = scPaths::getFilenamePath($this->_tempImageFilename, scPaths::RESOURCE_TEMP_IMAGE, scPaths::PATH_WEB);
			}
			if (miGetParamDefault('tempThumbFilename', '') != '') {
				$this->_tempThumbFilename = miGetParam('tempThumbFilename');
				$this->_tempThumbPath = scPaths::getFilenamePath($this->_tempThumbFilename, scPaths::RESOURCE_TEMP_THUMB, scPaths::PATH_WEB);
			}
		}
		
		/**
		 * Checks if there have been images that were successfully uploaded using the preview function 
		 *
		 * @return bool
		 */
		protected function hasPreviewedImage()
		{
			return (miGetParamDefault('tempImageFilename', '') != '') or (miGetParamDefault('tempThumbFilename', '') != '');
		}
	}
	
	/**
	 * When preview button is clicked
	 */
	class scImagePreviewAction extends scImageUploadAction {
		public function doAction()
		{
			try {
				$this->initFlags();
				$this->restoreTempFiles();
				
				if (array_key_exists('uploadedFile', $_FILES)) {
					if ($_FILES['uploadedFile']['error'] != UPLOAD_ERR_NO_FILE)
						$this->saveTempFiles();
					else
						$this->_exceptionMessage = _t('Please upload an image.');
				}
			} catch (miException $exception) {
				$this->_exceptionMessage = $exception->getMessage();
			}
		
			return parent::doAction();
		}
	}

	/**
	 * When close button is clicked
	 */
	class scImageCloseAction extends scImageUploadAction {
		public function doAction()
		{
			try {
				$this->initFlags();
				$this->restoreTempFiles();
				
				if (array_key_exists('uploadedFile', $_FILES)) {
					if ($_FILES['uploadedFile']['error'] != UPLOAD_ERR_NO_FILE) {
						$this->saveTempFiles();
						$this->_onOk = true;
					} elseif ($this->hasPreviewedImage()) {
						$this->_onOk = true;
						return parent::doAction();
					} else {
						$this->_exceptionMessage = _t('Please upload an image.');
					}
				}
			} catch (miException $exception) {
				$this->_exceptionMessage = $exception->getMessage();
			}
			
			return parent::doAction();
		}
	}

	/**
	 * When the Apply now button is clicked
	 */
	class scImageSaveAction extends scImageUploadAction {
		public function doAction()
		{
			try {
				$this->initFlags();
				$this->restoreTempFiles();
				
				if (array_key_exists('uploadedFile', $_FILES)) {
					if ($_FILES['uploadedFile']['error'] != UPLOAD_ERR_NO_FILE) {
						$this->saveTempFiles();
						$this->_directApply = $this->_onOk = true;
					} elseif ($this->hasPreviewedImage()) {
						$this->_directApply = $this->_onOk = true;
						return parent::doAction();
					} else {
						$this->_exceptionMessage = _t('Please upload an image.');
					}
				}
			} catch (miException $exception) {
				$this->_exceptionMessage = $exception->getMessage();
			}
			
			return parent::doAction();
		}
	}
	
	$imageUpload = new scImageUploadView();
	$page = new miPage(array($imageUpload));
	
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scImagePreviewAction');
	$page->getDispatcher()->setActionHandler('dmPreview', 'scImagePreviewAction');
	$page->getDispatcher()->setActionHandler('dmClose', 'scImageCloseAction');
	$page->getDispatcher()->setActionHandler('dmSave', 'scImageSaveAction');
	
	$page->dispatchAndShow();
?>