<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_PRODUCTS);
	
	class scProductsView extends scView {
		
		const DEFAULT_PRODUCT_DESCRIPTIONS_XHTML = 1;
		const DEFAULT_PRODUCT_WEIGHT = 0;
		const DEFAULT_PRODUCT_QUANTITY = 0;
		
		const TEMPLATE_ID_DELETE = 'delete';
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/products/product_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/products/product_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/products/product_create.tmpl',
			self::TEMPLATE_ID_DELETE => 'admin/products/product_delete.tmpl',
		);
		
		protected $_defaultFilterValues = array(
			'ProductPrice' => array(0 => '', 1 => ''),
		);
		
		protected $_dataFields = array(
       		// Tab 1: General
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductCode',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Code'
        	),
       		array(
				'field' => 'scOptionalSelectWidget',
				'data' => 'ManufacturerID',
				'displayName' => 'Manufacturer'
        	),
			array(
				'field' => 'scCategorySelect',
				'data' => 'CategoryID',
				'displayName' => 'Category'
        	),
       		array(
				'field' => 'miCheckboxWidget',
				'data' => 'ProductIsActive',
				'properties' => array('defaultValue' => true),
				'displayName' => 'Active'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'ProductName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name'
        	),
			array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'ProductDescription',
				'displayName' => 'Description'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductDescriptionXHTML',
       			'properties' => array(
       				'defaultValue' => self::DEFAULT_PRODUCT_DESCRIPTIONS_XHTML,
       			),
				'displayName' => 'Description Is HTML'
        	),
			array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'ProductDetailedDescription',
				'displayName' => 'Detailed Description'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductDetailedDescriptionXHTML',
       			'properties' => array(
       				'defaultValue' => self::DEFAULT_PRODUCT_DESCRIPTIONS_XHTML,
       			),
				'displayName' => 'Detailed Description Is HTML'
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductPrice',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Price'
        	),
			array(
				'field' => 'scProductFixedPricesWidget',
				'data' => 'ProductFixedPrices',
				'modelStrategy' => 'scViewMapperProductFixedPrices',
			),
       		array(
				'field' => 'scWeightWidget',
				'data' => 'ProductWeight',
				'displayName' => 'Weight',
       			'properties' => array(
       				'defaultValue' => self::DEFAULT_PRODUCT_WEIGHT,
       			),
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductQuantity',
				'displayName' => 'Quantity',
       			'properties' => array(
       				'defaultValue' => self::DEFAULT_PRODUCT_QUANTITY,
       			),
				'validator' => array(
       				'scValidatorSignedInt',
       			),
			),
			array(
				'field' => 'miCheckboxWidget',
				'data' => 'ProductIsEgood',
				'properties' => array(
					'defaultValue' => false,
					'id' => 'ProductIsEgood',
				),
				'displayName' => 'Product Is E-Good'
			),
			
       		// Tab 2: Pictures
       		array(
				'field' => 'scProductImagesUploadWidget',
				'data' => 'RSProductImages',
				'modelStrategy' => 'scViewMapperProductImages',
				'properties' => array(
					'template' => 'admin/products/product_images_rs.tmpl',
					'table' => 'ProductImages',
       				'sortBy' => 'ProductImageSort',
					'foreignKey' => 'ProductID',
       				'foreignTablePrimaryKey' => 'ProductImageID',
					'fields' => array( 
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductImageID'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductImageThumbWidth',
							'properties' => array('readOnly' => true)
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductImageThumbHeight',
							'properties' => array('readOnly' => true)
			        	),
						array(
							'field' => 'scProductImagesRadio',
							'data' => 'ProductImageIsDefault'
			        	),
			       		array(
							'field' => 'scWebFormWidgetFileUpload',
							'data' => 'ProductImagePicture',
							'properties' => array(
								'primaryKey' => 'ProductImageID',
							),
						),
			       		array(
							'field' => 'scWebFormWidgetFileUpload',
							'data' => 'ProductImageThumb',
							'properties' => array(
								'primaryKey' => 'ProductImageID',
							),
						),
			        ),	
				),
        	),
        	
			// Tab 3: SEO
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'ProductPageTitle'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'ProductMetaKeywords'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'ProductMetaDescription'
        	),
			array(
				'field' => 'scCustomUrlsWidget',
				'data' => 'CustomUrls',
				'modelStrategy' => 'scViewMapperProductCustomUrls',
				'properties' => array(
					'template' => 'admin/custom_urls/custom_url.tmpl',
					
					'table' => 'CustomUrls',
					'foreignKey' => 'CustomUrlEntityID',
					'foreignTablePrimaryKey' => 'CustomUrlID',
					
					'fields' => array(
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CustomUrlText',
			       			'properties' => array('minLength' => 1),
			       			'validator' => 'scValidatorUniqueCustomUrl'
			        	),
			       		array(
							'field' => 'miCheckboxWidget',
							'data' => 'CustomUrlIsCurrent'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomUrlID'
			        	)
			        ),
			        'mlFields' => array('CustomUrlText')
				),
			),
        	
			// Tab 4: Categories
			array(
				'field' => 'scDoubleListBoxWidget',
				'data' => 'Categories',
				// _t('Available Categories'), _t('Selected Categories')
				'properties' => array('leftCaption' => 'Available Categories', 'rightCaption' => 'Selected Categories'),
				'modelStrategy' => 'scViewMapperProductCategories',
			),
			
			// Tab 5: Options
			array(
				'field' => 'scTableCheckboxWidget',
				'data' => 'ProductOptionGroups',
       			'modelStrategy' => 'scViewMapperProductVariantOptions',
				'properties' => array(
       				'template' => 'admin/products/product_global_options_widget.tmpl'
       			),
			),
			array(
				'field' => 'scLocalOptionGroupWidget',
				'data' => 'LocalOptionGroups',
				'modelStrategy' => 'scViewMapperProductVariantOptions',
				'properties' => array(
					'template' => 'admin/products/product_local_options_rs.tmpl',
					
					'table' => 'LocalOptionGroups',
					'sortField' => 'LocalOptionGroupSort',
					'sortBy' => 'LocalOptionGroupSort',
					'foreignKey' => 'ProductID',
					'foreignTablePrimaryKey' => 'LocalOptionGroupID',
					
					'valueTable' => 'LocalOptions',
					'valueKey' => 'LocalOptionID',
					'valueSortBy' => 'LocalOptionSort',
					
					'fields' => array( 
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'LocalOptionGroupID'
			        	),
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'LocalOptionGroupName',
			       			'properties' => array('minLength' => 1)
			        	),
			       		array(
							'field' => 'miSelectWidget',
							'data' => 'LocalOptionGroupType',
			        	),
			        ),
			        'mlFields' => array('LocalOptionGroupName')
				),
			),
			
			// Tab 6: E-Goods
			array(
				'field' => 'scProductEgoodsWidget',
				'data' => 'RSProductEgoods',
				'modelStrategy' => 'scViewMapperProductEgoods',
				'properties' => array(
					scProductEgoodsWidget::EGOODS_DIR_FIELD => 'EgoodsDirName',
					scProductEgoodsWidget::EGOODS_SHARED_DIR_FIELD => 'SharedEgoodsDirName',
					'template' => 'admin/products/product_egoods_rs.tmpl',
					'table' => 'ProductEgoods',
					'foreignKey' => 'ProductID',
					'foreignTablePrimaryKey' => 'ProductEgoodID',
					'fields' => array( 
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductEgoodID'
			        	),
			       		array(
							'field' => 'scEgoodWidget',
							'data' => 'ProductEgoodFile',
			        	),
			        ),
				),
        	),
        	
        	// Tab 7: Prices
        	array(
        		'field' => 'scProductPricesWidget',
        		'data' => 'RSProductPrices',
        		'modelStrategy' => 'scViewMapperProductPrices',
        		'properties' => array(
					'template' => 'admin/products/product_prices_rs.tmpl',
					'table' => 'ProductWholesalePrices',
					'foreignKey' => 'ProductID',
					'foreignTablePrimaryKey' => 'ProductWholesalePriceID',
					'fields' => array( 
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductWholesalePriceQuantity'
			        	),
			        ),
        		),
        	),
        	
        	// Tab 8: Attributes
			array(
				'field' => 'scOptionalSelectWidget',
				'data' => 'ProductClassID',
				'displayName' => 'Class'
        	),
			array(
				'field' => 'scProductAttributesWidget',
				'data' => 'ProductAttributes',
				'modelStrategy' => 'scViewMapperProductAttributes',
				'validator' => 'scValidatorProductAttributes',
        	),
			
			// Tab 9: Variants
			array(
				'field' => 'scProductVariantsWidget',
				'data' => 'ProductVariants',
				'modelStrategy' => 'scViewMapperProductVariants',
				'properties' => array(
					'template' => 'admin/products/product_variants_rs.tmpl',
					'table' => 'ProductVariants',
					'foreignKey' => 'ProductID',
					'foreignTablePrimaryKey' => 'ProductVariantID',
					'wholesalePriceField' => 'ProductVariantWholesalePriceAmount',
					'fields' => array(
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductVariantID',
						),
						array(
							'field' => 'scDefaultDynRowWidget',
							'data' => 'ProductVariantIsDefault',
						),
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductVariantCode',
						),
						array(
							'field' => 'scProductVariantPriceWidget',
							'data' => 'ProductVariantPrice'
						),
						array(
							'field' => 'scOptionalTextWidget',
							'data' => 'ProductVariantBonusPointsPrice',
							'validator' => 'scValidatorOptionalInt',
						),
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductVariantWeight',
						),
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductVariantQuantity',
						),
						array(
							'field' => 'scProductVariantImageWidget',
							'data' => 'ProductVariantImage',
							'properties' => array(
								'size' => 4,
							),
						),
						array(
							'field' => 'scProductVariantOptionsListWidget',
							'data' => 'ProductVariantOptions',
						),
					),
				),
			),
			// Tab 10: Related products
        	array(
				'field' => 'scDynRowsWidget',
				'data' => 'RSRelatedProducts',
				'modelStrategy' => 'scViewMapperRelatedProducts',
				'properties' => array(
					'id' => 'Products',
					'template' => 'admin/products/related_products_rs.tmpl',
					'table' => 'RelatedProducts',
					'foreignKey' => 'ProductID',
					'foreignTablePrimaryKey' => 'RelatedProductID',
					'fields' => array(
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'RelatedProductID',
						),
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductName',
							'properties' => array('readOnly' => true),
						),
						array(
							'field' => 'miBaseCheckBoxWidget',
							'data' => 'BiDirectionalLink',
						),
					),
				)
        	),
        	
        	// Tab 11: Bonus Points
        	array(
        		'field' => 'miSelectWidget',
        		'data' => 'ProductBonusPointsMode',
        	),
        	array(
        		'field' => 'scOptionalTextWidget',
        		'data' => 'ProductBonusPointsPrice',
        		'validator' => 'scValidatorOptionalInt',
        	),
        	
        	// Tab 12: Bundle
        	array(
        		'field' => 'miCheckboxWidget',
        		'data' => 'ProductIsBundle',
        	),
        	array(
        		'field' => 'miCheckboxWidget',
        		'data' => 'ProductBundleAutoQuantity',
        	),
        	array(
        		'field' => 'scBundleProductsWidget',
        		'data' => 'BundleProducts',
        		'modelStrategy' => 'scViewMapperBundleProducts',
        		'properties' => array(
        			'id' => 'BundleProducts',
					'template' => 'admin/products/bundle_products_rs.tmpl',
					'table' => 'BundleProducts',
					'sortBy' => 'BundleProductSort',
					'sortField' => 'BundleProductSort',
        			'foreignKey' => 'ProductID',
					'foreignTablePrimaryKey' => 'BundleProductID',
					'fields' => array(
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'BundleProductID',
						),
        				array(
							'field' => 'miBaseTextWidget',
							'data' => 'SubProductID',
						),
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'ProductName',
							'properties' => array('readOnly' => true),
						),
						array(
							'field' => 'miBaseTextWidget',
							'data' => 'BundleProductQuantity',
							'properties' => array(
								'defaultValue' => 1,
							),
						),
					),
        		),
        	),
        );
        
        protected $_domainObjectDisplayName = 'Product';	// _t('Product');
        
        public static function getCategoryBranchForExport($categoryId)
        {
        	$branch = scCategories::getCategoryFullBranch($categoryId, self::getCurrentDataLanguageId());
        	return '|' . implode('|', $branch);
        }
		        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
			
			// Create dropdown filter for category
			$dummyForm = new miWebForm(array());
			$categorySelectWidget = new scCategorySelect($dummyForm, array());
			$categoriesArray = $categorySelectWidget->getCategoriesTree(0, true);
			$categoriesArray = array('' => _t('All categories')) + $categoriesArray;
			scUIUtil::createDropdownFilter($this, $table, 'CategoryID', $categoriesArray, 'category', '');
			$this->getTableFilterObj()->setConditionHandler('category', 'scSqlFilterProductsCategoryId');
			
			$manufacturersArray = array('' => _t('All manufacturers')) + scUtil::getManufacturersArray();
			scUIUtil::createDropdownFilter($this, $table, 'ManufacturerID', $manufacturersArray, '=', '');
			
			$productClassesArray = array('' => _t('All product classes')) + scUtil::getProductClassesArray();
			scUIUtil::createDropdownFilter($this, $table, 'ProductClassID', $productClassesArray, '=', '');
			
			$active = array('' => _t('All statuses'), 1 => _t('Active'), 0 => _t('Inactive'));
			scUIUtil::createDropdownFilter($this, $table, 'ProductIsActive', $active, '=', '');
			
			$this->getTableFilterObj()->setConditionHandler('descriptions', 'scSqlFilterProductsDescriptions');
			$this->getTableFilterObj()->setConditionHandler('productCode', 'scSqlFilterProductsCode');
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$form->addMainPageElements(
				array(
					'%%DUPLICATE%%' => 'false',
					'%%DUPLICATEACTION%%' => 'dmExecCreate',
					'%%TITLE%%' => miI18N::htmlEscape(_t('Create')),
					'%%BONUSPOINTSCONVERSIONRATE%%' => scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_CONVERSION_RATE),
				)
			);
			$form->addMainPageElements(array(
				'%%CKEDITOR_CONTENTCSS%%' => scSkins::singleton()->getCurrentSkin()->getCssFilesForWysiwygEditor(),
				'%%CKEDITOR_LANGUAGE%%' => scCMS::getCkEditorLanguage()
			));
			
			// Initialize main category select
			$field = $form->getWidget('CategoryID');
			$field->setCategoryTree(0, true);
			
			$field = $form->getWidget('ProductClassID');
			$field->setOptions(array(scOptionalSelectWidget::NONE_VALUE => '') + scUtil::getProductClassesArray());
			
			$field = $form->getWidget('ManufacturerID');
			$field->setOptions(array(scOptionalSelectWidget::NONE_VALUE => '') + scUtil::getManufacturersArray());
			
			$field = $form->getWidget('ProductBonusPointsMode');
			$defaultValue = scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_PRODUCTS_DEFAULT_MODE);
			$field->setOptions(scBonusPoints::getAllProductCustomerPointModes(), $defaultValue);
			
			// Initialize additional categories multi select
			$primaryKeyValue = (int)miGetParamDefault('ProductID', null);
			$field = $form->getWidget('Categories');
			$field->setMode(scDoubleListBoxWidget::MODE_COPY_VALUES);
			$field->setLeftToRightJsHandler('SC.Products.copySelectedCategories');
			$field->setRightToLeftJsHandler('SC.Products.removeSelectedCategories');
			
			$dummyForm = new miWebForm(array());
			$categorySelectWidget = new scCategorySelect($dummyForm, array());
			$categoriesArray = $categorySelectWidget->getCategoriesTree(0, true);
			$field->setOptions($categoriesArray);			
			$field->setSelectedItemsTextFilter(create_function('$name', 'return preg_replace("/.*▢/", "", $name);'));
			
			// Initialize taxes
			$form->addTemplateSectionInfo(scUIProducts::getTaxesSection($primaryKeyValue));
			
			// Initialize product option groups
			$field = $form->getWidget('ProductOptionGroups');
			$globalOptions = new scSqlRecordsetML('OptionGroups', 'OptionGroupID');
			$globalOptions->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$field->setTable(new scOptionGroupsTable($globalOptions));
			
			// Handle currency
			list($currencySymbolLeft, $currencySymbolRight) = scCurrencyUtil::getBaseCurrencySymbols();
			$this->addMainPageElements(array('%%CURRENCYSYMBOLLEFT%%' => miI18N::htmlEscape($currencySymbolLeft), '%%CURRENCYSYMBOLRIGHT%%' => miI18N::htmlEscape($currencySymbolRight)));
			
			// Set the product code validator
			$record = new scDomainToRecordAdapter($this->getDomainObject());
			$form->addValidator('ProductCode', new scValidatorUniqueFields($form, array('ProductCode'), $record));
			
			// assign product name in current language for the breadcrumb
			if ($this->getController()->getDispatcher()->getActionObj() instanceof miEditAction) {
				$productName = $this->_viewMapperController->getDomainObject()->get('ProductName');
				$productName = $productName[scMultiLanguage::singleton()->getCurrentLanguageId()];
				$form->addMainPageElements(array('%%FMT_PRODUCTNAME%%' => $productName));
			}
			
			// Handle the SFBrowser language
			$langCode = scMultiLanguage::getCurrentLanguageCode();
			$sfBrowserLangCode = ($langCode == 'BG') ? 'bg' : 'en';
			$form->addMainPageElements(array('%%SFBROWSER_LANGCODE%%' => $sfBrowserLangCode));
		}
		
		/**
		 * Called after an action
		 */
		public function postProcess($actionObj, $actionResult)
		{
			parent::postProcess($actionObj, $actionResult);
			
			if ($actionResult === false)
				return;

			if (($actionObj instanceof miExecEditAction) or ($actionObj instanceof miExecCreateAction) or ($actionObj instanceof miExecDeleteAction)) {
				scCategories::updateProductCounts();
			}
			
			if (($actionObj instanceof miExecEditAction) or ($actionObj instanceof miExecCreateAction)) {	
				
				$productId = $this->getDomainObject()->get('ProductID');
				
				// get TaxClasses for each Tax
				$taxClasses = scTax::getAllTaxClassIds();
				// Remove old ProductTaxClasses
				miStaticDBUtil::execDelete('ProductTaxClasses', 'ProductID', $productId);
				// Save ProductTaxClasses
				$selectedTaxClasses = miGetParamDefault('ProductTaxClasses', array());
				$productTaxes = miGetParamDefault('ProductTaxes', array());
				foreach ($selectedTaxClasses as $iSelectedTaxClass) {
					// If the TaxID of the selected tax class is in among the enabled taxes, add the tax class to the product
					if (in_array($taxClasses[$iSelectedTaxClass], $productTaxes)) {
						$values = array(
							'ProductID' => $productId,
							'TaxClassID' => (int) $iSelectedTaxClass
						);
						miStaticDBUtil::execInsert('ProductTaxClasses', $values);
					}
				}
				
				$currenciesWithFixedPrices = scCurrencyUtil::getCurrenciesWithFixedPrices();
				foreach ($currenciesWithFixedPrices as $currency) {
					scCurrencyUtil::updateProductPrices($currency['CurrencyCode'], $currency['CurrencyRate'], $productId);
				}
			}
			
			if ($actionObj instanceof miExecDeleteAction) {
				// Check for coupons or dicounts referring to this product and make them inactive if no products are left
				$discounts = $this->getDomainObject()->getDeactivatedDiscounts();
				$coupons = $this->getDomainObject()->getDeactivatedCoupons();
				
				$message = array($this->getDomainObjectDisplayName() . _t(' successfully deleted.'));
				$messageType = miMessage::MSG_TYPE_INFO;
				if (count($discounts) > 0) {
					$changedDiscounts =  scDiscountService::getDiscountsNames('Discounts', 'DiscountID', 'DiscountName', $discounts);
					$message[] = _t('Discounts: ') .
						scDiscountService::formatDiscountNames($changedDiscounts) .
						_t(' were automatically disabled to prevent unpredicted behaviour.');
						
					$messageType = miMessage::MSG_TYPE_WARNING;
				}
				if (count($coupons) > 0) {
					$changedCoupons = scDiscountService::getDiscountsNames('Coupons', 'CouponID', 'CouponCode', $coupons);
					$message[] = _t('Coupons: ') .
						scDiscountService::formatDiscountNames($changedCoupons) .
						_t(' were automatically disabled to prevent unpredicted behaviour.');
						
					$messageType = miMessage::MSG_TYPE_WARNING;
				}
				
				$message = implode("\r\n", $message);
				$this->addRedirectToListControllerCommand($message, $messageType);
			}
		}
		
		public function addRedirectToEditControllerCommand($productId)
		{
			$url = 'products.php?action=dmEdit&ProductID=' . $productId;
			$this->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
		}

		/**
		 * @return scProductDomainObject
		 */
        protected function createDomainObject()
		{
			return new scProductDomainObject();
		}
	}
	
	class scProductFixedPricesWidget extends miBaseTextWidget {
		
		public function getValue()
		{
			$currenciesWithFixedPrices = miStaticDBUtil::execSelect(
				'SELECT *, "" AS ProductPrice, 1 AS ProductPriceAuto FROM Currencies
				INNER JOIN CurrenciesML USING(CurrencyID)
				WHERE CurrencyAllowFixedPrices AND Currencies.CurrencyID != ? AND LanguageID = ?',
				array(scCurrencyUtil::getBaseCurrency()->getCurrencyId(), scMultiLanguage::singleton()->getCurrentLanguageId())
			);
			
			$productFixedPrices = parent::getValue();
			if (is_null($productFixedPrices)) {
				return $currenciesWithFixedPrices;
			}
			
			foreach ($currenciesWithFixedPrices as &$currency) {
				$currencyCode = $currency['CurrencyCode'];
				if (!isset($productFixedPrices[$currencyCode]))
					continue;
				
				$currency = array_merge($currency, $productFixedPrices[$currencyCode]);
			}
			return $currenciesWithFixedPrices;
		}
		
		public function getEditableControl()
		{
			$productFixedPrices = array_values($this->getValue());
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/products/multi_currency_prices.tmpl');
			$t->addSectionInfo(new miTemplateParserSectionInfo('prices', count($productFixedPrices), miUIUtil::templetizeRows(miUIUtil::transposeRows($productFixedPrices))));
			return array($this->_fieldName => $t->templateParse());
		}
		
		public function getData()
		{
			$currenciesWithFixedPrices = scCurrencyUtil::getCurrenciesWithFixedPrices();
			if (empty($currenciesWithFixedPrices)) {
				return array();
			}
			
			$productFixedPrices = array();
			foreach ($currenciesWithFixedPrices as $currency) {
				$productFixedPrices[$currency['CurrencyCode']] = $currency + array(
					'ProductPrice' => miGetParamDefault('ProductPrice' . $currency['CurrencyCode'], 0),
					'ProductPriceAuto' => miGetParamDefault('ProductPrice' . $currency['CurrencyCode'] . 'Auto', 1)
				);
			}
			
			return array($this->_fieldName => $productFixedPrices);
		}
	}
	
	class scViewMapperProductFixedPrices extends miViewMapper {
		
		protected $_productFixedPrices = array();
		
		public function read()
		{
			$domainObj = $this->_viewMapperController->getDomainObject();
			$productData = $domainObj->getRow();
			if (empty($productData)) {
				return;
			}
			
			$currenciesWithFixedPrices = scCurrencyUtil::getCurrenciesWithFixedPrices();
			foreach ($currenciesWithFixedPrices as &$currency) {
				$priceFieldName = 'ProductPrice' . $currency['CurrencyCode'];
				if (!isset($productData[$priceFieldName])) {
					continue;
				}
				
				$productPrice = $productData[$priceFieldName];
				if ($productPrice == 0) {
					$productPrice = '';
				}
				
				$this->_productFixedPrices[$currency['CurrencyCode']] = array(
					'ProductPrice' => $productPrice,
					'ProductPriceAuto' => $productData[$priceFieldName . 'Auto']
				);
			}
		}
		
		public function get($fieldName)
		{
			return $this->_productFixedPrices;
		}
		
		public function set($fieldName, $fieldValue)
		{
			if (empty($fieldValue)) {
				return;
			}
			
			$domainObj = $this->_viewMapperController->getDomainObject();
			foreach ($fieldValue as $currencyCode => $currency) {
				// The product price in this currency will be updated in the postProcess
				if ($currency['ProductPrice'] == 0) {
					$domainObj->set('ProductPrice' . $currencyCode . 'Auto', 1);
					continue;
				}
				
				// The product price in this currency will be updated in the postProcess
				$domainObj->set('ProductPrice' . $currencyCode . 'Auto', $currency['ProductPriceAuto']);
				if ($currency['ProductPriceAuto'])
					continue;
				
				$domainObj->set('ProductPrice' . $currencyCode, $currency['ProductPrice']);
			}
			
			// Update the base currency product price (since it's hidden)
			$baseCurrency = scCurrencyUtil::getBaseCurrency()->getCurrencyRow();
			if ($baseCurrency['CurrencyAllowFixedPrices']) {
				$domainObj->set('ProductPrice' . $baseCurrency['CurrencyCode'], $domainObj->get('ProductPrice'));
			}
		}
	}
	
	/**
	 * Represents an E-Good by two fields
	 * - The default _fieldName field that is assumed to contain a path to a file
	 * - A prefixed field containing only the basename()
	 */
	class scEgoodWidget extends miBaseTextWidget {
		
		public function getEditableControl()
		{
			$return = parent::getEditableControl();
			$return['FMT_' . $this->_fieldName] = basename($return[$this->_fieldName]);
			return $return;
		}
	}
	
	class scViewMapperProductCategories extends scViewMapperMultiValues {
		
		var $_dafaultCategoryId;
		
		public function __construct($model)
		{
			$this->_tableName = 'ProductXCategories';
			$this->_tableKeyName = 'ProductID';
			$this->_tableValueName = 'CategoryID';
			$this->_sortKeyName = 'ProductSortInCategory';
			parent::__construct($model);
		}
		
		public function update()
		{
			$itemId = (int)$this->getItemId();
			
			if (count($this->_subItemIds) == 0) {
				miStaticDBUtil::execSQL('DELETE FROM ProductXCategories WHERE ProductID = ?', array($itemId));
				return;
			}
			
			miStaticDBUtil::execSQL('DELETE FROM ProductXCategories WHERE ProductID = ? AND CategoryID NOT IN (' . implode(',',$this->_subItemIds) . ')', array($itemId));
			
			$defaultCategoryId = $this->getDefaultCategoryId();
			
			$currentAdditionalCategories = miUtil::getDBArray('SELECT CategoryID FROM ProductXCategories WHERE ProductID = ' . $itemId, 'CategoryID', 'CategoryID');
			$newAdditionalCategories = array_diff($this->_subItemIds, $currentAdditionalCategories, array($defaultCategoryId));
			
			// Save subitems
			$records = array();
			if ($newAdditionalCategories) {
				foreach ($newAdditionalCategories as $subItemId) {
					// Skip the default category
					if ($defaultCategoryId == $subItemId)
						continue;
					
					// Get sort number
					if (scSettings::getSettingValue(scSettings::SETTING_NEW_PRODUCTS_INFRONT_OF_CATEGORY_LISTING) == 1) {
						$newSortId = 1;
						$sql = 'UPDATE ProductXCategories SET ProductSortInCategory = ProductSortInCategory + 1 WHERE CategoryID IN (' . implode(',',$this->_subItemIds) . ') ORDER BY ProductSortInCategory DESC';
						miStaticDBUtil::execSQL($sql);
					} else {
						$newSortId = scProductSortService::getMaxSortInCategory($subItemId) + 1;
					}
						
					$records[] = array(
						$this->_tableKeyName => $itemId,
						$this->_tableValueName => $subItemId,
						$this->_sortKeyName => $newSortId,
					);
				}
			}
			
			if (!empty($records)) {
				miStaticDBUtil::execMultiInsert($this->_tableName, $records);
			}
		}
		
		/**
		 * Get default category id of the current product
		 */
		protected function getDefaultCategoryId()
		{
			return scUtil::getDbValue('Products', 'CategoryID', 'ProductID', $this->getItemId());
		}
	}
	
	class scViewMapperProductCustomUrls extends scViewMapperCustomUrls {
		protected $_entityCode = scFriendlyUrl::TYPE_PRODUCT;
		protected $_tableKeyName = 'ProductID';
	}
	
	class scBundleProductsWidget extends scDynRowsWidget {
		protected function setWebFormData()
		{
			// Set the ProductName, otherwise when there is error in the form the product name is empty
			$records = $this->_webForm->getFormData($this->_fieldName);
			if ($records == null)
				$records = array();
			
			foreach ($records as &$record) {
				$product = miStaticDBUtil::execSelect('SELECT ProductName FROM ProductsML WHERE ProductID=? AND LanguageID=?', array($record['SubProductID'], scMultiLanguage::getCurrentLanguageId()));
				if (!empty($product))
					$record['ProductName'] = $product[0]['ProductName'];
			}
			$this->_webForm->setFormData($this->_fieldName, $records);
			
			return parent::setWebFormData();
		}
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			parent::validateData($errors);
			
			if ($errors->hasErrors()) {
				return;
			}
			
			$isBundle = $this->_webForm->getSubmittedData('ProductIsBundle');
			if (!$isBundle) {
				return;
			}
			
			$bundleProducts = $this->getData();
			if (!isset($bundleProducts['BundleProducts']) || empty($bundleProducts['BundleProducts'])) {
				$errors->addError('BundleProducts', _t('A product bundle must have subproducts.'));
				return;
			}
			
			$bundleId = $this->_webForm->getSubmittedData('ProductID');
			foreach ($bundleProducts['BundleProducts'] as $bundledProduct) {
				if ((int)$bundledProduct['BundleProductQuantity'] <= 0) {
					$errors->addError('BundleProducts', _t('All bundled products must have a minimum quantity of 1.'));
					return;
				}
				if (is_numeric($bundleId) && $bundleId > 0) {
					// There is a ProductID. This means that's not a create action.
					// It is possible to add the same ProductID as subproduct.
					if ($bundleId == $bundledProduct['SubProductID']) {
						$errors->addError('BundleProducts', _t('A product bundle can contain other bundles but cannot contain itself.'));
						return;
					}
					try {
						// It is still possible to include a subproduct that contains the ProductID product.
						$bundledProductObject = new scProductDomainObject();
						$bundledProductObject->read($bundledProduct['SubProductID']);
						if ($bundledProductObject->get('ProductIsBundle')) {
							$bundledProductObject->checkBundleRecursion(array($bundleId));
						}
					} catch (miException $e) {
						$errors->addError('BundleProducts', $e->getMessage());
						return;
					}
				}
			}
			
			$optionGroups = $this->_webForm->getSubmittedData('ProductOptionGroups');
			if (!empty($optionGroups)) {
				$recordset = new miSqlRecordset('OptionGroups');
				$recordset->addFilter(new miSqlFilterIn('OptionGroupID', $optionGroups));
				$recordset->addFilter(new miSqlFilterEqual('OptionGroupType', scProductOptions::GROUP_TYPE_VARIANT));
				if ($recordset->getRecordsCount() > 0) {
					$errors->addError('ProductIsBundle', _t('A product bundle cannot have variant options.'));
					return;
				}
			}
			
			$localOptionGroups = $this->_webForm->getSubmittedData('LocalOptionGroups');
			foreach ($localOptionGroups as $localOptionGroup) {
				if ($localOptionGroup['LocalOptionGroupType'] == scLocalOptions::GROUP_TYPE_VARIANT) {
					$errors->addError('ProductIsBundle', _t('A product bundle cannot have variant options.'));
					return;
				}
			}
		}
	}
	
	class scViewMapperBundleProducts extends scViewMapperDynRows {
		
		public function insert()
		{
			// Check if the primary key is set. If it is unset it.
			// This way duplicated product won't "steal" the bundled
			// products from the one it is being copied from.
			foreach ($this->_rsValues['BundleProducts'] as &$item) {
				$itemIdField = $this->_rsProperties['BundleProducts']['foreignTablePrimaryKey'];
				if (isset($item[$itemIdField])) {
					$item[$itemIdField] = null;
				}
			}
			return parent::insert();
		}
		
		public function update()
		{
			$return = parent::update();
			$this->getProductDomainObject()->retrieveBundleData(); // Set data to product.
			$this->getProductDomainObject()->update(); // And update it.
			return $return;
		}
		
		protected function readRecords($rsProperties)
		{
			$records = parent::readRecords($rsProperties);
			if (empty($records)) {
				return $records;
			}
			
			$subProductIds = miUtil::getKeyValueArray($records, 'SubProductID', 'SubProductID');
			$recordset = new scSqlRecordsetML('Products', 'ProductID');
			$recordset->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$recordset->setSelectFields(array('Products.ProductID, ProductName, ProductCode'));
			$recordset->addFilter(new miSqlFilterIn('Products.ProductID', $subProductIds));
			$rows = $recordset->getAllRecords();
			$rows = miUtil::getArrayWithKeys($rows, 'ProductID');
			
			foreach ($records as &$record) {
				$record['ProductName'] = $rows[$record['SubProductID']]['ProductName'] .
					' (' . $rows[$record['SubProductID']]['ProductCode'] . ')';
			}
			return $records;
		}
		
		/**
		 * @return scProductDomainObject
		 */
		protected function getProductDomainObject()
		{
			$product = $this->_viewMapperController->getView()->getDomainObject();
			if (!($product instanceof scProductDomainObject)) {
				throw new miException(_i('scProductDomainObject expected.'));
			}
			return $product;
		}
	}
	
	class scProductAttributesDataProvider implements IFieldsDataProvider {
		
		public function getFieldsTable()
		{
			return 'ProductClassAttributes';
		}
		
		public function getValuesTable()
		{
			return 'ProductAttributeValues';
		}
		
		public function getFieldIdColumn()
		{
			return 'ProductClassAttributeID';
		}
		
		public function getFieldTypeColumn()
		{
			return 'ProductClassAttributeType';
		}
		
		public function getValueIdColumn()
		{
			return 'ProductAttributeValueID';
		}
		
		public function getValueFieldForType($type)
		{
			return scProductAttributes::singleton()->getValueFieldForType($type);
		}
		
		public function getFieldTypesWithValues()
		{
			return scProductAttributes::singleton()->getFieldTypesWithValues();
		}
		
		public function getFieldsToIgnore()
		{
			return scProductAttributes::singleton()->getSystemAttributes();
		}
	}
	
	class scViewMapperProductAttributes extends scViewMapperFields {
		
		public function __construct(miViewMapperControllerDefault $viewMapperController)
		{
			parent::__construct($viewMapperController);
			
			if (!($this->_domainObject instanceof scProductDomainObject)) {
				throw new miException(_i('scProductDomainObject needed.'));
			}
		}
		
		protected function createDataProvider()
		{
			return new scProductAttributesDataProvider();
		}
		
		protected function createFieldValueDomainObject()
		{
			return new scProductAttributeValue();
		}
		
		protected function getValues($productId)
		{
			$recordset = new miSqlRecordset('ProductAttributeValues');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$result = $recordset->getAllRecords();
			$values = miUtil::getArrayWithKeys($result, 'ProductAttributeValueID');
			
			$attributeValueIds = array_keys($values);
			
			$recordset = new miSqlRecordset('ProductAttributeValuesML');
			$recordset->addFilter(new miSqlFilterIn('ProductAttributeValueID', $attributeValueIds));
			$result = $recordset->getAllRecords();
			$mlValues = array();
			foreach ($result as $row) {
				$mlValues[$row['ProductAttributeValueID']][$row['LanguageID']] = $row['ProductAttributeValueML'];
			}
			
			$recordset = new miSqlRecordset('ProductAttributeMultiValues');
			$recordset->addFilter(new miSqlFilterIn('ProductAttributeValueID', $attributeValueIds));
			$result = $recordset->getAllRecords();
			$multiValues = array();
			foreach ($result as $row) {
				$multiValues[$row['ProductAttributeValueID']][] = $row['ProductClassAttributeMultiValue'];
			}
			
			foreach ($values as &$value) {
				$valueId = $value['ProductAttributeValueID'];
				
				$value['ProductAttributeValueML'] = array();
				if (isset($mlValues[$valueId])) {
					$value['ProductAttributeValueML'] = $mlValues[$valueId];
				}
				
				$value['ProductClassAttributeMultiValue'] = array();
				if (isset($multiValues[$valueId])) {
					$value['ProductClassAttributeMultiValue'] = $multiValues[$valueId];
				}
			}
			return $values;
		}
	}
	
	class scProductAttributesWidget extends miBaseWidget {
		
		public function getEditableControl()
		{
			$productClassId = (int)$this->_webForm->getFormData('ProductClassID');
			if ($productClassId == 0) {
				return array($this->_fieldName => '');
			}
			$values = $this->_webForm->getFormData($this->_fieldName);
			
			$html = scProductAttributes::singleton()->getProductClassAttributesHtml($productClassId, $values);
			return array($this->_fieldName => $html);
		}
		
		function getData()
		{
			$productClassId = (int)miGetParamDefault('ProductClassID', null);
			$attributes = scProductAttributes::singleton()->getProductClassAttributes($productClassId);
			$values = miGetParamDefault($this->_fieldName, array());
			$attributeValues = array();
			foreach ($values as $attributeId => $value) {
				$attributeType = $attributes[$attributeId]['ProductClassAttributeType'];
				$valueField = scProductAttributes::singleton()->getValueFieldForType($attributeType);
				
				$attributeValues[$attributeId] = array(
					'ProductClassAttributeID' => $attributeId,
					$valueField => $value,
				);
			}
			
			return array($this->_fieldName => $attributeValues);
		}
	}
	
	class scValidatorProductAttributes extends miValidator {
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$productClassId = (int)$this->_webForm->getSubmittedData('ProductClassID');
			$attributes = scProductAttributes::singleton()->getProductClassAttributes($productClassId);
			$validators = scProductAttributes::singleton()->getProductClassValidators($productClassId);
			
			$values = $this->_webForm->getSubmittedData($this->_fieldName);
			foreach ($values as $value) {
				$attributeId = $value['ProductClassAttributeID'];
				$attributeType = $attributes[$attributeId]['ProductClassAttributeType'];
				$valueField = scProductAttributes::singleton()->getValueFieldForType($attributeType);
				if (empty($value[$valueField])) {
					// Skip empty values. If "no empty" validator is supported this check has to be reworked.
					continue;
				}
				
				$displayName = $attributes[$attributeId]['ProductClassAttributeName'];
				switch ($validators[$attributeId]) {
					case scProductAttributes::VALIDATOR_NUMBER:
						if (!preg_match('#^[0-9]{1,10}\.?([0-9]){0,10}$#', $value[$valueField]))
							$errors->addError($this->_fieldName, $displayName . ': '. _t('Invalid number.'));
						break;
				}
			}
		}
	}
	
	class scProductAttributeValue extends scDomainObject {
		
		protected $_multiValuesTable = 'ProductAttributeMultiValues';
		protected $_multiValuesField = 'ProductClassAttributeMultiValue';
		protected $_multiValues = array();
		
		public function __construct()
		{
			parent::__construct(new scSqlRecordML('ProductAttributeValues', 'ProductAttributeValueID',
				array('ProductAttributeValueML'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
		
		/**
		 * Allow setting the PK.
		 *
		 * @param string $name
		 * @param mixed $value
		 */
		public function set($name, $value)
		{
			if ($name == $this->_multiValuesField) {
				$this->_multiValues = $value;
			} else {
				$this->_record->set($name, $value);
			}
		}
		
		public function read($pkValue)
		{
			parent::read($pkValue);
			
			$q = 'SELECT ' . $this->_multiValuesField . ' FROM ' . $this->_multiValuesTable .
				' WHERE ' . $this->getPKName() . ' = ' . $this->get($this->getPKName());
			$this->_multiValues = miStaticDBUtil::execSelect($q);
			$this->_multiValues = array_values(miUtil::getKeyValueArray($this->_multiValues, $this->_multiValuesField, $this->_multiValuesField));
		}
		
		public function insert()
		{
			$attributeValueId = parent::insert();
			
			$multiInsertValues = $this->getMultiValuesForMultiInsert($attributeValueId);
			if (!empty($multiInsertValues)) {
				miStaticDBUtil::execMultiInsert($this->_multiValuesTable, $multiInsertValues);
			}
			
			return $attributeValueId;
		}
		
		public function update()
		{
			$attributeValueId = parent::update();
			
			// INSER IGNORE multi-values
			$multiInsertValues = $this->getMultiValuesForMultiInsert($attributeValueId);
			if (!empty($multiInsertValues)) {
				miStaticDBUtil::execMultiInsert($this->_multiValuesTable, $multiInsertValues, miDBUtilImpl::FLAG_IGNORE);
			}
			
			// DELETE left behind values
			$newValues = array_map('sql_escape_string', $this->_multiValues);
			$deleteQuery = 'DELETE FROM ' . $this->_multiValuesTable . ' WHERE ' . $this->getPKName() . ' = ' . $attributeValueId;
			if (!empty($newValues)) {
				$deleteQuery .= ' AND ' . $this->_multiValuesField . ' NOT IN (' . implode(', ', $newValues) . ')';
			}
			miStaticDBUtil::execSQL($deleteQuery);
			
			return $attributeValueId;
		}
		
		protected function getMultiValuesForMultiInsert($attributeValueId)
		{
			if (empty($this->_multiValues)) {
				return array();
			}
			
			$values = array();
			foreach ($this->_multiValues as $value) {
				$values[] = array(
					$this->getPKName() => $attributeValueId,
					$this->_multiValuesField => $value,
				);
			}
			return $values;
		}
	}
	
	class scViewMapperRelatedProducts extends scViewMapperDynRows {
		
		protected function insertRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			$mainIdField = $rsProperties['foreignKey']; // e.g. ProductID
			$relatedIdField = $rsProperties['foreignTablePrimaryKey']; // e.g. RelatedProductID
			
			$data = $record->getRow();
			$values = array();
			
			$values[] = array(
					$relatedIdField => $data[$relatedIdField],
					$mainIdField => $data[$mainIdField],
					'RelatedProductSort' => $rowIndex + 1
				);
			
			if ($data['BiDirectionalLink']) {
				// Get max sort + 1
				$maxRowIndex = miStaticDBUtil::execSelect('SELECT MAX(RelatedProductSort) AS RelatedProductSort FROM RelatedProducts' .
					' WHERE ProductID = ?', array($data[$relatedIdField]));
				$maxRowIndex = (empty($maxRowIndex) ? 1 : $maxRowIndex[0]['RelatedProductSort'] + 1);
				
				$values[] = array(
					$relatedIdField => $data[$mainIdField],
					$mainIdField => $data[$relatedIdField],
					'RelatedProductSort' => $maxRowIndex
				);
			} else {
				// delete the bi-directional link, if one exists
				miStaticDBUtil::execDelete($rsProperties['table'], array($mainIdField, $relatedIdField), array($data[$relatedIdField], $data[$mainIdField]));
			}
			
			miStaticDBUtil::execMultiInsert($rsProperties['table'], $values, miDBUtilImpl::FLAG_IGNORE);
			return $data[$relatedIdField];
		}
		
		protected function updateRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			return $this->insertRecord($record, $rsValues, $rsProperties, $rowIndex);
		}
		
		protected function saveDynRows($rsValues, $rsProperties)
		{
			miStaticDBUtil::execDelete($rsProperties['table'], array($rsProperties['foreignKey']),
				array($this->getForeignKeyValue($rsProperties['foreignKey'])));
			parent::saveDynRows($rsValues, $rsProperties);
		}
		
		protected function readRecords($rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			
			// get products related to this one, as well as product this one is related to
			$selectQuery = 'SELECT * FROM ' . $rsProperties['table'] . ' WHERE  ' .$rsProperties['foreignKey'].' = ?';
			$selectQuery .= ' OR ' . $rsProperties['foreignTablePrimaryKey'] . '= ?';
			$selectQuery .= ' ORDER BY RelatedProductSort';
			$rows = miStaticDBUtil::execSelect($selectQuery, array($foreignKeyValue, $foreignKeyValue));
			
			// map products to their relations
			$relatedItems = array();
			foreach ($rows as $row)
				$relatedItems[$row[$rsProperties['foreignKey']]][] = $row[$rsProperties['foreignTablePrimaryKey']];
			
			// if this product have no relations, return.
			// Note: it can be related to other products without other products to be related to it  
			if (!isset($relatedItems[$foreignKeyValue]))
				return array();
			
			$rows = $this->getRelatedItemsData($relatedItems[$foreignKeyValue]);
			
			// check which related products have back relation
			foreach ($rows as $key => $value)
				$rows[$key]['BiDirectionalLink'] = isset($relatedItems[$value['ProductID']]) ? 1 : 0;

			return $rows;
		}
		
		protected function getRelatedItemsData($relatedItemIds)
		{
			$query = 'SELECT ProductsML.*, ProductsML.ProductID AS RelatedProductID FROM Products' .
				' INNER JOIN ProductsML USING(ProductID)' .
				' WHERE LanguageID = ' . scMultiLanguage::getCurrentLanguageId() . ' AND Products.ProductID IN (' . implode(',', $relatedItemIds) . ')' .
				' ORDER BY Products.ProductID = ' . implode(' DESC, Products.ProductID = ', $relatedItemIds) . ' DESC';
			$result = miStaticDBUtil::execSelect($query);
			
			return $result;
		}
	}
	
	/**
	 * 
	 */
	class scProductImagesUploadWidget extends scFileUploadDynRowsWidget {
		protected function createFieldObjects()
		{
			parent::createFieldObjects();
			
			$rsFields = $this->getRsFieldObjs('ProductImagePicture');
			foreach ($rsFields as $field)
				$field->setProps(
					scPaths::getFilenamePath(scPaths::getFilenamePrefix(scPaths::RESOURCE_PRODUCT_IMAGE), scPaths::RESOURCE_PRODUCT_IMAGE),
					scPaths::getStoreWebPath() . scPaths::getFilenamePath(scPaths::getFilenamePrefix(scPaths::RESOURCE_PRODUCT_IMAGE), scPaths::RESOURCE_PRODUCT_IMAGE, scPaths::PATH_WEB),
					scPaths::getFilenameSuffix(scPaths::RESOURCE_PRODUCT_IMAGE));
					
			$rsFields = $this->getRsFieldObjs('ProductImageThumb');
			foreach ($rsFields as $field)
				$field->setProps(
					scPaths::getFilenamePath(scPaths::getFilenamePrefix(scPaths::RESOURCE_PRODUCT_THUMB), scPaths::RESOURCE_PRODUCT_THUMB),
					scPaths::getStoreWebPath() . scPaths::getFilenamePath(scPaths::getFilenamePrefix(scPaths::RESOURCE_PRODUCT_THUMB), scPaths::RESOURCE_PRODUCT_THUMB, scPaths::PATH_WEB),
					scPaths::getFilenameSuffix(scPaths::RESOURCE_PRODUCT_THUMB));
		}
	}
	
	/**
	 * Duplicates the product with all of it's attributes, files etc
	 */
	class scDuplicateProductAction extends miCreateAction {		
		public function doAction()
		{
			// Create the webform if not created by the view
			$form = $this->getWebForm();	// Dummy

			$form->addMainPageElements(
				array(
					'%%DUPLICATE%%' => 'true',
					'%%DUPLICATEACTION%%' => 'dmExecDuplicate',
					'%%TITLE%%' => miI18N::htmlEscape(_t('Duplicate'))
				)
			);
					
			$vmController = $this->_view->getViewMapperController();
			$vmController->getDomainObject()->set('ProductID', miGetParam('ProductID'));
			$vmController->read();
			$vmController->setDataToWebForm();

			$this->_view->callPlugin($this, 'preShowForm');
			
			$form = $this->getWebForm();
			$this->showForm($form, $this->_view->getTemplateFileName(scProductsView::TEMPLATE_ID_CREATE), true);
			return true;
		}
	}
	
	/**
	 * Performs the product duplication
	 */
	class scExecDuplicateAction extends miExecCreateAction {
		public function doAction()
		{
			$form = $this->getWebForm();	// Dummy

			$form->addMainPageElements(
				array(
					'%%DUPLICATE%%' => 'true',
					'%%DUPLICATEACTION%%' => 'dmExecDuplicate',
					'%%TITLE%%' => miI18N::htmlEscape(_t('Duplicate'))
				)
			);
			
			return parent::doAction();
		}
	}
	
	class scCreateProductAction extends miCreateAction {
		public function doAction()
		{
			$dummyForm = new miWebForm(array());
			$categorySelectWidget = new scCategorySelect($dummyForm, array());
			$categoriesArray = $categorySelectWidget->getCategoriesTree(0, true);
			if (count($categoriesArray) == 0) {
				$msg = _t('Create a category first.');
				$this->_view->addRedirectToListControllerCommand($msg);
				return false;
			}
			parent::doAction();
		}
	}
	
	class scDeleteProductAction extends miExecDeleteAction {
		
		public function doAction()
		{
			try {
				// Basic initialization
				$this->_view->initMainPageElements();
				$parser = new scTemplateParser();
				$parser->assignArray($this->_view->getMainPageElements());
				
				$productId = (int)miGetParam('ProductID');
				$productEgoods = $this->_view->getDomainObject()->getDeletableProductEgoods($productId);
				if (empty($productEgoods)) {
					// The product does not contain any deletable E-Goods. We don't need user's confirmation.
					return parent::doAction();
				}
				
				$languageId = scMultiLanguage::getCurrentLanguageId();
				
				// Assign the product id
				$parser->assign('%%PRODUCTID%%', $productId);
				
				// Get the name of the product marked for deletion
				$record = new scSqlRecordML('Products', 'ProductID', array('ProductName'), $languageId);
				$record->readPK($productId);
				$productName = $record->get('ProductName');
				$productName = $productName[$languageId];
				// Assign the product name
				$parser->assign('%%PRODUCTNAME%%', $productName);
				
				$egoodsCount = count($productEgoods);
				$parser->assign('%%EGOODSCOUNT%%', $egoodsCount);
				
				$msg = new miMessage();
				$parser->assignArray($msg->getMessageTemplateVars());
				
				$section = new miTemplateParserSectionInfo('egoods');
				$section->setValues(miUIUtil::templetizeRows(miUIUtil::transposeRows($productEgoods)));
				$section->setIterationsCount($egoodsCount);
				$parser->addSectionInfo($section);
				
				$parser->readTemplate($this->_view->getTemplateFileName(scProductsView::TEMPLATE_ID_DELETE));
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $parser->templateParse()));
				return false;// Do not call postProcess()
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_DELETE_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scSaveAndEditProductAction extends miExecEditAction {
		
		/**
		 * @param array $viewMapperControllerResult
		 * @return true
		 */
		protected function onSuccess($viewMapperControllerResult = null)
		{
			$recordId = $this->parseViewMapperControllerResult($viewMapperControllerResult);
			$msg = sprintf($this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG'), $recordId);
			$loc = '?action=dmEdit&ProductID=' . $recordId .
				'&'. miMessage::PARAM_MESSAGE .'='. urlencode($msg) .
				'&'. miMessage::PARAM_MESSAGE_TYPE .'='. urlencode(miMessage::MSG_TYPE_INFO);
			
			$stateParams = miAppFactory::singleton()->getStateObj()->getStateUrlParams();
			if ($stateParams != '') {
				$loc .= '&' . $stateParams;
			}
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($cmd);
			return true;
		}
	}
	
	class scDownloadProductEgoodAction extends miAction {
		
		public function doAction()
		{
			try {
				$productEgood = new scProductEgood((int)miGetParam('ProductEgoodID'));
				$productEgood->download();
			} catch (miException $e) {
				$this->redirectToDelete($productEgood->get('ProductID'), $e->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			return true;
		}
		
		/**
		 * @param int $orderId
		 * @param string $msg
		 * @param int $msgType
		 */
		protected function redirectToDelete($productId, $msg, $msgType)
		{
			$loc = 'products.php?action=dmDelete&ProductID='. $productId .
				'&'. miMessage::PARAM_MESSAGE .'='. urlencode($msg) .
				'&'. miMessage::PARAM_MESSAGE_TYPE .'='. urlencode($msgType);
			scUtil::redirect($loc);
		}
	}
	
	class scGetProductClassAttributesAction extends miActionWithWebForm {
		
		public function doAction()
		{
			$html = scProductAttributes::singleton()->getProductClassAttributesHtml((int)miGetParam('ProductClassID'), array());
			$command = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			
			$this->_view->addControllerCommand($command);
			return true;
		}
	}
	
	class scSqlFilterProductsCategoryId extends miSqlFilterEqual {
		
		public function getSql()
		{
			if ((string)$this->_value == '')
				return '';
			return '(Products.' . sql_escape_string($this->_name) . $this->_operator . '"' . sql_escape_string($this->_value) . '" OR ProductXCategories.'.sql_escape_string($this->_name) . $this->_operator .' "'.$this->_value.'")';
		}
	}
	
	class scSqlFilterProductsDescriptions extends miSqlFilterSubstring {
		
		public function getSql()
		{
			if ((string)$this->_value == '')
				return '';
			$fields = array(
				'ProductDescription',
				'ProductDetailedDescription',
			);
			
			$phrase = miStaticDBUtil::escapeStringForLike($this->_value);
			
			$sql = array();
			foreach ($fields as $field) {
				$sql[] = $field . ' LIKE "%' . $phrase . '%"';
			}
			
			return '((' . implode(') OR (', $sql) . '))';
		}
	}
	
	class scSqlFilterProductsCode extends miSqlFilterSubstring {
		
		public function getSql()
		{
			if ($this->_value == '')
				return '';
			$productSql = parent::getSql();
			
			$variantsFilter = new miSqlFilterSubstring('ProductVariantCode', $this->_value);
			$variantsSql = 'Products.ProductID IN (SELECT ProductID FROM ProductVariants WHERE ' . $variantsFilter->getSql() . ')';
			
			return '((' . $productSql . ') OR (' . $variantsSql . '))';
		}
	}
	
	class scSaveCategorySort extends miAction {
		
		public function doAction()
		{
			try {
				miStaticDBUtil::startTransaction();
				
				$sourceProductId = (int)miGetParamDefault('dragSortSourceProductId', 0);
				$targetProductId = (int)miGetParamDefault('dragSortTargetProductId', 0);
				$insertBeforeTarget = miGetParamDefault('dragSortInsertBeforeTarget', 0);
				$categoryId = (int)miGetParamDefault('CategoryID', 0);
				
				$sourceProductOldSort = scProductSortService::getProductSortInCategory($sourceProductId, $categoryId);
				$targetProductSort = scProductSortService::getProductSortInCategory($targetProductId, $categoryId);
				
				if (($sourceProductId <= 0) || ($categoryId <= 0) || ($targetProductSort === false) || ($sourceProductOldSort === false)) {
					throw new miException(_i('Update failed.'));
				}
				
				if ($insertBeforeTarget) {
					// This can happen if someone drops the first item onto itself
					if ($sourceProductOldSort < $targetProductSort)
						die(_i('OK'));
					
					// Product dragged to the beginning of the list
					$this->updateProductSorting(true, '<=', '>=', $targetProductSort, $sourceProductOldSort, $categoryId, $sourceProductId, $targetProductSort);
				} else {
					// Product dragged anywhere except at the beginning of the list
					if ($targetProductSort < $sourceProductOldSort) {
						// Drag product up
						$this->updateProductSorting(true, '<=', '>', $targetProductSort, $sourceProductOldSort, $categoryId, $sourceProductId, $targetProductSort + 1);		
					} else {
						// Drag product down
						$this->updateProductSorting(false, '>', '<=', $targetProductSort, $sourceProductOldSort, $categoryId, $sourceProductId, $targetProductSort);
					}
				}
				miStaticDBUtil::commit();
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				die($exception->getMessage());
			}
			
			die(_i('OK'));
		} 
		
		
		protected function updateProductSorting($pushItemsDown, $sign1, $sign2, $targetProductSort, $sourceProductOldSort,
			$categoryId, $sourceProductId, $sourceProductNewSort)
		{
			// If the product is dropped in the same place that it was before - do nothing
			if ($sourceProductOldSort == $sourceProductNewSort)
				return true;
			
			// Check for duplicates
			$query = 'SELECT COUNT(1) AS N FROM Products WHERE CategoryID = ? AND ProductSortInCategory = ? AND ProductID <> ?';
			$res = miStaticDBUtil::execSelect($query, array($categoryId, $sourceProductOldSort, $sourceProductId));
			$duplicatesInProducts = $res[0]['N'];
			$query = 'SELECT COUNT(1) AS N FROM ProductXCategories WHERE CategoryID = ? AND ProductSortInCategory = ? AND ProductID <> ?';
			$res = miStaticDBUtil::execSelect($query, array($categoryId, $sourceProductOldSort, $sourceProductId));
			$duplicatesInCats = $res[0]['N'];
			
			if ($duplicatesInProducts > 0) {
				$query = 'UPDATE Products SET ProductSortInCategory = ProductSortInCategory + ? WHERE CategoryID = ? AND ProductSortInCategory >= ? ORDER BY ProductSortInCategory DESC';
				miStaticDBUtil::execSQL($query, array($duplicatesInProducts, $categoryId, $sourceProductOldSort));
			}
			if ($duplicatesInCats > 0) {
				$query = 'UPDATE ProductXCategories SET ProductSortInCategory = ProductSortInCategory + ? WHERE CategoryID = ? AND ProductSortInCategory >= ? ORDER BY ProductSortInCategory DESC';
				miStaticDBUtil::execSQL($query, array($duplicatesInCats, $categoryId, $sourceProductOldSort));
			}
			
			// Temporarily put the source product in last place (prevent the db unique key from errors)
			$maxSortInCategory = scProductSortService::getMaxSortInCategory($categoryId);
			$query = 'UPDATE Products SET ProductSortInCategory = ? WHERE CategoryID = ? AND ProductSortInCategory = ?';
			miStaticDBUtil::execSQL($query, array($maxSortInCategory + 1, $categoryId, $sourceProductOldSort));
			$query = 'UPDATE ProductXCategories SET ProductSortInCategory = ? WHERE CategoryID = ? AND ProductSortInCategory = ?';
			miStaticDBUtil::execSQL($query, array($maxSortInCategory + 2, $categoryId, $sourceProductOldSort));
			
			// Fix unique key overlaping
			if ($pushItemsDown)
				$sqlOrder = 'ORDER BY ProductSortInCategory DESC';
			else
				$sqlOrder = 'ORDER BY ProductSortInCategory ASC';
				
			// Change the sorting of the products the fall in the desired diapason
			$query = 'UPDATE Products SET ProductSortInCategory = ProductSortInCategory ' . (($pushItemsDown) ? '+' : '-') . ' 1' .
				' WHERE ProductSortInCategory ' .$sign1. ' ? AND ProductSortInCategory '.$sign2.' ? AND CategoryID = ? ' . $sqlOrder;
			miStaticDBUtil::execSQL($query, array($sourceProductOldSort, $targetProductSort, $categoryId));
			$query = 'UPDATE ProductXCategories SET ProductSortInCategory = ProductSortInCategory ' . (($pushItemsDown) ? '+' : '-') . ' 1' .
				' WHERE ProductSortInCategory ' .$sign1. ' ? AND ProductSortInCategory '.$sign2.' ? AND CategoryID = ?' . $sqlOrder;
			miStaticDBUtil::execSQL($query, array($sourceProductOldSort, $targetProductSort, $categoryId));
			
			// Put the source product in the desired sorting position
			$query = 'UPDATE Products SET ProductSortInCategory = ? WHERE ProductID = ? AND CategoryID = ?';
			miStaticDBUtil::execSQL($query, array($sourceProductNewSort, $sourceProductId, $categoryId));
			$query = 'UPDATE ProductXCategories SET ProductSortInCategory = ? WHERE ProductID = ? AND CategoryID = ?';
			miStaticDBUtil::execSQL($query, array($sourceProductNewSort, $sourceProductId, $categoryId));
		}
	}
	
	/**
	 * Set the CategoryIDFilter condition, even if it is not passed as parameter
	 */
	class scProductsTableCustom extends scProductsTable {
		
		public function getState()
		{
			$params = parent::getState();
			$params['CategoryIDCondition'] = miGetParamDefault('CategoryIDCondition', 'category');
			return $params;
		}
		
		protected function addMainUiElements(&$t)
		{
			parent::addMainUiElements($t);
			
			// Enable ajax sorting if in category
			if ((miGetParamDefault('CategoryID' . miTableFilters::FILTER_NAME_SUFFIX, '') != '') &&
				(miGetParamDefault(miTableSorter::PARAM_SORTBY, '') == 'ProductIDSort') &&
				(miGetParamDefault(miTableSorter::PARAM_SORTDIR, '') == 'ASC' )) {
					$t->assign('%%AJAXSORTABLEID%%' , 'sortable_products_in_cat');
			} else {
				$t->assign('%%AJAXSORTABLEID%%', 'notsortable_products_in_cat');
			}
			$t->assign('%%CATEGORYIDFORSORTING%%', (int)miGetParamDefault('CategoryID' . miTableFilters::FILTER_NAME_SUFFIX, 0));
		}
	}
	
	class scProductsExportCsvAction extends scExportCSVAction {
		
		public function getExportFields()
		{
			$this->getPrintExportUtil()->setExtraFieldsCallback('CategoryID', 'CategoryBranch', array('scProductsView', 'getCategoryBranchForExport'));
			$exportFields = parent::getExportFields();
			
			// Add the price for each currency that allows fixed prices
			$currenciesWithFixedPrices = scCurrencyUtil::getCurrenciesWithFixedPrices();
			foreach ($currenciesWithFixedPrices as $currency) {
				$exportFields[] = 'ProductPrice' . $currency['CurrencyCode'];
			}
			
			return $exportFields;
		}
	}
	
	class scOptionalTextWidget extends miBaseTextWidget {
		
		public function getData()
		{
			$data = parent::getData();
			if ($data[$this->_fieldName] == '') {
				$data[$this->_fieldName] = null;
			}
			return $data;
		}
	}
	
	class scProductVariantsWidget extends scDynRowsWidget {
		
		// Just for adding a reset button in template for variants wholesale prices
		protected function parseTemplate($sectionInfo)
		{
			$properties = $this->getProperties();
			$productId = miGetParamDefault($properties['foreignKey'], 0);
			$hasWholesalePricesSection = new miTemplateParserSectionInfo('hasWholesalePrices', 0, array('%%PRODUCTID%%' => $productId));
			
			if (!empty($properties['wholesalePriceField']) && $productId)
				$hasWholesalePricesSection->setIterationsCount(1);
			
			$rsTpl = new scTemplateParser();
			$rsTpl->readTemplate($this->_template);
			$this->assignTemplateVars($rsTpl);
			$rsTpl->setSectionInfos(array($sectionInfo, $hasWholesalePricesSection));
			return $rsTpl->templateParse();
		}
	}
	
	class scResetVariantsWholesalePricesAction extends miAction {
		
		public function doAction()
		{
			$productId = intval(miGetParam('ProductID'));
			
			miStaticDBUtil::startTransaction();
			miStaticDBUtil::execSQL('DELETE FROM ProductWholesalePrices WHERE ProductID = ? AND ProductVariantID IS NOT NULL', array($productId));
			
			$productWholesalePrices = miStaticDBUtil::execSelect('SELECT * FROM ProductWholesalePrices WHERE ProductID = ?', array($productId));
			$productVariants = miUtil::getDBArray(sprintf('SELECT ProductVariantID FROM ProductVariants WHERE ProductID = %d', $productId),
				'ProductVariantID', 'ProductVariantID');
			$variantPrices = array();
			foreach ($productVariants as $variantId)
				foreach ($productWholesalePrices as $productWholesalePrice)
					$variantPrices[] = array(
						'ProductID' => $productId,
						'ProductVariantID' => $variantId,
						'CustomerGroupID' => $productWholesalePrice['CustomerGroupID'],
						'ProductWholesalePriceQuantity' => $productWholesalePrice['ProductWholesalePriceQuantity'],
						'ProductWholesalePriceAmount' => $productWholesalePrice['ProductWholesalePriceAmount']
					);
			
			miStaticDBUtil::execMultiInsert('ProductWholesalePrices', $variantPrices);
			miStaticDBUtil::commit();
			
			$url = 'products.php?action=dmEdit&ProductID=' . $productId;
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
			return true;
		}
	}
	
	class scProductVariantPriceWidget extends miBaseTextWidget {
		
		protected $_realFieldName = 'ProductVariantPrice';
		public static $variantsWholesalePrices = array();
		
		protected function getFormattedSubmittedData()
		{
			$properties = $this->_webForm->getWidget('ProductVariants')->getProperties();
			
			$rowIndex = scDynRowsWidget::getRowIndexByFieldName($this->_fieldName);
			$variantPrices = array();
			$n = 0;
			
			if (empty(self::$variantsWholesalePrices))
				self::$variantsWholesalePrices = miGetParamDefault($properties['wholesalePriceField'], array());
			
			$keys = array_keys(self::$variantsWholesalePrices);
			$variantId = array_shift($keys);
			$customerGroups = array_shift(self::$variantsWholesalePrices);
			foreach ($customerGroups as $customerGroupId => $variantQuantities) {
				foreach ($variantQuantities as $variantQuantity => $priceAmount) {
					$variantPrices[] = array(
						'ProductID' => $this->_webForm->getSubmittedData('ProductID'),
						'ProductVariantID' => $variantId,
						'CustomerGroupID' => $customerGroupId,
						'ProductWholesalePriceQuantity' => $variantQuantity,
						'ProductWholesalePriceAmount' => $priceAmount,
					);
				}
			}
			
			if (!empty($keys) && !empty(self::$variantsWholesalePrices))
				self::$variantsWholesalePrices = array_combine($keys, self::$variantsWholesalePrices);
			
			return $variantPrices;
		}
		
		protected function getFormatedData($productVariants)
		{
			$customerGroupsArray = array();
			foreach ($productVariants as $productVariant)
				$customerGroupsArray[$productVariant['CustomerGroupID']][] = $productVariant['ProductWholesalePriceQuantity'];
			
			return $customerGroupsArray;
		}
		
		public function getData()
		{
			$properties = $this->_webForm->getWidget('ProductVariants')->getProperties();
			
			if (empty($properties['wholesalePriceField']) || !miHasParam($properties['wholesalePriceField']))
				return parent::getData();
			
			return array($this->_realFieldName => $this->getFormattedSubmittedData());
		}
		
		public function getEditableControl()
		{
			$value = $this->getValue();
			if (is_array($value)) {
				// Init arrays
				$forSection = array();
				$subsections = array();
				$byQuantity = array();
				$groupNames = array();
				
				$customerGroupNames = scCustomerGroups::getCustomerGroupNames();
				
				// Group by quantity
				foreach ($value as $val) {
					$byQuantity[$val['ProductWholesalePriceQuantity']][] = $val;
					$groupNames[$val['CustomerGroupID']] = array('groupName' => $customerGroupNames[$val['CustomerGroupID']]);
				}
				
				// Check if a new customer group is added
				$newGroups = array_diff(array_keys($customerGroupNames), array_keys($groupNames));
				foreach ($newGroups as $newGroupId) {
					$groupNames[$newGroupId] = array('groupName' => $customerGroupNames[$newGroupId]);
					foreach ($byQuantity as &$byQty) {
						$latestPrice = end($byQty);
						$latestPrice['ProductWholesalePriceAmount'] = '';
						$latestPrice['CustomerGroupID'] = $newGroupId;
						$byQty[] = $latestPrice;
					}
				}
				
				$byQuantity = array_values($byQuantity);
				$groupNames = array_values($groupNames);
				
				// Generate the subsections and the main section array
				foreach ($byQuantity as $quantity) {
					$forSection[] = array('ProductWholesalePriceQuantity' => $quantity[0]['ProductWholesalePriceQuantity']);
					$subsections[] = new miTemplateParserSectionInfo('priceFields', count($quantity), miUIUtil::templetizeRows(miUIUtil::transposeRows($quantity)));
				}
				
				$section = new miTemplateParserSectionInfo('variantsPrices', count($forSection), miUIUtil::templetizeRows(miUIUtil::transposeRows($forSection)));
				$section->addSubsectionsArray('priceFields', $subsections);
				
				// Generate the html
				$t = new scTemplateParser();
				$t->readTemplate('admin/products/variants_prices_rs.tmpl');
				$t->addSectionInfo(new miTemplateParserSectionInfo('groupNames', count($groupNames), miUIUtil::templetizeRows(miUIUtil::transposeRows($groupNames))));
				$t->addSectionInfo($section);
				
				$html = $t->templateParse();
			} else {
				$html = '<input type="text" name="' . $this->_fieldName . '" value="' . miI18N::htmlEscape($value) . '" />';
			}
			
			return array($this->_fieldName => $html);
		}
	}
	
	class scValidatorOptionalInt extends scValidatorInt {
		
		protected $_regEx = '#^[0-9]*$#';
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$formData = $this->_webForm->getSubmittedDataRow();
			if (!isset($formData[$this->_fieldName])) {
				return;
			}
			return parent::validate($errors);
		}
	}
	
	class scProductsSqlRecordset extends scSqlRecordsetML {
		
		public function __construct()
		{
			parent::__construct('Products', 'ProductID');
		}
		
		/**
		 * Acts as if there are not _havingFilterNames set as the "ProductIDSort"
		 * does not have an impact on the records count.
		 */
		public function getRecordsCount()
		{
			$filterClause = $this->getSqlFilterClause();
			if ($filterClause != '')
				$filterClause = ' WHERE ' . $filterClause;
			
			$query = 'SELECT COUNT(1) AS N ' . $this->getTablesClause() . $filterClause  . $this->getGroupByClause();				
			
			$rows = miStaticDBUtil::execSelect($query);
			if (isset($rows[0])) {
				return (int)$rows[0]['N'];
			}
			return 0;
		}
	}
	
	$products = new scProductsSqlRecordset();
	$products->setHavingFilterNames(array('ProductIDSort'));
	
	$categoryId = (int)miGetParamDefault('CategoryID' . miTableFilters::FILTER_NAME_SUFFIX, '');
	$sortBy = miGetParamDefault(miTableSorter::PARAM_SORTBY, '');
	$sortDir = miGetParamDefault(miTableSorter::PARAM_SORTDIR, '');
	if ($categoryId && ((($sortBy == '') || ($sortBy == 'ProductIDSort')) && (($sortDir == '') || ($sortDir == 'ASC')))) {
		// Set default sorting to "ProductSortInCategory"
		if (miGetParamDefault(miTableSorter::PARAM_SORTBY, '') == '') {
			miSetParam(miTableSorter::PARAM_SORTBY, 'ProductIDSort');
			miSetParam(miTableSorter::PARAM_SORTDIR, 'ASC');
		}
	} else if ($sortBy == '') {
		miSetParam(miTableSorter::PARAM_SORTBY, 'ProductID');
		miSetParam(miTableSorter::PARAM_SORTDIR, 'DESC');
	}
	
	$selectFields = array();
	$selectFields[] = 'Products.*';
	$selectFields[] = 'ProductsML.LanguageID';
	$selectFields[] = 'ProductsML.ProductName';
	$selectFields[] = 'ProductsML.ProductDescription';
	$selectFields[] = 'ProductsML.ProductDetailedDescription';
	
	if ($categoryId) {
		$selectFields[] = 'IF(ProductXCategories.ProductSortInCategory, ProductXCategories.ProductSortInCategory, Products.ProductSortInCategory) AS ProductIDSort';
		$products->addJoinCondition('LEFT', 'ProductXCategories', ' ON Products.ProductID = ProductXCategories.ProductID AND ProductXCategories.CategoryID = ' . $categoryId);
	} else {
		$selectFields[] = 'Products.ProductID as ProductIDSort';
	}
	
	$products->setSelectFields($selectFields);

	$table = new scProductsTableCustom($products);
	
	$view = new scProductsView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	
	$page->getDispatcher()->setActionHandler('dmCreate', 'scCreateProductAction');
	$page->getDispatcher()->setActionHandler('dmDelete', 'scDeleteProductAction');
	$page->getDispatcher()->setActionHandler('dmDuplicate', 'scDuplicateProductAction');
	$page->getDispatcher()->setActionHandler('dmExecDuplicate', 'scExecDuplicateAction');
	$page->getDispatcher()->setActionHandler('dmSaveAndEdit', 'scSaveAndEditProductAction');
	$page->getDispatcher()->setActionHandler('dmExecEdit', 'miExecEditAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scProductsExportCsvAction');
	$page->getDispatcher()->setActionHandler('dmDownloadEgood', 'scDownloadProductEgoodAction');
	$page->getDispatcher()->setActionHandler('dmGetAttributes', 'scGetProductClassAttributesAction');
	$page->getDispatcher()->setActionHandler('dmSaveCategorySort', 'scSaveCategorySort');
	$page->getDispatcher()->setActionHandler('dmResetVariantsWholesalePrices', 'scResetVariantsWholesalePricesAction');
	$page->dispatchAndShow();
?>
