<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_PRODUCTS);
	
	class scRegenerateImagesView extends scView {
		
		const TEMPLATE_ID_REGENERATE_IMAGES = 'regenerate';
		
		protected $_templates = array(
			self::TEMPLATE_ID_REGENERATE_IMAGES => 'admin/regenerate_images/regenerate_images.tmpl',
		);
	}
	
	class scRegenerateImagesAction extends miAction {
		
		public function doAction()
		{
			try {
				$t = new scTemplateParser();
				$t->readTemplate($this->_view->getTemplateFileName(scRegenerateImagesView::TEMPLATE_ID_REGENERATE_IMAGES));
				$t->assign('%%PRODUCTIMAGES%%', $this->getProductImagesCount());
				$t->assign('%%VARIANTIMAGES%%', $this->getVariantImagesCount());
				$html = $t->templateParse();
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $e) {
				$loc = 'general_options.php?' . $this->_view->getRedirectParams($e->getMessage(), miMessage::MSG_TYPE_ERROR);
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
				$this->_view->addControllerCommand($cmd);
				return false;
			}
		}
		
		protected function getProductImagesCount()
		{
			$action = new scExecRegenerateProductImagesAction($this->_view);
			return $action->getImagesRecordset()->getRecordsCount();
		}
		
		protected function getVariantImagesCount()
		{
			$action = new scExecRegenerateVariantImagesAction($this->_view);
			return $action->getImagesRecordset()->getRecordsCount();
		}
	}
	
	class scExecRegenerateProductImagesAction extends miAction {
		
		const PROCESSING_SECONDS = 2;
		
		public function doAction()
		{
			try {
				$start = time();
				
				$resizeOffset = (int)miGetParamDefault('resizeOffset', 0);
				$recordset = $this->getImagesRecordset();
				$allImages = $recordset->getRecordsCount();
				
				$processedImages = $resizeOffset;
				
				$warning = '';
				try {
					while (($imageInfo = $recordset->getRecordsByIndex($resizeOffset, 1)) && !empty($imageInfo)) {
						$imageInfo = $imageInfo[0];
						$image = $this->getImageDomainObject();
						$image->setRow($imageInfo);
						$image->regenerateSizesFromOriginal();
						$processedImages += 1;
						$resizeOffset += 1;
						
						if ((time() - $start) >= self::PROCESSING_SECONDS) {
							break;
						}
					}
				} catch (Exception $exception) {
					$warning = $exception->getMessage();
					$processedImages += 1;
					$resizeOffset += 1;
				}
				
				$response = new scRegenerateAjaxResponse();
				$response->set(scRegenerateAjaxResponse::ALL_IMAGES_COUNT, $allImages);
				$response->set(scRegenerateAjaxResponse::DONE_IMAGES_COUNT, $processedImages);
				if ($allImages == 0) {
					$percentage = 100;
				} else {
					$percentage = round(($processedImages / $allImages) * 100, 2);
				}
				$response->set(scRegenerateAjaxResponse::PERCENTAGE, $percentage);
				if ($warning)
					$response->set(scRegenerateAjaxResponse::WARNING, $warning);
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $response->__toString());
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (Exception $e) {
				$response = new scRegenerateAjaxResponse();
				$response->set(scRegenerateAjaxResponse::ERROR, $e->getMessage());
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $response->__toString());
				$this->_view->addControllerCommand($cmd);
				return false;
			}
		}
		
		/**
		 * @return miSqlRecordset
		 */
		public function getImagesRecordset()
		{
			$recordset = new miSqlRecordset('ProductImages');
			$recordset->setOrder('ProductImageID');
			return $recordset;
		}
		
		/**
		 * @return scProductImageDomainObject
		 */
		protected function getImageDomainObject()
		{
			return new scProductImageDomainObject();
		}
	}
	
	class scExecRegenerateVariantImagesAction extends scExecRegenerateProductImagesAction {
		
		/**
		 * @return miSqlRecordset
		 */
		public function getImagesRecordset()
		{
			$recordset = new miSqlRecordset('ProductVariants');
			$recordset->setOrder('ProductVariantID');
			return $recordset;
		}
		
		/**
		 * @return scProductVariant
		 */
		protected function getImageDomainObject()
		{
			return new scProductVariant();
		}
	}
	
	class scRegenerateAjaxResponse {
		
		const ERROR = 'error';
		const WARNING = 'warning';
		const ALL_IMAGES_COUNT = 'all';
		const DONE_IMAGES_COUNT = 'done';
		const PERCENTAGE = 'percent';
		
		protected $_data = array(
			self::ERROR => '',
			self::WARNING => '',
			self::ALL_IMAGES_COUNT => 0,
			self::DONE_IMAGES_COUNT => 0,
			self::PERCENTAGE => 0,
		);
		
		public function __toString()
		{
			return scJson::jsonEncode($this->_data);
		}
		
		public function set($key, $value)
		{
			if (!isset($this->_data[$key])) {
				throw new miException(sprintf(_i('Unknown field "%s".'), $key));
			}
			
			$this->_data[$key] = $value;
		}
	}
	
	$page = new scAdminDisplayController(array(new scRegenerateImagesView()));
	$page->getDispatcher()->setActionHandler('', 'scRegenerateImagesAction');
	$page->getDispatcher()->setActionHandler('dmProductImages', 'scExecRegenerateProductImagesAction');
	$page->getDispatcher()->setActionHandler('dmVariantImages', 'scExecRegenerateVariantImagesAction');
	$page->dispatchAndShow();
?>