<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_RICH_TEXT_PAGES);

	class scRichTextPagesView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/rich_text_pages/rich_text_page_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/rich_text_pages/rich_text_page_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/rich_text_pages/rich_text_page_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CMSRichTextPageID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CMSRichTextPageTitle',
				'displayName' => 'Title',
				'validator' => 'scValidatorNoEmpty',
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'CMSRichTextPageContent',
				'displayName' => 'Content',
        	),
        	array(
				'field' => 'scCustomUrlsWidget',
				'data' => 'CustomUrls',
				'modelStrategy' => 'scViewMapperRichTextPageCustomUrls',
				'properties' => array(
					'template' => 'admin/custom_urls/custom_url.tmpl',
					
					'table' => 'CustomUrls',
					'foreignKey' => 'CustomUrlEntityID',
					'foreignTablePrimaryKey' => 'CustomUrlID',
					
					'fields' => array(
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CustomUrlText',
			       			'properties' => array('minLength' => 1),
			       			'validator' => 'scValidatorUniqueCustomUrl'
			        	),
			       		array(
							'field' => 'miCheckboxWidget',
							'data' => 'CustomUrlIsCurrent'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomUrlID'
			        	)
			        ),
			        'mlFields' => array('CustomUrlText')
				),
			),
        	array(
				'field' => 'scCMSLayoutSelect',
				'data' => 'CMSLayoutID',
				'displayName' => 'Layout'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'CMSRichTextPageSEOTitle'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'CMSRichTextPageSEOKeywords'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'CMSRichTextPageSEODescription'
        	),
        	array(
        		'field' => 'scCmsMenuItemsWidget',
        		'data' => 'CMSMenuItem',
        		'modelStrategy' => 'scRichTextToCmsMenuItemViewMapper'
        	)
        );
        
        protected $_domainObjectDisplayName = 'Rich text page';	// _t('Rich text page');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$form->addMainPageElements(array('%%STOREROOT%%' => miI18N::htmlEscape(scPaths::getHttpHost())));
			
			$form->addMainPageElements(array(
				'%%CKEDITOR_CONTENTCSS%%' => scSkins::singleton()->getCurrentSkin()->getCssFilesForWysiwygEditor(),
				'%%CKEDITOR_LANGUAGE%%' => scCMS::getCkEditorLanguage()
			));
		}
		
		protected function createDomainObject()
		{
			return new scRichTextPageDomainObject();
		}
	}
	
	class scRichTextPageDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new scSqlRecordML('CMSRichTextPages', 'CMSRichTextPageID',
				array('CMSRichTextPageTitle', 'CMSRichTextPageContent', 'CMSRichTextPageSEOTitle', 'CMSRichTextPageSEOKeywords', 'CMSRichTextPageSEODescription'),
				array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
	}
	
	/**
	 * Rich Text Pages Table
	 */
	class scRichTextPagesTable extends scTable {
		protected function addUiElements(&$rows)
		{
			if (count($rows) == 0)
				return;
			
			$layoutIds = array();
			foreach ($rows as $row)
				$layoutIds[] = (int)$row['CMSLayoutID'];
			 $sql = 'SELECT CMSLayoutID,CMSLayoutName FROM CMSLayouts WHERE CMSLayoutID IN(' . implode(',', $layoutIds) . ')';
			 $layoutNames = miUtil::getDBArray($sql, 'CMSLayoutID', 'CMSLayoutName');
			
			foreach ($rows as &$row) {
				$row['CMSLayoutName'] = $layoutNames[$row['CMSLayoutID']];
				$row['CMSRichTextPagePublicURL'] = scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_RICH_TEXT_PAGE, $row['CMSRichTextPageID'], $row['CMSRichTextPageTitle']), scUrlUtil::URL_TYPE_ABSOLUTE);
			}
		}
	}
	
	class scRichTextToCmsMenuItemViewMapper extends miViewMapper {
		
		protected $_data = array();
		
		public function __construct(miViewMapperController $viewMapperController)
		{
			parent::__construct($viewMapperController);
		}
		
		public function get($field)
		{
			return $this->_data;
		}
		
		public function set($field, $value)
		{
			$this->_data = $value;
		}
		
		public function read()
		{
			$recordset = new miSqlRecordset('CMSMenus');
			$pageID = miGetParamDefault('CMSRichTextPageID', 0);
			if ($pageID) {
				$recordset->addJoinCondition('LEFT', 'CMSMenuItems', 'ON CMSMenuItems.CMSMenuID = CMSMenus.CMSMenuID AND CMSMenuItems.CMSRichTextPageID = ' . intval($pageID));
				$selectFields[] = 'CMSMenus.*';
				$selectFields[] = 'CMSMenuItems.CMSMenuItemID';
				$recordset->setSelectFields($selectFields);
				
				$menus = $recordset->getAllRecords();
				foreach ($menus as &$menu)
					if ($menu['CMSMenuItemID'])
						$menu['CMSMenuItemName'] = miUtil::getDBArray('SELECT LanguageID, CMSMenuItemName FROM CMSMenuItemsML WHERE CMSMenuItemID = ' . $menu['CMSMenuItemID'], 'LanguageID', 'CMSMenuItemName');
			} else {
				$menus = $recordset->getAllRecords();
			}
			
			$this->_data = $menus;
		}
		
		public function insert()
		{
			$this->update();
		}
		
		/**
		 * Returns hash with CMSMenuID => NextSortValue
		 */
		public function getNextMenuItemSort()
		{
			return miUtil::getDBArray('SELECT CMSMenuID, MAX(CMSMenuItemSort) as CMSMaxSort FROM CMSMenuItems GROUP BY CMSMenuID', 'CMSMenuID', 'CMSMaxSort');
		}
		
		public function update()
		{
			miStaticDBUtil::startTransaction();
			
			$domainObject = $this->getViewMapperController()->getView()->getDomainObject();
			$pageId = $domainObject->get($domainObject->getPKName());
			
			$sorts = $this->getNextMenuItemSort();
			
			$activeLanguages = scMultiLanguage::singleton()->getActiveLanguages();
			
			// Insert new menu items
			foreach ($this->_data as $row) {
				if (!$row['CMSMenuItemIsActive']) {
					
					// If it has been deactivated - delete the menu item
					if ($row['CMSMenuItemID'])
						miStaticDBUtil::execDelete('CMSMenuItems', 'CMSMenuItemID', $row['CMSMenuItemID']);
					continue;
				}
				
				// If we have CMSMenuItemID then update
				if ($row['CMSMenuItemID']) {
					foreach ($activeLanguages as $activeLanguage) {
						$values = array('CMSMenuItemName' => $row['CMSMenuItemName'][$activeLanguage['LanguageID']]);
						miStaticDBUtil::execUpdate('CMSMenuItemsML', $values,
							array('CMSMenuItemID', 'LanguageID'),
							array($row['CMSMenuItemID'], $activeLanguage['LanguageID']));
					}
					continue;
				}
				
				$values = array(
					'CMSRichTextPageID' => $pageId,
					'CMSMenuID' => intval($row['CMSMenuID']),
					'CMSMenuItemSort' => ++$sorts[$row['CMSMenuID']],
					'CMSMenuItemLinkType' => scUserMenuItem::LINK_TYPE_RICH_TEXT_PAGE,
					'CMSFormID' => null
				);
				foreach ($activeLanguages as $activeLanguage)
					$values['CMSMenuItemName'][$activeLanguage['LanguageID']] = $row['CMSMenuItemName'][$activeLanguage['LanguageID']];
				
				$menuItemRecord = new scSqlRecordML('CMSMenuItems', 'CMSMenuItemID', array('CMSMenuItemName') , array_keys(scMultiLanguage::getActiveLanguageCodes()));
				$menuItemRecord->setRow($values);
				
				$menuItemRecord->insert();
			}
			
			miStaticDBUtil::commit();
		}
	}
	
	class scViewMapperRichTextPageCustomUrls extends scViewMapperCustomUrls {
		protected $_entityCode = scFriendlyUrl::TYPE_RICH_TEXT_PAGE;
		protected $_tableKeyName = 'CMSRichTextPageID';
	}
	
	/**
	 * Helper class for the rich text pages actions
	 */
	class scRichTextPagesUtil {
		public static function initFormData($view, $action)
		{
			$form = $action->getWebForm();
			
			$viewMapper = $view->getViewMapperController()->getViewMapper('scRichTextToCmsMenuItemViewMapper');
			$viewMapper->read();
			$view->callPlugin($action, 'preShowForm');
			
			$dataFields = $view->getDataFields();
			foreach ($dataFields as $dataField) {
				$viewMapperName = isset($dataField['modelStrategy']) ? $dataField['modelStrategy'] : '';
				if ($viewMapperName == 'scRichTextToCmsMenuItemViewMapper') {
					// Get the data read
					$form->setFormData('CMSMenuItem', $viewMapper->get($dataField['data']));
				}
			}
		}
	}

	// ---------------------------------------------------------------------------------------------------- //
	// Rich text page requres custom actions because of the initialization of the menus table				//
	// ---------------------------------------------------------------------------------------------------- //
	class scRichTextPageCreateAction extends miCreateAction {
		public function doAction()
		{
			scRichTextPagesUtil::initFormData($this->_view, $this);
			
			$form = $this->getWebForm();
			$this->showForm($form, $this->_view->getTemplateFileName(miDefaultView::TEMPLATE_ID_CREATE), true);
			return true;
		}
	}
	
	class scRichTextPageExecCreateAction extends miExecCreateAction {
		public function doAction()
		{
			scRichTextPagesUtil::initFormData($this->_view, $this);
			
			return parent::doAction();
		}
	}
	
	class scRichTextPageEditAction extends miEditAction {
		public function doAction()
		{
			scRichTextPagesUtil::initFormData($this->_view, $this);
			
			return parent::doAction();
		}
	}
	
	class scRichTextPageExecEditAction extends miExecEditAction {
		public function doAction()
		{
			scRichTextPagesUtil::initFormData($this->_view, $this);
			
			return parent::doAction();
		}
	}
	
	$richTextPages = new scSqlRecordsetML('CMSRichTextPages', 'CMSRichTextPageID');
	$table = new scRichTextPagesTable($richTextPages);
	
	$view = new scRichTextPagesView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmCreate', 'scRichTextPageCreateAction');
	$page->getDispatcher()->setActionHandler('dmExecCreate', 'scRichTextPageExecCreateAction');
	$page->getDispatcher()->setActionHandler('dmEdit', 'scRichTextPageEditAction');
	$page->getDispatcher()->setActionHandler('dmExecEdit', 'scRichTextPageExecEditAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>