<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	// Just check if there is administrator logged in
	scAdminAuth::singleton()->checkAuthorization();
	
	class scListSearchAction extends miAction {
		const FIELDS_COLUMNS = 5;
		
		public function doAction()
		{
			$this->_view->initMainPageElements();
			
			// Create the template parser
			$templateParser = new scTemplateParser();
			$templateParser->assignArray($this->_view->getMainPageElements());
			
			// Check the authorization status
			$auth = scAdminAuth::singleton();
			if ($auth->getAdministrator() === false) {
				// No session
				$this->_view->addRedirectToLoginControllerCommand();
				return true;
			}
			
			$this->execSearch($templateParser);
			
			$templateParser->readTemplate($this->_view->getTemplateFileName(scSearchView::TEMPLATE_ID_VIEW));
			$html = $templateParser->templateParse();
			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;
		}
		
		protected function execSearch(miTemplateParser $templateParser)
		{
			$auth = scAdminAuth::singleton();
			$productsAuthorization = $auth->isAuthorized(scSections::SECTION_PRODUCTS);
			$ordersAuthorization = $auth->isAuthorized(scSections::SECTION_ORDERS);
			if (!$productsAuthorization && !$ordersAuthorization) {
				// No permissions to view products or orders
				$templateParser->addSectionInfo(new miTemplateParserSectionInfo('noPermissions'));
			}
			else {
				$hasPermissionsSection = new miTemplateParserSectionInfo('hasPermissions');
				
				// Has permissions to view some data so initialize the search engine
				$searchEngine = scSearch::singleton();
				
				// Checks if user has permissions to view products
				if ($productsAuthorization) {
					// Find the products matching the criteria
					$products = $searchEngine->findProducts();
					
					$productsCount = count($products);
					if ($productsCount > 0) {
						// Create section that shows the product table
						$productsSection = new miTemplateParserSectionInfo('products');
						// Prepare the rows for parsing 
						$products = miUIUtil::templetizeRows(miUIUtil::transposeRows($products));
						// Create the rows section
						$productsSection->addSubsection(new miTemplateParserSectionInfo('rows', $productsCount, $products));

						$hasPermissionsSection->addSubsection($productsSection);
					}
					else		// No products matching the criteria
						$hasPermissionsSection->addSubsection(new miTemplateParserSectionInfo('noproducts'));
				}
				
				// Checks if user has permissions to view orders
				if ($ordersAuthorization) {
					// Find the orders matching the criteria
					$orders = $searchEngine->findOrders();
					
					$ordersCount = count($orders);
					if ($ordersCount > 0) {
						// Create section that shows the order table
						$ordersSection = new miTemplateParserSectionInfo('orders');
						// Prepare the rows for parsing 
						$orders = miUIUtil::templetizeRows(miUIUtil::transposeRows($orders));
						// Create the rows section
						$ordersSection->addSubsection(new miTemplateParserSectionInfo('rows', $ordersCount, $orders));

						$hasPermissionsSection->addSubsection($ordersSection);
					}
					else			// No orders matching the criteria
						$hasPermissionsSection->addSubsection(new miTemplateParserSectionInfo('noorders'));
				}
				
				$templateParser->addSectionInfo($hasPermissionsSection);
				
				$this->saveSearchOptions($templateParser);
			}
		}
		
		protected function saveSearchOptions(miTemplateParser $templateParser)
		{
			$searchEngine = scSearch::singleton();
			
			$templateParser->assign('%%PHRASE%%', $searchEngine->getSearchPhrase());
			
			// Get the section which shows if user has sufficient permissions to view products or orders
			$hasPermissionsSection = $templateParser->getSectionInfo('hasPermissions');
			
			// Add the product fields (used for customized searching)
			$productsSection = $this->getFieldsSection('productFields', 'productFieldsRow', $searchEngine->getProductFields());
			$hasPermissionsSection->addSubsection($productsSection);
			
			// Add the order fields (used for customized searching)
			$ordersSection = $this->getFieldsSection('orderFields', 'orderFieldsRow', $searchEngine->getOrderFields());
			$hasPermissionsSection->addSubsection($ordersSection);
		}
		
		protected function getFieldsSection($sectionName, $rowSectionName, $fields)
		{
			$fields = array_chunk($fields, self::FIELDS_COLUMNS);
			
			// Create the fields info rows sections
			$fieldsRowSection = array();
			foreach ($fields as $fieldsRow)			
				$fieldsRowSection[] = new miTemplateParserSectionInfo($rowSectionName, count($fieldsRow), miUIUtil::templetizeRows(miUIUtil::transposeRows($fieldsRow)));
			
			$fieldsSection = new miTemplateParserSectionInfo($sectionName, count($fieldsRowSection));
			$fieldsSection->addSubsectionsArray($rowSectionName, $fieldsRowSection);
			
			return $fieldsSection;
		}
	}
	
	class scSearchView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_VIEW => 'admin/search/search.tmpl',
		);
		
		public function addRedirectToLoginControllerCommand()
		{
			$url = 'login.php';
			$this->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
		}
	}
	
	$searchView = new scSearchView();

	$page = new scAdminDisplayController(array($searchView));
	
	// Overwrite the action hadnlers
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scListSearchAction');
	$page->getDispatcher()->setActionHandler('dmSearch', 'scListSearchAction');
	
	$page->dispatchAndShow();
?>