<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_MODULES_SHIPPING);

	class scShippingModulesView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_CREATE => 'admin/shipping_modules/shipping_modules_create.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/shipping_modules/shipping_modules_edit.tmpl',
			self::TEMPLATE_ID_LIST => 'admin/shipping_modules/shipping_modules_mgmt.tmpl',
		);
		
        protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ShippingModuleID'
        	),
       		array(
				'field' => 'miSelectWidget',
				'data' => 'ShippingZoneID',
				'displayName' => 'Shipping zone'
        	),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'ShippingModuleName',
				'properties' => array('readOnly' => true),
				'displayName' => 'Name'
        	),
			array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'ShippingModuleDisplayName',
				'displayName' => 'Display name'
        	),
       		array(
				'field' => 'miCheckboxWidget',
				'data' => 'ShippingModuleIsActive',
				'properties' => array('id' => 'isModuleActive')
        	),
       		array(
				'field' => 'scShippingMethodsWidget',
				'data' => 'ShippingModuleActiveMethods',
       			'modelStrategy' => 'scViewMapperShippingMethods'
        	),
       		array(
				'field' => 'scModuleOptionsWidget',
				'data' => 'ModuleOptionFields',
				'modelStrategy' => 'scViewMapperShippingModuleOptions',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ShippingModuleImage',
       			'properties' => array('readOnly' => 1),
        	),
		);
		
		protected $_domainObjectDisplayName = 'Shipping module';	// _t('Shipping module');
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$sql = 'SELECT ShippingZoneID,ShippingZoneName FROM ShippingZones ORDER BY ShippingZoneName';
			$shippingZones = miUtil::getDBArray($sql, 'ShippingZoneID', 'ShippingZoneName');
			$form->getWidget('ShippingZoneID')->setOptions(array(scShippingZone::ALL_ZONES => 'All') + $shippingZones);
			
			$shippingModule = $this->getModule();
			$shippingMethods = $shippingModule->getShippingMethods();
			foreach ($shippingMethods as $shippingMethodId => $shippingMethod)
				$shippingMethods[$shippingMethodId] = scInterfaceTexts::getTranslatedText($shippingMethod);
			
			$form->getWidget('ShippingModuleActiveMethods')->setShippingMethods($shippingMethods);
			$form->getWidget('ModuleOptionFields')->setupModule($this->getDomainObject()->get('ModuleID'), $shippingModule);
		}
		
		protected function createDomainObject()
		{
			return new scShippingModulesDomainObject($this->getRecord());
		}
		
		protected function getModule()
		{
			$domainObject = $this->getDomainObject();
			$domainObject->read((int)miGetParam('ShippingModuleID'));
			$className = scModulesFactory::getModuleClassName($domainObject->get('ModuleID'), 'IShippingModule');
			return new $className($domainObject->get('ModuleID'), $domainObject->get('ShippingModuleID'));
		}
		
		public function getStaticShippingModules()
		{
			return array(
				'com.summercart.shipping.digitaldelivery'
			);
		}
		
		public function initViewMapperController($viewMapperController)
		{
			$viewMapper = $viewMapperController->getViewMapper('scViewMapperShippingModuleOptions');
			$viewMapper->setModule($this->getModule());
			$viewMapper->setShippingModuleId((int)miGetParam('ShippingModuleID'));
		}
		
        public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function preProcess($actionObj)
		{
			parent::preProcess($actionObj);
			
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('ShippingModuleIsActive');
		}
	}
	
	class scShippingMethodsWidget extends miBaseWidget {
		
		/**
		 * Array with the supported shipping methods
		 */
		protected $_shippingMethods = array();
		
		/**
		 * Set the supported shipping methods
		 * 
		 * @param array $shippingMethods
		 */
		public function setShippingMethods($shippingMethods)
		{
			$this->_shippingMethods = $shippingMethods;
		}
		
		function getEditableControl()
		{
			$shippingMethodRows = $this->_webForm->getFormData($this->_fieldName);
			
			$displayNameWidgets = $this->getDisplayNameWidgets();
			
			$saveFieldName = $this->_fieldName;
			$widget = '';
			foreach ($this->_shippingMethods as $key => $shippingMethod) {
				// See if it is active
				$isActive = false;
				if (isset($shippingMethodRows[$key])) {
					$isActive = $shippingMethodRows[$key]['ShippingMethodIsActive'] ? true : false;
					$this->_webForm->setFormData($displayNameWidgets[$key]->getFieldName(), $shippingMethodRows[$key]['ShippingMethodDisplayName']);
				}
				
				// Get the shipping method display name control
				$shippingMethodDisplayNameWidget = $displayNameWidgets[$key];
				$shippingMethodDisplayNameData = $shippingMethodDisplayNameWidget->getEditableControl();
				$shippingMethodDisplayName = $shippingMethodDisplayNameData[$shippingMethodDisplayNameWidget->getFieldName()];
				
				$checked = $isActive ? ' checked="checked"' : '';
				$name = $this->_fieldName .  '-' . $key;
				$checkbox = '<input type="checkbox" name="' . $name . '" id="' . $name . '"' . $checked . ' value="1" />';
				$widget .= '<tr><td>' . $checkbox . '</td><td><label for="' . $name . '">' .
					miI18N::htmlEscape($shippingMethod) . '</label></td><td nowrap="nowrap">' . $shippingMethodDisplayName . '</td></tr>';
			}
			$this->setFieldName($saveFieldName);
			return array($this->_fieldName => $widget);
		}
		
		function getData()
		{
			$displayNameWidgets = $this->getDisplayNameWidgets();
			foreach ($this->_shippingMethods as $key => $shippingMethod) {
				$displayNameData = $displayNameWidgets[$key]->getData();
				$displayName = $displayNameData[$displayNameWidgets[$key]->getFieldName()];
				
				$shippingMethods[$key] = array(
					'ShippingMethodCode' => $key,
					'ShippingMethodIsActive' => miGetParamDefault('ShippingModuleActiveMethods-' . $key, '') != '' ? 1 : 0,
					'ShippingMethodDisplayName' => $displayName
				);
			}
			
			return array($this->_fieldName => $shippingMethods);
		}
		
		protected function getDisplayNameWidgets()
		{
			$widgets = array();
			foreach ($this->_shippingMethods as $key => $shippingMethod) {
				// Get the shipping method display name control
				$shippingMethodDisplayNameWidget = new scWebFormWidgetTextML($this->_webForm, array());
				$shippingMethodDisplayNameWidget->setFieldName('ShippingMethodDisplayName-' . $key);
				$widgets[$key] = $shippingMethodDisplayNameWidget;
			}
			return $widgets;
		}
	}
	
	/**
	 * Handle the reading and updating of shipping method display names and active flag 
	 */
	class scViewMapperShippingMethods extends miViewMapper {
		protected $_shippingMethodRows = array();
		
		public function getShippingModuleId()
		{
			return $this->getViewMapperController()->getDomainObject()->get('ShippingModuleID');
		}
		
		public function get($fieldName)
		{
			return $this->_shippingMethodRows;
		}
		
		public function set($fieldName, $fieldValue)
		{
			return $this->_shippingMethodRows = $fieldValue;
		}
		
		public function read()
		{
			$sql = 'SELECT ShippingMethodID FROM ShippingMethods WHERE ShippingModuleID = ?';
			$shippingMethods = miStaticDBUtil::execSelect($sql, array($this->getShippingModuleId()));
			
			foreach ($shippingMethods as $shippingMethod) {
				$shippingMethodRecord = new scSqlRecordML('ShippingMethods', 'ShippingMethodID', array('ShippingMethodDisplayName'), array_keys(scMultiLanguage::getActiveLanguageCodes()));
				$shippingMethodRecord->readPK($shippingMethod['ShippingMethodID']);
				$this->_shippingMethodRows[$shippingMethodRecord->get('ShippingMethodCode')] = $shippingMethodRecord->getRow();
			}
		}
		
		public function insert()
		{
			$this->update();
		}
		
		public function update()
		{
			$shippingModuleId = (int)$this->getShippingModuleId();
			
			$shippingMethodCodes = array();
			foreach ($this->_shippingMethodRows as $shippingMethod) {
				$shippingMethodCodes[] = sql_escape_string($shippingMethod['ShippingMethodCode']);
			}
			$sql = 'SELECT ShippingMethodID,ShippingMethodCode FROM ShippingMethods' .
				' WHERE ShippingMethodCode IN("' . implode('","', $shippingMethodCodes) . '")' .
				' AND ShippingModuleID = ' . sql_escape_string($shippingModuleId);
			$shippingMethodIds = miUtil::getDBArray($sql, 'ShippingMethodCode', 'ShippingMethodID');
			
			foreach ($this->_shippingMethodRows as $shippingMethod) {
				$shippingMethodRecord = new scSqlRecordML('ShippingMethods', 'ShippingMethodID', array('ShippingMethodDisplayName'), array_keys(scMultiLanguage::getActiveLanguageCodes()));
				$shippingMethodRecord->setRow($shippingMethod);
				$shippingMethodRecord->set('ShippingModuleID', $shippingModuleId);
				
				if (isset($shippingMethodIds[$shippingMethod['ShippingMethodCode']])) {
					$shippingMethodRecord->set('ShippingMethodID', $shippingMethodIds[$shippingMethod['ShippingMethodCode']]);
					$shippingMethodRecord->update();
				} else
					$shippingMethodRecord->insert();
			}
		}
	}
	
	class scShippingModulesTable extends scTable {
		public function addUiElements(&$rows)
		{
			foreach ($rows as &$row) {
				if ($row['ShippingZoneID'] == 0)
					$row['HTML_ShippingZoneName'] = _t('All');
				else
					$row['HTML_ShippingZoneName'] = '<a href="shipping_zones.php?action=dmEdit&amp;ShippingZoneID=' . $row['ShippingZoneID'] . '">' . $row['ShippingZoneName'] . '</a>';
			}
		}
	}
	
	class scNewShippingModuleAction extends miActionWithWebForm {
		public function getWebForm()
		{
			static $form = null;
			if ($form == null)
				$form = new miWebForm(array()); 
			return $form;
		}
		
		public function doAction()
		{
			try {
				$this->_view->callPlugin($this, 'preShowForm');
				
				$moduleClassesRs = new miSqlRecordset('ModuleClasses');
				$moduleClassesRs->addJoinCondition('INNER', 'Modules', 'USING(ModuleID)');
				$moduleClassesRs->addFilter(new miSqlFilterEqual('ModuleClassInterface', 'IShippingModule'));
				$moduleClassesRs->addFilter(new miSqlFilterEqual('ModuleIsActive', '1'));
				$moduleClassesRs->addFilter(new miSqlFilterNotIn('ModuleName', $this->_view->getStaticShippingModules()));
				$moduleClasses = $moduleClassesRs->getAllRecords();
				
				$modules = array();
				$warnings = array();
				foreach ($moduleClasses as $key => &$moduleClass) {
					try {
						$className = scModulesFactory::getModuleClassName($moduleClass['ModuleID'], 'IShippingModule');
						$shippingModule =  new $className($moduleClass['ModuleID'], 0);
						$moduleClass['ShippingModuleImage'] = $shippingModule->getShippingModuleImage();
						$modules[] = &$moduleClass;
					} catch (miException $e) {
						$warnings[] = $moduleClass['ModuleTitle'] . ': ' . $e->getMessage();
					}
				}
				
				$modulesSection = new miTemplateParserSectionInfo('modules', count($modules), miUIUtil::templetizeRows(miUIUtil::transposeRows($modules)));
				
				$breadcrumb = new scBreadcrumb();
				$this->_view->addMainPageElements(array(
					'%%HTML_BREADCRUMB%%' => $breadcrumb->getCurrentBreadcrumbHtml(),
					'%%HTML_MODULES_WARNINGS%%' => implode('<br />', $warnings),
					'%%MODULES_WARNINGS_STYLE%%' => (empty($warnings) ? 'hidden' : 'warning'),
				));
			} catch (miException $e) {
				return $this->onFailure($e);
			}
			
			$form = $this->getWebForm();
			$form->addTemplateSectionInfo($modulesSection);
			$this->showForm($form, $this->_view->getTemplateFileName(miDefaultView::TEMPLATE_ID_CREATE), true);
			return true;
		}
	}
	
	class scExecNewShippingModuleAction extends miExecCreateAction {
		public function doAction()
		{
			$moduleId = (int)miGetParam('ModuleID');
			$moduleName = scModuleUtil::getModuleName($moduleId);
			
			$nonCreatable = $this->_view->getStaticShippingModules();
			if (in_array($moduleName, $nonCreatable)) {
				$this->_view->addRedirectToListControllerCommand(
						_t('This shipping module must have one instance and cannot be added.'),
						miMessage::MSG_TYPE_ERROR
					);
				return false;
			}
			
			$shippingModule = $this->_view->getDomainObject();
			$shippingModule->set('ModuleID', $moduleId);
			$shippingModule->set('ShippingModuleName', $moduleName);
			$shippingModule->set('ShippingModuleIsActive', 0);
			$shippingModuleId = $shippingModule->insert();
			
			$loc = '?action=dmEdit&ShippingModuleID=' . $shippingModuleId . '&' .
				$this->_view->getRedirectParams(_t('Shipping module successfully created. Please setup the module.'), miMessage::MSG_TYPE_INFO);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($cmd);
		}
	}
	
	class scExecDeleteShippingModuleAction extends miExecDeleteAction {
		
		public function doAction()
		{
			$nonDeletable = $this->_view->getStaticShippingModules();
			$domainObject = $this->_view->getDomainObject();
			$domainObject->read(miGetParam($domainObject->getPKName()));
			if (in_array($domainObject->get('ShippingModuleName'), $nonDeletable)) {
				$this->_view->addRedirectToListControllerCommand(
						_t('This shipping module must have one instance and cannot be deleted.'),
						miMessage::MSG_TYPE_ERROR
					);
				return false;
			}
			parent::doAction();
		}
	}
	
	class scChangeIsActiveShippingModuleAction extends scExecChangeIsActiveAction {
		
		public function doAction()
		{
			$result = parent::doAction();
			if ($result === true) {
				$domainObject = $this->_view->getDomainObject();
				$newValue = $domainObject->get($this->_fieldName);
				if ((bool)$newValue) {
					miStaticDBUtil::execUpdate('ShippingMethods', array('ShippingMethodIsActive' => 1),
						'ShippingModuleID', $domainObject->get('ShippingModuleID'));
				}
			}
		}
	}
	
	$shippingModules = new scSqlRecordsetML('ShippingModules', 'ShippingModuleID');
	$shippingModules->addJoinCondition('LEFT', 'ShippingZones', 'USING(ShippingZoneID)');
	
	$table = new scShippingModulesTable($shippingModules);
	
	$view = new scShippingModulesView();
	$view->setRecord(new scSqlRecordML('ShippingModules', 'ShippingModuleID',
						array('ShippingModuleDisplayName'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
	$view->setUpSortColumn('ShippingModuleSort');
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmExecDelete', 'scExecDeleteShippingModuleAction');
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scChangeIsActiveShippingModuleAction');
	$page->getDispatcher()->setActionHandler('dmNewShippingModule', 'scNewShippingModuleAction');
	$page->getDispatcher()->setActionHandler('dmExecNewShippingModule', 'scExecNewShippingModuleAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>