<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_SHIPPING_ZONES);
	
	class scShippingZonesView extends scView {
		
        protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/shipping_zones/shipping_zone_mgmt.tmpl',
        	self::TEMPLATE_ID_EDIT => 'admin/shipping_zones/shipping_zone_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/shipping_zones/shipping_zone_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ShippingZoneID'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'ShippingZoneName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name'
        	),
        	array(
				'field' => 'scDoubleListBoxWidget',
				'data' => 'Countries',
				// _t('Available Countries'), _t('Selected Countries')
				'properties' => array('leftCaption' => 'Available Countries', 'rightCaption' => 'Selected Countries'),
				'modelStrategy' => 'scViewMapperShippingZoneCountries',
        	),
        	array(
				'field' => 'scDoubleListBoxWidget',
				'data' => 'States',
				// _t('Available States'), _t('Selected States')
        		'properties' => array('leftCaption' => 'Available States', 'rightCaption' => 'Selected States'),
				'modelStrategy' => 'scViewMapperShippingZoneStates',
        	),
        );
		
        protected $_domainObjectDisplayName = 'Shipping zone';	// _t('Shipping zone');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Set Countries
			$fieldCountries = $form->getWidget('Countries');
			$fieldCountries->setOptions(scCountry::getCountryNames());
			
			// Set States
			$fieldStates = $form->getWidget('States');
			$languageId = (int)scView::getCurrentDataLanguageId();
			$sql = 'SELECT States.StateID,CONCAT(CountryName," - ",StateName) as StateName FROM States' .
				' LEFT JOIN StatesML ON States.StateID=StatesML.StateID AND StatesML.LanguageID=' . $languageId .
				' LEFT JOIN CountriesML ON States.CountryID=CountriesML.CountryID AND CountriesML.LanguageID=' . $languageId;
			$states = miUtil::getDBArray($sql, 'StateID', 'StateName');
			$fieldStates->setOptions($states);
		}
		
		protected function createDomainObject()
		{
			return new scShippingZonesDomainObject(new miSqlRecord('ShippingZones', 'ShippingZoneID'));
		}
	}
	
	class scViewMapperShippingZoneCountries extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'ShippingZoneCountries';
			$this->_tableKeyName = 'ShippingZoneID';
			$this->_tableValueName = 'CountryID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperShippingZoneStates extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'ShippingZoneStates';
			$this->_tableKeyName = 'ShippingZoneID';
			$this->_tableValueName = 'StateID';
			parent::__construct($model);
		}
	}
	
	$shippingZones = new miSqlRecordset('ShippingZones');
	$table = new scTable($shippingZones);
	
	$view = new scShippingZonesView();
	$view->setRecord(new miSqlRecord('ShippingZones', 'ShippingZoneID'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>