<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_SKINS);
	
	class scSkinSettingsView extends scView {
		
		protected $_persistVars = array(
			'SkinID',
		);
		
		protected $_templates = array(
			self::TEMPLATE_ID_EDIT => 'admin/skin_settings/skin_edit.tmpl',
		);
		
		protected $_dataFields = array(
			array(
				'field' => 'scSkinSettingsWidget',
				'data' => 'SkinSettings',
				'modelStrategy' => 'scSkinSettingsViewMapper',
			),
		);
		
		/**
		 * @param miWebForm $form
		 */
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$widget = $form->getWidget('SkinSettings');
			$widget->setWidgets(scSkinSettingsUtil::getWidgets($this->getDomainObject(), $form));
		}
		
		public function initMainPageElements()
		{
			parent::initMainPageElements();
			
			$data = $this->getDomainObject()->getRow();
			$data = miUIUtil::templetizeAndEscapeRows($data);
			$this->addMainPageElements($data);
		}
		
		public function preProcess($actionObj)
		{
			parent::preProcess($actionObj);
			
			if (($actionObj instanceof miEditAction) && !($actionObj instanceof miExecEditAction)) {
				try {
					$this->upgradeSettings();
				} catch (scSkinConfigurationException $e) {
					$msg = _i('Skin configuration error: ') . $e->getMessage();
					$loc = 'skins.php?' . miMessage::PARAM_MESSAGE .'='. urlencode($msg) .
						'&'. miMessage::PARAM_MESSAGE_TYPE .'='. urlencode(miMessage::MSG_TYPE_ERROR);
					scUtil::redirect($loc);
				}
			}
		}
		
		protected function upgradeSettings()
		{
			$domainObject = $this->getDomainObject();
			$skinId = (int)miGetParam($domainObject->getPKName());
			
			// Read the current skin
			$domainObject->read($skinId);
			
			// Get current settings
			$currentSettings = $domainObject->getSettings();
			
			// Get upgrade settings
			$skinXml = scSkinSettingsUtil::getSkinXml($domainObject);
			if ($skinXml === false) {
				return;
			}
			
			$upgradeSettings = scSkinSettingsUtil::getSkinSettingsFromXml($skinXml);
			
			scSkinSettingsUtil::upgradeSkinSettings($skinId, $currentSettings, $upgradeSettings);
			
			// Reread settings after they have been updated
			$domainObject->getSettings(scSkinDomainObject::FORCE_READ_SETTINGS);
		}
		
		protected function createDomainObject()
		{
			return new scSkinDomainObject();
		}
	}
	
	class scSkinSettingsViewMapper extends miViewMapper {
		
		protected $_data = array();
		
		public function read()
		{
			$skin = $this->_viewMapperController->getDomainObject();
			
			$this->_data = $skin->getSettings();
		}
		
		public function get($field)
		{
			$skin = $this->_viewMapperController->getDomainObject();
			foreach ($this->_data as &$value) {
				switch ($value['SkinSettingType']) {
					case scSkinSettingsUtil::SETTING_TYPE_IMAGE:
						$settingValue = explode(':', $value['SkinSettingValue']);
						$resouceType = array_shift($settingValue);
						$settingValue = implode(':', $settingValue);
						
						if ($resouceType == scPaths::RESOURCE_SKIN_DEFAULT_IMAGE) {
							$settingValue = 'skins/' . $skin->get('SkinLocation') . '/' . $settingValue;
						}
						
						$image = array(
							'imageFileName' => '',
							'thumbFileName' => '',
							'imageWebPath' => $settingValue,
							'thumbWebPath' => $settingValue,
						);
						
						$value = $image;
						break;
					default:
						$value = $value['SkinSettingValue'];
						break;
				}
			}
			
			return $this->_data;
		}
		
		public function set($field, $value)
		{
			$this->_data = $value;
		}
		
		public function update()
		{
			$this->prepareDbData();
			
			$skinId = $this->_viewMapperController->getDomainObject()->get('SkinID');
			miStaticDBUtil::startTransaction();
			scSkinSettingsUtil::deleteSkinSettings($skinId, array_keys($this->_data));
			scSkinSettingsUtil::createSkinSettings($skinId, $this->_data);
			miStaticDBUtil::commit();
		}
		
		protected function prepareDbData()
		{
			$skin = $this->_viewMapperController->getDomainObject();
			$originalSettings = $skin->getSettings();
			foreach ($this->_data as $settingName => $setting) {
				$settingType = $originalSettings[$settingName]['SkinSettingType'];
				
				switch ($settingType) {
					case scSkinSettingsUtil::SETTING_TYPE_IMAGE:
						$this->_data[$settingName] = $originalSettings[$settingName];
						
						// If the image is marked for deletion
						if (isset($setting['delete']) && (bool)$setting['delete']) {
							$newPath = scSkinSettingsUtil::getSkinSettingDefaultValue($skin, $settingName);
							
							$this->_data[$settingName]['SkinSettingValue'] = $newPath;
							$this->deleteSettingImage($setting['imageWebPath']);
							break;
						}
						
						// If the image has not changed
						if (empty($setting['imageFileName'])) {
							break;
						}
						
						// If the picture has been changed
						$newValue = $this->moveSettingImage($skin, $settingName, $setting);
						$dbValue = scPaths::RESOURCE_SKIN_UPLOADED_IMAGE . ':' . $newValue;
						$this->_data[$settingName]['SkinSettingValue'] = $dbValue;
						break;
					
					case scSkinSettingsUtil::SETTING_TYPE_CHECKBOX:
						$this->_data[$settingName] = $originalSettings[$settingName];
						$this->_data[$settingName]['SkinSettingValue'] = (int)(bool)$setting;
						break;
					
					default:
						$this->_data[$settingName] = $originalSettings[$settingName];
						$this->_data[$settingName]['SkinSettingValue'] = $setting;
						break;
				}
			}
		}
		
		/**
		 * @param scSkinDomainObject $skin
		 * @param string $settingName
		 * @param array $image
		 * @return unknown
		 */
		protected function moveSettingImage(scSkinDomainObject $skin, $settingName, $image)
		{
			$maxWidth = scSkinSettingsUtil::getSkinSettingAttribute($skin, $settingName, 'maxWidth');
			$maxHeight = scSkinSettingsUtil::getSkinSettingAttribute($skin, $settingName, 'maxHeight');
			
			// 	Move the image
			$imageFilePath = scPaths::getFilenamePath($skin->get('SkinLocation') . '/' . $image['imageFileName'], scPaths::RESOURCE_SKIN_UPLOADED_IMAGE);
			$tempImageFilePath = scPaths::getFilenamePath($image['imageFileName'], scPaths::RESOURCE_TEMP_IMAGE);
			
			$dirName = dirname($imageFilePath);
			if (!file_exists($dirName) && !mkdir($dirName, 0777, true)) {
				throw new miException(sprintf(_t('Cannot create directory "%s".'), $dirName));
			}
			
			scImageHandler::resizeImage($tempImageFilePath, $imageFilePath, $maxWidth, $maxHeight);
			
			return scPaths::getFilenamePath($skin->get('SkinLocation') . '/' . $image['imageFileName'], scPaths::RESOURCE_SKIN_UPLOADED_IMAGE, scPaths::PATH_WEB);
		}
		
		protected function deleteSettingImage($imageWebPath)
		{
			if (empty($imageWebPath)) {
				return;
			}
			
			$imageFilePath = scPaths::getStoreRootDir() . $imageWebPath;
			unlink($imageFilePath);
		}
	}
	
	$view = new scSkinSettingsView();
	
	$page = new scAdminDisplayController(array($view));
	
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'miEditAction');
	$page->getDispatcher()->setActionHandler('dmExecEdit', 'miExecEditAction');
	
	$page->dispatchAndShow();
?>