<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_SKINS);
	
	class scSkinsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_EDIT => 'admin/skins/skins_edit.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'scTableRadioWidget',
				'data' => 'SkinAdmin',
       			'properties' => array(
       				'template' => 'admin/skins/skins_radio_widget.tmpl',
       			),
        	),
       		array(
				'field' => 'scTableRadioWidget',
				'data' => 'SkinCustomer',
       			'properties' => array(
       				'template' => 'admin/skins/skins_radio_widget.tmpl',
       			),
        	),
        );
        
        protected $_domainObjectDisplayName = 'Skin selection';	// _t('Skin selection');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Set Admin Skins
			$field = $form->getWidget('SkinAdmin');
			$skinsAdmin = new miSqlRecordset('Skins');
			$skinsAdmin->addFilter(new miSqlFilterEqual('SkinHasAdmin', 1));
			$field->setTable(new scSkinsTable($skinsAdmin));
			
			// Set Customer Skins
			$field = $form->getWidget('SkinCustomer');
			$skinsCustomer = new miSqlRecordset('Skins');
			$skinsCustomer->addFilter(new miSqlFilterEqual('SkinHasUser', 1));
			$field->setTable(new scSkinsTable($skinsCustomer));
		}		
	}
	
	class scSelectSkinAction extends miEditAction
	{
		protected $_skippedSkins = array();
		
		public function getWebForm()
		{
			parent::getWebForm();
			
			if (count($this->_skippedSkins)) {
				$section = new miTemplateParserSectionInfo('manualSkinInstall');
				$subsection = new miTemplateParserSectionInfo();
				$subsection->setSectionInfo('skins', count($this->_skippedSkins), miUIUtil::templetizeRows(miUIUtil::transposeRows($this->_skippedSkins)));
				$section->addSubSection($subsection);
				$this->_webForm->addTemplateSectionInfo($section);
			}			
			
			return $this->_webForm;
		}
		
		/**
		 * @param string $dir SkinLocation
		 * @param SimpleXMLElement $xml Parsed skin.xml
		 * @return array An array(SkinLocation => SkinLocation) of all installed skins
		 */
		protected function installSkin($dir, $xml)
		{
			if ((string)$xml->SkinLocation != $dir)
				throw new miException(sprintf(_i('SkinLocation "%s" does not match the actual location "%s".'), (string)$xml->SkinLocation, $dir));
			
			miStaticDBUtil::startTransaction();
			$installedSkins = array();
			try {
				$record = new scSkinDomainObject();
				$record->set('SkinLocation', $dir);
				$record->set('SkinName', $xml->Name);
				$record->set('SkinScreenshot', $xml->Screenshot);
				$record->set('SkinHasAdmin', ($xml->Areas->Admin == 'true') ? 1 : 0);
				$record->set('SkinHasUser', ($xml->Areas->Customer == 'true') ? 1 : 0);
				if ($xml->SkinParentLocation) {
					$parentSkinId = scUtil::getDbValue('Skins', 'SkinID', 'SkinLocation', (string)$xml->SkinParentLocation, 0);
					if (!$parentSkinId) {
						$parentSkinLocation = (string)$xml->SkinParentLocation;
						$parentSkinPath = scSkins::getSkinsRoot() . $parentSkinLocation;
						if (!file_exists($parentSkinPath) || !is_dir($parentSkinPath)) {
							throw new miException(sprintf(_t('Skin location "%s" is not a valid directory.'), $parentSkinPath));
						}
						$parentSkinXml = $parentSkinPath . '/skin.xml';
						if (!file_exists($parentSkinXml)) {
							throw new miException(sprintf(_t('Skin description "%s" not found.'), $parentSkinXml));
						}
						
						$parentSkinData = file_get_contents($parentSkinXml);
						if ($parentSkinData == false) {
							throw new miException(sprintf(_t('Unable to read skin description "%s".'), $parentSkinXml));
						}
						
						$parentSkinXml = new SimpleXMLElement($parentSkinData);
						if ((string)$xml->SCVersion != scVersion::getVersion()) {
							throw new miException(sprintf(_t('The skin "%s" is for another version of the software.'), (string)$xml->Name));
						}
						
						$parentSkins = $this->installSkin($parentSkinLocation, $parentSkinXml);
						$installedSkins = array_merge($installedSkins, $parentSkins);
						
						$parentSkinId = scUtil::getDbValue('Skins', 'SkinID', 'SkinLocation', (string)$xml->SkinParentLocation, 0);
						if (!$parentSkinId) {
							throw new miException(_t(sprintf('The skin "%s" depends on skin with location "%s", which is not installed.', (string)$xml->Name, (string)$xml->SkinParentLocation)));
						}
					}
					$record->set('SkinParentID', $parentSkinId);
				}
				$skinId = $record->insert();
				$installedSkins[$dir] = $dir;
				
				$skinSettings = scSkinSettingsUtil::getSkinSettingsFromXml($xml);
				
				scSkinSettingsUtil::createSkinSettings($skinId, $skinSettings);
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e;
			}
			miStaticDBUtil::commit();
			return $installedSkins;
		}
		
		/**
		 * Checks and installs new skins
		 * 
		 * @return int number of installed skins
		 */
		protected function checkForNewSkins()
		{
			$installedSkins = 0;
			
			// get Summer Cart Version. Autoinstall only skins for current version
			$version = scVersion::getVersion();
			
			// get existing skins
			$existingSkins = miUtil::getDBArray('SELECT * FROM Skins', 'SkinLocation', 'SkinLocation');
			
			// check for new skins
			$path = scSkins::getSkinsRoot();
			$dh = opendir($path);
			while (($file = readdir($dh)) !== false) {
				if (($file == '.') || ($file == '..'))
					continue;
				
				if (is_dir($path . $file) == false)
					continue;
				
				if (isset($existingSkins[$file]))	
					continue;
				
				if (file_exists($path . $file . '/skin.xml') == false)
					continue;
				
				$skinData = file_get_contents($path . $file . '/skin.xml');
				if ($skinData == false)
					continue;
				
				$xml = new SimpleXMLElement($skinData);
				if ((string)$xml->SCVersion != $version) {
					$this->_skippedSkins[] = array(
						'location' => $file,
						'name' => $xml->Name,
						'version' => $xml->SCVersion, 
					);
					continue;
				}
				$installedSkinLocations = $this->installSkin($file, $xml);
				$existingSkins = array_merge($existingSkins, $installedSkinLocations);
				$installedSkins += count($installedSkinLocations);
			}
			
			return $installedSkins;
		}
		
		public function doAction()
		{
			try {
				$installedSkins = $this->checkForNewSkins();
				if ($installedSkins > 0)
					miSetParam(miMessage::PARAM_MESSAGE, sprintf(_t('Automatically installed %d skin(s).'), $installedSkins) . miGetParamDefault('msg', ''));
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_EDIT_FAILED_MSG') . $exception->getMessage();
				miSetParam(miMessage::PARAM_MESSAGE, $msg);
				miSetParam(miMessage::PARAM_MESSAGE_TYPE, miMessage::MSG_TYPE_ERROR);
			}
			return parent::doAction();
		}
	}
	
	class scInstallSkinAction extends scSelectSkinAction
	{
		public function doAction()
		{
			try {
				$path = scSkins::getSkinsRoot();
				$file = basename(miGetParam('skin'));
				$skinData = file_get_contents($path . $file . '/skin.xml');
				if ($skinData == false) {
					throw new miException(_t('Cannot read "skin.xml".'));
				}
				$xml = new SimpleXMLElement($skinData);
				$this->installSkin($file, $xml);
				$this->_view->addRedirectToListControllerCommand(_t('Skin installed successfully.'));
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_EDIT_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}

	class scSkinsTable extends miTable implements IOptionsWidgetTable {
		
		protected $_checkedValues;
		protected $_checkboxValueField = 'SkinID';
		
		const CHECKED_TEMPLATE_VAR = 'HTML_CHECKED';
		const CLASS_VAR = 'CLASS_ACTIVE';
		const BUTTON_DUSABLED_VAR = 'HTML_BUTTON_DISABLED';
		const BUTTON_CAPTION_VAR = 'BUTTON_CAPTION';
		const SCREENSHOT_VAR = 'SCREENSHOT_FILE';
		
		public function setCheckedValues($checkedValues)
		{
			$this->_checkedValues = $checkedValues;
		}
		
		protected function addUiElements(&$rows)
		{
			parent::addUiElements($rows);
			
			foreach ($rows as $key => $row) {
				$checkboxValue = $row[$this->_checkboxValueField];
				
				$screenshotFile = 'skins/' . $rows[$key]['SkinLocation'] . '/' . $rows[$key]['SkinScreenshot'];
				if (!file_exists(scPaths::getStoreRootDir() . $screenshotFile)) {
					$screenshotFile = 'skins/skeleton/skin-screenshot.png';
				}
				$rows[$key][self::SCREENSHOT_VAR] = scPaths::getUrl($screenshotFile);
				
				$rows[$key][self::CHECKED_TEMPLATE_VAR] = '';
				$rows[$key][self::CLASS_VAR] = '';
				$rows[$key][self::BUTTON_DUSABLED_VAR] = '';
				$rows[$key][self::BUTTON_CAPTION_VAR] = _t('Activate');
				
				if (in_array($checkboxValue, $this->_checkedValues)) {
					$rows[$key][self::CHECKED_TEMPLATE_VAR] = ' checked="checked" ';
					$rows[$key][self::CLASS_VAR] = ' active-skin ';
					$rows[$key][self::BUTTON_DUSABLED_VAR] = ' disabled="disabled" ';
					$rows[$key][self::BUTTON_CAPTION_VAR] = _t('Current');
				}
			}
		}
	}
	
	miSetParam('', 0);
	
	$skins = new miSqlRecordset('Settings');
	$table = new miTable($skins);
	
	$view = new scSkinsView();
	$view->setRecord(new miPropertiesRecord('Settings', 'SettingName', 'SettingValue'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scSelectSkinAction');
	$page->getDispatcher()->setActionHandler('dmBack', 'scSelectSkinAction');
	$page->getDispatcher()->setActionHandler('dmExecEdit', 'miExecEditAction');
	$page->getDispatcher()->setActionHandler('dmInstall', 'scInstallSkinAction');
	
	$page->dispatchAndShow();
?>