<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_STATES);

	class scStatesView extends scView {
		
		/**
         * @var array with Countries
         * @access protected
         */
        protected $_countries;  
        
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/states/state_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/states/state_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/states/state_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'StateID'
        	),
       		array(
				'field' => 'miSelectWidget',
				'data' => 'CountryID',
				'displayName' => 'Country'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'StateCode',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'State Code'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'StateName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'State'
        	)
        );
        
		protected $_domainObjectDisplayName = 'State';	// _t('State');
        
		/**
         * Constructor
         */
        function __construct()
        {
        	parent::__construct();
        	
            // Select countries
			$this->_countries = scCountry::getCountryNames();
		}  
        	
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());

			// Create dropdown filter countries
			$countries = array('' => _t('All countries')) + $this->_countries;
			scUIUtil::createDropdownFilter($this, $table, 'CountryID', $countries, 'equalNameTable', '');
			$this->getTableFilterObj()->setConditionHandler('equalNameTable', 'scStateSqlFilterSimple');
		}
		
		public function createDomainObject()
		{
			return new scStateDomainObject($this->_record);
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Set CountryID options
			$field = $form->getWidget('CountryID');
			$field->setOptions($this->_countries, scSettings::getSettingValue(scSettings::SETTING_DEFAULT_COUNTRY));
			
			// Add a special validator - the unique key
			$form->addValidator('StateCode', new scValidatorUniqueFields($form, array('StateCode', 'CountryID'), $this->getRecord()));
		}
	}

	/**
	 * Filter class for operator filters
	 */
	class scStateSqlFilterSimple extends miSqlFilterSimple {
		public function getSql()
		{
			if ($this->_value == '')
				return '';
			return 'States.CountryID="' . sql_escape_string($this->_value) . '"';
		}
	}
	
	if (!miHasParam('sortBy')) {
		miSetParam('sortBy', 'CountryID');
	}
	
	$states = new scSqlRecordsetML('States', 'StateID');
	$table = new scTable($states);
	$view = new scStatesView();
	
	$selectFields[] = 'States.*';
	$selectFields[] = 'StatesML.StateName';
	$selectFields[] = 'CountriesML.CountryName as CountryName';
	$states->setSelectFields($selectFields);
	$languageId = (int)scView::getCurrentDataLanguageId();
	$states->addJoinCondition('LEFT', 'CountriesML', 'ON CountriesML.CountryID=States.CountryID AND CountriesML.LanguageID=' . $languageId);
	
	$view->setRecord(new scSqlRecordML('States', 'StateID',
		array('StateName'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>