<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_TAXES);

	class scTaxRatesView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/tax_rates/tax_rates_mgmt.tmpl',
		);
        
		protected $_domainObjectDisplayName = 'Tax rates';	// _t('Tax rates');
		
		public function initTable($table)
		{
			parent::initTable($table);
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function addRedirectToListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = 'taxes.php?' . $this->getRedirectParams($msg, $msgType);
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
	}
	
	/**
	 * Shows data from the database in a table format
	 */
	class scListAction extends miListAction {
		
		protected function doList()
		{
			// Get Tax
			$taxId = (int)miGetParamDefault('TaxID', 0);
			$fieldValues['%%TAXID%%'] = $taxId;
			$fieldValues['%%TAXNAME%%'] = scTax::getTaxName($taxId);
			
			// Get Tax Classes
			$classRecordset = new miSqlRecordset('TaxClasses');
			$classRecordset->addFilter(new miSqlFilterEqual('TaxID', $taxId));
			$taxClasses = $classRecordset->getAllRecords();
			// Set classes section
			$classesSection = new miTemplateParserSectionInfo();
			$classesSection->setSectionInfo('classes', count($taxClasses), miUIUtil::templetizeRows(miUIUtil::transposeRows($taxClasses)));
			
			// Get Tax Zones
			$zoneRecordset = new miSqlRecordset('TaxZones');
			$zoneRecordset->addFilter(new miSqlFilterEqual('TaxID', $taxId));
			$taxZones = $zoneRecordset->getAllRecords();
			// Set zones section
			$zonesSection = new miTemplateParserSectionInfo();
			$zonesSection->setSectionInfo('zones', count($taxZones), miUIUtil::templetizeRows(miUIUtil::transposeRows($taxZones)));

			if (!count($taxZones) || !count($taxClasses)) {
				$this->_view->addRedirectToListControllerCommand(_t('You need to define tax classes and tax zones first.'), miMessage::MSG_TYPE_ERROR);
				return;
			}
			
			// Get Tax Rate
			$taxRates = array();
			for ($z = 0; $z < count($taxZones); $z++) {
				for ($c = 0; $c < count($taxClasses); $c++) {
					$sql = 'SELECT TaxRate FROM TaxRates WHERE TaxZoneID=? AND TaxClassID=?';
					$taxRate = miStaticDBUtil::execSelect($sql, array($taxZones[$z]['TaxZoneID'], $taxClasses[$c]['TaxClassID']));
					if (count($taxRate)) {
						$taxRates[$c]['TaxRate'] = $taxRate[0]['TaxRate'];
					} else {
						$taxRates[$c]['TaxRate'] = '';
					}
					$taxRates[$c]['TaxClassID'] = $taxClasses[$c]['TaxClassID'];
				}
				$taxRatesSection[$z] = new miTemplateParserSectionInfo();
				$taxRatesSection[$z]->setSectionInfo('rates', count($taxRates), miUIUtil::templetizeRows(miUIUtil::transposeRows($taxRates)));
			}
			// Set tax rate sections
			$zonesSection->addSubsectionsArray('rates', $taxRatesSection);
			
			// Show page
			$template = new scTemplateParser();
			miTable::escapeAndAssignArray($template, $this->_view->getMainPageElements());
			$template->assignArray($fieldValues);
			$template->readTemplate($this->_view->getTemplateFileName(miDefaultView::TEMPLATE_ID_LIST));
			$template->setSectionInfos(array($classesSection, $zonesSection));

			$html = $template->templateParse();
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($cmd);
		}
	}
	
	/**
	 * Takes the values from the user input and update a record in the database
	 */
	class scExecEditAction extends miEditAction {
		public function doAction()
		{
			// Get Tax ID
			$taxId = (int)miGetParamDefault('TaxID', 0);
			
			try {				
				// Get Tax Classes
				$taxClasses = miUtil::getDBArray('SELECT TaxClassID,TaxClassName FROM TaxClasses WHERE TaxID=' . $taxId, 'TaxClassID', 'TaxClassName');
				
				// Get Tax Zones
				$taxZones = miUtil::getDBArray('SELECT TaxZoneID,TaxZoneName FROM TaxZones WHERE TaxID=' . $taxId, 'TaxZoneID', 'TaxZoneName');

				foreach ($taxZones as $zoneId => $zone) {
					foreach ($taxClasses as $classId => $class) {
						
						miStaticDBUtil::execSQL('DELETE FROM TaxRates WHERE TaxZoneID=' . $zoneId . ' AND TaxClassID=' . $classId);
						
						$taxRate = miGetParamDefault('rate_' . $zoneId . '_' . $classId, 0);
						
						if ($taxRate != '') {
							$values = array(
								'TaxZoneID' => $zoneId,
								'TaxClassID' => $classId,
								'TaxRate' => (float)$taxRate
							);
							miStaticDBUtil::execInsert('TaxRates', $values);
						}
					}
				}
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_UPDATE_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			$msg = sprintf($this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG'), $taxId);
			$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	
	$taxRates = new miSqlRecordset('TaxRates');
	$table = new miTable($taxRates);
	
	$view = new scTaxRatesView();
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('', 'scListAction');
	$page->getDispatcher()->setActionHandler('dmExecEdit', 'scExecEditAction');
	$page->dispatchAndShow();
?>