<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_TAXES);
	
	class scTaxZonesView extends scView {
		
        protected $_templates = array(
			self::TEMPLATE_ID_EDIT => 'admin/tax_zones/tax_zone_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/tax_zones/tax_zone_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'TaxZoneID'
        		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'TaxZoneName',
				'validator' => 'scValidatorNoEmpty',
        		),
       		array(
				'field' => 'scWebFormFieldHidden',
				'data' => 'TaxID'
        		),
        	array(
				'field' => 'scDoubleListBoxWidget',
				'data' => 'Countries',
        		// _t('Available Countries'), _t('Selected Countries')
				'properties' => array('leftCaption' => 'Available Countries', 'rightCaption' => 'Selected Countries'),
				'modelStrategy' => 'scViewMapperTaxZoneCountries',
        		),
        	array(
				'field' => 'scDoubleListBoxWidget',
				'data' => 'States',
				// _t('Available States'), _t('Selected States')
				'properties' => array('leftCaption' => 'Available States', 'rightCaption' => 'Selected States'),
				'modelStrategy' => 'scViewMapperTaxZoneStates',
        		),
        );
        
        protected $_domainObjectDisplayName = 'Tax zone';	// _t('Tax zone');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Set Countries
			$fieldCountries = $form->getWidget('Countries');
			$fieldCountries->setOptions(scCountry::getCountryNames());
			
			// Set States
			$fieldStates = $form->getWidget('States');
			$languageId = (int)scView::getCurrentDataLanguageId();
			$sql = 'SELECT States.StateID,CONCAT(CountryName," - ",StateName) as StateName FROM States' .
				' LEFT JOIN StatesML ON States.StateID=StatesML.StateID AND StatesML.LanguageID=' . $languageId .
				' LEFT JOIN CountriesML ON States.CountryID=CountriesML.CountryID AND CountriesML.LanguageID=' . $languageId;
			$states = miUtil::getDBArray($sql, 'StateID', 'StateName');
			$fieldStates->setOptions($states);
		}
		
		/**
		 * Redirects the browser to the list of items.
		 */
		public function addRedirectToListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = 'taxes.php?action=dmEdit&' . $this->getRedirectParams($msg, $msgType) .
				'&' . 'TaxID=' . urlencode(miGetParamDefault('TaxID', ''));
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
	}
	
	class scViewMapperTaxZoneCountries extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'TaxZoneCountries';
			$this->_tableKeyName = 'TaxZoneID';
			$this->_tableValueName = 'CountryID';
			parent::__construct($model);
		}
	}
	
	class scViewMapperTaxZoneStates extends scViewMapperMultiValues {
		public function __construct($model)
		{
			$this->_tableName = 'TaxZoneStates';
			$this->_tableKeyName = 'TaxZoneID';
			$this->_tableValueName = 'StateID';
			parent::__construct($model);
		}
	}
	
	
	$taxZones = new miSqlRecordset('TaxZones');
	$table = new miTable($taxZones);
	
	$view = new scTaxZonesView();
	$view->setRecord(new miSqlRecord('TaxZones', 'TaxZoneID'));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->dispatchAndShow();
?>