<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_TAXES);

	class scTaxesView extends scView {
		
		const TEMPLATE_ID_LIST_CLASSES = 'listClass';
		const TEMPLATE_ID_LIST_ZONES = 'listZone';
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'admin/taxes/tax_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'admin/taxes/tax_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'admin/taxes/tax_create.tmpl',
			
			self::TEMPLATE_ID_LIST_CLASSES => 'admin/tax_classes/tax_class_mgmt.tmpl',
			self::TEMPLATE_ID_LIST_ZONES => 'admin/tax_zones/tax_zone_mgmt.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'TaxID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextML',
				'data' => 'TaxName',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Name'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'TaxRegNumber',
				'displayName' => 'Registration Number'
        	),
       		array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'TaxApplyToShipping'
        	),
        	array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'TaxApplyByDefault'
        	),
        	array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'TaxIncludeWithDisplayPrice'
        	)
        );
        
        protected $_domainObjectDisplayName = 'Tax';	// _t('Tax');
        
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function preProcess($actionObj)
		{
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('TaxApplyToShipping');

			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsDefault')
				$actionObj->setFieldName('TaxApplyByDefault');
		}
	}
	
	/**
	 * Show the page for updating a record in the database
	 */
	class scEditAction extends miEditAction {
		public function doAction()
		{
			try {
				if (miGetParamDefault('load', '') != 'table')
					$this->getWebForm()->addTemplateSectionInfo(new miTemplateParserSectionInfo('static'));
				
				$taxId = miGetParam('TaxID');
				$filter = new miSqlFilterEqual('TaxID', $taxId);
				
				/// List Classes
				$classes = new miSqlRecordset('TaxClasses');
				$classes->setOrder('TaxClassSort', 'ASC');
				$classes->addFilter($filter);
				$table = new scTable($classes);
				$table->assign('%%TAXID%%', $taxId);
			
				$html = $table->parse($this->_view->getTemplateFileName(scTaxesView::TEMPLATE_ID_LIST_CLASSES));
				$this->getWebForm()->addMainPageElements(array('%%HTML_TAX_CLASSES%%' => $html));
				
				/// List Zones
				$zones = new miSqlRecordset('TaxZones');
				$zones->addFilter($filter);
				$table = new miTable($zones);
				$table->assign('%%TAXID%%', $taxId);
				
				$html = $table->parse($this->_view->getTemplateFileName(scTaxesView::TEMPLATE_ID_LIST_ZONES));
				$this->getWebForm()->addMainPageElements(array('%%HTML_TAX_ZONES%%' => $html));
				
				return parent::doAction();
				
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_LIST_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	$taxes = new scSqlRecordsetML('Taxes', 'TaxID');
	$table = new scTable($taxes);
	
	$view = new scTaxesView();
	$view->setRecord(new scSqlRecordML('Taxes', 'TaxID',
		array('TaxName'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
	$view->setTable($table);
	
	$page = new scAdminDisplayController(array($view));
	$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmChangeIsDefault', 'scExecChangeIsActiveAction');
	$page->getDispatcher()->setActionHandler('dmEdit', 'scEditAction');
	$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
	$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
	
	$page->dispatchAndShow();
?>