<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	class scUpdater {
		
		public static function update()
		{
			if (!scAdminAuth::singleton()->isAuthorized()) {
				return false;
			}
			
			$path = scPaths::getStoreRootDir() . 'userfiles/upgrades/patch/';
			if (!file_exists($path)) {
				return false;
			}
			$handle = opendir($path);
			if ($handle === false) {
				return false;
			}
			
			$iniFile = scPaths::getStoreRootDir() . 'userfiles/upgrades/patch-updater.ini';
			
			$alreadyDone = array();
			if (file_exists($iniFile)) {
				$alreadyDone = parse_ini_file($iniFile);
				if ($alreadyDone === false) {
					return false;
				}
			}
			
			$log = @fopen($iniFile, 'ab');
			if ($log === false) {
				return false;
			}
			
			while (false !== ($item = readdir($handle))) {
				if (in_array($item, array('.', '..', '.svn'))) {
					continue;
				}
				$hash = md5($item);
				if (isset($alreadyDone[$hash])) {
					continue;
				}
				fwrite($log, $hash . ' = 1' . PHP_EOL);
				
				ob_start();
				include_once($path . $item);
				
				$message = ob_get_clean();
				if (!empty($message)) {
					$message = sprintf('"%s" output: ', $item) . $message;
					scLog::singleton()->log(scLog::SEVERITY_ERROR, $message);
				}
			}
			closedir($handle);
			
			return true;
		}
		
		public static function insertTexts($texts, $textType)
		{
			$languageCodes = scMultiLanguage::singleton()->getLanguageCodes();
			foreach ($texts as $textOriginal => $textTranslations) {
				$text = array(
					'TextType' => $textType,
					'TextOriginal' => $textOriginal,
					'TextOriginalHash' => crc32($textOriginal),
				);
				$textId = miStaticDBUtil::execInsert('Texts', $text, miDBUtilImpl::FLAG_IGNORE);
				if (!$textId) {
					continue;
				}
				
				// If there is no translation in English then use the original text as translation
				if (!isset($textTranslations['EN'])) {
					$textTranslations['EN'] = $textOriginal;
				}
				foreach ($languageCodes as $languageCode => $languageId) {
					$values = array(
						'TextID' => $textId,
						'LanguageID' => $languageId,
						'TextOriginalHash' => $text['TextOriginalHash'],
						'TextTranslated' => !empty($textTranslations[$languageCode]) ? $textTranslations[$languageCode] : '',
						'TextIsTranslated' => !empty($textTranslations[$languageCode]) ? 1 : 0,
					);
					miStaticDBUtil::execInsert('TextsTranslated', $values, miDBUtilImpl::FLAG_IGNORE);
				}
			}
		}
	}
	
	// Output a 1x1 transparent gif
	ignore_user_abort(false);
	header('Content-Type: image/gif');
	header('Connection: close');
	header('Content-Length: 43');
	echo base64_decode('R0lGODlhAQABAIABAP///wAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==');
	while (ob_get_length() !== false) {
		ob_end_flush();
	}
	flush();
	
	register_shutdown_function(array('scUpdater', 'update'));
?>
