<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scCityNamesRetreiver extends scAutoCompleteDataRetriever {
		
		const VILIGE_PREFIX = 'с.';
		const TOWN_PREFIX = 'гр.';
		
		public function outputData()
		{
			if (empty($this->_searchStr)) {
				return;
			}
			
			$stateId = miGetParamDefault('StateID', 0);
			$languageId = scMultiLanguage::getCurrentLanguageId();
			
			// We are using CML2 to retreive the city name in the current language,
			// while searching in all languages because we don't know 
			// the alphabet the user uses.
			if (empty($stateId)) {
				$countryId = miGetParamDefault('CountryID', 0);
				
				// If there isn't country or state print nothing - we don't search in all cities around the world
				if (empty($countryId))
					return;
				
				$sql = '
					SELECT 
						CML.CityID, CML2.CityName, Cities.StateID, StatesML.StateName
					FROM CitiesML CML
						INNER JOIN Cities USING(CityID)
						INNER JOIN States ON (States.StateID = Cities.StateID AND CountryID = ?)
						INNER JOIN StatesML ON (States.StateID = StatesML.StateID AND StatesML.LanguageID = ?)
						INNER JOIN CitiesML CML2 ON (CML2.CityID = CML.CityID AND CML2.LanguageID = ?)
					WHERE CML.CityName LIKE "' . miStaticDBUtil::escapeStringForLike($this->_searchStr) . '%"
					GROUP BY CML.CityID
					LIMIT 0, ' . (int)$this->_recordsLimit;
				$res = miStaticDBUtil::execSelect($sql, array($countryId, $languageId, $languageId));
				
			} else {
				$sql = '
					SELECT 
						CML.CityID, CML2.CityName, Cities.StateID 
					FROM 
						Cities 
						INNER JOIN CitiesML CML USING(CityID) 
						INNER JOIN CitiesML CML2 ON (CML2.CityID = CML.CityID AND CML2.LanguageID = ?)
					WHERE 
						Cities.StateID =? AND CML.CityName LIKE "' . miStaticDBUtil::escapeStringForLike($this->_searchStr) . '%"
					GROUP BY 
						CML.CityID 
					LIMIT 0, ' . (int)$this->_recordsLimit;
				$res = miStaticDBUtil::execSelect($sql, array($languageId, $stateId));
			}
			$result = '';
			foreach ($res as $row) {
				if (!empty($row['StateName'])) {
					$displayName = $row['CityName'] . ' (' . $row['StateName'] . ')'; 
					$stateName = ' (' . $row['StateName'] . ')';
				}else{
					$displayName = $row['CityName'];
					$stateName = '';
				}
				$result .= $this->sanitize($displayName) . $this->_cellSeparator;
				$result .= $this->sanitize($row['CityID']) . $this->_cellSeparator;
				$result .= $this->sanitize($row['StateID']) . $this->_cellSeparator;
				$result .= $this->sanitize($stateName) . "\n";
			}
			echo $result;
		}
	}
	
	try {
		$search = miGetParam('q');
		$search = str_replace(scCityNamesRetreiver::VILIGE_PREFIX, '', $search);
		$search = str_replace(scCityNamesRetreiver::TOWN_PREFIX, '', $search);
		$search = trim($search); // remove white spaces
		
		$retriever = new scCityNamesRetreiver($search, miGetParamDefault('limit', scAutoCompleteDataRetriever::DEFAULT_RESULTS));
		$retriever->outputData();
	} catch (miException $e) {
		$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($e->getMessage());
		scUtil::redirect($url);
		die;
	}
?>