<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiBundleProductOperation {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiBundleProduct
		 */
		public $BundleProduct;
	}
	
	class scSoapApiBundleProductOperationResult {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * BundleProduct id
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $BundleProductId;
		
		/**
		 * Your bundleProduct reference id. Will be returned only if there is sync info for this bundleProduct
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $BundleProductSyncId;
	}
	
	class scSoapApiBundleProductKey {
		
		/**
		 * BundleProduct id. Read only
		 * @var int
		 */
		public $BundleProductId;
		
		/**
		 * Your bundleProduct reference id. Must be unique
		 * @var string
		 */
		public $BundleProductSyncId;
	}
	
	/**
	 * BundleProduct object
	 */
	class scSoapApiBundleProduct {
		
		/**
		 * BundleProduct id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $BundleProductId;
		
		/**
		 * @var scSoapApiProductKey
		 * @xsd minOccurs 0
		 */
		public $ProductKey;
		
		/**
		 * @var scSoapApiProductKey
		 * @xsd minOccurs 0
		 */
		public $SubProductKey;
		
		/**
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $BundleProductQuantity;
		
		/**
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $BundleProductSort;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $BundleProductCustomData;
		
		/**
		 * BundleProduct sync info
		 * @var scSoapApiBundleProductSyncInfo
		 * @xsd minOccurs 0
		 */
		public $BundleProductSyncInfo;
	}
	
	
	class scSoapApiBundleProductSelector {
		
		/**
		 * @var int[]
		 */
		public $BundleProductId;
		
		/**
		 * @var int[]
		 */
		public $ProductId;
	}
	
	/**
	 * BundleProduct related operations implementation
	 */
	class scSoapApiServiceBundleProductOperations extends scSoapApiServiceImpl {
		
		/**
		 * Perform get operations
		 * @param scSoapApiBundleProductSelector $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			if (empty($selector->BundleProductId) && empty($selector->ProductId)) {
				return array();
			}
			
			$recordset = $this->getRecordset();
			if (!empty($selector->BundleProductId)) {
				$recordset->addFilter(new miSqlFilterIn('BundleProductID', $selector->BundleProductId));
			}
			if (!empty($selector->ProductId)) {
				$recordset->addFilter(new miSqlFilterIn('ProductID', $selector->ProductId));
			}
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, $recordset->getPrimaryKey(), $recordset->getPrimaryKey());
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record[$recordset->getPrimaryKey()]])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record[$recordset->getPrimaryKey()]];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		/**
		 * 
		 * @param scSoapApiBundleProductOperationResult $operationResult
		 * @param scSoapApiBundleProductOperation $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->BundleProduct->BundleProductSyncInfo))
				$operationResult->BundleProductSyncId = $operation->BundleProduct->BundleProductSyncInfo->BundleProductSyncId;
		}
		
		protected function getRecordset()
		{
			return new scAllLangsRecordset('BundleProducts', 'BundleProductID', array(), array());
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiBundleProduct
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiBundleProduct();
			$item->BundleProductId = $record['BundleProductID'];
			$item->ProductKey = array('ProductId' => $record['ProductID']);
			$item->SubProductKey = array('ProductId' => $record['SubProductID']);
			$item->BundleProductQuantity = $record['BundleProductQuantity'];
			$item->BundleProductSort = $record['BundleProductSort'];
			return $item;
		}
		
		/**
		 * @param scBundleProductDomainObject $object
		 * @param scSoapApiBundleProduct $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->ProductKey) and $isInsert)
				throw new miException(_i('ProductKey must be set on insert.'));
			if (!isset($item->SubProductKey) and $isInsert)
				throw new miException(_i('SubProductKey must be set on insert.'));
			if (!isset($item->BundleProductQuantity) and $isInsert)
				throw new miException(_i('BundleProductQuantity must be set on insert.'));
			if (isset($item->BundleProductQuantity) && ($item->BundleProductQuantity < 1))
				throw new miException(_i('If present BundleProductQuantity must be greater than 0.'));
			
			if (isset($item->ProductKey))
				$object->set('ProductID', scSoapApiServiceImpl::getLocalId('Product', $item->ProductKey));
			if (isset($item->SubProductKey))
				$object->set('SubProductID', scSoapApiServiceImpl::getLocalId('Product', $item->SubProductKey));
			if (isset($item->BundleProductQuantity))
				$object->set('BundleProductQuantity', (int)$item->BundleProductQuantity);
			
			if (isset($item->BundleProductSort)) {
				$sort = (int)$item->BundleProductSort;
				if ($isInsert or ($sort != $object->get('BundleProductSort'))) {
					$this->freeSort('BundleProducts', 'BundleProductSort', $sort, 'ProductID = ' . intval($object->get('ProductID')));
					$object->set('BundleProductSort', $sort);
				}
			} elseif ($isInsert) {
				$object->set('BundleProductSort', $this->getNextSort('BundleProducts', 'BundleProductSort', 'ProductID = ' . intval($object->get('ProductID'))));
			}
		}
		
		protected function createDomainObject()
		{
			return new scBundleProductDomainObject();
		}
	}
?>