<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Category operation
	 */
	class scSoapApiCategoryOperation {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiCategory
		 */
		public $Category;
	}
	
	/**
	 * Category operation result
	 */
	class scSoapApiCategoryOperationResult {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * Category id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $CategoryId;
		
		/**
		 * Your category reference id. Will be returned only if there is sync info for this category
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CategorySyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiCategoryKey {
		/**
		 * Category id. Read only
		 * @var int
		 */
		public $CategoryId;
		
		/**
		 * Your category reference id. Must be unique
		 * @var string
		 */
		public $CategorySyncId;
	}
	
	/**
	 * Category object
	 */
	class scSoapApiCategory {
		/**
		 * Category id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $CategoryId;
		
		/**
		 * Category parent
		 * @var scSoapApiCategoryKey
		 * @xsd minOccurs 0
		 */
		public $CategoryParentKey;
		
		/**
		 * The category name
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $CategoryName;
		
		/**
		 * The category description
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $CategoryDescription;
		
		/**
		 * @var scSoapApiImage
		 * @xsd minOccurs 0
		 */
		public $CategoryImage;
		
		/**
		 * The category sort order
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $CategorySort;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CategoryCustomData;
		
		/**
		 * Category sync info
		 * @var scSoapApiCategorySyncInfo
		 * @xsd minOccurs 0
		 */
		public $CategorySyncInfo;
	}
	
	/**
	 * Selector for categories
	 */
	class scSoapApiCategorySelector {
		/**
		 * @var int[]
		 */
		public $CategoryId;
	}
	
	/**
	 * Category related operations implementation
	 */
	class scSoapApiServiceCategoryOperations extends scSoapApiServiceImpl {
		
		/**
		 * Perform modify category operations
		 * 
		 * @param array $categoryOperations
		 * @return array
		 */
		public function doModifyOperation($entityName, $categoryOperations, $allowPartialSuccess)
		{
			$results = parent::doModifyOperation($entityName, $categoryOperations, $allowPartialSuccess);
			
			scNestedSet::buildNestedSet('Categories', 'CategoryID', 'CategoryParentID', 'CategoryLeft', 'CategoryRight');
			
			return $results;
		}
		
		/**
		 * 
		 * @param scSoapApiCategoryOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->Category->CategorySyncInfo))
				$operationResult->CategorySyncId = $operation->Category->CategorySyncInfo->CategorySyncId;
		}
		
		/**
		 * Return recordset for categories
		 * @return scAllLangsRecordset
		 */
		protected function getRecordset()
		{
			$recordMlFields = array('CategoryName', 'CategoryDescription', 'CategorySEOKeywords', 'CategorySEODescription');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('Categories', 'CategoryID', $recordMlFields, $recordLanguages);
		}
		
		/**
		 * Returns SOAP entity from record
		 * @param array $record
		 * @return scSoapApiCategory
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiCategory;
			$item->CategoryId = $record['CategoryID'];
			$item->CategoryParentKey = array('CategoryId' => ($record['CategoryParentID'] == null) ? 0 : $record['CategoryParentID']);
			$item->CategoryName = $this->mlToXml($record['CategoryName']);
			$item->CategorySort = $record['CategorySort'];
			$item->CategoryDescription = $this->mlToXml($record['CategoryDescription']);
			if (!empty($record['CategoryImage'])) {
				$item->CategoryImage = new scSoapApiImage();
				$item->CategoryImage->ImageUri = scPaths::getUrl($record['CategoryImage']);
			}
			return $item;
		}
		
		/**
		 * Set SOAP item data into the domain object
		 * @param scCategoryDomainObject $object
		 * @param scSoapApiCategory $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->CategoryParentKey) and $isInsert)
				throw new miException(_i('CategoryParentKey must be set on insert.'));
			
			if (isset($item->CategoryName))
				$object->set('CategoryName', $this->xmlToMl($item, 'CategoryName'));
			if (isset($item->CategoryDescription))
				$object->set('CategoryDescription', $this->xmlToMl($item, 'CategoryDescription'));
			
			if (isset($item->CategoryImage)) {
				$imageTempPath = $this->processImage($item->CategoryImage);
				$object->set('CategoryImage', $imageTempPath);
			}
			
			$categoryParentId = isset($item->CategoryParentKey) ? scSoapApiServiceImpl::getLocalId('Category', $item->CategoryParentKey) : null;
			if ($categoryParentId == 0)
				$categoryParentId = null;
			
			// Set the category parent only if set in the SOAP item
			if (isset($item->CategoryParentKey))
				$object->set('CategoryParentID', $categoryParentId);
			$categoryParentId = $object->get('CategoryParentID');
			
			// Handle category sort
			$additonalClause = ($categoryParentId == 0) ? 'CategoryParentID IS NULL' : 'CategoryParentID=' . intval($categoryParentId);
			if (isset($item->CategorySort)) {
				$sort = (int)$item->CategorySort;
				if ($isInsert or ($sort != $object->get('CategorySort'))) {
					$this->freeSort('Categories', 'CategorySort', $sort, $additonalClause);
					$object->set('CategorySort', $sort);
				}
			} elseif ($isInsert) {
				$object->set('CategorySort', $this->getNextSort('Categories', 'CategorySort', $additonalClause));
			}
		}
		
		/**
		 * Inserts a new category
		 * 
		 * @param scSoapApiCategory $item
		 * @return int category id
		 */
		protected function doInsert($entityName, $item)
		{
			$categoryObj = new scCategoryDomainObject();
			$this->setItemDataToDomainObject($categoryObj, $item, true);
			
			$categoryObj->set('CMSLayoutID', $this->getDefaultCmsLayoutId());
			
			miStaticDBUtil::startTransaction();
			try {
				$itemId = $categoryObj->insert();
				
				if ($categoryObj->has('CategoryImage')) {
					$oldPath = scPaths::getStoreRootDir() . $categoryObj->get('CategoryImage');
					$imagePathFile = scPaths::getItemPath($itemId, scPaths::RESOURCE_CATEGORY_IMAGE, scPaths::PATH_FILE);
					$imagePathWeb = scPaths::getItemPath($itemId, scPaths::RESOURCE_CATEGORY_IMAGE, scPaths::PATH_WEB);
					if (@rename($oldPath, $imagePathFile) == false) {
						throw new miException(_i('Cannot move file.'));
					}
					$categoryObj->set('CategoryImage', $imagePathWeb);
					$categoryObj->update();
				}
				
				miStaticDBUtil::commit();
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e; // Rethrow
			}
			
			if (isset($item->CategorySyncInfo)) {
				$this->getSyncImpl($entityName)->insertSyncInfo($itemId, $item->CategorySyncInfo->CategorySyncId, $item->CategorySyncInfo->HashCode);
			}
			return $itemId;
		}
		
		/**
		 * Update category
		 * 
		 * @param scSoapApiCategory $item
		 * @return int category id
		 */
		protected function doUpdate($entityName, $item)
		{
			if (!isset($item->CategoryId))
				throw new miException(_i('CategoryId is required.'));
			
			try {
				miStaticDBUtil::startTransaction();
				$categoryObj = new scCategoryDomainObject();
				$categoryObj->read($item->CategoryId);
				
				$oldImage = $categoryObj->get('CategoryImage');
				
				$this->setItemDataToDomainObject($categoryObj, $item, false);
				$itemId = $categoryObj->update();
				
				if ($categoryObj->get('CategoryImage') != $oldImage) {
					$oldPath = scPaths::getStoreRootDir() . $categoryObj->get('CategoryImage');
					$imagePathFile = scPaths::getItemPath($itemId, scPaths::RESOURCE_CATEGORY_IMAGE, scPaths::PATH_FILE);
					$imagePathWeb = scPaths::getItemPath($itemId, scPaths::RESOURCE_CATEGORY_IMAGE, scPaths::PATH_WEB);
					if (@rename($oldPath, $imagePathFile) == false) {
						throw new miException(_i('Cannot move file.'));
					}
					$categoryObj->set('CategoryImage', $imagePathWeb);
					$categoryObj->update();
				}
				
				miStaticDBUtil::commit();
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e; // Rethrow
			}
			if (isset($item->CategorySyncInfo)) {
				$this->getSyncImpl($entityName)->updateSyncInfo($itemId, $item->CategorySyncInfo->CategorySyncId, $item->CategorySyncInfo->HashCode);
			}
			return $itemId;
		}
		
		protected function doDelete($entityName, $item)
		{
			if (!scApiPrivs::hasAccess($this->_apiAccountPrivs, scApiPrivs::API_METHOD_MODIFY_DELETE_CODE)) {
				$xmlLocalId = scSoapApiMapping::getEntityField($entityName, scSoapApiMapping::XML_LOCAL_ID);
				return $item->{$xmlLocalId};
			}
			return parent::doDelete($entityName, $item);
		}
		
		protected function createDomainObject()
		{
			return new scCategoryDomainObject();
		}
	}
?>