<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiCustomerAddressOperation {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiCustomerAddress
		 */
		public $CustomerAddress;
	}
	
	class scSoapApiCustomerAddressOperationResult {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * CustomerAddress id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $CustomerAddressId;
		
		/**
		 * Your customerAddress reference id. Will be returned only if there is sync info for this customerAddress
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiCustomerAddressKey {
		
		/**
		 * CustomerAddress id. Read only
		 * @var int
		 */
		public $CustomerAddressId;
		
		/**
		 * Your customerAddress reference id. Must be unique
		 * @var string
		 */
		public $CustomerAddressSyncId;
	}
	
	/**
	 * CustomerAddress object
	 */
	class scSoapApiCustomerAddress {
		
		/**
		 * CustomerAddress id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressId;
		
		/**
		 * @var scSoapApiCustomerKey
		 * @xsd minOccurs 0
		 */
		public $CustomerKey;
		
		/**
		 * @var bool
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressIsDefaultBilling;
		
		/**
		 * @var bool
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressIsDefaultShipping;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressFirstName;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressLastName;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressCompany;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressPhone;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressFax;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressLine1;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressLine2;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressCity;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressStateCode;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressStateName;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressCountryName;
		
		/**
		 * ISO 3166-1 alpha-2 country code
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressCountryCode;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressZip;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressVatNumber;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressCustomData;
		
		/**
		 * CustomerAddress sync info
		 * @var scSoapApiCustomerAddressSyncInfo
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressSyncInfo;
	}
	
	class scSoapApiCustomerAddressSelector {
		
		/**
		 * @var int[]
		 */
		public $CustomerAddressId;
		
		/**
		 * @var int[]
		 */
		public $CustomerId;
	}
	
	/**
	 * CustomerAddress related operations implementation
	 */
	class scSoapApiServiceCustomerAddressOperations extends scSoapApiServiceImpl {
		
		/**
		 * @param scSoapApiCustomerAddressOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->CustomerAddress->CustomerAddressSyncInfo))
				$operationResult->CustomerAddressSyncId = $operation->CustomerAddress->CustomerAddressSyncInfo->CustomerAddressSyncId;
		}
		
		protected function getRecordset()
		{
			return new scAllLangsRecordset('CustomerAddresses', 'CustomerAddressID', array(), array());
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiCustomerAddress
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiCustomerAddress;
			$item->CustomerAddressId = $record['CustomerAddressID'];
			$item->CustomerKey = array('CustomerId' => $record['CustomerID']);
			$item->CustomerAddressIsDefaultBilling = (bool)$record['CustomerAddressIsDefaultBilling'];
			$item->CustomerAddressIsDefaultShipping = (bool)$record['CustomerAddressIsDefaultShipping'];
			$item->CustomerAddressFirstName = $record['CustomerAddressFirstName'];
			$item->CustomerAddressLastName = $record['CustomerAddressLastName'];
			$item->CustomerAddressCountryCode = scCountry::getCountryCode2($record['CountryID']);
			$item->CustomerAddressCountryName = scCountry::getCountryName($record['CountryID'], scMultiLanguage::getDefaultLanguageId());
			
			if (!empty($record['StateID'])) {
				$item->CustomerAddressStateCode = scCountry::getStateCode($record['StateID']);
				$item->CustomerAddressStateName = scCountry::getStateName($record['StateID'], scMultiLanguage::getDefaultLanguageId());
			}
			if (!empty($record['CustomerAddressCompany']))
				$item->CustomerAddressCompany = $record['CustomerAddressCompany'];
			if (!empty($record['CustomerAddressPhone']))
				$item->CustomerAddressPhone = $record['CustomerAddressPhone'];
			if (!empty($record['CustomerAddressFax']))
				$item->CustomerAddressFax = $record['CustomerAddressFax'];
			if (!empty($record['CustomerAddressLine1']))
				$item->CustomerAddressLine1 = $record['CustomerAddressLine1'];
			if (!empty($record['CustomerAddressLine2']))
				$item->CustomerAddressLine2 = $record['CustomerAddressLine2'];
			if (!empty($record['CustomerAddressZip']))
				$item->CustomerAddressZip = $record['CustomerAddressZip'];
			if (!empty($record['CustomerAddressVatNumber']))
				$item->CustomerAddressVatNumber = $record['CustomerAddressVatNumber'];
			if (!empty($record['CustomerAddressCity']))
				$item->CustomerAddressCity = $record['CustomerAddressCity'];
			
			return $item;
		}
		
		/**
		 * @param scCustomerAddress $object
		 * @param scSoapApiCustomerAddress $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (isset($item->CustomerAddressCountryName))
				throw new miException(_i('Passing CustomerAddressCountryName is not supported.'));
			if (isset($item->CustomerAddressStateName))
				throw new miException(_i('Passing CustomerAddressStateName is not supported.'));
			if ($isInsert && !isset($item->CustomerKey))
				throw new miException(_i('CustomerKey is required on insert.'));
			if ($isInsert && !isset($item->CustomerAddressCountryCode))
				throw new miException(_i('CustomerAddressCountryCode is required on insert.'));
			if ($isInsert && isset($item->CustomerAddressStateCode) && !isset($item->CustomerAddressCountryCode))
				throw new miException(_i('CustomerAddressCountryCode is required on insert when CustomerAddressStateCode is specified.'));
			
			if (!$isInsert && isset($item->CustomerAddressStateCode) && !isset($item->CustomerAddressCountryCode) && !is_numeric(@$object->get('CountryID'))) {
				throw new miException(_i('Unknown country when CustomerAddressStateCode is specified.'));
			}
			
			if (isset($item->CustomerAddressCountryCode)) {
				$countryId = scCountry::getCountryIdByCode2($item->CustomerAddressCountryCode);
				$object->set('CountryID', $countryId);
			}
			if (!isset($countryId)) {
				$countryId = $object->get('CountryID');
			}
			if (isset($item->CustomerAddressStateCode)) {
				$stateId = scCountry::getStateId($countryId, $item->CustomerAddressStateCode);
				$object->set('StateID', $stateId);
			}
			
			if (isset($item->CustomerKey))
				$object->set('CustomerID', scSoapApiServiceImpl::getLocalId('Customer', $item->CustomerKey));
			if (isset($item->CustomerAddressIsDefaultBilling))
				$object->set('CustomerAddressIsDefaultBilling', $item->CustomerAddressIsDefaultBilling);
			if (isset($item->CustomerAddressIsDefaultShipping))
				$object->set('CustomerAddressIsDefaultShipping', $item->CustomerAddressIsDefaultShipping);
			if (isset($item->CustomerAddressFirstName))
				$object->set('CustomerAddressFirstName', $item->CustomerAddressFirstName);
			if (isset($item->CustomerAddressLastName))
				$object->set('CustomerAddressLastName', $item->CustomerAddressLastName);
			if (isset($item->CustomerAddressCompany))
				$object->set('CustomerAddressCompany', $item->CustomerAddressCompany);
			if (isset($item->CustomerAddressPhone))
				$object->set('CustomerAddressPhone', $item->CustomerAddressPhone);
			if (isset($item->CustomerAddressFax))
				$object->set('CustomerAddressFax', $item->CustomerAddressFax);
			if (isset($item->CustomerAddressLine1))
				$object->set('CustomerAddressLine1', $item->CustomerAddressLine1);
			if (isset($item->CustomerAddressLine2))
				$object->set('CustomerAddressLine2', $item->CustomerAddressLine2);
			if (isset($item->CustomerAddressCity))
				$object->set('CustomerAddressCity', $item->CustomerAddressCity);
			if (isset($item->CustomerAddressZip))
				$object->set('CustomerAddressZip', $item->CustomerAddressZip);
			if (isset($item->CustomerAddressVatNumber))
				$object->set('CustomerAddressVatNumber', $item->CustomerAddressVatNumber);
		}
		
		/**
		 * Perform get operations
		 * @param scSoapApiCustomerAddressSelector $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			if (empty($selector->CustomerAddressId) && empty($selector->CustomerId)) {
				return array();
			}
			
			$recordset = $this->getRecordset();
			if (!empty($selector->CustomerAddressId)) {
				$recordset->addFilter(new miSqlFilterIn('CustomerAddressID', $selector->CustomerAddressId));
			}
			if (!empty($selector->CustomerId)) {
				$recordset->addFilter(new miSqlFilterIn('CustomerID', $selector->CustomerId));
			}
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, $recordset->getPrimaryKey(), $recordset->getPrimaryKey());
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record[$recordset->getPrimaryKey()]])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record[$recordset->getPrimaryKey()]];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		protected function createDomainObject()
		{
			return new scCustomerAddress();
		}
	}
?>