<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiCustomerOperation {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiCustomer
		 */
		public $Customer;
	}
	
	class scSoapApiCustomerOperationResult {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * Customer id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $CustomerId;
		
		/**
		 * Your customer reference id. Will be returned only if there is sync info for this customer
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiCustomerKey {
		
		/**
		 * Customer id. Read only
		 * @var int
		 */
		public $CustomerId;
		
		/**
		 * Your customer reference id. Must be unique
		 * @var string
		 */
		public $CustomerSyncId;
	}
	
	/**
	 * Customer object
	 */
	class scSoapApiCustomer {
		
		/**
		 * Customer id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $CustomerId;
		
		/**
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $CustomerGroupId;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerEmail;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerPassword;
		
		/**
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $CustomerBonusPoints;
		
		/**
		 * @var scSoapApiCustomerStatus
		 * @xsd minOccurs 0
		 */
		public $CustomerStatus;
		
		/**
		 * @var scSoapApiCustomerKey
		 * @xsd minOccurs 0
		 */
		public $CustomerReferrerKey;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $CustomerCustomData;
		
		/**
		 * Customer sync info
		 * @var scSoapApiCustomerSyncInfo
		 * @xsd minOccurs 0
		 */
		public $CustomerSyncInfo;
	}
	
	class scSoapApiCustomerStatus {
		
		const ACTIVE = 'ACTIVE';
		const DISABLED = 'DISABLED';
	}
	
	class scSoapApiCustomerSelector {
		
		/**
		 * @var int[]
		 */
		public $CustomerId;
		
		/**
		 * Match customer by e-mail (exact match)
		 * @var string[]
		 */
		public $CustomerEmail;
	}
	
	/**
	 * Customer related operations implementation
	 */
	class scSoapApiServiceCustomerOperations extends scSoapApiServiceImpl {
		
		protected $_statusesMapping = array(
			scCustomerStatuses::CUSTOMER_STATUS_ACTIVE => scSoapApiCustomerStatus::ACTIVE,
			scCustomerStatuses::CUSTOMER_STATUS_DISABLED => scSoapApiCustomerStatus::DISABLED,
		);
		
		/**
		 * Perform get operations
		 * @param scSoapApiCustomerSelector $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			if (empty($selector->CustomerId) && empty($selector->CustomerEmail)) {
				return array();
			}
			
			$recordset = $this->getRecordset();
			if (!empty($selector->CustomerId)) {
				$recordset->addFilter(new miSqlFilterIn('CustomerID', $selector->CustomerId));
			}
			if (!empty($selector->CustomerEmail)) {
				$recordset->addFilter(new miSqlFilterIn('CustomerEmail', $selector->CustomerEmail));
			}
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, 'CustomerID', 'CustomerID');
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record['CustomerID']])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record['CustomerID']];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		/**
		 * @param scSoapApiCustomerOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->Customer->CustomerSyncInfo))
				$operationResult->CustomerSyncId = $operation->Customer->CustomerSyncInfo->CustomerSyncId;
		}
		
		protected function getRecordset()
		{
			return new scAllLangsRecordset('Customers', 'CustomerID', array(), array());
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiCustomer
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiCustomer;
			$item->CustomerId = $record['CustomerID'];
			$item->CustomerEmail = $record['CustomerEmail'];
			
			// Do not return the customer password - security
			// $item->CustomerPassword = $record['CustomerPassword'];
			
			$item->CustomerStatus = $this->statusScToXml($record['CustomerStatus']);
			$item->CustomerBonusPoints = $record['CustomerBonusPoints'];
			if (!is_null($record['CustomerReferrerID'])) {
				$item->CustomerReferrerKey = array('CustomerId' => $record['CustomerReferrerID']);
			}
			return $item;
		}
		
		/**
		 * @param scCustomerDomainObject $object
		 * @param scSoapApiCustomer $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (isset($item->CustomerBonusPoints))
				throw new miException(_i('Passing CustomerBonusPoints is not supported.'));
			if ($isInsert && !isset($item->CustomerEmail))
				throw new miException(_i('CustomerEmail is required on insert.'));
			
			if (isset($item->CustomerEmail))
				$object->set('CustomerEmail', $item->CustomerEmail);
			if (isset($item->CustomerGroupId)) {
				$object->set('CustomerGroupID', $item->CustomerGroupId);
			} else if ($isInsert) {
				$object->set('CustomerGroupID', scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP));
			}
			if (isset($item->CustomerPassword)) {
				$object->set('CustomerPassword', $item->CustomerPassword);
			}
			if (isset($item->CustomerStatus))
				$object->set('CustomerStatus', $this->statusXmlToSc($item->CustomerStatus));
			if (isset($item->CustomerReferrerKey))
				$object->set('CustomerReferrerID', scSoapApiServiceImpl::getLocalId('Customer', $item->CustomerReferrerKey));
		}
		
		protected function statusXmlToSc($xmlValue)
		{
			$scValue = array_search($xmlValue, $this->_statusesMapping);
			if ($scValue === false) {
				throw new miException(sprintf(_i('Invalid status "%s".'), $xmlValue));
			}
			return $scValue;
		}
		
		protected function statusScToXml($scValue)
		{
			if (!isset($this->_statusesMapping[$scValue])) {
				throw new miException(sprintf(_i('Invalid status "%s".'), $scValue));
			}
			return $this->_statusesMapping[$scValue];
		}
		
		protected function createDomainObject()
		{
			return new scCustomerDomainObject();
		}
	}
?>