<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiLocalOptionGroupOperation {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiLocalOptionGroup
		 */
		public $LocalOptionGroup;
	}
	
	class scSoapApiLocalOptionGroupOperationResult {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * Local option group id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $LocalOptionGroupId;
		
		/**
		 * Your option group reference id. Will be returned only if there is sync info for this option group
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiLocalOptionGroupKey {
		
		/**
		 * Local option group id. Read only
		 * @var int
		 */
		public $LocalOptionGroupId;
		
		/**
		 * Your option group reference id. Must be unique
		 * @var string
		 */
		public $LocalOptionGroupSyncId;
	}
	
	/**
	 * Local Option Group object
	 */
	class scSoapApiLocalOptionGroup {
		
		/**
		 * LocalOptionGroup id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupId;
		
		/**
		 * Product to which option group is attached
		 * @var scSoapApiProductKey
		 * @xsd minOccurs 0
		 */
		public $ProductKey;
		
		/**
		 * @var scSoapApiLocalOptionGroupType
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupType;
		
		/**
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupSort;
		
		/**
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupName;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupCustomData;
		
		/**
		 * LocalOptionGroup sync info
		 * @var scSoapApiLocalOptionGroupSyncInfo
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupSyncInfo;
	}
	
	class scSoapApiLocalOptionGroupType {
		
		const PRICE_MODIFIER = 'PRICE_MODIFIER';
		const VARIANT = 'VARIANT';
		const USER_TEXT = 'USER_TEXT';
		const USER_IMAGE = 'USER_IMAGE';
	}
	
	class scSoapApiLocalOptionGroupSelector {
		
		/**
		 * @var int[]
		 */
		public $LocalOptionGroupId;
		
		/**
		 * @var int[]
		 */
		public $ProductId;
	}
	
	/**
	 * LocalOptionGroup related operations implementation
	 */
	class scSoapApiServiceLocalOptionGroupOperations extends scSoapApiServiceImpl {
		
		protected $_groupTypeMapping = array(
			scLocalOptions::GROUP_TYPE_PRICE_MODIFIER => scSoapApiLocalOptionGroupType::PRICE_MODIFIER,
			scLocalOptions::GROUP_TYPE_VARIANT => scSoapApiLocalOptionGroupType::VARIANT,
			scLocalOptions::GROUP_TYPE_USER_TEXT => scSoapApiLocalOptionGroupType::USER_TEXT,
			scLocalOptions::GROUP_TYPE_USER_IMAGE => scSoapApiLocalOptionGroupType::USER_IMAGE,
		);
		
		/**
		 * Perform get operations
		 * @param scSoapApiLocalOptionGroupSelector $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			if (empty($selector->LocalOptionGroupId) && empty($selector->ProductId)) {
				return array();
			}
			
			$recordset = $this->getRecordset();
			if (!empty($selector->LocalOptionGroupId)) {
				$recordset->addFilter(new miSqlFilterIn('LocalOptionGroupID', $selector->LocalOptionGroupId));
			}
			if (!empty($selector->ProductId)) {
				$recordset->addFilter(new miSqlFilterIn('ProductID', $selector->ProductId));
			}
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, $recordset->getPrimaryKey(), $recordset->getPrimaryKey());
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record[$recordset->getPrimaryKey()]])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record[$recordset->getPrimaryKey()]];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		/**
		 * 
		 * @param scSoapApiLocalOptionGroupOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->LocalOptionGroup->LocalOptionGroupSyncInfo))
				$operationResult->LocalOptionGroupSyncId = $operation->LocalOptionGroup->LocalOptionGroupSyncInfo->LocalOptionGroupSyncId;
		}
		
		protected function getRecordset()
		{
			$recordMlFields = array('LocalOptionGroupName');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('LocalOptionGroups', 'LocalOptionGroupID', $recordMlFields, $recordLanguages);
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiLocalOptionGroup
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiLocalOptionGroup();
			$item->LocalOptionGroupId = $record['LocalOptionGroupID'];
			$item->ProductKey = array('ProductId' => $record['ProductID']);
			$item->LocalOptionGroupName = $this->mlToXml($record['LocalOptionGroupName']);
			$item->LocalOptionGroupSort = $record['LocalOptionGroupSort'];
			$item->LocalOptionGroupType = $this->groupTypeScToXml($record['LocalOptionGroupType']);
			return $item;
		}
		
		/**
		 * @param scLocalOptionGroupDomainObject $object
		 * @param scSoapApiLocalOptionGroup $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->ProductKey) and $isInsert)
				throw new miException(_i('ProductKey must be set on insert.'));
			
			if (isset($item->ProductKey)) {
				$productId = scSoapApiServiceImpl::getLocalId('Product', $item->ProductKey);
				if (empty($productId)) {
					throw new miException(_i('Invalid ProductKey.'));
				}
				$object->set('ProductID', $productId);
			}
			if (isset($item->LocalOptionGroupType))
				$object->set('LocalOptionGroupType', $this->groupTypeXmlToSc($item->LocalOptionGroupType));
			if (isset($item->LocalOptionGroupName))
				$object->set('LocalOptionGroupName', $this->xmlToMl($item, 'LocalOptionGroupName'));
		
			if (isset($item->LocalOptionGroupSort)) {
				$sort = (int)$item->LocalOptionGroupSort;
				if ($isInsert or ($sort != $object->get('LocalOptionGroupSort'))) {
					$this->freeSort('LocalOptionGroups', 'LocalOptionGroupSort', $sort, 'ProductID = ' . intval($object->get('ProductID')));
					$object->set('LocalOptionGroupSort', $sort);
				}
			} elseif ($isInsert) {
				$object->set('LocalOptionGroupSort', $this->getNextSort('LocalOptionGroups', 'LocalOptionGroupSort', 'ProductID = ' . intval($object->get('ProductID'))));
			}
		}
		
		protected function groupTypeXmlToSc($xmlValue)
		{
			$scValue = array_search($xmlValue, $this->_groupTypeMapping);
			if ($scValue === false) {
				throw new miException(sprintf(_i('Invalid local option group type "%s".'), $xmlValue));
			}
			return $scValue;
		}
		
		protected function groupTypeScToXml($scValue)
		{
			if (!isset($this->_groupTypeMapping[$scValue])) {
				throw new miException(sprintf(_i('Invalid local option group type "%s".'), $scValue));
			}
			return $this->_groupTypeMapping[$scValue];
		}
		
		protected function createDomainObject()
		{
			return new scLocalOptionGroupDomainObject();
		}
	}
?>