<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiLocalOptionOperation {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiLocalOption
		 */
		public $LocalOption;
	}
	
	class scSoapApiLocalOptionOperationResult {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * Local option id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $LocalOptionId;
		
		/**
		 * Your option reference id. Will be returned only if there is sync info for this option
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $LocalOptionSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiLocalOptionKey {
		
		/**
		 * Local option id. Read only
		 * @var int
		 */
		public $LocalOptionId;
		
		/**
		 * Your option reference id. Must be unique
		 * @var string
		 */
		public $LocalOptionSyncId;
	}
	
	/**
	 * Local Option object
	 */
	class scSoapApiLocalOption {
		
		/**
		 * LocalOption id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $LocalOptionId;
		
		/**
		 * @var scSoapApiLocalOptionGroupKey
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupKey;
		
		/**
		 * @var scSoapApiLocalOptionPriceModType
		 * @xsd minOccurs 0
		 */
		public $LocalOptionPriceModType;
		
		/**
		 * @var decimal
		 * @xsd minOccurs 0
		 */
		public $LocalOptionPriceModValue;
		
		/**
		 * @var bool
		 * @xsd minOccurs 0
		 */
		public $LocalOptionRequired;
		
		/**
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $LocalOptionSort;
		
		/**
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $LocalOptionName;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $LocalOptionCustomData;
		
		/**
		 * LocalOption sync info
		 * @var scSoapApiLocalOptionSyncInfo
		 * @xsd minOccurs 0
		 */
		public $LocalOptionSyncInfo;
	}
	
	class scSoapApiLocalOptionPriceModType {
		
		const NONE = 'NONE';
		const AMOUNT = 'AMOUNT';
		const PERCENT = 'PERCENT';
	}
	
	class scSoapApiLocalOptionSelector {
		
		/**
		 * @var int[]
		 */
		public $LocalOptionId;
		
		/**
		 * @var int[]
		 */
		public $LocalOptionGroupId;
		
		/**
		 * @var int[]
		 */
		public $ProductId;
	}
	
	/**
	 * LocalOption related operations implementation
	 */
	class scSoapApiServiceLocalOptionOperations extends scSoapApiServiceImpl {
		
		protected $_priceModTypeMapping = array(
			scLocalOptions::PRODUCT_OPTION_TYPE_NONE => scSoapApiLocalOptionPriceModType::NONE,
			scLocalOptions::PRODUCT_OPTION_TYPE_AMOUNT => scSoapApiLocalOptionPriceModType::AMOUNT,
			scLocalOptions::PRODUCT_OPTION_TYPE_PERCENT => scSoapApiLocalOptionPriceModType::PERCENT,
		);
		
		/**
		 * Perform get operations
		 * @param scSoapApiLocalOptionSelector $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			if (empty($selector->LocalOptionId) && empty($selector->LocalOptionGroupId) && empty($selector->ProductId)) {
				return array();
			}
			
			$recordset = $this->getRecordset();
			if (!empty($selector->LocalOptionId)) {
				$recordset->addFilter(new miSqlFilterIn('LocalOptionID', $selector->LocalOptionId));
			}
			if (!empty($selector->LocalOptionGroupId)) {
				$recordset->addFilter(new miSqlFilterIn('LocalOptionGroupID', $selector->LocalOptionGroupId));
			}
			if (!empty($selector->ProductId)) {
				$recordset->addJoinCondition('INNER', 'LocalOptionGroups', 'USING(LocalOptionGroupID)');
				$recordset->addFilter(new miSqlFilterIn('ProductID', $selector->ProductId));
			}
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, $recordset->getPrimaryKey(), $recordset->getPrimaryKey());
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record[$recordset->getPrimaryKey()]])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record[$recordset->getPrimaryKey()]];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		/**
		 * 
		 * @param scSoapApiLocalOptionOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->LocalOption->LocalOptionSyncInfo))
				$operationResult->LocalOptionSyncId = $operation->LocalOption->LocalOptionSyncInfo->LocalOptionSyncId;
		}
		
		/**
		 * @return scAllLangsRecordset
		 */
		protected function getRecordset()
		{
			$recordMlFields = array('LocalOptionName');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('LocalOptions', 'LocalOptionID', $recordMlFields, $recordLanguages);
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiLocalOption
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiLocalOption();
			$item->LocalOptionGroupKey = array('LocalOptionGroupId' => $record['LocalOptionGroupID']);
			$item->LocalOptionId = $record['LocalOptionID'];
			$item->LocalOptionName = $this->mlToXml($record['LocalOptionName']);
			$item->LocalOptionSort = $record['LocalOptionSort'];
			$item->LocalOptionPriceModType = $this->priceModTypeScToXml($record['LocalOptionPriceModType']);
			$item->LocalOptionPriceModValue = $record['LocalOptionPriceModValue'];
			$item->LocalOptionRequired = $record['LocalOptionRequired'];
			return $item;
		}
		
		/**
		 * @param scLocalOptionDomainObject $object
		 * @param scSoapApiLocalOption $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->LocalOptionGroupKey) and $isInsert)
				throw new miException(_i('LocalOptionGroupKey must be set on insert.'));
			
			if (isset($item->LocalOptionGroupKey))
				$object->set('LocalOptionGroupID', scSoapApiServiceImpl::getLocalId('LocalOptionGroup', $item->LocalOptionGroupKey));
			if (isset($item->LocalOptionPriceModType))
				$object->set('LocalOptionPriceModType', $this->priceModTypeXmlToSc($item->LocalOptionPriceModType));
			if (isset($item->LocalOptionPriceModValue))
				$object->set('LocalOptionPriceModValue', $item->LocalOptionPriceModValue);
			if (isset($item->LocalOptionRequired))
				$object->set('LocalOptionRequired', $item->LocalOptionRequired);
			if (isset($item->LocalOptionName))
				$object->set('LocalOptionName', $this->xmlToMl($item, 'LocalOptionName'));
			
			if (isset($item->LocalOptionSort)) {
				$sort = (int)$item->LocalOptionSort;
				if ($isInsert or ($sort != $object->get('LocalOptionSort'))) {
					$this->freeSort('LocalOptions', 'LocalOptionSort', $sort, 'LocalOptionGroupID = ' . intval($object->get('LocalOptionGroupID')));
					$object->set('LocalOptionSort', $sort);
				}
			} elseif ($isInsert) {
				$object->set('LocalOptionSort', $this->getNextSort('LocalOptions', 'LocalOptionSort', 'LocalOptionGroupID = ' . intval($object->get('LocalOptionGroupID'))));
			}
		}
		
		protected function priceModTypeXmlToSc($xmlValue)
		{
			$scValue = array_search($xmlValue, $this->_priceModTypeMapping);
			if ($scValue === false) {
				throw new miException(sprintf(_i('Invalid local option price mod type "%s".'), $xmlValue));
			}
			return $scValue;
		}
		
		protected function priceModTypeScToXml($scValue)
		{
			if (!isset($this->_priceModTypeMapping[$scValue])) {
				throw new miException(sprintf(_i('Invalid local option price mod type "%s".'), $scValue));
			}
			return $this->_priceModTypeMapping[$scValue];
		}
		
		protected function createDomainObject()
		{
			return new scLocalOptionDomainObject();
		}
	}
?>