<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiManufacturerOperation {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiManufacturer
		 */
		public $Manufacturer;
	}
	
	class scSoapApiManufacturerOperationResult {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * Manufacturer id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $ManufacturerId;
		
		/**
		 * Your manufacturer reference id. Will be returned only if there is sync info for this manufacturer
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ManufacturerSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiManufacturerKey {
		/**
		 * Manufacturer id. Read only
		 * @var int
		 */
		public $ManufacturerId;
		
		/**
		 * Your manufacturer reference id. Must be unique
		 * @var string
		 */
		public $ManufacturerSyncId;
	}
	
	/**
	 * Manufacturer object
	 */
	class scSoapApiManufacturer {
		/**
		 * Manufacturer id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ManufacturerId;
		
		/**
		 * The manufacturer name
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ManufacturerName;
		
		/**
		 * The manufacturer description
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ManufacturerDescription;
		
		/**
		 * @var scSoapApiImage
		 * @xsd minOccurs 0
		 */
		public $ManufacturerImage;
		
		/**
		 * The manufacturer sort order
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ManufacturerSort;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ManufacturerCustomData;
		
		/**
		 * Manufacturer sync info
		 * @var scSoapApiManufacturerSyncInfo
		 * @xsd minOccurs 0
		 */
		public $ManufacturerSyncInfo;
	}
	
	
	class scSoapApiManufacturerSelector {
		/**
		 * @var int[]
		 */
		public $ManufacturerId;
	}
	
	/**
	 * Manufacturer related operations implementation
	 */
	class scSoapApiServiceManufacturerOperations extends scSoapApiServiceImpl {
		
		/**
		 * 
		 * @param scSoapApiManufacturerOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->Manufacturer->ManufacturerSyncInfo))
				$operationResult->ManufacturerSyncId = $operation->Manufacturer->ManufacturerSyncInfo->ManufacturerSyncId;
		}
		
		protected function getRecordset()
		{
			$recordMlFields = array('ManufacturerName', 'ManufacturerDescription');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('Manufacturers', 'ManufacturerID', $recordMlFields, $recordLanguages);
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiManufacturer
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiManufacturer;
			$item->ManufacturerId = $record['ManufacturerID'];
			$item->ManufacturerName = $this->mlToXml($record['ManufacturerName']);
			$item->ManufacturerSort = $record['ManufacturerSort'];
			$item->ManufacturerDescription = $this->mlToXml($record['ManufacturerDescription']);
			if (!empty($record['ManufacturerImage'])) {
				$item->ManufacturerImage = new scSoapApiImage();
				$item->ManufacturerImage->ImageUri = scPaths::getUrl($record['ManufacturerImage']);
			}
			return $item;
		}
		
		/**
		 * @param scManufacturerDomainObject $object
		 * @param scSoapApiManufacturer $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (isset($item->ManufacturerName))
				$object->set('ManufacturerName', $this->xmlToMl($item, 'ManufacturerName'));
			if (isset($item->ManufacturerDescription))
				$object->set('ManufacturerDescription', $this->xmlToMl($item, 'ManufacturerDescription'));
			
			if (isset($item->ManufacturerImage)) {
				$imageTempPath = $this->processImage($item->ManufacturerImage);
				$object->set('ManufacturerImage', $imageTempPath);
			}
			
			// Handle manufacturer sort
			if (isset($item->ManufacturerSort)) {
				$sort = (int)$item->ManufacturerSort;
				if ($isInsert or ($sort != $object->get('ManufacturerSort'))) {
					$this->freeSort('Manufacturers', 'ManufacturerSort', $sort);
					$object->set('ManufacturerSort', $sort);
				}
			} elseif ($isInsert) {
				$object->set('ManufacturerSort', $this->getNextSort('Manufacturers', 'ManufacturerSort'));
			}
		}
		
		/**
		 * Inserts a new manufacturer
		 * 
		 * @param scSoapApiManufacturer $item
		 * @return int
		 */
		protected function doInsert($entityName, $item)
		{
			$manufacturerObj = new scManufacturerDomainObject();
			$this->setItemDataToDomainObject($manufacturerObj, $item, true);
			
			miStaticDBUtil::startTransaction();
			try {
				$itemId = $manufacturerObj->insert();
				
				if ($manufacturerObj->has('ManufacturerImage')) {
					$oldPath = scPaths::getStoreRootDir() . $manufacturerObj->get('ManufacturerImage');
					$imagePathFile = scPaths::getItemPath($itemId, scPaths::RESOURCE_MANUFACTURER_IMAGE, scPaths::PATH_FILE);
					$imagePathWeb = scPaths::getItemPath($itemId, scPaths::RESOURCE_MANUFACTURER_IMAGE, scPaths::PATH_WEB);
					if (@rename($oldPath, $imagePathFile) == false) {
						throw new miException(_i('Cannot move file.'));
					}
					$manufacturerObj->set('ManufacturerImage', $imagePathWeb);
					$manufacturerObj->update();
				}
				
				miStaticDBUtil::commit();
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e; // Rethrow
			}
			
			if (isset($item->ManufacturerSyncInfo)) {
				$this->getSyncImpl($entityName)->insertSyncInfo($itemId, $item->ManufacturerSyncInfo->ManufacturerSyncId, $item->ManufacturerSyncInfo->HashCode);
			}
			return $itemId;
		}
		
		protected function doUpdate($entityName, $item)
		{
			if (!isset($item->ManufacturerId))
				throw new miException(_i('ManufacturerId is required.'));
			
			try {
				miStaticDBUtil::startTransaction();
				$manufacturerObj = new scManufacturerDomainObject();
				$manufacturerObj->read($item->ManufacturerId);
				
				$oldImage = $manufacturerObj->get('ManufacturerImage');
				
				$this->setItemDataToDomainObject($manufacturerObj, $item, false);
				$itemId = $manufacturerObj->update();
				
				if ($manufacturerObj->get('ManufacturerImage') != $oldImage) {
					$oldPath = scPaths::getStoreRootDir() . $manufacturerObj->get('ManufacturerImage');
					$imagePathFile = scPaths::getItemPath($itemId, scPaths::RESOURCE_MANUFACTURER_IMAGE, scPaths::PATH_FILE);
					$imagePathWeb = scPaths::getItemPath($itemId, scPaths::RESOURCE_MANUFACTURER_IMAGE, scPaths::PATH_WEB);
					if (@rename($oldPath, $imagePathFile) == false) {
						throw new miException(_i('Cannot move file.'));
					}
					$manufacturerObj->set('ManufacturerImage', $imagePathWeb);
					$manufacturerObj->update();
				}
				
				miStaticDBUtil::commit();
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e; // Rethrow
			}
			
			if (isset($item->ManufacturerSyncInfo)) {
				$this->getSyncImpl($entityName)->updateSyncInfo($itemId, $item->ManufacturerSyncInfo->ManufacturerSyncId, $item->ManufacturerSyncInfo->HashCode);
			}
			return $itemId;
		}
		
		protected function createDomainObject()
		{
			return new scManufacturerDomainObject();
		}
	}
?>