<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiProductAttributeValueOperation {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiProductAttributeValue
		 */
		public $ProductAttributeValue;
	}
	
	class scSoapApiProductAttributeValueOperationResult {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * ProductAttributeValue id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $ProductAttributeValueId;
		
		/**
		 * Your productAttributeValue reference id. Will be returned only if there is sync info for this productAttributeValue
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiProductAttributeValueKey {
		/**
		 * ProductAttributeValue id. Read only
		 * @var int
		 */
		public $ProductAttributeValueId;
		
		/**
		 * Your productAttributeValue reference id. Must be unique
		 * @var string
		 */
		public $ProductAttributeValueSyncId;
	}
	
	/**
	 * Contains one of the 4 possible values that a product attribute can have
	 * @xsd order choice
	 */
	class scSoapApiProductAttributeValueChoice {
		/**
		 * The product attribute value if the attribute type is text
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueText;
		
		/**
		 * The product attribute value if the attribute type is multilanguage text
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueML;
		
		/**
		 * The product attribute value if the attribute type is drop-down or radio buttons
		 * @var scSoapApiProductClassAttributeValueKey
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueKey;
		
		/**
		 * The product attribute value if the attribute type is multiple select
		 * @var scSoapApiProductClassAttributeValueKey[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeMultiValueKey;
	}
	
	/**
	 * ProductAttributeValue object
	 */
	class scSoapApiProductAttributeValue {
		/**
		 * ProductAttributeValue id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueId;
		
		/**
		 * Product to which attribute value is attached
		 * @var scSoapApiProductKey
		 * @xsd minOccurs 0
		 */
		public $ProductKey;
		
		/**
		 * The attribute that this value specifies
		 * @var scSoapApiProductClassAttributeKey
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeKey;
		
		/**
		 * 
		 * @var scSoapApiProductAttributeValueChoice
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValue;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueCustomData;
		
		/**
		 * ProductAttributeValue sync info
		 * @var scSoapApiProductAttributeValueSyncInfo
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueSyncInfo;
	}
	
	
	class scSoapApiProductAttributeValueSelector {
		
		/**
		 * @var int[]
		 */
		public $ProductAttributeValueId;
		
		/**
		 * @var int[]
		 */
		public $ProductId;
	}
	
	/**
	 * ProductAttributeValue related operations implementation
	 */
	class scSoapApiServiceProductAttributeValueOperations extends scSoapApiServiceImpl {
		
		/**
		 * Perform get operations
		 * @param scSoapApiProductAttributeValueSelector $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			if (empty($selector->ProductAttributeValueId) && empty($selector->ProductId)) {
				return array();
			}
			
			$recordset = $this->getRecordset();
			if (!empty($selector->ProductAttributeValueId)) {
				$recordset->addFilter(new miSqlFilterIn('ProductAttributeValueID', $selector->ProductAttributeValueId));
			}
			if (!empty($selector->ProductId)) {
				$recordset->addFilter(new miSqlFilterIn('ProductID', $selector->ProductId));
			}
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, $recordset->getPrimaryKey(), $recordset->getPrimaryKey());
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record[$recordset->getPrimaryKey()]])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record[$recordset->getPrimaryKey()]];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		/**
		 * 
		 * @param scSoapApiProductAttributeValueOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->ProductAttributeValue->ProductAttributeValueSyncInfo))
				$operationResult->ProductAttributeValueSyncId = $operation->ProductAttributeValue->ProductAttributeValueSyncInfo->ProductAttributeValueSyncId;
		}
		
		protected function getRecordset()
		{
			$recordMlFields = array('ProductAttributeValueName', 'ProductAttributeValueDescription');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('ProductAttributeValues', 'ProductAttributeValueID', $recordMlFields, $recordLanguages);
		}
		
		protected function getProductClassAttributeType($productClassAttributeId)
		{
			return scUtil::getDbValue('ProductClassAttributes', 'ProductClassAttributeType', 'ProductClassAttributeID', $productClassAttributeId, 0);
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiProductAttributeValue
		 */
		protected function getItemDataFromRecord($record)
		{
			$attributeType = $this->getProductClassAttributeType($record['ProductClassAttributeID']);
			
			$item = new scSoapApiProductAttributeValue;
			$item->ProductAttributeValueId = $record['ProductAttributeValueID'];
			$item->ProductKey = array('ProductId' => $record['ProductID']);
			$item->ProductClassAttributeKey = array('ProductClassAttributeId' => $record['ProductClassAttributeID']);
			$item->ProductAttributeValue = array();
			switch ($attributeType) {
				case scProductAttributes::ATTRIBUTE_TYPE_TEXT:
					$item->ProductAttributeValue = array('ProductAttributeValueText' => $record['ProductAttributeValue']);
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_TEXT_ML:
					$item->ProductAttributeValue = array('ProductAttributeValueML' => $this->mlToXml($record['ProductAttributeValueML']));
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_SELECT:
				case scProductAttributes::ATTRIBUTE_TYPE_RADIO_BUTTONS:
					$item->ProductAttributeValue = array('ProductClassAttributeValueKey' => array('ProductClassAttributeValueId' => $record['ProductClassAttributeValueID']));
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_MULTIPLE_SELECT:
					$valueIds = $this->getMultiSelectedAttributeValuesId($record['ProductAttributeValueID']);
					$item->ProductAttributeValue = array('ProductClassAttributeMultiValueKey' => $valueIds);
					break;
			}
			return $item;
		}
		
		protected function getMultiSelectedAttributeValuesId($productAttributeValueId)
		{
			$query = 'SELECT ProductClassAttributeMultiValue FROM ProductAttributeMultiValues WHERE ProductAttributeValueID=' . $productAttributeValueId;
			$valudIds = miUtil::getDBArray($query, 'ProductClassAttributeMultiValue', 'ProductClassAttributeMultiValue');
			
			$result = array();
			foreach ($valudIds as $valueId) {
				$result[] = array('ProductClassAttributeValueId' => $valueId);
			}
			return $result;
		}
		
		protected function updateMultiSelectAttributeValueIds($productAttributeValueId, $values)
		{
			if (!is_array($values))
				$values = array($values);
			
			miStaticDBUtil::execDelete('ProductAttributeMultiValues', 'ProductAttributeValueID', $productAttributeValueId);
			
			foreach ($values as $value) {
				$localId = scSoapApiServiceImpl::getLocalId('ProductClassAttributeValue', $value);
				miStaticDBUtil::execInsert('ProductAttributeMultiValues', array('ProductAttributeValueID' => $productAttributeValueId, 'ProductClassAttributeMultiValue' => $localId));
			}
		}
		
		/**
		 * @param scProductAttributeValueDomainObject $object
		 * @param scSoapApiProductAttributeValue $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->ProductClassAttributeKey) and $isInsert)
				throw new miException(_i('ProductClassAttributeKey must be set on insert.'));
			
			if (isset($item->ProductClassAttributeKey))
				$object->set('ProductClassAttributeID', scSoapApiServiceImpl::getLocalId('ProductClassAttribute', $item->ProductClassAttributeKey));
				
			if (!isset($item->ProductKey) and $isInsert)
				throw new miException(_i('ProductKey must be set on insert.'));
			if (isset($item->ProductKey))
				$object->set('ProductID', scSoapApiServiceImpl::getLocalId('Product', $item->ProductKey));
			
			$attributeType = $this->getProductClassAttributeType($object->get('ProductClassAttributeID'));
			switch ($attributeType) {
				case scProductAttributes::ATTRIBUTE_TYPE_TEXT:
					if (isset($item->ProductAttributeValue->ProductAttributeValueText))
						$object->set('ProductAttributeValue', $item->ProductAttributeValue->ProductAttributeValueText);
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_TEXT_ML:
					if (isset($item->ProductAttributeValue->ProductAttributeValueML))
						$object->set('ProductAttributeValueML', $this->xmlToMl($item->ProductAttributeValue, 'ProductAttributeValueML'));
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_SELECT:
				case scProductAttributes::ATTRIBUTE_TYPE_RADIO_BUTTONS:
					if (isset($item->ProductAttributeValue->ProductClassAttributeValueKey)) {
						$object->set('ProductClassAttributeValueID', scSoapApiServiceImpl::getLocalId('ProductClassAttributeValue', $item->ProductAttributeValue->ProductClassAttributeValueKey));
					}
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_MULTIPLE_SELECT:
					// The multivalues are updated from doInsert()/doUpdate()
					break;
			}
		}
		
		protected function createDomainObject()
		{
			return new scProductAttributeValueDomainObject();
		}
		
		protected function doInsert($entityName, $item)
		{
			$itemId = parent::doInsert($entityName, $item);
			
			if (isset($item->ProductAttributeValue->ProductClassAttributeMultiValueKey))
				$this->updateMultiSelectAttributeValueIds($itemId, $item->ProductAttributeValue->ProductClassAttributeMultiValueKey);
			
			return $itemId;
		}
		
		protected function doUpdate($entityName, $item)
		{
			$itemId = parent::doUpdate($entityName, $item);
			
			if (isset($item->ProductAttributeValue->ProductClassAttributeMultiValueKey))
				$this->updateMultiSelectAttributeValueIds($itemId, $item->ProductAttributeValue->ProductClassAttributeMultiValueKey);
				
			return $itemId;
		}
	}
	
	class scProductAttributeValueDomainObject extends scDomainObject {
		
		public function __construct()
		{
			$record = new scSqlRecordML('ProductAttributeValues', 'ProductAttributeValueID', array('ProductAttributeValueML'), 
				array_keys(scMultiLanguage::getActiveLanguageCodes()));
			parent::__construct($record);
		}
	}
?>