<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiProductClassAttributeOperation {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiProductClassAttribute
		 */
		public $ProductClassAttribute;
	}
	
	class scSoapApiProductClassAttributeOperationResult {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * ProductClassAttribute id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $ProductClassAttributeId;
		
		/**
		 * Your productClassAttribute reference id. Will be returned only if there is sync info for this productClassAttribute
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeSyncId;
	}
	
	/**
	 */
	class scSoapApiProductClassAttributeType {
		const TEXT = 'TEXT';
		const TEXT_ML = 'TEXT_ML';
		const SELECT = 'SELECT';
		const RADIO_BUTTONS = 'RADIO_BUTTONS';
		const MULTIPLE_SELECT = 'MULTIPLE_SELECT';
	}
	
	class scSoapApiProductClassAttributeValidator {
		const NONE = 'NONE';
		const NUMBER = 'NUMBER';
	}
	
//	class scSoapApiProductClassAttributeValue {
//		/**
//		 * 
//		 * @var int
//		 */
//		public $ProductClassAttributeValueId;
//		
//		/**
//		 * The product class attribute value text
//		 * @var scSoapApiMultilanguageText[]
//		 * @xsd minOccurs 0
//		 */
//		public $ProductClassAttributeValueText;
//	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiProductClassAttributeKey {
		/**
		 * ProductClassAttribute id. Read only
		 * @var int
		 */
		public $ProductClassAttributeId;
		
		/**
		 * Your productClassAttribute reference id. Must be unique
		 * @var string
		 */
		public $ProductClassAttributeSyncId;
	}
	
	/**
	 * ProductClassAttribute object
	 */
	class scSoapApiProductClassAttribute {
		/**
		 * ProductClassAttribute id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeId;
		
		/**
		 * Product class key
		 * @var scSoapApiProductClassKey
		 * @xsd minOccurs 0
		 */
		public $ProductClassKey;
		
		/**
		 * The productClassAttribute name
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeName;
		
		/**
		 * The type of the attribute
		 * @var scSoapApiProductClassAttributeType
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeType;
		
		/**
		 * The validator of the attribute. Depends on the type of the attribute
		 * @var scSoapApiProductClassAttributeValidator
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValidator;
		
//		/**
//		 * The attribute values
//		 * Used when the attribute type is drop-down, radio buttons or multiple select
//		 * @var scSoapApiProductClassAttributeValue[]
//		 */
//		public $ProductClassAttributeValue;
		
		/**
		 * The product class attribute sort order
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeSort;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeCustomData;
		
		/**
		 * ProductClassAttribute sync info
		 * @var scSoapApiProductClassAttributeSyncInfo
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeSyncInfo;
	}
	
	
	class scSoapApiProductClassAttributeSelector {
		
		/**
		 * @var int[]
		 */
		public $ProductClassAttributeId;
		
		/**
		 * @var int[]
		 */
		public $ProductClassId;
	}
	
	/**
	 * ProductClassAttribute related operations implementation
	 */
	class scSoapApiServiceProductClassAttributeOperations extends scSoapApiServiceImpl {
		
		protected $_attributeTypesMap = array(
			scSoapApiProductClassAttributeType::TEXT => scProductAttributes::ATTRIBUTE_TYPE_TEXT,
			scSoapApiProductClassAttributeType::TEXT_ML => scProductAttributes::ATTRIBUTE_TYPE_TEXT_ML,
			scSoapApiProductClassAttributeType::SELECT => scProductAttributes::ATTRIBUTE_TYPE_SELECT,
			scSoapApiProductClassAttributeType::MULTIPLE_SELECT => scProductAttributes::ATTRIBUTE_TYPE_MULTIPLE_SELECT,
			scSoapApiProductClassAttributeType::RADIO_BUTTONS => scProductAttributes::ATTRIBUTE_TYPE_RADIO_BUTTONS
		);
		protected $_attributeValidatorsMap = array(
			scSoapApiProductClassAttributeValidator::NONE => scProductAttributes::VALIDATOR_NONE,
			scSoapApiProductClassAttributeValidator::NUMBER => scProductAttributes::VALIDATOR_NUMBER
		);
		
		/**
		 * Perform get operations
		 * @param scSoapApiProductClassAttributeSelector $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			if (empty($selector->ProductClassAttributeId) && empty($selector->ProductClassId)) {
				return array();
			}
			
			$recordset = $this->getRecordset();
			if (!empty($selector->ProductClassAttributeId)) {
				$recordset->addFilter(new miSqlFilterIn('ProductClassAttributeID', $selector->ProductClassAttributeId));
			}
			if (!empty($selector->ProductClassId)) {
				$recordset->addFilter(new miSqlFilterIn('ProductClassID', $selector->ProductClassId));
			}
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, $recordset->getPrimaryKey(), $recordset->getPrimaryKey());
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record[$recordset->getPrimaryKey()]])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record[$recordset->getPrimaryKey()]];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		/**
		 * 
		 * @param scSoapApiProductClassAttributeOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->ProductClassAttribute->ProductClassAttributeSyncInfo))
				$operationResult->ProductClassAttributeSyncId = $operation->ProductClassAttribute->ProductClassAttributeSyncInfo->ProductClassAttributeSyncId;
		}
		
		protected function getRecordset()
		{
			$recordMlFields = array('ProductClassAttributeName', 'ProductClassAttributeDescription');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('ProductClassAttributes', 'ProductClassAttributeID', $recordMlFields, $recordLanguages);
		}
		
		protected function mapSoapAttrTypeToSc($soapAttrType)
		{
			if (!isset($this->_attributeTypesMap[$soapAttrType]))
				throw new miException(sprintf(_i('Invalid soap product class attribute type "%s"'), $soapAttrType));
			return $this->_attributeTypesMap[$soapAttrType];
		}
		
		protected function mapScToSoapAttrType($scAttrType)
		{
			$soapAttrType = array_search($scAttrType, $this->_attributeTypesMap);
			if ($soapAttrType === false) {
				return scSoapApiProductClassAttributeType::TEXT;
				//throw new miException(sprintf(_i('Invalid sc product class attribute type "%s"'), $scAttrType));
			}
			return $soapAttrType;
		}
		
		protected function mapSoapAttrValidatorToSc($soapAttrValidator)
		{
			if (!isset($this->_attributeValidatorsMap[$soapAttrValidator]))
				throw new miException(sprintf(_i('Invalid soap product class attribute validator "%s"'), $soapAttrValidator));
			return $this->_attributeValidatorsMap[$soapAttrValidator];
		}
		
		protected function mapScToSoapAttrValidator($scAttrValidator)
		{
			$soapAttrValidator = array_search($scAttrValidator, $this->_attributeValidatorsMap);
			if ($soapAttrValidator === false) {
				return scSoapApiProductClassAttributeValidator::NONE;
				//throw new miException(sprintf(_i('Invalid sc product class attribute validator "%s"'), $scAttrValidator));
			}
			return $soapAttrValidator;
		}
		
//		protected function getAttributeValues($productClassAttributeId)
//		{
//			$recordMlFields = array('ProductClassAttributeValue');
//			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
//			
//			$recordset = new scAllLangsRecordset('ProductClassAttributeValues', 'ProductClassAttributeValueID', $recordMlFields, $recordLanguages);
//			$recordset->addFilter(new miSqlFilterEqual('ProductClassAttributeID', $productClassAttributeId));
//			$records = $recordset->getAllRecords();
//			
//			$items = array();
//			foreach ($records as $record) {
//				$item = new scSoapApiProductClassAttributeValue;
//				$item->ProductClassAttributeValueId = $record['ProductClassAttributeValueID'];
//				$item->ProductClassAttributeValueText = $this->mlToXml($record['ProductClassAttributeValue']);
//				$items[] = $item;
//			}
//			return $items;
//		}
//		
//		protected function setAttributeValues($productClassAttributeId, $values)
//		{
//			if (!is_array($values))
//				$values = array($values);
//			
//			miStaticDBUtil::execDelete('ProductClassAttributeValues', 'ProductClassAttributeID', $productClassAttributeId);
//			
//			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
//			foreach ($values as $value) {
//				if (!isset($value->ProductClassAttributeValueText))
//					continue;
//				
//				$record = new scSqlRecordML('ProductClassAttributeValues', 'ProductClassAttributeValueID', array('ProductClassAttributeValue'), $recordLanguages);
//				$record->set('ProductClassAttributeID', $productClassAttributeId);
//				$record->set('ProductClassAttributeValue', $this->xmlToMl($value, 'ProductClassAttributeValueText'));
//				$record->insert();
//			}
//		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiProductClassAttribute
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiProductClassAttribute;
			$item->ProductClassAttributeId = $record['ProductClassAttributeID'];
			$item->ProductClassKey = array('ProductClassId' => $record['ProductClassID']);
			$item->ProductClassAttributeName = $this->mlToXml($record['ProductClassAttributeName']);
			$item->ProductClassAttributeType = $this->mapScToSoapAttrType($record['ProductClassAttributeType']);
			$item->ProductClassAttributeValidator = $this->mapScToSoapAttrValidator($record['ProductClassAttributeValidator']);
//			$item->ProductClassAttributeValue = $this->getAttributeValues($record['ProductClassAttributeID']);
			$item->ProductClassAttributeSort = $record['ProductClassAttributeSort'];
			return $item;
		}
		
		/**
		 * @param scProductClassAttributeDomainObject $object
		 * @param scSoapApiProductClassAttribute $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->ProductClassKey) and $isInsert)
				throw new miException(_i('ProductClassKey must be set on insert.'));
			
			if (isset($item->ProductClassKey))
				$object->set('ProductClassID', scSoapApiServiceImpl::getLocalId('ProductClass', $item->ProductClassKey));
			
			if (isset($item->ProductClassAttributeName))
				$object->set('ProductClassAttributeName', $this->xmlToMl($item, 'ProductClassAttributeName'));
			
			if (!isset($item->ProductClassAttributeType) and $isInsert)
				throw new miException(_i('ProductClassAttributeType must be set on insert.'));
			
			if (isset($item->ProductClassAttributeType))
				$object->set('ProductClassAttributeType', $this->mapSoapAttrTypeToSc($item->ProductClassAttributeType));
			
			if (!isset($item->ProductClassAttributeValidator) and $isInsert)
				$object->set('ProductClassAttributeValidator', scProductAttributes::VALIDATOR_NONE);
			if (isset($item->ProductClassAttributeValidator))
				$object->set('ProductClassAttributeValidator', $this->mapSoapAttrValidatorToSc($item->ProductClassAttributeValidator));
			
			// Handle product class attribute sort
			$additonalClause = 'ProductClassID=' . intval($object->get('ProductClassID'));
			if (isset($item->ProductClassAttributeSort)) {
				$sort = (int)$item->ProductClassAttributeSort;
				if ($isInsert or ($sort != $object->get('ProductClassAttributeSort'))) {
					$this->freeSort('ProductClassAttributes', 'ProductClassAttributeSort', $sort, $additonalClause);
					$object->set('ProductClassAttributeSort', $sort);
				}
			} elseif ($isInsert) {
				$object->set('ProductClassAttributeSort', $this->getNextSort('ProductClassAttributes', 'ProductClassAttributeSort', $additonalClause));
			}
		}
		
		protected function createDomainObject()
		{
			return new scProductClassAttributeDomainObject();
		}
	}
?>