<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiProductClassAttributeValueOperation {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiProductClassAttributeValue
		 */
		public $ProductClassAttributeValue;
	}
	
	class scSoapApiProductClassAttributeValueOperationResult {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * ProductClassAttributeValue id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $ProductClassAttributeValueId;
		
		/**
		 * Your productClassAttributeValue reference id. Will be returned only if there is sync info for this productClassAttributeValue
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiProductClassAttributeValueKey {
		/**
		 * ProductClassAttributeValue id. Read only
		 * @var int
		 */
		public $ProductClassAttributeValueId;
		
		/**
		 * Your productClassAttributeValue reference id. Must be unique
		 * @var string
		 */
		public $ProductClassAttributeValueSyncId;
	}
	
	/**
	 * ProductClassAttributeValue object
	 */
	class scSoapApiProductClassAttributeValue {
		/**
		 * ProductClassAttributeValue id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueId;
		
		/**
		 * The product class attribute to which this value belongs
		 * @var scSoapApiProductClassAttributeKey
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeKey;
		
		/**
		 * The product class attribute value
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueML;
		
		/**
		 * The product class attribute value sort order
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueSort;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueCustomData;
		
		/**
		 * ProductClassAttributeValue sync info
		 * @var scSoapApiProductClassAttributeValueSyncInfo
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueSyncInfo;
	}
	
	
	class scSoapApiProductClassAttributeValueSelector {
		
		/**
		 * @var int[]
		 */
		public $ProductClassAttributeValueId;
		
		/**
		 * @var int[]
		 */
		public $ProductClassAttributeId;
		
		/**
		 * @var int[]
		 */
		public $ProductClassId;
	}
	
	/**
	 * ProductClassAttributeValue related operations implementation
	 */
	class scSoapApiServiceProductClassAttributeValueOperations extends scSoapApiServiceImpl {
		
		/**
		 * Perform get operations
		 * @param scSoapApiProductClassAttributeSelector $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			if (empty($selector->ProductClassAttributeValueId) && empty($selector->ProductClassAttributeId) && empty($selector->ProductClassId)) {
				return array();
			}
			
			$recordset = $this->getRecordset();
			if (!empty($selector->ProductClassAttributeValueId)) {
				$recordset->addFilter(new miSqlFilterIn('ProductClassAttributeValueID', $selector->ProductClassAttributeValueId));
			}
			if (!empty($selector->ProductClassAttributeId)) {
				$recordset->addFilter(new miSqlFilterIn('ProductClassAttributeID', $selector->ProductClassAttributeId));
			}
			if (!empty($selector->ProductClassId)) {
				$recordset->addJoinCondition('INNER', 'ProductClassAttributes', 'USING(ProductClassAttributeID)');
				$recordset->addFilter(new miSqlFilterIn('ProductClassID', $selector->ProductClassId));
			}
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, $recordset->getPrimaryKey(), $recordset->getPrimaryKey());
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record[$recordset->getPrimaryKey()]])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record[$recordset->getPrimaryKey()]];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		/**
		 * 
		 * @param scSoapApiProductClassAttributeValueOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->ProductClassAttributeValue->ProductClassAttributeValueSyncInfo))
				$operationResult->ProductClassAttributeValueSyncId = $operation->ProductClassAttributeValue->ProductClassAttributeValueSyncInfo->ProductClassAttributeValueSyncId;
		}
		
		/**
		 * @return scAllLangsRecordset
		 */
		protected function getRecordset()
		{
			$recordMlFields = array('ProductClassAttributeValueName', 'ProductClassAttributeValueDescription');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('ProductClassAttributeValues', 'ProductClassAttributeValueID', $recordMlFields, $recordLanguages);
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiProductClassAttributeValue
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiProductClassAttributeValue;
			$item->ProductClassAttributeValueId = $record['ProductClassAttributeValueID'];
			$item->ProductClassAttributeKey = array('ProductClassAttributeId' => $record['ProductClassAttributeID']);
			$item->ProductClassAttributeValueML = $this->mlToXml($record['ProductClassAttributeValueML']);
			$item->ProductClassAttributeValueSort = $record['ProductClassAttributeValueSort'];
			return $item;
		}
		
		/**
		 * @param scProductClassAttributeValueDomainObject $object
		 * @param scSoapApiProductClassAttributeValue $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->ProductClassAttributeKey) and $isInsert)
				throw new miException(_i('ProductClassAttributeKey must be set on insert.'));
			
			if (isset($item->ProductClassAttributeKey))
				$object->set('ProductClassAttributeID', scSoapApiServiceImpl::getLocalId('ProductClassAttribute', $item->ProductClassAttributeKey));
			
			if (isset($item->ProductClassAttributeValueML))
				$object->set('ProductClassAttributeValueML', $this->xmlToMl($item, 'ProductClassAttributeValueML'));
			
			// Handle product class attribute value sort
			$additonalClause = 'ProductClassAttributeID=' . intval($object->get('ProductClassAttributeID'));
			if (isset($item->ProductClassAttributeValueSort)) {
				$sort = (int)$item->ProductClassAttributeValueSort;
				if ($isInsert or ($sort != $object->get('ProductClassAttributeValueSort'))) {
					$this->freeSort('ProductClassAttributeValues', 'ProductClassAttributeValueSort', $sort, $additonalClause);
					$object->set('ProductClassAttributeValueSort', $sort);
				}
			} elseif ($isInsert) {
				$object->set('ProductClassAttributeValueSort', $this->getNextSort('ProductClassAttributeValues', 'ProductClassAttributeValueSort', $additonalClause));
			}
		}
		
		protected function createDomainObject()
		{
			return new scProductClassAttributeValueDomainObject();
		}
	}
	
	class scProductClassAttributeValueDomainObject extends scDomainObject {
		
		public function __construct()
		{
			$record = new scSqlRecordML('ProductClassAttributeValues', 'ProductClassAttributeValueID', array('ProductClassAttributeValueML'), 
				array_keys(scMultiLanguage::getActiveLanguageCodes()));
			parent::__construct($record);
		}
	}
?>