<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiProductClassOperation {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiProductClass
		 */
		public $ProductClass;
	}
	
	class scSoapApiProductClassOperationResult {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * ProductClass id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $ProductClassId;
		
		/**
		 * Your productClass reference id. Will be returned only if there is sync info for this productClass
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductClassSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiProductClassKey {
		/**
		 * ProductClass id. Read only
		 * @var int
		 */
		public $ProductClassId;
		
		/**
		 * Your productClass reference id. Must be unique
		 * @var string
		 */
		public $ProductClassSyncId;
	}
	
	/**
	 * ProductClass object
	 */
	class scSoapApiProductClass {
		/**
		 * ProductClass id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductClassId;
		
		/**
		 * The productClass name
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassName;
		
		/**
		 * The productClass sort order
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductClassSort;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductClassCustomData;
		
		/**
		 * ProductClass sync info
		 * @var scSoapApiProductClassSyncInfo
		 * @xsd minOccurs 0
		 */
		public $ProductClassSyncInfo;
	}
	
	
	class scSoapApiProductClassSelector {
		/**
		 * @var int[]
		 */
		public $ProductClassId;
	}
	
	/**
	 * ProductClass related operations implementation
	 */
	class scSoapApiServiceProductClassOperations extends scSoapApiServiceImpl {
		
		/**
		 * 
		 * @param scSoapApiProductClassOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->ProductClass->ProductClassSyncInfo))
				$operationResult->ProductClassSyncId = $operation->ProductClass->ProductClassSyncInfo->ProductClassSyncId;
		}
		
		protected function getRecordset()
		{
			$recordMlFields = array('ProductClassName');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('ProductClasses', 'ProductClassID', $recordMlFields, $recordLanguages);
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiProductClass
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiProductClass;
			$item->ProductClassId = $record['ProductClassID'];
			$item->ProductClassName = $this->mlToXml($record['ProductClassName']);
//			$item->ProductClassSort = $record['ProductClassSort'];
			return $item;
		}
		
		/**
		 * @param scProductClassDomainObject $object
		 * @param scSoapApiProductClass $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (isset($item->ProductClassName))
				$object->set('ProductClassName', $this->xmlToMl($item, 'ProductClassName'));
			
			// Handle productClass sort
//			if (isset($item->ProductClassSort)) {
//				$sort = (int)$item->ProductClassSort;
//				if ($isInsert or ($sort != $object->get('ProductClassSort'))) {
//					$this->freeSort('ProductClass', 'ProductClassSort', $sort);
//					$object->set('ProductClassSort', $sort);
//				}
//			} elseif ($isInsert) {
//				$object->set('ProductClassSort', $this->getNextSort('ProductClass', 'ProductClassSort'));
//			}
		}
		
		protected function createDomainObject()
		{
			return new scProductClassDomainObject();
		}
	}
?>