<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiProductImageOperation {
		
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiProductImage
		 */
		public $ProductImage;
	}
	
	class scSoapApiProductImageOperationResult {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * ProductImage id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $ProductImageId;
		
		/**
		 * Your productImage reference id. Will be returned only if there is sync info for this productImage
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductImageSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiProductImageKey {
		/**
		 * ProductImage id. Read only
		 * @var int
		 */
		public $ProductImageId;
		
		/**
		 * Your ProductImage reference id. Must be unique
		 * @var string
		 */
		public $ProductImageSyncId;
	}
	
	/**
	 * ProductImage object
	 */
	class scSoapApiProductImage {
		
		/**
		 * ProductImage id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductImageId;
		
		/**
		 * Product to which attribute value is attached
		 * @var scSoapApiProductKey
		 * @xsd minOccurs 0
		 */
		public $ProductKey;
		
		/**
		 * @var bool
		 * @xsd minOccurs 0
		 */
		public $ProductImageIsDefault;
		
		/**
		 * 
		 * @var scSoapApiImage
		 * @xsd minOccurs 0
		 */
		public $ProductImageOriginal;
		
		/**
		 * 
		 * @var scSoapApiImage
		 * @xsd minOccurs 0
		 */
		public $ProductImageThumb;
		
		/**
		 * 
		 * @var scSoapApiImage
		 * @xsd minOccurs 0
		 */
		public $ProductImageBox;
		
		/**
		 * 
		 * @var scSoapApiImage
		 * @xsd minOccurs 0
		 */
		public $ProductImageLarge;
		
		/**
		 * 
		 * @var scSoapApiImage
		 * @xsd minOccurs 0
		 */
		public $ProductImage;
		
		/**
		 * 
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductImageSort;
		
		/**
		 * 
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductImageCustomData;
		
		/**
		 * ProductImage sync info
		 * @var scSoapApiProductImageSyncInfo
		 * @xsd minOccurs 0
		 */
		public $ProductImageSyncInfo;
	}
	
	class scSoapApiProductImageSelector {
		
		/**
		 * @var int[]
		 */
		public $ProductImageId;
	}
	
	/**
	 * ProductImage related operations implementation
	 */
	class scSoapApiServiceProductImageOperations extends scSoapApiServiceImpl {
		
		/**
		 * 
		 * @param scSoapApiProductImageOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->ProductImage->ProductImageSyncInfo))
				$operationResult->ProductImageSyncId = $operation->ProductImage->ProductImageSyncInfo->ProductImageSyncId;
		}
		
		protected function getRecordset()
		{
			return new scAllLangsRecordset('ProductImages', 'ProductImageID', array(), array());
		}
		
		/**
		 * @param array $record
		 * @return scSoapApiProductImage
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiProductImage();
			$item->ProductImageId = $record['ProductImageID'];
			$item->ProductKey = array('ProductId' => $record['ProductID']);
			$item->ProductImageSort = $record['ProductImageSort'];
			
			$item->ProductImageOriginal = new scSoapApiImage();
			$item->ProductImageOriginal->ImageUri = scPaths::getUrl($record['ProductImageOriginal']);
			
			$item->ProductImageThumb = new scSoapApiImage();
			$item->ProductImageThumb->ImageUri = scPaths::getUrl($record['ProductImageThumb']);
			
			$item->ProductImageBox = new scSoapApiImage();
			$item->ProductImageBox->ImageUri = scPaths::getUrl($record['ProductImageBox']);
			
			$item->ProductImageLarge = new scSoapApiImage();
			$item->ProductImageLarge->ImageUri = scPaths::getUrl($record['ProductImageLarge']);
			
			$item->ProductImage = new scSoapApiImage();
			$item->ProductImage->ImageUri = scPaths::getUrl($record['ProductImage']);
			
			return $item;
		}
		
		/**
		 * @param scProductImageDomainObject $object
		 * @param scSoapApiProductImage $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->ProductKey) and $isInsert)
				throw new miException(_i('ProductKey must be set on insert.'));
			if (!isset($item->ProductImageOriginal) and $isInsert)
				throw new miException(_i('ProductImageOriginal must be set on insert.'));
			if (isset($item->ProductImage))
				throw new miException(_i('Passing ProductImage is not supported. Use ProductImageOriginal instead.'));
			if (isset($item->ProductImageThumb))
				throw new miException(_i('Passing ProductImageThumb is not supported. Use ProductImageOriginal instead.'));
			if (isset($item->ProductImageBox))
				throw new miException(_i('Passing ProductImageBox is not supported. Use ProductImageOriginal instead.'));
			if (isset($item->ProductImageLarge))
				throw new miException(_i('Passing ProductImageLarge is not supported. Use ProductImageOriginal instead.'));
			
			if (isset($item->ProductImageOriginal)) {
				$imageTempPath = $this->processImage($item->ProductImageOriginal);
				$object->set('ProductImageOriginal', $imageTempPath);
			}
			
			if (isset($item->ProductKey))
				$object->set('ProductID', scSoapApiServiceImpl::getLocalId('Product', $item->ProductKey));
			if (isset($item->ProductImageIsDefault))
				$object->set('ProductImageIsDefault', (bool)$item->ProductImageIsDefault);
		
			if (isset($item->ProductImageSort)) {
				$sort = (int)$item->ProductImageSort;
				if ($isInsert or ($sort != $object->get('ProductImageSort'))) {
					$this->freeSort('ProductImages', 'ProductImageSort', $sort, 'ProductID = ' . $object->get('ProductID'));
					$object->set('ProductImageSort', $sort);
				}
			} elseif ($isInsert) {
				$object->set('ProductImageSort', $this->getNextSort('ProductImages', 'ProductImageSort'));
			}
		}
		
		/**
		 * Inserts a new ProductImage
		 * 
		 * @param scSoapApiProductImage $item
		 * @return int
		 */
		protected function doInsert($entityName, $item)
		{
			$domainObj = new scProductImageDomainObject();
			$this->setItemDataToDomainObject($domainObj, $item, true);
			
			miStaticDBUtil::startTransaction();
			try {
				$itemId = $domainObj->insert();
				
				// In order to create the correct filename an ID is needed.
				$oldPath = scPaths::getStoreRootDir() . $domainObj->get('ProductImageOriginal');
				$imagePathFile = scPaths::getItemPath($itemId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE, scPaths::PATH_FILE);
				$imagePathWeb = scPaths::getItemPath($itemId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE, scPaths::PATH_WEB);
				if (@rename($oldPath, $imagePathFile) == false) {
					throw new miException(_i('Cannot move file.'));
				}
				$domainObj->set('ProductImageOriginal', $imagePathWeb);
				$domainObj->set('ProductImageOriginalName', basename($imagePathWeb));
				$domainObj->update();
				$domainObj->regenerateSizesFromOriginal();
				
				miStaticDBUtil::commit();
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e; // Rethrow
			}
			
			if (isset($item->ProductImageSyncInfo)) {
				$this->getSyncImpl($entityName)->insertSyncInfo($itemId, $item->ProductImageSyncInfo->ProductImageSyncId, $item->ProductImageSyncInfo->HashCode);
			}
			return $itemId;
		}
		
		protected function doUpdate($entityName, $item)
		{
			if (!isset($item->ProductImageId))
				throw new miException(_i('ProductImageId is required.'));
			
			try {
				miStaticDBUtil::startTransaction();
				$domainObj = new scProductImageDomainObject();
				$domainObj->read($item->ProductImageId);
				
				$oldOriginalImage = $domainObj->get('ProductImageOriginal');
				$this->setItemDataToDomainObject($domainObj, $item, false);
				$itemId = $domainObj->update();
				
				if ($domainObj->get('ProductImageOriginal') != $oldOriginalImage) {
					$oldPath = scPaths::getStoreRootDir() . $domainObj->get('ProductImageOriginal');
					$imagePathFile = scPaths::getItemPath($itemId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE, scPaths::PATH_FILE);
					$imagePathWeb = scPaths::getItemPath($itemId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE, scPaths::PATH_WEB);
					if (@rename($oldPath, $imagePathFile) == false) {
						throw new miException(_i('Cannot move file.'));
					}
					$domainObj->set('ProductImageOriginal', $imagePathWeb);
					$domainObj->set('ProductImageOriginalName', basename($imagePathWeb));
					$domainObj->update();
					$domainObj->regenerateSizesFromOriginal();
				}
				
				miStaticDBUtil::commit();
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e; // Rethrow
			}
			
			if (isset($item->ProductImageSyncInfo)) {
				$this->getSyncImpl($entityName)->updateSyncInfo($itemId, $item->ProductImageSyncInfo->ProductImageSyncId, $item->ProductImageSyncInfo->HashCode);
			}
			return $itemId;
		}
		
		protected function createDomainObject()
		{
			return new scProductImageDomainObject();
		}
	}
?>