<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 
	 */
	class scSoapApiProductOperation {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * @var scSoapApiProduct
		 */
		public $Product;
	}
	
	class scSoapApiProductOperationResult {
		/**
		 * Operation: one of INSERT, UPDATE or DELETE
		 * 
		 * @var scSoapApiOperationType
		 */
		public $Operation;
		
		/**
		 * Operation status
		 * @var scSoapApiOperationStatusCode
		 */
		public $OperationStatusCode;
		
		/**
		 * Error message
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $OperationStatusMessage;
		
		/**
		 * Product id
		 * @xsd minOccurs 0
		 * @var int
		 */
		public $ProductId;
		
		/**
		 * Your product reference id. Will be returned only if there is sync info for this product
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductSyncId;
	}
	
	/**
	 * 
	 * @xsd order choice
	 */
	class scSoapApiProductKey {
		/**
		 * Product id. Read only
		 * @var int
		 */
		public $ProductId;
		
		/**
		 * Your product reference id. Must be unique
		 * @var string
		 */
		public $ProductSyncId;
	}
	
	
	/**
	 * Product object
	 */
	class scSoapApiProduct {
		/**
		 * Product id. Read only
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductId;
		
		/**
		 * Category
		 * @var scSoapApiCategoryKey
		 * @xsd minOccurs 0
		 */
		public $CategoryKey;
		
		/**
		 * Manufacturer
		 * @var scSoapApiManufacturerKey
		 * @xsd minOccurs 0
		 */
		public $ManufacturerKey;
		
		/**
		 * Product class key
		 * @var scSoapApiProductClassKey
		 * @xsd minOccurs 0
		 */
		public $ProductClassKey;
		
		/**
		 * Product code. Must be set on insert and must be unique
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductCode;
		
		/**
		 * Product creation timestamp
		 * @var dateTime
		 * @xsd minOccurs 0
		 */
		public $ProductCreatedTimestamp;
		
		/**
		 * The product name
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductName;
		
		/**
		 * The product description
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductDescription;
		
		/**
		 * The product detailed description
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductDetailedDescription;
		
		/**
		 * Product price
		 * @var decimal
		 * @xsd minOccurs 0
		 */
		public $ProductPrice;
		
		/**
		 * Product weight
		 * @var decimal
		 * @xsd minOccurs 0
		 */
		public $ProductWeight;
		
		/**
		 * Product quantity
		 * @var int
		 * @xsd minOccurs 0
		 */
		public $ProductQuantity;
		
		/**
		 * Flag if product is active
		 * @var boolean
		 * @xsd minOccurs 0
		 */
		public $ProductIsActive;
		
		/**
		 * The product page title. Product name will be used if page title is empty
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductPageTitle;
		
		/**
		 * The product meta keywords
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductMetaKeywords;
		
		/**
		 * The product meta keywords
		 * @var scSoapApiMultilanguageText[]
		 * @xsd minOccurs 0
		 */
		public $ProductMetaDescription;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ProductCustomData;
		
		/**
		 * Product sync info
		 * @var scSoapApiProductSyncInfo
		 * @xsd minOccurs 0
		 */
		public $ProductSyncInfo;
	}
	
	
	class scSoapApiProductSelector {
		/**
		 * @var int[]
		 */
		public $ProductId;
	}
	
	/**
	 * Product related operations implementation
	 */
	class scSoapApiServiceProductOperations extends scSoapApiServiceImpl {
		
		/**
		 * 
		 * @param scSoapApiProductOperationResult $operationResult
		 * @param object $operation
		 */
		protected function setSyncId($operationResult, $operation)
		{
			if (isset($operation->Product->ProductSyncInfo))
				$operationResult->ProductSyncId = $operation->Product->ProductSyncInfo->ProductSyncId;
		}
		
		protected function getRecordset()
		{
			$recordMlFields = array('ProductName', 'ProductDescription');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			return new scAllLangsRecordset('Products', 'ProductID', $recordMlFields, $recordLanguages);
		}
		
		/**
		 * 
		 * @param array $record
		 * @return scSoapApiProduct
		 */
		protected function getItemDataFromRecord($record)
		{
			$item = new scSoapApiProduct;
			$item->ProductId = $record['ProductID'];
			$item->CategoryKey = array('CategoryId' => $record['CategoryID']);
			$item->ManufacturerKey = array('ManufacturerId' => ($record['ManufacturerID'] == null) ? 0 : $record['ManufacturerID']);
			$item->ProductClassKey = array('ProductClassId' => ($record['ProductClassID'] == null) ? 0 : $record['ProductClassID']);
			$item->ProductCode = $record['ProductCode'];
			$item->ProductCreatedTimestamp = scDateTime::timestampToIso8601($record['ProductCreatedTimestamp']);
			$item->ProductName = $this->mlToXml($record['ProductName']);
			$item->ProductDescription = $this->mlToXml($record['ProductDescription']);
			$item->ProductDetailedDescription = $this->mlToXml($record['ProductDetailedDescription']);
			$item->ProductPrice = $record['ProductPrice'];
			$item->ProductWeight = $record['ProductWeight'];
			$item->ProductQuantity = $record['ProductQuantity'];
			$item->ProductIsActive = $record['ProductIsActive'] ? true : false;
			$item->ProductPageTitle = $this->mlToXml($record['ProductPageTitle']);
			$item->ProductMetaKeywords = $this->mlToXml($record['ProductMetaKeywords']);
			$item->ProductMetaDescription = $this->mlToXml($record['ProductMetaDescription']);
			return $item;
		}
		
		/**
		 * @param scProductDomainObject $object
		 * @param scSoapApiProduct $item
		 * @param bool $isInsert
		 */
		protected function setItemDataToDomainObject(miDomainObject $object, $item, $isInsert)
		{
			if (!isset($item->CategoryKey) and $isInsert)
				throw new miException(_i('CategoryKey must be set on insert.'));
			
			if (isset($item->CategoryKey))
				$object->set('CategoryID', scSoapApiServiceImpl::getLocalId('Category', $item->CategoryKey));
			
			$manufacturerId = isset($item->ManufacturerKey) ? scSoapApiServiceImpl::getLocalId('Manufacturer', $item->ManufacturerKey) : null;
			if ($manufacturerId == 0)
				$manufacturerId = null;
			// Set the manufacturer only if set in the SOAP item
			if (isset($item->ManufacturerKey))
				$object->set('ManufacturerID', $manufacturerId);
			
			$productClassId = isset($item->ProductClassKey) ? scSoapApiServiceImpl::getLocalId('ProductClass', $item->ProductClassKey) : null;
			if ($productClassId == 0)
				$productClassId = null;
			// Set the product class only if set in the SOAP item
			if (isset($item->ProductClassKey))
				$object->set('ProductClassID', $productClassId);
			
			if (!isset($item->ProductCode) and $isInsert)
				throw new miException(_i('ProductCode must be set on insert.'));
			
			if (isset($item->ProductCode))
				$object->set('ProductCode', $item->ProductCode);
			if (isset($item->ProductCreatedTimestamp))
				$object->set('ProductCreatedTimestamp', scDateTime::iso8601toTimestamp($item->ProductCreatedTimestamp));
			if (isset($item->ProductName))
				$object->set('ProductName', $this->xmlToMl($item, 'ProductName'));
			if (isset($item->ProductDescription))
				$object->set('ProductDescription', $this->xmlToMl($item, 'ProductDescription'));
			if (isset($item->ProductDetailedDescription))
				$object->set('ProductDetailedDescription', $this->xmlToMl($item, 'ProductDetailedDescription'));
			if (isset($item->ProductPrice))
				$object->set('ProductPrice', $item->ProductPrice);
			if (isset($item->ProductWeight))
				$object->set('ProductWeight', $item->ProductWeight);
			if (isset($item->ProductQuantity))
				$object->set('ProductQuantity', $item->ProductQuantity);
			
			if (isset($item->ProductIsActive))
				$object->set('ProductIsActive', $item->ProductIsActive ? 1 : 0);
			elseif ($isInsert)
				$object->set('ProductIsActive', 1);
			
			if (isset($item->ProductPageTitle))
				$object->set('ProductPageTitle', $this->xmlToMl($item, 'ProductPageTitle'));
			if (isset($item->ProductMetaKeywords))
				$object->set('ProductMetaKeywords', $this->xmlToMl($item, 'ProductMetaKeywords'));
			if (isset($item->ProductMetaDescription))
				$object->set('ProductMetaDescription', $this->xmlToMl($item, 'ProductMetaDescription'));
		}
		
		public function doModifyOperation($entityName, $operations, $allowPartialSuccess)
		{
			$result = parent::doModifyOperation($entityName, $operations, $allowPartialSuccess);
			scCategories::updateProductCounts();
			return $result;
		}
		
		/**
		 * @param string $entityName
		 * @param scSoapApiProduct $item
		 * @return int
		 */
		protected function doDelete($entityName, $item)
		{
			if (!scApiPrivs::hasAccess($this->_apiAccountPrivs, scApiPrivs::API_METHOD_MODIFY_DELETE_CODE)) {
				$item->ProductIsActive = false;
				return parent::doUpdate($entityName, $item);
			}
			return parent::doDelete($entityName, $item);
		}
		
		protected function createDomainObject()
		{
			return new scProductDomainObject();
		}
	}
?>