<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Recordset that retrieves records along with all multilanguage info for each record
	 */
	class scAllLangsRecordset extends miSqlRecordset {
		protected $_primaryKey;
		protected $_mlFields;
		protected $_languages = array();
		
		public function __construct($table, $primaryKey, $mlFields, $languages)
		{
			parent::__construct($table);
			
			$this->_primaryKey = $primaryKey;
			$this->_mlFields = $mlFields;
			$this->_languages = $languages;
		}
		
		public function getPrimaryKey()
		{
			return $this->_primaryKey;
		}
		
		public function getRecordsByIndex($fromIndex, $numRecords)
		{
			$records = parent::getRecordsByIndex($fromIndex, $numRecords);
			
			if (empty($records))
				return $records;
			
			if (empty($this->_mlFields))
				return $records;
			
			$recordIds = array();
			foreach ($records as $record) {
				$recordIds[] = (int)$record[$this->_primaryKey];
			}
			
			// Retrieve all multi-language fields
			$query = 'SELECT * FROM ' . $this->_table . 'ML WHERE ' . $this->_primaryKey . ' IN(' . implode(',', $recordIds) . ')';
			$mlRecords = miStaticDBUtil::execSelect($query);
			if (empty($mlRecords))
				return $records;
			
			$fields = array();
			foreach ($mlRecords[0] as $key => $value) {
				if (($key == 'LanguageID') or ($key == $this->_primaryKey))
					continue;
				$fields[] = $key;
			}
			
			foreach ($records as &$record) {
				foreach ($fields as $field) {
					$record[$field] = array();
					foreach ($mlRecords as $mlRecord) {
						if ($mlRecord[$this->_primaryKey] != $record[$this->_primaryKey])
							continue;
												
						$langId = $mlRecord['LanguageID'];
						$record[$field][$langId] = $mlRecord[$field];
					}
				}
			}
			
			return $records;
		}
	}
?>