<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSoapApiMapping {
		
		const HANDLER = 'handler';	// Unique identifier of the entity
		
		const XML_ELEMENT = 'element';	// XML element name of the entity
		const XML_LOCAL_ID = 'localIdElement';	// XML element name of the local primary key
		const XML_FOREIGN_ID = 'foreignIdElement';	// XML element name of the third-party system primary key
		
		const CLASS_SYNC_INFO = 'classSyncInfo';	// XML element name of the sync info (same as class name)
		const CLASS_SYNC_INFO_RESULT = 'classSyncInfoResult';	// XML element name of the sync info result (same as class name)
		const CLASS_OPERATIONS = 'classOperations'; // Handler (getter / setter) of the entity
		const CLASS_OPERATION_RESULT = 'classOperationResult'; // XML element name containing the result status of entity operation (same as class name)
		
		const TABLE_NAME = 'tableName';	// Database table containing the entities
		const TABLE_PK = 'tablePk';	// Primary key column name
		const PRIVS_CODE = 'privsCode';	// Code of the set scApiPrivs::API_ENTITY_*
		
		protected static $_entities = array(
			'Customer' => array(
				self::HANDLER => 'scXmlSyncCustomers',
				self::XML_ELEMENT => 'Customer',
				self::XML_LOCAL_ID => 'CustomerId',
				self::XML_FOREIGN_ID => 'CustomerSyncId',
				self::TABLE_NAME => 'Customers',
				self::TABLE_PK => 'CustomerID',
				self::CLASS_SYNC_INFO => 'scSoapApiCustomerSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiCustomerSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceCustomerOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiCustomerOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_CUSTOMER_CODE,
			),
			'CustomerAddress' => array(
				self::HANDLER => 'scXmlSyncCustomerAddresses',
				self::XML_ELEMENT => 'CustomerAddress',
				self::XML_LOCAL_ID => 'CustomerAddressId',
				self::XML_FOREIGN_ID => 'CustomerAddressSyncId',
				self::TABLE_NAME => 'CustomerAddresses',
				self::TABLE_PK => 'CustomerAddressID',
				self::CLASS_SYNC_INFO => 'scSoapApiCustomerAddressSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiCustomerAddressSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceCustomerAddressOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiCustomerAddressOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_CUSTOMER_ADDRESS_CODE,
			),
			'Category' => array(
				self::HANDLER => 'scXmlSyncCategories',
				self::XML_ELEMENT => 'Category',
				self::XML_LOCAL_ID => 'CategoryId',
				self::XML_FOREIGN_ID => 'CategorySyncId',
				self::TABLE_NAME => 'Categories',
				self::TABLE_PK => 'CategoryID',
				self::CLASS_SYNC_INFO => 'scSoapApiCategorySyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiCategorySyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceCategoryOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiCategoryOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_CATEGORY_CODE,
			),
			'Manufacturer' => array(
				self::HANDLER => 'scXmlSyncManufacturers',
				self::XML_ELEMENT => 'Manufacturer',
				self::XML_LOCAL_ID => 'ManufacturerId',
				self::XML_FOREIGN_ID => 'ManufacturerSyncId',
				self::TABLE_NAME => 'Manufacturers',
				self::TABLE_PK => 'ManufacturerID',
				self::CLASS_SYNC_INFO => 'scSoapApiManufacturerSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiManufacturerSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceManufacturerOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiManufacturerOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_MANUFACTURER_CODE,
			),
			'ProductClass' => array(
				self::HANDLER => 'scXmlSyncProductClass',
				self::XML_ELEMENT => 'ProductClass',
				self::XML_LOCAL_ID => 'ProductClassId',
				self::XML_FOREIGN_ID => 'ProductClassSyncId',
				self::TABLE_NAME => 'ProductClasses',
				self::TABLE_PK => 'ProductClassID',
				self::CLASS_SYNC_INFO => 'scSoapApiProductClassSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiProductClassSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceProductClassOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiProductClassOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_PRODUCT_CLASS_CODE,
			),
			'ProductClassAttribute' => array(
				self::HANDLER => 'scXmlSyncProductClassAttributes',
				self::XML_ELEMENT => 'ProductClassAttribute',
				self::XML_LOCAL_ID => 'ProductClassAttributeId',
				self::XML_FOREIGN_ID => 'ProductClassAttributeSyncId',
				self::TABLE_NAME => 'ProductClassAttributes',
				self::TABLE_PK => 'ProductClassAttributeID',
				self::CLASS_SYNC_INFO => 'scSoapApiProductClassAttributeSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiProductClassAttributeSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceProductClassAttributeOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiProductClassAttributeOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_PRODUCT_CLASS_ATTRIBUTE_CODE,
			),
			'ProductClassAttributeValue' => array(
				self::HANDLER => 'scXmlSyncProductClassAttributeValues',
				self::XML_ELEMENT => 'ProductClassAttributeValue',
				self::XML_LOCAL_ID => 'ProductClassAttributeValueId',
				self::XML_FOREIGN_ID => 'ProductClassAttributeValueSyncId',
				self::TABLE_NAME => 'ProductClassAttributeValues',
				self::TABLE_PK => 'ProductClassAttributeValueID',
				self::CLASS_SYNC_INFO => 'scSoapApiProductClassAttributeValueSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiProductClassAttributeValueSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceProductClassAttributeValueOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiProductClassAttributeValueOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_PRODUCT_CLASS_ATTRIBUTE_VALUE_CODE,
			),
			'Product' => array(
				self::HANDLER => 'scXmlSyncProducts',
				self::XML_ELEMENT => 'Product',
				self::XML_LOCAL_ID => 'ProductId',
				self::XML_FOREIGN_ID => 'ProductSyncId',
				self::TABLE_NAME => 'Products',
				self::TABLE_PK => 'ProductID',
				self::CLASS_SYNC_INFO => 'scSoapApiProductSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiProductSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceProductOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiProductOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_PRODUCT_CODE,
			),
			'ProductAttributeValue' => array(
				self::HANDLER => 'scXmlSyncProductAttributeValues',
				self::XML_ELEMENT => 'ProductAttributeValue',
				self::XML_LOCAL_ID => 'ProductAttributeValueId',
				self::XML_FOREIGN_ID => 'ProductAttributeValueSyncId',
				self::TABLE_NAME => 'ProductAttributeValues',
				self::TABLE_PK => 'ProductAttributeValueID',
				self::CLASS_SYNC_INFO => 'scSoapApiProductAttributeValueSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiProductAttributeValueSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceProductAttributeValueOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiProductAttributeValueOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_PRODUCT_ATTRIBUTE_VALUE_CODE,
			),
			'ProductImage' => array(
				self::HANDLER => 'scXmlSyncProductImages',
				self::XML_ELEMENT => 'ProductImage',
				self::XML_LOCAL_ID => 'ProductImageId',
				self::XML_FOREIGN_ID => 'ProductImageSyncId',
				self::TABLE_NAME => 'ProductImages',
				self::TABLE_PK => 'ProductImageID',
				self::CLASS_SYNC_INFO => 'scSoapApiProductImageSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiProductImageSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceProductImageOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiProductImageOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_PRODUCT_IMAGE_CODE,
			),
			'BundleProduct' => array(
				self::HANDLER => 'scXmlSyncBundleProducts',
				self::XML_ELEMENT => 'BundleProduct',
				self::XML_LOCAL_ID => 'BundleProductId',
				self::XML_FOREIGN_ID => 'BundleProductSyncId',
				self::TABLE_NAME => 'BundleProducts',
				self::TABLE_PK => 'BundleProductID',
				self::CLASS_SYNC_INFO => 'scSoapApiBundleProductSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiBundleProductSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceBundleProductOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiBundleProductOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_BUNDLE_PRODUCT_CODE,
			),
			'LocalOptionGroup' => array(
				self::HANDLER => 'scXmlSyncLocalOptionGroups',
				self::XML_ELEMENT => 'LocalOptionGroup',
				self::XML_LOCAL_ID => 'LocalOptionGroupId',
				self::XML_FOREIGN_ID => 'LocalOptionGroupSyncId',
				self::TABLE_NAME => 'LocalOptionGroups',
				self::TABLE_PK => 'LocalOptionGroupID',
				self::CLASS_SYNC_INFO => 'scSoapApiLocalOptionGroupSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiLocalOptionGroupSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceLocalOptionGroupOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiLocalOptionGroupOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_LOCAL_OPTIONS_CODE,
			),
			'LocalOption' => array(
				self::HANDLER => 'scXmlSyncLocalOptions',
				self::XML_ELEMENT => 'LocalOption',
				self::XML_LOCAL_ID => 'LocalOptionId',
				self::XML_FOREIGN_ID => 'LocalOptionSyncId',
				self::TABLE_NAME => 'LocalOptions',
				self::TABLE_PK => 'LocalOptionID',
				self::CLASS_SYNC_INFO => 'scSoapApiLocalOptionSyncInfo',
				self::CLASS_SYNC_INFO_RESULT => 'scSoapApiLocalOptionSyncInfoResult',
				self::CLASS_OPERATIONS => 'scSoapApiServiceLocalOptionOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiLocalOptionOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_LOCAL_OPTIONS_CODE,
			),
			'Order' => array(
				self::HANDLER => 'scXmlSyncOrders',
				self::XML_ELEMENT => 'Order',
				self::XML_LOCAL_ID => 'OrderId',
				self::XML_FOREIGN_ID => 'OrderSyncId',
				self::TABLE_NAME => 'Orders',
				self::TABLE_PK => 'OrderID',
				self::CLASS_OPERATIONS => 'scSoapApiServiceOrderOperations',
				self::CLASS_OPERATION_RESULT => 'scSoapApiOrderOperationResult',
				self::PRIVS_CODE => scApiPrivs::API_ENTITY_ORDER_CODE,
			),
		);
		
		/**
		 * @return array
		 */
		public static function getEntities()
		{
			return self::$_entities;
		}
		
		/**
		 * @param string $entityName
		 * @return array
		 */
		public static function getEntity($entityName)
		{
			if (!isset(self::$_entities[$entityName])) {
				throw new miException(sprintf(_i('Unknown entity "%s".'), $entityName));
			}
			return self::$_entities[$entityName];
		}
		
		public static function getEntityField($entityName, $field)
		{
			if (!isset(self::$_entities[$entityName])) {
				throw new miException(sprintf(_i('Unknown entity "%s".'), $entityName));
			}
			if (!isset(self::$_entities[$entityName][$field])) {
				throw new miException(sprintf(_i('Unknown entity field "%s" for entity "%s".'), $field, $entityName));
			}
			return self::$_entities[$entityName][$field];
		}
	}
?>