<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/SoapApiServiceImpl.php');
	require_once(dirname(__FILE__) . '/AllLangsRecordset.php');
	require_once(dirname(__FILE__) . '/SoapApiMapping.php');
	require_once(dirname(__FILE__) . '/../sync/SoapApiSyncImpl.php');
	require_once(dirname(__FILE__) . '/../sync/SyncOperation.php');
	require_once(dirname(__FILE__) . '/../operations/CustomerAddressOperations.php');
	require_once(dirname(__FILE__) . '/../operations/CustomerOperations.php');
	require_once(dirname(__FILE__) . '/../operations/CategoryOperations.php');
	require_once(dirname(__FILE__) . '/../operations/LocalOptionOperations.php');
	require_once(dirname(__FILE__) . '/../operations/LocalOptionGroupOperations.php');
	require_once(dirname(__FILE__) . '/../operations/ManufacturerOperations.php');
	require_once(dirname(__FILE__) . '/../operations/OrderOperations.php');
	require_once(dirname(__FILE__) . '/../operations/ProductOperations.php');
	require_once(dirname(__FILE__) . '/../operations/ProductClassOperations.php');
	require_once(dirname(__FILE__) . '/../operations/ProductClassAttributeOperations.php');
	require_once(dirname(__FILE__) . '/../operations/ProductClassAttributeValueOperations.php');
	require_once(dirname(__FILE__) . '/../operations/ProductAttributeValueOperations.php');
	require_once(dirname(__FILE__) . '/../operations/ProductImageOperations.php');
	require_once(dirname(__FILE__) . '/../operations/BundleProductOperations.php');
		
	class scSyncTransactionResultOperationType {
		
		const OP_INSERT = 1;
		const OP_UPDATE = 2;
		const OP_DELETE = 3;
		
		public static function getSoapApiOperationType($syncTransactionType)
		{
			switch ($syncTransactionType) {
				case self::OP_INSERT:
					return scSoapApiOperationType::OP_INSERT;
				case self::OP_UPDATE:
					return scSoapApiOperationType::OP_UPDATE;
				case self::OP_DELETE:
					return scSoapApiOperationType::OP_DELETE;
			}
			throw new miException(sprintf(_i('Unknown operation type "%s".'), $syncTransactionType));
		}
	}
	
	
	/****************** BASE TYPES ******************/
	
	class scSoapApiOperationType {
		const OP_INSERT = 'INSERT';
		const OP_UPDATE = 'UPDATE';
		const OP_DELETE = 'DELETE';
	}
	
	class scSoapApiOperationStatusCode {
		const SUCCESS = 'SUCCESS';
		const EXCEPTION = 'EXCEPTION';
	}
	
	/**
	 * SOAP API Image
	 */
	class scSoapApiImage {
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ImageUri;
		
		/**
		 * @var string
		 * @xsd minOccurs 0
		 */
		public $ImageEncoded;
	}
	
	/****************** MODIFY CALL REQUEST & RESPONSE ******************/
	
	/**
	 * Perform operations.
	 * An operation can be INSERT, UPDATE or DELETE.
	 * There can be any number of operations on any object
	 */
	class  scSoapApiModifyRequest {
		
		/**
		 * @var scSoapApiCustomerOperation[]
		 * @xsd minOccurs 0
		 */
		public $CustomerOperation;
		
		/**
		 * @var scSoapApiCustomerAddressOperation[]
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressOperation;
		
		/**
		 * 
		 * @var scSoapApiCategoryOperation[]
		 * @xsd minOccurs 0
		 */
		public $CategoryOperation;
		
		/**
		 * @var scSoapApiManufacturerOperation[]
		 * @xsd minOccurs 0
		 */
		public $ManufacturerOperation;
		
		/**
		 * @var scSoapApiProductClassOperation[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassOperation;
		
		/**
		 * @var scSoapApiProductClassAttributeOperation[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeOperation;
		
		/**
		 * @var scSoapApiProductClassAttributeValueOperation[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueOperation;
		
		/**
		 * @var scSoapApiProductAttributeValueOperation[]
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueOperation;
		
		/**
		 * @var scSoapApiProductOperation[]
		 * @xsd minOccurs 0
		 */
		public $ProductOperation;
		
		/**
		 * @var scSoapApiProductImageOperation[]
		 * @xsd minOccurs 0
		 */
		public $ProductImageOperation;
		
		/**
		 * 
		 * @var scSoapApiBundleProductOperation[]
		 * @xsd minOccurs 0
		 */
		public $BundleProductOperation;
		
		/**
		 * @var scSoapApiLocalOptionGroupOperation[]
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupOperation;
		
		/**
		 * @var scSoapApiLocalOptionOperation[]
		 * @xsd minOccurs 0
		 */
		public $LocalOptionOperation;
		
		/**
		 * @var scSoapApiOrderOperation[]
		 * @xsd minOccurs 0
		 */
		public $OrderOperation;
		
		/**
		 * Allow partial success.
		 * If is set to false either all operations will succeed, or if there is an error no operation will take place
		 * 
		 * @var boolean
		 */
		public $AllowPartialSuccess;
	}
	
	class scSoapApiModifyResponse {
		
		/**
		 * Results of customer operations
		 * 
		 * @var scSoapApiCustomerOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $CustomerOperationResult;
		
		/**
		 * Results of customer address operations
		 * 
		 * @var scSoapApiCustomerAddressOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressOperationResult;
		
		/**
		 * Results of category operations
		 * 
		 * @var scSoapApiCategoryOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $CategoryOperationResult;
		
		/**
		 * Results of manufacturer operations
		 * 
		 * @var scSoapApiManufacturerOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $ManufacturerOperationResult;
		
		/**
		 * Results of product class operations
		 * 
		 * @var scSoapApiProductClassOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassOperationResult;
		
		/**
		 * Results of product class attribute operations
		 * 
		 * @var scSoapApiProductClassAttributeOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeOperationResult;
		
		/**
		 * Results of product class attribute value operations
		 * 
		 * @var scSoapApiProductClassAttributeValueOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueOperationResult;
		
		/**
		 * Results of product attribute value operations
		 * 
		 * @var scSoapApiProductAttributeValueOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueOperationResult;
		
		/**
		 * Results of product operations
		 * 
		 * @var scSoapApiProductOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $ProductOperationResult;
		
		/**
		 * Results of product image operations
		 * 
		 * @var scSoapApiProductImageOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $ProductImageOperationResult;
		
		/**
		 * Results of bundle product operations
		 * 
		 * @var scSoapApiBundleProductOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $BundleProductOperationResult;
		
		/**
		 * @var scSoapApiLocalOptionGroupOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupOperationResult;
		
		/**
		 * @var scSoapApiLocalOptionOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $LocalOptionOperationResult;
		
		/**
		 * @var scSoapApiOrderOperationResult[]
		 * @xsd minOccurs 0
		 */
		public $OrderOperationResult;
	}
	
	class scSoapApiGetRequest {
		
		/**
		 * Selector which customer to be returned
		 * @var scSoapApiCustomerSelector
		 * @xsd minOccurs 0
		 */
		public $CustomerSelector;
		
		/**
		 * Selector which customer address to be returned
		 * @var scSoapApiCustomerAddressSelector
		 * @xsd minOccurs 0
		 */
		public $CustomerAddressSelector;
		
		/**
		 * Selector which categories to be returned
		 * @var scSoapApiCategorySelector
		 * @xsd minOccurs 0
		 */
		public $CategorySelector;
		
		/**
		 * Selector which manufacturers to be returned
		 * @var scSoapApiManufacturerSelector
		 * @xsd minOccurs 0
		 */
		public $ManufacturerSelector;
		
		/**
		 * Selector which product classes to be returned
		 * @var scSoapApiProductClassSelector
		 * @xsd minOccurs 0
		 */
		public $ProductClassSelector;
		
		/**
		 * Selector which product class attributes to be returned
		 * @var scSoapApiProductClassAttributeSelector
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeSelector;
		
		/**
		 * Selector which product class attribute values to be returned
		 * @var scSoapApiProductClassAttributeValueSelector
		 * @xsd minOccurs 0
		 */
		public $ProductClassAttributeValueSelector;
		
		/**
		 * Selector which product attribute values to be returned
		 * @var scSoapApiProductAttributeValueSelector
		 * @xsd minOccurs 0
		 */
		public $ProductAttributeValueSelector;
		
		/**
		 * Selector which products to be returned
		 * @var scSoapApiProductSelector
		 * @xsd minOccurs 0
		 */
		public $ProductSelector;
		
		/**
		 * Selector which product image to be returned
		 * @var scSoapApiProductImageSelector
		 * @xsd minOccurs 0
		 */
		public $ProductImageSelector;
		
		/**
		 * Selector which bundle products to be returned
		 * @var scSoapApiBundleProductSelector
		 * @xsd minOccurs 0
		 */
		public $BundleProductSelector;
		
		/**
		 * Selector which option groups to be returned
		 * @var scSoapApiLocalOptionGroupSelector
		 * @xsd minOccurs 0
		 */
		public $LocalOptionGroupSelector;
		
		/**
		 * Selector which option groups to be returned
		 * @var scSoapApiLocalOptionSelector
		 * @xsd minOccurs 0
		 */
		public $LocalOptionSelector;
		
		/**
		 * Selector which orders to be returned
		 * @var scSoapApiOrderSelector
		 * @xsd minOccurs 0
		 */
		public $OrderSelector;
	}
	
	class scSoapApiGetResponse {
		
		/**
		 * @var scSoapApiCustomer[]
		 */
		public $Customer;
		
		/**
		 * @var scSoapApiCustomerAddress[]
		 */
		public $CustomerAddress;
		
		/**
		 * @var scSoapApiCategory[]
		 */
		public $Category;
		
		/**
		 * @var scSoapApiManufacturer[]
		 */
		public $Manufacturer;
		
		/**
		 * @var scSoapApiProductClass[]
		 */
		public $ProductClass;
		
		/**
		 * @var scSoapApiProductClassAttribute[]
		 */
		public $ProductClassAttribute;
		
		/**
		 * @var scSoapApiProductClassAttributeValue[]
		 */
		public $ProductClassAttributeValue;
		
		/**
		 * @var scSoapApiProductAttributeValue[]
		 */
		public $ProductAttributeValue;
		
		/**
		 * @var scSoapApiProduct[]
		 */
		public $Product;
		
		/**
		 * @var scSoapApiProductImage[]
		 */
		public $ProductImage;
		
		/**
		 * @var scSoapApiBundleProduct[]
		 */
		public $BundleProduct;
		
		/**
		 * @var scSoapApiLocalOptionGroup[]
		 */
		public $LocalOptionGroup;
		
		/**
		 * @var scSoapApiLocalOption[]
		 */
		public $LocalOption;
		
		/**
		 * @var scSoapApiOrder[]
		 */
		public $Order;
	}
	
	class scSoapApiGetOperationsLogRequest {
		
		/**
		 * @var bool
		 */
		public $GetOrderOperations;
	}
	
	class scSoapApiGetOperationsLogResponse {
		
		/**
		 * @var scSoapApiOperationsLogOrder[]
		 */
		public $Order;
	}
	
	class scSoapApiMarkOperationsLogSyncedRequest {
		
		/**
		 * @var scSoapApiOperationsLogSyncedOrder[]
		 */
		public $Order;
	}
	
	class scSoapApiMarkOperationsLogSyncedResponse {
		
	}
	
	/**
	 * SOAP Request header
	 */
	class scSoapApiRequestHeader {
		
		/**
		 * Username used to authenticate the client to the web service
		 * @var string
		 */
		public $Username;
		
		/**
		 * Password assigned to the client
		 * @var string
		 */
		public $Password;
	}
	
	/**
	 * SOAP Response header
	 * Currently has no values
	 */
	class scSoapApiResponseHeader {
	}
	
	/**
	 * Base Summer Cart Soap Api exception
	 */
	class scSoapApiException extends SoapFault {
		
		const ERROR_CODE_UNAUTHORISED = 1;
		const ERROR_CODE_NOT_ENOUGH_PERMISSIONS = 2;
		
		public function __construct($faultString, $errorCode)
		{
			$this->ErrorCode = $errorCode;
			parent::__construct('Client', $faultString, null, $this);
		}
		
		/**
		 * 
		 * @var string
		 */
		public $ErrorCode;
	}
	
//	class scSoapApiAuthException {
//	}
	
//	class scSoapApiError {
//		/**
//		 * 
//		 * @var int
//		 */
//		public $Type;
//	}
	
//	class scSoapApiErrorCategoryNotFound {
//		/**
//		 * Category that was not found
//		 * @var int
//		 */
//		public $CategoryId;
//	}
	
	/**
	 * Summer Cart SOAP service
	 */
	class scSoapApiService {
		
		const TRANSACTION_ACTIVITY_HOURS = 4;
		
		protected $_isAuth = false;
		protected $_apiAccountPrivs = array();
		protected $_apiAccountId = null;
		
		/**
		 * @param scSoapApiRequestHeader $requestHeader
		 * @xsd skip
		 */
		public function scSoapApiRequestHeader($requestHeader)
		{
			$sql = 'SELECT * FROM ApiAccounts WHERE ApiAccountUsername=? AND ApiAccountPassword=?';
			$result = miStaticDBUtil::execSelect($sql, array(strval($requestHeader->Username), strval($requestHeader->Password)));
			if (count($result) > 0) {
				$this->_apiAccountId = $result[0]['ApiAccountID'];
				$this->_isAuth = true;
				$this->_apiAccountPrivs = miUtil::getDBArray('SELECT ApiAccountPrivEntityCode, ApiAccountPrivOperations FROM ApiAccountsPrivs WHERE ApiAccountID = ' . $result[0]['ApiAccountID'], 'ApiAccountPrivEntityCode', 'ApiAccountPrivOperations');
			}
		}
		
		/**
		 * @param scSoapApiStartSyncTransactionRequest $request
		 * @return scSoapApiStartSyncTransactionResponse
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function startSyncTransaction($request)
		{
			$this->checkAuth();
			
			$this->deleteOldTransactions();
			
			$response = new scSoapApiStartSyncTransactionResponse;
			$response->SyncTransactionId = $this->getNewSyncTransactionId();
			return $response;
		}
		
		/**
		 * @param scSoapApiAddSyncInfoRequest $request
		 * @return scSoapApiAddSyncInfoResponse
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function addSyncInfo($request)
		{
			$this->checkAuth();
			
			$transactionId = (int)$request->SyncTransactionId;
			if ($transactionId < 1) {
				throw new miException(_i('Invalid SyncTransactionId.'));
			}
			
			$response = new scSoapApiAddSyncInfoResponse();
			
			$transactionInfoArrays = array();
			foreach (scSoapApiMapping::getEntities() as $entityName => $entity) {
				$entityElement = $entityName . 'SyncInfo';
				if (!isset($request->{$entityElement})) {
					continue;
				}
				
				$sync = new scSoapApiSyncImpl($entityName);
				$this->checkPriv($entity[scSoapApiMapping::PRIVS_CODE], scApiPrivs::API_METHOD_GET_SYNC_CODE);
				$syncInfos = $sync->createSyncTransactionInfos($request->{$entityElement});
				$this->insertTransactionSyncInfosToDb($transactionId, $syncInfos);
				$this->markTransactionActivity($transactionId);
				unset($syncInfos, $sync);
			}
			
			return $response;
		}
		
		/**
		 * @param scSoapApiPerformSyncTransactionRequest $request
		 * @return scSoapApiPerformSyncTransactionResponse
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function performSyncTransaction($request)
		{
			$this->checkAuth();
			
			$transactionId = (int)$request->SyncTransactionId;
			if ($transactionId < 1) {
				throw new miException(_i('Invalid SyncTransactionId.'));
			}
			
			$response = new scSoapApiPerformSyncTransactionResponse;
			
			$recordset = new miSqlRecordset('SyncTransactionItems');
			$recordset->addFilter(new miSqlFilterEqual('SyncTransactionID', $transactionId));
			$recordset->setGroupBy('SyncTransactionItemHandler');
			$recordset = $recordset->getAllRecords();
			$fetchEntities = array();
			foreach ($recordset as $row) {
				$fetchEntities[$row['SyncTransactionItemHandler']] = '"' . sql_escape_string($row['SyncTransactionItemHandler']) . '"';
			}
			
			$entities = scSoapApiMapping::getEntities();
			foreach ($entities as $entityName => $entity) {
				// Check if the entity is in the FETCH list.
				if (!isset($fetchEntities[$entity[scSoapApiMapping::HANDLER]])) {
					continue;
				}
				// Check if the entity has a CLASS_SYNC_INFO defined to check if it supports sync.
				if (!isset($entity[scSoapApiMapping::CLASS_SYNC_INFO])) {
					continue;
				}
				
				$sync = new scSoapApiSyncImpl($entityName);
				$sync->cleanupSyncInfo();
			}
			
			if (!empty($fetchEntities)) {
				// Generate DELETE operations
				$sql = 'INSERT INTO SyncTransactionResults (SyncTransactionResultOperation, SyncTransactionID, SyncTransactionResultHandler, SyncTransactionResultLocalID)
					(
						SELECT ?, ?, SyncItemHandler, SyncItemLocalID
						FROM SyncItems
						LEFT JOIN SyncTransactionItems ON
							SyncTransactionItemHandler = SyncItemHandler
							AND SyncTransactionItemForeignID = SyncItemForeignID
							AND SyncTransactionID = ?
						WHERE SyncTransactionItemID IS NULL
						AND SyncItemHandler IN (' . implode(', ', $fetchEntities) . ')
					)';
				miStaticDBUtil::execSQL($sql, array(scSyncTransactionResultOperationType::OP_DELETE, $transactionId, $transactionId));
			}
			
			if (isset($request->ReturnNonSynced) && (bool)$request->ReturnNonSynced) {
				$entities = scSoapApiMapping::getEntities();
				foreach ($entities as $entity) {
					if (!isset($fetchEntities[$entity[scSoapApiMapping::HANDLER]])) {
						continue;
					}
					
					// Prepare the SELECT query using recordset and don't bother with escaping.
					$recordset = new miSqlRecordset('SyncItems');
					$recordset->setSelectFields(array('SyncItemLocalID'));
					$recordset->addFilter(new miSqlFilterEqual('SyncItemHandler', $entity[scSoapApiMapping::HANDLER]));
					
					$localIdsQuery = $recordset->getSelectQuery();
					$filterQuery = sql_escape_string($entity[scSoapApiMapping::TABLE_PK]) . ' NOT IN (' . $localIdsQuery . ')';
					
					$recordset = new miSqlRecordset($entity[scSoapApiMapping::TABLE_NAME]);
					$recordset->setSelectFields(array(
						scSyncTransactionResultOperationType::OP_DELETE,
						$transactionId,
						'"' . $entity[scSoapApiMapping::HANDLER] . '"',
						$entity[scSoapApiMapping::TABLE_PK],
					));
					$recordset->addFilter(new miSqlFilterCustom($entity[scSoapApiMapping::TABLE_PK], $filterQuery));
					
					// INSERT the entities in the RESULTS table.
					$insertQuery = 'INSERT INTO SyncTransactionResults (SyncTransactionResultOperation, SyncTransactionID, SyncTransactionResultHandler, SyncTransactionResultLocalID)' .
						' (' . $recordset->getSelectQuery() . ')';
					miStaticDBUtil::execSQL($insertQuery);
				}
			}
			
			// Generate UPDATE operations
			$sql = 'INSERT INTO SyncTransactionResults (SyncTransactionResultOperation, SyncTransactionID, SyncTransactionResultHandler, SyncTransactionResultLocalID, SyncTransactionResultForeignID, SyncTransactionResultHashCode)
				(
					SELECT ?, ?, SyncItemHandler, SyncItemLocalID, SyncItemForeignID, SyncItemHashCode
					FROM SyncItems
					INNER JOIN SyncTransactionItems ON
						SyncTransactionItemHandler = SyncItemHandler
						AND SyncTransactionItemForeignID = SyncItemForeignID
					WHERE SyncTransactionID = ? AND SyncTransactionItemHashCode <> SyncItemHashCode
				)';
			miStaticDBUtil::execSQL($sql, array(scSyncTransactionResultOperationType::OP_UPDATE, $transactionId, $transactionId));
			
			// Generate INSERT operations
			$sql = 'INSERT INTO SyncTransactionResults (SyncTransactionResultOperation, SyncTransactionID, SyncTransactionResultHandler, SyncTransactionResultForeignID, SyncTransactionResultHashCode)
				(
					SELECT ?, ?, SyncTransactionItemHandler, SyncTransactionItemForeignID, SyncTransactionItemHashCode
					FROM SyncTransactionItems
					LEFT JOIN SyncItems ON
						SyncTransactionItemHandler = SyncItemHandler
						AND SyncTransactionItemForeignID = SyncItemForeignID
					WHERE SyncItemID IS NULL AND SyncTransactionID = ?
				)';
			miStaticDBUtil::execSQL($sql, array(scSyncTransactionResultOperationType::OP_INSERT, $transactionId, $transactionId));
			
			$sql = 'SELECT COUNT(1) AS N FROM SyncTransactionResults WHERE SyncTransactionID = ?';
			$result = miStaticDBUtil::execSelect($sql, array($transactionId));
			$response->OperationsCount = $result[0]['N'];
			return $response;
		}
		
		/**
		 * @param scSoapApiGetSyncTransactionResultsRequest $request
		 * @return scSoapApiGetSyncTransactionResultsResponse
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function getSyncTransactionResults($request)
		{
			$response = new scSoapApiGetSyncTransactionResultsResponse();
			
			$transactionId = (int)$request->SyncTransactionId;
			if ($transactionId < 1) {
				throw new miException(_i('Invalid SyncTransactionId.'));
			}
			$pageNum = (int)$request->PageNumber;
			if ($pageNum < 1) {
				throw new miException(_i('Invalid PageNumber.'));
			}
			$itemsPerPage = (int)$request->OperationsPerPage;
			if ($itemsPerPage < 1) {
				throw new miException(_i('Invalid OperationsPerPage.'));
			}
			$offset = (($pageNum - 1) * $itemsPerPage);
			
			
			$results = new miSqlRecordset('SyncTransactionResults');
			$results->addFilter(new miSqlFilterEqual('SyncTransactionID', $transactionId));
			$results->setOrder('SyncTransactionResultID');
			$results = $results->getRecordsByIndex($offset, $itemsPerPage);
			
			// Group the results by handler
			$byHandlerResults = array();
			foreach ($results as $result) {
				$byHandlerResults[$result['SyncTransactionResultHandler']][] = $result;
			}
			unset($results);
			
			$handlerToEntity = $this->getHandlerToEntityMapping();
			foreach ($byHandlerResults as $handler => $results) {
				$entityName = $handlerToEntity[$handler];
				
				// Check if entity has SyncInfoResult object
				$entity = scSoapApiMapping::getEntity($entityName);
				if (!isset($entity[scSoapApiMapping::CLASS_SYNC_INFO_RESULT])) {
					continue;
				}
				
				$sync = new scSoapApiSyncImpl($entityName);
				$resultElement = $entityName . 'SyncInfoResult';
				$response->{$resultElement} = $sync->createSyncInfoResults($results);
			}
			unset($byHandlerResults);
			
			return $response;
		}
		
		/**
		 * Obtain syncronization info
		 * 
		 * @param scSoapApiSyncRequest $request
		 * @return scSoapApiSyncResponse
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function getSync($request)
		{
			$this->checkAuth();
			
			$response = new scSoapApiSyncResponse;
			
			$returnNonSynced = (bool)$request->ReturnNonSynced;
			foreach (scSoapApiMapping::getEntities() as $entityName => $entity) {
				$requestElement = $entityName . 'SyncInfo';
				$resultElement = $entityName . 'SyncInfoResult';
				if (!isset($request->{$requestElement})) {
					continue;
				}
				$sync = new scSoapApiSyncImpl($entityName);
				$this->checkPriv($entity[scSoapApiMapping::PRIVS_CODE], scApiPrivs::API_METHOD_GET_SYNC_CODE);
				$response->{$resultElement} = $sync->doGetSyncOperation($request->{$requestElement}, $returnNonSynced);
			}
			
			return $response;
		}
		
		/**
		 * Get information from the store
		 * @param scSoapApiGetRequest $request
		 * @return scSoapApiGetResponse
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function get($request)
		{
			$this->checkAuth();
			
			$response = new scSoapApiGetResponse;
			
			foreach (scSoapApiMapping::getEntities() as $entityName => $entity) {
				$handler = new $entity[scSoapApiMapping::CLASS_OPERATIONS];
				$requestElement = $entityName . 'Selector';
				$resultElement = $entityName;
				if (!isset($request->{$requestElement})) {
					continue;
				}
				$this->checkPriv($entity[scSoapApiMapping::PRIVS_CODE], scApiPrivs::API_METHOD_GET_CODE);
				$response->{$resultElement} = $handler->doGetOperation($entityName, $request->{$requestElement});
				unset($handler);
			}
			
			return $response;
		}
		
		/**
		 * Get items that have (probably) been modified since last call
		 * 
		 * @param scSoapApiGetOperationsLogRequest $request
		 * @return scSoapApiGetOperationsLogResponse 
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function getOperationsLog($request)
		{
			$this->checkAuth();
			
			$response = new scSoapApiGetOperationsLogResponse();
			
			if (isset($request->GetOrderOperations) && $request->GetOrderOperations) {
				$this->checkPriv(scApiPrivs::API_ENTITY_ORDER_CODE, scApiPrivs::API_METHOD_GET_OPERATIONS_LOG_CODE);
				$orderServiceImpl = new scSoapApiServiceOrderOperations;
				$response->Order = $orderServiceImpl->doGetOperationsLog();
			}
			
			return $response;
		}
		
		/**
		 * Mark specified items synced to point
		 * 
		 * @param scSoapApiMarkOperationsLogSyncedRequest $request
		 * @return scSoapApiMarkOperationsLogSyncedResponse 
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function markOperationsLogSynced($request)
		{
			$this->checkAuth();
			
			$response = new scSoapApiMarkOperationsLogSyncedResponse();
			
			if (isset($request->Order) && !empty($request->Order)) {
				$this->checkPriv(scApiPrivs::API_ENTITY_ORDER_CODE, scApiPrivs::API_METHOD_MARK_OPERATIONS_LOG_SYNCED_CODE);
				$orderServiceImpl = new scSoapApiServiceOrderOperations;
				$orderServiceImpl->doMarkOperationsLogSynced($request->Order);
				// TODO: May say if there are newer sync operations or not.
			}
			
			return $response;
		}
		
		/**
		 * Modify the store
		 * 
		 * @param scSoapApiModifyRequest $request
		 * @return scSoapApiModifyResponse 
		 * @xsd fault scSoapApiException
		 * @xsd soaprequestheader scSoapApiRequestHeader
		 * @xsd soapresponseheader scSoapApiResponseHeader
		 */
		public function modify($request)
		{
			$this->checkAuth();
			$allowPartialSuccess = $request->AllowPartialSuccess;
			
			$response = new scSoapApiModifyResponse;
			
			foreach (scSoapApiMapping::getEntities() as $entityName => $entity) {
				$handler = new $entity[scSoapApiMapping::CLASS_OPERATIONS];
				$requestElement = $entityName . 'Operation';
				$resultElement = $entityName . 'OperationResult';
				if (!isset($request->{$requestElement})) {
					continue;
				}
				$this->checkPriv($entity[scSoapApiMapping::PRIVS_CODE], scApiPrivs::API_METHOD_MODIFY_CODE);
				$handler->setApiAccountPrivs($this->_apiAccountPrivs[$entity[scSoapApiMapping::PRIVS_CODE]]);
				$response->{$resultElement} = $handler->doModifyOperation($entityName, $request->{$requestElement}, $allowPartialSuccess);
				unset($handler);
			}
			
			return $response;
		}
		
		protected function checkAuth()
		{
			if (!$this->_isAuth)
				throw new scSoapApiException(_i('Authorization failed.'), scSoapApiException::ERROR_CODE_UNAUTHORISED);
		}
		
		protected function checkPriv($entityCode, $priv)
		{
			if (!isset($this->_apiAccountPrivs[$entityCode]) || !scApiPrivs::hasAccess($this->_apiAccountPrivs[$entityCode], $priv)) {
				$msg = sprintf(_i('You do not have permissions to perform "%s" operation on "%s".'), scApiPrivs::getMethodNameByCode($priv), scApiPrivs::getEntityNameByCode($entityCode));
				throw new scSoapApiException($msg, scSoapApiException::ERROR_CODE_NOT_ENOUGH_PERMISSIONS);
			}
		}
		
		/**
		 * @return int
		 */
		protected function getNewSyncTransactionId()
		{
			$syncTransaction = new miSqlRecord('SyncTransactions', 'SyncTransactionID');
			$syncTransaction->set('ApiAccountID', $this->_apiAccountId);
			$syncTransaction->set('SyncTransactionLastActivity', time());
			return $syncTransaction->insert();
		}
		
		/**
		 * @param int $transactionId
		 * @return int
		 */
		protected function markTransactionActivity($transactionId)
		{
			miStaticDBUtil::execUpdate('SyncTransactions', array('SyncTransactionLastActivity' => time()), 'SyncTransactionID', $transactionId);
		}
		
		protected function deleteOldTransactions()
		{
			$activitySeconds = self::TRANSACTION_ACTIVITY_HOURS * 60 * 60;
			miStaticDBUtil::execSQL('DELETE FROM SyncTransactions WHERE SyncTransactionLastActivity < ?', array(time() - $activitySeconds));
		}
		
		/**
		 * @param scSyncTransactionInfo[] $syncInfos
		 */
		protected function insertTransactionSyncInfosToDb($transactionId, $syncInfos)
		{
			$multiValues = array();
			foreach ($syncInfos as $syncInfo) {
				if (!($syncInfo instanceof scSyncTransactionInfo)) {
					throw new miException(_i('scSyncTransactionInfo expected.'));
				}
				$syncInfo->set('SyncTransactionID', $transactionId);
				
				$multiValues[] = $syncInfo->getRow();
			}
			$columns = array_keys($multiValues[0]);
			miStaticDBUtil::multiValueListsInsert('SyncTransactionItems', $columns, $multiValues, scSyncTransactionInfo::COLUMN_HASH);
		}
		
		protected function getHandlerToEntityMapping()
		{
			$entities = scSoapApiMapping::getEntities();
			$mapping = array();
			foreach ($entities as $entityName => $entity) {
				$mapping[$entity[scSoapApiMapping::HANDLER]] = $entityName;
			}
			return $mapping;
		}
	}
?>