<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scCartReferer {
		public static function getReferer()
		{
			if (isset($_SERVER['HTTP_REFERER'])) {
				$isCart = scUrlResolver::singleton()->is($_SERVER['HTTP_REFERER'], scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART);
				$path = parse_url($_SERVER['HTTP_REFERER'], PHP_URL_PATH);
				$isCheckout = !substr_compare($path, 'checkout.php', -12);
				if (!$isCart && !$isCheckout)
					$_SESSION['CartReferrer'] = $_SERVER['HTTP_REFERER'];
			}
			
			if (isset($_SESSION['CartReferrer']))
				return $_SESSION['CartReferrer'];
			
			return '';
		}
	}
	
	class scCartView extends miDefaultView implements ICMSBlock {
		public function __construct()
		{
			parent::__construct(null);
		}
		
		protected $_templates = array(
			self::TEMPLATE_ID_VIEW => 'customer/cart.tmpl'
		);
		
		public function addRedirectToViewCartControllerCommand($message = '')
		{
			$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART);
			if ($message != '')
				$url = scUrlUtil::addQueryParams($url, 'msg=' . urlencode($message));
			$url = scUrlUtil::convertUrl($url, scUrlUtil::URL_TYPE_DOCROOT);
			$this->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
		}
		
		public function addRedirectToCheckoutControllerCommand($moduleName)
		{
			$url = scPaths::getSecureUrl('checkout.php?ModuleName=' . urlencode($moduleName));			
			$this->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
		}
	}
	
	/**
	 * Shows the contents of the shopping cart
	 */
	class scShowCartAction extends miAction {
		public function doAction()
		{
			try {
				$this->viewCart();
			} catch (Exception $exception) {
				header('Location: ' . scPaths::getUrl('error.php') . '?msg=' . urlencode($exception->getMessage()));
			}
		}
		
		protected function viewCart()
		{
			$uiCart = new scUICart();
			
			$t = new scTemplateParser();
			$t->readTemplate($this->_view->getTemplateFileName(scCartView::TEMPLATE_ID_VIEW));
			$t->assign('%%REFERRER%%', htmlspecialchars(scCartReferer::getReferer(), ENT_QUOTES));
			$t->setSectionInfos(array($uiCart->getCartTemplateSection(), $uiCart->getCheckoutModulesSection(), $uiCart->getCouponsTemplateSection()));
			$t->addSectionInfo(new miTemplateParserSectionInfo('cartHasPerItemDiscount', scCart::getCart()->hasPerItemDiscount()));
			
			$msg = miGetParamDefault('msg', '');
			$t->assign('%%MSG%%', miI18N::htmlEscape($msg));
			$t->assign('%%MSG_STYLE%%', $msg ? 'error' : 'hidden');
			
			$t->addSectionInfo($uiCart->getOrderTotalLinesTemplateSection());
			$t->assign('%%GRANDTOTAL%%', scPrice::factory(scCart::getCart()->getOrderTotal()->getGrandTotalAmount())->getDisplayPrice());
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('cart.php'));		
			
			$html = $t->templateParse();
			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;
		}
	}

	/**
	 * Adds product to the shopping cart 
	 */
	class scAddToCartAction extends miAction {
		public function doAction()
		{
			try {
				// Create the cart item object
				$productId = miGetParam('ProductID');
				$quantity = miGetParam('Quantity');
				$options = scCartItemService::singleton()->getOptionsParam('Options');
				$variantId = miGetParamDefault('ProductVariantID', 0);
				
				$item = scCartItemService::singleton()->createCartItemProduct($productId, $quantity, $options, $variantId);
				
				// Add the item / update existing item in the cart
				scCart::getCart()->addItem($item);
				
				$this->_view->addRedirectToViewCartControllerCommand();
				return true;
			}
			catch (miException $exception)
			{
				$this->_view->addRedirectToViewCartControllerCommand($exception->getMessage());
				return false;
			}
		}
	}
	
	class scRemoveItemAction extends miAction {
		public function doAction()
		{
			try {
				scCart::getCart()->removeItem(miGetParam('CartItemCode'));
				
				$this->_view->addRedirectToViewCartControllerCommand();
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToViewCartControllerCommand($exception->getMessage());
				return false;
			}
		}
	}
	
	/**
	 * Updates the cart contents
	 */
	class scUpdateCartAction extends miAction {
		public function doAction()
		{
			try {
				$this->updateCart();
				
				$this->_view->addRedirectToViewCartControllerCommand();
				return true;
			} catch (miException $exception)	{
				$this->_view->addRedirectToViewCartControllerCommand($exception->getMessage());
				return false;
			}
		}
		
		public function updateCart()
		{
			$cartItemQuantity = miGetParamDefault('CartItemQuantity', array());
			
			$cart = scCart::getCart();
			foreach ($cartItemQuantity as $cartItemCode => $quantity) {
				if (!is_int($quantity) && !ctype_digit($quantity))
					continue;
				
				$quantity = intval($quantity);
				if ($quantity < 0)
					continue;
				
				if ($quantity == 0)
					$cart->removeItem($cartItemCode);
				
				$item = $cart->getItem($cartItemCode);
				if ($item == null)
					continue;
				
				$item->setQuantity($quantity);
			}
		}
	}
	
	/**
	 * Removes all items from the cart
	 */
	class scEmptyCartAction extends miAction {
		public function doAction()
		{
			try {
				scCart::getCart()->removeAllItems();
				scCart::getCart()->removeCoupon();
				
				$this->_view->addRedirectToViewCartControllerCommand();
				return true;
			} catch (miException $exception)	{
				$this->_view->addRedirectToViewCartControllerCommand($exception->getMessage());
				return false;
			}
		}
	}
	
	/**
	 * Sends user to the checkout wizard
	 */
	class scCheckoutAction extends scUpdateCartAction {
		public function doAction()
		{
			try {
				$this->updateCart();
				$this->_view->addRedirectToCheckoutControllerCommand(urlencode(miGetParamDefault('ModuleName', '')));
				
				return true;
			} catch (miException $exception)	{
				$this->_view->addRedirectToViewCartControllerCommand($exception->getMessage());
				return false;
			}
		}
	}
	
	/**
	 * Redirects the user to the last visited page (before entering the cart page)
	 */
	class scContinueShoppingAction extends scUpdateCartAction {
		public function doAction()
		{
			try {
				$this->updateCart();
				
				$referer = scCartReferer::getReferer();
				if ($referer == '')
					$referer = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_INDEX);
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $referer));
				return true;
			} catch (miException $exception)	{
				$this->_view->addRedirectToViewCartControllerCommand($exception->getMessage());
				return false;
			}
		}
	}
	
	/**
	 * Applies a coupon to the product
	 */
	class scApplyCouponAction extends miAction {
		public function doAction()
		{
			try {
				$couponCode = miGetParam('CouponCode');
				scCart::getCart()->applyCoupon($couponCode);
				scCart::getCart()->incHashSeed();
				$this->_view->addRedirectToViewCartControllerCommand(_t('Coupon applied.'));
				return true;
			}
			catch (miException $exception)
			{
				$this->_view->addRedirectToViewCartControllerCommand($exception->getMessage());
				return false;
			}
		}
	}
	
	class scRemoveCouponAction extends miAction {
		public function doAction()
		{
			try {
				scCart::getCart()->removeCoupon();
				scCart::getCart()->incHashSeed();
				
				$this->_view->addRedirectToViewCartControllerCommand(_t('Coupon removed.'));
				return true;
			}
			catch (miException $exception)
			{
				$this->_view->addRedirectToViewCartControllerCommand($exception->getMessage());
				return false;
			}
		}
	}
	
	$userPageCart = new scCartView();
	
	$page = new scUserDisplayController($userPageCart);
	$page->setSeo(new scDynamicPageSeo('cart.php', _t('View Cart')));
	// Specify the cart actions
	$dispatcher = $page->getDispatcher(); 
	$dispatcher->clearActionHandlers();
	$dispatcher->setActionHandler('', 'scShowCartAction');
	$dispatcher->setActionHandler('dmAddToCart', 'scAddToCartAction');
	$dispatcher->setActionHandler('dmRemoveItem', 'scRemoveItemAction');
	$dispatcher->setActionHandler('dmUpdateCart', 'scUpdateCartAction');
	$dispatcher->setActionHandler('dmEmptyCart', 'scEmptyCartAction');
	$dispatcher->setActionHandler('dmCheckout', 'scCheckoutAction');
	$dispatcher->setActionHandler('dmContinueShopping', 'scContinueShoppingAction');
	$dispatcher->setActionHandler('dmApplyCoupon', 'scApplyCouponAction');
	$dispatcher->setActionHandler('dmRemoveCoupon', 'scRemoveCouponAction');
	
	$page->dispatchAndShow();
?>