<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
		
	class scUserPageCategories extends scView {
		const TEMPLATE_ID_VIEW_GRID = 'grid_view';
		
		protected $_templates = array(
			self::TEMPLATE_ID_VIEW => 'customer/categories/categories.tmpl',
			self::TEMPLATE_ID_VIEW_GRID => 'customer/categories/categories_grid.tmpl'
		);
		
		public function __construct()
		{
			parent::__construct(null);
		}
	}
	
	class scViewCategoryAction extends scProductsListingAction {
		public function doAction()
		{
			try {
				$html = $this->getCategoryHtml();
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			} catch (miException $exception) {
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($exception->getMessage());
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
			}
			
			return true;
		}
		
		protected function getCategoryHtml()
		{
			$categoryId = (int) miGetParamDefault('CategoryID', 0);
			
			// Init category row
			$categoryRow = $this->getCategoryRow($categoryId);
			
			// Get the view type - list/grid
			$view = scProductService::getViewType();
			if ($view == scProductService::VIEW_GRID) {
				$altView = scProductService::VIEW_LIST;
				$altViewString = 'list';
				$templateId = scUserPageCategories::TEMPLATE_ID_VIEW_GRID;
			} else {
				$altView = scProductService::VIEW_GRID;
				$altViewString = 'grid';
				$templateId = scUserPageCategories::TEMPLATE_ID_VIEW;
			}
			
			// Get featured products
			$featuredProductsSection = new miTemplateParserSectionInfo('featuredProducts', 1);
			$featuredProductsSection->addSubsection(scProductService::getProductSections('products', $this->getFeaturedProducts($categoryId), $view));
			
			// The main products section
			$productsSection = scProductService::getProductSections('products', $this->getProducts($categoryId), $view);
			
			// Breadcrumbs section
			$categoriesBreadcrumbSection = scUIUtil::getCategoriesBreadcrumb($categoryId);
			$subCategoriesSection = $this->getSubcategoriesSection($categoryRow);
			
			// Recalculate products count if filters have been applied
			$categoryRow['CategoryProductCount'] = $this->getProductsCount($categoryId, $categoryRow['CategoryProductCount']);
			
			// Parse the page
			$t = new scTemplateParser();
			$t->assignArray(miUIUtil::templetizeAndEscapeRows($categoryRow));
			$t->setSectionInfos(array($subCategoriesSection, $categoriesBreadcrumbSection, $productsSection, $featuredProductsSection));
			$t->readTemplate($this->_view->getTemplateFileName($templateId));
			
			$selfUrl = $this->getSelfUrl($categoryId, $categoryRow['CategoryName']);
			list($page, $numPages) = scCustomerAreaPager::calculatePages('page', $categoryRow['CategoryProductCount']);
			$altViewPage = scCustomerAreaPager::calculateAltViewPage($page, $view, $altView);
			$altViewUrl = scUrlUtil::addQueryParams($selfUrl, 'page=' . $altViewPage . '&view=' . $altViewString);
			
			$t->assign('%%HTML_CATEGORY_PAGES%%', scCustomerAreaPager::getPaging($page, $numPages, $selfUrl));
			$t->assign('%%PAGE%%', $page);
			$t->assign('%%ALT_VIEW_PAGE%%', $altViewPage);
			$t->assign('%%NUMPAGES%%', $numPages);
			
			$this->_view->getController()->getSeo()->setListingPageInfo($page, $numPages);
			
			// For the grid/list view links
			$t->assign('%%SELF_URL%%', $selfUrl);
			$t->assign('%%ALT_VIEW_URL%%', $altViewUrl);
			// Create the product sort dropdown
			$t->assign('%%HTML_SORT_PRODUCTS%%', scProductSortService::getSortDropdown());
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('categories.php'));		
			
			// Assign category image section
			$iterationsCount = 0;
			if (!empty($categoryRow['CategoryImage']) && file_exists(scPaths::getStoreRootDir() . $categoryRow['CategoryImage'])) {
				$iterationsCount = 1;
			}
			$imageInfo = array(
				'CategoryImage' => scPaths::getStoreWebPath() . $categoryRow['CategoryImage'],
				'CategoryImageWidth' => $categoryRow['CategoryImageWidth'],
				'CategoryImageHeight' => $categoryRow['CategoryImageHeight'],
			);
			$section = new miTemplateParserSectionInfo('categoryImage', $iterationsCount, miUIUtil::templetizeRows($imageInfo));
			$t->addSectionInfo($section);
			
			$this->countCategoryView($categoryId);
			
			return $t->templateParse();
		}
		
		protected function getSelfUrl($categoryId, $categoryName)
		{
			$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_CATEGORY, $categoryId, $categoryName);
			$urlParams = miAppFactory::singleton()->getStateObj()->getStateUrlParams();
			$url = scUrlUtil::addQueryParams($url, $urlParams);
			
			return $url;
		}
		
		protected function getSubcategoriesSection($category)
		{
			$categoriesRs = new scSqlRecordsetML('Categories', 'CategoryID');
			$categoriesRs->setSelectFields(array('*', 'CategoryDescription AS HTML_CategoryDescription'));
			$categoriesRs->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$categoriesRs->addFilter(new miSqlFilterEqual('CategoryParentID', (int)miGetParam('CategoryID')));
			$subCategories = $categoriesRs->getAllRecords();
			scFriendlyUrl::initFriendlyUrls($subCategories, scFriendlyUrl::ROW_TYPE_CATEGORIES);
			
			$additionalWhereConditions = $this->getAdditionalWhereConditions();
			$this->getFilteredSubcategories($subCategories, $category, $additionalWhereConditions);
			
			if (count($subCategories) > 0) {
				$hasSubcategoriesSection = new miTemplateParserSectionInfo('hasSubcategories', 1);
				
				$subCategoriesSection = new miTemplateParserSectionInfo();
				$subCategoriesSection->setSectionInfo('subcategories', count($subCategories), miUIUtil::templetizeRows(miUIUtil::transposeRows($subCategories)));
				
				$subCategoriesImageSections = array();
				foreach ($subCategories as $subCategory) {
					$imageSection = new miTemplateParserSectionInfo('categoryImage', 0);
					if (!empty($subCategory['CategoryImage']) && file_exists(scPaths::getStoreRootDir() . $subCategory['CategoryImage'])) {
						$imageSection->setIterationsCount(1);
						$subCategory['CategoryImage'] = scPaths::getStoreWebPath() . $subCategory['CategoryImage'];
						$imageSection->addValues(miUIUtil::templetizeRows($subCategory));
					}
					$subCategoriesImageSections[] = $imageSection; 
				}
				$subCategoriesSection->addSubsectionsArray('categoryImage', $subCategoriesImageSections);
				
				$hasSubcategoriesSection->addSubsection($subCategoriesSection);
			}
			else
				$hasSubcategoriesSection = new miTemplateParserSectionInfo('hasSubcategories', 0);
			
			return $hasSubcategoriesSection;
		}
		
		protected function getFilteredSubcategories(&$subCategories, $category, $additionalWhereConditions)
		{
			if (!empty($additionalWhereConditions)) {
				$hideOutOfStockSql = scProductService::hideOutOfStock() ? ' AND ProductQuantity > 0 ' : '';
				
				$additionalWhereConditions = implode(' ', $additionalWhereConditions);
				$main = ' SELECT Categories.CategoryID, COUNT(Products.ProductID) AS N, Categories.CategoryLeft, Categories.CategoryRight' . 
					' FROM Categories' .
					' INNER JOIN Products USING(CategoryID)' .
					' WHERE ' . $category['CategoryLeft'] . ' < CategoryLeft AND CategoryRight < ' . $category['CategoryRight'] .
					' AND ProductIsActive <> 0' .
					$hideOutOfStockSql .
					$additionalWhereConditions .
					' GROUP BY Categories.CategoryID';
				
				$additional = 'SELECT Categories.CategoryID, COUNT(Products.ProductID) AS N, Categories.CategoryLeft, Categories.CategoryRight' .
					' FROM Categories' .
					' INNER JOIN ProductXCategories USING(CategoryID)' .
					' INNER JOIN Products USING(ProductID)' .
					' WHERE ' . $category['CategoryLeft'] . ' < CategoryLeft AND CategoryRight < ' . $category['CategoryRight'] .
					' AND ProductIsActive <> 0' .
					$hideOutOfStockSql .
					$additionalWhereConditions .
					' GROUP BY Categories.CategoryID';
				
				$deepProductsCount = 'SELECT Categories.CategoryID, SUM(Table2.CategoryProductCount) AS CategoryDeepProductCount' .
					' FROM Categories'.
					' INNER JOIN (' .
						'SELECT CategoryLeft, CategoryRight, SUM(N) AS CategoryProductCount FROM (' .
							'(' . $main . ') UNION ALL (' . $additional . ')' .
						') AS Table1 GROUP BY CategoryID' .
					' ) AS Table2 ON Categories.CategoryLeft <= Table2.CategoryLeft AND Table2.CategoryRight <= Categories.CategoryRight' .
					' WHERE Categories.CategoryParentID = ' . $category['CategoryID'] .
					' GROUP BY Categories.CategoryID';
				
				$deepProductsCount = miStaticDBUtil::execSelect($deepProductsCount);
				$deepProductsCount = miUtil::getKeyValueArray($deepProductsCount, 'CategoryID', 'CategoryDeepProductCount');
				
				$stateObj = miAppFactory::singleton()->getStateObj();
				$oldParams = $params = $stateObj->getArray();
				$params['page'] = 1;
				$stateObj->setArray($params);
				$urlParams = $stateObj->getStateUrlParams();
				$stateObj->setArray($oldParams);
				
				foreach ($subCategories as &$subCategory) {
					$subCategoryId = $subCategory['CategoryID'];
					if (isset($deepProductsCount[$subCategoryId])) {
						$subCategory['CategoryDeepProductCount'] = $deepProductsCount[$subCategoryId];
					} else {
						$subCategory['CategoryDeepProductCount'] = 0;
					}
					
					$subCategory['CategoryUrl'] = scUrlUtil::addQueryParams($subCategory['CategoryUrl'], $urlParams);
				}
				unset($subCategory);
			}
		}
		
		/**
		 * Return the products from this category
		 * 
		 * @param int $categoryId
		 * @return array
		 */
		protected function getProducts($categoryId)
		{
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0 ' : '';
			$variantSql = 'LEFT JOIN ProductVariants ON (ProductVariants.ProductID = Products.ProductID AND ProductVariantIsDefault<>0) ';
			
			$additionalWhereConditions = $this->getAdditionalWhereConditions();
			$additionalWhereConditions = implode(' ', $additionalWhereConditions);
			
			$main = 'SELECT Products.ProductSortInCategory as ProductIDSort, Products.*, ProductsML.*, ProductImages.*, ProductVariants.*, Products.ProductID AS ProductID, Products.CategoryID AS CategoryID' .
				' FROM Products' .
				' LEFT JOIN ProductsML USING(ProductID)' .
				' LEFT JOIN ProductImages ON(Products.ProductID = ProductImages.ProductID AND ProductImageIsDefault<>0)' .
				$variantSql .
				' WHERE LanguageID = ?' .
				' AND Products.CategoryID = ?' .
				' AND ProductIsActive <> 0 ' . $hideOutOfStockSql .
				$additionalWhereConditions .
				' GROUP BY Products.ProductID';
			
			$additional = 'SELECT ProductXCategories.ProductSortInCategory as ProductIDSort, Products.*, ProductsML.*, ProductImages.*, ProductVariants.*, Products.ProductID AS ProductID, Products.CategoryID AS CategoryID' .
				' FROM Products' .
				' LEFT JOIN ProductsML USING(ProductID)' .
				' LEFT JOIN ProductXCategories ON(Products.ProductID = ProductXCategories.ProductID)' .
				' LEFT JOIN ProductImages ON(Products.ProductID = ProductImages.ProductID AND ProductImageIsDefault<>0)' .
				$variantSql .
				' WHERE LanguageID = ?' .
				' AND (ProductXCategories.CategoryID = ?)' .
				' AND ProductIsActive <> 0 ' . $hideOutOfStockSql .
				$additionalWhereConditions .
				' GROUP BY Products.ProductID';
			
			$internalLimitString = ' ORDER BY ' . scProductSortService::getSortClause() . ' LIMIT ' . scCustomerAreaPager::getProductsPerPage() * intval(miGetParamDefault('page', 1));
			$sql = '(' . $main . $internalLimitString . ') UNION DISTINCT (' . $additional . $internalLimitString . ') ' .
				' ORDER BY ' . scProductSortService::getSortClause() .
				scCustomerAreaPager::getLimitString('page');
			
			$languageId = scMultiLanguage::getCurrentLanguageId();
			return miStaticDBUtil::execSelect($sql, array($languageId, $categoryId, $languageId, $categoryId));
		}
		
		protected function getProductsCount($categoryId, $cachedValue = 0)
		{
			$additionalWhereConditions = $this->getAdditionalWhereConditions();
			if (empty($additionalWhereConditions) && !empty($cachedValue))
				return $cachedValue;
			
			$additionalWhereConditions = implode(' ', $additionalWhereConditions);
			
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0 ' : '';
			
			$main = 'SELECT Products.ProductID' .
				' FROM Products' .
				' WHERE Products.CategoryID = ?' .
				' AND ProductIsActive <> 0 ' . $hideOutOfStockSql .
				$additionalWhereConditions .
				' GROUP BY Products.ProductID';
			
			$additional = 'SELECT Products.ProductID' .
				' FROM Products' .
				' LEFT JOIN ProductXCategories ON(Products.ProductID = ProductXCategories.ProductID)' .
				' WHERE (ProductXCategories.CategoryID = ?)' .
				' AND ProductIsActive <> 0 ' . $hideOutOfStockSql .
				$additionalWhereConditions .
				' GROUP BY Products.ProductID';
			
			$sql = 'SELECT COUNT(*) AS N FROM ((' . $main . ') UNION DISTINCT (' . $additional . ')) AS Table1';
			
			$result = miStaticDBUtil::execSelect($sql, array($categoryId, $categoryId));
			if (!isset($result[0]) || !isset($result[0]['N'])) {
				return 0;
			}
			return (int)$result[0]['N'];
		}
		
		/**
		 * Return the featured priducts for the current category
		 * 
		 * @param int $categoryId
		 * @return array
		 */
		protected function getFeaturedProducts($categoryId)
		{
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0 ' : '';
			$variantSql = 'LEFT JOIN ProductVariants ON (ProductVariants.ProductID = Products.ProductID AND ProductVariantIsDefault<>0) ';
			
			$additionalWhereConditions = $this->getAdditionalWhereConditions();
			$additionalWhereConditions = implode(' ', $additionalWhereConditions);
			
			$sql = 	'SELECT *, Products.ProductID as ProductID, FeaturedProductSort as ProductIDSort FROM FeaturedProductCategories ' . 
					'INNER JOIN FeaturedProducts USING(FeaturedProductID) ' .
					'INNER JOIN Products USING(ProductID) ' .
					'LEFT JOIN ProductsML USING(ProductID) ' .
					'LEFT JOIN ProductImages ON(Products.ProductID = ProductImages.ProductID AND ProductImageIsDefault<>0) ' .
					$variantSql .
					'WHERE LanguageID = ? ' .
					'AND FeaturedProductCategories.CategoryID = ? ' .
					'AND ProductIsActive <> 0 ' . $hideOutOfStockSql .
					$additionalWhereConditions .
					'ORDER BY ' . scProductSortService::getSortClause();
			
			return miStaticDBUtil::execSelect($sql, array(scMultiLanguage::getCurrentLanguageId(), $categoryId));
		}
		
		/**
		 * Return the category row for a category id
		 * 
		 * @return array the category row
		 */
		protected function getCategoryRow($categoryId)
		{
			$sql = 'SELECT *, CategoryDescription AS HTML_CategoryDescription FROM Categories JOIN CategoriesML USING(CategoryID) WHERE Categories.CategoryID=? AND LanguageID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($categoryId, scMultiLanguage::getCurrentLanguageId()));
			if (count($rows) == 0)
				throw new miException(_t('Cannot find category.'));
			return $rows[0];
		}
		
		protected function countCategoryView($categoryId)
		{
			if (!isset($_SESSION['categoryViewsCount']))
				$_SESSION['categoryViewsCount'] = array();
			
			if (in_array($categoryId, $_SESSION['categoryViewsCount'])) {
				return '';
			}
			
			$_SESSION['categoryViewsCount'][] = $categoryId;
			
			$q = 'INSERT INTO StatCategoryViews (CategoryID, StatCategoryViewsDate, StatCategoryViewsCount) VALUES' .
				' (?, ?, 1) ON DUPLICATE KEY UPDATE StatCategoryViewsCount = StatCategoryViewsCount + 1';
			miStaticDBUtil::execSQL($q, array($categoryId, date(MI_MYSQL_DATE_FORMAT)));
		}
	}
	
	$page = new scUserDisplayController(new scUserPageCategories());
	$page->setSeo(new scCategorySeo());
	$page->getDispatcher()->setActionHandler('', 'scViewCategoryAction');
	$page->dispatchAndShow();
?>