<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scUserPageCheckoutFinalAction extends miAction {
		public function doAction()
		{
			$orderId = (int)miGetParamDefault('OrderID', 0);
			
			if (!scOrdersInProgress::singleton()->checkOrderId($orderId)) {
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode(sprintf(_t('You are not allowed to view order %d!'), $orderId));
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
				return true;
			}
			
			$order = new scOrder($orderId);
			
			// Check for message
			$message = miGetParamDefault('Message', '');
			
			$paymentStatus = $order->get('OrderPaymentStatus');
			
			$isSuccess = scPaymentStatus::isOrderSuccess($paymentStatus);
			$isPending = scPaymentStatus::isPaymentPending($paymentStatus); 
			
			try {// get payment module implementation from order's PaymentModuleID 
				$paymentModuleDomainObj = new scPaymentModule($order->get('PaymentModuleID'));
				$moduleId = $paymentModuleDomainObj->get('ModuleID');
				$paymentModule = scModulesFactory::getImplementation($moduleId, 'IPaymentModule');
			} catch (miDBException $e) {
				$paymentModule = null;
			}
			
			// Order success - empty the cart
			if ($isSuccess or $isPending) {
				scCart::getCart()->removeAllItems();
				return $this->doSuccess($order, $message, $paymentStatus, $isPending, $paymentModule);
			}
			
			// Failure
			$template = 'customer/checkout/checkout_failure.tmpl'; 
			
			$t = new scTemplateParser();
			$t->readTemplate($template);
			$t->assign('%%MESSAGE%%', miI18n::htmlescape($message));
			
			$paymentStatuses = scPaymentStatus::getPaymentStatuses();
			$t->assign('%%PAYMENTSTATUS%%', isset($paymentStatuses[$paymentStatus]) ? $paymentStatuses[$paymentStatus] : '');
			$t->assign('%%ORDERID%%', miI18n::htmlescape(miGetParamDefault('OrderID', 0)));
			$t->assign('%%IPADDRESS%%', miI18n::htmlescape($_SERVER['REMOTE_ADDR']));
			
			$t->assign('%%PAYMENTDETAILS%%', '');
			if (!is_null($paymentModule)) {
				// assign notes from the payment module
				$t->assign('%%PAYMENTDETAILS%%', $paymentModule->getPaymentContent(IPaymentModule::CONTENT_TYPE_CHECKOUT_FAILURE, array('OrderID' => $order->get('OrderID'))));
			}
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('checkout_final.php?failure'));		
			
			$html = $t->templateParse();
			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			return true;
		}
		
		/**
		 * Process successful order
		 * Note that the payment may be pending
		 * 
		 * @param scOrder $order
		 * @param string $message message received in the Message= param
		 * @param int $paymentStatus one scPaymentStatus::PAYMENT_STATUS_ constants
		 * @param boolean $isPending true if the payment is pending
		 * @param IPaymentModule $paymentModule the payment module that processed the payment
		 * @return boolean returns true
		 */
		protected function doSuccess(scOrder $order, $message, $paymentStatus, $isPending, IPaymentModule $paymentModule = null)
		{
			$orderDetailsHtml = scCustomerOrdersUI::singleton()->getOrderDetailsHtml($order);
			
			$t = new scTemplateParser();
			if ($isPending) {
				$template = 'customer/checkout/checkout_pending.tmpl';
				$dynamicPage = 'checkout_final.php?pending';
				$contentType = IPaymentModule::CONTENT_TYPE_CHECKOUT_PENDING;
			} else {
				$template = 'customer/checkout/checkout_success.tmpl';
				$dynamicPage = 'checkout_final.php?success';
				$contentType = IPaymentModule::CONTENT_TYPE_CHECKOUT_SUCCESS;
			}
			$t->readTemplate($template);
			$t->assign('%%MESSAGE%%', miI18n::htmlescape($message));
			$t->assign('%%ORDER_DETAILS%%', $orderDetailsHtml);
			
			$paymentStatuses = scPaymentStatus::getPaymentStatuses();
			$t->assign('%%PAYMENTSTATUS%%', isset($paymentStatuses[$paymentStatus]) ? $paymentStatuses[$paymentStatus] : '');
			
			$t->assign('%%PAYMENTDETAILS%%', '');
			if (!is_null($paymentModule)) {
				// assign notes from the payment module
				$t->assign('%%PAYMENTDETAILS%%', $paymentModule->getPaymentContent($contentType, array('OrderID' => $order->get('OrderID'))));
			}
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars($dynamicPage));		
			
			$html = $t->templateParse();
			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			return true;
		}
	}
	
	class scCheckoutFinalSeo extends scDynamicPageSeo {
		
		protected function addBodyClasses()
		{
			parent::addBodyClasses();
			$this->_bodyClasses[] = 'dynamic-page-checkout-final';
		}
 		
		public function getPagePath()
		{
			$orderId = (int)miGetParamDefault('OrderID', 0);
			$order = new scOrder($orderId);
			$paymentStatus = $order->get('OrderPaymentStatus');
			
			$isSuccess = scPaymentStatus::isOrderSuccess($paymentStatus);
			
			if (!$isSuccess) {
				return 'checkout_final.php?failure';
			}
			
			$isPending = scPaymentStatus::isPaymentPending($paymentStatus);
			
			if (!$isPending) {
				return 'checkout_final.php?success';
			}
			
			return 'checkout_final.php?pending';
		}
	}
	
	$page = new scUserDisplayController(array(new scView(null)));
	$page->setSeo(new scCheckoutFinalSeo(_t('Checkout Final')));
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scUserPageCheckoutFinalAction');
	
	$page->dispatchAndShow();
?>