<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	scDateTime::setTimezone();
	
	class scDownloadEgoodAction extends miAction {
		
		public function doAction()
		{
			try {
				$orderItemEgood = new scOrderItemEgood(miGetParam('EgoodID'));
				$order = $orderItemEgood->getOrderItem()->getOrder();
				
				if (scCustomerAuth::singleton()->isLoggedIn()) {
					$customer = scCustomerAuth::singleton()->getCustomer();
					if ($order->get('CustomerID') != $customer->get('CustomerID')) {
						$this->redirectToEgoods($orderItemEgood, _t('Access denied.'), miMessage::MSG_TYPE_ERROR);
						return false;
					}
				} else if (miGetParamDefault('AccessKey', '') !== $order->getAccessKey()) {
					$orderId = $order->get('OrderID');
					$loginUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN);
					$loginUrl = scUrlUtil::addQueryParams($loginUrl, 'Referer=' . urlencode('order_egoods.php?OrderID=' . $orderId));
					$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loginUrl));
					
					return false;
				}
				
				$orderItemEgood->download();
			} catch (miException $e) {
				$this->htmlRedirectToEgoods(($order instanceof scOrder ? $order->get('OrderID') : 0), _t('Access denied.') .' '. $e->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			return true;
		}
		
		/**
		 * @param int $orderId
		 * @param string $msg
		 * @param int $msgType
		 * @return string
		 */
		protected function getRedirectLocation($orderId, $msg, $msgType)
		{
			return 'order_egoods.php?OrderID='. $orderId .
				'&'. miMessage::PARAM_MESSAGE .'='. urlencode($msg) .
				'&'. miMessage::PARAM_MESSAGE_TYPE .'='. urlencode($msgType);
		}
		
		/**
		 * @param int $orderId
		 * @param string $msg
		 * @param int $msgType
		 */
		protected function htmlRedirectToEgoods($orderId, $msg, $msgType)
		{
			$loc = $this->getRedirectLocation($orderId, $msg, $msgType);
			print '<html><head><script type="text/javascript">window.parent.location = "' . $loc .
				'"</script></head></html>';
			exit;
		}
	}
	
	$view = new miDefaultView(null);
	$page = new miPage(array($view));
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scDownloadEgoodAction');
	$page->dispatchAndShow();
?>