<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scForgotPasswordAction extends miAction {
		public function doAction()
		{
			$t = new scTemplateParser();
			$t->readClientAreaTemplate('customer/forgot_pass/forgot_pass.tmpl');
			
			$email = miGetParamDefault('Email', '');
			$msg = miGetParamDefault('msg', '');
			
			$t->assignArray(array(
				'%%EMAIL%%' => miI18N::htmlEscape($email),
				'%%MSG%%' => miI18N::htmlEscape($msg),
				'%%MSG_STYLE%%' =>  $msg ? 'error' : 'hidden',
			));
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('forgot_pass.php'));		
			
			$html = $t->templateParse();	
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($cmd);
		}
	}
	
	class scExecForgotPasswordAction extends miAction {
		public function doAction()
		{
			try {
				$email = miGetParamDefault('Email', '');
				if (empty($email)) {
					throw new miException(_t('Please enter email.'));
				}
				$user = new miSqlRecord('Customers', 'CustomerEmail');
				$user->readPK($email);
				
				# send mail
				$oEmail = new scEmailNotifications;
				$oEmail->sendForgotPass($user->get('CustomerID'));
				
			} catch (miException $exception) {
				
				$msg = $exception->getMessage();
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					$msg = _t('Email not found. Please try again.');
				}
				$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_FORGOT_PASS);
				$url = scUrlUtil::addQueryParams($url, 'msg=' . urlencode($msg) . '&Email=' . urlencode($email));
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url);
				$this->_view->addControllerCommand($cmd);
				return false;
			}
			
			$msg = sprintf(_t('Your password has been sent to %s. Please check your mail.'), $email);
			$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_FORGOT_PASS);
			$url = scUrlUtil::addQueryParams($url, 'msg=' . urlencode($msg));
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url);
			$this->_view->addControllerCommand($cmd);
			return true;
		}
	}
	
	class scResetPasswordAction extends miAction {
		public function doAction()
		{
			try {
				$msg = miGetParamDefault('msg', '');
				$customerId = (int)miGetParamDefault('CustomerID', 0);
				if (empty($customerId)) {
					throw new miException(_i('Invalid CustomerID.'));
				}
				
				$secret = miGetParamDefault('secret', 0);
				$hash = scUtil::getDbValue('Customers', 'CustomerPassword', 'CustomerID', $customerId, '');
				if ($secret != md5($hash)) {
					throw new miException(_t('You have already changed your password.'));
				}
				
				$t = new scTemplateParser();
				$t->readClientAreaTemplate('customer/forgot_pass/reset_pass.tmpl');
				
				$t->assignArray(array(
					'%%CUSTOMERID%%' => miI18N::htmlEscape($customerId),
					'%%SECRET%%' => miI18N::htmlEscape($secret),
					'%%MSG_STYLE%%' => empty($msg) ? 'hidden' : 'error',
					'%%MSG%%' => $msg,
				));
				
				// Assign dynamic page areas values
				$t->assignArray(scDynamicPages::getTemplateVars('forgot_pass.php'));
				
				$html = $t->templateParse();	
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				
			} catch (miException $exception) {
				$msg = $exception->getMessage();
				$url = scPaths::getUrl('error.php?msg=' . urlencode($msg));
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url);
				$this->_view->addControllerCommand($cmd);
				return false;
			}
		}
	}
	
	class scExecResetPasswordAction extends miAction {
		public function doAction()
		{
			try {
				$customerId = (int)miGetParamDefault('CustomerID', 0);
				if (empty($customerId)) {
					throw new miException(_i('Invalid CustomerID.'));
				}
				
				$secret = miGetParamDefault('secret', 0);
				
				$customer = new scCustomerDomainObject();
				$customer->read($customerId);
				if ($secret != md5($customer->get('CustomerPassword'))) {
					throw new miException(_t('You have already changed your password.'));
				}
				
				$password = miGetParam('newPass');
				$password2 = miGetParam('newPass2');
				if ($password != $password2) {
					throw new miException(_t('Passwords mismatch.'));
				}
				
				$customer->set('CustomerPassword', $password);
				$customer->update();
				
				$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_FORGOT_PASS);
				$url = scUrlUtil::addQueryParams($url, 'action=resetPassSuccess');
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url);
				$this->_view->addControllerCommand($cmd);
				
				return true;
			} catch (miException $exception) {
				$msg = $exception->getMessage();
				$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_FORGOT_PASS);
				$url = scUrlUtil::addQueryParams($url, 'action=resetPass&msg=' . urlencode($msg) . '&CustomerID=' . $customerId . '&secret=' . urlencode($secret));
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url);
				$this->_view->addControllerCommand($cmd);
				return false;
			}
		}
	}
	
	class scResetPassSuccessAction extends miAction {
		public function doAction()
		{
			$t = new scTemplateParser();
			$t->readClientAreaTemplate('customer/forgot_pass/success.tmpl');
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('forgot_pass.php'));
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $t->templateParse());
			$this->_view->addControllerCommand($cmd);
			return true;
		}
	}
	
	$page = new scUserDisplayController(new scView());
	$page->setSeo(new scDynamicPageSeo('forgot_pass.php', _t('Forgotten Password')));
	$page->getDispatcher()->setActionHandler('', 'scForgotPasswordAction');
	$page->getDispatcher()->setActionHandler('execForgotPass', 'scExecForgotPasswordAction');
	$page->getDispatcher()->setActionHandler('resetPass', 'scResetPasswordAction');
	$page->getDispatcher()->setActionHandler('execResetPass', 'scExecResetPasswordAction');
	$page->getDispatcher()->setActionHandler('resetPassSuccess', 'scResetPassSuccessAction');
	$page->dispatchAndShow();
?>