<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scUserPageForms extends miDefaultView implements ICMSBlock {
	    protected $_templates = array(
			'form' => 'customer/forms/form.tmpl',
			'confirmation' => 'customer/forms/confirmation.tmpl',
		);
		
		protected $_hasStates = false;
		
		protected function addTableFeatures($table)
		{
			// Overload parent and do nothing
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$record = $this->getRecord();
			$record->readPK((int)miGetParam('FormID'));
			$this->addMainPageElements($this->getFormVars($record));

			// Assign dynamic page areas values
			$this->addMainPageElements(scDynamicPages::getTemplateVars('form.php'));		
			
            $form->setErrorsHandler(new scByFieldErrorsHandler);
		}
		
		public function getFormVars($record)
		{
			$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_FORM, $record->get('CMSFormID'), 'form');
			
			return array(
				'%%FORMURL%%' => $url,
				'%%FORMID%%' => $record->get('CMSFormID'),
				'%%FORMTITLE%%' => $record->get('CMSFormTitle'),
				'%%FORMDESCRIPTION%%' => $record->get('CMSFormDescription'),
				'%%FORMCONFIRMTEXT%%' => $record->get('CMSFormConfirmText')
			);
		}
		
		public function getFormFieldsSection($form)
		{
            $items = $this->readItems();
            $itemValues = $this->readItemValues($items);
            $formFileds =  $this->createFormFields($form, $items, $itemValues);
			
            $itemSections = array();
			$headerSections = array();
			$separatorSections = array();
			
            foreach ($formFileds as $key => $formFiled) {
            	$data = $formFiled->getEditableControl();
            	$values = array('%%FORMFIELDNAME%%' => $items[substr($key, 4)]['CMSFormItemName'], '%%FORMFIELD%%' => $data[$key], '%%FIELDID%%' => $key);
            	$cmsFormItemType = $items[substr($key, 4)]['CMSFormItemType'];
            	
				$isHeader = ($cmsFormItemType == scFormItem::ITEM_TYPE_HEADER);
				$headerSections[] = new miTemplateParserSectionInfo('header', $isHeader ? 1 : 0, $values);
				
				$isSeparator = ($cmsFormItemType == scFormItem::ITEM_TYPE_SEPARATOR);
				$separatorSections[] = new miTemplateParserSectionInfo('separator', $isSeparator ? 1 : 0, $values);
				
            	$isItem = !($isHeader or $isSeparator);
            	$itemSections[] = new miTemplateParserSectionInfo('item', $isItem ? 1 : 0, $values);
            }
            
			$section = new miTemplateParserSectionInfo('items', count($items));
			$section->addSubsectionsArray('item', $itemSections);
			$section->addSubsectionsArray('header', $headerSections);
			$section->addSubsectionsArray('separator', $separatorSections);
			return $section;
		}
		
		public function getFormFields($form, $items, $itemValues)
		{
            $formFields = $this->createFormFields($form, $items, $itemValues);
            $validators = $this->getValidators($form, $items);
            return array($formFields, $validators);
		}
		
		public function hasStates()
		{
			return $this->_hasStates;
		}
		
		public function readItems()
		{ 
			return miStaticDBUtil::execSelect(
				'SELECT * FROM CMSFormItems INNER JOIN CMSFormItemsML USING(CMSFormItemID) ' .
				'WHERE LanguageID = ? AND CMSFormID = ? AND CMSFormItemIsActive = "1" ' . 
				'ORDER BY CMSFormItems.CMSFormItemSort ASC',
				array(
					scMultiLanguage::getCurrentLanguageId(),
					miGetParam('FormID')
				)
			);
		}
		
		public function readItemValues($items)
		{
            foreach ($items as $row) {
            	$itemIds[] = $row['CMSFormItemID'];
            }
            
            $itemValuesForm = new miSqlRecordset('CMSFormItemValues');
            $itemValuesForm->addJoinCondition('INNER', 'CMSFormItemValuesML', 'USING(CMSFormItemValueID)');
			$itemValuesForm->addFilter(new miSqlFilterIn('CMSFormItemID', implode(',', $itemIds)));
			$itemValuesForm->addFilter(new miSqlFilterEqual('LanguageID', scMultiLanguage::getCurrentLanguageId()));
			$itemValuesForm->setOrder('CMSFormItemValues.CMSFormItemValueID', 'ASC');
            return $itemValuesForm->getAllRecords();
		}
		
		protected function getValidators($form, $items)
		{
			$validators = array();
			foreach ($items as $key => $row) {
				if ($row['CMSFormItemIsRequired']) {
					switch ($row['CMSFormItemType']) {
						case scFormItem::ITEM_TYPE_TEXT:
						case scFormItem::ITEM_TYPE_TEXT_AREA:
						case scFormItem::ITEM_TYPE_CHECKBOX:
							$validators[] = new scValidatorNoEmpty($form, $this->getFieldName($key));
					}
				}
				
				switch ($row['CMSFormItemType']) {
					case scFormItem::ITEM_TYPE_EMAIL:
						$validators[] = new scValidatorEmail($form, $this->getFieldName($key));
						$validators[] = new scValidatorNoEmpty($form, $this->getFieldName($key));
						break;
					case scFormItem::ITEM_TYPE_NUMBER:
						$validators[] = new scValidatorDecimal($form, $this->getFieldName($key));
						$validators[] = new scValidatorNoEmpty($form, $this->getFieldName($key));
						break;
				}
			}
			return $validators;
		}
		
		protected function getFieldName($key)
		{
			return 'form' . $key;
		}
		
		protected function createFormFields(miWebForm $form, $items, $itemValues)
		{
			$formFields = array();
			foreach ($items as $key => $row) {
            	switch ($row['CMSFormItemType']) {
					case scFormItem::ITEM_TYPE_TEXT:
						$widget = new miTextWidget($form);
						break;
					case scFormItem::ITEM_TYPE_TEXT_AREA:
						$widget = new scWebFormWidgetTextarea($form);
						break;
					case scFormItem::ITEM_TYPE_CHECKBOX:
						$widget = new miCheckboxWidget($form);
						break;
					case scFormItem::ITEM_TYPE_RADIO_BUTTONS:
						$widget = new miRadioWidget($form);
						$widget->setRadioButtons($this->getItemValues($row['CMSFormItemID'], $itemValues));
						break;
					case scFormItem::ITEM_TYPE_SELECT:
						$widget = new miSelectWidget($form);
						$widget->setOptions($this->getItemValues($row['CMSFormItemID'], $itemValues));
						break;
					case scFormItem::ITEM_TYPE_EMAIL:
						$widget = new miTextWidget($form);
						break;
					case scFormItem::ITEM_TYPE_NUMBER:
						$widget = new miTextWidget($form);
						break;
					case scFormItem::ITEM_TYPE_COUNTRIES:
						$widget = new miSelectWidget($form, array('class' => 'country'));
						$widget->setOptions(scCountry::getCountryNames(), scSettings::getSettingValue(scSettings::SETTING_DEFAULT_COUNTRY));
						break;
					case scFormItem::ITEM_TYPE_STATES:
						$this->_hasStates = true;
						$widget = new miSelectWidget($form, array('class' => 'state'));
						$widget->setOptions(scCountry::getStateNames());
						break;
					case scFormItem::ITEM_TYPE_HEADER:
						$widget = new scFormsWidgetHeader($form);
						break;
					case scFormItem::ITEM_TYPE_SEPARATOR:
						$widget = new scFormsWidgetSeparator($form);
						break;
					case scFormItem::ITEM_TYPE_FILE_UPLOAD:
						if ($row['CMSFormItemIsRequired']) {
							$widget = new scFileUploadWidget($form);
						} else {
							$widget = new scNotRequiredFileUploadWidget($form);
						}
						break;
					case scFormItem::ITEM_TYPE_CUSTOM_HTML:
						$widget = new scFormsWidgetCustomHtml($form);
						$widget->setProperty('content', $row['CMSFormItemCustomHtml']);
						break;
				}
				
				$widget->setProperty('id', $this->getFieldName($key));
				$widget->setFieldName($this->getFieldName($key));
				$formFields[$this->getFieldName($key)] = $widget;
			}
			return $formFields;
		}
		
		protected function getItemValues($itemKey, $itemValues)
		{
			$options = array();
			foreach ($itemValues as $row) {
				if ($row['CMSFormItemID'] == $itemKey) {
					$options[$row['CMSFormItemValueID']] = $row['CMSFormItemValue'];
				}
			}
			return $options;
		}
	}
	
	class scNotRequiredFileUploadWidget extends scFileUploadWidget {
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			$data = $this->getData();
			if (empty($data)) {
				return;
			}
			
			parent::validateData($errors);
		}
	}
	
	class scFormAction extends miActionWithWebForm {
		
		protected function showForm(&$form, $templateName, $isEditable)
		{
			$form->addMainPageElements($this->_view->getMainPageElements());
			$form->addTemplateSectionInfo($this->_view->getFormFieldsSection($form));

			if ($this->_view->getDomainObject()->get('CMSFormEnableCaptcha')) {
				$captcha = new scCaptchaWidget($form);
				$captcha->setProperty('id', 'captcha');
				$captcha->setFieldName('captcha');
				$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('captcha', 1, miUIUtil::templetizeRows($captcha->getEditableControl())));
			}
			
			$statesData = $this->_view->hasStates() ? scUIUtil::getStatesJsData() : '';
			$form->addMainPageElements(array('%%STATES_JS_DATA%%' => $statesData));
			
			$html = $form->parse($templateName, $isEditable);
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($cmd);
		}
		
		public function doAction()
		{
			try {
				$this->_view->callPlugin($this, 'preShowForm');
				$form = $this->getWebForm();
				$this->showForm($form, $this->_view->getTemplateFileName('form'), true);
				return true;
			} catch (miException $exception) {
				$msg = _t('View form failed: ') . $exception->getMessage();
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, scPaths::getStoreWebPath() . 'error.php?' . $this->_view->getRedirectParams($msg, miMessage::MSG_TYPE_ERROR));
				$this->_view->addControllerCommand($cmd);
				return false;
			}
		}
	}
	
	
	class scSubmitFormAction extends scFormAction {
		
		public function doAction()
		{
			$record = $this->_view->getRecord();
			$record->readPK((int)miGetParam('FormID'));
			
			$form = $this->getWebForm();
            $items = $this->_view->readItems();
            $itemValues = $this->_view->readItemValues($items);
			list($formFields, $validators) = $this->_view->getFormFields($form, $items, $itemValues);
			
			foreach ($formFields as $key => $formField)
				$form->addWidget($key, $formField);
			foreach ($validators as $validator)
				$form->addValidator($validator->getFieldName(), $validator);
			
			$isOk = $form->processSubmit();
			$data = $form->getSubmittedDataRow();
			$data = array_merge($data, $this->getCustomHtmlFieldsData($data));
			
			$form->setFormDataRow($data);
			
			if ($record->get('CMSFormEnableCaptcha')) {
				$isOk = ($isOk and $this->validateCaptcha()); // Keep the parenthesis!
			}
			
			if ($isOk === false) {
				parent::doAction();
				return false;
			}
			
			$this->showConfirmation($record);
			
			$to = trim($record->get('CMSFormEmail'));
			if (empty($to)) {
				$to = scSettings::getSettingValue(scSettings::SETTING_EMAIL_HELP_SUPPORT);
			}
			
			$oEmail = new scEmailNotifications;
			$oEmail->sendFormEmail($to, $record->get('CMSFormTitle'), $items, $formFields, $data);
			
			return true;
		}
		
		protected function getCustomHtmlFieldsData($formData = array())
		{
			$filteredFields = array();
			$data = array_diff_key($_REQUEST, $formData);
			foreach ($data as $key => $val)
				if (substr($key, 0, strlen(scFormItem::CUSTOM_HTML_FIELD_PREFIX)) == scFormItem::CUSTOM_HTML_FIELD_PREFIX)
					$filteredFields[$key] = $val;
			
			return $filteredFields;
		}
		
		protected function showConfirmation($record)
		{
			$t = new scTemplateParser();
			$t->readTemplate($this->_view->getTemplateFileName('confirmation'));
			$t->assignArray($this->_view->getFormVars($record));
			
			$html = $t->templateParse();
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($cmd);
		}
		
		protected function validateCaptcha()
		{
			$captchaSecurity = new scCaptchaSecurity();
			$code = $captchaSecurity->getCode();
			
			if (empty($code) or (strtolower($code) != strtolower(miGetParam('captcha')))) {
				$err = $this->getWebForm()->getWebFormErrorsHandler();
				$err->addError('captcha', _t('Security code does not match.'));
				return false;
			}
			return true;
		}
	}
	
	class scFormsWidgetHeader extends miBaseWidget {
		public function getEditableControl()
		{
			return array($this->_fieldName => '');
		}
	}
	
	class scFormsWidgetCustomHtml extends miBaseWidget {
		
		public function getEditableControl()
		{
			return array($this->_fieldName => $this->_properties['content']);
		}
		
	}
	
	class scFormsWidgetSeparator extends miBaseWidget {
		public function getEditableControl()
		{
			return array($this->_fieldName => '');
		}
	}
	
	/**
	 * Generates captcha security image
	 */
	class scCaptchaImageAction extends miAction {
		public function doAction()
		{
			$captchaSecurity = new scCaptchaSecurity();
			$captchaSecurity->generateCode();
			
			$captchaImage = new scCaptchaImage();
			$captchaImage->generateImage($captchaSecurity->getCode());
			exit;
		}
	}
	
	$view = new scUserPageForms(null);
	$view->setRecord(new scSqlRecord('CMSForms', 'CMSFormID', true));
	
	$page = new scUserDisplayController(array($view));
	$page->setSeo(new scFormSeo());	
	$page->getDispatcher()->setActionHandler('', 'scFormAction');
	$page->getDispatcher()->setActionHandler('SubmitForm', 'scSubmitFormAction');
	$page->getDispatcher()->setActionHandler('acCaptchaImage', 'scCaptchaImageAction');
	
	$page->dispatchAndShow();
?>