<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scGalleriesView extends miDefaultView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_VIEW => 'customer/gallery.tmpl',
		);
			
		public function __construct()
		{
			parent::__construct(null);
			
			$this->_persistVars[] = 'GalleryID';
		}
	}
	
	class scGalleryViewAction extends miAction {
		
		protected function setStatePersistVars(IProductsListingFilter $filter)
		{
			$persistVars = $filter->getFilterVars();
			if (empty($persistVars)) {
				return;
			}
			
			$state = miAppFactory::singleton()->getStateObj();
			$statePersistVars = $state->getPersistVars();
			$statePersistVars = array_merge($statePersistVars, $persistVars);
			$statePersistVars = array_unique($statePersistVars);
			$state->setPersistVars($statePersistVars);
		}
		
		public function doAction()
		{
			$galleryId = (int)miGetParam('GalleryID');
			
			$sql =  'SELECT *, Galleries.GalleryID AS GalleryID FROM Galleries' .
				' LEFT JOIN GalleriesML ON Galleries.GalleryID = GalleriesML.GalleryID AND LanguageID = ?' .
				' WHERE Galleries.GalleryID = ?';
			
			$galleryInfo = miStaticDBUtil::execSelect($sql, array(scMultiLanguage::getCurrentLanguageId(), $galleryId));
			if (empty($galleryInfo)) {
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode(_t('Cannot find gallery.'));
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
				return true;
			}
			$galleryInfo = reset($galleryInfo);
			
			$recordset = new scSqlRecordsetML('GalleryItems', 'GalleryItemID');
			$recordset->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$recordset->addFilter(new miSqlFilterEqual('GalleryID', $galleryId));
			$recordset->setOrder('GalleryItemSort', 'ASC');
			
			$itemsPerRow = scSettings::getSettingValue(scSettings::SETTING_GALLERY_SHOW_TABLE_X);
			$rowsPerPage = scSettings::getSettingValue(scSettings::SETTING_GALLERY_SHOW_TABLE_Y);
			$itemsPerPage = $itemsPerRow * $rowsPerPage;			
			
			$offset = scCustomerAreaPager::getLimitOffset('page', $itemsPerPage);
			$rows = $recordset->getRecordsByIndex($offset, $itemsPerPage);
			
			$imagesSection = $this->getImagesSection($rows, $itemsPerRow);
			
			$t = new scTemplateParser();
			$t->readTemplate($this->_view->getTemplateFileName(miDefaultView::TEMPLATE_ID_VIEW));
			$t->assignArray($this->_view->getMainPageElements());
			$t->assignArray(miUIUtil::templetizeAndEscapeRows($galleryInfo));
			$t->assignArray(scDynamicPages::getTemplateVars('gallery.php'));
			
			$t->addSectionInfo($imagesSection);
			if (!empty($galleryInfo['GalleryDescription'])) {
				$t->addSectionInfo(new miTemplateParserSectionInfo('hasDescription', 1));
			}
			
			$selfUrl = $this->getSelfUrl($galleryId, $galleryInfo['GalleryName']);
			
			list($page, $numPages) = scCustomerAreaPager::calculatePages('page', $recordset->getRecordsCount(), $itemsPerPage);
			$t->assign('%%HTML_GALLERY_PAGES%%', scCustomerAreaPager::getPaging($page, $numPages, $selfUrl));
			$t->assign('%%PAGE%%', $page);
			$t->assign('%%NUMPAGES%%', $numPages);
			
			$this->_view->getController()->getSeo()->setListingPageInfo($page, $numPages);
			
			$html = $t->templateParse();
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;					
		}
		
		protected function getImagesSection($rows, $imagesPerRow)
		{
			$rows = array_chunk($rows, $imagesPerRow);
			
			$rowsSection = new miTemplateParserSectionInfo('rows', count($rows));
			$images = array();
			foreach ($rows as &$row) {
				$keys = array_keys(reset($row));
				$noImage = array();
				foreach ($keys as $key) {
					$noImage[$key] = '';
				}
				$row = array_pad($row, $imagesPerRow, null);
				
				$imageSections = array();
				foreach ($row as &$image) {
					if (is_null($image)) {
						$imageSections[] = new miTemplateParserSectionInfo('image', 0);
						continue;
					}
					$image['GalleryItemImage'] = scUrlUtil::convertUrl($image['GalleryItemImage'], scUrlUtil::URL_TYPE_DOCROOT);
					$image['GalleryItemImageThumb'] = scUrlUtil::convertUrl($image['GalleryItemImageThumb'], scUrlUtil::URL_TYPE_DOCROOT);
					$imageSection = new miTemplateParserSectionInfo('image', 1);
					if (!empty($image['GalleryItemURL'])) {
						$imageSection->addSubsection(new miTemplateParserSectionInfo('hasUrl', 1));
					}
					$imageSections[] = $imageSection;
				}
				unset($image);
				
				$section = new miTemplateParserSectionInfo('image', count($row), miUIUtil::templetizeRows(miUIUtil::transposeRows($row)));
				$section->addSubsectionsArray('image', $imageSections);
				$images[] = $section;
			}
			unset($row);
			$rowsSection->addSubsectionsArray('image', $images);
			
			$section = new miTemplateParserSectionInfo('hasImages', 1);
			$section->addSubsection($rowsSection);
			
			return $section;
		}
		
		protected function getSelfUrl($galleryId, $galleryName)
		{
			$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_GALLERY, $galleryId, $galleryName) . '?';
			$urlParams = miAppFactory::singleton()->getStateObj()->getStateUrlParams();
			$url = scUrlUtil::addQueryParams($url, $urlParams);
			
			return $url;
		}
	}
	
	$view = new scGalleriesView();
	
	$page = new scUserDisplayController(array($view));
	$page->setSeo(new scGallerySeo());
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scGalleryViewAction');
	
	$page->dispatchAndShow();
?>