<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Change IsActive in the database  
	 */
	class scExecChangeIsActiveAction extends miAction {
		
		protected $_fieldName;
		
		/**
		 * Sets the field name
		 *
		 * @access public
		 * @param int $field current field name
		 */
		public function setFieldName($field)
		{
			$this->_fieldName = $field;
		}
		
		public function doAction()
		{
			try {
				$domainObject = $this->_view->getDomainObject();
				$domainObject->read(miGetParam($domainObject->getPKName()));
				$newIsActive = $domainObject->get($this->_fieldName) ? 0 : 1;
				$domainObject->set($this->_fieldName, $newIsActive);
				$pkValue = $domainObject->update();
				
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_UPDATE_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			$msg = sprintf($this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG'), $pkValue);
			$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}

	/**
	 * Back Action - fill in form
	 */
	class scBackAction extends miListAction {
		
		public function doAction()
		{
			// Save the current params
			$getParams = $_GET;
			$postParams = $_POST;
			$requestParams = $_REQUEST;
			
			unset($getParams['action']);
			unset($postParams['action']);
			unset($requestParams['action']);
			
			scNavigationUtil::restoreNavigationParams(get_class($this->_view));
			
			// Make sure we don't have leftover messages
			unset($_GET['msg']);
			unset($_POST['msg']);
			unset($_REQUEST['msg']);

			// Merge the current params with the restored ones
			$_GET = array_merge($_GET, $getParams);
			$_POST = array_merge($_POST, $postParams);
			$_REQUEST = array_merge($_REQUEST, $requestParams);
			
			parent::doAction();
			
			return true;
		}
	}
	
	/**
	 * Action for printing data
	 */
	class scPrintAction extends miAction {
		const PRINT_COLUMNS = 2;
		
		protected $_allowedPrintFields = null;
		protected $_selectedPrintFields = null;
		
		/**
		 * The processing method
		 */
		public function doAction()
		{
			if (miHasParam('print'))
				$controllerCommand = $this->doPrint();
			else
				$controllerCommand = $this->showSelectPrintFields();
			
			$this->_view->addControllerCommand($controllerCommand);
			return true;
		}
		
		/**
		 * @return scPrintExportUtil
		 */
		protected function createPrintExportUtil()
		{
			return new scPrintExportUtil($this->_view);
		}
		
		/**
		 * @return scPrintExportUtil
		 */
		protected function getPrintExportUtil()
		{
			static $printExportUtil = null;
			if ($printExportUtil == null) {
				$printExportUtil = $this->createPrintExportUtil();
			}
			return $printExportUtil;
		}
		
		/**
		 * Returns the selected fields for printing
		 * Throws an exception on invalid field name
		 *
		 * @return array
		 * @throws miException
		 */
		protected function getSelectedPrintFields()
		{
			if (is_null($this->_selectedPrintFields)) {
				// Do the print selection
				$fields = trim(miGetParamDefault('fields', ''), '|');
				if (empty($fields))
					throw new miException(_t('No print fields selected.'));
				
				$fields = explode('|', $fields);
				$this->_selectedPrintFields = array_intersect($fields, array_keys($this->getPrintFields()));
			}

			return $this->_selectedPrintFields;
		}
		
		/**
		 * Return the headers (translated) for the selected print columns
		 *
		 * @return array
		 */
		protected function getPrintHeaders()
		{
			return array_intersect_key($this->getPrintFields(), array_flip($this->getSelectedPrintFields()));
		}
		
		
		
		/**
		 * Used to show the print page
		 * 
		 * @return miControllerCommand
		 */
		protected function doPrint()
		{
			try {
				$fields = $this->getSelectedPrintFields();
				$recordset = $this->getPrintExportUtil()->getRecordset();
				$this->getPrintExportUtil()->setupRecordset($recordset, $fields);
				$rows = $this->getPrintExportUtil()->getRecords($recordset, $fields);
				$headers = $this->getPrintHeaders();
			}
			catch (miException $exception)
			{
				return new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, '?msg=' . $exception->getMessage());
			}
			
			$parser = new scTemplateParser();

			if (count($rows) > 0) {
				// Create the header for the table
				$headerSection = new miTemplateParserSectionInfo('header', count($headers), array('%%HEADER%%' => array_values($headers)));
				$parser->addSectionInfo($headerSection);
				
				// Create the column section
				foreach ($rows as $row)
					$columnSections[] = new miTemplateParserSectionInfo('cols', count($row), array('%%PRINT_DATA%%' => array_values($row)));
				
				// Create the rows section
				$rowsSection = new miTemplateParserSectionInfo('rows', count($columnSections));
				$rowsSection->addSubsectionsArray('cols', $columnSections);
				
				$parser->addSectionInfo($rowsSection);
			}
			else
				$rowsSection = new miTemplateParserSectionInfo('norows');

			$parser->readTemplate('admin/popup/print.tmpl');
			$parser->addSectionInfo($rowsSection);
			$parser->assign('%%TITLE%%', $this->_view->getDomainObjectDisplayName());
			$html = $parser->templateParse();
			
			
			return new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
		}
		
		/**
		 * Used to show the fields available for printing
		 *
		 * @return miControllerCommand
		 */
		protected function showSelectPrintFields()
		{
			$printFields = array_chunk($this->getPrintFields(), self::PRINT_COLUMNS, true);
			
			$displayRows = array();
			foreach ($printFields as $row) {
				$displayRow = array(
					'%%FIELDNAME%%' => array_keys($row),
					'%%FIELDDISPLAYNAME%%' => array_values($row)
				);
				
				$displayRows[] = new miTemplateParserSectionInfo('row', count($row), $displayRow);
			}
			
			$fieldsSection = new miTemplateParserSectionInfo('rows', count($displayRows));
			$fieldsSection->addSubsectionsArray('row', $displayRows);
			
			$parser = new scTemplateParser();
			$parser->readTemplate('admin/popup/print_fields.tmpl');
			$parser->addSectionInfo($fieldsSection);
			$parser->assign('%%MODULENAME%%', miGetParamDefault('ModuleName', ''));			
			$table = $this->_view->getTable();
			$this->_view->initTable($table); 
			
			$parser->assignArray($table->getFeatureValues());
			
			$html = $parser->templateParse();
			
			return new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
		}
		
		/**
		 * Used to retrieve the fields available for printing (field names as keys field translated descriptions as values)
		 * 
		 * @return array
		 */
		protected function getPrintFields()
		{
			return $this->getPrintExportUtil()->getFields();
		}
	}
	
	/**
	 * Export as CSV action
	 */
	class scExportCSVAction extends miCsvExportAction {
				
		/**
		 * @return scPrintExportUtil
		 */
		protected function createPrintExportUtil()
		{
			return new scPrintExportUtil($this->_view);
		}
		
		/**
		 * @return scPrintExportUtil
		 */
		protected function getPrintExportUtil()
		{
			static $printExportUtil = null;
			if ($printExportUtil == null) {
				$printExportUtil = $this->createPrintExportUtil();
			}
			return $printExportUtil;
		}
		
		public function getExportFields()
		{
			return array_keys($this->getPrintExportUtil()->getFields());
		}
		
		public function doAction()
		{
			$fields = $this->getExportFields();
			// Init the recordset
			$recordSet = $this->getPrintExportUtil()->getRecordset();
			$this->getPrintExportUtil()->setupRecordset($recordSet, $fields);
			
			// Perform the export
			if (headers_sent()) {
				throw new miConfigurationException('Cannot export as CSV. Headers have already been sent', miConfigurationException::EXCEPTION_HEADERS_ALREADY_SENT);
			}
			
			// Remove possible header and do gzip encoding for speeding up the CSV file download
			if (ob_get_level()) {
				ob_end_clean();
				header('Content-Encoding: none');
			}
			ob_start('ob_gzhandler');
			
			$this->showCsvExportHeaders();
			$this->exportRecords($this->getPrintExportUtil()->getRecords($recordSet, $fields));
			exit;
		}
	}
?>