<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Base class for sort moving up/down action
	 */
	class scSortAction extends miAction {
		static protected $_sortColumn;
		static protected $_foreignKeyClause = false;
		
		protected $_tableName;
		protected $_primaryKeyColumn;
		
		public function doAction()
		{
			try {
				miStaticDBUtil::startTransaction();
				$record = new scDomainToRecordAdapter($this->_view->getDomainObject());
				$this->_tableName = $record->getTableName();
				$this->_primaryKeyColumn = $record->getPrimaryKeyColumn();
				$record->readPK(miGetParam($this->_primaryKeyColumn));
				
				$this->doSortAction($record->getPK(), $record->get(self::$_sortColumn));
				miStaticDBUtil::commit();
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				
				$msg = $this->_view->getMessage('MI_RECORD_UPDATE_FAILED_MSG');
				$this->_view->addAjaxErrorRedirect($msg . $exception->getMessage());
				return false;
			}
			$msg = sprintf($this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG'), $record->getPK());
			$this->_view->addAjaxRefreshListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
		
		/**
		 * Set the sort column name
		 * 
		 * @param string $sortColumnName the sort column name
		 */
		static public function setSortColumnName($sortColumnName)
		{
			self::$_sortColumn = $sortColumnName;
		}
		
		/**
		 * Set the foreign key clause
		 * 
		 * @param string $foreignKeyClause the foreign key clause
		 */
		static public function setForeignKeyClause($foreignKeyClause)
		{
			self::$_foreignKeyClause = $foreignKeyClause;
		}
		
		/**
		 * Set the sorts of two records
		 * Avoids duplicate key error by making a temporary update
		 */
		protected function setSort($recordId1, $recordSort1, $recordId2, $recordSort2)
		{
			// Find the max sort
			$sql = 'SELECT MAX(' . self::$_sortColumn . ') AS N FROM ' . $this->_tableName;
			if (self::$_foreignKeyClause)
				$sql .= ' WHERE ' . self::$_foreignKeyClause;
			$rows = miStaticDBUtil::execSelect($sql);
			$maxSort = $rows[0]['N'] + 1;
			
			// Temporary set the first record to max+1
			// This will avoid duplicate key error
			$sql = 'UPDATE ' . $this->_tableName . ' SET ' . self::$_sortColumn . '=' . $maxSort .
				' WHERE ' .	$this->_primaryKeyColumn . '=' .  $recordId1;
			//die($sql);
			miStaticDBUtil::execSQL($sql);
			
			// Set the sorts
			$sql = 'UPDATE ' . $this->_tableName . ' SET ' . self::$_sortColumn . '=' . $recordSort2 .
				' WHERE ' .	$this->_primaryKeyColumn . '=' .  $recordId2;
			miStaticDBUtil::execSQL($sql);
				
			$sql = 'UPDATE ' . $this->_tableName . ' SET ' . self::$_sortColumn . '=' . $recordSort1 .
				' WHERE ' .	$this->_primaryKeyColumn . '=' .  $recordId1;
			miStaticDBUtil::execSQL($sql);
		}
		
		/**
		 * Retrieves the next record in sort order
		 * 
		 * @param int $recordSort the current record sort
		 * @param string $direction the direction - 'ASC' or 'DESC'
		 */
		protected function getNextRecord($recordId, $recordSort, $direction)
		{
			$operator = ($direction == 'ASC') ? '>' : '<';
			$sql = 'SELECT ' . $this->_primaryKeyColumn . ',' . self::$_sortColumn . ' FROM ' . $this->_tableName .
				' WHERE ' . self::$_sortColumn . $operator . $recordSort;
			if (self::$_foreignKeyClause)
				$sql .= ' AND ' . self::$_foreignKeyClause;
			$sql .= ' ORDER BY ' . self::$_sortColumn . ' ' . $direction . ' LIMIT 1';
			return miStaticDBUtil::execSelect($sql);
		}
		
		/**
		 * Perform sort action
		 * 
		 * @param int $recordId
		 * @param int $recordSort
		 */
		protected function doSortAction($recordId, $recordSort)
		{
			// Find the next record
			$rows = $this->getNextRecord($recordId, $recordSort, $this->_direction);
			if (count($rows) == 0)
				return false;
			
			$otherRecordId = $rows[0][$this->_primaryKeyColumn];
			$otherRecordSort = $rows[0][self::$_sortColumn];
			
			// Exchange the sorts
			$this->setSort($recordId, $otherRecordSort, $otherRecordId, $recordSort);
		}
	}

	/**
	 * Move the record up in sort order
	 */
	class scExecMoveUpAction extends scSortAction {
		protected $_direction = 'DESC';
	}
	
	/**
	 * Move the record down in sort order
	 */
	class scExecMoveDownAction extends scSortAction {
		protected $_direction = 'ASC';
	}
?>