<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scUserDisplayController extends miPage {
		
		/**
		 * @var scSkin
		 */
		protected $_skin;
		
		/**
		 * @var scSeo
		 */
		protected $_seo = null;
		
		/**
		 * @var bool
		 */
		protected $_disableLanguageSelection = false;
		
		public function __construct($controllers)
		{
			parent::__construct($controllers);
			$this->_skin = scSkins::singleton()->getCurrentSkin();
			
			$this->setHeader('customer/header.tmpl');
			$this->setFooter('customer/footer.tmpl');			
		}
		
		/**
		 * Set if the language selection drop-down should be disabled
		 */
		public function setDisableLanguageSelection($disable)
		{
			$this->_disableLanguageSelection = $disable;
		}
		
		/**
		 * Set the seo object
		 * 
		 * @param scSeo $seo
		 */
		public function setSeo(scSeo $seo)
		{
			$this->_seo = $seo;
		}
		
		/**
		 * Returns the seo object. Creates scSeo object if neccessary
		 * 
		 * @return scSeo
		 */
		public function getSeo()
		{
			if ($this->_seo == null)
				$this->_seo = new scSeo();
			return $this->_seo;
		}
		
		public function getHeader()
		{
			if (!$this->_headerFileName)
				return '';
			
			$fastMenuSection = scUserMenus::getMenuTemplateSection(scUserMenus::MENU_LOCATION_FASTMENU, 'fastMenuItem');
			$topMenuSection = scUserMenus::getMenuTemplateSection(scUserMenus::MENU_LOCATION_TOP, 'topMenuItem');
			
			$cartSection = $this->getCartSection();
			
			$t = new scTemplateParser();
			$t->readTemplate($this->_headerFileName);
			
			$meta = $this->getSeo()->getMetaData();
			$t->assign('%%COMPANY_NAME%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME)));
			$t->assign('%%TITLE%%', miI18N::htmlEscape($meta[scSeo::TITLE]));
			$t->assign('%%BODY_CLASSES%%', miI18N::htmlEscape($meta[scSeo::BODY_CLASSES]));
			$t->assign('%%META_DESCRIPTION%%', miI18N::htmlEscape($meta[scSeo::DESCRIPTION]));
			$t->assign('%%META_KEYWORDS%%', miI18N::htmlEscape($meta[scSeo::KEYWORDS]));
			$t->assign('%%STORE_ROOT_URL%%', 'http://' . scPaths::getHttpHost() . scPaths::getStoreWebPath());
			$t->assign('%%DISABLE_LANGUAGE%%', $this->_disableLanguageSelection ? 'disabled="disabled"' : '');
			$t->assign('%%LANGUAGE_DIRECTION%%', (scMultiLanguage::getCurrentLanguageDirection() == scLanguageDomainObject::LANGUAGE_DIRECTION_RTL) ? ' dir="rtl"' : '');
			
			// For the currencies
			$currencySelectorHtml = scUIUtil::getConstDropdown('CurrencyID', scCurrencyUtil::getCurrentCurrencyId(), scCurrencyUtil::getActiveCurrencies(), array('id' => 'CurrencyID'));
			$t->assign('%%CURRENCY_SELECTOR%%', $currencySelectorHtml);
			
			$loginSection = new miTemplateParserSectionInfo();
			$auth = scCustomerAuth::singleton();
			if ($auth->isLoggedIn()) {
				$customerFirstName = $auth->getCustomerFirstName();
				$customerLastName = $auth->getCustomerLastName();
				$loginSection->setSectionInfo('logged', 1,
					array('%%CUSTOMERFIRSTNAME%%' => $customerFirstName, '%%CUSTOMERLASTNAME%%' => $customerLastName));
			} else { 
				$loginSection->setSectionInfo('notlogged', 1);
			}
			
			// Language selection
			$languageSelection = new miTemplateParserSectionInfo('lang', 1);
			$this->addLanguagesSection($languageSelection);
			
			// For the search
			$t->assign('%%PHRASE%%', miI18N::htmlEscape(miGetParamDefault(scSearchData::KEY_PHRASE, '')));
			$t->setSectionInfos(array($fastMenuSection, $topMenuSection, $loginSection, $cartSection, $languageSelection));
			
			return $t->templateParse();
		}
		
		/**
		 * Return section with cart info
		 * Can be used in the header/footer
		 * @return miTemplateParserSectionInfo
		 */
		protected function getCartSection()
		{
			$cart = scCart::getCart();
			$itemsCount = $cart->getItemsCount();
			if (!$itemsCount) {
				return new miTemplateParserSectionInfo('cart_empty', 1);
			}
			
			$values = array();
			$values['%%CART_ITEMS_COUNT%%'] = $itemsCount;
			$values['%%CART_ITEMS_WORD%%'] = ($itemsCount == 1) ? _t('item') : _t('items');
			
			$subtotal = scPrice::factory($cart->getSubtotal());
			$values['%%SUBTOTAL%%'] = $subtotal->getDisplayPrice();
			
			return new miTemplateParserSectionInfo('cart_not_empty', 1, $values);
		}
		
		public function getFooter()
		{
			$footerMenuSection = scUserMenus::getMenuTemplateSection(scUserMenus::MENU_LOCATION_FOOTER, 'footerMenuItem');
			
			$t = new scTemplateParser;
			$t->readTemplate($this->_footerFileName);
			
			// Assign company info
			$countryId = miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_COUNTRY));
			$countryName = $countryId ? scCountry::getCountryName($countryId) : '';
			
			$stateId = miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_STATE));
			$stateName = $stateId ? ', ' . scCountry::getStateName($stateId) : '';
			
			$t->assign('%%COMPANY_ADDRESS%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_ADDRESS)));
			$t->assign('%%COMPANY_ADDRESS2%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_ADDRESS2)));
			$t->assign('%%COMPANY_CITY%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_CITY)));
			$t->assign('%%COMPANY_ZIP%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_ZIP)));
			$t->assign('%%COMPANY_COUNTRY%%', miI18N::htmlEscape($countryName));
			$t->assign('%%COMPANY_STATE%%', miI18N::htmlEscape($stateName));
			$t->assign('%%COMPANY_PHONE%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_PHONE)));
			$t->assign('%%COMPANY_PHONE2%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_PHONE2)));
			$t->assign('%%COMPANY_FAX%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_FAX)));
			
			$t->assign('%%HELP_SUPPORT_EMAIL%%', miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_EMAIL_HELP_SUPPORT)));
			
			$t->setSectionInfos(array($footerMenuSection));
			return $t->templateParse();
		}
		
		public function showPage()
		{
			// XXX: Implement a router/locator - an object that knows what page is being processed.
			$file = '';
			if ($this->getSeo() instanceof scDynamicPageSeo) {
				$file = $this->getSeo()->getPagePath();
			}
			
			scBonusPoints::handleBonusPointsReferral();
			
			$mainControllerCommands = array();
			$isHtml = true;
			foreach ($this->_views as $view) {
				$viewCommands = $view->getControllerCommands();
				foreach ($viewCommands as $command) {
					if ($command->getCommandType() != miControllerCommand::CONTROLLER_COMMAND_HTML) {
						$isHtml = false;
						break;
					}
				}
				$mainControllerCommands = array_merge($mainControllerCommands, $viewCommands);
			}
			
			$commands = array();
			if ($isHtml && (miGetParamDefault('load', '') != 'table')) {// If this is not an AJAX request
				$html = $this->getHeader();
				$commands[] = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$html = $this->_skin->getColumn(scLayout::COLUMN_TOP, scLayout::AREA_CUSTOMER, $file);
				$commands[] = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$html = $this->_skin->getColumn(scLayout::COLUMN_LEFT, scLayout::AREA_CUSTOMER, $file);
				$commands[] = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			}
			
			$commands = array_merge($commands, $mainControllerCommands);
			
			if ($isHtml && (miGetParamDefault('load', '') != 'table')) {// If this is not an AJAX request
				$html = $this->_skin->getColumn(scLayout::COLUMN_RIGHT, scLayout::AREA_CUSTOMER, $file);
				$commands[] = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$html = $this->_skin->getColumn(scLayout::COLUMN_BOTTOM, scLayout::AREA_CUSTOMER, $file);
				$commands[] = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$html = $this->getFooter();
				$commands[] = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			}
			
			// Do not flush the output unless we say so
			ob_implicit_flush(false);
			
			$this->processControllerCommands($commands);
			
			// Flush the gzipped output buffer.
			ob_end_flush();
			// Get the gzipped length for the Content-Length header
			header('Content-Length: ' . ob_get_length());
			header('Connection: close');
			// Flush the contents
			ob_end_flush();
			flush();
		}
		
		/**
		 * Process the controller commands
		 * 
		 * @param miControllerCommand[] $controllerCommands
		 */
		public function processControllerCommands($controllerCommands)
		{
			parent::processControllerCommands($controllerCommands);
			
			$countPageView = true;
			foreach ($controllerCommands as $command) {
				if ($command->getCommandType() == miControllerCommand::CONTROLLER_COMMAND_REDIRECT) {
					$countPageView = false;
					break;
				}
			}
			
			if ($countPageView) {
				$this->countPageView();
			}
		}
		
		protected function countPageView()
		{
			if (!isset($_SERVER['HTTP_USER_AGENT']) || ((strpos($_SERVER['HTTP_USER_AGENT'], 'Mozilla') === false) && (strpos($_SERVER['HTTP_USER_AGENT'], 'Opera') === false))) {
				// It is not a known browser.
				return;
			}
			
			$uniqueVisitsIncrement = 0;
			if (!isset($_SESSION['uniqueVisit'])) {
				$_SESSION['uniqueVisit'] = 1;
				$uniqueVisitsIncrement = 1;
			}
			
			$q = 'INSERT INTO StatViewsVisits (StatViewsVisitsDate, StatViewsCount, StatVisitsCount) VALUES' .
				' (?, 1, 1) ON DUPLICATE KEY UPDATE' .
					' StatViewsCount = StatViewsCount + 1,' .
					' StatVisitsCount = StatVisitsCount + ?';
			miStaticDBUtil::execSQL($q, array(date(MI_MYSQL_DATE_FORMAT), $uniqueVisitsIncrement));
		}
		
		protected function addLanguagesSection(miTemplateParserSectionInfo $main)
		{
			$hasLanguageSelection = $this->getSeo()->isMultilanguage();
			$languages = scMultiLanguage::singleton()->getActiveLanguages();
			$curLanguageCode = scMultiLanguage::getCurrentLanguageCode();
			
			if ($hasLanguageSelection) {
				$urlData = $this->getSeo()->getUrlParams();
				$resolver = scUrlResolverFactory::getUrlResolverForPageType($urlData[scSeoUrlTypes::PAGE_TYPE]);
				$names = $resolver->getNames($urlData[scSeoUrlTypes::PAGE_ID]);
			}
			
			$isSelectedSections = array();
			$sectionLanguages = array();
			foreach ($languages as &$language) {
				$isSelected = ($language['LanguageCode'] == $curLanguageCode);
				if (!$hasLanguageSelection && !$isSelected) {
					continue;
				}
				
				$name = '';
				if (isset($names[$language['LanguageID']])) {
					$name = $names[$language['LanguageID']];
				}
				
				if ($hasLanguageSelection) {
					$url = scFriendlyUrl::get($urlData[scSeoUrlTypes::PAGE_TYPE], $urlData[scSeoUrlTypes::PAGE_ID], $name, $language['LanguageCode']);
					$language['LanguageUrl'] = $url;
					$language['LanguageSelected'] = ($isSelected ? ' selected="selected" ' : '');
					$language['LanguageImage'] = scPaths::getStoreWebPath() . $language['LanguageImage'];
				}
				
				$isSelectedSections[] = new miTemplateParserSectionInfo('isSelected', $isSelected);
				$sectionLanguages[] = $language;
				
				if ($isSelected) {
					$selectedSection = new miTemplateParserSectionInfo('selectedLanguage');
					$selectedSection->setIterationsCount(1);
					$selectedSection->addValues(miUIUtil::templetizeAndEscapeRows($language));
					$main->addSubsection($selectedSection);
				}
			}
			
			if ($hasLanguageSelection) {
				$languagesSection = new miTemplateParserSectionInfo('languages', count($sectionLanguages), miUIUtil::templetizeRows(miUIUtil::transposeRows($sectionLanguages)));
				$languagesSection->addSubsectionsArray('isSelected', $isSelectedSections);
				$hasLanguageSelectionSection = new miTemplateParserSectionInfo('hasLanguageSelection', 1);
				$hasLanguageSelectionSection->addSubsection($languagesSection);
				$main->addSubsection($hasLanguageSelectionSection);
			}
		}
	}
?>