<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scAdministratorDomainObject extends miDefaultDomainObject {
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('Administrators', 'AdministratorID'));
		}
		
		public function set($name, $value)
		{
			// Hash the password
			if ('AdministratorPasswordHashMode' == $name)
				return;
			
			if ('AdministratorPassword' == $name) {
				$hashMode = scSettings::getSettingValue(scSettings::SETTING_PASSWORD_HASH_MODE);
				$this->_record->set('AdministratorPasswordHashMode', $hashMode);
				
				$auth = scAdminAuth::singleton();
				$value = $auth->hashPassword($value);
			}
			$this->_record->set($name, $value);
		}
		
		public function getTableName()
		{
			return $this->_record->getTableName();
		}
		
		public function delete($pkValue = null)
		{
			if ($pkValue !== null) {
				$this->_record->set($this->_record->getPrimaryKeyColumn(), $pkValue);
			}
			
			$toDeleteId = $this->_record->get($this->_record->getPrimaryKeyColumn());
			
			if (scAdminAuth::singleton()->getAdministratorId() == $toDeleteId) {
				throw new miException(_t('You cannot delete the account you are currently logged in with.'));
			}
			
			return parent::delete($pkValue);
		}
		
		/**
		 * @throws miException
		 */
		public function validateOnUpdate()
		{
			// The new values are already set.
			$newAccessLevel = $this->get('AdministratorAccessLevel');
			// We need the old ones.
			$record = clone $this->_record;
			$record->readPK($this->get('AdministratorID'));
			$oldAccessLevel = $record->get('AdministratorAccessLevel');
			
			if ($oldAccessLevel == $newAccessLevel)
				return;// The access level has not been modified
			
			// The old access level differs from the new one.
			// We need to check if the old one was "Full access".
			if ($oldAccessLevel != scAdminAuth::ADMINISTRATOR_ACCESS_LEVEL_FULL_ACCESS)
				return;
			
			$recordset = new miSqlRecordset('Administrators');
			$recordset->addFilter(new miSqlFilterEqual('AdministratorAccessLevel', scAdminAuth::ADMINISTRATOR_ACCESS_LEVEL_FULL_ACCESS));
			$currentFullAccessAdmins = $recordset->getRecordsCount();
			
			if ($currentFullAccessAdmins == 1) {
				throw new miException(_t('You cannot limit and/or disable the only full access account.'));
			}
		}
	}
?>