<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scBonusPointsTransaction extends scDomainObject {
		
		/**
		 * @param int $actionType
		 * @param int $pointsAmount
		 * @param string $actionIdentifier
		 * @param int $customerId
		 * @param string $description
		 * @param int $description
		 * @return scBonusPointsTransaction
		 */
		public static function create($actionType, $pointsAmount, $actionIdentifier, $customerId, $description, $reason = 0, $autoApprove = false)
		{
			$pointsAction = new scBonusPointsTransaction();
			$pointsAction->set('CustomerID', $customerId);
			$pointsAction->set('BonusPointsTransactionType', $actionType);
			$pointsAction->set('BonusPointsTransactionDescription', $description);
			$pointsAction->set('BonusPointsAmount', $pointsAmount);
			$pointsAction->set('BonusPointsTransactionIdentifier', $actionIdentifier);
			$pointsAction->set('BonusPointsTransactionReason', $reason);
			$pointsAction->insert();
			
			if (($actionType === scBonusPoints::ACTION_SPEND) || !scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_MANUAL_APPROVAL) || $autoApprove) {
				// If manual approval is not turned on the transaction may be approved.
				$pointsAction->approve();
			}
			
			return $pointsAction;
		}
		
		/**
		 * @return miSqlRecordset
		 */
		public static function getRecordset($actionType, $actionIdentifier, $customerId, $reason = 0)
		{
			$recordset = new miSqlRecordset('BonusPointsTransactions');
			$recordset->addFilter(new miSqlFilterEqual('CustomerID', $customerId));
			$recordset->addFilter(new miSqlFilterEqual('BonusPointsTransactionType', $actionType));
			$recordset->addFilter(new miSqlFilterEqual('BonusPointsTransactionReason', $reason));
			$recordset->addFilter(new miSqlFilterEqual('BonusPointsTransactionIdentifier', $actionIdentifier));
			return $recordset;
		}
		
		/**
		 * @var scBonusPointsTransaction
		 */
		protected $_readCopy = null;
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('BonusPointsTransactions', 'BonusPointsTransactionID'));
		}
		
		public function insert()
		{
			if (!$this->has('BonusPointsTransactionStatus')) {
				$this->set('BonusPointsTransactionStatus', scBonusPoints::TRANSACTION_STATUS_PENDING);
			}
			if (!$this->has('BonusPointsTransactionTime')) {
				$this->set('BonusPointsTransactionTime', time());
			}
			
			$return = parent::insert();
			
			$this->updateBonusPointsBalance();
			
			return $return;
		}
		
		public function update()
		{
			$return = parent::update();
			
			$this->updateBonusPointsBalance();
			
			return $return;
		}
		
		public function approve()
		{
			miStaticDBUtil::startTransaction();
			$this->set('BonusPointsTransactionStatus', scBonusPoints::TRANSACTION_STATUS_APPROVED);
			$this->update();
//			$this->applyBonusPoints();
			miStaticDBUtil::commit();
		}
		
		public function void()
		{
			miStaticDBUtil::startTransaction();
			$this->set('BonusPointsTransactionStatus', scBonusPoints::TRANSACTION_STATUS_VOID);
			$this->update();
//			$this->revertBonusPoints();
			miStaticDBUtil::commit();
		}
		
		public function delete($pkValue = null)
		{
			$pkValue = $this->getPkValueArray($pkValue);
			if (empty($pkValue)) {
				return $pkValue;
			}
			
			if (count($pkValue) == 1) {
				if (!$this->has('BonusPointsAmount') || !$this->has('BonusPointsTransactionType')) {
					$this->read(reset($pkValue));
				}
//				$this->revertBonusPoints();
				$result = parent::delete($pkValue);
				$this->updateBonusPointsBalance();
				return $result;
			}
			
			$data = new miSqlRecordset('BonusPointsTransactions');
			$data->addFilter(new miSqlFilterIn($pkValue));
			$data = $data->getAllRecords();
			$toDelete = new scBonusPointsTransaction();
			foreach ($data as $row) {
				$toDelete->setRow($row);
				$toDelete->delete();
			}
			
			return true;
		}
		
		public function read($pkValue)
		{
			$result = parent::read($pkValue);
			
			$this->_readCopy = clone $this;
			
			return $result;
		}
		
		/**
		 * @return scBonusPointsTransaction
		 */
		protected function getReadCopy()
		{
			if (is_null($this->_readCopy)) {
				throw new miException(_i('Object has not been read.'));
			}
			
			return $this->_readCopy;
		}
		
/*		protected function applyBonusPoints()
		{
			$pointsModification = (int)$this->get('BonusPointsAmount');
			if ($pointsModification == 0) {
				// Nothing to do.
				return;
			}
			
			$type = $this->get('BonusPointsTransactionType');
			if ($type == scBonusPoints::ACTION_SPEND) {
				// Points should be spent
				$pointsModification = - $pointsModification;
			}
			
			$this->updateBonusPoints($pointsModification);
		}
		
		protected function revertBonusPoints()
		{
			$pointsModification = (int)$this->get('BonusPointsAmount');
			if ($pointsModification == 0) {
				// Nothing to do.
				return;
			}
			
 			$previousStatus = $this->getReadCopy()->get('BonusPointsTransactionStatus');
 			if ($previousStatus != scBonusPoints::TRANSACTION_STATUS_APPROVED) {
 				// The transaction was not approved so nothing has to be reverted
 				return;
 			}
			
			$type = $this->get('BonusPointsTransactionType');
 			if ($type == scBonusPoints::ACTION_ASSIGN) {
				// Given points should be taken back
 				$pointsModification = - $pointsModification;
			}
			
			$this->updateBonusPoints($pointsModification);
		}
		
		protected function updateBonusPoints($pointsModification)
		{
			$sql = 'UPDATE Customers SET CustomerBonusPoints = CustomerBonusPoints + ? WHERE CustomerID = ?';
			miStaticDBUtil::execSQL($sql, array($pointsModification, $this->get('CustomerID')));
		}*/
		
		protected function updateBonusPointsBalance()
		{
			miStaticDBUtil::execSQL('UPDATE Customers SET CustomerBonusPoints = ? WHERE CustomerID = ?', array(scBonusPoints::recalculateCurrentBalance($this->get('CustomerID')), $this->get('CustomerID')));
		}
	}
?>