<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCategoryDomainObject extends scDomainObject {
		
		/**
		 * @var array
		 */
		protected $_deactivatedDiscounts = array();
		
		/**
		 * @var array
		 */
		protected $_deactivatedCoupons = array();
		
		protected $_oldParentID = null;
		
		public function __construct()
		{
			$recordMlFields = array('CategoryName', 'CategoryDescription', 'CategorySEOTitle', 'CategorySEOKeywords', 'CategorySEODescription');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			parent::__construct(new scSqlRecordML('Categories', 'CategoryID', $recordMlFields, $recordLanguages));
		}
		
		/**
		 * @return array
		 */
		public function getDeactivatedDiscounts()
		{
			return $this->_deactivatedDiscounts;
		}
		
		/**
		 * @return array
		 */
		public function getDeactivatedCoupons()
		{
			return $this->_deactivatedCoupons;
		}
		
		/**
		 * Set domain object value
		 *
		 * @param string $name
		 * @param mixed $value
		 */
		public function set($name, $value)
		{
			if ($name == 'CategoryParentID' && intval($value) == 0)
				parent::set($name, null);
			else
				parent::set($name, $value);
		}
		
		public function read($pkValue)
		{
			parent::read($pkValue);
			$this->_oldParentID = $this->get('CategoryParentID');
		}
		
		public function update()
		{
			if ($this->get('CategoryParentID') != $this->_oldParentID) {
				$sql = 'SELECT MAX(CategorySort) AS N FROM Categories';
				if (is_null($this->get('CategoryParentID'))) {
					$sql .= ' WHERE CategoryParentID IS NULL';
				} else {
					$sql .= ' WHERE CategoryParentID = ' . intval($this->get('CategoryParentID'));
				}
				$rows = miStaticDBUtil::execSelect($sql);
				$this->set('CategorySort', $rows[0]['N']+1);
			}
			
			return parent::update();
		}
		
		protected function onDelete($pkValue)
		{
			$this->deleteSubcategories($pkValue);
			$this->deleteProducts($pkValue);
			
			$this->deactivateDiscounts($pkValue);
		}
		
		protected function deleteSubcategories($pkValue)
		{
			// Get direct subcatergories
			$sql = sprintf('SELECT * FROM Categories WHERE CategoryParentID IN (%s)', implode(',', $pkValue));
			$rows = miUtil::getDBArray($sql, 'CategoryID', 'CategoryID');
			if (count($rows)) {
				// Delete direct subcategories if such exist
				$this->delete($rows);
			}
		}
		
		protected function deleteProducts($categoryIds)
		{
			// Get all products in those categories
			$productsQuery = 'SELECT ProductID FROM Products WHERE CategoryID IN (' . implode(', ', $categoryIds) . ')';
			$productIds = miUtil::getDBArray($productsQuery, 'ProductID', 'ProductID');
			
			// Delete the products
			$products = new scProductDomainObject();
			$products->delete($productIds);
			$this->addDeactivatedDiscounts($products->getDeactivatedDiscounts());
			$this->addDeactivatedCoupons($products->getDeactivatedCoupons());
		}
		
		/**
		 * @param array $productIds
		 */
		protected function deactivateDiscounts($categoryIds)
		{
			foreach ($categoryIds as $categoryId) {
				$discounts = scDiscountService::validateDiscounts(
					'Discounts', 'DiscountID', 'DiscountCategories', 'CategoryID', 'DiscountIsActive', $categoryId);
				$this->addDeactivatedDiscounts($discounts);
				
				$coupons = scDiscountService::validateDiscounts(
					'Coupons', 'CouponID', 'CouponCategories', 'CategoryID', 'CouponIsActive', $categoryId);
				$this->addDeactivatedCoupons($coupons);
			}
		}
		
		/**
		 * @param array $discounts
		 */
		protected function addDeactivatedDiscounts($discounts)
		{
			$this->_deactivatedDiscounts = array_merge($this->_deactivatedDiscounts, $discounts);
		}
		
		/**
		 * @param array $coupons
		 */
		protected function addDeactivatedCoupons($coupons)
		{
			$this->_deactivatedCoupons = array_merge($this->_deactivatedCoupons, $coupons);
		}
	}
?>