<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCustomUrlDomainObject extends scDomainObject {
		
		public static $fieldNameByEntityCode = array(
			scFriendlyUrl::TYPE_PRODUCT => 'ProductID',
			scFriendlyUrl::TYPE_CATEGORY => 'CategoryID',
			scFriendlyUrl::TYPE_MANUFACTURER => 'ManufacturerID',
			scFriendlyUrl::TYPE_DYNAMIC_FORM => 'CMSFormID',
			scFriendlyUrl::TYPE_RICH_TEXT_PAGE => 'CMSRichTextPageID',
			scFriendlyUrl::TYPE_GALLERY => 'GalleryID',
			scFriendlyUrl::TYPE_NEWS => 'NewsID',
			scFriendlyUrl::TYPE_DYNAMIC_PAGE => 'scFriendlyUrl_Dynamic_Page',
		);
		
		public static function getEntityName($entityCode)
		{
			$entityNames = self::getAllEntities();
			return $entityNames[$entityCode];
		}
		
		public static function getAllEntities()
		{
			$entityNames = array(
				scFriendlyUrl::TYPE_PRODUCT => _t('Product'),
				scFriendlyUrl::TYPE_CATEGORY => _t('Category'),
				scFriendlyUrl::TYPE_MANUFACTURER => _t('Manufacturer'),
				scFriendlyUrl::TYPE_DYNAMIC_FORM => _t('Form'),
				scFriendlyUrl::TYPE_RICH_TEXT_PAGE => _t('Rich Text Page'),
				scFriendlyUrl::TYPE_GALLERY => _t('Gallery'),
				scFriendlyUrl::TYPE_NEWS => _t('News'),
				scFriendlyUrl::TYPE_DYNAMIC_PAGE => _t('Dynamic Page'),
			);
			return $entityNames;
		}
		
		public function __construct()
		{
			parent::__construct(new scSqlRecordML('CustomUrls', 'CustomUrlID',
				array('CustomUrlText', 'CustomUrlHash'),
				array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
		
		public function insert()
		{
			$this->calculateHashes();
			
			return parent::insert();
		}
		
		public function update()
		{
			$this->calculateHashes();
			
			return parent::update();
		}
		
		protected function calculateHashes()
		{
			$texts = $this->get('CustomUrlText');
			foreach ($texts as $languageId => $url) {
				// Calculate URL hashes based on the base path.
				// e.g. for the URL "/sad/blah.php?something=2" base the hash on "/sad/blah.php".
				// This would cause same hashes for that URL and for "/sad/blah.php?x=1&y=2"
				// Checking if the query strings match will be done dynamically.
				$parsed = scUrlUtil::parse($url);
				if (isset($parsed[scUrlUtil::QUERY])) {
					unset($parsed[scUrlUtil::QUERY]);
				}
				$baseUrl = scUrlUtil::glue($parsed);
				
				$hashes[$languageId] = sprintf('%u', crc32($baseUrl));
			}
			$this->set('CustomUrlHash', $hashes);
		}
	}
?>