<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCustomerAddress extends scDomainObject {
		
		const BILLING = 1;
		const SHIPPING = 2;
		
		protected static $_mapping = array(
			self::BILLING => array(
				'CustomerBillingFirstName' => 'CustomerAddressFirstName',
				'CustomerBillingLastName' => 'CustomerAddressLastName',
				'CustomerBillingCompany' => 'CustomerAddressCompany',
				'CustomerBillingAddress1' => 'CustomerAddressLine1',
				'CustomerBillingAddress2' => 'CustomerAddressLine2',
				'CustomerBillingCountry' => 'CountryID',
				'CustomerBillingState' => 'StateID',
				'CustomerBillingZip' => 'CustomerAddressZip',
				'CustomerBillingCity' => 'CustomerAddressCity',
				'CustomerBillingCityID' => 'CityID',
				'CustomerBillingPhone' => 'CustomerAddressPhone',
				'CustomerBillingFax' => 'CustomerAddressFax',
				'CustomerBillingVatNumber' => 'CustomerAddressVatNumber',
			),
			
			self::SHIPPING => array(
				'CustomerShippingFirstName' => 'CustomerAddressFirstName',
				'CustomerShippingLastName' => 'CustomerAddressLastName',
				'CustomerShippingCompany' => 'CustomerAddressCompany',
				'CustomerShippingAddress1' => 'CustomerAddressLine1',
				'CustomerShippingAddress2' => 'CustomerAddressLine2',
				'CustomerShippingCountry' => 'CountryID',
				'CustomerShippingState' => 'StateID',
				'CustomerShippingZip' => 'CustomerAddressZip',
				'CustomerShippingCity' => 'CustomerAddressCity',
				'CustomerShippingCityID' => 'CityID',
				'CustomerShippingPhone' => 'CustomerAddressPhone',
				'CustomerShippingFax' => 'CustomerAddressFax',
				'CustomerShippingVatNumber' => 'CustomerAddressVatNumber',
			)
		);
		
		public static function getAddressFields($type)
		{
			return self::$_mapping[$type];
		}
		
		/**
		 * Maps customer address to billing/shipping address
		 * @param int $addressType
		 * @param array $address
		 * @return array
		 */
		public static function getMappedAddressFields($addressType, $address)
		{
			$data = array();
			$mapping = scCustomerAddress::getAddressFields($addressType);
			foreach ($mapping as $checkoutField => $addressField) {
				$data[$checkoutField] = $address[$addressField];
			}
			return $data;
		}
		
		/**
		 * @param array $address
		 * @return string
		 */
		public static function getFormattedAddress($address)
		{
			static $countryNames = null;
			static $stateNames = null;
			
			$languageId = scMultiLanguage::getCurrentLanguageId();
			if (is_null($countryNames) || !isset($countryNames[$languageId])) {
				$countryNames[$languageId] = scCountry::getCountryNames();
			}
			if (is_null($stateNames) || !isset($stateNames[$languageId])) {
				$stateNames[$languageId] = scCountry::getStateNames();
			}
			
			$address['CountryName'] = (isset($countryNames[$languageId][$address['CountryID']]) ? $countryNames[$languageId][$address['CountryID']] : '');
			$address['StateName'] = (isset($stateNames[$languageId][$address['StateID']]) ? $stateNames[$languageId][$address['StateID']] : '');
			
			$lines = array();
			$lines[] = $address['CustomerAddressFirstName'] . ' ' . $address['CustomerAddressLastName'];
			
			$line = array_filter(array($address['CustomerAddressCity'], $address['CountryName'], $address['StateName']));
			$lines[] = (!empty($address['CustomerAddressZip']) ? $address['CustomerAddressZip'] . ' ' : '') . implode(', ', $line);
			
			$lines[] = $address['CustomerAddressLine1'];
			$lines[] = $address['CustomerAddressLine2'];
			
			if (!empty($address['CustomerAddressPhone'])) {
				$lines[] = _t('Phone:') . $address['CustomerAddressPhone'];
			}
			if (!empty($address['CustomerAddressFax'])) {
				$lines[] = _t('Fax:') . $address['CustomerAddressFax'];
			}
			
			return implode('<br />', array_filter($lines));
		}
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('CustomerAddresses', 'CustomerAddressID'));
		}
		
		public function set($field, $value)
		{
			if (($field == 'StateID') && empty($value)) {
				$value = null;
			}
			// Allow setting the PK value
			$this->_record->set($field, $value);
		}
		
		public function insert()
		{
			$this->updateCityId();
			return parent::insert();
		}
		
		public function update()
		{
			$this->updateCityId();
			$customerAddressId = parent::update();
			
			// Clear the hash code so that a third party system can find out that we updated the client record
			miStaticDBUtil::execUpdate('SyncItems', array('SyncItemHashCode' => ''),
				array('SyncItemHandler', 'SyncItemLocalID'), array('scXmlSyncCustomerAddresses', $customerAddressId));
			
			return $customerAddressId;
		}
		
		public function delete($pkValue = null)
		{
			if (is_array($pkValue)) {
				throw new miException(_i('Multiple customer addresses cannot be deleted at once at this point.'));
			}
			
			miStaticDBUtil::startTransaction();
			// Read the address before deleting it.
			$this->read($pkValue);
			
			// Check if that's not the customer's only address.
			$query = 'SELECT COUNT(CustomerAddressID) AS N FROM CustomerAddresses WHERE CustomerID = ?';
			$result = miStaticDBUtil::execSelect($query, array($this->get('CustomerID')));
			if (!is_array($result) || !isset($result[0]) || !isset($result[0]['N'])) {
				throw new miException(_i('Unable to get customer addresses count.'));
			}
			
			if ($result[0]['N'] <= 1) {
				throw new miException(_t('Customer\'s last address cannot be deleted.'));
			}
			
			parent::delete($pkValue);
			
			// That may have been the default billing and/or shipping address so another one may need to be set as such.
			if ($this->get('CustomerAddressIsDefaultBilling')) {
				$query = 'UPDATE CustomerAddresses SET CustomerAddressIsDefaultBilling = 1' .
					' WHERE CustomerID = ? LIMIT 1';
				miStaticDBUtil::execSQL($query, array($this->get('CustomerID')));
			}
			if ($this->get('CustomerAddressIsDefaultShipping')) {
				$query = 'UPDATE CustomerAddresses SET CustomerAddressIsDefaultShipping = 1' .
					' WHERE CustomerID = ? LIMIT 1';
				miStaticDBUtil::execSQL($query, array($this->get('CustomerID')));
			}
			
			miStaticDBUtil::commit();
			return true;
		}
		
		/**
		 * Updates the CityID from the other info in the address
		 * Checks if the supplied CityName and CityID are for the same record.
		 * The problem lies if there is a CityID passed that doesn't correspond to the CityName
		 * @return int or boolean
		 */
		protected function updateCityId() 
		{
			if (!$this->has('CustomerAddressCity')) {
				return;
			}
			
			$cityId = $this->has('CityID') ? $this->get('CityID') : null;
			$stateId = $this->has('StateID') ? $this->get('StateID') : null;
			
			// if there is not CityID set try to get one
			if (empty($cityId)) {
				$cityId = self::findCityId($stateId, $this->get('CustomerAddressCity'));
				$this->set('CityID', $cityId);
				return;
			}
			
			// check if the CityID correspond to the CityName
			$sql = 'SELECT COUNT(1) AS N FROM CitiesML WHERE CityName = ? AND CityID = ?';
			$res = miStaticDBUtil::execSelect($sql, array($this->get('CustomerAddressCity'), $cityId));
			if ($res[0]['N'] == 0) {
				$this->set('CityID', null);
			}
		}
		
		/**
		 * Returns the CityID for this city
		 * If not found returns null
		 * 
		 * @param int $stateId
		 * @param string $cityName the city name, could be in any language
		 * @return int|null
		 */
		public static function findCityId($stateId, $cityName)
		{
			if (empty($stateId)) {
				return null;
			}
			
			$sql = 'SELECT CitiesML.CityID FROM CitiesML INNER JOIN Cities USING(CityID)
				WHERE CityName=? AND StateID=? LIMIT 1';
			$res = miStaticDBUtil::execSelect($sql, array($cityName, $stateId));
			if (empty($res)) {
				return null;
			}
			
			return $res[0]['CityID'];
		}
		
		/**
		 * Returns the Addresses Count by CustomerID
		 * 
		 * @param int $customerId
		 */
		public static function getAddressesCountByCustomerId($customerId)
		{
			$addressesCount = miStaticDBUtil::execSelect('SELECT COUNT(CustomerAddressID) AS N FROM CustomerAddresses WHERE CustomerID = ?', array($customerId));
			if (!empty($addressesCount))
				return $addressesCount[0]['N'];
			
			return 0;
		}
	}
?>