<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scDomainObject extends miDefaultDomainObject {
		
		/**
		 * Allows deletion of more than one record.
		 * @param int|array|null $pkValue
		 * @return int
		 */
		public function delete($pkValue = null)
		{
			$pkValue = $this->getPkValueArray($pkValue);
			if (empty($pkValue)) {
				return $pkValue;
			}
			
			$this->onDelete($pkValue);
			
			$deleteQuery = 'DELETE FROM ' . $this->getTableName() . ' WHERE ' .
				$this->getPKName() . ' IN (' . implode(', ', $pkValue) . ')';
			miStaticDBUtil::execSQL($deleteQuery);
			
			return true;
		}
		
		/**
		 * @return string
		 */
		public function getTableName()
		{
			return $this->_record->getTableName();
		}
		
		/**
		 * Subclasses may delete from tables with foreign key referencing the table of this record.
		 * Called just before the main record is deleted
		 * @param array $pkValueArray
		 */
		protected function onDelete($pkValueArray)
		{
		}
		
		/**
		 * Returns an array of primary key values to be used by delete()
		 * @return array
		 * @throws miDBException
		 */
		protected function getPkValueArray($pkValue)
		{
			if (is_null($pkValue)) {
				// id is not passed.
				// We'll try to find it in the data row.
				$dataRow = $this->getRow();
				$pkName = $this->getPKName();
				if (!isset($dataRow[$pkName]) || empty($dataRow[$pkName])) {
					throw new miDBException(_i('Primary key not set.'));
				}
				$pkValue = array($dataRow[$pkName]);
			}
			
			if (!is_array($pkValue)) {
				$pkValue = array(intval($pkValue));
			}
			
			return array_map('intval', $pkValue);
		}
	}
?>