<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	abstract class scEgoodDomainObject extends miDefaultDomainObject {
		
		/**
		 * The method would be called on subclasses to tell this abstract class
		 * which is the File field containing the path to the E-Good
		 * @return string
		 */
		abstract protected function getFileField();
		
		/**
		 * Subclasses may implement this method in order to give a reason
		 * for the file not being available for download.
		 * That is the first method called in download()
		 */
		public function checkIsDownloadable() {}
		
		/**
		 * Sends appropriate headers and output the E-Good.
		 */
		public function download()
		{
			$this->checkIsDownloadable();
			$filePath = $this->getFilePath();
			$handle = $this->getHandle($filePath);
			$this->sendHeaders($filePath);
			$this->increaseDownloadCount();
			$this->passThruHandle($handle);
		}
		
		/**
		 * @return string
		 */
		protected function getFilePath()
		{
			return scPaths::getItemPath($this->get($this->getFileField()), scPaths::RESOURCE_BASE_EGOOD);
		}
		
		/**
		 * @param string $filePath
		 * @return resource
		 * @throws miException
		 */
		protected function getHandle($filePath)
		{
			$handle = @fopen($filePath, 'rb');
			if ($handle === false) {
				throw new miException(sprintf(_t('File "%s" not found.'), basename($filePath)));
			}
			return $handle;
		}
		
		/**
		 * @param string $filePath
		 */
		protected function sendHeaders($filePath)
		{
			if (ob_get_length() !== false) {
				ob_end_clean();
			}
			header('Content-Encoding: none');
			header('Pragma: public');
			header('Cache-control: private, must-revalidate, max-age=0');
			header('Expires: 0');
			header('Content-Type: ' . $this->getFileContentType($filePath));
			header('Content-length: ' . filesize($filePath));
			header('Content-disposition: attachment; filename="' . basename($filePath) . '"');
			header('Connection: close');
		}
		
		/**
		 * Subclass may implement this method to count the number of downloads.
		 * It is called right after sendHeaders()
		 */
		protected function increaseDownloadCount() {}
		
		/**
		 * @param resource $handle
		 */
		protected function passThruHandle(&$handle)
		{
			fpassthru($handle);
			fclose($handle);
		}
		
		/**
		 * @return string
		 */
		protected function getFileContentType($filePath)
		{
			$contentType = '';
			if (function_exists('mime_content_type'))
				$contentType = mime_content_type($filePath);
			
			if ($this->isValidContentType($contentType))
				return $contentType;
			
			return 'application/octet-stream';
		}
		
		/**
		 * @param string $contentType
		 * @return bool
		 */
		protected function isValidContentType($contentType)
		{
			if (strpos($contentType, ' ') !== false)
				return false;
			if (count(explode('/', $contentType)) != 2)
				return false;
			return true;
		}
	}
?>