<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scGalleryDomainObject extends scDomainObject {
		
		public function __construct()
		{
			parent::__construct(new scSqlRecordML('Galleries', 'GalleryID',
				array('GalleryTitle', 'GalleryDescription', 'GallerySEOTitle'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
		
		public function __clone()
		{
			$this->_record = clone $this->_record;
		}
		
		public function insert()
		{
			try {
				miStaticDBUtil::startTransaction();
				$dirName = scFilesUtil::escapeStringForFilename(trim($this->get('GalleryName')));
				$this->set('GalleryDirName', $dirName);
				
				$galleryId = parent::insert();
				
				if (empty($dirName)) {
					$this->set('GalleryDirName', $galleryId);
					$this->update();
				}
				
				$galleriesPath = scPaths::getPath(scPaths::RESOURCE_GALLERY_IMAGE);
				$fullPath = scPaths::getStoreRootDir() . $galleriesPath . $this->get('GalleryDirName') . '/';
				if (!file_exists($fullPath) && !mkdir($fullPath, 0777, true)) {
					throw new miException(_t('Could not create gallery directory.'));
				}
				
				$largeImages = $fullPath . scPaths::GALLERY_LARGE_IMAGES;
				if (!file_exists($largeImages) && !mkdir($largeImages, 0777, true)) {
					throw new miException(_t('Could not create gallery images directory.'));
				}
				
				$thumbImages = $fullPath . scPaths::GALLERY_THUMB_IMAGES;
				if (!file_exists($thumbImages) && !mkdir($thumbImages, 0777, true)) {
					throw new miException(_t('Could not create gallery thumbnails directory.'));
				}
				
				miStaticDBUtil::commit();
				return $galleryId;
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e;
			}
		}
		
		public function update()
		{
			$galleryId = (int)$this->get('GalleryID');
			
			$clone = clone $this;
			$clone->read($galleryId);
			if ($clone->get('GalleryName') != $this->get('GalleryName')) {
				// the gallery name has changed
				$dirName = scFilesUtil::escapeStringForFilename(trim($this->get('GalleryName'))); // get the new dirname
				if (empty($dirName)) {
					$dirName = $galleryId;
				}
				
				$galleriesPath = scPaths::getStoreRootDir() . scPaths::getPath(scPaths::RESOURCE_GALLERY_IMAGE);
				// Check if such a directory already exists
				if (file_exists($galleriesPath . $dirName)) {
					// If it exists add the id to the end and try again.
					$dirName .= '_' . $galleryId;
					if (file_exists($galleriesPath . $dirName)) {
						throw new miException(_t('Gallery renaming failed: duplicate directory name.'));
					}
				}
				
				$oldPath = $galleriesPath . $clone->get('GalleryDirName');
				$newPath = $galleriesPath . $dirName;
				if (!@rename($oldPath, $newPath)) {
					throw new miException(_t('Gallery renaming failed.'));
				}
				$this->set('GalleryDirName', $dirName);
			}
			
			return parent::update();
		}
		
		public function delete($pkValue = null)
		{
			$galleryIds = $this->getPkValueArray($pkValue);
			
			$this->emptyGalleries($galleryIds);
			
			return parent::delete($pkValue);
		}
		
		public function emptyGalleries($galleryIds)
		{
			$galleryIds = array_map('intval', $galleryIds);
			$galleryIds = array_filter($galleryIds);
			
			$sql = 'SELECT * FROM GalleryItems' .
				' WHERE GalleryID IN (' . implode(', ', $galleryIds) . ')';
			$items = miStaticDBUtil::execSelect($sql);
			
			$galleryItem = new scGalleryItemDomainObject();
			foreach ($items as $itemData) {
				$galleryItem->setRow($itemData);
				$galleryItem->delete();
			}
		}
	}
?>