<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scGalleryItemDomainObject extends scDomainObject {

		public function __construct()
		{
			parent::__construct(new scSqlRecordML('GalleryItems', 'GalleryItemID', 
				array('GalleryItemDescription'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
		
		public function __clone()
		{
			$this->_record = clone $this->_record;
		}
		
		public function insert()
		{
			$this->set('GalleryItemCreateTimestamp', time());
			return parent::insert();
		}
		
		public function update()
		{
			$dbRecord = clone $this;
			$dbRecord->read($this->get('GalleryItemID'));
			
			if ($dbRecord->get('GalleryItemName') != $this->get('GalleryItemName')) {
				throw new miException(_i('Changing a gallery item\'s name is not allowed.'));
			}
			
			return parent::update();
		}
		
		public function delete($pkValue = null)
		{
			// Check if item(s) to be deleted is(are) passed as a parameter.
			if (!is_null($pkValue)) {
				// If it's an array of item ids select them all and delete them one by one
				if (is_array($pkValue)) {
					miStaticDBUtil::startTransaction();
					$sql = 'SELECT * FROM GalleryItems WHERE GalleryItemID IN (' . implode(', ', $pkValue) . ')';
					$items = miStaticDBUtil::execSelect($sql);
					$item = new scGalleryItemDomainObject();
					foreach ($items as $itemData) {
						$item->setRow($itemData);
						if (!$item->delete()) {
							miStaticDBUtil::rollback();
							throw new miException(sprintf(_i('Unable to delete item "%s".'), $item->get('GalleryItemID')));
						}
					}
					miStaticDBUtil::commit();
					return true;
				}
				
				// It's not an array of ids so if we just read it and proceed as it was read originally
				$this->read($pkValue);
			}
			
			// GalleryItemID is not passed as a parameter.
			// Try to find it in the data row.
			$dataRow = $this->getRow();
			if (!isset($dataRow['GalleryItemID']) || empty($dataRow['GalleryItemID'])) {
				throw new miException(_i('Unable to delete: unknown gallery item.'));
			}
			
			// Delete files
			$image = scPaths::getStoreRootDir() . $this->get('GalleryItemImage');
			$thumb = scPaths::getStoreRootDir() . $this->get('GalleryItemImageThumb');
			if (file_exists($image) && !is_writable($image)) {
				throw new miException(sprintf(_i('Unable to delete: cannot delete file "%s".'), $image));
			}
			if (file_exists($thumb) && !is_writable($thumb)) {
				throw new miException(sprintf(_i('Unable to delete: cannot delete file "%s".'), $thumb));
			}
			
			if (file_exists($image)) {
				@unlink($image);
			}
			if (file_exists($thumb)) {
				@unlink($thumb);
			}
			
			// Delete DB record
			return parent::delete($pkValue);
		}
	}
?>