<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scLanguageDomainObject extends miDefaultDomainObject
	{
		const LANGUAGE_DIRECTION_LTR = 0;
		const LANGUAGE_DIRECTION_RTL = 1;
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('Languages', 'LanguageID'));
		}
		
		public static function getLanguageDirections()
		{
			return array(
				self::LANGUAGE_DIRECTION_LTR => _t('Left-to-right'),
				self::LANGUAGE_DIRECTION_RTL => _t('Right-to-left'),
			);
		}
		
		public function getTableName()
		{
			return $this->_record->getTableName();
		}
		
		public function set($field, $value)
		{
			if ($field == 'LanguageCode')
				$value = strtoupper($value);
			parent::set($field, $value);
		}
		
		public function insert()
		{
			if (!$this->has('LanguageSort')) {
				$maxSort = miStaticDBUtil::execSelect('SELECT MAX(LanguageSort) AS N FROM Languages');
				if (isset($maxSort[0]) && isset($maxSort[0]['N'])) {
					$this->set('LanguageSort', $maxSort[0]['N'] + 1);
				}
			}
			return parent::insert();
		}
		
		/**
		 * Checks if the language exists
		 * @param string $languageCode
		 * @return bool
		 */
		public static function checkLanguageExists($languageCode)
		{
			$result = scUtil::getDbFirstRow('SELECT COUNT(1) AS N FROM Languages WHERE LanguageCode=?', array($languageCode));
			return $result['N'] ? true : false;
		}
		
		/**
		 * @throws miException
		 */
		public function validateOnUpdate()
		{
			parent::validateOnUpdate();
			
			$oldRecord = clone $this->_record;
			$oldRecord->readPK($this->get('LanguageID'));
			
			if ((int)$oldRecord->get('LanguageIsActive') == (int)$this->get('LanguageIsActive'))
				return;// The value has not been modified.
			
			if ((int)$this->get('LanguageIsActive') == 1)
				return;// We are activating a language.
			
			$recordset = new miSqlRecordset('Languages');
			$recordset->addFilter(new miSqlFilterEqual('LanguageIsActive', 1));
			$currentActiveLangs = $recordset->getRecordsCount();
			if ($currentActiveLangs == 1) {
				throw new miException(_t('You cannot disable the only active language.'));
			}
		}
	}
?>