<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOptionGroupDomainObject extends scDomainObject implements ISet, ICartesianProduct {
		
		protected $_values = null;
		
		protected $_isNew = false;
		
		/**
		 * @var bool
		 */
		protected $_justBecameVariant = false;
		
		public function __construct()
		{
			parent::__construct(new scSqlRecordML('OptionGroups', 'OptionGroupID',
				array('OptionGroupName'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
		}
		
		public function __clone()
		{
			$this->_record = clone $this->_record;
		}
		
		public function insert()
		{
			$id = parent::insert();
			$this->_isNew = true;
			$this->_values = null;
			return $id;
		}
		
		public function update()
		{
			$copy = clone $this;
			$copy->read($this->get('OptionGroupID'));
			
			if ($this->get('OptionGroupType') == scProductOptions::GROUP_TYPE_VARIANT) {
				if ($copy->get('OptionGroupType') != scProductOptions::GROUP_TYPE_VARIANT) {
					$this->_justBecameVariant = true;
				}
			} else if ($copy->get('OptionGroupType') == scLocalOptions::GROUP_TYPE_VARIANT) {
				// The option has been of type "variant" and is now something else.
				// Delete all data in the variants as if the option was deleted.
				// From Variant's point of view that's exactly what has happened to the option.
				$sql = 'DELETE FROM ProductVariantOptions' .
					' WHERE OptionID IN (SELECT OptionID FROM Options WHERE OptionGroupID = ?)';
				miStaticDBUtil::execSQL($sql, array($this->get('OptionGroupID')));
			}
			
			$this->_values = null;
			
			return parent::update();
		}
		
		/**
		 * @return bool
		 */
		public function getJustBecameVariant()
		{
			return $this->_justBecameVariant;
		}
		
		public function isVariant()
		{
			return ($this->get('OptionGroupType') == scProductOptions::GROUP_TYPE_VARIANT);
		}
		
		/**
		 * ISet::add
		 */
		public function add($value)
		{
			throw new miException(_i('Not implemented.'));
		}
		
		public function setIsNew($isNew)
		{
			$this->_isNew = (bool)$isNew;
			// Reset values cache so it is rebuilt.
			$this->_values = null;
		}
		
		/**
		 * ISet::getValues
		 * @return scProductVariantOption[]
		 */
		public function getValues()
		{
			if (is_null($this->_values)) {
				$recordset = new miSqlRecordset('Options');
				$recordset->setSelectFields(array('OptionID'));
				$recordset->addFilter(new miSqlFilterEqual($this->getPKName(), $this->get($this->getPKName())));
				$options = $recordset->getAllRecords();
				$values = array();
				
				foreach ($options as $option) {
					$variantOption = new scProductVariantOption($option['OptionID'], scProductVariantOption::TYPE_GLOBAL);
					$variantOption->setIsNewGroup($this->_isNew || $this->_justBecameVariant);
					$values[] = array($variantOption);
				}
				$this->_values = $values;
			}
			return $this->_values;
		}
		
		/**
		 * @param scProductVariantOption[] $values
		 */
		public function setValues($values)
		{
			$this->_values = $values;
		}
		
		/**
		 * ISet::isEmpty
		 * @return bool
		 */
		public function isEmpty()
		{
			$values = $this->getValues();
			return empty($values);
		}
		
		/**
		 * ICartesianProduct::cartesianProduct
		 * @param ISet $set
		 * @return ISet
		 */
		public function cartesianProduct(ISet $set)
		{
			if ($this->isEmpty()) {
				$set = new scSet($set->getValues());
				$set->cartesianProduct($this);
				return $set;
			}
			
			$cartesianProduct = array();
			foreach ($this->getValues() as $value1) {
				if (!is_array($value1)) {
					$value1 = array($value1);
				}
				foreach ($set->getValues() as $value2) {
					if (!is_array($value2)) {
						$value2 = array($value2);
					}
					$cartesianProduct[] = array_merge($value1, $value2);
				}
			}
			
			return new scSet($cartesianProduct);			
		}
	}
?>