<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOrderDomainObject extends miDefaultDomainObject {
		
		/**
		 * @param int $orderId
		 */
		public function __construct($orderId = 0)
		{
			parent::__construct(new miSqlRecord('Orders', 'OrderID'));
			if ($orderId > 0) {
				$this->read($orderId);
			}
		}
	}
	
	class scOrder extends scOrderDomainObject {
		
		/**
		 * @var scOrderItem[]
		 */
		protected $_orderItems = null;
		
		/**
		 * Holds OrderItemEgoods grouped by OrderItemID
		 * @var array
		 */
		protected static $_orderItemEgoods = null;
		
		/**
		 * @var array
		 */
		protected $_orderTotalLines = null;
		
		/**
		 * @var scCurrency
		 */
		protected $_baseCurrency = null;
		
		/**
		 * @var scCurrency
		 */
		protected $_customerCurrency = null;
		
		/**
		 * Gives the Order (domain object) the abillity to set the value of the PK.
		 * @param string $name
		 * @param mixed $value
		 */
		public function set($name, $value)
		{
			switch ($name) {
				case 'OrderBillingCityID':
				case 'OrderShippingCityID':
					if (empty($value))
						$value = null;
					break;
			}
			$this->_record->set($name, $value);
		}
		
		/**
		 * Backwards campatability with scOrder SqlRecord
		 * @return int
		 * @deprecated Use DomainObject's get(getPKName)
		 */
		public function getPK()
		{
			return $this->get($this->getPKName());
		}
		
		/**
		 * @param array $customerInfo
		 */
		public function setCustomerInfo(array $customerInfo)
		{
			$shippingModuleId = 0;
			$shippingMethodCode = '';
			if (count(explode('-', $customerInfo['ShippingModuleAndMethod'])) == 2)
				list($shippingModuleId, $shippingMethodCode) = explode('-', $customerInfo['ShippingModuleAndMethod']);
			
			list($shippingModuleName, $shippingMethodName) = scOrdersInProgress::singleton()->getShippingModuleAndMethodNames($shippingModuleId, $shippingMethodCode);
			
			list($paymentModuleName, $paymentModuleIntegrationMethod) = scOrdersInProgress::singleton()->getPaymentModuleName($customerInfo['PaymentModuleID']);
			
			$orderDetails = array(
				// Shipping
				'ShippingModuleID' => $shippingModuleId,
				'OrderShippingModuleName' => $shippingModuleName,
				'OrderShippingMethodCode' => $shippingMethodCode,
				'OrderShippingMethodName' => $shippingMethodName,
				
				// Payment
				'PaymentModuleID' => $customerInfo['PaymentModuleID'],
				'OrderPaymentModuleName' => $paymentModuleName,
				'OrderPaymentModuleIntegration' => $paymentModuleIntegrationMethod,
				'OrderPaymentNotes' => '',
				
				'OrderCustomerInstructions' => empty($customerInfo['OrderCustomerInstructions']) ? '' : $customerInfo['OrderCustomerInstructions'],
				'OrderCustomerEmail' => $customerInfo['CustomerEmail'],
				'CustomerID' => $customerInfo['CustomerID'],
				
				'OrderBillingFirstName' => $customerInfo['CustomerBillingFirstName'],
				'OrderBillingLastName' => $customerInfo['CustomerBillingLastName'],
				'OrderBillingCompany' => $customerInfo['CustomerBillingCompany'],
				'OrderBillingPhone' => $customerInfo['CustomerBillingPhone'],
				'OrderBillingFax' => $customerInfo['CustomerBillingFax'],
				'OrderBillingAddress1' => $customerInfo['CustomerBillingAddress1'],
				'OrderBillingAddress2' => $customerInfo['CustomerBillingAddress2'],
				'OrderBillingCity' => $customerInfo['CustomerBillingCity'],
				'OrderBillingCityID' => $customerInfo['CustomerBillingCityID'],
				'OrderBillingState' => scCountry::getStateName($customerInfo['CustomerBillingState'], scMultiLanguage::getDefaultLanguageId()),
				'OrderBillingStateCode' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'OrderBillingCountry' => scCountry::getCountryName($customerInfo['CustomerBillingCountry'], scMultiLanguage::getDefaultLanguageId()),
				'OrderBillingCountryCode2' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'OrderBillingZip' => $customerInfo['CustomerBillingZip'],
				'OrderBillingVatNumber' => $customerInfo['CustomerBillingVatNumber'],
				
				'OrderShipToBillingAddress' => $customerInfo['CustomerShipToBillingAddress'] ? 1 : 0,
				
				'OrderShippingFirstName' => $customerInfo['CustomerShippingFirstName'],
				'OrderShippingLastName' => $customerInfo['CustomerShippingLastName'],
				'OrderShippingCompany' => $customerInfo['CustomerShippingCompany'],
				'OrderShippingPhone' => $customerInfo['CustomerShippingPhone'],
				'OrderShippingFax' => $customerInfo['CustomerShippingFax'],
				'OrderShippingAddress1' => $customerInfo['CustomerShippingAddress1'],
				'OrderShippingAddress2' => $customerInfo['CustomerShippingAddress2'],
				'OrderShippingCity' => $customerInfo['CustomerShippingCity'],
				'OrderShippingCityID' => $customerInfo['CustomerShippingCityID'],
				'OrderShippingState' => scCountry::getStateName($customerInfo['CustomerShippingState'], scMultiLanguage::getDefaultLanguageId()),
				'OrderShippingStateCode' => scCountry::getStateCode($customerInfo['CustomerShippingState']),
				'OrderShippingCountry' => scCountry::getCountryName($customerInfo['CustomerShippingCountry'], scMultiLanguage::getDefaultLanguageId()),
				'OrderShippingCountryCode2' => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']),
				'OrderShippingZip' => $customerInfo['CustomerShippingZip'],
				'OrderShippingVatNumber' => $customerInfo['CustomerShippingVatNumber'],
			);
			
			foreach ($orderDetails as $field => $value) {
				if (is_null($value))
					$value = '';
				
				$this->set($field, $value);
			}
		}
		
		public function insert()
		{
			$this->set('OrderTimestamp', time());
			$this->set('OrderIPAddress', sprintf('%u', ip2long($_SERVER['REMOTE_ADDR'])));
			$this->set('OrderStatus', scOrderStatus::ORDER_STATUS_UNFINISHED);
			$this->set('OrderPaymentStatus', scPaymentStatus::PAYMENT_STATUS_NONE);
			$this->set('OrderLanguageCode', scMultiLanguage::getCurrentLanguageCode());
			
			$orderId = parent::insert();
			scOperationsLog::storeOperation(scOperationsLog::ITEM_TYPE_ORDER, $orderId, scOperationsLog::OPERATION_INSERT);
			return $orderId;
		}
		
		public function update()
		{
			$orderId = parent::update();
			scOperationsLog::storeOperation(scOperationsLog::ITEM_TYPE_ORDER, $orderId, scOperationsLog::OPERATION_UPDATE);
			return $orderId;
		}
		
		public function delete($pkValue = null)
		{
			if (scSettings::getSettingValue(scSettings::SETTING_INVENTORY_TRACKING)) {
				$this->returnInventory();
			}
			$orderId = parent::delete();
			scOperationsLog::storeOperation(scOperationsLog::ITEM_TYPE_ORDER, $orderId, scOperationsLog::OPERATION_DELETE);
			return $orderId;
		}
		
		/**
		 * @param int $orderId
		 * @return array
		 */
		public static function getOrderEgoods($orderId)
		{
			if (is_null(self::$_orderItemEgoods)) {
				$recordset = new miSqlRecordset('OrderItems');
				$recordset->addJoinCondition('INNER', 'OrderItemEgoods', 'ON OrderItems.OrderItemID = OrderItemEgoods.OrderItemID');
				$recordset->addFilter(new miSqlFilterEqual('OrderID', $orderId));
				$recordset->setSelectFields(array(
					'OrderItemEgoods.*',
					'OrderItemQty * OrderItemEgoodMaxDownloads AS CalculatedEgoodMaxDownloads'
				));
				
				$egoods = $recordset->getAllRecords();
				
				self::$_orderItemEgoods = array();
				foreach ($egoods as $egood) {
					$egood['OrderItemEgoodFileName'] = basename($egood['OrderItemEgoodFile']);
					
					self::$_orderItemEgoods[$egood['OrderItemID']][] = $egood;
				}
			}
			return self::$_orderItemEgoods;
		}
		
		/**
		 * @param int $newStatus {@see scOrderStatus}
		 */
		public function updateOrderStatus($newStatus)
		{
			$this->setOrderStatus($newStatus);
			$this->update();
		}
		
		/**
		 * Sets the order's status to the specified value.
		 * Does not trigger order's update() method. This should be done manually.
		 * @see scOrder::updateOrderStatus()
		 * @param int $newStatus
		 */
		public function setOrderStatus($newStatus)
		{
			$oldStatus = $this->get('OrderStatus');
			if ($newStatus == $oldStatus) {
				// Check if the status has not been changed.
				return;
			}
			
			$orderId = $this->getPK();
			
			// Set the new status to the object.
			$this->set('OrderStatus', $newStatus);
			
			// E-mail notifications
			if ($newStatus == scOrderStatus::ORDER_STATUS_NEW) {
				$notifications = new scEmailNotifications();
				$result = $notifications->sendNewOrderEmail($this->getPK());
				if (!$result) {
					scLog::singleton()->warning(sprintf(_t('Could not send new order email for order %s.'), $orderId));
				}
				
				// Handle the bestsellers module
				// TODO: V1.1 Implement module callback / notification system
				$bestsellers = scModulesFactory::loadCmsBoxClassByName('scCustomerBestSellersBox');
				if ($bestsellers instanceof scCustomerBestSellersBox) {
					$bestsellers->updateBestsellers();
				}
				
				// Autounlock of E-Goods (if any)
				if ($this->hasEgoods() and scSettings::getSettingValue(scSettings::SETTING_AUTO_UNLOCK_EGOODS)) {
					$this->unlockAllEgoods($orderId);
				}
			} else {
				$notifications = new scEmailNotifications();
				$notifications->sendStatusChangeNotification($this, $newStatus);
			}
			
			$this->processBonusPoints($newStatus);
			$this->inventoryTracking($newStatus);
		}
		
		/**
		 * Unlock all egoods and send notification to customer
		 * 
		 * @param int $orderId
		 */
		protected function unlockAllEgoods($orderId)
		{
			miStaticDBUtil::execSQL('UPDATE OrderItemEgoods SET OrderItemEgoodIsLocked = 0 ' .
				'WHERE OrderItemID IN (SELECT OrderItemID FROM OrderItems WHERE OrderID = ?)', array($orderId));
			
			$notifications = new scEmailNotifications();
			$notifications->sendOrderEgoods($orderId);
		}
		
		/**
		 * Processes order-based inventory changes.
		 * Returns and reduces stock item quantities when needed.
		 * Does not trigger order's update() method. This should be done manually.
		 * @param unknown_type $newOrderStatus
		 */
		public function inventoryTracking($newOrderStatus)
		{
			if (!scSettings::getSettingValue(scSettings::SETTING_INVENTORY_TRACKING)) {
				// Check if inventory tracking is turned off.
				return;
			}
			
			// Check if this status supposes reduce or not
			$inventoryAction = scOrderStatuses::singleton()->getOrderStatusInventoryAction($newOrderStatus);
			
			// reduceInventory() and returnInventory() both check if they have already been applied
			// so quantities are not reduced (or returned) twice.
			if ($inventoryAction == scOrderStatusInventoryAction::REDUCE_QUANTITIES) {
				$this->reduceInventory();
			} else if ($inventoryAction == scOrderStatusInventoryAction::RETURN_QUANTITIES) {
				$this->returnInventory();
			}
		}
		
		public function updateOrderPaymentStatus($newStatus)
		{
			$values = array('OrderPaymentStatus' => $newStatus);
			$orderId = $this->get($this->getPKName());
			miStaticDBUtil::execUpdate('Orders', $values, 'OrderID', $orderId);
			
			scOperationsLog::storeOperation(scOperationsLog::ITEM_TYPE_ORDER, $orderId, scOperationsLog::OPERATION_UPDATE);
			
			$this->set('OrderPaymentStatus', $newStatus);
		}
		
		/**
		 * @return scOrderItem[]
		 */
		public function getOrderItems()
		{
			if (is_null($this->_orderItems)) {
				$recordset = new miSqlRecordset('OrderItems');
				$recordset->addFilter(new miSqlFilterEqual('OrderID', $this->get($this->getPKName())));
				$orderItems = $recordset->getAllRecords();
				$this->_orderItems = array();
				foreach ($orderItems as $orderItemData) {
					$orderItem = new scOrderItem();
					$orderItem->setOrder($this);
					$orderItem->setRow($orderItemData);
					$this->_orderItems[] = $orderItem;
				}
			}
			return $this->_orderItems;
		}
		
		/**
		 * @return bool
		 */
		public function hasPerItemDiscount()
		{
			foreach ($this->getOrderItems() as $item) {
				if ($item->get('OrderItemDiscount') > 0) {
					return true;
				}
			}
			return false;
		}
		
		/**
		 * @return array
		 */
		public function getEgoods()
		{
			return self::getOrderEgoods($this->get($this->getPKName()));
		}
		
		/**
		 * @return bool
		 */
		public function hasEgoods()
		{
			return (bool)count($this->getEgoods());
		}
		
		/**
		 * @return string
		 */
		public function getAccessKey()
		{
			$orderId = $this->get($this->getPKName());
			return md5($orderId . $this->get('OrderIPAddress') . $this->get('OrderTimestamp') . $this->get('OrderBillingFirstName'));
		}
		
		/**
		 * Returns ModuleID of the module that processed the payment
		 * @return int
		 */
		public function getModuleIdForPayment()
		{
			return scUtil::getDbValue('PaymentModules', 'ModuleID', 'PaymentModuleID', $this->get('PaymentModuleID'), 0);
		}
		
		/**
		 * Reduces the available inventory based on the ordered items.
		 * Does not trigger order's update() method. This should be done manually.
		 */
		public function reduceInventory()
		{
			if ($this->get('OrderQuantitiesInStockReduced')) {
				return;
			}
			
			$this->reduceStockItemQuantities();
			$this->set('OrderQuantitiesInStockReduced', 1);
		}
		
		/**
		 * Return the available inventory based on the ordered items.
		 * Does not trigger order's update() method. This should be done manually.
		 */
		public function returnInventory()
		{
			if (!$this->get('OrderQuantitiesInStockReduced')) {
				return;
			}
			
			$this->returnStockItemQuantities();
			$this->set('OrderQuantitiesInStockReduced', 0);
		}
		
		/**
		 * Checks if the billing and shipping addresses are the same
		 *
		 * @return boolean true if the shipping and billing addresses are the same
		 */
		public function isShippingAddressSameAsBilling()
		{
			$checkFields = array(
				'OrderBillingFirstName' => 'OrderShippingFirstName',
				'OrderBillingLastName' => 'OrderShippingLastName',
				'OrderBillingCompany' => 'OrderShippingCompany',
				'OrderBillingPhone' => 'OrderShippingPhone',
				'OrderBillingFax' => 'OrderShippingFax',
				'OrderBillingAddress1' => 'OrderShippingAddress1',
				'OrderBillingAddress2' => 'OrderShippingAddress2',
				'OrderBillingCity' => 'OrderShippingCity',
				'OrderBillingState' => 'OrderShippingState',
				'OrderBillingStateCode' => 'OrderShippingStateCode',
				'OrderBillingCountry' => 'OrderShippingCountry',
				'OrderBillingCountryCode2' => 'OrderShippingCountryCode2',
				'OrderBillingZip' => 'OrderShippingZip',
			);
			foreach ($checkFields as $field1 => $field2) {
				if ($this->get($field1) != $this->get($field2))
					return false;
			}
			return true;
		}
		
		/**
		 * @return scCurrency
		 */
		public function getBaseCurrency()
		{
			if (is_null($this->_baseCurrency)) {
				try {
					$this->_baseCurrency = new scCurrency($this->get('OrderCurrencyCode'));
				} catch (miException $e) {
					$this->_baseCurrency = new scUnsupportedCurrency($this->get('OrderCurrencyCode'));
				}
			}
			return $this->_baseCurrency;
		}
		
		public function getBaseCurrencyTotal()
		{
			return $this->get('OrderTotal');
		}
		
		/**
		 * @return scCurrency
		 */
		public function getCustomerCurrency()
		{
			if ($this->get('OrderCustomerCurrencyCode') == '') {
				return $this->getBaseCurrency();
			}
			
			if (is_null($this->_customerCurrency)) {
				try {
					$this->_customerCurrency = new scCurrency($this->get('OrderCustomerCurrencyCode'));
				} catch (miException $e) {
					$this->_customerCurrency = new scUnsupportedCurrency($this->get('OrderCustomerCurrencyCode'));
				}
			}
			return $this->_customerCurrency;
		}
		
		public function getCustomerCurrencyTotal()
		{
			if ($this->get('OrderCustomerCurrencyCode') == '') {
				return $this->getBaseCurrencyTotal();
			}
			
			return $this->get('OrderCustomerCurrencyTotal');
		}
		
		/**
		 * @return scOrderTotalLineDomainObject[]
		 */
		public function getOrderTotalLines()
		{
			if (is_null($this->_orderTotalLines)) {
				$recordset = new miSqlRecordset('OrderTotalLines');
				$recordset->addFilter(new miSqlFilterEqual('OrderID', $this->getPK()));
				$recordset->setOrder('OrderTotalLineSort');
				$orderTotalLines = $recordset->getAllRecords();
				$this->_orderTotalLines = array();
				foreach ($orderTotalLines as $totalLine) {
					$line = new scOrderTotalLineDomainObject();
					$line->setRow($totalLine);
					$this->_orderTotalLines[] = $line;
				}
			}
			return $this->_orderTotalLines;
		}
		
		
		public function getTotalWeight($weightUnits)
		{
			$totalWeight = 0;
			$orderItems = $this->getOrderItems();
			foreach ($orderItems as $item) {
				$totalWeight += $item->get('OrderItemQty') * $item->getListingItem()->getWeight()->getWeightInUnits($weightUnits);
			}
			return $totalWeight;
		}
		
		public function getDeclaredValueForShipping(scCurrency $orderBaseCurrency, scCurrency $returnCurrency)
		{
			$sql = 'SELECT OrderTotalLineAmount, OrderTotalLineType FROM OrderTotalLines WHERE OrderID = ? ORDER BY OrderTotalLineSort ASC';
			$orderTotalLines = miStaticDBUtil::execSelect($sql, array($this->getPK()));
			$total = scMultiCurrencyAmount::getZero();
			foreach ($orderTotalLines as $line) {
				// get all total lines before shipping
				if ($line['OrderTotalLineType'] == IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING) {
					break;
				}
				// ignore the bonus points total line
				if ($line['OrderTotalLineType'] != IOrderTotalModule::ORDER_TOTAL_TYPE_POINTS) {
					$amount = scMultiCurrencyAmount::getZero();
					$amount->set($orderBaseCurrency, $line['OrderTotalLineAmount']);
					$total = scMultiCurrencyAmountMath::add($total, $amount);
				}
			}
			$price = scPrice::factory($total);
			return $price->getAmountInCurrency($returnCurrency);
		}
		
		protected function processBonusPoints($newOrderStatus)
		{
			$this->processReferralPoints($newOrderStatus);
			
			// Check if there is a registered customer assigned to this order
			if (($this->get('CustomerID') <= 0) || ($this->get('CustomerID') == ICheckoutModule::GUEST_USER_ID)) {
				return;
			}
			
			// Check if the customer still exists as there are no foreign keys in order's data
			try {
				$customer = new scCustomerDomainObject();
				$customer->read($this->get('CustomerID'));
			} catch (miDBException $exception) {
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					return;
				}
				throw $exception;
			}
			
			if ($newOrderStatus == scOrderStatus::ORDER_STATUS_NEW)
				$this->spendPoints();
			else if ($newOrderStatus == scOrderStatus::ORDER_STATUS_CANCELLED)
				$this->returnSpentPoints();
			
			if (scBonusPoints::isOrderStatusAssign($newOrderStatus))
				$this->assignPoints();
			else {
				$identifier = $this->getPK();
				$customerId = $this->get('CustomerID');
				scBonusPoints::deassignEarnedPoints($identifier, $customerId);
			}
		}
		
		protected function spendPoints()
		{
			$spentPoints = $this->getOrderSpendPoints();
			$spendingDescription = _t('Order') . ': ' . $this->getPK();
			scBonusPoints::spendBonusPoints($this->get('CustomerID'), $spentPoints, scBonusPoints::SPENT_REASON_SPEND, $spendingDescription, $this->getPK());
		}
		
		protected function returnSpentPoints()
		{
			$identifier = $this->getPK();
			$customerId = $this->get('CustomerID');
			$spentPoints = $this->getOrderSpendPoints();
			scBonusPoints::returnSpentPoints($identifier, $customerId, $spentPoints);
		}
		
		protected function getOrderSpendPoints()
		{
			$orderItems = $this->getOrderItems();
			$spentPoints = 0;
			foreach ($orderItems as $orderItem)
				$spentPoints += $orderItem->get('OrderItemBonusPointsSpent');
			
			return $spentPoints;
		}
		
		protected function assignPoints()
		{
			// Assign points for purchasing
			$pointsAssignmentReason = scBonusPoints::ASSIGNMENT_REASON_PURCHASE;
			$pointsAssignmentDescription = _t('New order') . ': ' . $this->getPK();
			scBonusPoints::assignBonusPoints($this->get('CustomerID'), $pointsAssignmentReason, $pointsAssignmentDescription, $this->getPK());
			
			// Assign points based on spent amount
			$orderTotalLines = $this->getOrderTotalLines(); // Get the order's total lines
			$orderTotalTypes = scUtil::getActiveOrderTotalLineTypes(IOrderTotalModule::ORDER_TOTAL_TYPE_POINTS); // Get the line types that are before the points type
			
			$spentAmount = 0;
			foreach ($orderTotalLines as $key => $totalLine) {
				if (!in_array($totalLine->get('OrderTotalLineType'), $orderTotalTypes)) {
					break;
				}
				$spentAmount += $totalLine->get('OrderTotalLineAmount');
			}
			
			$customerId = $this->get('CustomerID');
			$pointsAssignmentReason = scBonusPoints::ASSIGNMENT_REASON_SPEND;
			$pointsAssignmentDescription = sprintf(_t('Based on spent amount for order %s.'), $this->getPK());
			scBonusPoints::assignBonusPoints($customerId, $pointsAssignmentReason, $pointsAssignmentDescription, $this->getPK(), $spentAmount);
		}
		
		protected function processReferralPoints($newOrderStatus)
		{
			$referringHash = (string)$this->get('OrderCustomerReferringHash');
			if ($referringHash == '') {
				// There is no referring hash assigned to the order.
				$orderCustomerId = $this->get('CustomerID');
				if ((int)$orderCustomerId <= 0) {
					// The customer has not registered.
					return;
				}
				if ($orderCustomerId == ICheckoutModule::GUEST_USER_ID) {
					// The customer made an guest order.
					return;
				}
				
				try {
					$orderCustomer = new scCustomerDomainObject();
					$orderCustomer->read($orderCustomerId);
					$referrerId = $orderCustomer->get('CustomerReferrerID');
					$referredEmailFromHash = '';
				} catch (miException $e) {
					return;
				}
			} else {
				list($referrerId, $referredEmailFromHash) = scBonusPoints::decodeReferringHash($referringHash);
			}
			
			$referrerId = (int)$referrerId;
			if ($referrerId <= 0) {
				// There is no valid referrer ID
				return;
			}
			
			$referredEmail = (string)$this->get('OrderCustomerEmail');
			if (trim($referredEmail) == '') {
				// There is no customer e-mail in the order. Use the one from the hash instead.
				$referredEmail = $referredEmailFromHash;
			}
			
			if ((trim($referredEmail) == '') || (!miValidatorEmail::isValid($referredEmail))) {
				// There is no valid referred e-mail
				return;
			}
			
			try {
				$referrer = new scCustomerDomainObject();
				$referrer->read($referrerId);
			} catch (miException $e) {
				// The referring customer was not found
				return;
			}
			
			if (scBonusPoints::isOrderStatusAssign($newOrderStatus)) {
				$this->assignReferralPoints($referrerId, $referredEmail);
			} else {
				$this->deassignReferralPoints($referrerId, $referredEmail);
			}
		}
		
		protected function assignReferralPoints($referrerId, $referredEmail)
		{
			$assignmentDescription = sprintf(_t('Referred customer %s.'), $referredEmail);
			scBonusPoints::assignBonusPoints($referrerId, scBonusPoints::ASSIGNMENT_REASON_REFERRAL, $assignmentDescription, $referredEmail);
		}
		
		protected function deassignReferralPoints($referrerId, $referredEmail)
		{
			$sql = 'SELECT SUM(BonusPointsAmount) AS BonusPointsAmount FROM BonusPointsTransactions' .
				' WHERE BonusPointsTransactionType = ' . scBonusPoints::ACTION_ASSIGN .
				' AND BonusPointsTransactionReason = ' . scBonusPoints::ASSIGNMENT_REASON_REFERRAL .
				' AND BonusPointsTransactionIdentifier = ?';
			$points = miStaticDBUtil::execSelect($sql, array($referredEmail));
			
			if (isset($points[0]) && isset($points[0]['BonusPointsAmount']) && $points[0]['BonusPointsAmount'] > 0) {
				// Remove them from customer's account
				$sql = 'UPDATE Customers SET CustomerBonusPoints = CustomerBonusPoints - ? WHERE CustomerID = ?';
				miStaticDBUtil::execSQL($sql, array($points[0]['BonusPointsAmount'], $referrerId));
				
				// Delete the history logs that say points have been assigned
				$sql = 'DELETE FROM BonusPointsTransactions' .
					' WHERE BonusPointsTransactionType = ' . scBonusPoints::ACTION_ASSIGN .
					' AND BonusPointsTransactionReason = ' . scBonusPoints::ASSIGNMENT_REASON_REFERRAL .
					' AND BonusPointsTransactionIdentifier = ?';
				miStaticDBUtil::execSQL($sql, array($referredEmail));
			}
		}
		
		protected function reduceStockItemQuantities()
		{
			$stockItems = $this->getStockItemsData();
			foreach ($stockItems as $stockItemData) {
				$stockItem = $this->getStockItemFromData($stockItemData);
				$stockItem->reduceQuantityInStock($stockItemData['OrderItemQty']);
			}
		}
		
		protected function returnStockItemQuantities()
		{
			$stockItems = $this->getStockItemsData();
			foreach ($stockItems as $stockItemData) {
				$stockItem = $this->getStockItemFromData($stockItemData);
				$stockItem->setQuantityInStock($stockItem->getQuantityInStock() + $stockItemData['OrderItemQty']);
			}
		}
		
		/**
		 * @return array
		 */
		protected function getStockItemsData()
		{
			$query = 'SELECT ProductID, ProductVariantID, OrderItemQty FROM OrderItems WHERE OrderID=?';
			return miStaticDBUtil::execSelect($query, array($this->get($this->getPKName())));
		}
		
		/**
		 * @param array $stockItemData
		 * @return IStockable
		 */
		protected function getStockItemFromData($stockItemData)
		{
			if (!empty($stockItemData['ProductVariantID'])) {
				$stockItem = new scProductVariant();
				$stockItem->read($stockItemData['ProductVariantID']);
			} else {
				$stockItem = new scProductDomainObject();
				$stockItem->read($stockItemData['ProductID']);
			}
			return $stockItem;
		}
	}
?>