<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOrderItem extends miDefaultDomainObject {
		
		protected $_orderItemOptions = null;
		protected $_orderItemLocalOptions = null;
		
		/**
		 * @var scOrder
		 */
		protected $_order = null;
		
		public function __construct($orderItemId = null)
		{
			parent::__construct(new miSqlRecord('OrderItems', 'OrderItemID'));
			if (!is_null($orderItemId)) {
				$this->read($orderItemId);
			}
		}
		
		/**
		 * @return scOrder
		 */
		public function getOrder()
		{
			if (is_null($this->_order)) {
				$this->_order = new scOrder($this->get('OrderID'));
			}
			return $this->_order;
		}
		
		/**
		 * @return scOrderItemOption[]
		 */
		public function getOrderItemOptions()
		{
			return scOrdersCache::getOrderItemOptions($this);
		}
		
		/**
		 * @return scOrderItemLocalOption[]
		 */
		public function getOrderItemLocalOptions()
		{
			return scOrdersCache::getOrderItemLocalOptions($this);
		}
		
		/**
		 * @return scPrice
		 */
		public function getPrice()
		{
			$amount = scMultiCurrencyAmount::factory();
			$amount->set($this->getOrder()->getBaseCurrency(), $this->get('OrderItemPrice'));
			$amount->set($this->getOrder()->getCustomerCurrency(), $this->get('OrderItemCustomerCurrencyPrice'));
			return scPrice::factory($amount);
		}
		
		/**
		 * @return scPrice
		 */
		public function getTotal()
		{
			$amount = scMultiCurrencyAmount::factory();
			$amount->set($this->getOrder()->getBaseCurrency(), $this->get('OrderItemTotal'));
			$amount->set($this->getOrder()->getCustomerCurrency(), $this->get('OrderItemCustomerCurrencyTotal'));
			return scPrice::factory($amount);
		}
		
		/**
		 * Set the order item's order so it is not read every time it is needed.
		 * If friend classes were provided that method would've been set to friend instead of public
		 * so only one scOrder can give itself to its item.
		 * @param scOrder $order
		 */
		public function setOrder(scOrder $order)
		{
			$this->_order = $order;
		}
		
		/**
		 * @return IProductListingItem or null
		 */
		public function getListingItem()
		{
			try {
				$productVariantId = $this->get('ProductVariantID');
				if (!is_null($productVariantId)) {
					$listingItem = new scProductVariant();
					$listingItem->read($productVariantId);
					return $listingItem;
				}
				
				$productId = $this->get('ProductID');
				$listingItem = new scProductSL();
				$listingItem->read($productId);
				return $listingItem;
				
			} catch (miDBException $e) {
				if ($e->getCode() !== miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					throw $e;
				}
				return null;
			}
		}
	}
?>