<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOrderItemEgood extends scEgoodDomainObject {
		
		const ORDER_ITEM_EGOOD_LOCKED = 1;
		const ORDER_ITEM_EGOOD_UNLOCKED = 0;
		
		/**
		 * @param array $array
		 * @return scOrderItemEgood
		 */
		public static function fromArray($array)
		{
			$orderItemEgood = new scOrderItemEgood();
			foreach ($array as $key => $value) {
				$orderItemEgood->set($key, $value);
			}
			return $orderItemEgood;
		}
		
		public function __construct($orderItemEgoodId = null)
		{
			parent::__construct(new miSqlRecord('OrderItemEgoods', 'OrderItemEgoodID'));
			if (!is_null($orderItemEgoodId)) {
				$this->read($orderItemEgoodId);
			}
		}
		
		/**
		 * @return int
		 */
		public function insert()
		{
			$this->set('OrderItemEgoodIsLocked', 1);
			$this->set('OrderItemEgoodDownloads', 0);
			$this->set('OrderItemEgoodMaxDownloads', (int)scSettings::getSettingValue(scSettings::SETTING_DOWNLOAD_TTL_DOWNLOADS));
			$this->set('OrderItemEgoodExpirationDate', $this->getExpirationDate());
			return parent::insert();
		}
		
		/**
		 * @return scOrderItem
		 */
		public function &getOrderItem()
		{
			static $orderItem = null;
			if (is_null($orderItem)) {
				$orderItem = new scOrderItem($this->get('OrderItemID'));
			}
			return $orderItem;
		}
		
		/**
		 * @return int
		 */
		public function getMaxDownloads()
		{
			$itemQty = $this->getOrderItem()->get('OrderItemQty');
			$egoodMaxDownloads = $this->get('OrderItemEgoodMaxDownloads');
			
			return ($egoodMaxDownloads * $itemQty);
		}
		
		/**
		 * @return bool
		 */
		public function isDownloadable()
		{
			try {
				$this->checkIsDownloadable();
			} catch (miException $e) {
				return false;
			}
			return true;
		}
		
		/**
		 * @throws miException
		 */
		public function checkIsDownloadable()
		{
			if ($this->get('OrderItemEgoodDownloads') >= $this->getMaxDownloads()) {
				throw new miException(_t('Maximum number of downloads reached.'));
			}
			if (date(MI_MYSQL_DATE_FORMAT) > $this->get('OrderItemEgoodExpirationDate')) {
				throw new miException(_t('E-Good download link has expired.'));
			}
			if ($this->isLocked()) {
				throw new miException(_t('E-Good is locked.'));
			}
		}
		
		/**
		 * @return bool
		 */
		public function isLocked()
		{
			return ($this->get('OrderItemEgoodIsLocked') == self::ORDER_ITEM_EGOOD_LOCKED);
		}
		
		/**
		 * @return string
		 */
		protected function getFileField()
		{
			return 'OrderItemEgoodFile';
		}
		
		protected function increaseDownloadCount()
		{
			$this->set('OrderItemEgoodDownloads', $this->get('OrderItemEgoodDownloads') + 1);
			$this->update();
		}
		
		/**
		 * @return string A date in the Y-m-d (yyyy-mm-dd) format
		 */
		protected function getExpirationDate()
		{
			$settingTtlDays = scSettings::getSettingValue(scSettings::SETTING_DOWNLOAD_TTL_DAYS);
			$ttlSeconds = $settingTtlDays * 24 * 60 * 60;
			
			$unixExpiration = time() + $ttlSeconds;
			
			return date(MI_MYSQL_DATE_FORMAT, $unixExpiration);
		}
	}
?>