<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductEgood extends scEgoodDomainObject {
		
		public function __construct($productEgoodId = null)
		{
			parent::__construct(new miSqlRecord('ProductEgoods', 'ProductEgoodID'));
			if (!is_null($productEgoodId)) {
				$this->read($productEgoodId);
			}
		}
		
		/**
		 * @return string
		 */
		protected function getFileField()
		{
			return 'ProductEgoodFile';
		}
		
		/**
		 * Deletes an e-good or more.
		 * 
		 * If array is supplied as $pkValue the method attempts to delete all e-goods with IDs in the array.
		 * @param int|array|null $pkValue
		 * @return void
		 * @throws miException
		 */
		public function delete($pkValue = null)
		{
			if (is_null($pkValue)) {
				// EgoodID is not passed.
				// We'll try to find it in the data row.
				$dataRow = $this->getRow();
				$pkName = $this->getPKName();
				if (!isset($dataRow[$pkName]) || empty($dataRow[$pkName])) {
					throw new miException(_i('Unable to delete: unknown e-good.'));
				}
				$pkValue = array($dataRow[$pkName]);
			}
			
			if (!is_array($pkValue)) {
				$pkValue = array($pkValue);
			}
			
			if (empty($pkValue)) {
				return true;
			}
			
			$pkValue = array_map('intval', $pkValue);
			
			// We've made sure that $pkValue holds an array of IDs
			$this->deleteFiles($pkValue);
			$query = 'DELETE FROM ProductEgoods WHERE ProductEgoodID IN (' . implode(', ', $pkValue) . ')';
			miStaticDBUtil::execSQL($query);
			return true;
		}
		
		/**
		 * Removes non-deletable e-egoods from the passed array.
		 * Note: The method will not preserve the array's keys. It does not use references either.
		 * @param array $egoodsArray
		 * @return array
		 */
		public function getDeletableEgoodsFromArray($egoodsArray)
		{
			$return = array();
			foreach ($egoodsArray as $egood) {
				if (strpos($egood[$this->getFileField()], scPaths::EGOODS_PRODUCTS) === 0) {
					$return[] = $egood;
				}
			}
			
			return $return;
		}
		
		/**
		 * Unlinks all deletable files.
		 * @param array $egoodIds
		 */
		protected function deleteFiles($egoodIds)
		{
			$recordset = new miSqlRecordset('ProductEgoods');
			$recordset->addFilter(new miSqlFilterIn('ProductEgoodID', $egoodIds));
			$egoods = $recordset->getAllRecords();
			// $egoods now contains all e-goods. Some of them should not be deleted.
			$egoods = $this->getDeletableEgoodsFromArray($egoods);
			foreach ($egoods as $egood) {
				$egoodFile = scPaths::getItemPath($egood[$this->getFileField()], scPaths::RESOURCE_BASE_EGOOD);
				@unlink($egoodFile);
			}
		}
	}
?>