<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductImageDomainObject extends scDomainObject {
		
		const WIDTH = 0;
		const HEIGHT = 1;
		const FIELD = 2;
		
		protected $_imageSizes = array(
			scPaths::RESOURCE_PRODUCT_THUMB => array(
				self::WIDTH => scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH,
				self::HEIGHT => scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_HEIGHT,
				self::FIELD => 'ProductImageThumb'
			),
			scPaths::RESOURCE_PRODUCT_BOX_IMAGE => array(
				self::WIDTH => scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_WIDTH,
				self::HEIGHT => scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_HEIGHT,
				self::FIELD => 'ProductImageBox'
			),
			scPaths::RESOURCE_PRODUCT_LARGE_IMAGE => array(
				self::WIDTH => scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_WIDTH,
				self::HEIGHT => scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_HEIGHT,
				self::FIELD => 'ProductImageLarge'
			),
			scPaths::RESOURCE_PRODUCT_IMAGE => array(
				self::WIDTH => scSettings::SETTING_PRODUCT_IMAGE_MAX_WIDTH,
				self::HEIGHT => scSettings::SETTING_PRODUCT_IMAGE_MAX_HEIGHT,
				self::FIELD => 'ProductImage'
			),
		);
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('ProductImages', 'ProductImageID'));
		}
		
		public function regenerateSizesFromOriginal()
		{
			$original = scPaths::getStoreRootDir() . $this->get('ProductImageOriginal');
			if (!file_exists($original)) {
				throw new miException(sprintf(_t('Original image not found. Product ID: %s.'), $this->get('ProductID')));
			}
			
			foreach ($this->_imageSizes as $resourceType => $size) {
				$width = scSettings::getSettingValue($size[self::WIDTH]);
				$height = scSettings::getSettingValue($size[self::HEIGHT]);
				$path = ($this->has($size[self::FIELD]) ? $this->get($size[self::FIELD]) : '');
				if (empty($path)) {
					$path = scPaths::getItemPath($this->get($this->getPKName()), $resourceType, scPaths::PATH_WEB);
				}
				$target = scPaths::getStoreRootDir() . $path;
				scImageHandler::resizeImage($original, $target, $width, $height);
				
				list($resultWidth, $resultHeight) = scImageHandler::getImageSize($target);
				$resultSize = filesize($target);
				
				$this->set($size[self::FIELD], $path);
				$this->set($size[self::FIELD] . 'Width', $resultWidth);
				$this->set($size[self::FIELD] . 'Height', $resultHeight);
				$this->set($size[self::FIELD] . 'Size', $resultSize);
			}
			
			return $this->update();
		}
		
		protected function onDelete($pkValue)
		{
			foreach ($pkValue as $imageId) {
				$this->unlinkFiles($imageId);
			}
		}
		
		/**
		 * Unlinks the product image and its thumbnail.
		 * Throws a miException if the passed $imageId is not an unsigned integer.
		 * @param int $imageId
		 * @throws miException
		 */
		protected function unlinkFiles($imageId)
		{
			if (!preg_match('/^[0-9]+$/', $imageId))
				throw new miException('Invalid image ID.');
			
			$imageFile = scPaths::getItemPath($imageId, scPaths::RESOURCE_PRODUCT_IMAGE);
			$originalImageFile = scPaths::getItemPath($imageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE);
			$largeImageFile = scPaths::getItemPath($imageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE);
			$boxImageFile = scPaths::getItemPath($imageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE);
			$thumbFile = scPaths::getItemPath($imageId, scPaths::RESOURCE_PRODUCT_THUMB);
			@unlink($imageFile);
			@unlink($originalImageFile);
			@unlink($largeImageFile);
			@unlink($boxImageFile);
			@unlink($thumbFile);
		}
	}
?>