<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductVariant extends scDomainObject implements IStockable, IProductListingItem {
		
		const WIDTH = 0;
		const HEIGHT = 1;
		const FIELD = 2;
		
		/**
		 * @var array([scProductVariantOption])
		 */
		protected $_options = array();
		
		protected $_productSl = null;
		
		/**
		 * @var scPrice
		 */
		protected $_price = null;
		
		protected $_imageSizes = array(
			array(
				self::WIDTH => scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH,
				self::HEIGHT => scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_HEIGHT,
				self::FIELD => 'ProductVariantImageThumb'
			),
			array(
				self::WIDTH => scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_WIDTH,
				self::HEIGHT => scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_HEIGHT,
				self::FIELD => 'ProductVariantImageBox'
			),
			array(
				self::WIDTH => scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_WIDTH,
				self::HEIGHT => scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_HEIGHT,
				self::FIELD => 'ProductVariantImageLarge'
			),
			array(
				self::WIDTH => scSettings::SETTING_PRODUCT_IMAGE_MAX_WIDTH,
				self::HEIGHT => scSettings::SETTING_PRODUCT_IMAGE_MAX_HEIGHT,
				self::FIELD => 'ProductVariantImage'
			),
		);
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('ProductVariants', 'ProductVariantID'));
		}
		
		public function __clone()
		{
			$this->_record = clone $this->_record;
		}
		
		public function set($name, $value)
		{
			if ($name == 'ProductVariantPrice' && is_array($value)) {
				// Set the variant's price to the one set for the default customer group for quantity=1
				$price = 0;
				foreach ($value as $wholesale) {
					if ($wholesale['ProductWholesalePriceQuantity'] != 1) {
						continue;
					}
					if ($wholesale['CustomerGroupID'] != scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP)) {
						continue;
					}
					$price = $wholesale['ProductWholesalePriceAmount'];
					break;
				}
				$value = $price;
			}
			
			// Allow setting PK value
			$this->_record->set($name, $value);
		}
		
		/**
		 * @param array([scProductVariantOption]) $options
		 */
		public function setOptions($options)
		{
			$this->_options = $options;
		}
		
		/**
		 * @return scProductSL
		 */
		public function getProductSL()
		{
			if (is_null($this->_productSl)) {
				$this->_productSl = $this->createProductSl();
				
				$row = $this->getRow();
				if (isset($row['ProductCode']))			// If we have the ProductCode assume we have enough product info
					$this->_productSl->setRow($this->getRow()); // FIXME: the data row consists of data of product, image and variant
				else
					$this->_productSl->read($this->get('ProductID'));
			}
			return $this->_productSl;
		}
		
		public function getSelectedOptionsArray()
		{
			$recordset = new miSqlRecordset('ProductVariantOptions');
			$recordset->addJoinCondition('INNER', 'Options', 'USING(OptionID)');
			$recordset->addFilter(new miSqlFilterEqual('ProductVariantID', $this->get('ProductVariantID')));
			$recordset->setSelectFields(array(
				'OptionGroupID',
				'ProductVariantOptions.OptionID'
			));
			$result[scProductVariantOption::TYPE_GLOBAL] = miUtil::getKeyValueArray($recordset->getAllRecords(), 'OptionGroupID', 'OptionID');
			
			$recordset = new miSqlRecordset('ProductVariantLocalOptions');
			$recordset->addJoinCondition('INNER', 'LocalOptions', 'ON ProductVariantLocalOptions.LocalOptionID = LocalOptions.LocalOptionID');
			$recordset->addFilter(new miSqlFilterEqual('ProductVariantID', $this->get('ProductVariantID')));
			$recordset->setSelectFields(array(
				'LocalOptionGroupID',
				'ProductVariantLocalOptions.LocalOptionID'
			));
			$result[scProductVariantOption::TYPE_LOCAL] = miUtil::getKeyValueArray($recordset->getAllRecords(), 'LocalOptionGroupID', 'LocalOptionID');
			
			return $result;
		}
		
		public function getSelectedOptionsDetails()
		{
			$recordset = new miSqlRecordset('ProductVariantOptions');
			$recordset->addJoinCondition('INNER', 'Options', 'USING(OptionID)');
			$recordset->addJoinCondition('LEFT', 'OptionsML', 'ON Options.OptionID = OptionsML.OptionID AND OptionsML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId());
			$recordset->addFilter(new miSqlFilterEqual('ProductVariantID', $this->get('ProductVariantID')));
			$result[scProductVariantOption::TYPE_GLOBAL] = $recordset->getAllRecords();
			
			$recordset = new miSqlRecordset('ProductVariantLocalOptions');
			$recordset->addJoinCondition('INNER', 'LocalOptions', 'ON ProductVariantLocalOptions.LocalOptionID = LocalOptions.LocalOptionID');
			$recordset->addJoinCondition('LEFT', 'LocalOptionsML', 'ON LocalOptions.LocalOptionID = LocalOptionsML.LocalOptionID AND LocalOptionsML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId());
			$recordset->addFilter(new miSqlFilterEqual('ProductVariantID', $this->get('ProductVariantID')));
			$result[scProductVariantOption::TYPE_LOCAL] = $recordset->getAllRecords();
			
			return $result;
		}
		
		/**
		 * @param bool $strict Should this be an exact match?
		 * @return int|null
		 */
		public function readByOptions($strict = true)
		{
			if (empty($this->_options)) {
				return null;
			}
			
			$filter = array();
			// Count variant options by table so the count can be later checked.
			$optionTables = array();
			$filter[] = 'ProductID = ' . intval($this->get('ProductID'));
			foreach ($this->_options as $option) {
				if (($strict || !$option->isNew()) || (!$option->isNewGroup() && $option->isNew())) {
					// Count the number of expected options.
					if (!isset($optionTables[$option->getTableName()])) {
						$optionTables[$option->getTableName()] = 0;
					}
					$optionTables[$option->getTableName()] += 1;
				}
				if (!$strict && $option->isNew()) {
					continue;
				}
				// Add filter by specific option
				$optionName = ($option->getTableName() == 'ProductVariantLocalOptions') ? 'LocalOptionID' : 'OptionID';
				$filter[] = 'ProductVariantID IN (SELECT ProductVariantID FROM ' . $option->getTableName() . ' WHERE '.$optionName.' = ' . intval($option->getOptionId()) . ')';
			}
			
			// Generate options count checks
			foreach ($optionTables as $table => $items) {
				$optionName = ($table == 'ProductVariantLocalOptions') ? 'LocalOptionID' : 'OptionID';
				$filter[] = '(SELECT COUNT('.$optionName.') FROM ' . $table . ' WHERE ProductVariantID = ProductVariants.ProductVariantID) = ' . intval($items) . '';
			}
			
			// Combine all filters
			$filter = '(' . implode(') AND (', $filter) . ')';
			
			$query = 'SELECT ProductVariantID FROM ProductVariants WHERE ' . $filter;
			if (!$strict) {
				$query .= ' LIMIT 1';
			}
			$result = miStaticDBUtil::execSelect($query);
			
			if (!empty($result) && isset($result[0]) && isset($result[0]['ProductVariantID'])) {
				if ($strict && (count($result) > 1)) {
					throw new miException(sprintf(_i('%s product variants found in strict reading mode.'), count($result)));
				}
				
				$this->read((int)$result[0]['ProductVariantID']);
				return (int)$result[0]['ProductVariantID'];
			}
			
			return null;
		}
		
		public function insert()
		{
			$variantId = parent::insert();

			$values = array();
			foreach ($this->_options as $option) {
				$optionName = ($option->getTableName() == 'ProductVariantLocalOptions') ? 'LocalOptionID' : 'OptionID';
				$values[$option->getTableName()][] = array(
					'ProductVariantID' => $variantId,
					$optionName => $option->getOptionId()
				);
			}
			
			foreach ($values as $table => $multiValues) {
				miStaticDBUtil::execMultiInsert($table, $multiValues);
			}
			
			return $variantId;
		}
		
		public function update()
		{
			$data = $this->getRow();
			$updatedFields = array_keys($data);
			$updatedFields = array_intersect($updatedFields, array('ProductVariantImage', 'ProductVariantImageThumb', 'ProductVariantQuantity'));
			
			if (!empty($updatedFields)) {
				// Read a copy of the variant
				$copy = clone $this;
				$copy->read($this->get($this->getPKName()));
				
				if (isset($data['ProductVariantImage']) && $copy->get('ProductVariantImage') != $data['ProductVariantImage']) {
					$filename = scPaths::getStoreRootDir() . $copy->get('ProductVariantImage');
					if (file_exists($filename) && is_file($filename)) {
						unlink($filename);
					}
				}
				if (isset($data['ProductVariantImageThumb']) && $copy->get('ProductVariantImageThumb') != $data['ProductVariantImageThumb']) {
					$filename = scPaths::getStoreRootDir() . $copy->get('ProductVariantImageThumb');
					if (file_exists($filename) && is_file($filename)) {
						unlink($filename);
					}
				}
				if (isset($data['ProductVariantQuantity']) && ($copy->get('ProductVariantQuantity') != $data['ProductVariantQuantity'])) {
					$this->onQuantityChange($copy);
				}
			}
			
			$this->updateVariantOptions();
			
			return parent::update();
		}
		
		protected function onQuantityChange(scProductVariant $old)
		{
			$newQuantity = $this->get('ProductVariantQuantity');
			$oldQuantity = $old->get('ProductVariantQuantity');
			
			if ($newQuantity == $oldQuantity) {
				// Nothing to do.
				return;
			}
			
			if (($oldQuantity <= 0) && ($newQuantity <= 0)) {
				// Quantity was negative or zero and is still negative or zero.
				// This should not affect the products.
				return;
			}
			
			// Calculate the difference that should be copied to the product.
			$quantityDiff = max(0, $newQuantity) - max(0, $oldQuantity);
			
			$product = new scProductDomainObject();
			$product->read($this->get('ProductID'));
			$product->set('ProductQuantity', max(0, $product->get('ProductQuantity') + $quantityDiff));
			$product->update();
		}
		
		public function delete($pkValue = null)
		{
			if (is_null($pkValue)) {
				$pkValue = array($this->get($this->getPKName()));
			}
			
			if (!is_array($pkValue)) {
				$pkValue = array($pkValue);
			}
			
			$images = array(
				'ProductVariantImage',
				'ProductVariantImageLarge',
				'ProductVariantImageBox',
				'ProductVariantImageThumb',
			);
			
			foreach ($pkValue as $id) {
				$this->read($id);
				
				foreach ($images as $imageField) {
					$image = scPaths::getStoreRootDir() . $this->get($imageField);
					if (is_file($image)) {
						unlink($image);
					}
				}
			}
			
			parent::delete($pkValue);
		}
			
		public function getQuantityInStock()
		{
			return $this->get('ProductVariantQuantity');
		}
			
		public function reduceQuantityInStock($itemsRemoved)
		{
			$newQuantity = $this->get('ProductVariantQuantity') - $itemsRemoved;
			$this->setQuantityInStock($newQuantity);
		}
		
		public function setQuantityInStock($newQuantityInStock)
		{
			$this->set('ProductVariantQuantity', $newQuantityInStock);
			$this->update();
		}
		
		public function isOutOfStock($quantity = 1)
		{
			if ((int)$this->getQuantityInStock() >= (int)$quantity)
				return false;
			
			$outOfStockHandling = scSettings::getSettingValue(scSettings::SETTING_OUT_OF_STOCK_HANDLING);
			if (($outOfStockHandling == scProductService::OUT_OF_STOCK_HANDLING_TAG) or ($outOfStockHandling == scProductService::OUT_OF_STOCK_HANDLING_HIDE))
				return true;
			return false;
		}
		
		/**
		 * @return scProductImageInfo
		 */
		public function getImageInfo()
		{
			$thumb = $this->get('ProductVariantImageThumb');
			if (empty($thumb) || (!file_exists(scPaths::getStoreRootDir() . $thumb))) {
				return $this->getProductSL()->getImageInfo();
			}
			
			$imageInfo = new scProductImageInfo($this->get('ProductVariantImage'), $this->get('ProductVariantImageWidth'), $this->get('ProductVariantImageHeight'), $this->get('ProductVariantImageSize'));
			$imageInfo->setThumbnail(new scImageInfo($this->get('ProductVariantImageThumb'), $this->get('ProductVariantImageThumbWidth'), $this->get('ProductVariantImageThumbHeight'), $this->get('ProductVariantImageThumbSize')));
			$imageInfo->setBoxImage(new scImageInfo($this->get('ProductVariantImageBox'), $this->get('ProductVariantImageBoxWidth'), $this->get('ProductVariantImageBoxHeight'), $this->get('ProductVariantImageBoxSize')));
			$imageInfo->setLargeImage(new scImageInfo($this->get('ProductVariantImageLarge'), $this->get('ProductVariantImageLargeWidth'), $this->get('ProductVariantImageLargeHeight'), $this->get('ProductVariantImageLargeSize')));
			
			return $imageInfo;
		}
		
		public function hasImage()
		{
			$thumb = $this->get('ProductVariantImageThumb');
			if (empty($thumb) || (!file_exists(scPaths::getStoreRootDir() . $thumb))) {
				return false;
			}
			
			return true;
		}
		
		public function getPK()
		{
			return $this->get($this->getPKName());
		}
		
		public function getName()
		{
			return $this->getProductSL()->getName();
		}
		
		public function getDescription()
		{
			return $this->getProductSL()->getDescription();
		}
		
		/**
		 * @return scPrice
		 */
		public function getPrice()
		{
			if (is_null($this->_price)) {
				$amount = scMultiCurrencyAmount::factoryBaseCurrency($this->get('ProductVariantPrice'));
				// Add the amount in the current currency so they are both set in the object and they are correctly used in calculations.
				$amount->set(scCurrencyUtil::getCurrentCurrency(), scCurrencyUtil::getCurrentCurrencyAmount($this->get('ProductVariantPrice')));
				$this->_price = scPrice::factory($amount);
			}
			return $this->_price;
		}
		
		/**
		 * @return string
		 */
		public function getUrl()
		{
			return $this->getProductSL()->getUrl();
		}
		
		/**
		 * @return int
		 */
		public function getProductId()
		{
			return (int)$this->getProductSL()->getProductId();
		}
		
		/**
		 * @return int
		 */
		public function getCategoryId()
		{
			return (int)$this->getProductSL()->getCategoryId();
		}
		
		/**
		 * @return int[]
		 */
		public function getAllCategoryIds()
		{
			return $this->getProductSL()->getAllCategoryIds();
		}
		
		/**
		 * @return int
		 */
		public function getProductClassId()
		{
			return (int)$this->getProductSL()->getProductClassId();
		}
		
		public function getCreateTimestamp()
		{
			return $this->getProductSL()->getCreateTimestamp();
		}
		
		/**
		 * @return string
		 */
		public function getSkuCode()
		{
			return $this->get('ProductVariantCode');
		}
		
		/**
		 * @return scWeight
		 */
		public function getWeight()
		{
			return new scWeight($this->get('ProductVariantWeight'), scSettings::getSettingValue(scSettings::SETTING_LOCALIZATION_WEIGHT_UNIT));
		}
		
		/**
		 * @return int or null
		 */
		public function getManufacturerId()
		{
			return $this->getProductSL()->getManufacturerId();
		}
		
		/**
		 * @return scManufacturerDomainObject or null
		 */
		public function getManufacturer()
		{
			return $this->getProductSL()->getManufacturer();
		}
		
		public function regenerateSizesFromOriginal()
		{
			$original = scPaths::getStoreRootDir() . $this->get('ProductVariantImageOriginal');
			if (!file_exists($original)) {
				throw new miException(sprintf(_t('Original variant image not found. Product ID: %s'), $this->get('ProductID')));
			}
			
			foreach ($this->_imageSizes as $size) {
				$width = scSettings::getSettingValue($size[self::WIDTH]);
				$height = scSettings::getSettingValue($size[self::HEIGHT]);
				$target = scPaths::getStoreRootDir() . $this->get($size[self::FIELD]);
				scImageHandler::resizeImage($original, $target, $width, $height);
				
				list($resultWidth, $resultHeight) = scImageHandler::getImageSize($target);
				$resultSize = filesize($target);
				
				$this->set($size[self::FIELD] . 'Width', $resultWidth);
				$this->set($size[self::FIELD] . 'Height', $resultHeight);
				$this->set($size[self::FIELD] . 'Size', $resultSize);
			}
			
			return $this->update();
		}
		
		/**
		 * @return bool
		 */
		public function isEgood()
		{
			return $this->getProductSL()->isEgood();
		}
		
		/**
		 * @return scProductSL
		 */
		protected function createProductSl()
		{
			return new scProductSL();
		}
		
		protected function updateVariantOptions()
		{
			if (empty($this->_options)) {
				return;
			}
			
			$productVariantId = $this->getPK();
			
			$newOptions = array();
			foreach ($this->_options as $newOption) {
				$newOptions[$newOption->getTableName()][] = $newOption->getOptionId();
			}
			
			$currentOptions = array();
			
			$recordset = new miSqlRecordset('ProductVariantOptions');
			$recordset->addFilter(new miSqlFilterEqual('ProductVariantID', $productVariantId));
			$recordset->setSelectFields(array('OptionID'));
			$records = $recordset->getAllRecords();
			$currentOptions['ProductVariantOptions'] = array();
			foreach ($records as $record) {
				$currentOptions['ProductVariantOptions'][] = $record['OptionID'];
			}
			
			$recordset = new miSqlRecordset('ProductVariantLocalOptions');
			$recordset->addFilter(new miSqlFilterEqual('ProductVariantID', $productVariantId));
			$recordset->setSelectFields(array('LocalOptionID'));
			$records = $recordset->getAllRecords();
			$currentOptions['ProductVariantLocalOptions'] = array();
			foreach ($records as $record) {
				$currentOptions['ProductVariantLocalOptions'][] = $record['LocalOptionID'];
			}
			
			foreach ($currentOptions as $tableName => $options) {
				$toInsert = array();
				$toDelete = array();
				if (!isset($newOptions[$tableName])) {
					$toDelete = $options;
				} else {
					$toDelete = array_diff($options, $newOptions[$tableName]);
					$toInsert = array_diff($newOptions[$tableName], $options);
				}
				
				$fieldName = ($tableName == 'ProductVariantOptions' ? 'OptionID' : 'LocalOptionID');
				
				if (!empty($toDelete)) {
					$sql = 'DELETE FROM ' . $tableName . ' WHERE ProductVariantID = ' . $productVariantId .
						' AND ' . $fieldName . ' IN (' . implode(', ', $toDelete) . ')';
					miStaticDBUtil::execSQL($sql);
				}
				if (!empty($toInsert)) {
					$values = array();
					foreach ($toInsert as $optionId) {
						$values[] = array(
							'ProductVariantID' => $productVariantId,
							$fieldName => $optionId,
						);
					}
					miStaticDBUtil::execMultiInsert($tableName, $values);
				}
			}
		}
	}
?>