<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Objects implementing ICartItem interface can be added to the shopping cart
	 */
	interface ICartItem {
		
		/**
		 * The IProductListingItem price.
		 */
		const PRICE_TYPE_BASE = 1;
		
		/**
		 * The price, provided by the "Prices" module.
		 */
		const PRICE_TYPE_WHOLESALE = 2;
		
		/**
		 * The price after option price modifications have been applied.
		 * If there is a wholesale price, the option modifications should be applied on it.
		 * If there is no wholesale price, the option mods are applued on the BASE price.
		 */
		const PRICE_TYPE_OPTIONS_MODIFIED = 3;
		
		/**
		 * The price with applied all IProductPriceModifier implementations.
		 * Those are applied on the OPTIONS price.
		 */
		const PRICE_TYPE_MODIFIED = 4;
		
		/**
		 * Change the quantity of this item in the cart
		 * 
		 * @param int $quantity
		 */
		public function setQuantity($quantity);
		
		/**
		 * Set the price object with the item price.
		 * 
		 * @param int $priceType PRICE_TYPE_*
		 * @param scPrice $priceObj
		 */
		public function setPrice($priceType, scPrice $priceObj = null);
		
		/**
		 * Set the cart item code
		 * 
		 * @param string $code
		 */
		public function setCartItemCode($code);
		
		/**
		 * Set the weight of the item
		 * 
		 * @param scWeight $weight
		 */
		public function setWeight(scWeight $weight);
		
		/**
		 * Returns the total amount of item price - item price multiplied by quantity
		 * 
		 * @return scMultiCurrencyAmount
		 */
		public function getTotal();
		
		/**
		 * Gets the item quantity in cart
		 * 
		 * @return int
		 */
		public function getQuantity();
		
		/**
		 * Gets the price object with the item price
		 * 
		 * @param int $priceType PRICE_TYPE_*
		 * @return scPrice
		 */
		public function getPrice($priceType = self::PRICE_TYPE_MODIFIED);
		
		/**
		 * Gets the item name
		 * 
		 * @return string
		 */
		public function getName();
		
		/**
		 * Gets the item code
		 * 
		 * @return string
		 */
		public function getCartItemCode();
		
		/**
		 * Gets the item weight
		 * 
		 * @return scWeight
		 */
		public function getWeight();
		
		/**
		 * Gets hash of the cart item used to compare two cart items
		 * The hash should not include quantity
		 * 
		 * @return string
		 */
		public function getHash();
		
		/**
		 * Gets item image info
		 * @return scImageInfo
		 */
		public function getImageInfo();
	}
?>