<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * All payment modules implement this interface
	 *
	 */
	interface IPaymentModule {
		
		const CONTENT_TYPE_PAYMENT_FORM = 1; // returns the payment form
		const CONTENT_TYPE_CHECKOUT_PENDING = 2; // returns message in checkout_final when payment is pending
		const CONTENT_TYPE_CHECKOUT_SUCCESS = 3; // returns message in checkout_final when payment is successfull
		const CONTENT_TYPE_CHECKOUT_FAILURE = 4; // returns message in checkout_final when payment is failed
		const CONTENT_TYPE_VIEW_ORDER = 5; // returns message in customer profile view order
		
		/**
		 * Payment module constructor
		 */
		public function __construct($moduleId);
		
		/**
		 * Return the payment content
		 *
		 * @param int $contentType the content type requested
		 * @param array $params additional parameter
		 * @return string the html of the data entry form
		 */
		public function getPaymentContent($contentType, $params = array());
		
		/**
		 * Return the 3-letter payment currency code
		 * If the result is null then it's the base currency
		 * 
		 * @return string the payment currency code
		 */
		public function getPaymentCurrencyCode();
		
		/**
		 * Process the payment
		 * 
		 * @param string $payAmount the amount to be paid
		 * @param string $payCurrencyCode the currency
		 * @param scCart $cart the shopping cart object
		 * @param array $paramsPost any parameters posted by the user from the payment form
		 * @return scPaymentProcessResult the payment process result
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost);
		
		/**
		 * Does the payment module add a payment step in the checkout process
		 * If false, after the confirmation, the customer is transferred directly to checkout success/failure page
		 * If true, the customer has to go through additional payment step (e.g. redirect to third-party payment processor)
		 * @return boolean
		 */
		public function hasCheckoutStep();
	}
?>