<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Payment modules that support additional operations implement this interface
	 */
	interface IPaymentModuleOperations {
		/**
		 * Return supported operations
		 *
		 * @return array array of scPaymentOperationDefinition objects
		 */
		public function getPaymentOperationDefinitions(scOrder $order);
	}
	
	/**
	 * Definition of a payment related operation
	 */
	class scPaymentOperationDefinition {
		const OPERATION_TYPE_OTHER = 0;
		const OPERATION_TYPE_REFUND = 1;
		const OPERATION_TYPE_PARTIAL_REFUND = 2;
		const OPERATION_TYPE_VOID = 3;
		const OPERATION_TYPE_CAPTURE = 4;
		const OPERATION_TYPE_AUTHORIZATION = 5;
		const OPERATION_TYPE_REAUTHORIZATION = 6;
		const OPERATION_TYPE_NON_REF_CREDIT = 7;
		
		protected $_operationType;
		protected $_operationCode;
		protected $_operationDisplayName;
		protected $_operationFunc;
		
		/**
		 * Create the object
		 * 
		 * @param int $operationType the type of the payment operation
		 * @param string $operationCode the operation code, unique for this module
		 * @param callback $operationFunc function callback to call to process the operation
		 * @param string $operationDisplayName if empty there won't be a button for this operation
		 */
		public function __construct($operationType, $operationCode, $operationFunc, $operationDisplayName = '')
		{
			$this->_operationType = $operationType;
			$this->_operationCode = $operationCode;
			$this->_operationFunc = $operationFunc;
			$this->_operationDisplayName = $operationDisplayName;
		}
		
		public function getOperationType()
		{
			return $this->_operationType;
		}
		
		public function getOperationCode()
		{
			return $this->_operationCode;
		}
		
		public function getOperationFunc()
		{
			return $this->_operationFunc;
		}
		
		public function getOperationDisplayName()
		{
			return $this->_operationDisplayName;
		}
	}
?>