<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Used to modify the product prices offered to the customers (e.g. in the product listing and product details pages)
	 */
	interface IProductPriceModifier
	{
		/**
		 * Returns the price modifications for $products
		 * Returns null if there are no price modifications
		 * 
		 * @param array $products array of IProductListingItem objects
		 * @return array array of scProductPriceModification objects
		 */
		public function getPriceModifications($products);
		
		/**
		 * Returns any price modifcations that should be applied to the product when added to the cart
		 *
		 * @param scCartItem $cartItem
		 * @return array array of scProductPriceModification objects
		 */
		public function getCartPriceModifications(scCartItem $cartItem);
	}
	
	/**
	 * Contains info for a price modification
	 */
	class scProductPriceModification
	{
		const DISPLAY_MODE_DISCOUNT = 1;
		const DISPLAY_MODE_PRODUCT_PRICE = 2;
		
		protected $_productId;
		
		/**
		 * @var scMultiCurrencyAmount
		 */
		protected $_amount;
		
		/**
		 * @var float
		 */
		protected $_percent;
		
		/**
		 * @var string
		 */
		protected $_description = '';
		
		/**
		 * @var bool
		 */
		protected $_isVisible = false;
		
		/**
		 * @var int
		 */
		protected $_displayMode;
		
		/**
		 * Constructs product price modification object
		 *
		 * @param int $productId the product id to which this price modification applies
		 * @param scMultiCurrencyAmount $amount the amount
		 * @param string $description the description of the price modification
		 * @param boolean $isVisible if the price modification is shown to the customer separately
		 */
		public function __construct($productId, scMultiCurrencyAmount $amount, $percent, $description = '', $isVisible = false, $displayMode = self::DISPLAY_MODE_DISCOUNT)
		{
			$this->_productId = $productId;
			$this->_amount = $amount;
			$this->_percent = $percent;
			$this->_description = $description;
			$this->_isVisible = $isVisible;
			$this->_displayMode = $displayMode;
		}
		
		public function getProductId()
		{
			return $this->_productId;
		}
		
		public function setAmount(scMultiCurrencyAmount $amount)
		{
			$this->_amount = $amount;
		}
		
		/**
		 * @return scMultiCurrencyAmount
		 */
		public function getAmount()
		{
			return $this->_amount; 
		}
		
		public function setPercent($percent)
		{
			$this->_percent = $percent;
		}
		
		/**
		 * @return float
		 */
		public function getPercent()
		{
			return $this->_percent;
		}
		
		public function setDescription($description)
		{
			$this->_description = $description;
		}
		
		public function getDescription()
		{
			return $this->_description;
		}
		
		public function setIsVisible($isVisible)
		{
			$this->_isVisible = $isVisible;
		}
		
		public function getIsVisible()
		{
			return $this->_isVisible;
		}
		
		public function getDisplayMode()
		{
			return $this->_displayMode;
		}
	}
?>