<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * All shipping modules implement this interface
	 *
	 */
	interface IShippingModule {
		const SHIPPING_RATES = 'rates';
		const SHIPPING_RATES_CURRENCY = 'currency';
		const SHIPPING_RATES_MESSAGE = 'message';
		
		/**
		 * Returns array with the support shipping methods by this module
		 * The shipping module code must be unique within the module:
		 * array(ShippingMethodCode => ShippingMethodName, ...);
		 * 
		 * @return array shipping methods, eg. array(1 => 'Free Shipping')
		 */
		function getShippingMethods();
		
		/**
		 * Returns the rates for shipping using a particlular shipping method
		 * If a rate for a particular method is not returned then the shipping method is unavailable
		 *
		 * @param scCart $cart the shopping cart object
		 * @param array $shippingMethods an array with the shipping methods we are interested in
		 * @return array associative array with SHIPPING_RATES, SHIPPING_RATES_CURRENCY and SHIPPING_RATES_MESSAGE as keys
		 */
		function getShippingRates(scCart $cart, $shippingMethods);
		
		/**
		 * Returns the image that will be shown in the admin area for this module
		 */
		function getShippingModuleImage();
	}
?>