<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Adapts a miDefaultDomainObject to miSqlRecord
	 */
	class scDomainToRecordAdapter extends miSqlRecord {
		
		/**
		 * @var miDefaultDomainObject
		 */
		protected $_domainObject;
		
		public function __construct(miDefaultDomainObject $domainObject)
		{
			$this->_domainObject = $domainObject;
		}
		
		/**
		 * @throws miException
		 */
		public function read($key, $value)
		{
			throw new miException('Not implementable: Domain Object can read only by primary key.');
		}
		
		/**
		 * Tries to get the table name from the domain object.
		 * If the domain object does not have a getTableName method an exception will be thrown.
		 * @throws miException
		 */
		public function getTableName()
		{
			if (!method_exists($this->_domainObject, 'getTableName')) {
				throw new miException('Not implementable: Domain Object does not know the table name.');
			}
			return $this->_domainObject->getTableName();
		}
		
		public function getPrimaryKeyColumn()
		{
			return $this->_domainObject->getPKName();
		}
		
		public function get($fieldName)
		{
			return $this->_domainObject->get($fieldName);
		}
		
		public function getPK()
		{
			return $this->_domainObject->get($this->_domainObject->getPKName());
		}
		
		public function has($name)
		{
			return $this->_domainObject->has($name);
		}
		
		public function set($fieldName, $fieldValue)
		{
			$this->_domainObject->set($fieldName, $fieldValue);
		}
		
		public function getRow()
		{
			return $this->_domainObject->getRow();
		}
		
		public function setRow($row)
		{
			$this->_domainObject->setRow($row);
		}
		
		public function readPK($pkValue)
		{
			$this->_domainObject->read($pkValue);
		}
		
		public function insert()
		{
			return $this->_domainObject->insert();
		}
		
		public function update()
		{
			return $this->_domainObject->update();
		}
		
		public function delete()
		{
			return $this->_domainObject->delete();
		}
	}
?>