<?php
	/**
	 * General Options Record Class
	 * 
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Handle and manage deleting and reading of data from the
	 * database.
	 *
	 * @copyright Copyright (c) 2003-2006 Mirchev Ideas Ltd. All rights reserved.
	 */
	class scPropertiesRecordML extends miPropertiesRecord {
		const LANGUAGE_ID = 'LanguageID';
		
		protected $_languages = array();
		
		
		/**
		 * scPropertiesRecordML constructor
		 * 
		 * @access public
		 * @param string $table database table name
		 * @param string $propertyField the name of the property field
		 * @param string $valueField the name of the value field
		 * @param array $mlFields
		 * @param array $languages
		 */
		public function __construct($table, $propertyField, $valueField, $languages)
		{
			parent::__construct($table, $propertyField, $valueField);
			$this->_languages = $languages;
		}
		
		
		/**
		 * Reads all properties from the db table
		 *
		 * Example:
		 * <code>
		 * <?php
		 * $propertiesRecord = new miPropertiesRecord('tableName', 'propertyField', 'valueField');
		 * $propertiesRecord->readPK($value);
		 * $row = $propertiesRecord->getRow();
		 * ?>
		 * </code>
		 * 
		 * @return void
		 * @throws miDBException
		 */
		public function readPK($value)
		{
			parent::readPK($value);
			$this->readML();
		}
		
		
		/**
		 * Read the multilanguage properties
		 */
		protected function readML()
		{
			// Read the multilanguage rows
			$query = 'SELECT * FROM ' . $this->_table . 'ML';
			$rows = miStaticDBUtil::execSelect($query);
			foreach ($rows as $row) {
				$languageId = $row[self::LANGUAGE_ID];
				$property = $row[$this->_propertyField];
				
				// Sanity check
				if (isset($this->_row[$property]) and !is_array($this->_row[$property]))
					unset($this->_row[$property]);
				
				$this->_row[$property][$languageId] = $row[$this->_valueField];
			}
		}
		
		/**
		 * Updates the properties table
		 *
		 * @access public
		 * @return void
		 * @throws miDBException
		 */
		public function update()
		{
			foreach ($this->_row as $property => $value) {
				if (is_array($value))
					continue;
				
				$row = array($this->_valueField => $value);
				miStaticDBUtil::execUpdate($this->_table, $row, $this->_propertyField, $property);
			}
			
			$this->updateML();
		}
		
		protected function updateML()
		{
			$names = array();
			foreach ($this->_row as $property => $value) {
				if (!is_array($value))
					continue;
				$names[] = '"' . sql_escape_string($property) . '"';
			}
			
			$sql = 'DELETE FROM ' . $this->_table . 'ML WHERE ' . $this->_propertyField . ' IN(' . implode(',', $names) . ')';
			miStaticDBUtil::execSQL($sql);
			
			foreach ($this->_row as $propertyName => $value) {
				if (!is_array($value))
					continue;
				
				foreach ($value as $languageId => $propertyValue) {
					$values = array(
						self::LANGUAGE_ID => $languageId,
						$this->_propertyField => $propertyName,
						$this->_valueField => $propertyValue,
					);
					
					miStaticDBUtil::execInsert($this->_table . 'ML', $values);
				}
			}
		}
	}
?>