<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Date Period Filter
	 */
	class scSqlFilterDatePeriod extends miSqlFilterOneValue {
		
		// Date periods
		const DATE_PERIODS_TODAY = 1;
		const DATE_PERIODS_YESTERDAY = 2;
		const DATE_PERIODS_LAST_7_DAYS = 3;
		const DATE_PERIODS_THIS_MONTH = 4;
		const DATE_PERIODS_LAST_MONTH = 5;
		const DATE_PERIODS_THIS_YEAR = 6;
		const DATE_PERIODS_LAST_YEAR = 7;
		const DATE_PERIODS_ALL_TIME = 8;
		
		// Set params position
		const PARAM_PERIOD_TYPE = 0;
		const PARAM_PREDEFINED_PERIOD = 1;
		const PARAM_DATE_FROM = 2;
		const PARAM_DATE_TO = 3;
		
		const PERIOD_TYPE_PREDEFINED = 0;
		const PERIOD_TYPE_CUSTOM = 1;
		
		static public $datePeriods;

		static public function setDatePeriods()
		{
			self::$datePeriods[self::DATE_PERIODS_TODAY] = _t('Today');
			self::$datePeriods[self::DATE_PERIODS_YESTERDAY] = _t('Yesterday');
			self::$datePeriods[self::DATE_PERIODS_LAST_7_DAYS] = _t('Last 7 days');
			self::$datePeriods[self::DATE_PERIODS_THIS_MONTH] = _t('This Month');
			self::$datePeriods[self::DATE_PERIODS_LAST_MONTH] = _t('Last Month');
			self::$datePeriods[self::DATE_PERIODS_THIS_YEAR] = _t('This Year');
			self::$datePeriods[self::DATE_PERIODS_LAST_YEAR] = _t('Last Year');
			self::$datePeriods[self::DATE_PERIODS_ALL_TIME] = _t('All time');
		}
		
		/**
		 * @return string
		 */
		public function getSql()
		{
			$values = explode(',', $this->_value);
			$filters = array();
			
			// Date period
			if ($values[self::PARAM_PERIOD_TYPE] == self::PERIOD_TYPE_PREDEFINED) {
				$filters = $this->getFiltersForPredefinedPeriod($values);
			}
			
			// Date interval
			if ($values[self::PARAM_PERIOD_TYPE] == self::PERIOD_TYPE_CUSTOM) {
				$filters = $this->getFiltersForCustomPeriod($values);
			}
			
			return implode(' AND ', $filters);
		}
		
		protected function getFiltersForPredefinedPeriod($values)
		{
			$filters = array();
			
			$day = gmdate('d');
			$month = gmdate('m');
			$year = gmdate('Y');
			
			switch ($values[self::PARAM_PREDEFINED_PERIOD]) {
				case self::DATE_PERIODS_TODAY:
						$filters[] = $this->getFilterFromDate($day, $month, $year);
						$filters[] = $this->getFilterToDate($day, $month, $year);
						break;
				case self::DATE_PERIODS_YESTERDAY:
						$filters[] = $this->getFilterFromDate($day-1, $month, $year);
						$filters[] = $this->getFilterToDate($day-1, $month, $year);
						break;
				case self::DATE_PERIODS_LAST_7_DAYS:
						$filters[] = $this->getFilterFromDate($day-6, $month, $year);
						$filters[] = $this->getFilterToDate($day, $month, $year);
						break;
				case self::DATE_PERIODS_THIS_MONTH:
						$filters[] = $this->getFilterFromDate(1, $month, $year);
						$filters[] = $this->getFilterToDate($day, $month, $year);
						break;
				case self::DATE_PERIODS_LAST_MONTH:
						$filters[] = $this->getFilterFromDate(1, $month-1, $year);
						$filters[] = $this->getFilterToDate(0, $month, $year);
						break;
				case self::DATE_PERIODS_THIS_YEAR:
						$filters[] = $this->getFilterFromDate(1, 1, $year);
						$filters[] = $this->getFilterToDate($day, $month, $year);
						break;
				case self::DATE_PERIODS_LAST_YEAR:
						$filters[] = $this->getFilterFromDate(1, 1, $year-1);
						$filters[] = $this->getFilterToDate(0, 1, $year);
						break;
			}
			
			return $filters;
		}
		
		protected function getFiltersForCustomPeriod($values)
		{
			$filters = array();
			
			// Date from
			$dateFromArray = explode('/', $values[self::PARAM_DATE_FROM], 3);
 			if (count($dateFromArray) == 3) {
				list($dayFrom, $monthFrom, $yearFrom) = $dateFromArray;
				if (checkdate($monthFrom, $dayFrom, $yearFrom))
					$filters[] = $this->getFilterFromDate($dayFrom, $monthFrom, $yearFrom);
 			}
			
			// Date to
 			$dateToArray = explode('/', $values[self::PARAM_DATE_TO], 3);
 			if (count($dateToArray) == 3) {
				list($dayTo, $monthTo, $yearTo) = $dateToArray;
				if (checkdate($monthTo, $dayTo, $yearTo))
					$filters[] = $this->getFilterToDate($dayTo, $monthTo, $yearTo);
 			}
			
			return $filters;
		}
		
		/**
		 * Returns sql for filter from date (inclusive)
		 */
		protected function getFilterFromDate($day, $month, $year)
		{
			return sql_escape_string($this->_name) . ' >= ' . gmmktime(0, 0, 0, $month, $day, $year);
		}
		
		/**
		 * Returns sql for filter to date (inclusive)
		 */
		protected function getFilterToDate($day, $month, $year)
		{
			return sql_escape_string($this->_name) . ' < ' . gmmktime(0, 0, 0, $month, $day+1, $year);
		}
	}
	
	/**
	 * Checkbox Filter
	 */
	class scSqlFilterCheckbox extends miSqlFilterOneValue {
		public function getSql()
		{
			$values = explode(',', $this->_value);
			
			foreach ($values as $value)
				if($value != '')
					$filters[] = sql_escape_string($this->_name) . ' = ' . sql_escape_string($value);
			
			// Set sql
			if (isset($filters)) {
				$sql = '(' . implode(' OR ', $filters) . ')';
			} else {
				$sql = '';
			}
				
			return $sql;
		}
	}
	
	/**
	 * Checks if given field is null
	 */
	class scSqlFilterIsNull extends miSqlFilter {
		public function getSql()
		{
			return sql_escape_string($this->_name) . ' IS NULL';
		}
	}
	
	/**
	 * Builds sql filter, combining other sql filters with OR
	 */
	class scSqlFilterOr extends miSqlFilterOneValue {
		public function getSql()
		{
			$sqlStatements = array();
			foreach ($this->_value as $value)
				$sqlStatements[] = '(' . $value->getSql() . ')';
				
			return '(' . implode(' OR ', $sqlStatements) . ')';
		}
	}
	
	// Translate the date periods
	scSqlFilterDatePeriod::setDatePeriods();
?>