<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Handle and manage the inserting, updating, deleting and reading of data from the
	 * database. This class works only with one record row specified by a table primary key or
	 * unique field.
	 */
	class scSqlRecordML extends miSqlRecord {
		const LANGUAGE_ID = 'LanguageID';
		
		/**
		 * @access protected
		 */
		protected $_mlRows = array();
		protected $_mlFields;
		protected $_languages = array();
		
		/**
		 * miSqlRecord constructor. It takes two parameters table name and 
		 * table primary key
		 * 
		 * @access public
		 * @param string $table database table name
		 * @param string $primaryKey database table primary key
		 * @param array $mlFields multilanguage fields
		 */
		public function __construct($table, $primaryKey, $mlFields, $languages)
		{
			parent::__construct($table, $primaryKey);
			$this->_mlFields = $mlFields;
			$this->_languages = $languages;
		}
		
		/**
		 * Read a record where the specified field represented by $key
		 * is equal to the value represented by $value
		 * If there are is than one record matching the first one will be read in
		 *
		 * @param string $key the name of the field
		 * @param string $value the value 
		 * @return void
		 * @throws miDBException
		 */
		public function read($key, $value)
		{
			parent::read($key, $value);
			$this->readML($key, $value);
		}
		
		protected function readML($key, $value)
		{
			// Read the multilanguage rows
			$query = 'SELECT * FROM ' . $this->_table . 'ML WHERE ' . $key . '="' . sql_escape_string($value) . '"';
			$rows = miStaticDBUtil::execSelect($query);
			foreach ($rows as $row) {
				$languageId = $row[self::LANGUAGE_ID];
				foreach ($row as $dataKey => $dataValue) {
					if ($dataKey == $this->_primaryKey)
						continue;
					
					$this->_row[$dataKey][$languageId] = $dataValue;
				}
			}
		}
		
		/**
		 * Insert a record in the db table
		 * It also updates the the PK value of the inserted row
		 *
		 * @access public
		 * @return PK the primary key of the newly created record
		 * @throws miDBException
		 */
		public function insert()
		{
			$this->_row[$this->_primaryKey] = miStaticDBUtil::execInsert($this->_table, $this->getMainRow());
			$this->insertMultilanguage();
			return $this->_row[$this->_primaryKey];
		}
		
		/**
		 * Update a record in the db table
		 *
		 * @access public
		 * @return PK the primary key of the updated record
		 * @throws miDBException
		 */
		public function update()
		{
			$this->updateMultilanguage();
			
			miStaticDBUtil::execUpdate($this->_table, $this->getMainRow(), $this->_primaryKey, $this->_row[$this->_primaryKey]);
			return $this->_row[$this->_primaryKey];
		}
		
		/**
		 * Inserts multilanguage records
		 */
		protected function insertMultilanguage()
		{
			// Create the new multilanguage rows
			foreach ($this->_languages as $languageId) {
				$values = array();
				foreach ($this->_mlFields as $mlField) {
					$values[$mlField] = isset($this->_row[$mlField][$languageId]) ? $this->_row[$mlField][$languageId] : '';
				}
				$values[$this->_primaryKey] = $this->_row[$this->_primaryKey];
				$values[self::LANGUAGE_ID] = $languageId;
				
				miStaticDBUtil::execInsert($this->_table . 'ML', $values);
			}
		}
		
		/**
		 * Updates the multilanguage table - first deletes and then inserts
		 */
		protected function updateMultilanguage()
		{
			$sql = 'DELETE FROM ' . $this->_table . 'ML WHERE ' . $this->_primaryKey . '=' . sql_escape_string($this->_row[$this->_primaryKey]) .
				' AND ' . self::LANGUAGE_ID . ' IN(' . implode(',', $this->_languages) . ')';
			miStaticDBUtil::execSQL($sql);
			
			$this->insertMultilanguage();
		}
		
		/**
		 * Returns the row that should go into the main table (instead of the multilanguage table)
		 */
		protected function getMainRow()
		{
			$row = $this->_row;
			foreach ($this->_mlFields as $mlField)
				unset($row[$mlField]);
			unset($row[self::LANGUAGE_ID]);
			return $row;
		}
		
		/**
		 * Delete a record in the db table
		 *
		 * @access public
		 * @return PK the primary key of the deleted record
		 * @throws miDBException
		 */
		public function delete()
		{
			miStaticDBUtil::execDelete($this->_table . 'ML', $this->_primaryKey, $this->_row[$this->_primaryKey]);
			return parent::delete();
		}
	}
?>