<?php
	/**
	 * SQL Multilanguage Recordset Class
	 * 
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */

	/**
	 * Handle and manage the reading of data from database.
	 * This class only read data from a given database table with necessary
	 * ordering and filtering.
	 *
	 * @copyright Copyright (c) 2003-2006 Mirchev Ideas Ltd. All rights reserved.
	 */	
	class scSqlRecordsetML extends miSqlRecordset {
		protected $_primaryKey; 
		protected $_languageId = 0;
		
		/**
		 * scSqlRecordsetML constructor
		 * 
		 * @access public
		 * @param string|array $table database table name, or several table names
		 * @param string $primaryKey the table primary key
		 */
		public function __construct($table, $primaryKey)
		{
			parent::__construct($table);
			
			$this->_primaryKey = $primaryKey;
		}
		
		/**
		 * Sets the language id
		 * Once the language id has been set it cannot be changed
		 *
		 * @param int $languageId
		 */
		public function setLanguageId($languageId)
		{
			if ($this->_languageId or !$languageId)
				return;
			
			$this->_languageId = (int)$languageId;
			$joinCondition = 'ON ' . $this->_table . '.' . $this->_primaryKey . '=' . $this->_table . 'ML.' . $this->_primaryKey .
				' AND ' . $this->_table . 'ML.LanguageID=' . $this->_languageId;
			
			// Add the ML join in the beginning of the $_joinConditions array
			// otherwise it is mandatory to call setLanguageId() before any addJoinCondition() call to work properly
			array_unshift($this->_joinConditions, array('RIGHT', $this->_table, $joinCondition));

			$this->_table .= 'ML';
		}
		
		/**
		 * Retrieves the language id
		 *
		 * @return int
		 */
		public function getLanguageId()
		{
			return $this->_languageId;
		}
		
		/**
		 * Return the order clause
		 *
		 * @return string order clause
		 */
		protected function getSqlOrderClause()
		{
			// Handle the special case - sorting by primaty key
			// Prefix the key with the table name
			if ($this->_sortBy == $this->_primaryKey) {
				if (($this->_sortDir != 'ASC') and ($this->_sortDir != 'DESC') and ($this->_sortDir != ''))
					throw new miConfigurationException('Invalid sort direction: ' . $this->_sortDir);
				
				return ' ORDER BY ' . $this->_table . '.' . $this->_sortBy . ' ' . $this->_sortDir;
			}
			return parent::getSqlOrderClause();
		}
	}
?>