<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSqlTreeRecordset extends miSqlRecordset {
		
		const MODE_NORMAL = 0;
		const MODE_EXPANDED_NODES_ONLY = 1;
		const MODE_PREORDER = 2;
		
		/**
		 * @access private
		 */
		var $_mode;
		var $_primaryKey;
		var $_parentKey;
		var $_expandedNodesIds = array();
		var $_allowedOperations = array('like', '=', '>', '<', '>=', '<=', '!=', 'ignore');
		/**
		 * Contructor
		 * @access public
		 */
		function __construct($table, $primaryKey, $parentKey)
		{
			parent::__construct($table);
			$this->_primaryKey = $primaryKey;
			$this->_parentKey = $parentKey;
		}
		
		/**
		 * Sets the mode of post-processing of the tree
		 * $mode is one of MODE_NORMAL or MODE_EXPANDED_NODES_ONLY
		 * or MODE_PREORDER
		 * Note that MODE_EXPANDED_NODES_ONLY implies MODE_PREORDER
		 * 
		 * @access public
		 */
		function setMode($mode)
		{
			$this->_mode = $mode;
		}
		
		/**
		 * Sets the expanded Nodes IDs
		 * @access public
		 * @param $expandedIds array containing the keys to the nodes that are expanded
		 */
		function setExpandedNodes($expandedNodesIds)
		{
			$this->_expandedNodesIds = $expandedNodesIds;
		}
		
		
		/**
		 * 
		 * @return array|false returns an array of records, or false on error
		 */
		
		function getRecordsByIndex($fromIndex, $numRecords)
		{
			$numRecords = PHP_INT_MAX;
			$rows = parent::getRecordsByIndex($fromIndex, $numRecords);
			if ($rows === false)
				return false;
			
			$treeUtil = new scTreeUtil;
			if (count($this->_filters) > 0) {
				$r = array_merge($rows, $this->getParentRecords($rows));
				return $treeUtil->makeTree($r, $this->_primaryKey, $this->_parentKey);	
			}	
			
			if ($this->_mode == self::MODE_NORMAL)
				return $rows;
			
			$rows = $treeUtil->makeTree($rows, $this->_primaryKey, $this->_parentKey);
			
			if (($this->_mode & self::MODE_EXPANDED_NODES_ONLY) == false)
				return $rows;
			
			// Make sure we return only expanded rows
			return $treeUtil->getExpandedNodesOnly($rows, $this->_expandedNodesIds, $this->_parentKey);
		}
		
		
		/**
		 * Retrieves all parent records to the given records
		 */
		function getParentRecords(&$rows)
		{
			// Get the parent rows
			$parentIds = array();
			foreach ($rows as $r) {
				if ($r[$this->_parentKey])
					$parentIds[] = $r[$this->_parentKey];
			}
			$parents = $this->getAllParentsByIds($parentIds);
			
			// Find and return only unique parent rows
			$recordIds = array();
			foreach ($rows as $r)
				$recordIds[] = $r[$this->_primaryKey];
			
			$parentRecords = array();
			foreach ($parents as $parent) {
				if (!in_array($parent[$this->_primaryKey], $recordIds)) {
					$recordIds[$parent[$this->_primaryKey]] = $parent;
					$parentRecords[] = $parent;	
				}
			}
			return $parentRecords;
		}
		
		protected function getNonTreeRecordset()
		{
			return new miSqlRecordset($this->_table);
		}
		
		protected function getNonTreeRecordsetPK()
		{
			return $this->_primaryKey;
		}
		
		/**
		 * Retrieves all parent rows for the given $childIds
		 */
		function getAllParentsByIds(&$childIds)
		{
			if (count($childIds) == 0)
				return array();
			
			$t = $this->getNonTreeRecordset();
			$t->addFilter(new miSqlFilterIn($this->getNonTreeRecordsetPK(), array_unique($childIds)));

			$rows = $t->getAllRecords();
			
			// Do a recursive check
			$ids = array();
			foreach ($rows as $row) {
				if ($row[$this->_parentKey])
					$ids[] = $row[$this->_parentKey];
			}
			if (count($ids) == 0)
				return $rows;
			
			return array_merge($rows, $this->getAllParentsByIds($ids));
		}
	}
?>
