<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperCmsMenuItemImage extends miViewMapperDefault {
		
		protected $_values = array();
		
		public function __construct(miViewMapperController $controller)
		{
			parent::__construct($controller, $controller->getView()->getDomainObject(), miGetParamDefault('CMSMenuItemID', 0));
		}
		
		public function read()
		{
			// The domain object has already been read.
		}
		
		public function get($fieldName)
		{
			if (!isset($this->_values[$fieldName])) {
				$this->_values[$fieldName] = array(
					$fieldName => $this->_domainObj->get($fieldName),
					$fieldName . 'Type' => $this->_domainObj->get($fieldName . 'Type')
				);
			}
			
			return $this->_values[$fieldName];
		}
		
		public function set($fieldName, $fieldValue)
		{
			$this->_values[$fieldName] = array(
				$fieldName => $fieldValue[$fieldName],
				$fieldName . 'Type' => $fieldValue[$fieldName . 'Type'],
			);
		}
		
		public function insert()
		{
			$this->update();
		}
		
		public function update()
		{
			$this->_pkValue = $this->_domainObj->get('CMSMenuItemID');
			
			$this->uploadImages();
			$this->_domainObj->update();
		}
		
		protected function uploadImages()
		{
			$dataRow = $this->_domainObj->getRow();
			
			foreach ($this->_values as $fieldName => $data) {
				if ($data[$fieldName . 'Type'] == scMenuItemImageTypes::TYPE_CUSTOM) {
					// Upload the custom image
					$fileName = scPaths::getItemFilename($this->_pkValue, scPaths::RESOURCE_MENU_ITEM_IMAGE);
					
					$oldPath = scPaths::getFilenamePath($data[$fieldName]['imageFileName'], scPaths::RESOURCE_TEMP_IMAGE, scPaths::PATH_FILE);
					$newPath = scPaths::getFilenamePath($fileName, scPaths::RESOURCE_MENU_ITEM_IMAGE, scPaths::PATH_FILE);
					
					// Upload the image
					if (!$data[$fieldName]['delete'] && is_file($oldPath)) {
						// Get the max width and height for the image
						$maxWidth = (int) scSettings::getSettingValue(scSettings::SETTING_MENU_ITEM_IMAGE_MAX_WIDTH);
						$maxHeight = (int) scSettings::getSettingValue(scSettings::SETTING_MENU_ITEM_IMAGE_MAX_HEIGHT);
						
						// Resize the new image if needed	
						scImageHandler::resizeImage($oldPath, $newPath, $maxWidth, $maxHeight);
					}
					
					// Check if the image is valid
					if ($data[$fieldName]['delete'] || !is_file($newPath)) {
						if (is_file($newPath))
							@unlink($newPath);
						
						$dataRow[$fieldName . 'Type'] = scMenuItemImageTypes::TYPE_NO_IMAGE;	// Not valid uploaded file
						$dataRow[$fieldName] = '';
						
						continue;
					}
					
					$dataRow[$fieldName] = scPaths::getFilenamePath($fileName, scPaths::RESOURCE_MENU_ITEM_IMAGE, scPaths::PATH_WEB);
					$dataRow[$fieldName . 'Type'] = $data[$fieldName . 'Type'];
					$dataRow[$fieldName . 'X'] = intval(scImageHandler::getImageWidth($newPath));
					$dataRow[$fieldName . 'Y'] = intval(scImageHandler::getImageHeight($newPath));
				} else {
					$dataRow[$fieldName] = $data[$fieldName];
					$dataRow[$fieldName . 'Type'] = $data[$fieldName . 'Type'];
					$dataRow[$fieldName . 'X'] = intval(scImageHandler::getImageWidth($data[$fieldName]));
					$dataRow[$fieldName . 'Y'] = intval(scImageHandler::getImageHeight($data[$fieldName]));
				}
				
				$this->_values[$fieldName] = $dataRow[$fieldName];
				$this->_values[$fieldName . 'Type'] = $dataRow[$fieldName . 'Type'];
			}
			
			$this->_domainObj->setRow($dataRow);
		}
	}
?>