<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	abstract class scViewMapperCustomUrls extends miViewMapper {
		
		/**
		 * @var array
		 */
		protected $_values = array();
		
		/**
		 * @var scProductDomainObject
		 */
		protected $_domainObject = null;
		
		// This must be set
		protected $_entityCode = '';
		protected $_tableKeyName = '';
		
		public function __construct(miViewMapperControllerDefault $viewMapperController)
		{
			parent::__construct($viewMapperController);
			$this->_domainObject = $viewMapperController->getDomainObject();
		}
		
		public function read()
		{
			$recordset = new miSqlRecordset('CustomUrls');
			$recordset->addFilter(new miSqlFilterEqual('CustomUrlEntityCode', $this->_entityCode));
			$recordset->addFilter(new miSqlFilterEqual('CustomUrlEntityID', $this->getEntityId()));
			$customUrls = $recordset->getAllRecords();
			
			foreach ($customUrls as &$customUrl) {
				$recordset = new miSqlRecordset('CustomUrlsML');
				$recordset->addFilter(new miSqlFilterEqual('CustomUrlID', $customUrl['CustomUrlID']));
				$urlTexts = $recordset->getAllRecords();
				$customUrl['CustomUrlText'] = miUtil::getKeyValueArray($urlTexts, 'LanguageID', 'CustomUrlText');
			}
			
			$this->setValues(array_values($customUrls));
		}
		
		public function get($fieldName)
		{
			return $this->_values;
		}
		
		public function set($fieldName, $value)
		{
			$this->setValues($value);
		}
		
		public function insert()
		{
			return $this->update();
		}
		
		public function update()
		{
			miStaticDBUtil::startTransaction();
			
			$entityId = $this->getEntityId();
			miStaticDBUtil::execDelete('CustomUrls', array('CustomUrlEntityCode', 'CustomUrlEntityID'), array($this->_entityCode, $entityId));
			
			$useCustomUrls = miGetParamDefault('UseCustomUrls', 0);
			if (!$useCustomUrls || empty($this->_values)) {
				miStaticDBUtil::commit();
				return $entityId;
			}
			
			foreach ($this->_values as $customUrlData) {
				$customUrl = new scCustomUrlDomainObject();
				$customUrl->set('CustomUrlEntityCode', $this->_entityCode);
				$customUrl->set('CustomUrlEntityID', $entityId);
				$customUrl->set('CustomUrlIsCurrent', (int)$customUrlData['CustomUrlIsCurrent']);
				$customUrl->set('CustomUrlText', $customUrlData['CustomUrlText']);
				$customUrl->insert();
			}
			
			miStaticDBUtil::commit();
			
			return $entityId;
		}
		
		protected function setValues($values)
		{
			$this->_values = $values;
		}
		
		protected function getEntityId()
		{
			return (int)$this->_domainObject->get($this->_tableKeyName);
		}
	}
?>