<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperDynRows extends miViewMapper {
		protected $_rsValues;
		
		/**
		 * The properties for each repeating section
		 * The key is the repeating section name
		 */
		protected $_rsProperties = array();
		
		public function __construct(miViewMapperController $model)
		{
			parent::__construct($model);
			
			$this->initRsProperties();
		}
		
		protected function initRsProperties()
		{
			// Setup $this->_rsProperties
			$dataFields = $this->_viewMapperController->getView()->getDataFields();
			
			$className = get_class($this);
			foreach ($dataFields as $dataField) {
				if (!isset($dataField['modelStrategy']) or ($dataField['modelStrategy'] != $className))
					continue;
				$this->_rsProperties[$dataField['data']] = $dataField['properties'];
			}
		}
		
		/**
		 * Return the repeating section values
		 */
		public function get($fieldName)
		{
			return $this->_rsValues[$fieldName];
		}
		
		public function set($fieldName, $fieldValue)
		{
			$this->_rsValues[$fieldName] = $fieldValue;
		}
		
		/**
		 * Read the module options
		 */
		public function read()
		{
			foreach ($this->_rsProperties as $fieldName => $rsProperties) {
				$this->_rsValues[$fieldName] = $this->readRecords($rsProperties);
			}
		}
		
		/**
		 * Reads the multilanguage fields data into records
		 * 
		 * @param array $records the records will be updated
		 */
		protected function readMLFieldData(&$records, $rsProperties)
		{
			if (empty($records))
				return;
			
			$foreignTablePrimaryKey = $rsProperties['foreignTablePrimaryKey'];

			// Find all foreign key ids
			$ids = array();
			foreach ($records as $record) {
				$ids[] = (int)$record[$rsProperties['foreignTablePrimaryKey']];
			}
			
			$query = 'SELECT * FROM ' . $rsProperties['table'] . 'ML WHERE ' .
				$rsProperties['foreignTablePrimaryKey'] . ' IN(' . implode(',', $ids) . ')';
			$rows = miStaticDBUtil::execSelect($query);
			
			// Index the records for easier search
			$keyedRows = array();
			foreach ($rows as $row) {
				$keyedRows[$row[$foreignTablePrimaryKey]][$row[scSqlRecordML::LANGUAGE_ID]] = $row;
			}
			
			// Put the data into each of the records
			foreach ($records as &$record) {
				foreach ($rsProperties['mlFields'] as $mlField) {
					$record[$mlField] = array();
				}
				
				if (!isset($keyedRows[$record[$foreignTablePrimaryKey]]))
					continue;
				
				foreach ($keyedRows[$record[$foreignTablePrimaryKey]] as $languageId => $row) {
					foreach ($rsProperties['mlFields'] as $mlField) {
						$record[$mlField][$languageId] = $row[$mlField];
					}
				}
			}
		}
		
		/**
		 * Reads the records
		 *
		 * @param int $foreignKeyValue
		 * @return array the records
		 */
		protected function readRecords($rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			
			// If there are multi-language fields
			if (!empty($rsProperties['mlFields'])) {
				$recordset = new miSqlRecordset($rsProperties['table']); 
				$recordset->addFilter(new miSqlFilterEqual($rsProperties['foreignKey'], intval($foreignKeyValue)));
				if (isset($rsProperties['sortBy']))
					$recordset->setOrder($rsProperties['sortBy']);
				$result = $recordset->getAllRecords();
				$this->readMLFieldData($result, $rsProperties);
				return $result;
			}
			
			$selectQuery = 'SELECT * FROM '.$rsProperties['table'].' WHERE '.$rsProperties['foreignKey'].' = '. intval($foreignKeyValue) . 
				(isset($rsProperties['sortBy']) ? ' ORDER BY ' . $rsProperties['sortBy'] : '');
			return miStaticDBUtil::execSelect($selectQuery);
		}
		
		public function insert()
		{
			$this->update();
		}
		
		public function update()
		{
			foreach ($this->_rsProperties as $fieldName => $rsProperties) {
				$this->saveDynRows($this->_rsValues[$fieldName], $rsProperties);
			}
		}
		
		/**
		 * Return the value for the foreign key
		 * Expects that the foreign key is a regular data field
		 * 
		 * @param string $foreignKey
		 * @return string|int the foreign key value
		 */
		protected function getForeignKeyValue($foreignKey)
		{
			return $this->getViewMapperController()->getDomainObject()->get($foreignKey);
		}
		
		protected function saveDynRows($rsValues, $rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			
			$recordIds = array();
			foreach ($rsValues as $rowIndex => $values) {
				$record = $this->createRecord($rsProperties);
				$record->set($rsProperties['foreignKey'], $foreignKeyValue);
				
				$action = $this->setValuesToRecord($record, $values, $rsProperties, $rowIndex);
				
				if ($action == 'UPDATE') {
					$recordIds[] = $this->updateRecord($record, $rsValues, $rsProperties, $rowIndex);
				} else {
					$recordIds[] = $this->insertRecord($record, $rsValues, $rsProperties, $rowIndex);
				}
			}
			
			$this->deleteOldRecords($recordIds, $foreignKeyValue, $rsProperties);
		}
		
		protected function deleteOldRecords($recordIds, $foreignKeyValue, $rsProperties)
		{
			$deleteQuery = 'DELETE FROM ' . $rsProperties['table'] .
				' WHERE ' . $rsProperties['foreignKey'] . ' = "' . sql_escape_string($foreignKeyValue) . '"';
			if (!empty($recordIds)) {
				$recordIds = array_map('intval', $recordIds);
				$deleteQuery .= ' AND ' . $rsProperties['foreignTablePrimaryKey'] . ' NOT IN (' . implode(', ', $recordIds) . ')';
			}
			miStaticDBUtil::execSQL($deleteQuery);
		}
		
		/**
		 * @param array $rsProperties
		 * @return miSqlRecord
		 */
		protected function createRecord($rsProperties)
		{
			$mlFields = array();
			if (isset($rsProperties['mlFields'])) {
				$mlFields = $rsProperties['mlFields'];
			}
			
			if (!empty($mlFields)) {
				$record = new scSqlRecordML($rsProperties['table'], $rsProperties['foreignTablePrimaryKey'],
					$mlFields, array_keys(scMultiLanguage::getActiveLanguageCodes()));
			} else {
				$record = new miSqlRecord($rsProperties['table'], $rsProperties['foreignTablePrimaryKey']);
			}
			
			return $record;
		}
		
		/**
		 * @param miSqlRecord $record
		 * @param array $values
		 * @param array $rsProperties
		 * @return string
		 */
		protected function setValuesToRecord($record, $values, $rsProperties, $rowIndex)
		{
			$action = 'INSERT';
			
			foreach ($rsProperties['fields'] as $field) {
				$name = $field['data'];
				if ($name == $record->getPrimaryKeyColumn()) {
					if (empty($values[$name])) {
						$values[$name] = null;
					} else {
						// The PK has been found and it has a non-empty value.
						// The record should be updated.
						$action = 'UPDATE';
					}
				}
				if ((isset($field['properties']['readOnly']) && $field['properties']['readOnly'])
					|| (isset($rsProperties['sortField']) && isset($field['name']) && ($rsProperties['sortField'] == $field['name'])))
					continue;
				
				$record->set($name, $values[$name]);
			}
			
			if (isset($rsProperties['sortField'])) {
				$sort = $rowIndex + 1;
				$this->freeSortPosition($rsProperties['table'], $rsProperties['sortField'], $sort, $rsProperties['foreignKey'], $this->getForeignKeyValue($rsProperties['foreignKey']));
				$record->set($rsProperties['sortField'], $sort);
			}
			
			return $action;
		}
		
		/**
		 * Insert the record
		 */
		protected function insertRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			return $record->insert();
		}
		
		/**
		 * Update the record
		 */
		protected function updateRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			return $record->update();
		}
		
		public function delete()
		{
			foreach ($this->_rsProperties as $fieldName => $rsProperties) {
				$this->deleteDynRows($rsProperties);
			}
		}
		
		protected function deleteDynRows($rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			miStaticDBUtil::execDelete($rsProperties['table'], $rsProperties['foreignKey'], $foreignKeyValue);			
		}
		
		/**
		 * Frees the sort position by setting the sort to max+1 of the used sort
		 * @param string $table
		 * @param string $sortField
		 * @param int $sortPosition
		 * @param string $foreignKey
		 * @param mixed $foreignKeyValue
		 */
		protected function freeSortPosition($table, $sortField, $sortPosition, $foreignKey, $foreignKeyValue)
		{
			$query = 'UPDATE ' . $table . ' SET ' . $sortField . ' = ' . $sortField . ' + 1' .
				' WHERE ' . $foreignKey . ' = "' . sql_escape_string($foreignKeyValue) . '" AND ' . $sortField . ' >= ' . intval($sortPosition) .
				' ORDER BY ' . $sortField . ' DESC';
			miStaticDBUtil::execSQL($query);
		}
	}
?>