<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	abstract class scViewMapperFields extends miViewMapper {
		
		/**
		 * @var miDefaultDomainObject
		 */
		protected $_domainObject;
		
		/**
		 * @var array
		 */
		protected $_values;
		
		/**
		 * @var IFieldsDataProvider
		 */
		protected $_dataProvider = null;
		
		/**
		 * @return IFieldsDataProvider
		 */
		abstract protected function createDataProvider();
		
		/**
		 * @return miDefaultDomainObject
		 */
		abstract protected function createFieldValueDomainObject();
		
		public function __construct(miViewMapperControllerDefault $viewMapperController)
		{
			parent::__construct($viewMapperController);
			$this->_domainObject = $viewMapperController->getDomainObject();
		}
		
		public function read()
		{
			$this->_values = $this->getValues((int)$this->_domainObject->get($this->_domainObject->getPKName()));
		}
		
		public function insert()
		{
			$this->update();
		}
		
		public function update()
		{
			$namesProvider = $this->getDataProvider();
			
			$customerId = (int)$this->_domainObject->get($this->_domainObject->getPKName());
			$recordset = new miSqlRecordset($namesProvider->getFieldsTable());
			$recordset->addFilter(new miSqlFilterIn($namesProvider->getFieldIdColumn(), array_keys($this->_values)));
			$fields = $recordset->getAllRecords();
			$fields = miUtil::getArrayWithKeys($fields, $namesProvider->getFieldIdColumn());
			
			$currentValues = $this->getValues($customerId);
			$currentValues = miUtil::getArrayWithKeys($currentValues, $namesProvider->getFieldIdColumn());
			
			$ignoreFields = $namesProvider->getFieldsToIgnore();
			
			$valueIds = array();
			foreach ($this->_values as $fieldId => $value) {
				if (in_array($fieldId, $ignoreFields)) {
					continue;
				}
				
				$fieldType = $fields[$fieldId][$namesProvider->getFieldTypeColumn()];
				
				$valueField = $namesProvider->getValueFieldForType($fieldType);
				if (empty($value[$valueField]) && in_array($fieldType, $namesProvider->getFieldTypesWithValues())) {
					$value[$valueField] = null;
				}
				
				$valueObject = $this->createFieldValueDomainObject();
				$valueObject->set($this->_domainObject->getPKName(), $customerId);
				$valueObject->set($namesProvider->getFieldIdColumn(), $fieldId);
				$valueObject->set($valueField, $value[$valueField]);
				if (isset($currentValues[$fieldId])) {
					$valueObject->set($valueObject->getPKName(), $currentValues[$fieldId][$namesProvider->getValueIdColumn()]);
					$valueIds[] = $valueObject->update();
				} else {
					$valueIds[] = $valueObject->insert();
				}
			}
			
			$deleteQuery = 'DELETE FROM ' . $namesProvider->getValuesTable() . ' WHERE ' . $this->_domainObject->getPKName() . ' = ' . $customerId;
			if (!empty($valueIds)) {
				$deleteQuery .= ' AND ' . $namesProvider->getValueIdColumn() . ' NOT IN (' . implode(', ', $valueIds) . ')';
			}
			miStaticDBUtil::execSQL($deleteQuery);
		}
		
		public function get($field)
		{
			return $this->_values;
		}
		
		public function set($field, $values)
		{
			$this->_values = $values;
		}
		
		protected function getValues($pkValue)
		{
			$recordset = new miSqlRecordset($this->getDataProvider()->getValuesTable());
			$recordset->addFilter(new miSqlFilterEqual($this->_domainObject->getPKName(), $pkValue));
			$result = $recordset->getAllRecords();
			$values = miUtil::getArrayWithKeys($result, $this->getDataProvider()->getValueIdColumn());
			return $values;
		}
		
		protected function getDataProvider()
		{
			if (is_null($this->_dataProvider)) {
				$this->_dataProvider = $this->createDataProvider();
			}
			return $this->_dataProvider;
		}
	}
?>