<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperGalleryItemImage extends miViewMapperDefault {
		
		protected $_values = array();
		
		public function __construct(miViewMapperController $controller)
		{
			parent::__construct($controller, $controller->getView()->getDomainObject(), miGetParamDefault('GalleryItemID', 0));
		}
		
		public function set($fieldName, $fieldValue)
		{
			$this->getViewMapperController()->getViewMapper()->set($fieldName, $fieldValue['imageWebPath']);
			$this->getViewMapperController()->getViewMapper()->set($fieldName . 'Thumb', $fieldValue['thumbWebPath']);
			
			$this->_values[$fieldName] = $fieldValue;
		}
		
		public function insert()
		{
			$this->update();
		}
		
		public function update()
		{
			$this->_pkValue = $this->_domainObj->get('GalleryItemID');
			$this->uploadImages();
			$this->_domainObj->update();
		}
		
		public function delete()
		{
			// Do nothing (Prevent from executing the domain object delete)
		}
		
		public function get($fieldName)
		{
			$imageWebPath = $this->getViewMapperController()->getViewMapper()->get($fieldName);
			$thumbWebPath = $this->getViewMapperController()->getViewMapper()->get($fieldName . 'Thumb');
			
			$this->_values[$fieldName] = array(
				'imageFileName' => '',
				'imageWebPath' => $imageWebPath,
				'thumbFileName' => '',
				'thumbWebPath' => $thumbWebPath,
				'delete' => false,
			);
			
			return $this->_values[$fieldName];
		}
		
		protected function uploadImages()
		{
			$galleryId = (int)$this->_domainObj->get('GalleryID');
			if ($galleryId <= 0) {
				throw new miException(_i('Unknown gallery.'));
			}
			$galleryDirName = scUtil::getDbValue('Galleries', 'GalleryDirName', 'GalleryID', $galleryId, $galleryId);
			
			foreach ($this->_values as $fieldName => $fieldValue) {
				if (isset($fieldValue['imageFileName']) && !empty($fieldValue['imageFileName'])) {
					// Process images
					$maxWidth = (int) scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_MAX_WIDTH);
					$maxHeight = (int) scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_MAX_HEIGHT);
					$imagesDir = $galleryDirName . '/' . scPaths::GALLERY_LARGE_IMAGES;
					list($file, $width, $height) = $this->resizeUploadedImage($imagesDir, $fieldValue['imageFileName'], scPaths::RESOURCE_TEMP_IMAGE, scPaths::RESOURCE_GALLERY_IMAGE, $maxWidth, $maxHeight);
					$this->_domainObj->set($fieldName, $file);
					$this->_domainObj->set($fieldName . 'Width', $width);
					$this->_domainObj->set($fieldName . 'Height', $height);
				}
				
				if (isset($fieldValue['thumbFileName']) && !empty($fieldValue['thumbFileName'])) {
					// Process thumb
					$maxWidth = (int) scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_THUMB_MAX_WIDTH);
					$maxHeight = (int) scSettings::getSettingValue(scSettings::SETTING_GALLERY_ITEM_IMAGE_THUMB_MAX_HEIGHT);
					$imagesDir = $galleryDirName . '/' . scPaths::GALLERY_THUMB_IMAGES;
					list($file, $width, $height) = $this->resizeUploadedImage($imagesDir, $fieldValue['thumbFileName'], scPaths::RESOURCE_TEMP_THUMB, scPaths::RESOURCE_GALLERY_IMAGE_THUMB, $maxWidth, $maxHeight);
					$this->_domainObj->set($fieldName . 'Thumb', $file);
					$this->_domainObj->set($fieldName . 'ThumbWidth', $width);
					$this->_domainObj->set($fieldName . 'ThumbHeight', $height);
				}
			}
		}
		
		/**
		 * @return array(path, width, height)
		 */
		protected function resizeUploadedImage($imagesDir, $fileName, $tempImageType, $imageType, $maxWidth, $maxHeight)
		{
			$tempImage = scPaths::getFilenamePath($fileName, $tempImageType, scPaths::PATH_FILE);
			
			$imageBaseName = $imagesDir . scPaths::getItemFilename($this->_pkValue, $imageType);
			$imagePath = scPaths::getFilenamePath($imageBaseName, $imageType, scPaths::PATH_FILE);
			
			// Copy and resize (if needed) the image
			scImageHandler::resizeImage($tempImage, $imagePath, $maxWidth, $maxHeight);
			
			// Get the new width and height
			list($resultWidth, $resultHeight) = scImageHandler::getImageSize($imagePath);
			
			return array(
				scPaths::getFilenamePath($imageBaseName, $imageType, scPaths::PATH_WEB),
				$resultWidth,
				$resultHeight
			);
		}
	}
?>