<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	abstract class scViewMapperItemsWithValues extends scViewMapperTwoLevelDynRows {
		
		abstract protected function getTypesWithValues();
		
		protected function readValues($rsProperties, &$records)
		{
			$foreignTable = $rsProperties['table'];
			$foreignPK = $rsProperties['foreignTablePrimaryKey'];
			
			$valueTable = $rsProperties['valueTable'];
			$valueKey = $rsProperties['valueKey'];
			$ids = array();
			
			foreach ($records as $record)
				$ids[] = '"' . sql_escape_string($record[$foreignPK]) . '"';
			
			$query = 'SELECT * ' .
				' FROM ' . $foreignTable .
				' INNER JOIN ' . $valueTable . ' USING(' . $foreignPK . ')' .
				' LEFT JOIN ' . $valueTable . 'ML USING(' . $valueKey . ')' .
				' WHERE ' . $foreignTable . '.' . $foreignPK . ' IN(' . implode(',', $ids) . ')' .
				(isset($rsProperties['valueSortBy']) ? ' ORDER BY ' . $rsProperties['valueSortBy'] : '');
			return miStaticDBUtil::execSelect($query);
		}
		
		protected function deleteOldValues($rsProperties)
		{
			return; // Do not delete values. They will be deleted in updateRecordValues()
		}
		
		protected function addValuesToRecords($rsProperties, $records, $values)
		{
			$valueTable = $rsProperties['valueTable'];
			$valueKey = $rsProperties['valueKey'];
			
			foreach ($records as &$record) {
				$record[$valueTable] = array();
			}
			
			foreach ($values as $row) {
				$key = $this->findRecord($rsProperties, $records, $row[$rsProperties['foreignTablePrimaryKey']]);
				if ($key === null) {
					continue;
				}
				
				$this->addRecordValue($rsProperties, $records[$key], $row);
			}
			return $records;
		}
		
		protected function addRecordValue($rsProperties, &$record, $valueData)
		{
			$valueTable = $rsProperties['valueTable'];
			$valueKey = $rsProperties['valueKey'];
			$valueField = $rsProperties['valueField'];
			
			$valueId = $valueData[$valueKey];
			if (isset($record[$valueTable][$valueId])) {
				$record[$valueTable][$valueId][$valueData['LanguageID']] = $valueData[$valueField];
			} else {
				$record[$valueTable][$valueId] = array($valueData['LanguageID'] => $valueData[$valueField]);
			}
		}
		
		protected function findRecord($rsProperties, $records, $id)
		{
			foreach ($records as $key => $record)
				if ($record[$rsProperties['foreignTablePrimaryKey']] == $id)
					return $key;
			return null;
		}
		
		protected function insertRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			$itemId = parent::insertRecord($record, $rsValues, $rsProperties, $rowIndex);
			
			$this->updateRecordValues($record, $rsValues, $rsProperties, $rowIndex);
			
			return $itemId;
		}
		
		protected function updateRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			$itemId = parent::updateRecord($record, $rsValues, $rsProperties, $rowIndex);
			
			$this->updateRecordValues($record, $rsValues, $rsProperties, $rowIndex);
			
			return $itemId;
		}
		
		protected function updateRecordValues($record, $rsValues, $rsProperties, $rowIndex)
		{
			$itemId = $record->getPK();
			// Save form item values
			$itemValueIds = array();
			foreach ($rsValues[$rowIndex][$rsProperties['valueTable']] as $key => $itemValues) {
				if (trim(implode($itemValues)) == '')
					continue;
				$valueId = '';
				if (isset($rsValues[$rowIndex][$rsProperties['valueKey']][$key])) {
					$valueId = $rsValues[$rowIndex][$rsProperties['valueKey']][$key];
				}
				if (!empty($valueId)) {
					$itemValueIds[] = $this->updateItemValues($rsProperties, $itemId, $itemValues, $key, $valueId);
				} else {
					$itemValueIds[] = $this->insertItemValues($rsProperties, $itemId, $itemValues, $key);
				}
			}
			
			$deleteQuery = 'DELETE FROM ' . $rsProperties['valueTable'] . ' WHERE ' . $rsProperties['foreignTablePrimaryKey'] . ' = ' . sql_escape_string($itemId);
			if (!empty($itemValueIds)) {
				$itemValueIds = array_map('intval', $itemValueIds);
				$deleteQuery .= ' AND ' . $rsProperties['valueKey'] . ' NOT IN (' . implode(', ', $itemValueIds) . ')';
			}
			miStaticDBUtil::execSQL($deleteQuery);
		}
		
		protected function updateItemValues($rsProperties, $itemId, $itemValues, $itemValueKey, $valueId)
		{
			$valueId = intval($valueId);
			
			$record = $this->getItemValuesRecord($rsProperties, $itemId, $itemValues, $itemValueKey, $valueId);
			return $record->update();
		}
		
		protected function insertItemValues($rsProperties, $itemId, $itemValues, $itemValueKey)
		{
			$record = $this->getItemValuesRecord($rsProperties, $itemId, $itemValues, $itemValueKey);
			return $record->insert();
		}
		
		/**
		 * @return scSqlRecordML
		 */
		protected function getItemValuesRecord($rsProperties, $itemId, $itemValues, $itemValueKey, $valueId = 0)
		{
			$record = new scSqlRecordML($rsProperties['valueTable'], $rsProperties['valueKey'], array($rsProperties['valueField']), array_keys($itemValues));
			if ($valueId > 0) {
				$record->readPK($valueId);
			}
			$record->set($rsProperties['foreignTablePrimaryKey'], $itemId);
			$record->set($rsProperties['valueField'], $itemValues);
			
			if (isset($rsProperties['valueSortField'])) {
				$sort = $itemValueKey + 1;
				$this->freeSortPosition($rsProperties['valueTable'], $rsProperties['valueSortField'], $sort, $rsProperties['foreignTablePrimaryKey'], $itemId);
				$record->set($rsProperties['valueSortField'], $sort);
			}
			
			return $record;
		}
	}
?>