<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperLocalOptions extends scViewMapperItemsWithValues implements IOptionGroupsProvider {
		
		protected $_optionGroups = array();
		
		protected $_newOptionGroupIds = array();
		
		public function insert()
		{
			foreach ($this->_rsValues['LocalOptionGroups'] as &$item) {
				$itemIdField = $this->_rsProperties['LocalOptionGroups']['foreignTablePrimaryKey'];
				if (isset($item[$itemIdField])) {
					$item[$itemIdField] = null;
				}
				
				$itemValueField = $this->_rsProperties['LocalOptionGroups']['valueTable'];
				foreach ($item[$itemValueField] as &$itemValue) {
					$itemValueIdField = $this->_rsProperties['LocalOptionGroups']['valueKey'];
					if (isset($itemValue[$itemValueIdField])) {
						$itemValue[$itemValueIdField] = null;
					}
				}
			}
			$this->update();
		}
		
		public function setRsProperties($fieldName, $rsProperties)
		{
			$this->_rsProperties[$fieldName] = $rsProperties;
		}
		
		public function getOptionGroups()
		{
			return $this->_optionGroups;
		}
		
		public function getNewOptionGroupIds()
		{
			return $this->_newOptionGroupIds;
		}
		
		protected function insertRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			// Skip scViewMapperItemsWithValues and call scViewMapperTwoLevelDynRows
			// to make sure the PK is known before inserting subitems.
			$optionGroupId = scViewMapperTwoLevelDynRows::insertRecord($record, $rsValues, $rsProperties, $rowIndex);
			// Add to the array that is going to be given to the variants view mapper.
			$this->addOptionGroupFromRecord($record);
			// Mark the option group "new".
			$this->_newOptionGroupIds[] = $optionGroupId;
			// Copy the behaviour of scViewMapperItemsWithValues
			$this->updateRecordValues($record, $rsValues, $rsProperties, $rowIndex);
			
			return $optionGroupId;
		}
		
		protected function updateRecord($record, $rsValues, $rsProperties, $rowIndex)
		{
			// This is update. Record already has its PK.
			$copy = clone $record;
			$copy->readPK($record->getPK());
			
			$this->addOptionGroupFromRecord($record);
			
			$itemId = parent::updateRecord($record, $rsValues, $rsProperties, $rowIndex);
			
			if ($record->get('LocalOptionGroupType') == scLocalOptions::GROUP_TYPE_VARIANT) {
				if ($copy->get('LocalOptionGroupType') != scLocalOptions::GROUP_TYPE_VARIANT) {
					// The option's type has just been changed to "variant". Mark it new.
					$this->_newOptionGroupIds[] = $itemId;
				}
			} else if ($copy->get('LocalOptionGroupType') == scLocalOptions::GROUP_TYPE_VARIANT) {
				// The option has been of type "variant" and is now something else.
				// Delete all data in the variants as if the option was deleted.
				// From Variant's point of view that's exactly what has happened to the option.
				$sql = 'DELETE FROM ProductVariantLocalOptions' .
					' WHERE LocalOptionID IN (SELECT LocalOptionID FROM LocalOptions WHERE LocalOptionGroupID = ?)';
				miStaticDBUtil::execSQL($sql, array($itemId));
			}
			
			return $itemId;
		}
		
		/**
		 * Add to the array that is going to be given to the variants view mapper.
		 */
		protected function addOptionGroupFromRecord($record)
		{
			$optionGroup = new scLocalOptionGroupDomainObject();
			$optionGroup->setRow($record->getRow());
			$this->_optionGroups[$optionGroup->get('LocalOptionGroupID')] = $optionGroup;
		}
		
		protected function getTypesWithValues()
		{
		}
		
		protected function addRecordValue($rsProperties, &$record, $valueData)
		{
			$valueId = $valueData['LocalOptionID'];
			if (!isset($record['LocalOptions'][$valueId])) {
				$record['LocalOptions'][$valueId] = $valueData;
			}
			
			foreach ($valueData as $inRowKey => $inRowValue) {
				if (in_array($inRowKey, array('LocalOptionName'))) {
					if (!is_array($record['LocalOptions'][$valueId][$inRowKey])) {
						$record['LocalOptions'][$valueId][$inRowKey] = array();
					}
					$record['LocalOptions'][$valueId][$inRowKey][$valueData['LanguageID']] = $inRowValue;
					unset($record['LocalOptions'][$valueId]['LanguageID']);
				}
			}
		}
		
		protected function updateRecordValues($record, $rsValues, $rsProperties, $rowIndex)
		{
			$itemId = $record->getPK();
			
			// Update option values: set sort = sort + 999999 so that the sort unique keys is satisified
			miStaticDBUtil::execSQL('UPDATE LocalOptions SET LocalOptionSort = LocalOptionSort + 999999' .
				' WHERE LocalOptionGroupID = ?' .
				' ORDER BY LocalOptionSort DESC', array($itemId));
			
			$itemValueIds = $insertedValueIds = array();
			foreach ($rsValues[$rowIndex][$rsProperties['valueTable']] as $key => $itemValue) {
				$valueId = '';
				if (isset($itemValue[$rsProperties['valueKey']])) {
					$valueId = $itemValue[$rsProperties['valueKey']];
				}
				if (!empty($valueId)) {
					$itemValueIds[] = $this->updateItemValue($rsProperties, $itemId, $itemValue, $valueId, $key + 1);
				} else {
					$valueId = $this->insertItemValue($rsProperties, $itemId, $itemValue, $key + 1);
					$insertedValueIds[] = $valueId;
					$itemValueIds[] = $valueId;
				}
			}
			
			// The newly inserted options should be marked "new" for the variants generator to be let know.
			if (!empty($insertedValueIds)) {
				// If the number of processed values ($itemValueIds) is the same as the number of inserted values ($insertedValueIds)
				// then all the processed items have been inserted. That means the group should be considered "new" by the generator.
				$groupIsNew = (bool)(count($itemValueIds) == count($insertedValueIds));
				// In insertRecord() and updateRecord() we've made sure the option group is set in the _optionGroups
				// array before starting to process the subitems.
				$optionGroup = $this->_optionGroups[$itemId];
				// $optionGroup is an ISet therefore it has getValues() method.
				$allValues = $optionGroup->getValues();
				foreach ($allValues as $value) {
					// Each $value is an array containing one scProductVariantOption.
					$value[0]->setIsNewGroup($groupIsNew);
					if (in_array($value[0]->getOptionId(), $insertedValueIds)) {
						$value[0]->setIsNew(true);
					}
				}
			}
			
			$deleteQuery = 'DELETE FROM ' . $rsProperties['valueTable'] . ' WHERE ' . $rsProperties['foreignTablePrimaryKey'] . ' = ' . sql_escape_string($itemId);
			if (!empty($itemValueIds)) {
				$itemValueIds = array_map('intval', $itemValueIds);
				$deleteQuery .= ' AND ' . $rsProperties['valueKey'] . ' NOT IN (' . implode(', ', $itemValueIds) . ')';
			}
			miStaticDBUtil::execSQL($deleteQuery);
		}
		
		protected function insertItemValue($rsProperties, $itemId, $itemValue, $sort)
		{
			if (empty($itemValue))
				continue;
			
			$this->fixItemValueForDb($itemValue);
			
			$itemValueId = miStaticDBUtil::execInsert('LocalOptions', array(
				'LocalOptionGroupID' => $itemId,
				'LocalOptionPriceModValue' => $itemValue['LocalOptionPriceModValue'],
				'LocalOptionPriceModType' => $itemValue['LocalOptionPriceModType'],
				'LocalOptionRequired' => $itemValue['LocalOptionRequired'],
				'LocalOptionSort' => $sort,
			));
			
			foreach ($itemValue['LocalOptionName'] as $languageId => $name) {
				$array = array (
					'LocalOptionID' => $itemValueId,
					'LanguageID' => $languageId,
					'LocalOptionName' => $name,
				);
				miStaticDBUtil::execInsert('LocalOptionsML', $array);
			}
			
			return $itemValueId;
		}
		
		protected function updateItemValue($rsProperties, $itemId, $itemValue, $itemValueId, $sort)
		{
			if (empty($itemValue))
				continue;
			
			$this->fixItemValueForDb($itemValue);
			
			$itemValueId = (int)$itemValueId;
			$values = array(
				'LocalOptionGroupID' => $itemId,
				'LocalOptionPriceModValue' => $itemValue['LocalOptionPriceModValue'],
				'LocalOptionPriceModType' => $itemValue['LocalOptionPriceModType'],
				'LocalOptionRequired' => $itemValue['LocalOptionRequired'],
				'LocalOptionSort' => $sort,
			);
			
			miStaticDBUtil::execUpdate('LocalOptions', $values, 'LocalOptionID', $itemValueId);
			
			foreach ($itemValue['LocalOptionName'] as $languageId => $name) {
				$sql = 'INSERT INTO LocalOptionsML (LocalOptionID, LanguageID, LocalOptionName)' .
					' VALUES (' . $itemValueId . ', ' . $languageId . ', "' . sql_escape_string($name) . '")' .
					' ON DUPLICATE KEY UPDATE LocalOptionName = "' . sql_escape_string($name) . '"';
				miStaticDBUtil::execSQL($sql);
			}
			
			return $itemValueId;
		}
		
		protected function fixItemValueForDb(&$itemValue)
		{
			if (!is_numeric($itemValue['LocalOptionPriceModType'])) {
				if ($itemValue['LocalOptionPriceModType'] != '') {
					throw new miException(sprintf(_i('Invalid price mod type "%s".'), $itemValue['LocalOptionPriceModType']));
				}
				$itemValue['LocalOptionPriceModType'] = scLocalOptions::PRODUCT_OPTION_TYPE_NONE;
			}
			
			$itemValue['LocalOptionPriceModValue'] = (float)$itemValue['LocalOptionPriceModValue'];
			$itemValue['LocalOptionRequired'] = (bool)$itemValue['LocalOptionRequired'];
		}
	}
?>