<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperModuleOptions extends miViewMapper {
		protected $_module;
		protected $_optionValues;
		
		/**
		 * Set the module
		 */
		public function setModule($module)
		{
			$this->_module = $module;
		}
		
		/**
		 * Get the module
		 */
		public function getModule()
		{
			return $this->_module;
		}
		
		/**
		 * Return the option values
		 * Only one field with option values is supported
		 */
		public function get($fieldName)
		{
			return $this->_optionValues;
		}
		
		public function set($fieldName, $fieldValue)
		{
			$this->_optionValues = $fieldValue;
		}
		
		public function getModuleOption($optionName)
		{
			return $this->_optionValues[$optionName];
		}
		
		public function setModuleOption($optionName, $optionValue)
		{
			$this->_optionValues[$optionName] = $optionValue;
		}
		
		/**
		 * Return the module id
		 * 
		 * @return int module id
		 */
		protected function getModuleId()
		{
			return $this->getViewMapperController()->getDomainObject()->get('ModuleID');
		}
		
		/**
		 * Read the module options
		 */
		public function read()
		{
			$this->_optionValues = scModuleOptions::getModuleOptions($this->getModuleId(), array(), scModuleOptions::MODULE_OPTIONS_INCLUDE_FULL_ML);
			$this->callModuleModelStrategies('read');
		}
		
		public function insert()
		{
			$this->updateModuleOptions();
			$this->callModuleModelStrategies('insert');
			$this->afterUpdate();
		}
		
		public function update()
		{
			$this->updateModuleOptions();
			$this->callModuleModelStrategies('update');
			$this->afterUpdate();
		}
		
		protected function updateModuleOptions()
		{
			// Get the option values which are not handled by strategies
			$strategies = $this->getModuleModelStrategies();
			$optionValues = array();
			foreach ($this->_optionValues as $optionName => $optionValue) {
				if (isset($strategies[$optionName]))
					continue;
				$optionValues[$optionName] = $optionValue;
			}
			
			$this->updateModuleOptionValues($this->getModuleId(), $optionValues);
		}
		
		/**
		 * Saves the module option values
		 * 
		 * @param int $moduleId
		 * @param array $moduleOptions
		 */
		protected function updateModuleOptionValues($moduleId, $moduleOptions)
		{
			// Start transaction
			miStaticDBUtil::startTransaction();
			
			// Delete the old options
			miStaticDBUtil::execDelete('ModuleOptions', 'ModuleID', $moduleId);
			miStaticDBUtil::execDelete('ModuleOptionsML', 'ModuleID', $moduleId);
			
			// Insert the option values
			foreach ($moduleOptions as $moduleOptionName => $moduleOptionValue) {
				scModuleOptions::createModuleOption($moduleId, $moduleOptionName, $moduleOptionValue);
			}
			
			// Commit
			miStaticDBUtil::commit();
		}
		
		public function delete()
		{
			$this->callModuleModelStrategies('delete');
		}
		
		protected function callModuleModelStrategies($operation)
		{
			$strategies = $this->getModuleModelStrategies();
			foreach ($strategies as $strategy)
				call_user_func(array($strategy, $operation));
		}
		
		/**
		 * Returns field strategy objects
		 * 
		 * @return array
		 */
		protected function getModuleModelStrategies()
		{
			static $strategies = null;
			
			if ($strategies == null) {
				$strategies = array();
				
				if (($this->_module == null) or !($this->_module instanceof IModuleOptions))
					return $strategies;
				$options = $this->_module->getOptionFields(new miWebForm(array()));
				if ($options == null)
					return $strategies;
				
				$strategyNames =  $options->getModelStrategies();
				foreach ($strategyNames as $fieldName => $strategyClassName)
					$strategies[$fieldName] = new $strategyClassName($this->getViewMapperController());
			}
			return $strategies;
		}
		
		protected function afterUpdate()
		{
			scModuleOptions::callOnUpdateCallbacks($this->getModuleId(), $this->_optionValues);
		}
	}
?>