<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Base class for multivalues view mapper
	 * 
	 * Manages subitems in another table linked to an item from the original table by a foreign key (1:M relation)
	 * (e.g. multiple categories linked to one product)
	 */
	class scViewMapperMultiValues extends miViewMapper {
		protected $_tableName;
		protected $_tableKeyName;
		protected $_tableValueName;
		protected $_subItemIds = array();
		
		public function getItemId()
		{
			return $this->getViewMapperController()->getDomainObject()->get($this->_tableKeyName);
		}
		
		public function get($fieldName)
		{
			return $this->_subItemIds;
		}
		
		public function set($fieldName, $fieldValue)
		{
			return $this->_subItemIds = $fieldValue;
		}
		
		public function read()
		{
			$sql = 'SELECT ' . $this->_tableValueName . ' FROM ' . $this->_tableName . ' WHERE ' .
				$this->_tableKeyName . '="' . sql_escape_string($this->getItemId()) . '"';
			$this->_subItemIds = miUtil::getDBArray($sql, $this->_tableValueName, $this->_tableValueName);
		}
		
		public function insert()
		{
			$this->update();
		}
		
		public function update()
		{
			$itemId = $this->getItemId();
			
			// Remove old subitems
			miStaticDBUtil::execDelete($this->_tableName, $this->_tableKeyName, $itemId);

			if (count($this->_subItemIds) == 0)
				return;

			// Save subitems
			$records = array();
			foreach ($this->_subItemIds as $subItemId) {
				$values = array(
					$this->_tableKeyName => $itemId,
					$this->_tableValueName => $subItemId
				);
				
				$records[] = "('" . implode("', '", $values) . "')";
			}

			$sql = 'INSERT INTO ' . $this->_tableName . '(' . $this->_tableKeyName . ',' . $this->_tableValueName . ') VALUES ' . implode(', ', $records);
			miStaticDBUtil::execSQL($sql);
		}
		
		public function delete()
		{
			$defaultViewMapper = $this->getViewMapperController()->getViewMapper(miViewMapperControllerDefault::DEFAULT_VIEW_MAPPER_ID);
			$pkValue = $defaultViewMapper->getPkValue();
			miStaticDBUtil::execDelete($this->_tableName, $this->_tableKeyName, $pkValue);
		}
	}
?>