<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperNewsletters extends miViewMapper {
		
		/**
		 * @var scCustomerDomainObject
		 */
		protected $_customer = null;
		
		protected $_values = array();
		
		public function __construct(miViewMapperController $viewMapperController)
		{
			parent::__construct($viewMapperController);
			
			$this->_customer = $viewMapperController->getView()->getDomainObject();
		}
		
		public function read()
		{
			$recordset = new miSqlRecordset('NLNewsletters');
			$recordset->addFilter(new miSqlFilterEqual('NewsletterIsActive', 1));
			$recordset->addJoinCondition('LEFT', 'NLSubscribers', 'ON (' .
				' NLNewsletters.NewsletterID = NLSubscribers.NewsletterID' .
				' AND (CustomerID = ' . intval($this->_customer->get('CustomerID')) . ' OR SubscriberEmail = "' . sql_escape_string($this->_customer->get('CustomerEmail')) . '")' .
				' AND SubscriberIsActive)');
			$recordset->setSelectFields(array('NLNewsletters.NewsletterID', 'MAX(SubscriberIsActive) AS SubscriberIsActive'));
			$recordset->setGroupBy('NLNewsletters.NewsletterID');
			$subscribed = $recordset->getAllRecords();
			$subscribed = miUtil::getKeyValueArray($subscribed, 'NewsletterID', 'SubscriberIsActive');
			$this->_values = $subscribed;
		}
		
		public function get($field)
		{
			return $this->_values;
		}
		
		public function set($field, $values)
		{
			$this->_values = $values;
		}
		
		public function insert()
		{
			return $this->update();
		}
		
		public function update()
		{
			// Load module classes
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.newsletter');
			scModulesFactory::getImplementation($moduleId, 'IUserModule');
			
			// All the newsletters that he is subscribing to
			$newsletterIds = array();
			foreach ($this->_values as $newsletterId => $isActive) {
				if (!$isActive) {
					continue;
				}
				$newsletter = new scNewsletter();
				$newsletter->read($newsletterId);
				$newsletter->subscribe($this->_customer->get('CustomerEmail'), $this->_customer->get('CustomerID'));
				$newsletterIds[] = $newsletterId;
			}
			
			// Do not unsubscribe from inactive newsletters
			$inactive = new miSqlRecordset('NLNewsletters');
			$inactive->addFilter(new miSqlFilterEqual('NewsletterIsActive', 0));
			$inactive->setSelectFields(array('NewsletterID'));
			$inactive = $inactive->getAllRecords();
			$inactive = miUtil::getKeyValueArray($inactive, 'NewsletterID', 'NewsletterID');
			$newsletterIds = array_merge($newsletterIds, array_values($inactive));
			
			$query = 'UPDATE NLSubscribers SET SubscriberIsActive = 0' .
				' WHERE (SubscriberEmail = ? OR CustomerID = ?)';
			if (!empty($newsletterIds)) {
				$query .= ' AND NewsletterID NOT IN (' . implode(', ', $newsletterIds) . ')';
			}
			
			$params = array(
				$this->_customer->get('CustomerEmail'),
				$this->_customer->get('CustomerID'),
			);
			miStaticDBUtil::execSQL($query, $params);
			
			return (int)$this->_customer->get('CustomerID');
		}
	}
?>