<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperProductEgoods extends scViewMapperDynRows {
		
		/**
		 * Read the module options
		 */
		public function read()
		{
			foreach ($this->_rsProperties as $fieldName => $rsProperties) {
				$this->_rsValues[$fieldName] = $this->readRecords($rsProperties);
			}
		}
		
		/**
		 * @param array $rsValues
		 * @param array $rsProperties
		 */
		protected function saveDynRows($rsValues, $rsProperties)
		{
			$action = $this->_viewMapperController->getView()->getController()->getDispatcher()->getActionObj();
			
			$productId = $this->getForeignKeyValue($rsProperties['foreignKey']);
			$productsTable = $rsProperties['table'];
			
			$webForm = $this->_viewMapperController->getWebForm();
			$egoodsWidget = $webForm->getWidget('RSProductEgoods');
			
			if ($action instanceof scExecDuplicateAction) {
				$oldIds = array();
				$newIds = $this->duplicateEgoods($rsValues, $productsTable, $productId);
			} else if ($action instanceof miExecCreateAction) {
				$oldIds = array();
				$newIds = $this->createEgoods($rsValues, $productsTable, $productId);
			} else if ($action instanceof miExecEditAction) {
				$oldIds = $this->getProductEgoodIds($rsProperties);
				$newIds = $this->editEgoods($rsValues, $productsTable, $productId);
			}
			
			$this->deleteRemovedEgoods($oldIds, $newIds);
		}
		
		protected function duplicateEgoods($rsValues, $productsTable, $productId)
		{
			$webForm = $this->_viewMapperController->getWebForm();
			$egoodsWidget = $webForm->getWidget('RSProductEgoods');
			
			// Get the directory where files were uploaded.
			$oldDir = $egoodsWidget->getDirName();
			// Get the directory where files should reside.
			$newDir = scPaths::EGOODS_PRODUCTS . $productId;
			
			$sharedEgoodsDir = $egoodsWidget->getSharedDirName();
			
			$newIds = array();
			// The product is being copied. All known items should be copied.
			// All items that have just been uploaded should be moved.
			foreach ($rsValues as $rowIndex => $egood) {
				// Check if this is a shared egood. If it is then it should not be moved.
				if (strpos($egood['ProductEgoodFile'], $sharedEgoodsDir) === 0) {
					$values = array(
						'ProductID' => $productId,
						'ProductEgoodFile' => $egood['ProductEgoodFile']
					);
					$newIds[] = miStaticDBUtil::execInsert($productsTable, $values);
					continue;
				}
				
				// This is not a shared egood. It should be COPIED or MOVED to the current product's egoods dir.
				$oldLocation = $egood['ProductEgoodFile'];
				$newLocation = str_replace($oldDir, $newDir, $oldLocation);
				
				// Get full paths
				$oldPath = scPaths::getItemPath($oldLocation, scPaths::RESOURCE_BASE_EGOOD);
				$newPath = scPaths::getItemPath($newLocation, scPaths::RESOURCE_BASE_EGOOD);
				
				if ($egood['ProductEgoodID'] > 0) {
					// This e-good is one that is being duplicated.
					// COPY the file (creating subfolders if necessary)
					$oldumask = umask(0);
					if (!scFilesUtil::recursiveCopy($oldPath, $newPath)) {
						umask($oldumask);
						throw new miException(_t('Copying e-good file failed.'));
					}
					umask($oldumask);
				} else {
					// This e-good has just been uploaded.
					// MOVE the file
					$oldumask = umask(0);
					if (!scFilesUtil::recursiveCopy($oldPath, $newPath)) {
						umask($oldumask);
						throw new miException(_t('Copying e-good file failed.'));
					}
					umask($oldumask);
				}
				
				// Insert record in the database
				$values = array(
					'ProductID' => $productId,
					'ProductEgoodFile' => $newLocation
				);
				$newIds[] = miStaticDBUtil::execInsert($productsTable, $values);
				continue;
			}
			
			return $newIds;
		}
		
		protected function createEgoods($rsValues, $productsTable, $productId)
		{
			// All NOT SHARED items should be moved.
			$webForm = $this->_viewMapperController->getWebForm();
			$egoodsWidget = $webForm->getWidget('RSProductEgoods');
			
			$newIds = array();
			
			// MOVING may be done by just moving renaming (moving) the temp directory:
			// i.e. mv path/temp path/product_egoods_dir
			
			// Get the directory where files were uploaded.
			$oldDir = $egoodsWidget->getDirName();
			// Get the directory where files should reside.
			$newDir = scPaths::EGOODS_PRODUCTS . $productId;
			
			if ($oldDir !== $newDir) {
				$oldPath = scPaths::getItemPath($oldDir, scPaths::RESOURCE_BASE_EGOOD);
				$newPath = scPaths::getItemPath($newDir, scPaths::RESOURCE_BASE_EGOOD);
				
				if (file_exists($oldPath)) {
					$oldumask = umask(0);
					if (!scFilesUtil::recursiveMove($oldPath, $newPath)) {
						umask($oldumask);
						throw new miException(_t('Moving e-good files failed.'));
					}
					umask($oldumask);
				}
			}
			
			foreach ($rsValues as $rowIndex => $rsValuesRow) {
				$values = array(
					'ProductID' => $productId,
					'ProductEgoodFile' => str_replace($oldDir, $newDir, $rsValuesRow['ProductEgoodFile'])
				);
				$newIds[] = miStaticDBUtil::execInsert($productsTable, $values);
			}
			
			return $newIds;
		}
		
		protected function editEgoods($rsValues, $productsTable, $productId)
		{
			$webForm = $this->_viewMapperController->getWebForm();
			$egoodsWidget = $webForm->getWidget('RSProductEgoods');
			
			$newIds = array();
			
			foreach ($rsValues as $rowIndex => $rsValuesRow) {
				if (isset($rsValuesRow['ProductEgoodID']) && $rsValuesRow['ProductEgoodID'] > 0) {
					$newIds[] = (int)$rsValuesRow['ProductEgoodID'];
					continue;
				}
				
				$values = array(
					'ProductID' => $productId,
					'ProductEgoodFile' => $rsValuesRow['ProductEgoodFile']
				);
				$newIds[] = miStaticDBUtil::execInsert($productsTable, $values);
			}
			
			return $newIds;
		}
		
		protected function getProductEgoodIds($rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			
			$query = 'SELECT ProductEgoodID FROM ' . $rsProperties['table'] .
				' WHERE ' . $rsProperties['foreignKey'] . '="' . sql_escape_string($foreignKeyValue) . '"';
			return miUtil::getDBArray($query, 'ProductEgoodID', 'ProductEgoodID');
		}
		
		protected function deleteRemovedEgoods($oldIds, $newIds)
		{
			$toDelete = array_diff($oldIds, $newIds);
			if (empty($toDelete))
				return;
			
			$selectQuery = 'SELECT ProductEgoodID, ProductEgoodFile FROM ProductEgoods' .
				' WHERE ProductEgoodID IN (' . implode(', ', $toDelete) . ')';
			$dataToDelete = miUtil::getDBArray($selectQuery, 'ProductEgoodID', 'ProductEgoodID');
			
			foreach ($dataToDelete as $egood) {
				$egoodPath = scPaths::getItemPath($egood, scPaths::RESOURCE_BASE_EGOOD);
				@unlink($egoodPath);
			}
			
			$deleteQuery = 'DELETE FROM ProductEgoods  WHERE ProductEgoodID IN (' . implode(', ', $toDelete) . ')';
			miStaticDBUtil::execSQL($deleteQuery);
		}
	}
?>