<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperProductImages extends scViewMapperDynRows {
		protected static $_error = '';
		 
		public static function errorHandler($errorNumber, $errorMessage)
		{
			self::$_error = $errorMessage;
		
			return false;
		}
		
		public static function isProductImageNameAvailable($imageName, $fullFilename)
		{
			try {
				$record = new miSqlRecord('ProductImages', 'ProductImageID');
				$record->read('ProductImageOriginalName', $fullFilename);
			} catch (miDBException $e) {
				if ($e->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					return true;
				}
			}
			return false;
		}
		
		public function getLastError()
		{
			return self::$_error;
		}
		
		protected function saveDynRows($rsValues, $rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			$foreignTable = $rsProperties['table'];
			$foreignKeyName = $rsProperties['foreignKey'];
			
			$oldProductImageIds = $this->getProductImageIds($rsProperties);
			
			$newProductImageIds = array();
			$isThereDefaultImage = false;
			
			// insert new records
			$rsRowCount = miGetParamDefault('RSProductImages' . '__rowCount', 0);
			
			foreach ($rsValues as $rowIndex => $rsValuesRow) {
				$values = array($foreignKeyName => $foreignKeyValue);
				foreach ($rsValuesRow as $key => $value)
					$values[$key] = $value;
				
				// Special processing of the ProductImageIsDefault radio button
				if (!$isThereDefaultImage && ($values['ProductImageID'] == miGetParamDefault('ProductImageIsDefault', 0))) {
					$values['ProductImageIsDefault'] = 1;
					$isThereDefaultImage = true;
				} else {
					$values['ProductImageIsDefault'] = 0;
				}
				
				if (!intval($values['ProductImageID'])) {
					// Do not create records if nothing has been uploaded
					if (empty($_FILES['ProductImagePicture::'.($rowIndex+1).'::']['name']) and empty($_FILES['ProductImageThumb::'.($rowIndex+1).'::']['name']))
						continue;
					
					// The ProductImageID is equal to 0 (zero). We need to unset it for MySQL in STRICT MODE
					unset($values['ProductImageID']);
					miStaticDBUtil::execSQL('UPDATE ProductImages SET ProductImageSort = ProductImageSort + 1 WHERE ProductID = ? AND ProductImageSort > ? ORDER BY ProductImageSort DESC', array($values['ProductID'], $rowIndex));
					$values['ProductImageSort'] = $rowIndex + 1;
					$productImageId = miStaticDBUtil::execInsert($foreignTable, $values);
					
					$newProductImageIds[] = $this->processImage($productImageId, $rowIndex+1);
					continue;
				}
				
				// TODO: V1.1 Not a good practice to use miGetParam in the view mapper
				if (miGetParamDefault('duplicate', 'false') != 'true') {
					$productImageId = intval($values['ProductImageID']);
					miStaticDBUtil::execUpdate($foreignTable, $values, 'ProductImageID', $productImageId);
					$newProductImageIds[] = $this->processImage($productImageId, $rowIndex+1);
					continue;
				}
				
				if (!empty($_FILES['ProductImagePicture::'.($rowIndex+1).'::']['name']) or !empty($_FILES['ProductImageThumb::'.($rowIndex+1).'::']['name'])) {
					unset($values['ProductImageID']);
					$productImageId = miStaticDBUtil::execInsert($foreignTable, $values);
					$newProductImageIds[] = $this->processImage($productImageId, $rowIndex+1);
					continue;
				}
				
				// A newer image has not been uploaded.
				// Get the information from the database
				$sql = 'SELECT * FROM ProductImages WHERE ProductImageID = ?';
				$imageInfo = miStaticDBUtil::execSelect($sql, array($values['ProductImageID']));
				$imageInfo = reset($imageInfo);
				
				// Modify the image info to prepare for copying
				$imageInfo['ProductID'] = $values['ProductID'];
				$imageInfo['ProductImageIsDefault'] = $values['ProductImageIsDefault'];
				unset($imageInfo['ProductImageID']);
				
				// Reinsert the image information
				$newImageId = miStaticDBUtil::execInsert($foreignTable, $imageInfo);
				$newProductImageIds[] = $newImageId;
				
				$oldImageOriginalPath = scPaths::getItemPath($values['ProductImageID'], scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE);
				$oldImagePath = scPaths::getItemPath($values['ProductImageID'], scPaths::RESOURCE_PRODUCT_IMAGE);
				$oldImageThumbPath = scPaths::getItemPath($values['ProductImageID'], scPaths::RESOURCE_PRODUCT_THUMB);
				$oldImageBoxPath = scPaths::getItemPath($values['ProductImageID'], scPaths::RESOURCE_PRODUCT_BOX_IMAGE);
				$oldImageLargePath = scPaths::getItemPath($values['ProductImageID'], scPaths::RESOURCE_PRODUCT_LARGE_IMAGE);
				
				$newImageOriginalPath = scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE);
				$newImagePath = scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_IMAGE);
				$newImageThumbPath = scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_THUMB);
				$newImageBoxPath = scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE);
				$newImageLargePath = scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE);
				
				// Not quite good performance, but works
				set_error_handler(array(__CLASS__, 'errorHandler'), E_WARNING);		// Used when copying functions
				if (!@copy($oldImageOriginalPath, $newImageOriginalPath)
					|| !@copy($oldImagePath, $newImagePath)
					|| !@copy($oldImageThumbPath, $newImageThumbPath)
					|| !@copy($oldImageBoxPath, $newImageBoxPath)
					|| !@copy($oldImageLargePath, $newImageLargePath)) {
					$error = $this->getLastError();
					restore_error_handler();
					
					@unlink($newImageOriginalPath);
					@unlink($newImagePath);
					@unlink($newImageBoxPath);
					@unlink($newImageThumbPath);
					@unlink($newImageLargePath);
					
					miStaticDBUtil::execDelete('ProductImages', 'ProductImageID', $newImageId);
					
					// Prepare somewhat friendly errors
					$message = _t('Duplicating image failed: ') . $error;
					
					throw new miException($message);
				}
				
				restore_error_handler();
				
				$updateParams = array(
					'ProductImageOriginal' => scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE, scPaths::PATH_WEB),
					'ProductImage' => scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_IMAGE, scPaths::PATH_WEB),
					'ProductImageThumb' => scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_THUMB, scPaths::PATH_WEB),
					'ProductImageBox' => scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE, scPaths::PATH_WEB),
					'ProductImageLarge' => scPaths::getItemPath($newImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE, scPaths::PATH_WEB),
				);
				miStaticDBUtil::execUpdate('ProductImages', $updateParams, 'ProductImageID', $newImageId);
			}
			
			// If there is no default image, set the first one as default
			if (!$isThereDefaultImage and count($newProductImageIds) > 0) {
				miStaticDBUtil::execUpdate($foreignTable, array('ProductImageIsDefault' => 1), 'ProductImageID', $newProductImageIds[0]);
			}
			
			$this->deleteRemovedImages($oldProductImageIds, $newProductImageIds);
		}
		
		protected function processImage($productImageId, $rowIndex)
		{
			try {
				$this->uploadImage($productImageId, $rowIndex);
				return $productImageId;
			} catch (miException $exception) {
				@unlink(scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE));
				@unlink(scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_IMAGE));
				@unlink(scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE));
				@unlink(scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_THUMB));
				@unlink(scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE));
				miStaticDBUtil::execDelete('ProductImages', 'ProductImageID', $productImageId);
				throw $exception;
			}
		}
		
		/**
		 * Uploads image
		 * Optionally creates thumb
		 *
		 * @param int $productImageId
		 * @throws miException
		 * @throws scImageException
		 */
		protected function uploadImage($productImageId, $rowIndex)
		{
			$originalImageDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE);
			$imageDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_IMAGE);
			$boxImageDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE);
			$thumbDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_THUMB);
			$largeImageDest = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE);
			
			$imageOriginalName = '';
			$imageUploaded = $thumbUploaded = false;
			
			if (!empty($_FILES['ProductImagePicture::'.$rowIndex.'::']['name'])) {	
				if (move_uploaded_file($_FILES['ProductImagePicture::'.$rowIndex.'::']['tmp_name'], $originalImageDest) === false)
					throw new miException('Cannot upload file. Saving into "' . $originalImageDest . '" failed.');
				chmod($originalImageDest, 0644);
				$imageUploaded = true;
				$imageOriginalName = $_FILES['ProductImagePicture::'.$rowIndex.'::']['name'];
			}
			if (!empty($_FILES['ProductImageThumb::'.$rowIndex.'::']['name'])) {			
				if (move_uploaded_file($_FILES['ProductImageThumb::'.$rowIndex.'::']['tmp_name'], $thumbDest) === false)
					throw new miException('Cannot upload file. Saving into "' . $thumbDest . '" failed.');
				chmod($thumbDest, 0644);
				$thumbUploaded = true;
			}
			
			// If nothing has been uploaded, do nothing
			if (!$imageUploaded and !$thumbUploaded)
				return;
			
			$maxThumbWidth = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH);
			$maxThumbHeight = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_HEIGHT);
			
			// If image is uploaded but thumbnail is not
			if ($imageUploaded and !$thumbUploaded) {
				// Create thumbnail from image
				$resizeResult = scImageHandler::resizeImage($originalImageDest, $thumbDest, $maxThumbWidth, $maxThumbHeight);
			}
			// If thumbnail is uploaded but image is not
			elseif (!$imageUploaded and $thumbUploaded) {
				// Create image from thumbnail
				rename($thumbDest, $imageDest);
				$resizeResult = scImageHandler::resizeImage($originalImageDest, $thumbDest, $maxThumbWidth, $maxThumbHeight);					
			}
			// If both thumbnail and image is uploaded
			elseif ($imageUploaded and $thumbUploaded) {
				// Resize the thumbnail
				$resizeResult = scImageHandler::resizeImage($thumbDest, $thumbDest, $maxThumbWidth, $maxThumbHeight);					
			}
			
			// BOX IMAGE
			$maxBoxImageWidth = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_WIDTH);
			$maxBoxImageHeight = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_HEIGHT);
			$resizeResult = scImageHandler::resizeImage($originalImageDest, $boxImageDest, $maxBoxImageWidth, $maxBoxImageHeight);					
			
			// LARGE IMAGE
			$maxLargeImageWidth = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_WIDTH);
			$maxLargeImageHeight = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_HEIGHT);
			$resizeResult = scImageHandler::resizeImage($originalImageDest, $largeImageDest, $maxLargeImageWidth, $maxLargeImageHeight);					
			
			// Make sure that the fullsize image is not too large
			$maxImageWidth = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_MAX_WIDTH);
			$maxImageHeight = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_MAX_HEIGHT);
			$resizeResult = scImageHandler::resizeImage($originalImageDest, $imageDest, $maxImageWidth, $maxImageHeight);
			
			// Save the new dimensions
			self::setProductImageDimensions($productImageId, $originalImageDest, $imageDest, $largeImageDest, $boxImageDest, $thumbDest, $imageOriginalName);
		}
		
		public static function setProductImageDimensions($productImageId, $originalImageDest, $imageDest, $largeImageDest, $boxImageDest, $thumbDest, $imageOriginalName = '')
		{
			$updateParams = array();
			if (!empty($imageOriginalName)) {
				$updateParams['ProductImageOriginalName'] = scFilesUtil::generateFilenameFromString($imageOriginalName, array('scViewMapperProductImages', 'isProductImageNameAvailable'));
			}
			if (is_file($originalImageDest)) {
				$updateParams['ProductImageOriginal'] = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE, scPaths::PATH_WEB);
			}
			if (is_file($imageDest)) {
				list($updateParams['ProductImageWidth'], $updateParams['ProductImageHeight']) = scImageHandler::getImageSize($imageDest);
				$updateParams['ProductImageSize'] = filesize($imageDest);
				$updateParams['ProductImage'] = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_IMAGE, scPaths::PATH_WEB);
			}
			if (is_file($largeImageDest)) {
				list($updateParams['ProductImageLargeWidth'], $updateParams['ProductImageLargeHeight']) = scImageHandler::getImageSize($largeImageDest);
				$updateParams['ProductImageLargeSize'] = filesize($imageDest);
				$updateParams['ProductImageLarge'] = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE, scPaths::PATH_WEB);
			}
			if (is_file($thumbDest)) {
				list($updateParams['ProductImageThumbWidth'], $updateParams['ProductImageThumbHeight']) = scImageHandler::getImageSize($thumbDest);					
				$updateParams['ProductImageThumbSize'] = filesize($thumbDest);
				$updateParams['ProductImageThumb'] = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_THUMB, scPaths::PATH_WEB);
			}
			if (is_file($boxImageDest)) {
				list($updateParams['ProductImageBoxWidth'], $updateParams['ProductImageBoxHeight']) = scImageHandler::getImageSize($boxImageDest);					
				$updateParams['ProductImageBoxSize'] = filesize($boxImageDest);
				$updateParams['ProductImageBox'] = scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE, scPaths::PATH_WEB);
			}
			if (!empty($updateParams)) {
				miStaticDBUtil::execUpdate('ProductImages', $updateParams, 'ProductImageID', $productImageId);
			}
		}
		
		protected function deleteRemovedImages($oldProductImageIds, $newProductImageIds)
		{
			foreach ($oldProductImageIds as $oldProductImageId) {
				if (!in_array($oldProductImageId, $newProductImageIds)) {
					miStaticDBUtil::execDelete('ProductImages', 'ProductImageID', $oldProductImageId);
					
					$originalImageFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE);
					$imageFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_IMAGE);
					$largeImageFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE);
					$boxImageFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE);
					$thumbFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_THUMB);
					@unlink($originalImageFileName);
					@unlink($imageFileName);
					@unlink($largeImageFileName);
					@unlink($boxImageFileName);
					@unlink($thumbFileName);
				}
			}
			$sort = 0;
			foreach ($newProductImageIds as $newProductImageId) {
				$sort++;
				miStaticDBUtil::execSQL('UPDATE ProductImages SET ProductImageSort = ? WHERE ProductImageID = ?', array($sort, $newProductImageId));
			}
		}
		
		protected function deleteDynRows($rsProperties)
		{
			$oldProductImageIds = $this->getProductImageIds($rsProperties);
			
			parent::deleteDynRows($rsProperties);
			
			// Delete all images
			foreach ($oldProductImageIds as $oldProductImageId) {
				$originalImageFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_ORIGINAL_IMAGE);
				$imageFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_IMAGE);
				$largeImageFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_LARGE_IMAGE);
				$boxImageFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_BOX_IMAGE);
				$thumbFileName = scPaths::getItemPath($oldProductImageId, scPaths::RESOURCE_PRODUCT_THUMB);
				@unlink($originalImageFileName);
				@unlink($imageFileName);
				@unlink($largeImageFileName);
				@unlink($boxImageFileName);
				@unlink($thumbFileName);
			}
		}
		
		protected function getProductImageIds($rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			
			$query = 'SELECT ProductImageID FROM ' . $rsProperties['table'] .
				' WHERE ' . $rsProperties['foreignKey'] . '="' . sql_escape_string($foreignKeyValue) . '"';
			return miUtil::getDBArray($query, 'ProductImageID', 'ProductImageID');
		}
	}
?>