<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperProductOptionGroups extends miViewMapper implements IOptionGroupsProvider {
		
		/**
		 * @var array
		 */
		protected $_values = array();
		
		/**
		 * @var scProductDomainObject
		 */
		protected $_domainObject = null;
		
		protected $_optionGroups = array();
		
		protected $_newOptionGroupIds = array();
		
		public function __construct(miViewMapperControllerDefault $viewMapperController)
		{
			parent::__construct($viewMapperController);
			$this->_domainObject = $viewMapperController->getDomainObject();
			if (!($this->_domainObject instanceof scProductDomainObject)) {
				throw new miException(_i('scProductDomainObject expected.'));
			}
		}
		
		public function read()
		{
			$productId = (int)$this->_domainObject->get($this->_domainObject->getPKName());
			
			$recordset = new miSqlRecordset('ProductOptionGroups');
			$recordset->addFilter(new miSqlFilterEqual($this->_domainObject->getPKName(), $productId));
			$optionGroups = $recordset->getAllRecords();
			$optionGroups = miUtil::getKeyValueArray($optionGroups, 'OptionGroupID', 'OptionGroupID');
			
			$this->setValues(array_values($optionGroups));
		}
		
		public function get($fieldName)
		{
			return $this->_values;
		}
		
		public function set($fieldName, $value)
		{
			$this->setValues(array_unique(array_filter($value)));
		}
		
		public function insert()
		{
			return $this->update();
		}
		
		public function update()
		{
			$productId = (int)$this->_domainObject->get('ProductID');
			
			// Get current ProductOptionGroups
			$recordset = new miSqlRecordset('ProductOptionGroups');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$recordset->setSelectFields(array('OptionGroupID'));
			$current = $recordset->getAllRecords();
			$current = miUtil::getKeyValueArray($current, 'OptionGroupID', 'OptionGroupID');
			
			// array_diff: ...all the entries from array1 that are not present array2.
			$toDelete = array_diff($current, $this->_values);
			$toInsert = array_diff($this->_values, $current);
			
			// Check if there is anything to DELETE
			if (!empty($toDelete)) {
				$deleteSql = 'DELETE FROM ProductOptionGroups WHERE ProductID = ' . $productId .
					' AND OptionGroupID IN (' . implode(', ', $toDelete) . ')';
				miStaticDBUtil::execSQL($deleteSql);
			}
			
			// Check if there is anything to INSERT
			if (empty($toInsert)) {
				return $productId;
			}
			
			$multiValues = array();
			foreach ($toInsert as $optionGroupId) {
				$multiValues[] = array(
					'ProductID' => $productId,
					'OptionGroupID' => (int)$optionGroupId
				);
				// Store it as a newOptionGroupId (see IOptionGroupsProvider)
				$this->_newOptionGroupIds[] = $optionGroupId;
			}
			miStaticDBUtil::execMultiInsert('ProductOptionGroups', $multiValues);
			
			return $productId;
		}
		
		public function getOptionGroups()
		{
			if (is_null($this->_optionGroups)) {
				$this->_optionGroups = array();
				foreach ($this->_values as $optionGroupId) {
					$optionGroup = new scOptionGroupDomainObject();
					$optionGroup->read($optionGroupId);
					$this->_optionGroups[] = $optionGroup;
				}
			}
			
			return $this->_optionGroups;
		}
		
		public function getNewOptionGroupIds()
		{
			return $this->_newOptionGroupIds;
		}
		
		protected function setValues($values)
		{
			$this->_values = $values;
			$this->_optionGroups = null;
		}
	}
?>