<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperProductPrices extends scViewMapperDynRows {
		public function read()
		{
			parent::read();
			
			$records = $this->_rsValues['RSProductPrices'];
			
			$quantities = array();
			foreach ($records as $record) {
				$quantities[$record['ProductWholesalePriceQuantity']] = $record['ProductWholesalePriceQuantity'];
			}
			sort($quantities);
			
			// Flatten the records - one record per quantity level
			$newRecords = array();
			foreach ($quantities as $quantity) {
				foreach ($records as $record) {
					if ($record['ProductWholesalePriceQuantity'] != $quantity)
						continue;
					
					if (isset($newRecords[$record['ProductWholesalePriceQuantity']])) {
						$newRecords[$record['ProductWholesalePriceQuantity']]['ProductWholesalePriceAmount' . $record['CustomerGroupID']] = $record['ProductWholesalePriceAmount'];
						continue;
					}
					
					$newRecords[$record['ProductWholesalePriceQuantity']] = array(
						'ProductID' => $record['ProductID'],
						'ProductWholesalePriceQuantity' => $record['ProductWholesalePriceQuantity'],
						'ProductWholesalePriceAmount' . $record['CustomerGroupID'] => $record['ProductWholesalePriceAmount']
					);
				}
			}
			
			$this->_rsValues['RSProductPrices'] = array_values($newRecords);
		}
		
		protected function getProductVariants($rsProperties)
		{
			$productVariantsRs = new miSqlRecordset('ProductVariants');
			$productVariantsRs->addJoinCondition('LEFT', $rsProperties['table'], 'USING(ProductVariantID)');
			$productVariantsRs->addFilter(new miSqlFilterEqual('ProductVariants.ProductID', $this->getForeignKeyValue($rsProperties['foreignKey'])));
			$productVariantsRs->setSelectFields(array(
				'ProductVariants.ProductVariantID AS ProductVariantID',
				sprintf('`%s`.CustomerGroupID AS CustomerGroupID', $rsProperties['table']),
				sprintf('`%s`.ProductWholesalePriceQuantity AS ProductWholesalePriceQuantity', $rsProperties['table'])
			));
			$productVariants = $productVariantsRs->getRecords();
			
			$productVariantsArray = array();
			foreach ($productVariants as $productVariant)
				$productVariantsArray[$productVariant['ProductVariantID']][$productVariant['CustomerGroupID']][] = $productVariant['ProductWholesalePriceQuantity'];
			
			return $productVariantsArray;
		}
		
		protected function saveDynRows($rsValues, $rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			$foreignKeyName = $rsProperties['foreignKey'];
			$defaultCustomerGroup = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP);
			$productVariantsArray = $this->getProductVariants($rsProperties);
			
			miStaticDBUtil::startTransaction();
			miStaticDBUtil::execSQL(sprintf('DELETE FROM `%s` WHERE `%s` = %d AND ProductVariantID IS NULL', $rsProperties['table'], $foreignKeyName, $foreignKeyValue));
			
			$variantsValues = array();
			foreach ($rsValues as $rowIndex => $rsValuesRow) {
				$values = array($foreignKeyName => $foreignKeyValue);
				$values['ProductWholesalePriceQuantity'] = $rsValuesRow['ProductWholesalePriceQuantity'];
				
				// Insert record for each price amount
				foreach ($rsValuesRow as $key => $value) {
					if (substr_compare('ProductWholesalePriceAmount', $key, 0, strlen('ProductWholesalePriceAmount')) != 0)
						continue;
					
					$customerGroupId = substr($key, strlen('ProductWholesalePriceAmount'));
					$values['CustomerGroupID'] = $customerGroupId;
					$values['ProductWholesalePriceAmount'] = $value;
					$values['ProductVariantID'] = null;
					
					miStaticDBUtil::execInsert($rsProperties['table'], $values);
					
					// Check if variant with this customer group and quantity exists
					// If not save one
					foreach ($productVariantsArray as $variantId => $variantCustomerGroups) {
						if (!empty($variantCustomerGroups[$customerGroupId]) && in_array($values['ProductWholesalePriceQuantity'], $variantCustomerGroups[$customerGroupId]))
							continue;
						else {
							$variantValues = $values;
							$variantValues['ProductVariantID'] = $variantId;
							$variantsValues[] = $variantValues;
						}
					}
					
					if (($values['ProductWholesalePriceQuantity'] == 1) && ($customerGroupId == $defaultCustomerGroup)) {
						$product = $this->_viewMapperController->getDomainObject();
						$product->set('ProductPrice', $value);
						$product->update();
					}
				}
			}
			
			miStaticDBUtil::execMultiInsert($rsProperties['table'], $variantsValues);
			miStaticDBUtil::commit();
		}
		
		// Just because we want only wholesale prices for this product (ProductVariantID IS NULL)
		protected function readRecords($rsProperties)
		{
			$foreignKeyValue = $this->getForeignKeyValue($rsProperties['foreignKey']);
			
			// Check for ProductVariantID to be NULL
			$selectQuery = 'SELECT * FROM '.$rsProperties['table'].' WHERE '.$rsProperties['foreignKey'].' = '. intval($foreignKeyValue) . ' AND ProductVariantID IS NULL';
			return miStaticDBUtil::execSelect($selectQuery);
		}
	}
?>