<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	interface IOptionGroupsProvider {
		
		/**
		 * @return scOptionGroupDomainObject[]
		 */
		public function getOptionGroups();
		
		/**
		 * @return int[]
		 */
		public function getNewOptionGroupIds();
	}
	
	class scViewMapperProductVariantOptions extends miViewMapper {
		
		protected $_viewMappers = array();
		
		/**
		 * @var scProductDomainObject
		 */
		protected $_domainObject;
		
		public function __construct(miViewMapperController $viewMapperController)
		{
			parent::__construct($viewMapperController);
			$this->_domainObject = $viewMapperController->getDomainObject();
			if (!($this->_domainObject instanceof scProductDomainObject)) {
				throw new miException(_i('scProductDomainObject expected.'));
			}
			
			$this->_viewMappers['ProductOptionGroups'] = new scViewMapperProductOptionGroups($viewMapperController);
			$this->_viewMappers['LocalOptionGroups'] = new scViewMapperLocalOptions($viewMapperController);
			
			// Setup LocalOptionGroups->_rsProperties
			$dataFields = $viewMapperController->getView()->getDataFields();
			foreach ($dataFields as $dataField) {
				if ($dataField['data'] != 'LocalOptionGroups')
					continue;
				$this->_viewMappers['LocalOptionGroups']->setRsProperties('LocalOptionGroups', $dataField['properties']);
				break;
			}
		}
		
		public function read()
		{
			foreach ($this->_viewMappers as $viewMapper) {
				$viewMapper->read();
			}
		}
		
		public function set($fieldName, $value)
		{
			$this->_viewMappers[$fieldName]->set($fieldName, $value);
		}
		
		public function get($fieldName)
		{
			return $this->_viewMappers[$fieldName]->get($fieldName);
		}
		
		public function insert()
		{
			$variantGroups = new scSet();
			foreach ($this->_viewMappers as $viewMapper) {
				$viewMapper->insert();
				$groups = $this->getVariantOptions($viewMapper);
				foreach ($groups as $group) {
					$variantGroups = $variantGroups->cartesianProduct($group);
				}
			}
			
			scProductVariantsUtil::generateVariants($this->_domainObject, $variantGroups);
			
			return $this->_domainObject->get($this->_domainObject->getPKName());
		}
		
		public function update()
		{
			$variantGroups = new scSet();
			foreach ($this->_viewMappers as $viewMapper) {
				$viewMapper->update();
				$groups = $this->getVariantOptions($viewMapper);
				foreach ($groups as $group) {
					$variantGroups = $variantGroups->cartesianProduct($group);
				}
			}
			
			scProductVariantsUtil::generateVariants($this->_domainObject, $variantGroups);
			return $this->_domainObject->get($this->_domainObject->getPKName());
		}
		
		/**
		 * @param IOptionGroupsProvider $groupsProvider
		 * @return array
		 */
		protected function getVariantOptions(IOptionGroupsProvider $groupsProvider)
		{
			$optionGroups = $groupsProvider->getOptionGroups();
			$newOptionGroupIds = $groupsProvider->getNewOptionGroupIds();
			$variantGroups = array();
			foreach ($optionGroups as $optionGroup) {
				if (!$optionGroup->isVariant()) {
					continue;
				}
				$options = $optionGroup->getValues();
				if (empty($options)) {
					continue;
				}
				
				$optionGroupId = $optionGroup->get($optionGroup->getPKName());
				if (in_array($optionGroupId, $newOptionGroupIds)) {
					$optionGroup->setIsNew(true);
				}
				$variantGroups[] = $optionGroup;
			}
			return $variantGroups;
		}
	}
?>