<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperProductVariants extends scViewMapperDynRows {
		
		protected $_productQuantity = 0;
		
		/**
		 * @var scProductDomainObject
		 */
		protected $_productDomainObject = null;
		
		public static function isImageNameAvailable($imageName, $fullFilename)
		{
			try {
				$record = new miSqlRecord('ProductVariants', 'ProductVariantID');
				$record->read('ProductVariantImageOriginalName', $fullFilename);
			} catch (miDBException $e) {
				if ($e->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					return true;
				}
			}
			return false;
		}
		
		public function __construct(miViewMapperController $model)
		{
			parent::__construct($model);
			
			$this->_productDomainObject = $model->getView()->getDomainObject();
		}
		
		public function update()
		{
			$result = parent::update();
			
			if ($this->_productDomainObject->hasVariants()) {
				$this->_productDomainObject->set('ProductQuantity', $this->_productQuantity);
				$defaultVariantPrice = $this->_productDomainObject->getDefaultVariant()->get('ProductVariantPrice');
				$this->_productDomainObject->set('ProductPrice', $defaultVariantPrice);
				$this->_productDomainObject->update();
				$this->updateVariantsWholesalePrices();
			}
			
			return $result;
		}
		
		/**
		 * Do not depend on registered fields. If a widget returns data (getData()) then it's probably needed.
		 */
		protected function setValuesToRecord($record, $values, $rsProperties, $rowIndex)
		{
			foreach ($values as $name => $value) {
				switch ($name) {
					case 'ProductVariantQuantity':
						// Store variants' quantity so the sum can later be set to the product.
						// A product that has variants should have the sum quantity of its variants.
						// Negative quantity only means "not avaialable for sale".
						// But if a product has quantity of one variant and none of the other
						// that doesn't make the product unavailable. That's why only positive
						// values should be accounted.
						$this->_productQuantity += max(0, $value);
						break;
					
					case 'ProductVariantImage':
						if ($value instanceof scUploadedFile) {
							$imageName = $this->_productDomainObject->get($this->_productDomainObject->getPKName()) . '_' . $record->getPK();
							$image = $this->uploadImage($imageName, $value);
							if (!empty($image)) {
								$imageOriginalName = scFilesUtil::generateFilenameFromString($value->getName(), array('scViewMapperProductVariants', 'isImageNameAvailable'));
								$record->set('ProductVariantImageOriginalName', $imageOriginalName);
								$record->set('ProductVariantImageOriginal', $image[0]);
								
								$imagePath = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_IMAGE);
								$record->set('ProductVariantImage', $image[1]);
								$record->set('ProductVariantImageWidth', scImageHandler::getImageWidth($imagePath));
								$record->set('ProductVariantImageHeight', scImageHandler::getImageHeight($imagePath));
								$record->set('ProductVariantImageSize', filesize($imagePath));
								
								$largeImagePath = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_LARGE_IMAGE);
								$record->set('ProductVariantImageLarge', $image[2]);
								$record->set('ProductVariantImageLargeWidth', scImageHandler::getImageWidth($largeImagePath));
								$record->set('ProductVariantImageLargeHeight', scImageHandler::getImageHeight($largeImagePath));
								$record->set('ProductVariantImageLargeSize', filesize($largeImagePath));
								
								$boxImagePath = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_BOX_IMAGE);
								$record->set('ProductVariantImageBox', $image[3]);
								$record->set('ProductVariantImageBoxWidth', scImageHandler::getImageWidth($boxImagePath));
								$record->set('ProductVariantImageBoxHeight', scImageHandler::getImageHeight($boxImagePath));
								$record->set('ProductVariantImageBoxSize', filesize($boxImagePath));
								
								$thumbPath = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_THUMB);
								$record->set('ProductVariantImageThumb', $image[4]);
								$record->set('ProductVariantImageThumbWidth', scImageHandler::getImageWidth($thumbPath));
								$record->set('ProductVariantImageThumbHeight', scImageHandler::getImageHeight($thumbPath));
								$record->set('ProductVariantImageThumbSize', filesize($thumbPath));
							}
							continue(2); // Do not set data to the record.
						}
						
					case 'ProductVariantOptions':
						// This is a visualization-only field.
						continue(2);
				}
				
				$record->set($name, $value);
			}
			// Variants are created by scViewMapperProductVariantOptions.
			// The only thing this view mapper would do is update.
			return 'UPDATE';
		}
		
		protected function deleteOldRecords($recordIds, $foreignKeyValue, $rsProperties)
		{
			// Variants are deleted and inserted by scViewMapperProductVariantOptions.
			return;
		}
		
		protected function createRecord($rsProperties)
		{
			return new scDomainToRecordAdapter(new scProductVariant());
		}
		
		protected function readRecords($rsProperties)
		{
			$variants = parent::readRecords($rsProperties);
			
			foreach ($variants as &$variantData) {
				$variant = new scProductVariant();
				$variant->setRow($variantData);
				$variantData['ProductVariantOptions'] = $variant->getSelectedOptionsDetails();
				
				// Get variant wholesale prices
				$variantPrices = miStaticDBUtil::execSelect('SELECT * FROM ProductWholesalePrices WHERE ProductVariantID = ?', array($variantData['ProductVariantID']));
				if (!empty($variantPrices))
					$variantData['ProductVariantPrice'] = $variantPrices;
			}
			unset($variantData);
			
			return $variants;
		}
		
		protected function uploadImage($imageName, scUploadedFile $uploadedFile)
		{
			$return = array();
			
			list($fieldName, $suffix) = $this->getFieldNameAndSuffix($uploadedFile);
			
			// Move the image to its new location (from the temp one)
			$originalImage = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_ORIGINAL_IMAGE, scPaths::PATH_WEB);
			$originalImageFile = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_ORIGINAL_IMAGE);
			$uploadedFile->move($originalImageFile);
			
			// LARGE IMAGE
			$largeImage = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_LARGE_IMAGE);
			$largeImageWidth = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_WIDTH);
			$largeImageHeight = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_HEIGHT);
			
			scImageHandler::resizeImage($originalImageFile, $largeImage, $largeImageWidth, $largeImageHeight);
			$largeImage = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_LARGE_IMAGE, scPaths::PATH_WEB);
			
			// BOX IMAGE
			$boxImage = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_BOX_IMAGE);
			$boxImageWidth = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_WIDTH);
			$boxImageHeight = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_HEIGHT);
			
			scImageHandler::resizeImage($originalImageFile, $boxImage, $boxImageWidth, $boxImageHeight);
			$boxImage = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_BOX_IMAGE, scPaths::PATH_WEB);
			
			// THUMBNAIL
			$thumb = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_THUMB);
			$thumbWidth = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH);
			$thumbHeight = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_HEIGHT);
			
			scImageHandler::resizeImage($originalImageFile, $thumb, $thumbWidth, $thumbHeight);
			$thumb = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_THUMB, scPaths::PATH_WEB);
			
			// IMAGE
			$image = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_IMAGE);
			$imageWidth = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_MAX_WIDTH);
			$imageHeight = scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_MAX_HEIGHT);
			
			scImageHandler::resizeImage($originalImageFile, $image, $imageWidth, $imageHeight);
			$image = scPaths::getItemPath($imageName, scPaths::RESOURCE_PRODUCT_VARIANT_IMAGE, scPaths::PATH_WEB);
			
			return array($originalImage, $image, $largeImage, $boxImage, $thumb);
		}
		
		protected function getFieldNameAndSuffix(scUploadedFile $uploadedFile)
		{
			$fieldNameWithSuffix = explode('::', $uploadedFile->getFieldName());
			$fieldName = array_shift($fieldNameWithSuffix);
			$suffix = '::' . implode('::', $fieldNameWithSuffix);
			return array($fieldName, $suffix);
		}
		
		protected function updateVariantsWholesalePrices()
		{
			$productId = $this->_productDomainObject->get('ProductID');
			$variantsWholesalePrices = miGetParamDefault($this->_rsProperties['ProductVariants']['wholesalePriceField'], array());
			
			$productQuantities = miUtil::getDBArray(sprintf('SELECT ProductWholesalePriceQuantity FROM ProductWholesalePrices' .
				' WHERE ProductID = %d AND ProductVariantID IS NULL' .
				' GROUP BY ProductWholesalePriceQuantity', $productId),
				'ProductWholesalePriceQuantity', 'ProductWholesalePriceQuantity');
			
			foreach ($variantsWholesalePrices as $variantId => $customerGroups) {
				foreach ($customerGroups as $customerGroupId => $variantQuantities) {
					foreach ($variantQuantities as $variantQuantity => $priceAmount) {
						// Check product quantity exists
						// In the case that the product wholesale price line has been deleted
						if (!in_array($variantQuantity, $productQuantities)) {
							miStaticDBUtil::execSQL('DELETE FROM ProductWholesalePrices' .
								' WHERE ProductID = ? AND ProductVariantID = ? AND CustomerGroupID = ? AND ProductWholesalePriceQuantity = ?',
								array($productId, $variantId, $customerGroupId, $variantQuantity));
							continue;
						}
							
						miStaticDBUtil::execUpdate('ProductWholesalePrices', array('ProductWholesalePriceAmount' => $priceAmount),
							array('ProductID', 'ProductVariantID', 'CustomerGroupID', 'ProductWholesalePriceQuantity'),
							array($productId, $variantId, $customerGroupId, $variantQuantity));
					}
				}
			}
		}
	}
?>