<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scViewMapperUploadImage extends miViewMapper {
		protected $_images = array();
		protected $_savedImageWebPaths = array();
		
		protected $_properties = array();
		
		public function __construct(miViewMapperController $vmController)
		{
			parent::__construct($vmController);
			
			$dataFields = $vmController->getView()->getDataFields();
			
			$className = get_class($this);
			foreach ($dataFields as $dataField) {
				if (!isset($dataField['modelStrategy']) || ($dataField['modelStrategy'] != $className))
					continue;
				
				$this->_properties[$dataField['data']] = $dataField['properties'];
			}
		}
		
		public function get($fieldName)
		{
			$imageWebPath = $this->getViewMapperController()->getViewMapper()->get($fieldName);
			$thumbWebPath = str_replace('image_', 'thumb_', $imageWebPath);
			
			$this->_savedImageWebPaths[$fieldName] = $imageWebPath;
			$this->_images[$fieldName] = array(
				'imageFileName' => '',
				'imageWebPath' => $imageWebPath,
				'thumbFileName' => '',
				'thumbWebPath' => $thumbWebPath,
				'delete' => false,
			);

			return $this->_images[$fieldName];
		}
		
		public function set($fieldName, $fieldValue)
		{
			$domainObject = $this->getViewMapperController()->getView()->getDomainObject();
			$this->_savedImageWebPaths[$fieldName] = $domainObject->has($fieldName) ? $domainObject->get($fieldName) : '';
			
			if ($fieldValue['delete']) {
				// If picture is marked for deletion, update the record
				$this->getViewMapperController()->getViewMapper()->set($fieldName, '');
			} else {
				if (!empty($fieldValue['imageFileName'])) {
					// If changes in the picture are made update it
					$imageWebPath = scPaths::getFilenamePath($fieldValue['imageFileName'], $fieldValue['resource'], scPaths::PATH_WEB);
					$this->getViewMapperController()->getViewMapper()->set($fieldName, $imageWebPath);
				}
			}

			$this->_images[$fieldName] = $fieldValue;
		}
		
		protected function deleteOldImage($imageWebPath)
		{
			if ($imageWebPath == '')
				return;
			
			$imageFilePath = scPaths::getStoreRootDir() . $imageWebPath;
			if (@unlink($imageFilePath) === false)
				scLog::singleton()->warning(sprintf(_t('Failed to delete image "%s".'), $imageFilePath));
		}
		
		public function update()
		{
			$this->insert();
		}
		
		public function insert()
		{
			foreach ($this->_images as $fieldName => $image) {
				if ($image['delete']) {
					$this->deleteOldImage($this->_savedImageWebPaths[$fieldName]);
					continue;
				}
				
				if (empty($image['imageFileName']))
					continue;

				$this->moveImage($fieldName, $image);
				$this->moveThumb($fieldName, $image);
				$this->deleteOldImage($this->_savedImageWebPaths[$fieldName]);
			}
		}
		
		public function preDelete()
		{
			/*
			 * Get reference to the record and call it's readPK method
			 * The data read will be later used in the delete method
			 */
			$domainObject = $this->getViewMapperController()->getView()->getDomainObject();
			$domainObject->read(miGetParam($domainObject->getPKName()));
		}
		
		/**
		 * Called when record is deleted. Deletes all associated images
		 */
		public function delete()
		{
			$dataFields = $this->getViewMapperController()->getViewMapperDataFields(get_class());
			$domainObject = $this->getViewMapperController()->getView()->getDomainObject();
			
			foreach ($dataFields as $dataField) {
				$imageWebPath = $domainObject->get($dataField['data']);
				$this->deleteOldImage($imageWebPath);
			}
		}
		
		/**
		 * Move the temp thumb image into it's permanent location, resizing it if needed
		 * 
		 * @param string $fieldName
		 * @param array $image image info from the widget 
		 * @throws miException
		 */
		protected function moveThumb($fieldName, $image)
		{
			if (array_key_exists('noThumb', $this->_properties[$fieldName]) && $this->_properties[$fieldName]['noThumb'])
				return;			// There will be no thumbnail for this image
			
			// Move the thumb
			$thumbFilePath = scPaths::getFilenamePath($image['thumbFileName'], $image['resource']);
			$tempThumbFilePath = scPaths::getFilenamePath($image['thumbFileName'], scPaths::RESOURCE_TEMP_THUMB);
			
			// Check if we have max width and max height - if yes call auto-resize
			if (isset($this->_properties[$fieldName]['thumbMaxWidthSetting']) and isset($this->_properties[$fieldName]['thumbMaxHeightSetting'])) {
				$maxWidth = (int) scSettings::getSettingValue($this->_properties[$fieldName]['thumbMaxWidthSetting']);
				$maxHeight = (int) scSettings::getSettingValue($this->_properties[$fieldName]['thumbMaxHeightSetting']);
				scImageHandler::resizeImage($tempThumbFilePath, $thumbFilePath, $maxWidth, $maxHeight);
			} else {
				// No width and height - just move the thumb
				if (@rename($tempThumbFilePath, $thumbFilePath) === false) {
					scPaths::checkFileWritable($thumbFilePath, true);
					throw new miException(sprintf(_t('Thumb copying from "%s" to "%s" failed.'), $tempThumbFilePath, $thumbFilePath));
				}
			}
		}
		
		/**
		 * Move the temp full-size image into it's permanent location, resizing it if needed
		 * 
		 * @param string $fieldName
		 * @param array $image image info from the widget
		 * @throws miException
		 */
		protected function moveImage($fieldName, $image)
		{
			// 	Move the image
			$imageFilePath = scPaths::getFilenamePath($image['imageFileName'], $image['resource']);
			$tempImageFilePath = scPaths::getFilenamePath($image['imageFileName'], scPaths::RESOURCE_TEMP_IMAGE);
			
			// Check if we have max width and max height - if yes call auto-resize
			if (isset($this->_properties[$fieldName]['imageMaxWidthSetting']) and isset($this->_properties[$fieldName]['imageMaxHeightSetting'])) {
				$maxWidth = (int) scSettings::getSettingValue($this->_properties[$fieldName]['imageMaxWidthSetting']);
				$maxHeight = (int) scSettings::getSettingValue($this->_properties[$fieldName]['imageMaxHeightSetting']);
				scImageHandler::resizeImage($tempImageFilePath, $imageFilePath, $maxWidth, $maxHeight);
			} else {
				// No width and height - just move the image
				if (@rename($tempImageFilePath, $imageFilePath) === false) {
					scPaths::checkFileWritable($imageFilePath, true);
					throw new miException(sprintf(_t('Image copying from "%s" to "%s" failed.'), $tempImageFilePath, $imageFilePath));
				}
			}
			
			if (isset($this->_properties[$fieldName]['imageWidthField']) and isset($this->_properties[$fieldName]['imageHeightField'])) {
				list($width, $height) = scImageHandler::getImageSize($imageFilePath);
				$viewMapper = $this->getViewMapperController()->getViewMapper();
				$viewMapper->set($this->_properties[$fieldName]['imageWidthField'], $width);
				$viewMapper->set($this->_properties[$fieldName]['imageHeightField'], $height);
				$viewMapper->update();
			}
		}
	}
?>