<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scReportColumnsFormatter {
		static protected $_reportFormatMethods = array(
			scReportDefinitions::REPORT_FIELD_SALES => 'formatColumnMoney',
			scReportDefinitions::REPORT_FIELD_ORDER_SHIPPING_COST => 'formatColumnMoney',
			scReportDefinitions::REPORT_FIELD_ORDER_PAYMENT_COST => 'formatColumnMoney',
			scReportDefinitions::REPORT_FIELD_ORDER_AVERAGE_AMOUNT => 'formatColumnMoney',
			scReportDefinitions::REPORT_FIELD_TAXES => 'formatColumnMoney',
			scReportDefinitions::REPORT_FIELD_PRODUCT_SALES => 'formatColumnMoney',
			
			scReportDefinitions::REPORT_FIELD_ORDER_STATUS => 'formatColumnOrderStatus',
			scReportDefinitions::REPORT_FIELD_CUSTOMER_COUNTRY => 'formatColumnCustomerCountry',
			scReportDefinitions::REPORT_FIELD_ORDER_SHIPPING_METHOD => 'formatColumnShippingMethod',
			scReportDefinitions::REPORT_FIELD_ORDER_PAYMENT_METHOD => 'formatColumnPaymentMethod',
		);
		
		static protected $_reportColumnClasses = array(
			scReportDefinitions::REPORT_FIELD_SALES => 'report_money',
			scReportDefinitions::REPORT_FIELD_NUMBER_OF_ORDERS => 'report_number',
			scReportDefinitions::REPORT_FIELD_ORDERED_ITEM_NAME => '',
			scReportDefinitions::REPORT_FIELD_ORDERED_ITEM_CATEGORY_NAME => '',
			scReportDefinitions::REPORT_FIELD_ORDER_CUSTOMER_NAME => '',
			scReportDefinitions::REPORT_FIELD_ORDER_CUSTOMER_COUNTRY => '',
			scReportDefinitions::REPORT_FIELD_ORDER_STATUS => '',
			scReportDefinitions::REPORT_FIELD_ORDER_SHIPPING_METHOD => '',
			scReportDefinitions::REPORT_FIELD_ORDER_SHIPPING_COST => 'report_money',
			scReportDefinitions::REPORT_FIELD_ORDER_PAYMENT_METHOD => '',
			scReportDefinitions::REPORT_FIELD_ORDER_PAYMENT_COST => 'report_money',
			scReportDefinitions::REPORT_FIELD_CUSTOMER_COUNTRY => '',
			scReportDefinitions::REPORT_FIELD_CUSTOMER_NAME => '',
			scReportDefinitions::REPORT_FIELD_ORDER_NUM_ITEMS => 'report_number',
			scReportDefinitions::REPORT_FIELD_ORDER_AVERAGE_AMOUNT => 'report_money',
			scReportDefinitions::REPORT_FIELD_TAXES => 'report_money',
			scReportDefinitions::REPORT_FIELD_PRODUCT_SALES => 'report_money',
		);
		
		static protected $_reportNumericColumns = array(
			scReportDefinitions::REPORT_FIELD_SALES => true,
			scReportDefinitions::REPORT_FIELD_NUMBER_OF_ORDERS => true,
			scReportDefinitions::REPORT_FIELD_ORDER_SHIPPING_COST => true,
			scReportDefinitions::REPORT_FIELD_ORDER_PAYMENT_COST => true,
			scReportDefinitions::REPORT_FIELD_ORDER_NUM_ITEMS => true,
			scReportDefinitions::REPORT_FIELD_ORDER_AVERAGE_AMOUNT => true,
			scReportDefinitions::REPORT_FIELD_TAXES => true,
			scReportDefinitions::REPORT_FIELD_PRODUCT_SALES => true,
			scReportDefinitions::REPORT_FIELD_COUNT_CUSTOMERS_IN_COUNTRY => true,
		);
		
		static protected $_moneyColumns = array(
			scReportDefinitions::REPORT_FIELD_SALES,
			scReportDefinitions::REPORT_FIELD_ORDER_SHIPPING_COST,
			scReportDefinitions::REPORT_FIELD_ORDER_PAYMENT_COST,
			scReportDefinitions::REPORT_FIELD_ORDER_AVERAGE_AMOUNT,
			scReportDefinitions::REPORT_FIELD_TAXES,
			scReportDefinitions::REPORT_FIELD_PRODUCT_SALES
		);
		
		/**
		 * Returns if the columns value is of type money
		 *
		 * @param int $column
		 * @return boolean
		 */
		public static function isMoneyColumn($column)
		{
			return in_array($column, self::$_moneyColumns);
		}
		
		public function getFormattedColumns($rows, $dataColumns)
		{
			$formatFuncs = array(
				'T' => array($this, 'formatColumnT')
			);
			
			$columnFieldIds = array();
			foreach ($dataColumns as $dataColumns)
				$columnFieldIds[substr($dataColumns, 1)] = $dataColumns;
			
			$result = array();
			foreach ($rows as $row) {
				$data = array();
				foreach ($columnFieldIds as $fieldId => $dataColumn) {
					
					if (isset(self::$_reportFormatMethods[$fieldId])) {
						$data[$dataColumn] = call_user_func(array($this, self::$_reportFormatMethods[$fieldId]), $row[$dataColumn]);
					} elseif ($dataColumn == 'T')
						$data[$dataColumn] = $this->formatColumnT($row[$dataColumn]);
					else
						$data[$dataColumn] = $row[$dataColumn];
				}
				$result[] = $data;
			}
			
			return $result;
		}
		
		/**
		 * Returns the column classes for each of data rows
		 * 
		 * @param array
		 * @return array
		 */
		static public function getColumnClasses($dataColumns)
		{
			$columnClasses = array();
			foreach (array_keys($dataColumns) as $dataColumn) {
				$fieldId = substr($dataColumn, 1);
				if (isset(self::$_reportColumnClasses[$fieldId]))
					$columnClasses[$dataColumn] = self::$_reportColumnClasses[$fieldId];
			}
			return $columnClasses;
		}
		
		/**
		 * Returns array with the numeric columns
		 */
		static public function getNumericColumns($dataColumns)
		{
			$numericColumns = array();
			foreach (array_keys($dataColumns) as $dataColumn) {
				$fieldId = substr($dataColumn, 1);
				if (isset(self::$_reportNumericColumns[$fieldId]))
					$numericColumns[$dataColumn] = $dataColumn;
			}
			return $numericColumns;
		}
		
		protected function formatColumnMoney($fieldValue)
		{
			return number_format($fieldValue, 2, '.', '');
		}
		
		protected function formatColumnOrderStatus($fieldValue)
		{
			return scOrderStatuses::singleton()->getOrderStatusName($fieldValue);
		}
		
		protected function formatColumnCustomerCountry($fieldValue)
		{
			return scCountry::getCountryName($fieldValue);
		}
		
		protected function formatColumnShippingMethod($fieldValue)
		{
			list($shippingModuleId, $methodId) = explode('-', $fieldValue);
			
			// TODO: V1.1 Performance improvement
			$moduleId = scUtil::getDbValue('ShippingModules', 'ModuleID', 'ShippingModuleID', $shippingModuleId);
			if (empty($moduleId))
				return sprintf(_t('Unknown (%s)'), $fieldValue);
			
			$className = scModulesFactory::getModuleClassName($moduleId, 'IShippingModule');
			$shippingModule =  new $className($moduleId, $shippingModuleId);
			
			$shippingMethods = $shippingModule->getShippingMethods();
			
			if (isset($shippingMethods[$methodId]))
				return scInterfaceTexts::getTranslatedText($shippingMethods[$methodId]);
			
			return _t('Unknown');
		}
		
		protected function formatColumnPaymentMethod($fieldValue)
		{
			// TODO: V1.1 Performance improvement
			$moduleName = scUtil::getDbValue('PaymentModules', 'PaymentModuleName', 'PaymentModuleID', $fieldValue);
			if ($moduleName)
				return $moduleName;
			else
				return _t('Payment Module ') . $fieldValue;
		}
		
		static public function formatColumnT($value)
		{
			$formatFuncs = array(
				scReportUnitOfTime::UNIT_OF_TIME_DAILY => array('self', 'formatColumnTimeDay'),
				scReportUnitOfTime::UNIT_OF_TIME_WEEKLY => array('self', 'formatColumnTimeWeek'),
				scReportUnitOfTime::UNIT_OF_TIME_MONTHLY => array('self', 'formatColumnTimeMonth'),
				scReportUnitOfTime::UNIT_OF_TIME_QUARTERLY => array('self', 'formatColumnTimeQuarter'),
				scReportUnitOfTime::UNIT_OF_TIME_YEARLY => false,
			);
			if ($formatFuncs[scReportUnitOfTime::getUnitOfTime()]) {
				return call_user_func($formatFuncs[scReportUnitOfTime::getUnitOfTime()], $value);
			}
			return $value;
		}
		
		static function formatColumnTimeDay($value)
		{
			// 86400 seconds = 1 day
			return gmdate('d-M-Y', $value * 86400);
		}
		
		static function formatColumnTimeWeek($value)
		{
			// 604800 seconds = 7 days (1 week)
			// 518400 seconds = 6 days
			return gmdate('d-M-Y', $value*604800) . ' to ' . gmdate('d-M-Y', $value*604800 + 518400);
		}
		
		static function formatColumnTimeMonth($value)
		{
			return gmdate('M-Y', gmmktime(0,0,0,$value%100,1,$value/100));
		}
		
		static function formatColumnTimeQuarter($value)
		{
			return intval($value/10) . ' Q' . ($value%10);
		}
	}
?>