<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scReportDefinitions {
		const REPORT_TYPE_ORDER_CUSTOMERS = 1;
		const REPORT_TYPE_ORDER_PRODUCTS = 2;
		const REPORT_TYPE_ORDER_CATEGORIES = 3;
		const REPORT_TYPE_ORDER_CUSTOMER_COUNTRIES = 4;
		const REPORT_TYPE_ORDER_SHIPPING_METHODS = 5;
		const REPORT_TYPE_ORDER_PAYMENT_METHODS = 6;
		const REPORT_TYPE_ORDER_STATUSES = 7;
		const REPORT_TYPE_ORDER_TAXES = 8;
		const REPORT_TYPE_ORDER_ITEMS = 9;
		const REPORT_TYPE_CUSTOMER_COUNTRIES = 10;
		const REPORT_TYPE_ORDER = 11;
		const REPORT_TYPE_PRODUCT_VIEWS = 12;
		const REPORT_TYPE_CATEGORY_VIEWS = 13;
		const REPORT_TYPE_DAILY_VIEWS = 14;
		
		const REPORT_INFO_MAIN_TABLE = 'table';
		
		const REPORT_INFO_JOINS = 'joins';
		const REPORT_INFO_JOIN_TYPE = 0;
		const REPORT_INFO_JOIN_TABLE = 1;
		const REPORT_INFO_JOIN_CONDITION = 2;
		
		const REPORT_INFO_GROUP_BY_FIELD = 'groupByField';
		const REPORT_INFO_TIMESTAMP_FIELD = 'timestampField';
		const REPORT_INFO_SORT_FIELD = 'sortField';
		const REPORT_INFO_SORT_DIRECTION = 'sortDirection';
		const REPORT_INFO_FIELDS = 'fields';
		const REPORT_INFO_DERIVED_JOIN_FIELD = 'derivedJoinField';
		const REPORT_INFO_CURRENCY_CODE = 'currencyCode';
		const REPORT_INFO_ORDERSTATUS_FIELD  = 'orderStatus';
		
		const REPORT_FIELD_SALES = 1;
		const REPORT_FIELD_NUMBER_OF_ORDERS = 2;
		const REPORT_FIELD_ORDERED_ITEM_NAME = 3;
		const REPORT_FIELD_ORDERED_ITEM_CATEGORY_NAME = 4;
		const REPORT_FIELD_ORDER_CUSTOMER_NAME = 5;
		const REPORT_FIELD_ORDER_CUSTOMER_COUNTRY = 6;
		const REPORT_FIELD_ORDER_STATUS = 7;
		const REPORT_FIELD_ORDER_SHIPPING_METHOD = 8;
		const REPORT_FIELD_ORDER_SHIPPING_COST = 9;
		const REPORT_FIELD_ORDER_PAYMENT_METHOD = 10;
		const REPORT_FIELD_ORDER_PAYMENT_COST = 11;
		const REPORT_FIELD_CUSTOMER_COUNTRY = 12;
		const REPORT_FIELD_CUSTOMER_NAME = 13;
		const REPORT_FIELD_ORDER_NUM_ITEMS = 14;
		const REPORT_FIELD_ORDER_AVERAGE_AMOUNT = 15;
		const REPORT_FIELD_TAXES = 16;
		const REPORT_FIELD_PRODUCT_SALES = 17;
		const REPORT_FIELD_COUNT_CUSTOMERS_IN_COUNTRY = 18;
		const REPORT_FIELD_PRODUCT_NAME = 19;
		const REPORT_FIELD_PRODUCT_VIEWS = 20;
		const REPORT_FIELD_CATEGORY_NAME = 21;
		const REPORT_FIELD_CATEGORY_VIEWS = 22;
		const REPORT_FIELD_DAILY_VALUE = 23;
		const REPORT_FIELD_DAILY_DISPLAY = 24;
		
		/**
		 * Definition of each of the available report types
		 */
		static protected $_reportInfos = array(
			self::REPORT_TYPE_ORDER_CUSTOMERS => array(
				self::REPORT_INFO_MAIN_TABLE => 'OrderItems',
				self::REPORT_INFO_JOINS => 'JOIN Orders ON (Orders.OrderID = OrderItems.OrderID)',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_ORDER_CUSTOMER_NAME,
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_PRODUCT_SALES,
				self::REPORT_INFO_SORT_DIRECTION => 'DESC',
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'CONCAT(OrderBillingFirstName , " ", OrderBillingLastName)',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_PRODUCT_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS,
				),
			),
			self::REPORT_TYPE_ORDER_PRODUCTS => array(
				self::REPORT_INFO_MAIN_TABLE => 'OrderItems',
				self::REPORT_INFO_JOINS => 'JOIN Orders ON (Orders.OrderID = OrderItems.OrderID)',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_ORDERED_ITEM_NAME,
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_PRODUCT_SALES,
				self::REPORT_INFO_SORT_DIRECTION => 'DESC',
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'OrderItems.OrderItemProductName',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_PRODUCT_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS,
				),
			),
			self::REPORT_TYPE_ORDER_CATEGORIES => array(
				self::REPORT_INFO_MAIN_TABLE => 'OrderItems',
				self::REPORT_INFO_JOINS => 'JOIN Orders ON (Orders.OrderID = OrderItems.OrderID)',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_ORDERED_ITEM_CATEGORY_NAME,
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_PRODUCT_SALES,
				self::REPORT_INFO_SORT_DIRECTION => 'DESC',
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'OrderItems.OrderItemCategoryName',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_PRODUCT_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS,
				),
			),
			self::REPORT_TYPE_ORDER_CUSTOMER_COUNTRIES => array(
				self::REPORT_INFO_MAIN_TABLE => 'OrderItems',
				self::REPORT_INFO_JOINS => 'JOIN Orders ON (Orders.OrderID = OrderItems.OrderID)',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_ORDER_CUSTOMER_COUNTRY,
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_PRODUCT_SALES,
				self::REPORT_INFO_SORT_DIRECTION => 'DESC',
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'Orders.OrderBillingCountry',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_PRODUCT_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS,
				),
			),
			self::REPORT_TYPE_ORDER_SHIPPING_METHODS => array(
				self::REPORT_INFO_MAIN_TABLE => 'Orders',
				// IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING = 2
				self::REPORT_INFO_JOINS => ' JOIN (SELECT OrderID,OrderTotalLineAmount FROM OrderTotalLines WHERE OrderTotalLineType=2) AS ta USING(OrderID)',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_ORDER_SHIPPING_METHOD,
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_SALES,
				self::REPORT_INFO_SORT_DIRECTION => 'DESC',
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'Orders.OrderShippingMethodCode',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS, self::REPORT_FIELD_ORDER_SHIPPING_COST
				),
			),
			self::REPORT_TYPE_ORDER_PAYMENT_METHODS => array(
				self::REPORT_INFO_MAIN_TABLE => 'Orders',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_ORDER_PAYMENT_METHOD,
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_SALES,
				self::REPORT_INFO_SORT_DIRECTION => 'DESC',
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'Orders.PaymentModuleID',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS //, self::REPORT_FIELD_ORDER_PAYMENT_COST
				),
			),
			self::REPORT_TYPE_ORDER_STATUSES => array(
				self::REPORT_INFO_MAIN_TABLE => 'Orders',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_ORDER_STATUS,
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_ORDER_STATUS,
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'Orders.OrderStatus',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS,
				),
			),
			self::REPORT_TYPE_ORDER_ITEMS => array(
				self::REPORT_INFO_MAIN_TABLE => 'Orders',
				self::REPORT_INFO_JOINS => 'JOIN (SELECT Orders.OrderID,COUNT(Orders.OrderID) AS NumItems FROM Orders JOIN OrderItems USING(OrderID) GROUP BY OrderItems.OrderID) AS X ON X.OrderID=Orders.OrderID',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_ORDER_NUM_ITEMS,
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_ORDER_NUM_ITEMS,
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'X.NumItems',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_NUMBER_OF_ORDERS,
				),
			),
			self::REPORT_TYPE_CUSTOMER_COUNTRIES => array(
				self::REPORT_INFO_MAIN_TABLE => 'Customers',
				self::REPORT_INFO_JOINS => 'JOIN CustomerAddresses ON CustomerAddresses.CustomerAddressIsDefaultBilling = 1 AND CustomerAddresses.CustomerID = Customers.CustomerID',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_CUSTOMER_COUNTRY,
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_CUSTOMER_COUNTRY,
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_COUNT_CUSTOMERS_IN_COUNTRY,
				),
			),
			self::REPORT_TYPE_ORDER_TAXES => array(
				self::REPORT_INFO_MAIN_TABLE => 'Orders',
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_TAXES, self::REPORT_FIELD_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS
				),
			),
			self::REPORT_TYPE_ORDER => array(
				self::REPORT_INFO_MAIN_TABLE => 'Orders',
				self::REPORT_INFO_TIMESTAMP_FIELD => 'OrderTimestamp',
				self::REPORT_INFO_ORDERSTATUS_FIELD => 'OrderStatus',
				self::REPORT_INFO_CURRENCY_CODE => 'OrderCurrencyCode',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_ORDER_AVERAGE_AMOUNT, self::REPORT_FIELD_SALES, self::REPORT_FIELD_NUMBER_OF_ORDERS
				),
			),
			self::REPORT_TYPE_PRODUCT_VIEWS => array(
				self::REPORT_INFO_MAIN_TABLE => 'Products',
				self::REPORT_INFO_JOINS => 'JOIN StatProductViews USING(ProductID) JOIN ProductsML ON Products.ProductID = ProductsML.ProductID AND LanguageID = :LanguageID',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_PRODUCT_NAME,
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_PRODUCT_VIEWS,
				self::REPORT_INFO_SORT_DIRECTION => 'DESC',
				self::REPORT_INFO_TIMESTAMP_FIELD => '(UNIX_TIMESTAMP(StatProductViewsDate) + 86400)', // the end of the day
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'ProductsML.ProductName',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_PRODUCT_VIEWS
				),
			),
			self::REPORT_TYPE_CATEGORY_VIEWS => array(
				self::REPORT_INFO_MAIN_TABLE => 'Categories',
				self::REPORT_INFO_JOINS => 'JOIN StatCategoryViews USING(CategoryID) JOIN CategoriesML ON Categories.CategoryID = CategoriesML.CategoryID AND LanguageID = :LanguageID',
				self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_CATEGORY_NAME,
				self::REPORT_INFO_SORT_FIELD => self::REPORT_FIELD_CATEGORY_VIEWS,
				self::REPORT_INFO_SORT_DIRECTION => 'DESC',
				self::REPORT_INFO_TIMESTAMP_FIELD => '(UNIX_TIMESTAMP(StatCategoryViewsDate) + 86400)', // the end of the day
				self::REPORT_INFO_DERIVED_JOIN_FIELD => 'CategoriesML.CategoryName',
				self::REPORT_INFO_FIELDS => array(
					self::REPORT_FIELD_CATEGORY_VIEWS
				),
			),
		);
		
		/**
		 * The names of all report fields
		 */
		static protected $_fieldNames = null;
		
		/**
		 * The SQL to select each of the report fields
		 */
		static protected $_fieldSqls = null;
		
		/**
		 * The predefined reports
		 */
		static protected $_reportDefinitions = null;
		
		/**
		 * Return the report definition for the requested report
		 */
		static public function getReportDefinition($reportId)
		{
			if (self::$_reportDefinitions == null) {
				self::$_reportDefinitions = array(
					'ViewsVisits' => array(
						'name' => _t('Page Views / Unique Visits'),
						'type' => self::REPORT_TYPE_DAILY_VIEWS
					),
					'ViewsByProduct' => array(
						'name' => _t('Views By Product'),
						'type' => self::REPORT_TYPE_PRODUCT_VIEWS
					),
					'ViewsByCategory' => array(
						'name' => _t('Views By Category'),
						'type' => self::REPORT_TYPE_CATEGORY_VIEWS
					),
					'SalesByProduct' => array(
						'name' => _t('Sales By Product'),
						'type' => self::REPORT_TYPE_ORDER_PRODUCTS
					),
					'SalesByCategory' => array(
						'name' => _t('Sales By Category'),
						'type' => self::REPORT_TYPE_ORDER_CATEGORIES
					),
					'SalesByCustomer' => array(
						'name' => _t('Sales By Customer'),
						'type' => self::REPORT_TYPE_ORDER_CUSTOMERS
					),
					'SalesByCountry' => array(
						'name' => _t('Sales By Country'),
						'type' => self::REPORT_TYPE_ORDER_CUSTOMER_COUNTRIES
					),
					'CustomerCountries' => array(
						'name' => _t('Customer Countries'),
						'type' => self::REPORT_TYPE_CUSTOMER_COUNTRIES
					),
					'AverageOrderAmount' => array(
						'name' => _t('Average Order Amount'),
						'type' => self::REPORT_TYPE_ORDER
					),
					'OrderStatuses' => array(
						'name' => _t('Order Statuses'),
						'type' => self::REPORT_TYPE_ORDER_STATUSES
					),
					'OrderShippingMethods' => array(
						'name' => _t('Shipping Methods'),
						'type' => self::REPORT_TYPE_ORDER_SHIPPING_METHODS
					),
					'OrderPaymentMethods' => array(
						'name' => _t('Payment Methods'),
						'type' => self::REPORT_TYPE_ORDER_PAYMENT_METHODS
					),
					'ItemsPerOrder' => array(
						'name' => _t('Items Per Order'),
						'type' => self::REPORT_TYPE_ORDER_ITEMS
					),
					'OrderTaxes' => array(
						'name' => _t('Taxes'),
						'type' => self::REPORT_TYPE_ORDER_TAXES
					)
				);
			}
			return self::$_reportDefinitions[$reportId];
		}
		
		static public function getReportInfo($reportDefinition)
		{
			if (!isset(self::$_reportInfos[self::REPORT_TYPE_DAILY_VIEWS])) {
				$sql = 'SELECT ? AS DailyStatsType, StatViewsVisitsDate AS DailyStatsDate, StatViewsCount AS DailyStatsValue FROM StatViewsVisits' .
					' UNION ' .
					'SELECT ?, StatViewsVisitsDate, StatVisitsCount FROM StatViewsVisits';
				$sql = miStaticDBUtil::getParsedQuery($sql, array(_t('Page Views'), _t('Unique Visitors')));
				self::$_reportInfos[self::REPORT_TYPE_DAILY_VIEWS] = array(
					self::REPORT_INFO_MAIN_TABLE => '(' . $sql . ') AS MainTable',
					self::REPORT_INFO_GROUP_BY_FIELD => self::REPORT_FIELD_DAILY_DISPLAY,
					self::REPORT_INFO_TIMESTAMP_FIELD => '(UNIX_TIMESTAMP(DailyStatsDate) + 86400)', // the end of the day
					self::REPORT_INFO_DERIVED_JOIN_FIELD => 'DailyStatsType',
					self::REPORT_INFO_FIELDS => array(
						self::REPORT_FIELD_DAILY_VALUE
					),
				);
			}
			return self::$_reportInfos[$reportDefinition['type']];
		}
		
		static public function getFieldSql($fieldId)
		{
			if (is_null(self::$_fieldSqls)) {
				// Initialize the fields sqls
				self::$_fieldSqls = array(
					self::REPORT_FIELD_SALES => 'SUM(OrderTotal)',
					self::REPORT_FIELD_NUMBER_OF_ORDERS => 'COUNT(Orders.OrderID)',
					self::REPORT_FIELD_ORDERED_ITEM_NAME => 'OrderItemProductName',
					self::REPORT_FIELD_ORDERED_ITEM_CATEGORY_NAME => 'OrderItemCategoryName',
					self::REPORT_FIELD_ORDER_CUSTOMER_NAME => 'CONCAT(OrderBillingFirstName , " ", OrderBillingLastName)',
					self::REPORT_FIELD_ORDER_CUSTOMER_COUNTRY => 'OrderBillingCountry',
					self::REPORT_FIELD_ORDER_STATUS => 'OrderStatus',
					self::REPORT_FIELD_ORDER_SHIPPING_METHOD => 'CONCAT(ShippingModuleID,"-",OrderShippingMethodCode)',
					self::REPORT_FIELD_ORDER_PAYMENT_METHOD => 'PaymentModuleID',
					self::REPORT_FIELD_ORDER_SHIPPING_COST => 'SUM(OrderTotalLineAmount)',
					self::REPORT_FIELD_CUSTOMER_COUNTRY => 'CountryID',
					self::REPORT_FIELD_CUSTOMER_NAME => 'CONCAT(CustomerFirstName , " ", CustomerLastName)',
					self::REPORT_FIELD_ORDER_NUM_ITEMS => 'NumItems',
					self::REPORT_FIELD_ORDER_AVERAGE_AMOUNT => 'SUM(OrderTotal) / COUNT(Orders.OrderID)',
					self::REPORT_FIELD_TAXES => 'SUM((SELECT SUM(OrderTotalLineAmount) FROM OrderTotalLines WHERE OrderTotalLines.OrderID=Orders.OrderID AND OrderTotalLineType=' . IOrderTotalModule::ORDER_TOTAL_TYPE_TAX . '))',
					self::REPORT_FIELD_PRODUCT_SALES => 'SUM(OrderItemTotal)',
					self::REPORT_FIELD_COUNT_CUSTOMERS_IN_COUNTRY => 'COUNT(Customers.CustomerID)',
					self::REPORT_FIELD_PRODUCT_NAME => 'ProductName',
					self::REPORT_FIELD_PRODUCT_VIEWS => 'SUM(StatProductViews.StatProductViewsCount)',
					self::REPORT_FIELD_CATEGORY_NAME => 'CategoryName',
					self::REPORT_FIELD_CATEGORY_VIEWS => 'SUM(StatCategoryViews.StatCategoryViewsCount)',
					self::REPORT_FIELD_DAILY_VALUE => 'SUM(DailyStatsValue)',
					self::REPORT_FIELD_DAILY_DISPLAY => 'DailyStatsType',
				);
			}
			
			return self::$_fieldSqls[$fieldId];
		}
		
		static public function getFieldName($fieldId)
		{
			if (self::$_fieldNames == null) {
				self::$_fieldNames = array(
					self::REPORT_FIELD_SALES => _t('Sales'),
					self::REPORT_FIELD_NUMBER_OF_ORDERS => _t('Number of orders'),
					self::REPORT_FIELD_ORDERED_ITEM_NAME => _t('Product name'),
					self::REPORT_FIELD_ORDERED_ITEM_CATEGORY_NAME => _t('Category name'),
					self::REPORT_FIELD_ORDER_CUSTOMER_NAME => _t('Customer name'),
					self::REPORT_FIELD_ORDER_CUSTOMER_COUNTRY => _t('Customer country'),
					self::REPORT_FIELD_ORDER_STATUS => _t('Order status'),
					self::REPORT_FIELD_ORDER_SHIPPING_METHOD => _t('Shipping method'),
					self::REPORT_FIELD_ORDER_SHIPPING_COST => _t('Shipping costs'),
					self::REPORT_FIELD_ORDER_PAYMENT_METHOD => _t('Payment method'),
//					self::REPORT_FIELD_ORDER_PAYMENT_COST => _t('Payment costs'),
					self::REPORT_FIELD_CUSTOMER_COUNTRY => _t('Customer country'),
					self::REPORT_FIELD_CUSTOMER_NAME => _t('Customer name'),
					self::REPORT_FIELD_ORDER_NUM_ITEMS => _t('Number of items'),
					self::REPORT_FIELD_ORDER_AVERAGE_AMOUNT => _t('Average Order Amount'),
					self::REPORT_FIELD_TAXES => _t('Tax'),
					self::REPORT_FIELD_PRODUCT_SALES => _t('Product sales'),
					self::REPORT_FIELD_COUNT_CUSTOMERS_IN_COUNTRY => _t('Customers'),
					self::REPORT_FIELD_PRODUCT_NAME => _t('Product Name'),
					self::REPORT_FIELD_PRODUCT_VIEWS => _t('Product views'),
					self::REPORT_FIELD_CATEGORY_NAME => _t('Category Name'),
					self::REPORT_FIELD_CATEGORY_VIEWS => _t('Category views'),
					self::REPORT_FIELD_DAILY_VALUE => _t('Page Views / Unique Visits'),
					self::REPORT_FIELD_DAILY_DISPLAY => _t('Page Views / Unique Visits'),
				);
			}
			
			return self::$_fieldNames[$fieldId];
		}
		
		/**
		 * Returns the column names
		 * 
		 * @return array the column names
		 */
		static public function getColumnNames($reportDefinition)
		{
			$reportInfo = self::getReportInfo($reportDefinition);
			$columnNames = array();
			if (isset($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]))
				$columnNames[self::getFieldSqlName($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD])] = self::getFieldName($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]);
			if (isset($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD]) and !scReportUnitOfTime::isUnitOfTimeSummary())
				$columnNames['T'] = scReportUnitOfTime::getUnitOfTimeDisplayColumnName();
			foreach ($reportInfo[self::REPORT_INFO_FIELDS] as $reportField) {
				$columnNames[self::getFieldSqlName($reportField)] = self::getFieldName($reportField);
			}
			return $columnNames;
		}
		
		/**
		 * Builds the report query
		 * 
		 * @param array $reportDefinition
		 * @return string sql query
		 */
		public static function getReportQuery($reportDefinition, $numberOfRecords)
		{
			$reportInfo = self::getReportInfo($reportDefinition);
			
			// Build the unit of time sql
			if (isset($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD]) and
				(scReportUnitOfTime::getUnitOfTime() != scReportUnitOfTime::UNIT_OF_TIME_SUMMARY))
			{
				$unitOfTimeValueSql = scReportUnitOfTime::getUnitOfTimeSql($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD]) . ' AS T';
				$unitOfTimeGroupSql = 'T';
			} else
				$unitOfTimeValueSql = $unitOfTimeGroupSql = '';
			
			// Build the report value
			$value = array();
			if (isset($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]))
				$value[$reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]] = self::getFieldSql($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]) . ' AS ' . self::getFieldSqlName($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]);
			foreach ($reportInfo[self::REPORT_INFO_FIELDS] as $reportField) {
				$value[$reportField] = self::getFieldSql($reportField) . ' AS ' . self::getFieldSqlName($reportField);
			}
			
			// Combine the final query
			$sql = 'SELECT ' . implode(',', $value) . $unitOfTimeValueSql . ' FROM ' . $reportInfo[self::REPORT_INFO_MAIN_TABLE];
			
			// Build the join sql
			$joinSql = isset($reportInfo[self::REPORT_INFO_JOINS]) ? ' ' . $reportInfo[self::REPORT_INFO_JOINS] : '';
			$sql .= ' ' . $joinSql;
			
			if (($numberOfRecords > 0) and $unitOfTimeGroupSql and isset($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD])) {
				$sql .= ' JOIN (' . self::getDerivedJoinSubquery($reportInfo, $numberOfRecords) . ') AS data';
				$sql .= ' ON data.' . self::getFieldSqlName($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]);
				$sql .= '=' . $reportInfo[self::REPORT_INFO_DERIVED_JOIN_FIELD];
				//$sql .= '=' . $reportInfo[self::REPORT_INFO_MAIN_TABLE] . '.' . self::getFieldSql($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]);
				
				// Don't limit below
				$numberOfRecords = 0;
			}
			
			// Where: Add time limit and currency selection
			$whereSql = array();
			
			if (isset($reportInfo[self::REPORT_INFO_ORDERSTATUS_FIELD])) {
				$whereSql[] = self::getOrderStatusesFilterSql($reportInfo[self::REPORT_INFO_ORDERSTATUS_FIELD]);
			}
			
			if (isset($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD])) {
				$timeSql = self::getTimeSelectorFilterSql($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD]);
				if ($timeSql)
					$whereSql[] = $timeSql; 
			}
			if (isset($reportInfo[self::REPORT_INFO_CURRENCY_CODE]))
				$whereSql[] = $reportInfo[self::REPORT_INFO_CURRENCY_CODE] . '="' . sql_escape_string(self::getCurrencyCode($reportDefinition)) . '"';
			if (count($whereSql))
				$sql .= ' WHERE ' . implode(' AND ', $whereSql);
			
			// Group
			$groupBy = array();
			if (isset($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]))
				$groupBy[] = self::getFieldSqlName($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]);
			if ($unitOfTimeGroupSql)
				$groupBy[] = $unitOfTimeGroupSql;
			if (count($groupBy))
				$sql .= ' GROUP BY ' . implode(',', $groupBy);
			
			// Order
			$orderBy = array();
			if ($unitOfTimeGroupSql)
				$orderBy[] = $unitOfTimeGroupSql;
			if (isset($reportInfo[self::REPORT_INFO_SORT_FIELD])) {
				$sortField = self::getFieldSqlName($reportInfo[self::REPORT_INFO_SORT_FIELD]);
				if (isset($reportInfo[self::REPORT_INFO_SORT_DIRECTION]))
					$sortField .= ' ' . $reportInfo[self::REPORT_INFO_SORT_DIRECTION];
				$orderBy[] = $sortField;
			}
			if (count($orderBy))
				$sql .= ' ORDER BY ' . implode(',', $orderBy);
			
			if ($numberOfRecords > 0)
				$sql .= ' LIMIT ' . $numberOfRecords;
			
			// Replace named parameters
			$sql = str_replace(':LanguageID', scMultiLanguage::getCurrentLanguageId(), $sql);
			
			return $sql;
		}
		
		/**
		 * Returns a subquery which selects the top $numberOfRecords items
		 */
		static protected function getDerivedJoinSubquery($reportInfo, $numberOfRecords)
		{
			$sql = '';
			
			$value = array();
			if (isset($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]))
				$value[$reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]] = self::getFieldSql($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]) . ' AS ' . self::getFieldSqlName($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]);
			if (isset($reportInfo[self::REPORT_INFO_SORT_FIELD]))
				$value[$reportInfo[self::REPORT_INFO_SORT_FIELD]] = self::getFieldSql($reportInfo[self::REPORT_INFO_SORT_FIELD]) . ' AS ' . self::getFieldSqlName($reportInfo[self::REPORT_INFO_SORT_FIELD]);
			
			$sql = 'SELECT ' . implode(',', $value) . ' FROM ' . $reportInfo[self::REPORT_INFO_MAIN_TABLE];
			
			$joinSql = isset($reportInfo[self::REPORT_INFO_JOINS]) ? ' ' . $reportInfo[self::REPORT_INFO_JOINS] : '';
			$sql .= ' ' . $joinSql;
			
			if (isset($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD])) {
				$timeFilterSql = self::getTimeSelectorFilterSql($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD]);
				if ($timeFilterSql != '')
					$sql .= ' WHERE ' . $timeFilterSql;
			}
			
			if (isset($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]))
				$sql .= ' GROUP BY ' . self::getFieldSqlName($reportInfo[self::REPORT_INFO_GROUP_BY_FIELD]);
			
			if (isset($reportInfo[self::REPORT_INFO_SORT_FIELD])) {
				$sortField = self::getFieldSqlName($reportInfo[self::REPORT_INFO_SORT_FIELD]);
				if (isset($reportInfo[self::REPORT_INFO_SORT_DIRECTION]))
					$sortField .= ' ' . $reportInfo[self::REPORT_INFO_SORT_DIRECTION];
				$sql .= ' ORDER BY ' . $sortField;
			}
			
			if ($numberOfRecords > 0)
				$sql .= ' LIMIT ' . $numberOfRecords;
			
			return $sql;
		}
		
		/**
		 * Return the currencies in the current report selection
		 */
		static public function getCurrencies($reportDefinition)
		{
			$reportInfo = self::getReportInfo($reportDefinition);
			
			// Check if we have currency field
			if (!isset($reportInfo[self::REPORT_INFO_CURRENCY_CODE]))
				return array();
			
			$sql = 'SELECT ' . $reportInfo[self::REPORT_INFO_CURRENCY_CODE] . ' AS CurrencyCode FROM ' . $reportInfo[self::REPORT_INFO_MAIN_TABLE];
			$sql .= isset($reportInfo[self::REPORT_INFO_JOINS]) ? ' ' . $reportInfo[self::REPORT_INFO_JOINS] : '';
			
			if (isset($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD])) {
				$timeSql = self::getTimeSelectorFilterSql($reportInfo[self::REPORT_INFO_TIMESTAMP_FIELD]);
				if ($timeSql)
					$sql .= ' WHERE ' . $timeSql;
			}
			$sql .= ' GROUP BY CurrencyCode';
			return miUtil::getDBArray($sql, 'CurrencyCode', 'CurrencyCode');
		}
		
		/**
		 * Returns the currency code to use in the report
		 * 
		 * @param array $reportDefinition
		 * @return string Currency Code
		 */
		public static function getCurrencyCode($reportDefinition)
		{
			$currencyCode = miGetParamDefault('CurrencyCode', '');
			if ($currencyCode != '')
				return $currencyCode;
			
			// Currency code is not passed, return the first found currency
			$currencies = self::getReportCurrencies($reportDefinition);
			if (empty($currencies))
				return '';
			
			return reset($currencies);
		}
		
		public static function getReportCurrencies($reportDefinition)
		{
			static $_currencies = null;
			if (is_null($_currencies)) {
				$_currencies = scReportDefinitions::getCurrencies($reportDefinition);
				$_currencies = array_filter($_currencies);
			}
			return $_currencies;
		}
		
		
		/**
		 * Return SQL for the time selector filter
		 * 
		 * @param string $field the name of the field that we filter on
		 * @return string the sql
		 */
		static protected function getTimeSelectorFilterSql($field)
		{
			$sqlFilter = new scSqlFilterDatePeriod($field, implode(',', scReportGenerator::getTimeSelectorFilterValues()));
			return $sqlFilter->getSql();
		}
		
		/**
		 * Return miSqlFilter for the time selector filter
		 * 
		 * @param string $field the name of the field that we filter on
		 * @return miSqlFilter
		 */
		static protected function getTimeSelectorFilter($field)
		{
			$sqlFilter = new scSqlFilterDatePeriod($field, implode(',', scReportGenerator::getTimeSelectorFilterValues()));
			return $sqlFilter;
		}
		
		/**
		 * Get the SQL field name for the field
		 * 
		 * @param int $fieldId
		 * @return string the SQL field name
		 */
		static public function getFieldSqlName($fieldId)
		{
			return 'F' . $fieldId;
		}
		
		/**
		 * returns if the report have status so we know if we should display
		 * the status filter
		 * @param array $reportDefinition
		 * @return boolean 
		 */
		static public function hasOrderStatus($reportDefinition)
		{
			$reportInfo = self::getReportInfo($reportDefinition);
			return isset($reportInfo[self::REPORT_INFO_ORDERSTATUS_FIELD]);
		}
		
		static public function getOrderStatusesFilterSql($field)
		{
			// The default statuses are all, except the Unfinished status
			$defaultValues = scOrderStatuses::singleton()->getOrderStatuses();
			unset($defaultValues[scOrderStatus::ORDER_STATUS_UNFINISHED]);
			
			$filter = new miSqlFilterIn($field, miGetParamDefault('OrderStatus', array_keys($defaultValues)));
			return $filter->getSql();
		}
	}
?>