<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * scReportGenerator class
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 * Generates the HTML for the report table as well as the XML needed for flash chart
	 */
	class scReportGenerator {
		
		// Default filter value for date range filter
		const DAYS_AGO_FILTER = 29;
		
		const MAX_ENTRIES_IN_XML = 200;
		const MAX_LINES_IN_CHART = 20;
		
		const MAX_Y_AXIS_CHARS = 50;
		const MAX_LEGEND_CHARS = 35;
		
		const DEFAULT_NUMBER_OF_RECORDS = 10;
		static protected $_numberOfRecordsTable = array(5 => 5, 10 => 10, 20 => 20, 50 => 50, 100 => 100, '' => 'All');
		static protected $_numberOfRecordsLineChart = array(5 => 5, 10 => 10, 20 => 20);
		static protected $_numberOfRecordsColumnChart = array(5 => 5, 10 => 10, 20 => 20, 50 => 50);
		
		protected $_reportDefinition;
		
		public function __construct()
		{
//			foreach ($this->_fieldNames as &$fieldName)
//				$fieldName = scMultiLanguage::getTranslatedText($fieldName);
//			foreach ($this->_reportDefinitions as &$reportDefinition)
//				$reportDefinition['name'] = scMultiLanguage::getTranslatedText($reportDefinition['name']);
		}
		
		public function generate()
		{
			$this->_reportDefinition = scReportDefinitions::getReportDefinition(miGetParam('report'));
			
			if (miGetParamDefault('action', '') == 'XMLReportSettings') {
				$this->displayReportSettings();
			}
			
			// Make the report query
			$numberOfRecords = (int)miGetParamDefault('NumberOfRecords', self::DEFAULT_NUMBER_OF_RECORDS);
			$sql = scReportDefinitions::getReportQuery($this->_reportDefinition, $numberOfRecords);
			
			$rows = $this->getData($sql);
			
			// Assign the column names
			$columnNames = scReportDefinitions::getColumnNames($this->_reportDefinition);
//			dump($columnNames);
			$formatter = new scReportColumnsFormatter;
			$formattedRows = $formatter->getFormattedColumns($rows, array_keys($columnNames));
			
			// Check if the user requested CSV export
			if (miGetParamDefault('action', '') == 'CsvExport') {
				$this->exportAsCSV($formattedRows, $columnNames);
			}
			if (miGetParamDefault('action', '') == 'displayXML') {
				$this->displayXml($rows, $formattedRows, $columnNames);
				exit;
			}
			
			if (miGetParamDefault('load', '') == 'table') {
				if (miGetParamDefault('display', '') == 'table') {
					return $this->displayTable($formattedRows, $columnNames);
				}
				return $this->displayChart($formattedRows, $columnNames);
			}
						
			return $this->displayReport($formattedRows, $columnNames);
		}
		
		protected function sumByKey($array, $key)
		{
			$sum = 0;
			foreach ($array as $arr) {
				$sum += $arr[$key];
			}
			return $sum;
		}
		
		protected function displayChart($rows, $columnNames)
		{
			$flashSection = new miTemplateParserSectionInfo();
			$chartType = scReportUnitOfTime::isUnitOfTimeSummary() ? 'columnChart' : 'lineChart';
			$flashSection->setSectionInfo($chartType, 1);
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/reports/report_chart.tmpl');
			$t->assign('%%CHARTVALUE_DROPDOWN%%', $this->getChartValueDropdown($this->_reportDefinition));
			$t->assign('%%NUMBEROFRECORDS_CHART_DROPDOWN%%', $this->getNumberOfRecordsDropdown($chartType));
			$this->assignCurrenciesDropdown($t);
			
			$numRows = min(count($rows), miGetParamDefault('NumberOfRecords', self::DEFAULT_NUMBER_OF_RECORDS));
			$t->assign('%%NUM_ROWS%%', $numRows);
			
			$t->setSectionInfos(array($flashSection));
			return $t->templateParse();
		}
		
		protected function displayTable($rows, $columnNames)
		{
			$columnNamesSection = new miTemplateParserSectionInfo();
			$columnNamesSection->setSectionInfo('columnNameRows', count($columnNames),
				array('%%COLUMNNAME%%' => miI18N::htmlEscape(array_values($columnNames))));
			
			$columnClasses = scReportColumnsFormatter::getColumnClasses($columnNames);
			$numericColumns = scReportColumnsFormatter::getNumericColumns($columnNames);
			
			// If need, get the currency prefix and suffix
			$currencyPrefix = $currencySuffix = '';
			foreach ($columnClasses as $columnClass)
				if ($columnClass == 'report_money') {
					list($currencyPrefix, $currencySuffix) = $this->getReportCurrency();
					break;
				}
			
			$productColumnKey = scReportDefinitions::getFieldSqlName(scReportDefinitions::REPORT_FIELD_ORDERED_ITEM_NAME);
			
			// Compute totals
			$totals = array();
			$showTotals = false;
			if (count($rows)) {
				// init totals
				foreach ($rows[0] as $key => $val) {
					$totals[$key] = '';
					if (isset($numericColumns[$key])) {
						$totals[$key] = $this->sumByKey($rows, $key);
						$showTotals = true;
					}
				}
			}
			
			// Assign the data
			$data = array();
			foreach ($rows as $row) {
				$rowData = array();
				foreach ($row as $key => $value) {
					if (isset($columnClasses[$key]) and $columnClasses[$key]) {
						$class = ' class="' . $columnClasses[$key] . '"';
						if ($columnClasses[$key] == 'report_money')
							$value = $currencyPrefix . $value . $currencySuffix;
					} else
						$class = '';
					
					$rowData[] = '<td' . $class . '>' .
						($key == $productColumnKey ? '<a href="products.php?ProductNameFilter=' . miI18N::htmlEscape($value) . '">' . miI18N::htmlEscape($value)  . '</a>' : miI18N::htmlEscape($value)) .
						'</td>';
				}
				$data[] = implode($rowData);
			}
			
			$dataSection = new miTemplateParserSectionInfo();
			$dataSection->setSectionInfo('rows', count($data), array('%%DATA%%' => $data));
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/reports/report_table.tmpl');
			$t->assign('%%NUMBEROFRECORDS_TABLE_DROPDOWN%%', $this->getNumberOfRecordsDropdown('Table'));
			$this->assignCurrenciesDropdown($t);
			$t->setSectionInfos(array($columnNamesSection, $dataSection));
			
			if ($showTotals) {
				$rowData = array();
				foreach ($totals as $key => $value) {
					if (isset($columnClasses[$key]) and $columnClasses[$key]) {
						$class = ' class="' . $columnClasses[$key] . '"';
						if ($columnClasses[$key] == 'report_money')
							$value = $currencyPrefix . $value . $currencySuffix;
					} else
						$class = '';
					$rowData[] = '<td' . $class . '>' . miI18N::htmlEscape($value) . '</td>';
				}
				$rowData[0] = '<td>' . _t('Total') . '</td>';
				$totalRow = implode($rowData);
			} else
				$totalRow = '';
			
			$t->assign('%%TOTALS%%', $totalRow);
			
			if (count($rows) == 0)
				$t->addSectionInfo(new miTemplateParserSectionInfo('norows'));
			
			return $t->templateParse();
		}
		
		protected function displayReport($rows, $columnNames)
		{
			$reportInfo = scReportDefinitions::getReportInfo($this->_reportDefinition);
			$defaultField = $reportInfo[scReportDefinitions::REPORT_INFO_FIELDS][0];

			// Call the parser
			$t = new scTemplateParser();
			$t->readTemplate('admin/reports/report.tmpl');
			if (scReportDefinitions::hasOrderStatus($this->_reportDefinition)) {
				$statuses = scOrderStatuses::singleton()->getOrderStatuses();
				// setup default values... They should be all checked excepting 'unfinished'
				$defaultValues = $statuses;
				unset($defaultValues[scOrderStatus::ORDER_STATUS_UNFINISHED]);
				
				$activeValues = miGetParamDefault('OrderStatus', array_keys($defaultValues));
				$t->addSectionInfo(new miTemplateParserSectionInfo('orderStatus'));
				$t->assign('%%HTML_ORDERSTATUS_FILTER%%', scUIUtil::createCheckboxTable('OrderStatus', $statuses, $activeValues));
			}
			//$t->setSectionInfos(array($dataSection));
			$t->assignArray(miI18N::htmlEscape(array(
				'%%REPORT%%' => miGetParam('report'),
				'%%REPORTNAME%%' => $this->_reportDefinition['name'],
				'%%TIME%%' => miGetParamDefault('UnitOfTime', scReportUnitOfTime::UNIT_OF_TIME_SUMMARY),
				'%%CHARTVALUE%%' => miGetParamDefault('ChartValue', $defaultField),
				'%%NUMBEROFRECORDS%%' => miGetParamDefault('NumberOfRecords', self::DEFAULT_NUMBER_OF_RECORDS),
				'%%CURRENCYCODE%%' => scReportDefinitions::getCurrencyCode($this->_reportDefinition)
			)));
			
			$t->assign('%%PERIODWIDGET%%', scUIUtil::getTimePeriodWidgetHtml(self::getTimeSelectorFilterValues()));
			$t->assign('%%UNITOFTIME%%', scReportUnitOfTime::getUnitOfTimeDropdown());
						
			return $t->templateParse();
		}
		
		/**
		 * Returns html with the available chart values
		 * 
		 * @return string the html
		 */
		protected function getChartValueDropdown()
		{
			$reportInfo = scReportDefinitions::getReportInfo($this->_reportDefinition);
			$fields = array();
			foreach ($reportInfo[scReportDefinitions::REPORT_INFO_FIELDS] as $fieldId)
				$fields[$fieldId] = scReportDefinitions::getFieldName($fieldId);
			$defaultField = $reportInfo[scReportDefinitions::REPORT_INFO_FIELDS][0];
			return miUIUtil::getConstDropdown('ChartValue', miGetParamDefault('ChartValue', $defaultField), $fields);
		}
		
		protected function getNumberOfRecordsDropdown($type)
		{
			if ($type == 'Table')
				$name = 'NumberOfRecordsTable';
			else
				$name = 'NumberOfRecordsChart';
			
			if ($type == 'Table')
				$records = self::$_numberOfRecordsTable;
			elseif ($type == 'lineChart')
				$records = self::$_numberOfRecordsLineChart;
			else
				$records = self::$_numberOfRecordsColumnChart;
				
			$default = miGetParamDefault('NumberOfRecords', self::DEFAULT_NUMBER_OF_RECORDS);
			if (!isset($records[$default]))
				$default = self::DEFAULT_NUMBER_OF_RECORDS;
			return miUIUtil::getConstDropdown($name, $default, $records);
		}
		
		protected function assignCurrenciesDropdown(&$t)
		{
			$currencies = scReportDefinitions::getReportCurrencies($this->_reportDefinition);
			
			if (empty($currencies)) {
				$dropdown = '';
				$show = false;
			} elseif (count($currencies) == 1) {
				$dropdown = reset($currencies);
				$show = true;
			} else {
				$dropdown = miUIUtil::getConstDropdown('CurrencyCode', miGetParamDefault('CurrencyCode', reset($currencies)), $currencies);
				$show = true;
			}
			$t->assign('%%CURRENCIES_TABLE_DROPDOWN%%', $dropdown);
			$t->assign('%%CURRENCIES_TABLE_SHOW%%', $show ? '' : 'display: none');
		}
		
		protected function displayReportSettings()
		{
			// Remove possible header and do gzip encoding for speeding up the XML file download
			if (ob_get_level()) {
				ob_end_clean();
				header('Content-Encoding: none');
			}
			ob_start('ob_gzhandler');
			
			$chartValue = miGetParam('ChartValue');
			$chartValueName = scReportDefinitions::getFieldName($chartValue);
			
			$t = new scTemplateParser();
			$t->readTemplate($this->getSettingsTemplateName());
			$t->assign('%%REPORT%%', htmlspecialchars(miGetParam('report')));
			$t->assign('%%REPORTNAME%%', htmlspecialchars($this->_reportDefinition['name']));
			$t->assign('%%CHARTVALUENAME%%', htmlspecialchars($chartValueName));
			$t->assign('%%CHARTVALUENAMELC%%', htmlspecialchars(strtolower($chartValueName)));
			$t->assign('%%COMPANYNAME%%', htmlspecialchars(scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME)));
			$t->assign('%%AMLINE_BOTTOM%%', intval(miGetParamDefault('NumRows', 0) / 2) * 25 + 60);
			
			if (scReportColumnsFormatter::isMoneyColumn($chartValue))
				list($currencyPrefix, $currencySuffix) = $this->getReportCurrency();
			else
				$currencyPrefix = $currencySuffix = '';
			
			$t->assign('%%CURRENCYPREFIX%%', htmlspecialchars($currencyPrefix));
			$t->assign('%%CURRENCYSUFFIX%%', htmlspecialchars($currencySuffix));
			
			// amcharts limitation - the unit can be only left or right, not both
			if ($currencyPrefix != '') {
				$t->assign('%%UNIT%%', htmlspecialchars($currencyPrefix));
				$t->assign('%%UNITPOSITION%%', 'left');
			} else {
				$t->assign('%%UNIT%%', htmlspecialchars($currencySuffix));
				$t->assign('%%UNITPOSITION%%', 'right');
			}
			
			$t->templateShow();
			exit;
		}
		
		protected function getSettingsTemplateName()
		{
			if (scReportUnitOfTime::isUnitOfTimeSummary())
				return 'admin/reports/amcolumn_settings.xml';
			return 'admin/reports/amline_settings.xml';
		}
		
		/**
		 * Print the data XML for amCharts
		 */
		protected function displayXml($rows, $formattedRows, $columnNames)
		{
			// Remove possible header and do gzip encoding for speeding up the CSV file download
			if (ob_get_level()) {
				ob_end_clean();
				header('Content-Encoding: none');
			}
			ob_start('ob_gzhandler');
			
			// Find the display column name
			$reportInfo = scReportDefinitions::getReportInfo($this->_reportDefinition);
			
			// Find the value column name
			$chartValue = miGetParamDefault('ChartValue', $reportInfo[scReportDefinitions::REPORT_INFO_FIELDS][0]);
			$valueColumn = scReportDefinitions::getFieldSqlName($chartValue);
			
			if (isset($reportInfo[scReportDefinitions::REPORT_INFO_GROUP_BY_FIELD]))
				$displayColumn = scReportDefinitions::getFieldSqlName($reportInfo[scReportDefinitions::REPORT_INFO_GROUP_BY_FIELD]);
			else
				$displayColumn = $valueColumn;
			$isOneLine = !isset($reportInfo[scReportDefinitions::REPORT_INFO_GROUP_BY_FIELD]);
			
			if (scReportUnitOfTime::isUnitOfTimeSummary())
				$this->displayXmlColumn($displayColumn, $valueColumn, $formattedRows, $isOneLine);
			else
				$this->displayXmlLine($displayColumn, $valueColumn, $rows, $formattedRows, $isOneLine);
		}
		
		/**
		 * Display the XML for line chart
		 */
		protected function displayXmlLine($displayColumn, $valueColumn, $rows, $formattedRows, $isOneLine)
		{
			if (empty($rows)) {
				// output default xml
				echo '<chart><xaxis><value xid="0">-</value></xaxis></chart>';
				return;
			}
			
			list($min, $max) = $this->getMinMaxTimes($rows);
			$timesArray = scReportUnitOfTime::getTimesArray($min, $max, scReportUnitOfTime::getUnitOfTime());
			$timesMapping = $this->reduceTimesCount($timesArray);
						
			$graphs = array();
			
			if ($isOneLine)
				$graphs[] = scReportDefinitions::getFieldName(substr($displayColumn, 1));
			else {
				foreach ($formattedRows as $row) {
					$graphs[] = $row[$displayColumn];
				}
				$graphs = array_unique($graphs);
			}
			
			$values = array();
			// fill all dates with zeroes
			$zeroValues = array_combine(array_values($timesArray), array_fill(0, count($timesArray), 0));
			foreach ($graphs as $graph) {
			 	$values[$graph] = $zeroValues;
			}
			
			// save non-zero values for each graph so we can decide which graph to remove later
			$graphMap = array_flip($graphs);
			$graphValues = array_combine(array_values($graphMap), array_fill(0, count($graphs), 0));			
			
			foreach ($formattedRows as $rowKey => $row) {
				$graph = $isOneLine ? $graphs[0] : $row[$displayColumn];
				$graphValues[$graphMap[$graph]] += $row[$valueColumn];
				// find original time in the timesArray
				$originalTime = $timesArray[$timesMapping[$row['T']]];				
				if (!is_array($values[$graph][$originalTime])) {
					$values[$graph][$originalTime] = array();
				}					
				$values[$graph][$originalTime][] = $row[$valueColumn];
			}
			
			if (count($graphs) > self::MAX_LINES_IN_CHART) {
				arsort($graphValues);
				$i = 0;
				foreach ($graphValues as $k => $v) {
					if ($i >= self::MAX_LINES_IN_CHART) {
						unset($graphs[$k]);
					}
					$i++;
				}
			}	
			
			// now arrange and calculate averages for values by time array
			$this->calcAvgValues($values);
			
			echo '<chart>';
			echo '<xaxis>';
			foreach ($timesArray as $timeValue) {
				echo '<value xid="' . htmlspecialchars($timeValue) . '">';
				echo htmlspecialchars($timeValue);
				echo '</value>';
			}
			echo '</xaxis>';

			if (scReportColumnsFormatter::isMoneyColumn(substr($valueColumn, 1)))
				list($currencyPrefix, $currencySuffix) = $this->getReportCurrency();
			else
				$currencyPrefix = $currencySuffix = '';
						
			echo '<graphs>';
			foreach ($graphs as $key => $graph) {
				$graphName = scUtf8::truncate($graph, 0, self::MAX_LEGEND_CHARS);
				echo '<graph gid="' . htmlspecialchars($key) . '" type="line" line_width="3" title="' . htmlspecialchars($graphName) .
					'" balloon_text="' . htmlspecialchars($graphName . ' (' . $currencyPrefix . '{value}' . $currencySuffix . ')') . '" bullet="round_outlined" bullet_size="10">';
				 
				foreach ($values[$graph] as $time => $value) {
					echo '<value xid="' . htmlspecialchars($time) . '">';
					echo htmlspecialchars($value);
					echo '</value>';
				}
				echo '</graph>';
			}
			echo '</graphs>';
			
			echo '</chart>';
		}
		
		/**
		 * Decreases size of the arrays by leaving one element for every n elements
		 * @param array $timesArray the values
		 * @return array $timeMapping - mapping of the original values to the new ones    
		 */
		protected function reduceTimesCount(&$timesArray)
		{
			$count = count($timesArray);
			if ($count <= self::MAX_ENTRIES_IN_XML) {
				// no need to optimize
				return array_flip($timesArray);
			}
			
			$newCount = $count / self::MAX_ENTRIES_IN_XML;
			$newCount = ceil($newCount);
			
			// Get the n-th element of $values in $chunks
			$chunks = array_chunk($timesArray, $newCount);
			foreach ($chunks as $key => &$chunk) {	
				foreach ($chunk as $value) {
					$timeMapping[$value] = $key;
				}
				$chunk = array_shift($chunk);				
			}
			$timesArray = $chunks;			
			
			return $timeMapping;
		}
		
		/**
		 * Compute the average value for every element in the array
		 * @param array $values The values to compute
		 * @return none
		 */
		protected function calcAvgValues(&$values)
		{
			foreach ($values as $graph => $graphValues) {
				foreach ($graphValues as $time => $value) {
					if (is_array($value))
						$values[$graph][$time] = array_sum($value) / count($value);
				}
			}
		}
		
		/**
		 * Find the min and max time in $rows
		 * 
		 * @param array $rows the rows
		 * @return array min and max values
		 */
		protected function getMinMaxTimes($rows)
		{
			if (count($rows) == 0)
				return array(0,0);
			
			$min = $max = $rows[0]['T'];
			foreach ($rows as $row) {
				$min = min($min, $row['T']);
				$max = max($max, $row['T']);
			}
			
			return array($min, $max);
		}
		
		protected function displayXmlColumn($displayColumn, $valueColumn, $rows, $isOneLine)
		{
			if (empty($rows)) {
				// output default xml
				echo '<chart></chart>';
				return;
			}
			
			echo '<chart>';
			echo '<series>';
			
			foreach ($rows as $rowKey => $row) {
				echo '<value xid="' . htmlspecialchars($rowKey) . '">';
				
				if ($isOneLine)
					echo scReportDefinitions::getFieldName(substr($displayColumn, 1));
				else if ($row[$displayColumn] == '')
					echo _t('-Empty name-');
				else
					echo scUtf8::truncate($row[$displayColumn], 0, self::MAX_Y_AXIS_CHARS);
				
				echo '</value>';
			}
			echo '</series>';
			
			echo '<graphs>';
			echo '<graph gid="1">';
			
			$productColumnKeys = array(
				scReportDefinitions::getFieldSqlName(scReportDefinitions::REPORT_FIELD_ORDERED_ITEM_NAME),
				scReportDefinitions::getFieldSqlName(scReportDefinitions::REPORT_FIELD_PRODUCT_NAME),
			);
			
			foreach ($rows as $rowKey => $row) {
				echo '<value xid="' . htmlspecialchars($rowKey);
				if (in_array($displayColumn, $productColumnKeys))
					echo '" url="' . htmlspecialchars('products.php?ProductNameFilter=' . urlencode($row[$displayColumn]));
				echo '" description="' . htmlspecialchars($row[$displayColumn]) . '">';
				echo htmlspecialchars($row[$valueColumn]);
				echo '</value>';
			}
			
			echo '</graph>';
			echo '</graphs>';
			
			echo '</chart>';
		}
		
		/**
		 * Export the report as CSV
		 * This function never returns
		 * 
		 * @param array $rows
		 * @param array $columnNames
		 */
		protected function exportAsCSV($rows, $columnNames)
		{
			$csvExportAction = new scReportCsvExport;
			$csvExportAction->export($rows, $columnNames);
		}
		
		/**
		 * Returns array with the time selector filter values
		 * 
		 * @return array the filter values
		 */
		static public function getTimeSelectorFilterValues()
		{
			// The dafult period is this month
			$orderTimestampFilter[scSqlFilterDatePeriod::PARAM_PERIOD_TYPE] = miGetParamDefault('typePeriod', scSqlFilterDatePeriod::PERIOD_TYPE_CUSTOM);
			$orderTimestampFilter[scSqlFilterDatePeriod::PARAM_PREDEFINED_PERIOD] = miGetParamDefault('period', scSqlFilterDatePeriod::DATE_PERIODS_THIS_MONTH);
			$orderTimestampFilter[scSqlFilterDatePeriod::PARAM_DATE_FROM] = miGetParamDefault('StartDate', date('d/m/Y', time() - self::DAYS_AGO_FILTER * 60 * 60 * 24));
			$orderTimestampFilter[scSqlFilterDatePeriod::PARAM_DATE_TO] = miGetParamDefault('EndDate', date('d/m/Y'));
			return $orderTimestampFilter;
		}
		
		protected function getData($sql)
		{
			return miStaticDBUtil::execSelect($sql);
		}
		
		protected function getReportCurrency()
		{
			$currencyCode = scReportDefinitions::getCurrencyCode($this->_reportDefinition);
			
			try {
				$currency = new scCurrency($currencyCode);
			} catch (miDBException $exception) {
				if ($exception->getCode() != miDBException::EXCEPTION_RECORD_NOT_FOUND)
					throw $exception;
				
				// Unknown currency - no prefix nor suffix
				return array('', '');
			}
			
			$currencyPrefix = $currency->getCurrencySymbolLeft();
			$currencySuffix = $currency->getCurrencySymbolRight();
			return array($currencyPrefix, $currencySuffix);
		}
	}
	
	
	/**
	 * Exports records
	 */
	class scReportCsvExport extends miCsvExportAction {
		public function __construct()
		{
		}
		
		/**
		 * Perform the actual CSV printing
		 *
		 * @param array $records the records to be exported
		 * @param array $columnNames the column names
		 */
		protected function exportToCsv($records, $columnNames)
		{
			if (count($records) == 0)
				return;
				
			// Print header line
			$header = array();
			foreach ($columnNames as $columnName)
				$header[] = $this->escapeCsv($columnName);
			echo implode(',', $header);
			echo miCsvExportAction::CSV_LINE_DELIMITER;
			
			// Print the values
			foreach ($records as $record) {
				$line = array();
				foreach ($record as $value)
					$line[] = $this->escapeCsv($value);
				echo implode(',', $line);
				echo miCsvExportAction::CSV_LINE_DELIMITER;
			}
		}
		
		/**
		 * Perform CSV export
		 * 
		 * @param array $records the records to be exported
		 * @param array $columnNames the column names
		 */
		public function export($rows, $columnNames)
		{
			// Remove possible header and do gzip encoding for speeding up the CSV file download
			if (ob_get_level()) {
				ob_end_clean();
				header('Content-Encoding: none');
			}
			ob_start('ob_gzhandler');
			
			$this->showCsvExportHeaders();
			$this->exportToCsv($rows, $columnNames);
			exit;
		}
	}
?>