<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Class to encapsulate report UnitOfTime handling
	 * All timestamps in the database are in UTC
	 */
	class scReportUnitOfTime {
		const UNIT_OF_TIME_SUMMARY = 1;
		const UNIT_OF_TIME_DAILY = 2;
		const UNIT_OF_TIME_WEEKLY = 3;
		const UNIT_OF_TIME_MONTHLY = 4;
		const UNIT_OF_TIME_QUARTERLY = 5;
		const UNIT_OF_TIME_YEARLY = 6;
		
		/**
		 * Retutn SQL for selecting the unit of time
		 * 
		 * @param string $columnName
		 * @return string the sql code
		 */
		static public function getUnitOfTimeSql($columnName)
		{
			$timeUnitValueSql = array(
				scReportUnitOfTime::UNIT_OF_TIME_DAILY => ',FLOOR(%s/86400)',
				scReportUnitOfTime::UNIT_OF_TIME_WEEKLY => ',FLOOR(%s/604800)',
				scReportUnitOfTime::UNIT_OF_TIME_MONTHLY => ',EXTRACT(YEAR_MONTH FROM FROM_UNIXTIME(%s))',
				scReportUnitOfTime::UNIT_OF_TIME_QUARTERLY => ',CONCAT(YEAR(FROM_UNIXTIME(%s)), QUARTER(FROM_UNIXTIME(%s)))',
				scReportUnitOfTime::UNIT_OF_TIME_YEARLY => ',YEAR(FROM_UNIXTIME(%s))',
				scReportUnitOfTime::UNIT_OF_TIME_SUMMARY => '',
			);
			return sprintf($timeUnitValueSql[self::getUnitOfTime()], $columnName, $columnName);
		}
		
		static public function getUnitOfTimeDisplayColumnName()
		{
			$columnName = array(
				self::UNIT_OF_TIME_DAILY => _t('Day'),
				self::UNIT_OF_TIME_WEEKLY => _t('Week'),
				self::UNIT_OF_TIME_MONTHLY => _t('Month'),
				self::UNIT_OF_TIME_QUARTERLY => _t('Quarter'),
				self::UNIT_OF_TIME_YEARLY => _t('Year'),
			);
			return $columnName[self::getUnitOfTime()];
		}
		
		static public function getTimesArray($min, $max, $unitOfTime)
		{
			$result = array();
			
			$current = $min;
			while ($current <= $max) {
				$result[] = scReportColumnsFormatter::formatColumnT($current);
				$current++;
				
				switch ($unitOfTime) {
					case self::UNIT_OF_TIME_MONTHLY:
						if (($current % 100) == 13)
							$current += 88;
						break;
					
					case self::UNIT_OF_TIME_QUARTERLY:
						if (($current % 10) == 5)
							$current += 6;
						break;
				}
			}
			
			return $result;
		}
		
		/**
		 * Returns the currently selected unit of time
		 * 
		 * @return int the unit of time
		 */
		static public function getUnitOfTime()
		{
			return miGetParamDefault('UnitOfTime', self::UNIT_OF_TIME_SUMMARY);
		}
		
		/**
		 * Returns html for the unit of time drop-down
		 * 
		 * @return string the html
		 */
		static public function getUnitOfTimeDropdown()
		{
			$unitOfTimes = array(
				self::UNIT_OF_TIME_SUMMARY => _t('Summary'),
				self::UNIT_OF_TIME_DAILY => _t('Daily'),
				self::UNIT_OF_TIME_WEEKLY => _t('Weekly'),
				self::UNIT_OF_TIME_MONTHLY => _t('Monthly'),
				self::UNIT_OF_TIME_QUARTERLY => _t('Quarterly'),
				self::UNIT_OF_TIME_YEARLY => _t('Yearly'),
			);
			return miUIUtil::getConstDropdown('UnitOfTime', self::getUnitOfTime(), $unitOfTimes);
		}
		
		/**
		 * Returns if the selected unit of time is Summary
		 * 
		 * @return bool if the unit of time is Summary
		 */
		static public function isUnitOfTimeSummary()
		{
			$unitOfTime = self::getUnitOfTime();
			return $unitOfTime == scReportUnitOfTime::UNIT_OF_TIME_SUMMARY;
		}
	}
?>