<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scDateTime {
		const DATE_FORMAT = 'd M Y';
		const DATE_TIME_FORMAT = 'd M Y H:i';
		const TIME_FORMAT = 'H:i';
		
		public static function getFormattedDate($mysqlDate)
		{
			list($year, $month, $day) = explode('-', $mysqlDate);
			$date = gmmktime(0, 0, 0, $month, $day, $year);
			return self::translateMonth(gmdate(scDateTime::DATE_FORMAT, $date));
		}
		
		public static function getLocalFormattedTimestamp($timestamp, $dateOnly = false)
		{
			return self::translateMonth(date(($dateOnly ? self::DATE_FORMAT : self::DATE_TIME_FORMAT), $timestamp));
		}
		
		public static function getUTCFormattedTimestamp($timestamp)
		{
			return self::translateMonth(gmdate(scDateTime::DATE_FORMAT, $timestamp));
		}
		
		/**
		 * Convert ISO 8601 formatted date/time to timestamp
		 * Negative dates are not supported. Fractional seconds are ignored.
		 * @param string $dateTime
		 * @return int unix timestamp in UTC
		 */
		public static function iso8601toTimestamp($dateTime)
		{
			if (preg_match('/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(\.\d*)?([+-])(\d{2}):(\d{2})$/', $dateTime, $matches) === 1) {
				$offset = ($matches[9] * 3600) + ($matches[10] * 60);
				if ($matches[8] == '-')
					$offset = -$offset;
			} elseif (preg_match('/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(\.\d*)?Z$/', $dateTime, $matches) === 1) {
				// no time zone info - UTC
				$offset = 0;
			} else {
				throw new miException(_i('Invalid ISO 8601 date/time format.'));
			}
			
			$time = gmmktime($matches[4], $matches[5], $matches[6], $matches[2], $matches[3], $matches[1]);
			return $time + $offset;
		}
		
		/**
		 * Converts timestamp to ISO 8601 formatted date/time
		 * 
		 * @param int $timestamp unix timestamp in UTC
		 * @return string the date/time
		 */
		public static function timestampToIso8601($timestamp)
		{
			return gmdate('Y-m-d\TH:i:s\Z', $timestamp);
		}
		
		public static function setTimezone()
		{
			// TODO: V1.1 Allow setting the timezone by the user
			
			// This is already done in Init.php
			// @date_default_timezone_set(date_default_timezone_get());
		}
		
		/**
		 * Translates the english short 3-character month to the current language
		 * 
		 * @param string $str
		 * @return string
		 */
		protected static function translateMonth($str)
		{
			$bgMonths = array(
				'Jan' => _t('Jan'),
				'Feb' => _t('Feb'),
				'Mar' => _t('Mar'),
				'Apr' => _t('Apr'),
				'May' => _t('May'),
				'Jun' => _t('Jun'),
				'Jul' => _t('Jul'),
				'Aug' => _t('Aug'),
				'Sep' => _t('Sep'),
				'Oct' => _t('Oct'),
				'Nov' => _t('Nov'),
				'Dec' => _t('Dec'),
			);
			return strtr($str, $bgMonths);
		}
	}
?>