<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scHttp {
		const TIMEOUT_SECONDS = 5;
		
		/**
		 * Makes HTTP request
		 * 
		 * @param string $url
		 * @param string|false $contents the POST contents. If false then it is a GET request
		 * @param array $options
		 * @throws miException
		 */
		static protected function request($url, $contents, $options = array())
		{
			$ch = curl_init($url);
			
			if (isset($options['timeout']) and ((int)$options['timeout'] > 0)) {
				curl_setopt($ch, CURLOPT_TIMEOUT, (int)$options['timeout']);
			} else {
				curl_setopt($ch, CURLOPT_TIMEOUT, self::TIMEOUT_SECONDS);
			}
			
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_USERAGENT, 'Summer Cart');
			
			if ($contents !== false) {
				curl_setopt($ch, CURLOPT_POST, true);
				curl_setopt($ch, CURLOPT_POSTFIELDS, $contents);
			}
			if (isset($options['username']) and isset($options['password'])) {
				curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
				curl_setopt($ch, CURLOPT_USERPWD, $options['username'] . ':' . $options['password']);
			}
			if (isset($options['httpHeaders'])) {
				curl_setopt($ch, CURLOPT_HTTPHEADER, $options['httpHeaders']);
			}
			if (isset($options['sslVersion'])) {
				curl_setopt($ch, CURLOPT_SSLVERSION, $options['sslVersion']);
			}
			if (isset($options['sslVerifyPeer'])) {
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $options['sslVerifyPeer']);
			}
			if (isset($options['sslCert'])) {
				curl_setopt($ch, CURLOPT_SSLCERT, $options['sslCert']);
			}
			if (isset($options['sslKey'])) {
				curl_setopt($ch, CURLOPT_SSLKEY, $options['sslKey']);
			}
			
			$response = curl_exec($ch);
			if ($response === false) {
				$error = curl_error($ch) . ' (' . curl_errno($ch) . ')';
				curl_close($ch);
				throw new miException($error);
			}
			
			$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			if ($httpCode and ($httpCode != 200)) {
				curl_close($ch);
				throw new scHttpException(sprintf(_t('Curl request failed for "%s". Response code "%d".'), $url, $httpCode), $httpCode, $response);
			}
			
			curl_close($ch);
			
			return $response;
		}
		
		/**
		 * Retrieves a resource using HTTP GET
		 */
		static public function get($url, $options = array())
		{
			return self::request($url, false, $options);
		}
		
		/**
		 * Makes a HTTP POST request
		 */
		static public function post($url, $contents, $options = array())
		{
			return self::request($url, $contents, $options);
		}
	}
?>