<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scLog {
		const SEVERITY_CRITICAL = 1;
		const SEVERITY_ERROR = 2;
		const SEVERITY_WARNING = 3;
		const SEVERITY_INFO = 4;
		const SEVERITY_DEBUG = 5;
		
		const CATEGORY_NONE = 0;
		const CATEGORY_PAYMENT = 1;
		const CATEGORY_SHIPPING = 2;
		const CATEGORY_CURRENCY_RATES = 3;
		const CATEGORY_NEWSLETTER = 4;
		
		protected $_severity;
		protected $_category;
		
		public function __construct()
		{
			$this->_severity = array(
				self::SEVERITY_CRITICAL => _t('Critical'),
				self::SEVERITY_ERROR => _t('Error'),
				self::SEVERITY_WARNING => _t('Warning'),
				self::SEVERITY_INFO => _t('Info'),
				self::SEVERITY_DEBUG => _t('Debug')
			);
			
			$this->_category = array(
				self::CATEGORY_NONE => _t('None'),
				self::CATEGORY_PAYMENT => _t('Payment'),
				self::CATEGORY_SHIPPING => _t('Shipping'),
				self::CATEGORY_CURRENCY_RATES => _t('Currency rates'),
				self::CATEGORY_NEWSLETTER => _t('Newsletter'),
			);
		}
		
		/**
		 * @return scLog
		 */
		public static function singleton()
		{
			static $obj = null;
			
			if ($obj == null)
				$obj = new scLog();
			return $obj;
		}
		
		public function log($severity, $message, $category = 0)
		{
			$values = array(
				'LogTimestamp' => time(),
				'LogSeverity' => $severity,
				'LogMessage' => $message,
				'LogCategory' => $category
			);
			
			$adminAuth = new scAdminAuth;
			$admin = $adminAuth->getAdministrator();
			if ($admin !== false)
				$values['LogAdministratorUsername'] = $admin->get('AdministratorUsername');
			
			$customer = scCustomerAuth::singleton()->getCustomer();
			if ($customer !== false)
				$values['LogCustomerEmail'] = $customer->get('CustomerEmail');
			
			miStaticDBUtil::execInsert('Logs', $values);
		}
		
		public function critical($message, $category = 0)
		{
			$this->log(self::SEVERITY_CRITICAL, $message, $category);
		}
		
		public function error($message, $category = 0)
		{
			$this->log(self::SEVERITY_ERROR, $message, $category);
		}
		
		public function warning($message, $category = 0)
		{
			$this->log(self::SEVERITY_WARNING, $message, $category);
		}
		
		public function info($message, $category = 0)
		{
			$this->log(self::SEVERITY_INFO, $message, $category);
		}
		
		public function debug($message, $category = 0)
		{
			$this->log(self::SEVERITY_DEBUG, $message, $category);
		}
		
		public function getSeverityText($severity)
		{
			if (!array_key_exists($severity, $this->_severity))
				return _t('Unknown');
				
			return $this->_severity[$severity];
		}
		
		public function getAllSeverityTexts()
		{
			return $this->_severity;
		}
		
		public function getCategoryText($category)
		{
			if (!array_key_exists($category, $this->_category))
				return _t('Unknown');
				
			return $this->_category[$category];
		}
	
		public function getAllCategoryTexts()
		{
			return $this->_category;
		}
	}
?>