<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Class for handling web and file paths
	 */
	class scPaths
	{
		/**
		 * Report low free space if the free space is less (in percents)
		 */
		const PERCENT_FREE = 5;
		
		const PATH_FILE = 1;
		const PATH_WEB = 2;

		const RESOURCE_PRODUCT_IMAGE = 1;
		const RESOURCE_PRODUCT_THUMB = 2;
		const RESOURCE_MANUFACTURER_IMAGE = 3;
		const RESOURCE_MENU_ITEM_IMAGE = 4;
		const RESOURCE_TEMP_IMAGE = 5;
		const RESOURCE_TEMP_THUMB = 6;
		const RESOURCE_LANGUAGE_IMAGE = 7;
		const RESOURCE_WYSIWYG_EDITOR_IMAGE = 8;
		const RESOURCE_BASE_EGOOD = 9;
		const RESOURCE_CATEGORY_IMAGE = 10;
		const RESOURCE_SKIN_DEFAULT_IMAGE = 11;
		const RESOURCE_SKIN_UPLOADED_IMAGE = 12;
		const RESOURCE_PRODUCT_VARIANT_IMAGE = 13;
		const RESOURCE_PRODUCT_VARIANT_THUMB = 14;
		const RESOURCE_IMPORT_IMAGE = 15;
		const RESOURCE_ORDER_ITEM_OPTION_USER_IMAGE = 16;
		const RESOURCE_ORDER_ITEM_LOCAL_OPTION_USER_IMAGE = 17;
		const RESOURCE_GALLERY_IMAGE = 18;
		const RESOURCE_GALLERY_IMAGE_THUMB = 19;
		
		const RESOURCE_PRODUCT_BOX_IMAGE = 20;
		const RESOURCE_PRODUCT_VARIANT_BOX_IMAGE = 21;
		const RESOURCE_PRODUCT_LARGE_IMAGE = 22;
		const RESOURCE_PRODUCT_VARIANT_LARGE_IMAGE = 23;
		const RESOURCE_PRODUCT_ORIGINAL_IMAGE = 24;
		const RESOURCE_PRODUCT_VARIANT_ORIGINAL_IMAGE = 25;
		
		const RESOURCE_SHIPPING_LABEL_CACHE_FILE = 26;
		
		const USER_FILES_PATH = '/userfiles/';
		
		const EGOODS_TEMP = 'temp/';
		const EGOODS_PRODUCTS = 'products/';
		const EGOODS_SHARED = 'shared/';
		
		const GALLERY_LARGE_IMAGES = 'large/';
		const GALLERY_THUMB_IMAGES = 'thumb/';
		
		const SECURE_SERVER_PORT = 443;
		
		static protected $_paths = array(
			self::RESOURCE_PRODUCT_IMAGE => 'userfiles/productimages/',
			self::RESOURCE_PRODUCT_ORIGINAL_IMAGE => 'userfiles/productimages/',
			self::RESOURCE_PRODUCT_THUMB => 'userfiles/productthumbs/',
			self::RESOURCE_PRODUCT_BOX_IMAGE => 'userfiles/productboximages/',
			self::RESOURCE_PRODUCT_LARGE_IMAGE => 'userfiles/productlargeimages/',
			self::RESOURCE_MANUFACTURER_IMAGE => 'userfiles/manufacturer/',
			self::RESOURCE_MENU_ITEM_IMAGE => 'userfiles/menuitems/',
			self::RESOURCE_TEMP_IMAGE => 'userfiles/temp_images/',
			self::RESOURCE_TEMP_THUMB => 'userfiles/temp_images/', // If the temp thumb dir is changed, add it to the garbage collector
			self::RESOURCE_LANGUAGE_IMAGE => 'userfiles/languages/',
			self::RESOURCE_WYSIWYG_EDITOR_IMAGE => 'userfiles/editor/',
			self::RESOURCE_BASE_EGOOD => 'userfiles/egoods/',
			self::RESOURCE_CATEGORY_IMAGE => 'userfiles/categoryimages/',
			self::RESOURCE_SKIN_DEFAULT_IMAGE => 'skins/',
			self::RESOURCE_SKIN_UPLOADED_IMAGE => 'userfiles/skins/',
			self::RESOURCE_PRODUCT_VARIANT_IMAGE => 'userfiles/productimages/',
			self::RESOURCE_PRODUCT_VARIANT_ORIGINAL_IMAGE => 'userfiles/productimages/',
			self::RESOURCE_PRODUCT_VARIANT_THUMB => 'userfiles/productthumbs/',
			self::RESOURCE_PRODUCT_VARIANT_BOX_IMAGE => 'userfiles/productboximages/',
			self::RESOURCE_PRODUCT_VARIANT_LARGE_IMAGE => 'userfiles/productlargeimages/',
			self::RESOURCE_IMPORT_IMAGE => 'userfiles/csvimport/',
			self::RESOURCE_ORDER_ITEM_OPTION_USER_IMAGE => 'userfiles/order_item_images/',
			self::RESOURCE_ORDER_ITEM_LOCAL_OPTION_USER_IMAGE => 'userfiles/order_item_images/',
			self::RESOURCE_GALLERY_IMAGE => 'userfiles/galleries/',
			self::RESOURCE_GALLERY_IMAGE_THUMB => 'userfiles/galleries/', // the same as for the images
			self::RESOURCE_SHIPPING_LABEL_CACHE_FILE => 'userfiles/shippinglabels/',
		);
		static protected $_filenamePrefixes = array(
			self::RESOURCE_PRODUCT_IMAGE => 'product_',
			self::RESOURCE_PRODUCT_ORIGINAL_IMAGE => 'original_',
			self::RESOURCE_PRODUCT_THUMB => 'thumb_',
			self::RESOURCE_PRODUCT_BOX_IMAGE => 'product_',
			self::RESOURCE_PRODUCT_LARGE_IMAGE => 'product_',
			self::RESOURCE_MANUFACTURER_IMAGE => 'manufacturer_',
			self::RESOURCE_MENU_ITEM_IMAGE => 'menuitem_',
			self::RESOURCE_TEMP_IMAGE => 'image_',
			self::RESOURCE_TEMP_THUMB => 'thumb_',
			self::RESOURCE_LANGUAGE_IMAGE => 'language_',
			self::RESOURCE_WYSIWYG_EDITOR_IMAGE => '',
			self::RESOURCE_BASE_EGOOD => '',
			self::RESOURCE_CATEGORY_IMAGE => 'category_',
			self::RESOURCE_SKIN_DEFAULT_IMAGE => '',
			self::RESOURCE_SKIN_UPLOADED_IMAGE => '',
			self::RESOURCE_PRODUCT_VARIANT_IMAGE => 'variant_',
			self::RESOURCE_PRODUCT_VARIANT_ORIGINAL_IMAGE => 'variant_original_',
			self::RESOURCE_PRODUCT_VARIANT_THUMB => 'variant_thumb_',
			self::RESOURCE_PRODUCT_VARIANT_BOX_IMAGE => 'variant_',
			self::RESOURCE_PRODUCT_VARIANT_LARGE_IMAGE => 'variant_',
			self::RESOURCE_ORDER_ITEM_OPTION_USER_IMAGE => 'option_image_',
			self::RESOURCE_ORDER_ITEM_LOCAL_OPTION_USER_IMAGE => 'local_option_image_',
			self::RESOURCE_GALLERY_IMAGE => 'gallery_',
			self::RESOURCE_GALLERY_IMAGE_THUMB => 'gallery_thumb_',
			self::RESOURCE_SHIPPING_LABEL_CACHE_FILE => '',
		);
		static protected $_filenameSuffixes = array(
			self::RESOURCE_PRODUCT_IMAGE => '.jpg',
			self::RESOURCE_PRODUCT_ORIGINAL_IMAGE => '.jpg',
			self::RESOURCE_PRODUCT_THUMB => '.jpg',
			self::RESOURCE_PRODUCT_BOX_IMAGE => '.jpg',
			self::RESOURCE_PRODUCT_LARGE_IMAGE => '.jpg',
			self::RESOURCE_MANUFACTURER_IMAGE => '.jpg',
			self::RESOURCE_MENU_ITEM_IMAGE => '.jpg',
			self::RESOURCE_TEMP_IMAGE => '.jpg',
			self::RESOURCE_TEMP_THUMB => '.jpg',
			self::RESOURCE_LANGUAGE_IMAGE => '.jpg',
			self::RESOURCE_WYSIWYG_EDITOR_IMAGE => '.jpg',
			self::RESOURCE_BASE_EGOOD => '',
			self::RESOURCE_CATEGORY_IMAGE => '.jpg',
			self::RESOURCE_SKIN_DEFAULT_IMAGE => '.jpg',
			self::RESOURCE_SKIN_UPLOADED_IMAGE => '.jpg',
			self::RESOURCE_PRODUCT_VARIANT_IMAGE => '.jpg',
			self::RESOURCE_PRODUCT_VARIANT_ORIGINAL_IMAGE => '.jpg',
			self::RESOURCE_PRODUCT_VARIANT_THUMB => '.jpg',
			self::RESOURCE_PRODUCT_VARIANT_BOX_IMAGE => '.jpg',
			self::RESOURCE_PRODUCT_VARIANT_LARGE_IMAGE => '.jpg',
			self::RESOURCE_ORDER_ITEM_OPTION_USER_IMAGE => '.jpg',
			self::RESOURCE_ORDER_ITEM_LOCAL_OPTION_USER_IMAGE => '.jpg',
			self::RESOURCE_GALLERY_IMAGE => '.jpg',
			self::RESOURCE_GALLERY_IMAGE_THUMB => '.jpg',
			self::RESOURCE_SHIPPING_LABEL_CACHE_FILE => '',
		);
		
		static protected $_isSslEnabled = null;
		
		static public function isSslEnabled()
		{
			if (self::$_isSslEnabled == null)
				self::$_isSslEnabled = scSettings::getSettingValue(scSettings::SETTING_SSL_ENABLED);
			return self::$_isSslEnabled;
		}
		
		/**
		 * Returns true if the currently loaded page is using SSL
		 * @return string
		 */
		static public function isCurrentPageSsl()
		{
			return ((isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] != 'off')) || (isset($_SERVER['SERVER_PORT']) && ($_SERVER['SERVER_PORT'] == self::SECURE_SERVER_PORT)));
		}
		
		static public function getHttpHost()
		{
			if (miSettings::singleton()->get('SC_HOST') != '')
				return miSettings::singleton()->get('SC_HOST');
			return isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '';
		}
		
		static public function getHttpsHost()
		{
			if (miSettings::singleton()->get('SC_HOST') != '')
				return miSettings::singleton()->get('SC_HOST');
			return isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '';
		}
		
		/**
		 * Get the current script name
		 * Strips store web path. The script does NOT start with a slash
		 * 
		 * @return string returns the current script name without the store web path
		 */
		public static function getCurrentScriptPathName()
		{
			$page = (isset($_SERVER['SCRIPT_NAME']) ? $_SERVER['SCRIPT_NAME'] : $_SERVER['PHP_SELF']);
			$webDir = scPaths::getStoreWebPath();
			if (substr_compare($page, $webDir, 0, strlen($webDir)) == 0)
				$page = substr($page, strlen($webDir));
			
			return $page;
		}
		
		/**
		 * Returns the basename of the current script, i.e. something.php
		 * 
		 * @return string the basename of the current script
		 */
		public static function getCurrentScriptBaseName()
		{
			return basename(isset($_SERVER['SCRIPT_NAME']) ? $_SERVER['SCRIPT_NAME'] : $_SERVER['PHP_SELF']);
		}
		
		/**
		 * Returns the web path where the store is installed
		 * It always starts and ends with slash (/)
		 * Uses the SC_WEBPATH setting
		 *
		 * @return string the web path
		 */
		static public function getStoreWebPath()
		{
			$scWebDir = miSettings::singleton()->get('SC_WEBPATH');
			if (empty($scWebDir))
				return '/';
			
			if ($scWebDir[0] != '/')
				$scWebDir = '/' . $scWebDir;
			
			if (substr($scWebDir, -1) != '/')
				$scWebDir .= '/';
			return $scWebDir;
		}
		
		/**
		 * Returns the directory where the store is installed
		 */
		static public function getStoreRootDir()
		{
			return realpath(dirname(__FILE__) . '/../../../') . '/';
		}
		
		/**
		 * Return absolute HTTP url to the resource $relativePath
		 */
		static public function getUrl($relativePath = '')
		{
			if (!empty($relativePath) and ($relativePath[0] == '/'))
				$relativePath = substr($relativePath, 1);
			return 'http://' . self::getHttpHost() . self::getStoreWebPath() . $relativePath;
		}
		
		/**
		 * Get absolute HTTPS url to the resource $relativePath
		 */
		static public function getHttpsUrl($relativePath = '')
		{
			if (!empty($relativePath) and ($relativePath[0] == '/'))
				$relativePath = substr($relativePath, 1);
			return 'https://' . self::getHttpsHost() . self::getStoreWebPath() . $relativePath;
		}
		
		/**
		 * Return secure absolute url to the resource $relativePath.
		 * 
		 * If SSL is enabled returns HTTPS, otherwise - HTTP
		 */
		static public function getSecureUrl($relativePath = '')
		{
			if (self::isSslEnabled()) {
				return self::getHttpsUrl($relativePath);
			}
			return self::getUrl($relativePath);
		}
		
		/**
		 * Return absolute url to the resource $relativePath.
		 * It will keep the current HTTP or HTTPS protocol scheme
		 * 
		 * The $_SERVER['HTTPS'] is used to check if we should return secure url or not.
		 */
		static public function getAbsoluteUrl($relativePath = '')
		{
			if (self::isCurrentPageSsl()) {
				return self::getHttpsUrl($relativePath);
			}
			return self::getUrl($relativePath);
		}
		
		static public function getPath($item)
		{
			return self::$_paths[$item];
		}
		
		static public function getFilenamePrefix($item)
		{
			return self::$_filenamePrefixes[$item];
		}
		
		static public function getFilenameSuffix($item)
		{
			return self::$_filenameSuffixes[$item];
		}
		
		
		/**
		 * Returns the full path of an item. The actual filename should be passed as the first param
		 * If the $pathType is PATH_FILE the returned path is filesystem-based, if it is PATH_WEB then it is web-based
		 * 
		 * @param string $filename the filename of the item
		 * @param int $item the item type
		 * @param int $pathType PATH_FILE or PATH_WEB
		 */
		static public function getFilenamePath($filename, $item, $pathType = self::PATH_FILE)
		{
			$result = self::getPath($item) . $filename;
			if ($pathType == self::PATH_FILE)
				$result = self::getStoreRootDir() . $result;
			return $result;
		}
		
		/**
		 * If the $pathType is PATH_FILE the returned path is filesystem-based, if it is PATH_WEB then it is web-based
		 * 
		 * @param string $name the identification of the item, eg. the image id
		 * @param int $item the item type, eg. RESOURCE_PRODUCT_IMAGE
		 * @param int $pathType PATH_FILE or PATH_WEB
		 */
		static public function getItemPath($name, $item, $pathType = self::PATH_FILE)
		{
			return self::getFilenamePath(self::getItemFilename($name, $item), $item, $pathType);
		}
		
		/**
		 * Returns the filename of an item
		 * 
		 * @param string $name the identification of the item, eg. the image id
		 * @param int $item the item type, eg. RESOURCE_PRODUCT_IMAGE
		 */
		static public function getItemFilename($name, $item)
		{
			return self::getFilenamePrefix($item) . $name . self::getFilenameSuffix($item);
		}
		
		/**
		 * Checks if directory exists and is writable
		 */
		static public function checkDirWritable($dir)
		{
			if (!is_dir($dir))
				throw new miException(sprintf(_t('Directory %s does not exist!'), $dir));
			if (!is_writable($dir))
				throw new miException(sprintf(_t('Directory %s is not writtable!'), $dir));
		}
		
		/**
		 * Checks that a given file $path is writable, and there is enough free space
		 * Does not return result. If there is a problem throws exception
		 * 
		 * @param string $path
		 * @param boolean $checkFreeSpace pass true to check for free space (default false)
		 * @throws miException
		 */
		static public function checkFileWritable($path, $checkFreeSpace = false)
		{
			$dir = dirname($path); 
			self::checkDirWritable($dir);
			
			if (!$checkFreeSpace)
				return;
			
			$total = disk_total_space($dir);
			$free = disk_free_space($dir);
			$percent_free = ($free / $total) * 100;
			if ($percent_free < self::PERCENT_FREE) {
				throw new miException(sprintf(_t('The free disk space is low. You have %s bytes free space (%2.2f%% of total).'),
					number_format($free, 0, '', ','), $percent_free));
			}
		}
	}
?>