<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	 
	 /**
	  * PrintExportUtil - used by Print and CSVExport actions
	  */
	class scPrintExportUtil {
		protected $_view;
		protected $_fields = null;
		
		protected $_tablePrimaryKeys = array(
			'ProductID' => 'ProductsML',
			'ManufacturerID' => 'ManufacturersML',
			'CMSLayoutID' => 'CMSLayouts',
			'ProductClassID' => 'ProductClassesML',
			'CategoryID' => 'CategoriesML',
		);
		
		protected $_fieldSubstitutes = array(
			'ProductID' => 'ProductName',
			'CategoryID' => 'CategoryName',
			'ManufacturerID' => 'ManufacturerName',
			'CMSLayoutID' => 'CMSLayoutName',
			'ShippingZoneID' => 'ShippingZoneName',
			'CountryID' => 'CountryName',
			'ProductClassID' => 'ProductClassName',
		);
		
		protected $_extraFields = array();
		
		protected $_extraFieldsCallback = array();
		
		public function __construct(miView $view)
		{
			$this->_view = $view;
		}
		
		/**
		 * Return the field substitutes
		 */
		public function getFieldSubstitues()
		{
			return $this->_fieldSubstitutes;
		}
		
		public function setExtraFieldsCallback($originalField, $extraField, $callback)
		{
			$this->_extraFields[$originalField] = $extraField;
			$this->_extraFieldsCallback[$extraField] = $callback;
		}
		
		/**
		 * Returns all records. Only fields in $fields will be returned
		 * Replaces IDs with the names as specified in $_fieldSubstitutes
		 * 
		 * @param miSqlRecordset $recordset
		 * @param array $fields
		 * @return array
		 */
		public function getRecords(miSqlRecordset $recordset, $fields)
		{
			$allRows = $recordset->getAllRecords();
			$rows = array();
			
			foreach ($allRows as $selectedRow) {
				$row = array();
				foreach ($fields as $field) {
					if (array_key_exists($field, $this->_fieldSubstitutes) && array_key_exists($this->_fieldSubstitutes[$field], $selectedRow)) {
						$row[$this->_fieldSubstitutes[$field]] = $selectedRow[$this->_fieldSubstitutes[$field]];
					} else {
						$row[$field] = $selectedRow[$field];
					}
					if (array_key_exists($field, $this->_extraFields)) {
						$extraField = $this->_extraFields[$field];
						$row[$extraField] = call_user_func_array($this->_extraFieldsCallback[$extraField], array($selectedRow[$field]));
					}
				}
				
				$rows[] = $row;
			}
			
			return $rows;
		}
		
		/**
		 * Modifies the recordset used to get the data for printing functionality
		 * 
		 * @param miSqlRecordset $recordset
		 * @param array $fields the fields that will be used
		 */
		protected function prepareRecordset($recordset, $fields) 
		{
			$table = $recordset->getTable();
			if (!empty($table) && (substr_compare($table, 'ML', -2, 2, false) == 0))
				$table = substr($table, 0, strlen($table) - 2);
			
			if (($recordset instanceof scSqlRecordsetML) or ($recordset instanceof scSqlTreeRecordsetML)) {
				$recordset->setLanguageId(scView::getCurrentDataLanguageId());
			}
			
			foreach ($fields as $field) {
				// Add joins where needed
				if (array_key_exists($field, $this->_tablePrimaryKeys) && array_key_exists($field, $this->_fieldSubstitutes)) {
					$joinTable = $this->_tablePrimaryKeys[$field];
					
					if (!in_array($joinTable, $recordset->getAllTables())) {
						$joinCondition = 'ON ' . $table . '.' . $field . ' = ' . $joinTable . '.' . $field;
						
						// If joining multilanguage table add LanguageID
						if (substr_compare($joinTable, 'ML', -2, 2, false) == 0)
							$joinCondition .= ' AND ' . $joinTable . '.LanguageID=' . scView::getCurrentDataLanguageId();
						$recordset->addJoinCondition('LEFT', $joinTable, $joinCondition);
					}
					
					$substituteField = $joinTable . '.' . $this->_fieldSubstitutes[$field];
					
					$selectFields = $recordset->getAllSelectFields();
					if (!in_array($substituteField, $selectFields)) {
						$recordset->addSelectField($substituteField);
					}
				}
			}
		}	
		
		public function getFields()
		{
			if (is_null($this->_fields)) {
				$fieldSubstitues = $this->getFieldSubstitues();
				
				// Find the export fields
				$this->_fields = array();
				foreach ($this->_view->getDataFields() as $dataField) {
					if (array_key_exists('displayName', $dataField))
						$this->_fields[$dataField['data']] = scInterfaceTexts::getTranslatedText($dataField['displayName']);
				}
			}
			
			return $this->_fields;
		}
		
		/**
		 * @return miSqlRecordset
		 */
		public function getRecordset()
		{
			return $this->_view->getTable()->getRecordset();
		}
		
		public function setupRecordset(miSqlRecordset $recordSet, $fields)
		{
			$this->prepareRecordset($recordSet, $fields);
			$recordSet->setOrder(miGetParamDefault(miTableSorter::PARAM_SORTBY, ''), miGetParamDefault(miTableSorter::PARAM_SORTDIR, ''));
			
			$table = $this->_view->getTable();
			$stateObj = miAppFactory::singleton()->getStateObj();
			$table->setState($stateObj->getArray());
			
			$this->_view->initTable($table);
			
			$filterObjects = $this->_view->getTableFilterObj()->getFilterObjs();
			$recordSet->setFieldsValidation(true);
			$recordSet->addFilters($filterObjects);
		}
	}
?>