<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	interface ISet {
		
		/**
		 * @param mixed $value
		 */
		public function add($value);
		
		/**
		 * @return array
		 */
		public function getValues();
		
		/**
		 * @return bool
		 */
		public function isEmpty();
	}
	
	interface ICartesianProduct extends ISet {
		
		/**
		 * @param ISet $set
		 * @return ISet
		 */
		public function cartesianProduct(ISet $set);
	}
	
	class scSet implements ISet {
		
		/**
		 * @var array
		 */
		protected $_values = array();
		
		public function __construct(array $values = array())
		{
			$this->_values = $values;
			foreach ($this->_values as $key => $value) {
				if (!is_array($value)) {
					$this->_values[$key] = array($value);
				}
			}
		}
		
		/**
		 * @param mixed $value
		 */
		public function add($value)
		{
			$this->_values[] = array($value);
		}
		
		/**
		 * @return array
		 */
		public function getValues()
		{
			return $this->_values;
		}
		
		/**
		 * @return bool
		 */
		public function isEmpty()
		{
			return empty($this->_values);
		}
		
		/**
		 * @param scValuesSet $set
		 * @return scValuesSet
		 */
		public function cartesianProduct(ISet $set)
		{
			if ($this->isEmpty()) {
				$set = new scSet($set->getValues());
				if (!$set->isEmpty()) {
					$set->cartesianProduct($this);
				}
				return $set;
			}
			
			$product = array();
			foreach ($this->_values as $value1) {
				if (!is_array($value1)) {
					$value1 = array($value1);
				}
				foreach ($set->getValues() as $value2) {
					if (!is_array($value2)) {
						$value2 = array($value2);
					}
					$product[] = array_merge($value1, $value2);
				}
			}
			
			return new scSet($product);
		}
	}
?>