<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scTemplateParser extends miTemplateParser {
		/**
		 * Reads the template file
		 *
		 * @access public
		 * @param string $filename
		 */
		public function readTemplate($filename)
		{
			$pathname = miSettings::singleton()->get('MI_TEMPLATE_BASEDIR') . scSkins::singleton()->getCurrentSkin()->getTemplatePath($filename);
			$fd = fopen($pathname, 'rb');
			$this->_templateFileContents = fread($fd, filesize($pathname));
			fclose($fd);
			
			$this->_templateDir = dirname($pathname) . '/';
			
			$this->parseSections();
		}
		
		/**
		 * Reads the template file from the client area
		 *
		 * @access public
		 * @param string $filename
		 */
		public function readClientAreaTemplate($filename)
		{
			$pathname = miSettings::singleton()->get('MI_TEMPLATE_BASEDIR') . scSkins::singleton()->getClientAreaSkin()->getTemplatePath($filename);
			$fd = fopen($pathname, 'rb');
			$this->_templateFileContents = fread($fd, filesize($pathname));
			fclose($fd);
			
			$this->parseSections();
		}
		
		/**
		 * Parses the template and returns the result
		 *
		 * @access public
		 * @return string the parsed contents
		 */
		public function templateParse()
		{
			// Summer Cart modifications
			$this->assignSystemVariables();
			$preprocessedSections = $this->preprocessTemplateSections($this->_templateSections);
			
			$html = $this->templateParseSubsection($preprocessedSections, $this->_sectionInfos);
			return self::postprocessTemplate(str_replace(array_keys($this->_templateVars), $this->_templateVars, $html));
		}
		
		/**
		 * Parses a part of the template
		 * 
		 * @param array $subsectionPath
		 * @param array $sectionInfos array of miTemplateParserSectionInfo objects
		 * @return string parsed part of template
		 */
		public function templateParsePart($subsectionPath, $sectionInfos)
		{
			$templateSections = $this->_templateSections;
			
			foreach ($subsectionPath as $part) {
				foreach ($templateSections as $templateSection) {
					if ($templateSection['name'] == $part) {
						$templateSections = $templateSection['subsections'];
						continue 2;
					}
				}
				
				throw new miException(sprintf('Cannot find subsection named "%s".', $part));
			}
			
			// Summer Cart modifications
			$this->assignSystemVariables();
			$preprocessedSections = $this->preprocessTemplateSections($templateSections);
			
			$html =  $this->templateParseSubsection($preprocessedSections, $sectionInfos[end($subsectionPath)]->_subsections);
			return self::postprocessTemplate(str_replace(array_keys($this->_templateVars), $this->_templateVars, $html));
		}
		
		protected function assignSystemVariables()
		{
			$this->assign('%%SKIN_URL%%', scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl());
			$this->assignArray(scSkins::singleton()->getCurrentSkin()->getTemplateVars());
			$this->assign('%%HTTPS%%', scPaths::getSecureUrl());
			$this->assign('%%CURRENT_LANGUAGE_CODE%%', strtolower(scMultiLanguage::singleton()->getCurrentLanguageCode()));
			
			$urls = $this->getAssignableUrls();
			$this->assignArray($urls);
		}
		
		protected function getAssignableUrls()
		{
			static $urlsCache = array();
			
			$languageId = scMultiLanguage::singleton()->getCurrentLanguageId();
			if (isset($urlsCache[$languageId])) {
				return $urlsCache[$languageId];
			}
			
			if (scFriendlyUrl::hasFriendlyUrls()) {
				scCustomUrls::prepareItems(scFriendlyUrl::TYPE_DYNAMIC_PAGE, array_keys(scFriendlyUrl::getLinkableDynamicPages()));
			}
			
			$secureUrlType = scPaths::isSslEnabled() ? scUrlUtil::URL_TYPE_HTTPS : scUrlUtil::URL_TYPE_DOCROOT;
			$urlsCache[$languageId] = array(
				'%%URL_MANUFACTURERS%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_MANUFACTURERS), scUrlUtil::URL_TYPE_DOCROOT),
				'%%URL_CART%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_DOCROOT),
				'%%URL_SEARCH%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_SEARCH), scUrlUtil::URL_TYPE_DOCROOT),
				'%%URL_INDEX%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_INDEX), scUrlUtil::URL_TYPE_DOCROOT),
				'%%URL_NEWS%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_NEWS), scUrlUtil::URL_TYPE_DOCROOT),
				
				'%%URL_LOGIN%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN), $secureUrlType),
				'%%URL_FORGOT_PASS%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_FORGOT_PASS), $secureUrlType),
				'%%URL_REGISTER%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_REGISTER), $secureUrlType),
				'%%URL_PROFILE%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_PROFILE), $secureUrlType),
				'%%URL_ORDERS%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_ORDERS), $secureUrlType),
				'%%URL_POINTS%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_POINTS), $secureUrlType),
				'%%URL_WISHLIST%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_WISHLIST), $secureUrlType),
				'%%URL_ADDRESSES%%' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_ADDRESSES), $secureUrlType),
				
				'%%URL_PRODUCT%%' => scUrlUtil::convertUrl('product.php', scUrlUtil::URL_TYPE_DOCROOT),
				'%%URL_SEND_TO_FRIEND%%' => scUrlUtil::convertUrl('send_to_friend.php', scUrlUtil::URL_TYPE_DOCROOT),
				'%%URL_MODULE%%' => scUrlUtil::convertUrl('module.php', scUrlUtil::URL_TYPE_DOCROOT),
				
				'%%URL_CHECKOUT%%' => scPaths::getSecureUrl('checkout.php'),
				'%%URL_LOGOUT%%' => scPaths::getSecureUrl('logout.php'),
				'%%URL_VIEW_ORDER%%' => scPaths::getSecureUrl('view_order.php'),
				'%%URL_ORDER_EGOODS%%' => scPaths::getSecureUrl('order_egoods.php'),
				'%%URL_DOWNLOAD%%' => scPaths::getSecureUrl('download.php'),
			);
			
			return $urlsCache[$languageId];
		}
		
		
		/**
		 * Translated a template string
		 * Find all string between <mi:text> and </mi:text> tags and translates them
		 *
		 * @param string $string
		 * @return string
		 */
/*		protected static function translateTemplateString($string)
		{
			$offset = $end = 0;
			$result = '';
			
			while (true) {
				$start = strpos($string, '<mi:text>', $offset);
				if ($start === false)
					break;
				
				$result .= substr($string, $offset, $start - $offset);
				
				$end = strpos($string, '</mi:text>', $start + 9);
				if ($end === false)
					throw new miException(sprintf(_i('Unclosed <mi:text> found at "%s".'), $string));
				
				$str = substr($string, $start + 9, $end - $start - 9);
				$result .= scInterfaceTexts::getTranslatedText($str);
				
				$offset = $end + 10;
			}
			
			$result .= substr($string, $offset);
			return $result;
		}
		
		protected static function skinurlTemplateString($string)
		{
			$skin = scSkins::singleton()->getCurrentSkin();
			
			$offset = $end = 0;
			$result = '';
			
			while (true) {
				$start = strpos($string, '<mi:skinurl>', $offset);
				if ($start === false)
					break;
				
				$result .= substr($string, $offset, $start - $offset);
				
				$end = strpos($string, '</mi:skinurl>', $start + 12);
				if ($end === false)
					throw new miException(sprintf(_i('Unclosed <mi:skinurl> found at "%s".'), $string));
				
				$str = substr($string, $start + 12, $end - $start - 12);
				
				// Get the path to the file in the current, or in a parent skin
				$result .= $skin->getTemplatePath($str);
				
				$offset = $end + 13;
			}
			
			$result .= substr($string, $offset);
			return $result;
		}*/
		
/*		
		// This method is slower the preg_replace_callback
		protected static function preprocessTemplateString($string)
		{
			$skin = scSkins::singleton()->getCurrentSkin();
			$preprocessFuncs = array(
				'text' => array('scMultiLanguage', 'getTranslatedText'),
				'skinurl' => array($skin, 'getTemplatePath')
			);
			
			$offset = $end = 0;
			$result = '';
			
			while (true) {
				$start = strpos($string, '<mi:', $offset);
				if ($start === false)
					break;
				
				foreach ($preprocessFuncs as $tag => $func) {
					if (substr_compare($string, '<mi:' . $tag . '>', $start, strlen($tag) + 5) === 0) {
						$result .= substr($string, $offset, $start - $offset);
						
						$textstart = $start + strlen($tag) + 5;
						$end = strpos($string, '</mi:' . $tag . '>', $textstart);
						if ($end === false)
							throw new miException(sprintf(_i('Unclosed <mi:%s> found at "%s".'), $tag, $string));
						
						$str = substr($string, $textstart, $end - $textstart);
						$result .= call_user_func($func, $str);
						
						$offset = $end + strlen($tag) + 6;
						continue 2;
					}
				}
				$result .= $string[$offset];
				$offset++;
			}
			
			$result .= substr($string, $offset);
			return $result;
		}*/
		
		protected function postprocessTemplate($string)
		{
			return preg_replace_callback('#<mi:skinurl>(.*)</mi:skinurl>#sSU', array('self', 'skinurlTemplateStringCallback'), $string);
		}
		
		protected static function skinurlTemplateStringCallback($matches)
		{
			$templatePath = scSkins::singleton()->getCurrentSkin()->getTemplatePath($matches[1]);
			return scPaths::getStoreWebPath() . $templatePath;
		}
		
		/**
		 * Replace the <mi:text> tag contents
		 * 
		 * @param string $string
		 * @return string
		 */
		protected static function preprocessTemplateString($string)
		{
			return preg_replace_callback('#<mi:text( escape="js")?>(.*)</mi:text>#sSU', array('self', 'translateTemplateStringCallback'), $string);
		}
		
		protected static function translateTemplateStringCallback($matches)
		{
			$translated = scInterfaceTexts::getTranslatedText($matches[2]);
			if ($matches[1] == '')
				return $translated;
			return str_replace(array('\'', '"'), array('\\\'', '\\"'), $translated);
		}
		
		/**
		 * Preprocess the sections
		 * Each section is a string
		 * 
		 * @param array $sections the sections
		 * @return array the preprocessed sections
		 */
		protected function preprocessTemplateSections($sections)
		{
			// Variant 1
//			$translateCallback = array('scTemplateParser', 'translateTemplateString');
//			$skinurlCallback = array('scTemplateParser', 'skinurlTemplateString');
//			
//			$preprocessedSection = array();
//			foreach ($sections as $section) {
//				 $section = $this->preprocessTemplateSection($section, $translateCallback);
//				 $section = $this->preprocessTemplateSection($section, $skinurlCallback);
//				 $preprocessedSection[] = $section;
//			}
//			return $preprocessedSection;
//			
			// Variant 2
			$preprocessedSection = array();
			foreach ($sections as $section) {
				$preprocessedSection[] = $this->preprocessTemplateSection($section);
			}
			return $preprocessedSection;
		}
		
		/**
		 * Preprocess template section
		 *
		 * @param array $section
		 * @return array the preprocessed section
		 */
		protected function preprocessTemplateSection($section)
		{
			$preprocessedSection = array();
			$preprocessedSection['name'] = $section['name'];
			if (isset($section['prefix']))
				$preprocessedSection['prefix'] = ($section['prefix'] == '') ? '' : self::preprocessTemplateString($section['prefix']);
			if (isset($section['contents']))
				$preprocessedSection['contents'] = ($section['contents'] == '') ? '' : self::preprocessTemplateString($section['contents']);
			if (isset($section['subsections']))
				$preprocessedSection['subsections'] = $this->preprocessSubsections($section['subsections']);
			return $preprocessedSection;
		}
		
		/**
		 * Recurrsive function to preprocess array of subsections
		 * 
		 * @param array $subsections subsections to preprocess
		 * @return array the preprocessed subsections
		 */
		protected function preprocessSubsections($subsections)
		{
			$preprocessed = array();
			foreach ($subsections as $subsection) {
				$preprocessed[] = $this->preprocessTemplateSection($subsection);
			}
			return $preprocessed;
		}
	}
?>