<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Util static class. Contains various utility methods
	 * @copyright Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	class scUtil {
		
		public static function disablePageCache() 
		{
			header("Cache-Control: no-cache, must-revalidate"); // HTTP/1.1
			header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); // Date in the past			
		}
		
		public static function getMLDbValue($table, $column, $key, $keyValue, $languageId = false)
		{
			if ($languageId === false)
				$languageId = scMultiLanguage::getCurrentLanguageId();
			
			$sql = 'SELECT ' . $column . ' FROM ' . $table . ' WHERE ' . $key . '=? AND LanguageID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($keyValue, $languageId));
			if (count($rows) == 0)
				return '';
			
			return $rows[0][$column];
		}
		
		public static function getDbValue($table, $column, $key, $keyValue, $default = '')
		{
			$sql = 'SELECT ' . $column . ' FROM ' . $table . ' WHERE ' . $key . '=?';
			$rows = miStaticDBUtil::execSelect($sql, array($keyValue));
			if (count($rows) == 0)
				return $default;
			
			return $rows[0][$column];
		}
		
		public static function getDbFirstRow($sql, $params = array())
		{
			$rows = miStaticDBUtil::execSelect($sql, $params);
			if (count($rows) != 1)
				throw new miDBException(sprintf(_i('Expected 1 row. Result rows: %d, query: %s.'), count($rows), $sql), miDBException::EXCEPTION_RECORD_NOT_FOUND);
			
			return $rows[0];
		}
		
		public static function getManufacturersArray()
		{
			$sql = 'SELECT Manufacturers.ManufacturerID, ManufacturerName '.
				'FROM Manufacturers JOIN ManufacturersML USING(ManufacturerID) WHERE LanguageID=' .
				scMultiLanguage::getCurrentLanguageId() . ' ORDER BY ManufacturerSort';
			
			return miUtil::getDBArray($sql, 'ManufacturerID', 'ManufacturerName');
		}
		
		public static function getProductClassesArray()
		{
			$sql = 'SELECT ProductClasses.ProductClassID, ProductClassName' .
				' FROM ProductClasses JOIN ProductClassesML USING(ProductClassID)' .
				' WHERE LanguageID=' . scMultiLanguage::getCurrentLanguageId() .
				' ORDER BY ProductClassName ASC';
			return miUtil::getDBArray($sql, 'ProductClassID', 'ProductClassName');
		}
		
		public static function splitFirstLastName($fullName)
		{
			$pos = strrpos($fullName, ' ');
			if ($pos === false)
				return array($fullName, '');
			
			return array(substr($fullName, 0, $pos), substr($fullName, $pos));
		}
		
		public static function redirect($url)
		{
			header('Location: '.$url);
			exit;
		}
		
		/**
		 * Returns an array of all keys that exist in all elements' rows.
		 * @param array([miDefaultDomainObject]) $domainObjects
		 * @return array
		 * @uses miDefaultDomainObject::getRow()
		 */
		public static function getCommonKeysInRow($domainObjects)
		{
			if (empty($domainObjects)) {
				return array();
			}
			
			foreach ($domainObjects as $domainObject) {
				if (!($domainObject instanceof miDomainObject)) {
					throw new miException(_i('Domain object required.'));
				}
				if (!isset($commonFields)) {
					$commonFields = array_keys($domainObject->getRow());
					continue;
				}
				if (empty($commonFields)) {
					break;
				}
				$commonFields = array_intersect($commonFields, array_keys($domainObject->getRow()));
			}
			
			return $commonFields;
		}
		
	    public static function hasCustomerBoughtProduct($customerId, $productId)
	    {
	    	if (!$customerId || !$productId)
	    		return false;
	    	
	    	$sql = 'SELECT Orders.OrderID FROM Orders' . 
	    		' INNER JOIN OrderItems USING(OrderID)' . 
	    		' WHERE Orders.CustomerID = ? AND OrderItems.ProductID = ? LIMIT 1';
	    	$result = miStaticDBUtil::execSelect($sql, array($customerId, $productId));
			
	    	if (count($result))
	    		return true;
	    	
	    	return false;
	    }
		
		/**
		 * Returns an array of order total line types
		 * with the values the same as the keys.
		 * If the optional $toType is specified all types AFTER it will be excluded.
		 * @todo Move to scModulesFactory where getActiveOrderTotalModules is.
		 * @param int $toType Order total line type to reach
		 * @return array
		 */
		public static function getActiveOrderTotalLineTypes($toType = false)
		{
			$recordset = new miSqlRecordset('OrderTotalModules');
			$recordset->setOrder('OrderTotalModuleSort', 'ASC');
			$recordset->addFilter(new miSqlFilterEqual('OrderTotalModuleIsActive', 1));
			$recordset->setSelectFields(array('OrderTotalModuleType'));
			$types = $recordset->getAllRecords();
			
			$return = array();
			foreach ($types as $type) {
				$return[$type['OrderTotalModuleType']] = $type['OrderTotalModuleType'];
				if (($toType !== false) && ($type['OrderTotalModuleType'] == $toType)) {
					return $return;
				}
			}
			return $return;
		}
	}
?>