<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Xml helper class
	 */
	class scXml {
		const TAG = 'tag';
		const DATA = 'data';
		const CDATA = 'cdata';
		const ATTRIBUTES = 'attributes';
		const CHILDREN = 'children';
		const XML = 'xml';
		
		// OBSOLETE: These are specifically chosen to be invalid XML names
		const OLD_DATA = '';
		const OLD_CDATA = '<cdata>';
		const OLD_ATTRIBUTES = '<attributes>';
		const OLD_XML = '<xml>';
		
		/**
		 * OBSOLETE: Build XML from values
		 * 
		 *
		 * Example:
		 * <example>
		 	$xml = scXml::buildXmlValues(array('operation' => array(
					scXml::OLD_ATTRIBUTES => array('name' => 'someName'),
					'operation' => array(scXml::OLD_ATTRIBUTES => array('soapAction' => '')),
					'input' => array(
						'body' => array(scXml::OLD_ATTRIBUTES => array('use' => 'literal'), scXml::OLD_XML => $someXmlString),
					),
					'output' => array(
						'body' => array(scXml::OLD_DATA => 'data'),
					),
				)));
		 * </example>
		 *
		 * @obsolete Use xmlWriter
		 * @param string|array $values
		 * @return string serialized values as XML
		 */
		public static function buildXmlValues($values)
		{
			// OBSOLETE FUNCTION. DO NOT USE IN NEW CODE!
			
			if (!is_array($values))
				return self::escape($values);
			
			if (isset($values[self::OLD_XML]))
				return $values[self::OLD_XML];
				
			$xml = '';
			foreach ($values as $key => $value) {
				if ($key === self::OLD_ATTRIBUTES)
					continue;
				$attributesXml = (is_array($values[$key]) and isset($values[$key][self::OLD_ATTRIBUTES])) ? self::getAttributes($values[$key][self::OLD_ATTRIBUTES]) : '';
				
				if ($key === self::OLD_DATA) {
					$xml .= self::buildXmlValues($value);
					continue;
				}
				
				if ($key === self::OLD_CDATA) {
					// If the CDATA data contain the CDATA end markup fallback to normal escape
					if (strpos($value, ']]>') !== false) {
						$xml .= self::buildXmlValues($value);
						continue;
					}
						
					$xml .= '<![CDATA[' . $value . ']]>';
					continue;
				}
				
				// Empty value - use short close tag
				if ($value === '') {
					$xml .= '<' . $key . $attributesXml . '/>';
					continue;
				}
				
				$xml .= '<' . $key . $attributesXml . '>';
				$xml .= self::buildXmlValues($value);
				$xml .= '</' . $key . '>';
			}
			return $xml;
		}
		
		
		/**
		 * XML Writer
		 * 
		 * @param array $xmlDefinitions
		 * @param int $level the indent level
		 * @return string
		 */
		public static function xmlWriter($xmlDefinitions, $level = 0)
		{
			if (!is_array($xmlDefinitions))
				return self::escapeData($xmlDefinitions);
			
			$xml = '';
			foreach ($xmlDefinitions as $xmlDefinition) {
				if (!is_array($xmlDefinition))
					throw new miException(_i('xml definition must be an array.'));
				
				// Has pre-generated XML ?
				if (isset($xmlDefinition[self::XML])) {
					$xml .= $xmlDefinition[self::XML];
					continue;
				}
				
				$tag = $xmlDefinition[self::TAG];
				
				// Get tag attributes
				if (isset($xmlDefinition[self::ATTRIBUTES])) {
					if (!is_array($xmlDefinition[self::ATTRIBUTES]))
						throw new miException(_i('Attributes value must be an array.'));
					$attributesXml = self::getAttributes($xmlDefinition[self::ATTRIBUTES]);
				} else {
					$attributesXml = '';
				}

				$xml .= self::getIndent($level);
				
				// Has children ?
				if (isset($xmlDefinition[self::CHILDREN])) {
					$xml .= '<' . $tag . $attributesXml . ">\n";
					$xml .= self::xmlWriter($xmlDefinition[self::CHILDREN], $level + 1);
					$xml .= self::getIndent($level);
					$xml .= '</' . $tag . ">\n";
					continue;
				}
				
				// Has data ?
				if (isset($xmlDefinition[self::DATA])) {
					$xml .= '<' . $tag . $attributesXml . '>';
					$xml .= self::escapeData($xmlDefinition[self::DATA]);
					$xml .= '</' . $tag . ">\n";
					continue;
				}
				
				// Has CDATA ?
				if (isset($xmlDefinition[self::CDATA])) {
					$xml .= '<' . $tag . $attributesXml . '>';
					if (strpos($xmlDefinition[self::CDATA], ']]>') !== false) {	// If the CDATA data contain the CDATA end markup fallback to normal escape
						$xml .= self::escapeData($xmlDefinition[self::CDATA]);
					} else {
						$xml .= '<![CDATA[' . $xmlDefinition[self::CDATA] . ']]>';
					}
					$xml .= '</' . $tag . ">\n";
					continue;
				}
				
				// No children, data, cdata or xml - then should be an empty element
				$xml .= '<' . $tag . $attributesXml . "/>\n";
			}
			return $xml;
		}
		
		/**
		 * Returns indent for the requested level - 2 spaces per level
		 * 
		 * @param int $level
		 * @return string
		 */
		protected static function getIndent($level)
		{
			return str_repeat(' ', $level);
		}
		
		protected static function getAttributes($attributes)
		{
			$xml = array();
			foreach ($attributes as $key => $value) {
				$xml[] = $key . '="' . self::escape($value) . '"';
			}
			return ' ' . implode(' ', $xml);
		}
		
		static protected $_fromArray = array('&', '"', '<', '>', "'");
		static protected $_toArray = array('&amp;', '&quot;', '&lt;', '&gt;', '&apos;');
		
		/**
		 * Static XML escape function
		 * 
		 * @param string|array $values a string, or array of strings to be escaped
		 * @return string|array the escaped string, or array of strings
		 */
		public static function escape($values)
		{
			return str_replace(self::$_fromArray, self::$_toArray, $values);
		}
		
		/**
		 * Static XML escape data function
		 * This function escapes '&', '<' and '>', and is suitable for escaping XML data, but not for escaping attribute value
		 * 
		 * @param string|array $values a string, or array of strings to be escaped
		 * @return string|array the escaped string, or array of strings
		 */
		public static function escapeData($data)
		{
			return htmlspecialchars($data, ENT_NOQUOTES);
		}
	}
?>