<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scArchiveZip {
		/**
		 * Reads the first 4 bytes of the file and checks if the signature matches a zip file
		 * @param string $zipFilename
		 * @return bool
		 */
		public function isZipArchive($zipFilename)
		{
			if (filesize($zipFilename) < 5)
				return false;
			
			$f = fopen($zipFilename, 'rb');
			$signature = fread($f, 4);
			fclose($f);
			
			if ($signature == "PK\03\04")
				return true;
			
			return false; 
		}
		
		/**
		 * Extracts all files from the zip file and returns an array with the contents
		 * @param string $zipFilename
		 * @return array array(ZipEntryName => ZipEntryContents)
		 */
		public function getZipFiles($zipFilename)
		{
			$zipFiles = array();
			
			$zip = zip_open($zipFilename);
			if (!is_resource($zip)) {
				throw new miException(sprintf(_t('Cannot read ZIP archive. Error %d: %s'), $zip, $this->getZipErrorMessage($zip)));
			}
			
			while ($zipEntry = zip_read($zip)) {
				if (!zip_entry_open($zip, $zipEntry)) {
					throw new miException(_t('Cannot open ZIP archive entry.'));
				}
				
				$entryContents = zip_entry_read($zipEntry, zip_entry_filesize($zipEntry));
				$zipFiles[zip_entry_name($zipEntry)] = $entryContents;
				zip_entry_close($zipEntry);
			}
			
			zip_close($zip);
			
			return $zipFiles;
		}
		
		/**
		 * Return text description of zip related error
		 * @param int $errorNo
		 * @return string
		 */
		public static function getZipErrorMessage($errorNo)
		{
			$errors = array(
				1 => 'Multi-disk zip archives not supported.',
				2 => 'Renaming temporary file failed.',
				3 => 'Closing zip archive failed.',
				4 => 'Seek error.',
				5 => 'Read error.',
				6 => 'Write error.',
				7 => 'CRC error.',
				8 => 'Containing zip archive was closed.',
				9 => 'No such file.',
				10 => 'File already exists.',
				11 => 'Can\'t open file.',
				12 => 'Failure to create temporary file.',
				13 => 'Zlib error.',
				14 => 'Memory allocation failure.',
				15 => 'Entry has been changed.',
				16 => 'Compression method not supported.',
				17 => 'Premature EOF.',
				18 => 'Invalid argument.',
				19 => 'Not a zip archive.',
				20=> 'Internal error.',
				21 => 'Zip archive inconsistent.',
				22 => 'Can\'t remove file.',
				23 => 'Entry has been deleted.',
			);
			
			if (!isset($errors[$errorNo]))
				return 'Unknown error.';
			
			return $errors[$errorNo];
		}
	}
?>