<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Administrators Auth class
	 */
	class scAdminAuth extends scAuth {
		const ADMINISTRATOR_ACCESS_LEVEL_DISABLED = 0;
		const ADMINISTRATOR_ACCESS_LEVEL_LIMITED = 1;
		const ADMINISTRATOR_ACCESS_LEVEL_FULL_ACCESS = 2;
		
		public static function getAdministratorAccessLevels()
		{
			return array(
				self::ADMINISTRATOR_ACCESS_LEVEL_DISABLED => _t('Disabled'),
				self::ADMINISTRATOR_ACCESS_LEVEL_LIMITED => _t('Limited'),
				self::ADMINISTRATOR_ACCESS_LEVEL_FULL_ACCESS => _t('Full Access'),
			);
		}
		
		/**
		 * @return scAdminAuth
		 */
		public static function singleton()
		{
			static $obj = null;
			
			if ($obj == null)
				$obj = new scAdminAuth;
			return $obj;
		}
		
		public function logIn($administratorId)
		{
			parent::logIn($administratorId);
			$_SESSION['AdministratorID'] = $administratorId;
		}
		
		public function logOut()
		{
			scSession::singleton()->openSession();
			unset($_SESSION['AdministratorID']);
			parent::logOut();
		}
		
		/**
		 * Checks if the password is correct
		 * @return bool
		 */
		public function isCorrectPassword($admin, $username, $password)
		{
			return parent::checkPasswordHash($admin->get('AdministratorUsername'), $admin->get('AdministratorPassword'), $admin->get('AdministratorPasswordHashMode'), $username, $password);
		}
		
		/**
		 * Returns miSqlRecord with the currently logged in administrator
		 * 
		 * @return miSqlRecord
		 */
		public function getAdministrator()
		{
			static $admin = false;
			
			if ($admin !== false)
				return $admin;
			
			if (!array_key_exists('AdministratorID', $_SESSION))
				return false;
			
			try {
				$admin = new miSqlRecord('Administrators', 'AdministratorID');
				$admin->readPK($_SESSION['AdministratorID']);
			} catch (miDBException $exception) {
				$admin = false;
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND)
					return false;
				throw $exception;
			}
				
			return $admin;
		}
		
		public function getAdministratorId()
		{
			if (!isset($_SESSION['AdministratorID']))
				return false;
			
			return $_SESSION['AdministratorID'];
		}
		
		public function getAdministratorUsername()
		{
			$employee = $this->getAdministrator();
			if ($employee === false)
				return '';
			return $employee->get('AdministratorUsername');
		}

		public function getAdministratorPassword()
		{
			$employee = $this->getAdministrator();
			if ($employee === false)
				return '';
			return $employee->get('AdministratorPassword');
		}

		public function getAdministratorAccessLevel()
		{
			$employee = $this->getAdministrator();
			if ($employee === false)
				return self::ADMINISTRATOR_ACCESS_LEVEL_DISABLED;
			return $employee->get('AdministratorAccessLevel');
		}

		/**
		 * Redirects user to login screen if not authorized
		 */	
		public function checkAuthorization($section = null)
		{
			if ($this->isAuthorized($section))
				return;
			
			// No session
			if ($this->getAdministrator() === false) {
				
				// If the session has expired and this is ajax request - use javascript
				if (miGetParamDefault('load', '') == 'table') {
					echo '<script type="text/javascript">window.location.href="' .
						htmlentities(urlencode('login.php')) . '"</script>';
				} else {
					header('Location: login.php?Referer=' . urlencode($_SERVER['REQUEST_URI']));
				}
				exit;
			}
			
			// No access to section
			header('Location: denied.php?section=' . $section);
			exit;
		}
		
		/**
		 * Verifies that the user has logged in and has clearance
		 */
		public function isAuthorized($section = null)
		{
			scSession::singleton()->openSession();
			
			$admin = $this->getAdministrator();
			if ($admin === false)
				return false;
			
			switch ($admin->get('AdministratorAccessLevel')) {
				case scAdminAuth::ADMINISTRATOR_ACCESS_LEVEL_DISABLED:
					return false;
				
				case scAdminAuth::ADMINISTRATOR_ACCESS_LEVEL_FULL_ACCESS:
					return true;
				
				case scAdminAuth::ADMINISTRATOR_ACCESS_LEVEL_LIMITED:
					if ($section == null)
						return true;
					return $this->isSectionAuthorized($admin->get('AdministratorID'), $section);
			}
			
			// Unknown access level - deny access
			return false;
		}
		
		protected function isSectionAuthorized($administratorId, $section)
		{
			$sql = 'SELECT COUNT(*) AS N FROM AdministratorPrivs WHERE AdministratorID=? AND AdministratorPrivSection=?';
			$rows = miStaticDBUtil::execSelect($sql, array($administratorId, $section));
			return $rows[0]['N'] ? true : false;
		}
	}
?>