<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scApiPrivs {
		
		const API_ENTITY_CATEGORY_CODE = 1;
		const API_ENTITY_MANUFACTURER_CODE = 2;
		const API_ENTITY_PRODUCT_CLASS_CODE = 3;
		const API_ENTITY_PRODUCT_CLASS_ATTRIBUTE_CODE = 4;
		const API_ENTITY_PRODUCT_CLASS_ATTRIBUTE_VALUE_CODE = 5;
		const API_ENTITY_PRODUCT_ATTRIBUTE_VALUE_CODE = 6;
		const API_ENTITY_PRODUCT_CODE = 7;
		const API_ENTITY_ORDER_CODE = 8;
		const API_ENTITY_BUNDLE_PRODUCT_CODE = 9;
		const API_ENTITY_LOCAL_OPTIONS_CODE = 10;
		const API_ENTITY_PRODUCT_IMAGE_CODE = 11;
		const API_ENTITY_CUSTOMER_CODE = 12;
		const API_ENTITY_CUSTOMER_ADDRESS_CODE = 13;
		
		const API_METHOD_GET_CODE = 1;
		const API_METHOD_GET_SYNC_CODE = 2;
		const API_METHOD_MODIFY_CODE = 4;
		const API_METHOD_GET_OPERATIONS_LOG_CODE = 8;
		const API_METHOD_MARK_OPERATIONS_LOG_SYNCED_CODE = 16;
		const API_METHOD_MODIFY_DELETE_CODE = 32;
		
		protected static $_entityNames = array(
			self::API_ENTITY_CATEGORY_CODE => 'Category', // _t('Category')
			self::API_ENTITY_MANUFACTURER_CODE => 'Manufacturer', // _t('Manufacturer')
			self::API_ENTITY_PRODUCT_CLASS_CODE => 'Product Class', // _t('Product Class')
			self::API_ENTITY_PRODUCT_CLASS_ATTRIBUTE_CODE => 'Product Class Attribute', // _t('Product Class Attribute')
			self::API_ENTITY_PRODUCT_CLASS_ATTRIBUTE_VALUE_CODE => 'Product Class Attribute Value', // _t('Product Class Attribute Value')
			self::API_ENTITY_PRODUCT_ATTRIBUTE_VALUE_CODE => 'Product Attribute', // _t('Product Attribute')
			self::API_ENTITY_PRODUCT_CODE => 'Product', // _t('Product')
			self::API_ENTITY_BUNDLE_PRODUCT_CODE => 'Bundle Product', // _t('Bundle Product')
			self::API_ENTITY_ORDER_CODE => 'Order', // _t('Order')
			self::API_ENTITY_LOCAL_OPTIONS_CODE => 'Local Option', // _t('Local Option')
			self::API_ENTITY_PRODUCT_IMAGE_CODE => 'Product Image', // _t('Product Image')
			self::API_ENTITY_CUSTOMER_CODE => 'Customer', // _t('Customer')
			self::API_ENTITY_CUSTOMER_ADDRESS_CODE => 'Customer Address', // _t('Customer Address')
		);
		
		protected static $_methodNames = array(
			self::API_METHOD_GET_CODE => 'Get',
			self::API_METHOD_GET_SYNC_CODE => 'Get Sync',
			self::API_METHOD_MODIFY_CODE => 'Modify',
			self::API_METHOD_GET_OPERATIONS_LOG_CODE => 'Get Operations Log',
			self::API_METHOD_MARK_OPERATIONS_LOG_SYNCED_CODE => 'Mark Synced',
			self::API_METHOD_MODIFY_DELETE_CODE => 'Delete',
		);
		
		protected static $_entityMethods = array(
			self::API_ENTITY_CATEGORY_CODE => 						array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE, self::API_METHOD_MODIFY_DELETE_CODE),
			self::API_ENTITY_MANUFACTURER_CODE => 					array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_PRODUCT_CLASS_CODE => 					array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_PRODUCT_CLASS_ATTRIBUTE_CODE => 		array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_PRODUCT_CLASS_ATTRIBUTE_VALUE_CODE => 	array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_PRODUCT_ATTRIBUTE_VALUE_CODE => 		array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_PRODUCT_CODE => 						array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE, self::API_METHOD_MODIFY_DELETE_CODE),
			self::API_ENTITY_BUNDLE_PRODUCT_CODE => 				array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_ORDER_CODE => 							array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE, self::API_METHOD_GET_OPERATIONS_LOG_CODE, self::API_METHOD_MARK_OPERATIONS_LOG_SYNCED_CODE),
			self::API_ENTITY_LOCAL_OPTIONS_CODE => 					array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_PRODUCT_IMAGE_CODE => 					array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_CUSTOMER_CODE => 						array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
			self::API_ENTITY_CUSTOMER_ADDRESS_CODE => 				array(self::API_METHOD_GET_CODE, self::API_METHOD_GET_SYNC_CODE, self::API_METHOD_MODIFY_CODE),
		);
		
		/**
		 * Return translated to current language entity names
		 * @return array
		 */
		public static function getTranslatedEntityNames()
		{
			$entities = array();
			foreach (self::$_entityNames as $entityCode => $entityName)
				$entities[$entityCode] = scInterfaceTexts::getTranslatedText($entityName);
			
			return $entities;
		}
		
		/**
		 * Return the entity name by code
		 * 
		 * @param int $entityCode
		 * @return string
		 */
		public static function getEntityNameByCode($entityCode)
		{
			return self::$_entityNames[$entityCode];
		}
		
		public static function getEntityMethods()
		{
			return self::$_entityMethods;
		}
		
		public static function getMethodNameByCode($code)
		{
			if (isset(self::$_methodNames[$code]))
				return self::$_methodNames[$code];
		}
		
		public static function hasAccess($privs, $priv)
		{
			if ($privs & $priv)
				return true;
			
			return false;
		}
	}
?>