<?php
	/**
	 * Authentication Class
	 * 
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Class for authorization session management of miProject
	 */
	class scAuth {
		const HASH_MODE_NONE = 0;
		const HASH_MODE_SALTED_HMAC_SHA1 = 1;
		const HASH_MODE_MD5 = 1001;
		
		public static function getHashModes()
		{
			return array(
				self::HASH_MODE_NONE => _t('Cleartext'),
				self::HASH_MODE_SALTED_HMAC_SHA1 => _t('Secure one-way hash (salted HMAC-SHA1)'),
				self::HASH_MODE_MD5 => _t('One-way hash (MD5)'),
			);
		}
		
		/**
		 * Checks if the password is correct
		 * @return bool
		 */
		public function checkPasswordHash($currentEmail, $currentPassword, $currentPasswordHashMode, $email, $password)
		{
			if ($currentEmail != $email)
				return false;
			
			switch ($currentPasswordHashMode) {
				case self::HASH_MODE_NONE:
					if ($currentPassword == $password)
						return true;
					break;
				
				case self::HASH_MODE_SALTED_HMAC_SHA1:
					@list($salt, $hashedPassword)  = explode('$', $currentPassword);
					if (scCryptUtil::hmac($salt, $password, scCryptUtil::HASH_SHA1) == $hashedPassword)
						return true;
					break;
				
				case self::HASH_MODE_MD5:
					if ($currentPassword == md5($password))
						return true;
					break;
			}
			
			return false;
		}
		
		public function hashPassword($password)
		{
			$hashMode = scSettings::getSettingValue(scSettings::SETTING_PASSWORD_HASH_MODE);
			
			switch ($hashMode) {
				case self::HASH_MODE_NONE:
					return $password;
				
				case self::HASH_MODE_SALTED_HMAC_SHA1:
					// 128 bits salt, pretty random. rand() and mt_rand() are seeded by PHP automatically, and they depend on microtime() and getpid() 
					$userAgent = (isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '');
					$salt = md5(mt_rand() . $_SERVER['REMOTE_ADDR'] . $userAgent . rand(), true);
					$salt = base64_encode($salt);
					return $salt . '$' . scCryptUtil::hmac($salt, $password, scCryptUtil::HASH_SHA1);
				
				case self::HASH_MODE_MD5:
					return md5($password);
			}
			throw new miException(_i('Unknown hash mode.'));
		}
		
		public function logIn($id)
		{
			scSession::singleton()->createSession();
		}
		
		/**
		 * Clears the session
		 * If the user is logged out from both the administrator and customer area destroy the session
		 */
		public function logOut()
		{
			if (isset($_SESSION['AdministratorID']) or (isset($_SESSION['CustomerID'])))
				return;
			
			scSession::singleton()->destroySession();
		}
	}
?>