<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCustomerAuth extends scAuth {
		
		/**
		 * @var scCustomerAuth
		 */
		protected static $_instance = null;
		
		/**
		 * Returns the one and only instance of the class
		 *
		 * @return scCustomerAuth
		 */
		public static function singleton()
		{
			if (is_null(self::$_instance))
				self::$_instance = new scCustomerAuth;
			return self::$_instance;
		}
		
		public static function setCustomerAuth(scCustomerAuth $auth)
		{
			self::$_instance = $auth;
		}
		
		public function logIn($customerId)
		{
			parent::logIn($customerId);
			$_SESSION['CustomerID'] = $customerId;
		}
		
		public function logOut()
		{
			scSession::singleton()->openSession();
			// TODO: Hold all Customer data in a subarray of $_SESSION
			unset($_SESSION['CustomerID']);
			unset($_SESSION['Cart']);
			parent::logOut();
		}
		
		public function isLoggedIn()
		{
			if (isset($_SESSION['CustomerID']))
				return true;
			return false;
		}
		
		/**
		 * Checks if the password is correct
		 * @return bool
		 */
		public function isCorrectPassword($user, $email, $password)
		{
			return parent::checkPasswordHash($user->get('CustomerEmail'), $user->get('CustomerPassword'), $user->get('CustomerPasswordHashMode'), $email, $password);
		}
		
		/**
		 * Returns the id of the currently logged user or false if user is not logged in
		 * 
		 * @return int
		 */
		public function getCustomerId()
		{
			if (!array_key_exists('CustomerID', $_SESSION))
				return false;
			
			return $_SESSION['CustomerID'];
		}
		
		/**
		 * Returns scCustomerDomainObject with the currently logged in customer
		 * 
		 * @return scCustomerDomainObject
		 */
		public function getCustomer()
		{
			static $customer = false;
			
			if ($customer !== false)
				return $customer;
			
			if (!array_key_exists('CustomerID', $_SESSION))
				return false;
			
			try {
				$customer = new scCustomerDomainObject();
				$customer->read($_SESSION['CustomerID']);
			} catch (miDBException $exception) {
				$customer = false;
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					unset($_SESSION['CustomerID']);	// No such ID in the database - customer deleted
					return false;
				}
				throw $exception;
			}
			
			return $customer;
		}
		
		public function getCustomerFirstName()
		{
			$customer = $this->getCustomer();
			if ($customer == false)
				return '';
			$billing = $customer->getBillingAddress();
			if (!isset($billing['CustomerAddressFirstName']))
				return ''; 
			return $billing['CustomerAddressFirstName'];
		}

		public function getCustomerLastName()
		{
			$customer = $this->getCustomer();
			if ($customer == false)
				return '';
			$billing = $customer->getBillingAddress();
			if (!isset($billing['CustomerAddressLastName']))
				return ''; 
			return $billing['CustomerAddressLastName'];
		}
		
		/**
		 * Returns the currrently logged-in user customer group
		 * If there isn't a logged-in user, return the default customer group
		 *
		 * @return int customer group id
		 */
		public function getCustomerGroupId()
		{
			$customer = $this->getCustomer();
			if ($customer === false)
				return scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP);
			return $customer->get('CustomerGroupID');
		}
	}
?>