<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scAdminMenus {
		const MENU_TOP_ORDERS = 1;
		const MENU_TOP_PRODUCTS = 2;
		const MENU_TOP_MODULES = 3;
		const MENU_TOP_LOCATIONS = 4;
		const MENU_TOP_CMS = 5;
		const MENU_TOP_CONFIG = 6;
		const MENU_TOP_LANGUAGES = 7;
		const MENU_TOP_REPORTS = 8;
		
		protected $_topMenus;
		protected $_adminMenu;
		
		/**
		 * Constructs the object
		 */
		protected function __construct()
		{
			$this->_topMenus = array(
				self::MENU_TOP_ORDERS => _t('Sales'),
				self::MENU_TOP_PRODUCTS => _t('Products & Categories'),
				self::MENU_TOP_MODULES => _t('Modules'),
				self::MENU_TOP_LOCATIONS => _t('Locations and Taxes'),
				self::MENU_TOP_CMS => _t('Website Content'),
				self::MENU_TOP_CONFIG => _t('Store Configuration'),
				self::MENU_TOP_LANGUAGES => _t('Languages & Currencies'),
				self::MENU_TOP_REPORTS => _t('Reports'),
			);
			
			$this->_adminMenu = array(
				self::MENU_TOP_ORDERS => array(
					array('name' => _t('Orders'), 'url' => 'orders.php'),
					array('name' => _t('Customers'), 'url' => 'customers.php'),
				),
				self::MENU_TOP_PRODUCTS => array(
					array('name' => _t('Product List'), 'url' => 'products.php'),
					array('name' => _t('Product Options'), 'url' => 'option_groups.php'),
					array('name' => _t('Categories'), 'url' => 'categories.php'),
					array('name' => _t('Featured Products'), 'url' => 'featured_products.php'),
					array('name' => _t('Manufacturers'), 'url' => 'manufacturers.php'),
					array('name' => _t('Discounts'), 'url' => 'discounts.php'),
					array('name' => _t('Coupons'), 'url' => 'coupons.php'),
					array('name' => _t('Import/Export Catalog'), 'url' => ''),
				),
				self::MENU_TOP_MODULES => array(
					array('name' => _t('Shipping'), 'url' => 'shipping_modules.php'),
					array('name' => _t('Payment'), 'url' => 'payment_modules.php'),
					array('name' => _t('Order Total'), 'url' => 'order_total_modules.php'),
					array('name' => _t('Currency Rate'), 'url' => 'currency_rate_modules.php'),
				),
				self::MENU_TOP_LOCATIONS => array(
					array('name' => _t('Countries'), 'url' => 'countries.php'),
					array('name' => _t('States'), 'url' => 'states.php'),
					array('name' => _t('Shipping Zones'), 'url' => 'shipping_zones.php'),
					array('name' => _t('Taxes'), 'url' => 'taxes.php'),
				),
				self::MENU_TOP_CMS => array(
					array('name' => _t('Pages'), 'url' => 'pages.php'),
					array('name' => _t('Forms'), 'url' => 'forms.php'),
					array('name' => _t('Boxes'), 'url' => 'boxes.php'),
					array('name' => _t('Menus'), 'url' => 'menus.php'),
					array('name' => _t('Layouts'), 'url' => 'layouts.php'),
					array('name' => _t('Skins'), 'url' => 'skins.php'),
				),
				self::MENU_TOP_CONFIG => array(
					array('name' => _t('General Options'), 'url' => 'general_options.php'),
					array('name' => _t('Store Owner'), 'url' => 'company.php'),
					array('name' => _t('Emails'), 'url' => 'emails.php'),
					array('name' => _t('Security'), 'url' => 'security.php'),
					array('name' => _t('SEO'), 'url' => 'seo.php'),
					array('name' => _t('Administrators'), 'url' => 'administrators.php'),
					array('name' => _t('Localization'), 'url' => 'localization.php'),
					array('name' => _t('Order Statuses'), 'url' => 'order_statuses.php'),
				),
				self::MENU_TOP_LANGUAGES => array(
					array('name' => _t('Languages'), 'url' => 'languages.php'),
					array('name' => _t('Currencies'), 'url' => 'currencies.php'),
				),
				self::MENU_TOP_REPORTS => array(
					array('name' => _t('Reports Center'), 'url' => 'reports.php'),
				),
			);
			
			$this->mergeModuleMenus();
		}
		
		/**
		 * Merge-in menus defined in modules
		 */
		protected function mergeModuleMenus()
		{
			$menuObjects = scModulesFactory::getImplementations('IMenu');
			$menus = array();
			foreach ($menuObjects as $menuObject) {
				$menus = $menuObject->getMenus();
			
				foreach ($menus as $topMenu => $menuItems) {
					$this->_adminMenu[$topMenu] = array_merge($this->_adminMenu[$topMenu], $menuItems);
				}
			}
		}
		
		/**
		 * Return the instance of the admin menus
		 * 
		 * @return scAdminMenus the object
		 */
		public static function singleton()
		{
			static $instance;
			
			if (!isset($instance))
				$instance = new scAdminMenus;
			
			return $instance;
		}
		
		/**
		 * Returns the top menus
		 *
		 * @return array
		 */
		public function getTopMenus()
		{
			return $this->_topMenus;
		}
		
		/**
		 * Return admin menus
		 * 
		 * @return array
		 */
		public function getAdminMenus()
		{
			return $this->_adminMenu;
		}
	}
?>