<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCmsBoxTypes
	{
		const CMS_BOX_TYPE_SYSTEM = 1;
		const CMS_BOX_TYPE_MENU = 2;
		const CMS_BOX_TYPE_XHTML = 3;
		const CMS_BOX_TYPE_NEWSLETTER = 4;
		
		protected static $_cmsBoxModuleClassNames = array(
			self::CMS_BOX_TYPE_XHTML => 'scCmsHtmlBox',
			self::CMS_BOX_TYPE_NEWSLETTER => 'scNewsletterCmsBox',
		);
				
		protected static $_cmsBoxEditableTypes = array(
			self::CMS_BOX_TYPE_XHTML,
			self::CMS_BOX_TYPE_NEWSLETTER,
		);
		
		/**
		 * @return array
		 */
		public static function getAllTypes()
		{
			return array(
				self::CMS_BOX_TYPE_SYSTEM => _t('System Box'),
				self::CMS_BOX_TYPE_MENU => _t('Menu Box'),
				self::CMS_BOX_TYPE_XHTML => _t('XHTML Box'),
				self::CMS_BOX_TYPE_NEWSLETTER => _t('Newsletter Box'),
			);
		}
		
		/**
		 * @param int $typeId
		 * @return string
		 */
		public static function getBoxType($typeId)
		{
			$boxTypes = self::getAllTypes();
			if (!isset($boxTypes[$typeId])) {
				throw new miException(_i('Unknown box type.'));
			}
			return $boxTypes[$typeId];
		}
		
		/**
		 * @return array
		 */
		public static function getAllEditableTypes()
		{
			$editableTypes = array_flip(self::$_cmsBoxEditableTypes);
			return array_intersect_key(self::getAllTypes(), $editableTypes);
		}
		
		/**
		 * @param int $type
		 * @return bool
		 */
		public static function isNameEditableForType($type)
		{
			return in_array($type, self::$_cmsBoxEditableTypes);
		}
		
		/**
		 * @param int $type
		 * @return bool
		 */
		public static function isDeletableType($type)
		{
			return in_array($type, self::$_cmsBoxEditableTypes);
		}
		
		public static function getModuleClassId($typeId)
		{
			return self::readModuleClassId(self::$_cmsBoxModuleClassNames[$typeId]);
		}
		
		protected static function readModuleClassId($className)
		{
			$moduleClassesRecordset = new miSqlRecordset('ModuleClasses');
			$moduleClassesRecordset->addFilter(new miSqlFilterEqual('ModuleClassName', $className));
			$moduleClassesRecordset->addFilter(new miSqlFilterEqual('ModuleClassInterface', 'ICMSBox'));
			
			$foundClasses = $moduleClassesRecordset->getRecordsCount();
			
			if ($foundClasses < 1) {
				throw new miException('Module class not found.');
			}
			
			if ($foundClasses > 1) {
				throw new miException('More than one module class found.');
			}
			
			$moduleClass = $moduleClassesRecordset->getAllRecords();
			return $moduleClass[0]['ModuleClassID'];
		}
	}
?>