<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scDynamicPages {
		protected static $dynamicLayoutId = array();
		
		/**
		 * Returns array with template vars for each dynamic page area
		 * Note: The dynamic page areas are retrieved by path and not by ID, in order to support
		 * dynamic pages added by 3rd parties 
		 */
		public static function getTemplateVars($dynamicPagePath)
		{
			$result = array();
			
			$query = 'SELECT CMSDynamicPageID FROM CMSDynamicPages WHERE CMSDynamicPagePath = ?';
			$resultRS = miStaticDBUtil::execSelect($query, array($dynamicPagePath));
			if (!isset($resultRS[0]['CMSDynamicPageID'])) {
				return $result;
			}
			
			$dynamicPageId = $resultRS[0]['CMSDynamicPageID'];

			// Get areas for the dynamic page
			$dynamicPageAreasRecordset = new scSqlRecordsetML('CMSDynamicPageAreas', 'CMSDynamicPageAreaID');
			$dynamicPageAreasRecordset->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$dynamicPageAreasRecordset->addFilter(new miSqlFilterEqual('CMSDynamicPageID', $dynamicPageId));
			$dynamicPageAreas = $dynamicPageAreasRecordset->getAllRecords();
			foreach ($dynamicPageAreas as $dynamicPageArea) {
				$result['%%DYNAMIC_PAGE_AREA_' . strtoupper($dynamicPageArea['CMSDynamicPageAreaName']) . '%%'] = $dynamicPageArea['CMSDynamicPageAreaText']; 
			}
			return $result;
		}
		
		public static function getLayoutId($dynamicPagePath)
		{
			if (!isset(self::$dynamicLayoutId[$dynamicPagePath])) {
				$record = new miSqlRecord('CMSDynamicPages', 'CMSDynamicPageID');
				$record->read('CMSDynamicPagePath', $dynamicPagePath);
				self::$dynamicLayoutId[$dynamicPagePath] = $record->get('CMSLayoutID');
			}
			return self::$dynamicLayoutId[$dynamicPagePath];
		}
		
		/**
		 * Returns a CMSDynamicPageID TO CMSDynamicPageName mapping
		 */
		public static function getLinkablePageNames()
		{
			$dynPages = self::getLinkablePages();
			
			$pageNames = array();
			foreach ($dynPages as $page) {
				$pageNames[$page['CMSDynamicPageID']] = $page['CMSDynamicPageName'];
			}
			return $pageNames;
		}
		
		/**
		 * Returns a CMSDynamicPageID TO scFriendlyUrl::DYN_PAGE_* mapping
		 */
		public static function getLinkablePageCodes()
		{
			$dynPages = self::getLinkablePages();
			$codes = scFriendlyUrl::getLinkableDynamicPages();
			
			$idsToCodes = array();
			foreach ($dynPages as $page) {
				$idsToCodes[$page['CMSDynamicPageID']] = array_search($page['CMSDynamicPagePath'], $codes);
			}
			return $idsToCodes;
		}
		
		public static function getLinkablePages()
		{
			static $dynPages = null;
			if (!is_null($dynPages)) {
				return $dynPages;
			}
			
			$linkablePages = scFriendlyUrl::getLinkableDynamicPages();
			
			$sql = 'SELECT CMSDynamicPageID, CMSDynamicPagePath, CMSDynamicPageName FROM CMSDynamicPages' .
				' WHERE CMSDynamicPagePath IN ("' . implode('", "', array_map('sql_escape_string', $linkablePages)) . '")';
			$dynPages = miStaticDBUtil::execSelect($sql);
			return $dynPages;
		}
	}
?>