<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scLayout {
		const COLUMN_LEFT = 1;
		const COLUMN_MIDDLE = 2;
		const COLUMN_RIGHT = 3;
		const COLUMN_TOP = 4;
		const COLUMN_BOTTOM = 5;
		
		const AREA_ADMIN = 1;
		const AREA_CUSTOMER = 2;
		
		protected static $_defaultLayout = array();
		protected static $_noBoxes = false;
		
		public static function getAreas()
		{
			return array(
				scLayout::AREA_ADMIN => _t('Admin Area'),
				scLayout::AREA_CUSTOMER => _t('Customer Area')
			);
		}
		
		/**
		 * Returns all layouts
		 *
		 * @return array the layouts array(LayoutID => LayoutName)
		 */
		public static function getLayouts()
		{
			return miUtil::getDBArray('SELECT CMSLayoutID, CMSLayoutName FROM CMSLayouts WHERE CMSLayoutArea=' . self::AREA_CUSTOMER, 'CMSLayoutID', 'CMSLayoutName');
		}
		
		public static function getBoxes($column, $area, $file)
		{
			if ($area == self::AREA_ADMIN)
				return self::getAdminBoxes($column);
			else if ($area == self::AREA_CUSTOMER)
				return self::getCustomerBoxes($file, $column);
			
			return array();
		}
		
		protected static function getBox($moduleId, $cmsBoxId, $moduleClassName, $moduleClassLocation)
		{
			scModulesFactory::loadClassByName($moduleClassName, $moduleClassLocation);
			return new $moduleClassName($moduleId, $cmsBoxId);
		}
		
		protected static function getLayoutId($file)
		{
			if (array_key_exists($file, self::$_defaultLayout))
				return self::$_defaultLayout[$file];
			
			try {
				switch ($file) {
					case 'categories.php':
						$categoryId = miGetParamDefault('CategoryID', 0);
						
						$record = new miSqlRecord('Categories', 'CategoryID');
						$record->readPK($categoryId);
						
						return $record->get('CMSLayoutID');
					
					// Rich text pages
					case 'page.php':
						$pageId = miGetParamDefault('PageID', 0);
						
						$record = new miSqlRecord('CMSRichTextPages', 'CMSRichTextPageID');
						$record->readPK($pageId);
						
						return $record->get('CMSLayoutID');
					
					// Dynamic pages
					default:
						return scDynamicPages::getLayoutId($file);
				}
			} catch (miDBException $exception) {
				// If the category or the page cannot be found
				if ($exception->getCode() != miDBException::EXCEPTION_RECORD_NOT_FOUND)
					throw $exception;
			}
			
			// Return the default layout id
			$record = new miSqlRecord('CMSLayouts', 'CMSLayoutID');
			$record->read('CMSLayoutIsDefault', '1');
			self::$_defaultLayout[$file] = $record->getPK();
			
			return self::$_defaultLayout[$file];
		}
		
		protected static function getAdminBoxes($column)
		{
			static $boxesCache = array();
			
			if (isset($boxesCache[$column]))
				return $boxesCache[$column];
			
			// TODO: V1.1 Admin boxes should be dynamic
			switch ($column) {
				case self::COLUMN_LEFT:
					$boxes = array(
						'quick_links' => scLayout::getBox(0, 0, 'scAdminQuickLinksBox', 'system/blocks/admin/quick_links_box.php')
					);
					
					// Check if "Setup wizard" box is active
					$setupWizardBoxInfo = miStaticDBUtil::execSelect('SELECT CMSBoxID,CMSBoxIsActive FROM CMSBoxes INNER JOIN ModuleClasses USING(ModuleClassID) WHERE ModuleClassName = "scAdminSetupWizardBox" LIMIT 1');
					if (count($setupWizardBoxInfo) > 0 && ((bool) intval($setupWizardBoxInfo[0]['CMSBoxIsActive'])))
						$boxes['setup_wizard'] = scLayout::getBox(0, $setupWizardBoxInfo[0]['CMSBoxID'], 'scAdminSetupWizardBox', 'system/blocks/admin/setup_wizard_box.php');
					
					$boxes['last_orders'] = scLayout::getBox(0, 0, 'scAdminLastOrdersBox', 'system/blocks/admin/last_orders_box.php');
					$boxes['help_zone'] = scLayout::getBox(0, 0, 'scAdminHelpBox', 'system/blocks/admin/help_box.php');
					break;
					
				case self::COLUMN_RIGHT:
				default:
					return array();
			}
			
			$boxStates = self::getAdminBoxesStates();
			foreach ($boxes as $boxId => $box) {
				if (array_key_exists($boxId, $boxStates) and ($boxStates[$boxId] == 0)) {
					$box->setVisible(false);
				}
			}
			
			$boxesCache[$column] = $boxes;
			return $boxes;
		}
		
		
		public static function forceNoBoxes($noBoxes = true)
		{
			self::$_noBoxes = $noBoxes;
		}
		
		public static function getLayoutCssClassNames($layoutId)
		{
			static $cssClassNames = array();
			
			if (!isset($cssClassNames[$layoutId])) {
				$cssClassNames[$layoutId] = scUtil::getDbValue('CMSLayouts', 'CMSLayoutCssClassNames', 'CMSLayoutID', $layoutId);
			}
			
			return $cssClassNames[$layoutId];
		}
		
		protected static function getCustomerBoxes($file, $column)
		{
			// Don't show any boxes during wizard checkout
			// TODO: V1.1 allow custom layout
			if ($file == 'checkout.php') {
				if (miGetParamDefault('ModuleName', '') == 'com.summercart.checkout.wizard')
					return array();
			}
			
			if (self::$_noBoxes)
				return array();
			
			return self::readLayoutBoxes($file, $column);
		}
		
		protected static function readLayoutBoxes($file, $column)
		{
			static $boxesCache = array();
			
			// Check the cache
			if (isset($boxesCache[$file])) {
				if (isset($boxesCache[$file][$column]))
					return $boxesCache[$file][$column];
				return array();
			}
			
			// Read the layout boxes
			$layoutId = self::getLayoutId($file);
			
			$layoutBoxesRs = new miSqlRecordset('CMSLayoutBoxes');
			$layoutBoxesRs->setOrder('CMSLayoutBoxSort', 'ASC');
			$layoutBoxesRs->setSelectFields(array('CMSLayoutBoxes.CMSBoxID', 'ModuleID', 'ModuleClassName', 'ModuleClassLocation', 'CMSLayoutBoxColumn', 'CMSLayoutBoxSort'));
			$layoutBoxesRs->addJoinCondition('INNER', 'CMSBoxes', 'USING(CMSBoxID)');
			$layoutBoxesRs->addJoinCondition('INNER', 'ModuleClasses', 'USING(ModuleClassID)');
			$layoutBoxesRs->addFilter(new miSqlFilterEqual('CMSLayoutID', $layoutId));
			
			$layoutBoxesRs->addFilter(new miSqlFilterNotEqual('CMSBoxIsActive', 0));
			
			$layoutBoxesRows = $layoutBoxesRs->getAllRecords();
			
			$boxesCache[$file] = array();
			foreach ($layoutBoxesRows as $layoutBoxRow) {
				try {
					$boxesCache[$file][$layoutBoxRow['CMSLayoutBoxColumn']][$layoutBoxRow['CMSLayoutBoxSort']] = scLayout::getBox($layoutBoxRow['ModuleID'], $layoutBoxRow['CMSBoxID'],
						$layoutBoxRow['ModuleClassName'], $layoutBoxRow['ModuleClassLocation']);
				} catch (miException $exception) {
					scLog::singleton()->error($exception->getMessage());
				}
			}
			
			foreach ($boxesCache[$file] as &$columnBoxes) {
				ksort($columnBoxes);
			}
			
			if (isset($boxesCache[$file][$column]))
				return $boxesCache[$file][$column];
			return array();
		}
		
		/**
		 * Returns array with visibility of the boxes
		 * 
		 * @return array returns array(boxid => visibility)
		 */
		protected static function getAdminBoxesStates()
		{
			if (!isset($_COOKIE['SCBoxes']))
				return array();
			
			$boxStates = array();
			$boxes = explode('|', $_COOKIE['SCBoxes']);
			foreach ($boxes as $box) {
				$vars = explode('=', $box);
				if (count($vars) != 2)
					continue;
				
				$boxStates[$vars[0]] = $vars[1];
			}
			
			return $boxStates;
		}
		
		protected static function getAdminBox()
		{
		}
	}
?>