<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSkins {
		
		protected $_layout = scLayout::AREA_CUSTOMER;
		protected $_currentSkin;
		protected $_clientAreaSkin;
		
		protected function __construct()
		{
		}
		
		/**
		 * @return scSkins
		 */
		public static function singleton()
		{
			static $instance = null;
			if (!$instance) {
				$instance = new scSkins();
			}
			
			return $instance;
		}
		
		public function setLayout($layout)
		{
			$this->_layout = $layout;
		}
		
		/**
		 * @return scSkin
		 */
		public function getCurrentSkin()
		{
			if (!$this->_currentSkin) {
				$this->_currentSkin = new scSkin($this->_layout);
			}
			return $this->_currentSkin;
		}
		
		/**
		 * @return scSkin
		 */
		public function getClientAreaSkin()
		{
			if (!$this->_clientAreaSkin) {
				$this->_clientAreaSkin = new scSkin(scLayout::AREA_CUSTOMER);
			}
			return $this->_clientAreaSkin;
		}
		
		/**
		 * Returns the root folder where all skins reside
		 * 
		 * @param bool $absolute
		 * @return string
		 */
		public static function getSkinsRoot($absolute = true)
		{
			if (!$absolute) {
				return 'skins/';
			}
			return scPaths::getStoreRootDir() . 'skins/';
		}
	}
	
	class scSkin {
		
		protected $_layout;
		protected $_path;
		protected $_locationMap;
		protected $_parentMap;
		
		public function __construct($layout = scLayout::AREA_CUSTOMER)
		{
			$this->_layout = $layout;
			
			if ($layout == scLayout::AREA_ADMIN) {
				$name = scSettings::SETTING_SKIN_ADMIN;
			} else {
				$name = scSettings::SETTING_SKIN_CUSTOMER;
			}
			$skinId = scSettings::getSettingValue($name);
			
			// init skin maps
			$rows = miStaticDBUtil::execSelect('SELECT SkinID,SkinLocation,SkinParentID FROM Skins');
			foreach ($rows as $row) {
				$this->_locationMap[$row['SkinID']] = $row['SkinLocation'];
				$this->_parentMap[$row['SkinLocation']] = $row['SkinParentID'];
			}
			
			if (!isset($this->_locationMap[$skinId]))
				throw new miException(_t('Skin not found.'));
			
			$skinLocation = $this->_locationMap[$skinId];
			
			$this->_path = $skinLocation;
			if (is_readable($this->getSkinPath()) && is_dir($this->getSkinPath()))
				return;
			
			// try using default skin
			$this->_path = 'default';
			if (is_readable($this->getSkinPath()) && is_dir($this->getSkinPath()))
				return;
			
			throw new miException(sprintf(_t('Skin location "%s" is invalid.'), $skinLocation));
		}
		
		public function getArea()
		{
			return $this->_layout;
		}
		
		/**
		 * Returns a list of CSS files used by the current skin
		 * 
		 * @return array
		 */
		protected function getCssFiles()
		{
			$currentSkin = scSkins::singleton()->getClientAreaSkin();
			$cssGlobPath = $currentSkin->getSkinPath() . 'customer/css/*.css';
			return glob($cssGlobPath);
		}

		/**
		 * Returns a list of CSS files used by the current skin
		 * 
		 * @return string
		 */
		public function getCssFilesForWysiwygEditor()
		{
			$cssFiles = $this->getCssFiles();
			
			$currentSkin = scSkins::singleton()->getClientAreaSkin();
			$currentSkinCssUrl = $currentSkin->getCurrentSkinUrl(true) . 'customer/css/';
			
			$result = array();
			foreach ($cssFiles as $cssFile) {
				$cssFile = scPaths::getAbsoluteUrl($currentSkinCssUrl . basename($cssFile));
				$result[] = '"' . $cssFile . '"';
			}
			return implode(',', $result);
		}
		
		/**
		 * Returns the area path
		 * @return string
		 */
		public function getAreaPath()
		{
			if ($this->_layout == scLayout::AREA_ADMIN)
				return 'admin';
			else
				return 'customer';
		}
		
		/**
		 * Returns absolute filename path to the current skin folder
		 * 
		 * @param bool $absolute
		 * @return string
		 */
		public function getSkinPath($absolute = true)
		{
			return scSkins::getSkinsRoot($absolute) . $this->_path . '/';
		}
		
		/**
		 * Returns the full template path to a template filename.
		 * Searches into parent skins (if any) if the template file cannot be found in the current skin
		 *
		 * @param string $path
		 * @return string
		 * @throws miException if the template file cannot be found
		 */
		public function getTemplatePath($path)
		{
			$root = scSkins::getSkinsRoot();
			
			$currentPath = $this->_path;
			while ($currentPath) {
				if (file_exists($root . $currentPath . '/' . $path))
					return 'skins/' . $currentPath . '/' . $path;
				
				if (!isset($this->_parentMap[$currentPath]))
					throw new miException(sprintf(_i('Template file "%s" not found.'), $path));
				
				$currentPath = $this->_locationMap[$this->_parentMap[$currentPath]];
			}
		}
		
		/**
		 * Returns the skin's URL.
		 * 
		 * If the page being processed was requested through HTTPS
		 * and the layout ( $this->_layout ) is customer's one,
		 * the returned URL will be HTTPS (absolute).
		 * The $_SERVER['HTTPS'] is used to check the above condition.
		 * @link http://www.php.net/manual/en/reserved.variables.server.php
		 * @return string
		 */
		public function getCurrentSkinUrl()
		{
			$skinUrl = $this->getSkinPath(false);
			return scPaths::getStoreWebPath() . $skinUrl;
		}
		
		public function getColumnTemplate($column, $area)
		{
			if ($area == scLayout::AREA_ADMIN) {
				if ($column == scLayout::COLUMN_LEFT)
					return 'admin/column_left.tmpl';
				elseif ($column == scLayout::COLUMN_RIGHT)
					return 'admin/column_right.tmpl';
			} elseif ($area == scLayout::AREA_CUSTOMER) {
				if ($column == scLayout::COLUMN_LEFT)
					return 'customer/column_left.tmpl';
				elseif ($column == scLayout::COLUMN_RIGHT)
					return 'customer/column_right.tmpl';
				elseif ($column == scLayout::COLUMN_TOP)
					return 'customer/column_top.tmpl';
				elseif ($column == scLayout::COLUMN_BOTTOM)
					return 'customer/column_bottom.tmpl';
			}
			throw new miException(_t('Cannot find column template!'));
		}
		
		public function getColumn($column, $area, $file = '')
		{
			if ($file == '') {
				$file = scPaths::getCurrentScriptBaseName();
			}
			
			$boxes = scLayout::getBoxes($column, $area, $file);
			
			$values = array();
			foreach ($boxes as $box)
				$values[] = $box->getBoxContents();
			$section = new miTemplateParserSectionInfo();
			$section->setSectionInfo('rows', count($boxes), array('%%BOXES%%' => $values));
			
			$t = new scTemplateParser();
			$t->setSectionInfos(array($section));
			$t->readTemplate($this->getColumnTemplate($column, $area));
			return $t->templateParse();
		}
		
		public function getTemplateVars()
		{
			$skinId = array_search($this->_path, $this->_locationMap);
			$settings = scSkinSettingsUtil::getSkinSettings($skinId);
			
			$templateVars = array();
			$templateSections = array();
			
			foreach ($settings as $settingName => $setting) {
				$identifier = 'SkinSetting' . $settingName;
				$value = scSkinSettingsUtil::getSkinSettingValueForType($this->_path, $setting['SkinSettingValue'], $setting['SkinSettingType']);
				$templateVars[$identifier] = $value;
			}
			
			return miUIUtil::templetizeRows($templateVars);
		}
	}
?>