<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scUserMenuItem {
		const LINK_TYPE_URL = 0;
		const LINK_TYPE_RICH_TEXT_PAGE = 1;
		const LINK_TYPE_FORM = 2;
		const LINK_TYPE_GALLERY = 3;
		const LINK_TYPE_DYN_PAGE = 4;
		
		public static function getLinkTypes()
		{
			return array(
				self::LINK_TYPE_URL => _t('URL'),
				self::LINK_TYPE_RICH_TEXT_PAGE => _t('Rich Text Page'),
				self::LINK_TYPE_FORM => _t('Form'),
				self::LINK_TYPE_GALLERY => _t('Gallery'),
				self::LINK_TYPE_DYN_PAGE => _t('Dynamic Page'),
			);
		}
	}

	/**
	 * 
	 */
	class scUserMenus {
		const MENU_LOCATION_BOX = 0;
		const MENU_LOCATION_FASTMENU = 1;
		const MENU_LOCATION_TOP = 2;
		const MENU_LOCATION_FOOTER = 3;
		
		protected static $_menuItems = null;
		
		/**
		 * Returns the menu locations
		 * 
		 * @return array
		 */
		public static function getMenuLocations()
		{
			return array(
				self::MENU_LOCATION_BOX => _t('Box'),
				self::MENU_LOCATION_FASTMENU => _t('Fast Menu'),
				self::MENU_LOCATION_TOP => _t('Top'),
				self::MENU_LOCATION_FOOTER => _t('Footer'),
			);
		}
		
		/**
		 * Returns array with menu item rows
		 * 
		 * @param int $menuLocation one of: MENU_LOCATION_BOX, MENU_LOCATION_FASTMENU, MENU_LOCATION_TOP, MENU_LOCATION_FOOTER
		 * @param int $cmsBoxId used if the $menuLocation is self::MENU_LOCATION_BOX, to specify which box to get menu items for (optional)
		 * @return array the rows with the menu items
		 */
		public static function getMenuItems($menuLocation, $cmsBoxId = null)
		{
			// If not loaded load the items
			if (is_null(self::$_menuItems)) {
				$query = 'SELECT * FROM CMSMenus' .
					' LEFT JOIN CMSMenusML ON CMSMenus.CMSMenuID = CMSMenusML.CMSMenuID' .
						' AND CMSMenusML.LanguageID = ?' .
					' LEFT JOIN CMSMenuItems ON CMSMenus.CMSMenuID = CMSMenuItems.CMSMenuID' .
					' JOIN CMSMenuItemsML ON CMSMenuItems.CMSMenuItemID = CMSMenuItemsML.CMSMenuItemID' .
						' AND CMSMenuItemsML.LanguageID = ?' .
					' WHERE CMSMenuIsActive AND CMSMenuItemName <> ""' .
					' ORDER BY CMSMenuItemSort ASC';
				$items = miStaticDBUtil::execSelect($query, array(scMultiLanguage::getCurrentLanguageId(), scMultiLanguage::getCurrentLanguageId()));
				
				// Init friendly urls
				self::setUrls($items);
				// Group the items
				self::$_menuItems = array();
				foreach ($items as $item) {
					if (!empty($item['CMSMenuItemImage'])) {
						$path = '';
						if ($item['CMSMenuItemImageType'] == scMenuItemImageTypes::TYPE_PREDEFINED) {
							$path = scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl() . scMenuItemPredefinedImagesWidget::MENU_ITEMS_IMAGES;
						}
						$item['CMSMenuItemImage'] = scPaths::getAbsoluteUrl($path . $item['CMSMenuItemImage']);
					}
					
					if ($item['CMSMenuLocation'] == self::MENU_LOCATION_BOX)
						self::$_menuItems[$item['CMSMenuLocation']][$item['CMSBoxID']][] = $item;
					else
						self::$_menuItems[$item['CMSMenuLocation']][] = $item;
				}
			}
			
			// Get the items according to the defined parameters
			if ($menuLocation == self::MENU_LOCATION_BOX) {
				if (is_null($cmsBoxId))
					throw new miException(_t('Missing CMSBoxID when getting box menu items!'));
					
				if (array_key_exists(self::MENU_LOCATION_BOX, self::$_menuItems) && array_key_exists($cmsBoxId, self::$_menuItems[self::MENU_LOCATION_BOX]))
					return self::$_menuItems[self::MENU_LOCATION_BOX][$cmsBoxId];
			}
			else {
				if (array_key_exists($menuLocation, self::$_menuItems))
					return self::$_menuItems[$menuLocation];
			}
			
			return array();
		}
		
		/**
		 * Returns array with templetized menu item rows
		 * 
		 * @param int $menuLocation
		 * @return array the rows
		 */
		public static function getMenuItemsTemplateRows($menuLocation, $cmsBoxId = null)
		{
			$menuItemsRows = self::getMenuItems($menuLocation, $cmsBoxId);
			return miUIUtil::templetizeRows(miUIUtil::transposeRows($menuItemsRows));
		}
		
		/**
		 * Find the actual url to use for the menu link
		 * 
		 * @param array $rows
		 */
		protected static function setUrls(&$rows)
		{
			scFriendlyUrl::initFriendlyUrls($rows, scFriendlyUrl::ROW_TYPE_MENU_ITEMS);
		}
		
		/**
		 * Return template section object defining the menu
		 * 
		 * @param int $menuLocation
		 * @return miTemplateParserSectionInfo
		 */
		public static function getMenuTemplateSection($menuLocation, $menuSectionName)
		{
			$menuItems = self::getMenuItemsTemplateRows($menuLocation);
			
			$menuItemHasImageSections = self::getMenuItemsImageSections($menuItems, $menuSectionName);
			
			$menuSection = new miTemplateParserSectionInfo($menuSectionName);
			if (count($menuItems) == 0) {
				$menuSection->setIterationsCount(0);
				return $menuSection;
			}
			
			$menuSection->setIterationsCount(count($menuItems['%%CMSMENUITEMID%%']));
			$menuSection->setValues($menuItems);
			$menuSection->addSubsectionsArray($menuSectionName . 'HasImage', $menuItemHasImageSections);
			
			// Create and assign separator subsection
			$seperatorSubsections = array();
			for ($i=0; $i < count($menuItems['%%CMSMENUITEMID%%'])-1; $i++)
				$seperatorSubsections[] = new miTemplateParserSectionInfo('separator', 1);
			$seperatorSubsections[] = new miTemplateParserSectionInfo('separator', 0);
			$menuSection->addSubsectionsArray('separator', $seperatorSubsections);
			
			return $menuSection;
		}
		
		protected static function getMenuItemsImageSections(&$menuItems, $menuSectionName)
		{
			$menuItemImage = '%%CMSMENUITEMIMAGE%%';
			
			$hasImageSection = new miTemplateParserSectionInfo($menuSectionName . 'HasImage', 1);
			$hasNoImageSection = new miTemplateParserSectionInfo($menuSectionName . 'HasImage', 0);
			
			if (!isset($menuItems[$menuItemImage])) {
				return array();
			}
			
			$hasImageSections = array();
			foreach ($menuItems[$menuItemImage] as $image) {
				if (!empty($image)) {
					$hasImageSections[] = $hasImageSection;
				} else {
					$hasImageSections[] = $hasNoImageSection;
				}
			}
			
			return $hasImageSections;
		}
	}
?>