<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCurrency {
		const CURRENCY_AUTOUPDATE_INTERVAL = 86400; // 24 hours in seconds
		const CURRENCY_RETRY_INTERVAL = 7200; // 2 hours in seconds
		
		protected $_currencyId;
		protected $_row;
		
		protected static $_currenciesCache = array();
		
		/**
		 * Constructs currency object
		 * The param can be either CurrencyID or the 3-letter currency code
		 * 
		 * @param int|string $currencyId the CurrencyID or the 3-letter currency code
		 */
		public function __construct($currencyId)
		{
			$this->_currencyId = $currencyId;
			
			if (isset(self::$_currenciesCache[$currencyId])) {
				$this->_row = self::$_currenciesCache[$currencyId]->_row;
				return;
			}
			
			// Check if we are passed 3-letter currency code
			if ((strlen($currencyId) == 3) and ctype_alpha($currencyId))
				$this->readCurrencyByCode($currencyId);
			else
				$this->readCurrencyById($currencyId);
			
			$this->updateRate();
			
			self::$_currenciesCache[$currencyId] = $this;
		}
		
		protected function readCurrencyById($currencyId)
		{
			$sql = 'SELECT * FROM Currencies JOIN CurrenciesML ON Currencies.CurrencyID=CurrenciesML.CurrencyID ' .
				'WHERE Currencies.CurrencyID = ? AND LanguageID=? ORDER BY CurrencySort';
			$rows = miStaticDBUtil::execSelect($sql, array($currencyId, scMultiLanguage::getCurrentLanguageId()));
			if (empty($rows)) {
				$rows = miStaticDBUtil::execSelect($sql, array($currencyId, scMultiLanguage::getDefaultLanguageId()));
				if (empty($rows)) {
					throw new miDBException(_t('Currency not found.'), miDBException::EXCEPTION_RECORD_NOT_FOUND);
				}
			}
			
			$this->_row = $rows[0];
		}
		
		protected function readCurrencyByCode($currencyCode)
		{
			$sql = 'SELECT * FROM Currencies JOIN CurrenciesML ON Currencies.CurrencyID=CurrenciesML.CurrencyID ' .
				'WHERE Currencies.CurrencyCode = ? AND LanguageID= ? ORDER BY CurrencySort';
			$rows = miStaticDBUtil::execSelect($sql, array($currencyCode, scMultiLanguage::getCurrentLanguageId()));
			if (empty($rows)) {
				throw new miDBException(_t('Currency not found.'), miDBException::EXCEPTION_RECORD_NOT_FOUND);
			}
			$this->_row = $rows[0];
		}
		
		protected function updateRate()
		{
			// If it has been updated recently - do nothing
			if ($this->_row['CurrencyRateLastUpdated'] >= (time() - self::CURRENCY_AUTOUPDATE_INTERVAL))
				return;
			
			if (scSettings::getSettingValue(scSettings::SETTING_LAST_CURRENCYRATE_UPDATE) < (time() - self::CURRENCY_RETRY_INTERVAL)) {
				// Update the rates in the background
				register_shutdown_function(array('scCurrencyUtil', 'updateInBackground'));
			}	
		}
		
		public function getCurrencyId()
		{
			return $this->_row['CurrencyID'];
		}
		
		public function getCurrencySymbolLeft()
		{
			return $this->_row['CurrencySymbolLeft'];
		}
		
		public function getCurrencySymbolRight()
		{
			return $this->_row['CurrencySymbolRight'];
		}
		
		public function getCurrencyDecimalSymbol()
		{
			return $this->_row['CurrencyDecimalSymbol'];
		}
		
		public function getCurrencyThousandsSeparator()
		{
			return $this->_row['CurrencyThousandsSeparator'];
		}
		
		public function getCurrencyNumDecimals()
		{
			return $this->_row['CurrencyNumDecimals'];
		}
		
		public function getCurrencyCode()
		{
			return $this->_row['CurrencyCode'];
		}
		
		public function getCurrencyRate()
		{
			return $this->_row['CurrencyRate'];
		}
		
		public function getCurrencyRow()
		{
			return $this->_row;
		}
	}
	
	/**
	 * A currency that no longer exists.
	 * Used for the purposes of Orders where data is copied.
	 */
	class scUnsupportedCurrency extends scCurrency {
		
		public function __construct($currencyCode)
		{
			// Suppose $currencyCode is actually a code
			$this->_row = array(
				'CurrencyID' => $currencyCode,
				'CurrencyCode' => $currencyCode,
				'CurrencySymbolLeft' => $currencyCode . ' ',
				'CurrencySymbolRight' => '',
				'CurrencyDecimalSymbol' => ',',
				'CurrencyThousandsSeparator' => '',
				'CurrencyNumDecimals' => 2,
				'CurrencyRate' => 1,
			);
			
			self::$_currenciesCache[$currencyCode] = $this;
		}
	}
?>