<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scDiscountService
	{
		/**
		 * The function checks the discount conditions (and disables it if the conditions are no conditions left)
		 * @param string $pTable - The discount table
		 * @param string $pKey - The discount table primary key column
		 * @param string $fTable - The condition table
		 * @param string $fKey - The condition table primary key column
		 * @param string $updateField - The status field in the discount table
		 * @param string $fKeyValue - The value of the field specified by $fKey
		 * 
		 * @return array - Array with ids of the disabled discounts
		 */
		public static function validateDiscounts($pTable, $pKey, $fTable, $fKey, $updateField, $fKeyValue)
		{
			// Find the discounts which should be disabled
			$discounts = self::getDiscountsWithLastCondition($pTable, $pKey, $fTable, $fKey, $updateField, $fKeyValue);
			
			// Disable discounts found
			miStaticDBUtil::execSQL(
				'UPDATE ' . $pTable . ' SET ' . $updateField . ' = 0 ' .
				'WHERE ' . $pKey . ' IN ("' . implode('", "', $discounts) . '")'
			);
			
			return array_values($discounts);
		}
		
		/**
		 * The query returns the discount names for given array with IDs
		 * 
		 * @param string $table - The discount table
		 * @param string $pKey - The discount table primary key column
		 * @param string $nameField - The column name used to get the discount name
		 * @param array $discountIds - Array with ids for which the name should be fetched 
		 */
		public static function getDiscountsNames($table, $pKey, $nameField, $discountIds)
		{
			$sql = 'SELECT ' . $nameField . ' FROM ' . $table . ' WHERE ' . $pKey . ' IN("' . implode('", "', $discountIds) . '")';
			return array_values(miUtil::getDBArray($sql, $nameField, $nameField));
		}
		
		public static function formatDiscountNames($names)
		{
			return '"' . implode('", "', $names) . '"';
		}
		
		protected static function getDiscountsWithLastCondition($pTable, $pKey, $fTable, $fKey, $updateField, $fKeyValue)
		{
			$discountWithCondition = 'SELECT ' . $pTable . '.' . $pKey .
				' FROM ' . $pTable . ' INNER JOIN ' . $fTable . ' USING(' . $pKey . ')' .
				' WHERE ' . $updateField . ' <> 0' .
				' AND ' . $fKey . ' = ' . $fKeyValue .
				' GROUP BY ' . $pTable . '.' . $pKey . ', ' . $fTable . '.' . $fKey . '';
			
			$sql = 'SELECT ' . $pKey . ', COUNT(' . $fKey . ') AS DiscountConditionsCount' .
				' FROM ' . $fTable . ' WHERE ' . $pKey . ' IN (' . $discountWithCondition . ')' .
				' GROUP BY ' . $pKey . ' HAVING DiscountConditionsCount = 1';
			
			return miUtil::getDBArray($sql, $pKey, $pKey);
		}
	}
?>