<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scZones {
		
		/**
		 * Returns the applicable tax zone id for this customer
		 * 
		 * @param int $taxId the tax we need info about
		 * @param array $customerInfo the customer info
		 * @return int the tax zone id, or 0 if no matching tax zone
		 */
		public static function getTaxZoneId($taxId, $customerInfo)
		{
			$customerState = (isset($customerInfo['CustomerShippingState']) ? $customerInfo['CustomerShippingState'] : '');
			$customerCountry = (isset($customerInfo['CustomerShippingCountry']) ? $customerInfo['CustomerShippingCountry'] : '');
			if ($customerCountry == '') {
				$customer = scCustomers::getGuestCustomer();
				$shipping = $customer->getShippingAddress();
				$customerState = $shipping['StateID'];
				$customerCountry = $shipping['CountryID'];
			}
			
			// Check if the states match
			$sql = 'SELECT TaxZones.TaxZoneID FROM TaxZones JOIN TaxZoneStates USING(TaxZoneID) WHERE TaxID=? AND StateID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($taxId, $customerState));
			if (count($rows) > 0)
				return $rows[0]['TaxZoneID'];
			
			// Check if countries match
			$sql = 'SELECT TaxZones.TaxZoneID FROM TaxZones JOIN TaxZoneCountries USING(TaxZoneID) WHERE TaxID=? AND CountryID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($taxId, $customerCountry));
			if (count($rows) > 0)
				return $rows[0]['TaxZoneID'];
			
			// No matching zone
			return 0;
		}
	}
?>