<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Finds all texts that need to be translated
	 */
	class scFindTexts {
		const ENGLISH_LANGUAGE_ID = 1;
		
		protected $_paths = array();
		protected $_extensions = array();
		protected $_parser = array();
		protected $_languageIds = null;
		
		public function __construct()
		{
			$this->_paths[scInterfaceTexts::TEXT_TYPE_TEMPLATE] = array(dirname(__FILE__) . '/../../../../skins/');
			$this->_paths[scInterfaceTexts::TEXT_TYPE_CODE] = array(dirname(__FILE__) . '/../../../../');
			
			$this->_extensions[scInterfaceTexts::TEXT_TYPE_TEMPLATE] = array('.tmpl', '.html');
			$this->_extensions[scInterfaceTexts::TEXT_TYPE_CODE] = array('.php', '.inc');
			
			$this->_parser[scInterfaceTexts::TEXT_TYPE_TEMPLATE] = array($this, 'parseTemplate');
			$this->_parser[scInterfaceTexts::TEXT_TYPE_CODE] = array($this, 'parsePHP');
		}
		
		/**
		 * Finds all files in the specified paths, and their subdirectories
		 *
		 * @param array $paths array of directory names
		 * @return array array of filenames
		 */
		protected function getAllFiles($paths, $extensions)
		{
			$allFiles = array();
			foreach ($paths as $path) {
				
				if (!is_readable($path)) {
					echo $path . ' is not readable.<br/>';
					continue;
				}
					
				$files = scandir($path);
				foreach ($files as $file) {
					if (($file == '.') or ($file == '..') or ($file == 'CVS') or ($file == '.svn') or ($file == 'userfiles'))
						continue;
					
					if (is_file($path . $file)) {
						$pos = strrpos($file, '.');
						if ($pos === false)
							continue;
						
						if (!in_array(substr($file, $pos), $extensions))
							continue;
						
						$allFiles[] = $path . $file;
					} elseif (is_dir($path . $file)) {
						$allFiles = array_merge($allFiles, $this->getAllFiles(array($path . $file . DIRECTORY_SEPARATOR), $extensions));
					}
				}
			}
			return $allFiles;
		}
		
		/**
		 * Finds all translatable texts in a given template
		 */
		protected function parseTemplate($text)
		{
			$result = preg_match_all('#<mi:text>(.+)</mi:text>#sUu', $text, $translatableTexts);
			if ($result > 0)
				return $translatableTexts[1];
			
			return array();
		}

		protected function parsePHP($text)
		{
			$result = array();
			if (preg_match_all('#_t\(([\'"].+[\'"])\)#sUu', $text, $translatableTexts) > 0) {
				foreach ($translatableTexts[1] as $quotedText) {
					eval('$quotedText = ' . $quotedText . ';');
					$result[] = $quotedText;
				}
			}
			
			if (preg_match_all('#[\'"]displayName[\'"] +=> +[\'"](.+)[\'"]#sUu', $text, $translatableTexts) > 0) {
				$result = array_merge($result, $translatableTexts[1]);
			}
			
			return $result;
		}

		protected function findTexts($textType)
		{
			$allFiles = $this->getAllFiles($this->_paths[$textType], $this->_extensions[$textType]);
			$translatableTexts = array();
			
			foreach ($allFiles as $file) {
				$contents = file_get_contents($file);
				$translatableTexts = array_merge($translatableTexts, call_user_func($this->_parser[$textType], $contents));
			}
			
			$translatableTexts = array_unique($translatableTexts);
			return $translatableTexts;
		}
		
		protected function findNewTexts($foundTexts, $currentTexts)
		{
			$newTexts = array();
			foreach ($foundTexts as $foundText) {
				if (!isset($currentTexts[$foundText]))
					$newTexts[] = $foundText;
			}
			return $newTexts;
		}
		
		protected function findObsoleteTexts($foundTexts, $currentTexts)
		{
			$obsoleteTextIds = array();
			foreach ($currentTexts as $currentText => $currentTextId) {
				if (!in_array($currentText, $foundTexts))
					$obsoleteTextIds[] = $currentTextId;
			}
			return $obsoleteTextIds;
		}
		
		protected function insertTexts($texts, $templateTexts, $codeTexts)
		{
			$languageIds = $this->getAllLanguageIds();
			
			$multiValues = array();
			foreach ($texts as $text) {
				$textType = in_array($text, $templateTexts) ? scInterfaceTexts::TEXT_TYPE_TEMPLATE : 0;
				$textType += in_array($text, $codeTexts) ? scInterfaceTexts::TEXT_TYPE_CODE : 0;
				
				$values = array(
					'TextOriginal' => $text,
					'TextOriginalHash' => sprintf('%u', crc32($text)),
					'TextType' => $textType
				);
				$textId = miStaticDBUtil::execInsert('Texts', $values);
				
				// Insert placeholder translations for all availabe languages
				foreach ($languageIds as $languageId) {
					$multiValues[] = array(
						$textId,
						$languageId,
						sprintf('%u', crc32($text)),
						($languageId == self::ENGLISH_LANGUAGE_ID) ? $text : '',
						0
					);
				}
			}
			
			miStaticDBUtil::multiValueListsInsert('TextsTranslated', array('TextID', 'LanguageID', 'TextOriginalHash', 'TextTranslated', 'TextIsTranslated'), $multiValues);
		}
		
		protected function getAllLanguageIds()
		{
			if ($this->_languageIds === null) {
				$sql = 'SELECT LanguageID FROM Languages';
				$this->_languageIds = miUtil::getDBArray($sql, 'LanguageID', 'LanguageID');
			}
			return $this->_languageIds;
		}
		
		protected function deleteTexts($textIds)
		{
			foreach ($textIds as $textId) {
				miStaticDBUtil::execDelete('Texts', 'TextID', $textId);
				miStaticDBUtil::execDelete('TextsTranslated', 'TextID', $textId);
			}
		}
		
		/**
		 * Perform the text finding
		 * @return array statistics
		 */
		public function process()
		{
			$templateTexts = $this->findTexts(scInterfaceTexts::TEXT_TYPE_TEMPLATE);
			$codeTexts = $this->findTexts(scInterfaceTexts::TEXT_TYPE_CODE);
			$allTexts = array_unique(array_merge($templateTexts, $codeTexts));
			
			$sql = 'SELECT * FROM Texts';
			$currentTexts = miUtil::getDBArray($sql, 'TextOriginal', 'TextID');
			
			$newTexts = $this->findNewTexts($allTexts, $currentTexts);
			
			$obsoleteTextIds = $this->findObsoleteTexts($allTexts, $currentTexts);
			
			$this->insertTexts($newTexts, $templateTexts, $codeTexts);
			$this->deleteTexts($obsoleteTextIds);
			
			return array(
				'TemplateTextsCount' => count($templateTexts),
				'CodeTextsCount' => count($codeTexts),
				'AllUniqueTextsCount' => count($allTexts),
				
				'PreviousTextsCount' => count($currentTexts),
				'NewTextsCount' => count($newTexts),
				'ObsoleteTextsCount' => count($obsoleteTextIds),
			);
		}
	}
?>