<?php
	/**
	 * Manufacturers cache class
	 * 
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scManufacturersCache {
		
		protected static $_prepared = array();
		
		protected static $_cache = array();
		
		/**
		 * Add items that will be used later in the script
		 * @param int $itemsType
		 * @param array $itemsIds
		 * @param int $languageId
		 */
		public static function prepareItems($manufacturerIds)
		{
			$manufacturerIds = array_map('intval', $manufacturerIds);
			
			self::$_prepared = array_merge(self::$_prepared, $manufacturerIds);
			self::$_prepared = array_unique(self::$_prepared);
		}
		
		/**
		 * @return scManufacturerDomainObject
		 */
		public static function getDomainObject($manufacturerId)
		{
			$object = new scManufacturerDomainObject();
			$object->setRow(self::getData($manufacturerId));
			return $object;
		}
		
		public static function getData($manufacturerId)
		{
			// Check if there is data in the cache and use it.
			if (isset(self::$_cache[$manufacturerId])) {
				return self::$_cache[$manufacturerId];
			}
			
			// Check if the requested item is already preapred.
			// Add it to the prepare list if it is not.
			if (!in_array($manufacturerId, self::$_prepared)) {
				self::$_prepared[] = intval($manufacturerId);
			}
			
			$mlFields = array(
				'ManufacturerName',
				'ManufacturerDescription',
				'ManufacturerSEOTitle',
			);
			$recordset = new miSqlRecordset('ManufacturersML');
			$recordset->addJoinCondition('RIGHT', 'Manufacturers', 'USING(ManufacturerID)');
			$recordset->addFilter(new miSqlFilterIn('Manufacturers.ManufacturerID', self::$_prepared));
			$recordset->setSelectFields(array('Manufacturers.*'));
			$recordset->addSelectField('ManufacturersML.LanguageID');
			foreach ($mlFields as $mlField) {
				$recordset->addSelectField('ManufacturersML.' . $mlField);
			}
			$result = $recordset->getAllRecords();
			
			// Store FALSE values for all the items that were just fetched.
			// Those values are later (a few rows later) replaced with real values where available.
			foreach (self::$_prepared as $preparedId) {
				self::$_cache[$preparedId] = false;
			}
			self::$_prepared = array();
			
			if (empty($result)) {
				// Nothing to cache.
				return false;
			}
			
			// Add the results to the cache.
			$manufacturers = array();
			foreach ($result as $row) {
				if (!isset($manufacturers[$row['ManufacturerID']])) {
					$manufacturers[$row['ManufacturerID']] = array_diff_key($row, array_flip($mlFields));
				}
				foreach ($mlFields as $fieldName) {
					$manufacturers[$row['ManufacturerID']][$fieldName][$row['LanguageID']] = $row[$fieldName];
				}
			}
			self::$_cache = $manufacturers + self::$_cache;
			return self::$_cache[$manufacturerId];
		}
	}
?>