<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOrderStatuses {
		
		static protected $_orderStatuses = array(
			scOrderStatus::ORDER_STATUS_UNFINISHED,
			scOrderStatus::ORDER_STATUS_PAYMENT_FAILED,
			scOrderStatus::ORDER_STATUS_PAYMENT_PENDING,
			scOrderStatus::ORDER_STATUS_NEW,
			scOrderStatus::ORDER_STATUS_IN_PROGRESS,
			scOrderStatus::ORDER_STATUS_CANCELLED,
			scOrderStatus::ORDER_STATUS_ON_HOLD,
			scOrderStatus::ORDER_STATUS_DELIVERED,
			scOrderStatus::ORDER_STATUS_RETURNED,
			scOrderStatus::ORDER_STATUS_QUEUED
		);
		
		/**
		 * Order status names as extracted from DB.
		 * @var string[]
		 */
		static protected $_cachedOrderStatuses = null;
		
		/**
		 * Order statuses that are used in orders (there are orders with them set)
		 * @var string[]
		 */
		static protected $_cachedOrderStatusesInUse = null;
		
		/**
		 * Including inactive ones
		 * @var string[]
		 */
		static protected $_allOrderStatusNames = null;
		
		/**
		 * Should notifications be sent on status
		 * @var bool[]
		 */
		static protected $_orderStatusNotifications = null;
		
		/**
		 * Order status to scOrderStatusInventoryAction mapping 
		 * @var int[]
		 */
		static protected $_orderStatusInventoryActions = null;
		
		/**
		 * @return scOrderStatuses
		 */
		public static function singleton()
		{
			static $obj = null;
			
			if ($obj == null)
				$obj = new scOrderStatuses;
			
			return $obj;
		}
		
		/**
		 * Return the order status name
		 *
		 * @param int $orderStatusId
		 * @return string the order status name
		 */
		public function getOrderStatusName($orderStatusId)
		{
			$this->loadOrderStatuses();
			return self::$_allOrderStatusNames[$orderStatusId];
		}
		
		/**
		 * @return bool
		 */
		public function shouldSendNotification($orderStatusId)
		{
			$this->loadOrderStatuses();
			return (bool)self::$_orderStatusNotifications[$orderStatusId];
		}
		
		/**
		 * @return int
		 */
		public function getOrderStatusInventoryAction($orderStatusId)
		{
			$this->loadOrderStatuses();
			return self::$_orderStatusInventoryActions[$orderStatusId];
		}
		
		/**
		 * Return the predefined order statues
		 *
		 * @return array
		 */
		public function getPredefinedOrderStatusIds()
		{
			return self::$_orderStatuses;
		}
		
		/**
		 * Return all order statuses
		 *
		 * @return array
		 */
		public function getOrderStatuses()
		{
			$this->loadOrderStatuses();
			return self::$_cachedOrderStatuses;
		}
		
		/**
		 * Return all order statuses
		 *
		 * @return array
		 */
		public function getAllOrderStatuses()
		{
			$this->loadOrderStatuses();
			return self::$_allOrderStatusNames;
		}
		
		public function loadOrderStatuses()
		{
			if (!is_null(self::$_cachedOrderStatuses)) {
				return;
			}
			$languageId = scMultiLanguage::getCurrentLanguageId();
			$query = 'SELECT * FROM OrderStatuses JOIN OrderStatusesML USING(OrderStatusID)' .
				' WHERE LanguageID=' . $languageId . ' ORDER BY OrderStatusSort';
			$result = miStaticDBUtil::execSelect($query);
			
			self::$_cachedOrderStatuses = array();
			self::$_allOrderStatusNames = array();
			self::$_orderStatusNotifications = array();
			self::$_orderStatusInventoryActions = array();
			foreach ($result as $row) {
				self::$_allOrderStatusNames[$row['OrderStatusID']] = $row['OrderStatusName'];
				self::$_orderStatusNotifications[$row['OrderStatusID']] = $row['OrderStatusSendNotification'];
				self::$_orderStatusInventoryActions[$row['OrderStatusID']] = $row['OrderStatusInventoryAction'];
				if ($row['OrderStatusIsActive']) {
					self::$_cachedOrderStatuses[$row['OrderStatusID']] = $row['OrderStatusName'];
				}
			}
		}
	}
?>