<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Orders cache / Identity Map
	 */
	class scOrdersCache {
		
		/**
		 * @var scOrder[]
		 */
		protected static $_orders = array();
		
		/**
		 * @var scOrderItem[]
		 */
		protected static $_orderItems = array();
		
		/**
		 * @var scOrderItem[]
		 */
		protected static $_unsortedOrderItems = array();
		
		/**
		 * @var scOrderItemOption[]
		 */
		protected static $_orderItemOptions = array();
		
		/**
		 * @var scOrderItemLocalOption[]
		 */
		protected static $_orderItemLocalOptions = array();
		
		/**
		 * Prepare an order's data so it can later be retrieved by not making separate queries.
		 * @param int $orderId
		 */
		public static function fullLoadOrderData($orderId)
		{
			// Load order
			self::getOrder($orderId);
			
			// Load items
			self::loadOrderItemsData($orderId);
		}
		
		public static function getOrder($orderId)
		{
			if (!isset(self::$_orders[$orderId])) {
				self::$_orders[$orderId] = new scOrder($orderId);
			}
			return self::$_orders[$orderId];
		}
		
		public static function getOrderItems($orderId, $parentOrderItemId = null)
		{
			$key = $orderId;
			if (!is_null($parentOrderItemId)) {
				$key .= '-' . $parentOrderItemId;
			}
			
			if (isset(self::$_orderItems[$key])) {
				return self::$_orderItems[$key];
			}
			
			$recordset = new miSqlRecordset('OrderItems');
			$recordset->addFilter(new miSqlFilterEqual('OrderID', $orderId));
			$orderItems = $recordset->getAllRecords();
			
			self::$_orderItems[$orderId] = array();
			foreach ($orderItems as $orderItemData) {
				$orderItem = new scOrderItem();
				$orderItem->setRow($orderItemData);
				if (isset(self::$_orders[$orderId])) {
					$orderItem->setOrder(self::$_orders[$orderId]);
				}
				$itemKey = $orderId;
				if (!is_null($orderItem->get('ParentOrderItemID'))) {
					$itemKey .= '-' . $orderItem->get('ParentOrderItemID');
				}
				self::$_orderItems[$itemKey][$orderItem->get('OrderItemID')] = $orderItem;
				self::$_unsortedOrderItems[$orderItem->get('OrderItemID')] = $orderItem;
			}
			
			if (isset(self::$_orderItems[$key])) {
				return self::$_orderItems[$key];
			}
			
			return array();
		}
		
		/**
		 * @param int $orderItemId
		 * @return scOrderItem
		 */
		public static function getOrderItem($orderItemId)
		{
			if (isset(self::$_unsortedOrderItems[$orderItemId])) {
				return self::$_unsortedOrderItems[$orderItemId];
			}
			
			throw new miException(_i('Order item not loaded yet.'));
		}
		
		/**
		 * @param scOrderItem $orderItem
		 * @return scOrderItemOption[]
		 */
		public static function getOrderItemOptions(scOrderItem $orderItem)
		{
			$orderId = $orderItem->get('OrderID');
			$orderItemId = $orderItem->get('OrderItemID');
			if (isset(self::$_orderItemOptions[$orderId][$orderItemId])) {
				return self::$_orderItemOptions[$orderId][$orderItemId];
			}
			return array();
		}
		
		/**
		 * @param scOrderItem $orderItem
		 * @return scOrderItemLocalOption[]
		 */
		public static function getOrderItemLocalOptions(scOrderItem $orderItem)
		{
			$orderId = $orderItem->get('OrderID');
			$orderItemId = $orderItem->get('OrderItemID');
			if (isset(self::$_orderItemLocalOptions[$orderId][$orderItemId])) {
				return self::$_orderItemLocalOptions[$orderId][$orderItemId];
			}
			return array();
		}
		
		protected static function loadOrderItemsData($orderId)
		{
			self::getOrderItems($orderId);
			
			self::loadOrderItemsOptionsData($orderId);
			self::loadOrderItemsLocalOptionsData($orderId);
		}
		
		protected static function loadOrderItemsOptionsData($orderId)
		{
			$optionsRecordset = new miSqlRecordset('OrderItemOptions');
			$optionsRecordset->addJoinCondition('INNER', 'OrderItems', 'USING(OrderItemID)');
			$optionsRecordset->setSelectFields(array('OrderItemOptions.*'));
			$optionsRecordset->addFilter(new miSqlFilterEqual('OrderID', $orderId));
			
			foreach ($optionsRecordset->getAllRecords() as $option) {
				$optionObj = new scOrderItemOption();
				$optionObj->setRow($option);
				$optionObj->setOrderItem(self::getOrderItem($option['OrderItemID']));
				self::$_orderItemOptions[$orderId][$option['OrderItemID']][$option['OrderItemOptionID']] = $optionObj;
			}
		}
		
		protected static function loadOrderItemsLocalOptionsData($orderId)
		{
			$optionsRecordset = new miSqlRecordset('OrderItemLocalOptions');
			$optionsRecordset->addJoinCondition('INNER', 'OrderItems', 'USING(OrderItemID)');
			$optionsRecordset->setSelectFields(array('OrderItemLocalOptions.*'));
			$optionsRecordset->addFilter(new miSqlFilterEqual('OrderID', $orderId));
			
			foreach ($optionsRecordset->getAllRecords() as $option) {
				$optionObj = new scOrderItemLocalOption();
				$optionObj->setRow($option);
				$optionObj->setOrderItem(self::getOrderItem($option['OrderItemID']));
				self::$_orderItemLocalOptions[$orderId][$option['OrderItemID']][$option['OrderItemLocalOptionID']] = $optionObj;
			}
		}
	}
?>