<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * The result from calling a payment module
	 */
	class scPaymentProcessResult {
		protected $_orderId;
		protected $_paymentStatus = scPaymentStatus::PAYMENT_STATUS_NONE;
		protected $_transactionAmount;
		protected $_transactionCurrencyCode;
		protected $_transactionCode;
		protected $_transactionNote;
		protected $_paymentNotes = '';
		protected $_validatorErrors = array();
		protected $_details = array();
		
		protected $_thirdPartyRedirectUrl = '';
		protected $_thirdPartyRedirectParams = array();
		protected $_thirdPartyRedirectMethod = '';
		protected $_thirdPartyRedirectOptions = array();
		
		public function __construct($orderId = 0, $transactionAmount = 0, $transactionCurrencyCode = '', $transactionCode = '', $transactionNote = '')
		{
			$this->_orderId = $orderId;
			$this->_transactionAmount = $transactionAmount;
			$this->_transactionCurrencyCode = $transactionCurrencyCode;
			$this->_transactionCode = $transactionCode;
			$this->_transactionNote = $transactionNote;
		}
		
		/**
		 * Marks that the payment process should proceed with redirect to a third party processor
		 * If the url is empty there is no redirect, and the payment is handled directly in processPayment() 
		 * 
		 * @param string $url the url to redirect to
		 * @param array $params the params sent to the url
		 * @param string $method the redirect method - get, post or iframe (optional), post by default 
		 * @param array $options redirect options, depend on the redirect method
		 */
		public function setThirdPartyRedirect($url, $params, $method = 'post', $options = array())
		{
			$this->_thirdPartyRedirectUrl = $url;
			$this->_thirdPartyRedirectParams = $params;
			$this->_thirdPartyRedirectMethod = $method;
			$this->_thirdPartyRedirectOptions = $options;
		}
		
		/**
		 * Returns array with the third party redirect info
		 */
		public function getThirdPartyRedirect()
		{
			return array($this->_thirdPartyRedirectUrl, $this->_thirdPartyRedirectParams, $this->_thirdPartyRedirectMethod, $this->_thirdPartyRedirectOptions);
		}
		
		public function setOrderId($orderId)
		{
			$this->_orderId = $orderId;
		}
		
		public function getOrderId()
		{
			return $this->_orderId;
		}
		
		public function setPaymentStatus($paymentStatus)
		{
			$this->_paymentStatus = $paymentStatus;
		}
		
		public function getPaymentStatus()
		{
			return $this->_paymentStatus;
		}
		
		public function setTransactionAmount($transactionAmount)
		{
			$this->_transactionAmount = $transactionAmount;
		}
		
		public function getTransactionAmount()
		{
			return $this->_transactionAmount;
		}
		
		public function setTransactionCurrencyCode($transactionCurrencyCode)
		{
			$this->_transactionCurrencyCode = $transactionCurrencyCode;
		}
		
		public function getTransactionCurrencyCode()
		{
			return $this->_transactionCurrencyCode;
		}
		
		public function setTransactionCode($transactionCode)
		{
			$this->_transactionCode = $transactionCode;
		}
		
		public function getTransactionCode()
		{
			return $this->_transactionCode;
		}
		
		/**
		 * Sets any transacation related notes
		 * Usually these come from the payment processor
		 *
		 * @param string $transactionNote
		 */
		public function setTransactionNote($transactionNote)
		{
			$this->_transactionNote = $transactionNote;
		}
		
		public function getTransactionNote()
		{
			return $this->_transactionNote;
		}
		
		/**
		 * Sets a payment related message to be displayed to the customer at checkout success page
		 *
		 * @param string $paymentNotes
		 */
		public function setPaymentNotes($paymentNotes)
		{
			$this->_paymentNotes = $paymentNotes;
		}
		
		public function getPaymentNotes()
		{
			return $this->_paymentNotes;
		}
		
		/**
		 * Sets transaction detail
		 * 
		 * @param string $detailName
		 * @param string $detailValue
		 */
		public function setTransactionDetail($detailName, $detailValue)
		{
			$this->_details[$detailName] = $detailValue;
		}
		
		public function getTransactionDetail($detailName)
		{
			if (isset($this->_details[$detailName]))
				return $this->_details[$detailName];
			else
				return null;
		}
		
		public function setAllTransactionDetails($details)
		{
			$this->_details = $details;
		}
		
		public function getAllTransactionDetails()
		{
			return $this->_details;
		}
		
		/**
		 * Returns if there are validator errors
		 * 
		 * @return boolean
		 */
		public function hasValidatorErrors()
		{
			return !empty($this->_validatorErrors);
		}
		
		/**
		 * Add an error
		 * If $fieldName is set then the error is associated with that field
		 * 
		 * @param string $errorMessage
		 * @param string $fieldName (optional)
		 */
		public function addValidatorError($errorMessage, $fieldName = '')
		{
			$this->_validatorErrors[$fieldName] = $errorMessage;
		}
		
		/**
		 * Return the validator errors
		 *
		 * @return array validator errors in the format array(fieldName => errorMessage) 
		 */
		public function getValidatorErrors()
		{
			return $this->_validatorErrors;
		}
	}
?>