<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scPaymentStatus {
		const PAYMENT_STATUS_NONE = 0;		// There is no payment
		const PAYMENT_STATUS_AT_PROCESSOR = 1;	// We are waiting for the processor to decide
		const PAYMENT_STATUS_DECLINED = 2;	// Payment declined
		const PAYMENT_STATUS_PENDING = 3;	// Received payment pending from processor
		const PAYMENT_STATUS_AUTH = 4;		// Received authorization confirmation from processor
		const PAYMENT_STATUS_CAPTURE = 5;	// Received capture confirmation from processor
		const PAYMENT_STATUS_PARTIAL_REFUND = 6;
		const PAYMENT_STATUS_REFUND = 7;
		const PAYMENT_STATUS_CHARGEBACK = 8;
		const PAYMENT_STATUS_OTHER = 9;		// Unknown payment status received from processor
		const PAYMENT_STATUS_OFFLINE = 10;	// The payment will happen offline
		const PAYMENT_STATUS_PARTIAL_CAPTURE = 11;	// Some of the amount has been captured
		const PAYMENT_STATUS_VOID = 12;	// Payment voided
		
		protected static $_successStatuses = array(self::PAYMENT_STATUS_AUTH, self::PAYMENT_STATUS_CAPTURE, self::PAYMENT_STATUS_OFFLINE);
		
		public static function getPaymentStatuses()
		{
			return array(
				self::PAYMENT_STATUS_NONE => '',
				self::PAYMENT_STATUS_AT_PROCESSOR => _t('Pending payment info from processor'),
				self::PAYMENT_STATUS_DECLINED => _t('Declined'),
				self::PAYMENT_STATUS_PENDING => _t('Pending'),
				self::PAYMENT_STATUS_AUTH => _t('Auth'),
				self::PAYMENT_STATUS_CAPTURE => _t('Capture'),
				self::PAYMENT_STATUS_PARTIAL_REFUND => _t('Partial Refund'),
				self::PAYMENT_STATUS_REFUND => _t('Refund'),
				self::PAYMENT_STATUS_CHARGEBACK => _t('Chargeback'),
				self::PAYMENT_STATUS_OTHER => _t('Other'),
				self::PAYMENT_STATUS_OFFLINE => _t('Offline'),
				self::PAYMENT_STATUS_PARTIAL_CAPTURE => _t('Partial Capture'),
				self::PAYMENT_STATUS_VOID => _t('Void'),
			);
		}
		
		/**
		 * Return if the payment status indicates order success
		 *
		 * @param int $paymentStatus
		 * @return boolean
		 */
		public static function isOrderSuccess($paymentStatus)
		{
			return in_array($paymentStatus, self::$_successStatuses);
		}
		
		/**
		 * Returns if the payment status indicates that the order payment is pending
		 *
		 * @param int $paymentStatus
		 * @return boolean
		 */
		public static function isPaymentPending($paymentStatus)
		{
			return ($paymentStatus == scPaymentStatus::PAYMENT_STATUS_PENDING) or ($paymentStatus == scPaymentStatus::PAYMENT_STATUS_AT_PROCESSOR);
		}
	}
?>