<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scTransaction {
		protected $_orderId;
		protected $_moduleId;
		protected $_transactionAmount;
		protected $_transactionCurrencyCode;
		protected $_transactionCode;
		protected $_paymentStatus;
		protected $_note;
		protected $_transactionId = null;
		
		protected $_details = array();
		
		public function __construct($orderId = 0, $moduleId = 0, $transactionAmount = 0, $transactionCurrencyCode = '',
			$transactionCode= '', $paymentStatus = scPaymentStatus::PAYMENT_STATUS_NONE, $note = '')
		{
			$this->_orderId = $orderId;
			$this->_moduleId = $moduleId;
			$this->_transactionAmount = $transactionAmount;
			$this->_transactionCurrencyCode = $transactionCurrencyCode;
			$this->_transactionCode = $transactionCode;
			$this->_paymentStatus = $paymentStatus;
			$this->_note = $note;
		}
		
		public function setOrderId($orderId)
		{
			$this->_orderId = $orderId;
		}
		
		public function getOrderId()
		{
			return $this->_orderId;
		}
		
		public function setModuleId($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getModuleId()
		{
			return $this->_moduleId;
		}
		
		public function setTransactionAmount($transactionAmount)
		{
			$this->_transactionAmount = $transactionAmount;
		}
		
		public function getTransactionAmount()
		{
			return $this->_transactionAmount;
		}
		
		public function setTransactionCurrencyCode($transactionCurrencyCode)
		{
			$this->_transactionCurrencyCode = $transactionCurrencyCode;
		}
		
		public function getTransactionCurrencyCode()
		{
			return $this->_transactionCurrencyCode;
		}
		
		public function setTransactionCode($transactionCode)
		{
			$this->_transactionCode = $transactionCode;
		}
		
		public function getTransactionCode()
		{
			return $this->_transactionCode;
		}
		
		public function setPaymentStatus($paymentStatus)
		{
			$this->_paymentStatus = $paymentStatus;
		}
		
		public function getPaymentStatus()
		{
			return $this->_paymentStatus;
		}
		
		public function setNote($note)
		{
			$this->_note = $note;
		}
		
		public function getNote()
		{
			return $this->_note;
		}
		
		public function setTransactionDetail($detailName, $detailValue)
		{
			$this->_details[$detailName] = $detailValue;
		}
		
		public function getTransactionDetail($detailName)
		{
			if (isset($this->_details[$detailName]))
				return $this->_details[$detailName];
			else
				return null;
		}
		
		public function setAllTransactionDetails($details)
		{
			$this->_details = $details;
		}
		
		public function getAllTransactionDetails()
		{
			return $this->_details;
		}
		
		/**
		 * 
		 * @return int
		 */
		public function insertTransaction()
		{
			$values = array(
				'OrderID' => $this->_orderId,
				'ModuleID' => $this->_moduleId,
				'TransactionTimestamp' => time(),
				'TransactionAmount' => $this->_transactionAmount,
				'TransactionCurrencyCode' => $this->_transactionCurrencyCode,
				'TransactionCode' => $this->_transactionCode,
				'TransactionPaymentStatus' => $this->_paymentStatus,
				'TransactionNote' => $this->_note
			);
			$this->_transactionId = miStaticDBUtil::execInsert('Transactions', $values);
			
			// Insert the transaction details
			$this->insertTransactionDetails($this->_transactionId);
			
			return $this->_transactionId; 
		}
		
		public function updateTransaction()
		{
			$values = array(
				'OrderID' => $this->_orderId,
				'ModuleID' => $this->_moduleId,
				'TransactionTimestamp' => time(),
				'TransactionAmount' => $this->_transactionAmount,
				'TransactionCurrencyCode' => $this->_transactionCurrencyCode,
				'TransactionCode' => $this->_transactionCode,
				'TransactionPaymentStatus' => $this->_paymentStatus,
				'TransactionNote' => $this->_note
			);
			miStaticDBUtil::execUpdate('Transactions', $values, 'TransactionID', $this->_transactionId);
		}
		
		/**
		 * Inserts the transaction details
		 */
		public function insertTransactionDetails($transactionId)
		{
			foreach ($this->_details as $detailName => $detailValue) {
				$values = array(
					'TransactionID' => $transactionId,
					'TransactionDetailName' => $detailName,
					'TransactionDetailValue' => $detailValue
				);
				miStaticDBUtil::execInsert('TransactionDetails', $values);
			}
		}
		
		/**
		 * Returns true if the transaction exists
		 *
		 * @param int $orderId
		 * @param string $transactionCode
		 * @param int $paymentStatus
		 * @return boolean true if the transaction exists
		 */
		static public function transactionExists($orderId, $transactionCode, $paymentStatus = null)
		{
			$sql = 'SELECT COUNT(*) AS N FROM Transactions WHERE OrderID="' . sql_escape_string($orderId) .
				'" AND TransactionCode="' . sql_escape_string($transactionCode) . '"';
			if (!is_null($paymentStatus))
				$sql .=	' AND TransactionPaymentStatus="' . sql_escape_string($paymentStatus) . '"';
			
			$rows = miStaticDBUtil::execSelect($sql);
			return $rows[0]['N'] ? true : false;
		}
		
		static public function getOrderIdByTransactionCode($transactionCode, $moduleId)
		{
			$sql = 'SELECT OrderID FROM Transactions WHERE TransactionCode=? AND ModuleID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($transactionCode, $moduleId));
			if (count($rows) == 0)
				throw new miException(sprintf(_t('Cannot find OrderID with TransactionCode=%s and ModuleID=%s.'), $transactionCode, $moduleId));
			return $rows[0]['OrderID'];
		}
	}
?>