<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scImageInfo {
		
		protected $_imageLocation;
		protected $_width;
		protected $_height;
		protected $_size;
		
		public function __construct($imageLocation, $width = null, $height = null, $size = null)
		{
			$this->_imageLocation = $imageLocation;
			$this->_width = $width;
			if (is_null($this->_width)) {
				$this->_width = scImageHandler::getImageWidth(scPaths::getStoreRootDir() . $imageLocation);
			}
			$this->_height = $height;
			if (is_null($this->_height)) {
				$this->_height = scImageHandler::getImageHeight(scPaths::getStoreRootDir() . $imageLocation);
			}
			$this->_size = $size;
			if (is_null($this->_size)) {
				$this->_size = filesize(scPaths::getStoreRootDir() . $imageLocation);
			}
		}
		
		public function exists()
		{
			if (empty($this->_imageLocation))
				return false;
			
			$filename = scPaths::getStoreRootDir() . $this->_imageLocation;
			return (file_exists($filename) && is_file($filename));
		}
		
		public function getDataArray($key = 'Image')
		{
			return array(
				$key => scPaths::getAbsoluteUrl($this->_imageLocation),
				$key . 'Width' => $this->_width,
				$key . 'Height' => $this->_height,
				$key . 'Size' => $this->_size,
				$key . 'Exists' => $this->exists(),
			);
		}
	}
	
	class scProductImageInfo extends scImageInfo {
		
		/**
		 * @var scProductListingItemImageInfo
		 */
		protected $_thumbnail;
		
		/**
		 * @var scProductListingItemImageInfo
		 */
		protected $_boxImage;
		
		/**
		 * @var scProductListingItemImageInfo
		 */
		protected $_largeImage;
		
		/**
		 * @var int
		 */
		protected $_productImageId = 0;
		
		/**
		 * @var bool
		 */
		protected $_isDefault = false;
		
		public function getDataArray($key = 'Image')
		{
			if (!$this->exists()) {
				$noImage = new scProductNoImageImageInfo();
				$noImage->setLargeImage(new scProductNoLargeImageImageInfo());
				$noImage->setThumbnail(new scProductNoThumbImageInfo());
				$noImage->setBoxImage(new scProductNoBoxImageInfo());
				return $noImage->getDataArray($key);
			}
			
			$return = parent::getDataArray($key);
			
			$return = array_merge($return, array($key . 'IsDefault' => $this->_isDefault, $key . 'ID' => $this->_productImageId));
			
			if ($this->_thumbnail instanceof scImageInfo) {
				$return = array_merge($return, $this->_thumbnail->getDataArray($key . 'Thumb'));
			}
			
			if ($this->_boxImage instanceof scImageInfo) {
				$return = array_merge($return, $this->_boxImage->getDataArray($key . 'Box'));
			}
			
			if ($this->_largeImage instanceof scImageInfo) {
				$return = array_merge($return, $this->_largeImage->getDataArray($key . 'Large'));
			}
			
			return $return;
		}
		
		public function setThumbnail(scImageInfo $image)
		{
			$this->_thumbnail = $image;
		}
		
		public function setBoxImage(scImageInfo $image)
		{
			$this->_boxImage = $image;
		}
		
		public function setLargeImage(scImageInfo $image)
		{
			$this->_largeImage = $image;
		}
		
		public function getLargeImage()
		{
			return $this->_largeImage;
		}
		
		public function setProductImageId($productImageId)
		{
			$this->_productImageId = $productImageId;
		}
		
		public function getProductImageId()
		{
			return $this->_productImageId;
		}
		
		public function setIsDefault($isDefault)
		{
			$this->_isDefault = (bool)$isDefault;
		}
	}
	
	class scProductNoImageImageInfo extends scProductImageInfo {
		
		public function __construct()
		{
			$sizes = scProductImage::getDefaultSizes();
			
			$skinPath = scSkins::singleton()->getCurrentSkin()->getSkinPath(false);
			
			$defaultImageLocation = $skinPath . scProductImage::getDefaultProductImageUrl();
			
			parent::__construct($defaultImageLocation, $sizes['DefaultImageWidth'], $sizes['DefaultImageHeight']);
		}
		
		public function exists()
		{
			return true;
		}
	}
	
	class scProductNoLargeImageImageInfo extends scImageInfo {
		
		public function __construct()
		{
			$sizes = scProductImage::getDefaultSizes();
			
			$skinPath = scSkins::singleton()->getCurrentSkin()->getSkinPath(false);
			
			$defaultImageLocation = $skinPath . scProductImage::getDefaultProductImageUrl();
			
			parent::__construct($defaultImageLocation, $sizes['DefaultImageWidth'], $sizes['DefaultImageHeight']);
		}
		
		public function exists()
		{
			return true;
		}
	}
	
	class scProductNoThumbImageInfo extends scImageInfo {
		
		public function __construct()
		{
			$sizes = scProductImage::getDefaultSizes();
			
			$skinPath = scSkins::singleton()->getCurrentSkin()->getSkinPath(false);
			
			$defaultThumbLocation = $skinPath . scProductImage::getDefaultProductThumbUrl();
			
			parent::__construct($defaultThumbLocation, $sizes['DefaultImageThumbWidth'], $sizes['DefaultImageThumbHeight']);
		}
	
		public function exists()
		{
			return true;
		}
	}
	
	class scProductNoBoxImageInfo extends scImageInfo {
		
		public function __construct()
		{
			$sizes = scProductImage::getDefaultSizes();
			
			$skinPath = scSkins::singleton()->getCurrentSkin()->getSkinPath(false);
			
			$defaultLocation = $skinPath . scProductImage::getDefaultProductBoxImageUrl();
			
			parent::__construct($defaultLocation, $sizes['DefaultImageBoxWidth'], $sizes['DefaultImageBoxHeight']);
		}
	
		public function exists()
		{
			return true;
		}
	}
?>