<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scPrice {
		const EPSILON = 0.000001;
		
		/**
		 * @var scMultiCurrencyAmount
		 */
		protected $_amount;
		
		/**
		 * Convinience method to create scPrice objects
		 * @param scMultiCurrencyAmount $amount
		 * @return scPrice
		 */
		public static function factory(scMultiCurrencyAmount $amount)
		{
			return new scPrice($amount);
		}
		
		/**
		 * Construct the price
		 *
		 * @param scMultiCurrencyAmount $amount
		 */
		protected function __construct(scMultiCurrencyAmount $amount)
		{
			$this->setAmount($amount);
		}
		
		/**
		 * Clone the scPrice.
		 * When cloning it clone the scMultiCurrencyAmount as well
		 * as the price is a wrapper around it.
		 */
		public function __clone()
		{
			$this->_amount = clone $this->_amount;
		}
		
		/**
		 * Sets the amount
		 *
		 * @param scMultiCurrencyAmount $amount
		 */
		public function setAmount(scMultiCurrencyAmount $amount)
		{
			$this->_amount = $amount;
		}
		
		/**
		 * Return the amount
		 *
		 * @return scMultiCurrencyAmount
		 */
		public function getAmount()
		{
			return $this->_amount;
		}
		
		/**
		 * Round the amount using $precision decimal digits
		 * Always rounds up
		 * 
		 * @param string $amount amount
		 * @param int $precision precision
		 * @return string rounded amount
		 */
		public function roundAmount($amount, $precision)
		{
			$rounded = round($amount, $precision);
			if ($amount - $rounded > self::EPSILON)
				$rounded += 1 / pow(10, $precision);
			
			return $rounded;
		}
		
		/**
		 * Return formatted amount in the specified currency
		 * 
		 * @param scCurrency $currency
		 * @param boolean $convert true if the amount should be converted to the specified currency
		 * @return string
		 */
		public function getAmountInCurrency(scCurrency $currency)
		{
			$amount = $this->_amount->get($currency);
			return number_format($amount, $currency->getCurrencyNumDecimals(), '.', '');
		}
		
		/**
		 * Return amount in currency currency
		 * @return string
		 */
		public function getCurrentCurrencyAmount()
		{
			return $this->getAmountInCurrency(scCurrencyUtil::getCurrentCurrency());
		}
		
		/**
		 * Converts to currency and formats the price
		 *
		 * @param boolean $convert true if the amount should be converted to the current currency
		 * @param scCurrency $currency if null uses the current currency
		 * @return string
		 */
		public function getDisplayPrice(scCurrency $currency = null)
		{
			if (is_null($currency))
				$currency = scCurrencyUtil::getCurrentCurrency();
			
			$amount = $this->getAmountInCurrency($currency);
			
			$negativeSign = ($amount < 0) ? '-' : '';
			return 	$negativeSign . 
					$currency->getCurrencySymbolLeft() .
					number_format(abs($amount), $currency->getCurrencyNumDecimals(), $currency->getCurrencyDecimalSymbol(), $currency->getCurrencyThousandsSeparator()) .
					$currency->getCurrencySymbolRight();
		}
	}
?>