<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductAttributes {
		
		const ATTRIBUTE_TYPE_TEXT = 1;
		const ATTRIBUTE_TYPE_SELECT = 2;
		const ATTRIBUTE_TYPE_RADIO_BUTTONS = 3;
		const ATTRIBUTE_TYPE_TEXT_ML = 4;
		const ATTRIBUTE_TYPE_MULTIPLE_SELECT = 5;
		
		const VALIDATOR_NONE = 1;
		const VALIDATOR_NUMBER = 2;
		
		const SYSTEM_ATTRIBUTE_PRICE = 'price';
		const SYSTEM_ATTRIBUTE_MANUFACTURER = 'manufacturer';
		
		/**
		 * Get system attributes only if others are available.
		 */
		const GET_SYSTEM_ATTRIBUTES_INCLUDE = 1;
		
		/**
		 * Always get system attributes no matter others are available or not.
		 */
		const GET_SYSTEM_ATTRIBUTES_ALWAYS = 2;
		
		/**
		 * Do not get system attributes.
		 */
		const GET_SYSTEM_ATTRIBUTES_NONE = 3;
		
		protected $_attributeTypesWithValues = array(
			self::ATTRIBUTE_TYPE_SELECT,
			self::ATTRIBUTE_TYPE_RADIO_BUTTONS,
			self::ATTRIBUTE_TYPE_MULTIPLE_SELECT,
		);
		
		protected $_mlAttributeTypes = array(
			self::ATTRIBUTE_TYPE_TEXT_ML,
		);
		
		protected $_attributeValidatorsByType = array(
			self::ATTRIBUTE_TYPE_TEXT => array(
				self::VALIDATOR_NONE,
				self::VALIDATOR_NUMBER,
			),
			self::ATTRIBUTE_TYPE_TEXT_ML => array(
				self::VALIDATOR_NONE,
			),
			self::ATTRIBUTE_TYPE_SELECT => array(
				self::VALIDATOR_NONE,
			),
			self::ATTRIBUTE_TYPE_RADIO_BUTTONS => array(
				self::VALIDATOR_NONE,
			),
			self::ATTRIBUTE_TYPE_MULTIPLE_SELECT => array(
				self::VALIDATOR_NONE,
			),
		);
		
		protected $_attributeTypeValueFields = array(
			self::ATTRIBUTE_TYPE_TEXT => 'ProductAttributeValue',
			self::ATTRIBUTE_TYPE_SELECT => 'ProductClassAttributeValueID',
			self::ATTRIBUTE_TYPE_RADIO_BUTTONS => 'ProductClassAttributeValueID',
			self::ATTRIBUTE_TYPE_TEXT_ML => 'ProductAttributeValueML',
			self::ATTRIBUTE_TYPE_MULTIPLE_SELECT => 'ProductClassAttributeMultiValue',
		);
		
		protected $_systemAttributes = array(
			'ProductPrice' => self::SYSTEM_ATTRIBUTE_PRICE,
			'ManufacturerID' => self::SYSTEM_ATTRIBUTE_MANUFACTURER,
		);
		
		protected function __construct()
		{
		}
		
		/**
		 * @return scProductAttributes
		 */
		public static function singleton()
		{
			static $obj = null;
			if ($obj == null)
				$obj = new scProductAttributes();
			return $obj;
		}
		
		/**
		 * @return array
		 */
		public function getFieldTypes()
		{
			return array(
				self::ATTRIBUTE_TYPE_TEXT => _t('Text'),
				self::ATTRIBUTE_TYPE_TEXT_ML => _t('Multi-Language Text'),
				self::ATTRIBUTE_TYPE_SELECT => _t('Drop-Down List'),
				self::ATTRIBUTE_TYPE_RADIO_BUTTONS => _t('Radio Buttons'),
				self::ATTRIBUTE_TYPE_MULTIPLE_SELECT => _t('Multiple Select'),
			);
		}
		
		/**
		 * @return array
		 */
		public function getFieldTypesWithValues()
		{
			return $this->_attributeTypesWithValues;
		}
		
		/**
		 * @return array
		 */
		public function getAttributeValidators()
		{
			return array(
				self::VALIDATOR_NONE => _t('No validator'),
				self::VALIDATOR_NUMBER => _t('Number'),
			);
		}
		
		/**
		 * @param int $attributeType
		 * @return array
		 */
		public function getAttributeValidatorsByType($attributeType = 0)
		{
			if ($attributeType != 0) {
				if (!isset($this->_attributeValidatorsByType[$attributeType])) {
					throw new miException(sprintf(_i('Attribute type "%s" not supported'), $attributeType));
				}
				return $this->_attributeValidatorsByType[$attributeType];
			}
			return $this->_attributeValidatorsByType;
		}
		
		/**
		 * @param int $attributeType
		 * @return string
		 */
		public function getValueFieldForType($attributeType)
		{
			if (!isset($this->_attributeTypeValueFields[$attributeType])) {
				throw new miException(sprintf(_i('Attribute type "%s" not supported'), $attributeType));
			}
			return $this->_attributeTypeValueFields[$attributeType];
		}
		
		public function getSystemAttributes()
		{
			return $this->_systemAttributes;
		}
		
		/**
		 * @param int $classId
		 * @param bool $includeSystemAttributes
		 * @return array
		 */
		public function getProductClassAttributes($classId, $includeSystemAttributes = self::GET_SYSTEM_ATTRIBUTES_INCLUDE)
		{
			$recordset = new scSqlRecordsetML('ProductClassAttributes', 'ProductClassAttributeID');
			$recordset->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$recordset->addFilter(new miSqlFilterEqual('ProductClassID', $classId));
			$recordset->setSelectFields(array(
				'ProductClassAttributes.ProductClassAttributeID',
				'ProductClassAttributeName',
				'ProductClassAttributeType',
				'ProductClassAttributeValidator',
			));
			$recordset->setOrder('ProductClassAttributeSort', 'ASC');
			$attributes = $recordset->getAllRecords();
			$attributes = miUtil::getArrayWithKeys($attributes, 'ProductClassAttributeID');
			
			if (empty($attributes) && $includeSystemAttributes != self::GET_SYSTEM_ATTRIBUTES_ALWAYS) {
				return array();
			}
			
			if (in_array($includeSystemAttributes, array(self::GET_SYSTEM_ATTRIBUTES_ALWAYS, self::GET_SYSTEM_ATTRIBUTES_INCLUDE))) {
				$attributes = $attributes + $this->getSystemAttributesData();
			}
			
			$values = $this->getFieldsPossibleValues($classId, $attributes);
			
			$this->addValuesToFields($attributes, $values);
			
			return $attributes;
		}
		
		public function getProductClassSystemAttributes()
		{
			$attributes = $this->getSystemAttributesData();
			
			$values = $this->getFieldsPossibleValues(0, $attributes);
			
			$this->addValuesToFields($attributes, $values);
			
			return $attributes;
		}
		
		public function getProductAttributeValues($productId, $includeSystemAttributes = scProductAttributes::GET_SYSTEM_ATTRIBUTES_NONE)
		{
			// Get values, valueIDs and ML values
			$recordset = new miSqlRecordset('ProductAttributeValues');
			$recordset->setSelectFields(array('*', 'ProductAttributeValues.ProductAttributeValueID'));
			$recordset->addJoinCondition('LEFT', 'ProductAttributeValuesML', 'ON(ProductAttributeValues.ProductAttributeValueID=ProductAttributeValuesML.ProductAttributeValueID AND LanguageID=' . scMultiLanguage::getCurrentLanguageId() . ')');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			// Key with by their PK
			$productAttributeValues = miUtil::getArrayWithKeys($recordset->getAllRecords(), 'ProductAttributeValueID');
			
			if ($includeSystemAttributes != self::GET_SYSTEM_ATTRIBUTES_NONE) {
				if (!empty($productAttributeValues) || ($includeSystemAttributes == self::GET_SYSTEM_ATTRIBUTES_ALWAYS)) {
					$productAttributeValues = $productAttributeValues + $this->getSystemAttributeProductValues($productId);
				}
			}
			
			// Get value IDs to extract needed multiValues
			$valueIds = array_keys($productAttributeValues);
			
			$recordset = new miSqlRecordset('ProductAttributeMultiValues');
			$recordset->addJoinCondition('INNER', 'ProductClassAttributeValues', 'ON ProductAttributeMultiValues.ProductClassAttributeMultiValue = ProductClassAttributeValues.ProductClassAttributeValueID');
			$recordset->addJoinCondition('LEFT', 'ProductClassAttributeValuesML', 'USING(ProductClassAttributeValueID)');
			$recordset->addFilter(new miSqlFilterIn('ProductAttributeValueID', $valueIds));
			$recordset->addFilter(new miSqlFilterIn('LanguageID', scMultiLanguage::getCurrentLanguageId()));
			$recordset->setOrder('ProductClassAttributeValueSort', 'ASC');
			$recordset->setSelectFields(array(
				'ProductAttributeValueID',
				'ProductClassAttributeMultiValue',
				'ProductClassAttributeValueML',
			));
			$result = $recordset->getAllRecords();
			
			// Group multiValues by the value they are selected for
			$multiValues = array();
			foreach ($result as $value) {
				$valueId = $value['ProductAttributeValueID'];
				if (!isset($multiValues[$valueId])) {
					$multiValues[$valueId] = array();
				}
				$multiValues[$valueId][$value['ProductClassAttributeMultiValue']] = $value['ProductClassAttributeValueML'];
			}
			
			// Add multiValues to product values
			$newResult = array();
			foreach ($productAttributeValues as $value) {
				$valueId = $value['ProductAttributeValueID'];
				if (!isset($newResult[$valueId])) {
					$newResult[$valueId] = $value;
					$newResult[$valueId]['ProductClassAttributeMultiValue'] = array();
				}
				if (isset($multiValues[$valueId])) {
					$newResult[$valueId]['ProductClassAttributeMultiValue'] = $multiValues[$valueId];
				}
			}
			
			// Set ProductClassAttributeID as key
			$newResult = miUtil::getArrayWithKeys($newResult, 'ProductClassAttributeID');
			
			return $newResult;
		}
		
		/**
		 * @param int $classId
		 * @return array
		 */
		public function getProductClassValidators($classId)
		{
			$recordset = new miSqlRecordset('ProductClassAttributes');
			$recordset->addFilter(new miSqlFilterEqual('ProductClassID', $classId));
			$recordset->setSelectFields(array(
				'ProductClassAttributeID',
				'ProductClassAttributeValidator'
			));
			$validators = $recordset->getAllRecords();
			return miUtil::getKeyValueArray($validators, 'ProductClassAttributeID', 'ProductClassAttributeValidator');
		}
		
		/**
		 * @param int $type
		 * @param miWebForm $webForm
		 * @return miBaseWidget
		 */
		public function getWidget($type, miWebForm $webForm)
		{
			switch ($type) {
				case self::ATTRIBUTE_TYPE_TEXT:
					$widget = new miTextWidget($webForm, array());
					break;
				case self::ATTRIBUTE_TYPE_TEXT_ML:
					$widget = new scWebFormWidgetTextML($webForm, array());
					break;
				case self::ATTRIBUTE_TYPE_SELECT:
					$widget = new miSelectWidget($webForm, array());
					break;
				case self::ATTRIBUTE_TYPE_RADIO_BUTTONS:
					$widget = new miRadioWidget($webForm, array());
					break;
				case self::ATTRIBUTE_TYPE_MULTIPLE_SELECT:
					$widget = new scMultipleCheckboxesWidget($webForm, array());
					break;
				default:
					throw new miException(sprintf(_i('Attribute type "%s" not supported'), $type));
			}
			return $widget;
		}
		
		/**
		 * @param miBaseWidget $widget
		 * @param array $values
		 */
		public function setWidgetValues(miBaseWidget $widget, array $values, $notSelectedLabel = 'Not selected') // _t('Not selected')
		{
			if ($widget instanceof miBaseSelectWidget) {
				$widget->setOptions(array('' => scInterfaceTexts::getTranslatedText($notSelectedLabel)) + $values['ProductClassAttributeValues']);
			} else if ($widget instanceof scMultipleCheckboxesWidget) {
				$widget->setRadioButtons($values['ProductClassAttributeValues']);
			} else if ($widget instanceof miBaseRadioWidget) {
				$widget->setRadioButtons(array('' => scInterfaceTexts::getTranslatedText($notSelectedLabel)) + $values['ProductClassAttributeValues']);
			} else {
				throw new miException(_i('Unknown widget type.'));
			}
		}
		
		/**
		 * @param int $productClassId
		 * @param array $values
		 * @return string
		 */
		public function getProductClassAttributesHtml($productClassId, $values)
		{
			$attributes = $this->getProductClassAttributes($productClassId, self::GET_SYSTEM_ATTRIBUTES_NONE);
			
			$formDataRow = array();
			foreach ($values as $value) {
				$fieldName = 'ProductAttributes[' . $value['ProductClassAttributeID'] . ']';
				if (!isset($attributes[$value['ProductClassAttributeID']])) {
					// There is a value for an unsupported attribute
					continue;
				}
				$attributeType = $attributes[$value['ProductClassAttributeID']]['ProductClassAttributeType'];
				
				$valueField = $this->getValueFieldForType($attributeType);
				
				$formDataRow[$fieldName] = $value[$valueField];
			}
			
			$typesWithValues = $this->getFieldTypesWithValues();
			$webForm = new miWebForm($formDataRow);
			$sectionValues = array();
			foreach ($attributes as $attribute) {
				$classAttrId = $attribute['ProductClassAttributeID'];
				$fieldName = 'ProductAttributes[' . $classAttrId . ']';
				
				$widget = $this->getWidget($attribute['ProductClassAttributeType'], $webForm);
				$widget->setFieldName($fieldName);
				if (in_array($attribute['ProductClassAttributeType'], $typesWithValues)) {
					$this->setWidgetValues($widget, $attribute);
				}
				
				$control = $widget->getEditableControl();
				$sectionValues[] = array(
					'ProductClassAttributeName' => $attribute['ProductClassAttributeName'],
					'HTML_ProductAttribute' => $control[$fieldName],
				);
			}
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/products/product_attributes.tmpl');
			$t->addSectionInfo(new miTemplateParserSectionInfo('productAttributes', count($sectionValues), miUIUtil::templetizeRows(miUIUtil::transposeRows($sectionValues))));
			return $t->templateParse();
		}
		
		public function getSqlFilters($productClassId, $attributeValues)
		{
			if ($attributeValues == 0)
				return array();
			
			$filters = array();
			
			$classAttributes = $this->getProductClassAttributes($productClassId, self::GET_SYSTEM_ATTRIBUTES_ALWAYS);
			foreach ($attributeValues as $attributeId => $value) {
				
				// Check if the product class has such an attribute
				if (!isset($classAttributes[$attributeId])) {
					continue;
				}
				// Ignore non-specified filters
				if (empty($value)) {
					continue;
				}
				
				// Check if that's a system attribute (price, manufacturer, etc.)
				if (!in_array($attributeId, $this->_systemAttributes)) {
					// If it's not a system one, it must be INT
					$attributeId = (int)$attributeId;
					// Get its value field as defined for this type of attribute.
					$valueField = $this->getValueFieldForType($classAttributes[$attributeId]['ProductClassAttributeType']);
				} else {
					// It's a system attribute. The field that is to be searched is set as key in the system attributes array.
					$valueField = array_search($attributeId, $this->_systemAttributes);
				}
				
				$whereClause = $this->getAttributeFilterForValue($classAttributes[$attributeId], $valueField, $value);
				if (empty($whereClause)) {
					continue;
				}
				
				switch ($valueField) {
					case 'ProductAttributeValueML':
						$filter =  'SELECT ProductID FROM ProductAttributeValues' .
							' LEFT JOIN ProductAttributeValuesML ON ProductAttributeValues.ProductAttributeValueID = ProductAttributeValuesML.ProductAttributeValueID' .
							' AND ProductAttributeValuesML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId() .
							' WHERE ProductClassAttributeID = ' . intval($attributeId) . ' AND (' . $whereClause . ')';
						break;
					
					case 'ProductAttributeValue':
					case 'ProductClassAttributeValueID':
						$filter = 'SELECT ProductID FROM ProductAttributeValues' .
							' WHERE ProductClassAttributeID = ' . intval($attributeId) . ' AND (' . $whereClause . ')';
						break;
					
					case 'ProductClassAttributeMultiValue':
						$filter =  'SELECT ProductID FROM ProductAttributeValues' .
							' INNER JOIN ProductAttributeMultiValues USING(ProductAttributeValueID)' .
							' WHERE ProductClassAttributeID = ' . intval($attributeId) . ' AND (' . $whereClause . ')' .
							' GROUP BY ProductID' .
							' HAVING COUNT(ProductClassAttributeMultiValue) = ' . count($value);
						break;
					
					default:
						$filter = 'SELECT ProductID FROM Products WHERE ' . $whereClause;
				}
				
				$filters[] = $filter;
			}
			
			return $filters;			
		}
		
		protected function getSystemAttributesData()
		{
			$attributes = array();
			$attributes[self::SYSTEM_ATTRIBUTE_PRICE] = array(
				'ProductClassAttributeID' => self::SYSTEM_ATTRIBUTE_PRICE,
				'ProductClassAttributeName' => _t('Price'),
				'ProductClassAttributeType' => self::ATTRIBUTE_TYPE_TEXT,
				'ProductClassAttributeValidator' => self::VALIDATOR_NUMBER,
			);
			$attributes[self::SYSTEM_ATTRIBUTE_MANUFACTURER] = array(
				'ProductClassAttributeID' => self::SYSTEM_ATTRIBUTE_MANUFACTURER,
				'ProductClassAttributeName' => _t('Manufacturer'),
				'ProductClassAttributeType' => self::ATTRIBUTE_TYPE_SELECT,
				'ProductClassAttributeValidator' => self::VALIDATOR_NONE,
			);
			return $attributes;
		}
		
		protected function getSystemAttributeProductValues($productId)
		{
			$values = array();
			
			$product = new scProductSL();
			$product->read($productId);
			
			if ($product->getProductDomainObject()->hasVariants()) {
				$listingItem = $product->getProductDomainObject()->getDefaultVariant();
			} else {
				$listingItem = $product;
			}
			
			$valueId = $listingItem->getProductId() . '-' . self::SYSTEM_ATTRIBUTE_PRICE;
			
			$listingItemPrice = $listingItem->getPrice()->getAmount();
			$priceModifications = scProductPrice::getSingleProductPriceModifications($listingItem);
			foreach ($priceModifications as $priceModification) {
				if ($priceModification->getDisplayMode() == scProductPriceModification::DISPLAY_MODE_PRODUCT_PRICE)
					continue;
				
				$listingItemPrice = scMultiCurrencyAmountMath::add($listingItemPrice, $priceModification->getAmount());
			}
			$listingItemTaxedPrice = scProductPrice::getTaxedAmount($listingItemPrice, $productId);
			$displayPricesTaxMode = scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE);
			#FIXME: move the HTML code below to a template file
			if (scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE) == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
				$displayPrice = sprintf('<span class="price-value">%s</span>', scPrice::factory($listingItemTaxedPrice)->getDisplayPrice());
			} else if (scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE) == scTax::DISPLAY_PRICES_TAX_MODE_WITH_AND_WITHOUT_TAX) {
				$displayPrice = sprintf('<span class="price-value">%s</span> ', scPrice::factory($listingItemPrice)->getDisplayPrice());
				$displayPrice .= sprintf('<span class="price-value-tax-label">%s</span> ', _t('(without tax)'));
				$displayPrice .= sprintf('<span class="taxed-price-value">%s</span> ', scPrice::factory($listingItemTaxedPrice)->getDisplayPrice());
				$displayPrice .= sprintf('<span class="taxed-price-value-tax-label">%s</span> ', _t('(with tax)'));
			}
			$values[$valueId] = array(
	            'ProductAttributeValueID' => $valueId,
	            'ProductID' => $listingItem->getProductId(),
	            'ProductClassAttributeID' => self::SYSTEM_ATTRIBUTE_PRICE,
	            'ProductClassAttributeValueID' => null,
	            'ProductAttributeValue' => $displayPrice,
	            'LanguageID' => scMultiLanguage::getCurrentLanguageId(),
	            'ProductAttributeValueML' => ''
			);
			
			$manufacturer = $listingItem->getManufacturer();
			if ($manufacturer != null) {
				$valueId = $listingItem->getProductId() . '-' . self::SYSTEM_ATTRIBUTE_MANUFACTURER;
				$values[$valueId] = array(
		            'ProductAttributeValueID' => $valueId,
		            'ProductID' => $listingItem->getProductId(),
		            'ProductClassAttributeID' => self::SYSTEM_ATTRIBUTE_MANUFACTURER,
		            'ProductClassAttributeValueID' => $manufacturer->get($manufacturer->getPKName()),
		            'ProductAttributeValue' => $manufacturer->getName(),
		            'LanguageID' => scMultiLanguage::getCurrentLanguageId(),
		            'ProductAttributeValueML' => ''
				);
			}
			
			return $values;
		}
		
		protected function getFieldsPossibleValues($classId, $attributes)
		{
			$systemAttributes = array();
			foreach ($attributes as $attributeId => $attribute) {
				if (!in_array($attributeId, $this->_systemAttributes)) {
					continue;
				}
				$systemAttributes[] = $attributeId;
				unset($attributes[$attributeId]);
			}
			
			$attributeIds = array_keys($attributes);
			$recordset = new scSqlRecordsetML('ProductClassAttributeValues', 'ProductClassAttributeValueID');
			$recordset->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$recordset->addFilter(new miSqlFilterIn('ProductClassAttributeID', $attributeIds));
			$values = $recordset->getAllRecords();
			
			foreach ($systemAttributes as $attributeId) {
				$values = array_merge($values, $this->getSystemAttributePossibleValues($attributeId));
			}
			
			return $values;
		}
		
		protected function addValuesToFields(&$fields, $values)
		{
			foreach ($fields as &$field) {
				$field['ProductClassAttributeValues'] = array();
			}
			
			foreach ($values as $value) {
				$attrId = $value['ProductClassAttributeID'];
				$valueId = $value['ProductClassAttributeValueID'];
				
				$fields[$attrId]['ProductClassAttributeValues'][$valueId] = $value['ProductClassAttributeValueML'];
			}
		}
		
		protected function getAttributeFilterForValue($classAttribute, $valueField, $value)
		{
			if (!is_array($value)) {
				if ($classAttribute['ProductClassAttributeValidator'] != self::VALIDATOR_NUMBER) {
					return $valueField . ' = "' . sql_escape_string($value) . '"';
				}
				
				$newValue = explode('-', $value);
				if (count($newValue) != 2) {
					throw new miException(sprintf(_i('Invalid search data for attribute "%s"', $classAttribute['ProductClassAttributeName'])));
				}
				$value = array();
				$value['From'] = $newValue[0];
				$value['To'] = $newValue[1];
			}
			
			if (isset($value['From']) && isset($value['To']) && ($classAttribute['ProductClassAttributeValidator'] == self::VALIDATOR_NUMBER)) {
				return $this->getFilterForFromToValue($valueField, $value);
			}
			
			if ($classAttribute['ProductClassAttributeType'] == self::ATTRIBUTE_TYPE_MULTIPLE_SELECT) {
				return $this->getFilterForMultiValue($valueField, $value);
			}
			
			throw new miException(sprintf(_i('Invalid search data for attribute "%s"', $classAttribute['ProductClassAttributeName'])));
		}
		
		protected function getFilterForFromToValue($valueField, array $value)
		{
			if ($valueField == 'ProductPrice') {
				$valueField = scCurrencyUtil::getProductPriceColumnName();
				$valueField = scProductPrice::getProductPriceFieldForQuery($valueField);
			}
			
			$filter = array();
			// Values are kept in text fields so we need to cast them to numbers to compare them with such.
			if (!empty($value['From'])) {
				$filter[] = '0+' . $valueField . ' >= 0+"' . sql_escape_string($value['From']) . '"';
			}
			if (!empty($value['To'])) {
				$filter[] = '0+' . $valueField . ' <= 0+"' . sql_escape_string($value['To']) . '"';
			}
			if (empty($filter)) {
				return '';
			}
			// Since there is at least one (From and/or To) condition filtering by this criteria is needed.
			// Empty strings would be casted to 0 when comparing with numbers so they have to be filtered out as well.
			array_unshift($filter, $valueField . ' <> ""');
			
			return implode(' AND ', $filter);
		}
		
		protected function getFilterForMultiValue($valueField, array $value)
		{
			$value = array_map('intval', $value);
			return $valueField . ' IN (' . implode(', ', $value) . ')';
		}
		
		protected function getSystemAttributePossibleValues($attributeId)
		{
			switch ($attributeId) {
				case self::SYSTEM_ATTRIBUTE_MANUFACTURER:
					$manufacturers = scUtil::getManufacturersArray();
					$return = array();
					foreach ($manufacturers as $manufacturerId => $manufacturerName) {
						$return[] = array(
							'ProductClassAttributeValueID' => $manufacturerId,
		            		'ProductClassAttributeValueML' => $manufacturerName,
		            		'ProductClassAttributeID' => $attributeId,
						);
					}
					return $return;
					
				default:
					return array();
			}
		}
	}
?>