<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductImage {
		
		protected static $_defaultImageSize = null;
		
		public static function getDefaultProductImageUrl()
		{
			$areaPath = scSkins::singleton()->getCurrentSkin()->getAreaPath();
			return $areaPath . '/images/image-not-available/default_product_image.gif';
		}
		
		public static function getDefaultProductThumbUrl()
		{
			$areaPath = scSkins::singleton()->getCurrentSkin()->getAreaPath();
			return $areaPath . '/images/image-not-available/default_product_thumb.gif';
		}
		
		public static function getDefaultProductBoxImageUrl()
		{
			$areaPath = scSkins::singleton()->getCurrentSkin()->getAreaPath();
			return $areaPath . '/images/image-not-available/default_product_box_image.gif';
		}
		
		/**
		 * Return the image section for given product
		 * 
		 * @param scProductSL $product
		 * @return miTemplateParserSectionInfo
		 */
		public static function getImageSection(IProductListingItem $listingItem)
		{
			$imageSection = new miTemplateParserSectionInfo('image');
			
			$imageInfo = $listingItem->getImageInfo();
			$imageSection->setValues(miUIUtil::templetizeAndEscapeRows($imageInfo->getDataArray('ProductImage')));
			return $imageSection;
		}
		
		/**
		 * Return all images information for a product (the first image in the array is the default image)
		 * 
		 * @param int $productId
		 * @return array
		 */
		public static function getImages($productId)
		{
			$recordset = new miSqlRecordset('ProductImages');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$recordset->setOrder('ProductImageIsDefault', 'DESC');
			
			$images = $recordset->getRecords();
			$result = array();
			
			if (count($images) == 0)
				return array(self::getImageInfo());
			
			foreach ($images as $image)
				$result[] = self::getImageInfo($image);
			
			return $result;
		}
		
		/**
		 * Return only the default image
		 * 
		 * @param int $productId
		 * @return scProductImageInfo 
		 */
		public static function getImage($productId)
		{
			$recordset = new miSqlRecordset('ProductImages');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$recordset->setOrder('ProductImageIsDefault', 'DESC');
			$recordset->setRecordsLimit(0, 1);
			
			$image = $recordset->getRecords();
			if (count($image) > 0)
				$image = reset($image);		// Get only the first record
			else
				$image = array();
			
			return self::getImageInfo($image);
		}
		
		/**
		 * @param int $productImageId
		 * @return scProductImageInfo
		 */
		public static function getProductImageById($productImageId)
		{
			$recordset = new miSqlRecordset('ProductImages');
			$recordset->addFilter(new miSqlFilterEqual('ProductImageID', $productImageId));
			$recordset->setRecordsLimit(0, 1);
			
			$image = $recordset->getRecords();
			if (count($image) > 0)
				$image = reset($image);		// Get only the first record
			else
				$image = array();
			
			return self::getImageInfo($image);
		}
		
		/**
		 * Return image information (if no image information is available it is filled with the default values)
		 * Any other fields in the $image array are preserved
		 * 
		 * @return scProductImageInfo
		 */
		public static function getImageInfo($image)
		{
			self::initDefaultSize();
			
			if (empty($image)) {
				$imageInfo = new scProductNoImageImageInfo();
				$imageInfo->setLargeImage(new scProductNoLargeImageImageInfo());
				$imageInfo->setThumbnail(new scProductNoThumbImageInfo());
				$imageInfo->setBoxImage(new scProductNoBoxImageInfo());
				return $imageInfo;
			}
			
			$productId = isset($image['ProductID']) ? $image['ProductID']: 0;
			
			if (!isset($image['ProductImageID'])) {
				$image['ProductImageID'] = 0;
			}
			$imageId = $image['ProductImageID'];
			
			$imagePath = scPaths::getStoreRootDir() . $image['ProductImage'];
			$largeImagePath = scPaths::getStoreRootDir() . $image['ProductImageLarge'];
			$boxImagePath = scPaths::getStoreRootDir() . $image['ProductImageBox'];
			$thumbPath = scPaths::getStoreRootDir() . $image['ProductImageThumb'];
			
			if (file_exists($imagePath)) {
				$imageInfo = new scProductImageInfo($image['ProductImage'], $image['ProductImageWidth'], $image['ProductImageHeight'], $image['ProductImageSize']);
				$imageInfo->setIsDefault($image['ProductImageIsDefault']);;
			} else {
				$imageInfo = new scProductNoImageImageInfo();
				$imageInfo->setIsDefault($imageId == 0);
			}
			
			if (file_exists($largeImagePath)) {
				$imageInfo->setLargeImage(new scImageInfo($image['ProductImageLarge'], $image['ProductImageLargeWidth'], $image['ProductImageLargeHeight'], $image['ProductImageLargeSize']));
			} else {
				$imageInfo->setLargeImage(new scProductNoLargeImageImageInfo());
			}
			
			if (file_exists($boxImagePath)) {
				$imageInfo->setBoxImage(new scImageInfo($image['ProductImageBox'], $image['ProductImageBoxWidth'], $image['ProductImageBoxHeight'], $image['ProductImageBoxSize']));
			} else {
				$imageInfo->setBoxImage(new scProductNoBoxImageInfo());
			}
			
			if (file_exists($thumbPath)) {
				$imageInfo->setThumbnail(new scImageInfo($image['ProductImageThumb'], $image['ProductImageThumbWidth'], $image['ProductImageThumbHeight'], $image['ProductImageThumbSize']));
			} else {
				$imageInfo->setThumbnail(new scProductNoThumbImageInfo());
			}
			
			$imageInfo->setProductImageId($image['ProductImageID']);
			
			return $imageInfo;
		}
		
		public static function getDefaultSizes()
		{
			self::initDefaultSize();
			return self::$_defaultImageSize;
		}
		
		/**
		 * Reads the default image dimensions from the database if necessary
		 * 
		 * @return void
		 */
		protected static function initDefaultSize()
		{
			if (!is_null(self::$_defaultImageSize))
				return;
			
			// Get the default image size
			self::$_defaultImageSize = array(
				'ThumbWidth' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH),
				'ThumbHeight' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_HEIGHT),
				'BoxWidth' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_WIDTH),
				'BoxHeight' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_HEIGHT),
				'Width' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_MAX_WIDTH),
				'Height' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_MAX_HEIGHT),
				'LargeWidth' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_WIDTH),
				'LargeHeight' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_HEIGHT),
				
				// The default image is the same size as the default thumb
				'DefaultImageThumbWidth' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH),
				'DefaultImageThumbHeight' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_HEIGHT),
				'DefaultImageBoxWidth' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_WIDTH),
				'DefaultImageBoxHeight' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_BOX_IMAGE_MAX_HEIGHT),
				'DefaultImageWidth' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH),
				'DefaultImageHeight' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_IMAGE_THUMB_MAX_WIDTH),
				'DefaultImageLargeWidth' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_WIDTH),
				'DefaultImageLargeHeight' => scSettings::getSettingValue(scSettings::SETTING_PRODUCT_LARGE_IMAGE_MAX_HEIGHT),
			);
		}
	}
?>