<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductOption {
		protected $_optionId;
		protected $_optionInfo;
		
		protected $_fieldPriceModType = 'OptionPriceModType';
		protected $_fieldPriceModValue = 'OptionPriceModValue';
		
		public function __construct($optionId, $optionInfo)
		{
			$this->_optionId = $optionId;
			$this->_optionInfo = $optionInfo;
		}
		
		public function getOptionId()
		{
			return $this->_optionId;
		}
		
		/**
		 * @param scMultiCurrencyAmount $productPrice
		 * @return scMultiCurrencyAmount
		 */
		public function getOptionPriceChange(scMultiCurrencyAmount $productPrice)
		{
			if ($this->_optionInfo[$this->_fieldPriceModType] == scProductOptions::PRODUCT_OPTION_TYPE_PERCENT) {
				$priceChange = scMultiCurrencyAmountMath::multiply($productPrice, ($this->_optionInfo[$this->_fieldPriceModValue] / 100.0));
			} else {
				$priceChange = scMultiCurrencyAmount::factoryBaseCurrency($this->_optionInfo[$this->_fieldPriceModValue]);
			}
			
			return $priceChange;
		}
		
		public function getOptionInfo(scMultiCurrencyAmount $productPrice)
		{
			$optionInfo = $this->_optionInfo;
			
			$price = scPrice::factory($this->getOptionPriceChange($productPrice));
			$optionInfo['OptionPrice'] = $price->getDisplayPrice();
			if ($optionInfo['OptionPriceModValue'] > 0)
				$optionInfo['OptionPrice'] = '+' . $optionInfo['OptionPrice'];
			return $optionInfo;
		}
		
		/**
		 * @return scOptionGroupDomainObject
		 */
		public function getOptionGroup()
		{
			$optionGroup = new scOptionGroupDomainObject();
			$optionGroup->setRow(array(
				'OptionGroupID' => $this->_optionInfo['OptionGroupID'],
				'OptionGroupType' => $this->_optionInfo['OptionGroupType'],
				'OptionGroupName' => $this->_optionInfo['OptionGroupName'],
				'LanguageID' => $this->_optionInfo['LanguageID'],
			));
			return $optionGroup;
		}
		
		/**
		 * @return filesystem path to temporary option file
		 */
		public function getOptionUploadedFilePath()
		{
			return empty($this->_optionInfo['OptionUploadedFileName']) ? '' : scPaths::getFilenamePath($this->_optionInfo['OptionUploadedFileName'], scPaths::RESOURCE_TEMP_IMAGE);
		}
		
		/**
		 * @return web path to temporary option file, useful to display the image with lightbox in cart before checkout
		 */
		public function getOptionUploadedFilePathForWeb()
		{
			return  empty($this->_optionInfo['OptionUploadedFileName']) ? '' : scPaths::getFilenamePath($this->_optionInfo['OptionUploadedFileName'], scPaths::RESOURCE_TEMP_IMAGE, scPaths::PATH_WEB);
		}
	}
?>