<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductOptions {
		const GROUP_TYPE_PRICE_MODIFIER = 1;
		const GROUP_TYPE_VARIANT = 2;
		const GROUP_TYPE_USER_IMAGE = 3;
		const GROUP_TYPE_USER_TEXT = 4;
		
		const PRODUCT_OPTION_TYPE_NONE = 0;
		const PRODUCT_OPTION_TYPE_AMOUNT = 1;
		const PRODUCT_OPTION_TYPE_PERCENT = 2;
		
		public static function getGroupTypes()
		{
			return array(
				self::GROUP_TYPE_PRICE_MODIFIER => _t('Price modifier'),
				self::GROUP_TYPE_VARIANT => _t('Variant'),
				self::GROUP_TYPE_USER_IMAGE => _t('User Image'),
				self::GROUP_TYPE_USER_TEXT => _t('User Text'),
			);
		}
		
		/**
		 * @return array
		 */
		public static function getUserValueGroupTypes()
		{
			return array(
				self::GROUP_TYPE_USER_IMAGE,
				self::GROUP_TYPE_USER_TEXT,
			);
		}
		
		/**
		 * @return array
		 */
		public static function getOptionTypes()
		{
			return array(
				self::PRODUCT_OPTION_TYPE_AMOUNT => _t('Amount'),
				self::PRODUCT_OPTION_TYPE_PERCENT => '%',
			);
		}
		
		public static function getOptionsInfo($optionIds, $langaugeId = null)
		{
			if (count($optionIds) == 0)
				return array();
			
			$optionIds = array_map('sql_escape_string', $optionIds);
			$optionIds = '"' . implode('", "', $optionIds) . '"';
			$sql =  'SELECT *, Options.OptionID AS OptionID, Options.OptionGroupID AS OptionGroupID FROM Options ' .
					'LEFT JOIN OptionsML USING(OptionID) ' .
					'INNER JOIN OptionGroups ON (Options.OptionGroupID = OptionGroups.OptionGroupID) ' .
					'LEFT JOIN OptionGroupsML ON (OptionGroups.OptionGroupID = OptionGroupsML.OptionGroupID) ' .
					'WHERE Options.OptionID IN (' . $optionIds . ') AND OptionsML.LanguageID = ? AND OptionGroupsML.LanguageID = ?';
			
			if (is_null($langaugeId)) {
				$langaugeId = scMultiLanguage::getCurrentLanguageId();
			}
			$options = miStaticDBUtil::execSelect($sql, array($langaugeId, $langaugeId));
			$options = miUtil::getArrayWithKeys($options, 'OptionID');
			
			foreach ($options as $optionId => $optionData) {
				$options[$optionId]['OptionUploadedFileName'] = self::getSessionTempUserFileName('ProductOptions', $optionData['OptionGroupID'], $optionId);
				$options[$optionId]['OptionUserText'] = self::getSessionTempUserText('ProductOptions', $optionData['OptionGroupID'], $optionId);
			}
			
			return $options;
		}
		
		public static function setSessionTempUserText($optionType, $optionGroupId, $optionId, $text)
		{
			$_SESSION['ProductUserText'][$optionType][$optionGroupId][$optionId] = $text;
		}
		
		public static function getSessionTempUserText($optionType, $optionGroupId, $optionId)
		{
			if (!isset($_SESSION['ProductUserText'][$optionType][$optionGroupId][$optionId]))
				return '';
			
			return $_SESSION['ProductUserText'][$optionType][$optionGroupId][$optionId];
		}
		
		public static function setSessionTempUserFileName($optionType, $optionGroupId, $optionId, $tempFilename)
		{
			$_SESSION['ProductUserImages'][$optionType][$optionGroupId][$optionId] = $tempFilename;
		}
		
		public static function getSessionTempUserFileName($optionType, $optionGroupId, $optionId)
		{
			if (!isset($_SESSION['ProductUserImages'][$optionType][$optionGroupId][$optionId]) || !is_file(scPaths::getFilenamePath($_SESSION['ProductUserImages'][$optionType][$optionGroupId][$optionId], scPaths::RESOURCE_TEMP_IMAGE)))
				return '';
			
			return $_SESSION['ProductUserImages'][$optionType][$optionGroupId][$optionId];
		}
	}
?>