<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductService {
		
		const VIEW_LIST = 1;
		const VIEW_GRID = 2;
		
		const GRID_ROW_ITEMS_NUMBER = 4;
		
		const OUT_OF_STOCK_HANDLING_SELL = 0;
		const OUT_OF_STOCK_HANDLING_HIDE = 1;
		const OUT_OF_STOCK_HANDLING_TAG = 2;
		
		public static function getViewTypes()
		{
			return array(
				self::VIEW_LIST => _t('List view'),
				self::VIEW_GRID => _t('Grid view'),
			);
		}
		
		public static function getOutOfStockHandlings()
		{
			return array(
				self::OUT_OF_STOCK_HANDLING_SELL => _t('Keep selling'),
				self::OUT_OF_STOCK_HANDLING_HIDE => _t('Hide product'),
				self::OUT_OF_STOCK_HANDLING_TAG => _t('"Out of stock" tag')
			);
		}
		
		public static function getViewType()
		{
			if (!miHasParam('view')) {
				// Try to get from session
				if (array_key_exists('_productListingViewType', $_SESSION))
					return $_SESSION['_productListingViewType'];
				
				// Default view type
				$view = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CATEGORY_VIEW_TYPE);
			}
			else {
				// Get from request
				$view = miGetParam('view');
				if ($view == 'grid')
					$view = self::VIEW_GRID;
				else
					$view = self::VIEW_LIST;
			}
			
			// Save to session
			$_SESSION['_productListingViewType'] = $view;
			
			return $view;
		}
				
		/**
		 * @param string $sectionName
		 * @param array $products
		 * @param int $viewType
		 * @return miTemplateParserSectionInfo
		 */
		public static function getProductSections($sectionName, $products, $viewType = self::VIEW_LIST)
		{
			$listingItems = self::getListingItemsFromArray($products);
			
			if (!empty($listingItems)) {
				self::replaceOutOfStockVariants($listingItems);
			}
			
			return scProductListingService::singleton()->getSection($sectionName, $listingItems, $viewType);
		}
		
		/**
		 * @param array $products
		 * @return array([IProductListingItem])
		 */
		public static function getListingItemsFromArray($products)
		{
			$listingItems = array(); // Array of IProductListingItem
			foreach ($products as $productInfo) {
				$productDomainObject = new scProductDomainObject();
				$productDomainObject->setRow($productInfo);
				
				$hasVariants = array_key_exists('ProductVariantID', $productInfo) ? ($productInfo['ProductVariantID'] != null) : $productDomainObject->hasVariants();
				if ($hasVariants) {
					if (array_key_exists('ProductVariantID', $productInfo)) {
						$listingItem = new scProductVariant();
						$listingItem->setRow($productInfo);
					} else {
						$listingItem = $productDomainObject->getDefaultVariant();
					}
					$listingItems[] = $listingItem;
				} else {
					$product = new scProductSL();
					$product->setRow($productInfo);
					$listingItems[] = $product;
				}
			}
			return $listingItems;
		}
		
		public static function getProductsOptions($productIds = array(), $languageId = null)
		{
			// Check for empty request
			$productIds = array_map('intval', $productIds);
			if (empty($productIds)) {
				return array();
			}
			
			// Check for language
			if (is_null($languageId)) {
				$languageId = scMultiLanguage::getCurrentLanguageId();
			}
			
			// Initialize the return array
			$return = array();
			foreach ($productIds as $productId) {
				$return[$productId] = array(
					'ProductOptions' => array(),
					'LocalOptions' => array(),
				);
			}
			
			$productIdsString = implode(', ', $productIds);
			
			// Load the options information
			$sql =  'SELECT * FROM ProductOptionGroups LEFT JOIN OptionGroups USING (OptionGroupID) ' .
					'LEFT JOIN OptionGroupsML ON (OptionGroups.OptionGroupID = OptionGroupsML.OptionGroupID AND OptionGroupsML.LanguageID = ?) ' .
					'LEFT JOIN Options ON (ProductOptionGroups.OptionGroupID = Options.OptionGroupID) ' .
					'LEFT JOIN OptionsML ON (Options.OptionID = OptionsML.OptionID AND OptionsML.LanguageID = ?) ' .
					'WHERE ProductID IN (' . $productIdsString . ')'; 
			$optionGroups = miStaticDBUtil::execSelect($sql, array($languageId, $languageId));
			// Add options data to the return array
			foreach ($optionGroups as $optionGroup) {
				$optionProductId = $optionGroup['ProductID'];
				$optionGroupId = $optionGroup['OptionGroupID'];
				
				$return[$optionProductId]['ProductOptions'][$optionGroupId]['Options'][$optionGroup['OptionID']] = array(
					'OptionPriceModType' => $optionGroup['OptionPriceModType'],
					'OptionPriceModValue' => $optionGroup['OptionPriceModValue'],
					'OptionName' => $optionGroup['OptionName'],
					'OptionRequired' => $optionGroup['OptionRequired'],
				);
				
				$return[$optionProductId]['ProductOptions'][$optionGroupId]['OptionGroupName'] = $optionGroup['OptionGroupName'];
				$return[$optionProductId]['ProductOptions'][$optionGroupId]['OptionGroupType'] = $optionGroup['OptionGroupType'];
			}
			
			// Load the local options information
			$sql =  'SELECT * FROM LocalOptionGroups ' .
				'LEFT JOIN LocalOptionGroupsML ON (LocalOptionGroups.LocalOptionGroupID = LocalOptionGroupsML.LocalOptionGroupID AND LocalOptionGroupsML.LanguageID = ?) ' .
				'LEFT JOIN LocalOptions ON (LocalOptionGroups.LocalOptionGroupID = LocalOptions.LocalOptionGroupID) ' .
				'LEFT JOIN LocalOptionsML ON (LocalOptions.LocalOptionID = LocalOptionsML.LocalOptionID AND LocalOptionsML.LanguageID = ?) ' .
				'WHERE ProductID IN (' . $productIdsString . ') ORDER BY LocalOptionGroupSort,LocalOptionSort'; 
			$optionGroups = miStaticDBUtil::execSelect($sql, array($languageId, $languageId));
			// Add local options data to the return array
			foreach ($optionGroups as $optionGroup) {
				$optionProductId = $optionGroup['ProductID'];
				$optionGroupId = $optionGroup['LocalOptionGroupID'];
				
				$return[$optionProductId]['LocalOptions'][$optionGroupId]['Options'][$optionGroup['LocalOptionID']] = array(
					'OptionPriceModType' => $optionGroup['LocalOptionPriceModType'],
					'OptionPriceModValue' => $optionGroup['LocalOptionPriceModValue'],
					'OptionName' => $optionGroup['LocalOptionName'],
					'OptionRequired' => $optionGroup['LocalOptionRequired'],
				);
				
				$return[$optionProductId]['LocalOptions'][$optionGroupId]['OptionGroupName'] = $optionGroup['LocalOptionGroupName'];
				$return[$optionProductId]['LocalOptions'][$optionGroupId]['OptionGroupType'] = $optionGroup['LocalOptionGroupType'];
			}
			
			return $return;
		}
		
		/**
		 * Return the product records for the given category id and the subcategories
		 * 
		 * @param int $categoryId The category id
		 * @param array $selectFields If not empty, select only the given fields
		 * @param string $orderBy Order by these fields in the order they are in the array
		 */
		public static function getProductsInCategory($categoryId, $selectFields = array(), $orderBy)
		{
			$categoryId = intval($categoryId);
			
			$result = array();
				
			if (!empty($selectFields))
				$selectFieldsStr = implode(',', $selectFields);
			else
				$selectFieldsStr = 'Products.*, ProductsML.*';
			
			if ($categoryId > 0) {
				// Get the left and right weight of the category record
				$record = new miSqlRecord('Categories', 'CategoryID');
				$record->readPK($categoryId);
				
				$left = $record->get('CategoryLeft');
				$right = $record->get('CategoryRight');
				
				// Get all the subcategories and the current category
				$categories = miUtil::getDBArray('SELECT CategoryID FROM Categories WHERE CategoryLeft >= '. $left . ' AND CategoryRight <= ' . $right, 'CategoryID', 'CategoryID');
			}
			else {
				// Since the top category is selected get all the categories
				$categories = miUtil::getDBArray('SELECT CategoryID FROM Categories', 'CategoryID', 'CategoryID');	
			}
			
			$categories = '"' . implode('", "', $categories) . '"';
			
			// Get all the available products
			$languageId = scMultiLanguage::getCurrentLanguageId();
			$query = 'SELECT ' . $selectFieldsStr . ' FROM Products '.
					 'LEFT JOIN ProductsML ON ProductsML.ProductID = Products.ProductID ' .
					 'WHERE Products.CategoryID IN (' . $categories . ') AND ProductsML.LanguageID = ? ' .
					 'UNION DISTINCT ' .
					 'SELECT ' . $selectFieldsStr . ' FROM ProductXCategories PC ' .
					 'JOIN Products ON Products.ProductID = PC.ProductID ' .
					 'LEFT JOIN ProductsML ON ProductsML.ProductID = Products.ProductID ' .
					 'WHERE PC.CategoryID IN (' . $categories . ') AND ProductsML.LanguageID = ? ' .
					 'ORDER BY ' . $orderBy;
			
			return miStaticDBUtil::execSelect($query, array($languageId, $languageId));
		}
		
		public static function getProductsArrayForBox($products)
		{
			$displayItems = array();
			foreach ($products as $productInfo) {
				$product = new scProductSL();
				$product->setRow($productInfo);
				
				$productDomainObject = new scProductDomainObject();
				$productDomainObject->setRow($productInfo);
				
				$hasVariants = array_key_exists('ProductVariantID', $productInfo) ? ($productInfo['ProductVariantID'] != null) : $productDomainObject->hasVariants();
				if ($hasVariants) {
					if (array_key_exists('ProductVariantID', $productInfo)) {
						$listingItem = new scProductVariant();
						$listingItem->setRow($productInfo);
					} else {
						$listingItem = $productDomainObject->getDefaultVariant();
					}
				} else {
					$listingItem = $product;
				}
				
				$displayItem = array();
				$displayItem['ProductID'] = $listingItem->getProductId();
				$displayItem['ProductName'] = $listingItem->getName();
				$displayItem['ProductUrl'] = $listingItem->getUrl();
				$displayItem['ProductPrice'] = $listingItem->getPrice()->getAmount()->get();
				$displayItem['HTML_ProductDescription'] = $listingItem->getDescription();
				$displayItem['CategoryID'] = $listingItem->getCategoryId();
				$displayItem['ProductClassID'] = $listingItem->getProductClassId();
				
				$imageInfo = $listingItem->getImageInfo();
				$displayItem = array_merge($displayItem, $imageInfo->getDataArray('ProductImage'));
				
				$displayItem['Price'] = scPrice::factory($listingItem->getPrice()->getAmount())->getDisplayPrice();
				$displayItems[] = $displayItem;
			}
			
			return $displayItems;
		}
		
		/**
		 * Returns section with product info
		 *
		 * @param scProductSL $product array with product properties
		 * @param scProductVariant $productVariant
		 * @param array $selectedOptions
		 * @return miTemplateParserSectionInfo
		 */
		public static function getSingleProductSection(scProductSL $product, scProductVariant $productVariant = null, $selectedOptions = array())
		{
			if (is_null($productVariant) && $product->getProductDomainObject()->hasVariants()) {
				$productVariant = $product->getProductDomainObject()->getDefaultVariant();
				// The default variant may be out of stock. If there is a variant that is in stock
				// it should be shown instead of the default. The ready method replaceOutOfStockVariants() may be used.
				$listingItems = array($productVariant);
				self::replaceOutOfStockVariants($listingItems);
				$productVariant = reset($listingItems);
			}
			
			// Get a reference to the correct implementation of IProductListingItem in $listingItem
			if (is_null($productVariant)) {
				$listingItem =& $product;
				$hasOptionsSection = self::getProductOptionsSection($product, array(), $selectedOptions);
			} else {
				$listingItem =& $productVariant;
				$hasOptionsSection = self::getProductOptionsSection($product, $productVariant->getSelectedOptionsArray(), $selectedOptions);
			}
			
			$images = $product->getImages();
			
			// Product image enlarge section
			$defaultImage = self::getProductDefaultImage($product, $images, $productVariant);
			$imageEnlargeSection = new miTemplateParserSectionInfo('imageEnlarge', 0);
			if ($defaultImage->getLargeImage()->exists() and !($defaultImage->getLargeImage() instanceof scProductNoImageImageInfo) and !($defaultImage->getLargeImage() instanceof scProductNoLargeImageImageInfo)) {
				$imageEnlargeSection->setIterationsCount(1);
			}
			$defaultImageSection = new miTemplateParserSectionInfo('defaultImage', 1);
			$defaultImageSection->addSubsection($imageEnlargeSection);
			
			// Images gallery section
			$imagesData = array();
			$imageEnlargeSubsections = array();
			
			foreach ($images as $image) {
				$imagesData[] = $image->getDataArray('ProductImage');
				$hasLarge = $image->getLargeImage()->exists();
				$imageEnlargeSubsections[] = new miTemplateParserSectionInfo('imageEnlarge', (int)$hasLarge);
			}
			$productImagesSection = new miTemplateParserSectionInfo('productImages', count($imagesData), miUIUtil::templetizeRows(miUIUtil::transposeRows($imagesData)));
			$productImagesSection->addSubsectionsArray('imageEnlarge', $imageEnlargeSubsections);
			
			// Product price HTML
			$priceSection = scProductPrice::getPriceSection($listingItem, scProductPrice::getSingleProductPriceModifications($listingItem));
			
			// outOfStock and addToCart sections
			$isOutOfStockSection = new miTemplateParserSectionInfo('outOfStock', 0);
			$addToCartSection = new miTemplateParserSectionInfo('addToCart', 1);
			if (!$product->get('ProductIsActive')) {
				$isOutOfStockSection->setIterationsCount(0);
				$addToCartSection->setIterationsCount(0);
			} else if ($listingItem->isOutOfStock()) {
				$isOutOfStockSection->setIterationsCount(1);
				$addToCartSection->setIterationsCount(0);
			}
			
			// Set the description variables
			$description = $product->getDescription();;
			$detailedDescription = $product->getDetailedDescription();
			
			$listingItemData = array();
			
			// SKU
			$listingItemData['%%PRODUCTCODE%%'] = $listingItem->getSkuCode();
			
			// Weight
			$weight = $listingItem->getWeight();
			$listingItemData['%%PRODUCTWEIGHT%%'] = $weight->getWeight();
			$listingItemData['%%WEIGHTUNIT%%'] = miI18N::htmlEscape(scWeightUnits::getWeightUnitsName($weight->getWeightUnits()));
			$hasWeightSection = new miTemplateParserSectionInfo('productHasWeight', (($weight->getWeight() > 0) ? 1 : 0));
			
			// Manufacturer
			$manufacturer = $listingItem->getManufacturer();
			$manufacturerName = '';
			$manufacturerUrl = '';
			$hasManufacturerSection = new miTemplateParserSectionInfo('productHasManufacturer', 0);
			if (!is_null($manufacturer)) {
				$hasManufacturerSection->setIterationsCount(1);
				$manufacturerName = $manufacturer->getName();
				$manufacturerUrl = $manufacturer->getUrl();
				if (!empty($manufacturerUrl)) {
					$manufacturerLinkSection = new miTemplateParserSectionInfo('manufacturerLink', 1);
					$hasManufacturerSection->addSubsection($manufacturerLinkSection);
				}
			}
			$listingItemData['%%MANUFACTURERNAME%%'] = $manufacturerName;
			$listingItemData['%%MANUFACTURERURL%%'] = $manufacturerUrl;
			
			if (!is_null($productVariant)) {
				$listingItemData = array_merge(miUIUtil::templetizeRows($productVariant->getRow()), $listingItemData);
			}
			
			$productSection = new miTemplateParserSectionInfo('product');
			$productSection->setValues($listingItemData);
			
			$productSection->addSubsection($defaultImageSection);
			$productSection->addSubsection($productImagesSection);
			$productSection->addSubsection($isOutOfStockSection);
			$productSection->addSubsection($addToCartSection);
			$productSection->addSubsection($priceSection);
			$productSection->addSubsection($hasOptionsSection);
			$productSection->addSubsection($hasManufacturerSection);
			$productSection->addSubsection($hasWeightSection);
			$productSection->addSubsection(self::getNewProductSection($product));
			$productSection->addSubsection(self::getEgoodProductSection($product));
			$productSection->addSubsection(self::getProductAttributesSection($product));
			
			$productSection->addValues(miUIUtil::templetizeAndEscapeRows($product->getRow()));
			$productSection->addValues(miUIUtil::templetizeRows($defaultImage->getDataArray('ProductImage')));
			$productSection->addValues(array(
				'%%HTML_PRODUCTDETAILEDDESCRIPTION%%' => $detailedDescription,
				'%%HTML_PRODUCTDESCRIPTION%%' => $description,
				'%%CURRENT_CURRENCY_CODE%%' => scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), 
				'%%FINAL_PRICE%%' => scProductPrice::$_finalPrice
			));
			$variantsJson = $variantOptionsJson = scJson::jsonEncode(array());
			if ($productVariant != null) {
				$variantsJson = self::getProductVariantsJson($product);
				$variantOptionsJson = self::getProductVariantOptionsJson($product);
			}
			$productSection->addValues(array(
				'%%PRODUCTVARIANTID%%' => ($listingItem instanceof scProductVariant ? $listingItem->getPK() : ''),
				'%%PRODUCT_VARIANTS_JSON%%' => $variantsJson,
				'%%PRODUCT_VARIANT_OPTIONS_JSON%%' => $variantOptionsJson,
			));
			
			self::getModuleProductDetails($product, $productSection);
			
			return $productSection;
		}
		
		/**
		 * @param scProductSL $productObject
		 * @return array
		 */
		public static function getProductAttributes($productId, $productClassId, $includeSystemAttributes = scProductAttributes::GET_SYSTEM_ATTRIBUTES_NONE)
		{
			if (!$productClassId)
				return array();
			
			$productAttributeValues = scProductAttributes::singleton()->getProductAttributeValues($productId, $includeSystemAttributes);
			$productAttributeValues = miUtil::getArrayWithKeys($productAttributeValues, 'ProductClassAttributeID');
			
			$productClassAttributes = scProductAttributes::singleton()->getProductClassAttributes($productClassId, $includeSystemAttributes);
			
			$productAttributes = array();
			foreach ($productClassAttributes as $productClassAttributeId => $productClassAttribute) {
				if (!isset($productAttributeValues[$productClassAttributeId])) {
					continue; // Skip attributes that have not been selected for that product.
				}
				
				$productAttribute = array();
				$productAttribute['AttributeID'] = $productClassAttribute['ProductClassAttributeID'];
				$productAttribute['AttributeName'] = $productClassAttribute['ProductClassAttributeName'];
				$valueField = scProductAttributes::singleton()->getValueFieldForType($productClassAttribute['ProductClassAttributeType']);
				$value = $productAttributeValues[$productClassAttributeId][$valueField];
				if (empty($value)) {
					continue; // Skip attributes for which the value is empty.
				}
				
				// Check if it's an attribute with selectable values
				if (in_array($productClassAttribute['ProductClassAttributeType'], scProductAttributes::singleton()->getFieldTypesWithValues())) {
					if (!is_array($value) && !isset($productClassAttribute['ProductClassAttributeValues'][$value])) {
						continue; // Skip values that are no longer supported.
					}
					
					// Check if it's an attribute with multiple values
					if (is_array($value)) {
						$productAttribute['AttributeMultiValue'] = $value;
						$productAttribute['AttributeValue'] = '';
					} else {
						$productAttribute['AttributeValue'] = $productClassAttribute['ProductClassAttributeValues'][$value];
						$productAttribute['AttributeMultiValue'] = array();
					}
				} else {
					$productAttribute['AttributeValue'] = $value;
				}
				
				$productAttributes[] = $productAttribute;
			}
			
			return $productAttributes;
		}
		
		/**
		 * Returns if the zero quantity 
		 *
		 * @return boolean if the zero quantity products should be hidden
		 */
		public static function hideOutOfStock()
		{
			return scSettings::getSettingValue(scSettings::SETTING_OUT_OF_STOCK_HANDLING) == scProductService::OUT_OF_STOCK_HANDLING_HIDE;
		}
		
		public static function getProductGridColumns()
		{
			$cols = (int)scSettings::getSettingValue(scSettings::SETTING_GRID_VIEW_COLUMNS);
			
			if ($cols <= 0) {
				return self::GRID_ROW_ITEMS_NUMBER;
			}
			
			return $cols;
		}
		
		/**
		 * @param scProductSL $productObject
		 * @param array $images
		 * @return scProductImageInfo
		 */
		protected static function getProductDefaultImage(scProductSL $productObject, &$images, scProductVariant $selectedVariant = null)
		{
			if ($selectedVariant != null) {
				$variantImageInfo = $selectedVariant->getImageInfo();
				if ($variantImageInfo->exists())
					return $variantImageInfo;
			}
			
			if (count($images) == 0) {
				return scProductImage::getImageInfo(array());
			}
			
			return array_shift($images);
		}
		
		/**
		 * @param scProductSL $product
		 * @return miTemplateParserSectionInfo
		 */
		public static function getNewProductSection(IProductListingItem $listingItem)
		{
			$daysProductIsNew = (int)scSettings::getSettingValue(scSettings::SETTING_NEW_PRODUCTS_DAYS);
			$isNew = ($listingItem->getCreateTimestamp() >= (time() - ($daysProductIsNew * 24 * 60 * 60)));
			return new miTemplateParserSectionInfo('isNewProduct', (int)$isNew);
		}
		
		/**
		 * @param scProductSL $product
		 * @return miTemplateParserSectionInfo
		 */
		public static function getEgoodProductSection(IProductListingItem $listingItem)
		{
			return new miTemplateParserSectionInfo('isEgoodProduct', $listingItem->isEgood() ? 1 : 0);
		}
		
		public static function getProductPriceHtml(IProductListingItem $listingItem)
		{
			$priceSection = scProductPrice::getPriceSection($listingItem, scProductPrice::getSingleProductPriceModifications($listingItem));
			
			$t = new scTemplateParser();
			$t->assign('%%CURRENT_CURRENCY_CODE%%', scCurrencyUtil::getCurrentCurrency()->getCurrencyCode());
			$t->readTemplate('customer/product.tmpl');
			return $t->templateParsePart(array('product', 'price'), array($priceSection->getSectionName() => $priceSection));
		}
		
		public static function getWholesalePricesHtml(IProductListingItem $listingItem)
		{
			$product = new scProductSL();
			$product->read($listingItem->getProductId());
			
			$moduleClassId = scUtil::getDbValue('ModuleClasses', 'ModuleClassID', 'ModuleClassName', 'scWholesaleProductDetails');
			list($className, $moduleId) = scModulesFactory::loadClass($moduleClassId);
			$wholesalePricesModule = new $className;
			list($priceRows, $wholesalePriceSectionsArray, $wholesaleTaxedPriceSectionsArray) = $wholesalePricesModule->getPriceRows($product);
			
			$parser = new scTemplateParser();
			$parser->readTemplate('modules/system/wholesale/product_prices.tmpl');
			
			$pricesSection = new miTemplateParserSectionInfo('prices', count($priceRows), miUIUtil::templetizeRows(miUIUtil::transposeRows($priceRows)));
			$pricesSection->addSubsectionsArray('wholesalePrice', $wholesalePriceSectionsArray);
			$pricesSection->addSubsectionsArray('wholesaleTaxedPrice', $wholesaleTaxedPriceSectionsArray);
			$parser->addSectionInfo($pricesSection);
			
			return $parser->templateParse();
		}
		
		public static function getProductVariantImage(scProductVariant $variant)
		{
			$imagePath = scPaths::getStoreRootDir() . $variant->get('ProductVariantImage');
			if (!file_exists($imagePath)) {
				$productId = (int)$variant->get('ProductID');
				$product = new scProductSL();
				$product->read($productId);
				$images = $product->getImages();
				$imageInfo = self::getProductDefaultImage($product, $images, $variant);
			} else {
				$imageInfo = $variant->getImageInfo();
			}
			
			$section = new miTemplateParserSectionInfo('defaultImage', 1);
			
			$enlargeSection = new miTemplateParserSectionInfo('imageEnlarge', 0);
			if ($imageInfo->getLargeImage()->exists() and !($imageInfo->getLargeImage() instanceof scProductNoImageImageInfo) and !($imageInfo->getLargeImage() instanceof scProductNoLargeImageImageInfo)) {
				$enlargeSection->setIterationsCount(1);
			}
			
			$section->addSubsection($enlargeSection);
			
			$templateData = $variant->getRow();
			$templateData = array_merge($templateData, $variant->getProductSL()->getRow());
			$templateData = array_merge($templateData, $imageInfo->getDataArray('ProductImage'));
			
			$templateData = miUIUtil::templetizeAndEscapeRows($templateData);
			
			$t = new scTemplateParser();
			$t->readTemplate('customer/product.tmpl');
			$t->assignArray($templateData);
			return $t->templateParsePart(array('product', 'defaultImage'), array($section->getSectionName() => $section));
		}
		
		/**
		 * @return array([scProductVariant])
		 */
		public static function replaceOutOfStockVariants(&$listingItems, $variantClass = 'scProductVariant')
		{
			if (empty($listingItems)) {
				return;
			}
			
			// Check for variants that are out of stock and collect their product IDs in $outOfStock.
			$outOfStock = array();
			foreach ($listingItems as $listingItem) {
				if (!($listingItem instanceof IProductListingItem)) {
					throw new miException(_i('Each item must implement IProductListingItem interface.'));
				}
				if (($listingItem instanceof scProductVariant) && $listingItem->isOutOfStock()) {
					$outOfStock[] = $listingItem->getProductId();
				}
			}
			
			if (empty($outOfStock)) {
				// No variants are out of stock.
				return;
			}
			
			// Get in-stock variants for products whose default variant is out of stock.
			$recordset = new miSqlRecordset('ProductVariants');
			$recordset->addFilter(new miSqlFilterIn('ProductID', $outOfStock));
			$recordset->addFilter(new miSqlFilterBiggerThan('ProductVariantQuantity', 0));
			$recordset->setGroupBy('ProductID');
			$recordset->setOrder('ProductVariantPrice', 'ASC');
			$variants = $recordset->getAllRecords();
			$variants = miUtil::getArrayWithKeys($variants, 'ProductID');
			
			// Replace the out of stock variants with variants that are in stock.
			// FIXME: That may cause incorrect sorting of the products if the replacement variant
			// has different value for the column that is used to sort them.
			foreach ($listingItems as $key => $listingItem) {
				$productId = $listingItem->getProductId();
				if (isset($variants[$productId])) {
					$replacementVariant = new $variantClass();
					$replacementVariant->setRow($variants[$productId]);
					$listingItems[$key] = $replacementVariant;
				}
			}
		}
		
		/**
		 * @param array $product
		 * @param array $productVariantOptions
		 * @return miTemplateParserSectionInfo
		 */
		public static function getProductOptionsSection(scProductSL &$product, $productVariantOptions, $selectedOptions = array())
		{
			$options = $product->getOptionsData();
			if (!isset($selectedOptions['LocalOptions'])) {
				$selectedOptions['LocalOptions'] = array();
			}
			if (!isset($selectedOptions['ProductOptions'])) {
				$selectedOptions['ProductOptions'] = array();
			}
			if (array_key_exists(scProductVariantOption::TYPE_GLOBAL, $productVariantOptions)) {
				$selectedOptions['ProductOptions'] = $productVariantOptions[scProductVariantOption::TYPE_GLOBAL] + $selectedOptions['ProductOptions'];
			}
			if (array_key_exists(scProductVariantOption::TYPE_LOCAL, $productVariantOptions)) {
				$selectedOptions['LocalOptions'] = $productVariantOptions[scProductVariantOption::TYPE_LOCAL] + $selectedOptions['LocalOptions'];
			}
			
			list($optionsCount, $optionsSectionData) = self::getOptionsSectionData($options, $product, $selectedOptions);
			
			$hasOptionsSection = new miTemplateParserSectionInfo('hasOptions', 0);
			if ($optionsCount > 0) {
				$hasOptionsSection->setIterationsCount(1);
				
				$optionsSection = new miTemplateParserSectionInfo('options', $optionsCount, miUIUtil::templetizeRows($optionsSectionData));
				$hasOptionsSection->addSubsection($optionsSection);
			}
			
			return $hasOptionsSection;
		}
		
		/**
		 * Calls each module implementing IProductDetails and assigns the product details
		 *
		 * @param unknown_type $product
		 * @param miTemplateParserSectionInfo $productSection
		 */
		protected static function getModuleProductDetails($product, miTemplateParserSectionInfo $productSection)
		{
			$productDetails = '';
			
			$productDetailsModules = scModulesFactory::getImplementations('IProductDetails');
			foreach ($productDetailsModules as $productDetailsModule) {
				$details = $productDetailsModule->getProductDetailsHtml($product, $productSection);
				if ($details == '')
					continue;
				$productDetails .= $details;
			}
			
			$productSection->addValues(array('%%MODULES_PRODUCT_DETAILS%%' => $productDetails));
		}
		
		/**
		 * @param scProductSL $productObject
		 * @return miTemplateParserSectionInfo
		 */
		protected static function getProductAttributesSection($productObject)
		{
			$attributes = self::getProductAttributes($productObject->get('ProductID'), $productObject->get('ProductClassID'));
			$section = new miTemplateParserSectionInfo('productHasAttributes', 0);
			
			if (!empty($attributes)) {
				$section->setIterationsCount(1);
				
				$multiValueSectionArray = array();
				foreach ($attributes as &$attribute) {
					$multiValueSection = new miTemplateParserSectionInfo('multiValue', 0);
					if (!empty($attribute['AttributeMultiValue'])) {
						$multiValueSection->setIterationsCount(count($attribute['AttributeMultiValue']));
						$values = array('AttributeMultiValue' => array_values($attribute['AttributeMultiValue']));
						$multiValueSection->setValues(miUIUtil::templetizeAndEscapeRows($values));
					}
					$multiValueSectionArray[] = $multiValueSection;
					
					$attribute['AttributeMultiValue'] = '';
				}
				unset($attribute);
				
				$subsectionValues = miUIUtil::templetizeRows(miUIUtil::transposeRows($attributes));
				$subsection = new miTemplateParserSectionInfo('productAttributes', count($attributes), $subsectionValues);
				$subsection->addSubsectionsArray('multiValue', $multiValueSectionArray);
				$section->addSubsection($subsection);
			}
			
			return $section;
		}
		
		/**
		 * Returns a json array of all option groups (local and global) for this product that are of type "variant"
		 * @return string
		 */
		protected static function getProductVariantOptionsJson(scProductSL $productObject)
		{
			$productId = (int)$productObject->get('ProductID');
			
			$options = array(
				scProductVariantOption::TYPE_GLOBAL => array(),
				scProductVariantOption::TYPE_LOCAL => array(),
			);
			
			$recordset = new miSqlRecordset('ProductOptionGroups');
			$recordset->addJoinCondition('INNER', 'OptionGroups', 'USING(OptionGroupID)');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$recordset->addFilter(new miSqlFilterEqual('OptionGroupType', scProductOptions::GROUP_TYPE_VARIANT));
			$recordset->setSelectFields(array('OptionGroups.OptionGroupID'));
			$options[scProductVariantOption::TYPE_GLOBAL] = miUtil::getKeyValueArray($recordset->getAllRecords(), 'OptionGroupID', 'OptionGroupID');
			
			$recordset = new miSqlRecordset('LocalOptionGroups');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$recordset->addFilter(new miSqlFilterEqual('LocalOptionGroupType', scProductOptions::GROUP_TYPE_VARIANT));
			$recordset->setSelectFields(array('LocalOptionGroups.LocalOptionGroupID'));
			$options[scProductVariantOption::TYPE_LOCAL] = miUtil::getKeyValueArray($recordset->getAllRecords(), 'LocalOptionGroupID', 'LocalOptionGroupID');
			
			return scJson::jsonEncode($options);
		}
		
		/**
		 * Returns variant details for the specified product
		 * @return string
		 */
		protected static function getProductVariantsJson(scProductSL $productObject)
		{
			$productId = $productObject->get('ProductID');
			
			$recordset = new miSqlRecordset('ProductVariants');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$variants = $recordset->getAllRecords();
			if (empty($variants))
				return '';
			
			$variantIds = array_values(miUtil::getKeyValueArray($variants, 'ProductVariantID', 'ProductVariantID'));
			
			$identifiers = self::getVariantIdentifiers($variantIds);
			
			$output = array();
			foreach ($variants as $variant) {
				$output[$identifiers[$variant['ProductVariantID']]] = array(
					'ProductVariantID' => $variant['ProductVariantID'],
					'ProductVariantCode' => $variant['ProductVariantCode'],
					'ProductVariantPrice' => $variant['ProductVariantPrice'],
					'ProductVariantWeight' => $variant['ProductVariantWeight'],
					'ProductVariantQuantity' => $variant['ProductVariantQuantity'],
					'ProductVariantImage' => $variant['ProductVariantImage'],
				);
			}
			
			return scJson::jsonEncode($output);
		}
		
		protected static function getVariantIdentifiers($variantIds)
		{
			$options = array(
				scProductVariantOption::TYPE_GLOBAL => array(),
				scProductVariantOption::TYPE_LOCAL => array(),
			);
			$recordset = new miSqlRecordset('ProductVariantOptions');
			$recordset->addFilter(new miSqlFilterIn('ProductVariantID', $variantIds));
			$recordset->setOrder('OptionID');
			$variantOptions = $recordset->getAllRecords();
			foreach ($variantOptions as $option) {
				if (!isset($options[scProductVariantOption::TYPE_GLOBAL][$option['ProductVariantID']])) {
					$options[scProductVariantOption::TYPE_GLOBAL][$option['ProductVariantID']] = scProductVariantOption::TYPE_GLOBAL . ':';
				}
				$options[scProductVariantOption::TYPE_GLOBAL][$option['ProductVariantID']] .= $option['OptionID'] . '-';
			}
			
			$recordset = new miSqlRecordset('ProductVariantLocalOptions');
			$recordset->addFilter(new miSqlFilterIn('ProductVariantID', $variantIds));
			$recordset->setOrder('LocalOptionID');
			$variantOptions = $recordset->getAllRecords();
			foreach ($variantOptions as $option) {
				if (!isset($options[scProductVariantOption::TYPE_LOCAL][$option['ProductVariantID']])) {
					$options[scProductVariantOption::TYPE_LOCAL][$option['ProductVariantID']] = scProductVariantOption::TYPE_LOCAL . ':';
				}
				$options[scProductVariantOption::TYPE_LOCAL][$option['ProductVariantID']] .= $option['LocalOptionID'] . '-';
			}
			
			$identifier = array();
			foreach ($variantIds as $variantId) {
				$identifier[$variantId] = '';
				if (isset($options[scProductVariantOption::TYPE_GLOBAL][$variantId])) {
					$identifier[$variantId] .= $options[scProductVariantOption::TYPE_GLOBAL][$variantId];
				}
				$identifier[$variantId] .= '|';
				if (isset($options[scProductVariantOption::TYPE_LOCAL][$variantId])) {
					$identifier[$variantId] .= $options[scProductVariantOption::TYPE_LOCAL][$variantId];
				}
			}
			
			return $identifier;
		}
		
		/**
		 * Uses the passed $optionsArray to generate Section Data array.
		 * Retuns an array of Options Count and Options Section Data.
		 * 
		 * @param array $optionsArray
		 * @param scProductSL $product Product data
		 * @return array(Options Count, Options Section Data)
		 */
		public static function getOptionsSectionData($optionsArray, $product, $selectedOptions = array())
		{
			$optionsSectionData = array();
			$optionsCount = 0;
			$productId = $product->getProductId();
			$productPrice = $product->getPrice()->getAmount();
			foreach ($optionsArray as $optionType => $options) {
				// Add the options section
				foreach ($options as $optionGroupId => $optionGroup) {
					switch ($optionGroup['OptionGroupType']) {
						case scProductOptions::GROUP_TYPE_VARIANT:
						case scProductOptions::GROUP_TYPE_PRICE_MODIFIER:
							$optionNames = array();
							foreach ($optionGroup['Options'] as $optionId => $option) {
								$optionInfo = $option;
								$optionInfo['OptionID'] = $optionId;
								$optionInfo['OptionGroupID'] = $optionGroupId;
								$optionInfo['OptionGroupName'] = $optionGroup['OptionGroupName'];
								$optionInfo['OptionGroupType'] = $optionGroup['OptionGroupType'];
								$optionObject = new scProductOption($optionId, $optionInfo);
								
								$priceChange = $optionObject->getOptionPriceChange($productPrice);
								$displayPricesTaxMode = scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE);
								if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
									$priceChange = scProductPrice::getTaxedAmount($priceChange, $productId);
								}
								$priceChange = scPrice::factory($priceChange);
								$priceChange = $priceChange->getDisplayPrice();
								// Construct the price variation
								if ($option['OptionPriceModValue'] != 0) {
									if ($option['OptionPriceModValue'] > 0) {
										$priceChange = '+ ' . $priceChange;
									}
									if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_AND_WITHOUT_TAX) {
										$priceChange .= ' ' . _t('without tax');
									}
									$option['OptionName'] .= ' (' . $priceChange . ')';
								}
								
								$optionNames[$optionId] = $option['OptionName'];
							}
							
							$selected = 0;
							if (isset($selectedOptions[$optionType]) && isset($selectedOptions[$optionType][$optionGroupId])) {
								$selected = $selectedOptions[$optionType][$optionGroupId];
							}
							$optionField = scUIUtil::getConstDropdown('Options[' . $optionType . '][' . $optionGroupId . '][]', $selected, $optionNames);
							break;
							
						case scProductOptions::GROUP_TYPE_USER_IMAGE:
							$optionField = '';
							foreach ($optionGroup['Options'] as $optionId => $option) {
								$optionField .= self::getFileuploadOptionField( 'Options[' . $optionType . '][' . $optionGroupId . ']' . '[' . $optionId . ']', (bool)$option['OptionRequired'], $option['OptionName']);
							}
							break;
							
						case scProductOptions::GROUP_TYPE_USER_TEXT:
							$optionField = '';
							foreach ($optionGroup['Options'] as $optionId => $option) {
								$optionField .= self::getUserTextOptionField('Options[' . $optionType . '][' . $optionGroupId . ']' . '[' . $optionId . ']', (bool)$option['OptionRequired'], $option['OptionName']);
							}
							break;
					}
					
					$optionsSectionData['Options'][] = $optionField;
					$optionsSectionData['OptionGroup'][] = $optionGroup['OptionGroupName'];
					
					$optionsCount++;
				}
			}
			
			return array(
				$optionsCount,
				$optionsSectionData
			);
		}
		
		protected static function getOptionField($attributes, $required = false, $label = false)
		{
			$html = '';
			$html .= '<div class="product_option' . (isset($attributes['div.class']) ? ' ' . $attributes['div.class']  : '') . '">';
			$html .= '<div>';
			if ($label !== false)
				$html .= '<label> ' . $label . ' </label>';
			if ($required !== false) 
				$html .= '<span class="asterisk">*</span>';
			$html .= '</div>';
			
			$attrString = array();
			foreach ($attributes as $attribute => $value) {
				$attrString[] = $attribute . '="' . $value . '"';
			}
			$html .= '<input ' . implode(' ', $attrString) . '/>';
			
			$html .= '</div>'; 
			return $html;
		}
		
		protected static function getFileUploadOptionField($name, $required = false, $label = false, $id = false)
		{
			$attributes = array(
				'name' => $name,
				'type' => 'file',
			);
			if ($required)
				$attributes['class'] = 'required';
			if ($id)
				$attributes['id'] = $id;
			return self::getOptionField($attributes, $required, $label);
		}
		
		protected static function getUserTextOptionField($name, $required = false, $label = false, $id = false)
		{
			$attributes = array(
				'name' => $name,
				'type' => 'text',
			);
			if ($required)
				$attributes['class'] = 'required';
			if ($id)
				$attributes['id'] = $id;
			return self::getOptionField($attributes, $required, $label);
		}
	}
?>