<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductSortService {
		const SORT_BY_RELEVANCE = 0;
		const SORT_BY_PRICE_ASC = 1;
		const SORT_BY_PRICE_DESC = 2;
		const SORT_BY_NAME = 3;
//		const SORT_BY_CODE = 4;

		protected static function getSortCondition()
		{
			if (miHasParam('ProductSort')) {
				$sort = miGetParam('ProductSort');
				$_SESSION['_productListingSortCondition'] = $sort;
				return $sort;
			}
			else {
				if (array_key_exists('_productListingSortCondition', $_SESSION))
					return $_SESSION['_productListingSortCondition'];
			}
			
			return scSettings::getSettingValue(scSettings::SETTING_DEFAULT_PRODUCT_ORDER);
		}
		
		public static function getSortOptionsArray()
		{
			return array(
					self::SORT_BY_RELEVANCE => _t('Sort by relevance'),
					self::SORT_BY_PRICE_ASC => _t('Sort by price (low to high)'),
					self::SORT_BY_PRICE_DESC => _t('Sort by price (high to low)'),
					self::SORT_BY_NAME => _t('Sort by product name'),
//					self::SORT_BY_CODE => _t('Sort by product code')
			);
		}
		
		public static function getSortDropdown()
		{
			$sortCondition = self::getSortCondition();
			$options = self::getSortOptionsArray();
			
			if (!array_key_exists($sortCondition, $options))
				$sortCondition = self::SORT_BY_RELEVANCE;

			return miUIUtil::getConstDropdown('ProductSort', $sortCondition, $options);
		}
		
		public static function getSortClause()
		{			
			switch (intval(self::getSortCondition()))
			{
				case self::SORT_BY_PRICE_ASC:
					return scCurrencyUtil::getProductPriceColumnName() . ' ASC';
					break;
				case self::SORT_BY_PRICE_DESC:
					return scCurrencyUtil::getProductPriceColumnName() . ' DESC';
					break;
				case self::SORT_BY_NAME:
					return 'ProductName ASC';
					break;
//				case self::SORT_BY_CODE:
//					return 'Products.ProductCode ASC';
//					break;
				case self::SORT_BY_RELEVANCE:
				default:
					return 'ProductIDSort';
			}
		}
		
	    public static function getMaxSortInCategory($categoryId)
	    {
	    	$categoryId = (int)$categoryId;
	    	
	    	$query = 'SELECT MAX(MaxProductSortInCategory) as MaxSic FROM ' .
	    				'(SELECT MAX(ProductSortInCategory) AS MaxProductSortInCategory FROM Products WHERE Products.CategoryID = ? '.
						'UNION ' .
						'SELECT MAX(ProductSortInCategory) AS MaxProductSortInCategory FROM ProductXCategories WHERE ProductXCategories.CategoryID = ?' .
						') AS temp';
	    	$result = miStaticDBUtil::execSelect($query, array($categoryId, $categoryId));
	  		
	    	return $result[0]['MaxSic'];
	    }
	    
	    public static function getProductSortInCategory($productId, $categoryId)
	    {
	    	$query = 'SELECT ProductSortInCategory FROM Products WHERE CategoryID = ? AND ProductID = ? ' .
						'UNION ' .
					 'SELECT ProductSortInCategory FROM ProductXCategories WHERE CategoryID = ? AND ProductID = ?';
	    	$result = miStaticDBUtil::execSelect($query, array($categoryId, $productId, $categoryId, $productId));
	    	if (!isset($result[0]['ProductSortInCategory']))
	    		return false;
	    	return $result[0]['ProductSortInCategory'];
	    }
	}
?>