<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductVariantsUtil {
		
		const VARIANT_SKU_SEPARATOR = '-';
		
		/**
		 * Generate product variants (removing old variants, and creating new ones)
		 */
		public static function generateVariants(scProductDomainObject $product, ISet $variantGroups)
		{
			$variants = $variantGroups->getValues();
			
			$variantNum = 0;
			$insertVariants = array();
			$existingVariants = array();
			$existingCodes = array();
			foreach ($variants as $variantOptions) {
				$variant = new scProductVariant();
				$variant->set('ProductID', $product->get('ProductID'));
				$variant->setOptions($variantOptions);
				$variantId = $variant->readByOptions(false);
				
				if (!is_null($variantId) && !isset($existingVariants[$variantId])) {
					$variantNum += 1;
					$codeNum = str_replace($product->get('ProductCode'), '', $variant->get('ProductVariantCode'));
					if (is_numeric($codeNum)) {
						// Store codes that may cause duplication in $existingCodes
						$existingCodes[] = $codeNum;
						// Make it easier for the generator to find an unused product code.
						$variantNum = max($variantNum, $codeNum);
					}
					$existingVariants[$variantId] = $variant;
					continue;
				}
				
				$variant->set('ProductVariantID', null);
				$variant->set('ProductVariantIsDefault', false);
				if (!$variant->has('ProductVariantPrice')) {
					$variant->set('ProductVariantPrice', $product->get('ProductPrice'));
					$variant->set('ProductVariantWeight', $product->get('ProductWeight'));
					$variant->set('ProductVariantQuantity', 0);
				}
				$insertVariants[] = $variant;
			}
			
			// Find and delete all product variants that were not recognized as $existingVariants
			$query = 'SELECT ProductVariantID FROM ProductVariants WHERE ProductID = ' . intval($product->get('ProductID'));
			if (!empty($existingVariants)) {
				$query .= ' AND ProductVariantID NOT IN (' . implode(', ', array_keys($existingVariants)) . ')';
			}
			$deleteIds = miStaticDBUtil::execSelect($query);
			
			if (!empty($deleteIds)) {
				// Get the IDs of all variants that need to be deleted
				$deleteIds = array_values(miUtil::getKeyValueArray($deleteIds, 'ProductVariantID', 'ProductVariantID'));
				
				// Delete them using the domain object
				$domainObject = new scProductVariant();
				$domainObject->delete($deleteIds);
			}
			
			// Some options may have been converted from type "variant" to some other type.
			// In case a variant generated for those still exists ($existingVariants) records
			// also exist in ProductVariantLocalOptions and ProductVariantOptions.
			// If the type has been changed those records have to be deleted.
			
			if (!empty($existingVariants)) {
				foreach ($existingVariants as $variant) {
					$variant->update();
				}
			}
			
			// $insertVariants contains an array of scProductVariant domain objects that need to be inserted.
			foreach ($insertVariants as $variant) {
				$variantNum += 1;
				// Generate a SKU code for the variant
				$variantSku = self::getVariantSku($existingCodes, $product->get('ProductCode'), $variantNum);
				$variant->set('ProductVariantCode', $variantSku);
				$variant->insert();
			}
			
			self::setDefaultVariant($product->get('ProductID'));
		}
		
		/**
		 * Makes sure there is default product variant
		 * @param int $productId
		 */
		protected static function setDefaultVariant($productId)
		{
			$sql = 'SELECT ProductVariantIsDefault FROM ProductVariants WHERE ProductID=' . intval($productId);
			$rows = miUtil::getDBArray($sql, 'ProductVariantIsDefault', 'ProductVariantIsDefault');
			
			// If there are no variants: return
			if (empty($rows))
				return;
			
			// If there is a default variant: return
			foreach ($rows as $isDefault)
				if ($isDefault)
					return;
			
			// There are variants, but no default variant
			$sql = 'UPDATE ProductVariants SET ProductVariantIsDefault=1 WHERE ProductID=? ORDER BY ProductVariantID LIMIT 1';
			miStaticDBUtil::execSQL($sql, array($productId));
		}
		
		/**
		 * @param array $takenSkus Taken = Already in use
		 * @param string $productSku
		 * @param int $variantNum
		 * @return string
		 */
		protected static function getVariantSku(&$takenSkus, $productSku, &$variantNum)
		{
			$variantCode = $productSku . self::VARIANT_SKU_SEPARATOR . $variantNum;
			$checkExists = scUtil::getDbValue('ProductVariants', 'ProductVariantID', 'ProductVariantCode', $variantCode);
			
			if (in_array($variantNum, $takenSkus) || $checkExists) {
				return self::getVariantSku($takenSkus, $productSku, ++$variantNum);
			}
			
			$takenSkus[] = $variantNum;
			return $variantCode;
		}
	}
?>