<?php
	class scWholesalePrices {
		
		protected static $_productWholesalePricesCache = array();
		
		protected static $_variantWholesalePricesCache = array();
		
		/**
		 * @param IProductListingItem $listingItem
		 * @return scPrice
		 */
		public static function getListingItemWholesalePrice(IProductListingItem $listingItem, $quantity = 1)
		{
			// Check that wholesale prices are enabled
			if (!scSettings::getSettingValue(scSettings::SETTING_WHOLESALE_PRICES))
				return $listingItem->getPrice();
			
			$customerGroupId = (int)scCustomerAuth::singleton()->getCustomerGroupId();
			
			$productKey = md5($customerGroupId . '-' . $quantity . '-' . $listingItem->getProductId());
			if (isset(self::$_productWholesalePricesCache[$productKey])) {
				return self::$_productWholesalePricesCache[$productKey];
			}
			
			if ($listingItem instanceof scProductVariant) {
				$variantKey = md5($customerGroupId . '-' . $quantity . '-' . $listingItem->getPK());
				if (isset(self::$_variantWholesalePricesCache[$variantKey])) {
					return self::$_variantWholesalePricesCache[$variantKey];
				}
				
				$price = miStaticDBUtil::execSelect('SELECT ProductWholesalePriceAmount FROM ProductWholesalePrices' .
					' WHERE CustomerGroupID = ? AND ProductWholesalePriceQuantity <= ? AND ProductVariantID = ?' .
					' ORDER BY ProductWholesalePriceQuantity DESC LIMIT 1',
					array($customerGroupId, $quantity, $listingItem->getPK()));
				if (!empty($price)) {
					$amount = scMultiCurrencyAmount::factoryBaseCurrency($price[0]['ProductWholesalePriceAmount']);
					$price = scPrice::factory($amount);
					self::$_variantWholesalePricesCache[$variantKey] = $price;
					return $price;
				}
			}
			
			$sql = 'SELECT ProductWholesalePriceAmount FROM ProductWholesalePrices WHERE CustomerGroupID = ?' .
				' AND ProductWholesalePriceQuantity <= ? AND ProductID = ? AND ProductVariantID IS NULL ORDER BY ProductWholesalePriceQuantity DESC LIMIT 1';
			$price = miStaticDBUtil::execSelect($sql, array($customerGroupId, $quantity, $listingItem->getProductId()));
			if (count($price) == 0) {
				$price = $listingItem->getPrice();
				self::$_productWholesalePricesCache[$productKey] = $price;
				return $price;
			}
			
			$amount = scMultiCurrencyAmount::factoryBaseCurrency($price[0]['ProductWholesalePriceAmount']);
			$price = scPrice::factory($amount);
			self::$_productWholesalePricesCache[$productKey] = $price;
			return $price;
		}
		
		public static function productHasWholesalePrice($productId)
		{
			$sql = 'SELECT COUNT(1) AS N FROM ProductWholesalePrices WHERE CustomerGroupID = ?' .
				' AND ProductWholesalePriceQuantity <= 1 AND ProductID = ? AND ProductVariantID IS NULL';
			$count = miStaticDBUtil::execSelect($sql, array(scCustomerAuth::singleton()->getCustomerGroupId(), $productId));
			return ($count[0]['N'] > 0) ? true : false;
		}
	}
?>