<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSession {
		const MI_SESSION_NAME = 'MIPHPF_SESSION';
		
		/**
		 * Returns the scSession object
		 * Singleton pattern
		 *
		 * @return scSession the session object
		 */
		public static function singleton()
		{
			static $obj = null;
			
			if ($obj == null)
				$obj = new scSession;
			return $obj;
		}
		
		
		public function createSession()
		{
			# Create new session
			session_set_cookie_params(0, scPaths::getStoreWebPath(), self::getSessionCookieDomain());
			session_name($this->get_session_cookie_name());
			
			@session_start();
			$oldSessionId = session_id();
			session_regenerate_id();
			scSessionStorageDB::singleton()->destroy($oldSessionId);
		}
		
		public function destroySession()
		{
			session_destroy();
		}
		
		public function flushSession()
		{
			session_write_close();
		}
		
		public function getSessionId()
		{
			return session_id();
		}
		
		/**
		 * Opens the session
		 * 
		 * @param string|null $sessionId the session id (optional)
		 */
		function openSession($sessionId = null)
		{
			// Prevent reopening the session
			$currentSessionId = @session_id();
			if ($currentSessionId != '')
				return;
			
			// Open the session
			session_set_cookie_params(0, scPaths::getStoreWebPath(), self::getSessionCookieDomain());
			session_name($this->get_session_cookie_name());
			if ($sessionId != null)
				session_id($sessionId);

			session_start();
		}
		
		protected function getSessionCookieDomain()
		{
			if (!isset($_SERVER['HTTP_HOST'])) {
				// HTTP_HOST may be unavailable
				return null;
			}
			if ($_SERVER['HTTP_HOST'] == 'localhost') {
				return null;
			}
			if (ip2long($_SERVER['HTTP_HOST']) !== false) {
				// If HTTP_HOST is an IP address
				return null;
			}
			
			// Remove www.
			$domain = $_SERVER['HTTP_HOST'];
			if (substr($domain, 0, 4) == 'www.')
				$domain = substr($domain, 4);
			// Add all-subdomains dot
			$domain = '.' . $domain;
			return $domain;
		}
		
		
		/**
		 * Use the server name for the computation of the session name, so that we can have mutliple
		 * concurrent sessions on a single server
		 */
		protected function get_session_cookie_name()
		{ 
			return self::MI_SESSION_NAME . crc32($_SERVER['SERVER_NAME']);
		}
	}
?>