<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/../../../miphpf/database/DBUtilImpl.php');

	/**
	 * Session storage in DB
	 *
	 */
	class scSessionStorageDB {
		protected static $_instance = null;
		protected $maxLifeTime;
		protected $maxSessionLockSeconds = 2;	// Don't allow someone to keep the lock for more than 2 seconds
		protected $sessionLock = null;
		
		protected function __construct()
		{
			session_set_save_handler(
				array($this, 'open'),
				array($this, 'close'),
				array($this, 'read'),
				array($this, 'write'),
				array($this, 'destroy'),
				array($this, 'gc')
			);
			// TODO: V1.1 make this configurable
			$this->maxLifeTime = 60 * 60;	// One hour
			//$this->maxLifeTime = ini_get('session.gc_maxlifetime');
		}
		
		/**
		 * Return the only instance of the object
		 * @return scSessionStorageDB
		 */
		public static function singleton()
		{
			if (self::$_instance == null)
				self::$_instance = new scSessionStorageDB;
			return self::$_instance;
		}
		
		public function open()
		{
			return true;
		}
		
		public function close()
		{
			if ($this->sessionLock !== null) {
				$sql = 'DO RELEASE_LOCK(?)';
				miStaticDBUtil::execSQL($sql, array($this->sessionLock));
				$this->sessionLock = null;
			}
			return true;
		}
		
		public function read($sessionId)
		{
			$sessionId = sprintf('%u', crc32($sessionId));
			
			$this->sessionLock = miSettings::singleton()->get('MI_DEFAULT_DB_NAME') . '.sc.' . $sessionId;
			$sql = 'SELECT GET_LOCK(?, ?) AS N';
			$result = miStaticDBUtil::execSelect($sql, array($this->sessionLock, $this->maxSessionLockSeconds));
			if ($result[0]['N'] != 1)
				$this->sessionLock = null;	// Was not obtained, but continue anyway
			
			$sql = 'SELECT SessionData FROM Sessions WHERE SessionID=? AND SessionLastAccess > ' . (time() - $this->maxLifeTime);
			$rows = miStaticDBUtil::execSelect($sql, array($sessionId));
			if (count($rows) != 1)
				return '';
			return $rows[0]['SessionData'];
		}
		
		public function write($sessionId, $sessionData)
		{
			ignore_user_abort(true);
			flush();
			
			// In newer PHP (5.2.10) the order of class destruction has been reversed
			if (!class_exists('miSettings', false)) {
				require(dirname(__FILE__) . '/../../../miphpf/util/Settings.php');
				require(dirname(__FILE__) . '/../../../miphpf/Config.php');
			}
			if (!class_exists('miLoader', false))
				require(dirname(__FILE__) . '/../../../miphpf/util/Loader.php');
			if (!class_exists('miStaticDBUtil', false))
				require(dirname(__FILE__) . '/../../../miphpf/database/StaticDBUtil.php');
			if (!class_exists('miDBUtilImpl', false))
				require(dirname(__FILE__) . '/../../../miphpf/database/DBUtilImpl.php');
			if (!class_exists('miException', false))
				require(dirname(__FILE__) . '/../../../miphpf/util/Exceptions.php');
			
			$sessionId = sprintf('%u', crc32($sessionId));
			$sql = 'REPLACE INTO  Sessions(SessionID,SessionData,SessionLastAccess) VALUES(?,?,?)';
			miStaticDBUtil::execSQL($sql, array($sessionId, $sessionData, time()));
			
/*			$sql = 'SELECT COUNT(*) AS N FROM Sessions WHERE SessionID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($sessionId));
			if ($rows[0]['N'] > 0) {
				$values = array('SessionData' => $sessionData, 'SessionLastAccess' => time());
				miStaticDBUtil::execUpdate('Sessions', $values, 'SessionID', $sessionId);
			} else {
				// Do not create empty records
				if ($sessionData == '')
					return true;
				
				$values = array('SessionID' => $sessionId, 'SessionData' => $sessionData, 'SessionLastAccess' => time());
				miStaticDBUtil::execInsert('Sessions', $values);
			}*/
			return true;
		}
		
		public function destroy($sessionId)
		{
			miStaticDBUtil::execDelete('Sessions', 'SessionID', sprintf('%u', crc32($sessionId)));
			return true;
		}
		
		public function gc($maxLifeTime)
		{
			$sql = 'DELETE FROM Sessions WHERE SessionLastAccess < ' . (time() - $maxLifeTime);
			miStaticDBUtil::execSQL($sql);
			return true;
		}
	}
?>