<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scVatNumbersUtil {
		
		protected static $_vatRegEx = array(
			// EU countries
			'AT' => 'U[0-9]{8}',
			'BE' => '[0-9]{10}',
			'BG' => '[0-9]{9,10}',
			'CY' => '[0-9]{8}[A-Za-z]',
			'CZ' => '[0-9]{8,10}',
			'DE' => '[0-9]{9}',
			'DK' => '[0-9]{2} ?[0-9]{2} ?[0-9]{2} ?[0-9]{2}',
			'EE' => '[0-9]{9}',
			'EL' => '[0-9]{9}',
			'ES' => '([A-Za-z0-9][0-9]{7}[A-Za-z0-9])',
			'FI' => '[0-9]{8}',
			'FR' => '[A-Za-z0-9]{2} ?[0-9]{9}',
			'GB' => '([0-9]{3} ?[0-9]{4} ?[0-9]{2}|[0-9]{3} ?[0-9]{4} ?[0-9]{2} ?[0-9]{3})',
			'HU' => '[0-9]{8}',
			'IE' => '[0-9][A-Za-z0-9\+\*][0-9]{5}[A-Za-z]',
			'IT' => '[0-9]{11}',
			'LT' => '([0-9]{9}|[0-9]{12})',
			'LU' => '[0-9]{8}',
			'LV' => '[0-9]{11}',
			'MT' => '[0-9]{8}',
			'NL' => '[0-9]{9}B[0-9]{2}',
			'PL' => '[0-9]{10}',
			'PT' => '[0-9]{9}',
			'RO' => '[0-9]{2,10}',
			'SE' => '[0-9]{12}',
			'SI' => '[0-9]{8}',
			'SK' => '[0-9]{10}',
			
			// Other countries
		);
		
		protected static $_countryVatValidators = array(
			'BG' => 'scValidatorBgVatNumber',
		);
		
		public static function isCountryFormatKnown($countryCode)
		{
			return isset(self::$_vatRegEx[$countryCode]);
		}
		
		public static function isKnownFormat($vatNumber)
		{
			$countryCode = substr($vatNumber, 0, 2);
			if (!isset(self::$_vatRegEx[$countryCode])) {
				return false;
			}
			return true;
		}
		
		public static function isValidNumber($vatNumber)
		{
			if (!self::isKnownFormat($vatNumber)) {
				return false;
			}
			
			$countryCode = substr($vatNumber, 0, 2);
			$vatNumber = substr($vatNumber, 2);
			
			if (!(bool)preg_match('/^' . self::$_vatRegEx[$countryCode] . '$/', $vatNumber)) {
				return false;
			}
			
			if (!isset(self::$_countryVatValidators[$countryCode])) {
				return true;
			}
			
			$className = self::$_countryVatValidators[$countryCode];
			return call_user_func_array(array($className, 'isValidNumber'), array($vatNumber));
		}
		
		public static function getCountryFromNumber($vatNumber)
		{
			if (!self::isKnownFormat($vatNumber)) {
				return '';
			}
			
			return substr($vatNumber, 0, 2);
		}
	}
	
	class scValidatorVat extends miValidator {
		
		/**
		 * @param string $number
		 */
		public static function isValidNumber($vatNumber)
		{
			throw new miException(_i('Not implemented.'));
		}
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = (string)$this->_webForm->getSubmittedData($this->_fieldName);
			if (strlen($fieldValue) == 0) {
				return;
			}
			
			if (!self::isValidNumber($fieldValue)) {
				$errors->addError($this->_fieldName, _t('Invalid VAT number.'));
			}
		}
	}
	
	class scValidatorBgVatNumber extends scValidatorVat {
		
		public static function isValidNumber($number)
		{
			if (strlen($number) == 10) {
				return scValidatorBgEgn::isValidNumber($number);
			}
			
			if (in_array(strlen($number), array(9, 11, 13, 15))) {
				return scValidatorBgEik::isValidNumber($number);
			}
			
			return false;
		}
	}
	
	class scValidatorBgEgn extends scValidatorVat {
		
		protected static $_weights = array(2, 4, 8, 5, 10, 9, 7, 3, 6);
		
		public static function isValidNumber($number)
		{
			if (!preg_match('#^[0-9]{10}$#', $number)) {
				return false;
			}
			
			if (!checkdate($number[2] . $number[3], $number[4] . $number[5], strftime('%C') . $number[0] . $number[1])) {
				return false;
			}
			
			$sum = 0;
			for ($n = 0; $n < 9; $n++) {
				$sum += $number[$n] * self::$_weights[$n];
			}
			$control = $sum % 11;
			
			if ($control == 10) {
				if ($number[9] != 0) {
					return false;
				}
			} else if ($control != $number[9]) {
				return false;
			}
			return true;
		}
	}
	
	class scValidatorBgEik extends scValidatorVat {
		
		protected static $_weights = array(1, 2, 3, 4, 5, 6, 7, 8, 2, 7, 3, 5);
		
		public static function isValidNumber($number)
		{
			$number = preg_replace('/^BG/i', '', $number);
		
			if (!preg_match('#^[0-9]+$#', $number)) {
				return false;
			}
			
			$length = strlen($number);
			if (!in_array($length, array(9, 13))) {
				return false;
			}
			
			if (($length == 9) && !self::isValid9($number)) {
				return false;
			} else if ($length == 13) {
				if (!self::isValid9($fieldValue) || !self::isValid13($fieldValue)) {
					return false;
				}
			}
			return true;
		}
		
		protected static function isValid9($fieldValue)
		{
			$sum = 0;
			for ($n = 0; $n < 8; $n++)
				$sum += $fieldValue[$n] * self::$_weights[$n];
			$control = $sum % 11;
			
			if ($control == 10) {
				$sum = 0;
				for ($n = 0; $n < 8; $n++)
					$sum += $fieldValue[$n] * (self::$_weights[$n] + 2);
				$control = $sum % 11;
				
				if ($control == 10) {
					$control = 0;
				}
			}
			if ($control != $fieldValue[8])
				return false;
			
			return true;
		}
		
		protected static function isValid13($fieldValue)
		{
			$sum = 0;
			for ($n = 8; $n < 12; $n++)
				$sum += $fieldValue[$n] * self::$_weights[$n];
			$control = $sum % 11;
			
			if ($control == 10) {
				$sum = 0;
				for ($n = 8; $n < 12; $n++)
					$sum += $fieldValue[$n] * (self::$_weights[$n] + 2);
				$control = $sum % 11;
				
				if ($control == 10) {
					if ($fieldValue[12] != 0)
						return false;
				} else if ($control != $fieldValue[12])
					return false;
			} else if ($control != $fieldValue[12])
				return false;
			
			return true;
		}
	}
?>