<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCustomUrls {
		
		const PRECACHE_RECORDS = 100;
		
		protected static $_prepared = array();
		
		protected static $_cache = array();
		
		/**
		 * Will be set to TRUE when precaching has been done.
		 */
		protected static $_precached = false;
		
		/**
		 * Will be set to TRUE only if all the CustomUrls have been loaded.
		 * This may happen if precaching has been done and the requested items were more than the received.
		 */
		protected static $_allPrecached = false;
		
		/**
		 * Add items that will be used later in the script
		 * @param int $itemsType
		 * @param array $itemsIds
		 * @param int $languageId
		 */
		public static function prepareItems($itemsType, $itemsIds, $languageId = null)
		{
			if (is_null($languageId)) {
				$languageId = scMultiLanguage::getCurrentLanguageId();
			}
			
			$itemsIds = array_map('intval', $itemsIds);
			
			if (!isset(self::$_prepared[$languageId][$itemsType])) {
				self::$_prepared[$languageId][$itemsType] = array();
			}
			self::$_prepared[$languageId][$itemsType] = array_merge(self::$_prepared[$languageId][$itemsType], $itemsIds);
			self::$_prepared[$languageId][$itemsType] = array_unique(self::$_prepared[$languageId][$itemsType]);
		}
		
		/**
		 * Return the primary url for an item
		 * 
		 * @param int $itemType
		 * @param int $itemId
		 * @param int $languageId
		 */
		public static function getPrimaryUrl($itemType, $itemId, $languageId = null)
		{
			if (is_null($languageId)) {
				$languageId = scMultiLanguage::getCurrentLanguageId();
			}
			$languageId = (int)$languageId;
			$itemType = (int)$itemType;
			$itemId = (int)$itemId;
			
			if (!self::$_precached) {
				self::precache();
			}
			
			// Check if the item has already been cached.
			if (isset(self::$_cache[$languageId][$itemType][$itemId])) {
				return self::$_cache[$languageId][$itemType][$itemId];
			}
			
			// Check if all custom urls have been precached.
			// If they are and there is no record in the $_cache for the requested one
			// there is no record for it at all.
			if (self::$_allPrecached) {
				return false;
			}
			
			// Check if it is currently prepared.
			if (!isset(self::$_prepared[$languageId][$itemType]) || !in_array($itemId, self::$_prepared[$languageId][$itemType])) {
				// Add it to the prepare list if needed.
				self::$_prepared[$languageId][$itemType][] = intval($itemId);
			}
			
			$idsSql = implode(', ', self::$_prepared[$languageId][$itemType]);
			
			$sql = 'SELECT CustomUrlEntityID, CustomUrlText FROM CustomUrls' .
				' INNER JOIN CustomUrlsML USING(CustomUrlID)' .
				' WHERE CustomUrlIsCurrent AND LanguageID = ' . $languageId . ' AND CustomUrlEntityCode = ' . $itemType . ' AND CustomUrlEntityID IN (' . $idsSql . ')';
			$result = miStaticDBUtil::execSelect($sql);
			
			// Store FALSE values for all the items that were just fetched.
			foreach (self::$_prepared[$languageId][$itemType] as $preparedId) {
				self::$_cache[$languageId][$itemType][$preparedId] = false;
			}
			unset(self::$_prepared[$languageId][$itemType]);
			
			if (empty($result)) {
				// Nothing to cache.
				return false;
			}
			
			// Add the results to the cache.
			foreach ($result as $pair) {
				self::$_cache[$languageId][$itemType][$pair['CustomUrlEntityID']] = $pair['CustomUrlText'];
			}
			
			return self::$_cache[$languageId][$itemType][$itemId];
		}
		
		protected static function precache()
		{
			self::$_precached = true;
			
			$sql = 'SELECT CustomUrlEntityCode, LanguageID, CustomUrlEntityID, CustomUrlText' .
				' FROM CustomUrls INNER JOIN CustomUrlsML USING(CustomUrlID)' .
				' WHERE CustomUrlIsCurrent LIMIT ' . self::PRECACHE_RECORDS;
			$result = miStaticDBUtil::execSelect($sql);
			if (count($result) < self::PRECACHE_RECORDS) {
				self::$_allPrecached = true;
			}
			
			foreach ($result as $row) {
				self::$_cache[$row['LanguageID']][$row['CustomUrlEntityCode']][$row['CustomUrlEntityID']] = $row['CustomUrlText'];
			}
		}
	}
?>