<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scFriendlyUrl {
		
		const TYPE_CATEGORY = 1;
		const TYPE_PRODUCT = 2;
		const TYPE_PRODUCT_VARIANT = 3;
		const TYPE_RICH_TEXT_PAGE = 4;
		const TYPE_DYNAMIC_FORM = 5;
		const TYPE_MANUFACTURER = 6;
		const TYPE_GALLERY = 7;
		const TYPE_NEWS = 8;
		const TYPE_WISHLIST = 9;
		const TYPE_DYNAMIC_PAGE = 10;
		const TYPE_PRODUCT_COMPARISON = 11;
		
		const ROW_TYPE_MENU_ITEMS = 1;
		const ROW_TYPE_PRODUCTS = 2;
		const ROW_TYPE_CART_ITEMS = 3;
		const ROW_TYPE_CATEGORIES = 4;
		const ROW_TYPE_MANUFACTURERS = 5;
		const ROW_TYPE_NEWS = 6;
		const ROW_TYPE_FORMS = 7;
		
		const DYN_PAGE_MANUFACTURERS = 101;
		const DYN_PAGE_CART = 102;
		const DYN_PAGE_SEARCH = 103;
		const DYN_PAGE_LOGIN = 104;
		const DYN_PAGE_FORGOT_PASS = 105;
		const DYN_PAGE_REGISTER = 106;
		const DYN_PAGE_PROFILE = 107;
		const DYN_PAGE_ORDERS = 108;
		const DYN_PAGE_POINTS = 109;
		const DYN_PAGE_WISHLIST = 110;
		const DYN_PAGE_NEWS = 111;
		const DYN_PAGE_INDEX = 112;
		const DYN_PAGE_ADDRESSES = 113;
		const DYN_PAGE_ERROR = 114;
		const DYN_PAGE_PRODUCT_COMPARISON = 115;
		
		protected static $_dynamicPages = array(
			self::DYN_PAGE_MANUFACTURERS => 'manufacturers.php',
			
			self::DYN_PAGE_CART => 'cart.php',
			self::DYN_PAGE_SEARCH => 'search.php',
			self::DYN_PAGE_LOGIN => 'login.php',
			self::DYN_PAGE_FORGOT_PASS => 'forgot_pass.php',
			self::DYN_PAGE_REGISTER => 'register.php',
			self::DYN_PAGE_PROFILE => 'profile.php',
			self::DYN_PAGE_ORDERS => 'orders.php',
			self::DYN_PAGE_POINTS => 'points.php',
			self::DYN_PAGE_INDEX => 'index.php',
			self::DYN_PAGE_ADDRESSES => 'addresses.php',
			self::DYN_PAGE_ERROR => 'error.php',
			
			self::DYN_PAGE_WISHLIST => 'module.php?ModuleName=com.summercart.wishlist',
			self::DYN_PAGE_NEWS => 'module.php?ModuleName=com.summercart.news',
			self::DYN_PAGE_PRODUCT_COMPARISON => 'module.php?ModuleName=com.summercart.productcomparison',
		);
		
		/**
		 * Returns if friendly urls are enabled
		 * 
		 * @return boolean
		 */
		public static function hasFriendlyUrls()
		{
			return scSettings::getSettingValue(scSettings::SETTING_FRIENDLY_URLS) ? true : false;
		}
		
		public static function transliterateCyrillic($string)
		{
			// Official Streamlined System
			// http://en.wikipedia.org/wiki/Streamlined_System_for_the_Romanization_of_Bulgarian
			static $cyrTable = array(
				'А' => 'A', 'а' => 'a',
				'Б' => 'B', 'б' => 'b',
				'В' => 'V', 'в' => 'v',
				'Г' => 'G', 'г' => 'g',
				'Д' => 'D', 'д' => 'd',
				'Е' => 'E', 'е' => 'e',
				'Ж' => 'Zh', 'ж' => 'zh',
				'З' => 'Z', 'з' => 'z',
				'И' => 'i', 'и' => 'i',
				'Й' => 'Y', 'й' => 'y',
				'К' => 'K', 'к' => 'k',
				'Л' => 'L', 'л' => 'l',
				'М' => 'M', 'м' => 'm',
				'Н' => 'N', 'н' => 'n',
				'О' => 'O', 'о' => 'o',
				'П' => 'P', 'п' => 'p',
				'Р' => 'R', 'р' => 'r',
				'С' => 'S', 'с' => 's',
				'Т' => 'T', 'т' => 't',
				'У' => 'U', 'у' => 'u',
				'Ф' => 'F', 'ф' => 'f',
				'Х' => 'H', 'х' => 'h',
				'Ц' => 'Ts', 'ц' => 'ts',
				'Ч' => 'Ch', 'ч' => 'ch',
				'Ш' => 'Sh', 'ш' => 'sh',
				'Щ' => 'Sht', 'щ' => 'sht',
				'Ъ' => 'A', 'ъ' => 'a',
				'Ь' => 'Y', 'ь' => 'y',
				'Ю' => 'Yu', 'ю' => 'yu',
				'Я' => 'Ya', 'я' => 'ya',
				'ИЯ' => 'IA', 'ия' => 'ia',
			);
			return strtr($string, $cyrTable);
		}
		
		public static function transliterateTurkish($string)
		{
			static $table = array(
				'Ç' => 'C', 'ç' => 'c',
				'Ğ' => 'G', 'ğ' => 'g',
				'İ' => 'I', 'ı' => 'i',
				'Ö' => 'O', 'ö' => 'o',
				'Ş' => 'S', 'ş' => 's',
				'Ü' => 'U', 'ü' => 'u',
			);
			return strtr($string, $table);
		}
		
		public static function get($type, $id, $name = null, $languageCode = null)
		{
			if ($name === null) {
				$name = scUrlResolverFactory::getUrlResolverForPageType($type)->getName($id, $languageCode);
			}
			if (self::hasFriendlyUrls()) {
				return self::getFriendlyUrl($type, $id, $name, $languageCode);
			}
			return self::getUrl($type, $id, $languageCode);
		}
		
		public static function getFriendlyUrl($type, $id, $name, $languageCode = null)
		{
			if (is_null($languageCode)) {
				// If the language code is not passed suppose a url to the current one is requsted.
				$languageCode = scMultiLanguage::getCurrentLanguageCode();
			}
			
			$languageId = scMultiLanguage::singleton()->getLanguageIdByCode($languageCode);
			$customUrl = scCustomUrls::getPrimaryUrl($type, $id, $languageId);
			if ($customUrl !== false) {
				return scPaths::getStoreWebPath() . $customUrl;
			}
			
			if ($type == self::TYPE_DYNAMIC_PAGE) {
				return scPaths::getStoreWebPath() . self::getLinkablePageFriendlyUrl($id, $languageCode);
			}
			
			$pageUrlId = self::getPageUrlId($type, $id, $name, $languageCode);
			
			// Get the friendly url prefix
			$prefix = '';
			$prefixes = self::getFriendlyUrlPrefixes();
			if (isset($prefixes[$type])) {
				$prefix = $prefixes[$type];
			}
			
			if ($languageCode == scMultiLanguage::getDefaultLanguageCode()) {
				// If the requested language is the default one do not add it in the url.
				$languageCode = null;
			}
			
			return scPaths::getStoreWebPath() . self::getTempletizedUrl($prefix, $pageUrlId, $languageCode);
		}
		
		public static function getUrl($type, $id, $languageCode = null)
		{
			if (is_null($languageCode)) {
				// If the language code is not passed suppose a url to the current one is requsted.
				$languageCode = scMultiLanguage::getCurrentLanguageCode();
			}
			
			$id = intval($id);
			
			if ($type == self::TYPE_DYNAMIC_PAGE) {
				$pages = self::getLinkableDynamicPages();
				$prefix = $pages[$id];
				$id = '';
			} else {
				$prefixes = self::getUrlPrefixes();
				$prefix = $prefixes[$type];
			}
			
			$url = scPaths::getStoreWebPath() . $prefix . $id;
			if (($languageCode != scMultiLanguage::getDefaultLanguageCode())
				|| (scMultiLanguage::getCurrentLanguageCode() != scMultiLanguage::getDefaultLanguageCode()))
			{
				$url = scUrlUtil::addQueryParams($url, '&UILanguage=' . strtoupper($languageCode));
			}
			return $url;
		}
		
		public static function initFriendlyUrls(&$rows, $rowType)
		{
			$friendlyUrlsEnabled = self::hasFriendlyUrls();

			$getDefaultNames = false;
			
			// Display all links in the current language
//			if (scMultiLanguage::getCurrentLanguageId() != scMultiLanguage::getDefaultLanguageId())
//				$getDefaultNames = true;
			
			switch ($rowType) {
				case self::ROW_TYPE_CART_ITEMS:
					if (!$friendlyUrlsEnabled)
						self::initCartItemsUrls($rows);
					else
						self::initCartItemsFriendlyUrls($rows, $getDefaultNames);
					break;	
					
				case self::ROW_TYPE_CATEGORIES:
					if (!$friendlyUrlsEnabled)
						self::initCategoriesUrls($rows);
					else
						self::initCategoriesFriendlyUrls($rows, $getDefaultNames);
					break;
					
				case self::ROW_TYPE_MANUFACTURERS:
					if (!$friendlyUrlsEnabled)
						self::initManufacturersUrls($rows);
					else
						self::initManufacturersFriendlyUrls($rows, $getDefaultNames);
					break;
					
				case self::ROW_TYPE_FORMS:
					if (!$friendlyUrlsEnabled)
						self::initFormUrls($rows);
					else
						self::initFormFriendlyUrls($rows, $getDefaultNames);
					break;
					
				case self::ROW_TYPE_MENU_ITEMS:
					if (!$friendlyUrlsEnabled)
						self::initMenuItemsUrls($rows);
					else
						self::initMenuItemsFriendlyUrls($rows, $getDefaultNames);
					break;
					
				case self::ROW_TYPE_PRODUCTS:
					if (!$friendlyUrlsEnabled)
						self::initProductsUrls($rows);
					else
						self::initProductsFriendlyUrls($rows, $getDefaultNames);
					break;
					
				case self::ROW_TYPE_NEWS:
					if (!$friendlyUrlsEnabled)
						self::initNewsUrls($rows);
					else
						self::initNewsFriendlyUrls($rows, $getDefaultNames);
					break;
			}
		}
		
		public static function getFriendlyUrlPrefixes()
		{
			static $prefixes = array(
				self::TYPE_CATEGORY => 'category',
				self::TYPE_PRODUCT => 'product',
				self::TYPE_PRODUCT_VARIANT => 'productv',
				self::TYPE_RICH_TEXT_PAGE => 'page',
				self::TYPE_DYNAMIC_FORM => 'form',
				self::TYPE_MANUFACTURER => 'manufacturer',
				self::TYPE_GALLERY => 'gallery',
				self::TYPE_NEWS => 'news',
			);
			return $prefixes;
		}
		
		public static function getUrlPrefixes()
		{
			static $prefixes = array(
				self::TYPE_CATEGORY => 'categories.php?CategoryID=',
				self::TYPE_PRODUCT => 'product.php?ProductID=',
				self::TYPE_PRODUCT_VARIANT => 'product.php?ProductVariantID=',
				self::TYPE_RICH_TEXT_PAGE => 'page.php?PageID=',
				self::TYPE_DYNAMIC_FORM => 'form.php?FormID=',
				self::TYPE_MANUFACTURER => 'manufacturers.php?ManufacturerID=',
				self::TYPE_GALLERY => 'gallery.php?GalleryID=',
				self::TYPE_NEWS => 'module.php?ModuleName=com.summercart.news&NewsID=',
			);
			return $prefixes;
		}
		
		public static function getLinkableDynamicPages()
		{
			return self::$_dynamicPages;
		}
		
		public static function getLinkablePageFriendlyUrl($pageType, $languageCode)
		{
			if (!isset(self::$_dynamicPages[$pageType])) {
				throw new miException(sprintf(_i('Page type "%s" is not a linkable one.'), $pageType));
			}
			
			switch ($pageType) {
				case self::DYN_PAGE_MANUFACTURERS:
					$prefix = 'manufacturers';
					break;
				case self::DYN_PAGE_NEWS:
					$prefix = 'news';
					break;
				case self::DYN_PAGE_WISHLIST:
					$prefix = 'wishlist';
					break;
				case self::DYN_PAGE_PRODUCT_COMPARISON:
					$prefix = 'product_comparison';
					break;
				default:
					$prefix = basename(self::$_dynamicPages[$pageType], '.php');
					break;
			}
			
			if ($languageCode == scMultiLanguage::getDefaultLanguageCode()) {
				$languageCode = null;
			}
			
			return self::getTempletizedUrl($prefix, null, $languageCode);
		}
		
		protected static function getTempletizedUrl($name, $pageUrlId, $languageCode)
		{
			$urlType = scSettings::getSettingValue(scSettings::SETTING_SEO_URLS_TYPE);
			switch ($urlType) {
				case scSeoUrlTypes::LANGUAGE_NAME_HTML:
					$url = (!is_null($languageCode) ? strtolower($languageCode) . '/' : '');
					$url .= $name;
					$url .= (!empty($pageUrlId) ? '/' . $pageUrlId : '');
					$url .= '.html';
					return $url;
				
				case scSeoUrlTypes::NAME_HTML_LANGUAGE:
					$url = $name;
					$url .= (!empty($pageUrlId) ? '/' . $pageUrlId : '');
					$url .= '.html';
					$url .= (!is_null($languageCode) ? '.' . strtolower($languageCode) : '');
					return $url;
				
				case scSeoUrlTypes::NAME_LANGUAGE_HTML:
					$url = $name;
					$url .= (!empty($pageUrlId) ? '/' . $pageUrlId : '');
					$url .= (!is_null($languageCode) ? '.' . strtolower($languageCode) : '');
					$url .= '.html';
					return $url;
			}
			throw new miException(sprintf(_i('Unknown URL type'), $urlType));
		}
		
		/**
		 * @param int $type
		 * @param int $id
		 * @param string $name
		 * @return string
		 */
		protected static function getPageUrlId($type, $id, $name, $languageCode = null)
		{
			if (is_null($languageCode)) {
				// If the language code is not passed suppose a url to the current one is requsted.
				$languageCode = scMultiLanguage::getCurrentLanguageCode();
			}
			
			if ($languageCode == 'BG')
				$name = self::transliterateCyrillic($name);
			else if ($languageCode == 'TR')
				$name = self::transliterateTurkish($name);
			
			// Sanity check and cleanup
			$name = preg_replace('#[^a-z0-9\-]#i', '-', strtolower($name));
			$name = preg_replace('#(-{2,})#', '-', $name);
			$name = trim($name, '-');
			$name = urlencode($name);		// If something is forgotten
			
			$id = intval($id);
			
			return $id . '/' . $name;
		}
		
		// --- Normal URLs ---------------------------- //
		protected static function initCartItemsUrls(&$rows)
		{
			self::initProductsUrls($rows);
		}
		
		protected static function initCategoriesUrls(&$rows)
		{
			foreach ($rows as &$row)
				$row['CategoryUrl'] = self::getUrl(self::TYPE_CATEGORY, $row['CategoryID']);
		}
		
		protected static function initManufacturersUrls(&$rows)
		{
			foreach ($rows as &$row)
				$row['ManufacturerUrl'] = self::getUrl(self::TYPE_MANUFACTURER, $row['ManufacturerID']);
		}
		
		protected static function initNewsUrls(&$rows)
		{
			foreach ($rows as &$row)
				$row['NewsUrl'] = self::getUrl(self::TYPE_NEWS, $row['NewsID']);
		}
		
		protected static function initFormUrls(&$rows)
		{
			foreach ($rows as &$row)
				$row['FormUrl'] = self::getUrl(self::TYPE_DYNAMIC_FORM, $row['CMSFormID']);
		}
		
		protected static function initMenuItemsUrls(&$rows)
		{
			// Get menu item ids
			foreach ($rows as &$row) {
				switch ($row['CMSMenuItemLinkType']) {
					case scUserMenuItem::LINK_TYPE_URL:
						$row['MenuUrl'] = $row['CMSMenuItemURL'];
						break;
					
					case scUserMenuItem::LINK_TYPE_RICH_TEXT_PAGE:
						$row['MenuUrl'] = self::getUrl(self::TYPE_RICH_TEXT_PAGE, $row['CMSRichTextPageID']);
						break;
					
					case scUserMenuItem::LINK_TYPE_FORM:
						$row['MenuUrl'] = self::getUrl(self::TYPE_DYNAMIC_FORM, $row['CMSFormID']);
						break;
					
					case scUserMenuItem::LINK_TYPE_GALLERY:
						$row['MenuUrl'] = self::getUrl(self::TYPE_GALLERY, $row['GalleryID']);
						break;
					
					case scUserMenuItem::LINK_TYPE_DYN_PAGE:
						$codes = scDynamicPages::getLinkablePageCodes();
						$row['MenuUrl'] = self::getUrl(self::TYPE_DYNAMIC_PAGE, $codes[$row['CMSDynamicPageID']]);
						break;
				}
			}
		}
		
		protected static function initProductsUrls(&$rows)
		{
			foreach ($rows as &$row) {
				if (isset($row['ProductVariantID']) and ($row['ProductVariantID'] != 0))
					$row['ProductUrl'] = self::getUrl(self::TYPE_PRODUCT_VARIANT, $row['ProductVariantID']);
				else
					$row['ProductUrl'] = self::getUrl(self::TYPE_PRODUCT, $row['ProductID']);
			}
		}
		
		// --- Friendly URLs ---------------------- //
		protected static function initCategoriesFriendlyUrls(&$rows, $getDefaultNames)
		{
			if ($getDefaultNames && (count($rows) > 0)) {
				// Extract the names from the database in the default language
				$categories = array();
				foreach ($rows as $category)
					$categories[] = (int)$category['CategoryID'];
					
				$categories = '"' . implode('", "', $categories) . '"';
				$categories = miUtil::getDBArray(
					'SELECT CategoryID, CategoryName FROM CategoriesML ' .
					'WHERE CategoryID IN (' . $categories . ') AND LanguageID = ' . scMultiLanguage::getDefaultLanguageId(),
					'CategoryID', 'CategoryName'
				);
			}
			
			// Prepare custom URLs
			$categoryIds = miUtil::getKeyValueArray($rows, 'CategoryID', 'CategoryID');
			scCustomUrls::prepareItems(scFriendlyUrl::TYPE_CATEGORY, $categoryIds);
			
			foreach ($rows as &$row) {
				if ($getDefaultNames)
					$categoryName = $categories[$row['CategoryID']];
				else
					$categoryName = $row['CategoryName'];
					
				$row['CategoryUrl'] = self::getFriendlyUrl(self::TYPE_CATEGORY, $row['CategoryID'], $categoryName);
			}
		}
		
		protected static function initFormFriendlyUrls(&$rows, $getDefaultNames)
		{
			if ($getDefaultNames && (count($rows) > 0)) {
				$forms = array();
				foreach ($rows as $form)
					$forms[] = (int)$form['CMSFormID'];
					
				$forms = '"' . implode('", "', $forms) . '"';
				$forms = miUtil::getDBArray(
					'SELECT CMSFormID, CMSFormTitle FROM CMSFormsML ' .
					'WHERE CMSFormID IN(' . $forms . ') AND LanguageID = ' . scMultiLanguage::getDefaultLanguageId(),
					'CMSFormID', 'CMSFormTitle'
				);
			}
			
			// Prepare custom URLs
			$formIds = miUtil::getKeyValueArray($rows, 'CMSFormID', 'CMSFormID');
			scCustomUrls::prepareItems(scFriendlyUrl::TYPE_DYNAMIC_FORM, $formIds);
			
			foreach ($rows as &$row) {
				if ($getDefaultNames)
					$formName = $forms[$row['CMSFormID']];
				else
					$formName = $row['CMSFormTitle'];
				
				$row['FormUrl'] = self::getFriendlyUrl(self::TYPE_DYNAMIC_FORM, $row['CMSFormID'], $formName);
			}
		}
		
		protected static function initMenuItemsFriendlyUrls(&$rows, $getDefaultNames)
		{
			if ($getDefaultNames && (count($rows) > 0)) {
				$items = array();
				foreach ($rows as $item)
					$items[] = (int)$item['CMSMenuItemID'];
					
				$items = '"' . implode('", "', $items) . '"';
				$items = miUtil::getDBArray(
					'SELECT CMSMenuItemID, CMSMenuItemName FROM CMSMenuItemsML ' .
					'WHERE CMSMenuItemID IN(' . $items . ') AND LanguageID = ' . scMultiLanguage::getDefaultLanguageId(),
					'CMSMenuItemID', 'CMSMenuItemName'
				);
			}
			
			
			// Prepare custom URLs
			$dynamicPagesToCodes = scDynamicPages::getLinkablePageCodes();
			self::prepareMenuItemsCustomUrls($rows);
			
			foreach ($rows as &$row) {
				if ($getDefaultNames)
					$itemName = $items[$row['CMSMenuItemID']];
				else
					$itemName = $row['CMSMenuItemName'];
				
				$row['MenuUrl'] = '';
				try {
					switch ($row['CMSMenuItemLinkType']) {
						case scUserMenuItem::LINK_TYPE_URL:
							$linkName = $row['CMSMenuItemURL'];
							$parsed = scUrlUtil::parse($row['CMSMenuItemURL']);
							if (isset($parsed[scUrlUtil::SCHEME])) {
								$row['MenuUrl'] = $row['CMSMenuItemURL'];
							} else {
								$row['MenuUrl'] = scUrlUtil::convertUrl($row['CMSMenuItemURL'], scUrlUtil::URL_TYPE_DOCROOT);
							}
							break;
						
						case scUserMenuItem::LINK_TYPE_RICH_TEXT_PAGE:
							$row['MenuUrl'] = scFriendlyUrl::getFriendlyUrl(scFriendlyUrl::TYPE_RICH_TEXT_PAGE, $row['CMSRichTextPageID'], $itemName);
							break;
						
						case scUserMenuItem::LINK_TYPE_FORM:
							$row['MenuUrl'] = scFriendlyUrl::getFriendlyUrl(scFriendlyUrl::TYPE_DYNAMIC_FORM, $row['CMSFormID'], $itemName);
							break;
						
						case scUserMenuItem::LINK_TYPE_GALLERY:
							$row['MenuUrl'] = scFriendlyUrl::getFriendlyUrl(scFriendlyUrl::TYPE_GALLERY, $row['GalleryID'], $itemName);
							break;
						
						case scUserMenuItem::LINK_TYPE_DYN_PAGE:
							$dynamicPageCode = null;
							if (isset($dynamicPagesToCodes[$row['CMSDynamicPageID']])) {
								$dynamicPageCode = $dynamicPagesToCodes[$row['CMSDynamicPageID']];
							}
							$row['MenuUrl'] = scFriendlyUrl::getFriendlyUrl(scFriendlyUrl::TYPE_DYNAMIC_PAGE, $dynamicPageCode, $itemName);
							break;					
					}
				} catch (miException $e) {
					$msg = $e->getMessage() . ' (Menu: ' . $row['CMSMenuName'] . ', Item ID: ' . $row['CMSMenuItemID'] . ')';
					scLog::singleton()->log(scLog::SEVERITY_ERROR, $msg, scLog::CATEGORY_NONE);
					continue;
				}
			}
		}
		
		protected static function prepareMenuItemsCustomUrls(&$rows)
		{
			$richTextPageIds = array();
			$formIds = array();
			$galleryIds = array();
			foreach ($rows as &$row) {
				switch ($row['CMSMenuItemLinkType']) {
					case scUserMenuItem::LINK_TYPE_RICH_TEXT_PAGE:
						$richTextPageIds[] = $row['CMSRichTextPageID'];
						break;
					case scUserMenuItem::LINK_TYPE_FORM:
						$formIds[] = $row['CMSFormID'];
						break;
					case scUserMenuItem::LINK_TYPE_GALLERY:
						$galleryIds[] = $row['GalleryID'];
						break;
				}
			}
			if (!empty($richTextPageIds))
				scCustomUrls::prepareItems(scFriendlyUrl::TYPE_RICH_TEXT_PAGE, $richTextPageIds);
			if (!empty($formIds))
				scCustomUrls::prepareItems(scFriendlyUrl::TYPE_RICH_TEXT_PAGE, $formIds);
			if (!empty($galleryIds))
				scCustomUrls::prepareItems(scFriendlyUrl::TYPE_RICH_TEXT_PAGE, $galleryIds);
		}
		
		protected static function initManufacturersFriendlyUrls(&$rows, $getDefaultNames)
		{
			if ($getDefaultNames && (count($rows) > 0)) {
				$manufacturers = array();
				foreach ($rows as $manufacturer)
					$manufacturers[] = (int)$manufacturer['ManufacturerID'];
					
				$manufacturers = '"' . implode('", "', $manufacturers) . '"';
				$manufacturers = miUtil::getDBArray(
					'SELECT ManufacturerID, ManufacturerName FROM ManufacturersML ' .
					'WHERE ManufacturerID IN(' . $manufacturers . ') AND LanguageID = ' . scMultiLanguage::getDefaultLanguageId(),
					'ManufacturerID', 'ManufacturerName'
				);
			}
			
			// Prepare custom URLs
			$manufacturerIds = miUtil::getKeyValueArray($rows, 'ManufacturerID', 'ManufacturerID');
			scCustomUrls::prepareItems(scFriendlyUrl::TYPE_MANUFACTURER, $manufacturerIds);
			
			foreach ($rows as &$row) {
				if ($getDefaultNames)
					$manufacturerName = $manufacturers[$row['ManufacturerID']];
				else
					$manufacturerName = $row['ManufacturerName'];
					
				$row['ManufacturerUrl'] = self::getFriendlyUrl(self::TYPE_MANUFACTURER, $row['ManufacturerID'], $manufacturerName);
			}
		}
		
		protected static function initNewsFriendlyUrls(&$rows, $getDefaultNames)
		{
			if ($getDefaultNames && (count($rows) > 0)) {
				$news = array();
				foreach ($rows as $article)
					$news[] = (int)$article['NewsID'];
					
				$news = '"' . implode('", "', $news) . '"';
				$news = miUtil::getDBArray(
					'SELECT NewsID, NewsTitle FROM NewsML ' .
					'WHERE NewsID IN(' . $news . ') AND LanguageID = ' . scMultiLanguage::getDefaultLanguageId(),
					'NewsID', 'NewsTitle'
				);
			}
			
			// Prepare custom URLs
			$newsIds = miUtil::getKeyValueArray($rows, 'NewsID', 'NewsID');
			scCustomUrls::prepareItems(scFriendlyUrl::TYPE_NEWS, $newsIds);
			
			foreach ($rows as &$row) {
				if ($getDefaultNames)
					$name = $news[$row['NewsID']];
				else
					$name = $row['NewsTitle'];
					
				$row['NewsUrl'] = self::getFriendlyUrl(self::TYPE_NEWS, $row['NewsID'], $name);
			}
		}
		
		protected static function initProductsFriendlyUrls(&$rows, $getDefaultNames)
		{
			if ($getDefaultNames && (count($rows) > 0)) {
				$products = array();
				foreach ($rows as $product)
					$products[] = (int)$product['ProductID'];
					
				$products = '"' . implode('", "', $products) . '"';
				$products = miUtil::getDBArray(
					'SELECT ProductID, ProductName FROM ProductsML ' .
					'WHERE ProductID IN(' . $products . ') AND LanguageID = ' . scMultiLanguage::getDefaultLanguageId(),
					'ProductID', 'ProductName'
				);
			}
			
			// Prepare custom URLs
			$productIds = array();
			$productVariantIds = array();
			foreach ($rows as &$row) {
				if (isset($row['ProductVariantID']) and ($row['ProductVariantID'] != 0)) {
					$productVariantIds[] = $row['ProductVariantID'];
				} else {
					$productIds[] = $row['ProductID'];
				}
			}
			scCustomUrls::prepareItems(scFriendlyUrl::TYPE_PRODUCT, $productIds);
			if (!empty($productVariantIds)) {
				scCustomUrls::prepareItems(scFriendlyUrl::TYPE_PRODUCT_VARIANT, $productVariantIds);
			}
			
			foreach ($rows as &$row) {
				if ($getDefaultNames)
					$productName = $products[$row['ProductID']];
				else
					$productName = $row['ProductName'];
					
				if (isset($row['ProductVariantID']) and ($row['ProductVariantID'] != 0))
					$row['ProductUrl'] = self::getFriendlyUrl(self::TYPE_PRODUCT_VARIANT, $row['ProductVariantID'], $productName);
				else
					$row['ProductUrl'] = self::getFriendlyUrl(self::TYPE_PRODUCT, $row['ProductID'], $productName);
			}
		}
		
		protected static function initCartItemsFriendlyUrls(&$rows, $getDefaultNames)
		{
			if (!$getDefaultNames) {
				foreach ($rows as &$row)
					$row['ProductName'] = $row['CartItemName'];
			}
			
			self::initProductsFriendlyUrls($rows, $getDefaultNames);
		}
	}
?>