<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * UTF8 handling
	 * Has replacement functions for mbstring extension
	 */

	if (extension_loaded('mbstring')) {
		mb_internal_encoding('UTF-8');
		
		/**
		 * Wrapper around the mbstring extension
		 */
		class scUtf8 {
			public static function strlen($str)
			{
				return mb_strlen($str);
			}
			
			public static function substr($str, $start, $length)
			{
				return mb_substr($str, $start, $length);
			}
			
			public static function truncate($str, $start, $length, $trimmarker = '...')
			{
				if (mb_strlen($str) <= $length)
					return $str;
				return rtrim(mb_substr($str, $start, $length)) . $trimmarker;
			}
			
			public static function isAsciiPrint($string)
			{
				return !preg_match('#([\x00-\x1F\x80-\xFF])#', $string);
			}
			
			public static function strtoupper($string)
			{
				return mb_strtoupper($string);
			}
			
			public static function strtolower($string)
			{
				return mb_strtolower($string);
			}
			
			public static function ucfirst($string)
			{
				return self::strtoupper(self::substr($string, 0, 1)) . self::strtolower(self::substr($string, 1, self::strlen($string)));
			}
			
			public static function ucwords($string)
			{
				$parts = explode(' ', $string);
				if (empty($parts))
					return '';
				
				foreach ($parts as $k => &$part) {
					if ($part === '') {
						unset($parts[$k]); // remove double spaces
						continue;
					}
					$part = self::ucfirst($part);
				}
				return implode(' ', $parts);
			}
		}
		
	} else {
		
		/**
		 * UTF8 mbstring extension replacement functions
		 */
		class scUtf8 {
			public static function strlen($str)
			{
				return strlen(utf8_decode($str));
			}
			
			public static function substr($str, $start, $length)
			{
				if ($start == 0) {
					if (!is_int($length))
						return $str;
					
					preg_match('/^(([\x00-\x7F]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF][\x80-\xBF]|[\xF0-\xF7][\x80-\xBF][\x80-\xBF][\x80-\xBF]){0,' . $length . '})/s', $str, $arr);
					return $arr[1];
				}
				
				if (!is_int($length)) {
					preg_match(
						'/^(([\x00-\x7F]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF][\x80-\xBF]|[\xF0-\xF7][\x80-\xBF][\x80-\xBF][\x80-\xBF]){' . $start . '})' .
						'(.+)/s', $str, $arr);
				} else {
					preg_match(
						'/^(([\x00-\x7F]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF][\x80-\xBF]|[\xF0-\xF7][\x80-\xBF][\x80-\xBF][\x80-\xBF]){' . $start . '})' .
						'(([\x00-\x7F]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF][\x80-\xBF]|[\xF0-\xF7][\x80-\xBF][\x80-\xBF][\x80-\xBF]){0,' . $length . '})/s', $str, $arr);
				}
				return $arr[3];
			}
			
			public static function truncate($str, $start, $length, $trimmarker = '...')
			{
				if (self::strlen($str) <= $length)
					return $str;
				return rtrim(self::substr($str, $start, $length)) . $trimmarker;
			}
			
			public static function isAsciiPrint($string)
			{
				return !preg_match('#([\x00-\x1F\x80-\xFF])#', $string);
			}
			
			public static function strtoupper($string)
			{
				return strtr($string, scUtfCaseMappings::getLowerToUpper());
			}
			
			public static function strtolower($string)
			{
				return strtr($string, scUtfCaseMappings::getUpperToLower());
			}
			
			public static function ucfirst($string)
			{
				return self::strtoupper(self::substr($string, 0, 1)) . self::strtolower(self::substr($string, 1, self::strlen($string)));
			}
			
			public static function ucwords($string)
			{
				$parts = explode(' ', $string);
				if (empty($parts))
					return '';
				
				foreach ($parts as $k => &$part) {
					if ($part === '') {
						unset($parts[$k]); // remove double spaces
						continue;
					}
					$part = self::ucfirst($part);
				}
				return implode(' ', $parts);
			}
		}
	}
?>