<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCustomerAreaPager {
		const PRODUCTS_PER_PAGE = 10;
		
		public static function calculatePages($pageParam, $numItems, $itemsPerPage = 0)
		{
			if ($itemsPerPage == 0) {
				$itemsPerPage = self::getProductsPerPage();
			}
			$numPages = intval(($numItems-1) / $itemsPerPage) + 1;
			$page = (int)miGetParamDefault($pageParam, 1);
			if ($page < 1)
				$page = 1;
			elseif ($page > $numPages)
				$page = $numPages;
			
			return array($page, $numPages);
		}
		
		public static function calculateAltViewPage($page, $view, $altView)
		{
			$numItems = scCustomerAreaPager::getProductsPerPage($view);
			$altViewItems = scCustomerAreaPager::getProductsPerPage($altView);
			
			$firstProductNum = $numItems * ($page - 1) + 1;
			return ceil($firstProductNum / $altViewItems);
		}
		
		public static function getPaging($page, $numPages, $url)
		{
			$url = scUrlUtil::convertUrl($url, scUrlUtil::URL_TYPE_DOCROOT);
			
			$startPage = $page - 5;
			if ($startPage < 1)
				$startPage = 1;
			
			if ($page > 1) {
				$html = '<a class=""prev-page" href="' . miI18N::htmlEscape(scUrlUtil::addQueryParams($url, self::getStateParamsForPage($page-1))) . '">' . _t('Previous') . '</a>';
			} else
				$html = '<span class="prev-page">' . _t('Previous') . '</span>';
			for ($i=0; $i < 10; $i++) {
				if ($startPage == $page)
					$html .= '<span class="selected-page">' . $startPage . '</span>';
				else
					$html .= ' <a href="' . miI18N::htmlEscape(scUrlUtil::addQueryParams($url, self::getStateParamsForPage($startPage))) . '">' . $startPage . '</a>';
				$startPage++;
				if ($startPage > $numPages)
					break;
			}
			if ($page < $numPages)
				$html .= '<a class="next-page" href="' . miI18N::htmlEscape(scUrlUtil::addQueryParams($url, self::getStateParamsForPage($page+1))) . '">' . _t('Next') . '</a>';
			else
				$html .= '<span class="next-page">' . _t('Next') . '</span>';
			return $html;
		}
		
		/**
		 * @return string
		 */
		public static function getLimitString($pageParam, $productsPerPage = 0)
		{
			if ($productsPerPage <= 0) {
				$productsPerPage = self::getProductsPerPage();
			}
			$offset = self::getLimitOffset($pageParam, $productsPerPage);
			return ' LIMIT ' . $offset . ',' . $productsPerPage;
		}
		
		/**
		 * @return int
		 */
		public static function getLimitOffset($pageParam, $productsPerPage)
		{
			$page = max(1, (int)miGetParamDefault($pageParam, 1));
			return (($page - 1) * $productsPerPage);
		}
		
		/**
		 * @return int
		 */
		public static function getProductsPerPage($view = null)
		{
			if (is_null($view)) {
				$view = scProductService::getViewType();
			}
			
			if ($view == scProductService::VIEW_GRID) {
				$cols = scSettings::getSettingValue(scSettings::SETTING_GRID_VIEW_COLUMNS);
				$rows = scSettings::getSettingValue(scSettings::SETTING_GRID_VIEW_ROWS);
				$productsPerPage = intval($cols * $rows);
			} else {
				$productsPerPage = (int)scSettings::getSettingValue(scSettings::SETTING_LIST_VIEW_PRODUCTS_PER_PAGE);
			}
			
			if ($productsPerPage <= 0) {
				return self::PRODUCTS_PER_PAGE;
			}
			
			return $productsPerPage;
		}
		
		protected static function getStateParamsForPage($page)
		{
			$stateObj = miAppFactory::singleton()->getStateObj();
			$stateObj->setArray(array('page' => $page));
			$stateUrlParams = $stateObj->getStateUrlParams();
			
			if (empty($stateUrlParams)) {
				return '';
			}
			return '&' . $stateUrlParams;
		}
	}
?>