<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Customers List
	 */
	class scCustomersTable extends scTable {
		
		protected function addUiElements(&$rows)
		{
			parent::addUiElements($rows);
			$customerStatuses = scCustomerStatuses::singleton()->getStatuses();
			$customerBillingAddresses = $this->getCustomerBillingAddresses($rows);
			$countryNames = $this->getCountries($customerBillingAddresses);
			
			$totalsByCustomer = $this->getTotalsByCustomer(miUtil::getKeyValueArray($rows, 'CustomerID', 'CustomerID'));
			
			foreach ($rows as &$row) {
				$customerId = $row['CustomerID'];
				
				if (isset($customerStatuses[$row['CustomerStatus']])) {
					$row['CUSTOMERSTATUS'] = $customerStatuses[$row['CustomerStatus']];
					$row['CUSTOMERSTATUSCLASS'] = (($row['CustomerStatus'] == scCustomerStatuses::CUSTOMER_STATUS_ACTIVE) ? 'active' : 'disabled');
				} else {
					$row['CUSTOMERSTATUS'] = '';
					$row['CUSTOMERSTATUSCLASS'] = 'disabled';
				}
				
				$row['FMT_CustomerBillingCountry'] = '';
				$row['FMT_CustomerBillingAddress'] = _t('None');
				if (isset($customerBillingAddresses[$customerId])) {
					$billingAddress = $customerBillingAddresses[$customerId];
					$row['FMT_CustomerBillingCountry'] = isset($countryNames[$billingAddress['CountryID']]) ? $countryNames[$billingAddress['CountryID']] : '';
					$row['FMT_CustomerBillingAddress'] = $this->getFormattedBillingAddress($billingAddress);
				}
				$row['HTML_MoreAddresses'] = '';
				if ($billingAddress['CustomerAddressesCount'] > 1) {
					$additionalAddresses = $billingAddress['CustomerAddressesCount'] - 1;
					$additionalAddressesCaption = ($additionalAddresses == 1) ? _t('%d more address') : _t('%d more addresses');
					$row['HTML_MoreAddresses'] = '<br /><span class="customer-more-addresses">' . sprintf($additionalAddressesCaption, $additionalAddresses) . '</span>';
				}
				
				$amount = 0;
				if (isset($totalsByCustomer[$row['CustomerID']])) {
					$amount = $totalsByCustomer[$row['CustomerID']];
				}
				$row['FMT_OrderTotal'] = scPrice::factory(scMultiCurrencyAmount::factoryBaseCurrency($amount))->getDisplayPrice();
				
				if ($row['OrdersCount'] == 1) {
					$row['OrdersCountWord'] = _t('Order');
				} else {
					$row['OrdersCountWord'] = _t('Orders');
				}
			}
		}
		
		/**
		 * Gets the billing addresses of all customers.
		 * 
		 * Adds the count of all addresses the customer has.
		 * @param array $rows
		 * @return array
		 */
		protected function getCustomerBillingAddresses(&$rows)
		{
			$customerIds = miUtil::getKeyValueArray($rows, 'CustomerID', 'CustomerID');
			
			// Get all customers
			$recordset = new miSqlRecordset('Customers');
			// Get the number of all the addresses of each customer
			$recordset->addJoinCondition('LEFT', 'CustomerAddresses', 'USING(CustomerID)');
			// Get the customer's billing address
			$recordset->addJoinCondition('LEFT', 'CustomerAddresses',
				'ON CustomerAddresses.CustomerID = CustomerBillingAddresses.CustomerID' .
				' AND CustomerBillingAddresses.CustomerAddressIsDefaultBilling = 1',
				'CustomerBillingAddresses');
			$recordset->setSelectFields(array(
				'CustomerBillingAddresses.*',
				'COUNT(DISTINCT(CustomerAddresses.CustomerAddressID)) AS CustomerAddressesCount',
				'Customers.CustomerID',
			));
			$recordset->setGroupBy('Customers.CustomerID');
			$recordset->addFilter(new miSqlFilterIn('Customers.CustomerID', $customerIds));
			
			$addresses = $recordset->getAllRecords();
			$addresses = miUtil::getArrayWithKeys($addresses, 'CustomerID');
			
			return $addresses;
		}
		
		protected function getCountries(&$addresses)
		{
			$countryIds = miUtil::getKeyValueArray($addresses, 'CountryID', 'CountryID');
			if (count($countryIds) == 0)
				return array();
			
			return scCountry::getCountryNames(array_unique(array_filter($countryIds)));
		}
		
		protected function getFormattedBillingAddress($address)
		{
			$formatted = array();
			if (!empty($address['CustomerAddressCity'])) {
				$formatted[] = $address['CustomerAddressCity'];
			}
			if (!empty($address['CustomerAddressZip'])) {
				$formatted[] = $address['CustomerAddressZip'];
			}
			if (!empty($address['CustomerAddressLine1'])) {
				$formatted[] = $address['CustomerAddressLine1'];
			}
			return implode(', ', $formatted);
		}
		
		/**
		 * Get the sums of OrderTotals for the specified $customerIds and groups them by CustomerID.
		 */
		protected function getTotalsByCustomer($customerIds)
		{
			$customerIds = array_map('intval', $customerIds);
			$customerIds = implode(', ', $customerIds);
			$baseCurrencyCode = scCurrencyUtil::getBaseCurrency()->getCurrencyCode();
			$showUnfinishedOrders = scSettings::getSettingValue(scSettings::SETTING_SHOW_UNFINISHED_ORDERS);
			$hideUnfinishedOrdersSql = $showUnfinishedOrders ? '' : ' AND OrderStatus != ' . scOrderStatus::ORDER_STATUS_UNFINISHED;
			
			// Records for which the OrderTotal is stored in the current base currency
			$currentBaseCurrencySql = 'SELECT CustomerID, OrderCurrencyCode, SUM(OrderTotal) AS OrderTotal' .
				' FROM Orders' .
				' WHERE OrderCurrencyCode = "' . sql_escape_string($baseCurrencyCode) . '"' .
				' AND CustomerID IN (' . $customerIds . ')' .
				$hideUnfinishedOrdersSql .
				' GROUP BY CustomerID, OrderCurrencyCode';
			
			// Records for which the OrderCustomerCurrencyTotal is in the current base currency
			$previousNonBaseSql = 'SELECT CustomerID, OrderCustomerCurrencyCode, SUM(OrderCustomerCurrencyTotal)' .
				' FROM Orders' .
				' WHERE OrderCurrencyCode <> "' . sql_escape_string($baseCurrencyCode) . '"' .
				' AND OrderCustomerCurrencyCode = "' . sql_escape_string($baseCurrencyCode) . '"' .
				' AND CustomerID IN (' . $customerIds . ')' .
				$hideUnfinishedOrdersSql .
				' GROUP BY CustomerID, OrderCustomerCurrencyCode';
			
			// $currentBaseCurrencySql and $previousNonBaseSql both return amounts IN THE CURRENT BASE CURRENCY
			$baseCurrencySql = 
				$currentBaseCurrencySql .
				' UNION ALL ' .
				$previousNonBaseSql;
			
			$nonBaseCurrencySql = 'SELECT CustomerID, OrderCurrencyCode, SUM(OrderTotal)' .
				' FROM Orders' .
				' WHERE OrderCurrencyCode <> "' . sql_escape_string($baseCurrencyCode) . '"' .
				' AND OrderCustomerCurrencyCode <> "' . sql_escape_string($baseCurrencyCode) . '"' .
				' AND CustomerID IN (' . $customerIds . ')' .
				$hideUnfinishedOrdersSql .
				' GROUP BY CustomerID, OrderCurrencyCode';
			
			$sql = $baseCurrencySql . ' UNION ALL ' . $nonBaseCurrencySql;
			
			$totals = miStaticDBUtil::execSelect($sql);
			$totalsByCustomer = array();
			foreach ($totals as $total) {
				if (!isset($totalsByCustomer[$total['CustomerID']])) {
					$totalsByCustomer[$total['CustomerID']] = 0;
				}
				if ($total['OrderTotal'] == 0) {
					continue;
				}
				$amount = $total['OrderTotal'];
				if ($total['OrderCurrencyCode'] != $baseCurrencyCode) {
					$amount = scCurrencyUtil::convert($amount, $total['OrderCurrencyCode'], new scCurrency($baseCurrencyCode));
				}
				$totalsByCustomer[$total['CustomerID']] += $amount;
			}
			return $totalsByCustomer;
		}
		
		/**
		 * @return scMessage
		 */
		protected function getMessageObj()
		{
			if ($this->_messageObj == null)
				$this->_messageObj = new scMessage;
			return $this->_messageObj;
		}
	}
?>