<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scMenuItemsTable extends scTable {
		public function addUiElements(&$rows)
		{
			$cmsFormNames = $this->getCMSFormNames($rows);
			
			$linkTypes = scUserMenuItem::getLinkTypes();
			$linkablePageNames = scDynamicPages::getLinkablePageNames();
			$linkablePageCodes = scDynamicPages::getLinkablePageCodes();
			foreach ($rows as &$row) {
				$linkName = '';
				$linkUrl = '';
				try {
					switch ($row['CMSMenuItemLinkType']) {
						case scUserMenuItem::LINK_TYPE_FORM:
							$linkName = isset($cmsFormNames[$row['CMSFormID']]) ? $cmsFormNames[$row['CMSFormID']] : '';
							$linkUrl = scUrlUtil::convertUrl('form.php?FormID=' . $row['CMSFormID'], scUrlUtil::URL_TYPE_DOCROOT);
							break;
							
						case scUserMenuItem::LINK_TYPE_RICH_TEXT_PAGE:
							$linkName = $row['CMSRichTextPageTitle'];
							$linkUrl = scUrlUtil::convertUrl('page.php?PageID=' . $row['CMSRichTextPageID'], scUrlUtil::URL_TYPE_DOCROOT);
							break;
							
						case scUserMenuItem::LINK_TYPE_GALLERY:
							$linkName = $row['GalleryName'];
							$linkUrl = scUrlUtil::convertUrl('gallery.php?GalleryID=' . $row['GalleryID'], scUrlUtil::URL_TYPE_DOCROOT);
							break;
							
						case scUserMenuItem::LINK_TYPE_DYN_PAGE:
							$linkName = $linkablePageNames[$row['CMSDynamicPageID']];
							$linkUrl = scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, $linkablePageCodes[$row['CMSDynamicPageID']]), scUrlUtil::URL_TYPE_DOCROOT);
							break;
							
						default:
						case scUserMenuItem::LINK_TYPE_URL:
							$linkName = $row['CMSMenuItemURL'];
							$parsed = scUrlUtil::parse($row['CMSMenuItemURL']);
							if (isset($parsed[scUrlUtil::SCHEME]) && !in_array($parsed[scUrlUtil::SCHEME], array('http', 'https'))) {
								$linkUrl = $row['CMSMenuItemURL'];
							} else {
								$linkUrl = scUrlUtil::convertUrl($row['CMSMenuItemURL'], scUrlUtil::URL_TYPE_DOCROOT);
							}
							break;
					}
				} catch (miException $e) {
					$msg = $e->getMessage() . ' (Menu: ' . $row['CMSMenuName'] . ', Item ID: ' . $row['CMSMenuItemID'] . ')';
					scLog::singleton()->log(scLog::SEVERITY_ERROR, $msg, scLog::CATEGORY_NONE);
				}
				$row['LinkName'] = $linkName;
				$row['LinkURL'] = $linkUrl;
				
				$row['CMSMenuItemLinkType'] = $linkTypes[$row['CMSMenuItemLinkType']];
			}
		}
		
		protected function getCMSFormNames($rows)
		{
			$cmsFormIds = array();
			foreach ($rows as $row) {
				if ($row['CMSFormID'])
					$cmsFormIds[] = $row['CMSFormID'];
			}
			if (empty($cmsFormIds))
				return array();
			
			$sql = 'SELECT CMSFormID,CMSFormName FROM CMSForms WHERE CMSFormID IN(' . implode(',', $cmsFormIds) . ')';
			return miUtil::getDBArray($sql, 'CMSFormID', 'CMSFormName');
		}
	}
?>