<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOrdersUI {
		
		const OPTION_CHECK_FOR_EGOODS = 1;
		
		const ORDER_ITEMS_WITH_IMAGES = true;
		const ORDER_ITEMS_WITHOUT_IMAGES = false;
		
		const ORDER_ITEMS_EGOODS_REQUIRED = true;
		const ORDER_ITEMS_EGOODS_NOT_REQUIRED = true;
		
		const TEMPLATE_ORDER_DETAILS = 'orderDetails';
		const TEMPLATE_BUNDLED_ORDER_ITEMS = 'bundledOrderItems';
		
		protected $_orderItemOptions = array();
		
		/**
		 * @return scOrdersUI
		public static function singleton()
		{
			static $instance = null;
			
			if (is_null($instance)) {
				$instance = new scOrdersUI();
			}
			
			return $instance;
		}
		 */
		
		protected function __construct()
		{
		}
		
		/**
		 * Return the template filename
		 * @param string $templateCode
		 * @return string
		 */
		public function getTemplateFilename($templateCode)
		{
			return $this->_templates[$templateCode];
		}
		
		public function getOrderDetailsHtml(scOrder $order, $options = array())
		{
			$orderId = $order->get($order->getPKName());
			scOrdersCache::fullLoadOrderData($orderId);
			
			$orderArray = $order->getRow();
			$orderArray = array_merge($orderArray, $this->getOrderInfoVars($order));
			$orderArray['CompanyName'] = scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME);
			$orderArray['OrderStatus'] = scOrderStatuses::singleton()->getOrderStatusName($order->get('OrderStatus'));
			
			$sections = array();
			$sections[] = $this->getOrderItemsSection($order, self::ORDER_ITEMS_WITH_IMAGES);
			$sections[] = $this->getOrderTotalLinesSection($order);
			
			$section = new miTemplateParserSectionInfo('specialInstructions', 0);
			if (trim($orderArray['OrderCustomerInstructions']) != '')
				$section->setIterationsCount(1);
			$sections[] = $section;
			
			$section = new miTemplateParserSectionInfo('hasBillingVatNumber');
			if (trim($orderArray['OrderBillingVatNumber']) != '')
				$section->setIterationsCount(1);
			$sections[] = $section;
			
			$section = new miTemplateParserSectionInfo('hasShippingVatNumber');
			if (trim($orderArray['OrderShippingVatNumber']) != '')
				$section->setIterationsCount(1);
			$sections[] = $section;
			
			$section = new miTemplateParserSectionInfo('orderHasPerItemDiscount', 0);
			if ($order->hasPerItemDiscount())
				$section->setIterationsCount(1);
			$sections[] = $section;
			
			if (in_array(self::OPTION_CHECK_FOR_EGOODS, $options)) {
				if ($order->hasEgoods()) {
					$sections[] = new miTemplateParserSectionInfo('orderHasEgoods', 1);
				}
			}
			
			// Check if there is any difference between the order currency and total and the actually paid currency and total
			if (($order->get('OrderCurrencyCode') != $order->get('OrderTotalPayCurrencyCode')) or
				($order->get('OrderTotal') != $order->get('OrderTotalPayAmount'))) {
				$sections[] = new miTemplateParserSectionInfo('multicurrency');
			}
			
			$t = new scTemplateParser();
			$t->readTemplate($this->getTemplateFilename(self::TEMPLATE_ORDER_DETAILS));
			$t->setSectionInfos($sections);
			$t->assignArray(miUIUtil::templetizeRows($orderArray));
			return $t->templateParse();
		}
		
		/**
		 * Returns array with formatted order info template variables
		 * 
		 * @param array $order array with order info
		 * @param scCurrency $currency
		 * @return array formatted template variables
		 */
		public function getOrderInfoVars(scOrder $order)
		{
			scDateTime::setTimezone();
			
			$templateVars['ORDERDATE'] = scDateTime::getLocalFormattedTimestamp($order->get('OrderTimestamp'), true);
			$templateVars['ORDERTIME'] = date(scDateTime::TIME_FORMAT, $order->get('OrderTimestamp'));
			$templateVars['ORDERIPADDRESS'] = long2ip($order->get('OrderIPAddress'));
			
			$baseCurrency = $order->getBaseCurrency();
			$customerCurrency = $order->getCustomerCurrency();
			
			// Format the order total price
			$amount = scMultiCurrencyAmount::factory();
			$amount->set($baseCurrency, $order->getBaseCurrencyTotal());
			$amount->set($customerCurrency, $order->getCustomerCurrencyTotal());
			$price = scPrice::factory($amount);
			
			$templateVars['ORDERTOTAL'] = $price->getDisplayPrice($baseCurrency);
			$templateVars['ORDERCUSTOMERCURRENCYTOTAL'] = $price->getDisplayPrice($customerCurrency);
			
			return $templateVars;
		}
		
		/**
		 * Returns miTemplateParserSectionInfo containing the order total lines info
		 * 
		 * @param int $orderId
		 * @param scCurrency $currency
		 */
		public function getOrderTotalLinesSection(scOrder $order)
		{
			// Get order total lines
			// TODO: Load using IdentityMap
			$linesRecordset = new miSqlRecordset('OrderTotalLines');
			$linesRecordset->addFilter(new miSqlFilterEqual('OrderID', $order->getPK()));
			$linesRecordset->setOrder('OrderTotalLineSort', 'ASC');
			$lines = array();
			foreach ($linesRecordset->getAllRecords() as $line) {
				// Format order total lines prices
				$amount = scMultiCurrencyAmount::factory();
				$amount->set($order->getBaseCurrency(), $line['OrderTotalLineAmount']);
				$amount->set($order->getCustomerCurrency(), $line['OrderTotalLineCustomerCurrencyAmount']);
				$price = scPrice::factory($amount);
				$line['OrderTotalLineAmount'] = $price->getDisplayPrice($order->getBaseCurrency());
				$line['OrderTotalLineCustomerCurrencyAmount'] = $price->getDisplayPrice($order->getCustomerCurrency());
				$lines[] = $line;
			}
			// Set section lines
			$linesSection = new miTemplateParserSectionInfo();
			$linesSection->setSectionInfo('lines', count($lines), miUIUtil::templetizeRows(miUIUtil::transposeRows($lines)));
			$linesSection->addSubsection(new miTemplateParserSectionInfo('orderHasPerItemDiscount', $order->hasPerItemDiscount()));
			return $linesSection;
		}
		
		/**
		 * Returns miTemplateParserSectionInfo with the order items
		 */
		public function getOrderItemsSection(scOrder $order, $withImages, $egoods = array(), $onlyItemsHavingEgoods = false, $parentOrderItemId = null)
		{
			$orderItems = scOrdersCache::getOrderItems($order->getPK(), $parentOrderItemId);
			
			$products = array();
			$optionSections = array();
			$egoodsSections = array();
			$moreThanOneSections = array();
			$hasItemDiscountSections = array();
			$orderHasPerItemDiscountSections = array();// Available if at least one item in the order has "per item" discount
			$orderHasPerItemDiscountSection = new miTemplateParserSectionInfo('orderHasPerItemDiscount', 0);
			foreach ($orderItems as $orderItem) {
				$orderItemId = $orderItem->get('OrderItemID');
				
				if ($onlyItemsHavingEgoods == self::ORDER_ITEMS_EGOODS_REQUIRED) {
					if (!isset($egoods[$orderItemId])) {
						// There are no egoods for the current product and the $onlyItemsHavingEgoods is set to true
						continue;
					}
				}
				$egoodsSections[] = $this->getOrderItemEgoodsSections($orderItemId, $egoods);
				
				// Basic data
				$product = $orderItem->getRow();
				// Add prices
				$product = array_merge($product, $this->getFormatedOrderItemPrices($orderItem));
				
				$options = array_merge($orderItem->getOrderItemOptions(), $orderItem->getOrderItemLocalOptions());
				
				$optionsData = array();
				$hasPriceModificationSections = array();
				$hasImageOptionSubsections = array();
				$hasUserTextOptionSections = array();
				foreach ($options as &$option) {
					$hasPriceModification = new miTemplateParserSectionInfo('hasPriceModification', 1);
					if (($option->getOptionGroupType() == scProductOptions::GROUP_TYPE_VARIANT) || ($option->getPriceChange()->get($order->getBaseCurrency()) == 0)) {
						$hasPriceModification->setIterationsCount(0);
					}
					
					$hasImageOptionSubsection = new miTemplateParserSectionInfo('hasImageOption', 0);
					if ($option->getOptionGroupType() == scProductOptions::GROUP_TYPE_USER_IMAGE) {
						$hasImageOptionSubsection->setIterationsCount(1);
					}
					$hasImageOptionSubsections[] = $hasImageOptionSubsection;
					
					$hasUserTextOptionSection = new miTemplateParserSectionInfo('hasUserTextOption', 0);
					if ($option->getOptionGroupType() == scProductOptions::GROUP_TYPE_USER_TEXT) {
						$hasUserTextOptionSection->setIterationsCount(1);
					}
					$hasPriceModificationSections[] = $hasPriceModification;
					$hasUserTextOptionSections[] = $hasUserTextOptionSection;
					
					$optionPriceChange = scPrice::factory($option->getPriceChange());
					$optionsData[] = array(
						'OrderItemOptionName' => $option->getName(),
						'OrderItemOptionGroupName' => $option->getOptionGroupName(),
						'OrderItemOptionUserText' => $option->getUserText(),
						'OrderItemOptionPrice' => $optionPriceChange->getDisplayPrice($order->getBaseCurrency()),
						'OrderItemOptionCustomerCurrencyPrice' => $optionPriceChange->getDisplayPrice($order->getCustomerCurrency()),
						'OrderItemOptionImage' => $option->getImage()
					);
				}
				$optionsSection = new miTemplateParserSectionInfo('options', count($optionsData), miUIUtil::templetizeRows(miUIUtil::transposeRows($optionsData)));
				$optionsSection->addSubsectionsArray('hasPriceModification', $hasPriceModificationSections);
				$optionsSection->addSubsectionsArray('hasImageOption', $hasImageOptionSubsections);
				$optionsSection->addSubsectionsArray('hasUserTextOption', $hasUserTextOptionSections);
				$optionSections[] = $optionsSection;

				$moreThanOneSections[] = new miTemplateParserSectionInfo('quantityIsMoreThanOne', intval($orderItem->get('OrderItemQty') > 1));
				
				$t = new scTemplateParser();
				$t->readTemplate($this->getTemplateFilename(self::TEMPLATE_BUNDLED_ORDER_ITEMS));
				$t->addSectionInfo($this->getOrderItemsSection($order, $withImages, $egoods, $onlyItemsHavingEgoods, $orderItemId));
				$product['HTML_BundledItems'] = $t->templateParse();
				
				if ($withImages == self::ORDER_ITEMS_WITH_IMAGES) {
					$image = $this->getOrderItemImage($orderItem)->getDataArray('ProductImage');
					$product = array_merge($product, $image);
				}
				
				// hasItemDiscount
				$hasItemDiscount = ($orderItem->get('OrderItemDiscount') > 0);
				$hasItemDiscountSection = new miTemplateParserSectionInfo('hasItemDiscount', $hasItemDiscount);
				$hasItemDiscountSections[] = $hasItemDiscountSection;
				
				// cartHasItemDiscount
				if ($hasItemDiscount) {
					$orderHasPerItemDiscountSection->setIterationsCount(1);
				}
				$orderHasPerItemDiscountSections[] = &$orderHasPerItemDiscountSection; // A reference to the same object
				
				$products[] = $product;				
			}
			$productsSection = new miTemplateParserSectionInfo();
			$productsSection->setSectionInfo('products', count($products), miUIUtil::templetizeRows(miUIUtil::transposeRows($products)));
			$productsSection->addSubsectionsArray('options', $optionSections);
			$productsSection->addSubsectionsArray('egoods', $egoodsSections);
			$productsSection->addSubsectionsArray('quantityIsMoreThanOne', $moreThanOneSections);
			$productsSection->addSubsectionsArray('hasItemDiscount', $hasItemDiscountSections);
			$productsSection->addSubsectionsArray('orderHasPerItemDiscount', $orderHasPerItemDiscountSections);
			
			return $productsSection;
		}
		
		/**
		 * @return scProductImageInfo
		 */
		protected function getOrderItemImage(scOrderItem $orderItem)
		{
			// Try to get the image that the order item was originaly saved with.
			$image = scProductImage::getProductImageById($orderItem->get('ProductImageID'));
			// Check if the original image was not found.
			if ($image instanceof scProductNoImageImageInfo) {
				// Try to get the image from the listing item that the customer bought.
				$listingItem = $orderItem->getListingItem();
				if ($listingItem instanceof IProductListingItem) {
					return $listingItem->getImageInfo();
				}
			}
			return $image;
		}
		
		/**
		 * @param int $orderId
		 * @return miSqlRecordset
		 */
		protected function getOrderProductsRecordset($orderId, $withImages, $parentOrderItemId = null)
		{
			// Get order products
			$productRecordset = new miSqlRecordset('OrderItems');
			$productRecordset->addFilter(new miSqlFilterEqual('OrderID', intval($orderId)));
			if (is_null($parentOrderItemId)) {
				$productRecordset->addFilter(new scSqlFilterIsNull('ParentOrderItemID'));
			} else {
				$productRecordset->addFilter(new miSqlFilterEqual('ParentOrderItemID', $parentOrderItemId));
			}
			if ($withImages == self::ORDER_ITEMS_WITH_IMAGES) {
				$productRecordset->addJoinCondition('LEFT', 'ProductImages', 'USING(ProductImageID)');
				$productRecordset->setSelectFields(array('*, ProductImages.ProductImageID AS ProductImageID'));
			}
			return $productRecordset;
		}
		
		/**
		 * @return string[]
		 */
		protected function getFormatedOrderItemPrices(scOrderItem $orderItem)
		{
			$price = $orderItem->getPrice();
			
			$baseCurrency = $orderItem->getOrder()->getBaseCurrency();
			$customerCurrency = $orderItem->getOrder()->getCustomerCurrency();
			
			$data['OrderItemPrice'] = $price->getDisplayPrice($baseCurrency);
			$data['OrderItemCustomerCurrencyPrice'] = $price->getDisplayPrice($customerCurrency);
			
			$total = $orderItem->getTotal();
			
			$data['OrderItemTotal'] = $total->getDisplayPrice($baseCurrency);
			$data['OrderItemCustomerCurrencyTotal'] = $total->getDisplayPrice($customerCurrency);
			
			$discount = scMultiCurrencyAmount::factory();
			$discount->set($baseCurrency, $orderItem->get('OrderItemDiscount'));
			$discount->set($customerCurrency, $orderItem->get('OrderItemCustomerCurrencyDiscount'));
			$discount = scPrice::factory($discount);
			
			$data['OrderItemDiscount'] = $discount->getDisplayPrice($baseCurrency);
			$data['OrderItemCustomerCurrencyDiscount'] = $discount->getDisplayPrice($customerCurrency);
			
			$basePrice = scMultiCurrencyAmountMath::add($price->getAmount(), $discount->getAmount());
			$basePrice = scPrice::factory($basePrice);
			
			$data['OrderItemBasePrice'] = $basePrice->getDisplayPrice($baseCurrency);
			$data['OrderItemCustomerCurrencyBasePrice'] = $basePrice->getDisplayPrice($customerCurrency);
			
			return $data;
		}
		
		public function getOrderItemEgoodsSections($orderItemId, $egoodsArray)
		{
			// Prepare the egoods section
			$productEgoods = isset($egoodsArray[$orderItemId]) ? $egoodsArray[$orderItemId] : array();
			
			$isDownloadableSections = array();
			$isNotDownloadableSections = array();
			
			$isLockedSections = array();
			$isUnlockedSections = array();
			
			foreach ($productEgoods as $productEgood) {
				$orderItemEgood = scOrderItemEgood::fromArray($productEgood);
				
				list($isDownloadableSection, $isNotDownloadableSection) = $this->getIsDownloadableSections($orderItemEgood);
				list($isUnlockedSection, $isLockedSection) = $this->getIsLockedSections($orderItemEgood);
				
				$isDownloadableSections[] = $isDownloadableSection;
				$isNotDownloadableSections[] = $isNotDownloadableSection;
				
				$isLockedSections[] = $isLockedSection;
				$isUnlockedSections[] = $isUnlockedSection;
			}
			
			$egoodsSection = new miTemplateParserSectionInfo('egoods', count($productEgoods), miUIUtil::templetizeRows(miUIUtil::transposeRows($productEgoods)));
			$egoodsSection->addSubsectionsArray('egoodIsDownloadable', $isDownloadableSections);
			$egoodsSection->addSubsectionsArray('egoodIsNotDownloadable', $isNotDownloadableSections);
			$egoodsSection->addSubsectionsArray('egoodIsUnlocked', $isUnlockedSections);
			$egoodsSection->addSubsectionsArray('egoodIsLocked', $isLockedSections);
			
			return $egoodsSection;
		}
		
		protected function getIsDownloadableSections(scOrderItemEgood &$orderItemEgood)
		{
			$isDownloadableSection = new miTemplateParserSectionInfo('egoodIsDownloadable', 0);
			$isNotDownloadableSection = new miTemplateParserSectionInfo('egoodIsNotDownloadable', 0);
			
			try {
				$orderItemEgood->checkIsDownloadable();
				$isDownloadableSection->setIterationsCount(1);
			} catch (miException $e) {
				$isNotDownloadableSection->setIterationsCount(1);
				$isNotDownloadableSection->setValues(array('%%REASON%%' => $e->getMessage()));
			}
			
			return array($isDownloadableSection, $isNotDownloadableSection);
		}
		
		protected function getIsLockedSections(scOrderItemEgood &$orderItemEgood)
		{
			$isUnlockedSection = new miTemplateParserSectionInfo('egoodIsUnlocked', 0);
			$isLockedSection = new miTemplateParserSectionInfo('egoodIsLocked', 0);
			
			if ($orderItemEgood->isLocked()) {
				$isLockedSection->setIterationsCount(1);
			} else {
				$isUnlockedSection->setIterationsCount(1);
			}
			
			return array($isUnlockedSection, $isLockedSection);
		}
	}
	
	class scCustomerOrdersUI extends scOrdersUI {
		
		protected $_templates = array(
			self::TEMPLATE_ORDER_DETAILS => 'customer/myaccount/order_details.tmpl',
			self::TEMPLATE_BUNDLED_ORDER_ITEMS => 'customer/myaccount/bundled_order_items.tmpl',
		);
		
		/**
		 * @return scCustomerOrdersUI
		 */
		public static function singleton()
		{
			static $instance = null;
			
			if (is_null($instance)) {
				$instance = new scCustomerOrdersUI();
			}
			
			return $instance;
		}
		
		/**
		 * Returns the Orders Count by CustomerID
		 * Does not return UNFINISHED orders 
		 *
		 * @param int $customerId
		 */
		public static function getOrdersCountByCustomerId($customerId)
		{
			$ordersCount = miStaticDBUtil::execSelect('SELECT COUNT(OrderID) AS N FROM Orders' .
				' WHERE CustomerID = ? AND OrderStatus != ?', array($customerId, scOrderStatus::ORDER_STATUS_UNFINISHED));
			if (!empty($ordersCount))
				return $ordersCount[0]['N'];
			
			return 0;
		}
		
		protected function __construct()
		{
		}
	}
	
	class scEmailOrdersUI extends scCustomerOrdersUI {
		
		/**
		 * @return scEmailOrdersUI
		 */
		public static function singleton()
		{
			static $instance = null;
			
			if (is_null($instance)) {
				$instance = new scEmailOrdersUI();
			}
			
			return $instance;
		}
		
		public function getTemplateFilename($templateCode)
		{
			$area = scSkins::singleton()->getCurrentSkin()->getArea();
			switch ($area) {
				case scLayout::AREA_ADMIN:
					return scAdminOrdersUI::singleton()->getTemplateFilename($templateCode);
				
				case scLayout::AREA_CUSTOMER:
					return scCustomerOrdersUI::singleton()->getTemplateFilename($templateCode);
			}
			throw new miException(_i('Unknown area.'));
		}
		
		protected function __construct()
		{
		}
	}
	
	class scAdminOrdersUI extends scOrdersUI {
		
		protected $_templates = array(
			self::TEMPLATE_ORDER_DETAILS => 'customer/myaccount/order_details.tmpl',
			self::TEMPLATE_BUNDLED_ORDER_ITEMS => 'admin/orders/bundled_order_items.tmpl',
		);
		
		/**
		 * @return scAdminOrdersUI
		 */
		public static function singleton()
		{
			static $instance = null;
			
			if (is_null($instance)) {
				$instance = new scAdminOrdersUI();
			}
			
			return $instance;
		}
		
		protected function __construct()
		{
		}
	}
?>