<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Products List
	 */
	class scProductsTable extends scTable {
		const DEFAULT_PRODUCT_THUMB_URL = 'admin/images/image-not-available/default_product_thumb.gif';
		const DEFAULT_PRODUCT_THUMB_WIDTH = 100;
		const DEFAULT_PRODUCT_THUMB_HEIGHT = 100;
		
		protected function addUiElements(&$rows)
		{
			$defaultLanguageId = scMultiLanguage::getDefaultLanguageId();
			
			$manufacturerNames = $this->getManufacturerNames($rows);
			$categoryNames = $this->getCategoryNames($rows);
			$productImageIds = $this->getProductImages($rows);
			$productVariantImages = $this->getProductVariantImages($rows);
			
			$defaultImage = '<img src="'.scUrlUtil::convertUrl(scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl(), scUrlUtil::URL_TYPE_DOCROOT) .
				self::DEFAULT_PRODUCT_THUMB_URL . '" width="'.self::DEFAULT_PRODUCT_THUMB_WIDTH.'" height="'.self::DEFAULT_PRODUCT_THUMB_HEIGHT.'" alt="" />';
			
			foreach ($rows as &$row) {
				$productId = $row['ProductID'];
				
				$row['FMT_PRODUCTCATEGORY'] = '';
				if (isset($categoryNames[$row['CategoryID']])) {
					$row['FMT_PRODUCTCATEGORY'] = $categoryNames[$row['CategoryID']];
				} elseif (isset($categoryNames[$defaultLanguageId])) {
					$row['FMT_PRODUCTCATEGORY'] = $categoryNames[$defaultLanguageId];
				}
				$row['PRODUCTMANUFACTURER'] = @$manufacturerNames[$row['ManufacturerID']];
				
				$productImageId = isset($productImageIds[$productId]) ? $productImageIds[$productId]['ProductImageID'] : false;
				if ($productImageId && is_file(scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_THUMB))) {
					$width = $productImageIds[$row['ProductID']]['ProductImageThumbWidth'];
					$height = $productImageIds[$row['ProductID']]['ProductImageThumbHeight'];
					list($width, $height) = scImageHandler::getImageFitSize($width, $height, self::DEFAULT_PRODUCT_THUMB_WIDTH, self::DEFAULT_PRODUCT_THUMB_WIDTH);
					$row['HTML_PRODUCTIMAGETHUMB'] = '<img src="../' . scPaths::getItemPath($productImageId, scPaths::RESOURCE_PRODUCT_THUMB, scPaths::PATH_WEB) . '" width="'.$width.'" height="'.$height.'" alt="" />';
				} else if (isset($productVariantImages[$productId]) && is_file(scPaths::getStoreRootDir() . $productVariantImages[$productId]['ProductVariantImageThumb'])) {
					$image = $productVariantImages[$productId];
					$width = $image['ProductVariantImageThumbWidth'];
					$height = $image['ProductVariantImageThumbHeight'];
					list($width, $height) = scImageHandler::getImageFitSize($width, $height, self::DEFAULT_PRODUCT_THUMB_WIDTH, self::DEFAULT_PRODUCT_THUMB_WIDTH);
					$row['HTML_PRODUCTIMAGETHUMB'] = '<img src="../' . $image['ProductVariantImageThumb'] . '" width="'.$width.'" height="'.$height.'" alt="" />';
				} else {
					$row['HTML_PRODUCTIMAGETHUMB'] = $defaultImage;
				}
				
				// Escape for javascript string
				$description = $row['ProductDescriptionXHTML'] ? $row['ProductDescription'] : miI18N::htmlEscape($row['ProductDescription']);
				$escape = array("\r\n" => '\n', "\r" => '\n', "\n" => '\n', '"' => '\"', "'" => "\\'");
				$description = str_replace(array_keys($escape), array_values($escape), $description);
				
				$row['HTML_PRODUCTDESCRIPTION'] = $description;
			}
		}
		
		protected function getManufacturerNames($rows)
		{
			$manufacturerIds = array();
			foreach ($rows as $row) {
				$manufacturerIds[] = (int)$row['ManufacturerID'];
			}
			if (count($manufacturerIds) == 0)
				return array();
			
			$sql = 'SELECT ManufacturerID,ManufacturerName FROM ManufacturersML WHERE LanguageID=' . scMultiLanguage::getCurrentLanguageId() .
				' AND ManufacturerID IN(' . implode(',', array_unique($manufacturerIds)) . ')';
			return miUtil::getDBArray($sql, 'ManufacturerID', 'ManufacturerName');
		}
		
		protected function getProductImages($rows)
		{
			$productIds = array();
			foreach ($rows as $row) {
				$productIds[] = (int)$row['ProductID'];
			}
			if (count($productIds) == 0)
				return array();
			
			$sql = 'SELECT ProductID,ProductImageID,ProductImageThumbWidth,ProductImageThumbHeight FROM ProductImages WHERE ProductID IN(' .
				implode(',', $productIds) . ') AND ProductImageIsDefault<>0';
			$rows = miStaticDBUtil::execSelect($sql);
			
			return miUtil::getArrayWithKeys($rows, 'ProductID');
		}
		
		protected function getProductVariantImages($rows)
		{
			$productIds = array();
			foreach ($rows as $row) {
				$productIds[] = (int)$row['ProductID'];
			}
			if (count($productIds) == 0)
				return array();
			
			$sql = 'SELECT ProductID,ProductVariantImageThumb,ProductVariantImageThumbWidth,ProductVariantImageThumbHeight' .
				' FROM ProductVariants WHERE ProductID IN(' . implode(',', $productIds) . ')' .
				' AND ProductVariantIsDefault <> 0';
			$rows = miStaticDBUtil::execSelect($sql);
			
			return miUtil::getArrayWithKeys($rows, 'ProductID');
		}
		
		protected function getCategoryNames($rows)
		{
			$categoryIds = array();
			foreach ($rows as $row) {
				$categoryIds[] = (int)$row['CategoryID'];
			}
			if (count($categoryIds) == 0)
				return array();
			
			$sql = 'SELECT CategoryID,CategoryName FROM CategoriesML WHERE LanguageID=' . scMultiLanguage::getCurrentLanguageId() .
				' AND CategoryID IN(' . implode(',', array_unique($categoryIds)) . ')';
			return miUtil::getDBArray($sql, 'CategoryID', 'CategoryName');
		}
		
		/**
		 * @return scMessage
		 */
		protected function getMessageObj()
		{
			if ($this->_messageObj == null)
				$this->_messageObj = new scMessage;
			return $this->_messageObj;
		}
	}
?>