<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scTable extends miTable {
		
		protected static $_sortingState = 0;
		
		/**
		 * @var string
		 */
		protected $_sectionName = 'rows';
		
		/**
		 * @var miTemplateParserSectionInfo
		 */
		protected $_section = null;
		
		/**
		 * @param string $sectionName
		 * @throws miException
		 */
		public function setSectionName($sectionName)
		{
			if (!is_string($sectionName) || empty($sectionName)) {
				throw new miException(_i('Invalid section name'));
			}
			$this->_sectionName = $sectionName;
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		public function getSection()
		{
			if (!is_null($this->_section)) {
				return $this->_section;
			}
			
			$rows = $this->_recordset->getRecords();
			
			$this->_section = new miTemplateParserSectionInfo();
			if (count($rows) > 0) {
				$this->addUiElements($rows);
				$this->_section->setSectionInfo($this->_sectionName, count($rows), miUIUtil::templetizeRows(miUIUtil::transposeRows($rows)));
			} else {
				$this->_section->setSectionInfo('no' . $this->_sectionName, 1);
			}
			
			return $this->_section;
		}
		
		/**
		 * Left trim the HTML so that there are no empty lines when updating with AJAX.
		 * Uses the getSection() method.
		 * @param string $templateFilename
		 * @return string
		 */
		public function parse($templateFilename)
		{
			$section = $this->getSection();
			
			$t = new scTemplateParser();
			$t->addSectionInfo($section);
			$t->readTemplate($templateFilename);
			
			$this->assignFeatureValues($t);
			$t->assignArray($this->getMessageObj()->getMessageTemplateVars());
			self::escapeAndAssignArray($t, $this->_mainPageElements);
			$this->addMainUiElements($t);
			
			return ltrim($t->templateParse());
		}
		
		public static function setSortingState($sortingState)
		{
			self::$_sortingState = $sortingState;
		}
		
		protected function setSortColumn(&$t)
		{
			$sortSubsection = new miTemplateParserSectionInfo();
			$sortDir = miGetParamDefault(miTableSorter::PARAM_SORTDIR, 'ASC');
			$up = ($sortDir != 'DESC') ? _t('Up') : _t('Down');
			$iconUp = ($sortDir != 'DESC') ? 'icon_up' : 'icon_down';
			$down = ($sortDir != 'DESC') ? _t('Down') : _t('Up');
			$iconDown = ($sortDir != 'DESC') ? 'icon_down' : 'icon_up';
			$values = array('%%UP%%' => $up, '%%ICONUP%%' => $iconUp, '%%DOWN%%' => $down, '%%ICONDOWN%%' => $iconDown);
			$sortSubsection->setSectionInfo('sortColumn' . self::$_sortingState, 1, $values);
			
			$section = $t->getSectionInfo($this->_sectionName);
			if ($section instanceof miTemplateParserSectionInfo )
				$section->addSubsection($sortSubsection);
		}
		
		protected function addMainUiElements(&$t)
		{
			if (self::$_sortingState)
				$this->setSortColumn($t);
			
			if (miGetParamDefault('load', '') == 'table')
				return;
			
			$section = new miTemplateParserSectionInfo();
			$section->setSectionInfo('static', 1);
			$t->addSectionInfo($section);
		}
	}
?>