<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scUICart {
		
		/**
		 * @var scCart
		 */
		protected $_cart = null;
		
		/**
		 * @var scUICartTemplater
		 */
		protected $_templater = null;
		
		public function setCart(scCart $cart)
		{
			$this->_cart = $cart;
		}
		
		/**
		 * @return scUICartTemplater
		 */
		public function getTemplater()
		{
			if (is_null($this->_templater)) {
				$this->_templater = new scUICartTemplater();
			}
			return $this->_templater;
		}
		
		/**
		 * @return scCart
		 */
		protected function getCart()
		{
			if (is_null($this->_cart)) {
				$this->_cart = scCart::getCart();
			}
			return $this->_cart;
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		public function getCartTemplateSection()
		{
			return $this->getCartItemsSection($this->getCart()->getItems());
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		public function getBonusPointsUsageApprovalSection()
		{
			$section = new miTemplateParserSectionInfo('bonusPointsUsageApproval', 0);
			if (!scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_MANUAL_SPENDING)) {
				return $section;
			}
			
			$customer = scCustomerAuth::singleton()->getCustomer();
			if ($customer === false) {
				return $section;
			}
			
			$bonusPoints = $customer->get('CustomerBonusPoints');
			if ($bonusPoints > 0) {
				$section->setIterationsCount(1);
				$section->setValues(array(
					'%%CUSTOMERBONUSPOINTS%%' => $bonusPoints,
				));
			}
			return $section;
		}
		
		public function getCouponsTemplateSection()
		{
			try {
				$couponsSection = new miTemplateParserSectionInfo('coupons');
				if (!$this->hasActiveCoupons()) {
					$couponsSection->setIterationsCount(0);
					return $couponsSection;
				}
				
				if ($this->getCart()->canApplyCoupon()) {
					$couponsSection->addSubsection(new miTemplateParserSectionInfo('canApplyCoupon'));
					$couponsSection->addSubsection(new miTemplateParserSectionInfo('canRemoveCoupon', 0));
				} else {
					$couponsSection->addSubsection(new miTemplateParserSectionInfo('canApplyCoupon', 0));
					$couponsSection->addSubsection(new miTemplateParserSectionInfo('canRemoveCoupon'));
				}
				
				return $couponsSection;
			}
			catch (miException $exception)
			{
				return new miTemplateParserSectionInfo('coupons', 0);
			}
		}
		
		/**
		 * Checks if there are any applicable coupons.
		 * 
		 * That's coupons that are active, valid for the current moment
		 * and not used as many times as allowed.
		 * @return bool
		 */
		public function hasActiveCoupons()
		{
			$sql = 'SELECT COUNT(1) AS N FROM Coupons' .
				' WHERE CouponValidFrom <= CURDATE()' .
				' AND CouponValidTo >= CURDATE()' .
				' AND CouponUsageCount < CouponAllowedUsageCount' .
				' AND CouponIsActive';
			$result = miStaticDBUtil::execSelect($sql);
			if (isset($result[0]) && isset($result[0]['N'])) {
				return (bool)$result[0]['N'];
			}
			return false;
		}		
		
		protected function getCheckoutModules()
		{
			$sql = 'SELECT CheckoutModuleName,CheckoutModuleDisplayName,ModuleName FROM ' .
				'CheckoutModules LEFT JOIN CheckoutModulesML USING(CheckoutModuleID) JOIN Modules ON CheckoutModules.ModuleID=Modules.ModuleID '.
				'WHERE CheckoutModuleIsActive=1 AND LanguageID=? ORDER BY CheckoutModuleSort';
			return miStaticDBUtil::execSelect($sql, array(scMultiLanguage::getCurrentLanguageId()));
		}
		
		public function getCheckoutModulesSection()
		{
			$checkoutModules = $this->getCheckoutModules();
			
			$var = array();
			foreach ($checkoutModules as $key => $checkoutModule) {
				$moduleId = scModulesFactory::getModuleIdByName($checkoutModule['ModuleName']);
				$checkoutModuleButton = scModulesFactory::getImplementation($moduleId, 'ICheckoutModuleButton');
				
				$var['%%HTML_CHECKOUT_BUTTON%%'][$key] = $checkoutModuleButton->getButtonHtml($checkoutModule['ModuleName'],
					$checkoutModule['CheckoutModuleName'], $checkoutModule['CheckoutModuleDisplayName']);
			}
			return new miTemplateParserSectionInfo('checkoutModules', count($checkoutModules), $var);
		}
		
		public function getOrderTotalLinesTemplateSection()
		{
			$orderTotalLinesSection = new miTemplateParserSectionInfo();
			
			$cart = $this->getCart();
			if ($cart->getOrderTotal() != null) {
				$orderTotalLines = $cart->getOrderTotal()->getOrderTotalLines();
				
				$lines = array();
				foreach ($orderTotalLines as $key => $orderTotalLine) {
					$lines[$key]['amount'] = scPrice::factory($orderTotalLine->getAmount())->getDisplayPrice();
					$lines[$key]['description'] = $orderTotalLine->getTranslatedDescription();
				}
				$orderTotalLinesSection->setSectionInfo('orderTotalLines', count($lines), miUIUtil::templetizeRows(miUIUtil::transposeRows($lines)));
			}
			return $orderTotalLinesSection;
		}
		
		protected function getTemplatizedCartItems($cartItems)
		{
			if (count($cartItems) == 0)
				return array();
			
			$currentCurrency = scCurrencyUtil::getCurrentCurrency();
			
			// Add price sections
			$vars = array();
			$cartUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART);
			foreach ($cartItems as $cartItem) {
				$displayPrice = $cartItem->getPrice();
				$basePrice = $cartItem->getPrice(ICartItem::PRICE_TYPE_OPTIONS_MODIFIED);
				$discount = scPrice::factory(scMultiCurrencyAmountMath::subtract($displayPrice->getAmount(), $basePrice->getAmount()));
				
				$var = array(
					'CartItemCode' => $cartItem->getCartItemCode(),
					'CartItemName' => $cartItem->getName(),
					'CartItemPrice' => $displayPrice->getDisplayPrice(),
					'CartItemBasePrice' => $basePrice->getDisplayPrice(),
					'CartItemDiscount' => $discount->getDisplayPrice(),
					'CartItemQuantity' => $cartItem->getQuantity(),
					'CartItemTotal' => scPrice::factory($cartItem->getTotal())->getDisplayPrice(),
					'CartItemHash' => $cartItem->getHash(),
					'ProductID' => $cartItem->getProductId(),
					'ProductVariantID' => 0,
					'HTML_BundledItems' => '',
					'URL_RemoveItem' => scUrlUtil::addQueryParams($cartUrl, 'action=dmRemoveItem&CartItemCode=' . $cartItem->getHash()),
				) + $cartItem->getImageInfo()->getDataArray('ProductImage');
				
				if ($cartItem instanceof ICartItemProductVariant) {
					$var['ProductVariantID'] = $cartItem->getProductVariant()->getPK();
				}
				
				if ($cartItem instanceof ICartItemProductBundle) {
					$var['HTML_BundledItems'] = $this->getBundledItemsHtml($cartItem);
				}
				
				$vars[] = $var;
			}
			
			scFriendlyUrl::initFriendlyUrls($vars, scFriendlyUrl::ROW_TYPE_CART_ITEMS);
			return miUIUtil::templetizeRows(miUIUtil::transposeRows($vars));	
		}
		
		/**
		 * @param scCartItem $cartItem
		 * @return string
		 */
		protected function getBundledItemsHtml(scCartItem $cartItem)
		{
			$bundledItems = $cartItem->getBundledItems();
			if (count($bundledItems) < 1) {
				return '';
			}
			$t = new scTemplateParser();
			$t->readTemplate($this->getTemplater()->get(scUICartTemplater::BUNDLED_ITEMS));
			$hasBundledItemsSection = new miTemplateParserSectionInfo('hasBundledItems', 1);
			$hasBundledItemsSection->addSubsection($this->getCartItemsSection($bundledItems));
			$t->addSectionInfo($hasBundledItemsSection);
			return $t->templateParse();
		}
		
		/**
		 * @param array $cartItems
		 * @return miTemplateParserSectionInfo
		 */
		protected function getCartItemsSection($cartItems)
		{
			$itemsSection = new miTemplateParserSectionInfo();
			$itemsSection->setSectionInfo('cartItems', count($cartItems), $this->getTemplatizedCartItems($cartItems));
			
			if (count($cartItems) > 0) {
				// Populate the options sections
				$optionsSections = array();
				$moreThanOneSections = array();
				$hasItemDiscountSections = array();
				$cartHasItemDiscountSections = array();// Available if at least one item in the cart has "per item" discount
				$cartHasItemDiscountSection = new miTemplateParserSectionInfo('cartHasPerItemDiscount', 0);
				foreach ($cartItems as $cartItem) {
					if (!($cartItem instanceof scCartItemProduct))
						continue;
					$optionsSections[] = $this->getOptionsTemplateSection($cartItem);
					$moreThanOneSections[] = $this->getQuantityIsMoreThanOneTemplateSection($cartItem);
					
					$baseAmount = $cartItem->getPrice(scCartItem::PRICE_TYPE_OPTIONS_MODIFIED)->getAmount();
					$finalAmount = $cartItem->getPrice(scCartItem::PRICE_TYPE_MODIFIED)->getAmount();
					
					// hasItemDiscount
					$hasItemDiscount = !$finalAmount->equals($baseAmount);
					$hasItemDiscountSection = new miTemplateParserSectionInfo('hasItemDiscount', $hasItemDiscount);
					$hasItemDiscountSections[] = $hasItemDiscountSection;
					
					// cartHasItemDiscount
					if ($hasItemDiscount) {
						$cartHasItemDiscountSection->setIterationsCount(1);
					}
					$cartHasItemDiscountSections[] = &$cartHasItemDiscountSection; // A reference to the same object
				}
				$itemsSection->addSubsectionsArray('hasOptions', $optionsSections);
				$itemsSection->addSubsectionsArray('quantityIsMoreThanOne', $moreThanOneSections);
				$itemsSection->addSubsectionsArray('hasItemDiscount', $hasItemDiscountSections);
				$itemsSection->addSubsectionsArray('cartHasPerItemDiscount', $cartHasItemDiscountSections);
			}
			
			return $itemsSection;
		}
		
		/**
		 * @param scCartItem $item
		 * @return miTemplateParserSectionInfo
		 */
		protected function getQuantityIsMoreThanOneTemplateSection(scCartItem $item)
		{
			return new miTemplateParserSectionInfo('quantityIsMoreThanOne', intval($item->getQuantity() > 1));
		}
		
		protected function getOptionsTemplateSection(scCartItem $item)
		{
			$hasOptionsSection = new miTemplateParserSectionInfo();
			
			if ($item instanceof scCartItemProduct) {
				$options = array_merge($item->getOptions(), $item->getLocalOptions());
				if (count($options) == 0)
					$hasOptionsSection->setSectionInfo('hasOptions', 0);
				else {
					$hasOptionsSection->setSectionInfo('hasOptions', 1);
					$price = $item->getPriceObj(false)->getAmount();
					
					// Get options info
					$optionsInfo = array();
					$hasPriceModificationSections = array();
					$hasImageOptionSubsections = array();
					$hasUserTextOptionSections = array();
					foreach ($options as $option) {
						$optionInfo = $option->getOptionInfo($price);
						$subsection = new miTemplateParserSectionInfo('hasPriceModification', 1);
						if (($optionInfo['OptionPriceModType'] == scProductOptions::PRODUCT_OPTION_TYPE_NONE) || ($optionInfo['OptionPriceModValue'] == 0)) {
							$subsection->setIterationsCount(0);
						}
						
						$optionInfo['OptionUploadedFilePathWeb'] = $option->getOptionUploadedFilePathForWeb();
						
						$hasImageOptionSubsection = new miTemplateParserSectionInfo('hasImageOption', 0);
						if ($optionInfo['OptionGroupType'] == scProductOptions::GROUP_TYPE_USER_IMAGE) {
							$hasImageOptionSubsection->setIterationsCount(1);
						}
						$hasImageOptionSubsections[] = $hasImageOptionSubsection;
						
						$hasUserTextOptionSection = new miTemplateParserSectionInfo('hasUserTextOption', 0);
						if ($optionInfo['OptionGroupType'] == scProductOptions::GROUP_TYPE_USER_TEXT) {
							$hasUserTextOptionSection->setIterationsCount(1);
						}
						$hasUserTextOptionSections[] = $hasUserTextOptionSection;
						
						$hasPriceModificationSections[] = $subsection;
						$optionsInfo[] = $optionInfo;
					}
					
					// Add the options section
					$optionsSection = new miTemplateParserSectionInfo('options', count($optionsInfo), miUIUtil::templetizeRows(miUIUtil::transposeRows($optionsInfo)));
					$optionsSection->addSubsectionsArray('hasPriceModification', $hasPriceModificationSections);
					$optionsSection->addSubsectionsArray('hasImageOption', $hasImageOptionSubsections);
					$optionsSection->addSubsectionsArray('hasUserTextOption', $hasUserTextOptionSections);
					$hasOptionsSection->addSubsection($optionsSection);
				}
			}
			else
				$hasOptionsSection->setSectionInfo('hasOptions', 0);
			
			return $hasOptionsSection;
		}
	}
	
	class scUICartTemplater {
		
		const ITEMS = 'items';
		const BUNDLED_ITEMS = 'bundledItems';
		
		protected $_templateLocations = array(
			self::ITEMS => 'customer/cart_items.tmpl',
			self::BUNDLED_ITEMS => 'customer/bundled_cart_items.tmpl',
		);
		
		public function set($type, $templateLocation)
		{
			$this->_templateLocations[$type] = $templateLocation;
		}
		
		public function get($type)
		{
			if (!isset($this->_templateLocations[$type])) {
				throw new miException(sprintf(_i('Unknown template type "%s".'), $type));
			}
			return $this->_templateLocations[$type];
		}
	}
?>