<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * User Interface Util Class
	 *
	 * @copyright Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	class scUIUtil {
		
		// Default filter value for date range filter
		const DAYS_AGO_FILTER = 29;
		
		/**
		 * Returns HTML code for a dropdown menu ("select" element) from the specified array
		 * 
		 * @access public
		 * @param string $name name of the select element
		 * @param int $selected which key is selected
		 * @param array $optionsArray array whit the dropdown menu elements values
		 */
		public static function getConstDropdown($name, $selected, $optionsArray, $htmlAttributes = array())
		{
			$supportedAttributes = array('id', 'class', 'sc:defaultValue');
			$attributes = '';
			if (!empty($htmlAttributes)) {
				foreach ($htmlAttributes as $attribute => $value) {
					if (!in_array($attribute, $supportedAttributes)) {
						continue;
					}
					$attributes .= ' ' . $attribute . '="' . miI18N::htmlEscape($value) . '"';
				}
			}
			
			$html = '<select name="' . $name . '"';
			$html .= $attributes;
			$html .= '>' . "\n";
			$selected = (string)$selected;
			foreach ($optionsArray as $key => $value)
				$html .= '<option value="' . miI18N::htmlEscape($key) . '"' . ((string)$key === $selected?' selected="selected"':'') . '>' . miI18N::htmlEscape($value) . '</option>' . "\n";
			$html .= '</select>' . "\n";
			return $html;
		}
		
		/**
		 * Creates drop-down filter
		 * The filter is applied to the $table
		 */
		public static function createDropdownFilter(miView $view, miTable $table, $field, $values, $condition, $defaultValue = null)
		{
			$filterObj = $view->getTableFilterObj();
			$filterValues = $filterObj->getFilterValues();
			$selectedValue = isset($filterValues[$field]) ? $filterValues[$field] : '';
			
			// Make sure that the filter is applied
			if (!isset($_REQUEST[$field . 'Filter']))
				$filterObj->addAdditionalFilter($field, $selectedValue, $condition);
			
			$htmlAttributes = array();
			if (!is_null($defaultValue)) {
				$htmlAttributes['sc:defaultValue'] = $defaultValue;
			}
			$dropdown = scUIUtil::getConstDropdown($field . 'Filter', $selectedValue, $values, $htmlAttributes);
			$condition = '<input type="hidden" name="' . miI18N::htmlEscape($field) . 'Condition" value="' . miI18N::htmlEscape($condition) . '"/>';
			$table->assign('%%HTML_' . strtoupper($field) . '_FILTER%%', $dropdown . $condition);
		}
		
		/**
		 * Return a javascript array with all states in the database
		 * 
		 * @return string the javascript 
		 */
		public static function getStatesJsData($arrayName = 'states')
		{
			$rows = scCountry::getStateRows();
			
			$rowsByCountry = array();
			foreach ($rows as $row)
				$rowsByCountry[$row['CountryID']][] = $row;
			
			$result = 'var '.$arrayName.' = new Array();';
			foreach ($rowsByCountry as $countryId => $rows) {
				$result .= $arrayName . '[' . $countryId . ']=Array();';
				$result .= $arrayName . '[' . $countryId . '][0]="' . _t('Choose') . '";';
				foreach ($rows as $row) {
					$result .= $arrayName . '[' . $countryId . '][' . $row['StateID'] . ']="' . addslashes($row['StateName']) . '";';
				}
			}
			return $result;
		}
		
	
		/**
		 * Creates date period filter
		 * The filter is applied to the $table
		 */
		public static function createDatePeriodFilter(miDefaultView $view, miTable $table, $field, $condition)
		{
			if (miGetParamDefault($field . 'Filter', '') == '') {
				
				$orderTimestampFilter[scSqlFilterDatePeriod::PARAM_PERIOD_TYPE] = miGetParamDefault('typePeriod', scSqlFilterDatePeriod::PERIOD_TYPE_CUSTOM);
				$orderTimestampFilter[scSqlFilterDatePeriod::PARAM_PREDEFINED_PERIOD] = miGetParamDefault('period', scSqlFilterDatePeriod::DATE_PERIODS_THIS_MONTH);
				$orderTimestampFilter[scSqlFilterDatePeriod::PARAM_DATE_FROM] = miGetParamDefault('StartDate', date('d/m/Y', time() - self::DAYS_AGO_FILTER * 60 * 60 * 24));
				$orderTimestampFilter[scSqlFilterDatePeriod::PARAM_DATE_TO] = miGetParamDefault('EndDate', date('d/m/Y'));
				miSetParam($field . 'Filter', implode(',', $orderTimestampFilter));
				miSetParam($field . 'Condition', $condition);
			}
			
			// Make sure that the filter is applied
			if (!isset($_REQUEST[$field . 'Filter'])) {
				$filterObj = $view->getTableFilterObj();
				$filterObj->addAdditionalFilter($field, '', $condition);
			}
			
			$valueFilter = miGetParamDefault($field . 'Filter', 0);
			if ($valueFilter)
				$values = explode(',', $valueFilter);
			else
				$values = array_fill(0, 4, '');

			$html = self::getTimePeriodWidgetHtml($values);
			$html .= '<input type="hidden" name="' . $field . 'Filter" value=""/>';
			$html .= '<input type="hidden" name="' . $field . 'Condition" value="' . $condition . '"/>';
			
			$table->assign('%%HTML_' . strtoupper($field) . '_FILTER%%', $html);
		}
		
		
		public static function getTimePeriodWidgetHtml($values)
		{
			$parser = new scTemplateParser();
			$parser->readTemplate('admin/widgets/time_period_widget.tmpl');
			$parser->assignArray(array(
				'%%PARAM_DATE_FROM%%' => miI18N::htmlEscape($values[scSqlFilterDatePeriod::PARAM_DATE_FROM]),
				'%%PARAM_DATE_TO%%' => miI18N::htmlEscape($values[scSqlFilterDatePeriod::PARAM_DATE_TO]),
				'%%DATERANGEPICKERINPUT%%' => self::getFormatedDate($values[scSqlFilterDatePeriod::PARAM_DATE_FROM]) . ' - ' . self::getFormatedDate($values[scSqlFilterDatePeriod::PARAM_DATE_TO]),
			));
			$html = $parser->templateParse();
			return $html;
		}
		
		protected static function getFormatedDate($date)
		{
			list($day, $month, $year) = explode('/', $date);
			return date('M d, Y', mktime(0,0,0,$month, $day, $year));
		}
		
		/**
		 * Creates checkbox filter
		 * The filter is applied to the $table
		 */
		public static function createCheckboxFilter(miDefaultView $view, miTable $table, $field, $values, $condition)
		{			
			// Make sure that the filter is applied
			if (!isset($_REQUEST[$field . 'Filter'])) {
				$filterObj = $view->getTableFilterObj();
				$filterObj->addAdditionalFilter($field, '', $condition);
			}
			
			$activeValues = miGetParamDefault($field . 'Filter', false);
			$allChecked = ($activeValues === false);
			$activeValues = explode(',', $activeValues);
				
			$html = self::createCheckboxTable($field, $values, $activeValues, $allChecked);
			
			$html .= '<input type="hidden" name="' . miI18N::htmlEscape($field) . 'Condition" value="' . miI18N::htmlEscape($condition) . '" />';
			$html .= '<input type="hidden" name="' . $field . 'Filter" value="1" />';
			
			$table->assign('%%HTML_' . strtoupper($field) . '_FILTER%%', $html);
		}
		
		public static function createCheckboxTable($field, $values, $activeValues, $allChecked = false, $cells = 4)
		{
			$count = 1;
			$html = '<table><tr>';
			foreach ($values as $key => $value) {
				$status = (in_array($key, $activeValues) || ($allChecked && $key != scOrderStatus::ORDER_STATUS_UNFINISHED) ? 'checked="checked"' : '');
				$name = miI18N::htmlEscape($field);
				$html .= '<td nowrap="nowrap"><input name="' . $name . '[]" id="' . $name . $count . '" type="checkbox" value="' . miI18N::htmlEscape($key) . '" ' . $status . ' /> ';
				$html .= '<label for="' . $name . $count . '">' . miI18N::htmlEscape($value) .'</label></td>';
				if (($count % $cells == 0) and ($count != count($values)))
					$html .= '</tr><tr>';
				$count++;
			}
			$html .= '</tr></table>';
			return $html;
		}

		public static function formatMoney($amount)
		{
			return number_format($amount, 2);
		}
		
		public static function getCategoriesBreadcrumb($categoryId)
		{
			$sql = 'SELECT B.CategoryID, CategoryName FROM Categories A JOIN Categories B JOIN CategoriesML ON B.CategoryID=CategoriesML.CategoryID' .
				' WHERE A.CategoryID=? AND LanguageID=? AND A.CategoryLeft >= B.CategoryLeft AND A.CategoryRight <= B.CategoryRight' .
				' ORDER BY B.CategoryLeft';
			$categories = miStaticDBUtil::execSelect($sql, array($categoryId, scMultiLanguage::getCurrentLanguageId()));
			
			scFriendlyUrl::initFriendlyUrls($categories, scFriendlyUrl::ROW_TYPE_CATEGORIES);
			
			$categoriesBreadcrumbSection = new miTemplateParserSectionInfo();
			$categoriesBreadcrumbSection->setSectionInfo('categoriesBreadcrumb', count($categories), miUIUtil::templetizeRows(miUIUtil::transposeRows($categories)));
			return $categoriesBreadcrumbSection;
		}
	}
?>