<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scView extends miDefaultView {
		/**
		 * The name of the sort column.
		 * If there is no sort column the value is false
		 * 
		 * @var string
		 */
		protected $_sortColumnName = false;
		
		/**
		 * The display name of domain object
		 * It will be translated and displayed to the user
		 * 
		 * @var string
		 */
		protected $_domainObjectDisplayName = 'Record';	// _t('Record')

		/**
		 * Constructs the summer cart default view
		 */
		public function __construct()
		{
			parent::__construct();
		}
		
		public function initMainPageElements()
		{
			$breadcrumb = new scBreadcrumb();
			$this->addMainPageElements(array('%%HTML_BREADCRUMB%%' => $breadcrumb->getCurrentBreadcrumbHtml()));
			
			if (!($this->getTable() instanceof miTable)) {
				return;
			}
			
			if (($this->getTable()->getRecordset() instanceof scSqlRecordsetML) or ($this->getTable()->getRecordset() instanceof scSqlTreeRecordsetML)) {
				$dataLanguageId = scView::getCurrentDataLanguageId();
				$this->getTable()->getRecordset()->setLanguageId($dataLanguageId);
				
				$action = $this->getController()->getDispatcher()->getActionObj();
				$isListingAction = ($action instanceof miListAction);
				$this->addMLMainPageElements($dataLanguageId, $isListingAction);
			}
		}
		
		/**
		 * The language id for data
		 */
		public static function getCurrentDataLanguageId()
		{
			static $dataLanguageId = false;
			
			if ($dataLanguageId === false) {
				if (miGetParamDefault('DataLanguage', '') != '') {
					$_SESSION['DataLanguage'] = (int)miGetParamDefault('DataLanguage', '');
				}
				
				if (isset($_SESSION['DataLanguage'])) {
					$dataLanguageId = $_SESSION['DataLanguage'];
				}
				
				// Make sure that the data language id is valid
				if (!scMultiLanguage::singleton()->isValidLanguageId($dataLanguageId)) {
					$dataLanguageId = scMultiLanguage::getDefaultLanguageId();
				}
			}
			return $dataLanguageId;
		}
		
		/**
		 * @return scTablePager
		 */
		public function &getTablePagerObj()
		{
			static $pagerObj = null;
			if ($pagerObj == null)
				$pagerObj = new scTablePager($this->_table);
			return $pagerObj;
		}
		
		public function initWebForm(&$form)
		{
			$this->initMainPageElements();
			parent::initWebForm($form);
			
			// Show errors
//			$form->setErrorsHandler(new miWebFormMessageErrorsHandler);
			$form->setErrorsHandler(new scByFieldErrorsHandler);
		}
		
		/**
		 * Sets up sort column handling
		 * 
		 * @param string $sortColumnName
		 */
		public function setUpSortColumn($sortColumnName)
		{
			// By default use the sort order
			if (miGetParamDefault(miTableSorter::PARAM_SORTBY, '') == '')
				miSetParam(miTableSorter::PARAM_SORTBY, $sortColumnName);
				
			// Check if we are sorted by the correct column, and set sorting state
			scTable::setSortingState($sortColumnName == miGetParam(miTableSorter::PARAM_SORTBY) ? 1 : 2);
			
			scSortAction::setSortColumnName($sortColumnName);
			
			$this->_sortColumnName = $sortColumnName;
		}
		
		/**
		 * In the event that there is a sort column, set the sort column value upon exec create
		 */
		public function preProcess($actionObj)
		{
			// Save table filters
			scNavigationUtil::saveNavigationParams(get_class($this));

			parent::preProcess($actionObj);
			
			if (!$this->_sortColumnName)
				return;
			
			if ($actionObj instanceof miExecCreateAction) {
				$sql = 'SELECT MAX(' . $this->_sortColumnName . ') AS N FROM ' . $this->getDomainObject()->getTableName();
				$rows = miStaticDBUtil::execSelect($sql);
				$this->getDomainObject()->set($this->_sortColumnName, $rows[0]['N']+1);
			}
		}
		
		/**
		 * Add miControllerCommand redirect to display error message using javascript
		 * 
		 * @param string $msg the error message
		 */
		public function addAjaxErrorRedirect($msg)
		{
			$loc = $this->getScriptPath() . '?' . $this->getRedirectParams($msg, miMessage::MSG_TYPE_ERROR);
			$html = '<script>window.location.href="' . $loc . '";</script>';
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->addControllerCommand($cmd);
		}
		
		/**
		 * Adds refresh list controller command
		 * The command is redirect to list with "load=table" additional param
		 * 
		 * @access public
		 * @param string $msg message to display on the list page
		 * @param int $msgType the type of the message (optional)
		 */
		public function addAjaxRefreshListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = $this->getScriptPath() . '?load=table&' . $this->getRedirectParams($msg, miMessage::MSG_TYPE_INFO);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
		
		/**
		 * Return customized message
		 * Uses the domain object display name
		 * 
		 * @param string $msgCode
		 * @return string the message
		 */
		public function getMessage($msgCode)
		{
			switch ($msgCode) {
				case 'MI_RECORD_CREATED_SUCCESSFULLY_MSG':
					$msg = _t('%s successfully created.');
					break;
				case 'MI_RECORD_UPDATED_SUCCESSFULLY_MSG':
					$msg = _t('%s successfully updated.');
					break;
				case 'MI_RECORD_DELETED_SUCCESSFULLY_MSG':
					$msg = _t('%s successfully deleted.');
					break;

				// Unknown message
				default:
					return miI18N::getSystemMessage($msgCode);
			}
			
			return sprintf($msg, $this->getDomainObjectDisplayName());
		}
		
		/**
		 * Returns the domain object display name
		 * 
		 * @return string
		 */
		public function getDomainObjectDisplayName()
		{
			return scInterfaceTexts::getTranslatedText($this->_domainObjectDisplayName);
		}
		
		protected function addMLMainPageElements($dataLanguageId, $isListingAction)
		{
			$this->addMainPageElements(array(
				'%%HTML_DATA_LANGUAGES%%' => $this->getDataLanguagesHtml($isListingAction),
				'%%LANGUAGEID%%' => miI18N::htmlEscape($dataLanguageId)
			));
		}
		
		protected function getDataLanguagesHtml($isListingAction)
		{
			$activeLanguages = scMultiLanguage::singleton()->getActiveLanguages();
			
			// Init the languages icons
			$languageImageSections = array();
			foreach ($activeLanguages as &$activeLanguagesRow) {
				if (!empty($activeLanguagesRow['LanguageImage']) and file_exists(scPaths::getStoreRootDir() . $activeLanguagesRow['LanguageImage'])) {
					$languageImageSections[] = new miTemplateParserSectionInfo('image');
				} else
					$languageImageSections[] = new miTemplateParserSectionInfo('image', 0);
				
				if (!empty($activeLanguagesRow['LanguageImage']))
					$activeLanguagesRow['LanguageImage'] = scPaths::getStoreWebPath() . $activeLanguagesRow['LanguageImage'];
			}
			
			$section = new miTemplateParserSectionInfo();
			$section->setSectionInfo('languages', count($activeLanguages), miUIUtil::templetizeRows(miUIUtil::transposeRows($activeLanguages)));
			$section->addSubsectionsArray('image', $languageImageSections);
			
			$showAllLanguagesSection = new miTemplateParserSectionInfo('showAllLanguages', $isListingAction ? 0 : 1);
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/multilanguage/data_languages.tmpl');
			$t->setSectionInfos(array($section, $showAllLanguagesSection));
			return $t->templateParse();
		}
		
		/**
		 * @param miTable $table
		 */
		protected function addTableFeatures($table)
		{
			$this->initMainPageElements();
			new scTableSorter($table);
			new miMessage($table);
			// Init the table pager and table filter
			$this->getTablePagerObj();
			$this->getTableFilterObj();
		}
		
		/**
		 * @return scDomainObject
		 */
		protected function createDomainObject()
		{
			return new scDomainObject($this->_record);
		}
	}
?>