<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCategorySelect extends miSelectWidget {
		const CHAR_VERTICAL_LINE = "\342\224\202\302\240\302\240";
//		const CHAR_VERTICAL_LINE =  "\342\224\212\302\240\302\240";	// Use dashed vertical line since it gets misaligned otherwise
		const CHAR_MID_JUNCTION = "\342\224\234\342\224\200";
		const CHAR_TOP_JUNCTION = "\342\224\224\342\224\200";
		const CHAR_LINE = "\342\224\200";
		const CHAR_COPYRIGHT = "\302\251";
		const CHAR_EMPTY = "\302\240\302\240";
		
		/**
		 * Sets the category tree
		 * Excludes $excludeCategoryId and all subcategories
		 * If $excludeCategoryId is 0 the whole tree is listed
		 * 
		 * @param int $excludeCategoryId a category id to exclude from the tree
		 */
		public function setCategoryTree($excludeCategoryId, $noTopLevel = false)
		{
			$this->setOptions($this->getCategoriesTree($excludeCategoryId, $noTopLevel));
		}
		
		/**
		 * Returns array with categories with the subcategories indented
		 * The key is the category id, the value is the category name
		 * 
		 * The exclude category id subtree is not returned in the array
		 * 
		 * @param $excludeCategoryId the category id whose subtree won't be returned
		 * @param bool $noTopLevel if true top level element won't be returned
		 * @return array
		 */
		function getCategoriesTree($excludeCategoryId, $noTopLevel = false)
	   	{
			$categories = new scSqlTreeRecordsetML('Categories', 'CategoryID', 'CategoryParentID');
			$categories->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$categories->setMode(scSqlTreeRecordset::MODE_PREORDER);
			$categories->setOrder('CategorySort');
			$rows = $categories->getAllRecords();
			
			$treeUtil = new scTreeUtil;
			$rows = $treeUtil->removeSubTree($rows, $excludeCategoryId, 'CategoryID', true);
			$categoriessArray = array();
			if (!$noTopLevel)
	   			$categoriessArray[0] = _t('-- Top Level --');

	   		$stack = array($this->getDirectSubnodesCount($rows, 0, -1));
	   		$currentNodeLevel = -1;
	   		
	   		foreach ($rows as $key => $row) {
	   			
	   			$start = '';
	   			for ($i=0; $i < $row['NodeLevel']; $i++) {
	   				if ($stack[$i] > 0)
	   					$start .= self::CHAR_VERTICAL_LINE . self::CHAR_EMPTY;
	   				else
	   					$start .= self::CHAR_EMPTY . self::CHAR_EMPTY . self::CHAR_EMPTY;
	   			}
	   			if ($stack[$row['NodeLevel']] > 1)
	   				$start .= self::CHAR_MID_JUNCTION;
	   			else
	   				$start .= self::CHAR_TOP_JUNCTION;
	   			$start .= '▢' . self::CHAR_EMPTY;
	   				
	   			// Going deep
	   			if ($row['NodeLevel'] != $currentNodeLevel) {
	   				$stack[$row['NodeLevel']+1] = $this->getDirectSubnodesCount($rows, $key+1, $row['NodeLevel']);
	   			}
	   			while ($row['NodeLevel'] < $currentNodeLevel) {
	   				array_pop($stack);
	   				$currentNodeLevel--;
	   			}
	   			$currentNodeLevel = $row['NodeLevel'];
	   			$stack[$row['NodeLevel']]--;

				$categoriessArray[$row['CategoryID']] = $start . $row['CategoryName'];
	   		}
	   		
	    	return $categoriessArray;
	    }
	    
	    protected function getDirectSubnodesCount($rows, $startKey, $startLevel)
	    {
	    	$subnodes = 0;
	    	for ($i = $startKey; isset($rows[$i]); $i++) {
	    		if ($rows[$i]['NodeLevel'] == $startLevel+1)
	    			$subnodes++;
	    		
	    		if ($rows[$i]['NodeLevel'] <= $startLevel)
	    			break;
	    	}
	    	return $subnodes;
	    }
	}
?>