<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Checkout payment method select widget
	 * The option value is the ModuleID of the payment module (not PaymentModuleID)
	 */
	class scCheckoutPaymentMethodWidget extends miRadioWidget {
		public function __construct(miWebForm &$webForm, $properties)
		{
			parent::__construct($webForm, $properties);
			
			$paymentModules = self::getActivePaymentModules();
			
			$options = array();
			foreach ($paymentModules as $moduleRow) {
				$options[$moduleRow['PaymentModuleID']] = $moduleRow['PaymentModuleDisplayName'];
			}
			$this->setRadioButtons($options);
		}

		public function getEditableControl()
		{
			$value = $this->getValue();
			
			$widget = '';
			$idIndex = 0;
			foreach ($this->_radioButtons as $key => $option) {
				$checked = ($value == $key) ? ' checked="checked"' : '';
				$id = $this->_fieldName . '--' . ($idIndex++);
				$widget .= '<label for="' . $id . '">' . '<input type="radio" name="' . $this->_fieldName . '" id="' . $id . '" value="' . miI18N::htmlEscape($key) . '"' . $checked . '/>&nbsp;&nbsp;' .
					miI18N::htmlEscape($option) . '</label>';
			}
			return array($this->_fieldName => $widget);
		}
		
		public static function getActivePaymentModules()
		{
			$paymentModulesRs = new miSqlRecordset('PaymentModules');
			$paymentModulesRs->addJoinCondition('INNER', 'PaymentModulesML', 'USING(PaymentModuleID)');
			$paymentModulesRs->addFilter(new miSqlFilterEqual('PaymentModuleIsActive', 1));
			$paymentModulesRs->addFilter(new miSqlFilterEqual('LanguageID', scMultiLanguage::getCurrentLanguageId()));
			$paymentModulesRs->setOrder('PaymentModuleSort', 'ASC');
			$paymentModulesRows = $paymentModulesRs->getAllRecords();
			
			$paymentModules = array();
			foreach ($paymentModulesRows as $paymentModuleRow) {
				try {
					$module = scModulesFactory::getImplementation($paymentModuleRow['ModuleID'], 'IPaymentModule');
				} catch (miException $exception) {
				}
				
				$paymentModules[$paymentModuleRow['ModuleID']] = array(
					'Module' => $module,
					'PaymentModuleID' => $paymentModuleRow['PaymentModuleID'],
					'PaymentModuleName' => $paymentModuleRow['PaymentModuleName'],
					'PaymentModuleDisplayName' => $paymentModuleRow['PaymentModuleDisplayName']
				);
			}
			return $paymentModules;
		}
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			$value = miGetParamDefault($this->_fieldName, '');
			if (strlen($value) == 0) {
				$errors->addError($this->_fieldName, _t('There are no available billing methods for your order!'));
			}
			if (!array_key_exists($value, $this->_radioButtons)) {
				$errors->addError($this->_fieldName, _t('Unknown billing method selected.'));
			}
		}
	}
?>