<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Two multiple select fields to choose a list of values from the first one into the second one
	 */
	class scDoubleListBoxWidget extends miBaseSelectWidget {
		
		const MODE_MOVE_VALUES = 1; // Move the values from the first to the second control
		const MODE_COPY_VALUES = 2; // Copy the values from the first to the second control
		
		/**
         * @var mode of operation of the control
         * @access protected
         */
        protected $_mode = self::MODE_MOVE_VALUES;
        
		/**
         * @var custom javascript function to be called when sending values from left to right select
         * @access protected
         */
        protected $_leftToRightJsHandler = null;
        
		/**
         * @var custom javascript function to be called when sending values from right to left select
         * @access protected
         */
        protected $_rightToLeftJsHandler = null;
        
        
        protected $_selectedItemsTextFilter = null;
        
 
        /**
         * Set the operation mode of the control
         */
        public function setMode($mode)
        {
        	$this->_mode = $mode;
        }
        
        public function setLeftToRightJsHandler($jsHandlerFunctionName)
        {
        	$this->_leftToRightJsHandler = $jsHandlerFunctionName;
        }
        
        public function setRightToLeftJsHandler($jsHandlerFunctionName)
        {
        	$this->_rightToLeftJsHandler = $jsHandlerFunctionName;
        }
        
        public function setSelectedItemsTextFilter($filterFunction)
        {
        	$this->_selectedItemsTextFilter = $filterFunction;
        }
        
		/**
		 * Process the form submissions and returns the field data
		 */
		public function getData()
		{
			return array($this->_fieldName => miGetParamDefault($this->_fieldName, array()));
		}
		
		/**
		 * Returns the text field contents to be displayed
		 */
		public function getEditableControl()
		{
			$selectedItems = $this->_webForm->getFormData($this->_fieldName);
			if ($selectedItems == null)
				$selectedItems = array();
			
			$options = '';
			$optionsSelected = '';
			foreach ($this->_options as $optionValue => $optionText) {
				
				$optionTextFiltered = $optionText;
				
				if ($this->_selectedItemsTextFilter && is_callable($this->_selectedItemsTextFilter)) {
					$optionTextFiltered = call_user_func($this->_selectedItemsTextFilter, $optionText);
				}
				
				$option = '<option value="' . miI18N::htmlEscape($optionValue) . '">' . miI18N::htmlEscape($optionText) . '</option>';
				$optionFiltered = '<option value="' . miI18N::htmlEscape($optionValue) . '">' . miI18N::htmlEscape($optionTextFiltered) . '</option>';
				
				if (in_array($optionValue, $selectedItems)) {					
					$optionsSelected .= $optionFiltered;
					if ($this->_mode == self::MODE_COPY_VALUES) {
						// don't remove the selected values from the original list
						$options .= $option;	
					}
				} else {
					$options .= $option;
				}
			}
			
			if ($this->_mode == self::MODE_COPY_VALUES) {
				$leftToRightFunction = 'SC.DMS.copySelected';
				$rightToLeftFunction = 'SC.DMS.removeSelected';
			} else {
				$leftToRightFunction = $rightToLeftFunction = 'SC.DMS.moveSelected';
			} 
			
			// If a custom handler functions are set, override the mode
			if ($this->_leftToRightJsHandler) {
				$leftToRightFunction = $this->_leftToRightJsHandler;
			}
			if ($this->_rightToLeftJsHandler) {
				$rightToLeftFunction = $this->_rightToLeftJsHandler;
			}
			
			$html = '<table><tr><td>';
			if (isset($this->_properties['leftCaption']))
				$html .= scInterfaceTexts::getTranslatedText($this->_properties['leftCaption']) . '<br/>';
			$html .= '<select multiple="multiple" id="' . $this->_fieldName . 'Id" size="10">';
			$html .= $options . '</select></td><td class="selectArrows">';
			$html .= '<a href="javascript: void(0);" id="'.$this->_fieldName.'LTR">&raquo;</a><br /><br />';
			$html .= '<a href="javascript: void(0);" id="'.$this->_fieldName.'RTL">&laquo;</a></td><td>';
			if (isset($this->_properties['rightCaption']))
				$html .= scInterfaceTexts::getTranslatedText($this->_properties['rightCaption']) . '<br/>';
			$html .= '<select multiple="multiple" name="' . $this->_fieldName . '[]" id="' . $this->_fieldName . 'SelectedId" size="10">';
			$html .= $optionsSelected . '</select></td></tr></table>';
			$html .= '<script type="text/javascript">';
			$html .= '$(function() {';
			$html .= '$("#'.$this->_fieldName.'LTR").click(function(){'.$leftToRightFunction.'("' . $this->_fieldName . 'Id", "' . $this->_fieldName . 'SelectedId"); return false;});';
			$html .= '$("#'.$this->_fieldName.'RTL").click(function(){'.$rightToLeftFunction.'("' . $this->_fieldName . 'SelectedId", "' . $this->_fieldName . 'Id"); return false;});';
			$html .= '});';
			$html .= '</script>';
			
			return array($this->_fieldName => $html);
		}
	}
?>