<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scImageUploadWidget extends miBaseWidget {
		protected $_hasApplyButton = false;
		
		/**
		 * Used to indicate if the popup box should have "apply now" button
		 * 
		 * @param bool $hasApplyButton
		 * @return void
		 */
		public function setApplyButton($hasApplyButton = true)
		{
			$this->_hasApplyButton = $hasApplyButton;
		}
			
		public function getControl()
		{
			return $this->getEditableControl();
		}
		
		public function getEditableControl()
		{
			$image = $this->_webForm->getFormData($this->_fieldName);
			
			$imageWebPath = $image['imageWebPath'];
			$imageWebPathForSrc = $imageWebPath;
			if (!is_file(scPaths::getStoreRootDir() . $imageWebPath)) {
				$imageWebPathForSrc = scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl() . 'admin/images/image-not-available/image_not_available.jpg';
				$imageWebPath = null;
			}
			
			$validThumb = true;
			
			$thumbWebPath = $image['thumbWebPath'];
			if (!is_file(scPaths::getStoreRootDir() . $thumbWebPath)) {
				$thumbWebPath = scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl() . 'admin/images/image-not-available/image_not_available.jpg';
				$validThumb = false;
			}
			
			$name = $this->_fieldName;
			$hasApplyButton = $this->_hasApplyButton ? 'true' : 'false';
			$hasThumb = array_key_exists('noThumb', $this->_properties) ? !$this->_properties['noThumb'] : true;
			$resource = $this->_properties['resource'];
			$imageMaxWidth = (empty($this->_properties['imageMaxWidthSetting']) ? 0 : scSettings::getSettingValue($this->_properties['imageMaxWidthSetting']));
			$imageMaxHeight = (empty($this->_properties['imageMaxHeightSetting']) ? 0 : scSettings::getSettingValue($this->_properties['imageMaxHeightSetting']));
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/widgets/image_upload_widget.tmpl');
			$t->assignArray(array(
				'%%NAME%%' => $name,
				'%%IMAGE_MAX_WIDTH%%' => $imageMaxWidth,
				'%%IMAGE_MAX_HEIGHT%%' => $imageMaxHeight,
				'%%HAS_THUMB%%' => ($hasThumb ? 'true' : 'false'),
				'%%HASAPPLYBUTTON%%' => $hasApplyButton,
				'%%RESOURCE%%' => $resource,
				'%%SKIN_URL%%' => scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl(),
				'%%LINK_URL%%' => scUrlUtil::convertUrl($imageWebPath, scUrlUtil::URL_TYPE_DOCROOT),
			
				'%%IMAGEWEBPATHFORSRC%%' => scUrlUtil::convertUrl($imageWebPathForSrc, scUrlUtil::URL_TYPE_DOCROOT),
				'%%IMAGEWEBPATH%%' => $imageWebPath,
				'%%IMAGEFILENAME%%' => $image['imageFileName'],
				'%%IMAGEWEBPATH%%' => $imageWebPath,
				'%%THUMBFILENAME%%' => $image['thumbFileName'],
				'%%THUMBWEBPATH%%' => $thumbWebPath,
			));
			
			// hide the delete button only if specifically requested
			$deleteButtonSection = new miTemplateParserSectionInfo('deleteButton', 1);
			if (isset($this->_properties['deleteButton']) &&  $this->_properties['deleteButton'] === false) {
				$deleteButtonSection->setIterationsCount(0);
			}
			$t->addSectionInfo($deleteButtonSection);
			
			$dimensionsSection = new miTemplateParserSectionInfo('dimensions', 1);
			if (empty($imageMaxWidth) && empty($imageMaxHeight)) {
				$dimensionsSection->setIterationsCount(0);
			}
			$t->addSectionInfo($dimensionsSection);
			
			return array($this->_fieldName => $t->templateParse());
		}
		
		function getData()
		{
			$uploadedImage = array(
				'imageFileName' => miGetParamDefault('ImageFileName-' . $this->_fieldName, ''),
				'imageWebPath' => miGetParamDefault('ImageWebPath-' . $this->_fieldName, ''),
				'thumbFileName' => miGetParamDefault('ThumbFileName-' . $this->_fieldName, ''),
				'thumbWebPath' => miGetParamDefault('ThumbWebPath-' . $this->_fieldName, ''),
				'delete' => miGetParamDefault('ImageDelete-' . $this->_fieldName, 'false') == 'true' ? true : false,
				'resource' => $this->_properties['resource']
			);
			
			return array($this->_fieldName => $uploadedImage);
		}
	}
?>