<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scLocalOptionGroupWidget extends scDynRowsWidget {
		
		const INPUT_TYPE_TEXT = 'text';
		const INPUT_TYPE_HIDDEN = 'hidden';
		
		/**
		 * Returns the field contents to be displayed
		 * 
		 * @return array
		 */
		public function getEditableControl()
		{
			$this->setWebFormData();
			$this->createFieldObjects();
			
			$fieldValues = $this->getFieldValues();
			$itemsSectionInfo = $this->getItemsSectionInfo();
			foreach ($itemsSectionInfo as $itemsSections) {
				$count = count($itemsSections->_values['%%HTML_STYLE%%']);
				$fieldValues['%%STYLEVALUES%%'][] = $count > 1 ? '' : 'style="display:none"';
				$fieldValues['%%ROWVALUESCOUNT%%'][] = $count - 1;
			}
			$sectionInfo = $this->getSectionInfo($fieldValues);
			
			$sectionInfo->addSubsectionsArray($this->_fieldName.'_Items', $itemsSectionInfo);
			return array($this->_fieldName => $this->parseTemplate($sectionInfo));
		}
		
		public function getData()
		{
			$result = parent::getData();
			foreach ($result['LocalOptionGroups'] as $key => &$formItem) {
				$formItem['LocalOptions'] = $this->getAllItemsValues($key+1);
			}
			return $result;
		}
		
		protected function initField(miBaseWidget $field, $fieldName)
		{
			if (!($field instanceof miBaseSelectWidget)) {
				return;
			}
			
			switch ($fieldName) {
				case 'LocalOptionGroupType':
					$field->setOptions(scProductOptions::getGroupTypes());
					break;
			}
		}
		
		protected function getControlHtml($fieldName, $fieldData, $htmlClass = '', $inputType = self::INPUT_TYPE_TEXT)
		{
			return '<input type="' . $inputType . '" name="' . miI18N::htmlEscape($fieldName) . 
				'" value="' . miI18N::htmlEscape($fieldData) . '" ' . 
				(!empty($htmlClass) ? ' class="' . miI18N::htmlEscape($htmlClass) . '"' : '') . ' />';
		}
		
		protected function getMLControlHtml($rowIndex, $rowValueNum = '', $data = array())
		{
			$activeLanguages = scMultiLanguage::singleton()->getActiveLanguages();
			$widget = '';
			
			$fieldName = $this->_fieldName . '::' . $rowIndex . '::LocalOptionName--' . $rowValueNum . '--';
			
			foreach ($activeLanguages as $activeLanguage) {
				$languageId = $activeLanguage['LanguageID'];
				$fieldData = isset($data[$languageId]) ? $data[$languageId] : '';
				
				$widget .= '<span id="span' . miI18N::htmlEscape($fieldName . '-' . $languageId) . '" style="display:none">';
				$widget .= '<a href="javascript: SC.ML.nextLanguage(' . $languageId . ');" name="changeLanguage">';
				if (!empty($activeLanguage['LanguageImage']) and file_exists(scPaths::getStoreRootDir() . $activeLanguage['LanguageImage'])) {
					$widget .= '<img src="' . scPaths::getStoreWebPath() . $activeLanguage['LanguageImage'] . '" width="' . $activeLanguage['LanguageImageWidth'] . '" height="' . $activeLanguage['LanguageImageHeight'] . '"'.
						' alt="' . miI18N::htmlEscape($activeLanguage['LanguageName']) . '" title="' . miI18N::htmlEscape($activeLanguage['LanguageName']) . '"/>';
				} else {
					$widget .= miI18N::htmlEscape($activeLanguage['LanguageName']);
				}
				$widget .= '</a>';
				$widget .= $this->getControlHtml($fieldName . '[' . $languageId . ']', $fieldData);
				$widget .= '</span>';
			}
			
			$widget .= '<script type="text/javascript">SC.ML.registerMultilanguageControl("' . miI18N::htmlEscape($fieldName). '")</script>';
			return $widget;
		}
		
		/**
		 * Generates a list of radio buttons with labels.
		 * Parameters are the same as those of miUIUtil::getConstDropdown()
		 *
		 * @param string $name
		 * @param mixed $selected
		 * @param array $optionsArray
		 * @return unknown
		 */
		protected function getRadiosHtml($name, $selected = null, $optionsArray)
		{
			if (is_null($selected)) {
				$optionKeys = array_keys($optionsArray);
				$selected = reset($optionKeys);
				unset($optionKeys);
			}
			$selected = (string)$selected;
			$html = '';
			foreach ($optionsArray as $key => $value) {
				$html .= '<label><input type="radio" name="' . $name .'" id="' . $name . '" ' .
					'value="' . miI18N::htmlEscape($key) . '"' . ((string)$key === $selected ? ' checked="checked"' : '') . ' />' . miI18N::htmlEscape($value) . '</label> ';
			}
			return $html;
		}
		
		protected function getCheckboxHtml($name, $checked = false)
		{
			return '<input type="checkbox" name="' . $name . '" ' . ($checked ? 'checked="checked"' : '') . ' value="1" />';
		}
		
		protected function getItemsSectionInfo()
		{
			$itemsSections = array();
			
			$radioPriceModTypes = array(
				scProductOptions::PRODUCT_OPTION_TYPE_AMOUNT => _t('Amount'),
				scProductOptions::PRODUCT_OPTION_TYPE_PERCENT => _t('% of product price'),
			);
			
			$innerRowFieldName = $this->_fieldName . '::%s::%s--%s--';
			
			foreach ($this->_rsFormFields as $rowIndex => $rowIndexFormFields) {
				$itemsValue = array();
				$priceModValueFieldName = sprintf($innerRowFieldName, $rowIndex, 'LocalOptionPriceModValue', '');
				$priceModTypeFieldName = sprintf($innerRowFieldName, $rowIndex, 'LocalOptionPriceModType', '');
				$requiredFieldName = sprintf($innerRowFieldName, $rowIndex, 'LocalOptionRequired', '');
				$idFieldName = sprintf($innerRowFieldName, $rowIndex, 'LocalOptionID', '');
				
				$itemsValue[0] = array(
					'ROWVALUENUM' => '',
					'HTML_STYLE' => 'style="display:none"',
					'HTML_LOCALOPTIONID' => $this->getControlHtml($idFieldName, '', '', self::INPUT_TYPE_HIDDEN),
					'HTML_LOCALOPTIONNAME' => $this->getMLControlHtml($rowIndex),
					'HTML_LOCALOPTIONPRICEMODVALUE' => $this->getControlHtml($priceModValueFieldName, '', 'priceModValue'),
					'HTML_LOCALOPTIONPRICEMODTYPE' => $this->getRadiosHtml($priceModTypeFieldName, null, $radioPriceModTypes),
					'HTML_LOCALOPTIONREQUIRED' => $this->getCheckboxHtml($requiredFieldName, false),
				);
				
				if (intval($rowIndex) > 0) {
					$values = $this->_webForm->getFormData('LocalOptions::' . $rowIndex . '::');
					$values = array_values($values);
					foreach ($values as $key => $value) {
						$priceModValueFieldName = sprintf($innerRowFieldName, $rowIndex, 'LocalOptionPriceModValue', ($key+1));
						$priceModTypeFieldName = sprintf($innerRowFieldName, $rowIndex, 'LocalOptionPriceModType', ($key+1));
						$requiredFieldName = sprintf($innerRowFieldName, $rowIndex, 'LocalOptionRequired', ($key+1));
						$idFieldName = sprintf($innerRowFieldName, $rowIndex, 'LocalOptionID', ($key+1));
						
						$itemsValue[] = array(
							'ROWVALUENUM' => $key+1,
							'HTML_STYLE' => '',
							'HTML_LOCALOPTIONID' => $this->getControlHtml($idFieldName, $value['LocalOptionID'], '', self::INPUT_TYPE_HIDDEN),
							'HTML_LOCALOPTIONNAME' => $this->getMLControlHtml($rowIndex, $key+1, $value['LocalOptionName']),
							'HTML_LOCALOPTIONPRICEMODVALUE' => $this->getControlHtml($priceModValueFieldName, $value['LocalOptionPriceModValue'], 'priceModValue'),
							'HTML_LOCALOPTIONPRICEMODTYPE' => $this->getRadiosHtml($priceModTypeFieldName, $value['LocalOptionPriceModType'], $radioPriceModTypes),
							'HTML_LOCALOPTIONREQUIRED' => $this->getCheckboxHtml($requiredFieldName, $value['LocalOptionRequired']),
						);
					}
				}
				// Set items section
				$itemsSections[] = new miTemplateParserSectionInfo($this->_fieldName.'_Items', count($itemsValue), miUIUtil::templetizeRows(miUIUtil::transposeRows($itemsValue)));
			}
			return $itemsSections;
		}
		
		protected function getAllItemsValues($rowIndex)
		{
			$rowIndexFormFields = $this->_rsFormFields[$rowIndex];
			
			$rowCount = miGetParamDefault('LocalOptionGroups::' . $rowIndex . '::__rowCount', '');
			if ($rowCount == '')
				return array();
			
			$itemValues = array();
			for ($i = 1; $i <= $rowCount; $i++) {
				$prefix = 'LocalOptionGroups::' . $rowIndex . '::';
				$itemValues[] = array(
					'LocalOptionID' => miGetParamDefault($prefix . 'LocalOptionID--' . $i . '--', ''),
					'LocalOptionName' => miGetParamDefault($prefix . 'LocalOptionName--' . $i . '--', ''),
					'LocalOptionPriceModType' => miGetParamDefault($prefix . 'LocalOptionPriceModType--' . $i . '--', ''),
					'LocalOptionPriceModValue' => miGetParamDefault($prefix . 'LocalOptionPriceModValue--' . $i . '--', ''),
					'LocalOptionRequired' => (miGetParamDefault($prefix . 'LocalOptionRequired--' . $i . '--', '') == '' ? 0 : 1),
				);
			}
			
			return $itemValues;
		}
	}
?>