<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Widget to select predefined menu images
	 */
	class scMenuItemPredefinedImagesWidget extends miRadioWidget {
		const MENU_ITEMS_IMAGES = 'customer/images/menuitems/';
		
		public function __construct(miWebForm $webForm, $properties = array())
		{
			parent::__construct($webForm, $properties);
			
			$this->setRadioButtons($this->getPredefinedImages(), null);
		}
		
		public function getEditableControl()
		{
			$value = $this->getValue();
			
			$menuImagesUrl = scSkins::singleton()->getSkinsRoot(false) . 'skeleton/' . self::MENU_ITEMS_IMAGES;
			
			$widget = '';
			foreach ($this->_radioButtons as $key => $option) {
				$checked = ($value == $key) ? ' checked="checked"' : '';
				
				$widget .= '<span><label><input type="radio" name="' . $this->_fieldName . '" value="' . $key . '"' . $checked . ' />';
				if ($key == '')
					$widget .= miI18N::htmlEscape($option);
				else
					$widget .= '<img src="../' . $menuImagesUrl . miI18N::htmlEscape($option) . '" alt=""/>';
				$widget .= '</label></span>';
			}
			return array($this->_fieldName => $widget);
		}
		
		protected function getPredefinedImages()
		{
			// Set the menu item images
			$menuItemImagesPath = scSkins::singleton()->getSkinsRoot(true) . 'skeleton/' . self::MENU_ITEMS_IMAGES;
			if (!is_dir($menuItemImagesPath)) {
				return array();
			}
			$menuItemDirImages = scandir($menuItemImagesPath);
			
			$images = array();
			foreach ($menuItemDirImages as $imageFilename) {
				if (in_array(strtolower(substr($imageFilename, -4)), array('.jpg', '.png', '.gif')))
					$images[$imageFilename] = $imageFilename;
			}
			
			return $images;
		}
	}
	
	/**
	 * Widget to select menu item image
	 * List all images with a radio button for each image
	 */
	class scMenuItemImageWidget extends miBaseWidget {
		/**
		 * @var scMenuItemPredefinedImagesWidget $_predefinedImagesWidget
		 */
		protected $_predefinedImagesWidget;
		
		/**
		 * @var array $_imageUploadProperties
		 */
		protected $_imageUploadProperties = array(
			'resource' => scPaths::RESOURCE_MENU_ITEM_IMAGE,
			'noThumb' => true
		);
		
		/**
		 * @var scImageUploadWidget $_customImageWidget
		 */
		protected $_customImageWidget;
		
		/**
		 * @var miRadioWidget $_imageTypeSelectorWidget
		 */
		protected $_imageTypeSelectorWidget;

		/**
		 * The widget constructor
		 */
		public function __construct(miWebForm $webForm, $properties = array())
		{
			parent::__construct($webForm, $properties);
			
			$this->_predefinedImagesWidget = new scMenuItemPredefinedImagesWidget($webForm);
			$this->_customImageWidget = new scImageUploadWidget($webForm, $this->_imageUploadProperties);
			$this->_imageTypeSelectorWidget = new miRadioWidget($webForm);
			
			$this->_imageTypeSelectorWidget->setRadioButtons(scMenuItemImageTypes::getImageTypes(), scMenuItemImageTypes::TYPE_NO_IMAGE);
		}
		
		/**
		 * Set the appropriate field names
		 */
		public function setFieldName($fieldName)
		{
			parent::setFieldName($fieldName);
			
			$this->_imageTypeSelectorWidget->setFieldName($this->_fieldName . 'Type');
			$this->_predefinedImagesWidget->setFieldName($this->_fieldName . 'Predefined');
			$this->_customImageWidget->setFieldName($this->_fieldName . 'Custom');
		}
		
		/**
		 * Returns the prepared widget
		 * 
		 * @return array
		 */
		public function getEditableControl()
		{
			// Recreate the form data (appropriate for display)
			$imageData = $this->_webForm->getFormData($this->_fieldName);
			
			$this->_webForm->setFormData($this->_fieldName . 'Type', $imageData[$this->_fieldName . 'Type']);
			switch(intval($imageData[$this->_fieldName . 'Type']))
			{
				case scMenuItemImageTypes::TYPE_PREDEFINED:
					$this->_webForm->setFormData($this->_fieldName . 'Predefined', $imageData[$this->_fieldName]);
					break;
				case scMenuItemImageTypes::TYPE_CUSTOM:
					if (!is_array($imageData[$this->_fieldName])) {
						$imageData[$this->_fieldName] = array(
							'imageWebPath' => $imageData[$this->_fieldName],
							'thumbWebPath' => $imageData[$this->_fieldName],
							'imageFileName' => '',
							'thumbFileName' => ''
						);
					}
					$this->_webForm->setFormData($this->_fieldName . 'Custom', $imageData[$this->_fieldName]);
					break;
			}
			
			return array(
				$this->_fieldName . 'Type' => $this->getImageTypeSelector(),
				$this->_fieldName . 'Predefined' => $this->getPredefinedImageControl(),
				$this->_fieldName . 'Custom' => $this->getCustomImageControl()
			);
		}
		
		/**
		 * Return the widget data to the view mapper
		 */
		public function getData()
		{
			// Collect the appropriate data.
			$data = $this->_imageTypeSelectorWidget->getData();
			$imageType = (int)reset($data);
			
			$imageData = array('Delete' => false);
			
			switch($imageType)
			{
				case scMenuItemImageTypes::TYPE_CUSTOM:
					$data = $this->_customImageWidget->getData();
					$image = reset($data);
					break;
				case scMenuItemImageTypes::TYPE_PREDEFINED:
					$data = $this->_predefinedImagesWidget->getData();
					$image = reset($data);
					break;
				default:
					$image = '';
			}
			
			return array(
				$this->_fieldName => array(
					$this->_fieldName => $image,
					$this->_fieldName . 'Type' => $imageType
				)
			);
		}
		
		/**
		 * Returns the image type selector control
		 * 
		 * @return string
		 */
		protected function getImageTypeSelector()
		{
			$control = $this->_imageTypeSelectorWidget->getEditableControl();
			return reset($control);
		}
		
		/**
		 * Return the widget for selecting predefined images
		 * 
		 * @return string
		 */
		protected function getPredefinedImageControl()
		{
			$control = $this->_predefinedImagesWidget->getEditableControl();
			return reset($control);
		}
		
		/**
		 * Return the widget for uploading custom images
		 * 
		 * @return string
		 */
		protected function getCustomImageControl()
		{
			$control = $this->_customImageWidget->getEditableControl();
			return reset($control);
		}
	}
?>