<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scModuleOptionsWidget extends miBaseWidget {
		protected $_module;
		protected $_moduleId;
		protected $_widgetHtml = null;
		protected $_validators = array();
		protected $_options = null;
		
		public function setupModule($moduleId, $module)
		{
			$this->_module = $module;
			$this->_moduleId = $moduleId;
			
			if (!($this->_module instanceof IModuleOptions)) {
				$this->_widgetHtml = _t('This module has no options.');
				return;
			}
			
			$this->_options = $this->_module->getOptionFields($this->_webForm);
			if ($this->_options == null) {
				$this->_widgetHtml = _t('This module has no options.');
				return;
			}
			$this->_validators = $this->_options->getValidators();
			
		}
		
		public function setupModuleOptions($moduleId, $interfaceName)
		{
			$module = scModulesFactory::getImplementation($moduleId, $interfaceName);
			$this->setupModule($moduleId, $module);
		}
		
		/**
		 * Set the module options values as form data
		 */
		protected function setModuleOptionValues()
		{
			$options = $this->_webForm->getFormData($this->_fieldName);
			foreach ($options as $optionName => $optionValue)
				$this->_webForm->setFormData(scModuleOptions::MODULE_OPTION_PREFIX . $optionName, $optionValue);
		}
		
		/**
		 * 
		 */
		public function getEditableControl()
		{
			if ($this->_widgetHtml == null) {
				$this->setModuleOptionValues();
				if ($this->_options->getTemplateFile() == null)
					throw new miException(_t('Options template file not set.'));
				$this->_widgetHtml = $this->getModuleOptionsHtml($this->_webForm, $this->_moduleId, $this->_options);
			}
			
			return array($this->_fieldName => $this->_widgetHtml);
		}
		
		/**
		 * Retrieves all fields and subfields in the fieldset
		 * 
		 * @param array $fieldset array of fields / fieldsets
		 * @return array array of fields
		 */
		protected function getAllFields($fieldset)
		{
			$fields = array();
			foreach ($fieldset as $field) {
				if ($field instanceof scFieldSet)
					$fields = array_merge($fields, $this->getAllFields($field->getFields()));
				else {
					$field->setFieldName(scModuleOptions::MODULE_OPTION_PREFIX . $field->getFieldName());
					$fields[] = $field;
				}
			}
			return $fields;
		}
		
		protected function getModuleOptionsFromRequest($module)
		{
			if (!($module instanceof IModuleOptions))
				return array();
			
			$options = $module->getOptionFields(new miWebForm(array()));
			if (!$options instanceof scModuleOptionsInfo)
				return array();
			
			$fields = $this->getAllFields($options->getOptionFields());
			
			$moduleOptions = array();
			foreach ($fields as $field) {
				$dataArray = $field->getData();
				foreach ($dataArray as $moduleOptionName => $moduleOptionValue)
					$moduleOptions[substr($moduleOptionName, strlen(scModuleOptions::MODULE_OPTION_PREFIX))] = $moduleOptionValue;
			}
			return $moduleOptions;
		}
		
		
		public function getData()
		{
			$moduleOptions = $this->getModuleOptionsFromRequest($this->_module);
			
			$result = array();
			foreach ($moduleOptions as $moduleOptionName => $moduleOptionValue)
				$result[$moduleOptionName] = $moduleOptionValue;
				
			$forValidation = array();
			foreach ($result as $name => $value)
				$forValidation[scModuleOptions::MODULE_OPTION_PREFIX . $name] = $value;
			
			return array($this->_fieldName => $result) + $forValidation;
		}
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			// Call the validators
			foreach ($this->_validators as $validator) {
				$validator->setFieldName(scModuleOptions::MODULE_OPTION_PREFIX . $validator->getFieldName());
				$validator->validate($errors);
			}
			
			if ($this->_module instanceof IModuleOptions) {
				// Call the validateData() of the fields
				
				$moduleOptionsInfo = $this->_module->getOptionFields($this->_webForm);
				if (!($moduleOptionsInfo instanceof scModuleOptionsInfo)) {
					return;
				}
				
				$fields = $this->getAllFields($moduleOptionsInfo->getOptionFields());
				foreach ($fields as $field) {
					$field->validateData($errors);
				}
			}
		}
		
		/**
		 * Return the module options HTML
		 * 
		 * @param miWebForm $form
		 * @param int $moduleId
		 * @param scModuleOptionsInfo $options
		 * @return string the html
		 */
		protected function getModuleOptionsHtml(&$form, $moduleId, $options)
		{
			$t = new scTemplateParser();
			$t->readTemplate($options->getTemplateFile());
			
			$fields = $this->getAllFields($options->getOptionFields());
			foreach ($fields as $field) {
				$disp = $field->getEditableControl();
				
				$fieldValues = array();
				foreach ($disp as $key => $value) {
					$fieldValues['%%' . strtoupper($key) . '%%'] = $value;
				}
				$t->assignArray($fieldValues);
			}
			
			$mainPageElements = $form->getMainPageElements();
			$t->assignArray($mainPageElements);
			
			return $t->templateParse();
		}
		
		/**
		 * Return the module options HTML
		 * 
		 * @param miWebForm $form
		 * @param int $moduleId
		 * @param scModuleOptionsInfo $options
		 * @return string the html
		 */
/*
		protected function getModuleOptionsHtml(&$form, $moduleId, $options)
		{
			if (!$options instanceof scModuleOptionsInfo)
				return '';
			
			$fieldset = $options->getOptionFields();
			
			$html = '';
			foreach ($fieldset as $field) {
				if ($field instanceof scFieldSet)
					$html .= $this->getFieldsetHtml($field);
				else
					$html .= $this->getFieldHtml($field);
			}
			return $html;
		}

		protected function getFieldsetHtml(scFieldSet $fieldset)
		{
			$optionFields = $fieldset->getFields();
			
			$html = '';
			foreach ($optionFields as $optionField)
				$html .= $this->getFieldHtml($optionField);
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/modules/options_fieldset.tmpl');
			$t->assign('%%DESCRIPTION%%', miI18N::htmlEscape($fieldset->getDescription()));
			$legend = $fieldset->getName() ? '<legend>' . miI18N::htmlEscape($fieldset->getName()) . '</legend>' : '';
			$t->assign('%%LEGEND%%', $legend);
			$t->assign('%%FIELDS%%', $html);
			return $t->templateParse();
		}
		
		protected function getFieldHtml(miBaseWidget $optionField)
		{
			$properties = $optionField->getProperties();
			
			if ($optionField instanceof scDescriptionWidget) {
				$t = new scTemplateParser();
				$t->assign('%%DESCRIPTION%%', $properties['description']);
				$t->readTemplate('admin/modules/options_description.tmpl');
				return $t->templateParse();
			}
			
			$optionField->setFieldName(scModuleOptions::MODULE_OPTION_PREFIX . $optionField->getFieldName());
			
			$controls = array();
			$displayName = isset($properties['displayName']) ? $properties['displayName'] : substr($optionField->getFieldName(), strlen(scModuleOptions::MODULE_OPTION_PREFIX));
			$controls['%%DISPLAYNAME%%'] = miI18N::htmlEscape($displayName);
			$controls['%%PREFIX%%'] = isset($properties['prefix']) ? miI18N::htmlEscape($properties['prefix']) : '';
			$controls['%%SUFFIX%%'] = isset($properties['suffix']) ? miI18N::htmlEscape($properties['suffix']) : '';
			$controls['%%DESCRIPTION%%'] = isset($properties['description']) ? miI18N::htmlEscape($properties['description']) : '';
			$controls['%%FIELDNAME%%'] = miI18N::htmlEscape($optionField->getFieldName());
			
			$fieldControls = $optionField->getEditableControl();
			if (count($fieldControls) > 1)
				throw new miException(_t('Too many control variables.'));
			$controls['%%HTML_CONTROL%%'] = current($fieldControls);
			
			$t = new scTemplateParser();
			$t->assignArray($controls);
			$t->readTemplate('admin/modules/options_field.tmpl');
			return $t->templateParse();
		}
*/
	}
?>