<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scMultipleCMSTextareaWidget extends miBaseWidget {

		protected $_formFields;
		protected $_cmsAreas;

		protected $_cmsAreasTable = 'CMSDynamicPageAreas';
		protected $_cmsAreasTablePrimaryKey = 'CMSDynamicPageAreaID';
		protected $_cmsAreasTableTitleField = 'CMSDynamicPageAreaTitle';
		protected $_cmsAreasTableTextField = 'CMSDynamicPageAreaText';
		protected $_foreignKeyToParent = 'CMSDynamicPageID';
		protected $_foreignKeyValue;
		protected $_widgetTemplate = 'admin/dynamic_pages/dynamic_page_area.tmpl';
		
		/**
		 * Constructs the widget
		 */
		function __construct(miWebForm &$webForm, $properties = array())
		{
			$this->_foreignKeyValue = miGetParam($this->_foreignKeyToParent);
			parent::__construct($webForm, $properties);
		}

		/**
		 * Returns array with template variables to be displayed for non-editable control
		 * 
		 * @access public
		 * @return array
		 */
		public function getControl()
		{
			return array();
		}
		
		/**
		 * Returns array with template variables to be displayed for editable control
		 * 
		 * @access public
		 * @return array
		 */
		public function getEditableControl()
		{
			$formData = $this->_webForm->getFormData($this->_fieldName);

			$this->createFieldObjects(); // Create the textarea fields of the widget
			$numberOfTextAreas = count($this->_formFields);			
			$cmsTextAreasVars = array();
			
			foreach ($this->_formFields as $iIndex => $iFormField) {
				$cmsAreaId = (int) substr($iFormField->getFieldName(), strlen($this->_fieldName)); // extract ID from field name (e.g. 15 from CMSDynamicPageAreas15)				
				
				$dataFieldName = $this->_fieldName . $cmsAreaId;
				// Set CMS areas data to form - the data will be read by the individual widgets
				$this->_webForm->setFormData($dataFieldName, $formData[$cmsAreaId]['value']); // TODO: V1.1 maybe move to view mapper->setDataToWebForm()
				$textAreaControl = $iFormField->getEditableControl();

				$cmsTextAreasVars['%%CMSDYNAMICPAGEAREAID%%'][] = $cmsAreaId;
				$cmsTextAreasVars['%%CMSDYNAMICPAGEAREATITLE%%'][] = $formData[$cmsAreaId]['title'];
				$cmsTextAreasVars['%%CMSDYNAMICPAGEAREATEXT%%'][] = $textAreaControl[$dataFieldName];
			}

			$cmsTextAreas = new miTemplateParserSectionInfo();
			$cmsTextAreas->setSectionInfo('cmsTextareas', $numberOfTextAreas, $cmsTextAreasVars);
			
			$t = new scTemplateParser();
			$t->readTemplate($this->_widgetTemplate);
			$t->setSectionInfos(array($cmsTextAreas));
			$control = $t->templateParse();
			
			return array($this->_fieldName => $control);
		}
		
		/**
		 * Process the form submissions and returns the data
		 * 
		 * @access public
		 * @return array
		 */
		public function getData()
		{
			$this->createFieldObjects();

			$result = array();

			foreach ($this->_formFields as $iIndex => $iFormField) {
				$cmsAreaId = (int) substr($iFormField->getFieldName(), strlen($this->_fieldName)); // extract ID from field name (e.g. 15 from CMSDynamicPageAreas15)

				$fieldData = $iFormField->getData();

				$result[$cmsAreaId] = array(
					'id' => miGetParam($this->_cmsAreasTablePrimaryKey . $cmsAreaId),
					'title' => miGetParam($this->_cmsAreasTableTitleField . $cmsAreaId),
					'value' => $fieldData[$this->_cmsAreasTable . $cmsAreaId], 
				);
			}
			
			return array($this->_fieldName => $result);
		}

		/**
		 * Creates the field objects
		 * TODO: Should use the form data info, instead of reading the database
		 */
		protected function createFieldObjects()
		{
			$record = new miSqlRecordset($this->_cmsAreasTable);
			$record->addFilter(new miSqlFilterEqual($this->_foreignKeyToParent, $this->_foreignKeyValue));
			$this->_cmsAreas = $record->getAllRecords();
			
			$this->_formFields = array();

			foreach ($this->_cmsAreas as $iIndex => $iArea) {
				$properties = array();
				$dataFieldName = $this->_fieldName . $iArea[$this->_cmsAreasTablePrimaryKey];
				$field = new scWebFormWidgetTextAreaML($this->_webForm, $properties);
				$field->setFieldName($dataFieldName);
				$this->_formFields[] = $field;
			}
		}
	}
?>