<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scNewslettersWidget extends miBaseWidget {
		
		public function getEditableControl()
		{
			$t = new scTemplateParser();
			$t->readTemplate($this->_properties['template']);
			$t->addSectionInfo($this->getSectionInfo());
			
			return array($this->_fieldName => $t->templateParse());
		}
		
		public function getData()
		{
			return array($this->_fieldName => miGetParamDefault($this->_fieldName, array()));
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		protected function getSectionInfo()
		{
			$value = $this->_webForm->getFormData($this->_fieldName);
			if (!is_array($value))
				$value = array();
			
			$section = new miTemplateParserSectionInfo('hasNewsletters', 0);
			$newsletters = $this->getNewslettersArray();
			if (empty($newsletters)) {
				return $section;
			}
			
			// Mark the check newsletters
			if (empty($value)) {
				foreach ($newsletters as &$newsletter) {
					$newsletter['HTML_CHECKED'] = '';
					if ($newsletter['NewsletterOptions'] & scNewsletterOptions::CHECKED_BY_DEFAULT) {
						$newsletter['HTML_CHECKED'] = 'checked="checked"';
					}
				}
			} else {
				foreach ($newsletters as &$newsletter) {
					$newsletter['HTML_CHECKED'] = '';
					if (isset($value[$newsletter['NewsletterID']]) && ($value[$newsletter['NewsletterID']] == 1)) {
						$newsletter['HTML_CHECKED'] = 'checked="checked"';
					}
				}
			}
			
			$section->setIterationsCount(1);
			
			$subsection = new miTemplateParserSectionInfo('newsletters', count($newsletters), miUIUtil::templetizeRows(miUIUtil::transposeRows($newsletters)));
			$section->addSubsection($subsection);
			
			return $section;
		}
		
		protected function getNewslettersArray()
		{
			try {
				// Load module classes
				$moduleId = scModulesFactory::getModuleIdByName('com.summercart.newsletter');
				scModulesFactory::getImplementation($moduleId, 'IUserModule');
				
				$recordset = new miSqlRecordset('NLNewsletters');
				$recordset->addJoinCondition('INNER', 'NLNewslettersML', 'ON NLNewsletters.NewsletterID = NLNewslettersML.NewsletterID AND NLNewslettersML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId());
				$recordset->addFilter(new miSqlFilterEqual('NewsletterIsActive', 1));
				$recordset->setSelectFields(array('NLNewsletters.NewsletterID', 'NLNewslettersML.NewsletterName', 'NLNewsletters.NewsletterOptions'));
				return $recordset->getAllRecords();
			} catch (miException $e) {
				// If the newsletters module has not been installed correctly.
				return array();
			}
		}
	}
?>