<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductAttributesSearchWidget extends miBaseWidget {
		
		protected $_productClassId = null;
		protected $_template;
		protected $_includeSystemAttributes = scProductAttributes::GET_SYSTEM_ATTRIBUTES_NONE;
		
		public static function getProductClassAttributesHtml($productClassId, $values, $widgetName, $template, $includeSystemAttributes = scProductAttributes::GET_SYSTEM_ATTRIBUTES_INCLUDE)
		{
			if (!$productClassId)
				return '';
			
			$attributes = scProductAttributes::singleton()->getProductClassAttributes($productClassId, $includeSystemAttributes);
			
			$formDataRow = array();
			foreach ($values as $value) {
				$fieldName = $widgetName . '[' . $value['ProductClassAttributeID'] . ']';
				$attributeType = $attributes[$value['ProductClassAttributeID']]['ProductClassAttributeType'];
				
				$valueField = scProductAttributes::singleton()->getValueFieldForType($attributeType);
				
				$formDataRow[$fieldName] = $value[$valueField];
			}
			
			$typesWithValues = scProductAttributes::singleton()->getFieldTypesWithValues();
			$webForm = new miWebForm($formDataRow);
			$sectionValues = array();
			foreach ($attributes as $attribute) {
				$classAttrId = $attribute['ProductClassAttributeID'];
				$fieldName = $widgetName . '[' . $classAttrId . ']';
				
				$widget = self::getWidget($attribute, $webForm);
				$widget->setFieldName($fieldName);
				if (in_array($attribute['ProductClassAttributeType'], $typesWithValues)) {
					scProductAttributes::singleton()->setWidgetValues($widget, $attribute, 'Any'); // _t('Any')
				}
				
				$control = $widget->getEditableControl();
				$sectionValues[] = array(
					'ProductClassAttributeName' => $attribute['ProductClassAttributeName'],
					'HTML_ProductAttribute' => $control[$fieldName],
				);
			}
			
			$t = new scTemplateParser();
			$t->readTemplate($template);
			$t->addSectionInfo(new miTemplateParserSectionInfo('productAttributes', count($sectionValues), miUIUtil::templetizeRows(miUIUtil::transposeRows($sectionValues))));
			return $t->templateParse();
		}
		
		public static function getProductClassAttributesPreparedValues($productClassId, $values, $includeSystemAttributes = scProductAttributes::GET_SYSTEM_ATTRIBUTES_ALWAYS)
		{
			if (!$productClassId)
				return array();
			
			$attributes = scProductAttributes::singleton()->getProductClassAttributes($productClassId, $includeSystemAttributes);
			
			$attributeValues = array();
			foreach ($attributes as $attributeId => $attribute) {
				$attributeType = $attribute['ProductClassAttributeType'];
				$valueField = scProductAttributes::singleton()->getValueFieldForType($attributeType);
				
				$value = '';
				if (isset($values[$attributeId])) {
					$value = $values[$attributeId];
				}
				
				$attributeValues[$attributeId] = array(
					'ProductClassAttributeID' => $attributeId,
					$valueField => $value,
				);
			}
			
			return $attributeValues;
		}
		
		public function setProductClassId($productClassId)
		{
			$this->_productClassId = $productClassId;
		}
		
		public function setIncludeSystemAttributes($includeSystemAttributes)
		{
			$this->_includeSystemAttributes = $includeSystemAttributes;
		}
		
		public function setTemplate($template)
		{
			$this->_template = $template;
		}
		
		public function getEditableControl()
		{
			$values = $this->_webForm->getFormData($this->_fieldName);
			$values = self::getProductClassAttributesPreparedValues($this->_productClassId, miGetParamDefault($this->_fieldName, array()), $this->_includeSystemAttributes);
			
			$html = self::getProductClassAttributesHtml($this->_productClassId, $values, $this->_fieldName, $this->_template, $this->_includeSystemAttributes);
			return array($this->_fieldName => $html);
		}
		
		/**
		 * @param int $attributeType
		 * @param miWebForm $webForm
		 * @return miBaseWidget
		 */
		protected static function getWidget(array $attribute, miWebForm $webForm)
		{
			switch ($attribute['ProductClassAttributeType']) {
				case scProductAttributes::ATTRIBUTE_TYPE_TEXT:
					if ($attribute['ProductClassAttributeValidator'] == scProductAttributes::VALIDATOR_NUMBER) {
						$widget = new scNumberIntervalWidget($webForm, array());
					} else {
						$widget = new miTextWidget($webForm, array());
					}
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_TEXT_ML:
					$widget = new miTextWidget($webForm, array());
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_SELECT:
					$widget = new miSelectWidget($webForm, array());
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_RADIO_BUTTONS:
					$widget = new miRadioWidget($webForm, array());
					break;
				case scProductAttributes::ATTRIBUTE_TYPE_MULTIPLE_SELECT:
					$widget = new scMultipleCheckboxesWidget($webForm, array());
					break;
			}
			return $widget;
		}
	}

	class scNumberIntervalWidget extends miBaseTextWidget {
		
		public function getEditableControl()
		{
			$value = $this->getValue();
			if (!is_array($value)) {
				$newValue = explode('-', $value);
				$value = array();
				if (count($newValue) == 2) {
					$value['From'] = $newValue[0];
					$value['To'] = $newValue[1];
				}
			}
			
			if (!isset($value['From'])) {
				$value['From'] = '';
			}
			if (!isset($value['To'])) {
				$value['To'] = '';
			}
			$widget = _t('From') . ': <input class="valueFrom" type="text" name="' . $this->_fieldName . '[From]" value="' . miI18N::htmlEscape($value['From']) . '" /> ';
			$widget .= _t('To') . ': <input class="valueTo" type="text" name="' . $this->_fieldName . '[To]" value="' . miI18N::htmlEscape($value['To']) . '" />';
			return array($this->_fieldName => $widget);
		}
	}
?>