<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductOptionsSearchWidget extends miBaseWidget {
		
		protected $_template;
		
		public function setTemplate($template)
		{
			$this->_template = $template;
		}
		
		public function getEditableControl()
		{
			$t = new scTemplateParser();
			$t->readTemplate($this->_template);
			$t->addSectionInfo($this->getSectionInfo());
			$html = $t->templateParse();
			
			return array($this->_fieldName => $html);
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		protected function getSectionInfo()
		{
			$hasSearchableOptionsSection = new miTemplateParserSectionInfo('hasSearchableOptions', 0);
			
			$optionGroups = $this->getSearchableOptions();
			if (count($optionGroups) == 0) {
				return $hasSearchableOptionsSection;
			}
			
			$hasSearchableOptionsSection->setIterationsCount(1);
			
			$optionGroupIds = miUtil::getKeyValueArray($optionGroups, 'OptionGroupID', 'OptionGroupID');
			$options = $this->getOptionGroupsOptions($optionGroupIds);
			
			$searchableOptionsSection = new miTemplateParserSectionInfo('searchableOptions', count($optionGroups), miUIUtil::templetizeRows(miUIUtil::transposeRows($optionGroups)));
			$hasSearchableOptionsSection->addSubsection($searchableOptionsSection);
			
			$optionValuesSections = array();
			foreach ($optionGroups as $optionGroup) {
				$optionGroupId = $optionGroup['OptionGroupID'];
				
				$values = array(
					array(
						'OptionID' => '',
						'OptionName' => _t('Any'),
					)
				);
				
				$optionValuesSection = new miTemplateParserSectionInfo('optionValues', 0);
				if (isset($options[$optionGroupId])) {
					$values = array_merge($values, $options[$optionGroupId]);
				}
				
				$optionValuesSection->setIterationsCount(count($values));
				$optionValuesSection->setValues(miUIUtil::templetizeRows(miUIUtil::transposeRows($values)));
				
				$optionValuesSections[] = $optionValuesSection;
			}
			
			$searchableOptionsSection->addSubsectionsArray('optionValues', $optionValuesSections);
			
			return $hasSearchableOptionsSection;
		}
		
		protected function getSearchableOptions()
		{
			$recordset = new scSqlRecordsetML('OptionGroups', 'OptionGroupID');
			$recordset->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$recordset->addFilter(new miSqlFilterEqual('OptionGroupType', scProductOptions::GROUP_TYPE_VARIANT));
			return $recordset->getAllRecords();
		}
		
		protected function getOptionGroupsOptions($optionGroupIds)
		{
			$recordset = new scSqlRecordsetML('Options', 'OptionID');
			$recordset->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$recordset->addFilter(new miSqlFilterIn('OptionGroupID', $optionGroupIds));
			$options = $recordset->getAllRecords();
			
			$result = array();
			foreach ($options as $option) {
				$result[$option['OptionGroupID']][] = $option;
			}
			
			return $result;
		}
	}
?>