<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductPricesWidget extends scDynRowsWidget {
		var $_customerGroupNames = null;
		
		public function __construct(miWebForm &$webForm, $properties)
		{
			parent::__construct($webForm, $properties);
			
			// Add text fields to hold the price for each Customer Group
			$customerGroupNames = $this->getCustomerGroupNames();
			foreach ($customerGroupNames as $customerGroupId => $customerGroupName)
				$this->_rsDataFields[] = array(
					'field' => 'miBaseTextWidget',
					'data' => 'ProductWholesalePriceAmount' . $customerGroupId
				);
		}
		
		protected function getCustomerGroupNames()
		{
			if ($this->_customerGroupNames == null)
				$this->_customerGroupNames = scCustomerGroups::getCustomerGroupNames();
			return $this->_customerGroupNames;
		}
		
		/**
		 * @param miTemplateParser $templateParser
		 */
		protected function assignTemplateVars(miTemplateParser &$templateParser)
		{
			parent::assignTemplateVars($templateParser);
			
			$templateParser->assign('%%CUSTOMER_GROUPS_COUNT%%', count($this->getCustomerGroupNames()));
			$templateParser->assign('%%CUSTOMER_GROUPS_COUNT_2%%', count($this->getCustomerGroupNames())+2);
		}
		
		protected function getCustomerGroupNamesSectionInfo()
		{
			$customerGroupNames = $this->getCustomerGroupNames();
			
			$sectionInfo = new miTemplateParserSectionInfo();
			$sectionInfo->setSectionInfo('customerGroupNames', count($customerGroupNames),
				miUIUtil::templetizeAndEscapeRows(array('CustomerGroupName' => array_values($customerGroupNames))));
			
			return $sectionInfo;
		}
		
		protected function getPriceFieldsSectionInfo()
		{
			$customerGroupNames = $this->getCustomerGroupNames();
			
			$sectionInfo = new miTemplateParserSectionInfo();
			$sectionInfo->setSectionInfo('priceFields', count($customerGroupNames),
				miUIUtil::templetizeAndEscapeRows(array('CustomerGroupID' => array_keys($customerGroupNames), 'CustomerGroupName' => array_values($customerGroupNames))));
			
			return $sectionInfo;
		}
		
		/**
		 * Parse the repeating section template
		 * 
		 * @param miTemplateParserSectionInfo the section info object
		 * @return string the html
		 */
		protected function parseTemplate($sectionInfo)
		{
			$rsTpl = new scTemplateParser();
			$rsTpl->readTemplate($this->_template);
			$this->assignTemplateVars($rsTpl);
			
			$priceFieldsSectionInfo = $this->getPriceFieldsSectionInfo();
			$sectionInfo->addSubsection($priceFieldsSectionInfo);
			
			$rsTpl->setSectionInfos(array($sectionInfo, $this->getCustomerGroupNamesSectionInfo()));
			return $rsTpl->templateParse();
		}
		
		
		/**
		 * @return array
		 */
		public function getData()
		{
			$return = parent::getData();
			
//			$properties = $this->getProperties();
			
			return $return;
		}
		
		/**
		 * Checks that the data in the widget is valid
		 * 1. Check for valid quantity (1 or more)
		 * 2. Check for duplicate quantity on different rows
		 * 3. Checks that the first row quantity is 1 
		 *
		 * @param miWebFormErrorsHandler $errors
		 */
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			$data = $this->getData();
			$data = reset($data);
			
			if (count($data) == 0)
				return;
			
			// First row quantity must be 1
			$data[0]['ProductWholesalePriceQuantity'] = 1;
			
			foreach ($data as $rowNum => $dataRow) {
				if (($dataRow['ProductWholesalePriceQuantity'] < 1) or (!preg_match('#^[0-9]+$#', $dataRow['ProductWholesalePriceQuantity']))) {
					$errors->addError('ProductWholesalePriceQuantity' . $this->getSuffix($rowNum+1), _t('Invalid quantity.'));
				}
				
				foreach ($data as $otherRowNum => $otherRow) {
					if ($rowNum == $otherRowNum)
						continue;
						
					if ($dataRow['ProductWholesalePriceQuantity'] == $otherRow['ProductWholesalePriceQuantity'])
						$errors->addError('ProductWholesalePriceQuantity' . $this->getSuffix($rowNum+1), _t('Duplicate quantity.'));
				}
				
				// Check for valid data in the amount fields
				foreach ($dataRow as $field => $fieldValue) {
					if ($field == 'ProductWholesalePriceQuantity')
						continue;
					
					if (!preg_match('#^[0-9]{1,10}\.?([0-9]){0,2}$#', $fieldValue))
						$errors->addError($field . $this->getSuffix($rowNum+1), _t('Invalid amount.'));
				}
			}
			
//			if ($data[0]['ProductWholesalePriceQuantity'] != '1')
//				$errors->addError('ProductWholesalePriceQuantity::1::', _t('First row quantity must be 1.'));
		}
	}
?>