<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Class for the multi language text widget
	 */
	class scWebFormWidgetTextML extends miTextWidget {
		/**
		 * Returns the actual control html
		 * 
		 * @param string $fieldName the name of the field
		 * @param string $fieldData the data in the field
		 * @param int $languageDirection the direction of the language
		 * @return string html for the control
		 */
		protected function getControlHtml($fieldName, $fieldData, $languageDirection)
		{
			$dir = ($languageDirection == scLanguageDomainObject::LANGUAGE_DIRECTION_RTL) ? ' dir="rtl"' : '';
			$disabled = (isset($this->_properties['disabled']) && $this->_properties['disabled'] ? ' disabled="disabled"' : '');
			$class = (isset($this->_properties['class']) && ($this->_properties['class'] != '') ? ' class="' . miI18N::htmlEscape($this->_properties['class']) . '"' : '');
			return '<input type="text" name="' . miI18N::htmlEscape($fieldName) . '" value="' . miI18N::htmlEscape($fieldData) . '"' . $dir . $disabled . $class . '/>';
		}
		
		/**
		 * Function return array 
		 */
		public function getEditableControl()
		{
			$activeLanguages = scMultiLanguage::singleton()->getActiveLanguages();
			$widget = '<div id="' . miI18N::htmlEscape($this->_fieldName) . '">';
			
			$data = $this->_webForm->getFormData($this->_fieldName);
			
			// this allows default value for this widget
			if (empty($data) && !empty($this->_defaultValue))
				$data = $this->_defaultValue;
			
			foreach ($activeLanguages as $activeLanguage) {
				$languageId = $activeLanguage['LanguageID'];
				$fieldData = isset($data[$languageId]) ? $data[$languageId] : '';
				
				$widget .= '<div>';
				$widget .= '<span id="span' . miI18N::htmlEscape($this->_fieldName . '-' . $languageId) . '" style="display:block;overflow:hidden;height:0;">';
				$widget .= $this->getControlHtml($this->_fieldName . '[' . $languageId . ']', $fieldData, $activeLanguage['LanguageDirection']);
				$widget .= '<a href="javascript: SC.ML.nextLanguage(' . $languageId . ');" name="changeLanguage">';
				
				if (!empty($activeLanguage['LanguageImage']) and file_exists(scPaths::getStoreRootDir() . $activeLanguage['LanguageImage'])) {
					$widget .= '<img id="img' . miI18N::htmlEscape($this->_fieldName . '-' . $languageId) . '" ' .
						'src="' . scPaths::getStoreWebPath() . $activeLanguage['LanguageImage'] . '" style="width: ' . $activeLanguage['LanguageImageWidth'] . 'px; height: ' . $activeLanguage['LanguageImageHeight'] . 'px" ' .
						'alt="' . miI18N::htmlEscape($activeLanguage['LanguageName']) . '" title="' . miI18N::htmlEscape($activeLanguage['LanguageName']) . '"/>';
				}
				else
					$widget .=  miI18N::htmlEscape($activeLanguage['LanguageName']);
				
				$widget .= '</a>';
				$widget .= '</span>';
				$widget .= '</div>';
			}
			
			$widget .= '</div><script type="text/javascript">SC.ML.registerMultilanguageControl("' . miI18N::htmlEscape($this->_fieldName). '")</script>';
			return array($this->_fieldName => $widget);
		}
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			$data = miGetParamDefault($this->_fieldName, array());
			
			if (!isset($this->_properties['minLength']))
				return true;
			
			$min = $this->_properties['minLength'];
			foreach ($data as $key => $val) {
				if (strlen($val) >= $min)
					return true; // we have input for at least one language 
			}

			foreach ($data as $key => $val) {
				$errors->addError($this->_fieldName . '[' . $key . ']', _t('Field is empty.'));
			}
		}
	}
	
	/**
	 * Class for the multi language text area widget
	 */
	class scWebFormWidgetTextAreaML extends scWebFormWidgetTextML {
		/**
		 * Returns the actual control html
		 * 
		 * @param string $fieldName the name of the field
		 * @param string $fieldData the data in the field
		 * @param int $languageDirection the direction of the language
		 * @return string html for the control
		 */
		protected function getControlHtml($fieldName, $fieldData, $languageDirection)
		{
			$dir = ($languageDirection == scLanguageDomainObject::LANGUAGE_DIRECTION_RTL) ? ' dir="rtl"' : '';
			return '<textarea rows="4" cols="80" id="' . miI18N::htmlEscape($fieldName) . '" name="' . miI18N::htmlEscape($fieldName) . '"' . $dir . '>' . miI18N::htmlEscape($fieldData) . '</textarea>';
		}
	}
?>