<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Action to display a wizard page
	 */
	class miViewWizardPageAction extends miActionWithWebForm {
		public function doAction()
		{
			try {
				$this->_view->callPlugin($this, 'preShowForm');
				
				$form = $this->getWebForm();
				$this->setRecordData();
				
				$model = $this->_view->getViewMapperController();
				// Get data from session. If there is no data in session, load data from database
				// This will prevent loading data from database if user has modified the values
				$data = $this->_view->getPageData();
				if (empty($data)) {
					// it is possible to have no valid ID
					// if the user is not logged in, so just ignore record not found exceptions
					try {
						$model->read();
					} catch (miDBException $e) {
						if ($e->getCode() != miDBException::EXCEPTION_RECORD_NOT_FOUND)
							throw $e;
					}
				}
				
				// Suppres notices from display only wigets 
				@$model->setDataToWebForm();
				
				$form->addMainPageElements($this->_view->getMainPageElements());
				
				$html = $form->parse($this->_view->getTemplateFileName(miWizardPage::TEMPLATE_ID_WIZARD_PAGE), true);
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$msg = _t('Error processing your request: ');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		protected function setRecordData()
		{
			$pageData = $this->_view->getPageData();
			$dataFields = $this->_view->getDataFields();
			foreach ($dataFields as $dataField) {
				if (!isset($pageData[$dataField['data']]))
					$pageData[$dataField['data']] = '';
			}
			
			$this->_view->getDomainObject()->setRow($pageData);
		}
	}
	
	/**
	 * Action to save the wizard page state
	 */
	class miExecWizardPageAction extends miActionWithWebForm {
		public function doAction()
		{
			try {
				$model = $this->_view->getViewMapperController();
				
				$form = $this->getWebForm();
				$isOk = $form->processSubmit();
				$model->setDataFromWebForm();
				
				$form->addMainPageElements($this->_view->getMainPageElements());
				
				$this->_view->savePageData($model->getDomainObject()->getRow());
				
				return $isOk;
			} catch (miException $exception) {
				$msg = _t('Wizard exception: ');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}

?>