<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Wizard contoller class
	 */
	class miWizardController extends scUserDisplayController {
		const WIZARD_VAR_CURRENT_PAGE = '%%CURRENT_WIZARD_PAGE%%';
		const WIZARD_VAR_TOTAL_PAGES = '%%TOTAL_WIZARD_PAGES%%';

		/**
		 * @var miWizardDispatcher $_dispatcher
		 */
		protected $_dispatcher;
		
		public function __construct()
		{
			parent::__construct(array());
			new miWizardDispatcher($this);
		}
		
		/**
		 * Adds new wizard page
		 * 
		 * @param miWizardPage $wizardPage the wizard page object
		 */
		public function addWizardPage(miWizardPage $wizardPage)
		{
			$this->_views[] = $wizardPage;
			$wizardPage->setController($this);		// Self plug
		}
		
		/**
		 * Inits the wizard
		 */
		public function initWizard()
		{
			foreach ($this->_views as $wizardPage) {
				$wizardPage->clearPageData();
			}
			
			$this->onWizardStart();
		}
			
		/**
		 * @return miWizardDispatcher
		 */
		public function getDispatcher()
		{
			return $this->_dispatcher;
		}
		
		/**
		 * @return miWizardPage
		 */
		public function getPage($pageIndex)
		{
			return $this->_views[$pageIndex - 1];
		}		
		
		public function setDispatcher(miDispatcher $dispatcher)
		{
			$this->_dispatcher = $dispatcher;
		}
		
		public function getPagesNumber()
		{
			return count($this->_views);
		}
		
		public function showPage()
		{
			echo $this->getHeader();
			
			echo $this->_skin->getColumn(scLayout::COLUMN_LEFT, scLayout::AREA_CUSTOMER);
			
			$currentPage = $this->getPage($this->getDispatcher()->getCurrentPage());
			$this->processControllerCommands($currentPage->getControllerCommands());
			
			echo $this->_skin->getColumn(scLayout::COLUMN_RIGHT, scLayout::AREA_CUSTOMER);
			
			echo $this->getFooter();
		}
		
		public function getWizardPageElements()
		{
			$nextPageIndex = $this->getDispatcher()->getCurrentPage();
			return array(
				self::WIZARD_VAR_CURRENT_PAGE => $nextPageIndex,
				self::WIZARD_VAR_TOTAL_PAGES => count($this->_views)
			);
		}
		
		/**
		 * Dispatch an action and show a page with a one view
		 */
		public function dispatchAndShow()
		{
			// Init the actions depending on the wizard action			
			$this->getDispatcher()->processWizardAction();
			
			$this->showPage();
		}
		
		/**
		 * Called when the user selects to start the wizard
		 */
		public function onWizardStart()
		{
		}
		
		/**
		 * Called when the user selects to cancel the wizard
		 */
		public function onWizardCancel()
		{
		}
		
		/**
		 * Called when the user finishes the wizard
		 */
		public function onWizardFinish()
		{
		}
	}
?>