<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class miWizardDispatcher extends miDispatcher {	
		/**
		 * @var miWizardController $_wizardController
		 */
		protected $_wizardController;
		
		protected $_currentPage = 1;
		protected $_nextPage = 1;
				
		protected $_wizardAction;
		protected $_action;
		
		const WIZARD_CURRENT_PAGE_PARAM = 'currentWizardPage';

		const WIZARD_ACTION_PARAM = 'wizardAction';
		const WIZARD_ACTION_NEXT = 'next';
		const WIZARD_ACTION_PREVIOUS = 'previous';
		const WIZARD_ACTION_CANCEL = 'cancel';
		

		protected $_actionHandlers = array(
			'' => 'miViewWizardPageAction',
			'miExecWizardPageAction' => 'miExecWizardPageAction',
		);
		
		public function __construct(miWizardController $controller)
		{
			$this->_wizardController = $controller;
			// Self pluggable dispatcher
			$controller->setDispatcher($this);
			
			$this->_wizardAction = miGetParamDefault(self::WIZARD_ACTION_PARAM, '');
		}
				
		/**
		 * Wizard processing
		 */
		public function processWizardAction()
		{
			$this->findCurrentAndNextPages();
			
			if ($this->_wizardAction == self::WIZARD_ACTION_CANCEL) {
				$this->_wizardController->onWizardCancel();
				return;
			}

			// Process the real action
			$action = miGetParamDefault('action', '');
			
//			if ($this->_wizardAction == '' && $action == '')
//				$this->_wizardController->initWizard();
			
			$result = $this->doProcess($action);
			
			if (($this->_wizardAction == self::WIZARD_ACTION_NEXT) || ($this->_wizardAction == self::WIZARD_ACTION_PREVIOUS)) {
				// Allow going back even on error
				if ($result or ($this->_wizardAction == self::WIZARD_ACTION_PREVIOUS))
					$this->moveToNextPage();
				
				$this->doProcess('');
			}
		}
		
		protected function setPage($pageIndex)
		{
			$this->_view = $this->getWizardPage($pageIndex);
		}
		
		protected function doProcess($action)
		{
			$this->_action = $action;
			
			if ($this->_nextPage == $this->_wizardController->getPagesNumber() + 1) {
				$this->_wizardController->onWizardFinish();
				return;
			}
			
			$actionObj = $this->createActionObj($this->_action);
			if ($actionObj === false)
				return false;
			
			$this->_actionObj = $actionObj;
			
			$currentPage = $this->_wizardController->getPage($this->_currentPage);
			if ($this->_nextPage != 0)
				$currentPage->addMainPageElements($this->_wizardController->getWizardPageElements());
			
			$currentPage->preProcess($actionObj);
			$actionResult = $actionObj->doAction();
			$currentPage->postProcess($actionObj, $actionResult);
			
			return $actionResult;
		}
		
		public function process()
		{
			throw new miException('miWizardDispatcher::process() should not be called.');
		}
						
		public function getWizardPage($pageIndex)
		{
			return $this->_wizardController->getPage($pageIndex);
		}
		
		protected function createActionObj($action)
		{
			return new $this->_actionHandlers[$action]($this->_wizardController->getPage($this->_currentPage));
		}
		
		/**
		 * Sets the next page number
		 * Page numbers start from 0
		 * 
		 * @param int $nextPage the next page number
 		 */
		public function setNextPage($nextPage)
		{
			$this->_nextPage = $nextPage;
		}
		
		/**
		 * Returns next page number
		 * Page numbers start from 0
		 * 
		 * @return int the next page number
		 */
		public function getNextPage()
		{
			return $this->_nextPage;
		}
		
		/**
		 * Returns the current page number
		 * Page numbers start from 0
		 * 
		 * @return int the current page number
		 */
		public function getCurrentPage()
		{
			return $this->_currentPage;
		}
		
		public function moveToNextPage()
		{
			$this->_currentPage = $this->_nextPage;
		}
			
		/**
		 * Sets the current page number and computes the next page number
		 */
		public function findCurrentAndNextPages()
		{
			$this->_currentPage = (int) miGetParamDefault(self::WIZARD_CURRENT_PAGE_PARAM, 1);
			if ($this->_currentPage > $this->_wizardController->getPagesNumber() or ($this->_currentPage < 1))
				throw new miException('Invalid wizard page');
			
			$this->_nextPage = $this->_currentPage;
			if ($this->_wizardAction == self::WIZARD_ACTION_NEXT)
				$this->_nextPage++;
			if ($this->_wizardAction == self::WIZARD_ACTION_PREVIOUS)
				$this->_nextPage--;
		}
		
		public function getWizardAction()
		{
			return $this->_wizardAction;
		}
		
		public function setWizardAction($wizardAction)
		{
			$this->_wizardAction = $wizardAction;
		}
	}
?>