<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scCustomerLogin implements ICMSBlock {
		
		public function getControllerCommands()
		{
			$email = miGetParamDefault('Email', '');
			$password = miGetParamDefault('Password', '');
			$msg = miGetParamDefault('msg', '');
			
			if (($email == '') or ($password == '')) {
				if ($_SERVER['REQUEST_METHOD'] == 'POST')
					$msg = _t('Please enter e-mail and password.');
				return $this->getLoginPage($email, $password, $msg);
			}
			
			try {
				$user = new miSqlRecord('Customers', 'CustomerEmail');
				$user->readPK($email);
				
				$auth = scCustomerAuth::singleton();
				if (!$auth->isCorrectPassword($user, $email, $password)) {
					return $this->getLoginPage($email, $password, _t('Invalid username and/or password. Please try again.'));
				}
				if ($user->get('CustomerStatus') != scCustomerStatuses::CUSTOMER_STATUS_ACTIVE) {
					return $this->getLoginPage($email, $password, _t('Your account has been disabled.'));
				}
			} catch (miDBException $exception) {
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					return $this->getLoginPage($email, $password, _t('Invalid username and/or password. Please try again.'));
					exit;
				}
				throw $exception;
			}
	
			# Create new session
			$auth->logIn($user->get('CustomerID'));
			
			scCart::getCart()->refreshCustomerInfo($auth->getCustomer());
			
			$location = miGetParamDefault('Referer', '');
			if (!empty($location)) {
				$parsed = scUrlUtil::parse($location);
				if (isset($parsed[scUrlUtil::PATH]) && (basename($parsed[scUrlUtil::PATH]) == 'error.php')) {
					// If coming from an error page
					$location = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_INDEX);
				} else if (isset($parsed[scUrlUtil::HOST]) && ($parsed[scUrlUtil::HOST] != $_SERVER['SERVER_NAME'])) {
					// If coming from another domain
					$location = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_INDEX);
				} else if (isset($parsed[scUrlUtil::PATH]) && (scUrlResolver::singleton()->is($parsed[scUrlUtil::PATH], scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN))) {
					// If coming from the index page
					$location = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_INDEX);
				}
			} else {
				$location = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_INDEX);
			}
			
			return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $location));
		}
		
		protected function getLoginPage($email, $password, $msg)
		{
			$t = new scTemplateParser();
			$t->readTemplate('customer/login.tmpl');
			$t->assign('%%EMAIL%%', miI18N::htmlEscape($email));
			$t->assign('%%PASSWORD%%', miI18N::htmlEscape($password));
			
			$referer = miGetParamDefault('Referer', '');
			if (($referer == '') and isset($_SERVER['HTTP_REFERER'])) {
				$parsedReferer = parse_url($_SERVER['HTTP_REFERER']);
				if ($parsedReferer['path'] != '/checkout.php') {
					$referer = $_SERVER['HTTP_REFERER'];
				}
			}
			$t->assign('%%REFERER%%', miI18N::htmlEscape($referer));
			
			$t->assign('%%ENCODED_REFERER%%', urlencode($referer));
			
			$t->assign('%%MSG%%', miI18N::htmlEscape($msg));
			$t->assign('%%MSG_STYLE%%', $msg ? 'error' : 'hidden');
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('login.php'));

			$html = $t->templateParse();
			return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
		}
	}
	
	$userPageCustomerLogin = new scCustomerLogin;
	$page = new scUserDisplayController(array($userPageCustomerLogin));
	$page->setSeo(new scDynamicPageSeo('login.php', _t('Login')));
	$page->showPage();
?>