<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scManufacturersView extends miDefaultView {
		const TEMPLATE_ID_VIEW_GRID = 'grid_view';
		
		const DEFAULT_IMAGE_URL = 'customer/images/default_manufacturer_image.gif';
		const DEFAULT_IMAGE_WIDTH = 100;
		const DEFAULT_IMAGE_HEIGHT = 100;
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'customer/manufacturers/all_manufacturers.tmpl',
			self::TEMPLATE_ID_VIEW => 'customer/manufacturers/manufacturer.tmpl',
			self::TEMPLATE_ID_VIEW_GRID => 'customer/manufacturers/manufacturer_grid.tmpl'
		);
			
		public function __construct()
		{
			parent::__construct(null);
			
			$this->_persistVars[] = 'ManufacturerID';
		}
	}
	
	class scManufacturerList extends miAction {
		public function doAction()
		{
			$parser = new scTemplateParser();
			
			$manufacturers = $this->getManufacturers();
			$parser->addSectionInfo(new miTemplateParserSectionInfo('manufacturers', count($manufacturers), miUIUtil::templetizeRows(miUIUtil::transposeRows($manufacturers))));
			
			// Assign dynamic page areas values
			$parser->assignArray(scDynamicPages::getTemplateVars('manufacturers.php'));
			
			$parser->readTemplate($this->_view->getTemplateFileName(scManufacturersView::TEMPLATE_ID_LIST));
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $parser->templateParse()));
		}
		
		protected function getManufacturers()
		{
			$sql =  'SELECT *, Manufacturers.ManufacturerID AS ManufacturerID FROM Manufacturers ' .
					'LEFT JOIN ManufacturersML USING(ManufacturerID) WHERE LanguageID = ? ORDER BY ManufacturerSort ASC';
			$manufacturers = miStaticDBUtil::execSelect($sql, array(scMultiLanguage::getCurrentLanguageId()));
			
			foreach ($manufacturers as &$manufacturer) {
				if (empty($manufacturer['ManufacturerImage']) || !is_file(scPaths::getStoreRootDir() . $manufacturer['ManufacturerImage'])) {
					$manufacturer['ManufacturerImage'] = scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl() . scManufacturersView::DEFAULT_IMAGE_URL;
					$manufacturer['ManufacturerImageWidth'] = scManufacturersView::DEFAULT_IMAGE_WIDTH;
					$manufacturer['ManufacturerImageHeight'] = scManufacturersView::DEFAULT_IMAGE_HEIGHT;
				} else {
					$manufacturer['ManufacturerImage'] = scPaths::getStoreWebPath() . $manufacturer['ManufacturerImage'];
				}
			}
			
			scFriendlyUrl::initFriendlyUrls($manufacturers, scFriendlyUrl::ROW_TYPE_MANUFACTURERS);
			
			return $manufacturers;
		}
	}
	
	class scManufacturerView extends scProductsListingAction
	{
		public function doAction()
		{
			$manufacturerId = (int) miGetParam('ManufacturerID');
			
			// Get manufacturers info
			$sql =  'SELECT *, Manufacturers.ManufacturerID AS ManufacturerID FROM Manufacturers ' .
					'LEFT JOIN ManufacturersML USING(ManufacturerID) ' .
					'WHERE Manufacturers.ManufacturerID = ? AND LanguageID = ?';
			
			$manufacturerInfo = miStaticDBUtil::execSelect($sql, array($manufacturerId, scMultiLanguage::getCurrentLanguageId()));
			if (empty($manufacturerInfo)) {
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode(_t('Cannot find manufacturer.'));
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
				return true;
			}
			
			$manufacturerInfo = reset($manufacturerInfo);		// Get the first row
			if (empty($manufacturerInfo['ManufacturerImage']) || !is_file(scPaths::getStoreRootDir() . $manufacturerInfo['ManufacturerImage'])) {
				$manufacturerInfo['ManufacturerImage'] = scSkins::singleton()->getCurrentSkin()->getCurrentSkinUrl() . scManufacturersView::DEFAULT_IMAGE_URL;
				$manufacturerInfo['ManufacturerImageWidth'] = scManufacturersView::DEFAULT_IMAGE_WIDTH;
				$manufacturerInfo['ManufacturerImageHeight'] = scManufacturersView::DEFAULT_IMAGE_HEIGHT;
			} else {
				$manufacturerInfo['ManufacturerImage'] = scPaths::getStoreWebPath() . $manufacturerInfo['ManufacturerImage'];
			}
			
			// Get the view type - list/grid
			$view = scProductService::getViewType();
			if ($view == scProductService::VIEW_GRID) {
				$templateId = scManufacturersView::TEMPLATE_ID_VIEW_GRID;
				$altView = scProductService::VIEW_LIST;
				$altViewString = 'list';
			} else {
				$templateId = scManufacturersView::TEMPLATE_ID_VIEW;
				$altView = scProductService::VIEW_GRID;
				$altViewString = 'grid';
			}	
			
			// The main products section
			$products = $this->getProducts($manufacturerId);
			$productsSection = scProductService::getProductSections('products', $products, $view);
			
			// Parse and show the template
			$t = new scTemplateParser();
			
			$manufacturerInfo['ProductCount'] = $this->getProductsCount($manufacturerId);
			$t->assignArray($this->_view->getMainPageElements());
			$t->assignArray(miUIUtil::templetizeRows($manufacturerInfo));
			
			$t->addSectionInfo($productsSection);
			
			$template = $this->_view->getTemplateFileName($templateId);
			$t->readTemplate($template);
			
			$selfUrl = $this->getSelfUrl($manufacturerId, $manufacturerInfo['ManufacturerName']);
			list($page, $numPages) = scCustomerAreaPager::calculatePages('page', $manufacturerInfo['ProductCount']);
			$altViewPage = scCustomerAreaPager::calculateAltViewPage($page, $view, $altView);
			$altViewUrl = scUrlUtil::addQueryParams($selfUrl, 'page=' . $altViewPage . '&view=' . $altViewString);
			
			$t->assign('%%HTML_CATEGORY_PAGES%%', scCustomerAreaPager::getPaging($page, $numPages, $selfUrl));
			$t->assign('%%PAGE%%', $page);
			$t->assign('%%ALT_VIEW_PAGE%%', $altViewPage);
			$t->assign('%%NUMPAGES%%', $numPages);
			
			$this->_view->getController()->getSeo()->setListingPageInfo($page, $numPages);
			
			// Used for grid/list view
			$t->assign('%%SELF_URL%%', $selfUrl);
			$t->assign('%%ALT_VIEW_URL%%', $altViewUrl);
			// Used for product sorting
			$t->assign('%%HTML_SORT_PRODUCTS%%', scProductSortService::getSortDropdown());
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('manufacturers.php'));
			
			$html = $t->templateParse();			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;					
		}
		
		protected function getSelfUrl($manufacturerId, $manufacturerName)
		{
			$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_MANUFACTURER, $manufacturerId, $manufacturerName) . '?';
			$urlParams = miAppFactory::singleton()->getStateObj()->getStateUrlParams();
			$url = scUrlUtil::addQueryParams($url, $urlParams);
			
			return $url;
		}
		
		protected function getProducts($manufacturerId)
		{
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0 ' : '';
			
			$additionalWhereConditions = $this->getAdditionalWhereConditions();
			$additionalWhereConditions = implode(' ', $additionalWhereConditions);
			
			// Get the manufacturer's products info
			$sql = 	'SELECT *, Products.ProductID AS ProductID, Products.ProductID AS ProductIDSort, Products.CategoryID AS CategoryID FROM Products ' .
					'LEFT JOIN ProductsML ON(Products.ProductID=ProductsML.ProductID AND LanguageID = ?) ' .
					'LEFT JOIN ProductImages ON(Products.ProductID = ProductImages.ProductID AND ProductImageIsDefault<>0) ' .
					'WHERE ManufacturerID = ? ' .
					'AND ProductIsActive <> 0 ' . $hideOutOfStockSql .
					$additionalWhereConditions .
					'ORDER BY ' . scProductSortService::getSortClause() .
					scCustomerAreaPager::getLimitString('page');
			
			return miStaticDBUtil::execSelect($sql, array(scMultiLanguage::getCurrentLanguageId(), $manufacturerId));
		}
		
		protected function getProductsCount($manufacturerId)
		{
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? ' AND ProductQuantity > 0' : '';
			
			$additionalWhereConditions = $this->getAdditionalWhereConditions();
			$additionalWhereConditions = implode(' ', $additionalWhereConditions);
			
			$sql = 	'SELECT COUNT(*) AS N' .
				' FROM Products' .
				' WHERE ManufacturerID = ?' .
				' AND ProductIsActive <> 0' .
				$hideOutOfStockSql .
				$additionalWhereConditions;
			$result = miStaticDBUtil::execSelect($sql, array($manufacturerId));
			return $result[0]['N'];
		}
	}
	
	if (miHasParam('ManufacturerID'))
		miSetParam('action', 'dmView');
	
	$manufacturersView = new scManufacturersView();

	$page = new scUserDisplayController(array($manufacturersView));
	$page->setSeo(new scManufacturerSeo());
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scManufacturerList');
	$page->getDispatcher()->setActionHandler('dmView', 'scManufacturerView');
	
	$page->dispatchAndShow();
?>