<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Callback from Google Checkout to calculate shipping and taxes
	 */
	
	require_once(dirname(__FILE__) . '/../../../include/miphpf/Init.php');
	require_once(dirname(__FILE__) . '/../../../include/sc/Init.php');
	
	class scPaymentMethodGoogleCheckoutCalculation {
		const GCO_MODULE_NAME = 'com.summercart.checkout.google';
		
		/**
		 * Retrieve the rates for the asked shipping methods
		 * The rates are returned in $sellerAccountCurrencyCode
		 * NOTE: This method expects that the destination address has been already set in the cart
		 *
		 * @param array $shippingMethods
		 * @param string $sellerAccountCurrencyCode 3-char currency code
		 * @param int $countryId
		 * @param int $stateId
		 */
		protected function getShippingMethodRates($shippingMethods, $sellerAccountCurrencyCode, $countryId, $stateId)
		{
			$result = array();
			
			// Find all needed shipping modules
			$shippingModules = scModulesFactory::getActiveShippingModulesForZone($countryId, $stateId);
			
			$neededShippingModules = array();
			foreach ($shippingModules as $shippingModule) {
				
				$moduleShippingMethodDisplayNames = array();
				foreach ($shippingModule['ShippingModuleActiveMethods'] as $activeMethod) {
					$moduleShippingMethodDisplayNames[] = $activeMethod['ShippingMethodDisplayName'];
				}
				
				foreach ($shippingMethods as $method) {
					if (in_array((string)$method['name'], $moduleShippingMethodDisplayNames)) {
						$neededShippingModules[] = $shippingModule;
						break;
					}
				}
			}
			
			// Calculate the rates
			foreach ($neededShippingModules as $shippingModule) {
				$moduleId = $shippingModule['ModuleID'];
				$module = $shippingModule['object'];
				
				$shippingMethodRates = $module->getShippingRates(scCart::getCart(), array_keys($shippingModule['ShippingModuleActiveMethods']));
				if (!isset($shippingMethodRates[IShippingModule::SHIPPING_RATES]))
					continue;
				
				$ratesCurrency = isset($shippingMethodRates[IShippingModule::SHIPPING_RATES_CURRENCY]) ?
					$shippingMethodRates[IShippingModule::SHIPPING_RATES_CURRENCY] : scCurrencyUtil::getBaseCurrencyCode();
				$sellerCurrency = new scCurrency($sellerAccountCurrencyCode);
				
				$rates = $shippingMethodRates[IShippingModule::SHIPPING_RATES];
				$shippingMethods = $module->getShippingMethods();
				
				foreach ($shippingMethods as $shippingMethodCode => $shippingMethodName) {
					
					if (!isset($rates[$shippingMethodCode]))
						continue;
					
					if (!isset($shippingModule['ShippingModuleActiveMethods'][$shippingMethodCode]))
						continue;
					
					$rate = $rates[$shippingMethodCode];
					if ($ratesCurrency != $sellerAccountCurrencyCode) {
						$rate = scCurrencyUtil::convert($rate, $ratesCurrency, $sellerCurrency);
					}
					
					$result[] = array('name' => $shippingModule['ShippingModuleActiveMethods'][$shippingMethodCode]['ShippingMethodDisplayName'], 'rate' => round($rate, 2));
				}
			}
			return $result;
		}
		
	
		function calculateShipping($requestXml)
		{
			$result = '';
			
			$moduleId = scModulesFactory::getModuleIdByName(self::GCO_MODULE_NAME);
			$options = scModuleOptions::getModuleOptions($moduleId);
			
			$xml = new SimpleXMLElement($requestXml);
			
			scSession::singleton()->openSession((string)($xml->{'shopping-cart'}->{'merchant-private-data'}->SessionID));
			
			$currencyCode = (string)$xml->{'shopping-cart'}->{'merchant-private-data'}->CurrencyCode;
			$shippingMethods = $xml->calculate->shipping->method;
			$addresses = $xml->calculate->addresses->{'anonymous-address'};
			
			foreach ($addresses as $address) {
				$countryId = scCountry::getCountryIdByCode2((string)$address->{'country-code'});
				$stateId = scCountry::getStateId($countryId, (string)$address->{'region'});
				
				// Set the address info which will be used in the shipping modules called by getShippingMethodRates()
				$cart = scCart::getCart();
				$custInfo = $cart->getCustomerInfo();
				$custInfo['CustomerShippingCountry'] = $countryId;
				$custInfo['CustomerShippingCity'] = (string)$address->{'city'};
				$custInfo['CustomerShippingState'] = $stateId;
				$custInfo['CustomerShippingZip'] = (string)$address->{'postal-code'};
				$custInfo['CustomerShipToBillingAddress'] = false;
				$cart->setCustomerInfo($custInfo);
				
				$methodRates = $this->getShippingMethodRates($shippingMethods, $options['SellerAccountCurrencyCode'], $countryId, $stateId);
				
				foreach ($shippingMethods as $method) {
					
					$found = false;
					foreach ($methodRates as $methodRate) {
						if ($methodRate['name'] == (string)$method['name']) {
							$result .= scXml::buildXmlValues(array('result' => array(
								'<attributes>' => array('shipping-name' => (string)$method['name'], 'address-id' => (string)$address['id']),
								'shipping-rate' => array('' => $methodRate['rate'], '<attributes>' => array('currency' => $currencyCode)),
							)));
							$found = true;
							break;
						}
					}
					if (!$found) {
						$result .= scXml::buildXmlValues(array('result' => array(
							'<attributes>' => array('shipping-name' => (string)$method['name'], 'address-id' => (string)$address['id']),
							'shippable' => 'false',
						)));
					}
				}
			}

			return $result;
		}
		
		public function buildXmlResponse($request, $values)
		{
			$xml = '<?xml version="1.0" encoding="UTF-8" ?>';
			$xml .= '<' . $request .' xmlns="http://checkout.google.com/schema/2">';
			$xml .= scXml::buildXmlValues($values);
			$xml .= '</' . $request . '>';
			return $xml;
		}
		
		/**
		 * Checks the authenticity of the notification
		 * 
		 * TODO: V1.1 check basic auth from Google
		 * 
		 * @throws miException
		 */
		public function checkAuth()
		{
			$gkey = miGetParamDefault('gkey', '');
			$gid = miGetParamDefault('gid', '');
			
			$moduleId = scModulesFactory::getModuleIdByName(self::GCO_MODULE_NAME);
			$options = scModuleOptions::getModuleOptions($moduleId);
			
			if (($gid != $options['MerchantID']) or ($gkey != $options['MerchantKey']))
				throw new miException(_t('Merchat ID or Merchant Key do not match.'));
		}
	}
	
//	scDebug::logVariable('/tmp/google.txt', '_SERVER', $_SERVER);
	
	try {
		$requestXml = file_get_contents('php://input');
		if ($requestXml == '')
			exit;
	
//		scDebug::logVariable('/tmp/google.txt', '$requestXml', $requestXml);
		
		$gCalculator = new scPaymentMethodGoogleCheckoutCalculation();
		$gCalculator->checkAuth();
		
		$values = array('results' => array(scXml::OLD_XML => $gCalculator->calculateShipping($requestXml)));
	
		$responseXml = $gCalculator->buildXmlResponse('merchant-calculation-results', $values);
//		scDebug::logVariable('/tmp/google.txt', 'Response', $responseXml);
		echo $responseXml;
		
	} catch (Exception $exception) {
//		scDebug::logVariable('/tmp/google.txt', '$exception', $exception);
		scLog::singleton()->error(_t('Google Checkout Exception: ') . $exception->getMessage(), scLog::CATEGORY_PAYMENT);
		
		// Send error
		header('HTTP/1.1 500 Internal Error');
		echo $exception->getMessage();
	}
?>