<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Google Checkout module
	 * 
	 * Notes: Tax on shipping is not supported.
	 */
	class scPaymentModuleGoogleCheckout implements ICheckoutModule, IModuleOptions {
		const MODE_SANDBOX = 'Sandbox';
		const MODE_PRODUCTION = 'Production';
		
		// Taken from the source of this page: https://checkout.google.com/buyerSignup - shippableCountries - as of 2nd March 2010
		protected static $supportedShippingCountryCodes = 'VU,VN,EC,VI,DZ,VG,VE,DM,VC,DO,VA,DE,UZ,UY,DK,DJ,US,UM,UG,UA,ET,ES,ER,EH,EG,TZ,EE,TT,TW,TV,GD,GE,GF,GA,GB,FR,FO,FK,FJ,FM,FI,WS,GY,GW,GU,GT,GS,GR,GQ,GP,WF,GN,GM,GL,GI,GH,RE,RO,AT,AS,AR,AQ,AW,QA,AU,AZ,BA,PT,PW,AG,PR,AE,AF,PS,AL,AI,AO,PY,AM,AN,BW,TG,BV,TF,BY,TD,BS,TK,BR,TJ,BT,TH,TO,TN,TM,TL,CA,TR,BZ,BF,SV,BG,BH,ST,BI,BB,SZ,BD,BE,BN,BO,BJ,TC,BM,CZ,SC,CY,CX,SE,SH,CV,SG,SJ,CS,SI,SL,SK,SN,SM,SO,SR,CI,CG,CH,RU,RW,CF,CC,CD,CR,CO,CM,CN,SA,CK,CL,SB,LV,LU,LT,LS,LR,MG,MH,MK,ML,MC,MD,MA,MV,MU,MX,MW,MZ,MY,MN,MP,MO,MR,MQ,MT,MS,NF,NG,NI,NL,NA,NC,NE,NZ,NU,NR,NP,NO,OM,PL,PM,PN,PH,PK,PE,PF,PG,PA,HK,ZA,HN,HM,HR,HT,HU,ZM,ID,ZW,IE,IL,IN,IO,IQ,IS,YE,IT,YT,JP,JO,JM,KI,KH,KG,KE,KR,KM,KN,KW,KY,KZ,LA,LC,LB,LI,LK';
		
		protected $_moduleId;
		protected $_cachedAddressFilters = array();
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentCurrencyCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('SellerAccountCurrencyCode'));
			return $options['SellerAccountCurrencyCode'];
		}
		
		protected function buildXmlRequest($request, $values)
		{
			$xml = '<?xml version="1.0" encoding="UTF-8" ?>';
			$xml .= '<' . $request .' xmlns="http://checkout.google.com/schema/2">';
			$xml .= scXml::buildXmlValues($values);
			$xml .= '</' . $request . '>';
			return $xml;
		}
		
		/**
		 * Return all active shipping methods
		 * @return array array('ShippingMethodDisplayName' => array(ShippingZoneID, ShippingZoneID, ...), ...)
		 */
		protected function getShippingMethodsAndZones()
		{
			$result = array();
			
			$shippingModules = scModulesFactory::getActiveShippingModules();
			
			foreach ($shippingModules as $shippingModule) {
				$shippingMethods = $shippingModule['object']->getShippingMethods(); // All the methods supported by the module
				foreach ($shippingModule['ShippingModuleActiveMethods'] as $shippingMethodCode => $activeMethod) {
					if (empty($activeMethod['ShippingMethodDisplayName'])) {
						$activeMethod['ShippingMethodDisplayName'] = $shippingMethods[$shippingMethodCode];
					}
					if (isset($result[$activeMethod['ShippingMethodDisplayName']])) {
						$result[$activeMethod['ShippingMethodDisplayName']][] = $shippingModule['ShippingZoneID'];
						continue;
					}
					$result[$activeMethod['ShippingMethodDisplayName']] = array($shippingModule['ShippingZoneID']);
				}
			}
			return $result;
		}
		
		/**
		 * Return merchant-calculated-shipping XML subtree
		 * 
		 * @return string the XML
		 */
		protected function getShippingMethodsXml($options)
		{
			$result = '';
			$methods = $this->getShippingMethodsAndZones();
			foreach ($methods as $method => $shippingZoneIds) {
				$values = array('merchant-calculated-shipping' => array(
					'<attributes>' => array('name' => $method),
					'price' => array('' => $options['BackupShippingAmount'], '<attributes>' => array('currency' => $options['SellerAccountCurrencyCode'])),
					'address-filters' => array(scXml::OLD_XML => $this->getShippingZoneAddressFilter($shippingZoneIds))
				));
				
				// Shipping restrictions are applied when the merchant calculation fails
				if (!$options['UseBackupShippingAmount']) {
					$values['merchant-calculated-shipping']['shipping-restrictions'] =
						array('allowed-areas' => array('us-zip-area' => array('zip-pattern' => '00000')));
				}
				if ($options['DisallowUSPOBox']) {
					$values['merchant-calculated-shipping']['address-filters']['allow-us-po-box'] = 'false';
				}
				$result .= scXml::buildXmlValues($values);
			}
			
			return $result;
		}
		
		/**
		 * Returns address filters for the selected shipping zone
		 * The address filters designate areas which the shipping zone can ship to
		 * The merchant calculations won't be called for areas outside the allowed address-filters
		 * 
		 * The results of the method are cached
		 * 
		 * @param int $shippingZoneIds
		 * @return string the address filter XML
		 */
		public function getShippingZoneAddressFilter($shippingZoneIds)
		{
			if (count($shippingZoneIds) == 0)
				throw new scPaymentProcessException(_i('No shipping zones for shipping zone address filter.'));
			
			// Check if any of the zones is NULL - that means that it is world-wide
			foreach ($shippingZoneIds as $shippingZoneId) {
				if ($shippingZoneId === null)
					return scXml::buildXmlValues(array('allowed-areas' => array('world-area' => '')));
			}
			
//			if (isset($this->_cachedAddressFilters[$shippingZoneId]))
//				return $this->_cachedAddressFilters[$shippingZoneId];
			$shippingZoneIds = array_map('intval', $shippingZoneIds);
			$sql = 'SELECT CountryCode2 FROM Countries JOIN ShippingZoneCountries USING(CountryID) WHERE ShippingZoneID IN (' . implode(',', $shippingZoneIds) . ')';
			$countryCodes = miUtil::getDBArray($sql, 'CountryCode2', 'CountryCode2');
			
			// Only leave the supported country codes
			$countryCodes = array_intersect($countryCodes, explode(',', self::$supportedShippingCountryCodes));
			
			// Currenly only US state codes are support by Google Checkout
			$sql = 'SELECT CountryCode2,StateCode FROM ShippingZoneStates JOIN States USING(StateID) JOIN Countries USING(CountryID) WHERE ShippingZoneID IN (' . implode(',', $shippingZoneIds) . ') AND CountryCode2="US"';
			$stateCodes = miUtil::getDBArray($sql, 'StateCode', 'CountryCode2');
			
			// If there are no countries and states - nothing is allowed
			if ((count($countryCodes) == 0) and (count($stateCodes) == 0))
				return scXml::buildXmlValues((array('allowed-areas' => array('us-zip-area' => array('zip-pattern' => '00000')))));
			
			$addressFilters = scXml::buildXmlValues(array('allowed-areas' => array(scXml::OLD_XML => $this->getAreaXml($countryCodes, $stateCodes))));
			
//			$this->_cachedAddressFilters[$shippingZoneId] = $addressFilters;
			return $addressFilters;
		}
		
		/**
		 * Return area XML
		 * Only US state codes are used, for other countries they are ignored
		 * 
		 * @param string $name
		 * @param array $countryCodes
		 * @param array $stateCodes
		 * @return string the xml
		 */
		protected function getAreaXml($countryCodes, $stateCodes)
		{
			// Get the US state filters and remove the US states from $stateCodes
			$usStateFilters = $this->getUSStateAreaAddressFilters($stateCodes);
			$allCountries = array_merge($countryCodes, $stateCodes);
			
			$addressFilters = $usStateFilters;
			foreach (array_unique($allCountries) as $countryCode) {
				$addressFilters .= scXml::buildXmlValues(array('postal-area' => array('country-code' => $countryCode)));
			}
			return $addressFilters;
		}
		
		/**
		 * Return us-state-area address filter
		 * Removes US state codes from the passed in array
		 * 
		 * @param array $stateCodes array(CountryCode2 => StateCode)
		 * @return string the XML with us-state-area
		 */
		protected function getUSStateAreaAddressFilters(&$stateCodes)
		{
			$stateAreas = '';
			foreach ($stateCodes as $stateCode => $countryCode2) {
				if ($countryCode2 != 'US')
					continue;
				
				$stateAreas .= scXml::buildXmlValues(array('us-state-area' => array('state' => $stateCode)));
				unset($stateCodes[$stateCode]);
			}
			return $stateAreas;
		}
		
		public function checkout()
		{
			$payCurrencyCode = $this->getPaymentCurrencyCode();
			if (is_null($payCurrencyCode))
				$payCurrencyCode = scCurrencyUtil::getBaseCurrencyCode();
			
			$cart = scCart::getCart();
			$cartInfo = $cart->getCustomerInfo();
			$cartInfo['CustomerID'] = ICheckoutModule::GUEST_USER_ID;
			$cartInfo['ShippingModuleAndMethod'] = '';
			$cartInfo['PaymentModuleID'] = 0;
			$cart->setCustomerInfo($cartInfo);
			
			$cart->getOrderTotal()->deleteOrderTotalLinesByType(IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING);
			$cart->getOrderTotal()->deleteOrderTotalLinesByType(IOrderTotalModule::ORDER_TOTAL_TYPE_TAX);
			
			$payAmount = $cart->getOrderTotal()->getGrandTotalInCurrency(new scCurrency($payCurrencyCode));
			$checkoutModuleId = scUtil::getDbValue('CheckoutModules', 'CheckoutModuleID', 'ModuleID', $this->_moduleId);
			scOrdersInProgress::singleton()->createOrder($payAmount, $payCurrencyCode, $cart, $checkoutModuleId);
			
			try {
				$result = $this->processPayment($payAmount, $payCurrencyCode, $cart, $_POST);
			} catch (miException $exception) {
				scPaymentProcess::handlePaymentProcessException($exception, scPaymentProcess::CALL_TYPE_DIRECT);
			}
			
			$cart->incHashSeed();
			
			scPaymentProcess::handlePaymentProcessResult($result, $this->_moduleId, scPaymentProcess::CALL_TYPE_DIRECT);
		}
		
		/**
		 * Retrieve XML tax tables
		 * 
		 * @see http://code.google.com/apis/checkout/developer/Google_Checkout_XML_API_Taxes.html
		 */
		protected function getTaxTablesXml()
		{
			$taxRates = scTax::getAllTaxRates();
			if (empty($taxRates))
				return '';
			
			// Group them by tax class
			$taxClasses = array();
			foreach ($taxRates as $taxRate) {
				if ($taxRate['TaxRate'] == 0)
					continue;
				
				if (!isset($taxClasses[$taxRate['TaxClassID']]))
					$taxClasses[$taxRate['TaxClassID']] = array();
				$taxClasses[$taxRate['TaxClassID']][] = $taxRate;
			}
			
			$taxTablesXml = '';
			foreach ($taxClasses as $taxClassId => $taxClassRates) {
				
				$alternateTaxRulesXml = '';
				foreach ($taxClassRates as $taxClassRate) {
					$sql = 'SELECT CountryCode2 FROM Countries JOIN TaxZoneCountries USING(CountryID) WHERE TaxZoneID="' . sql_escape_string($taxClassRate['TaxZoneID']) . '"';
					$countryCodes = miUtil::getDBArray($sql, 'CountryCode2', 'CountryCode2');
					
					// Currenly only US state codes are support by Google Checkout 
					$sql = 'SELECT CountryCode2,StateCode FROM TaxZoneStates JOIN States USING(StateID) JOIN Countries USING(CountryID)
						WHERE TaxZoneID="' . sql_escape_string($taxClassRate['TaxZoneID']) . '" AND CountryCode2="US"'; // GCO supports only US states
					$stateCodes = miUtil::getDBArray($sql, 'StateCode', 'CountryCode2');
					
					if (empty($countryCodes) && empty($stateCodes)) {
						// The zone is probably for a non-US state.
						scLog::singleton()->warning(sprintf(_t('Tax zone "%s" cannot be used in Google Checkout.'), $taxClassRate['TaxZoneName']), scLog::CATEGORY_PAYMENT);
						continue;
					}
					
					$taxAreaXml = $this->getAreaXml($countryCodes, $stateCodes);
					$alternateTaxRulesXml .= scXml::buildXmlValues(array(
						'alternate-tax-rule' => array('rate' => round($taxClassRate['TaxRate'] / 100, 2), 'tax-areas' => array(scXml::OLD_XML => $taxAreaXml)
					)));
				}
				
				$taxTablesXml .= scXml::buildXmlValues(array('alternate-tax-table' => array(
					'<attributes>' => array('name' => 'TaxClass' . $taxClassId, 'standalone' => 'true'),
					'alternate-tax-rules' => array(scXml::OLD_XML => $alternateTaxRulesXml)
				)));
			}
			
			return scXml::buildXmlValues(array('alternate-tax-tables' => array(scXml::OLD_XML => $taxTablesXml)));
		}
		
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantKey', 'MerchantID', 'SellerAccountCurrencyCode', 'Mode'));
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$calculateUrl = 'modules/checkout/google/calculate.php?gkey=' . urlencode($options['MerchantKey']) . '&gid=' . urlencode($options['MerchantID']);
			$values = array(
				'shopping-cart' => array('items' => array(),
					'merchant-private-data' => array('OrderID' => $orderId, 'SessionID' => session_id(), 'CurrencyCode' => $options['SellerAccountCurrencyCode'])),
				'checkout-flow-support' => array('merchant-checkout-flow-support' => array(
					'edit-cart-url' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_ABSOLUTE),
					'continue-shopping-url' => scPaths::getSecureUrl('modules/checkout/google/return.php?OrderID=' . $orderId),
					'tax-tables' => array(scXml::OLD_XML => $this->getTaxTablesXml()),
					'shipping-methods' => array(),
					'merchant-calculations' => array('merchant-calculations-url' => scPaths::getSecureUrl($calculateUrl))
//					'request-buyer-phone-number' => 'true'
/*					'flat-rate-shipping' => array(
							'<attributes>' => array('name' => 'Test shipping'),
							'price' => array('' => '111', '<attributes>' => array('currency' => 'GBP')),
							'shipping-restrictions' => array('allowed-areas' =>
								array('postal-area' => array('country-code' => 'US', 'postal-code-pattern' => '90002')))
							)*/
				)),
				'order-processing-support' => array('request-initial-auth-details' => 'true')
			);
			
			$values['checkout-flow-support']['merchant-checkout-flow-support']['shipping-methods'][scXml::OLD_XML] =
				$this->getShippingMethodsXml($options);
			
			$items = $cart->getItems();
			$currency = new scCurrency($options['SellerAccountCurrencyCode']);
			$itemsXml = '';
			foreach ($items as $item) {
				// Find the tax classes
				$taxClasses = scTax::getProductTaxClasses($item->getProductId());
				if (count($taxClasses) > 1)
					throw new miException(_t('Google Checkout supports only one tax class per product.'));
				$taxClass = reset($taxClasses);
				
				$itemValues = array(
					'item-name' => $item->getName(),
					'item-description' => '',
					'unit-price' => array(
						'' => $item->getPriceObj()->getAmountInCurrency($currency),
						'<attributes>' => array('currency' => $options['SellerAccountCurrencyCode'])),
					'quantity' => $item->getQuantity(),
					'merchant-item-id' => $item->getCartItemCode(),
				);
				if ($taxClass)
					$itemValues['tax-table-selector'] = 'TaxClass' . $taxClass;
				$itemsXml .= scXml::buildXmlValues(array('item' => $itemValues));
			}
			
			$lines = $cart->getOrderTotal()->getOrderTotalLines();
			foreach ($lines as $line) {
				if (($line->getType() == IOrderTotalModule::ORDER_TOTAL_TYPE_SUBTOTAL) or ($line->getType() == IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING)) 
					continue;
				
				$price = scPrice::factory($line->getAmount());
				$itemsXml .= scXml::buildXmlValues(array('item' => array(
					'item-name' => $line->getDescription(),
					'item-description' => '',
					'unit-price' => array(
						'' => $price->getAmountInCurrency($currency),
						'<attributes>' => array('currency' => $options['SellerAccountCurrencyCode'])),
					'quantity' => 1,
				)));
			}
			
			$values['shopping-cart']['items'][scXml::OLD_XML] =  $itemsXml;
			
			$xmlRequest = $this->buildXmlRequest('checkout-shopping-cart', $values);
			
			// Sandbox / Production mode
			if ($options['Mode'] == self::MODE_SANDBOX)
				$url = 'https://sandbox.google.com/checkout/cws/v2/Merchant/' . $options['MerchantID'] . '/merchantCheckout';
			else
				$url = 'https://checkout.google.com/cws/v2/Merchant/' . $options['MerchantID'] . '/merchantCheckout';
			
			$httpCode = 200;
			try { 
				$responseXml = scHttp::post($url, $xmlRequest, array('username' => $options['MerchantID'], 'password' => $options['MerchantKey']));
			} catch (scHttpException $exception) {
				$responseXml = $exception->getResponse();
				$httpCode = $exception->getHttpCode();
			}
			
			try {
				// Try to find the actual error message, even if the HTTP code is not 200
				$xml = new SimpleXMLElement($responseXml);
				if ($xml->getName() == 'error') {
					$message = (string)$xml->{'error-message'};
					throw new miException(sprintf(_t('Google Checkout error: "%s".'), $message));
				}
			} catch (Exception $exception) {
				if ($exception instanceof miException)
					throw $exception;
				
				// We get here if the XML returned from GCO is invalid
				
				// If Google Checkout returned HTTP 200 OK, but invalid XML say so
				if ($httpCode == 200)
					throw new miException(sprintf(_i('Invalid XML from Google Checkout: "%s".'), $responseXml));
				
				// Otherwise XML is invalid and there is HTTP error, show the HTTP error
				throw new miException(sprintf(_i('Invalid HTTP response code from Google Checkout: "%d".'), $httpCode));
			}
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect((string)$xml->{'redirect-url'}, array(), 'get');
			return $result; 
		}
		
		public function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('OrderID'));
			
			$result = new scPaymentProcessResult($params['OrderID']);
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_AT_PROCESSOR);
			return $result;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'MerchantID',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'MerchantKey',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'BackupShippingAmount',
					'validator' => array('scValidatorDecimal', 'scValidatorNoEmpty')
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'UseBackupShippingAmount',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'DisallowUSPOBox',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'SellerAccountCurrencyCode',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('Mode')->setRadioButtons(array(self::MODE_PRODUCTION => _t('Production'), self::MODE_SANDBOX => _t('Sandbox')), self::MODE_SANDBOX);
			$options->getOptionField('SellerAccountCurrencyCode')->setRadioButtons(array('USD' => 'USD', 'GBP' => 'GBP'), 'USD');
			$options->setTemplateFile('modules/checkout/google/options.tmpl');
			return $options;
		}
	}
?>