<?php
//	ini_set('error_log', '/tmp/notify.log');
//	ini_set('error_reporting', E_ALL|E_STRICT);
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Notification Callback from Google Checkout
	 * 
	 * For more info:
	 * http://code.google.com/apis/checkout/developer/Google_Checkout_XML_API_Notification_API.html
	 */
	
	require_once(dirname(__FILE__) . '/../../../include/miphpf/Init.php');
	require_once(dirname(__FILE__) . '/../../../include/sc/Init.php');
	
	class scPaymentMethodGoogleCheckoutNotify {
		const GCO_MODULE_NAME = 'com.summercart.checkout.google';
		const PASSWORD_LENGTH = 8;
		
		protected function newOrder($xml)
		{
			scSession::singleton()->openSession((string)($xml->{'shopping-cart'}->{'merchant-private-data'}->SessionID));
			
			$orderId = (string)$xml->{'shopping-cart'}->{'merchant-private-data'}->OrderID;
			$order = new scOrder($orderId);
			
			// Update the new order billing address
			$contactName = (string)$xml->{'buyer-billing-address'}->{'contact-name'};
			list($firstName, $lastName) = scUtil::splitFirstLastName($contactName);
			if (isset($xml->{'buyer-billing-address'}->{'structured-name'})) {
				$firstName = (string)$xml->{'buyer-billing-address'}->{'structured-name'}->{'first-name'};
				$lastName = (string)$xml->{'buyer-billing-address'}->{'structured-name'}->{'last-name'};
			}
			$order->set('OrderBillingFirstName', $firstName);
			$order->set('OrderBillingLastName', $lastName);
			$order->set('OrderBillingCompany', (string)$xml->{'buyer-billing-address'}->{'company-name'});
			$order->set('OrderBillingPhone', (string)$xml->{'buyer-billing-address'}->phone);
			$order->set('OrderBillingFax', (string)$xml->{'buyer-billing-address'}->fax);
			$order->set('OrderBillingAddress1', (string)$xml->{'buyer-billing-address'}->address1);
			$order->set('OrderBillingAddress2', (string)$xml->{'buyer-billing-address'}->address2);
			$order->set('OrderBillingCity', (string)$xml->{'buyer-billing-address'}->city);
			$order->set('OrderBillingState', (string)$xml->{'buyer-billing-address'}->region);
			$order->set('OrderBillingCountry', (string)$xml->{'buyer-billing-address'}->{'country-code'});
			$order->set('OrderBillingZip', (string)$xml->{'buyer-billing-address'}->{'postal-code'});
			
			$order->set('OrderCustomerEmail', (string)$xml->{'buyer-billing-address'}->email);
			
			// Update the new order shipping address
			$contactName = (string)$xml->{'buyer-shipping-address'}->{'contact-name'};
			list($firstName, $lastName) = scUtil::splitFirstLastName($contactName);
			if (isset($xml->{'buyer-shipping-address'}->{'structured-name'})) {
				$firstName = (string)$xml->{'buyer-shipping-address'}->{'structured-name'}->{'first-name'};
				$lastName = (string)$xml->{'buyer-shipping-address'}->{'structured-name'}->{'last-name'};
			}
			$order->set('OrderShippingFirstName', $firstName);
			$order->set('OrderShippingLastName', $lastName);
			$order->set('OrderShippingCompany', (string)$xml->{'buyer-shipping-address'}->{'company-name'});
			$order->set('OrderShippingPhone', (string)$xml->{'buyer-shipping-address'}->phone);
			$order->set('OrderShippingFax', (string)$xml->{'buyer-shipping-address'}->fax);
			$order->set('OrderShippingAddress1', (string)$xml->{'buyer-shipping-address'}->address1);
			$order->set('OrderShippingAddress2', (string)$xml->{'buyer-shipping-address'}->address2);
			$order->set('OrderShippingCity', (string)$xml->{'buyer-shipping-address'}->city);
			$order->set('OrderShippingState', (string)$xml->{'buyer-shipping-address'}->region);
			$order->set('OrderShippingCountry', (string)$xml->{'buyer-shipping-address'}->{'country-code'});
			$order->set('OrderShippingZip', (string)$xml->{'buyer-shipping-address'}->{'postal-code'});
			
			// Check if the billing and shipping addresses are the same
			$order->set('OrderShipToBillingAddress', $this->isShippingAddressSameAsBilling($order));
			
			// Update the new order shipping method
			$calculatedShippingAdjustment = $xml->{'order-adjustment'}->shipping->{'merchant-calculated-shipping-adjustment'};
			$shippingMethodName = (string)$calculatedShippingAdjustment->{'shipping-name'};
			$shippingMethodAmount = (string)$calculatedShippingAdjustment->{'shipping-cost'};
			
			// Check that the shipping cost currency is the same as the order currency
			if ($order->get('OrderCurrencyCode') != (string)$calculatedShippingAdjustment->{'shipping-cost'}->attributes()->currency) {
				throw new miException(sprintf(_t('Shipping cost currency mismatch. Expected "%s", found "%s".'),
					$order->get('OrderCurrencyCode'), (string)$calculatedShippingAdjustment->{'shipping-cost'}->attributes()->currency));
			}
			
			// The tax check
			$totalTax = (string)$xml->{'order-adjustment'}->{'total-tax'};
			if ($order->get('OrderCurrencyCode') != (string)$xml->{'order-adjustment'}->{'total-tax'}->attributes()->currency) {
				throw new miException(sprintf(_t('Tax currency mismatch. Expected "%s", found "%s".'),
					$order->get('OrderCurrencyCode'), (string)$xml->{'order-adjustment'}->{'total-tax'}->attributes()->currency));
			}
			
			// Check: order_amount_from_gco - shipping - tax = our_order_amount
			$orderTotal = $xml->{'order-total'};
			scPaymentProcess::checkPaymentToOrder($orderId, ((string)$orderTotal) - $shippingMethodAmount - $totalTax, (string)$orderTotal->attributes()->currency);
			$order->set('OrderTotal', $order->get('OrderTotal') + $shippingMethodAmount + $totalTax);
			$order->set('OrderTotalPayAmount', (string)$orderTotal);
			
			list($shippingModuleId, $shippingMethodCode) = $this->getShippingModuleIdAndMethodCode($shippingMethodName);
			$order->set('ShippingModuleID', $shippingModuleId);
			$order->set('OrderShippingMethodCode', $shippingMethodCode);
			
			try {
				list($baseShippingModuleName, $baseShippingMethodName) = scOrdersInProgress::singleton()->getShippingModuleAndMethodNames($shippingModuleId, $shippingMethodCode);
				$order->set('OrderShippingModuleName', $baseShippingModuleName);
				$order->set('OrderShippingMethodName', $baseShippingMethodName);  
			} catch (miException $exception) {
				scLog::singleton()->warning($exception->getMessage(), scLog::CATEGORY_PAYMENT);
			}
			
			// If we can't get the base shipping method name, at least set what we have received
			if ($order->get('OrderShippingMethodName') == '')
				$order->set('OrderShippingMethodName', $shippingMethodName);  
			
			// Set state
			$order->set('OrderStatus', $this->getOrderStatus((string)$xml->{'fulfillment-order-state'}, (string)$xml->{'financial-order-state'}));
			
			// Save the google order number
			$order->set('OrderPaymentTransactionNo', (string)$xml->{'google-order-number'});  
			
			$order->set('CustomerID', $this->getCustomerId((string)$xml->{'buyer-billing-address'}->email, $order));
			
			$order->update();
			
//			$buyerId = (string)$xml->{'buyer-id'};
			
			// Add shipping and tax order total lines
			$this->insertOrderTotalLine($orderId, $shippingMethodAmount, _t('Shipping'), IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING);
			$this->insertOrderTotalLine($orderId, $totalTax, _t('Tax'), IOrderTotalModule::ORDER_TOTAL_TYPE_TAX);
			
			// Create transaction
			scPaymentProcess::createTransaction($order, scModulesFactory::getModuleIdByName(self::GCO_MODULE_NAME),
				(string)$orderTotal, (string)$orderTotal->attributes()->currency,
				(string)$xml->{'google-order-number'}, $this->getPaymentStatus((string)$xml->{'financial-order-state'}),
				(string)$xml->{'financial-order-state'});
		}
		
		/**
		 * Returns the customer id
		 * If the customer doesn't exists creates new customer
		 * FIXME: Use scCustomerDomainObject->getCustomerIdForNewOrder()
		 * 
		 * @param string $email
		 * @param scOrder $order
		 * @return int|false the customer id, or false if not found
		 */
		protected function getCustomerId($email, scOrder $order)
		{
			// Check if customer with the given email exists
			$customerId = scUtil::getDbValue('Customers', 'CustomerID', 'CustomerEmail', $email, false);
			
			$record = new scCustomerDomainObject();
			if ($customerId !== false)
				$record->read($customerId);
			else {
				$record->set('CustomerEmail', $email);
				
				// Generate random password no longer than PASSWORD_LENGTH
				$password = md5(uniqid(rand(), true));
				if (strlen($password) > self::PASSWORD_LENGTH)
					$password = substr($password, 0, self::PASSWORD_LENGTH);
				
				$auth = scCustomerAuth::singleton();
				$record->set('CustomerPassword', $auth->hashPassword($password));
				$record->set('CustomerPasswordHashMode', scSettings::getSettingValue(scSettings::SETTING_PASSWORD_HASH_MODE));
				$record->set('CustomerStatus', scCustomerStatuses::CUSTOMER_STATUS_ACTIVE);
				$record->set('CustomerGroupID', scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP));
			}
			
			// If it is existing customer - update
			if ($customerId !== false) {
				$record->update();
				return $customerId;
			}
			
			// If it is new customer - insert
			$customerId = $record->insert();
			
			$address = new scCustomerAddress();
			$address->set('CustomerID', $customerId);
			$address->set('CustomerAddressFirstName', $order->get('OrderBillingFirstName'));
			$address->set('CustomerAddressLastName', $order->get('OrderBillingLastName'));
			$address->set('CustomerAddressCompany', $order->get('OrderBillingCompany'));
			$address->set('CustomerAddressPhone', $order->get('OrderBillingPhone'));
			$address->set('CustomerAddressFax', $order->get('OrderBillingFax'));
			$address->set('CustomerAddressLine1', $order->get('OrderBillingAddress1'));
			$address->set('CustomerAddressLine2', $order->get('OrderBillingAddress2'));
			$address->set('CustomerAddressCity', $order->get('OrderBillingCity'));
			$address->set('CountryID', scCountry::getCountryIdByCode2($order->get('OrderBillingCountry')));
			$address->set('StateID', scCountry::getStateId($record->get('CustomerBillingCountry'), $order->get('OrderBillingState')));
			$address->set('CustomerAddressZip', $order->get('OrderBillingZip'));
			$address->set('CustomerAddressIsDefaultBilling', 1);
			$address->set('CustomerAddressIsDefaultShipping', $this->isShippingAddressSameAsBilling($order));
			$address->insert();
			
			if (!$this->isShippingAddressSameAsBilling($order)) {
				$address = new scCustomerAddress();
				$address->set('CustomerID', $customerId);
				$address->set('CustomerAddressFirstName', $order->get('OrderShippingFirstName'));
				$address->set('CustomerAddressLastName', $order->get('OrderShippingLastName'));
				$address->set('CustomerAddressCompany', $order->get('OrderShippingCompany'));
				$address->set('CustomerAddressPhone', $order->get('OrderShippingPhone'));
				$address->set('CustomerAddressFax', $order->get('OrderShippingFax'));
				$address->set('CustomerAddressLine1', $order->get('OrderShippingAddress1'));
				$address->set('CustomerAddressLine2', $order->get('OrderShippingAddress2'));
				$address->set('CustomerAddressCity', $order->get('OrderShippingCity'));
				$address->set('CountryID', scCountry::getCountryIdByCode2($order->get('OrderShippingCountry')));
				$address->set('StateID', scCountry::getStateId($record->get('CustomerShippingCountry'), $order->get('OrderShippingState')));
				$address->set('CustomerAddressZip', $order->get('OrderShippingZip'));
				$address->set('CustomerAddressIsDefaultBilling', 0);
				$address->set('CustomerAddressIsDefaultShipping', 1);
				$address->insert();
			}
			
			// Send notification email
			$notifications = new scEmailNotifications();
			$notifications->sendSignupNotification($customerId);
			return $customerId;
		}
		
		/**
		 * Checks if the billing and shipping addresses are the same
		 * FIXME: Use scOrder->isShippingAddressSameAsBilling
		 *
		 * @param scOrder $order
		 * @return boolean true if the shipping and billing addresses are the same
		 */
		protected function isShippingAddressSameAsBilling(scOrder $order)
		{
			$checkFields = array(
				'OrderBillingFirstName' => 'OrderShippingFirstName',
				'OrderBillingLastName' => 'OrderShippingLastName',
				'OrderBillingCompany' => 'OrderShippingCompany',
				'OrderBillingPhone' => 'OrderShippingPhone',
				'OrderBillingFax' => 'OrderShippingFax',
				'OrderBillingAddress1' => 'OrderShippingAddress1',
				'OrderBillingAddress2' => 'OrderShippingAddress2',
				'OrderBillingCity' => 'OrderShippingCity',
				'OrderBillingState' => 'OrderShippingState',
				'OrderBillingCountry' => 'OrderShippingCountry',
				'OrderBillingZip' => 'OrderShippingZip',
			);
			foreach ($checkFields as $field1 => $field2) {
				if ($order->get($field1) != $order->get($field2))
					return false;
			}
			return true;
		}
		
		/**
		 * Inserts new order total line
		 * TODO: Use scOrderService
		 * 
		 * @param int $orderId the order id
		 * @param string $amount the amount
		 * @param string $name the name of the order total line
		 * @param int $orderTotalType a IOrderTotalModule::ORDER_TOTAL_TYPE_* constant
		 */
		protected function insertOrderTotalLine($orderId, $amount, $name, $orderTotalType)
		{
			$sql = 'SELECT MAX(OrderTotalLineSort) AS Sort FROM OrderTotalLines WHERE OrderID=?';
			$result = miStaticDBUtil::execSelect($sql, array($orderId));
			$sort = $result[0]['Sort'];
			
			$values = array(
					'OrderID' => $orderId,
					'OrderTotalLineType' => $orderTotalType,
					'OrderTotalLineName' => $name,
					'OrderTotalLineAmount' => $amount,
					'OrderTotalLineSort' => $sort
				);
			miStaticDBUtil::execInsert('OrderTotalLines', $values);
		}
		
		/**
		 * Find the shipping module and method code by shipping method name
		 * Returns the shipping module id and the shipping method code
		 * 
		 * @param string $shippingMethodName the shipping method name
		 * @return array array($shippingModuleId,$shippingMethodCode)
		 */
		protected function getShippingModuleIdAndMethodCode($shippingMethodName)
		{
			$shippingModules = scModulesFactory::getActiveShippingModules();
			
			foreach ($shippingModules as $shippingModule) {
				$shippingModuleId = $shippingModule['ShippingModuleID'];
				$module = $shippingModule['object'];
				
				$shippingMethods = $module->getShippingMethods();
				
				foreach ($shippingMethods as $shippingMethodCode => $shippingMethod) {
					if (!isset($shippingModule['ShippingModuleActiveMethods'][$shippingMethodCode]))
						continue;
					
					if ($shippingMethod == $shippingMethodName)
						return array($shippingModuleId, $shippingMethodCode);
				}
			}
			
			// Not found
			return array(0, '');
		}
		
		protected function getOrderStatus($fulfillmentState, $financialOrderState)
		{
			switch ($fulfillmentState) {
				case 'NEW':
					if ($this->getPaymentStatus($financialOrderState) == scPaymentStatus::PAYMENT_STATUS_PENDING)
						return scOrderStatus::ORDER_STATUS_PAYMENT_FAILED;
					
					return scOrderStatus::ORDER_STATUS_NEW;
				case 'PROCESSING':
					return scOrderStatus::ORDER_STATUS_IN_PROGRESS;
				case 'DELIVERED':
					return scOrderStatus::ORDER_STATUS_DELIVERED;
				case 'WILL_NOT_DELIVER':
					return scOrderStatus::ORDER_STATUS_CANCELLED;
				
				// If the state is unknown put the order on hold
				default:
					return scOrderStatus::ORDER_STATUS_ON_HOLD;
			}
		}
		
		protected function getPaymentStatus($financialOrderState)
		{
			switch ($financialOrderState) {
				case 'REVIEWING':
				case 'CHARGEABLE':
				case 'CHARGING':
					return scPaymentStatus::PAYMENT_STATUS_PENDING;
				case 'CHARGED':
					return scPaymentStatus::PAYMENT_STATUS_CAPTURE;
				case 'PAYMENT_DECLINED':
				case 'CANCELLED':
				case 'CANCELLED_BY_GOOGLE':
					return scPaymentStatus::PAYMENT_STATUS_DECLINED;
				default:
					return scPaymentStatus::PAYMENT_STATUS_OTHER;
			}
		}
		
		protected function orderStateChangeNotification($xml)
		{
			$moduleId = scModulesFactory::getModuleIdByName(self::GCO_MODULE_NAME);
			$transactionCode = (string)$xml->{'google-order-number'};
			
			$orderId = scTransaction::getOrderIdByTransactionCode($transactionCode, $moduleId);
			$order = new scOrder($orderId);
			
			if ((string)$xml->{'new-financial-order-state'} != (string)$xml->{'previous-financial-order-state'}) {
				$note = (string)$xml->{'new-financial-order-state'};
				if ((string)$xml->reason)
					$note .= ' (' . $xml->reason . ')';
				scPaymentProcess::createTransaction($order, $moduleId, 0, '',
					$transactionCode, $this->getPaymentStatus((string)$xml->{'new-financial-order-state'}),	$note);
			}
			
			if ((string)$xml->{'new-fulfillment-order-state'} != (string)$xml->{'previous-fulfillment-order-state'}) {
				$order->updateOrderStatus($this->getOrderStatus((string)$xml->{'new-fulfillment-order-state'}, (string)$xml->{'new-financial-order-state'}));
				
				$note = sprintf(_t('New fulfillment state: %s.'), $xml->{'new-fulfillment-order-state'});
				scPaymentProcess::createTransaction($order, $moduleId, 0, '',
					$transactionCode, scPaymentStatus::PAYMENT_STATUS_NONE,	$note);
			}
		}
		
		protected function riskInformationNotification($xml)
		{
			$moduleId = scModulesFactory::getModuleIdByName(self::GCO_MODULE_NAME);
			$transactionCode = (string)$xml->{'google-order-number'};
			
			$orderId = scTransaction::getOrderIdByTransactionCode($transactionCode, $moduleId);
			$order = new scOrder($orderId);
			
			$riskInfo = $xml->{'risk-information'};
			
			$eligibleForProtection = (string)$riskInfo->{'eligible-for-protection'};
			$notes = sprintf(_t('Eligible for protection: %s.'), $eligibleForProtection ? _t('Yes') : _t('No'));
			$notes .= ' ' . sprintf(_t('Account age: %d days.'), $riskInfo->{'buyer-account-age'});
			$notes .= ' ' . _t('IP address: ') . $riskInfo->{'ip-address'};
			$order->set('OrderPaymentNotes', $notes);
			$order->update();
			
			// Create PaymentCreditCard record
			$this->updatePaymentCreditCard($orderId, (string)$riskInfo->{'avs-response'}, (string)$riskInfo->{'cvn-response'}, 
				str_pad($riskInfo->{'partial-cc-number'}, 16, '*', STR_PAD_LEFT));
		}
		
		protected function updatePaymentCreditCard($orderId, $avs, $cvn, $cardNum = '')
		{
			$payment = new scPaymentCreditCard($orderId);
			if (!empty($cardNum))
				$payment->set('PaymentCreditCardNumber', $cardNum);
			$payment->set('PaymentCreditCardProcessorAVS', $avs);
			$payment->set('PaymentCreditCardAVSStatus', $this->getAvsStatus($avs));
			$payment->set('PaymentCreditCardCVNStatus', $this->getCvnStatus($cvn));
			$payment->set('PaymentCreditCardProcessorCVN', $cvn);
			$payment->save();
		}
		
		protected function getCvnStatus($cvn)
		{
			$cvnMap = array(
				'M' => scPaymentCVN::CVN_STATUS_MATCH,
				'N' => scPaymentCVN::CVN_STATUS_NO_MATCH,
				'U' => scPaymentCVN::CVN_STATUS_NOT_AVAILABLE,
				'E' => scPaymentCVN::CVN_STATUS_ERROR
			);
			if (isset($cvnMap[$cvn]))
				return $cvnMap[$cvn];
			return scPaymentCVN::CVN_STATUS_UNKNOWN;
		}
		
		protected function getAvsStatus($avs)
		{
			$avsMap = array(
				'Y' => scPaymentAVS::AVS_STATUS_FULL_MATCH,
				'P' => scPaymentAVS::AVS_STATUS_ZIP_MATCH,
				'A' => scPaymentAVS::AVS_STATUS_STREET_MATCH,
				'N' => scPaymentAVS::AVS_STATUS_NO_MATCH,
				'U' => scPaymentAVS::AVS_STATUS_NOT_SUPPORTED
			);
			if (isset($avsMap[$avs]))
				return $avsMap[$avs];
			return scPaymentAVS::AVS_STATUS_UNKNOWN;
		}
		
		protected function processAmountNotification($xml, $paymentStatus, $latest, $total)
		{
			$moduleId = scModulesFactory::getModuleIdByName(self::GCO_MODULE_NAME);
			$transactionCode = (string)$xml->{'google-order-number'};
			
			$orderId = scTransaction::getOrderIdByTransactionCode($transactionCode, $moduleId);
			$order = new scOrder($orderId);
			
			$latestAmount = $xml->$latest;
			$latestCurrencyCode = (string)$latestAmount->attributes()->currency;
			
			$totalAmount = $xml->$total;
			$totalCurrencyCode = (string)$totalAmount->attributes()->currency;
			$note = sprintf(_t('Total: %s %s'), $totalAmount, $totalCurrencyCode);
			
			scPaymentProcess::createTransaction($order, $moduleId, $latestAmount, $latestCurrencyCode, $transactionCode, $paymentStatus, $note);
		}
		
		protected function chargeAmountNotification($xml)
		{
			$this->processAmountNotification($xml, scPaymentStatus::PAYMENT_STATUS_CAPTURE, 'latest-charge-amount', 'total-charge-amount');
		}
		
		protected function refundAmountNotification($xml)
		{
			$this->processAmountNotification($xml, scPaymentStatus::PAYMENT_STATUS_REFUND, 'latest-refund-amount', 'total-refund-amount');
		}
		
		protected function chargebackAmountNotification($xml)
		{
			$this->processAmountNotification($xml, scPaymentStatus::PAYMENT_STATUS_CHARGEBACK, 'latest-chargeback-amount', 'total-chargeback-amount');
		}
		
		protected function authorizationAmountNotification($xml)
		{
			$moduleId = scModulesFactory::getModuleIdByName(self::GCO_MODULE_NAME);
			$transactionCode = (string)$xml->{'google-order-number'};
			
			$orderId = scTransaction::getOrderIdByTransactionCode($transactionCode, $moduleId);
			$order = new scOrder($orderId);
			
			$amount = $xml->{'authorization-amount'};
			$currencyCode = (string)$amount->attributes()->currency;
			
			$note = sprintf(_t('Auth Expiration %s'), $xml->{'authorization-expiration-date'});
			scPaymentProcess::createTransaction($order, $moduleId, $amount, $currencyCode, $transactionCode,
				scPaymentStatus::PAYMENT_STATUS_AUTH, $note);
				
			$this->updatePaymentCreditCard($orderId, (string)$xml->{'avs-response'}, (string)$xml->{'cvn-response'});
		}
		
		/**
		 * Checks the authenticity of the notification
		 * 
		 * TODO: V1.1 check basic auth from Google
		 * 
		 * @throws miException
		 */
		protected function checkAuth()
		{
			$gkey = miGetParamDefault('gkey', '');
			$gid = miGetParamDefault('gid', '');
			
			$moduleId = scModulesFactory::getModuleIdByName(self::GCO_MODULE_NAME);
			$options = scModuleOptions::getModuleOptions($moduleId);
			
			if (($gid != $options['MerchantID']) or ($gkey != $options['MerchantKey']))
				throw new miException(_t('Merchat ID or Merchant Key do not match.'));
		}
		
		public function processNotification($xml)
		{
			$this->checkAuth();
			
			switch ($xml->getName()) {
				case 'new-order-notification':
					$this->newOrder($xml);
					break;
				
				case 'order-state-change-notification':
					$this->orderStateChangeNotification($xml);
					break;
				
				case 'risk-information-notification':
					$this->riskInformationNotification($xml);
					break;
				
				case 'charge-amount-notification':
					$this->chargeAmountNotification($xml);
					break;
				
				case 'refund-amount-notification':
					$this->refundAmountNotification($xml);
					break;
				
				case 'chargeback-amount-notification':
					$this->chargebackAmountNotification($xml);
					break;
				
				case 'authorization-amount-notification':
					$this->authorizationAmountNotification($xml);
					break;
					
				default:
					throw new miException(_t('Unknown Google Checkout notification: %s'), $xml->getName());
			}
			
		}
	}
	
//	scDebug::logVariable('/tmp/google.txt', 'Notify:', $_SERVER);
	
	try {
		$requestXml = file_get_contents('php://input');
//		scDebug::logVariable('/tmp/google.txt', '$requestXml', $requestXml);
		
		$xml = new SimpleXMLElement($requestXml);
		$gNotify = new scPaymentMethodGoogleCheckoutNotify;
		$gNotify->processNotification($xml);
	} catch (Exception $exception) {
		scLog::singleton()->error(_t('Google Checkout Exception: ') . $exception->getMessage(), scLog::CATEGORY_PAYMENT);
//		scDebug::logVariable('/tmp/google.txt', '$exception', $exception);
		
		// Send error
		header('HTTP/1.1 500 Internal Error');
		echo $exception->getMessage();
	}
?>