<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/../../payment/paypal/nvpapi.php');
	
	/**
	 * PayPal Express module button
	 */
	class scPaymentModulePayPalExpressButton implements ICheckoutModuleButton {
		protected $_moduleId;
		
		protected $_locales = array(
			'de_DE',
			'en_US',
			'en_AU',
			'nl_NL',
			'fr_FR',
			'zh_XC',
			'fr_XC',
			'es_XC',
			'zh_CN',
			'en_GB',
			'it_IT',
			'ja_JP',
			'pl_PL'
		);
		
		/**
		 * Checkout module constructor
		 */
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		/**
		 * Returns HTML for rendering the checkout button
		 */
		public function getButtonHtml($moduleName, $checkoutModuleName, $displayName)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'CheckoutButtonType', 'ApiUsername', 'ApiPassword', 'ApiSignature'));
			
			if ($options['CheckoutButtonType'] == scPayPalNvpApi::CHECKOUT_BUTTON_TYPE_DYNAMIC) {
				$dynamicButton = $this->getDynamicButton($moduleName, $checkoutModuleName, $displayName, $options);
				
				if ($dynamicButton !== false) {
					return $dynamicButton;
				}
			}
			
			return $this->getStaticButton($moduleName, $displayName);
		}
		
		/**
		 * The dynamic button includes "PAL", "Locale" and "OrderTotal" allowing PayPal to customize the apperance of the button
		 * 
		 * @param string $moduleName
		 * @param string $checkoutModuleName
		 * @param string $displayName
		 * @param array $options
		 * @return string
		 */
		protected function getDynamicButton($moduleName, $checkoutModuleName, $displayName, $options)
		{
			try {
				// Find the PAL value, checking the cache first
				if (isset($options['CachedPal']) and ($options['CachedPal'] != '')) { 
					$pal = $options['CachedPal'];
				} else {
					$ppApi = new scPayPalNvpApi($options['Mode'], $options['ApiUsername'], $options['ApiPassword'], $options['ApiSignature']);
					$palDetails = $ppApi->doGetPalDetails();
					$pal = $palDetails['PAL'];
					
					// Cache the PAL value 
					scModuleOptions::setModuleOption($this->_moduleId, 'CachedPal', $pal);
				}
			} catch (Exception $e) {
				return false;
			}
			
			$cart = scCart::getCart();
			$locale = $this->getLocale($cart);
			
			$currency = new scCurrency('USD');
			$orderTotal = $cart->getOrderTotal()->getGrandTotalInCurrency($currency);
			
			$t = new scTemplateParser;
			$t->readTemplate('modules/checkout/paypal/dynamic_button.tmpl');
			$t->assign('%%MODULENAME%%', miI18N::htmlEscape($moduleName));
			$t->assign('%%PAL%%', miI18N::htmlEscape($pal));
			$t->assign('%%ORDERTOTAL%%', miI18N::htmlEscape($orderTotal));
			$t->assign('%%LOCALE%%', miI18N::htmlEscape($locale));
			$t->assign('%%CHECKOUTMODULEDISPLAYNAME%%', miI18N::htmlEscape($displayName));
			return $t->templateParse();
		}
		
		protected function getStaticButton($moduleName, $displayName)
		{
			$t = new scTemplateParser;
			$t->readTemplate('modules/checkout/paypal/static_button.tmpl');
			$t->assign('%%MODULENAME%%', miI18N::htmlEscape($moduleName));
			$t->assign('%%SCHEME%%', scPaths::isCurrentPageSsl() ? 'https' : 'http');
			$t->assign('%%CHECKOUTMODULEDISPLAYNAME%%', miI18N::htmlEscape($displayName));
			return $t->templateParse();
		}
		
		/**
		 * Returns the locale
		 * 
		 * @param scCart $cart
		 * @return string
		 */
		protected function getLocale(scCart $cart)
		{
			$langCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$customerInfo = $cart->getCustomerInfo();
			$countryCode = strtoupper(scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']));
			
			$locale = $langCode . '_' . $countryCode;
			if (in_array($locale, $this->_locales))
				return $locale;
			
			// This locale is not supported, so just use language code
			foreach ($this->_locales as $supportedLocale) {
				if (substr($supportedLocale, 0, 2) == $langCode)
					return $supportedLocale;
			}
			
			// Use the default locale
			return '';
		}
	}
?>