<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/../../payment/paypal/nvpapi.php');
	
	class scPaymentModulePayPalExpress implements ICheckoutModule, IModuleOptions {
		protected $_moduleId;
		
		const PAYPAL_LIVE_URL = 'https://www.paypal.com/cgi-bin/webscr';
		const PAYPAL_SANDBOX_URL = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
		
		protected $_paypalCurrencies = array('AUD', 'BRL', 'CAD', 'CHF', 'CZK', 'DKK', 'EUR', 'GBP', 'HKD', 'HUF', 'ILS', 'JPY',
			'MYR', 'MXN', 'NOK', 'NZD', 'PHP', 'PLN', 'SGD', 'SEK', 'TWD', 'THB', 'USD');
		
		protected $_paymentActions = array(
			'sale' => 'Sale',
			'authorization' => 'Authorization',
//			'order' => 'Order',
		);
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentCurrencyCode()
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $this->_paypalCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		public function checkout()
		{
			try {
				if (miGetParamDefault('action', '') == 'scExecConfirmOrder') {
					$this->execConfirmOrder();
					return;
				}
				
				if (scCart::getCart()->getItemsCount() == 0) {
					throw new miException(_t('Cart must contain at least one item.'));
				}
				
				$cart = scCart::getCart();
				$cart->refreshCustomerInfo(scCustomerAuth::singleton()->getCustomer());
				
				$cartInfo = $cart->getCustomerInfo();
				$cartInfo['CustomerID'] = ICheckoutModule::GUEST_USER_ID;
				$cartInfo['ShippingModuleAndMethod'] = '';
				$cartInfo['PaymentModuleID'] = 0;
				$cart->setCustomerInfo($cartInfo);
				
//				$cart->getOrderTotal()->deleteOrderTotalLinesByType(IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING);
//				$cart->getOrderTotal()->deleteOrderTotalLinesByType(IOrderTotalModule::ORDER_TOTAL_TYPE_TAX);
				
				$payCurrencyCode = $this->getPaymentCurrencyCode();
				$payAmount = $cart->getOrderTotal()->getGrandTotalInCurrency(new scCurrency($payCurrencyCode));
				
				$checkoutModuleId = scUtil::getDbValue('CheckoutModules', 'CheckoutModuleID', 'ModuleID', $this->_moduleId);
				scOrdersInProgress::singleton()->createOrder($payAmount, $payCurrencyCode, $cart, $checkoutModuleId);
				
				$result = $this->processPayment($payAmount, $payCurrencyCode, $cart, $_POST);
			} catch (miException $exception) {
				scPaymentProcess::handlePaymentProcessException($exception, scPaymentProcess::CALL_TYPE_DIRECT);
			}
			
			$cart->incHashSeed();
			
			scPaymentProcess::handlePaymentProcessResult($result, $this->_moduleId, scPaymentProcess::CALL_TYPE_DIRECT);
		}
		
		protected function processPayment($payAmount, $payCurrencyCode, $cart, $postParams)
		{
			$custInfo = $cart->getCustomerInfo();
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PaymentAction', 'Mode', 'ApiUsername', 'ApiPassword', 'ApiSignature',
				'AllowNote', 'RequireConfirmedShippingAddress', 'LandingPage', 'UseConfirmPage'));
			
			$methods = $this->getShippingMethodsAndZones($payCurrencyCode, $custInfo['CustomerShippingCountry'], $custInfo['CustomerShippingState'], $cart->isAllIntangible(), true);
			// PayPal does not support different tax for each of the shipping methods during SetExpressCheckout
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$ppApi = new scPayPalNvpApi($options['Mode'], $options['ApiUsername'], $options['ApiPassword'], $options['ApiSignature']);
			$nvpResult = $ppApi->doSetExpressCheckout($cart, $payCurrencyCode, $options['PaymentAction'], $orderId, session_id(), $methods, $options['AllowNote'],
				$options['RequireConfirmedShippingAddress'], $isoLangCode, $options['LandingPage']);
			
			$url = ($options['Mode'] == scPayPalNvpApi::MODE_PRODUCTION) ? self::PAYPAL_LIVE_URL : self::PAYPAL_SANDBOX_URL;
			$params = array(
				'cmd' => '_express-checkout',
				'token' => $nvpResult['TOKEN'],
			);
			if (!$options['UseConfirmPage'])
				$params['useraction'] =  'commit';	// User commits to buying without additional review page
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($url, $params, 'get');
			return $result;
		}
		
		
		public function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('token', 'PayerID'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PaymentAction', 'Mode', 'ApiUsername', 'ApiPassword', 'ApiSignature', 'UseConfirmPage'));
			
			// Retrieve the checkout details from PayPal
			$ppApi = new scPayPalNvpApi($options['Mode'], $options['ApiUsername'], $options['ApiPassword'], $options['ApiSignature']);
			$expressCheckoutDetails = $ppApi->doGetExpressCheckoutDetails($params['token']);
			
			// Update the order according to customer selection
			$this->updateOrder($expressCheckoutDetails);
			
			// TODO: Update the customer addresses with the information from PayPal
			
			if ($options['UseConfirmPage']) {
				$this->showConfirm();
				die;
			}
			
			// Do finalize the order
			return $this->finalizeOrder($ppApi, $expressCheckoutDetails, $options['PaymentAction'], $params['token'], $expressCheckoutDetails['SHIPPINGAMT']);
		}
		
		protected function finalizeOrder(scPayPalNvpApi $ppApi, $expressCheckoutDetails, $paymentAction, $token, $shippingAmount)
		{
			$payerId = $expressCheckoutDetails['PAYERID'];
			$currencyCode = $expressCheckoutDetails['CURRENCYCODE'];
			
			// Try to complete the transaction
			$nvpResult = $ppApi->doDoExpressCheckoutPayment(scCart::getCart(), $currencyCode, $paymentAction, $token, $payerId, $shippingAmount);
			
			// Update the order with the final paid currency and amount
			$orderId = $expressCheckoutDetails['INVNUM'];
			miStaticDBUtil::execUpdate('Orders', array('OrderTotalPayCurrencyCode' => $currencyCode, 'OrderTotalPayAmount' => $nvpResult['AMT']), 'OrderID', $orderId);
			
			// Return the payment result
			$result = new scPaymentProcessResult($orderId, $nvpResult['AMT'], $nvpResult['CURRENCYCODE'], $nvpResult['TRANSACTIONID']);
			switch ($nvpResult['PAYMENTSTATUS']) {
				case 'Completed':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);	// Correct for operation 'Sale'
					break;
				
				case 'Denied':
				case 'Expired':
				case 'Failed':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					break;
				
				case 'In-Progress':
				case 'Pending':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
					break;
				
				default:
					// Others should not happen
					throw new scPaymentProcessException(_t('Unknown payment status.'), array('PAYMENTSTATUS' => $nvpResult['PAYMENTSTATUS']));
			}
			
			// Set the transaction details
			$transactionDetailFields = array('TOKEN', 'ORDERTIME', 'CORRELATIONID', 'TRANSACTIONTYPE', 'PAYMENTTYPE',
				'FEEAMT', 'SETTLEAMT', 'EXCHANGERATE', 'PAYMENTSTATUS', 'PENDINGREASON', 'REASONCODE', 'PROTECTIONELIGIBILITY');
			foreach ($transactionDetailFields as $transactionDetailField) {
				if (isset($nvpResult[$transactionDetailField]))
					$result->setTransactionDetail($transactionDetailField, $nvpResult[$transactionDetailField]);
			}
			$result->setTransactionDetail('PAYERID', $payerId);
			$result->setTransactionDetail('PAYERSTATUS', $expressCheckoutDetails['PAYERSTATUS']);
			$result->setTransactionDetail('ADDRESSSTATUS', $expressCheckoutDetails['ADDRESSSTATUS']);
			
			return $result;
		}
		
		/**
		 * Update the oder based on the info returned from GetExpressCheckoutDetails PayPal API call
		 * 
		 * @param array $nvpResult
		 */
		protected function updateOrder($nvpResult)
		{
			if ($nvpResult['CHECKOUTSTATUS'] == 'PaymentActionCompleted')
				throw new scPaymentProcessException(_t('Order already paid.'));
			
			$currencyCode = $nvpResult['CURRENCYCODE'];
			
			// Update the order with the checkout details
			$orderId = $nvpResult['INVNUM'];
			$order = new scOrder($orderId);
			
			$order->set('OrderBillingFirstName', $nvpResult['FIRSTNAME']);
			$order->set('OrderBillingLastName', $nvpResult['LASTNAME']);
			$order->set('OrderBillingCompany', isset($nvpResult['BUSINESS']) ? $nvpResult['BUSINESS'] : '');
			$order->set('OrderBillingPhone', isset($nvpResult['PHONENUM']) ? $nvpResult['PHONENUM'] : '');
			$order->set('OrderBillingFax', '');
			$order->set('OrderBillingAddress1', $nvpResult['SHIPTOSTREET']);
			$order->set('OrderBillingAddress2', isset($nvpResult['SHIPTOSTREET2']) ? $nvpResult['SHIPTOSTREET2'] : '');
			$order->set('OrderBillingCity', $nvpResult['SHIPTOCITY']);
			$order->set('OrderBillingState', $nvpResult['SHIPTOSTATE']);
			$order->set('OrderBillingCountry', $nvpResult['SHIPTOCOUNTRYNAME']);
			$order->set('OrderBillingZip', $nvpResult['SHIPTOZIP']);
			
			$shipToName = explode(' ', $nvpResult['SHIPTONAME'], 2);
			$order->set('OrderShippingFirstName', $shipToName[0]);
			$order->set('OrderShippingLastName', isset($shipToName[1]) ? $shipToName[1] : '');
			$order->set('OrderShippingCompany', isset($nvpResult['BUSINESS']) ? $nvpResult['BUSINESS'] : '');
			$order->set('OrderShippingPhone', isset($nvpResult['PHONENUM']) ? $nvpResult['PHONENUM'] : '');
			$order->set('OrderShippingFax', '');
			$order->set('OrderShippingAddress1', $nvpResult['SHIPTOSTREET']);
			$order->set('OrderShippingAddress2', isset($nvpResult['SHIPTOSTREET2']) ? $nvpResult['SHIPTOSTREET2'] : '');
			$order->set('OrderShippingCity', $nvpResult['SHIPTOCITY']);
			$order->set('OrderShippingState', $nvpResult['SHIPTOSTATE']);
			$order->set('OrderShippingCountry', $nvpResult['SHIPTOCOUNTRYNAME']);
			$order->set('OrderShippingZip', $nvpResult['SHIPTOZIP']);
			$order->set('OrderShipToBillingAddress', 1);
			
			$order->set('OrderCustomerEmail', $nvpResult['EMAIL']);
			
			$order->set('OrderTotalPayAmount', $nvpResult['AMT']);
			$order->set('OrderPaymentModuleName', 'PayPal Express');
			
			// Save the shipping module / method info
			list($shippingModuleId, $shippingMethodCode, $shippingMethodName) = $this->getShippingMethodNameAndCode($nvpResult['SHIPPINGOPTIONNAME'], $nvpResult);
			$order->set('ShippingModuleID', $shippingModuleId);
			$order->set('OrderShippingMethodName', $shippingMethodName);
			$order->set('OrderShippingMethodCode', $shippingMethodCode);
			
			if ($shippingModuleId) {
				list($shippingModuleName, $shippingMethodName) = scOrdersInProgress::singleton()->getShippingModuleAndMethodNames($shippingModuleId, $shippingMethodCode);
				$order->set('OrderShippingMethodName', $shippingMethodName);
				$order->set('OrderShippingModuleName', $shippingModuleName);
			}
			
			// Customer instructions and id
			$order->set('OrderCustomerInstructions', isset($nvpResult['NOTETEXT']) ? $nvpResult['NOTETEXT'] : '');
			$order->set('CustomerID', scCustomerDomainObject::getCustomerIdForNewOrder($nvpResult['EMAIL'], null));
			$order->update();
			
			// Add shipping and tax order total lines
			scOrderService::singleton()->deleteOrderTotalLinesByType($orderId, IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING);
			scOrderService::singleton()->deleteOrderTotalLinesByType($orderId, IOrderTotalModule::ORDER_TOTAL_TYPE_TAX);
			
			// Convert shipping / tax back to base currency
			$shippingAmount = scCurrencyUtil::convert($nvpResult['SHIPPINGAMT'], $currencyCode, scCurrencyUtil::getBaseCurrency());
			scOrderService::singleton()->insertOrderTotalLine($orderId, $shippingAmount, _t('Shipping'), IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING);
			if ($nvpResult['TAXAMT'] != 0) {
				$taxAmount = scCurrencyUtil::convert($nvpResult['TAXAMT'], $currencyCode, scCurrencyUtil::getBaseCurrency());
				scOrderService::singleton()->insertOrderTotalLine($orderId, $taxAmount, _t('Tax'), IOrderTotalModule::ORDER_TOTAL_TYPE_TAX);
			}
			scOrderService::singleton()->updateOrderTotal($orderId);
			
			// Save the shipping address and method selection in the cart
			scCart::getCart()->setCustomerInfoField('ShippingModuleAndMethod', $shippingModuleId . '-' . $shippingMethodCode);
			$this->setShippingAddressToCart($nvpResult);
		}
		
		protected function execConfirmOrder()
		{
			$params = $_POST;
			$token = $params['token'];
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PaymentAction', 'Mode', 'ApiUsername', 'ApiPassword', 'ApiSignature', 'UseConfirmPage'));
			
			// Retrieve the checkout details from PayPal
			$ppApi = new scPayPalNvpApi($options['Mode'], $options['ApiUsername'], $options['ApiPassword'], $options['ApiSignature']);
			$expressCheckoutDetails = $ppApi->doGetExpressCheckoutDetails($token);
			
			$orderId = $expressCheckoutDetails['INVNUM'];
			
			// Update the order according to customer selection
			$this->updateOrder($expressCheckoutDetails);
			
			// Get the correct shipping and tax
			$cart = scCart::getCart();
			$orderTotal = $cart->getOrderTotal();
			$shipping = $orderTotal->getSummedOrderTotalLinesAmount(IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING, true);
			$tax = $orderTotal->getSummedOrderTotalLinesAmount(IOrderTotalModule::ORDER_TOTAL_TYPE_TAX, true);

			// Update shipping and tax order total lines
			scOrderService::singleton()->deleteOrderTotalLinesByType($orderId, IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING);
			scOrderService::singleton()->deleteOrderTotalLinesByType($orderId, IOrderTotalModule::ORDER_TOTAL_TYPE_TAX);
			
			$orderTotalLines = $orderTotal->getCurrentOrderTotalLines();
			foreach ($orderTotalLines as $key => $orderTotalLine) {
				if (($orderTotalLine->getType() != IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING) and ($orderTotalLine->getType() != IOrderTotalModule::ORDER_TOTAL_TYPE_TAX))
					unset($orderTotalLines[$key]);
			}
			scOrderService::singleton()->saveOrderTotalLines($orderTotalLines, $orderId);
			scOrderService::singleton()->updateOrderTotal($orderId);
			
			$currencyCode = $expressCheckoutDetails['CURRENCYCODE'];
			$shippingAmountInPayPalCurrency = $shipping->get(new scCurrency($currencyCode));
			$paymentResult = $this->finalizeOrder($ppApi, $expressCheckoutDetails, $options['PaymentAction'], $params['token'], $shippingAmountInPayPalCurrency);
			
			$cart->incHashSeed();
			
			scPaymentProcess::handlePaymentProcessResult($paymentResult, $this->_moduleId, scPaymentProcess::CALL_TYPE_DIRECT);
		}
		
		/**
		 * Will show confirmation screen
		 */
		protected function showConfirm()
		{
			$view = new miView;
			$page = new scUserDisplayController(array($view));
			$page->setSeo(new scCheckoutPayPalExpressSeo(_t('Checkout')));
			scLayout::forceNoBoxes(true);
			
			$page->getDispatcher()->setActionHandler('', 'scPayPalExpressCheckoutConfirmAction');
			$page->dispatchAndShow();
		}
		
		protected function getShippingMethodNameAndCode($shippingOptionName, $params)
		{
			if (substr($shippingOptionName, -2) == ' :')
				$shippingOptionName = substr($shippingOptionName, 0, -2);
			
			$cart = scCart::getCart();
			$custInfo = $cart->getCustomerInfo();
			$custInfo['CustomerShippingAddress1'] = $params['SHIPTOSTREET'];
			$custInfo['CustomerShippingAddress2'] = isset($params['SHIPTOSTREET2']) ? $params['SHIPTOSTREET2'] : '';
			$custInfo['CustomerShippingCountry'] = scCountry::getCountryIdByCode2($params['SHIPTOCOUNTRYCODE']);
			$custInfo['CustomerShippingCity'] = $params['SHIPTOCITY'];
			$custInfo['CustomerShippingState'] = scCountry::getStateId($custInfo['CustomerShippingCountry'], $params['SHIPTOSTATE']);
			$custInfo['CustomerShippingZip'] = $params['SHIPTOZIP'];
			$custInfo['CustomerShipToBillingAddress'] = false;
			$cart->setCustomerInfo($custInfo);
			
			list($shippingModuleId, $shippingMethodCode) = scOrderService::singleton()->getShippingModuleIdAndMethodCode($shippingOptionName, $custInfo['CustomerShippingCountry'], $custInfo['CustomerShippingState'], $cart->isAllIntangible());
			
			return array($shippingModuleId, $shippingMethodCode, $shippingOptionName);
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'Account',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'PaymentAction',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ApiUsername',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ApiPassword',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ApiSignature',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'AllowNote',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'RequireConfirmedShippingAddress',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'LandingPage',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'UseConfirmPage',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'CheckoutButtonType',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('PaymentAction')->setOptions($this->_paymentActions);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_paypalCurrencies, $this->_paypalCurrencies));
			$options->getOptionField('LandingPage')->setOptions(array('Login' => 'Login', 'Billing' => 'Billing'));
			$options->getOptionField('Mode')->setRadioButtons(array(scPayPalNvpApi::MODE_SANDBOX => _t('Sandbox'), scPayPalNvpApi::MODE_PRODUCTION => _t('Production')), scPayPalNvpApi::MODE_SANDBOX);
			$options->getOptionField('CheckoutButtonType')->setRadioButtons(array(scPayPalNvpApi::CHECKOUT_BUTTON_TYPE_STATIC => _t('Static'), scPayPalNvpApi::CHECKOUT_BUTTON_TYPE_DYNAMIC => _t('Dynamic')), scPayPalNvpApi::CHECKOUT_BUTTON_TYPE_DYNAMIC);
			$options->setTemplateFile('modules/checkout/paypal/express_options.tmpl');
			return $options;
		}
		
		/**
		 * Set the shipping address in the cart from the address returned from PayPal
		 * 
		 * @param array $params
		 */
		protected function setShippingAddressToCart($params)
		{
			$cart = scCart::getCart();
			$custInfo = $cart->getCustomerInfo();
			$custInfo['CustomerShippingAddress1'] = $params['SHIPTOSTREET'];
			$custInfo['CustomerShippingAddress2'] = isset($params['SHIPTOSTREET2']) ? $params['SHIPTOSTREET2'] : '';
			$custInfo['CustomerShippingCountry'] = scCountry::getCountryIdByCode2(isset($params['SHIPTOCOUNTRYCODE']) ? $params['SHIPTOCOUNTRYCODE'] : $params['SHIPTOCOUNTRY']);
			$custInfo['CustomerShippingCity'] = $params['SHIPTOCITY'];
			$custInfo['CustomerShippingState'] = scCountry::getStateId($custInfo['CustomerShippingCountry'], $params['SHIPTOSTATE']);
			$custInfo['CustomerShippingZip'] = $params['SHIPTOZIP'];
			$custInfo['CustomerShipToBillingAddress'] = false;
			$cart->setCustomerInfo($custInfo);
		}
		
		public function processCallback($params)
		{
			ignore_user_abort(true);
			
//			$start = microtime(true);
//			file_put_contents('/tmp/callback.txt', 'Start: ' . var_export($start, true), FILE_APPEND);
//			file_put_contents('/tmp/callback.txt', var_export($params, true), FILE_APPEND);
			
			// Note: we create new session on each callback
			$this->setShippingAddressToCart($params);
			
			// Calculate the shipping
			$custInfo = scCart::getCart()->getCustomerInfo();
			$digitalDelivery = $this->hasDigitalDeliveryProductsCallback($params);
			$currencyCode = $params['CURRENCYCODE'];
			$methods = $this->getShippingMethodsAndZones($currencyCode, $custInfo['CustomerShippingCountry'], $custInfo['CustomerShippingState'], $digitalDelivery);
			
			// Calculate the taxes
			$methods = $this->addTaxesToShippingMethods($methods);
			
//			file_put_contents('/tmp/callback.txt', 'Methods: ' . var_export($methods, true), FILE_APPEND);
			
			// Build the response
			$ppApi = new scPayPalNvpApi(scPayPalNvpApi::MODE_SANDBOX);
			$params = $ppApi->getCallbackResponse($methods, new scCurrency($currencyCode));
			
			foreach ($params as $key => $value) {
				echo $key . '=' . $value . '&';
			}
			
//			$end = microtime(true);
//			file_put_contents('/tmp/callback.txt', 'End: ' . var_export($end - $start, true), FILE_APPEND);
//			file_put_contents('/tmp/callback.txt', var_export($params, true), FILE_APPEND);
			exit;
		}
		
		protected function addTaxesToShippingMethods($methods)
		{
			$cart = scCart::getCart();
			$orderTotal = $cart->getOrderTotal();
			foreach ($methods as &$method) {
				$orderTotal->setShippingAmount($method['Rate']);
				$method['Tax'] = $orderTotal->getSummedOrderTotalLinesAmount(IOrderTotalModule::ORDER_TOTAL_TYPE_TAX, true);
			}
			return $methods;
		}
		
		/**
		 * Return true if at least one of the products is digital product
		 * 
		 * @return boolean
		 */
		protected function hasDigitalDeliveryProductsCallback($params)
		{
			$productCodes = array();
			$n = 0;
			while (isset($params['L_NUMBER' . $n])) {
				$productCodes[] = $params['L_NUMBER' . $n];
				$n++;
			}
			
			// Check if we should allow digital delivery. We don't have the real cart here, so check each of the products
			$productCodes = array_map('sql_escape_string', $productCodes);
			$sql = 'SELECT ProductIsEgood FROM Products WHERE ProductCode IN ("' .  implode('","', $productCodes) . '") AND ProductIsEgood>0 LIMIT 1';
			$rows = miUtil::getDBArray($sql, 'ProductIsEgood', 'ProductIsEgood');
			return count($rows) ? true : false;
		}
		
		/**
		 * Handle notification
		 *
		 * @param array $params
		 */
		public function processNotify($params)
		{
//			scDebug::logVariable('/tmp/paypal.txt', 'POST', $params);
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Account', 'Mode'));
			
			// Validate notification
			$post = $params;
			$post['cmd'] = '_notify-validate';
			$url = ($options['Mode'] == scPayPalNvpApi::MODE_PRODUCTION) ? self::PAYPAL_LIVE_URL : self::PAYPAL_SANDBOX_URL;
			$result = scHttp::post($url, http_build_query($post));
//			scDebug::logVariable('/tmp/paypal.txt', 'Paypal result', $result);
			
			if ($result != 'VERIFIED')
				throw new scPaymentProcessException(_t('PayPal notification validation failed.'), array(_t('Result') => $result));
			
			$orderId = $params['invoice'];
			
			if (isset($params['test_ipn']) and $params['test_ipn'] and ($options['Mode'] != scPayPalNvpApi::MODE_SANDBOX)) {
				throw new scPaymentProcessException(_t('Test IPN to live system.'), array(_t('Result') => $result));
			}
			
			// Check the account is correct email address
			$this->checkPayPalAccount($options, $params);
			
			// Process the notification
			$result = new scPaymentProcessResult($orderId, $params['mc_gross'], $params['mc_currency'], $params['txn_id']);
			switch ($params['payment_status']) {
				case 'Completed':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					
					// Check that the currency and the amount match. If not, then this is a partial capture
					$order = new scOrder($params['invoice']);
					if (($order->get('OrderTotalPayCurrencyCode') != $params['mc_currency']) or (intval($order->get('OrderTotalPayAmount') * 100) != intval($params['mc_gross'] * 100))) {
						$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PARTIAL_CAPTURE);
					}
					break;
				
				case 'Pending':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
					$result->setTransactionNote($params['pending_reason']);
					break;
				
				case 'Denied':
				case 'Expired':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					$result->setTransactionNote($params['payment_status']);
					break;
				
				case 'Refunded':
				case 'Reversed':
					$paymentStatus = ($params['reason_code'] == 'chargeback') ? scPaymentStatus::PAYMENT_STATUS_CHARGEBACK : scPaymentStatus::PAYMENT_STATUS_REFUND;
					$result->setPaymentStatus($paymentStatus);
					$result->setTransactionNote($params['reason_code']);
					break;
				
				default:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_OTHER);
					$result->setTransactionNote($params['payment_status']);
			}
			
			$transactionDetailFields = array('mc_gross', 'mc_currency', 'invoice', 'settle_amount', 'protection_eligibility', 'address_status',
				'payer_id', 'payment_status', 'mc_fee', 'exchange_rate', 'settle_currency', 'payer_status', 'payment_type',
				'auth_id', 'auth_amount', 'auth_exp', 'auth_status', 'remaining_settle', 'payer_email', 'test_ipn', 'memo', 'parent_txn_id');
			foreach ($transactionDetailFields as $transactionDetailField) {
				if (isset($params[$transactionDetailField]))
					$result->setTransactionDetail($transactionDetailField, $params[$transactionDetailField]);
			}
			
			return $result;
		}
		
		protected function checkPayPalAccount($options, $params)
		{
			// Check if the account is using Secure Merchant Account ID (receiver_id)
			if (isset($params['receiver_id']) and ($options['Account'] == $params['receiver_id']))
				return;
			
			if (isset($params['business']) and ($options['Account'] == $params['business']))
				return;
			
			if ($options['Account'] == $params['receiver_email'])
				return;
				
			throw new miException(sprintf(_i('Invalid business "%s" and receiver_email "%s", expected "%s"'), @$params['business'], @$params['receiver_email'], $options['Account']));
		}
		
		protected function getShippingMethodsAndZones($currencyCode, $countryId = 0, $stateId = 0, $digitalDelivery = 'any', $skipRealtime = false)
		{
			$currency = new scCurrency($currencyCode);
			
			$result = array();
			
			$shippingModules = scModulesFactory::getActiveShippingModulesForZone($countryId, $stateId, $digitalDelivery);
			foreach ($shippingModules as $shippingModule) {
				try {
					$moduleId = $shippingModule['ModuleID'];
					$module = $shippingModule['object'];
					
					// If skip realtime option is given, skip all realtime shipping quotes
					if ($skipRealtime) {
						if (($module instanceof scFedEx) or ($module instanceof scUPS) or ($module instanceof scUSPS))
							continue;
					}
					
					$shippingMethodRates = $module->getShippingRates(scCart::getCart(), array_keys($shippingModule['ShippingModuleActiveMethods']));
					if (!isset($shippingMethodRates[IShippingModule::SHIPPING_RATES]))
						continue;
					
					$rates = $shippingMethodRates[IShippingModule::SHIPPING_RATES];
					if (isset($shippingMethodRates[IShippingModule::SHIPPING_RATES_CURRENCY])) {
						$rateCurrency = new scCurrency($shippingMethodRates[IShippingModule::SHIPPING_RATES_CURRENCY]);
					} else {
						$rateCurrency = scCurrencyUtil::getBaseCurrency();
					}
					$shippingMethods = $module->getShippingMethods();
					
					foreach ($shippingMethods as $shippingMethodCode => $shippingMethod) {
							if (!isset($rates[$shippingMethodCode]))
								continue;
							
							if (!isset($shippingModule['ShippingModuleActiveMethods'][$shippingMethodCode]))
								continue;
							
							$methodName = trim($shippingModule['ShippingModuleActiveMethods'][$shippingMethodCode]['ShippingMethodDisplayName']);
							if ($methodName == '')
								$methodName = $shippingMethod;
							
							$rate = scMultiCurrencyAmount::factory();
							$rate->set($rateCurrency, $rates[$shippingMethodCode]);
							// Calculate it in the processor's currency.
							$rate->get($currency);
							$result[] = array(
								'ShippingModuleID' => $moduleId,
								'ShippingMethodCode' => $shippingMethodCode,
								'ShippingMethodDisplayName' => $methodName,
								'Rate' => $rate
							);
						
						// There is a limit of PayPal shipping options. We don't need more than the limit
						if (count($result) >= scPayPalNvpApi::MAX_SHIPPING_OPTIONS)
							return $result;
					}
				} catch (miException $e) {
					scLog::singleton()->info(_i('PayPal Express shipping calculations: ') . $e->getMessage(), scLog::CATEGORY_SHIPPING);
				}
			}
			return $result;
		}
	}
	
	class scCheckoutPayPalExpressSeo extends scSeo {
		
		protected function addBodyClasses()
		{
			parent::addBodyClasses();
			$this->_bodyClasses[] = 'dynamic-page-checkout';
		}
	}
	
	class scPayPalExpressCheckoutConfirmAction extends miAction {
		const CONFIRM_PAGE = 'modules/checkout/paypal/checkout_confirm.tmpl';
		
		public function doAction()
		{
			try {
				$this->viewConfirmPage();
			} catch (Exception $exception) {
				header('Location: ' . scPaths::getUrl('error.php') . '?msg=' . urlencode($exception->getMessage()));
			}
		}
		
		protected function viewConfirmPage()
		{
			$uiCart = new scUICart();

			$t = new scTemplateParser();
			$t->readTemplate(self::CONFIRM_PAGE);
			$t->setSectionInfos(array($uiCart->getCartTemplateSection(), $uiCart->getOrderTotalLinesTemplateSection()));
			$t->addSectionInfo(new miTemplateParserSectionInfo('cartHasPerItemDiscount', scCart::getCart()->hasPerItemDiscount()));
			
			$grandTotal = scPrice::factory(scCart::getCart()->getOrderTotal()->getGrandTotalAmount());
			$t->assign('%%GRANDTOTAL%%', $grandTotal->getDisplayPrice());
			$t->assign('%%TOKEN%%', miGetParamDefault('token', ''));
			$t->assign('%%PAYERID%%', miGetParamDefault('PayerID', ''));
			
			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('cart.php'));
			
			$html = $t->templateParse();

			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;
		}
	}
?>