<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Checkout page
	 *
	 */
	class scUserPageCheckout extends scCheckoutWizardPage {
		protected $_templates = array(
			self::TEMPLATE_ID_WIZARD_PAGE => 'customer/checkout/wizard/checkout.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerEmail',
				'validator' => array('scValidatorEmail', 'scValidatorNoEmpty'),
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerPassword',
				'validator' => 'scValidatorNoEmpty'
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'LoginEmail',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'LoginPassword',
        	),
        );
        
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$errorsHandler = new scUserPageCheckoutErrorsHandler;
			$form->setErrorsHandler($errorsHandler);
			
			$form->addMainPageElements(array('%%LOGINERROR%%' => ''));
			
			// Captcha
			if (scSettings::getSettingValue(scSettings::SETTING_CAPTCHA_ON_REGISTER_IN_CHECKOUT)) {
				$captcha = new scCaptchaWidget($form);
				$captcha->setProperty('id', 'captcha');
				$captcha->setFieldName('captcha');
				$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('captcha', 1, miUIUtil::templetizeRows($captcha->getEditableControl())));
			}
			
			// Check if guest checkout is supported
			if (scWizardCheckout::hasGuestCheckout()) {
				$form->addTemplateSectionInfo($this->getHasGuestCheckoutSection());
				
				$optionalEmailWidget = new miTextWidget($form);
				$optionalEmailWidget->setFieldName('GuestOptionalEmail');
				$form->addWidget('GuestOptionalEmail', $optionalEmailWidget);
				
				$pageInfo = $this->getPageData();
				$form->setFormData('GuestOptionalEmail', @$pageInfo['CustomerEmail']);
			}
		}
		
		public function postProcess($actionObj, $result)
		{
			if ($actionObj instanceof scGuestAction) {
				$pageData = $this->getPageData();
				$pageData['CustomerEmail'] = miGetParamDefault('GuestOptionalEmail', '');
				$this->savePageData($pageData);
			}
			
			parent::postProcess($actionObj, $result);
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		public function getHasGuestCheckoutSection()
		{
			static $section = null;
			
			if (is_null($section)) {
				$section = new miTemplateParserSectionInfo('hasGuestCheckout', 1);
			}
			
			return $section;
		}
	}
	
	class scUserPageCheckoutErrorsHandler extends miWebFormErrorsHandler {
		
		const ERRORS_SEPARATOR = '<br/>';
		
		const ACTION_TYPE_LOGIN = 0;
		const ACTION_TYPE_REGISTER = 1;
		const ACTION_TYPE_GUEST = 2;
		
		protected $_isRegister = false;
		protected $_loginErrors = array();
		
		public function setIsRegister($isRegister)
		{
			$this->_isRegister = $isRegister;
		}
		
		public function setLoginError($errors)
		{
			$this->_loginErrors = $errors;
		}
		
		public function assignErrors(&$t)
		{
			$t->assign('%%LOGINERRORS%%', '');
			$t->assign('%%REGISTERERRORS%%', '');
			$t->assign('%%GUESTERRORS%%', '');
			
			switch ($this->_isRegister) {
				case self::ACTION_TYPE_REGISTER:
					$t->assign('%%REGISTERERRORS%%', $this->getHtmlErrors($this->_errors));
					break;
				
				case self::ACTION_TYPE_LOGIN:
					$t->assign('%%LOGINERRORS%%', $this->getHtmlErrors($this->_loginErrors));
					break;
				
				case self::ACTION_TYPE_GUEST:
					$t->assign('%%GUESTERRORS%%', $this->getHtmlErrors($this->_errors));
					break;
			}
		}
		
		protected function getHtmlErrors($errors)
		{
			$errorsArray = array();
			foreach ($this->_errors as $fieldName => $errors) {
				$errorsArray[] = $fieldName . ' : ' . $errors;
			}
			
			return implode(self::ERRORS_SEPARATOR, $errorsArray);
		}
	}
	
	class scLoginAction extends miExecWizardPageAction {
		/**
		 * Logs in the customer
		 *
		 * @return true|string true on success, error message on error
		 */
		protected function login()
		{
			$data = $this->_view->getPageData('scUserPageCheckout');
			
			$email = $data['CustomerEmail'] = $data['LoginEmail'];
			$password = $data['CustomerPassword'] = $data['LoginPassword'];
			
			if ($email == '')
				return _t('Please enter email.');
			
			try {
				$user = new scCustomerDomainObject();
				$user->read($email, 'CustomerEmail');
			} catch (miDBException $exception) {
				if ($exception->getCode() != miDBException::EXCEPTION_RECORD_NOT_FOUND)
					throw $exception;
				return _t('Invalid username or password.');
			}
			
			$auth = scCustomerAuth::singleton();
			if (!$auth->isCorrectPassword($user, $email, $password)) {
				return _t('Invalid username and/or password. Please try again.');
			}
			if ($user->get('CustomerStatus') != scCustomerStatuses::CUSTOMER_STATUS_ACTIVE) {
				return _t('Your account has been disabled.');
			}
			
			$data['WizardAuth'] = scWizardCheckout::WIZARD_AUTH_METHOD_LOGIN;
			$data['CustomerID'] = $user->get('CustomerID');
			$this->_view->savePageData($data, 'scUserPageCheckout');
			miSetParam('CustomerID', $user->get('CustomerID'));
			$auth = scCustomerAuth::singleton();
			@$auth->logIn($user->get('CustomerID'));
			
			// Set the address fields
			$data = $user->getBillingAndShippingAddresses();
			$this->_view->savePageData($data, 'scUserPageCheckoutAddress');
			
			scCart::getCart()->resetWholesalePrices();
			scCart::getCart()->applyPriceModifications();
			
			return true;
		}
		
		public function doAction()
		{
			try {
				$form = $this->getWebForm();
				$form->removeValidators('CustomerEmail');
				$form->removeValidators('CustomerPassword');
				
				$isOk = parent::doAction();
				$loginResult = '';
				if ($isOk === true) {
					$loginResult = $this->login();
				}
				
				if (($isOk === true) and ($loginResult === true)) {
					return true;
				}
				
				// Display the checkout screen with error
				$this->_view->getController()->getDispatcher()->setWizardAction('');
				$model = $this->_view->getViewMapperController();
				$model->setDataToWebForm();
				$error = new miTemplateParserSectionInfo('loginerror', 1, array('%%LOGINERROR%%' => miI18N::htmlEscape($loginResult)));
				$form->addTemplateSectionInfo($error);
				$form->addMainPageElements($this->_view->getMainPageElements());
				$form->addMainPageElements(array('%%LOGINERROR%%' => miI18N::htmlEscape($loginResult)));
				$html = $form->parse($this->_view->getTemplateFileName(miWizardPage::TEMPLATE_ID_WIZARD_PAGE), true);
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
			} catch (miException $exception) {
				$msg = _t('Exception during login: ');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scRegisterAction extends miExecWizardPageAction {
		/**
		 * Registers the customer
		 *
		 * @return true|array true on success, or array with error messages
		 * @throws miException
		 */
		protected function register()
		{
			$data = $this->_view->getPageData('scUserPageCheckout');
			
			// we cannot add CustomerPassword2 as a widget because it is not present in the
			// login form, which uses the same class
			if ($data['CustomerPassword'] != miGetParamDefault('CustomerPassword2', '') ) {
				return array(_t('Passwords do not match!'));
			}
			
			// check if such record does not exist
			$record = new miSqlRecord('Customers', 'CustomerID');
			
			try {
				$record->read('CustomerEmail', $data['CustomerEmail']);
			} catch (miDBException $exception) {
				// Record doesn't exists. The value is unqiue
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					$data = $this->_view->getPageData('scUserPageCheckout');
					$data['WizardAuth'] = scWizardCheckout::WIZARD_AUTH_METHOD_REGISTER;
					$data['CustomerID'] = false;	// Make sure that previous login/guest checkout won't affect the register
					$this->_view->savePageData($data, 'scUserPageCheckout');
					return true;
				}
				// Different exception. Rethrow
				throw $exception;
			}
			
			return array(_t('This email is already registered.'));
		}
		
		protected function validateCaptcha()
		{
			$captchaSecurity = new scCaptchaSecurity();
			$code = $captchaSecurity->getCode();
			
			if (empty($code) || (strtolower($code) != strtolower(miGetParam('captcha')))) {
				$err = $this->getWebForm()->getWebFormErrorsHandler();
				$err->addError('captcha', _t('Security code does not match.'));
				return false;
			}
			
			return true;
		}
		
		public function doAction()
		{
			try {
				$isOk = parent::doAction();
				
				if (scSettings::getSettingValue(scSettings::SETTING_CAPTCHA_ON_REGISTER_IN_CHECKOUT))
					$isOk = ($isOk && $this->validateCaptcha()); // Keep the parenthesis!
				
				$registerResult = array();
				if ($isOk === true)
					$registerResult = $this->register();
				
				if (($isOk === true) and ($registerResult === true)) {
					return true;
				}
				
				// Display the checkout screen with error
				$this->_view->getController()->getDispatcher()->setWizardAction('');
				
				$form = $this->getWebForm();
				
				$errorHandler = $form->getWebFormErrorsHandler();
				$errorHandler->setIsRegister(scUserPageCheckoutErrorsHandler::ACTION_TYPE_REGISTER);
				foreach ($registerResult as $registerError)
					$errorHandler->addError('', $registerError);
				$error = new miTemplateParserSectionInfo('registererror', 1);
				$form->addTemplateSectionInfo($error);
				$model = $this->_view->getViewMapperController();
				$model->setDataToWebForm();
				
				$form->addMainPageElements($this->_view->getMainPageElements());
				$html = $form->parse($this->_view->getTemplateFileName(miWizardPage::TEMPLATE_ID_WIZARD_PAGE), true);
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				
				return true;
			} catch (miException $exception) {
				$msg = _t('Exception during register: ');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scGuestAction extends miExecWizardPageAction {
		
		public function doAction()
		{
			$form = $this->getWebForm();
			$form->removeValidators('CustomerEmail');
			$form->removeValidators('CustomerPassword');
			
			$form->addValidator('GuestOptionalEmail', new scValidatorEmail($form, 'GuestOptionalEmail'));
			
			// Process the form data
			if (parent::doAction() === false) {
				// Display the checkout screen with error
				$this->_view->getController()->getDispatcher()->setWizardAction('');
				
				$errorHandler = $form->getWebFormErrorsHandler();
				$errorHandler->setIsRegister(scUserPageCheckoutErrorsHandler::ACTION_TYPE_GUEST);
				$error = new miTemplateParserSectionInfo('guesterror', 1);
				$this->_view->getHasGuestCheckoutSection()->addSubsection($error);
				$form->addTemplateSectionInfo($error);
				
				$form->addMainPageElements($this->_view->getMainPageElements());
				$html = $form->parse($this->_view->getTemplateFileName(miWizardPage::TEMPLATE_ID_WIZARD_PAGE), true);
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				
				return false;
			}
			
			try {
				if (!scWizardCheckout::hasGuestCheckout())
					throw new miException(_t('Guest checkout is not enabled.'));
				
				$data['WizardAuth'] = scWizardCheckout::WIZARD_AUTH_METHOD_GUEST;
				$data['CustomerID'] = ICheckoutModule::GUEST_USER_ID;
				$data['CustomerEmail'] = miGetParamDefault('AnonimousOptionalEmail', '');
				$this->_view->savePageData($data);
				return true;
			} catch (miDBException $exception) {
				$msg = _t('Exception during checkout: ');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	/**
	 * Shipping and Billing address page
	 *
	 */
	class scUserPageCheckoutAddress extends scCheckoutWizardPage {
		protected $_templates = array(
			self::TEMPLATE_ID_WIZARD_PAGE => 'customer/checkout/wizard/checkout_address.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerBillingFirstName',
				'validator' => 'scValidatorNoEmpty',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerBillingLastName',
				'validator' => 'scValidatorNoEmpty',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerBillingCompany',
        	),
        	array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerBillingPhone',
				'validator' => 'scValidatorNoEmpty',
        	),
        	array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerBillingFax',
        	),
        	array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerBillingAddress1',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerBillingAddress2'
       		),
       		array(
				'field' => 'scCityNameWidget',
				'data' => 'CustomerBillingCity',
       			'validator' => 'scValidatorNoEmpty',
       			'properties' => array(
       				'template' => 'customer/widgets/city_name_widget.tmpl',
       				'CityIDFieldId' => 'CustomerBillingCityID', 
       				'StateIDFieldId' => 'CustomerBillingState',
       				'CountryIDFieldId' => 'CustomerBillingCountry',
       				'tabindex' => 9
       				),
        		),
        	array(
        		'field' => 'miBaseTextWidget',
        		'data' => 'CustomerBillingCityID',
        		),
       		array(
				'field' => 'scStateSelectWidget',
				'data' => 'CustomerBillingState',
				'properties' => array('id' => 'CustomerBillingState', 'originalValueField' => 'OriginalCustomerBillingState', 'tabindex' => 8)
       		),
       		array(
				'field' => 'miBaseSelectWidget',
				'data' => 'CustomerBillingCountry',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerBillingZip',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'scVatNumberWidget',
				'data' => 'CustomerBillingVatNumber',
        	),
       		array(
				'field' => 'miBaseCheckboxWidget',
				'data' => 'CustomerShipToBillingAddress',
       		),
        	array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerShippingFirstName',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerShippingLastName',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerShippingCompany',
        	),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerShippingPhone',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerShippingFax',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerShippingAddress1',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerShippingAddress2'
       		),
       		array(
				'field' => 'scCityNameWidget',
				'data' => 'CustomerShippingCity',
       			'validator' => 'scValidatorNoEmpty',
       			'properties' => array(
       				'template' => 'customer/widgets/city_name_widget.tmpl',
       				'CityIDFieldId' => 'CustomerShippingCityID', 
       				'StateIDFieldId' => 'CustomerShippingState',
       				'CountryIDFieldId' => 'CustomerShippingCountry',
       				'tabindex' => 22
       				),
        		),
        	array(
        		'field' => 'miBaseTextWidget',
        		'data' => 'CustomerShippingCityID',
        		),
       		array(
				'field' => 'scStateSelectWidget',
				'data' => 'CustomerShippingState',
				'properties' => array('id' => 'CustomerShippingState', 'originalValueField' => 'OriginalCustomerShippingState', 'tabindex' => 21)
       		),
       		array(
				'field' => 'miBaseSelectWidget',
				'data' => 'CustomerShippingCountry',
       		),
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerShippingZip',
				'validator' => 'scValidatorNoEmpty',
       		),
       		array(
				'field' => 'scVatNumberWidget',
				'data' => 'CustomerShippingVatNumber',
        	),
       		
       		array(
       			'field' => 'scAddressSelectWidget',
       			'data' => 'CustomerBillingAddressID',
       			'properties' => array(
       				'id' => 'CustomerBillingAddressID',
       				'template' => 'customer/checkout/wizard/checkout_address_widget.tmpl',
       				'addressType' => scCustomerAddress::BILLING,
       				'tabindex' => 1
       			),
       		),
       		array(
       			'field' => 'scAddressSelectWidget',
       			'data' => 'CustomerShippingAddressID',
       			'properties' => array(
       				'id' => 'CustomerShippingAddressID',
       				'template' => 'customer/checkout/wizard/checkout_address_widget.tmpl',
       				'addressType' => scCustomerAddress::SHIPPING,
       				'tabindex' => 14
       			),
       		),
        );
        
        protected static $_errorsHandler;
        
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$data = $this->getPageData();
			if (!isset($data['CustomerShipToBillingAddress'])) {
				$data['CustomerShipToBillingAddress'] = true;
				$this->savePageData($data);
			}
			
			$countryNames = scCountry::getCountryNames();
			$field = $form->getWidget('CustomerBillingCountry');
			$field->setOptions($countryNames, scSettings::getSettingValue(scSettings::SETTING_DEFAULT_COUNTRY));
			
			$field = $form->getWidget('CustomerBillingVatNumber');
			$field->setCountryField('CustomerBillingCountry');
			
			$field = $form->getWidget('CustomerShippingCountry');
			$field->setOptions($countryNames, scSettings::getSettingValue(scSettings::SETTING_DEFAULT_COUNTRY));
			
			$field = $form->getWidget('CustomerShippingVatNumber');
			$field->setCountryField('CustomerShippingCountry');
			
			$stateNames = scCountry::getStateNames();
			$field = $form->getWidget('CustomerBillingState');
			$field->setOptions($stateNames);
			
			$field = $form->getWidget('CustomerShippingState');
			$field->setOptions($stateNames);
			
			$form->getWidget('CustomerShipToBillingAddress')->setDefaultValue(true);
			
			$customer = scCustomerAuth::singleton()->getCustomer();
			if ($customer) {
				$customerAddresses = $customer->getAddresses();
				$widget = $form->getWidget('CustomerBillingAddressID');
				$widget->setAddresses($customerAddresses, $customer->getBillingAddressId());
				
				$widget = $form->getWidget('CustomerShippingAddressID');
				$widget->setAddresses($customerAddresses, $customer->getShippingAddressId());
			}
			
			$this->addMainPageElements(array('%%STATES_JS_DATA%%' => scUIUtil::getStatesJsData()));
			
			$form->setErrorsHandler(self::$_errorsHandler ? self::$_errorsHandler : new scByFieldErrorsHandler);
			
			$form->addValidator('CustomerBillingCity', new scAddressValidator($form, 'CustomerBillingCityID', 'CustomerBillingCountry'));
			$form->addValidator('CustomerShippingCity', new scAddressValidator($form, 'CustomerShippingCityID', 'CustomerShippingCountry'));
		}
		
		public function preProcess($actionObj)
		{
			$authInfo = $this->getPageData('scUserPageCheckout');
			if ($authInfo['WizardAuth'] == scWizardCheckout::WIZARD_AUTH_METHOD_REGISTER) {
				$this->_dataFields[] = array(
	       			'field' => 'scNewslettersWidget',
	       			'data' => 'Newsletters',
					'properties' => array(
						'template' => 'customer/widgets/newsletters_widget.tmpl'
					),
	       		);
			} else {
				$this->addMainPageElements(array('%%NEWSLETTERS%%' => ''));
			}
			
			$shipToBillingAddress = (bool) miGetParamDefault('CustomerShipToBillingAddress', false);
			if ($shipToBillingAddress) {
				miSetParam('CustomerShippingFirstName', miGetParamDefault('CustomerBillingFirstName', ''));
				miSetParam('CustomerShippingLastName', miGetParamDefault('CustomerBillingLastName', ''));
				miSetParam('CustomerShippingAddress1', miGetParamDefault('CustomerBillingAddress1', ''));
				miSetParam('CustomerShippingAddress2', miGetParamDefault('CustomerBillingAddress2', ''));
				miSetParam('CustomerShippingCity', miGetParamDefault('CustomerBillingCity', ''));
				miSetParam('CustomerShippingCityID', miGetParamDefault('CustomerBillingCityID', ''));
				miSetParam('CustomerShippingState', miGetParamDefault('CustomerBillingState', ''));
				miSetParam('CustomerShippingCountry', miGetParamDefault('CustomerBillingCountry', ''));
				miSetParam('CustomerShippingZip', miGetParamDefault('CustomerBillingZip', ''));
				miSetParam('CustomerShippingPhone', miGetParamDefault('CustomerBillingPhone', ''));
				miSetParam('CustomerShippingVatNumber', miGetParamDefault('CustomerBillingVatNumber', ''));
			}
		}
		
		public function postProcess($actionObj, $actionResult)
		{
			parent::postProcess($actionObj, $actionResult);
			
			if ($actionObj instanceof miExecWizardPageAction) {
				$data = $this->getPageData();
				if (@$data['CustomerShipToBillingAddress']) {
					$data['CustomerShippingFirstName'] = $data['CustomerBillingFirstName'];
					$data['CustomerShippingLastName'] = $data['CustomerBillingLastName'];
					$data['CustomerShippingAddress1'] = $data['CustomerBillingAddress1'];
					$data['CustomerShippingAddress2'] = $data['CustomerBillingAddress2'];
					$data['CustomerShippingCity'] = $data['CustomerBillingCity'];
					$data['CustomerShippingState'] = $data['CustomerBillingState'];
					$data['CustomerShippingCountry'] = $data['CustomerBillingCountry'];
					$data['CustomerShippingZip'] = $data['CustomerBillingZip'];
					$data['CustomerShippingPhone'] = $data['CustomerBillingPhone'];
					$data['CustomerShippingVatNumber'] = $data['CustomerBillingVatNumber'];
					$this->savePageData($data);
				}
				
				self::$_errorsHandler = $actionObj->getWebForm()->getWebFormErrorsHandler();
			}
		}
	}
	
	class scUserPageCheckoutMethods extends scCheckoutWizardPage {
		protected $_templates = array(
			self::TEMPLATE_ID_WIZARD_PAGE => 'customer/checkout/wizard/checkout_methods.tmpl'
		);
		
		protected static $_errorsHandler;
		
		protected $_dataFields = array(
			array(
				'field' => 'scCheckoutShippingMethodWidget',
				'data' => 'ShippingModuleAndMethod',
			),
			array(
				'field' => 'scCheckoutPaymentMethodWidget',
				'data' => 'PaymentModuleID',
			),
			array(
				'field' => 'scCheckoutAllowPointsWidget',
				'data' => 'CustomerAllowedPoints',
			),
		);
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$form->setErrorsHandler(self::$_errorsHandler ? self::$_errorsHandler : new scUserPageCheckoutMethodsErrorsHandler);
			
			$data = $this->getPageData('scUserPageCheckoutAddress');
			$widget = $form->getWidget('ShippingModuleAndMethod');
			$widget->setProperty('stateId', $data['CustomerShippingState']);
			$widget->setProperty('countryId', $data['CustomerShippingCountry']);
			$widget->setCart(scCart::getCart());
			
			$addressVars = scWizardUI::getAddressTmplVars($data);
			$this->addMainPageElements($addressVars);
			
			// Check if user is allowed to enter order instructions and eventually show the textarea
			$moduleId = scModulesFactory::getModuleIdByName(scWizardCheckout::MODULE_NAME);
			if ((bool) intval(scModuleOptions::getModuleOption($moduleId, scWizardCheckout::OPTION_ORDER_INSTRUCTONS))) {
				$data = $this->getPageData();
				$section = new miTemplateParserSectionInfo('hasOrderInstructions');
				
				$instructions = @$data['OrderCustomerInstructions'];
				$section->setValues(array(
					'%%ORDERCUSTOMERINSTRUCTIONS%%' => $instructions,
					'%%SPECIALINSTRUCTIONSDISPLAY%%' => trim($instructions) == '' ? 'none' : ''));
				$form->addTemplateSectionInfo($section);
			}
			
			// Bonus points approval
			$widget = $form->getWidget('CustomerAllowedPoints');
			$widget->setDefaultValue(1);
			$uiCart = new scUICart();
			$form->addTemplateSectionInfo($uiCart->getBonusPointsUsageApprovalSection());
		}
		
		public function postProcess($actionObj, $actionResult)
		{
			if ($actionObj instanceof miExecWizardPageAction) {
				$data = $this->getPageData();
//				if (miGetParamDefault('ShippingModuleAndMethod', '') != '')
//					$data['ShippingModuleAndMethod'] = miGetParam('ShippingModuleAndMethod');
				
				if (miHasParam('OrderCustomerInstructions'))
					$data['OrderCustomerInstructions'] = miGetParam('OrderCustomerInstructions');
				else
					$data['OrderCustomerInstructions'] = '';
					
				$this->savePageData($data);
				
				// If the shipping method or payment method are not set - don't let the user proceed
//				if ((miGetParamDefault('ShippingModuleAndMethod', '') == '') or(miGetParamDefault('PaymentModuleID', '') == ''))
//					$this->getController()->getDispatcher()->setNextPage($this->getController()->getDispatcher()->getCurrentPage());

				self::$_errorsHandler = $actionObj->getWebForm()->getWebFormErrorsHandler();
			}
			
			parent::postProcess($actionObj, $actionResult);
		}
	}
	
	class scUserPageCheckoutMethodsErrorsHandler extends miWebFormErrorsHandler {
		public function assignErrors(&$t)
		{
			if (empty($this->_errors)) {
				$t->assign('%%ERRORS%%', '');
				$t->assign('%%ERROR_STYLE%%', 'hidden');
			} else {
				$t->assign('%%ERRORS%%', implode('<br/>', miI18N::htmlEscape($this->_errors)));
				$t->assign('%%ERROR_STYLE%%', 'error');
			}
		}
	}
	
	class scUserPageCheckoutConfirm extends scCheckoutWizardPage {
		protected $_templates = array(
			self::TEMPLATE_ID_WIZARD_PAGE => 'customer/checkout/wizard/checkout_confirm.tmpl'
		);
		
		protected $_moduleId;
		protected $_errors = array();
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		protected $_dataFields = array(
			array(
				'field' => 'miCheckboxWidget',
				'data' => 'AgreedToTerms',
			),
		);
		
		public function getModuleId()
		{
			return $this->_moduleId;
		}
			
		public function setErrors($errors)
		{
			$this->_errors = $errors;
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			// Apply the price modifications after the customer selects a payment module
			scCart::getCart()->applyPriceModifications();
			
			$uiCart = new scUICart();
			$form->addTemplateSectionInfo($uiCart->getCartTemplateSection());
			$form->addTemplateSectionInfo($uiCart->getOrderTotalLinesTemplateSection());
			$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('cartHasPerItemDiscount', scCart::getCart()->hasPerItemDiscount()));
			
			$data = $this->getPageData('scUserPageCheckoutMethods');
			$paymentModuleDomainObj = new scPaymentModule($data['PaymentModuleID']);
			$paymentModule = scModulesFactory::getImplementation($paymentModuleDomainObj->get('ModuleID'), 'IPaymentModule');
			$form->addMainPageElements(array('%%PAYMENT_FORM%%' => $paymentModule->getPaymentContent(IPaymentModule::CONTENT_TYPE_PAYMENT_FORM)));
			
			$cart = scCart::getCart();
			$grandTotal = scPrice::factory($cart->getOrderTotal()->getGrandTotalAmount());
			$this->addMainPageElements(array('%%GRANDTOTAL%%' => miI18N::htmlEscape($grandTotal->getDisplayPrice())));
			
			list($shippingModuleId, $shippingMethodCode) = explode('-', $data['ShippingModuleAndMethod']);
			list($shippingModuleName, $shippingMethodName) = scOrdersInProgress::singleton()->getShippingModuleAndMethodNames($shippingModuleId, $shippingMethodCode);
			$this->addMainPageElements(array(
				'%%SHIPPINGMODULENAME%%' => $shippingModuleName,
				'%%SHIPPINGMETHODNAME%%' => $shippingMethodName,
			));
			
			list($paymentModuleName, $paymentModuleIntegrationMethod) = scOrdersInProgress::singleton()->getPaymentModuleName($data['PaymentModuleID']);
			$this->addMainPageElements(array(
				'%%PAYMENTMODULENAME%%' => $paymentModuleName,
				'%%PAYMENTMODULEINTEGRATIONMETHOD%%' => $paymentModuleIntegrationMethod,
			));
			
			// next button caption
			if ($paymentModule->hasCheckoutStep() && ($grandTotal->getAmount()->get() > 0)) {
				$buttonCaption = _t('Proceed to payment');
			} else {
				$buttonCaption = _t('Finalize order');
			}
			$this->addMainPageElements(array(
				'%%NEXT_STEP_BUTTON_CAPTION%%' => $buttonCaption
			));
			
			// manage terms
			$moduleId = scModulesFactory::getModuleIdByName(scWizardCheckout::MODULE_NAME);
			$options = scModuleOptions::getModuleOptions($moduleId, array(), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			if (!empty($options['RequireAcceptTerms'])) {
				$this->addMainPageElements(array(
					'%%HTML_TERMS%%' => $options['Terms']
				));
				
				$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('RequireAgreeToTerms', 1));
				$form->addValidator('AgreedToTerms',  new scValidatorTerms($form, 'AgreedToTerms'));
			}
			$data = $this->getPageData('scUserPageCheckoutAddress');
			$addressVars = scWizardUI::getAddressTmplVars($data);
			$this->addMainPageElements($addressVars);
			
			$errorsHandler = new scCheckoutConfirmErrorsHandler;
			$form->setErrorsHandler($errorsHandler);
			if (count($this->_errors))
				$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('confirmerror', 1));
			foreach ($this->_errors as $fieldName => $error)
				$errorsHandler->addError($fieldName, $error);
		}
	}
	
	class scValidatorTerms extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (!$fieldValue) {
				$errors->addError($this->_fieldName, _t('Please agree to the terms.'));
			}
			return;
		}
	}
	
	class scCheckoutConfirmErrorsHandler extends miWebFormMessageErrorsHandler {
		public function getErrors()
		{
			return $this->_errors;
		}
		
		public function assignErrors(&$t)
		{
			$errorsArray = array();
			foreach ($this->_errors as $fieldName => $errors) {
				$errorsArray[] = $errors;
			}
			$t->assign('%%ERRORS%%', implode(self::ERRORS_SEPARATOR, miI18N::htmlEscape($errorsArray)));
		}
	}
	
	class scConfirmOrderAction extends miExecWizardPageAction {
		
		/**
		 * Updates the customer profile record with newly entered info
		 *
		 * @param array $cartInfo
		 */
		protected function updateProfile($cartInfo)
		{
			$customer = new scCustomerDomainObject();
			$customer->read($cartInfo['CustomerID']);
			
			$this->updateAddresses($cartInfo, $customer->getBillingAddressId(), $customer->getShippingAddressId());
		}
		
		protected function updateAddresses($cartInfo, $defaultBillingAddressId, $defaultShippingAddressId)
		{
			$billingAddressId = $cartInfo['CustomerBillingAddressID'];
			$mapping = scCustomerAddress::getAddressFields(scCustomerAddress::BILLING);
			if (!empty($billingAddressId) && ($billingAddressId != scAddressSelectWidget::NEW_ADDRESS)) {
				$this->updateAddress($cartInfo, $mapping, $billingAddressId);
			} else {
				$override = array();
				if (is_null($defaultBillingAddressId)) {
					$override['CustomerAddressIsDefaultBilling'] = 1;
				}
				if ($cartInfo['CustomerShipToBillingAddress']) {
					if (is_null($defaultShippingAddressId)) {
						$override['CustomerAddressIsDefaultShipping'] = 1;
					}
				}
				$billingAddressId = $this->createAddress($cartInfo, $mapping, $override);
			}
			
			$shippingAddressId = $billingAddressId;
			if (!$cartInfo['CustomerShipToBillingAddress']) {
				$shippingAddressId = $cartInfo['CustomerShippingAddressID'];
				$mapping = scCustomerAddress::getAddressFields(scCustomerAddress::SHIPPING);
				if (!empty($shippingAddressId) && ($shippingAddressId != scAddressSelectWidget::NEW_ADDRESS)) {
					$this->updateAddress($cartInfo, $mapping, $shippingAddressId);
				} else {
					$override = array();
					if (is_null($defaultShippingAddressId)) {
						$override['CustomerAddressIsDefaultShipping'] = 1;
					}
					$shippingAddressId = $this->createAddress($cartInfo, $mapping, $override);
				}
			}
			
			$data = $this->_view->getPageData('scUserPageCheckoutAddress');
			$data['CustomerBillingAddressID'] = $billingAddressId;
			$data['CustomerShippingAddressID'] = $shippingAddressId;
			$this->_view->savePageData($data, 'scUserPageCheckoutAddress');
		}
		
		protected function subscribeForNewsletters($cartInfo)
		{
			try {
				if (!isset($cartInfo['Newsletters'])) {
					return;
				}
				
				$moduleId = scModulesFactory::getModuleIdByName('com.summercart.newsletter');
				scModulesFactory::getImplementation($moduleId, 'IUserModule');
				
				foreach ($cartInfo['Newsletters'] as $newsletterId => $isActive) {
					$newsletter = new scNewsletter();
					$newsletter->read($newsletterId);
					$newsletter->subscribe($cartInfo['CustomerEmail'], $cartInfo['CustomerID']);
				}
			} catch (miException $e) {
				return;
			}
		}
		
		protected function updateAddress($cartInfo, $mapping, $addressId)
		{
			$address = new scCustomerAddress();
			$address->read($addressId);
			foreach ($mapping as $formField => $dbField) {
				$address->set($dbField, $cartInfo[$formField]);
			}
			return $address->update();
		}
		
		protected function createAddress($cartInfo, $mapping, $override = array())
		{
			$address = new scCustomerAddress();
			$address->set('CustomerID', $cartInfo['CustomerID']);
			foreach ($mapping as $formField => $dbField) {
				$address->set($dbField, $cartInfo[$formField]);
			}
			foreach ($override as $dbField => $value) {
				$address->set($dbField, $value);
			}
			return $address->insert();
		}
		
		protected function processPayment($form)
		{
			$data = $this->_view->getPageData('scUserPageCheckoutMethods');
			$paymentModuleDomainObj = new scPaymentModule($data['PaymentModuleID']);
			$moduleId = $paymentModuleDomainObj->get('ModuleID');
			$paymentModule = scModulesFactory::getImplementation($moduleId, 'IPaymentModule');
			
			$payCurrencyCode = $paymentModule->getPaymentCurrencyCode();
			if (is_null($payCurrencyCode))
				$payCurrencyCode = scCurrencyUtil::getBaseCurrencyCode();
			
			// Add CustomerID to the Cart's CustomerInfo
			// If the user has choosen to register for checkout, add it to the database
			$cart = scCart::getCart();
			$cartInfo = $cart->getCustomerInfo();
			$checkoutInfo = $this->getCheckoutInfo();
			$authInfo = $this->_view->getPageData('scUserPageCheckout');
			switch ($authInfo['WizardAuth'])
			{
				case scWizardCheckout::WIZARD_AUTH_METHOD_REGISTER:
					$auth = scCustomerAuth::singleton();
					
					if (isset($cartInfo['CustomerID']) and ($cartInfo['CustomerID'])) {
						// Login the customer
						$auth->logIn($cartInfo['CustomerID']);
						break;
					}
					$cartInfo['CustomerEmail'] = $authInfo['CustomerEmail'];
					$cartInfo['CustomerPassword'] = $authInfo['CustomerPassword'];
					
					// Create new Customer record
					$customer = new scCustomerDomainObject();
					$customer->setRow($cartInfo);
					$customer->set('CustomerStatus', scCustomerStatuses::CUSTOMER_STATUS_ACTIVE);
					$customer->set('CustomerGroupID', scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP));
					scBonusPoints::setNewCustomerReferrer($customer);
					$customerId = $customer->insert();
					
					$cartInfo['CustomerID'] = $customerId;
					
					$this->updateAddresses($cartInfo, null, null);
					
					$this->subscribeForNewsletters($cartInfo);
					
					// Send email
					$notifications = new scEmailNotifications();
					$notifications->sendSignupNotification($customerId);
					
					// Login the customer
					$auth->logIn($customerId);
					break;
				
				case scWizardCheckout::WIZARD_AUTH_METHOD_LOGIN:
					$customer = scCustomerAuth::singleton()->getCustomer();
					$cartInfo['CustomerID'] = $customer->get('CustomerID');
					$cartInfo['CustomerEmail'] = $customer->get('CustomerEmail');
					$this->updateProfile($cartInfo);
					break;
				
				case scWizardCheckout::WIZARD_AUTH_METHOD_GUEST:
					$cartInfo['CustomerID'] = $checkoutInfo['CustomerID'];
					$cartInfo['CustomerEmail'] = $checkoutInfo['CustomerEmail'];
					break;
			}
			$cartInfo['OrderCustomerInstructions'] = array_key_exists('OrderCustomerInstructions', $checkoutInfo) ? $checkoutInfo['OrderCustomerInstructions'] : '';
			
			$cart->setCustomerInfo($cartInfo);
			
			$payAmount = $cart->getOrderTotal()->getGrandTotalInCurrency(new scCurrency($payCurrencyCode));
			
			$checkoutModuleId = scUtil::getDbValue('CheckoutModules', 'CheckoutModuleID', 'ModuleID', $this->_view->getModuleId());
			$orderId = scOrdersInProgress::singleton()->createOrder($payAmount, $payCurrencyCode, $cart, $checkoutModuleId);
			
			miSetParam('OrderID', $orderId);
			
			try {
				if ($cart->getOrderTotal()->getGrandTotalAmount()->get() == 0) {
					$result = new scPaymentProcessResult($orderId);
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_OFFLINE);
				} else {
					$result = $paymentModule->processPayment($payAmount, $payCurrencyCode, $cart, $_POST);
				}
				
				if (!($result instanceof scPaymentProcessResult))
					throw new miException(_i('Payment module processPayment() method did not return scPaymentProcessResult object.'));
				
			} catch (miException $exception) {
				scPaymentProcess::handlePaymentProcessException($exception, scPaymentProcess::CALL_TYPE_DIRECT);
				exit;
			}
			
			if ($result->hasValidatorErrors())
				return $result;
			
			// If there are no validator errors then there is real attempt to make payment
			// Thus make sure that the orderId is not reused
			$cart->incHashSeed();
			
			// Mark any coupon as used
			$cart->markCouponUsed();
			
			scPaymentProcess::handlePaymentProcessResult($result, $moduleId, scPaymentProcess::CALL_TYPE_DIRECT);
			exit;
		}
		
		public function doAction()
		{
			try {
				$model = $this->_view->getViewMapperController();
				
				$form = $this->getWebForm();
				$isOk = $form->processSubmit();
				scCart::getCart()->checkMinimumPoints();
				$model->setDataFromWebForm();
				
				$this->_view->savePageData($model->getDomainObject()->getRow());
				
				if ($isOk === false) {
					$this->_view->setErrors($form->getWebFormErrorsHandler()->getErrors());
					return false;
				}
				$result = $this->processPayment($form);
				if ($result->hasValidatorErrors()) {
					$this->_view->setErrors($result->getValidatorErrors());
					return false;
				}
				
				return true;
			} catch (miException $exception) {
				$msg = _t('Exception during processing payment: ') . $exception->getMessage();
				$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART);
				$url = scUrlUtil::addQueryParams($url, 'msg=' . urlencode($msg));
				header('Location: ' . $url);
				exit;
			}
		}
		
		protected function getCheckoutInfo()
		{
			$methodsData = $this->_view->getPageData('scUserPageCheckoutMethods');
			$checkoutTypeData = $this->_view->getPageData('scUserPageCheckout');
			
			return array(
				'OrderCustomerInstructions' => @$methodsData['OrderCustomerInstructions'],
				'CustomerEmail' => @$checkoutTypeData['CustomerEmail'],
				'CustomerID' => @$checkoutTypeData['CustomerID']
			);
		}
	}
	
	/**
	 * After order confirmation
	 */
	class scUserPageCheckoutProcessOrder extends scCheckoutWizardPage {
		public function getControllerCommands()
		{
			$orderId = (int) miGetParamDefault('OrderID', 0);
			$message = miGetParamDefault('Message', '');
			scPaymentProcess::redirectToPaymentResult($orderId, $message);
			exit;
		}
	}
	
	/**
	 * Checkout wizard class
	 */
	class scWizardCheckout implements ICheckoutModule, IModuleOptions {
		const WIZARD_AUTH_METHOD_LOGIN = 1;
		const WIZARD_AUTH_METHOD_REGISTER = 2;
		const WIZARD_AUTH_METHOD_GUEST = 3;
		
		const OPTION_ALLOW_GUEST_CHECKOUT = 'AllowGuestCheckout';
		const OPTION_ORDER_INSTRUCTONS = 'AllowOrderInstructions';
		
		const OPTION_REQUIRE_ACCEPT_TERMS = 'RequireAcceptTerms';
		const OPTION_TERMS = 'Terms';
		
		const MODULE_NAME = 'com.summercart.checkout.wizard';
		
		protected $_moduleId;
		
		/**
		 * @return bool
		 */
		public static function hasGuestCheckout()
		{
			if (!scCart::getCart()->isAllTangible()) {
				// if the cart contains any e-goods do not allow
				return false;
			}
			
			$moduleId = scModulesFactory::getModuleIdByName(scWizardCheckout::MODULE_NAME);
			$optionValue = scModuleOptions::getModuleOption($moduleId, scWizardCheckout::OPTION_ALLOW_GUEST_CHECKOUT);
			return (bool)(int)$optionValue;
		}
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miCheckboxWidget',
					'data' => self::OPTION_ALLOW_GUEST_CHECKOUT,
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => self::OPTION_ORDER_INSTRUCTONS,
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => self::OPTION_REQUIRE_ACCEPT_TERMS,
				),
				array(
					'field' => 'scWebFormWidgetTextAreaML',
					'data' => self::OPTION_TERMS,
				),
			);
			
			$webForm->addMainPageElements(array(
				'%%CKEDITOR_CONTENTCSS%%' => scSkins::singleton()->getCurrentSkin()->getCssFilesForWysiwygEditor(),
				'%%CKEDITOR_LANGUAGE%%' => scCMS::getCkEditorLanguage()
			));
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->setTemplateFile('modules/checkout/wizard/options.tmpl');
			return $options;
		}
		
		public function checkout()
		{
			try {
				$wizard = new scCheckoutWizardController();
				$wizard->setSeo(new scCheckoutWizardSeo(_t('Checkout')));
				$wizard->setDisableLanguageSelection(true);
				
				$customer = scCustomerAuth::singleton()->getCustomer();
				if ($customer) {
					miSetParam('CustomerID', $customer->get('CustomerID'));
				}
				
				$customerRecord = new miSqlRecord('Customers', 'CustomerID');
				
				$wizardPage = new scUserPageCheckout();
				$wizardPage->setRecord($customerRecord);
				$wizard->addWizardPage($wizardPage);
				$wizard->getDispatcher()->setActionHandler('scLogin', 'scLoginAction');
				$wizard->getDispatcher()->setActionHandler('scRegister', 'scRegisterAction');
				$wizard->getDispatcher()->setActionHandler('scGuest', 'scGuestAction');
				
				$wizardPage = new scUserPageCheckoutAddress();
				$wizardPage->setRecord($customerRecord);
				$wizard->addWizardPage($wizardPage);
				
				$wizardPage = new scUserPageCheckoutMethods();
				$wizardPage->setRecord($customerRecord);
				$wizard->addWizardPage($wizardPage);
				
				$wizardPage = new scUserPageCheckoutConfirm($this->_moduleId);
				$wizardPage->setRecord($customerRecord);
				$wizard->addWizardPage($wizardPage);
				
				$wizard->getDispatcher()->setActionHandler('scConfirmOrder', 'scConfirmOrderAction');
				
				$wizardPage = new scUserPageCheckoutProcessOrder();
				$wizardPage->setRecord($customerRecord);
				$wizard->addWizardPage($wizardPage);
				
				$wizard->dispatchAndShow();
			} catch (Exception $exception) {
				$msg = _t('Exception during checkout: ') . $exception->getMessage();
				$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART);
				$url = scUrlUtil::addQueryParams($url, 'msg=' . urlencode($msg));
				$wizard->processControllerCommands(array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url)));
			}
		}
	}
	
	class scCheckoutWizardSeo extends scSeo {
		
		protected function addBodyClasses()
		{
			parent::addBodyClasses();
			$this->_bodyClasses[] = 'dynamic-page-checkout';
		}
	}
	
	class scAddressSelectWidget extends miSelectWidget {
		
		const NEW_ADDRESS = -1;
		
		protected $_addresses = array();
		
		public function __construct(miWebForm &$webForm, $properties = array())
		{
			parent::__construct($webForm, $properties);
			
			$this->_options = array(
				self::NEW_ADDRESS => _t('New address')
			);
		}
		
		public function setAddresses($addresses, $defaultAddressId)
		{
			$this->_addresses = $addresses;
			
			$options = array();
			foreach ($addresses as $address) {
				$options[$address['CustomerAddressID']] = $address['CustomerAddressZip'] . ', ' .
					$address['CustomerAddressCity'] . ', ' .
					$address['CustomerAddressLine1'];
			}
			$options[self::NEW_ADDRESS] = _t('New address');
			$this->setOptions($options, $defaultAddressId);
		}
		
		public function getEditableControl()
		{
			if (count($this->_options) == 1) {
				return array(
					$this->_fieldName => '<input type="hidden" name="' . $this->_fieldName . '" value="' . self::NEW_ADDRESS . '" />',
				);
			}
			
			$control = parent::getEditableControl();
			$control[$this->_fieldName] = $this->getAdditionalHtml($control[$this->_fieldName]);
			return $control;
		}
		
		protected function getAdditionalHtml($dropdownHtml = '')
		{
			$t = new scTemplateParser();
			$t->readTemplate($this->_properties['template']);
			
			$t->assign('%%NEW_ADDRESS%%', self::NEW_ADDRESS);
			$t->assign('%%FIELD_NAME%%', $this->_fieldName);
			$t->assign('%%ADDRESSES_JSON%%', scJson::jsonEncode($this->_addresses));
			$t->assign('%%HTML_DROPDOWN%%', $dropdownHtml);
			
			$addressType = $this->_properties['addressType'];
			$fieldsMapping = scCustomerAddress::getAddressFields($addressType);
			
			$t->assign('%%ADDRESSES_MAPPING%%', scJson::jsonEncode($fieldsMapping));
			return $t->templateParse();
		}
	}
	
	class scWizardUI {
		
		public static function getAddressTmplVars($data)
		{
			$addressVars = array(
				'%%CUSTOMERBILLINGFIRSTNAME%%' => @$data['CustomerBillingFirstName'],
				'%%CUSTOMERBILLINGLASTNAME%%' => @$data['CustomerBillingLastName'],
				'%%CUSTOMERBILLINGCOMPANY%%' => @$data['CustomerBillingCompany'],
				'%%CUSTOMERBILLINGADDRESS1%%' => @$data['CustomerBillingAddress1'],
				'%%CUSTOMERBILLINGADDRESS2%%' => @$data['CustomerBillingAddress2'],
				'%%CUSTOMERBILLINGADDRESSCITY%%' => @$data['CustomerBillingCity'],
				'%%CUSTOMERBILLINGADDRESSSTATE%%' => scCountry::getStateName($data['CustomerBillingState']),
				'%%CUSTOMERBILLINGADDRESSCOUNTRY%%' => scCountry::getCountryName($data['CustomerBillingCountry']),
				'%%CUSTOMERBILLINGADDRESSZIP%%' => @$data['CustomerBillingZip'],
				'%%CUSTOMERBILLINGADDRESSVATNUMBER%%' => @$data['CustomerBillingVatNumber'],
				'%%CUSTOMERBILLINGPHONE%%' => @$data['CustomerBillingPhone'],
				'%%CUSTOMERSHIPPINGFIRSTNAME%%' => @$data['CustomerShippingFirstName'],
				'%%CUSTOMERSHIPPINGLASTNAME%%' => @$data['CustomerShippingLastName'],
				'%%CUSTOMERSHIPPINGCOMPANY%%' => @$data['CustomerShippingCompany'],
				'%%CUSTOMERSHIPPINGADDRESS1%%' => @$data['CustomerShippingAddress1'],
				'%%CUSTOMERSHIPPINGADDRESS2%%' => @$data['CustomerShippingAddress2'],
				'%%CUSTOMERSHIPPINGADDRESSCITY%%' => @$data['CustomerShippingCity'],
				'%%CUSTOMERSHIPPINGADDRESSSTATE%%' => scCountry::getStateName($data['CustomerShippingState']),
				'%%CUSTOMERSHIPPINGADDRESSCOUNTRY%%' => scCountry::getCountryName($data['CustomerShippingCountry']),
				'%%CUSTOMERSHIPPINGADDRESSZIP%%' => @$data['CustomerShippingZip'],
				'%%CUSTOMERSHIPPINGADDRESSVATNUMBER%%' => @$data['CustomerShippingVatNumber'],
				'%%CUSTOMERSHIPPINGPHONE%%' => @$data['CustomerShippingPhone']
			);
			foreach ($addressVars as $key => $addressVar)
				$addressVars[$key] = miI18N::htmlEscape($addressVar);
			return $addressVars;
		}
	}
?>