<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 2Checkout V2 payment processing module
	 */
	class scPaymentModule2checkout implements IPaymentModule, IModuleOptions {
		protected $_moduleId;
		
		/**
		 * ISO 639-2 to 2checkout language code map
		 * @var array
		 */
		protected $_langCodeMap = array(
			// ISO 639-2 => 2checkout
			'es' => 'es_ib',	// European spanish
			'zh' => 'zh',
			'da' => 'da',
			'nl' => 'nl',
			'fr' => 'fr',
			'de' => 'gr',
			'el' => 'el',
			'it' => 'it',
			'ja' => 'jp',
			'no' => 'no',
			'pt' => 'pt',
			'sl' => 'sl',
			'sv' => 'sv'
		);

		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Return the payment currency
		 */
		public function getPaymentCurrencyCode()
		{
			return 'USD';
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('SID', 'DemoMode', 'SecretWord'));
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = isset($this->_langCodeMap[$isoLangCode]) ? $this->_langCodeMap[$isoLangCode] : 'en';
			
			$params = array(
				'sid' => $options['SID'],
				'total' => $payAmount,
				'cart_order_id' => scOrdersInProgress::singleton()->getLastOrderId(),
				'fixed' => 'Y',
				'demo' => $options['DemoMode'],
				'lang' => $lang,
				
				'card_holder_name' => $customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName'],
				'street_address' => $customerInfo['CustomerBillingAddress1'],
				'street_address2' => $customerInfo['CustomerBillingAddress2'],
				'city' => $customerInfo['CustomerBillingCity'],
				'state' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'zip' => $customerInfo['CustomerBillingZip'],
				'country' => scCountry::getCountryCode3($customerInfo['CustomerBillingCountry']),
				'email' => $customerInfo['CustomerEmail'],
				'phone' => $customerInfo['CustomerBillingPhone'],
				
				'ship_street_address' => $customerInfo['CustomerShippingAddress1'],
				'ship_city' => $customerInfo['CustomerShippingCity'],
				'ship_state' => scCountry::getStateCode($customerInfo['CustomerShippingState']),
				'ship_zip' => $customerInfo['CustomerShippingZip'],
				'ship_country' => scCountry::getCountryCode3($customerInfo['CustomerShippingCountry']),
				
				'return_url' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_ABSOLUTE),
				
				// NOTE: 2Checkout is currently ignoring this url
				// Make sure that the correct url is set under Look & Feel section
				'x_receipt_link_url' => scPaths::getSecureUrl('modules/payment/2checkout/return.php'),
				
				'id_type' => '1'
			);
			
			// Add the items
			$items = $cart->getItems();
			$n = 0;
			$currency = new scCurrency('USD');
			foreach ($items as $item) {
				$n++;
				$params['c_prod_' . $n] = $item->getCartItemCode() . ',' . $item->getQuantity();
				$params['c_name_' . $n] = $item->getName();
				$params['c_description_' . $n] = $item->getName();
				$params['c_price_' . $n] = $item->getPriceObj()->getAmountInCurrency($currency);
				$params['c_tangible_' . $n] = $item->isTangible() ? 'Y' : 'N';
			}
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect('https://www.2checkout.com/2co/buyer/purchase', $params);
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('demo', 'cart_order_id', 'total', 'key', 'credit_card_processed'));
			
			// Check the key
			$isDemo = ($params['demo'] == 'Y') ? true : false;
			$orderNumber = $isDemo ? 1 : $params['order_number'];
			$orderId = $params['cart_order_id'];
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId);
			$key = strtoupper(md5($options['SecretWord'] . $options['SID'] . $orderNumber . $params['total']));
			if ($params['key'] != $key)
				throw new scPaymentProcessException(_t('Invalid 2Checkout key. Please check you Secret Word.'), array(_t('OrderID') => $orderId));
			
			if ($isDemo and ($options['DemoMode'] != 'Y'))
				throw new scPaymentProcessException(_t('Order in demo mode.'), array(_t('OrderID') => $orderId));
			
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($params['cart_order_id'], $params['total'], 'USD');
			
			$result = new scPaymentProcessResult($orderId, $params['total'], 'USD', $orderNumber);
			if ($params['credit_card_processed'] == 'Y')
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			elseif ($params['credit_card_processed'] == 'K')
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
			else
				throw new scPaymentProcessException(_t('Unknown processing code.'), array(_t('Processing Code') => $params['credit_card_processed']));
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'SID',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'DemoMode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'SecretWord',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('DemoMode')->setRadioButtons(array('Y' => _t('Yes'), 'N' => _t('No')), 'N');
			$options->setTemplateFile('modules/payment/2checkout/options.tmpl');
			return $options;
		}
	}
?>