<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Atos payment processing module
	 */
	 
	 class scPaymentModuleAtos implements IPaymentModule, IModuleOptions {
		const URL_AUTORESPONSE = 'modules/payment/atos/notify.php';
		const URL_CANCEL = 'modules/payment/atos/return.php';
		const URL_RETURN = 'modules/payment/atos/return.php';
		
		const PATH_PATHFILE = 'modules/payment/atos/params/pathfile';
		const PATH_REQUEST = 'modules/payment/atos/executables/request';
		const PATH_RESPONSE = 'modules/payment/atos/executables/response';
		
		const FILE_NOT_EXECUTABLE = 126;
		
		protected static $_languages = array(
			'DE' => 'de',
			'EN' => 'en',
			'ES' => 'es',
			'FR' => 'fr'
		);
		
		protected static $_allowedCurrencies = array(
			'978' => 'EUR',
			'840' => 'USD',
			'756' => 'CHF',
			'826' => 'GBP',
			'124' => 'CAD',
			'392' => 'JPY',
			'484' => 'MXP',
			'792' => 'TRL',
			'036' => 'AUD',
			'554' => 'NZD',
			'578' => 'NOK',
			'986' => 'BRC',
			'032' => 'ARP',
			'116' => 'KHR',
			'901' => 'TWD',
			'752' => 'SEK',
			'208' => 'DKK',
			'410' => 'KRW',
			'702' => 'SGD'
		);
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
	 	public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), self::$_allowedCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'CountryID'));
			$customerInfo = $cart->getCustomerInfo();
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$params = array();
			$params[] = 'merchant_id=' . $options['MerchantID'];
			$params[] = 'merchant_country=' . strtolower(scCountry::getCountryCode2($options['CountryID']));
			// Amount must be in cents.
			$params[] = 'amount=' . round($payAmount * 100);
			$params[] = 'currency_code=' . array_search($payCurrencyCode, self::$_allowedCurrencies);
			$params[] = 'pathfile=' . self::PATH_PATHFILE;
			$params[] = 'transaction_id=' . substr(time(), 4);
//			$params[] = 'normal_return_url=http://internal.mirchevideas.com:803/' . self::URL_RETURN;
//			$params[] = 'cancel_return_url=http://internal.mirchevideas.com:803/' . self::URL_CANCEL;
//			$params[] = 'automatic_response_url=http://internal.mirchevideas.com:803/' . self::URL_AUTORESPONSE;
			$params[] = 'normal_return_url=' . scPaths::getSecureUrl() . self::URL_RETURN;
			$params[] = 'cancel_return_url=' . scPaths::getSecureUrl() . self::URL_CANCEL;
			$params[] = 'automatic_response_url=' . scPaths::getSecureUrl() . self::URL_AUTORESPONSE;
			$params[] = 'language=' . $this->getLanguageCode();
			$params[] = 'payment_means=CB,2,VISA,2,MASTERCARD,2';
			$params[] = 'header_flag=no';
			if (!empty($_SESSION['CustomerID']))
				$params[] = 'customer_id=' . $_SESSION['CustomerID'];
			$params[] = 'customer_email=' . $customerInfo['CustomerEmail'];
			$params[] = 'customer_ip_address=' . $_SERVER['REMOTE_ADDR'];
			$params[] = 'order_id=' . $orderId;
			
			// These options are custom for every store
			// If the store owner want them work
			// images, logos, texts must e-mailed to the Atos Technical Aid Center
			// (e-transactions@credit-agricole.fr)
//			$params[] = 'data=';
//			$params[] = 'return_context=';
//			$params[] = 'target=';
//			$params[] = 'capture_day=';
//			$params[] = 'capture_mode=';
//			$params[] = 'bgcolor=';
//			$params[] = 'block_align=';
//			$params[] = 'block_order=';
//			$params[] = 'textcolor=';
//			$params[] = 'receipt_complement=';
//			$params[] = 'normal_return_logo=';
//			$params[] = 'cancel_return_logo=';
//			$params[] = 'submit_logo=';
//			$params[] = 'logo_id=';
//			$params[] = 'logo_id2=';
//			$params[] = 'advert=';
//			$params[] = 'background_id=';
//			$params[] = 'caddie=mon_caddie';
//			$params[] = 'templatefile=template_ca_fr';
			
			$escapedParams = array();
			foreach ($params as $param)
				$escapedParams[] = escapeshellarg($param);
			
			chdir(scPaths::getStoreRootDir());
			$result = exec(self::PATH_REQUEST . ' ' . implode(' ', $escapedParams), $output, $returnVar);
			if ($returnVar == self::FILE_NOT_EXECUTABLE)
				throw new miException(sprintf(_i('%s is not executable.'), self::PATH_REQUEST));

			// function output : $result = !code!error!buffer!
			// - code = 0 : function generates a HTML page contained in the variable buffer
			// - code = -1 : The function returns an error message in the variable error

			$resultData = explode('!', $result);

			$code = $resultData[1];
			$error = $resultData[2];
			$message = $resultData[3];
			
			if ($code != 0)
				throw new miException(sprintf(_i('Error message: %s'), $error));
			
			$t = new miTemplateParser();
			$t->readTemplate('skins/default/modules/payment/atos/atos.tmpl');
			$t->assignArray(array(
				'%%ERROR%%' => $error,
				'%%MESSAGE%%' => $message
			));
			$t->templateShow();
			
			return new scPaymentProcessResult();
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			$resultData = $this->getResultData($params);
			
			scSession::singleton()->openSession();
			
			// Get latest transaction status
			$transactionStatus = 0;
			$transactions = miStaticDBUtil::execSelect('SELECT TransactionPaymentStatus FROM Transactions WHERE OrderID = ? ORDER BY TransactionID DESC', array($resultData['OrderId']));
			if (!empty($transactions))
				$transactionStatus = $transactions[0]['TransactionPaymentStatus'];
			
			// Amount is in cents.
			$amount = $resultData['Amount'] / 100;
			
			$result = new scPaymentProcessResult($resultData['OrderId'], $amount, self::$_allowedCurrencies[$resultData['CurrencyCode']], $resultData['TransactionId']);
			if (($resultData['CaptureMode'] != 'AUTHOR_CAPTURE') && ($transactionStatus != scPaymentStatus::PAYMENT_STATUS_CAPTURE)) {
				scOrdersInProgress::singleton()->addOrderId($resultData['OrderId']);
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes(_t('Payment cancelled by customer.'));
			}
			
			$result->setAllTransactionDetails($resultData);
			return $result;
		}
		
		/**
		 * Handles payment notification
		 * 
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processNotify($params)
		{
			$resultData = $this->getResultData($params);
			
			if ($resultData['Code'] != 0)
				throw new miException(sprintf(_i('Error message: %s'), $resultData['Error']));
			
			// Amount is in cents.
			$amount = $resultData['Amount'] / 100;
			scPaymentProcess::checkPaymentToOrder($resultData['OrderId'], $amount);
			
			$result = new scPaymentProcessResult($resultData['OrderId'], $amount, self::$_allowedCurrencies[$resultData['CurrencyCode']], $resultData['TransactionId']);
			if (empty($resultData['PaymentCertificate']) || empty($resultData['AuthorisationId'])) {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes(_t('Payment failed.'));
			} else
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			$result->setAllTransactionDetails($resultData);
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'MerchantID',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'CountryID',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultLanguageCode'
				)
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$options->getOptionField('CountryID')->setOptions(scCountry::getCountryNames());
			sort(self::$_allowedCurrencies);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine(self::$_allowedCurrencies, self::$_allowedCurrencies));
			
			$languageCodes = array_keys(self::$_languages);
			$options->getOptionField('DefaultLanguageCode')->setOptions(array_combine($languageCodes, $languageCodes));
			
			$options->setTemplateFile('modules/payment/atos/atos_options.tmpl');
			return $options;
		}
		
		protected function getLanguageCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultLanguageCode'));
			
			if (in_array(scMultiLanguage::getCurrentLanguageCode(), array_keys(self::$_languages)))
				return self::$_languages[scMultiLanguage::getCurrentLanguageCode()];
			
			return self::$_languages[$options['DefaultLanguageCode']];
		}
		
		/**
		 * Handles return data
		 *
		 * @param array $params
		 * @return array of returned payment params
		 */
		protected function getResultData($params)
		{
			$pathfile = escapeshellarg('pathfile=' . self::PATH_PATHFILE);
			$message = escapeshellarg('message=' . $params['DATA']);
			
			chdir(scPaths::getStoreRootDir());
			$resultValues = exec(self::PATH_RESPONSE . ' ' . $pathfile . ' ' . $message, $output, $returnVar);
			if ($returnVar == self::FILE_NOT_EXECUTABLE)
				throw new miException(sprintf(_i('%s is not executable.'), self::PATH_RESPONSE));
			
			$resultKeys = array(
				'',
				'Code', 'Error', 'MerchantId', 'MerchantCountry', 'Amount', 'TransactionId',
				'PaymentMeans', 'TransmissionDate', 'PaymentTime', 'PaymentDate', 'ResponseCode', 'PaymentCertificate',
				'AuthorisationId', 'CurrencyCode', 'CartNumber', 'CvvFlag', 'CvvResponseCode', 'BankResponseCode',
				'ComplementaryCode', 'ComplementaryInfo', 'ReturnContext', 'Caddie', 'ReceiptComplement', 'MerchantLanguage',
				'Language', 'CustomerId', 'OrderId', 'CustomerEmail', 'CustomerIpAddress', 'CaptureDay',
				'CaptureMode', 'Data'
			);

			// function output : !code!error!v1!v2!v3!...!v29
			// - code = 0 : function returns transaction info in the variables v1, v2, ...
			// : These variables are described on the PROGRAMMER'S GUIDE
			// - code = -1 : The function returns an error message in the variable error

			return array_combine($resultKeys, explode('!', $resultValues));
		}
	}
?>