<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Authorize.Net SIM payment processing module
	 */
	class scPaymentModuleAuthorizeSIM implements IPaymentModule, IModuleOptions {
		const AUTHORIZE_NET_URL = 'https://secure.authorize.net/gateway/transact.dll';
		const AUTHORIZE_NET_DEVELOPER_URL = 'https://test.authorize.net/gateway/transact.dll';
		
		const MODE_LIVE = 'Live';
		const MODE_DEMO = 'Demo';
		const MODE_DEVELOPER = 'Developer';
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		/**
		 * Compute HMAC-MD5
		 */
		protected function hmac_md5($key, $data)
		{
			if (strlen($key) > 64)
				$key = pack('H*', md5($key));
			$key = str_pad($key, 64, chr(0));
			
			$ipad = str_repeat(chr(0x36), 64);
			$opad = str_repeat(chr(0x5c), 64);
			
			return md5(($key ^ $opad) . pack('H*', md5(($key ^ $ipad) . $data)));
		}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			return 'USD';
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('LoginID', 'Mode', 'TransactionKey', 'MD5Hash', 'TimezoneOffset'));
			
			$x_fp_sequence = scOrdersInProgress::singleton()->getLastOrderId();
			$x_fp_timestamp = time() - $options['TimezoneOffset'];
			$x_fp_hash = $this->hmac_md5($options['TransactionKey'],
				$options['LoginID'] . '^' . $x_fp_sequence . '^' . $x_fp_timestamp . '^' . $payAmount . '^' . 'USD');
			
			$params = array(
				'x_login' => $options['LoginID'],
				'x_version' => '3.1',
				'x_test_request' => ($options['Mode'] != self::MODE_LIVE),
				
				'x_fp_hash' => $x_fp_hash,
				'x_fp_sequence' => $x_fp_sequence,
				'x_fp_timestamp' => $x_fp_timestamp,
				
				'x_show_form' => 'PAYMENT_FORM',
				
				'x_amount' => $payAmount,
				'x_currency_code' => 'USD',
				'x_invoice_num' => scOrdersInProgress::singleton()->getLastOrderId(),
				
				'x_relay_response' => 'FALSE',
				'x_relay_url' => scPaths::getSecureUrl('modules/payment/authorize/return.php'),
				
				'x_receipt_link_method' => 'GET',
				'x_receipt_link_text' => 'Text',
				'x_receipt_link_url' => scPaths::getSecureUrl('modules/payment/authorize/return.php'),
				
				'x_first_name' => $customerInfo['CustomerBillingFirstName'],
				'x_last_name' => $customerInfo['CustomerBillingLastName'],
				'x_company' => $customerInfo['CustomerBillingCompany'],
				'x_address' => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				'x_city' => $customerInfo['CustomerBillingCity'],
				'x_state' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'x_zip' => $customerInfo['CustomerBillingZip'],
				'x_country' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'x_email' => $customerInfo['CustomerEmail'],
				'x_phone' => $customerInfo['CustomerBillingPhone'],
				
				'x_ship_to_first_name' => $customerInfo['CustomerShippingFirstName'],
				'x_ship_to_last_name' => $customerInfo['CustomerShippingLastName'],
				'x_ship_to_address' => $customerInfo['CustomerShippingAddress1'] . ' ' . $customerInfo['CustomerShippingAddress2'],
				'x_ship_to_city' => $customerInfo['CustomerShippingCity'],
				'x_ship_to_state' => scCountry::getStateCode($customerInfo['CustomerShippingState']),
				'x_ship_to_zip' => $customerInfo['CustomerShippingZip'],
				'x_ship_to_country' => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']),
			);
			
			$url = ($options['Mode'] == self::MODE_DEVELOPER) ? self::AUTHORIZE_NET_DEVELOPER_URL : self::AUTHORIZE_NET_URL;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId);
			$key = md5($options['MD5Hash'] . $options['LoginID'] . $params['x_trans_id'] . $params['x_amount']);
			if (strtoupper($key) != $params['x_MD5_Hash'])
				throw new scPaymentProcessException(_t('MD5 Hash does not match.'), array(_t('OrderID') => $params['x_invoice_num']));
			
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($params['x_invoice_num'], $params['x_amount'], 'USD');
			
			$result = new scPaymentProcessResult($params['x_invoice_num'], $params['x_amount'], 'USD', $params['x_trans_id']);
			$result->setTransactionNote($params['x_response_reason_text'] . ' (' . $params['x_response_reason_code'] . ')');
			$result->setPaymentNotes($result->getTransactionNote());
			if ($params['x_response_code'] == 1)
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			elseif ($params['x_response_code'] == 2)
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			elseif ($params['x_response_code'] == 4) {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
			} else
				throw new scPaymentProcessException(_t('Processing error.'),
					array(_t('OrderID') => $params['x_invoice_num'],
					_t('Response Code') => $params['x_response_code'], _t('Response Subcode') => $params['x_response_subcode'],
					_t('Response Reason Text') => $params['x_response_reason_text'], _t('Response Reason Code') => $params['x_response_reason_code']));
			
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'LoginID',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'TransactionKey',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'MD5Hash',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'TimezoneOffset',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$modes = array(self::MODE_LIVE => _t('Live'), self::MODE_DEMO => _t('Demo'), self::MODE_DEVELOPER => _t('Developer'));
			$options->getOptionField('Mode')->setOptions($modes);
			$options->setTemplateFile('modules/payment/authorize/sim_options.tmpl');
			return $options;
		}
	}
?>