<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scPaymentModuleBoricaGateSetup implements IAdminModule {
		/**
		 * Initializes the action handlers for this module
		 * 
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('acBoricaSetupWizard', 'scPaymentModuleBoricaSetupWizardAction');
			$page->getDispatcher()->setActionHandler('acGenerateApgwKeyandCSR', 'scPaymentModuleBoricaGenerateApgwKeyandCSRAction');
			$page->getDispatcher()->setActionHandler('acGenerateEtlogKeyandCSR', 'scPaymentModuleBoricaGenerateEtlogKeyandCSRAction');
			$page->getDispatcher()->setActionHandler('acDownloadApgwCSR', 'scPaymentModuleBoricaDownloadApgwCSRAction');
			$page->getDispatcher()->setActionHandler('acDownloadApgwTestCSR', 'scPaymentModuleBoricaDownloadApgwTestCSRAction');
			$page->getDispatcher()->setActionHandler('acDownloadEtlogCSR', 'scPaymentModuleBoricaDownloadEtlogCSRAction');
			$page->getDispatcher()->setActionHandler('acGenerateP12File', 'scPaymentModuleBoricaGenerateP12FileAction');
			$page->getDispatcher()->setActionHandler('acReturnToBoricaModule', 'scPaymentModuleBoricaReturnToEditAction');
		}
		
		/**
		 * Returns the view that should be displayed
		 * 
		 * @return miDefaultView
		 */
		public function getView()
		{
			return new scView();
		}
	}
	
	class scPaymentModuleBoricaPKI {
		/**
		 * Generates new private key suitable for Borica
		 * 
		 * @return string
		 */
		public function generatePrivateKeyAndCsr()
		{
			// Generate private key
			$privateKey = openssl_pkey_new(array('private_key_type' => OPENSSL_KEYTYPE_RSA, 'private_key_bits' => 1024));
			if ($privateKey === false)
				$this->handleOpensslError();
			if (!openssl_pkey_export($privateKey, $privateKeyString))
				$this->handleOpensslError();
			
			// Generate CSR
			$dn = $this->getCertDistinguishedName();
			if ($dn['countryName'] == '')
				throw new miException(_t('Store owner country not set.'));
			if ($dn['localityName'] == '')
				throw new miException(_t('Store owner city not set.'));
			if ($dn['organizationName'] == '')
				throw new miException(_t('Store owner company name not set.'));
			if ($dn['emailAddress'] == '')
				throw new miException(_t('Administrator email address not set.'));
			
			$csr = openssl_csr_new($dn, $privateKey);
			if ($csr === false)
				$this->handleOpensslError();
			if (!openssl_csr_export($csr, $csrString))
				$this->handleOpensslError();
			
			return array($privateKeyString, $csrString);
		}
		
		protected function handleOpensslError()
		{
			$error = _i('OpenSSL error: ');
			while ($msg = openssl_error_string())
				$error .= $msg . "\n";
			throw new miException($error);
		}
		
		public function getCertDistinguishedName()
		{
			$dn = array(
				'countryName' => scCountry::getCountryCode2(scSettings::getSettingValue(scSettings::SETTING_COMPANY_COUNTRY)),
				'stateOrProvinceName' => scCountry::getStateName(scSettings::getSettingValue(scSettings::SETTING_COMPANY_STATE)),
				'localityName' => scSettings::getSettingValue(scSettings::SETTING_COMPANY_CITY),
				'organizationName' => scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME),
				'organizationalUnitName' => _i('Summer Cart CSR Generator'),
				'commonName' => scPaths::getHttpHost(),
				'emailAddress' => scSettings::getSettingValue(scSettings::SETTING_EMAIL_ADMINISTRATOR),
			);
			
			// If state is not set - set it the same as city
			if ($dn['stateOrProvinceName'] == '')
				$dn['stateOrProvinceName'] = $dn['localityName'];
			
			return $dn;
		}
		
		public function generateP12File($certitifcateString, $privateKeyString)
		{
			set_error_handler(array($this, 'errorHandler'), E_WARNING);
			$cert = openssl_x509_read($certitifcateString);
			openssl_pkcs12_export($cert, $out, $privateKeyString, null, array('friendly_name' => _i('Borica eTLog Certificate')));
			restore_error_handler();
			return $out;
		}
		
		public function errorHandler($errno, $errstr)
		{
			throw new miException($errstr);
		}
	}
	
	
	class scPaymentModuleBoricaActions extends miAction {
		
		/**
		 * @param string $filePath
		 * @param string $contents
		 */
		protected function downloadContents($filePath, $contents)
		{
			header('Pragma: public');
			header('Cache-control: private, must-revalidate, max-age=0');
			header('Expires: 0');
			header('Content-Type: application/octet-stream');
			header('Content-length: ' . strlen($contents));
			header('Content-disposition: attachment; filename="' . basename($filePath) . '"');
			header('Connection: close');
			
			echo $contents;
			die;
		}
		
		protected function redirectToBoricaSetup($msg = '', $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = scPaths::getCurrentScriptPathName() . '?ModuleName=com.summercart.payment.borica.gate&action=acBoricaSetupWizard';
			$loc = scUrlUtil::convertUrl($loc, scUrlUtil::URL_TYPE_DOCROOT);
			$loc = scUrlUtil::addQueryParams($loc, $this->_view->getRedirectParams($msg, $msgType));
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($controllerCommand);
		}
	}
	
	class scPaymentModuleBoricaGenerateApgwKeyandCSRAction extends scPaymentModuleBoricaActions
	{
		public function doAction()
		{
			try {
				$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.borica.gate');
				$pki = new scPaymentModuleBoricaPKI;
				
				list($privateKeyString, $csrString) = $pki->generatePrivateKeyAndCsr();
				scModuleOptions::setModuleOption($moduleId, 'ApgwPrivateKey', $privateKeyString);
				scModuleOptions::setModuleOption($moduleId, 'ApgwCertificateSignRequest', $csrString);
				
				list($privateKeyString, $csrString) = $pki->generatePrivateKeyAndCsr();
				scModuleOptions::setModuleOption($moduleId, 'ApgwTestPrivateKey', $privateKeyString);
				scModuleOptions::setModuleOption($moduleId, 'ApgwTestCertificateSignRequest', $csrString);
				
				$this->redirectToBoricaSetup(_t('Two private keys and CSRs generated.'), miMessage::MSG_TYPE_INFO);
			} catch (Exception $exception) {
				$this->redirectToBoricaSetup(_t('Error generating key: ') . $exception->getMessage());
			}
		}
	}
	
	class scPaymentModuleBoricaGenerateEtlogKeyandCSRAction extends scPaymentModuleBoricaActions
	{
		public function doAction()
		{
			try {
				$pki = new scPaymentModuleBoricaPKI;
				list($privateKeyString, $csrString) = $pki->generatePrivateKeyAndCsr();
				
				$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.borica.gate');
				
				scModuleOptions::setModuleOption($moduleId, 'EtlogPrivateKey', $privateKeyString);
				scModuleOptions::setModuleOption($moduleId, 'EtlogCertificateSignRequest', $csrString);
				scModuleOptions::setModuleOption($moduleId, 'EtlogCertificate', '');
				
				$this->redirectToBoricaSetup(_t('Private key and CSR generated.'), miMessage::MSG_TYPE_INFO);
			} catch (Exception $exception) {
				$this->redirectToBoricaSetup(_t('Error generating key: ') . $exception->getMessage());
			}
		}
	}
	
	class scPaymentModuleBoricaDownloadAction extends scPaymentModuleBoricaActions {
		var $_moduleOptionName = '';
		var $_fileName = '';
		
		public function doAction()
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.borica.gate');
			$csrString = scModuleOptions::getModuleOption($moduleId, $this->_moduleOptionName);
			if ($csrString == '') {
				$this->redirectToBoricaSetup(_t('Generate key first.'));
				return;
			}
			
			$this->downloadContents($this->_fileName, $csrString);
		}
	}
	
	class scPaymentModuleBoricaDownloadApgwCSRAction extends scPaymentModuleBoricaDownloadAction
	{
		var $_moduleOptionName = 'ApgwCertificateSignRequest';
		var $_fileName = 'apgw.csr';
	}
	
	class scPaymentModuleBoricaDownloadApgwTestCSRAction extends scPaymentModuleBoricaDownloadAction
	{
		var $_moduleOptionName = 'ApgwTestCertificateSignRequest';
		var $_fileName = 'apgwtest.csr';
	}
	
	class scPaymentModuleBoricaDownloadEtlogCSRAction extends scPaymentModuleBoricaDownloadAction
	{
		var $_moduleOptionName = 'EtlogCertificateSignRequest';
		var $_fileName = 'etlog.csr';
	}
	
	class scPaymentModuleBoricaGenerateP12FileAction extends scPaymentModuleBoricaActions
	{
		public function doAction()
		{
			try {
				$etlogCertificate = miGetParam('EtlogCertificate');
				if ((strpos($etlogCertificate, 'BEGIN CERTIFICATE') === false)) {
					$this->redirectToBoricaSetup(_t('Invalid certificate.'));
					return;
				}
				
				$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.borica.gate');
				$privateKeyString = scModuleOptions::getModuleOption($moduleId, 'EtlogPrivateKey');
				if ((strpos($privateKeyString, 'BEGIN RSA PRIVATE KEY') === false)) {
					$this->redirectToBoricaSetup(_t('Generate key first.'));
					return;
				}
				
				$pki = new scPaymentModuleBoricaPKI;
				$p12Contents = $pki->generateP12File($etlogCertificate, $privateKeyString);
				
				scModuleOptions::setModuleOption($moduleId, 'EtlogCertificate', $etlogCertificate);
				$this->downloadContents('etlog.p12', $p12Contents);
			} catch (miException $exception) {
				$this->redirectToBoricaSetup($exception->getMessage());
			}
		}
	}
	
	class scPaymentModuleBoricaReturnToEditAction extends scPaymentModuleBoricaActions
	{
		public function doAction()
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.borica.gate');
			$paymentModuleId = scUtil::getDbValue('PaymentModules', 'PaymentModuleID', 'ModuleID', $moduleId, 0);
			
			$loc = 'payment_modules.php?action=dmEdit&PaymentModuleID=' . $paymentModuleId;
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($controllerCommand);
		}
	}
	
	class scPaymentModuleBoricaSetupWizardAction extends miAction {
		public function doAction()
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.borica.gate');
			
			$pki = new scPaymentModuleBoricaPKI;
			$dn = $pki->getCertDistinguishedName();
			$dn = miUIUtil::templetizeAndEscapeRows($dn);
			
			$miMsg = new miMessage();
			
			$parser = new scTemplateParser();
			$parser->readTemplate('modules/payment/borica/generate_keys.tmpl');
			$parser->assign('%%HAS_APGW_KEY%%', $this->hasApgwKey($moduleId) ? 'true' : 'false');
			$parser->assign('%%HAS_ETLOG_KEY%%', $this->hasEtlogKey($moduleId) ? 'true' : 'false');
			$parser->assign('%%STEP1%%', $this->getStep1($moduleId));
			$parser->assign('%%STEP2%%', $this->getStep2($moduleId));
			$parser->assign('%%STEP3%%', $this->getStep3($moduleId));
			$parser->assign('%%STEP4%%', $this->getStep4($moduleId));
			$parser->assignArray($miMsg->getMessageTemplateVars());
			$parser->assignArray($dn);
			$html = $parser->templateParse();
			
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($controllerCommand);
		}
		
		protected function hasApgwKey($moduleId)
		{
			$privateKeyString = scModuleOptions::getModuleOption($moduleId, 'ApgwPrivateKey');
			$csrString = scModuleOptions::getModuleOption($moduleId, 'ApgwCertificateSignRequest');
			$privateKeyTestString = scModuleOptions::getModuleOption($moduleId, 'ApgwTestPrivateKey');
			$csrTestString = scModuleOptions::getModuleOption($moduleId, 'ApgwTestCertificateSignRequest');
			
			if ((strpos($privateKeyString, 'BEGIN RSA PRIVATE KEY') !== false) and (strpos($csrString, 'BEGIN CERTIFICATE REQUEST') !== false) and
			(strpos($privateKeyTestString, 'BEGIN RSA PRIVATE KEY') !== false) and (strpos($csrTestString, 'BEGIN CERTIFICATE REQUEST') !== false))
				return true;
			return false;
		}
		
		protected function hasEtlogKey($moduleId)
		{
			$privateKeyString = scModuleOptions::getModuleOption($moduleId, 'EtlogPrivateKey');
			$csrString = scModuleOptions::getModuleOption($moduleId, 'EtlogCertificateSignRequest');
			
			if ((strpos($privateKeyString, 'BEGIN RSA PRIVATE KEY') !== false) and (strpos($csrString, 'BEGIN CERTIFICATE REQUEST') !== false))
				return true;
			return false;
		}
		
		protected function getStep1($moduleId)
		{
			if ($this->hasApgwKey($moduleId))
				return _t('OK');
			return _t('Pending');
		}
		
		protected function getStep2($moduleId)
		{
			if ($this->hasEtlogKey($moduleId))
				return _t('OK');
			return _t('Pending');
		}
		
		protected function getStep3($moduleId)
		{
			$tid = scModuleOptions::getModuleOption($moduleId, 'TID');
			
			if (strlen($tid) > 0)
				return _t('OK');
			return _t('Pending');
		}
		
		protected function getStep4($moduleId)
		{
			$certificateString = scModuleOptions::getModuleOption($moduleId, 'EtlogCertificate');
			
			if ((strpos($certificateString, 'BEGIN CERTIFICATE') !== false))
				return _t('OK');
			return _t('Pending');
		}
	}
?>