<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/gate.php');

	class scPaymentModuleBoricaOperations implements IPaymentModuleOperations {
		const TEMPLATE_REFUND = 'modules/payment/borica/reversal.tmpl';
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentOperationDefinitions(scOrder $order)
		{
			$operations = array();
			$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_REFUND, 'Refund', array($this, 'refund'), _t('Reversal'));
			$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_REFUND, 'RefundConfirm', array($this, 'refundConfirm'));
			return $operations;
		}
		
		public function refund(scOrder $order, scView $view)
		{
			$alreadyCapturedAmount = $this->getAlreadyCapturedAmount($order);
			
//			$currency = new scCurrency($order->get('OrderTotalPayCurrencyCode'));
//			$amount = scMultiCurrencyAmount::factory();
//			$amount->set($currency, $alreadyCapturedAmount);
//			
//			$displayAlreadyCapturedAmount = $alreadyCapturedAmount ? scPrice::factory($amount)->getAmountInCurrency($currency) : '0';
			
			$t = new scTemplateParser();
			$t->readTemplate(self::TEMPLATE_REFUND);
//			$t->assign('%%ALREADY_CAPTURED_AMOUNT%%', miI18N::htmlEscape($displayAlreadyCapturedAmount));
			$t->assign('%%REFUNDAMOUNT%%', miI18N::htmlEscape($alreadyCapturedAmount));
			
			$this->displayForm($view, $order, $t);
		}
		
		public function refundConfirm(scOrder $order, scView $view)
		{
			try {
				$refundAmount = miGetParamDefault('RefundAmount', 0);
				
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('TID', 'Mode', 'AccountCurrencyCode', 'ApgwPrivateKey', 'ApgwTestPrivateKey', 'ApgwCertificate', 'ApgwTestCertificate'));
				
				$api = new scPaymentModuleBoricaApi($options['TID'], $options['Mode'], $options['ApgwPrivateKey'], $options['ApgwTestPrivateKey'], $options['ApgwCertificate'], $options['ApgwTestCertificate']);
				$result = $api->doReversal($order->getPK(), $refundAmount, $order->get('OrderTotalPayCurrencyCode'));
				
				// Insert transaction
				$paymentStatus = ($order->get('OrderTotalPayAmount') == $refundAmount) ? scPaymentStatus::PAYMENT_STATUS_REFUND : scPaymentStatus::PAYMENT_STATUS_PARTIAL_REFUND;
				$order->updateOrderPaymentStatus($paymentStatus);
				$transaction = new scTransaction($order->get('OrderID'), $this->_moduleId, $refundAmount, $order->get('OrderTotalPayCurrencyCode'), '', $paymentStatus);
				$transaction->insertTransaction();
								
				$view->addRedirectToEditControllerCommand(_t('Funds refunded successfully.'), miMessage::MSG_TYPE_INFO);
			} catch (scPaymentProcessException $exception) {
				throw new miException($exception->getMessage() . $exception->getDetailsAsString());
			}
		}
		
		protected function displayForm($view, $order, $t)
		{
			// Get the order information
			$view->addMainPageElements(miUIUtil::templetizeRows($order->getRow()));
			
			miTable::escapeAndAssignArray($t, $view->getMainPageElements());
			$t->assign('%%ORDERID%%', miI18N::htmlEscape($order->getPK()));
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $t->templateParse());
			$view->addControllerCommand($cmd);
		}
		
		protected function getAlreadyCapturedAmount(scOrder $order)
		{
			$sql = 'SELECT SUM(TransactionAmount) AS Sum FROM Transactions WHERE OrderID=? AND TransactionPaymentStatus IN (?,?)';
			$result = miStaticDBUtil::execSelect($sql, array($order->getPK(), scPaymentStatus::PAYMENT_STATUS_CAPTURE, scPaymentStatus::PAYMENT_STATUS_PARTIAL_CAPTURE));
			return $result[0]['Sum'];
		}
	}
?>