<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * CCNow payment processing module
	 */
	 
	 class scPaymentModuleCcnowHostedPage implements IPaymentModule, IModuleOptions {
		const CCNOW_URL = 'https://www.ccnow.com/cgi-local/transact.cgi';
		const CCNOW_IP_ADDRESS = '209.87.181.254';
		
		const METHOD_CC = 'CC';
		const METHOD_PAYPAL = 'PAYPAL';
		const METHOD_TEST = 'TEST';
		const METHOD_NONE = 'NONE';
		
		protected $_paymentMethods = array(
			self::METHOD_CC => 'Credit card',
			self::METHOD_PAYPAL => 'PayPal',
			self::METHOD_TEST => 'Test order',
			self::METHOD_NONE => 'CCNow checkout page'
		);
		
		protected static $_allowedCurrencies = array('USD', 'EUR', 'AUD', 'CAD', 'CHF', 'DKK', 'GBP', 'HKD', 'INR', 'JPY', 'NOK', 'NZD', 'SEK', 'ZAR');
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), self::$_allowedCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('ClientID', 'ActivationKey', 'Method'));
			
			$customerInfo = $cart->getCustomerInfo();
			
			$currency = new scCurrency($payCurrencyCode);
			$itemsData = $this->getItemsData($cart, $currency);
			list($shippingAmount, $shippingMethod, $discountAmount, $discountLabel) = $this->getShippingAndDiscount($cart, $itemsData, $payAmount, $currency);
			
			$clientId = $options['ClientID'];
			$activationKey = $options['ActivationKey'];
			$method = $options['Method'];
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$data = array(
				'x_login' => $clientId,
				'x_version' => '1.0',
				'x_fp_sequence' => $orderId . time(),
				'x_currency_code' => $payCurrencyCode,
				'x_method' => $method,
				'x_name' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName']),
				'x_company' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerBillingCompany']),
				'x_address' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerBillingAddress1']),
				'x_address2' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerBillingAddress2']),
				'x_city' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerBillingCity']),
				'x_state' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'x_zip' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerBillingZip']),
				'x_country' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'x_phone' => $customerInfo['CustomerBillingPhone'],
				'x_email' => $customerInfo['CustomerEmail'],
				'x_ship_to_name' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerShippingFirstName'] . ' ' . $customerInfo['CustomerShippingLastName']),
				'x_ship_to_company' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerShippingCompany']),
				'x_ship_to_address' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerShippingAddress1']),
				'x_ship_to_address2' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerShippingAddress2']),
				'x_ship_to_city' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerShippingCity']),
				'x_ship_to_state' => scCountry::getStateCode($customerInfo['CustomerShippingState']),
				'x_ship_to_zip' => scFriendlyUrl::transliterateCyrillic($customerInfo['CustomerShippingZip']),
				'x_ship_to_country' => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']),
				'x_ship_to_phone' => $customerInfo['CustomerShippingPhone'],
				'x_shipping_amount' => $shippingAmount,
				'x_shipping_label' => '',
				'x_shipping_method' => scFriendlyUrl::transliterateCyrillic($shippingMethod),
				'x_discount_amount' => $discountAmount,
				'x_discount_label' => scFriendlyUrl::transliterateCyrillic($discountLabel),
				'x_invoice_num' => $orderId,
				'x_instructions' => scFriendlyUrl::transliterateCyrillic($customerInfo['OrderCustomerInstructions']),
//				'x_callback_url' => 'http://internal.mirchevideas.com:803/test.php?orderid=#order_id#&invoice_num=#invoice_num#&order_amount=#order_amount#&order_method=#order_method#&order_status=#order_status#',
				'x_callback_url' => scPaths::getSecureUrl() . 'modules/payment/ccnow/callback.php?OrderId=#invoice_num#&OrderAmount=#order_amount#&Status=#order_status#',
				'x_amount' => $payAmount
			);
			
			$data += $itemsData;
			
			$this->calcFpHash($data, $clientId, $activationKey);
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(self::CCNOW_URL, $data);
			return $result;
		}
		
		protected function getShippingAndDiscount($cart, $itemsData, $payAmount, $currency)
		{
			$shippingMethod = $this->getShippingLineText($cart->getCustomerInfo());
			$shippingAmount = $cart->getOrderTotal()->getSummedOrderTotalLinesAmount(IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING)->get($currency);
			$shippingAmount = round($shippingAmount, 2);
			
			// The total amount sent to CCNow must be:
			// x_amount = sum(x_product_unitprice * x_product_quantity) + x_shipping_amount - x_discount_amount
			// Since it is possible that there are rounding errors we put any rounding errors in the discount amount
			$itemsAmount = 0;
			foreach ($itemsData as $key => $itemData) {
				if (substr($key, 0, 20) == 'x_product_unitprice_')
					$itemsAmount += $itemData * $itemsData['x_product_quantity_' . substr($key, 20)];
			}
			
			$discountLabel = '';
			$discountAmount = sprintf('%.2f', $itemsAmount + $shippingAmount - $payAmount);
			
			// Since CCNow does not allow negative discount nor negative shipping,
			// we need to move the amount from shipping to discount or vice versa to make it work
			if ($shippingAmount < 0) {
				$discountAmount -= $shippingAmount;
				$shippingAmount = 0;
			}
			if ($discountAmount < 0) {
				$shippingAmount -= $discountAmount;
				$discountAmount = 0;
			}
			
			$shippingAmount = sprintf('%.2f', $shippingAmount);
			$discountAmount = sprintf('%.2f', $discountAmount);
			
			return array($shippingAmount, $shippingMethod, $discountAmount, $discountLabel);
		}
		
		protected function getShippingLineText($customerInfo)
		{
			$shippingModuleId = 0;
			$shippingMethodCode = '';
			if (count(explode('-', $customerInfo['ShippingModuleAndMethod'])) == 2)
				list($shippingModuleId, $shippingMethodCode) = explode('-', $customerInfo['ShippingModuleAndMethod']);
			
			list(, $shippingMethodName) = scOrdersInProgress::singleton()->getShippingModuleAndMethodNames($shippingModuleId, $shippingMethodCode);
			
			return $shippingMethodName;
		}
		
		protected function calcFpHash(&$data, $clientId, $activationKey)
		{
			$args = array_keys($data);
			$args[] = 'x_fp_arg_list';
			$data['x_fp_arg_list'] = implode('^', $args);
			
			$values = array_values($data);
			$data['x_fp_hash'] = md5(implode('^', $values) . '^' . $activationKey);
		}
		
		protected function getItemsData($cart, $currency)
		{
			$items = $cart->getItems();
			$itemsData = array();
			$n = 0;
			foreach ($items as $item) {
				$n++;
				
				// Get the product details url
				$productInfo = $item->getProductInfo();
				$productUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_PRODUCT, $productInfo['ProductID'], $productInfo['ProductName']);
				
				$itemsData['x_product_sku_' . $n] = $item->getProductCode();
				$itemsData['x_product_title_' . $n] = scFriendlyUrl::transliterateCyrillic($item->getName());
				$itemsData['x_product_quantity_' . $n] = $item->getQuantity();
				$itemsData['x_product_unitprice_' . $n] = $item->getPriceObj()->getAmountInCurrency($currency);
				$itemsData['x_product_url_' . $n] = scPaths::getUrl($productUrl);
			}
			
			return $itemsData;
		}
		
		 /**
		  * Handles payment return
		  *
		  * @param array $params
		  * @return scPaymentProcessResult
		  */
		public function processReturn($params)
		{
			scSession::singleton()->openSession();
			$params['OrderId'] = scOrdersInProgress::singleton()->getLastOrderId();
			
			// Check transaction
			$transactionStatus = scUtil::getDbValue('Transactions', 'TransactionPaymentStatus', 'OrderID', $params['OrderId']);
			if ($transactionStatus == scPaymentStatus::PAYMENT_STATUS_CAPTURE)
				$params['Status'] = scPaymentStatus::PAYMENT_STATUS_CAPTURE;
			else
				$params['Status'] = scPaymentStatus::PAYMENT_STATUS_PENDING;
			
			scPaymentProcess::checkParamsExist($params, array('OrderId', 'Status'));
			
			$result = new scPaymentProcessResult($params['OrderId']);
			return $result;
		}
		
		/**
		 * Handles payment notification
		 * 
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processCallback($params)
		{
			// Security check
			if ((ip2long($_SERVER['REMOTE_ADDR']) | 255) != (ip2long(self::CCNOW_IP_ADDRESS) | 255)) {
				throw new scPaymentProcessException(_i('CCNow: Callback from unknown server.'), $params + $_SERVER);
			}
			
			scPaymentProcess::checkParamsExist($params, array('OrderId', 'Status', 'OrderAmount'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Method'));
			
			scPaymentProcess::checkPaymentToOrder($params['OrderId'], $params['OrderAmount']);
			
        	$result = new scPaymentProcessResult($params['OrderId'], $params['OrderAmount']);
        	switch ($params['Status']) {
				case 'accept':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					break;
				
				case 'test':
					if ($options['Method'] != self::METHOD_TEST)
						throw new scPaymentProcessException(_t('Test payment.'), $params);
					
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					$result->setTransactionNote(_t('Test payment.'));
					$result->setPaymentNotes(_t('Order in demo mode.'));
					break;
				
				// Everything else: decline
				default:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					break;
			}
			
			$result->setTransactionDetail('Status', $params['Status']);
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ClientID',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ActivationKey',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'Method',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$options->getOptionField('Method')->setOptions($this->_paymentMethods);
			sort(self::$_allowedCurrencies);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine(self::$_allowedCurrencies, self::$_allowedCurrencies), 'USD');
			$options->setTemplateFile('modules/payment/ccnow/ccnow_options.tmpl');
			return $options;
		}
	}
?>