<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Collect On Delivery payment processing module
	 */
	class scPaymentModuleCollectOnDelivery implements IPaymentModule, IModuleOptions {
		protected $_moduleId;

		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return false;
		}
		
		public function getPaymentCurrencyCode()
		{
			return null;
		}
		
		/**
		 * Process the payment
		 *
		 * @param string $payAmount the amount to be paid
		 * @param string $payCurrencyCode the currency
		 * @param scCart $cart the shopping cart object
		 * @param array $paramsPost any parameters posted by the user from the payment form
		 * @return scPaymentProcessResult the payment process result
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$result = new scPaymentProcessResult($orderId);
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_OFFLINE);
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MaximumAmount'));
			
			if ($payAmount > $options['MaximumAmount'])
				$result->addValidatorError(sprintf(_t('You exceeded the Collect On Delivery payment limit. Maximum amount is %d %s.'), $options['MaximumAmount'], $payCurrencyCode));
			
			return $result;
		}
		
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'scAmountWidget',
					'data' => 'MaximumAmount',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->setTemplateFile('modules/payment/cod/options.tmpl');
			return $options;
		}
	}
?>