<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Credit Card payment processing module
	 */
	class scPaymentModuleCreditCard implements IPaymentModule, IModuleOptions {
		protected $_moduleId;

		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		
		public function getPaymentContent($contentType, $params = array())
		{
			if ($contentType == self::CONTENT_TYPE_PAYMENT_FORM) 
				return scPaymentForms::getCreditCardForm();
		}
		
		public function hasCheckoutStep()
		{
			return false;
		}
		
		public function getPaymentCurrencyCode()
		{
			return null;
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$result = new scPaymentProcessResult($orderId);
			if (empty($paramsPost['NameOnCard']))
				$result->addValidatorError(_t('Please enter the name on card.'), 'NameOnCard');
			if (!scPaymentForms::isCardNumberValid($paramsPost['CardNumber']))
				$result->addValidatorError(_t('Invalid card number.'), 'CardNumber');
				
			if (!scPaymentForms::isCardExpirationValid($paramsPost['CardExpirationYear'], $paramsPost['CardExpirationMonth']))
				$result->addValidatorError(_t('Invalid expiration date.'), 'CardExpirationYear');
			
			if (!ctype_digit($paramsPost['CardCVV']) and strlen($paramsPost['CardCVV']))
				$result->addValidatorError(_t('Invalid CVV code.'), 'CardCVV');
			
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_OFFLINE);
			
			// Success - save the credit card info
			if (!$result->hasValidatorErrors())
				$this->saveCreditCardInfo($orderId, $paramsPost);
			
			return $result;
		}
		
		protected function saveCreditCardInfo($orderId, $paramsPost)
		{
			$paymentCreditCard = new scPaymentCreditCard($orderId);
			$paymentCreditCard->set('PaymentCreditCardNumber', str_replace(array(' ', '-'), '', $paramsPost['CardNumber']));
			$paymentCreditCard->set('PaymentCreditCardNameOnCard', $paramsPost['NameOnCard']);
			$paymentCreditCard->set('PaymentCreditCardExpMonth', $paramsPost['CardExpirationMonth']);
			$paymentCreditCard->set('PaymentCreditCardExpYear', $paramsPost['CardExpirationYear']);
			$paymentCreditCard->set('PaymentCreditCardCVN', $paramsPost['CardCVV']);
			$paymentCreditCard->save();
		}
		
		function getOptionFields(miWebForm &$webForm)
		{
			return null;
		}
	}
?>