<?php
	/**
	 * Copyright (c) 2008,2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Epay.bg payment processing module
	 */
	 
	 class scPaymentModuleEpayBgPayLogin implements IPaymentModule, IModuleOptions {
		const EPAY_LIVE_BG_URL = 'https://www.epay.bg/';
		const EPAY_LIVE_EN_URL = 'https://www.epay.bg/en/';
		const EPAY_DEVELOPER_URL = 'https://devep2.datamax.bg/ep2/epay2_demo/';
		
		const MODE_LIVE = 'Live';
		const MODE_DEVELOPER = 'Developer';
		
		protected $_moduleId;
		
		/**
		 * Returns the page parameter, sent to ePay, which actually is the ePayBg's payment method
		 */
				
		protected function getPageParam()
		{
			return 'paylogin';
		}
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			return 'BGN';
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			//$customerInfo = $cart->getCustomerInfo(); // not used
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MIN', 'SecretKey', 'ExpirationTimeHours', 'OrderDescription', 'Mode'));
			
			$loginId = $options['MIN'];
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			$expDate = date('d.m.Y H:i', time() + $options['ExpirationTimeHours'] * 60 * 60);
			$secret = $options['SecretKey'];
			
			$data = array(
				'MIN' => $loginId,
				'INVOICE' => $orderId,
				'AMOUNT' => $payAmount,
				'EXP_TIME' => $expDate,
				'CURRENCY' => 'BGN',
				'DESCR' => $options['OrderDescription'],
				'ENCODING' => 'utf-8'
			);
			
			$content = '';
			foreach ($data as $key => $val) {
				$content .= $key . '=' . $val . "\n";
			}
			$encoded = base64_encode($content);
			$checksum = scCryptUtil::hmac($secret, $encoded, scCryptUtil::HASH_SHA1);
			$params = array(
				'PAGE' => $this->getPageParam(),
				'ENCODED' => $encoded,
				'CHECKSUM' => $checksum,
				'URL_OK' => scPaths::getSecureUrl(sprintf('modules/payment/epay_bg/return.php?Status=%d&OrderId=%d', scPaymentStatus::PAYMENT_STATUS_PENDING, $orderId)),
				'URL_CANCEL' => scPaths::getSecureUrl(sprintf('modules/payment/epay_bg/return.php?Status=%d&OrderId=%d', scPaymentStatus::PAYMENT_STATUS_DECLINED, $orderId)),
			);
			
			$isoLangCode = strtoupper(scMultiLanguage::getCurrentLanguageCode());
			if ($options['Mode'] == self::MODE_DEVELOPER)
				$url = self::EPAY_DEVELOPER_URL;
			else if ($isoLangCode == 'BG')
				$url = self::EPAY_LIVE_BG_URL;
			else
				$url = self::EPAY_LIVE_EN_URL;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($url, $params);
			return $result; 
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('OrderId', 'Status'));
			
			$result = new scPaymentProcessResult($params['OrderId']);
			if ($params['Status'] == scPaymentStatus::PAYMENT_STATUS_DECLINED) {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			} else {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
			}
			
			return $result;
		}
		
		/**
		 * Handles payment notification
		 * 
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processNotify($params)
		{
        	$result = new scPaymentProcessResult($params['OrderId']);
        	switch ($params['Status']) {
				case 'PAID':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					$result->setTransactionCode($params['STAN']);
					$result->setTransactionNote(sprintf(_t('Pay time: %s, Borica code %s.'), $params['PayTime'], $params['BCODE']));
					break;
				
				case 'EXPIRED':
					$result->setPaymentNotes(_t('Payment expired.'));
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					$result->setTransactionNote(_t('Notification.'));
					break;
				
				case 'DENIED':
					$result->setPaymentNotes(_t('Payment cancelled by customer.'));
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					$result->setTransactionNote(_t('Notification.'));
					break;
			}
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'MIN',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'SecretKey',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ExpirationTimeHours',
					'validator' => 'scValidatorInt'
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'OrderDescription',
					'properties' => array('maxLength' => 100),
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'Mode',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$modes = array(self::MODE_LIVE => _t('Live'), self::MODE_DEVELOPER => _t('Developer'));
			$options->getOptionField('Mode')->setOptions($modes);
			$options->setTemplateFile('modules/payment/epay_bg/epay_options.tmpl');
			return $options;
		}
	}
	
	/**
	 * ePay.bg PayDirect method
	 * Process payments with credit cards
	 */
	class scPaymentModuleEpayBgPayDirect extends scPaymentModuleEpayBgPayLogin {
				
		protected function getPageParam()
		{
			return 'credit_paydirect';
		}
	}
?>