<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * GECAD epayment.ro romanian payment processing module	
	 */
	class scPaymentModuleEpaymentRo implements IPaymentModule, IModuleOptions {
		
		const EPAYMENT_URL = 'https://secure.epayment.ro/order/lu.php';
		
		const MODE_LIVE = 'Live';
		const MODE_TEST = 'Test';
		
		protected $_moduleId;
		
		/**
		 * ISO 639-2 to epayment language code map
		 * @var array
		 */
		protected $_langCodeMap = array(
			// ISO 639-2 => epayment
			'ro' => 'ro',
			'en' => 'en',
			'de' => 'de',
			'fr' => 'fr',
			'it' => 'it',
			'es' => 'es',
			'hu' => 'hu',
			'pl' => 'pl',
		);
		
		protected static $_allowedCurrencies = array(
			'AED', 'AUD', 'BGN', 'BRL', 'CAD', 'CHF', 'CNY', 'CZK', 'DKK', 'EEK', 'EGP', 'EUR',
			'GBP', 'HKD', 'HRK', 'HUF', 'IDR', 'INR', 'JPY', 'KRW', 'LTL', 'LVL', 'MDL', 'MXN',
			'MYR', 'NOK', 'NZD', 'PHP', 'PLN', 'RSD', 'RON', 'RUB', 'SEK', 'SGD', 'TRY', 'UAH',
			'USD', 'ZAR'
		);
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode() 
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), self::$_allowedCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantCode', 'Mode', 'SecretKey', 'SkipForms', 'DefaultCurrencyCode'));
			
			$currency = new scCurrency($payCurrencyCode);
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = isset($this->_langCodeMap[$isoLangCode]) ? $this->_langCodeMap[$isoLangCode] : 'en';
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$params = array(
				'MERCHANT' => $options['MerchantCode'],	
				'ORDER_REF' => $orderId,
				'ORDER_DATE' => date('Y-m-d H:i:s'),
				
				'ORDER_PNAME[]' => array(),
//				'ORDER_PGROUP[]' => array(''),
				'ORDER_PCODE[]' => array(),
				'ORDER_PINFO[]' => array(),
				'ORDER_PRICE[]' => array(),
				'ORDER_QTY[]' => array(),
				'ORDER_VAT[]' => array(),
//				'ORDER_VER[]' => array(''),
				
				'ORDER_SHIPPING' => 0,
				
				'PRICES_CURRENCY' => $payCurrencyCode,
				'DESTINATION_CITY' => $customerInfo['CustomerBillingCity'],
				'DESTINATION_STATE' => scCountry::getStateName($customerInfo['CustomerBillingState']),
				'DESTINATION_COUNTRY' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
			);
			
			$items = $cart->getItems();
			$currency = new scCurrency($payCurrencyCode);
			foreach ($items as $item) {
				$params['ORDER_PNAME[]'][] = $item->getName();
				//$params['ORDER_PGROUP[]'][] = '';
				$params['ORDER_PCODE[]'][] = $item->getCartItemCode();
				$params['ORDER_PINFO[]'][] = '';
				$params['ORDER_PRICE[]'][] = $item->getPriceObj()->getAmountInCurrency($currency);
				$params['ORDER_QTY[]'][] = $item->getQuantity();
				$params['ORDER_VAT[]'][] = '0';
				//$params['ORDER_VER[]'][] = '';
			}
			
			$orderTotalLines = $cart->getOrderTotal()->getOrderTotalLines();
			foreach ($orderTotalLines as $i => $orderTotalLine) {
				if ($orderTotalLine->getType() == IOrderTotalModule::ORDER_TOTAL_TYPE_SUBTOTAL)
					continue;
				
				$params['ORDER_PNAME[]'][] = $orderTotalLine->getDescription();
				//$params['ORDER_PGROUP[]'][] = '';
				$params['ORDER_PCODE[]'][] = 'ORDERTOTALLINE' . $i;
				$params['ORDER_PINFO[]'][] = '';
				$params['ORDER_PRICE[]'][] = $orderTotalLine->getAmount()->get($currency);
				$params['ORDER_QTY[]'][] = 1;
				$params['ORDER_VAT[]'][] = '0';
				//$params['ORDER_VER[]'][] = '';
			}
			
			$params['ORDER_HASH'] = $this->calculateChecksum($params, $options['SecretKey']);
			
			// Add the params that are not part of the hash
			$params += array(
				'BACK_REF' => scPaths::getAbsoluteUrl('/modules/payment/epayment_ro/return.php?OrderID=' . $orderId),
				//'BACK_REF' => scPaths::getAbsoluteUrl('/modules/payment/epayment.php/ipn.php'),
				
				'TESTORDER' => $options['Mode'] == self::MODE_TEST ? 'TRUE' : 'FALSE',
				'LANGUAGE' => $lang,
				//'DEBUG' => 1,
				'AUTOMODE' => $options['SkipForms'] ? 1 : 0,
				
				'BILL_FNAME' => $customerInfo['CustomerBillingFirstName'],	
				'BILL_LNAME' => $customerInfo['CustomerBillingLastName'],
				'BILL_COMPANY' => $customerInfo['CustomerBillingCompany'],
				'BILL_FISCALCODE' => $customerInfo['CustomerBillingVatNumber'],
				'BILL_ADDRESS' => $customerInfo['CustomerBillingAddress1'],
				'BILL_ADDRESS2' => $customerInfo['CustomerBillingAddress2'],	
				'BILL_CITY' => $customerInfo['CustomerBillingCity'],	
				'BILL_ZIPCODE' => $customerInfo['CustomerBillingZip'],
				'BILL_STATE' => scCountry::getStateName($customerInfo['CustomerBillingState']),
				'BILL_COUNTRYCODE' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'BILL_PHONE' => $customerInfo['CustomerBillingPhone'],
				'BILL_FAX' => 	$customerInfo['CustomerBillingFax'],
				'BILL_EMAIL' => $customerInfo['CustomerEmail'],
				
				'DELIVERY_FNAME' => $customerInfo['CustomerBillingFirstName'],	
				'DELIVERY_LNAME' => $customerInfo['CustomerBillingLastName'],
				'DELIVERY_COMPANY' => $customerInfo['CustomerBillingCompany'],
				'DELIVERY_ADDRESS' => $customerInfo['CustomerBillingAddress1'],
				'DELIVERY_ADDRESS2' => $customerInfo['CustomerBillingAddress2'],	
				'DELIVERY_CITY' => $customerInfo['CustomerBillingCity'],	
				'DELIVERY_ZIPCODE' => $customerInfo['CustomerBillingZip'],
				'DELIVERY_STATE' => scCountry::getStateName($customerInfo['CustomerBillingState']),
				'DELIVERY_COUNTRYCODE' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'DELIVERY_PHONE' => $customerInfo['CustomerBillingPhone'],
				
//				'ORDERSTYLE' => 'nLWs45S2gH4='
			);
			
			//var_dump($params);
			//die;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(self::EPAYMENT_URL, $params);
			
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'SecretKey'));
			
			//scPaymentProcess::checkParamsExist($params, array('OrderID', '3dsecure', 'date', 'payrefno', 'ctrl'));
			scPaymentProcess::checkParamsExist($params, array('OrderID', 'ctrl'));
			
			// Validate the return
			$query = $_SERVER['QUERY_STRING'];
			$pos = strrpos($query, '&ctrl=');
			if ($pos === false)
				throw new miException(_i('Invalid hash.'));
			
			$query = substr($query, 0, $pos);
			$data = scPaths::getAbsoluteUrl('/modules/payment/epayment_ro/return.php?') . $query;
			
			$data = strlen($data) . $data;
			$hmac = scCryptUtil::hmac($options['SecretKey'], $data, scCryptUtil::HASH_MD5);
			
			if ($hmac != $params['ctrl']) {
				throw new miException(_i('Invalid hash.'));
			}
			
			// Handle the return
			$orderId = (int)$params['OrderID'];
			
			$result = new scPaymentProcessResult($orderId);
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processIpn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('REFNOEXT', 'HASH'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('SecretKey'));
			
			// Check the received data
			$dataParams = $params;
			unset($dataParams['HASH']);
			if ($params['HASH'] != $this->calculateChecksum($dataParams, $options['SecretKey']))
				throw new miException(_i('Invalid hash. Please check the secret key.'));
			
			// Check that the amount matches
			$orderId = intval($params['REFNOEXT']);
			scPaymentProcess::checkPaymentToOrder($orderId, $params['IPN_TOTALGENERAL'], $params['CURRENCY']);
			
			//var_dump($params);
			//file_put_contents('/tmp/zzz.txt', var_export($params, true), FILE_APPEND);
			
			// Print the confirmation to ePayment
			$date = date('YmdHis');
			$hash = $this->calculateChecksum(array($params['IPN_PID'][0], $params['IPN_PNAME'][0], $params['IPN_DATE'], $date), $options['SecretKey']);
			echo '<EPAYMENT>' . $date . '|' . $hash . '</EPAYMENT>';
			
			// Return transaction info
			$result = new scPaymentProcessResult($orderId, $params['IPN_TOTALGENERAL'], $params['CURRENCY'], $params['REFNO'], $params['ORDERSTATUS']);
			switch ($params['ORDERSTATUS']) {
				case 'PAYMENT_AUTHORIZED':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_AUTH);
					break;
				case 'PAYMENT_RECEIVED':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					break;
				case 'CASH':
					// FIXME: What is this status
					break;	
				case 'COMPLETE':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					break;
				case 'TEST':
					$result->setPaymentNotes(_t('Test order'));
					break;
				default:
					// If unknown code is passed, leave the payment status as is
			}
			
			foreach ($params as &$param) {
				if (is_array($param))
					$param = var_export($param, true);
			}
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
		public function getOptionFields(miWebForm &$webForm) 
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'MerchantCode',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'SecretKey',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'SkipForms',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$modes = array(self::MODE_LIVE => _t('Live'), self::MODE_TEST => _t('Test'));
			$options->getOptionField('Mode')->setOptions($modes);
			
			sort(self::$_allowedCurrencies);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine(self::$_allowedCurrencies, self::$_allowedCurrencies));
			
			$options->setTemplateFile('modules/payment/epayment_ro/epayment_options.tmpl');
			return $options;
		}
		
		/**
		 * Calculate the checksum that will be checked by epayment.ro
		 * 
		 * @param string $params
		 * @param string $key
		 * @return string
		 */
		protected function calculateChecksum($params, $key)
		{
			$data = '';
			foreach ($params as $param) {
				if (is_array($param)) {
					foreach ($param as $paramEntry) {
						$data .= strlen($paramEntry) . $paramEntry;
					}
				} else {
					$data .= strlen($param) . $param;
				}
			}
			
			return scCryptUtil::hmac($key, $data, scCryptUtil::HASH_MD5);
		}
	}
?>