<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * EuPlătesc.ro romanian payment processing module	
	 */
	class scPaymentModuleEuplatescRo implements IPaymentModule, IModuleOptions {
		
		const EPAYMENT_URL = 'https://www.euplatesc.ro/plati-online/tdsprocess/tranzactd.php';
		
		protected $_moduleId;
		
		protected $_allowedCurrencies = array('EUR', 'RON', 'USD');
		
		protected $_allowedLanguages = array(
			'ro' => 'Romanian', // _t('Romanian')
			'en' => 'English', // _t('English')
			'hu' => 'Hungarian', // _t('Hungarian')
			'it' => 'Italian', // _t('Italian')
			'es' => 'Spanish', // _t('Spanish')
		);
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode() 
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $this->_allowedCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'DefaultCurrencyCode', 'LanguageCode', 'EncryptionKey', 'OrderDescription'));
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$params = array(
				'amount' => $payAmount,
				'curr' => $payCurrencyCode,
				'invoice_id' => sprintf('%07d', $orderId),
				'order_desc' => $options['OrderDescription'],
				'merch_id' => $options['MerchantID'],	
				'timestamp' => gmdate("YmdHis"),			
				'nonce' => md5(microtime() . mt_rand()),
			);
			$params['fp_hash'] = $this->calculateHash($params, $options['EncryptionKey']);
			
			$params += array(
				'lang' => strtolower($options['LanguageCode']),
				
				// Billing address
				'fname' => $customerInfo['CustomerBillingFirstName'],	
				'lname' => $customerInfo['CustomerBillingLastName'],
				'company' => $customerInfo['CustomerBillingCompany'],
				'add' => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				'city' => $customerInfo['CustomerBillingCity'],	
				'state' => scCountry::getStateName($customerInfo['CustomerBillingState']),
				'zip' => $customerInfo['CustomerBillingZip'],
				'country' => scCountry::getCountryName($customerInfo['CustomerBillingCountry']),
				'phone' => $customerInfo['CustomerBillingPhone'],
				'fax' => 	$customerInfo['CustomerBillingFax'],
				'email' => $customerInfo['CustomerEmail'],
				
				// Shipping address
				'sfname' => $customerInfo['CustomerShippingFirstName'],	
				'slname' => $customerInfo['CustomerShippingLastName'],
				'scompany' => $customerInfo['CustomerShippingCompany'],
				'sadd' => $customerInfo['CustomerShippingAddress1'] . ' ' . $customerInfo['CustomerShippingAddress2'],
				'scity' => $customerInfo['CustomerShippingCity'],	
				'sstate' => scCountry::getStateName($customerInfo['CustomerShippingState']),
				'szip' => $customerInfo['CustomerShippingZip'],
				'scountry' => scCountry::getCountryName($customerInfo['CustomerShippingCountry']),
				'sphone' => $customerInfo['CustomerShippingPhone'],
				'sfax' => $customerInfo['CustomerShippingFax'],
				'semail' => $customerInfo['CustomerEmail'],
			);
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(self::EPAYMENT_URL, $params);
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			if (empty($params['fp_hash'])) {
				throw new miException(_i('Invalid hash.'));
			}
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('EncryptionKey'));
			
			$params = array_map('trim', $params);
			$params = array_map('addslashes', $params);
			
			// Validate the return
			$response = $params;
			unset($response['fp_hash']);
			unset($response['lang']);
			
			$responseHash = $this->calculateHash($response, $options['EncryptionKey']);
			if ($responseHash !== strtolower($params['fp_hash'])) {
				throw new miException(_i('Invalid hash. Please check you Encryption Key.'));
			}
			
			// Handle the return
			$result = new scPaymentProcessResult(intval($params['invoice_id']), $params['amount'], $params['curr'], $params['action'], $params['message']);
			if ($params['action'] == 0) {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			} else {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes($params['message']);
			}
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
		public function getOptionFields(miWebForm &$webForm) 
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'MerchantID',
					'properties' => array('minLength' => 8, 'maxLength' => 50),
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'LanguageCode',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'EncryptionKey',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'OrderDescription',
					'properties' => array('minLength' => 1, 'maxLength' => 50),
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			sort($this->_allowedCurrencies);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_allowedCurrencies, $this->_allowedCurrencies));
			
			$options->getOptionField('LanguageCode')->setOptions($this->_allowedLanguages);
			
			$options->setTemplateFile('modules/payment/euplatesc_ro/euplatesc_options.tmpl');
			return $options;
		}
		
		/**
		 * Calculate the hash that will be checked by euplatesc.ro
		 * 
		 * @param string $params
		 * @param string $key
		 * @return string
		 */
		protected function calculateHash($params, $key)
		{
			$data = '';
			foreach($params as $param) {
				if (($param === null) || ($param == '')) {
					$data .= '-'; // null values ​​are replaced with -
				} else {
					$data .= strlen($param) . $param;
				}
			}
			
			$key = pack('H*', $key);
			$blocksize = 64;
			
			if(strlen($key) > $blocksize) {
				$key = pack('H*', md5($key));
			}
			
			$key  = str_pad($key, $blocksize, chr(0x00));
			$ipad = str_repeat(chr(0x36), $blocksize);
			$opad = str_repeat(chr(0x5c), $blocksize);
			
			$hmac = pack('H*', md5(($key ^ $opad) . pack('H*', md5(($key ^ $ipad) . $data))));
			return bin2hex($hmac);
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
	}
?>