<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * First Investment Bank - Bulgaria payment processing module	
	 * It is proccesed by cPay - www.cpay.com.mk
	 */
	
	/**
	 * This payment proccessor (cPay) supports multiple currencies and languages
	 * But since it is made for FIBank we can use only BGN an BG language 
	 */
	class scPaymentModuleFIBankBg implements IPaymentModule, IModuleOptions {
		
		const FIBANK_URL = 'https://www.cpay.com.mk/client/Page/default.aspx?xml_id=/bg-BG/.loginToPay/';
		
		const PARAMS_COUNT_LENGTH = 2; // nuber of digits to represent the parameters count in header
		const PARAMS_LENGTH_LENGTH = 3; // nuber of digits to represent each parameter lenth in header
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode() 
		{
			return 'BGN';
		}
		
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PayToMerchant', 'MerchantName', 'MerchantPassword', 'OrderDetails'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$currency = new scCurrency($payCurrencyCode);
			
			$countryName = scCountry::getCountryName($customerInfo['CustomerBillingCountry'], scMultiLanguage::singleton()->getCurrentLanguageId('BG'));
			
			$params = array(

				'PayToMerchant' => $options['PayToMerchant'],	
				'MerchantName' => $options['MerchantName'],
				
				'AmountToPay' => round($payAmount * 100), //requested by payment proccessor to provide integer	
				'AmountCurrency' => $payCurrencyCode,
				
				'Details1' => $options['OrderDetails'],	
				
				// since the proccessor doesn't have a dedicated field for order id we'll use the Details2 field
				'Details2' => scOrdersInProgress::singleton()->getLastOrderId(),
				
				'PaymentOKURL' => scPaths::getAbsoluteUrl('/modules/payment/fibank_bg/return.php?status=' . scPaymentStatus::PAYMENT_STATUS_CAPTURE),	
				'PaymentFailURL' => scPaths::getAbsoluteUrl('/modules/payment/fibank_bg/return.php?status=' . scPaymentStatus::PAYMENT_STATUS_DECLINED),	
			
				'FirstName' => $customerInfo['CustomerBillingFirstName'],	
				'LastName' => $customerInfo['CustomerBillingLastName'],	
				'Address' => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],	
				'City' => $customerInfo['CustomerBillingCity'],	
				'Zip' => $customerInfo['CustomerBillingZip'],	
				'Country' => $countryName,
				'Telephone' => $customerInfo['CustomerBillingPhone'],	
				'Email' => $customerInfo['CustomerEmail'],	
			);

			// cleanup empty fields - required by proccessor
			$params = array_filter($params);
			
			// checksum header and checksum are not parts of the params array
			$header = $this->getHeader($params);
			$checkSum = $this->getCheckSum($params, $header, $options['MerchantPassword']);
			
			$params['CheckSumHeader'] = $header;	
			$params['CheckSum'] = $checkSum;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(self::FIBANK_URL, $params);
			
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('status', 'Details2', 'ReturnCheckSumHeader', 'ReturnCheckSum'));
			
			// validate the return
			$validReturn = $this->checkReturnCheckSum($params);
			if (!$validReturn) {
				throw new scPaymentProcessException(_t('Invalid return from proccessor.'));
			}
			
			$orderId = (int)$params['Details2'];
			
			$result = new scPaymentProcessResult($orderId);
			if ($params['status'] == scPaymentStatus::PAYMENT_STATUS_DECLINED || 
				(!isset($params['cPayPaymentRef']) || empty($params['cPayPaymentRef']))
			) {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			} else {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			}
			return $result;
		}
		
		
		public function getOptionFields(miWebForm &$webForm) 
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'PayToMerchant',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'MerchantName',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'MerchantPassword',
				),
				array(
					'field' => 'scWebFormWidgetTextAreaML',
					'data' => 'OrderDetails',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$options->setTemplateFile('modules/payment/fibank_bg/fibank_options.tmpl');
			return $options;
		}
		
		/**
		 * Returns header string
		 * @param array $params
		 * @return string
		 */
		protected function getHeader($params)
		{
			// first we need the number of parameters being passed to the proccessor presented in 2 digits right-justified, padded left with zeros
			$string = sprintf('%0' . self::PARAMS_COUNT_LENGTH . 's', count($params));
			
			// than a list of all parameter names
			foreach (array_keys($params) as $key) {
				$string .= $key . ',';
			}
			
			// and last length of each parameter presented in 3 digits right-justified, padded left with zeros
			foreach ($params as $value) {
				$string .= sprintf('%0' . self::PARAMS_LENGTH_LENGTH . 's', scUtf8::strlen($value));
			}
			return $string;
		}
		
		/**
		 * 
		 * @param array $params
		 * @param string $head
		 * @param string $password
		 * @return string
		 */
		protected function getCheckSum($params, $head, $password)
		{
			$inputString = $head;
			foreach ($params as $param) {
				$inputString .= $param;
			}
			$inputString .= $password;
			
			return strtoupper(md5($inputString));
		}
		
		/**
		 * 
		 * @param array $params
		 * @return boolean
		 */
		protected function checkReturnCheckSum($params)
		{
			$checksumHeader = $params['ReturnCheckSumHeader'];
			$paramsCount = (int)substr($checksumHeader, 0, self::PARAMS_COUNT_LENGTH);
			$paramsNames = explode(',', substr($checksumHeader, self::PARAMS_COUNT_LENGTH));
			
			$paramsLengths = array_pop($paramsNames); // get the string with the legths
			$paramsLengths = chunk_split($paramsLengths, self::PARAMS_LENGTH_LENGTH, ','); // add a delimiter to explode it
			$paramsLengths = explode(',', $paramsLengths); // explode it to an array 
			array_pop($paramsLengths); // remove the last element of the array since it is empty - chunk_split
			
			if ((count($paramsNames) != $paramsCount) || (count($paramsLengths) != $paramsCount)) {
				return false;
			}
			
			$inputString = $checksumHeader;
			foreach ($paramsNames as $key => $param) {
				if (!isset($params[$param])) {
					return false; // a param is missing
				}
				
				if (scUtf8::strlen($params[$param]) != $paramsLengths[$key]) {
					return false; // a param's length is wrong
				}
				$inputString .= $params[$param];
			}
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantPassword'));
			$inputString .= $options['MerchantPassword'];
			
			if (strtoupper(md5($inputString)) == $params['ReturnCheckSum']) {
				return true;
			}
			return false;
		}
	}
?>