<?php
	/**
	 * Generates HSBC hash
	 */
	class scHsbcHashCalculator {
		
		/**
		 * Generates the hash using the TestHash.e executable file
		 * 
		public function generateHash($params, $sharedSecret)
		{
			$path = dirname(__FILE__);
			putenv('LD_LIBRARY_PATH=' . $path);
			
			$hashElements = '';
			foreach ($params as $param)
				$hashElements .= ' ' . escapeshellarg($param);
			
			$cmd = $path . '/TestHash.e ' . escapeshellarg($sharedSecret) . $hashElements;
			
			$result = exec($cmd);
			if (strpos($result, 'Hash value:  ') === false)
				throw new miException(_i('TestHash.e did not return hash value.'));
			
			return substr($result, 13);
		}
		*/
		
		protected $_internalKey;
		protected $_initIV;
		
		function __construct()
		{
			if (!function_exists('mcrypt_module_open'))
				throw new scPaymentProcessException(_i('HSBC module requires mcrypt PHP module.'));
			
			$s1 = 'KmJTwzVPwjoxQdWJb1BxbuhBSa2RuM05+/aUdgYoGdFWWf04CKIQTxtxLeKCp+5J';
			$s2 = 'y8YhmjsAoMUW9RxfXBSos0A6LwGd+5pXv/MRAKCYFLG';
			$s3 = 'BqRkPAG8DFFAdeN5SMAArktCYuUGXi2q88EDoOs3Ykw0k';
			$this->_initIV = chr(98) . chr(84) . chr(120) . chr(114) . chr(66) . chr(87) . chr(80) . chr(112);
			
			$this->_internalKey = $this->initKey($s1, $s2, $s3);
		}

		public function generateHash($vector, $sharedSecret)
		{
			sort($vector, SORT_STRING);
			$str = implode('', $vector);
			
			$secret = $this->decryptToBinary($sharedSecret);
			
			$ret = base64_encode(pack('H*', scCryptUtil::hmac($secret, $str . $secret, scCryptUtil::HASH_SHA1)));
			return $ret;
		}
		
		protected function encode($str) {
			return base64_encode($str);
		}
		
		protected function decode($s) {
			return base64_decode($s);
		}

		protected function encrypt($input, $key)
		{
			$td = mcrypt_module_open(MCRYPT_DES, '', MCRYPT_MODE_CBC, '');
			$keySize = mcrypt_enc_get_key_size($td);
			$key = substr($key, 0, $keySize);
			
			$input = $this->pkcs5Pad($input, mcrypt_get_block_size(MCRYPT_DES, MCRYPT_MODE_CBC));
			
			/* Intialize encryption */
			mcrypt_generic_init($td, $key, $this->_initIV);
			return mcrypt_generic($td, $input);
		}
		
		protected function decrypt($input, $key)
		{
			$td = mcrypt_module_open(MCRYPT_DES, '', MCRYPT_MODE_CBC, '');
			$keySize = mcrypt_enc_get_key_size($td);
			$key = substr($key, 0, $keySize);
			
			/* Intialize encryption */
			mcrypt_generic_init($td, $key, $this->_initIV);
			$ret = mdecrypt_generic($td, $input);
			
			$ret = $this->pkcs5Unpad($ret);
			return $ret;
		}
		
		protected function pkcs5Pad($text, $blocksize)
		{
			$pad = $blocksize - (strlen($text) % $blocksize);
			return $text . str_repeat(chr($pad), $pad);
		}
		
		protected function pkcs5Unpad($text)
		{
			$pad = ord($text[strlen($text)-1]);
			if ($pad > strlen($text))
				return false;
			if (strspn($text, chr($pad), strlen($text) - $pad) != $pad)
				return false;
			return substr($text, 0, -$pad);
		}

		protected function encryptEncode($abyte0, $abyte1)
		{
			return $this->encode($this->encrypt($abyte0, $abyte1));
		}
		
		protected function decodeDecrypt($s, $internalKey)
		{
			return $this->decrypt($this->decode($s), $internalKey);
		}
		
		protected function initKey($s, $s1, $s2)
		{
			$abyte0 = chr(0);
			$abyte1 = $s1;
			$abyte2 = $s2;
			$byte0 = 4;
			$i = $byte0 + 9;
			//    $j = rand(0, 29);	// Well, not so random :-)
			$j = 0;
			if ($j > $byte0 * $i)
				$j -= $byte0 * $i;
			
			$k = 0;
			for ($l = 0; $l < $byte0 * $i; $l++) {
				switch(($j + $l) % $i) {
					case 0:
						if ($k == 2) {
							$abyte0 = $this->encrypt($abyte1, $abyte2);
							$k++;
						}
						break;
					
					case 1:
						if ($k == 1) {
							$abyte2 = str_rot13($abyte1);
							$k++;
						}
						break;
					
					case 2:
						if ($k == 0) {
							$abyte1[0] = chr(48 + (ord($abyte1[0]) + 10) % 10);
							$k++;
						}
						break;
					
					case 3:
						if ($k == 3)
							$k++;
						break;
					
					case 5:
					case 7:
					case 10:
						if ($k < 2)
							$abyte0 = $this->encrypt($abyte1, $abyte2);
						break;
				}
			}
			return $this->decodeDecrypt($s, $abyte0);
		}
		
		public function decryptToBinary($s)
		{
			return $this->decodeDecrypt($s, $this->_internalKey);
		}
	}
?>