<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Moneris Hosted Page payment processing module
	 */
	class scPaymentModuleMonerisHostedPage implements IPaymentModule, IModuleOptions {
		
		const MONERIS_USA_DEMO_URL = 'https://esplusqa.moneris.com/DPHPP/index.php';
		const MONERIS_USA_URL = 'https://esplus.moneris.com/DPHPP/index.php';
		
		const MONERIS_CAN_DEMO_URL = 'https://esqa.moneris.com/HPPDP/index.php';
		const MONERIS_CAN_URL = 'https://www3.moneris.com/HPPDP/index.php';
		
		const RETURN_CODE_APPROVED = 50; // check if the response code is less than this
		const DEMO_MODE_PREFIX = 'sc_test_oid_'; // make unique orderID in moneris test enviroment
		
		protected $_moduleId;
		protected $_options = array();
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			// ensure the module is set up correctly
			$isOK = $this->checkSetup();
			if ($isOK !== true) {
				throw new scPaymentProcessException(_t('Moneris payment module is not set up correctly.'));
			}
			
			// get module options
			$options = $this->getOptions();
			
			// if the current currency is set up return it.
			$currentCurrencyCode = scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			if ($this->checkCurrencySetup($currentCurrencyCode)) {
				return $currentCurrencyCode;
			}
			
			// just return the default
			return $options['HostedPaymentPageDefaultCurrency'];
		}
		
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			$options = $this->getOptions();
			
			$params = array(
				$this->getParamName(self::PARAM_HPP_ID) => $options[$payCurrencyCode . 'HostedPaymentPageID'],
				$this->getParamName(self::PARAM_HPP_KEY) => $options[$payCurrencyCode . 'HostedPaymentPageKey'],
				$this->getParamName(self::PARAM_NOTE) => $options['PaymentNote'],
				$this->getParamName(self::PARAM_AMOUNT) => $payAmount,
				$this->getParamName(self::PARAM_ORDER_NO) => ($options['DemoMode'] ? self::DEMO_MODE_PREFIX : '') . scOrdersInProgress::singleton()->getLastOrderId(),
				$this->getParamName(self::PARAM_CLIENT_EMAIL) => $customerInfo['CustomerEmail'],
				'rvar_currencyCode' => $payCurrencyCode, // this is the only way to see which hhp_id we should use for verification
			);
			
			// set address params
			$this->setAddresses($params, $customerInfo);
			
			// set items params
			$this->setItems($params, $cart, $payCurrencyCode);

			// is it demo or live
			$url = $this->getUrl();
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			// we should always have order number in the response
			scPaymentProcess::checkParamsExist($params, array($this->getParamName(self::PARAM_ORDER_NO)));
			
			// if in demo mode remove the DEMO_MODE_PREFIX
			$options = $this->getOptions();
			if ($options['DemoMode']) {
				$orderId = (int)str_replace(self::DEMO_MODE_PREFIX, '', $params[$this->getParamName(self::PARAM_ORDER_NO)]);
			} else {
				$orderId = (int)$params[$this->getParamName(self::PARAM_ORDER_NO)];
			}

			// init the scPaymentProcessResult object
			$result = new scPaymentProcessResult($orderId);
			
			// first check if the transaction was canceled by the user at Moneris
			if (isset($params['cancel'])) {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes(_t('Payment cancelled.'));
				
			// process the transaction
			} else {
				scPaymentProcess::checkParamsExist($params, array($this->getParamName(self::PARAM_RESPONSE_CODE), $this->getParamName(self::PARAM_ORDER_NO), $this->getParamName(self::PARAM_VERIFY_KEY), 'rvar_currencyCode'));
				
				// get data again directly from Moneris
				// array_merge assures that data got from transaction verification from moneris overwrites the posted
				$params = array_merge($params, $this->validateTransaction($params));
				
				// Check that the amount matches
				scPaymentProcess::checkPaymentToOrder($orderId, $params[$this->getParamName(self::PARAM_AMOUNT)], $params['rvar_currencyCode']);
				
				if ($params[$this->getParamName(self::PARAM_RESPONSE_CODE)] < self::RETURN_CODE_APPROVED) {
					$result->setTransactionCode($params[$this->getParamName(self::PARAM_REF_NUM)]);
					$result->setTransactionAmount($params[$this->getParamName(self::PARAM_AMOUNT)]);
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					$result->setPaymentNotes($params['message']);
				} else {
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					$result->setPaymentNotes($params['message']);
				}
			}
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'USDHostedPaymentPageID',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'USDHostedPaymentPageKey',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'CADHostedPaymentPageID',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'CADHostedPaymentPageKey',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'HostedPaymentPageDefaultCurrency',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'DemoMode',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'HostedPaymentPageVersion',
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'PaymentNote',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('DemoMode')->setRadioButtons(array('Y' => _t('Yes'), 'N' => _t('No')), 'N');
			$options->getOptionField('HostedPaymentPageDefaultCurrency')->setRadioButtons(array('USD' => 'USD', 'CAD' => 'CAD'), 'CAD');
			$options->getOptionField('HostedPaymentPageVersion')->setRadioButtons(array(self::VERSION_USA => 'USA', self::VERSION_CANADA => 'Canada'), self::VERSION_USA);
			$options->setTemplateFile('modules/payment/moneris/options.tmpl');
			return $options;
		}
		
		protected function validateTransaction($params)
		{
			$options = $this->getOptions();
			$postParams = array(
				$this->getParamName(self::PARAM_HPP_ID) => $options[$params['rvar_currencyCode'] . 'HostedPaymentPageID'],
				$this->getParamName(self::PARAM_HPP_KEY) => $options[$params['rvar_currencyCode'] . 'HostedPaymentPageKey'],
				$this->getParamName(self::PARAM_VERIFY_KEY) => $params[$this->getParamName(self::PARAM_VERIFY_KEY)],
			);
			
			// check if we are in demo mode
			$url = $this->getUrl();
			
			// make the request
			$res = scHttp::post($url, $postParams);
			$xmlObj = new SimpleXMLElement($res);
			
			$originalData = array(
				$this->getParamName(self::PARAM_ORDER_NO) => $params[$this->getParamName(self::PARAM_ORDER_NO)],
				$this->getParamName(self::PARAM_RESPONSE_CODE) => $params[$this->getParamName(self::PARAM_RESPONSE_CODE)],
				$this->getParamName(self::PARAM_AMOUNT) => $params[$this->getParamName(self::PARAM_AMOUNT)],
			);
			$result = array(
				$this->getParamName(self::PARAM_ORDER_NO) => (string)$xmlObj->order_no,
				$this->getParamName(self::PARAM_RESPONSE_CODE) => (string)$xmlObj->response_code,
				$this->getParamName(self::PARAM_AMOUNT) => (string)$xmlObj->amount,
			);
			
			// this evaluates to true if the data in the both arrays is the same
			if ($originalData != $result) {
				throw new scPaymentProcessException(_t('Moneris transaction verification failed.'));
			}
			return $result;
		}
		
		protected function setAddresses(&$params, $customerInfo)
		{
			$currentLanguageId = scMultiLanguage::singleton()->getCurrentLanguageId();
			$params = $params + array(
				$this->getParamName(self::PARAM_SHIP_FIRSTNAME) => $customerInfo['CustomerShippingFirstName'],
				$this->getParamName(self::PARAM_SHIP_LASTNAME) => $customerInfo['CustomerShippingLastName'],
				$this->getParamName(self::PARAM_SHIP_COMPANY) => $customerInfo['CustomerShippingCompany'],
				$this->getParamName(self::PARAM_SHIP_ADDRESS) => $customerInfo['CustomerShippingAddress1'] . ' ' . $customerInfo['CustomerShippingAddress2'],
				$this->getParamName(self::PARAM_SHIP_CITY) => $customerInfo['CustomerShippingCity'],
				$this->getParamName(self::PARAM_SHIP_STATE) => scCountry::getStateName($customerInfo['CustomerShippingState'], $currentLanguageId),
				$this->getParamName(self::PARAM_SHIP_ZIPCODE) => $customerInfo['CustomerShippingZip'],
				$this->getParamName(self::PARAM_SHIP_COUNTRY) => scCountry::getCountryName($customerInfo['CustomerShippingCountry'], $currentLanguageId),
				$this->getParamName(self::PARAM_SHIP_PHONE) => $customerInfo['CustomerShippingPhone'],
				$this->getParamName(self::PARAM_SHIP_FAX) => $customerInfo['CustomerShippingFax'],
				
				$this->getParamName(self::PARAM_BILL_FIRSTNAME) => $customerInfo['CustomerBillingFirstName'],
				$this->getParamName(self::PARAM_BILL_LASTNAME) => $customerInfo['CustomerBillingLastName'],
				$this->getParamName(self::PARAM_BILL_COMPANY) => $customerInfo['CustomerBillingCompany'],
				$this->getParamName(self::PARAM_BILL_ADDRESS) => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				$this->getParamName(self::PARAM_BILL_CITY) => $customerInfo['CustomerBillingCity'],
				$this->getParamName(self::PARAM_BILL_STATE) => scCountry::getStateName($customerInfo['CustomerBillingState'], $currentLanguageId),
				$this->getParamName(self::PARAM_BILL_ZIPCODE) => $customerInfo['CustomerBillingZip'],
				$this->getParamName(self::PARAM_BILL_COUNTRY) => scCountry::getCountryName($customerInfo['CustomerBillingCountry'], $currentLanguageId),
				$this->getParamName(self::PARAM_BILL_PHONE) => $customerInfo['CustomerBillingPhone'],
				$this->getParamName(self::PARAM_BILL_FAX) => $customerInfo['CustomerBillingFax'],
			);
		}
		
		protected function setItems(&$params, scCart $cart, $payCurrencyCode)
		{
			$items = $cart->getItems();
			$n = 0;
			$currency = new scCurrency($payCurrencyCode);
			foreach ($items as $item) {
				$n++;
				$params[$this->getParamName(self::PARAM_ITEM_ID) . $n] = $item->getCartItemCode();
				$params[$this->getParamName(self::PARAM_ITEM_DESCRIPTION) . $n] = $item->getName();
				$params[$this->getParamName(self::PARAM_ITEM_QUANTITY) . $n] = $item->getQuantity();
				$params[$this->getParamName(self::PARAM_ITEM_PRICE) . $n] = $item->getPriceObj()->getAmountInCurrency($currency);
			}
		}
		
		/**
		 * Cache the module options
		 *
		 * @return array a module option name/value pairs
		 */
		protected function getOptions()
		{
			if (empty($this->_options)) {
				$this->_options = scModuleOptions::getModuleOptions($this->_moduleId, array(
					'HostedPaymentPageDefaultCurrency', 'DemoMode', 'PaymentNote',
					'CADHostedPaymentPageID', 'CADHostedPaymentPageKey',
					'USDHostedPaymentPageID', 'USDHostedPaymentPageKey',
					'HostedPaymentPageVersion',
				), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			}
			return $this->_options;
		}
		
		/**
		 * Checks if the payment module is setup correctly.
		 *
		 * @return boolean
		 */
		protected function checkSetup()
		{
			$options = $this->getOptions();
			// if we are missing atleast one of the config options for the two currencies - error
			if ((empty($options['CADHostedPaymentPageID']) || empty($options['CADHostedPaymentPageKey'])) &&
				(empty($options['USDHostedPaymentPageID']) || empty($options['USDHostedPaymentPageKey']))) {
				return false;
			}
			
			return $this->checkCurrencySetup($options['HostedPaymentPageDefaultCurrency']);
		}
		
		/**
		 * Checks if the payment module is setup correctly for currency
		 *
		 * @param string $currencyCode
		 * @return boolean
		 */
		protected function checkCurrencySetup($currencyCode)
		{
			$options = $this->getOptions();
			if (empty($options[$currencyCode . 'HostedPaymentPageID']) || empty($options[$currencyCode . 'HostedPaymentPageKey'])) {
				return false;
			}
			return true;
		}
		
		/**
		 * Returns the URL for te service depending ot version (USA|CAD) and demo mode
		 */
		protected function getUrl()
		{
			$options = $this->getOptions();
			$version = $options['HostedPaymentPageVersion'];
			$demoMode = $options['DemoMode'];
			if ($version == self::VERSION_USA) {
				return $demoMode == 'Y' ? self::MONERIS_USA_DEMO_URL : self::MONERIS_USA_URL;
			} else {
				return $demoMode == 'Y' ? self::MONERIS_CAN_DEMO_URL : self::MONERIS_CAN_URL;
			}
		}
		
		/**
		 * Returns the appropreate parama name depending on the
		 * current varsion setting. There is a difference between
		 * USA and Canadian versions of Moneris.
		 *
		 * @param string $param
		 */
		protected function getParamName($param)
		{
			$options = $this->getOptions();
			$version = $options['HostedPaymentPageVersion'];
			return $this->_paramNames[$version][$param];
		}
		
		// versions
		const VERSION_USA = 1;
		const VERSION_CANADA = 2;
		
		// core
		const PARAM_HPP_ID = 1;
		const PARAM_HPP_KEY = 2;
		const PARAM_NOTE = 3;
		const PARAM_AMOUNT = 4;
		const PARAM_ORDER_NO = 5;
		const PARAM_RESPONSE_ORDER_NO = 6;
		const PARAM_CLIENT_EMAIL = 7;
		const PARAM_VERIFY_KEY = 8;
		const PARAM_RESPONSE_CODE = 9;
		const PARAM_REF_NUM = 10;
		
		// shipping address
		const PARAM_SHIP_FIRSTNAME = 30;
		const PARAM_SHIP_LASTNAME = 31;
		const PARAM_SHIP_COMPANY = 32;
		const PARAM_SHIP_ADDRESS = 33;
		const PARAM_SHIP_CITY = 34;
		const PARAM_SHIP_STATE = 35;
		const PARAM_SHIP_ZIPCODE = 36;
		const PARAM_SHIP_COUNTRY = 37;
		const PARAM_SHIP_PHONE = 38;
		const PARAM_SHIP_FAX = 39;
		
		// billing address
		const PARAM_BILL_FIRSTNAME = 50;
		const PARAM_BILL_LASTNAME = 51;
		const PARAM_BILL_COMPANY = 52;
		const PARAM_BILL_ADDRESS = 53;
		const PARAM_BILL_CITY = 54;
		const PARAM_BILL_STATE= 55;
		const PARAM_BILL_ZIPCODE = 56;
		const PARAM_BILL_COUNTRY= 57;
		const PARAM_BILL_PHONE = 58;
		const PARAM_BILL_FAX = 59;
		
		// cart items
		const PARAM_ITEM_ID = 70;
		const PARAM_ITEM_DESCRIPTION = 71;
		const PARAM_ITEM_QUANTITY = 72;
		const PARAM_ITEM_PRICE = 73;
		
		protected $_paramNames = array(
			self::VERSION_USA => array(
				// core
				self::PARAM_HPP_ID => 'hpp_id',
				self::PARAM_HPP_KEY => 'hpp_key',
				self::PARAM_NOTE => 'note',
				self::PARAM_AMOUNT => 'amount',
				self::PARAM_ORDER_NO => 'order_no',
				self::PARAM_RESPONSE_ORDER_NO => 'order_no',
				self::PARAM_CLIENT_EMAIL => 'client_email',
				self::PARAM_VERIFY_KEY => 'verify_key',
				self::PARAM_RESPONSE_CODE => 'response_code',
				self::PARAM_REF_NUM => 'ref_num',
				
				// shipping address
				self::PARAM_SHIP_FIRSTNAME => 'od_ship_firstname',
				self::PARAM_SHIP_LASTNAME => 'od_ship_lastname',
				self::PARAM_SHIP_COMPANY => 'od_ship_company',
				self::PARAM_SHIP_ADDRESS => 'od_ship_address',
				self::PARAM_SHIP_CITY => 'od_ship_city',
				self::PARAM_SHIP_STATE => 'od_ship_state',
				self::PARAM_SHIP_ZIPCODE => 'od_ship_zipcode',
				self::PARAM_SHIP_COUNTRY => 'od_ship_country',
				self::PARAM_SHIP_PHONE => 'od_ship_phone',
				self::PARAM_SHIP_FAX => 'od_ship_fax',
				
				// billing address
				self::PARAM_BILL_FIRSTNAME => 'od_bill_firstname',
				self::PARAM_BILL_LASTNAME => 'od_bill_lastname',
				self::PARAM_BILL_COMPANY => 'od_bill_company',
				self::PARAM_BILL_ADDRESS => 'od_bill_address',
				self::PARAM_BILL_CITY => 'od_bill_city',
				self::PARAM_BILL_STATE => 'od_bill_state',
				self::PARAM_BILL_ZIPCODE => 'od_bill_zipcode',
				self::PARAM_BILL_COUNTRY => 'od_bill_country',
				self::PARAM_BILL_PHONE => 'od_bill_phone',
				self::PARAM_BILL_FAX => 'od_bill_fax',

				// cart items
				self::PARAM_ITEM_ID => 'li_id',
				self::PARAM_ITEM_DESCRIPTION => 'li_description',
				self::PARAM_ITEM_QUANTITY => 'li_quantity',
				self::PARAM_ITEM_PRICE => 'li_price',
			),
			
			
			self::VERSION_CANADA => array(
				// core
				self::PARAM_HPP_ID => 'ps_store_id',
				self::PARAM_HPP_KEY => 'hpp_key',
				self::PARAM_NOTE => 'note',
				self::PARAM_AMOUNT => 'charge_total',
				self::PARAM_ORDER_NO => 'order_id',
				self::PARAM_RESPONSE_ORDER_NO => 'response_order_id',
				self::PARAM_CLIENT_EMAIL => 'client_email',
				self::PARAM_VERIFY_KEY => 'transactionKey',
				self::PARAM_RESPONSE_CODE => 'response_code',
				self::PARAM_REF_NUM => 'bank_transaction_id',
				
				// shipping address
				self::PARAM_SHIP_FIRSTNAME => 'ship_firstname',
				self::PARAM_SHIP_LASTNAME => 'ship_lastname',
				self::PARAM_SHIP_COMPANY => 'ship_company',
				self::PARAM_SHIP_ADDRESS => 'ship_address',
				self::PARAM_SHIP_CITY => 'ship_city',
				self::PARAM_SHIP_STATE => 'ship_state',
				self::PARAM_SHIP_ZIPCODE => 'ship_zipcode',
				self::PARAM_SHIP_COUNTRY => 'ship_country',
				self::PARAM_SHIP_PHONE => 'ship_phone',
				self::PARAM_SHIP_FAX => 'ship_fax',
				
				// billing address
				self::PARAM_BILL_FIRSTNAME => 'bill_firstname',
				self::PARAM_BILL_LASTNAME => 'bill_lastname',
				self::PARAM_BILL_COMPANY => 'bill_company',
				self::PARAM_BILL_ADDRESS => 'bill_address',
				self::PARAM_BILL_CITY => 'bill_city',
				self::PARAM_BILL_STATE => 'bill_state',
				self::PARAM_BILL_ZIPCODE => 'bill_zipcode',
				self::PARAM_BILL_COUNTRY => 'bill_country',
				self::PARAM_BILL_PHONE => 'bill_phone',
				self::PARAM_BILL_FAX => 'bill_fax',

				// cart items
				self::PARAM_ITEM_ID => 'id',
				self::PARAM_ITEM_DESCRIPTION => 'description',
				self::PARAM_ITEM_QUANTITY => 'quantity',
				self::PARAM_ITEM_PRICE => 'price',
			),
		);
	}
?>