<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Moneybookers payment processing module
	 */
	class scPaymentModuleMoneybookers implements IPaymentModule, IModuleOptions {
		protected $_moduleId;
		
		const MONEYBOOKERS_URL = 'https://www.moneybookers.com/app/payment.pl';
		
		/**
		 * ISO 639-2 to Moneybookers language code map
		 * @var array
		 */
		protected $_langCodeMap = array(
			// ISO 639-2 => Moneybookers
			'EN' => 'EN',
			'DE' => 'DE',
			'ES' => 'ES',
			'FR' => 'FR',
			'IT' => 'IT',
			'PL' => 'PL',
			'EL' => 'GR',
			'RO' => 'RO',
			'RU' => 'RU',
			'TR' => 'TR',
			'CN' => 'CN',
			'CZ' => 'CZ',
			'NL' => 'NL',
			'DA' => 'DA',
			'SV' => 'SV',
			'FI' => 'FI',
		);
		
		protected $_supportedCurrencies = array('EUR', 'USD', 'GBP', 'HKD', 'SGD', 'JPY', 'CAD', 'AUD', 'CHF', 'DKK', 'SEK', 'NOK', 'ILS', 'MYR', 'NZD', 'TRY',
			'TWD', 'THB', 'CZK', 'HUF', 'SKK', 'EEK', 'BGN', 'PLN', 'ISK', 'INR', 'LVL', 'KRW', 'ZAR', 'RON', 'HRK', 'LTL');
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Return the payment currency
		 */
		public function getPaymentCurrencyCode()
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $this->_supportedCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PayToEmail', 'RecipientDescription', 'LogoURL', 'HideLogin', 'UseIframe', 'ConfirmationNote', 'ReturnURLText'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = isset($this->_langCodeMap[$isoLangCode]) ? $this->_langCodeMap[$isoLangCode] : 'EN';
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$params = array(
				'pay_to_email' => $options['PayToEmail'],
				'recipient_description' => $options['RecipientDescription'],
				'transaction_id' => $orderId,
				'return_url' => scPaths::getSecureUrl('modules/payment/moneybookers/return.php?OrderID=' . $orderId),
				'return_url_text' => $options['ReturnURLText'],
				'cancel_url' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_ABSOLUTE),
				'status_url' => scPaths::getSecureUrl('modules/payment/moneybookers/status.php'),
				'logo_url' => $options['LogoURL'],
				'confirmation_note' => $options['ConfirmationNote'],
				'hide_login' => $options['HideLogin'],
				'language' => $lang,
				
				'amount' => $payAmount,
				'currency' => $payCurrencyCode,
				
				'firstname' => $customerInfo['CustomerBillingFirstName'],
				'lastname' => $customerInfo['CustomerBillingLastName'],
				'address' => $customerInfo['CustomerBillingAddress1'],
				'address2' => $customerInfo['CustomerBillingAddress2'],
				'city' => $customerInfo['CustomerBillingCity'],
				'state' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'postal_code' => $customerInfo['CustomerBillingZip'],
				'country' => scCountry::getCountryCode3($customerInfo['CustomerBillingCountry']),
				'pay_from_email' => $customerInfo['CustomerEmail'],
				'phone_number' => $customerInfo['CustomerBillingPhone'],
				
				'prepare_only' => '1',
			);
			
			$redirectMethod = $options['UseIframe'] ? 'iframe' : 'post';
			$redirectOptions = $options['UseIframe'] ? array('iframe_height' => 500) : array();
			
			$result = new scPaymentProcessResult();
			$sid = scHttp::post(self::MONEYBOOKERS_URL, http_build_query($params));
			
			// Since there is no normal error reporting check if the SID contains space.
			//If it does there is an error, and use standard redirect so that the user can see the error message
			if (strpos($sid, ' ') !== false) {
				unset($params['prepare_only']);
				$result->setThirdPartyRedirect(self::MONEYBOOKERS_URL, $params, $redirectMethod, $redirectOptions);
				return $result;
			}
			
			$result->setThirdPartyRedirect(self::MONEYBOOKERS_URL, array('sid' => $sid), $redirectMethod, $redirectOptions);
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('OrderID'));
			
			/* DON'T NEED THIS PIECE OF CODE
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('SecretWord'));
			
			$merchantId = $this->getMerchantId($params['transaction_id']);
			
			$secret = $options['SecretWord'];
			$computedMd5 = md5(strtoupper($merchantId . $params['transaction_id'] . strtoupper(md5($secret))));
			if (strtoupper($computedMd5) != strtoupper($params['msid']))
				throw new miException(_i('Incorrect signature received from Moneybookers.'));
			*/

			$result = new scPaymentProcessResult($params['OrderID']);
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
			return $result;
		}
		
		/**
		 * Handles status notify return
		 *
		 * @param array $params
		 */
		function processStatus($params)
		{
			$statusMap = array(
				2 => scPaymentStatus::PAYMENT_STATUS_CAPTURE,
				0 => scPaymentStatus::PAYMENT_STATUS_PENDING,
				-1 => scPaymentStatus::PAYMENT_STATUS_DECLINED,
				-2 => scPaymentStatus::PAYMENT_STATUS_DECLINED,
				-3 => scPaymentStatus::PAYMENT_STATUS_CHARGEBACK,
			);
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PayToEmail', 'SecretWord'));
			
			scPaymentProcess::checkParamsExist($params, array('status', 'pay_to_email', 'pay_from_email', 'md5sig', 'mb_amount', 'mb_currency', 'amount', 'currency', 'transaction_id', 'mb_transaction_id', 'merchant_id'));
			
			// Check the MD5 signature
			$secret = $options['SecretWord'];
			$computedMd5 = strtoupper(md5($params['merchant_id'] . $params['transaction_id'] . strtoupper(md5($secret)) . $params['mb_amount'] . $params['mb_currency'] . $params['status']));
			
			if ($computedMd5 != $params['md5sig'])
				throw new miException(_i('Incorrect signature received from Moneybookers.'));
			
			$orderId = $params['transaction_id'];
			
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($orderId, $params['amount'], $params['currency']);
			
			// Check that it has been paid to the correct email
			if ($options['PayToEmail'] != $params['pay_to_email'])
				throw new miException(sprintf(_i('Not matching "Pay To Email" (%s) received from Moneybookers.'), $params['pay_to_email']));
			
			$result = new scPaymentProcessResult($orderId, $params['mb_amount'], $params['mb_currency'], $params['mb_transaction_id']);
			if (isset($statusMap[$params['status']])) {
				$result->setPaymentStatus($statusMap[$params['status']]);
				if ($params['status'] == -1)
					$result->setPaymentNotes(_t('Payment cancelled.'));
				if ($params['status'] == -2)
					$result->setPaymentNotes(_t('Payment failed.'));
			} else
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);	// Unknown status
			
			$result->setTransactionDetail('MerchantID', $params['merchant_id']);
			$result->setTransactionDetail('Amount', $params['amount']);
			$result->setTransactionDetail('Currency', $params['currency']);
			$result->setTransactionDetail('Status', $params['status']);
			$result->setTransactionDetail('PayFromEmail', $params['pay_from_email']);
			
			return $result;
		}
		
		/**
		 * Returns the MerchantID obtained during transaction registration
		 * 
		 * @param int $orderId
		 * @return string
		 */
		protected function getMerchantId($orderId)
		{
			$sql = 'SELECT TransactionDetailValue FROM Transactions JOIN TransactionDetails USING(TransactionID) WHERE OrderID=? AND TransactionDetailName=?';
			$rows = miStaticDBUtil::execSelect($sql, array($orderId, 'MerchantID'));
			if (count($rows) == 0)
				throw new scPaymentProcessException(sprintf(_i('Moneybookers MerchantID for order %s not found.'), $orderId));
			
			return $rows[0]['TransactionDetailValue'];
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'PayToEmail',
					'properties' => array('minLength' => 1)
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'RecipientDescription',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'LogoURL',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'HideLogin',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'UseIframe',
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'ConfirmationNote',
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'ReturnURLText',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'SecretWord',
					'properties' => array('minLength' => 1)
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			sort($this->_supportedCurrencies);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_supportedCurrencies, $this->_supportedCurrencies));
			$options->setTemplateFile('modules/payment/moneybookers/options.tmpl');
			return $options;
		}
	}
?>