<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Ogone payment processing module
	 */
	class scPaymentModuleOgone implements IPaymentModule, IModuleOptions {
		const OGONE_TEST_URL = 'https://secure.ogone.com/ncol/test/orderstandard.asp';
		const OGONE_PROD_URL = 'https://secure.ogone.com/ncol/prod/orderstandard.asp';
		
		protected $_moduleId;
		
		/**
		 * ISO 639-2 to Ogone language code map
		 * @var array
		 */
		protected $_langCodeMap = array(
			'en' => 'en_US',
			'dk' => 'dk_DK',
			'nl' => 'nl_NL',
			//'nl' => 'nl_BE',
			'fr' => 'fr_FR',
			'de' => 'de_DE',
			'it' => 'it_IT',
			'ja' => 'ja_JP',
			'no' => 'no_NO',
			'pt' => 'pt_PT',
			'es' => 'es_ES',
			'se' => 'se_SE',
			'tr' => 'tr_TR',
		);
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		const DEFAULT_CURRENCY_CODE = 'EUR';
		protected $_supportedCurrencies = array('AED', 'ANG', 'ARS', 'AUD', 'AWG', 'BGN', 'BRL', 'BYR',
			'CAD', 'CHF', 'CNY', 'CZK', 'DKK', 'EEK', 'EGP', 'EUR', 'GBP', 'GEL', 'HKD', 'HRK', 'HUF',
			'ILS', 'ISK', 'JPY', 'KRW', 'LTL', 'LVL', 'MAD', 'MXN', 'NOK', 'NZD', 'PLN', 'RON', 'RUB',
			'SEK', 'SGD', 'SKK', 'THB', 'TRY', 'UAH', 'USD', 'XAF', 'XOF', 'XPF', 'ZAR');
		
		/**
		 * Return the payment currency
		 */
		public function getPaymentCurrencyCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('SupportedCurrencyCodes', 'DefaultCurrencyCode'));
			$supportedCurrencyCodes = explode(',', $options['SupportedCurrencyCodes']);
			
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $supportedCurrencyCodes))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			return $options['DefaultCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PSPID', 'SHA1IN', 'TransactionType', 'PageTitle', 'DemoMode'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = isset($this->_langCodeMap[$isoLangCode]) ? $this->_langCodeMap[$isoLangCode] : 'en_US';
			
			$params = array(
				'PSPID' => $options['PSPID'],
				'orderID' => scOrdersInProgress::singleton()->getLastOrderId(),
				'amount' => round($payAmount * 100), // Ogone takes the value in cents
				'currency' => $payCurrencyCode,
				'language' => $lang,
				'Operation' => $options['TransactionType'],
				
				'CN' => $customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName'],
				'EMAIL' => $customerInfo['CustomerEmail'],
				'owneraddress' => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				'ownercty' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'ownertelno' => $customerInfo['CustomerBillingPhone'],
				'ownertown' => $customerInfo['CustomerBillingCity'],
				'ownerzip' => $customerInfo['CustomerBillingZip'],
				
				'title' => utf8_decode($options['PageTitle']), // Ogone works in 8859-1 character set
				
				'accepturl' => scPaths::getSecureUrl('modules/payment/ogone/return.php'),
				'declineurl' => scPaths::getSecureUrl('modules/payment/ogone/return.php'),
				'exceptionurl' => scPaths::getSecureUrl('modules/payment/ogone/return.php'),
				'cancelurl' => scPaths::getUrl('modules/payment/ogone/return.php'),
				
				//'homeurl' => scPaths::getUrl(),
				'catalogurl' => scPaths::getUrl(),
			);
			
			$params['SHASign'] = sha1($params['orderID'] . $params['amount'] . $params['currency'] . $params['PSPID'] . $params['Operation'] . $options['SHA1IN']);
			
			$result = new scPaymentProcessResult();
			$url = $options['DemoMode'] ? self::OGONE_TEST_URL : self::OGONE_PROD_URL;
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		protected $_statusMessages = array(
			0 => 'Incomplete or invalid ',
			1 => 'Cancelled by client ',
			2 => 'Authorization refused ',
			4 => 'Order stored',
			41 => 'Waiting client payment',
			5 => 'Authorized',
			51 => 'Authorization waiting',
			52 => 'Authorization not known',
			59 => 'Author. to get manually',
			6 => 'Authorized and canceled',
			61 => 'Author. deletion waiting',
			62 => 'Author. deletion uncertain',
			63 => 'Author. deletion refused',
			7 => 'Payment deleted',
			71 => 'Payment deletion pending',
			72 => 'Payment deletion uncertain',
			73 => 'Payment deletion refused',
			74 => 'Payment deleted (not accepted)',
			75 => 'Deletion processed by merchant',
			8 => 'Refund',
			81 => 'Refund pending',
			82 => 'Refund uncertain',
			83 => 'Refund refused',
			84 => 'Payment declined by the acquirer (will be debited)',
			85 => 'Refund processed by merchant',
			9 => 'Payment requested',
			91 => 'Payment processing',
			92 => 'Payment uncertain',
			93 => 'Payment refused',
			94 => 'Refund declined by the acquirer',
			95 => 'Payment processed by merchant ',
			97 => 'Being processed (intermediate technical status)',
			98 => 'Being processed (intermediate technical status)',
			99 => 'Being processed (intermediate technical status)',
		);
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{	
			scPaymentProcess::checkParamsExist($params, array('orderID', 'currency', 'amount', 'PM', 'ACCEPTANCE', 'STATUS', 'CARDNO', 'PAYID', 'NCERROR', 'BRAND', 'SHASIGN'));
			
			$orderId = $params['orderID'];
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('SHA1OUT', 'DemoMode'));
			$shaLocal = sha1(
				$params['orderID'] . 
				$params['currency'] .
				$params['amount'] .
				$params['PM'] .
				$params['ACCEPTANCE'] .
				$params['STATUS'] .
				$params['CARDNO'] .
				$params['PAYID'] .
				$params['NCERROR'] .
				$params['BRAND'] .
				$options['SHA1OUT']
			);
			
			if (strcasecmp($params['SHASIGN'], $shaLocal) != 0)
				throw new scPaymentProcessException(_t('Invalid security verification. Check your SHA-1-OUT signature.'), array(_t('OrderID') => $orderId));
			
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($params['orderID'], $params['amount'], $params['currency']);
			
			$result = new scPaymentProcessResult($orderId, $params['amount'], $params['currency'], $params['PAYID']);
			if ($options['DemoMode'])
				$result->setPaymentNotes(_t('Order in demo mode.'));
			
			if (!isset($this->_statusMessages[$params['STATUS']]))
				throw new scPaymentProcessException(_t('Unknown processing code.'), array(_t('Processing Code') => $params['STATUS']));
			$result->setTransactionNote(($this->_statusMessages[$params['STATUS']]));
			
			switch ($params['STATUS']) {
				case 9:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					break;
				case 5:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_AUTH);
					break;
				case 51:
				case 91:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
					break;
				case 52:
				case 92:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);	// Auth or Capture uncertain
					break;
				default:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			}
			
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'PSPID',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'SHA1IN',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'SHA1OUT',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'DemoMode',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'TransactionType',
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'PageTitle',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'scModuleOptionMultiCheckboxWidget',
					'data' => 'SupportedCurrencyCodes',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('DemoMode')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 0);
			$options->getOptionField('TransactionType')->setRadioButtons(array('RES' => _i('Authorization'), 'SAL' => _t('Sale')), 'SAL');
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_supportedCurrencies, $this->_supportedCurrencies));
			$options->getOptionField('SupportedCurrencyCodes')->setRadioButtons(array_combine($this->_supportedCurrencies, $this->_supportedCurrencies), self::DEFAULT_CURRENCY_CODE);
			$options->setTemplateFile('modules/payment/ogone/options.tmpl');
			return $options;
		}
	}
?>