<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */

	require_once(dirname(__FILE__) . '/nvpapi.php');

	class scPaymentModulePayPalWPSOperations implements IPaymentModuleOperations {
		const TEMPLATE_CAPTURE = 'modules/payment/paypal/capture.tmpl';
		const TEMPLATE_REFUND = 'modules/payment/paypal/refund.tmpl';
		const TEMPLATE_VOID = 'modules/payment/paypal/void.tmpl';
		const TEMPLATE_REAUTHORIZATION = 'modules/payment/paypal/reauthorization.tmpl';
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentOperationDefinitions(scOrder $order)
		{
			$operations = array();
			
			$authId = $this->getAuthId($order);
			if (($authId != '') and (!$this->isAuthCompleted($order))) {
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_CAPTURE, 'Capture', array($this, 'capture'), _t('Capture'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_CAPTURE, 'CaptureConfirm', array($this, 'captureConfirm'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_VOID, 'Void', array($this, 'void'), _t('Void'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_VOID, 'VoidConfirm', array($this, 'voidConfirm'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_REAUTHORIZATION, 'Reauthorization', array($this, 'reauthorization'), _t('Reauthorization'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_REAUTHORIZATION, 'ReauthorizationConfirm', array($this, 'reauthorizationConfirm'));
			}
			$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_REFUND, 'Refund', array($this, 'refund'), _t('Refund'));
			$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_REFUND, 'RefundConfirm', array($this, 'refundConfirm'));
			return $operations;
		}
		
		public function capture(scOrder $order, scView $view)
		{
			$currency = new scCurrency($order->get('OrderTotalPayCurrencyCode'));
			
			$alreadyCapturedAmount = $this->getAlreadyCapturedAmount($order);
			$alreadyCaptured = scMultiCurrencyAmount::factory();
			$alreadyCaptured->set($currency, $alreadyCapturedAmount);
			
			$totalCaptureAmount = $order->get('OrderTotalPayAmount');
			$totalCapture = scMultiCurrencyAmount::factory();
			$totalCapture->set($currency, $toCaptureAmount);
			
			$toCapture = scMultiCurrencyAmountMath::subtract($totalCapture, $alreadyCaptured);
			$toCapture = scMultiCurrencyAmountMath::abs($toCapture);
			
			$t = new scTemplateParser();
			$t->readTemplate(self::TEMPLATE_CAPTURE);
			$t->assign('%%ALREADY_CAPTURED_AMOUNT%%', miI18N::htmlEscape(scPrice::factory($alreadyCaptured)->getAmountInCurrency($currency)));
			$t->assign('%%CAPTUREAMOUNT%%', miI18N::htmlEscape(scPrice::factory($toCapture)->getAmountInCurrency($currency)));
			miTable::escapeAndAssignArray($t, $view->getMainPageElements());
			
			$this->displayForm($view, $order, $t);
		}
		
		public function captureConfirm(scOrder $order, scView $view)
		{
			try {
				$captureAmount = miGetParam('CaptureAmount');
				
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'ApiUsername', 'ApiPassword', 'ApiSignature'));
				$nvp = new scPayPalNvpApi($options['Mode'], $options['ApiUsername'], $options['ApiPassword'], $options['ApiSignature']);
				
				$authId = $this->getAuthId($order);
				if ($authId == '')
					throw new miException('Cannot find auth id.');
				
				$result = $nvp->doDoCapture($authId, $captureAmount, $order->get('OrderTotalPayCurrencyCode'), miGetParam('CompleteType'), miGetParam('note'));
				
				$view->addRedirectToEditControllerCommand(_t('Funds captured successfully.'), miMessage::MSG_TYPE_INFO);
			} catch (scPaymentProcessException $exception) {
				throw new miException($exception->getMessage() . $exception->getDetailsAsString());
			}
		}
		
		public function refund(scOrder $order, scView $view)
		{
			$currency = new scCurrency($order->get('OrderTotalPayCurrencyCode'));
			
			$alreadyCapturedAmount = $this->getAlreadyCapturedAmount($order);
			$alreadyCaptured = scMultiCurrencyAmount::factory();
			$alreadyCaptured->set($currency, $alreadyCapturedAmount);
			
			// Build a selecy widget so that the user can select the correct transaction to refund
			$captures = $this->getAllCaptures($order);
			if (empty($captures))
				throw new miException('No transactions to refund.');
			
			$options = array();
			foreach ($captures as $capture)
				$options[$capture['TransactionCode']] = sprintf('%s %s : %s (%s)', $capture['TransactionAmount'], $capture['TransactionCurrencyCode'],
					scDateTime::getLocalFormattedTimestamp($capture['TransactionTimestamp']), $capture['TransactionCode']);
			
			$selectWidget = new miSelectWidget(new miWebForm(array()));
			$selectWidget->setFieldName('TransactionCode');
			$selectWidget->setOptions($options);
			
			$t = new scTemplateParser();
			$t->readTemplate(self::TEMPLATE_REFUND);
			$t->assign('%%ALREADY_CAPTURED_AMOUNT%%', miI18N::htmlEscape(scPrice::factory($alreadyCaptured)->getAmountInCurrency($currency)));
			$t->assign('%%REFUNDAMOUNT%%', miI18N::htmlEscape(scPrice::factory(scMultiCurrencyAmountMath::abs($alreadyCaptured))->getAmountInCurrency($currency)));
			$t->assignArray(miUIUtil::templetizeRows($selectWidget->getEditableControl()));
			
			$this->displayForm($view, $order, $t);
		}
		
		public function refundConfirm(scOrder $order, scView $view)
		{
			try {
				$refundAmount = miGetParamDefault('RefundAmount', 0);
				$transactionId = miGetParam('TransactionCode');
				
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'ApiUsername', 'ApiPassword', 'ApiSignature'));
				$nvp = new scPayPalNvpApi($options['Mode'], $options['ApiUsername'], $options['ApiPassword'], $options['ApiSignature']);
				
				$result = $nvp->doDoRefund($transactionId, $refundAmount, miGetParam('RefundType'), miGetParam('note'));
				
				$view->addRedirectToEditControllerCommand(_t('Funds refunded successfully.'), miMessage::MSG_TYPE_INFO);
			} catch (scPaymentProcessException $exception) {
				throw new miException($exception->getMessage() . $exception->getDetailsAsString());
			}
		}
		
		public function void(scOrder $order, scView $view)
		{
			$t = new scTemplateParser();
			$t->readTemplate(self::TEMPLATE_VOID);
			
			$this->displayForm($view, $order, $t);
		}
		
		public function voidConfirm(scOrder $order, scView $view)
		{
			try {
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'ApiUsername', 'ApiPassword', 'ApiSignature'));
				$nvp = new scPayPalNvpApi($options['Mode'], $options['ApiUsername'], $options['ApiPassword'], $options['ApiSignature']);
				
				$authId = $this->getAuthId($order);
				if ($authId == '')
					throw new miException('Cannot find auth id.');
								
				$result = $nvp->doDoVoid($authId, miGetParam('note'));
				
				$view->addRedirectToEditControllerCommand(_t('Authorization voided successfully.'), miMessage::MSG_TYPE_INFO);
			} catch (scPaymentProcessException $exception) {
				throw new miException($exception->getMessage() . $exception->getDetailsAsString());
			}
		}
		
		public function reauthorization(scOrder $order, scView $view)
		{
			$currency = new scCurrency($order->get('OrderTotalPayCurrencyCode'));
			
			$alreadyCapturedAmount = $this->getAlreadyCapturedAmount($order);
			$alreadyCaptured = scMultiCurrencyAmount::factory();
			$alreadyCaptured->set($currency, $alreadyCapturedAmount);
			
			$totalCaptureAmount = $order->get('OrderTotalPayAmount');
			$totalCapture = scMultiCurrencyAmount::factory();
			$totalCapture->set($currency, $toCaptureAmount);
			
			$toCapture = scMultiCurrencyAmountMath::subtract($totalCapture, $alreadyCaptured);
			$toCapture = scMultiCurrencyAmountMath::abs($toCapture);
			
			$t = new scTemplateParser();
			$t->readTemplate(self::TEMPLATE_REAUTHORIZATION);
			$t->assign('%%ALREADY_CAPTURED_AMOUNT%%', miI18N::htmlEscape(scPrice::factory($alreadyCaptured)->getAmountInCurrency($currency)));
			$t->assign('%%REAUTHORIZATIONAMOUNT%%', miI18N::htmlEscape(scPrice::factory($toCapture)->getAmountInCurrency($currency)));
			miTable::escapeAndAssignArray($t, $view->getMainPageElements());
			
			$this->displayForm($view, $order, $t);
		}
		
		public function reauthorizationConfirm(scOrder $order, scView $view)
		{
			try {
				$reauthorizationAmount = miGetParam('ReauthorizationAmount');
				
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'ApiUsername', 'ApiPassword', 'ApiSignature'));
				$nvp = new scPayPalNvpApi($options['Mode'], $options['ApiUsername'], $options['ApiPassword'], $options['ApiSignature']);
				
				$authId = $this->getAuthId($order);
				if ($authId == '')
					throw new miException('Cannot find auth id.');
				
				$result = $nvp->doDoReauthorization($authId, $reauthorizationAmount, $order->get('OrderTotalPayCurrencyCode'));
				
				$view->addRedirectToEditControllerCommand(_t('Funds reauthorized successfully.'), miMessage::MSG_TYPE_INFO);
			} catch (scPaymentProcessException $exception) {
				throw new miException($exception->getMessage() . $exception->getDetailsAsString());
			}
		}
		
		protected function displayForm($view, $order, $t)
		{
			// Get the order information
			$view->addMainPageElements(miUIUtil::templetizeRows($order->getRow()));
			
			miTable::escapeAndAssignArray($t, $view->getMainPageElements());
			$t->assign('%%ORDERID%%', miI18N::htmlEscape($order->getPK()));
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $t->templateParse());
			$view->addControllerCommand($cmd);
		}
		
		protected function getAlreadyCapturedAmount(scOrder $order)
		{
			$sql = 'SELECT SUM(TransactionAmount) AS Sum FROM Transactions WHERE OrderID=? AND TransactionPaymentStatus IN (?,?)';
			$result = miStaticDBUtil::execSelect($sql, array($order->getPK(), scPaymentStatus::PAYMENT_STATUS_CAPTURE, scPaymentStatus::PAYMENT_STATUS_PARTIAL_CAPTURE));
			return $result[0]['Sum'];
		}
		
		/**
		 * Retrieve the authorization id
		 * Finds the last auth_id in the transaction list
		 * 
		 * @param $order
		 * @return string
		 */
		protected function getAuthId(scOrder $order)
		{
			$sql = 'SELECT TransactionDetailValue FROM TransactionDetails JOIN Transactions USING(TransactionID) WHERE OrderID=? AND TransactionDetailName=? ORDER BY Transactions.TransactionID DESC LIMIT 1';
			$result = miStaticDBUtil::execSelect($sql, array($order->getPK(), 'auth_id'));
			if (count($result))
				return $result[0]['TransactionDetailValue'];
			return '';
		}
		
		/**
		 * Retrieve the transation id
		 * The transaction id is the first TransactionCode saved as part if this order
		 * 
		 * @param scOrder $order
		 * @return string
		 */
		protected function getTransactionId(scOrder $order)
		{
			$sql = 'SELECT TransactionCode FROM Transactions WHERE OrderID=? ORDER BY TransactionID LIMIT 1';
			$result = miStaticDBUtil::execSelect($sql, array($order->getPK()));
			if (count($result))
				return $result[0]['TransactionCode'];
			return '';
		}
		
		/**
		 * Checks if the auth has been completed
		 * An auth is completed if the last status is not In_Progress or Pending
		 * 
		 * @param scOrder $order
		 * @return bool
		 */
		protected function isAuthCompleted(scOrder $order)
		{
			$sql = 'SELECT TransactionDetailValue FROM TransactionDetails JOIN Transactions USING(TransactionID) WHERE OrderID=? AND TransactionDetailName=? ORDER BY TransactionDetails.TransactionID DESC LIMIT 1';
			$result = miStaticDBUtil::execSelect($sql, array($order->getPK(), 'auth_status'));
			if (count($result))
				return ($result[0]['TransactionDetailValue'] != 'In_Progress') and ($result[0]['TransactionDetailValue'] != 'Pending');
			
			// There is no auth_status - return true for completed
			return true;
		}
		
		/**
		 * Return all captures for this order
		 * @param scOrder $order
		 * @return array
		 */
		protected function getAllCaptures(scOrder $order)
		{
			$sql = 'SELECT TransactionTimestamp, TransactionAmount, TransactionCurrencyCode, TransactionCode FROM Transactions WHERE'.
				' OrderID=? AND TransactionPaymentStatus IN(?,?) ORDER BY TransactionID';
			return miStaticDBUtil::execSelect($sql, array($order->getPK(), scPaymentStatus::PAYMENT_STATUS_CAPTURE, scPaymentStatus::PAYMENT_STATUS_PARTIAL_CAPTURE));
		}
	}
?>