<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * PayPal WPS payment processing module
	 */
	class scPaymentModulePayPalWPS implements IPaymentModule, IModuleOptions {
		const MODE_LIVE = 'Live';
		const MODE_SANDBOX = 'Sandbox';
		
		const PAYPAL_LIVE_URL = 'https://www.paypal.com/cgi-bin/webscr';
		const PAYPAL_SANDBOX_URL = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
		
		protected $_moduleId;
		
		protected $_paypalCurrencies = array('AUD', 'BRL', 'CAD', 'CHF', 'CZK', 'DKK', 'EUR', 'GBP', 'HKD', 'HUF', 'ILS', 'JPY',
			'MYR', 'MXN', 'NOK', 'NZD', 'PHP', 'PLN', 'SGD', 'SEK', 'TWD', 'THB', 'USD');
		
		protected $_paymentActions = array(
			'sale' => 'Sale',
			'authorization' => 'Authorization',
			'order' => 'Order',
		);
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $this->_paypalCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $params)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Account', 'PaymentAction', 'OrderDescription', 'DefaultCurrencyCode', 'Mode'));
			
			$params = array(
				'cmd' => '_xclick',
				'paymentaction' => $options['PaymentAction'],
				'business' => $options['Account'],
				'amount' => number_format($payAmount, 2, '.', ''),
				'currency_code' => $payCurrencyCode,
				'custom' => scOrdersInProgress::singleton()->getLastOrderId(),
				'item_name' => $options['OrderDescription'],
				
				'lc' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'charset' => 'utf-8',
				
				'no_shipping' => '1',
				'address_override' => '1',
				
				'first_name' => $customerInfo['CustomerBillingFirstName'],
				'last_name' => $customerInfo['CustomerBillingLastName'],
				'address1' => $customerInfo['CustomerBillingAddress1'],
				'address2' => $customerInfo['CustomerBillingAddress2'],
				'city' => $customerInfo['CustomerBillingCity'],
				'state' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'zip' => $customerInfo['CustomerBillingZip'],
				'country' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'email' => $customerInfo['CustomerEmail'],
				
				'night_phone_a' => $customerInfo['CustomerBillingPhone'],
				'night_phone_b' => '',
				'night_phone_c' => '',
				
				'bn' => 'MirchevIdeas_SummerCart',
				'mrb' => 'ZG2ZHG9AKUMCC',
				'pal' => 'ZG2ZHG9AKUMCC',
				
				'return' => scPaths::getSecureUrl('modules/payment/paypal/wps_return.php'),
				'cancel_return' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_ABSOLUTE),
				'notify_url' => scPaths::getSecureUrl('modules/payment/paypal/wps_notify.php'),
			);
			
			$result = new scPaymentProcessResult();
			$url = ($options['Mode'] == self::MODE_LIVE) ? self::PAYPAL_LIVE_URL : self::PAYPAL_SANDBOX_URL;
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		public function processReturn($params)
		{
			// Check if the tx param
			if (!isset($params['tx']))
				throw new miException(sprintf(_t('Please enable Payment Data Transfer.')));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Account', 'Mode', 'PDTIdentityToken'));
			
			$post = array(
				'cmd' => '_notify-synch',
				'tx' => $params['tx'],
				'at' => $options['PDTIdentityToken']
			);
			$url = ($options['Mode'] == self::MODE_LIVE) ? self::PAYPAL_LIVE_URL : self::PAYPAL_SANDBOX_URL;
			$result = scHttp::post($url, http_build_query($post));
			
			$lines = explode("\n", $result);
			if ($lines[0] != 'SUCCESS')
				throw new miException(sprintf(_t('Error verifing Payment Data Transfer. Please check your PDT identity token. %s'), $lines[1]));
			
			unset($lines[0]);
			
			$params = array();
			foreach ($lines as $line) {
				if (trim($line) == '')
					continue;
				
				list($key, $value) = explode('=', $line);
				$params[urldecode($key)] = urldecode($value);
			}
			
			$params['PDT'] = 'PDT';
			return $this->processPayPalInfo($options, $params);
		}
		
		/**
		 * Handle notification
		 *
		 * @param array $params
		 */
		public function processNotify($params)
		{
			//scDebug::logVariable('/tmp/paypal.txt', 'POST', $params);
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Account', 'Mode'));
			
			// Validate notification
			$post = $params;
			$post['cmd'] = '_notify-validate';
			$url = ($options['Mode'] == self::MODE_LIVE) ? self::PAYPAL_LIVE_URL : self::PAYPAL_SANDBOX_URL;
			$result = scHttp::post($url, http_build_query($post));
			//scDebug::logVariable('/tmp/paypal.txt', 'Paypal result', $result);
			
			if ($result != 'VERIFIED')
				throw new scPaymentProcessException(_t('PayPal notification validation failed.'), array(_t('Result') => $result));
			
			$params['IPN'] = 'IPN';
			return $this->processPayPalInfo($options, $params);
		}
		
		protected function processPayPalInfo($options, $params)
		{
			$this->checkPayPalAccount($options, $params);
			
			if (isset($params['test_ipn']) and $params['test_ipn'] and ($options['Mode'] != self::MODE_SANDBOX)) {
				throw new scPaymentProcessException(_t('Test IPN to live system.'), array(_t('Result') => $result));
			}
			
			scPaymentProcess::checkParamsExist($params, array('custom', 'mc_gross', 'mc_currency', 'txn_id'));
			
			// Process the notification
			$result = new scPaymentProcessResult($params['custom'], $params['mc_gross'], $params['mc_currency'], $params['txn_id']);
			switch ($params['payment_status']) {
				case 'Completed':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					
					// Check that the currency and the amount match. If not, then this is a partial capture
					$order = new scOrder($params['custom']);
					if (($order->get('OrderTotalPayCurrencyCode') != $params['mc_currency']) or (round($order->get('OrderTotalPayAmount'),2) != round($params['mc_gross'],2))) {
						$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PARTIAL_CAPTURE);
					}
					break;
				
				case 'Pending':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
					$result->setTransactionNote($params['pending_reason']);
					break;
				
				case 'Denied':
				case 'Expired':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					$result->setTransactionNote($params['payment_status']);
					break;
				
				case 'Refunded':
				case 'Reversed':
					$paymentStatus = ($params['reason_code'] == 'chargeback') ? scPaymentStatus::PAYMENT_STATUS_CHARGEBACK : scPaymentStatus::PAYMENT_STATUS_REFUND;
					$result->setPaymentStatus($paymentStatus);
					$result->setTransactionNote($params['reason_code']);
					break;
				
				default:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_OTHER);
					$result->setTransactionNote($params['payment_status']);
			}
			
			$result->setAllTransactionDetails($params);
//			$transactionDetailFields = array('mc_gross', 'invoice', 'settle_amount', 'protection_eligibility', 'address_status',
//				'payer_id', 'payment_status', 'mc_fee', 'exchange_rate', 'settle_currency', 'payer_status', 'payment_type',
//				'auth_id', 'auth_amount', 'auth_exp', 'auth_status', 'remaining_settle', 'payer_email', 'test_ipn', 'memo', 'parent_txn_id');
//			foreach ($transactionDetailFields as $transactionDetailField) {
//				if (isset($params[$transactionDetailField]))
//					$result->setTransactionDetail($transactionDetailField, $params[$transactionDetailField]);
//			}
			
			return $result;
		}
		
		protected function checkPayPalAccount($options, $params)
		{
			// Check if the account is using Secure Merchant Account ID (receiver_id)
			if (isset($params['receiver_id']) and ($options['Account'] == $params['receiver_id']))
				return;
			
			if (isset($params['business']) and ($options['Account'] == $params['business']))
				return;
			
			if ($options['Account'] == $params['receiver_email'])
				return;
				
			throw new miException(sprintf(_i('Invalid business "%s" and receiver_email "%s", expected "%s"'), @$params['business'], @$params['receiver_email'], $options['Account']));
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'Account',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'PaymentAction',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'OrderDescription',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'PDTIdentityToken',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ApiUsername',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ApiPassword',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ApiSignature',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_paypalCurrencies, $this->_paypalCurrencies));
			$options->getOptionField('PaymentAction')->setOptions($this->_paymentActions);
			$options->getOptionField('Mode')->setRadioButtons(array(self::MODE_SANDBOX => _t('Sandbox'), self::MODE_LIVE => _t('Live')), self::MODE_SANDBOX);
			$options->setTemplateFile('modules/payment/paypal/wps_options.tmpl');
			return $options;
		}
	}
?>