<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/plationline.php');
	
	class scPaymentModulePlatiOnlineRoOperations extends scPaymentOperations implements IPaymentModuleOperations {
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentOperationDefinitions(scOrder $order)
		{
			$operations = array();
			if ($order->get('OrderPaymentStatus') == scPaymentStatus::PAYMENT_STATUS_AUTH) {
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_CAPTURE, 'Capture', array($this, 'showCaptureForm'), _t('Capture'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_CAPTURE, 'CaptureConfirm', array($this, 'captureConfirm'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_VOID, 'Void', array($this, 'showVoidForm'), _t('Void'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_VOID, 'VoidConfirm', array($this, 'voidConfirm'));
			}
			
			if (($order->get('OrderPaymentStatus') == scPaymentStatus::PAYMENT_STATUS_CAPTURE) or ($order->get('OrderPaymentStatus') == scPaymentStatus::PAYMENT_STATUS_PENDING)) {
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_REFUND, 'Refund', array($this, 'showRefundForm'), _t('Refund'));
				$operations[] = new scPaymentOperationDefinition(scPaymentOperationDefinition::OPERATION_TYPE_REFUND, 'RefundConfirm', array($this, 'refundConfirm'));
			}
			return $operations;
		}
		
		public function captureConfirm(scOrder $order, scView $view)
		{
			try {
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'EncryptionKey', 'EncryptionMode'));
				$api = new scPaymentModulePlatiOnlineRoApi($options['EncryptionKey'], $options['EncryptionMode']);
				
				$transactionId = $this->getTransactionId($order);
				if ($transactionId == '')
					throw new miException('Cannot find transaction id.');
				
				$result = $api->performSettlement($options['MerchantID'], $this->getTransactionId($order), $order->getPK(), $order->get('OrderShippingModuleName') . ' / ' . $order->get('OrderShippingMethodName'), $order->get('OrderShippingTrackingNo'));
				
				switch ($result['X_RESPONSE_CODE']) {
					case 3:
						$view->addRedirectToEditControllerCommand(_t('Funds captured successfully.'), miMessage::MSG_TYPE_INFO);
						break;
					default:
					case 10:
						$view->addRedirectToEditControllerCommand(sprintf(_i('Error: %s / %s.'), $result['X_RESPONSE_REASON_CODE'], $result['X_RESPONSE_REASON_TEXT']), miMessage::MSG_TYPE_ERROR);
						break;
					case 13:
						$view->addRedirectToEditControllerCommand(sprintf(_i('The transaction was authorized but requires additional verifications: %s / %s.'), $result['X_RESPONSE_REASON_CODE'], $result['X_RESPONSE_REASON_TEXT']), miMessage::MSG_TYPE_WARNING);
						break;
				}
				
				// Update order payment status
				$this->updateOrderPaymentStatus($api, $order, $options['MerchantID'], $result['X_TRAN_ID']);
				
			} catch (scPaymentProcessException $exception) {
				throw new miException($exception->getMessage() . $exception->getDetailsAsString());
			}
		}
		
		public function voidConfirm(scOrder $order, scView $view)
		{
			try {
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'EncryptionKey', 'EncryptionMode'));
				$api = new scPaymentModulePlatiOnlineRoApi($options['EncryptionKey'], $options['EncryptionMode']);
				$transactionId = $this->getTransactionId($order);
				if ($transactionId == '')
					throw new miException('Cannot find transaction id.');
				
				$result = $api->performVoid($options['MerchantID'], $transactionId, $order->getPK());
				
				switch ($result['X_RESPONSE_CODE']) {
					case 7:
						$view->addRedirectToEditControllerCommand(_t('Authorization voided successfully.'), miMessage::MSG_TYPE_INFO);
						break;
					default:
					case 10:
						$view->addRedirectToEditControllerCommand(sprintf(_i('Error: %s / %s.'), $result['X_RESPONSE_REASON_CODE'], $result['X_RESPONSE_REASON_TEXT']), miMessage::MSG_TYPE_ERROR);
						break;
					case 13:
						$view->addRedirectToEditControllerCommand(sprintf(_i('The request was declined as the transaction status does not allow voiding: %s / %s.'), $result['X_RESPONSE_REASON_CODE'], $result['X_RESPONSE_REASON_TEXT']), miMessage::MSG_TYPE_WARNING);
						break;
				}
				
				// Update order payment status
				$this->updateOrderPaymentStatus($api, $order, $options['MerchantID'], $result['X_TRAN_ID']);
				
			} catch (scPaymentProcessException $exception) {
				throw new miException($exception->getMessage() . $exception->getDetailsAsString());
			}
		}
		
		public function refundConfirm(scOrder $order, scView $view)
		{
			try {
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'EncryptionKey', 'EncryptionMode'));
				$api = new scPaymentModulePlatiOnlineRoApi($options['EncryptionKey'], $options['EncryptionMode']);
				$transactionId = $this->getTransactionId($order);
				if ($transactionId == '')
					throw new miException('Cannot find transaction id.');
				
				$result = $api->performCredit($options['MerchantID'], $transactionId, $order->getPK());
				
				switch ($result['X_RESPONSE_CODE']) {
					case 1:
						$view->addRedirectToEditControllerCommand(_t('Funds refunded successfully.'), miMessage::MSG_TYPE_INFO);
						break;
					default:
					case 10:
						$view->addRedirectToEditControllerCommand(sprintf(_i('Error: %s / %s.'), $result['X_RESPONSE_REASON_CODE'], $result['X_RESPONSE_REASON_TEXT']), miMessage::MSG_TYPE_ERROR);
						break;
					case 13:
						$view->addRedirectToEditControllerCommand(sprintf(_i('The request was declined as the transaction status does not allow crediting: %s / %s.'), $result['X_RESPONSE_REASON_CODE'], $result['X_RESPONSE_REASON_TEXT']), miMessage::MSG_TYPE_WARNING);
						break;
				}
				
				// Update order payment status
				$this->updateOrderPaymentStatus($api, $order, $options['MerchantID'], $result['X_TRAN_ID']);
				
			} catch (scPaymentProcessException $exception) {
				throw new miException($exception->getMessage() . $exception->getDetailsAsString());
			}
		}
		
		/**
		 * Update the order payment status
		 * @param scPaymentModulePlatiOnlineRoApi $api
		 * @param scOrder $order
		 * @param string $merchantId
		 * @param string $transactionId
		 */
		protected function updateOrderPaymentStatus(scPaymentModulePlatiOnlineRoApi $api, scOrder $order, $merchantId, $transactionId)
		{
			$result = $api->getQueryResult($merchantId, $transactionId, $order->getPK());
			scPaymentProcess::handlePaymentProcessResult($result, $this->_moduleId, scPaymentProcess::CALL_TYPE_NOTIFY);
		}
	}
?>