<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * plationline.ro romanian payment processing module	
	 */
	class scPaymentModulePlatiOnlineRo implements IPaymentModule, IModuleOptions {
		
		const MODE_LIVE = 'Live';
		const MODE_TEST = 'Test';
		
		protected $_moduleId;
		
		/**
		 * ISO 639-2 to plationline.ro language code map
		 * @var array
		 */
		protected $_langCodeMap = array(
			// ISO 639-2 => plationline.ro
			'ro' => 'ro',
			'en' => 'en',
		);
		
		protected static $_allowedCurrencies = array('EUR', 'RON', 'USD');
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode() 
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), self::$_allowedCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'EncryptionKey', 'EncryptionMode', 'SkipForms', 'DefaultCurrencyCode'));
			
			$currency = new scCurrency($payCurrencyCode);
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = isset($this->_langCodeMap[$isoLangCode]) ? $this->_langCodeMap[$isoLangCode] : 'en';
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$params = array(
				'F_Show_Form' => $options['SkipForms'] ? 0 : 1,
				'F_Login' => $options['MerchantID'],
				'F_Action' => scPaymentModulePlatiOnlineRoApi::ACTION_SIMPLE_AUTH,
				'F_Lang' => $lang,
				'F_Language' => $lang,
				'F_Amount' => $payAmount,
				'F_Currency' => $payCurrencyCode,
				'F_Order_Number' => $orderId,
				
				'F_First_Name' => $customerInfo['CustomerBillingFirstName'],	
				'F_Last_Name' => $customerInfo['CustomerBillingLastName'],
				'F_Company' => $customerInfo['CustomerBillingCompany'],
				'F_CUI' => $customerInfo['CustomerBillingVatNumber'],
				'F_Address' => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				'F_City' => $customerInfo['CustomerBillingCity'],	
				'F_Zip' => $customerInfo['CustomerBillingZip'],
				'F_State' => scCountry::getStateName($customerInfo['CustomerBillingState']),
				'F_Country' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'F_Phone' => $customerInfo['CustomerBillingPhone'],
				'F_Email' => $customerInfo['CustomerEmail'],
				
				'F_Ship_To_First_Name' => $customerInfo['CustomerBillingFirstName'],	
				'F_Ship_To_Last_Name' => $customerInfo['CustomerBillingLastName'],
				'F_Ship_To_Company' => $customerInfo['CustomerBillingCompany'],
				'F_Ship_To_Address' => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				'F_Ship_To_City' => $customerInfo['CustomerBillingCity'],
				'F_Ship_To_Zipcode' => $customerInfo['CustomerBillingZip'],
				'F_Ship_To_State' => scCountry::getStateName($customerInfo['CustomerBillingState']),
				'F_Ship_To_Country' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'F_Ship_To_Phone' => $customerInfo['CustomerBillingPhone'],
				
				'F_Test_Request' => ($options['Mode'] == self::MODE_TEST) ? 1 : 0,
				
				// Check if these might work
//				'F_Relay_Method' => 'GET',
//				'F_Relay_Response_URL' => scPaths::getSecureUrl('/modules/payment/plationline_ro/relay.php'),
			);
			
			$params['F_Order_String'] = $this->getOrderString($cart, $params, $payCurrencyCode);
			
			$api = new scPaymentModulePlatiOnlineRoApi($options['EncryptionKey'], $options['EncryptionMode']);
			
			$params['F_Message'] = $api->getMessageForAuth($params['F_Login'], $params['F_Amount'], $params['F_Currency'], $params['F_Order_Number'], $params['F_Action']);
			$params['F_Crypt_Message'] = $api->getCryptMessage($params['F_Message'], $options['EncryptionKey'], $options['EncryptionMode']);
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(scPaymentModulePlatiOnlineRoApi::PLATIONLINE_GATEWAY_URL, $params);
			
			return $result;
		}
		
		/**
		 * Handles payment relay return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processRelay($params)
		{
			scPaymentProcess::checkParamsExist($params, array('F_MESSAGE_RELAY', 'F_CRYPT_MESSAGE_RELAY', 'F_AMOUNT', 'F_CURRENCY', 'X_RESPONSE_CODE',
				'F_ORDER_NUMBER', 'X_TRAN_ID', 'X_AUTH_CODE', 'F_ACTION', 'X_RESPONSE_REASON_CODE', 'X_RESPONSE_REASON_TEXT'));
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'EncryptionKey', 'EncryptionMode'));
			
			$api = new scPaymentModulePlatiOnlineRoApi($options['EncryptionKey'], $options['EncryptionMode']);
			
			$msgRelay = $params['F_MESSAGE_RELAY'];
			if (!$api->isHashCorrect($msgRelay, $params['F_CRYPT_MESSAGE_RELAY']))
				throw new miException(_i('Invalid hash. Please check encryption key and encryption mode.'));
			
			// Check the authencity of the params
			$params['MerchantID'] = $options['MerchantID'];
			$api->checkParamsAuthencity($msgRelay, $params, array(1 => 'MerchantID', 3 => 'F_AMOUNT', 4 => 'F_CURRENCY', 5 => 'X_RESPONSE_CODE', 6 => 'F_ORDER_NUMBER', 7 => 'X_TRAN_ID', 8 => 'X_AUTH_CODE', 9 => 'F_ACTION'));
			
			// Handle the return
			$orderId = (int)$params['F_ORDER_NUMBER'];
			
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($orderId, $params['F_AMOUNT'], $params['F_CURRENCY']);
			
			// Redirect the customer to the actual return script
			ignore_user_abort(true);
			$url = scPaths::getSecureUrl('/modules/payment/plationline_ro/return.php?OrderID=' . $orderId);

			$html = '<html><head>';
			$html .= '<meta http-equiv="refresh" content="0;url=' . miI18N::htmlEscape($url) . '">';
			$html .= '</head><body></body></html>';
			echo $html;
			
			// Build the payment result
			$result = new scPaymentProcessResult($orderId, $params['F_AMOUNT'], $params['F_CURRENCY'], $params['X_TRAN_ID'], $params['X_RESPONSE_REASON_TEXT']);
			switch ($params['X_RESPONSE_CODE']) {
				case 2:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_AUTH);
					break;
				case 13:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
					break;
				case 8:
				case 10:
				default:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					break;
			}
			$result->setPaymentNotes(sprintf(_i('%s (%s)'), $params['X_RESPONSE_REASON_TEXT'], $params['X_RESPONSE_REASON_CODE']));
			$result->setAllTransactionDetails($params);
			
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			$orderId = (int)$params['OrderID'];
			$result = new scPaymentProcessResult($orderId);
			return $result;
		}

		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processItsn($params)
		{
//			$params = array (
//  'start' => 'POITSN',
//  'F_AMOUNT' => '0.35',
//  'F_CURRENCY' => 'USD',
//  'X_TRAN_ID' => '322215',
//  'X_TRANS_ID' => '322215',
//  'F_ORDER_NUMBER' => '2739',
//  'F_MESSAGE_ITSN' => '377^DEMO www.mirchevideas.com^3/3/2011 6:06:03 PM^0.35^USD^322215^2739',
//  'F_CRYPT_MESSAGE_ITSN' => 'A861B28A6F777331B09BAA70F697E75735ABBF93',
//  'end' => 'POITSN',
//);
			scPaymentProcess::checkParamsExist($params, array('F_MESSAGE_ITSN', 'F_CRYPT_MESSAGE_ITSN', 'X_TRAN_ID', 'F_ORDER_NUMBER'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'EncryptionKey', 'EncryptionMode'));
			
			$api = new scPaymentModulePlatiOnlineRoApi($options['EncryptionKey'], $options['EncryptionMode']);
			
			$msgItsn = $params['F_MESSAGE_ITSN'];
			if (!$api->isHashCorrect($msgItsn, $params['F_CRYPT_MESSAGE_ITSN']))
				throw new miException(_i('Invalid hash. Please check encryption key and encryption mode.'));
			
			// Check the authencity of the params
			$params['MerchantID'] = $options['MerchantID'];
			$api->checkParamsAuthencity($msgItsn, $params, array(1 => 'MerchantID', 3 => 'F_AMOUNT', 4 => 'F_CURRENCY', 5 => 'X_TRAN_ID', 6 => 'F_ORDER_NUMBER'));
			
			// Perform Query to get the transaction status
			$result = $api->getQueryResult($options['MerchantID'], $params['X_TRAN_ID'], $params['F_ORDER_NUMBER'], $params['F_AMOUNT'], $params['F_CURRENCY']);
			
			// Print the expected result
			$xmlDef= array(
				scXml::TAG => 'itsn',
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'x_trans_id',
						scXml::DATA => $params['X_TRAN_ID']
					),
					array(
						scXml::TAG => 'merchServerStamp',
						scXml::DATA => date('m/d/Y h:i:s a')
					),
					array(
						scXml::TAG => 'f_response_code',
						scXml::DATA => 1	// 0 - fail, 1 - success
					),
				)
			);
			echo '<?xml version="1.0" encoding="UTF-8" ?>' . "\n" . scXml::xmlWriter(array($xmlDef));
			
			return $result;
		}
		
		public function getOptionFields(miWebForm &$webForm) 
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'MerchantID',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'EncryptionKey',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'EncryptionMode',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'SkipForms',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$modes = array(self::MODE_LIVE => _t('Live'), self::MODE_TEST => _t('Test'));
			$options->getOptionField('Mode')->setOptions($modes);
			
			sort(self::$_allowedCurrencies);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine(self::$_allowedCurrencies, self::$_allowedCurrencies));
			
			$options->setTemplateFile('modules/payment/plationline_ro/plationline_options.tmpl');
			return $options;
		}
		
		
		protected function getOrderString(scCart $cart, $params, $payCurrencyCode)
		{
			$itemsXml = array();
			
			$currency = new scCurrency($payCurrencyCode);
			$items = $cart->getItems();
			foreach ($items as $item) {
				$itemsXml[] = array(
					scXml::TAG => 'item',
					scXml::CHILDREN => array(
						array(
							scXml::TAG => 'ProdID',
							scXml::DATA => $item->getCartItemCode()
						),
						array(
							scXml::TAG => 'qty',
							scXml::DATA => $item->getQuantity()
						),
						array(
							scXml::TAG => 'itemprice',
							scXml::DATA => $item->getPriceObj()->getAmountInCurrency($currency)
						),
						array(
							scXml::TAG => 'name',
							scXml::DATA => $item->getName()
						),
						array(
							scXml::TAG => 'Prodtype_id',
							scXml::DATA => $item->isTangible() ? 1 : 0
						),
					)
				);
			}
			
			$xmlDef = array(
				scXml::TAG => 'start_string',
				scXml::CHILDREN => $itemsXml
			);
			
			return scXml::xmlWriter(array($xmlDef));
		}
		
	}
	
	/**
	 * PlatiOnline.ro API
	 */
	class scPaymentModulePlatiOnlineRoApi {
		const PLATIONLINE_GATEWAY_URL = 'https://secure2.plationline.ro/trans30ta.asp';
		const PLATIONLINE_API_URL = 'https://secure2.plationline.ro/trans30.asp';
		
		const ACTION_QUERY = 0;
		const ACTION_CREDIT = 1;
		const ACTION_SIMPLE_AUTH = 2;
		const ACTION_SETTLE = 3;
		const ACTION_VOID = 7;
		const ACTION_INSTALLMENTS_AUTH = 10;
		
		protected $keyEnc;
		protected $keyMod;
		
		public function __construct($keyEnc, $keyMod)
		{
			$this->keyEnc = $keyEnc;
			$this->keyMod = $keyMod;
		}
		
		/**
		 * Perform query of a transaction
		 * @param string $merchantId
		 * @param string $transactionId the transaction id as returned by plationline.ro
		 * @param int $orderNumber
		 * @return array
		 */
		public function performQuery($merchantId, $transactionId, $orderNumber)
		{
			return $this->performAction($merchantId, $transactionId, $orderNumber, self::ACTION_QUERY);
		}
		
		/**
		 * Credit a transaction
		 * @param string $merchantId
		 * @param string $transactionId the transaction id as returned by plationline.ro
		 * @param int $orderNumber
		 * @return array
		 */
		public function performCredit($merchantId, $transactionId, $orderNumber)
		{
			return $this->performAction($merchantId, $transactionId, $orderNumber, self::ACTION_CREDIT);
		}
		
		/**
		 * Void a transaction
		 * @param string $merchantId
		 * @param string $transactionId the transaction id as returned by plationline.ro
		 * @param int $orderNumber
		 * @return array
		 */
		public function performVoid($merchantId, $transactionId, $orderNumber)
		{
			return $this->performAction($merchantId, $transactionId, $orderNumber, self::ACTION_VOID);
		}
		
		/**
		 * Perform action
		 * @param string $merchantId
		 * @param string $transactionId the transaction id as returned by plationline.ro
		 * @param int $orderNumber
		 * @return array
		 */
		protected function performAction($merchantId, $transactionId, $orderNumber, $action)
		{
			$message = implode('^', array($this->getMessagePrefix($merchantId), $transactionId, $orderNumber, $action));
			
			$params = array(
				'F_Login' => $merchantId,
				'F_Message' => $message,
				'F_Crypt_Message' => $this->getCryptMessage($message),
				'X_Trans_ID' => $transactionId,
				'F_Order_Number' => $orderNumber,
				'F_Action' => $action,
				'F_XML' => 0
			);
			
			$result = scHttp::post(self::PLATIONLINE_API_URL, http_build_query($params));
			parse_str($result, $resultArray);
			return $resultArray;
		}
		
		/**
		 * Perform settlement of transaction (capture)
		 * @param string $merchantId
		 * @param string $transactionId
		 * @param int $orderNumber
		 * @param string $shippingCompany
		 * @param string $shippingTrackingNumber
		 * @return array
		 */
		public function performSettlement($merchantId, $transactionId, $orderNumber, $shippingCompany, $shippingTrackingNumber)
		{
			if (empty($shippingCompany))
				$shippingCompany = '-';
			if (empty($shippingTrackingNumber))
				$shippingTrackingNumber = '-';
			
			$message = implode('^', array($this->getMessagePrefix($merchantId), $transactionId, $orderNumber, $shippingTrackingNumber, $shippingCompany, self::ACTION_SETTLE));
			
			$params = array(
				'F_Login' => $merchantId,
				'F_Message' => $message,
				'F_Crypt_Message' => $this->getCryptMessage($message),
				'X_Trans_ID' => $transactionId,
				'F_Order_Number' => $orderNumber,
				'F_Shipping_Company' => $shippingCompany,
				'F_AWB' => $shippingTrackingNumber,
				'F_Action' => self::ACTION_SETTLE,
				'F_XML' => 0
			);
			
			$result = scHttp::post(self::PLATIONLINE_API_URL, http_build_query($params));
			parse_str($result, $resultArray);
			return $resultArray;
		}
		
		/**
		 * Return payment process result based on data returned from PlatiOnline.ro
		 * 
		 * @param array $params
		 * @param string $merchantId
		 * @param string $transactionId
		 * @param int $orderId
		 * @return scPaymentProcessResult
		 */
		public function getQueryResult($merchantId, $transactionId, $orderId, $amount = 0, $currency = '')
		{
			$orderId = (int)$orderId;
			
			$queryResult = $this->performQuery($merchantId, $transactionId, $orderId);
			
			$result = new scPaymentProcessResult($orderId, $amount, $currency, $transactionId, $queryResult['X_RESPONSE_REASON_TEXT']);
			switch ($queryResult['X_STARE_FIN1']) {
				case 2: // Authorized
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_AUTH);
					break;
				case 3: // Pending settle
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
					break;
				case 5: // Settled
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					break;
				case 6: //Pending void
					break;
				case 7: // Voided
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_VOID);
					break;
				case 8: // Declined
				case 9: // Expired
				case 10: // Error
				case 13: // Auth-call verify
				default:
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					break;
			}
			
			// FIXME: Handle X_STARE_FIN2
			
			$result->setAllTransactionDetails($queryResult);
			return $result;
		}
		
		protected function getMessagePrefix($merchantId)
		{
			$sequence = rand(1, 1000);
			$tstamp = date('m/d/Y h:i:s a');
			return implode('^', array($sequence, $merchantId, $tstamp));
		}
		
		public function getMessageForAuth($merchantId, $amount, $currency, $orderNumber, $action)
		{
			return implode('^', array($this->getMessagePrefix($merchantId), $amount, $currency, $orderNumber, $action));
		}
		
		/**
		 * Verify that the hash matches
		 * @param string $msg
		 * @param string $hash
		 * @return bool true if hash matches
		 */
		public function isHashCorrect($msg, $hash)
		{
			return strcasecmp($this->getCryptMessage($msg), $hash) == 0 ? true : false;
		}
		
		/**
		 * Checks if message matches tha params. This is used to check if the params have been tampered with
		 * @param string $msg
		 * @param array $varsToCheck
		 * @throws miException if the message is not correct
		 */
		public function checkParamsAuthencity($msg, $params, $varsToCheck)
		{
			$msgArray = explode('^', $msg);
			
			foreach ($varsToCheck as $index => $paramName) {
				if ($msgArray[$index] != $params[$paramName])
					throw new miException(_i('Invalid ' . $paramName . '.'));
			}
		}
		
		public function getCryptMessage($msg)
		{
			$stringRSA = $this->rsaEncrypt($msg, $this->keyEnc, $this->keyMod);
			
			$key = pack('H*', $this->keyEnc . $this->keyMod);
			return scCryptUtil::hmac($key, $stringRSA, scCryptUtil::HASH_SHA1);
		}
		
		/**
		 * Calculate $x^$pow(mod $keyMod)
		 * 
		 * @param int $x
		 * @param int $pow
		 * @param int $keyMod
		 * @return int
		 */
		protected function powmod($x, $pow, $keyMod)
		{
			$result = 1;
			
			while ($pow > 0) {
				while (($pow % 2) == 0) {
					$x = $this->modulo($x*$x, $keyMod);
					$pow /= 2;
				}
				$result = $this->modulo($x*$result, $keyMod);
				$pow--;
			}
			return $result;
		}
	
		protected function modulo($x, $y)
		{
			if ($y == 0)
				return 0;
			
			return $x - (intval($x / $y) * $y);
		}
	
		protected function rsaEncrypt($msg, $keyEnc, $keyMod)
		{
			$result = '';
			for ($i = 0; $i < strlen($msg); $i++) {
				$c = ord($msg[$i]);
				$result .= sprintf('%08s', dechex($this->powmod($c, $keyEnc, $keyMod)));
			}
			return strtoupper($result);
		}
	}
?>