<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scPaymentModuleSagePayVSP {
		protected $_moduleId;
		
		const VSP_SIMULATOR_MODE = 0;
		const VSP_TEST_MODE = 1;
		const VSP_LIVE_MODE = 2;
		
		const DEFAULT_CURRENCY_CODE = 'GBP';
		protected $_sagepayCurrencies = array('AUD', 'CAD', 'CHF', 'DKK', 'EUR', 'GBP', 'HKD',
			'IDR', 'JPY', 'LUF', 'NOK', 'NZD', 'SEK', 'SGD', 'TRL', 'USD');
		
		protected $_transactionTypes = array('PAYMENT', 'DEFERRED', 'AUTHENTICATE');
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		/**
		 * Return the redirect to url
		 * 
		 * @param int $mode
		 * @return string
		 */
		protected function getProxRedirectUrl($mode)
		{
			if (!isset($this->_sagepayRedirectUrls[$mode]))
				throw new miException(_i('Unknown Sage Pay VSP mode.'));
			return $this->_sagepayRedirectUrls[$mode];
		}
		
		/**
		 * Return the payment currency
		 */
		public function getPaymentCurrencyCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('SupportedCurrencyCodes', 'DefaultCurrencyCode'));
			$supportedCurrencyCodes = explode(',', $options['SupportedCurrencyCodes']);
			
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $supportedCurrencyCodes))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			return $options['DefaultCurrencyCode'];
		}
		
		/**
		 * Replaces colons with spaces, used to escape the product name in the basket
		 * 
		 * @param string $text
		 * @return string
		 */
		protected function noColon($text)
		{
			return str_replace(':', ' ', $text);
		}
		
		/**
		 * Return common to all integration methods data fields to be sent to Sage Pay
		 * 
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $customerInfo
		 * @param array $options
		 * @return array
		 */
		protected function getCommonDataFields($payAmount, $payCurrencyCode, scCart $cart, $customerInfo, $options)
		{
			$data = array(
				'VendorTxCode' => scOrdersInProgress::singleton()->getLastOrderId(),
				'ReferrerID' => 'BCF32138-93F4-4685-A551-30B487C1CA32',
				'Amount' => number_format($payAmount, 2, '.', ''),
				'Currency' => $payCurrencyCode,
				'Description' => $options['OrderDescription'],
				
				'BillingSurname' => $customerInfo['CustomerBillingLastName'],
				'BillingFirstnames' => $customerInfo['CustomerBillingFirstName'],
				'BillingAddress1' => $customerInfo['CustomerBillingAddress1'],
				'BillingAddress2' => $customerInfo['CustomerBillingAddress2'],
				'BillingCity' => $customerInfo['CustomerBillingCity'],
				'BillingPostCode' => $customerInfo['CustomerBillingZip'],
				'BillingCountry' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'BillingPhone' => $customerInfo['CustomerBillingPhone'],
			
				'DeliverySurname' => $customerInfo['CustomerShippingLastName'],
				'DeliveryFirstnames' => $customerInfo['CustomerShippingFirstName'],
				'DeliveryAddress1' => $customerInfo['CustomerShippingAddress1'],
				'DeliveryAddress2' => $customerInfo['CustomerShippingAddress2'],
				'DeliveryCity' => $customerInfo['CustomerShippingCity'],
				'DeliveryPostCode' => $customerInfo['CustomerShippingZip'],
				'DeliveryCountry' => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']),
				'DeliveryPhone' => $customerInfo['CustomerShippingPhone'],
			);
			if ($data['BillingCountry'] == 'US')
				$data['BillingState'] = scCountry::getStateCode($customerInfo['CustomerBillingState']);
			if ($data['DeliveryCountry'] == 'US')
				$data['DeliveryState'] = scCountry::getStateCode($customerInfo['CustomerShippingState']);
			
			// Fill-in the basket
			$items = $cart->getItems();
			$currency = new scCurrency('GBP');
			$basket = '';
			foreach ($items as $item) {
				$total = $item->getQuantity() * $item->getPriceObj()->getAmountInCurrency($currency);
				$basket .= ':' . $this->noColon($item->getName()) . ':' . $item->getQuantity() . ':' . $item->getPriceObj()->getAmountInCurrency($currency) . ':::' . $total;
			}
			$data['Basket'] = count($items) . $basket;
			
			return $data;
		}
		
		/**
		 * Returns the correct payment status depending on the status sent by Sage Pay
		 * 
		 * @param $sagepayStatus
		 * @return int
		 */
		protected function getPaymentStatus($sagepayStatus)
		{
			switch ($sagepayStatus) {
				case 'OK':
					return scPaymentStatus::PAYMENT_STATUS_CAPTURE;
				
				case 'AUTHENTICATED':
				case 'REGISTERED':
					return scPaymentStatus::PAYMENT_STATUS_AUTH;
				
				case 'NOTAUTHED':
				case 'MALFORMED':
				case 'INVALID':
				case 'ABORT':
				case 'REJECTED':
				case 'ERROR':
				default:	// Declined by default
					return scPaymentStatus::PAYMENT_STATUS_DECLINED;
			}
		}
		
		/**
		 * Explodes the Sage Pay result into key => value pairs
		 * 
		 * @param string $transactionRegistrationResult
		 * @return array
		 */
		protected function getResponseValues($transactionRegistrationResult)
		{
			$responseLines = explode("\r\n", $transactionRegistrationResult);
			
			$responseFields = array();
			foreach ($responseLines as $responseLine) {
				if ($responseLine == '')
					continue;
				
				list($key, $value) = explode('=', $responseLine, 2);
				$responseFields[$key] = $value;
			}
			return $responseFields;
		}
		
		/**
		 * Set the payment status and payment note
		 * 
		 * @param scPaymentProcessResult $ppr
		 * @param array $responseFields
		 */
		protected function setPaymentInfo(scPaymentProcessResult &$ppr, $responseFields)
		{
			$ppr->setPaymentStatus($this->getPaymentStatus($responseFields['Status']));
			
			$paymentNotes = $responseFields['Status'];
			if (isset($responseFields['StatusDetail']))
				$paymentNotes .= ' (' . $responseFields['StatusDetail'] . ')';
			$ppr->setPaymentNotes($paymentNotes);
		}
		
		
		/**
		 * Sets the transaction details to the payment process result
		 * 
		 * @param object $ppr scPaymentProcessResult or scTransaction object
		 * @param array $data
		 */
		protected function setTransactionDetails(scPaymentProcessResult &$ppr, $data)
		{
			$fields = array('TxType', 'TxAuthNo', 'AVSCV2',
				'AddressResult', 'PostCodeResult', 'CV2Result', 'GiftAid', '3DSecureStatus', 'CAVV', 'AddressStatus',
				'PayerStatus', 'CardType', 'Last4Digits');
			
			foreach ($fields as $field) {
				if (isset($data[$field]))
					$ppr->setTransactionDetail($field, $data[$field]);
			}
		}
	}
?>