<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/vsp.php');
	
	/**
	 * Sage Pay VSP Form payment processing module
	 */
	class scPaymentModuleSagePayVSPDirect extends scPaymentModuleSagePayVSP implements IPaymentModule, IModuleOptions {
		
		protected $_sagepayRedirectUrls = array(
			self::VSP_SIMULATOR_MODE => 'https://test.sagepay.com/Simulator/VSPDirectGateway.asp',
			self::VSP_TEST_MODE => 'https://test.sagepay.com/gateway/service/vspdirect-register.vsp',
			self::VSP_LIVE_MODE => 'https://live.sagepay.com/gateway/service/vspdirect-register.vsp'
		);
		
		protected $_3dSecureCallbackUrl = array(
			self::VSP_SIMULATOR_MODE => 'https://test.sagepay.com/Simulator/VSPDirectCallback.asp',
			self::VSP_TEST_MODE => 'https://test.sagepay.com/gateway/service/direct3dcallback.vsp',
			self::VSP_LIVE_MODE => 'https://live.sagepay.com/gateway/service/direct3dcallback.vsp'
		);
		
		public function getPaymentContent($contentType, $params = array())
		{
			if ($contentType == self::CONTENT_TYPE_PAYMENT_FORM) {
				$expMonths = array(0 => _t('Month'), 1 => '01', 2 => '02', 3 => '03', 4 => '04', 5 => '05',
					6 => '06', 7 => '07', 8 => '08', 9 => '09', 10 => '10', 11 => '11', 12 => '12');
				
				$expYears = array(0 => _t('Year'));
				$currentYear = gmdate('Y');
				for ($i = 0; $i < scPaymentForms::NUMBER_OF_YEARS; $i++)
					$expYears[$currentYear+$i] = $currentYear+$i;
				
				$startYears = array(0 => _t('Year'));
				for ($i = 0; $i < scPaymentForms::NUMBER_OF_YEARS; $i++)
					$startYears[$currentYear-$i] = $currentYear-$i;
				
				$form = new scPaymentModuleWebForm();
				$form->initWidgets(array(
					array(
						'field' => 'miSelectWidget',
						'data' => 'CardType'
					),
					array(
						'field' => 'miTextWidget',
						'data' => 'NameOnCard'
					),
					array(
						'field' => 'miTextWidget',
						'data' => 'CardNumber'
					),
					array(
						'field' => 'miTextWidget',
						'data' => 'IssueNumber'
					),
					array(
						'field' => 'miSelectWidget',
						'data' => 'CardStartMonth'
					),
					array(
						'field' => 'miSelectWidget',
						'data' => 'CardStartYear'
					),
					array(
						'field' => 'miSelectWidget',
						'data' => 'CardExpirationMonth'
					),
					array(
						'field' => 'miSelectWidget',
						'data' => 'CardExpirationYear'
					),
					array(
						'field' => 'miBaseTextWidget',
						'data' => 'CardCVV'
					),
				));
				
				$form->getWidget('CardExpirationMonth')->setOptions($expMonths);
				$form->getWidget('CardExpirationYear')->setOptions($expYears);
				$form->getWidget('CardStartMonth')->setOptions($expMonths);
				$form->getWidget('CardStartYear')->setOptions($startYears);
				
				$cardTypes = array(
					'VISA' => 'VISA Credit' ,
					'DELTA' => 'VISA Debit',
					'UKE' => 'VISA Electron',
					'MC' => 'MasterCard',
					'MAESTRO' => 'Maestro',
					'AMEX' => 'American Express',
					'DC' => 'Diners Club',
					'JCB' => 'JCB Card',
					'LASER' => 'Laser',
					'SOLO' => 'Solo',
				);
				$form->getWidget('CardType')->setOptions($cardTypes);
				
				return $form->parse('modules/payment/sagepay/vsp_direct_payment_form.tmpl', true);
			}
		}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		protected function validateForm(scPaymentProcessResult $ppr, $paramsPost)
		{
			if (empty($paramsPost['NameOnCard']))
				$ppr->addValidatorError(_t('Please enter the name on card.'), 'NameOnCard');
			
			if (!scPaymentForms::isCardNumberValid($paramsPost['CardNumber']))
				$ppr->addValidatorError(_t('Invalid card number.'), 'CardNumber');
			
			if ($paramsPost['CardStartMonth'] or $paramsPost['CardStartYear']) {
				if (!scPaymentForms::isCardStartValid($paramsPost['CardStartYear'], $paramsPost['CardStartMonth']))
					$ppr->addValidatorError(_t('Invalid start date.'), 'CardStartYear');
			}
				
			if (!scPaymentForms::isCardExpirationValid($paramsPost['CardExpirationYear'], $paramsPost['CardExpirationMonth']))
				$ppr->addValidatorError(_t('Invalid expiration date.'), 'CardExpirationYear');
			
			if (!ctype_digit($paramsPost['CardCVV']) and strlen($paramsPost['CardCVV']))
				$ppr->addValidatorError(_t('Invalid CVV code.'), 'CardCVV');
			
			if (!ctype_digit($paramsPost['IssueNumber']) and strlen($paramsPost['IssueNumber']))
				$ppr->addValidatorError(_t('Invalid issue number.'), 'IssueNumber');
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Vendor', 'TransactionType', 'Mode', 'OrderDescription'));
			
			$customerInfo = $cart->getCustomerInfo();
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$result = new scPaymentProcessResult($orderId);
			
			$cardNumber = str_replace(array(' ', '-'), '', $paramsPost['CardNumber']);
			
			// Validate the card info
			$this->validateForm($result, $paramsPost);
			if ($result->hasValidatorErrors())
				return $result;
			
			// Card Start is optional
			if ($paramsPost['CardStartMonth'] or $paramsPost['CardStartYear']) {
				$cardStart = sprintf('%02d%02d', $paramsPost['CardStartMonth'], substr($paramsPost['CardStartYear'], 2));
			} else {
				$cardStart = '';
			}
			
			// These are the VSP Direct specific fields
			$data = array(
				'VPSProtocol' => '2.23',
				'TxType' => $options['TransactionType'],
				'Vendor' => $options['Vendor'],
				
				'CardHolder' => $paramsPost['NameOnCard'],
				'CardNumber' => $cardNumber,
				'StartDate' => $cardStart,
				'ExpiryDate' => sprintf('%02d%02d', $paramsPost['CardExpirationMonth'], substr($paramsPost['CardExpirationYear'], 2)),
				'IssueNumber' => $paramsPost['IssueNumber'],
				'CV2' => $paramsPost['CardCVV'],
				'CardType' => $paramsPost['CardType'],
				
				'CustomerEMail' => $customerInfo['CustomerEmail'],
				'ClientIPAddress' => $_SERVER['REMOTE_ADDR']
			);
			
			// Add the common fields
			$data += $this->getCommonDataFields($payAmount, $payCurrencyCode, $cart, $customerInfo, $options);
			
//			var_dump($paramsPost);
//			var_dump($data);die;
			
			// Append the transaction id number to the order id so that it gets unique TxCode every time
			$transaction = new scTransaction($orderId, $this->_moduleId,
				$payAmount, $payCurrencyCode, $data['VendorTxCode']);
			$transactionId = $transaction->insertTransaction();
			$data['VendorTxCode'] = $orderId . '-' . $transactionId;
			
			$transactionRegistrationResult = scHttp::post($this->getProxRedirectUrl($options['Mode']), http_build_query($data));
			
			$responseFields = $this->getResponseValues($transactionRegistrationResult);
			
			// If we receive an error display it to user, and log it
			if (in_array($responseFields['Status'], array('MALFORMED', 'INVALID', 'ERROR'))) {
				$msg = sprintf(_t('Error status from Sage Pay: %s (%s).'), $responseFields['Status'], $responseFields['StatusDetail']);
				scLog::singleton()->warning($msg, scLog::CATEGORY_PAYMENT);
				$result->addValidatorError($msg, '');
				return $result;
			}
			
			// Check if it is a 3DAUTH response
			if ($responseFields['Status'] == '3DAUTH') {
				// Save the MD
				$transaction->setTransactionDetail('MD', $responseFields['MD']);
				$transaction->insertTransactionDetails($transactionId);
				
				return $this->redirectTo3dAuth($orderId, $responseFields);
			}
			
			// Save the transaction code
			$transaction->setTransactionCode($responseFields['VPSTxId']);
			$transaction->updateTransaction();
			
			$result->setTransactionCode($responseFields['VPSTxId']);
			
			// Save the security key as $responseFields detail
			$this->setTransactionDetails($result, $responseFields);
			$result->setTransactionDetail('SecurityKey', $responseFields['SecurityKey']);
			
			$this->setPaymentInfo($result, $responseFields);
			
			return $result;
		}
		
		protected function redirectTo3dAuth($orderId, $responseFields)
		{
			// TODO: Store in the transaction that we are going for 3DAUTH
			$result = new scPaymentProcessResult($orderId);
			$result->setThirdPartyRedirect($responseFields['ACSURL'],
				array(
					'MD' => $responseFields['MD'],
					'PaReq' => $responseFields['PAReq'],
					'TermUrl' => scPaths::getSecureUrl('modules/payment/sagepay/vsp_direct_3dauth.php')));
			return $result;
		}
		
		/**
		 * Handles 3D AUTH return
		 *
		 * @param array $params
		 */
		function process3dAuth($params)
		{
			scPaymentProcess::checkParamsExist($params, array('MD', 'PaRes'));
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode'));
			
			$callbackUrl = $this->_3dSecureCallbackUrl[$options['Mode']];
			
			$data = array('MD' => $params['MD'], 'PARes' => $params['PaRes']);
			$transaction3dAuthResult = scHttp::post($callbackUrl, http_build_query($data));
			$responseFields = $this->getResponseValues($transaction3dAuthResult);
			
//			scDebug::logVariable('/tmp/sagepay.log', '$responseFields', $responseFields);
			
			$orderId = $this->getOrderIdByMD($params['MD']);
			$result = new scPaymentProcessResult($orderId, 0, '', $responseFields['VPSTxId']);
			$this->setPaymentInfo($result, $responseFields);
			$this->setTransactionDetails($result, $responseFields);
			
//			scDebug::logVariable('/tmp/sagepay.log', '$result', $result);
			
			return $result;
		}
		
		protected function getOrderIdByMD($md)
		{
			$sql = 'SELECT OrderID FROM Transactions JOIN TransactionDetails USING(TransactionID) WHERE ModuleID=? AND TransactionDetailName="MD" AND TransactionDetailValue=?';
			$rows = miStaticDBUtil::execSelect($sql, array($this->_moduleId, $md));
			if (count($rows) == 0)
				throw new scPaymentProcessException(sprintf(_i('Sage Pay VSP Direct: Cannot find order for MD "%s".'), $md));
			
			return $rows[0]['OrderID'];
		}
		
		/**
		 * Returns the security key obtained during transaction registration
		 * 
		 * @param int $transactionId
		 * @return string
		 */
		protected function getSecurityKey($transactionId)
		{
			$sql = 'SELECT TransactionDetailValue FROM TransactionDetails WHERE TransactionID=? AND TransactionDetailName=?';
			$rows = miStaticDBUtil::execSelect($sql, array($transactionId, 'SecurityKey'));
			if (count($rows) == 0)
				throw new scPaymentProcessException(sprintf(_i('Sage Pay SecurityKey for transaction %s not found.'), $transactionId));
			
			return $rows[0]['TransactionDetailValue'];
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'Vendor',
					'properties' => array('maxLength' => 15)
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'TransactionType',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'OrderDescription',
					'properties' => array('maxLength' => 100),
				),
				array(
					'field' => 'scModuleOptionMultiCheckboxWidget',
					'data' => 'SupportedCurrencyCodes',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('TransactionType')->setOptions(array_combine($this->_transactionTypes, $this->_transactionTypes));
			
			$modes = array(self::VSP_SIMULATOR_MODE => _t('Simulator'), self::VSP_TEST_MODE => _t('Test'), self::VSP_LIVE_MODE => _t('Live'));
			$options->getOptionField('Mode')->setRadioButtons($modes, self::VSP_TEST_MODE);
			
			$options->getOptionField('SupportedCurrencyCodes')->setRadioButtons(array_combine($this->_sagepayCurrencies, $this->_sagepayCurrencies), self::DEFAULT_CURRENCY_CODE);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_sagepayCurrencies, $this->_sagepayCurrencies), self::DEFAULT_CURRENCY_CODE);
			
			$options->setTemplateFile('modules/payment/sagepay/vsp_direct_options.tmpl');
			return $options;
		}
	}
?>