<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/../borica/cp1251.php');	// FIXME

	/**
	 * Transcard payment processing module
	 */
	class scPaymentModuleTranscardBg implements IPaymentModule, IModuleOptions {
		const TRANSACRD_URL = 'https://www.transcard.bg/pay-gw/pay.php';
		const TRANSCARD_TEST_URL = 'https://www-t.transcard.bg/pay-gw/pay.php';
		
		const MODE_LIVE = 'Live';
		const MODE_TEST = 'Test';
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			return 'BGN';
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'OrderDescription', 'TestPrivateKey', 'LivePrivateKey'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$params = array(
				'x_Type' => 'AUTH_CAPTURE',
				'x_Login' => $options['MerchantID'],
				'x_Invoice_Num' => scOrdersInProgress::singleton()->getLastOrderId(),
				'x_Amount' => number_format($payAmount, 2, '.', ''),
				'x_Receipt_Link_URL' => scPaths::getSecureUrl('modules/payment/transcard_bg/return.php'),
				
				'x_Test_Request' => ($options['Mode'] != self::MODE_LIVE) ? 1 : 0,
				'x_Currency_Code' => scCp1251::utf8ToCp1251('лв.'),
				'x_Description' => scCp1251::utf8ToCp1251($options['OrderDescription']),
				'x_Receipt_Link_Method' => 'GET',
				
				'x_Version' => '1.0',
			);
			
			$privateKey = ($options['Mode'] == self::MODE_TEST) ? $options['TestPrivateKey'] : $options['LivePrivateKey'];
			
			$hash = $this->calculateHash($privateKey, $params['x_Login'], $params['x_Type'], $params['x_Invoice_Num'], $params['x_Amount']);
			$params['x_FP_Hash'] = $hash;
			
			$url = ($options['Mode'] == self::MODE_TEST) ? self::TRANSCARD_TEST_URL : self::TRANSACRD_URL;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($url, $params, 'get');
			return $result;
		}
		
		protected function calculateHash($privateKey, $loginId, $type, $invoiceNum, $amount)
		{
			// Can optionaly sign TransID
			
			$data = 'x_Login=' . $loginId . "\n";
			$data .= 'x_Type=' . $type . "\n";
			$data .= 'x_Invoice_Num=' . $invoiceNum . "\n";
			$data .= 'x_Ammount=' . $amount . "\n";
			
			// Sign the data with the private key
			$pkeyid = openssl_pkey_get_private($privateKey);
			openssl_sign($data, $signature, $pkeyid);
			openssl_free_key($pkeyid);
			
			return base64_encode($data) . '#' . base64_encode($signature);
		}
		
		/**
		 * Verifies the signature
		 * 
		 * @param string $data
		 * @param string $signature
		 * @param string $publicKey
		 * @return boolean
		 */
		protected function verifySignature($data, $signature, $publicKey)
		{
			$pubkeyid = openssl_pkey_get_public($publicKey);
			$ok = openssl_verify($data, $signature, $pubkeyid);
			openssl_free_key($pubkeyid);
			
			return ($ok == 1) ? true : false;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			if (strpos($params['x_FP_Hash'], '#') === false)
				throw new scPaymentProcessException(_i('Received invalid hash.'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'TestTranscardCertificate', 'LiveTranscardCertificate'));
			$publicKey = ($options['Mode'] == self::MODE_TEST) ? $options['TestTranscardCertificate'] : $options['LiveTranscardCertificate'];
			
			list($signedData, $signature) = explode('#', $params['x_FP_Hash']);
			$signedData = base64_decode($signedData);
			$signature = base64_decode($signature);
			$ok = $this->verifySignature($signedData, $signature, $publicKey);
			
			if (!$ok)
				throw new scPaymentProcessException(_i('Cannot verify signature.'));
			
			$lines = explode("\n", $signedData);
			$data = array();
			foreach ($lines as $line) {
				$line = trim($line);
				if ($line == '')
					continue;
				
				$parts = explode('=', $line, 2);
				if (count($parts) != 2)
					throw new scPaymentProcessException(sprintf(_i('Invalid signed data line "%s"'), $line));
				$data[$parts[0]] = $parts[1];
			}
			
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($data['x_Invoice_Num'], $data['x_Ammount'], 'BGN');
			
//			var_dump($params);
//			var_dump($data);
//			die;
			
			
			$result = new scPaymentProcessResult($data['x_Invoice_Num'], $data['x_Ammount'], 'BGN', $data['x_Trans_ID']);
			if ($data['x_Response_Code'] == 1)	// Success
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			if ($data['x_Response_Code'] == 2) { // User cancel
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes(_t('Payment cancelled by customer.'));
			}
			if ($data['x_Response_Code'] == 3)	// Decline
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			if ($data['x_Response_Code'] == 4) {	// Error
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes(_i('Transcard error.'));
			}
			if ($data['x_Response_Code'] == 5) {	// Offline
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes(_i('Transcard offline.'));
			}
			if ($data['x_Response_Code'] == 6) {	// Invalid parameter
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes(_i('Invalid parameter.'));
			}
			
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'MerchantID',
					'properties' => array('minLength' => 9, 'maxLength' => 9)
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'OrderDescription',
					'properties' => array('maxLength' => 125),
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'TestPrivateKey',
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'TestTranscardCertificate',
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'LivePrivateKey',
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'LiveTranscardCertificate',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$options->getOptionField('Mode')->setRadioButtons(array(self::MODE_LIVE => _t('Live'), self::MODE_TEST => _t('Test')), self::MODE_TEST);
			$options->setTemplateFile('modules/payment/transcard_bg/options.tmpl');
			return $options;
		}
	}
?>