<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/base.php');
	
	/**
	 * EST.com.tr 3D Pay Hosting (OOS) payment processing module
	 * Works with all turkish banks utilizing EST
	 */
	class scPaymentModuleEst3dPayHosting extends scPaymentModuleTurkishBanksBase implements IPaymentModule, IModuleOptions {
		const GARANTI_PP_URL = 'https://ccpos.garanti.com.tr/servlet/garpp';
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			var_dump($customerInfo);
//			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('ClientID', 'Bank', 'OrderDescription', 'CurrencyCode', 'StoreKey'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			$options = array();
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = ($isoLangCode == 'tr') ? 'tr' : 'eng';
			
			$params = array(
				'clientid' => '100102', // FIXME
				'name' => '100102pay', // FIXME
				'password' => '100102pass', // FIXME
				
				'tutar' => $payAmount,
				'parabirimi' => '840', // FIXME
				'islemtipi' => 'Auth', // FIXME
				
				'donadr' => scPaths::getSecureUrl('modules/payment/turkish_banks/return_3dpayhosting.php'),
				'taksit' => '',
				
				'action' => 'payform',
				'test' => 'Test', // FIXME
				'refr' => '2',	// refresh time in seconds
				'dil' => $lang,
				'firmaadi' => 'Den1eme Firm yönetim', // FIXME: convert from UTF-8 to ISO-8859-9
//				'firmlogoadr' => '',
				
				'email' => $customerInfo['CustomerEmail'],
				'siparisno' => scOrdersInProgress::singleton()->getLastOrderId(),
				
				'Ffirmaadi' => $customerInfo['CustomerBillingCompany'],
				'Fismi'  => $customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName'],
				'Fadres' => $customerInfo['CustomerBillingAddress1'],
				'Fadres2' => $customerInfo['CustomerBillingAddress2'],
				'Fil' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'Filce' => $customerInfo['CustomerBillingCity'],
				'Fpostakodu' => $customerInfo['CustomerBillingZip'],
				'tel' => $customerInfo['CustomerBillingPhone'],
//				'Fulkekod' => '792', // FIXME
				
				// Shipping
//				'NakliyeFirma' => $customerInfo['CustomerShippingCompany'],
				'tismi'  => $customerInfo['CustomerShippingFirstName'] . ' ' . $customerInfo['CustomerShippingLastName'],
				'tadres' => $customerInfo['CustomerShippingAddress1'],
				'tadres2' => $customerInfo['CustomerShippingAddress2'],
				'til' => scCountry::getStateCode($customerInfo['CustomerShippingState']),
				'tilce' => $customerInfo['CustomerShippingCity'],
				'tpostakodu' => $customerInfo['CustomerShippingZip'],
//				'tulkekod' => '792', // FIXME
			);
			// Pass the userid only if the user is not the guest user
			if ($customerInfo['CustomerID'] != ICheckoutModule::GUEST_USER_ID)
				$params['kullanici'] = $customerInfo['CustomerID'];
			
//			var_dump($params);die;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(self::GARANTI_PP_URL, $params);
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			var_dump($params);die;
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('ClientID', 'Bank', 'CurrencyCode', 'StoreKey'));
			
			if (!$this->verifyHash($params, $options['StoreKey']))
				throw new scPaymentProcessException(_t('Hash signature is incorrect. Please check your store key.'));
			
			scPaymentProcess::checkPaymentToOrder($params['oid'], $params['amount'], $params['CurrencyCode']);
				
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ClientID',
					'properties' => array('minLength' => 1),
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'Bank',
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'OrderDescription',
					'properties' => array('minLength' => 1, 'maxLength' => 255),
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'CurrencyCode',
//					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'StoreKey',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('CurrencyCode')->setOptions(array_combine($this->_supportedCurrencies, $this->_supportedCurrencies));
//			$options->getOptionField('Mode')->setRadioButtons(array('Live' => _t('Live'), 'Simulator' => _t('Simulator')), 'Live');
			$options->setTemplateFile('modules/payment/turkish_banks/options.tmpl');
			return $options;
		}
	}
?>