<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/base.php');
	
	class scPaymentModuleTurkishBanksSetup implements IAdminModule {
		
		public function __construct()
		{
			$this->setupAutoloading();
		}
		
		/**
		 * Initializes the action handlers for this module
		 * 
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('acEditBank', 'scPaymentModuleTurkishBanksSetupEditBankAction');
			$page->getDispatcher()->setActionHandler('acExecEditBank', 'scPaymentModuleTurkishBanksSetupExecEditBankAction');
			$page->getDispatcher()->setActionHandler('acChangeIsActive', 'scPaymentModuleTurkishBanksSetupChangeIsActiveAction');
		}
		
		/**
		 * Returns the view that should be displayed
		 * 
		 * @return miDefaultView
		 */
		public function getView()
		{
			$bankCode = self::getBankCode();
			
			if (in_array($bankCode, scPaymentModuleTurkishBanksBase::$_banksEst3d)) {
				$view = new scPaymentModuleTurkishBanksEst3dSetupView();
				$view->setRecord(new scPaymentModuleTurkishBanksEst3dModuleOptionsRecord());
			}
			elseif (in_array($bankCode, scPaymentModuleTurkishBanksBase::$_banksPosnet)) {
				$view = new scPaymentModuleTurkishBanksPosnetSetupView();
				$view->setRecord(new scPaymentModuleTurkishBanksPosnetModuleOptionsRecord());
			}
			elseif ($bankCode == scPaymentModuleTurkishBanksBase::BANK_CODE_GARANTI) {
				$view = new scPaymentModuleTurkishBanksGarantiSetupView();
				$view->setRecord(new scPaymentModuleTurkishBanksGarantiModuleOptionsRecord());
			}
			else {
				throw new miException(sprintf(_i('Unknown bank code %s.'), $bankCode));
			}
			
			return $view;
		}
		
		public static function getBankCode()
		{
			$bankCode = miGetParam('BankCode');
			
			// validate the bank code
			$bankNames = scPaymentModuleTurkishBanksBase::getBankNames();
			if (!isset($bankNames[$bankCode]))
				throw new miException(sprintf(_i('Unknown bank code %s.'), $bankCode));
			
			return $bankCode;
		}
		
		public static function redirectToEdit($view, $msg = '', $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = scPaths::getCurrentScriptPathName() . '?ModuleName=com.summercart.turkishbanks.virtualpos&action=acEditBank&BankCode=' . miGetParam('BankCode');
			$loc = scUrlUtil::convertUrl($loc, scUrlUtil::URL_TYPE_DOCROOT);
			$loc = scUrlUtil::addQueryParams($loc, $view->getRedirectParams($msg, $msgType));
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$view->addControllerCommand($controllerCommand);
		}
		
		public static function redirectToModuleOptions($view, $msg = '', $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.turkishbanks.virtualpos');
			$paymentModuleId = scUtil::getDbValue('PaymentModules', 'PaymentModuleID', 'ModuleID', $moduleId, 0);
			
			$loc = 'payment_modules.php?action=dmEdit&PaymentModuleID=' . $paymentModuleId;
			$loc .= '&' . $view->getRedirectParams($msg, $msgType);
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$view->addControllerCommand($controllerCommand);
		}
		
		protected function setupAutoloading()
		{
			$autoloadClasses = array(
				// View
				'scPaymentModuleTurkishBanksEst3dSetupView' => 'banks_setup_est3d.php',
				'scPaymentModuleTurkishBanksGarantiSetupView' => 'banks_setup_garanti.php',
				'scPaymentModuleTurkishBanksPosnetSetupView' => 'banks_setup_posnet.php',
				
				// Model
				'scPaymentModuleTurkishBanksEst3dModuleOptionsRecord' => 'banks_setup_est3d.php',
				'scPaymentModuleTurkishBanksGarantiModuleOptionsRecord' => 'banks_setup_garanti.php',
				'scPaymentModuleTurkishBanksPosnetModuleOptionsRecord' => 'banks_setup_posnet.php',
			);
			
			foreach ($autoloadClasses as $className => $fileName) {
				// Relative from the location of miLoader
				miLoader::addClass($className, '../../modules/payment/turkish_banks/' . $fileName);
			}
		}
	}
	
	class scPaymentModuleTurkishBanksSetupEditBankAction extends miEditAction {
		
		protected function onFailure(miException $exception = null)
		{
			$msg = '';
			if (!is_null($exception)) {
				$msg .= $exception->getMessage();
			}
			
			scPaymentModuleTurkishBanksSetup::redirectToModuleOptions($this->_view, $msg, miMessage::MSG_TYPE_ERROR);
			return false;
		}
	}
	
	class scPaymentModuleTurkishBanksSetupExecEditBankAction extends miExecEditAction {
		protected function onFailure(miException $exception = null)
		{
			$msg = '';
			if (!is_null($exception)) {
				$msg .= $exception->getMessage();
			}
			
			scPaymentModuleTurkishBanksSetup::redirectToEdit($this->_view, $msg, miMessage::MSG_TYPE_ERROR);
			return false;
		}
		
		protected function onSuccess($viewMapperControllerResult = null)
		{
			scPaymentModuleTurkishBanksSetup::redirectToModuleOptions($this->_view, _t('Bank options updated successfully.'), miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
	
	class scPaymentModuleTurkishBanksSetupChangeIsActiveAction extends miAction {
		public function doAction()
		{
			try {
				$bankCode = scPaymentModuleTurkishBanksSetup::getBankCode();
				$moduleId = scModulesFactory::getModuleIdByName('com.summercart.turkishbanks.virtualpos');
				
				$isActive = scModuleOptions::getModuleOption($moduleId, $bankCode . '_Active');
				if (!$isActive) {
					
					$isConfigured = true;
					if (in_array($bankCode, scPaymentModuleTurkishBanksBase::$_banksEst3d)) {
						$clientId = scModuleOptions::getModuleOption($moduleId, $bankCode . '_ClientID');
						if (empty($clientId)) {
							$isConfigured = false;
						}
					} elseif (in_array($bankCode, scPaymentModuleTurkishBanksBase::$_banksPosnet)) {
						$options = scModuleOptions::getModuleOptions($moduleId, array($bankCode . '_MerchantID', $bankCode . '_TerminalID', $bankCode . '_PosnetID'));
						if ((empty($options[$bankCode . '_MerchantID'])) || (empty($options[$bankCode . '_TerminalID'])) || (empty($options[$bankCode . '_PosnetID']))) {
							$isConfigured = false;
						}
					} elseif ($bankCode == scPaymentModuleTurkishBanksBase::BANK_CODE_GARANTI) {
						$options = scModuleOptions::getModuleOptions($moduleId, array(
							$bankCode . '_ProvisionUserID', $bankCode . '_ProvisionPassword', $bankCode . '_UserID', $bankCode . '_TerminalID',
							$bankCode . '_MerchantID', $bankCode . '_3DSecureMode', $bankCode . '_3DSecurePassword'
						));
						if ((empty($options[$bankCode . '_ProvisionUserID'])) || (empty($options[$bankCode . '_ProvisionPassword'])) || (empty($options[$bankCode . '_UserID'])) ||
							(empty($options[$bankCode . '_TerminalID'])) || (empty($options[$bankCode . '_MerchantID'])) || (empty($options[$bankCode . '_3DSecureMode'])) ||
							(empty($options[$bankCode . '_3DSecurePassword']))) {
							$isConfigured = false;
						}
					}
					
					if (!$isConfigured) {
						throw new miException(_t('Please configure the bank before activating it.'));
					}
				}
				scModuleOptions::setModuleOption($moduleId, $bankCode . '_Active', !$isActive);
				
				$msg = $isActive ? _t('Bank deactivated successfully.') : _t('Bank activated successfully.');
				scPaymentModuleTurkishBanksSetup::redirectToModuleOptions($this->_view, $msg, miMessage::MSG_TYPE_INFO);
			} catch (Exception $exception) {
				scPaymentModuleTurkishBanksSetup::redirectToModuleOptions($this->_view, $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
			}
		}
	}
	
	class scPaymentModuleTurkishBanksModuleOptionsRecord extends miSqlRecord {
		
		protected $_optionNames;
		
		public function __construct()
		{
			if (empty($this->_optionNames)) {
				throw new miException(_t('Bank options not specified.'));
			}
		}
		
		public function readPK($value)
		{
			// Set default values
			foreach ($this->_optionNames as $optionName)
				$this->_row[$optionName] = '';
				
			$bankCode = scPaymentModuleTurkishBanksSetup::getBankCode();
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.turkishbanks.virtualpos');
			$moduleOptions = scModuleOptions::getModuleOptions($moduleId);
			
			foreach ($moduleOptions as $moduleOptionName => $moduleOptionValue) {
				if (substr_compare($bankCode . '_', $moduleOptionName, 0, strlen($bankCode) + 1) === 0) {
					$this->_row[substr($moduleOptionName, strlen($bankCode) + 1)] = $moduleOptionValue;
				}
			}
		}
		
		public function update()
		{
			$bankCode = scPaymentModuleTurkishBanksSetup::getBankCode();
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.turkishbanks.virtualpos');
			
			foreach ($this->_row as $moduleOptionName => $moduleOptionValue)
				scModuleOptions::setModuleOption($moduleId, $bankCode . '_' . $moduleOptionName, $moduleOptionValue);
		}
	}
	
	class scPaymentModuleTurkishBanksSetupView extends scView {
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$widget = $form->getWidget('Mode');
			$widget->setRadioButtons(array(scPaymentModuleTurkishBanksBase::MODE_LIVE => _t('Live'), scPaymentModuleTurkishBanksBase::MODE_TEST => _t('Test')), scPaymentModuleTurkishBanksBase::MODE_TEST);
			
			$bankCode = scPaymentModuleTurkishBanksSetup::getBankCode();
			$bankNames = scPaymentModuleTurkishBanksBase::getBankNames();
			$form->addMainPageElements(array('%%BANKCODE%%' => miI18N::htmlEscape($bankCode)));
			$form->addMainPageElements(array('%%BANKNAME%%' => miI18N::htmlEscape($bankNames[$bankCode])));
		}	
	}
	
	class scPaymentModuleTurkishBanksInstallmentsViewMapper extends scViewMapperDynRows {
		/**
		 * Read the installments from [BankCode]_Installments module option
		 * @see saveDynRows for the formay
		 */
		protected function readRecords($rsProperties)
		{
			$bankCode = scPaymentModuleTurkishBanksSetup::getBankCode();
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.turkishbanks.virtualpos');
			$installmentsString = scModuleOptions::getModuleOption($moduleId, $bankCode . '_Installments');
			
			if (!$installmentsString)
				return array();
			
			$installments = explode(',', $installmentsString);
			
			$result = array();
			foreach ($installments as $installment) {
				list($numberOfPayments, $insterestRate) = explode(':', $installment);
				$result[] =	array(
					'InstallmentNumberOfPayments' => $numberOfPayments,
					'InstallmentInterestRate' => $insterestRate,
				);
			}
			return $result;
		}
		
		/**
		 * Save the installments into [BankCode]_Installments module option
		 * The format is Installments:InterestRate,Installments:InterestRate,Installments:InterestRate. Example 1:0,2:1.3,3:2.8
		 */
		protected function saveDynRows($rsValues, $rsProperties)
		{
			$unsortedInstallments = array();
			foreach ($rsValues as $rsValue) {
				$unsortedInstallments[intval($rsValue['InstallmentNumberOfPayments'])] = floatval($rsValue['InstallmentInterestRate']);
			}
			ksort($unsortedInstallments);
			
			$installments = array();
			foreach ($unsortedInstallments as $numberOfPayments => $interestRate) {
				$installments[] = $numberOfPayments . ':' . $interestRate;
			}
			
			$bankCode = scPaymentModuleTurkishBanksSetup::getBankCode();
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.turkishbanks.virtualpos');
			$installments = scModuleOptions::setModuleOption($moduleId, $bankCode . '_Installments', implode(',', $installments));
		}
	}
	
	class scPaymentModuleTurkishBanksInstallmentsWidget extends scDynRowsWidget {
		protected function validateDynRowsProperties($properties)
		{
			foreach (array('template', 'fields') as $propertyName) {
				if (!isset($properties[$propertyName])) {
					throw new miException('You must specify a "'.$propertyName.'" property for a repeating section!');
				}
			}
			
			$this->_template = $properties['template'];
			$this->_rsDataFields = $properties['fields'];
		}
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			$data = $this->getData();
			$data = reset($data);
			
			if (count($data) == 0)
				return;
			
			$installments = array();
			foreach ($data as $rowNum => $dataRow) {
				if (!ctype_digit($dataRow['InstallmentNumberOfPayments']))
					$errors->addError('InstallmentNumberOfPayments' . $this->getSuffix($rowNum+1), _t('Invalid number of installments.'));
				if (!preg_match('#^[0-9]{1,10}\.?([0-9]){0,2}$#', $dataRow['InstallmentInterestRate']))
					$errors->addError('InstallmentInterestRate' . $this->getSuffix($rowNum+1), _t('Invalid interest rate.'));
				
				if (in_array($dataRow['InstallmentNumberOfPayments'], $installments))
					$errors->addError('InstallmentNumberOfPayments' . $this->getSuffix($rowNum+1), _t('Dublicated number of installments.'));
				else
					$installments[] = $dataRow['InstallmentNumberOfPayments'];
			}
		}
	}
?>