<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/turkish_banks.php');
	
	class scTurkishBanksIntallmentsProductsDetails implements IProductDetails {
		
		public function getProductDetailsHtml(scProductSL $product, miTemplateParserSectionInfo $productSection)
		{
			try {
				$moduleId = scUtil::getDbValue('ModuleClasses', 'ModuleID', 'ModuleClassName', 'scPaymentModuleTurkishBanks');
				
				$paymentModuleIsActive = scUtil::getDbValue('PaymentModules', 'PaymentModuleIsActive', 'ModuleID', $moduleId, false);
				if (!$paymentModuleIsActive) {
					$productSection->addSubsection(new miTemplateParserSectionInfo('installmentsAreAvailable', 0));
					return '';
				}
				
				$base = new scPaymentModuleTurkishBanks($moduleId);
				
				$amount = $product->getPrice()->getAmount();
				try {
					if (class_exists('scDiscountCalculator', false))
						$discountCalculator = new scDiscountCalculator(scDiscountCalculator::MODE_FEED);
				} catch (miException $e) {
				}
				if (isset($discountCalculator)) {
					$discountAmount = $discountCalculator->getProductDiscount($product);
					if ($discountAmount->getAmount()->get() != 0) {
						$amount = scMultiCurrencyAmountMath::add($amount, $discountAmount->getAmount());
					}
				}
				
				$productTaxedPrice = scProductPrice::getTaxedAmount($amount, $product->getPK());
				$installments = $base->getInstallments($productTaxedPrice);
				if (empty($installments)) {
					$productSection->addSubsection(new miTemplateParserSectionInfo('installmentsAreAvailable', 0));
					return '';
				}
				
				// Build the header banks section
				$availableBanks = array();
				$installmentValues = array();
				$installmentsByBank = array();
				foreach ($installments as $bankName => $bankInstallments) {
					$bank = array('Name' => $bankName);
					foreach ($bankInstallments as $bankInstallment) {
						$installmentValues[] = $bankInstallment['NumberOfPayments'];
						
						$bank['Logo'] = $bankInstallment['Logo'];
						
						$installmentsByBank[$bankName][$bankInstallment['NumberOfPayments']] = array(
							'Installments' => array(
								'InstallmentPaymentPrice' => $bankInstallment['InstallmentPaymentPrice'],
								'TotalPrice' => $bankInstallment['TotalPrice'],
							),
						);
					}
					
					$availableBanks[] = $bank;
				}
				$banksSection = new miTemplateParserSectionInfo('banks', count($availableBanks), miUIUtil::templetizeAndEscapeRows(miUIUtil::transposeRows($availableBanks)));
				
				// Build the installments section
				$installmentValues = array_unique($installmentValues);
				sort($installmentValues);
				
				$availableInstallments = array();
				foreach ($installmentValues as $value) {
					$availableInstallments[]['Installments'] = $value;
				}
				$installmentsSection = new miTemplateParserSectionInfo('installments', count($availableInstallments), miUIUtil::templetizeRows(miUIUtil::transposeRows($availableInstallments)));
				
				// Build the installments banks subsections
				$lowestInstallmentPayment = PHP_INT_MAX;
				$installmentsByBankSections = array();
				foreach ($installmentValues as $value) {
					
					// Init the row values
					$bankInstallmentValues = array();
					foreach ($installmentsByBank as $bankName => $bankInstallments) {
						$values = array(
							'InstallmentPaymentPrice' => '-',
							'TotalPrice' => '-',
						);
						
						if (array_key_exists($value, $bankInstallments) !== false) {
							$values = $bankInstallments[$value]['Installments'];
						}
						
						$bankInstallmentValues[] = $values;
					}
					
					// Determine the lowest installment payment for the row
					$lowestRowValue = PHP_INT_MAX;
					foreach ($bankInstallmentValues as $key => $values) {
						if (!is_numeric($values['InstallmentPaymentPrice'])) {
							continue;
						}
						
						if ($values['InstallmentPaymentPrice'] < $lowestRowValue) {
							$lowestRowValue = $values['InstallmentPaymentPrice'];
						}
					}
					
					// Assign the appropriate CSS class for the values (lowest) that need to be highlighted on the row
					foreach ($bankInstallmentValues as $key => &$values) {
						$values['Class'] = '';
						if ($values['InstallmentPaymentPrice'] == $lowestRowValue) {
							$values['Class'] = 'lowest';
						}
					}
					
					// Determine the lowest installment of all
					if ($lowestRowValue < $lowestInstallmentPayment) {
						$lowestInstallmentPayment = $lowestRowValue;
					}
					
					$rowBanksSection = new miTemplateParserSectionInfo('installmentsByBank', count($bankInstallmentValues), miUIUtil::templetizeRows(miUIUtil::transposeRows($bankInstallmentValues)));
					$installmentsByBankSections[] = $rowBanksSection;
				}
				
				$installmentsSection->addSubsectionsArray('installmentsByBank', $installmentsByBankSections);
				
			} catch (miException $exception) {
				$productSection->addSubsection(new miTemplateParserSectionInfo('installmentsAreAvailable', 0));
				return '';
			}
			
			$productSection->addSubsection(new miTemplateParserSectionInfo('installmentsAreAvailable', 1));
			
			$lowestInstallmentSeciton = new miTemplateParserSectionInfo('showLowestInstallment', 0);
			if ($lowestInstallmentPayment != PHP_INT_MAX) {
				$lowestInstallmentSeciton->setIterationsCount(1);
				
				$lowestAmount = scMultiCurrencyAmount::factory();
				$lowestAmount->set(scCurrencyUtil::getCurrentCurrency(), $lowestInstallmentPayment);
				$lowestInstallmentPayment = scPrice::factory($lowestAmount)->getDisplayPrice();
			}
			
			$t = new scTemplateParser();
			$t->readTemplate('modules/payment/turkish_banks/product_details.tmpl');
			$t->setSectionInfos(array($banksSection, $installmentsSection));
			$t->assignArray(array(
				'%%CURRENCY%%' => scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(),
				'%%LOWEST_INSTALLMENT_PAYMENT%%' => $lowestInstallmentPayment
			));
			return $t->templateParse();
		}
	}
?>