<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/base.php');
	
	class scPaymentModuleTurkishBanks extends scPaymentModuleTurkishBanksBase implements IPaymentModule, IModuleOptions {
		
		const NUMBER_OF_YEARS = 15;
		
		public function __construct($moduleId)
		{
			parent::__construct($moduleId);
			$this->setupAutoloading();
		}
		
		public function getPaymentContent($contentType, $params = array())
		{
			if ($contentType == self::CONTENT_TYPE_PAYMENT_FORM) {
				$expMonths = array(0 => _t('Month'), 1 => '01', 2 => '02', 3 => '03', 4 => '04', 5 => '05',
					6 => '06', 7 => '07', 8 => '08', 9 => '09', 10 => '10', 11 => '11', 12 => '12');
				
				$expYears = array(0 => _t('Year'));
				$currentYear = gmdate('Y');
				for ($i = 0; $i < self::NUMBER_OF_YEARS; $i++)
					$expYears[$currentYear+$i] = $currentYear+$i;
				
				$form = new scPaymentModuleWebForm();
				$form->initWidgets(array(
					array(
						'field' => 'miSelectWidget',
						'data' => 'CardIssuer'
					),
					array(
						'field' => 'miTextWidget',
						'data' => 'CardNumber'
					),
					array(
						'field' => 'miSelectWidget',
						'data' => 'CardExpirationMonth'
					),
					array(
						'field' => 'miSelectWidget',
						'data' => 'CardExpirationYear'
					),
					array(
						'field' => 'miBaseTextWidget',
						'data' => 'CardCVV'
					),
					array(
						'field' => 'miBaseTextWidget',
						'data' => 'Installments'
					),
					// Used only for posnet banks
					array(
						'field' => 'miBaseCheckboxWidget',
						'data' => 'UsePoints'
					),
				));
				
				$form->getWidget('CardExpirationMonth')->setOptions($expMonths);
				$form->getWidget('CardExpirationYear')->setOptions($expYears);
				
				$defaultBankCode = scModuleOptions::getModuleOption($this->_moduleId, 'DefaultBankCode');
				$bankNames = scPaymentModuleTurkishBanksBase::getActiveBankNames();
				if (!isset($bankNames[$defaultBankCode]))
					$defaultBankCode = null;
				
				if (count($bankNames) == 0)
					throw new scPaymentProcessException(_t('There are no active banks.'));
				
				$form->getWidget('CardIssuer')->setOptions(array(scPaymentModuleTurkishBanksBase::BANK_CODE_OTHER => _t('Other card issuer')) + $bankNames, $defaultBankCode);
				
				$grandTotal = scCart::getCart()->getOrderTotal()->getGrandTotalAmount();
				$form->addMainPageElements(array(
					'%%INSTALLMENTS_JS%%' => $this->getInstallmentsJavascript($grandTotal),
					'%%GRAND_TOTAL_BEFORE_INSTALLMENTS%%' => $grandTotal->get(scCurrencyUtil::getCurrentCurrency()),
					'%%POSNET_BANKS%%' => scJson::jsonEncode(self::$_banksPosnet),
				));
				
				return $form->parse('modules/payment/turkish_banks/payment_form.tmpl', true);
			}
		}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $params)
		{
			if ($params['CardIssuer'] == scPaymentModuleTurkishBanksBase::BANK_CODE_OTHER) {
				$params['CardIssuer'] = scModuleOptions::getModuleOption($this->_moduleId, 'DefaultBankCode');
			}
			$bankProcessor = $this->getBankProcessor($params['CardIssuer']);
			return $bankProcessor->processPayment($payAmount, $payCurrencyCode, $cart, $params);
		}
		
		function processReturn($params)
		{
			$bankProcessor = $this->getBankProcessor($params['bank']);
			return $bankProcessor->processReturn($params);
		}
		
		function finalizePosnet($params)
		{
			$bankProcessor = new scPaymentModulePosnet($this->_moduleId);
			return $bankProcessor->finalizeOrder($params);
		}
		
		/**
		 * Returns javascript with definitions of the available installment options
		 * @param scMultiCurrencyAmount $amount
		 * @return string
		 */
		protected function getInstallmentsJavascript(scMultiCurrencyAmount $amount)
		{
			return scJson::jsonEncode($this->getInstallments($amount));
		}
		
		/**
		 * Returns array with definitions of the available installment options
		 * @param scMultiCurrencyAmount $amount
		 * @return array
		 */
		public function getInstallments(scMultiCurrencyAmount $amount)
		{
			$result = array();
			
			$banksInfo = self::getBanksInfo();
			$options = scModuleOptions::getModuleOptions($this->_moduleId);
			foreach ($banksInfo as $bankCode => $bankInfo) {
				if (!isset($options[$bankCode . '_Installments']))
					continue;
				
				if (!isset($options[$bankCode . '_Active']) or !$options[$bankCode . '_Active'])
					continue;
				
				if (!isset($options[$bankCode . '_Active']) or ($options[$bankCode . '_Installments'] == ''))
					continue;
				
				$installments = explode(',', $options[$bankCode . '_Installments']);
				$installmentsJs = array();
				foreach ($installments as $installment) {
					list($numberOfPayments, $interestRate) = explode(':', $installment);
					$totalAmount = scMultiCurrencyAmountMath::multiply($amount, floatval($interestRate) / 100 + 1);
					$installmentAmount = scMultiCurrencyAmountMath::divide($totalAmount, $numberOfPayments);
					$totalAmount = scMultiCurrencyAmountMath::multiply($installmentAmount, $numberOfPayments); // rounding fix
					$installmentsJs[] = array(
						'NumberOfPayments' => $numberOfPayments,
						'InterestRate' => number_format($interestRate, 2) . '%',
						'InstallmentPayment' => scPrice::factory($installmentAmount)->getDisplayPrice(),	// Equal payments
						'Total' => scPrice::factory($totalAmount)->getDisplayPrice(),
						'InstallmentPaymentPrice' => $installmentAmount->get(),
						'TotalPrice' => $totalAmount->get(),
						'Logo' => scPaths::getUrl('skins/default/modules/payment/turkish_banks/images/' . $bankInfo['logo'])
					);
				}
				$result[$bankCode] = $installmentsJs;
			}
			
			return $result;
		}
		
		protected function getBankProcessor($bank)
		{
			if (in_array($bank, scPaymentModuleTurkishBanksBase::$_banksEst3d)) {
				return new scPaymentModuleEst3d($this->_moduleId);
			} elseif (in_array($bank, scPaymentModuleTurkishBanksBase::$_banksPosnet)) {
				return new scPaymentModulePosnet($this->_moduleId);
			} elseif ($bank == scPaymentModuleTurkishBanksBase::BANK_CODE_GARANTI) {
				return new scPaymentModuleGaranti($this->_moduleId);
			}
			
			// Bank not found
			throw new miException(sprintf(_i('Unknown bank code %s.'), $bank));
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultBankCode',
				),
				array(
					'field' => 'scPaymentModuleTurkishBanksTableWidget',
					'data' => 'BanksList',
					'properties' => array('ModuleID' => $this->_moduleId)
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('DefaultBankCode')->setOptions(self::getBankNames());
			$options->setTemplateFile('modules/payment/turkish_banks/options_3d.tmpl');
			return $options;
		}
		
		/**
		 * Iterates the xml and sets all values as transaction details
		 * @param scPaymentProcessResult $result
		 * @param SimpleXMLElement $xml
		 */
		protected function setTransactionDetailsFromXml(scPaymentProcessResult $result, SimpleXMLElement $xml)
		{
			foreach ($xml as $key => $xmlElement) {
				if ($xmlElement->children()) {
					$this->setTransactionDetailsFromXml($result, $xmlElement);
					continue;
				}
				
				$result->setTransactionDetail($key, (string)$xmlElement);
			}
		}
		
		/**
		 * Validates the posted credit card. Will return validation errors in $result object if the data is incorrect
		 * 
		 * @param array $paramsPost
		 * @param scPaymentProcessResult &$result
		 */
		protected function validateCreditCard($paramsPost, scPaymentProcessResult &$result)
		{
			if (!scPaymentForms::isCardNumberValid($paramsPost['CardNumber']))
				$result->addValidatorError(_t('Invalid card number.'), 'CardNumber');
				
			if (!scPaymentForms::isCardExpirationValid($paramsPost['CardExpirationYear'], $paramsPost['CardExpirationMonth']))
				$result->addValidatorError(_t('Invalid expiration date.'), 'CardExpirationYear');
			
			$cvvLength = strlen($paramsPost['CardCVV']);
			if (!ctype_digit($paramsPost['CardCVV']) or (($cvvLength != 3) and ($cvvLength != 4)))
				$result->addValidatorError(_t('Invalid CVV code.'), 'CardCVV');
		}
		
		protected function setupAutoloading()
		{
			$autoloadClasses = array(
				'scPaymentModuleEst3d' => 'turkish_banks_est3d.php',
				'scPaymentModuleGaranti' => 'turkish_banks_garanti.php',
				'scPaymentModulePosnet' => 'turkish_banks_posnet.php',
			);
			
			foreach ($autoloadClasses as $className => $fileName) {
				// Relative from the location of miLoader
				miLoader::addClass($className, '../../modules/payment/turkish_banks/' . $fileName);
			}
		}
	}
	
	/**
	 * Widget to display all supported banks
	 */
	class scPaymentModuleTurkishBanksTableWidget extends scTableWidget {
		public function __construct(miWebForm &$webForm, $properties)
		{
			$properties['template'] = 'modules/payment/turkish_banks/options_3d_banks_table.tmpl';
			parent::__construct($webForm, $properties);
			
			$bankInfos = scPaymentModuleTurkishBanksBase::getBanksInfo();
			$options = scModuleOptions::getModuleOptions($properties['ModuleID']);
			
			$rows = array();
			foreach ($bankInfos as $bankCode => $bankInfo) {
				$rows[] = array(
					'BankCode' => $bankCode,
					'BankName' => $bankInfo['name'],
					'BankLogo' => $bankInfo['logo'],
					'BankIsActive' => (isset($options[$bankCode . '_Active']) and $options[$bankCode . '_Active']) ?  '1' : '0',
				);
			}
			$recordset = new scPreparedRecordset($rows);
			
			$table = new scTable($recordset);
			$this->setTable($table);
		}
		
		public function getData()
		{
			$moduleOptions = scModuleOptions::getModuleOptions($this->_properties['ModuleID']);
			
			// Unset options that are not handled by this widget
			if (isset($moduleOptions['DefaultBankCode'])) {
				unset($moduleOptions['DefaultBankCode']);
			}
			
			$data = array();
			foreach ($moduleOptions as $name => $value) {
				$data[scModuleOptions::MODULE_OPTION_PREFIX . $name] = $value;
			}
			return $data;
		}
	}
?>