<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * EST.com.tr 3D Model payment processing module
	 * Works with all turkish banks utilizing EST (est.com.tr)
	 */
	class scPaymentModuleEst3d extends scPaymentModuleTurkishBanks implements IPaymentModule, IModuleOptions {
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			// Validate the entered information
			$result = new scPaymentProcessResult();
			$this->validateCreditCard($paramsPost, $result);
			if ($result->hasValidatorErrors())
				return $result;
			
			$bank = $paramsPost['CardIssuer'];
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array($bank . '_ClientID', $bank . '_Mode', $bank . '_TransactionType', $bank . '_StoreKey'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$customerInfo = $cart->getCustomerInfo();
			$params = $this->getStandardParams($customerInfo, $options[$bank . '_ClientID'], $options[$bank . '_TransactionType'], $payAmount, $payCurrencyCode);
			$params += array(
				'okUrl' => scPaths::getSecureUrl('modules/payment/turkish_banks/return_3d.php'),
				'failUrl' => scPaths::getSecureUrl('modules/payment/turkish_banks/return_3d.php'),
				'storetype' => '3d',
				
				'bank' => $bank,
				
				'pan' => scPaymentForms::normalizeCardNumber($paramsPost['CardNumber']),
				'cv2' => $paramsPost['CardCVV'],
				'Ecom_Payment_Card_ExpDate_Year' => sprintf('%02d', $paramsPost['CardExpirationYear']),
				'Ecom_Payment_Card_ExpDate_Month' => sprintf('%02d', $paramsPost['CardExpirationMonth']),
			);
			if (isset($paramsPost['Installments']))
				$params['taksit'] = $paramsPost['Installments'];
			$params['hash'] = $this->calcHash($params, $options[$bank . '_StoreKey']);
			
//			if ($bank == self::BANK_CODE_HSBC)
//				$params += $this->getHsbcParams($payAmount, $payCurrencyCode, $options, $bank);
			
			$url = self::getBank3dGateUrl($bank, $options[$bank . '_Mode']);
			
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		/**
		 * FIXME: this function
		 * @return array
		 */
/*		protected function getHsbcParams($payAmount, $payCurrencyCode, $options, $bank)
		{
			$params = array(
				'clientid' => $options['ClientID'],
				'oid' => scOrdersInProgress::singleton()->getLastOrderId(),
				'amount' => $payAmount,
				'CurrencyCode' => $payCurrencyCode,
				'bank' => $bank,
				
				'pan' => '5555555555554444',
				'cv2' => '123',
				'Ecom_Payment_Card_ExpDate_Year' => '11', // MUST BE YY for HSBC
				'Ecom_Payment_Card_ExpDate_Month' => '10',
				'txtype' => 'Auth', // FIXME
				'taksit' => '',
				'ReturnURL' => scPaths::getSecureUrl('modules/payment/turkish_banks/return_3d.php'),
			);
			return $params;
		}*/
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		public function processReturn($params)
		{
			// Check if system error
			if (isset($params['ProcReturnCode']) and isset($params['ErrMsg']) and isset($params['oid']) and ($params['ProcReturnCode'] == 99)) {
				$result = new scPaymentProcessResult($params['oid']);
				$result->setPaymentNotes($params['ErrMsg']);
				return $result;
			}
			
			$bank = $params['bank'];
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array($bank . '_ClientID', $bank . '_Mode', $bank . '_TransactionType', $bank . '_StoreKey', $bank . '_ApiUsername', $bank . '_ApiPassword'));
			
			// Security checks
			if (!$this->verifyHash($params, $options[$bank . '_StoreKey']))
				throw new scPaymentProcessException(_t('Hash signature is incorrect. Please check your store key.'));
			
			scPaymentProcess::checkPaymentToOrder($params['oid'], $params['amount'], $params['CurrencyCode']);
			
			if ($params['clientid'] != $options[$bank . '_ClientID'])
				throw new scPaymentProcessException(_t('Merchant does not match.'));
			
			// Check if the 3D auth succeeded
			$successfulMdStatuses = array(1, 2, 3, 4);
			if (!in_array($params['mdStatus'], $successfulMdStatuses)) {
				// 3D auth failed
				$result = new scPaymentProcessResult($params['oid'], $params['amount']);
				$result->setPaymentNotes(sprintf(_t('3D authentication failed. Reason: %s'), $params['mdErrorMsg']));
				$result->setAllTransactionDetails($params);
				return $result;
			}
			
			// Process the payment
			$xmlResult = $this->makeCc5Request($params, $options[$bank . '_ClientID'], $options[$bank . '_ApiUsername'], $options[$bank . '_ApiPassword'], $options[$bank . '_Mode'], $options[$bank . '_TransactionType']);
			
			$xml = new SimpleXMLElement($xmlResult);
			$orderId = (string)$xml->OrderId;
			$response = (string)$xml->Response;
			$transId = (string)$xml->TransId;
			$errMsg = (string)$xml->ErrMsg;
			
			$result = new scPaymentProcessResult($params['oid'], $params['amount'], $params['CurrencyCode'], $transId);
			$this->setTransactionDetailsFromXml($result, $xml);
			
			if ($response == 'Approved')
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			else // Declined/Error
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			
			$result->setPaymentNotes($errMsg);
			
			return $result;
		}
		
		protected function makeCc5Request($params, $clientId, $apiUsername, $apiPassword, $mode, $transactionType)
		{
			$userId = isset($params['userid']) ? $params['userid'] : '';
			
			$productsXmlDefinitions = $this->getProductsXmlDefinitions($params['CurrencyCode']);
			
			$definitions = array(
				array(
					scXml::TAG => 'CC5Request',
					scXml::CHILDREN => array(
						array(scXml::TAG => 'Name', scXml::DATA => $apiUsername),
						array(scXml::TAG => 'Password', scXml::DATA => $apiPassword),
						array(scXml::TAG => 'ClientId', scXml::DATA => $clientId),
						array(scXml::TAG => 'IPAddress', scXml::DATA => $_SERVER['REMOTE_ADDR']),
						array(scXml::TAG => 'Email', scXml::DATA => $params['email']),
						array(scXml::TAG => 'Mode', scXml::DATA => 'P'),	// P = Production, T = Test; Only P works
						array(scXml::TAG => 'OrderId', scXml::DATA => $params['oid']),
						array(scXml::TAG => 'GroupId', scXml::DATA => ''),
						array(scXml::TAG => 'TransId', scXml::DATA => ''),
						array(scXml::TAG => 'UserId', scXml::DATA => $userId),
						array(scXml::TAG => 'Type', scXml::DATA => $transactionType),
						array(scXml::TAG => 'Number', scXml::DATA => $params['md']),
//						array(scXml::TAG => 'Expires', scXml::DATA => ''),
//						array(scXml::TAG => 'Cvv2Val', scXml::DATA => ''),
						array(scXml::TAG => 'Total', scXml::DATA => $params['amount']),
						array(scXml::TAG => 'Currency', scXml::DATA => $this->getIso4217NumCurrencyCode($params['CurrencyCode'])),
						array(scXml::TAG => 'Taksit', scXml::DATA => isset($params['taksit']) ? $params['taksit'] : ''),
						array(scXml::TAG => 'PayerTxnId', scXml::DATA => $params['xid']),
						array(scXml::TAG => 'PayerSecurityLevel', scXml::DATA => $params['eci']),
						array(scXml::TAG => 'PayerAuthenticationCode', scXml::DATA => $params['cavv']),
						array(scXml::TAG => 'CardholderPresentCode', scXml::DATA => '13'),
						array(scXml::TAG => 'BillTo',
							scXml::CHILDREN => array(
								array(scXml::TAG => 'Name', scXml::DATA => $params['BillingName']),
								array(scXml::TAG => 'Street1', scXml::DATA => $params['BillingAddress']),
								array(scXml::TAG => 'Street2', scXml::DATA => $params['BillingAddress2']),
								array(scXml::TAG => 'Street3', scXml::DATA => ''),	// FIXME: Consider passing the country here
								array(scXml::TAG => 'City', scXml::DATA => $params['BillingCity']),
								array(scXml::TAG => 'StateProv', scXml::DATA => $params['BillingState']),
								array(scXml::TAG => 'PostalCode', scXml::DATA => $params['BillingPostcode']),
								array(scXml::TAG => 'Country', scXml::DATA => $params['BillingCountry']), // This should be 3166-1 3-number numeric code
								array(scXml::TAG => 'Company', scXml::DATA => $params['BillingCompany']),
								array(scXml::TAG => 'TelVoice', scXml::DATA => $params['BillingPhone']),
								array(scXml::TAG => 'TelFax', scXml::DATA => $params['BillingFax']),
							)
						),
						array(scXml::TAG => 'ShipTo',
							scXml::CHILDREN => array(
								array(scXml::TAG => 'Name', scXml::DATA => $params['ShippingName']),
								array(scXml::TAG => 'Street1', scXml::DATA => $params['ShippingAddress']),
								array(scXml::TAG => 'Street2', scXml::DATA => $params['ShippingAddress2']),
								array(scXml::TAG => 'Street3', scXml::DATA => ''),
								array(scXml::TAG => 'City', scXml::DATA => $params['ShippingCity']),
								array(scXml::TAG => 'StateProv', scXml::DATA => $params['ShippingState']),
								array(scXml::TAG => 'PostalCode', scXml::DATA => $params['ShippingPostcode']),
								array(scXml::TAG => 'Country', scXml::DATA => $params['ShippingCountry']), // This should be 3166-1 3-number numeric code
								array(scXml::TAG => 'Company', scXml::DATA => $params['ShippingCompany']),
								array(scXml::TAG => 'TelVoice', scXml::DATA => $params['ShippingPhone']),
								array(scXml::TAG => 'TelFax', scXml::DATA => $params['ShippingFax']),
							)
						),
//						array(scXml::TAG => 'Extra'),
						$productsXmlDefinitions
					)
				),
			);
			
			$xml = 'DATA=<?xml version="1.0" encoding="UTF-8"?>' . scXml::xmlWriter($definitions);
			
			$url = self::getBankApiUrl($params['bank'], $mode);
			return scHttp::post($url, $xml);
		}
		
		protected function getProductsXmlDefinitions($currencyCode)
		{
			$items = scCart::getCart()->getItems();
			$currency = new scCurrency($currencyCode);
			$n = 0;
			
			$products = array();
			
			foreach ($items as $item) {
				$n++;
				$products[] = array(
					scXml::TAG => 'OrderItem',
					scXml::CHILDREN => array(
						array(scXml::TAG => 'ItemNumber', scXml::DATA => $n),
						array(scXml::TAG => 'ProductCode', scXml::DATA => $item->getCartItemCode()),
						array(scXml::TAG => 'Qty', scXml::DATA => $item->getQuantity()),
						array(scXml::TAG => 'Desc', scXml::DATA => $item->getName()),
						array(scXml::TAG => 'Id', scXml::DATA => $item->getCartItemCode()),
						array(scXml::TAG => 'Price', scXml::DATA => $item->getPriceObj()->getAmountInCurrency($currency)),
					)
				);
			}
			
			$definition = array(
				scXml::TAG => 'OrderItemList',
				scXml::CHILDREN => $products
			);
			return $definition;
		}
		
		public function getOptionFields(miWebForm &$webForm) {}
	}
?>