<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Garanti Virtual POS system payment module
	 */
	class scPaymentModuleGaranti extends scPaymentModuleTurkishBanks implements IPaymentModule, IModuleOptions {
		
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			// Validate the entered information
			$result = new scPaymentProcessResult();
			$this->validateCreditCard($paramsPost, $result);
			if ($result->hasValidatorErrors())
				return $result;
			
			$bank = $paramsPost['CardIssuer'];
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array(
				$bank . '_OrderDescription', $bank . '_ProvisionUserID', $bank . '_ProvisionPassword', $bank . '_UserID',
				$bank . '_TerminalID', $bank . '_MerchantID', $bank . '_Mode', $bank . '_3DSecureMode', $bank . '_3DSecurePassword'
			));
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			$customerInfo = $cart->getCustomerInfo();
			
			$params = array(
				// System params
				'secure3dsecuritylevel' => $options[$bank . '_3DSecureMode'],
				'mode' => ($options[$bank . '_Mode'] == scPaymentModuleTurkishBanksBase::MODE_LIVE) ? 'PROD' : 'TEST',
				'apiversion' => 'v0.01',
				'terminalprovuserid' => $options[$bank . '_ProvisionUserID'],
				'terminaluserid' => $options[$bank . '_UserID'],
				'terminalmerchantid' => $options[$bank . '_MerchantID'],
				'terminalid' => $options[$bank . '_TerminalID'],
				
				// Transaction params
				'txntype' => 'sales', // TransactionType
				'txnamount' => round($payAmount * 100),
				'txncurrencycode' => $this->getIso4217NumCurrencyCode($this->getPaymentCurrencyCode()),
				'txntimestamp' => time(),
				
				// Credit card params
				'cardnumber' => scPaymentForms::normalizeCardNumber($paramsPost['CardNumber']),
				'cardexpiredatemonth' => sprintf('%02d', $paramsPost['CardExpirationMonth']),
				'cardexpiredateyear' => sprintf('%02d', substr($paramsPost['CardExpirationYear'], 2)),
				'cardcvv2' => $paramsPost['CardCVV'],
				'cardholder' => $customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName'],
				
				// Order params
				'orderid' => $orderId,
				'orderdescription' => $this->normalizeDescription($options[$bank . '_OrderDescription']),
				
				// Customer params
				'customeripaddress' => $_SERVER['REMOTE_ADDR'],
				'customeremailaddress' => $customerInfo['CustomerEmail'],
//				'companyname' => '',
				
				// Return urls
				'successurl' => scPaths::getSecureUrl('modules/payment/turkish_banks/return_3d.php?bank=' . urlencode($bank)),
				'errorurl' => scPaths::getSecureUrl('modules/payment/turkish_banks/return_3d.php?bank=' . urlencode($bank)),
			);
			$installments = '';
			if ((isset($paramsPost['Installments'])) && (intval($paramsPost['Installments']) > 0)) {
				$params['txninstallmentcount'] = $paramsPost['Installments'];
				$installments = $paramsPost['Installments'];
			}
			$params['secure3dhash'] = $this->getSecure3dHash($orderId, 
				$options[$bank . '_TerminalID'], $options[$bank . '_ProvisionPassword'], $options[$bank . '_3DSecurePassword'],
				$params['txntype'], $params['txnamount'], $installments, $params['successurl'], $params['errorurl']
			);
			$params = array_merge($params, $this->setOrderItemsParams($cart));
			$params = array_merge($params, $this->setCustomerAddressesParams($customerInfo));
			if (!empty($customerInfo['OrderCustomerInstructions'])) {
				$params['ordercommentcount'] = 1;
				$params['ordercommentnumber1'] = 1;
				$params['ordercommenttext1'] = self::normalizeTextField($customerInfo['OrderCustomerInstructions']);
			}
			
			$url = self::getBank3dGateUrl($bank, $options[$bank . '_Mode']);
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		public function processReturn($params)
		{
			$bank = $params['bank'];
			if ($bank != scPaymentModuleTurkishBanksBase::BANK_CODE_GARANTI) {
				throw new scPaymentProcessException(_i('Expected bank Garanti.'));
			}
			
			$currencyCode = $this->getCurrencyCodeFromIso4217Num($params['txncurrencycode']);
			$result = new scPaymentProcessResult($params['oid'], $params['txnamount'], $currencyCode);
			
			if (isset($params['procreturncode']) && isset($params['errmsg']) && isset($params['oid']) && ($params['procreturncode'] == 99)) {
				$result->setPaymentNotes($params['errmsg']);
				$result->setAllTransactionDetails($params);
				return $result;
			}
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array(
				$bank . '_3DSecureMode', $bank . '_3DSecurePassword'
			));
			
			if (!isset($params['procreturnCode']) && isset($params['procreturncode'])) {
				$params['procreturnCode'] = $params['procreturncode']; // Garanti VPS bug workaround
			}
			
			// Verify data integrity
			if ($this->verifyHash($params, $options[$bank . '_3DSecurePassword']) === false) {
				throw new scPaymentProcessException(_t('Hash signature is incorrect. Please check your 3D secure password.'));
			}
			
			scPaymentProcess::checkPaymentToOrder($params['oid'], ($params['txnamount'] / 100), $currencyCode);
			
			if (($options[$bank . '_3DSecureMode'] == '3D_PAY') && (!in_array($params['mdstatus'], array(1, 2, 3, 4, 5))) || 
				($options[$bank . '_3DSecureMode'] == '3D_FULL') && ($params['mdstatus'] != 1) ||
				($options[$bank . '_3DSecureMode'] == '3D_HALF') && (!in_array($params['mdstatus'], array(1, 2, 3, 4)))) {
				// 3D auth failed
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes(sprintf(_t('3D authentication failed. Reason: %s'), $params['mderrormessage']));
				$result->setAllTransactionDetails($params);
				return $result;
			}
			
			if ($params['response'] == 'Approved') {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			} else if (($params['response'] == 'Declined') || ($params['response'] == 'Error')) {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				
				$errorMessage = sprintf(_t('Transaction failed. Reason: %s'), $params['errmsg']);
				$result->setPaymentNotes($errorMessage);
				$result->setTransactionNote($errorMessage);
			} else {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_OTHER);
			}
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
		protected function getSecure3dHash($orderId, $terminalId, $provisionPassword, $tdSecurePassword, $transactionType, $orderTotal, $installmentsCount, $successUrl, $errorUrl)
		{
			$securityData = strtoupper(sha1($provisionPassword . sprintf('%09d', $terminalId)));
			return strtoupper(sha1($terminalId . $orderId . $orderTotal . $successUrl . $errorUrl . $transactionType . $installmentsCount . $tdSecurePassword . $securityData));
		}
		
		protected function setOrderItemsParams(scCart $cart)
		{
			$orderItemsParams = array(
				'orderitemcount' => $cart->getItemsCount()
			);
			
			$currency = new scCurrency($this->getPaymentCurrencyCode());
			$cartItems = $cart->getItems();
			
			$cartItemNum = 1;
			foreach ($cartItems as $cartItem) {
				$productInfo = $cartItem->getProductInfo();
				
				$orderItemsParams['orderitemnumber' . $cartItemNum] = $cartItemNum;
				$orderItemsParams['orderitemproductid' . $cartItemNum] = $cartItem->getProductId();
				$orderItemsParams['orderitemproductcode' . $cartItemNum] = $cartItem->getProductCode();
				$orderItemsParams['orderitemquantity' . $cartItemNum] = $cartItem->getQuantity();
				$orderItemsParams['orderitemdescription' . $cartItemNum] = $this->normalizeDescription($cartItem->getName());
				
				$itemPrice = scPrice::factory($cartItem->getPriceObj()->getAmount());
				$itemPriceValue = $itemPrice->getAmountInCurrency($currency);
				$orderItemsParams['orderitemprice' . $cartItemNum] = round($itemPriceValue * 100);
				
				$totalItemAmount = scMultiCurrencyAmount::factory();
				$totalItemAmount->set($currency, $itemPriceValue);
				$totalItemAmount = scMultiCurrencyAmountMath::multiply($totalItemAmount, $cartItem->getQuantity());
				$totalItemPrice = $totalItemAmount->get($currency);
				$orderItemsParams['orderitemtotalamount' . $cartItemNum] = round($totalItemPrice * 100);
				
				$cartItemNum++;
			}
			
			return array_map(array('scPaymentModuleGaranti', 'normalizeTextField'), $orderItemsParams);
		}
		
		protected function setCustomerAddressesParams($customerInfo)
		{
			$addressParams = array(
				'orderaddresscount' => 2,
				'orderaddresstype1' => 'B', // Billing
				'orderaddresscity1' => $customerInfo['CustomerBillingCity'],
				'orderaddresscompany1' => $customerInfo['CustomerBillingCompany'],
				'orderaddresscountry1' => scCountry::getCountryName($customerInfo['CustomerBillingCountry']),
				'orderaddressfaxnumber1' => preg_replace('/\D/', '', $customerInfo['CustomerBillingFax']), // Remove all non-digit characters from the number
				'orderaddresslastname1' => $customerInfo['CustomerBillingLastName'],
				'orderaddressname1' => $customerInfo['CustomerBillingFirstName'],
				'orderaddressphonenumber1' => preg_replace('/\D/', '', $customerInfo['CustomerBillingPhone']), // Remove all non-digit characters from the number
				'orderaddresspostalcode1' => $customerInfo['CustomerBillingZip'],
				'orderaddresstext1' => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				
				'orderaddresstype2' => 'S', // Shipping
				'orderaddresscity2' => $customerInfo['CustomerShippingCity'],
				'orderaddresscompany2' => $customerInfo['CustomerShippingCompany'],
				'orderaddresscountry2' => scCountry::getCountryName($customerInfo['CustomerShippingCountry']),
				'orderaddressfaxnumber2' => preg_replace('/\D/', '', $customerInfo['CustomerShippingFax']), // Remove all non-digit characters from the number
				'orderaddresslastname2' => $customerInfo['CustomerShippingLastName'],
				'orderaddressname2' => $customerInfo['CustomerShippingFirstName'],
				'orderaddressphonenumber2' => preg_replace('/\D/', '', $customerInfo['CustomerShippingPhone']), // Remove all non-digit characters from the number
				'orderaddresspostalcode2' => $customerInfo['CustomerShippingZip'],
				'orderaddresstext2' => $customerInfo['CustomerShippingAddress1'] . ' ' . $customerInfo['CustomerShippingAddress2'],
			);
			
			return array_map(array('scPaymentModuleGaranti', 'normalizeTextField'), $addressParams);
		}
		
		/**
		 * Returns true if the hash is correct
		 * @param array $params
		 * @param string $storeKey
		 * @return bool
		 */
		protected function verifyHash($params, $storeKey)
		{
			if ((!isset($params['hashparams'])) || (!isset($params['hashparamsval'])) || (!isset($params['hash']))) {
				return false;
			}
			
			$params['HASHPARAMS'] = $params['hashparams'];
			$params['HASHPARAMSVAL'] = $params['hashparamsval'];
			$params['HASH'] = $params['hash'];
			return parent::verifyHash($params, $storeKey);
		}
		
		protected function normalizeDescription($description)
		{
			$specialTurkishChars = array('â', 'î', 'û', 'ç', 'Ç', 'ğ', 'Ğ', 'ı', 'İ', 'ö', 'Ö', 'ş', 'Ş', 'ü', 'Ü');
			$specialTurkishCharsReplacements = array('a', 'i', 'u', 'c', 'C', 'g', 'G', 'i', 'I', 'o', 'O', 's', 'S', 'u', 'U');
			$description = str_replace($specialTurkishChars, $specialTurkishCharsReplacements, $description);
			
			return preg_replace(array('/[\s]/', '/[^a-zA-Z0-9_]/'), array('_', ''), $description);
		}
		
		public static function normalizeTextField($text)
		{
			return preg_replace('/[\&\?\:\-]/', '', $text);
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep() {}
		
		public function getOptionFields(miWebForm &$webForm) {}
	}
?>