<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * 3D Secure payment processing module
	 * Works with all turkish banks utilizing Posnet
	 */
	class scPaymentModulePosnet extends scPaymentModuleTurkishBanks implements IPaymentModule, IModuleOptions {
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			// Validate the entered information
			$result = new scPaymentProcessResult();
			$this->validateCreditCard($paramsPost, $result);
			if ($result->hasValidatorErrors())
				return $result;
			
			$bank = $paramsPost['CardIssuer'];
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array(
				$bank . '_MerchantID', $bank . '_TerminalID', $bank . '_PosnetID', $bank . '_ForwardSaleCode',
				$bank . '_LanguageCode', $bank . '_Mode', $bank . '_TransactionType', $bank . '_ApiUsername', $bank . '_ApiPassword'
			));
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			$formattedOrderId = sprintf($bank . '_%016d', $orderId); // 20-character string (e.g. YKB_0000000000002036)
			$customerInfo = $cart->getCustomerInfo();
			
			$params = array(
				'bank' => $bank, // used for internal purposes, it is passed back by posnet
				'posnetid' => $options[$bank . '_PosnetID'],
				'ccno' => scPaymentForms::normalizeCardNumber($paramsPost['CardNumber']),
				'expDate' => sprintf('%02d%02d', substr($paramsPost['CardExpirationYear'], 2), $paramsPost['CardExpirationMonth']),
				'cvc' => $paramsPost['CardCVV'],
				'amount' => round($payAmount * 100),
				'currencyCode' => 'TL',
				'installment' => '0',
				//'orderID' => $formattedOrderId, // Without 3D Secure
				'XID' => $formattedOrderId, // With 3D Secure
				'cardHolderName' => $customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName'],
				'tranType' => $options[$bank . '_TransactionType'],
			);
			if (isset($paramsPost['Installments'])) {
				$params['installment'] = $paramsPost['Installments'];
			}
			
			$xmlResult = $this->prepareTranscation($params, $bank, $options[$bank . '_MerchantID'], $options[$bank . '_TerminalID'], $options[$bank . '_ApiUsername'], $options[$bank . '_ApiPassword'], $options[$bank . '_Mode']);
			$xml = new SimpleXMLElement($xmlResult);
			
			$result = new scPaymentProcessResult($orderId, $payAmount, $payCurrencyCode);
			$this->setTransactionDetailsFromXml($result, $xml);
			
			$approved = (int) $xml->approved;
			if (!$approved) {
				$result->setTransactionCode((string) $xml->respCode);
				$result->setTransactionNote((string) $xml->respText);
				
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes((string) $xml->respText);
				return $result;
			}
			
			$usePoints = (int) miGetParamDefault('UsePoints', 0);
			$params = array(
				'posnetData' => (string) $xml->oosRequestDataResponse->data1,
				'posnetData2' => (string) $xml->oosRequestDataResponse->data2,
				'digest' => (string) $xml->oosRequestDataResponse->sign,
				'mid' => $options[$bank . '_MerchantID'],
				'posnetID' => $options[$bank . '_PosnetID'],
				'vftCode' => $options[$bank . '_ForwardSaleCode'],
				'merchantReturnURL' => scPaths::getSecureUrl('modules/payment/turkish_banks/return_3d.php?bank=' . urlencode($bank) . '&usePoints=' . $usePoints),
				'url' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_DOCROOT),
				'lang' => $options[$bank . '_LanguageCode'],
				'openANewWindow' => 0,
			);
			
			$url = self::getBank3dGateUrl($bank, $options[$bank . '_Mode']);
			$result->setThirdPartyRedirect($url, $params, 'post');
			return $result;
		}
		
		public function processReturn($params)
		{
			$bank = $params['bank'];
			if (!in_array($bank, self::$_banksPosnet)) {
				throw new miException(_i('Expected Posnet bank.'));
			}
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array(
				$bank . '_MerchantID', $bank . '_TerminalID', $bank . '_PosnetID', $bank . '_EncryptionKey',
				$bank . '_3dSecureCheckMask', $bank . '_Mode', $bank . '_ApiUsername', $bank . '_ApiPassword'
			));
			
			$this->checkSignature($params['MerchantPacket'], $params['BankPacket'], $params['Sign'], $options[$bank . '_EncryptionKey']);
			
			$xmlResult = $this->checkAndResolveMerchantData($params, $bank, $options[$bank . '_MerchantID'], $options[$bank . '_TerminalID'], $options[$bank . '_ApiUsername'], $options[$bank . '_ApiPassword'], $options[$bank . '_Mode']);
			$xml = new SimpleXMLElement($xmlResult);
			
			$orderId = (int) str_replace($params['bank'] . '_', '', $params['Xid']);
			$result = new scPaymentProcessResult($orderId, $params['Amount']);
			$this->setTransactionDetailsFromXml($result, $xml);
			
			$approved = (int) $xml->approved;
			if (!$approved) {
				$result->setTransactionCode((string) $xml->respCode);
				$result->setTransactionNote((string) $xml->respText);
				
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes((string) $xml->respText);
				return $result;
			}
			
			$tdsStatus = (int) $xml->oosResolveMerchantDataResponse->mdStatus;
			$this->check3dSecureStatus($tdsStatus, $options[$bank . '_3dSecureCheckMask']);
			
			// Check whether the user has world points
			$points = (int) $xml->oosResolveMerchantDataResponse->point;
			$usePoints = miGetParam('usePoints');
			if (($points > 0) && $usePoints) {
				// User has world points and wants to use them
				$this->showPointsPage($params, $xml);
				die();
			}
			
			// World points are not going to be used, continue
			$params['WPAmount'] = 0;
			
			return $this->finalizeOrder($params, $options);
		}
		
		public function finalizeOrder($params, $options = array())
		{
			$bank = $params['bank'];
			if (!in_array($bank, self::$_banksPosnet)) {
				throw new miException(_i('Expected Posnet bank.'));
			}
			
			if (empty($options)) {
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array(
					$bank . '_MerchantID', $bank . '_TerminalID', $bank . '_PosnetID',
					$bank . '_Mode', $bank . '_ApiUsername', $bank . '_ApiPassword'
				));
			}
			
			$xmlResult = $this->doTDSTransaction($params, $bank, $options[$bank . '_MerchantID'], $options[$bank . '_TerminalID'], $options[$bank . '_ApiUsername'], $options[$bank . '_ApiPassword'], $options[$bank . '_Mode']);
			$xml = new SimpleXMLElement($xmlResult);
			
			$orderId = (int) str_replace($params['bank'] . '_', '', $params['Xid']);
			$result = new scPaymentProcessResult($orderId, $params['Amount']);
			$this->setTransactionDetailsFromXml($result, $xml);
			
			$approved = (int) $xml->approved;
			if (!$approved) {
				$result->setTransactionCode((string) $xml->respCode);
				$result->setTransactionNote((string) $xml->respText);
				
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes((string) $xml->respText);
				return $result;
			}
			
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			return $result;
		}
		
		protected function showPointsPage($paramsPost, $xmlResult)
		{
			scLayout::forceNoBoxes();
			$page = new scUserDisplayController(new scPaymentModulePosnetWorldPointsView($paramsPost, $xmlResult));
			$page->showPage();
		}
		
		/**
		 * Make request to Posnet with the card info
		 * 
		 * @param $params
		 * @param $bank
		 * @param $merchantId
		 * @param $terminalId
		 * @param $apiUsername
		 * @param $apiPassword
		 * @param $mode
		 */
		protected function prepareTranscation($params, $bank, $merchantId, $terminalId, $apiUsername, $apiPassword, $mode)
		{
			$definitions = array(
				array(
					scXml::TAG => 'posnetRequest',
					scXml::CHILDREN => array(
						array(scXml::TAG => 'mid', scXml::DATA => $merchantId),
						array(scXml::TAG => 'tid', scXml::DATA => $terminalId),
						array(scXml::TAG => 'username', scXml::DATA => $apiUsername),
						array(scXml::TAG => 'password', scXml::DATA => $apiPassword),
						array(scXml::TAG => 'oosRequestData',
							scXml::CHILDREN => array(
								array(scXml::TAG => 'posnetid', scXml::DATA => $params['posnetid']),
								array(scXml::TAG => 'ccno', scXml::DATA => $params['ccno']),
								array(scXml::TAG => 'expDate', scXml::DATA => $params['expDate']),
								array(scXml::TAG => 'cvc', scXml::DATA => $params['cvc']),
								array(scXml::TAG => 'amount', scXml::DATA => $params['amount']),
								array(scXml::TAG => 'currencyCode', scXml::DATA => $params['currencyCode']),
								array(scXml::TAG => 'installment', scXml::DATA => $params['installment']),
								array(scXml::TAG => 'XID', scXml::DATA => $params['XID']),
								array(scXml::TAG => 'cardHolderName', scXml::DATA => $params['cardHolderName']),
								array(scXml::TAG => 'tranType', scXml::DATA => $params['tranType']),
							)
						),
					)
				)
			);
			
			return $this->makePosnetRequest($bank, $mode, $definitions);
		}
		
		protected function checkAndResolveMerchantData($params, $bank, $merchantId, $terminalId, $apiUsername, $apiPassword, $mode)
		{
			$definitions = array(
				array(
					scXml::TAG => 'posnetRequest',
					scXml::CHILDREN => array(
						array(scXml::TAG => 'mid', scXml::DATA => $merchantId),
						array(scXml::TAG => 'tid', scXml::DATA => $terminalId),
						array(scXml::TAG => 'username', scXml::DATA => $apiUsername),
						array(scXml::TAG => 'password', scXml::DATA => $apiPassword),
						array(scXml::TAG => 'oosResolveMerchantData',
							scXml::CHILDREN => array(
								array(scXml::TAG => 'bankData', scXml::DATA => $params['BankPacket']),
								array(scXml::TAG => 'merchantData', scXml::DATA => $params['MerchantPacket']),
								array(scXml::TAG => 'sign', scXml::DATA => $params['Sign']),
							)
						),
					)
				)
			);
			
			return $this->makePosnetRequest($bank, $mode, $definitions);
		}
		
		/**
		 * Perform the 3D secure (TDS) transaction at Posnet and finalize it. 
		 * 
		 * @param $params
		 * @param $bank
		 * @param $merchantId
		 * @param $terminalId
		 * @param $apiUsername
		 * @param $apiPassword
		 * @param $mode
		 */
		protected function doTDSTransaction($params, $bank, $merchantId, $terminalId, $apiUsername, $apiPassword, $mode)
		{
			$definitions = array(
				array(
					scXml::TAG => 'posnetRequest',
					scXml::CHILDREN => array(
						array(scXml::TAG => 'mid', scXml::DATA => $merchantId),
						array(scXml::TAG => 'tid', scXml::DATA => $terminalId),
						array(scXml::TAG => 'username', scXml::DATA => $apiUsername),
						array(scXml::TAG => 'password', scXml::DATA => $apiPassword),
						array(scXml::TAG => 'oosTranData',
							scXml::CHILDREN => array(
								array(scXml::TAG => 'bankData', scXml::DATA => $params['BankPacket']),
								array(scXml::TAG => 'merchantData', scXml::DATA => $params['MerchantPacket']),
								array(scXml::TAG => 'sign', scXml::DATA => $params['Sign']),
								array(scXml::TAG => 'wpAmount', scXml::DATA => $params['WPAmount']),
							)
						),
					)
				)
			);
			
			return $this->makePosnetRequest($bank, $mode, $definitions);
		}
		
		public function makePosnetRequest($bank, $mode, $definitions)
		{
			$url = self::getBankApiUrl($bank, $mode);
			$url = parse_url($url);
			$xml = '<?xml version="1.0" encoding="UTF-8"?>' . scXml::xmlWriter($definitions);
			
			$ch = curl_init($url['scheme'] . '://' . $url['host'] . '/');
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			
			$requestBody = 'xmldata=' . urlencode($xml);
			$headers = array(
				'Host: ' . $url['host'],
				'User-Agent: Summer Cart',
				'Pragma: nocache',
				'Content-Type: application/x-www-form-urlencoded',
				'Content-Length: ' . strlen($requestBody)
			);
			$customRequest = 'POST ' . $url['path'] . " HTTP/1.1\r\n" . implode("\r\n", $headers) . "\r\n\r\n" . $requestBody;
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $customRequest);
			
			$response = curl_exec($ch);
			if ($response === false) {
				$error = curl_error($ch) . ' (' . curl_errno($ch) . ')';
				curl_close($ch);
				throw new miException($error);
			}
			
			$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			if ($httpCode and ($httpCode != 200)) {
				curl_close($ch);
				throw new scHttpException(sprintf(_t('Curl request failed for "%s". Response code "%d".'), $url, $httpCode), $httpCode, $response);
			}
			
			curl_close($ch);
			
			return $response;
		}
		
		protected function checkSignature($merchantPacket, $bankPacket, $signature, $encryptionKey)
		{
			if (empty($merchantPacket) || empty($bankPacket) || empty($signature)) {
				throw new miException(_t('Response error: Invalid merchant data.'));
			}
			
			$hash = strtoupper(md5($merchantPacket . $bankPacket . $encryptionKey));
			if (strcmp($hash, $signature) != 0) {
				throw new miException(_t('Response error: Invalid signature.'));
			}
		}
		
		protected function check3dSecureStatus($status, $checkMask)
		{
			$checkList = explode(',', $checkMask);
			if (in_array($status, $checkList))
				return;
			
			throw new miException(sprintf(_t('The transaction was refused by the store owner:  3D secure status "%s" is not allowed.'), $status));
		}
		
		public function getOptionFields(miWebForm &$webForm) {}
	}
	
	class scPaymentModulePosnetWorldPointsView implements ICMSBlock {
		
		protected $_params;
		protected $_xmlResult;
		
		public function __construct($params, $xmlResult)
		{
			$this->_params = $params;
			$this->_xmlResult = $xmlResult;
		}
		
		public function getControllerCommands()
		{
			$t = new scTemplateParser();
			$t->readTemplate('modules/payment/turkish_banks/world_points_form.tmpl');
			
			$t->assignArray(miUIUtil::templetizeRows(array(
				'bank' => $this->_params['bank'],
				'MerchantPacket' => $this->_params['MerchantPacket'],
				'BankPacket' => $this->_params['BankPacket'],
				'Sign' => $this->_params['Sign'],
				'XID' => $this->_xmlResult->oosResolveMerchantDataResponse->xid,
				'Amount' => $this->_xmlResult->oosResolveMerchantDataResponse->amount,
				'Currency' => $this->_xmlResult->oosResolveMerchantDataResponse->currency,
				'Points' => (int) $this->_xmlResult->oosResolveMerchantDataResponse->point,
				'PointsAmount' => $this->_xmlResult->oosResolveMerchantDataResponse->pointAmount
			)));
			
			$html = $t->templateParse();
			return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
		}
	}
?>