<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Ubb.bg payment processing module
	 */
	 
	 class scPaymentModuleUbbBgHostedPage implements IPaymentModule, IModuleOptions {
		
	 	const UBB_SERVLET = 'servlet/UniversalXMLServlet';
	 	const UBB_IP_ADDRESS = '195.34.112.25';
	 	
	 	const PAYMENT_TYPE_PURCHASE = 1;
	 	const PAYMENT_TYPE_AUTHORISATION = 4;
	 	
		protected $_moduleId;
		
		// ISO 4217 currency codes table
		protected static $currencyCodes = array(
			'BGN' => 975,
			'EUR' => 978,
			'USD' => 840,
			'GBP' => 826,
		);
		const DEFAULT_CURRENCY_CODE = 'BGN';
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('SupportedCurrencyCodes', 'DefaultCurrencyCode'));
			$supportedCurrencyCodes = explode(',', $options['SupportedCurrencyCodes']);
			
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $supportedCurrencyCodes))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			return $options['DefaultCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('TransactionType', 'SelectedAlias', 'Aliases'));
			
			$aliases = unserialize($options['Aliases']);
			if (!isset($aliases[$options['SelectedAlias']]))
				throw new scPaymentProcessException(_t('UBB module is not set up.'));
			
			$resourceContent = $aliases[$options['SelectedAlias']];
			$settings = scPaymentModuleUbbBgResourceFile::parseSettings($resourceContent);
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = ($isoLangCode == 'bg') ? 'bg_BG' : 'en_US';
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$data = array(
				'action' => $options['TransactionType'],
				'amt' => $payAmount,
				'currencycode' => self::$currencyCodes[$payCurrencyCode],
				'trackid' => $orderId,
				'langid' => $lang,
				'responseurl' => scPaths::getSecureUrl('modules/payment/ubb_bg/notify.php'),
				'errorurl' => scPaths::getSecureUrl('modules/payment/ubb_bg/error.php'),
				'udf1' => $orderId
			);
			$paymentInitXml = $this->getPaymentInitXml('PaymentInit', $settings, $data);
			$result = $this->sendTransaction($settings, $paymentInitXml);
			
			$resultXml = simplexml_load_string($result);
			
			// Check for errors
			$responseAttrs = $resultXml->response->attributes();
			if ($responseAttrs['type'] == 'error') {
				$details = array(
					'error_code_tag' => (string)$resultXml->response->error_code_tag,
					'error_service_tag' => (string)$resultXml->response->error_service_tag,
					'error_text' => (string)$resultXml->response->error_text,
				);
				throw new scPaymentProcessException(sprintf(_t('Error from ubb.bg: "%s"'), $resultXml->response->error_text), $details);
			}
			
			// Success
			$paymentPage = (string)$resultXml->response->paymentpage;
			$paymentId = (string)$resultXml->response->paymentid;
			
			// Save the payment id
			$transaction = new scTransaction($orderId, $this->_moduleId, $payAmount, $payCurrencyCode);
			$transaction->setTransactionDetail('PaymentID', $paymentId);
			$transactionId = $transaction->insertTransaction();
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($paymentPage, array('PaymentID' => $paymentId));
			return $result;
		}
		
		protected function getPaymentInitXml($transactionName, $settings, $data)
		{
			$requestChildren = array();
			$requestChildren[] = array(scXml::TAG => 'id', scXml::DATA => $settings['id']);
			$requestChildren[] = array(scXml::TAG => 'passwordhash', scXml::DATA => $settings['passwordhash']);
			$requestChildren[] = array(scXml::TAG => 'action', scXml::DATA => $data['action']);
			$requestChildren[] = array(scXml::TAG => 'amt', scXml::DATA => $data['amt']);
			$requestChildren[] = array(scXml::TAG => 'currencycode', scXml::DATA => $data['currencycode']);
			$requestChildren[] = array(scXml::TAG => 'trackid', scXml::DATA => $data['trackid']);
			$requestChildren[] = array(scXml::TAG => 'langid', scXml::DATA => $data['langid']);
			$requestChildren[] = array(scXml::TAG => 'responseurl', scXml::DATA => $data['responseurl']);
			$requestChildren[] = array(scXml::TAG => 'errorurl', scXml::DATA => $data['errorurl']);
			$requestChildren[] = array(scXml::TAG => 'udf1', scXml::DATA => $data['udf1']);
			
			$data = array(array(
				scXml::TAG => 'transaction',
				scXml::ATTRIBUTES => array('name' => $transactionName, 'version' => '1'),
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'request', 
						scXml::CHILDREN => $requestChildren
					)
				),
			));
			return scXml::xmlWriter($data);
		}
		
		protected function sendTransaction($settings, $paymentInitXml)
		{
			$url = 'https://' . $settings['webaddress'];
			if (!empty($settings['port']))
				$url .= ':' . $settings['port'];
			$url .= '/' . $settings['context'] . '/' . self::UBB_SERVLET;
			
			return scHttp::post($url, $paymentInitXml, array('sslVerifyPeer' => false)); // FIXME: is sslVerifyPeer needed
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			if (!isset($params['OrderId']) or empty($params['OrderId'])) {
				$params['OrderId'] = scOrdersInProgress::singleton()->getLastOrderId();
			}
			
			$result = new scPaymentProcessResult($params['OrderId']);
			if (isset($params['msg'])) {
				$result->setPaymentNotes($params['msg']);
			} else {
				$order = new scOrder($params['OrderId']);
				$result->setPaymentNotes($order->get('OrderPaymentNotes'));	// Set the same payment notes so that the customer can see the message
			}
			return $result;
		}
		
		/**
		 * Handles error return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processErrorReturn($params)
		{
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			$order = new scOrder($orderId);
			
			$result = new scPaymentProcessResult($orderId);
			$result->setPaymentNotes($order->get('OrderPaymentNotes'));	// Set the same payment notes so that the customer can see the message
			return $result;
		}
		
		/**
		 * Handles payment notification
		 * 
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processNotify($params)
		{
			static $responseStatusesMap = array(
				'CAPTURED' => scPaymentStatus::PAYMENT_STATUS_CAPTURE,
				'APPROVED' => scPaymentStatus::PAYMENT_STATUS_AUTH,
				'VOIDED' => scPaymentStatus::PAYMENT_STATUS_VOID,
				'NOT CAPTURED' => scPaymentStatus::PAYMENT_STATUS_DECLINED,
				'NOT APPROVED' => scPaymentStatus::PAYMENT_STATUS_DECLINED,
				'NOT VOIDED' => scPaymentStatus::PAYMENT_STATUS_NONE,
				'DENIED BY RISK' => scPaymentStatus::PAYMENT_STATUS_DECLINED,
				'FAILED AVS' => scPaymentStatus::PAYMENT_STATUS_DECLINED,
				'HOST TIMEOUT' => scPaymentStatus::PAYMENT_STATUS_DECLINED,
			);
			
			// Security check
			if ((ip2long($_SERVER['REMOTE_ADDR']) | 255) != (ip2long(self::UBB_IP_ADDRESS) | 255)) {
				throw new scPaymentProcessException(_i('UBB.bg: Notify from unknown server.'), $params + $_SERVER);
			}
			
			// Check if cancelled by user
			if ($params['result'] == 'CANCELED') {
				die('REDIRECT=' . scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_ABSOLUTE));
			}
			
			scPaymentProcess::checkParamsExist($params, array('paymentid'));
			
			$orderId = $this->getOrderIdByUbbPaymentId($params['paymentid']);
			
			// Check if this notification is for error
			if (isset($params['Error'])) {
				$result = new scPaymentProcessResult($orderId);
				$result->setAllTransactionDetails($params);
				$result->setPaymentNotes($params['ErrorText']);
				return $result;
			}
			
			if ($params['result'] == 'CAPTURED') {
				$paymentNotes = _t('Purchase');
			} else if ($params['result'] == 'APPROVED') {
				$paymentNotes = _t('Authorization');
			} else {
				$paymentNotes = $params['result'];
			}
			$paymentNotes .= ', ' . _t('Auth Code: ') . $params['auth'];
			
			// Try to process
			$result = new scPaymentProcessResult($orderId);
			if (isset($responseStatusesMap[$params['result']]))
				$result->setPaymentStatus($responseStatusesMap[$params['result']]);
			else
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			$result->setTransactionCode($params['tranid']);
			$result->setPaymentNotes($paymentNotes);
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
	 	protected function getOrderIdByUbbPaymentId($ubbPaymentId)
		{
			$sql = 'SELECT OrderID FROM Transactions JOIN TransactionDetails USING(TransactionID) WHERE ModuleID=? AND TransactionDetailName="PaymentID" AND TransactionDetailValue=?';
			$rows = miStaticDBUtil::execSelect($sql, array($this->_moduleId, $ubbPaymentId));
			if (count($rows) == 0)
				throw new scPaymentProcessException(sprintf(_i('UBB.bg: Cannot find order for PaymentID "%s".'), $ubbPaymentId));
			
			return $rows[0]['OrderID'];
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miRadioWidget',
					'data' => 'TransactionType',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'scModuleOptionMultiCheckboxWidget',
					'data' => 'SupportedCurrencyCodes',
				),
				array(
					'field' => 'scPaymentModuleUbbBgAliasesWidget',
					'data' => 'SelectedAlias',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'Aliases',	// This is set through the setup wizard
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('TransactionType')->setRadioButtons(array(self::PAYMENT_TYPE_PURCHASE => _t('Purchase'), self::PAYMENT_TYPE_AUTHORISATION => _i('Authorization')), self::PAYMENT_TYPE_PURCHASE);
			
			$currencies = array_keys(self::$currencyCodes);
			$currencies = array_combine($currencies, $currencies);
			$options->getOptionField('DefaultCurrencyCode')->setOptions($currencies, self::DEFAULT_CURRENCY_CODE);
			$options->getOptionField('SupportedCurrencyCodes')->setRadioButtons($currencies, self::DEFAULT_CURRENCY_CODE);
			$options->setTemplateFile('modules/payment/ubb_bg/options.tmpl');
			return $options;
		}
	}
	
	class scPaymentModuleUbbBgAliasesWidget extends miSelectWidget {
		public function getEditableControl()
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.ubb_bg.hostedpage');
			$options = scModuleOptions::getModuleOptions($moduleId);
			
			if (isset($options['Aliases']) and ($options['Aliases'] != '')) {
				$aliases = unserialize($options['Aliases']);
				$aliases = array_keys($aliases);
				$this->setOptions(array_combine($aliases, $aliases));
			} else {
				$aliases = array(0 => _t('Please use the setup wizard.'));
				$this->setOptions($aliases);
			}
			
			return parent::getEditableControl();
		}
	}
	
	class scPaymentModuleUbbBgResourceFile {
		protected static $xorKey = 'Those who profess to favour freedom and yet depreciate agitation are men who want rain without thunder and lightning';
		
		public static function getAliases($filename)
		{
			$tempFilename = tempnam('/tmp', 'ubb_resource');
			self::decryptZip(file_get_contents($filename), $tempFilename);
			
			$aliases = array();
			
			$zip = zip_open($tempFilename);
			while ($zipEntry = zip_read($zip)) {
				$name = zip_entry_name($zipEntry);
				if (substr($name, 0, 5) == 'TRAN_')
					continue;
				
				if (zip_entry_open($zip, $zipEntry, 'r')) {
					$xml = self::doXor(zip_entry_read($zipEntry, zip_entry_filesize($zipEntry)), self::$xorKey);
					zip_entry_close($zipEntry);
				}
				
				$name = substr($name, 0, -4); // Remove the .xml at the end
				$aliases[$name] = $xml;
			}
			zip_close($zip);
			
			unlink($tempFilename);
			
			return $aliases;
		}
		
		public static function parseSettings($xmlStr)
		{
			$xml = simplexml_load_string($xmlStr);
			
			$settings = array();
			foreach ($xml as $key => $value) {
				$settings[$key] = (string)$value;
			}
			return $settings;
		}
		
		protected static function decryptZip($contents, $outputFilename)
		{
			$decrypted = self::doXor($contents, self::$xorKey);
			file_put_contents($outputFilename, $decrypted);
		}
		
		protected static function doXor($input, $key)
		{
	    	$inputLen = strlen($input);
	    	$keyLen = strlen($key);
	    	
			$result = '';
	    	for ($i = 0; $i < $inputLen; $i++) {
	    		$result .= $input{$i} ^ $key{$i % $keyLen};
	    	}
			return $result;
		}
	}
?>