<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/ubb.php');
	
	class scPaymentModuleUbbBgSetup implements IAdminModule {
		/**
		 * Initializes the action handlers for this module
		 * 
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('acUbbBgSetupWizard', 'scPaymentModuleUbbBgSetupAction');
			$page->getDispatcher()->setActionHandler('acUbbBgUploadResourceFile', 'scPaymentModuleUbbBgUploadResourceFileAction');
			$page->getDispatcher()->setActionHandler('acReturnToUbbBgModule', 'scPaymentModuleUbbBgReturnToEditAction');
		}
		
		/**
		 * Returns the view that should be displayed
		 * 
		 * @return miDefaultView
		 */
		public function getView()
		{
			return new scView();
		}
	}
	
	class scPaymentModuleUbbBgSetupAction extends miAction {
		public function doAction()
		{
			$parser = new scTemplateParser();
			$parser->readTemplate('modules/payment/ubb_bg/ubb_setup.tmpl');
			
			$miMsg = new miMessage();
			$parser->assignArray($miMsg->getMessageTemplateVars());
			
			$html = $parser->templateParse();
			
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($controllerCommand);
		}
	}
	
	class scPaymentModuleUbbBgReturnToEditAction extends miAction
	{
		public function doAction()
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.ubb_bg.hostedpage');
			$paymentModuleId = scUtil::getDbValue('PaymentModules', 'PaymentModuleID', 'ModuleID', $moduleId, 0);
			
			$loc = 'payment_modules.php?action=dmEdit&PaymentModuleID=' . $paymentModuleId;
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($controllerCommand);
		}
	}
	
	class scPaymentModuleUbbBgUploadResourceFileAction extends miAction {
		public function doAction()
		{
			if (!function_exists('zip_open')) {
				$msg = _t('PHP extension ZIP is not installed. Please contact your hosting provider to enable it.');
				$this->redirectToUbbBgSetup($msg, miMessage::MSG_TYPE_ERROR);
				return;
			}
			
			$error = $_FILES['resourceFile']['error'];
			if ($error != UPLOAD_ERR_OK) {
				if ($error == UPLOAD_ERR_NO_FILE)
					$msg = _t('Please upload file.');
				else
					$msg = scFilesUtil::getUploadErrorDescription($error);
				$this->redirectToUbbBgSetup($msg);
				return;
			}
			
			$filename = $_FILES['resourceFile']['tmp_name'];
			
			$aliases = scPaymentModuleUbbBgResourceFile::getAliases($filename);
			reset($aliases);
			
			if (count($aliases) == 0) {
				$this->redirectToUbbBgSetup(_t('No terminal aliases found.'));
				return;
			}
			
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.payment.ubb_bg.hostedpage');
			scModuleOptions::setModuleOption($moduleId, 'Aliases', serialize($aliases));
			scModuleOptions::setModuleOption($moduleId, 'SelectedAlias', key($aliases));
			
			$msg = sprintf(_t('Success! Terminal aliases found: %s.'), implode(',', array_keys($aliases)));
			$this->redirectToUbbBgSetup($msg, miMessage::MSG_TYPE_INFO);
		}
		
		protected function redirectToUbbBgSetup($msg = '', $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$loc = scPaths::getCurrentScriptPathName() . '?ModuleName=com.summercart.payment.ubb_bg.hostedpage&action=acUbbBgSetupWizard';
			$loc = scUrlUtil::convertUrl($loc, scUrlUtil::URL_TYPE_DOCROOT);
			$loc = scUrlUtil::addQueryParams($loc, $this->_view->getRedirectParams($msg, $msgType));
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($controllerCommand);
		}
	}
?>