<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Wire (Bank) transfer payment processing module
	 */
	class scPaymentModuleWireTransfer implements IPaymentModule, IModuleOptions {
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array())
		{
			if ($contentType == self::CONTENT_TYPE_PAYMENT_FORM) {
				$t = new scTemplateParser();
				$t->readTemplate('modules/payment/wire/form.tmpl');
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('WireInstructions'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
				
				$hasBanksSection = new miTemplateParserSectionInfo('hasBanks', 0);
				
				$banks = @unserialize($options['WireInstructions']);
				
				// If the option value cannot be unserialized then we have a single bank
				if ($banks === false) {
					$t->assign('%%BANKWIREINSTRUCTIONS%%', $options['WireInstructions']);
					$t->addSectionInfo($hasBanksSection);
					return $t->templateParse();
				}
				
				// If we have a single bank which has no name then we should show only the bank instructions
				if ((count($banks) == 1) && (trim($banks[0]['BankName']) != '')) {
					$t->assign('%%BANKWIREINSTRUCTIONS%%', $banks[0]['BankWireInstructions']);
					$t->addSectionInfo($hasBanksSection);
					return $t->templateParse();
				}
				
				foreach ($banks as $rowIndex => &$bank) {
					$bank['HTML_BankWireInstructions'] = $bank['BankWireInstructions'];
					$bank['BankNumber'] = $rowIndex;
				}
				
				$t->assign('%%BANKWIREINSTRUCTIONS%%', '');
				$hasBanksSection->setIterationsCount(1);
				$hasBanksSection->addSubsection(new miTemplateParserSectionInfo('banks', count($banks), miUIUtil::templetizeAndEscapeRows(miUIUtil::transposeRows($banks))));
				$t->addSectionInfo($hasBanksSection);
				return $t->templateParse();
			}
		}
		
		public function hasCheckoutStep()
		{
			return false;
		}
		
		public function getPaymentCurrencyCode()
		{
			return null;
		}
		
		/**
		 * Process the payment
		 *
		 * @param string $payAmount the amount to be paid
		 * @param string $payCurrencyCode the currency
		 * @param scCart $cart the shopping cart object
		 * @param array $paramsPost any parameters posted by the user from the payment form
		 * @return scPaymentProcessResult the payment process result
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$result = new scPaymentProcessResult($orderId);
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_OFFLINE);
			
			if (!empty($paramsPost['WirePaymentBankName'])) {
				$result->setPaymentNotes(_t('Bank name: ') . $paramsPost['WirePaymentBankName']);
			}
			
			return $result;
		}
		
		public function getDataFields()
		{
			return array(
				array(
					'field' => 'scModuleOptionDynRowsWidget',
					'data' => 'RSBanks',
					'modelStrategy' => 'scViewMapperModuleOptionDynRows',
					'properties' => array(
						'template' => 'modules/payment/wire/banks_rs.tmpl',
						'fields' => array(
							array(
								'field' => 'scWebFormWidgetTextML',
								'data' => 'BankName',
							),
							array(
								'field' => 'scWebFormWidgetTextAreaML',
								'data' => 'BankWireInstructions',
							),
						),
					)
	        	),
			);
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = $this->getDataFields();
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->setTemplateFile('modules/payment/wire/options.tmpl');
			$options->setModelStrategies(array('RSBanks' => 'scViewMapperModuleOptionDynRows'));
			return $options;
		}
	}
	
	class scModuleOptionDynRowsWidget extends scDynRowsWidget {
		
		protected function validateDynRowsProperties($properties)
		{
			$this->_template = $properties['template'];
			$this->_rsDataFields = $properties['fields'];
		}
	}
	
	class scViewMapperModuleOptionDynRows extends scViewMapperDynRows {
		
		public function read()
		{
			$optionValues = $this->getModuleOptionsViewMapper()->get('WireInstructions');
			if (empty($optionValues)) {
				return;
			}
			
			foreach ($this->_rsProperties as $fieldName => $rsProperties) {
				
				$values = array();
				foreach ($optionValues['WireInstructions'] as $languageId => $value) {
					$unserializedValue = @unserialize($value);
					if ($unserializedValue === false) { // backwards compatability
						$rowData['BankName'][$languageId] = '';
						$rowData['BankWireInstructions'][$languageId] = $value;
						$values = array($rowData);
						continue;
					}
					
					foreach ($unserializedValue as $rowIndex => $rowData) {
						$values[$rowIndex]['BankName'][$languageId] = $rowData['BankName'];
						$values[$rowIndex]['BankWireInstructions'][$languageId] = $rowData['BankWireInstructions'];
					}
				}
				
				$this->_rsValues[$fieldName] = $values;
			}
			
			$this->getModuleOptionsViewMapper()->set('WireInstructions', $this->_rsValues);
		}
		
		public function update()
		{
			foreach ($this->_rsProperties as $fieldName => $rsProperties) {
				$values = $this->getModuleOptionsViewMapper()->get($fieldName);
				$this->saveDynRows($values[$fieldName], $rsProperties);
			}
		}
		
		protected function getModuleOptionsViewMapper()
		{
			static $viewMapper;
			if (is_null($viewMapper)) {
				$viewMapper = $this->_viewMapperController->getViewMapper('scViewMapperModuleOptions');
			}
			return $viewMapper;
		}
		
		protected function initRsProperties()
		{
			// Setup $this->_rsProperties
			$dataFields = $this->getModuleOptionsViewMapper()->getModule()->getDataFields();
			
			$className = get_class($this);
			foreach ($dataFields as $dataField) {
				if (!isset($dataField['modelStrategy']) or ($dataField['modelStrategy'] != $className))
					continue;
				$this->_rsProperties[$dataField['data']] = $dataField['properties'];
			}
		}
		
		protected function saveDynRows($rsValues, $rsProperties)
		{
			$optionsByLanguage = array();
			foreach ($rsValues as $rowIndex => $values) {
				
				foreach ($values['BankName'] as $languageId => $bankName) {
					$optionsByLanguage[$languageId][] = array(
						'BankName' => $bankName,
						'BankWireInstructions' => $values['BankWireInstructions'][$languageId],
					);
				}
			}
			
			miStaticDBUtil::startTransaction();
			
			$moduleId = $this->getModuleOptionsViewMapper()->getViewMapperController()->getDomainObject()->get('ModuleID');
			miStaticDBUtil::execDelete('ModuleOptionsML', 'ModuleID', $moduleId);
			
			foreach ($optionsByLanguage as &$optionValue) {
				$optionValue = serialize($optionValue);
			}
			scModuleOptions::createModuleOption($moduleId, 'WireInstructions', $optionsByLanguage);
			
			miStaticDBUtil::commit();
		}
	}
?>