<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Retrieves quotes from European Central Bank
	 */
	class scCurrencyRatesECB implements ICurrencyRates {
		const XML_URL = 'http://www.ecb.int/stats/eurofxref/eurofxref-daily.xml';
		
		/**
		 * Return the currency rates
		 * 
		 * @return array of the scCurrencyRate objects
		 * @throws miException
		 */
		public function getCurrencyRates()
		{
			$ratesXml = scHttp::get(self::XML_URL, array('timeout' => 15));
			$xml = new SimpleXMLElement($ratesXml);
			
			$timestamp = $this->getTimestamp((string)$xml->Cube->Cube['time']);
			
			$rates = array();
			foreach ($xml->Cube->Cube->Cube as $cube) {
				$rates[] = new scCurrencyRate('EUR', (string)$cube['currency'], (string)$cube['rate'], $timestamp);
			}
			
			return $rates;
		}
		
		/**
		 * Converts YYYY-MM-DD date into timestamp. Uses 12 o'clock for the time part
		 * 
		 * @param string $date the date to be converted
		 * @return int the timestamp
		 * @throws miException
		 */
		protected function getTimestamp($date)
		{
			list($year, $month, $day) = explode('-', $date, 3);
			if (!ctype_digit($year) or !ctype_digit($month) or !ctype_digit($day))
				throw new miException(sprintf('Invalid ECB rate date format: %s', $date));
			
			if (!checkdate($month, $day, $year))
				throw new miException(sprintf('Invalid ECB rate date: %s', $date));
			
			$timestamp = gmmktime(12, 0, 0, $month, $day, $year);
			return $timestamp;
		}
	}
?>