<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Retrieves quotes from Federal Reserve Bank of New Your
	 */
	class scCurrencyRatesRBA implements ICurrencyRates {
		const XML_URL = 'http://www.rba.gov.au/rss/rss-cb-exchange-rates.xml';
		
		/**
		 * Return the currency rates
		 * 
		 * @return array of the scCurrencyRate objects
		 * @throws miException
		 */
		public function getCurrencyRates()
		{
			$ratesXml = scHttp::get(self::XML_URL, array('timeout' => 15));
			$xml = new SimpleXMLElement($ratesXml);
			
			$rates = array();
			foreach ($xml->item as $item) {
				$dcElements = $item->children('http://purl.org/dc/elements/1.1/');
				$cbElements = $item->children('http://www.cbwiki.net/wiki/index.php/Specification_1.1');
				$timestamp = $this->getTimestamp((string)$dcElements->date);
				
				$rates[] = new scCurrencyRate((string)$cbElements->statistics->exchangeRate->baseCurrency, (string)$cbElements->statistics->exchangeRate->targetCurrency, (string)$cbElements->statistics->exchangeRate->value, $timestamp);
			}
			return $rates;
		}
		
		/**
		 * Converts the date into timestamp
		 * 
		 * @param string $date the date to be converted
		 * @return int the timestamp
		 * @throws miException
		 */
		protected function getTimestamp($date)
		{
			$pattern = '/(\d{4})-(\d{2})-(\d{2})/';
			if (preg_match($pattern, $date, $match)) {
				list($year, $month, $day) = array($match[1], $match[2], $match[3]);
            	$timestamp = gmmktime( 0, 0, 0, $month, $day, $year);
            	
				return $timestamp;
			}
			throw new miException(sprintf('Invalid date format: %s', $date));
		}
	}
?>