<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Retrieves quotes from Central Bank of the Republic of Turkey
	 */
	class scCurrencyRatesTCMB implements ICurrencyRates {
		const XML_URL = 'http://www.tcmb.gov.tr/kurlar/today.xml';
		
		/**
		 * Return the currency rates
		 * 
		 * @return array of the scCurrencyRate objects
		 * @throws miException
		 */
		public function getCurrencyRates()
		{
			$ratesXml = scHttp::get(self::XML_URL, array('timeout' => 15));
			$xml = new SimpleXMLElement($ratesXml);
			
			$timestamp = $this->getTimestamp((string)$xml->attributes()->Date);
			
			$rates = array();
			foreach ($xml->Currency as $currency) {
				if (!isset($currency->ForexSelling) or !isset($currency->Unit))
					continue;
				
				$forexSelling = (string)$currency->ForexSelling;
				if ($forexSelling == '')
					continue;
				
				$forexSelling /= (string)$currency->Unit;
				
				$rates[] = new scCurrencyRate((string)$currency->attributes()->CurrencyCode, 'TRY', $forexSelling, $timestamp);
			}
			return $rates;
		}
		
		/**
		 * Converts MM/DD/YYYY date into timestamp. Uses 15:30 EET (UTC+2) for the time part
		 * 
		 * @param string $date the date to be converted
		 * @return int the timestamp
		 * @throws miException
		 */
		protected function getTimestamp($date)
		{
			list($month, $day, $year) = explode('/', $date, 3);
			if (!ctype_digit($year) or !ctype_digit($month) or !ctype_digit($day))
				throw new miException(sprintf('Invalid Turkish Central Bank rate date format: %s', $date));
			
			if (!checkdate($month, $day, $year))
				throw new miException(sprintf('Invalid Turkish Central Bank rate date: %s', $date));
			
			// The currency rates are published 15:30 EET (UTC+2)
			// Converted to UTC this will be 13:30
			$timestamp = gmmktime(13, 30, 0, $month, $day, $year);
			return $timestamp;
		}
	}
?>