<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	class scEcontBg implements IShippingModule, IShippingModuleCodInfo, IModuleOptions {
		const ECONT_DELIVERY_TO_DOOR = 1;
		const ECONT_DELIVERY_TO_OFFICE = 2;
		
		protected $_moduleId;
		protected $_shippingModuleId;
		protected $_options;
		
		public function __construct($moduleId, $shippingModuleId)
		{
			$this->_moduleId = $moduleId;
			$this->_shippingModuleId = $shippingModuleId;
			$this->_options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
		}
		
		public function getShippingModuleImage()
		{
			return 'skins/default/modules/shipping/econt/logo_econt.png';
		}
		
		/** 
		 * Returns the status of shipments
		 *
		 * @param array $shipmentNumbers - shipment numbers
		 * @return array shipment status by shipment number
		 */
		/*public function getShipmentStatuses($shipmentNumbers) 
		{
			$xmlRequest = new SimpleXMLElement("<?xml version='1.0' encoding='UTF-8'?><request></request>");
			
			$xmlRequest->addChild('client');
				$xmlRequest->client->addChild('username');
				$xmlRequest->client->username = $this->_options['EEcontUsername'];
				$xmlRequest->client->addChild('password');
				$xmlRequest->client->password = $this->_options['EEcontPassword'];
			
			$xmlRequest->addChild('request_type', 'shipments');
			$xmlRequest->addChild('shipments');
			foreach ($shipmentNumbers as $shipmentNum) {
				$xmlRequest->shipments->addChild('num', $shipmentNum);
			}
				
			$request = $xmlRequest->asXML();
			
			$response = self::doRequest($request, econtConfig::EECONTURL . '/xml_service_tool.php');
			if (strpos($response,'API_ERR_LOGIN') !== false)
				throw new miException(_t('Invalid e-Econt username/password.'));
			
			$xmlResponse = new SimpleXMLElement($response);
			
			$shipmentStatuses = array();
			foreach ($xmlResponse->shipments->e as $shipment) {
				$shipmentNum = (string)$shipment->loading_num;
				$shipmentStatuses[$shipmentNum] = array(
					'storage'			=> (string)$shipment->storage,
					'receiver_person'	=> (string)$shipment->receiver_person,
					'receiver_phone'	=> (string)$shipment->receiver_phone,
					'receiver_courier'	=> (string)$shipment->receiver_courier,
					'receiver_courier_phone' => (string)$shipment->receiver_courier_phone,
					'receiver_time'		=> (string)$shipment->receiver_time,
					'CD_get_sum'		=> (string)$shipment->CD_get_sum,
					'CD_get_time'		=> (string)$shipment->CD_get_time,
					'error'				=> (string)$shipment->error,
					'error_code'		=> (string)$shipment->error_code
				);
			}
				
			return $shipmentStatuses;
		}*/
		
		
		/**
		 * sends http request to $url with $sRequest as post upload file and returns the response
		 *
		 * @param string $request
		 * @param string $url
		 * @param int $timeout timeout, in seconds
		 * @return string
		 */
		public static function doRequest($request, $url, $timeout = 12) 
		{
			if (!function_exists('sys_get_temp_dir')) {
				function sys_get_temp_dir() {
					if ($temp=getenv('TMP'))
						return $temp;
					if ($temp=getenv('TEMP'))
						return $temp;
					if ($temp=getenv('TMPDIR'))
						return $temp;
					$temp=tempnam(__FILE__,'');
					if (file_exists($temp)) {
						unlink($temp);
						return dirname($temp);
					}
					return null;
				}
			}
			
			$tmpFileName = tempnam(sys_get_temp_dir(), 'econt_request_');
			try {
				file_put_contents($tmpFileName, $request);	// FIXME: temp file?
		        $postData = array('file' => '@' . $tmpFileName);
				$response = scHttp::post($url, $postData, array('timeout' => $timeout));
			} catch (Exception $e) {
				unlink($tmpFileName);
				throw $e;
			}
			unlink($tmpFileName);
			return $response;
		}
		
		public function getShippingMethods() 
		{
			return array(
				self::ECONT_DELIVERY_TO_DOOR 	=> 'Econt delivery to address',
				self::ECONT_DELIVERY_TO_OFFICE 	=> 'Econt delivery to local Econt office'
			);
		}	
		
		/**
		 * Build the XML for loading request
		 * 
		 * @param array $loadings
		 * @param bool $validate
		 * @param bool $onlyCalculate
		 * @return string the XML for the request
		 */
		protected function buildLoadingsXml($loadings, $validate = true, $onlyCalculate = true)
		{
			$xmlRequest = new SimpleXMLElement("<?xml version='1.0' encoding='UTF-8'?><parcels></parcels>");
			$xmlRequest->addChild('is_summercart','true');
			$xmlRequest->addChild('system');
			$xmlRequest->system->addChild('validate');
			$xmlRequest->system->validate = ($validate) ? 1 : 0;
			$xmlRequest->system->addChild('response_type');
			$xmlRequest->system->response_type = 'XML';
			$xmlRequest->system->addChild('only_calculate');
			$xmlRequest->system->only_calculate = ($onlyCalculate) ? 1 : 0;
	
			$xmlRequest->addChild('client');
			$xmlRequest->client->addChild('username');
			$xmlRequest->client->username = $this->_options['EEcontUsername'];
			$xmlRequest->client->addChild('password');
			$xmlRequest->client->password = $this->_options['EEcontPassword'];
				
			$xmlRequest->addChild('loadings');
			foreach ($loadings as $loadingInfo) {
				$this->addLoadingToXml($loadingInfo, $xmlRequest->loadings);
			}
				
			return $xmlRequest->asXML();
		}
		
		/**
		 * adds a row to $xmlElement with data from $loading
		 *
		 * @param array $loading
		 * @param SimpleXMLElement $XML
		 */
		protected function addLoadingToXml($loading, SimpleXMLElement $xmlElement)
		{
			if (empty($loading))
				return;
			
			$row = $xmlElement->addChild('row');
			
			// sender
			$row->addChild('sender');
			foreach ($loading['sender'] as $name => $value) {
				$row->sender->addChild($name);
				$row->sender->$name = $value;
			}
			
			// receiver
			$row->addChild('receiver');
			foreach ($loading['receiver'] as $name => $value) {
				$row->receiver->addChild($name);
				$row->receiver->$name = $value;
			}
			
			// shipment
			$row->addChild('shipment');
			foreach ($loading['shipment'] as $name => $value) {
				// truncate package description to 70 chars
				if($name == 'description' && strlen(utf8_decode($value)) > 70) {
					$value = scUtf8::substr($value, 0, 67) . '...';
				}
				$row->shipment->addChild($name);
				$row->shipment->$name = $value;
			}
				
			// payment
			$row->addChild('payment');
			foreach ($loading['payment'] as $name => $value) {
				$row->payment->addChild($name);
				$row->payment->$name = $value;
			}
			
			// services
			$row->addChild('services');
			
			// Receipt (Обратна разписка)
			if (!empty($loading['services']['dc'])) {
				$row->services->addChild('dc');
				$row->services->dc = $loading['services']['dc'];
			}
			// Declared value (Обявена стойност)
			if (!empty($loading['services']['oc'])) {
				$row->services->addChild('oc');
				$row->services->oc = $loading['services']['oc'];
			}
			// Collect on delivery
			if (!empty($loading['services']['cd'])) {
				$row->services->addChild('cd');
				$row->services->cd = $loading['services']['cd'];
				$row->services->cd->addAttribute('type', 'GET');
			}
			if (!empty($loading['services']['cd_agreement_num'])) {
				$row->services->addChild('cd_agreement_num');
				$row->services->cd_agreement_num = $loading['services']['cd_agreement_num'];
			}
		}
		
		public function getReceiverSharePercent()
		{
			$sharePercent = (float) $this->_options['SenderPayPercent'];
			if ($sharePercent > 100)
				$sharePercent = 100;
			if ($sharePercent < 0)
				$sharePercent = 0;
			return (100 - $sharePercent);
		}
		
		/**
		 * Makes request to econt and calculated the shipping
		 * 
		 * @param scCart $cart
		 * @param array $shippingMethods
		 * @param bool $withCoD
		 */
		public function calcShipping(scCart $cart, $shippingMethods, $withCoD = false)
		{
			$currencyBgn = new scCurrency('BGN');
			
			$customerInfo = $cart->getCustomerInfo();
			$declaredValueForShipping = $cart->getOrderTotal()->getDeclaredValueForShipping($currencyBgn);
			
			$baseLoading = array(
				'sender' => array(
					'name' => $this->_options['CompanyName'],
					'name_person' => $this->_options['PersonName'],
					'phone_num' => trim($this->_options['Phone']),
					'city' => trim($this->_options['City']),
					'post_code' => trim($this->_options['Zip']),
					'street' => trim($this->_options['Address1'] . ' ' . $this->_options['Address2'])
				),
				'receiver' => array(
					'city' => $customerInfo['CustomerShippingCity'],
					'post_code' => $customerInfo['CustomerShippingZip'],
				),
				'shipment' => array(
					'shipment_type' => 'PACK',
					'pack_count' => '1',
					'weight' => $cart->getTotalWeight(scWeightUnits::WEIGHT_UNIT_KGS),
				),
				'payment' => array(
					'side' => 'SENDER',
					'method' => $this->_options['PaymentMethod'],
					'key_word' => $this->_options['CreditNum'],
					'share_percent' => $this->getReceiverSharePercent(),
					// if share_percent is 100 the payment side will be converted to RECEIVER and the method to CASH 
				),
				'services' => array(
					'dc' => ($this->_options['DeliveryReceipt'] == 'YES') ? 'ON' : '',
					'oc' => ($this->_options['OCMinAmount'] <= $declaredValueForShipping) ? $declaredValueForShipping : '',
				)
			);
			
			if ($withCoD) {
				$totalPrice = $cart->getOrderTotal()->getGrandTotalInCurrency($currencyBgn);
				$shippingPrice = $cart->getOrderTotal()->getSummedOrderTotalLinesAmount(IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING)->get($currencyBgn);
				
				$baseLoading['services']['cd'] = $totalPrice - $shippingPrice;
				if (!empty($this->_options['CDAgreementNum']))
					$baseLoading['services']['cd_agreement_num'] = $this->_options['CDAgreementNum'];
			}
			
			// door or office
			$loadings = array(0 => $baseLoading, 1 => $baseLoading);
			if ($this->_options['PickupFrom'] == 'OFFICE') {
				// to door
				$loadings[0]['shipment']['tariff_code'] = 3;
				$loadings[0]['shipment']['tariff_sub_code'] = 'OFFICE_DOOR';
				// to office
				$loadings[1]['shipment']['tariff_code'] = 2;
				$loadings[1]['shipment']['tariff_sub_code'] = 'OFFICE_OFFICE';
			} else {
				
				if ($baseLoading['sender']['post_code'] == $baseLoading['receiver']['post_code'])
					$tariffCode = 1;	// City courier
				else
					$tariffCode = 4;	// Whole country
				
				// to door
				$loadings[0]['shipment']['tariff_code'] = $tariffCode;
				$loadings[0]['shipment']['tariff_sub_code'] = 'DOOR_DOOR';
				// to office
				$loadings[1]['shipment']['tariff_code'] = 3;
				$loadings[1]['shipment']['tariff_sub_code'] = 'DOOR_OFFICE';
			}
			
			// do request
			$request = $this->buildLoadingsXml($loadings);
			
			return self::doRequest($request, econtConfig::EECONTURL . '/xml_parcel_import.php');
			
			/*
			 * Result pricing has four components, based on the following:
			 * C: Куриер (Courrier)
			 * DC: Обратна разписка (Receipt)
			 * OC: Обявена стойност (Declared Value)
			 * CD: Наложен платеж (Collect on delivery)
			 * 
			 * IMPORTANT: The prices returned from Econt include VAT.
			 */
		}
		
		public function getShippingRates(scCart $cart, $shippingMethods) 
		{
			// Econt works only in Bulgaria so if shipping address is not in Bulgaria don't even try to get rates
			$customerInfo = $cart->getCustomerInfo();
			$countryCode = scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']);
			if (strtoupper($countryCode) != 'BG') {
				return array();
			}
			
			// decide shipment types
			$toOfficeAllowed = (array_search(self::ECONT_DELIVERY_TO_OFFICE, $shippingMethods) !== false) ? true : false;
			$toDoorAllowed	 = (array_search(self::ECONT_DELIVERY_TO_DOOR,   $shippingMethods) !== false) ? true : false;
			if (!$toDoorAllowed && !$toOfficeAllowed) 
				return array();
			
			$response = $this->calcShipping($cart, $shippingMethods, false);
			
			try {
				$this->checkResponseForErrors($response);
			} catch (miException $e) {
				return array(self::SHIPPING_RATES_MESSAGE => $e->getMessage());
			}
			
			$shippingRates = array(
				self::SHIPPING_RATES => array(),
				self::SHIPPING_RATES_CURRENCY => 'BGN',
			);
			
			$currencyBgn = new scCurrency('BGN');
			$xmlResponse = new SimpleXMLElement($response);
			$e = $xmlResponse->result->e;
			
			// to door
			if ($toDoorAllowed) {
				$loadingPrice = (float)$e[0]->loading_price->receiver_total;
				$loadingCurrencyCode = $this->convertCurrencySign((string)$xmlResponse->result->e[0]->loading_price->currency);
				$shippingRates[self::SHIPPING_RATES][self::ECONT_DELIVERY_TO_DOOR] = scCurrencyUtil::convert($loadingPrice, $loadingCurrencyCode, $currencyBgn);
			}
			// to office
			if ($toOfficeAllowed) {
				$loadingPrice = (float)$e[1]->loading_price->receiver_total;
				$loadingCurrencyCode = $this->convertCurrencySign((string)$xmlResponse->result->e[1]->loading_price->currency);
				$shippingRates[self::SHIPPING_RATES][self::ECONT_DELIVERY_TO_OFFICE] = scCurrencyUtil::convert($loadingPrice, $loadingCurrencyCode, $currencyBgn);
			}
			
			return $shippingRates;
		}
		
		protected function checkResponseForErrors($response)
		{
			if ($response === false)
				throw new miException(_t('Connection Error.'));
			
			$xmlResponse = new SimpleXMLElement($response);
			$errorCode = (isset($xmlResponse->result->e[0]->error_code) ? (string)$xmlResponse->result->e[0]->error_code : '');
			if (!empty($errorCode) && ('0' != $errorCode)) {
				$err = econtConfig::getXmlApiError($xmlResponse);
				throw new miException($err);
			}
			
			if (empty($xmlResponse->result)) {
				if (strpos($response, 'API_ERR_LOGIN') !== false)
					throw new miException(_t('Invalid e-Econt username/password.'));
				else 
					throw new miException(_t('Invalid request.'));
			}
		}
		
		public function getCodFee(scCart $cart)
		{
			$shippingMethods = $this->getShippingMethods();
			
			$response = $this->calcShipping($cart, $shippingMethods, true);
			$this->checkResponseForErrors($response);
			
			$xmlResponse = new SimpleXMLElement($response);
			
			$shippingMethod = $cart->getShippingMethod();
			$e = $xmlResponse->result->e;
			if ($shippingMethod == self::ECONT_DELIVERY_TO_DOOR)
				$codPremium = (float)$e[0]->loading_price->CD;
			else
				$codPremium = (float)$e[1]->loading_price->CD;
				
			// apply shared shipping payment
			$codPremium = $codPremium * $this->getReceiverSharePercent() / 100;
			
			$amount = scMultiCurrencyAmount::factory();
			$amount->set(new scCurrency('BGN'), $codPremium);
			return $amount;
		}
		
		/**
		 * Converts currency sign from econt response to standard currency code
		 *
		 * @param string $currency
		 * @return string
		 */
		public static function convertCurrencySign($currency)
		{
			$currencyMapping = array(
				 '€' => 'EUR',
				 'лв' => 'BGN',
				 'leu' => 'RON',
				 '?' => 'GBP',
				 '$' => 'USD',
			);
			
			if (array_key_exists($currency, $currencyMapping))
				return $currencyMapping[$currency];
			else 
				return $currency;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				// auth
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'EEcontUsername',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'EEcontPassword',
				),
				// contact info
				array(
					'field' => 'miTextWidget',
					'data' => 'CompanyName',
					//'validator' => 'scValidatorNoEmpty'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'PersonName',
					//'validator' => 'scValidatorNoEmpty'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Phone',
					//'validator' => array('econtValidatorPhone')
				),
				//  address
				array(
					'field' => 'miSelectWidget',
					'data' => 'Country',
					'properties' => array('code' => 'country')
        		),
				array(
					'field' => 'miTextWidget',
					'data' => 'City',
					//'validator' => array('econtValidatorOptionsAddress','scValidatorNoEmpty')
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Zip',
					//'validator' => array('scValidatorNoEmpty')
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Address1',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Address2',
				),
				// options
				array(
					'field' => 'miRadioWidget',
					'data' => 'PickupFrom'
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'DeliveryReceipt'
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'PaymentMethod',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'OCMinAmount',
					//'validator' => array('miValidatorDecimal')
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'CreditNum'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'CDAgreementNum'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'SenderPayPercent'
				),
			);
			
			$moduleOptions = new scModuleOptionsInfo();
			$moduleOptions->buildOptionFields($fieldsDescriptions, $webForm);
			$moduleOptions->setTemplateFile('modules/shipping/econt/options.tmpl');
			
			// options
			$moduleOptions->getOptionField('PickupFrom')->setRadioButtons(
				array('DOOR' => _t('Door'), 'OFFICE' => _t('Office')), 'DOOR');
			$moduleOptions->getOptionField('PaymentMethod')->setRadioButtons(
				array('CASH' => _t('Cash'), 'CREDIT' => _t('Credit')), 'CASH');
			$moduleOptions->getOptionField('DeliveryReceipt')->setRadioButtons(
				array('YES' => _t('Yes'), 'NO' => _t('No')), 'NO');
			
			$moduleOptions->getOptionField('Country')->setOptions(array(0 => '') + scCountry::getCountryNames());
				
			return $moduleOptions;
		}
	}

	// FIXME Full remake of this validator is required. Should use different tables
	class econtValidatorOptionsAddress extends miValidator {
		
		protected $config = null;
		public function  __construct(miWebForm &$webForm, $fieldName) {
			$this->config = new econtConfig();
			parent::__construct($webForm,$fieldName);
		}
		
		public function validate(miWebFormErrorsHandler &$errors) {
			return;
			/*// validate only for BG
			if(scCountry::getCountryCode3($this->_webForm->getSubmittedData('ModuleOptionCountry')) != 'BGR') 
				return;
			
			$address['city'] =  $this->_webForm->getSubmittedData('ModuleOptionCity');
			$address['zip']  =  $this->_webForm->getSubmittedData('ModuleOptionZip');
			
			// check if empty
			if(empty($address['city']) || empty($address['zip'])) 
				return;
			
			try {
				// check tables
				$count = miStaticDBUtil::execSelect(
					'SELECT COUNT(*) as count FROM ' . $this->config->tableNames['cities'] . '  
					UNION SELECT COUNT(*) as count FROM ' . $this->config->tableNames['regions']
				);
				if($count[0]['count'] == 0 || $count[1]['count'] == 0) {
					return;
				}
				
				// city
				$cityQuery = 'SELECT Name, EcontCityPostCode FROM ' . $this->config->tableNames['cities'] . 
							'WHERE LOWER(Name) = LOWER(?) OR LOWER(NameEN) = LOWER(?)';

				$cityResult = miStaticDBUtil::execSelect($cityQuery,array($address['city'],$address['city']));
								
				if(sizeof($cityResult) == 0) {
					$errors->addError('ModuleOptionCity', _t('Invalid city.'));
					return;
				}
				
				// zip
				$zipQuery = 'SELECT c.Name, r.EcontCityRegionPostCode FROM ' . $this->config->tableNames['cities'] . ' c ' . 
							' LEFT JOIN ' . $this->config->tableNames['regions'] .' r USING(EcontCityID) ' . 
							' WHERE r.EcontCityRegionPostCode = ? OR c.EcontCityPostCode = ? ';
				
				$zipResult = miStaticDBUtil::execSelect($zipQuery,array($address['zip'],$address['zip']));
				if(sizeof($zipResult) == 0) {
					$errors->addError('ModuleOptionZip',_t('Invalid zip.'));
					return;
				}
				
				if($zipResult[0]['Name'] != $address['city']) {
					$errors->addError('ModuleOptionZip',_t('City and zip do not match.'));
					return;
				}
			}
			catch (miDBException $e) {
				return ;
			}*/
		}
	}
	
	class econtValidatorPhone extends miValidator
	{
		public function validate(miWebFormErrorsHandler &$errors) {
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (preg_match('/^[1-9]{5,}([ ]*,[ ]*[\d]{5,})*$/', $fieldValue))
				return;
			
			$errors->addError($this->_fieldName, _t('Phone is not valid - digits only, multiple numbers are seperated with ",".'));
		}
	}

	// FIXME Should remove this class
	class econtConfig
	{
		const EECONTURL = 'http://www.econt.com/e-econt';
		
		public $tableNames = array();
		public function __construct()
		{
			$this->tableNames['cities'] = 'EcontCities';
			$this->tableNames['regions'] = 'EcontCityRegions';
		}
		
		/**
		 * mapping from econt API errors to text
		 * @param string $errorCode
		 * @return string
		 */
		public static function getXmlApiError($xml)
		{
			$message = (string)$xml->result->e->error;
			if ((scMultiLanguage::singleton()->getCurrentLanguageCode() == 'BG') && !empty($message)) {
				return $message;
			}
			$errorCode = (string)$xml->result->e->error_code;
			
			$errors = array(
				'API_ERR_LOADING_NO_DATA' => 'No loadings to import.',
				'API_ERR_SENDER_NODE' => 'Missing <sender> in XML file.',
				'API_ERR_RECEIVER_NODE' => 'Missing <receiver> in XML file.',
				'API_ERR_SHIPMENT_NODE' => 'Missing <shipment> in XML file.',
				'API_ERR_PAYMENT_NODE' => 'Missing <payment> in XML file.',
				'API_ERR_SERVICES_NODE' => 'Missing <services> in XML file.',
				'API_ERR_SENDER_NAME' => 'Invalid sender name.',
				'API_ERR_SENDER_PHONE' => 'Invalid sender phone.',
				'API_ERR_SENDER_CITY' => 'Invalid sender city.',
				'API_ERR_SENDER_OFFICE' => 'Invalid sender office.',
				'API_ERR_SENDER_QUARTER' => 'Invalid sender quarter.',
				'API_ERR_SENDER_STREET' => 'Invalid sender street.',
				'API_ERR_SENDER_ADDRESS' => 'Invalid sender address.',
				'API_ERR_RECEIVER_NAME' => 'Invalid receiver name.',
				'API_ERR_RECEIVER_PHONE' => 'Invalid receiver phone.',
				'API_ERR_RECEIVER_CITY' => 'Invalid receiver city.',
				'API_ERR_RECEIVER_OFFICE' => 'Invalid receiver office.',
				'API_ERR_RECEIVER_QUARTER' => 'Invalid receiver quarter.',
				'API_ERR_RECEIVER_STREET' => 'Invalid receiver street.',
				'API_ERR_RECEIVER_ADDRESS' => 'Invalid receiver address.',
				'API_ERR_RECEIVER_SMS_PHONE' => 'Invalid SMS notify number.',
				'API_ERR_SHIPMENT_DESCRIPTION' => 'Invalid package description.',
				'API_ERR_SHIPMENT_PACK_COUNT' => 'Invalid pack count.',
				'API_ERR_SHIPMENT_WEIGHT' => 'Invalid weight.',
				'API_ERR_SHIPMENT_TARIFF_CODE' => 'Invalid tariff code.',
				'API_ERR_PAYMENT_SIDE' => 'Invalid payment side.',
				'API_ERR_INVALID_CLIENT_NUMBER' => 'Invalid client number.',
				'API_ERR_WRONG_CLIENT_NUMBER' => 'Invalid client number.',
				'API_ERR_PAYMENT_METHOD' => 'Invalid payment method.',
				'API_ERR_PAYMENT_METHOD_SENDER' => 'Sender\'s payment method is invalid.',
				'API_ERR_TARIFF_CODE_1' => 'Incorrect tariff code.',
				'API_ERR_TARIFF_CODE_2' => 'Incorrect tariff code.',
				'API_ERR_TARIFF_CODE_3' => 'Incorrect tariff code.',
				'API_ERR_TARIFF_CODE_4' => 'Incorrect tariff code.',
				'API_ERR_TARIFF_CODE_8' => 'Incorrect tariff code.',
				'API_ERR_SHIPMENT_TYPE' => 'Incorrect shipment type.',
				'API_ERR_SERVICE_CD' => 'Invalid CD amount.',
				'API_ERR_SERVICE_CD_AGREEMENT' => 'Invalid CD agreement number.',
				'API_ERR_SERVICE_CD_AGREEMENT_WRONG_CLIENT' => 'Invalid CD agreement number.',
			);
			
			if (array_key_exists($errorCode,$errors))
				return $errors[$errorCode];
			else 
				return $errorCode;
		}
	}
?>
