<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */

	class scSpeedyApi extends SoapClient {
		const SPEEDY_WS_URL = 'https://www.speedy.bg/eps/main01.wsdl';
		
		protected $_sessionId;
		protected $_clientId;
		
		protected $_options;
		
		public function __construct()
		{
			try {
				// on some php instalations SOAP wsdl and HTTPS do not work
				//$wsdlUrl = scPaths::getUrl('/modules/shipping/speedybg/speedy.wsdl');
				$wsdl = dirname(__FILE__) . '/speedy.wsdl';
				parent::__construct($wsdl);
			} catch (SoapFault $e) {
				throw new scSpeedyApiException(sprintf(_i('Could not connect to Speedy server. Additional information: %s: %s'), $e->getCode(), $e->getMessage()));
			}
		}
		
		public function makeCall($function, $parameters = array()) 
		{
			try {
				$parameters = array_merge(array('sessionId' => $this->_sessionId), $parameters);
				$return = $this->$function($parameters);
				return $return->return;
			} catch (SoapFault $e) {
				$message = $e->getMessage();
				if (isset($e->detail->PickingValidationException->errorType)) {
					$message = sprintf('%s. (%s)', $this->getErrorDescription($e->detail->PickingValidationException->errorType), $e->detail->PickingValidationException->errorType);
				}
				if (isset($e->detail->PickingValidationException->errorDescription)) {
					$message .= ': ' . $e->detail->PickingValidationException->errorDescription;
				}
				throw new scSpeedyApiException($message, $e->getCode());
			}
		}
		
		public function getErrorDescription($errorType)
		{
			$errors = array(
				'COMMON_ERROR' => 'Обща валидационна грешка (по принцип в "errorDescription" трябва да има допълнителна информация)',
				'AMOUNT_INSURANCE_OUT_OF_RANGE' => 'Застрахователната стойност е извън позволените граници',
				'AMOUNT_CASH_ON_DELIVERY_OUT_OF_RANGE' => 'Стойността на Наложения платеж е извън позволените граници',
				'SENDER_HAS_NO_ANNEX_FOR_CASH_ON_DELIVERY' => 'Подателят няма договор/анекс за Наложен платеж',
				'INSURANCE_CANNOT_BE_LESS_THAN_CASH_ON_DELIVERY' => 'Застрахователната стойност не може да бъде по-малка от Наложения платеж',
				'WEIGHT_NOT_IN_RANGE' => 'Теглото е извън позволените граници',
				'DOCUMENTS_NOT_ALLOWED_FOR_THIS_SERVICE_AND_SITES' => 'За избраните куриерска услуга и населени места, не са позволени документни пратки',
				'DOCUMENT_PICKINGS_CAN_CONSIST_OF_ONLY_ONE_PARCEL' => 'Документните пратки може да бъдат само еднопакетни',
				'DOCUMENT_PICKINGS_ARE_NOT_ALLOWED_TO_HAVE_INSURANCE' => 'Документните пратки не може да бъдат застраховани',
				'DOCUMENT_PICKINGS_CANNOT_BE_PALLETIZED' => 'Документните пратки не може да бъдат палетизирани',
				'FIXED_TIME_DELIVERY_NOT_ALLOWED_FOR_RECEIVER_SITE' => 'Допълнителната услуга "Фиксиран час на доставка" не е позволена за населеното място на получателя',
				'FIXED_TIME_DELIVERY_NOT_ALLOWED_FOR_SERVICE' => 'Допълнителната услуга "Фиксиран час на доставка" не е позволена за услугата',
				'INVALID_FIXED_TIME_FOR_DELIVERY' => 'Невалидна стойност за "Фиксиран час на доставка"',
				'NO_MORE_THAN_3_PHONE_NUMBERS_EXPECTED' => 'Не са позволени по повече от три телефонни номера за клиент',
				'INVALID_PHONE_NUMBER' => 'Невалиден телефонен номер',
				'LOGGED_CLIENT_OBJECT_MUST_BE_PAYER_OR_SENDER' => 'Фирменият обект на логнатия потребител трябва да бъде или платец, или подател',
				'INVALID_PARCELS_INFO' => 'Подадените "parcels" са с невалидни стойности',
				'INVALID_TAKING_DATE' => 'Неправилна "Дата на взимане"',
				'INVALID_RET_TO_CLIENT_ID' => ' Невалиден/непозволен клиент (този, който трябва да получи обратните документи и/или обратната разписка)',
				'MISSING_REQUIRED_VALUE_FIXED_TIME_FOR_DELIVERY' => 'Липсваща задължителна стойност: Фиксиран час на доставка',
				'MISSING_REQUIRED_VALUE_CONTENTS' => 'Липсваща задължителна стойност: Съдържание',
				'MISSING_REQUIRED_VALUE_PACKING' => 'Липсваща задължителна стойност: Опаковка',
				'MISSING_REQUIRED_VALUE_PARTNER_NAME' => 'Липсваща задължителна стойност: Име на клиент',
				'MISSING_REQUIRED_VALUE_ADDRESS' => 'Липсваща задължителна стойност: Адрес',
				'MISSING_REQUIRED_VALUE_PHONE_SENDER' => 'Липсваща задължителна стойност: Телефон на подателя',
				'MISSING_REQUIRED_VALUE_PHONE_RECEIVER' => 'Липсваща задължителна стойност: Телефон на получателя',
				'VALUE_OUT_OF_RANGE_PARCEL_COUNT' => 'Стойността е извън норма: Брой пакети',
				'VALUE_OUT_OF_RANGE_SIZE' => 'Стойността е извън норма: Размер на пратката',
				'VALUE_OUT_OF_RANGE_PACK_ID' => 'Стойността е извън норма: Номер на опаковка',
				'VALUE_OUT_OF_RANGE_CONTENTS' => 'Стойността е извън норма: Съдържание',
				'VALUE_OUT_OF_RANGE_PACKING' => 'Стойността е извън норма: Опаковка',
				'VALUE_OUT_OF_RANGE_NOTE_CLIENT' => 'Стойността е извън норма: Клиентска забележка',
				'VALUE_OUT_OF_RANGE_CLIENT_FIELD' => 'Стойността е извън норма: Поле за клиент (в "errorDescription" е описано кое точно)',
				'VALUE_OUT_OF_RANGE_ADDRESS_FIELD' => 'Стойността е извън норма: Поле за адрес (в "errorDescription" е описано кое точно)'
			);
			
			if (isset($errors[$errorType]))
				return $errors[$errorType];
			return '';
		}
		
		public function setOptions($options)
		{
			$this->_options = $options;
		}
		
		public function initSession()
		{
			if (!isset($this->_options['Username']) or !isset($this->_options['Password'])) {
				throw new scSpeedyApiException(_t('Please enter username and password.'));
			}
			
			try {
				$loginResult = $this->login(array('username' => $this->_options['Username'], 'password' => $this->_options['Password']));
				$this->_sessionId = $loginResult->return->sessionId;
				$this->_clientId = $loginResult->return->clientId;
			} catch (SoapFault $e) {
				throw new scSpeedyApiException(_t('Could not authenticate to Speedy.'));
			}
		}
		
		public function getClientId()
		{
			return $this->_clientId;
		}
	}
	
	class scSpeedyApiException extends miException {}
?>