<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	class scSpeedyShippingLabelModule implements IShippingLabelModule {
		
		const CHOOSE_OPTIONS_TEMPLATE = 'modules/shipping/speedybg/print_options.tmpl';
		
		public function getShippingLabel($orderId)
		{
			$shippingLabel = new scSpeedyShippingLabel($orderId);
			return $shippingLabel;
		}
		
		public function handlePrintAction($orderId)
		{
			$shippingLabel = $this->getShippingLabel($orderId);
			
			// DELETE cache
			if (miHasParam('deleteCache')) {
				
				// remove the slip cachefile
				$fileName = $shippingLabel->getCacheFileName($orderId);
				if (scShippingLabelCache::checkCacheExists($fileName)) {
					$filePath = scShippingLabelCache::getCacheFilePath($fileName);
					unlink($filePath);
					
					// remove all labels cache files
					$moduleName = 'com.summercart.shipping.speedybg';
					$moduleId = scModulesFactory::getModuleIdByName($moduleName);
					$labels = scOrderDetailsUtil::getDetail($orderId, $moduleId, 'ShippingLabels');
					$labels = unserialize($labels);
					foreach ($labels as $label) {
						$fileName = $shippingLabel->getCacheFileName($orderId, array('documentType' => IShippingLabel::DOCUMENT_TYPE_LABEL, 'parcelId' => $label));
						$filePath = scShippingLabelCache::getCacheFilePath($fileName);
						unlink($filePath);
					}
					
					// remove the data in the database
					scOrderDetailsUtil::removeDetail($orderId, $moduleId, 'ShippingLabels');
				}
				scUtil::redirect('orders.php?action=dmPrintSinglePackingSlip&OrderID='.$orderId);
				die;
			}
			
			// if we have parcelId get the label
			if (miHasParam('parcelId')) {
				$fileName = $shippingLabel->getCacheFileName($orderId, array('documentType'=>IShippingLabel::DOCUMENT_TYPE_LABEL, 'parcelId' => miGetParam('parcelId')));
				$this->getPrintContent($fileName, $orderId);
			}
			
			// print the shippingLabel
			if (miHasParam('printShippingLabel')) {
				$fileName = $shippingLabel->getCacheFileName($orderId);
				$this->getPrintContent($fileName, $orderId);
			}
			
			// if we have choosen parcelsCount get content from shippinglabel and print it out
			$parcelsCount = (int)miGetParamDefault('parcelsCount', 0);
			if ($parcelsCount > 0) {
				$shippingLabel->getContent($orderId, array('parcelsCount' => $parcelsCount, 'pickupDate' => miGetParam('pickupDate')));
				$order = new scOrder($orderId);
				$fileName = $shippingLabel->getCacheFileName($orderId);
				// FIXME: something is missing
			}
			
			// just show select options html 
			$html = $this->getOptionsSelectHtml($orderId);
			echo $html;
			die;
		}
		
		public function getPrintActionOpenType($orderId)
		{
			return IShippingLabelModule::OPEN_TYPE_INLINE;
		}
		
		protected function getOptionsSelectHtml($orderId)
		{
			$order = new scOrder($orderId);
				
			// Get shipping label implementation
			$shippingLabel = $this->getShippingLabel($orderId);
			
			// Create cache if none exists for that order
			$fileName = $shippingLabel->getCacheFileName($orderId);
			if (!scShippingLabelCache::checkCacheExists($fileName)) {
				$selectCountSection = new miTemplateParserSectionInfo('selectCount', 1);
				$cancelShippingLabelSection = new miTemplateParserSectionInfo('cancelShippingLabel', 0);
				$labelsSection = new miTemplateParserSectionInfo('labels', 0);
				$msg = _t('Please choose parcels count');
			} else {
				$moduleName = 'com.summercart.shipping.speedybg';
				$moduleId = scModulesFactory::getModuleIdByName($moduleName);
				$labels = scOrderDetailsUtil::getDetail($orderId, $moduleId, 'ShippingLabels');
				$labels = unserialize($labels);

				$selectCountSection = new miTemplateParserSectionInfo('selectCount', 0);
				$cancelShippingLabelSection = new miTemplateParserSectionInfo('cancelShippingLabel', 1);
				$labelsForSection = array();
				foreach ($labels as $label) {
					$labelsForSection[] = array(
						'labelId' => $label,
					);
				}
				$labelsSection = new miTemplateParserSectionInfo('labels', count($labels), miUIUtil::templetizeRows(miUIUtil::transposeRows($labelsForSection)));
				$msg = '';
			}
			$headerText = sprintf(_t('Print packing slip and labels for order #%d'), $orderId);
			
			$template = new scTemplateParser();
			$template->assignArray(array(
				'%%ORDERDATE%%' => scDateTime::getLocalFormattedTimestamp($order->get('OrderTimestamp'), true),
				'%%PICKUP_DATES%%' => $this->getPickupDatesHtml(7),
				'%%ORDERID%%' => $orderId,
				'%%MSG%%' => $msg,
				'%%HEADER_TEXT%%' => $headerText,
			));
			$template->setSectionInfos(array(
				'selectCount' => $selectCountSection,
				'cancelShippingLabel' => $cancelShippingLabelSection,
				'labels' => $labelsSection,
			));
			$template->readTemplate(self::CHOOSE_OPTIONS_TEMPLATE);
			$template->templateShow();
			die;
		}
		
		protected function getPickupDatesHtml($numDays)
		{
			scDateTime::setTimezone();
			
			$html = '';
			for ($i = 0; $i < $numDays; $i++) {
				$date = date('Y-m-d', time() + $i*86400);
				$html .= '<option value="' . $date .'">' . scDateTime::getFormattedDate($date) . '</option>';
			}
			return $html;
		}
		
		protected function getPrintContent($fileName, $orderId)
		{
			header('Cache-Control: public'); // needed for i.e.
			header('Content-Type: application/pdf');
			header('Content-Disposition: attachment; filename="' . $fileName . '"');
			echo scShippingLabelCache::getCachedContent($fileName);
			die;
		}
	}
	
	class scSpeedyShippingLabel extends scShippingLabelPdf implements IShippingLabel {

		public function getContent($orderId, $params = array())
		{
			$parcelsCount = $params['parcelsCount'];
			$pickupDate = $params['pickupDate'];
			
			$moduleName = 'com.summercart.shipping.speedybg';
			$moduleId = scModulesFactory::getModuleIdByName($moduleName);
			$shippingModuleId = scUtil::getDbValue('ShippingModules', 'ShippingModuleID', 'ModuleID', $moduleId);
			$className = scModulesFactory::getModuleClassName($moduleId, 'IShippingModule');
					
			$speedy = new $className($moduleId, $shippingModuleId);
			
			$apiClient = $speedy->getApiClient();
			
			$order = new scOrder();
			$order->read($orderId);
			
			$paymentModuleId = $order->getModuleIdForPayment('PaymentModuleID');
			
			$amountCodBase = null;
			$codModuleName = 'com.summercart.payment.cod';
			$codModuleId = scModulesFactory::getModuleIdByName($codModuleName);
			
			if ($paymentModuleId == $codModuleId) {
				$amountCodBaseMCA = scMultiCurrencyAmount::getZero();
				$amountCodBaseMCA->set($order->getBaseCurrency(), $order->getBaseCurrencyTotal());
				$amountCodBaseMCA->set($order->getCustomerCurrency(), $order->getCustomerCurrencyTotal());
				$amountCodBase = $amountCodBaseMCA->get(new scCurrency('BGN'));
			}
			
			$customerAddress = $order->get('OrderShippingCity');
			$customerAddress .= ', ' . $order->get('OrderShippingAddress1');
			$customerAddress .= $order->get('OrderShippingAddress2') ? ', ' . $order->get('OrderShippingAddress2') : '';
			
			$shippingAddress = new scAddress();
			$shippingAddress->setCity($order->get('OrderShippingCity'));
			$shippingAddress->setCityId($order->get('OrderShippingCityID'));
			
			$payerType = $this->getPayerType($paymentModuleId);
			
			$options = scModuleOptions::getShippingModuleOptions($shippingModuleId);
			$amountInsuranceBase = 0;
			$payerTypeInsurance = null;
			if ($options['AddInsurance']) {
				$amountInsuranceBase = $order->getDeclaredValueForShipping($order->getBaseCurrency(), new scCurrency('BGN'));
				$payerTypeInsurance = $payerType;
			}
			
			$sender = array(
				'clientId' => (float)$options['ClientId'],
				'partnerName'=> null,
				'objectName'=> null,
				'address'=> null,
				'contactName'=> null,
				'phones' => array(array('number' => preg_replace('/[^0-9]+/u', '', scSettings::getSettingValue(scSettings::SETTING_COMPANY_PHONE)))),
			);
			$receiver = array(
				'clientId' => null,
				'partnerName' => $order->get('OrderShippingFirstName') . ' ' . $order->get('OrderShippingLastName'),
				'objectName' => $order->get('OrderShippingCompany'),
				'address' => array(
					'siteId' => $speedy->getSpeedyCityCode($shippingAddress),
					'streetName' => null,
					'streetType' => null,
					'streetId' => null,
					'quarterName' => null,
					'quarterType' => null,
					'quarterId' => null,
					'streetNo' => null,
					'blockNo' => null,
					'entranceNo' => null,
					'floorNo' => null,
					'apartmentNo' => null,
					'addressNote' => $customerAddress,
					'commonObjectId' => null,
				),
				'contactName' => $order->get('OrderShippingFirstName') . ' ' . $order->get('OrderShippingLastName'),
				'phones' => array(array('number' => preg_replace('/[^0-9]+/u', '', $order->get('OrderShippingPhone')))),
			);
			
			$params = 
				array(
					'picking' => array(
						'takingDate' => $pickupDate,
						'serviceTypeId' => $order->get('OrderShippingMethodCode'),
						'officeToBeCalledId' => null,
						'fixedTimeDelivery' => null,
						'backDocumentsRequest' => (isset($options['backDocumentsRequest']) ? $options['backDocumentsRequest'] : 0),
						'backReceiptRequest' => (isset($options['backReceiptRequest']) ? $options['backReceiptRequest'] : 1),
						'willBringToOffice' => (isset($options['isBroughtToOffice']) ? $options['isBroughtToOffice'] : 0),
						'amountInsuranceBase' => $amountInsuranceBase,
						'amountCodBase' => $amountCodBase,
						'payCodToThirdParty' => 0,
						'parcelsCount' => $parcelsCount,
						'size' => null,
						'weightDeclared' => $order->getTotalWeight(scWeightUnits::WEIGHT_UNIT_KGS),
						'contents' => _t('Order') . ' ' . $order->get('OrderID'),
						'packing' => '.', // FIXME not good
						'packId' => null,
						'documents' => $options['IsDocuments'],
						'fragile' => $options['IsFragile'],
						'palletized' => 0,
						'sender' => $sender,
						'receiver' => $receiver,
						'payerType' => $payerType,
						'payerRefId' => null,
						'payerTypeInsurance' => $payerTypeInsurance,
						'payerRefInsuranceId' => null,
						'noteClient' => _t('Order') .  ' ' . $order->get('OrderID'),
						'discCalc' => null,
						'retToClientId' => null,
						'parcels' => null,
					),	
			);
			
			$shippingLabels = array();
			$res = $apiClient->makeCall('createBillOfLading', $params);
			if (isset($res->generatedParcels)) {
				$trackingNumber = null;
				if (is_array($res->generatedParcels)) {
					foreach ($res->generatedParcels as $parcel) {
						if (is_null($trackingNumber)) {
							$trackingNumber = $parcel->parcelId;
							self::writeCacheFile(IShippingLabel::DOCUMENT_TYPE_SLIP, $apiClient, $orderId, $parcel->parcelId);
						}
						$shippingLabels[] = $parcel->parcelId;
						self::writeCacheFile(IShippingLabel::DOCUMENT_TYPE_LABEL, $apiClient, $orderId, $parcel->parcelId);
					}
				} else {
					$trackingNumber = $res->generatedParcels->parcelId;
					$shippingLabels[] = $res->generatedParcels->parcelId;
					self::writeCacheFile(IShippingLabel::DOCUMENT_TYPE_SLIP, $apiClient, $orderId, $trackingNumber);
					self::writeCacheFile(IShippingLabel::DOCUMENT_TYPE_LABEL, $apiClient, $orderId, $trackingNumber);
				}
				// update the shipping tracking number in the order 
				$order->set('OrderShippingTrackingNo', $trackingNumber);
				$order->update();
				
				// save shipping labels ids in order details
				scOrderDetailsUtil::setDetail($orderId, $moduleId, 'ShippingLabels', serialize($shippingLabels));
				return $trackingNumber;
			}
		}
		
		protected function getPayerType($moduleId)
		{
			// We must check if the shpping price for the payment method
			// is owed on checkout or upon delivery so we are able to determine
			// the payment side for the shipping label
			$orderTotalShippingModuleId = scModulesFactory::getModuleIdByName('com.summercart.ordertotal.shipping');
			$shippingModuleId = scUtil::getDbValue('OrderTotalModules', 'OrderTotalModuleID', 'ModuleID', $orderTotalShippingModuleId);
			$orderTotalShippingClassName = scModulesFactory::getModuleClassName($orderTotalShippingModuleId, 'IOrderTotalModule');
			$paymentModuleId = scUtil::getDbValue('PaymentModules', 'PaymentModuleID', 'ModuleID', $moduleId);
			$orderTotalShippingObj = new $orderTotalShippingClassName($orderTotalShippingModuleId);
			
			$shippingPriceInTotal = $orderTotalShippingObj->shippingIncludedInTotal($paymentModuleId);
			if ($shippingPriceInTotal) {
				return scSpeedyBg::PAYER_TYPE_SENDER;
			}
			
			return scSpeedyBg::PAYER_TYPE_RECEIVER;
		}
		
		public function getCacheFileName($orderId, $params = array())
		{
			if (isset($params['documentType']) && ($params['documentType'] == self::DOCUMENT_TYPE_LABEL)) {
				if (empty($params['parcelId'])) {
					throw new miException(_i('Required parameter parcelId is missing.'));
				}
				return $orderId . '_' . self::DOCUMENT_TYPE_LABEL . '_' . $params['parcelId'] . '.' . $this->getExtension();
			}
			return $orderId . '_' . self::DOCUMENT_TYPE_SLIP . '.' . $this->getExtension();
		}
		
		protected function writeCacheFile($type, $apiClient, $orderId, $shippingLabelId)
		{
			if ($type == self::DOCUMENT_TYPE_LABEL) {
				$params = array(
					'parcelId' => $shippingLabelId,
					'includeAutoPrintJS' => 0,
				);
				$res = $apiClient->makeCall('createCustomTravelLabelPDFType1', $params);
				$fileName = $this->getCacheFileName($orderId, array('documentType' => self::DOCUMENT_TYPE_LABEL, 'parcelId' => $shippingLabelId));
			} else {
				$params = array(
					'billOfLading' => $shippingLabelId,
					'includeAutoPrintJS' => 0,
				);
				$res = $apiClient->makeCall('createBillOfLadingPDF', $params);
				$fileName = $this->getCacheFileName($orderId);
			}
			
			$path = scPaths::getFilenamePath($fileName, scPaths::RESOURCE_SHIPPING_LABEL_CACHE_FILE, scPaths::PATH_FILE);
			file_put_contents($path, $res);
		}
	}
?>