<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	$apiPath = '../../modules/shipping/speedybg/speedy_api.php';
	miLoader::addClass('scSpeedyApi', $apiPath);
	miLoader::addClass('scSpeedyApiException', $apiPath);
	
	class scSpeedyBg implements IShippingModule, IShippingModuleCodInfo, IModuleOptions {
	
		const HANDLING_FEE_TYPE_PERCENT = 1;
		const HANDLING_FEE_TYPE_FIXED = 2;
	
		const PAYER_TYPE_SENDER = 0;
		const PAYER_TYPE_RECEIVER = 1;
		const PAYER_TYPE_OTHER = 2;
		
		const BULGARIAN_VAT = 1.2;
		
		protected $_moduleId;
		protected $_shippingModuleId;
	
		protected $_apiClient;
	
		protected $_statesCache = array();
		protected static $_bgLangId = null;
	
		public function __construct($moduleId, $shippingModuleId)
		{
			$this->_moduleId = $moduleId;
			$this->_shippingModuleId = $shippingModuleId;
		}
	
		public function getShippingMethods()
		{
			$shippingMethods = array(
				2 => 'Експресна',
				3 => 'Икономична',
				4 => 'Карго',
				26 => 'Фиксиран час - Експресна',
				36 => 'Фиксиран час - Икономична',
				44 => 'Контейнер',
				45 => 'Евро Палет - Експрес',
				46 => 'Евро Палет - Карго',
				47 => 'Индустриален Палет - Експрес',
				48 => 'Индустриален Палет - Карго',
				55 => 'Пакетна',
				110 => 'Градски куриер - 90 минути',
				111 => 'Градски куриер - 3 часа',
				112 => 'Градски куриер - 6 часа',
				113 => 'Градски куриер - 1 ден',
				114 => 'Градски куриер - 2 дни',
//				882 => 'TB (RO) - Експрес',
//				883 => 'TB (RO) - Карго',
//				884 => 'TB (RO) - Контейнер',
//				885 => 'TB (RO) - Евро Палет Карго',
//				886 => 'TB (RO) - Евро Палет Експрес',
//				887 => 'TB (RO) - Инд. Палет Карго',
//				888 => 'TB (RO) - Инд. Палет Експрес',
//				992 => 'Трансбалкан - Експрес',
//				993 => 'Трансбалкан - Карго',
//				994 => 'Трансбалкан - Контейнер',
//				995 => 'Трансбалкан - Палет (Букурещ)',
			);
			foreach ($shippingMethods as $serviceTypeId => $serviceName)
				$shippingMethods[$serviceTypeId] = _i('Speedy: ') . $serviceName;
				
			return $shippingMethods;
		}
	
		public function getShippingRates(scCart $cart, $shippingMethods)
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			if (empty($options['Username'])) {
				throw new miException(_t('Speedy shipping module is not configured.'));
			}
			
			$params = $this->prepareParamsForGetRatesCall($cart, $options);
			
			$result = $this->getApiClient()->makeCall('calculateMultipleServices', array('calculation' => $params, 'serviceTypeIds' => $this->getActiveShippingMethodsIds()));
			$return = $this->formatSpeedyRates($result, $options);
			if (empty($return[self::SHIPPING_RATES]) && !empty($shippingMethods)) {
				throw new miException($this->formatNoMethodsException($result, $shippingMethods));
			}
			return $return;
		}
		
		public function getShippingModuleImage()
		{
			return 'skins/default/modules/shipping/speedybg/logo_speedy.png';
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'Username',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Password',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'IsFragile',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'AddInsurance',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'IsDocuments',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'IncludeVat',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'HandlingFeeType',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'HandlingFeeAmount',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ClientId',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'isBroughtToOffice',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'backReceiptRequest',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'backDocumentsRequest',
				),
			);
				
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('AddInsurance')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 0);
			$options->getOptionField('IsFragile')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 0);
			$options->getOptionField('IsDocuments')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 0);
			$options->getOptionField('IncludeVat')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 1);
			$options->getOptionField('HandlingFeeType')->setRadioButtons(array(self::HANDLING_FEE_TYPE_FIXED => _t('Fixed'), self::HANDLING_FEE_TYPE_PERCENT => _t('Percent')));
			$options->getOptionField('isBroughtToOffice')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 0);
			$options->getOptionField('backDocumentsRequest')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 0);
			$options->getOptionField('backReceiptRequest')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 0);
			$webForm->addMainPageElements(array('%%SHIPPINGMODULEID%%'=>$this->_shippingModuleId));
			$options->setTemplateFile('modules/shipping/speedybg/options.tmpl');
			return $options;
		}
		
		public function importLocations()
		{
			$cities = $this->getApiClient()->makeCall('listAllSites');
			miStaticDBUtil::startTransaction();
			try {
				if (!empty($cities)) {
					$sql = 'DELETE FROM CitySources WHERE CitySourceModuleID = ?';
					miStaticDBUtil::execSQL($sql, array($this->_moduleId));
				}
				
				$newCitiesImported = 0;
				$citiesUpdated = 0;
				$citySources = array();
				$cityPostCodes = array();
				
				$languageCodes = scMultiLanguage::singleton()->getActiveLanguageCodes();
				
				foreach ($cities as $city) {
					$stateId = $this->getStateId($city->region);
					$cityName = $city->name;
					$cityName = scUtf8::ucwords($cityName);
					
					// if the current city is in the static map change the name
					if (!empty($this->_speedyToLocalIdsMap[$city->id]) && !empty($this->_speedyToLocalIdsMap[$city->id]['add'])) {
						
						$newCityName = $cityName; 
						
						// if we need to add the post code add 'п.к.' to the name
						if ($this->_speedyToLocalIdsMap[$city->id]['add'] == 'postCode') {
							$newCityName .= ' (п.к. ' .  $city->{$this->_speedyToLocalIdsMap[$city->id]['add']} . ')';
						
						// if we ned to add the municipality make it ucwords
						} elseif($this->_speedyToLocalIdsMap[$city->id]['add'] == 'municipality') {
							$newCityName .= ' (' .  scUtf8::ucwords($city->{$this->_speedyToLocalIdsMap[$city->id]['add']}) . ')';
						
						// if this is a type 
						} elseif(!empty($city->{$this->_speedyToLocalIdsMap[$city->id]['add']})) {
							$newCityName .= ' (' .  $city->{$this->_speedyToLocalIdsMap[$city->id]['add']} . ')';

						// this is something hardcoded
						} else {
							$newCityName .= ' (' .  $this->_speedyToLocalIdsMap[$city->id]['add'] . ')';
						}
						
						// update the names with the new values
						$cityId = $this->_speedyToLocalIdsMap[$city->id]['CityID'];
							
						$sql = 'UPDATE CitiesML SET CityName = ? WHERE CityID = ? AND LanguageID = ? AND CityName = ?';
						miStaticDBUtil::execSQL($sql, array($newCityName, $cityId, $this->getBgLangId(), $cityName));
						
						$sql = 'UPDATE CitiesML SET CityName = ? WHERE CityID = ? AND LanguageID <> ? AND CityName = ?';
						miStaticDBUtil::execSQL($sql, array(scFriendlyUrl::transliterateCyrillic($newCityName), $cityId, $this->getBgLangId(), scFriendlyUrl::transliterateCyrillic($cityName)));
						
						$cityName = $newCityName;
					}
					
					// check if we have the city in our database
					$sql = 'SELECT Cities.CityID FROM Cities JOIN CitiesML USING(CityID) WHERE CityName = ? AND LanguageID = ? AND StateID=?';
					$res = miStaticDBUtil::execSelect($sql, array($cityName, $this->getBgLangId(), $stateId));
					if (empty($res)) {
						$cityObj = new scSqlRecordML('Cities', 'CityID', array('CityName'), array_keys($languageCodes));
						$cityNames = array();
						foreach ($languageCodes as $langId => $langCode) {
							$cityNames[$langId] = $this->transliterateByLanguage($langCode, $cityName);
						}
						$cityObj->setRow(array(
							'StateID' => $stateId,
							'CityName' => $cityNames,
						));
						$cityId = $cityObj->insert();
						$newCitiesImported++;
					} else {
						$cityId = $res[0]['CityID'];
						$citiesUpdated++;
					}
					
					$citySources[] = array(
						'CityID' => $cityId,
						'CitySourceModuleID' => $this->_moduleId,
						'CitySourceCode' => (string)$city->id
					);
					
					$cityPostCodes[] = array(
						'CityID' => $cityId,
						'CityPostCode' => $city->postCode,
					);
				}
				
				miStaticDBUtil::execMultiInsert('CitySources', $citySources);
				miStaticDBUtil::execMultiInsert('CityPostCodes', $cityPostCodes, miDBUtilImpl::FLAG_IGNORE);
				
			} catch(miDBException $e) {
				miStaticDBUtil::rollback();
				return array('type' => scMessage::MSG_TYPE_ERROR, 'message' => _t('An error occured while importing cities with message: ') . $e->getMessage());
			}
			miStaticDBUtil::commit();
			return array('type' => scMessage::MSG_TYPE_INFO, 'message' => sprintf(_t('Locations were imported successfully. New cities imported: %d. Cities updated: %d.'),$newCitiesImported, $citiesUpdated));
		}
	
		public function getAccountData()
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId, array('ClientId'));
			$apiClient = $this->getSpeedyApiInstance();
			$apiClient->setOptions(array(
				'Username' => miGetParam('ModuleOptionUsername'),
				'Password' => miGetParam('ModuleOptionPassword'),
				'ClientId' => miGetParam('ModuleOptionClientId'),
			));
			$apiClient->initSession();
			$response = $apiClient->makeCall('getClientById', array('clientId' => (float)miGetParam('ModuleOptionClientId')));
			
			$t = new scTemplateParser();
			$t->readTemplate('modules/shipping/speedybg/account_data.tmpl');
			$t->assignArray(array(
				'%%REGIONNAME%%' => $response->address->regionName,
	            '%%SITENAME%%' => $response->address->siteName,
	            '%%SITETYPE%%' => $response->address->siteType,
	            '%%STREETNAME%%' => (isset($response->address->streetName) ? $response->address->streetName : $response->address->addressNote),
	            '%%STREETNO%%' => (isset($response->address->streetNo) ? $response->address->streetNo : ''),
	            '%%STREETTYPE%%' => (isset($response->address->streetType) ? $response->address->streetType : ''),
			
	            '%%OBJECTNAME%%' => empty($response->objectName) ? '' : $response->objectName,
	            '%%PARTNERNAME%%' => empty($response->partnerName) ? '' : $response->partnerName,
			));
			return $t->templateParse();
		}
		
		public function getCodFee(scCart $cart)
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			if (empty($options['Username'])) {
				throw new miException(_t('Speedy shipping module is not configured.'));
			}
			$params = $this->prepareParamsForGetRatesCall($cart, $options, true);
			$result = $this->getApiClient()->makeCall('calculateMultipleServices', array('calculation' => $params, 'serviceTypeIds' => $this->getActiveShippingMethodsIds()));
			if (!is_array($result)) {
				$result = array($result);
			}
			
			$shippingMethod = $cart->getShippingMethod();
			if (empty($shippingMethod)) {
				// since CoD premium in Speedy depends on the contract signed between Speedy and the customer
				// don't return any cod fee if no shipping method is selected yet
				return null;
			}
			
			foreach ($result as $service) {
				if (isset($service->serviceTypeId) && ($service->serviceTypeId == $shippingMethod)) {
					if (isset($service->resultInfo->amounts->codPremium)) {
						$codPremium = $service->resultInfo->amounts->codPremium;
						if ($options['IncludeVat'])
							$codPremium *= self::BULGARIAN_VAT;
						$amount = scMultiCurrencyAmount::factory();
						$amount->set(new scCurrency('BGN'), $codPremium);
						return $amount;
					}
				}
			}
			return null;
		}
			
		/**
		 * Retrieves the SpeedyCityID from CitySources
		 * TODO make a util for retreiving CitySourceCode
		 * @param scAddress $shippingAddress
		 * @return string the city code, for Speedy this is SiteID
		 */
		public function getSpeedyCityCode(scAddress $shippingAddress)
		{
			$cityId = $shippingAddress->getCityId();
			
			// If city id is not known, try to find it
			if (empty($cityId)) {
				$stateId = $shippingAddress->getStateId();
				if (empty($stateId))
					throw new scSpeedyApiException(_t('Please specify state.'));
				
				$cityId = scCustomerAddress::findCityId($stateId, $shippingAddress->getCity());
				$shippingAddress->setCityId($cityId);
				
				if (empty($cityId)) {
					$stateName = scCountry::getStateName($stateId);
					throw new scSpeedyApiException((sprintf(_t('Could not find city "%s" in our city list for state "%s". Please check your spelling.'), $shippingAddress->getCity(), $stateName)));
				}
			}
			
			$sql = 'SELECT CitySourceCode FROM CitySources WHERE CitySourceModuleID = ? AND CityID = ?';
			$res = miStaticDBUtil::execSelect($sql, array($this->_moduleId, $cityId));
			if (empty($res)) {
				throw new scSpeedyApiException(sprintf(_t('Could not find CityID "%s". Please import locations using the Speedy module administration.'), $cityId));
			}
			if (count($res) > 1)
				throw new scSpeedyApiException(sprintf(_t('Found more than one city named "%s".'), $shippingAddress->getCity()));
			
			return $res[0]['CitySourceCode'];
		}
		
		public function getApiClient()
		{
			if (is_null($this->_apiClient)) {
				$this->initApiClient();
			}
			return $this->_apiClient;
		}
		
		/**
		 * creates a api client instance and logs in in Speedy's system
		 */
		protected function initApiClient()
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			$this->_apiClient = $this->getSpeedyApiInstance();
			$this->_apiClient->setOptions($options);
			$this->_apiClient->initSession();
		}
	
		/**
		 * maps and caches the state ids against state names
		 * @param string $stateName
		 * @return int $stateId
		 */
		protected function getStateId($stateName)
		{
			static $customStatesMappings = array(
				'СОФИЯ - ГРАД' => 'СОФИЯ - ГРАД'
			);
			
			static $bgCountryId = null;
			
			if ($bgCountryId === null) {
				$bgCountryId = scUtil::getDbValue('Countries', 'CountryID', 'CountryCode2', 'BG');
				if (empty($bgCountryId)) {
					throw new miException(_i('Cannot find country Bulgaria.'));
				}
			}
			
			// this is only for Speedy.
			if (isset($customStatesMappings[$stateName])) {
				$stateName = $customStatesMappings[$stateName];
			}
			
			if (array_key_exists($stateName, $this->_statesCache)) {
				return $this->_statesCache[$stateName];
			}
			
			$sql = 'SELECT States.StateID FROM States JOIN StatesML USING(StateID) WHERE LanguageID = ? AND StateName = ? AND CountryID=?';
			$res = miStaticDBUtil::execSelect($sql, array($this->getBgLangId(), $stateName, $bgCountryId));
			if (empty($res[0]['StateID'])) {
				throw new scSpeedyApiException(sprintf(_t('Could not find State %s.'), $stateName));
			}
			
			$this->_statesCache[$stateName] = $res[0]['StateID'];
			return $this->_statesCache[$stateName];
		}
		
		
		/**
		 * returns only the active shipping methods for speedy
		 */
		protected function getActiveShippingMethodsIds()
		{
			$sql = 'SELECT ShippingMethodCode FROM ShippingMethods WHERE ShippingModuleID = ? AND ShippingMethodIsActive = 1';
			$res = miStaticDBUtil::execSelect($sql, array($this->_shippingModuleId));
			if (empty($res)) {
				throw new scSpeedyApiException(_t('No shipping methods activated for Speedy.'));
			}
			$res = miUIUtil::transposeRows($res);
			$return = $res['ShippingMethodCode'];
			return $return;
		}
		
		/**
		 * Formats Speedy XML result to array
		 * @param array $result
		 */
		protected function formatSpeedyRates($result, $options)
		{
			$return = array();
			$return[self::SHIPPING_RATES_CURRENCY] = 'BGN';
			if (!is_array($result)) {
				$result = array($result);
			}
			$services = array();
			foreach ($result as $service) {
				if (isset($service->resultInfo)) {
					$amount = 0;
					$amount = $service->resultInfo->amounts->total;
					if (!$options['IncludeVat'])
						$amount /= self::BULGARIAN_VAT;
					if ($options['HandlingFeeAmount'] > 0) {
						if ($options['HandlingFeeType'] == self::HANDLING_FEE_TYPE_FIXED) {
							$handlingFee = scMultiCurrencyAmount::factoryBaseCurrency($options['HandlingFeeAmount']);
							$amount += $handlingFee->get(new scCurrency('BGN'));
						} else {
							$amount += $amount * $options['HandlingFeeAmount'] / 100;
						}
					}
					$services[$service->serviceTypeId] = $amount;
				}
			}
			$return[self::SHIPPING_RATES] = $services;
			return $return;
		}
		
		protected function formatNoMethodsException($apiResults, $shippingMethods)
		{
			$shippingMethodNames = $this->getShippingMethods();
			$shippingMethodNames = array_intersect_key($shippingMethodNames, array_flip($shippingMethods));
			$msg = '';
			if (!is_array($apiResults)) {
				$apiResults = array($apiResults);
			}
			foreach ($apiResults as $method) {
				if (!isset($shippingMethodNames[$method->serviceTypeId]))
					continue;
				if (empty($method->errorDescription))
					continue;
				$msg .= $shippingMethodNames[$method->serviceTypeId] . ': ';
				
				$description = explode('/', $method->errorDescription);
				if (count($description) != 3) {
					$msg .= $method->errorDescription;
				} else {
					$error = $this->getApiClient()->getErrorDescription(trim($description[1]));
					if (is_array($error)) {
						$msg .= $method->errorDescription;
					} else {
						$msg .= $error;
					}
				}
				$msg .= '<br />';
			}
			return $msg;
		}
		
		protected function prepareParamsForGetRatesCall(scCart $cart, $options, $includeCoD = false)
		{
			$shippingAddress = $cart->getShippingAddress();
			
			$insuraceBase = null;
			$payerTypeInsurance = null;
			if ($options['AddInsurance']) {
				$insuraceBase = $cart->getDeclaredValueForShipping(new scCurrency('BGN'));
				$payerTypeInsurance = self::PAYER_TYPE_SENDER;
			}
			
			$amountCodBase = null;
			if ($includeCoD)
				$amountCodBase = $cart->getOrderTotal()->getDeclaredValueForShipping(new scCurrency('BGN'));
			
			$params = array(
				'takingDate' => null,
				'autoAdjustTakingDate' => 1,
				'serviceTypeId' => 0, // empty for multiple services
				'broughtToOffice' => (isset($options['isBroughtToOffice']) ? $options['isBroughtToOffice'] : 0),
				'toBeCalled' => 0,
				'fixedTimeDelivery' => null,
				'amountInsuranceBase' => $insuraceBase,
				'amountCodBase' => $amountCodBase,
				'payCodToThirdParty' => 0,
				'parcelsCount' => 1,
				'weightDeclared' => $cart->getTotalWeight(scWeightUnits::WEIGHT_UNIT_KGS),
				'documents' => $options['IsDocuments'],
				'fragile' => $options['IsFragile'],
				'palletized' => 0,
				'senderId' => $this->getApiClient()->getClientId(),
				'senderSiteId' => null,
				'receiverId' => null,
				'receiverSiteId' => $this->getSpeedyCityCode($shippingAddress),
				'payerType' => self::PAYER_TYPE_SENDER,
				'payerRefId' => null,
				'payerTypeInsurance' => $payerTypeInsurance,
				'payerRefInsuranceId' => self::PAYER_TYPE_SENDER,
			);
			
			return $params;
		}
		
		/**
		 * Return the language id for Bulgarian language
		 * @return int
		 */
		protected function getBgLangId()
		{
			if (self::$_bgLangId == null) {
				self::$_bgLangId = scUtil::getDbValue('Languages', 'LanguageID', 'LanguageCode', 'BG');
				if (empty(self::$_bgLangId))
					throw new scSpeedyApiException(_i('Could not find Bulgarian language.'));
			}
			return self::$_bgLangId;
		}
		
		/**
		 * Transliterate the city names
		 * 
		 * @param string $langCode
		 * @param string $text
		 * @return string
		 */
		protected function transliterateByLanguage($langCode, $text)
		{
			$langCode = strtoupper($langCode);
			if ($langCode == 'BG')
				return $text;
			
			return scFriendlyUrl::transliterateCyrillic($text);
		}
		
		/**
		 * @return scSpeedyApi
		 */
		protected function getSpeedyApiInstance()
		{
			if (!class_exists('SoapClient', false))
				throw new miException(_i('SoapClient cannot be found. Please make sure that you have PHP SOAP extension enabled.'));
			
			return new scSpeedyApi();
		}
		
		/**
		 * FIXME add city prefixes and suffixes to make cities distinct in the autocomplete
		 * Use this map of speedyIds to localIds to 
		 * handle duplicate city names in a state
		 * 
		 * [key] - the speedyId for the city
		 * [element] - 
		 * 	array 
		 * 		CityID - Locat CityID
		 * 		add - what should we add to the CityName
		 * 
		 * If the 'add' is empty this is a town and no change to the name is required 
		 * if the 'add' is not a City property we add as a suffix in braces to the name e.g. (гара)
		 */
		
		protected $_speedyToLocalIdsMap = array(
			
			// Костенец
		    92063 => array('CityID' => 5324, 'add' => 'type'),
		    38902 => array('CityID' => 2492, 'add' => ''),
		    38916 => array('CityID' => 2493, 'add' => 'type'),
		    
			// Бов
		    4546 => array('CityID' => 325, 'add' => ''),
		    14461 => array('CityID' => 326, 'add' => 'гара'),
		    
			// Вълчовци
		    12615 => array('CityID' => 892, 'add' => 'postCode'),
		    12629 => array('CityID' => 893, 'add' => 'postCode'),
		    
			// Генчовци
		    14756 => array('CityID' => 986, 'add' => 'municipality'),
		    56304 => array('CityID' => 987, 'add' => 'municipality'),
		    
			// Елин Пелин
		    18490 => array('CityID' => 10662, 'add' => 'type'),
		    27303 => array('CityID' => 1808, 'add' => 'гара'),
		    
			// Елхово
		    27379 => array('CityID' => 1815, 'add' => 'postCode'),
		    27382 => array('CityID' => 1817, 'add' => 'postCode'),
		    
			// Искър
		    14888 => array('CityID' => 2115, 'add' => ''),
		    55782 => array('CityID' => 2117, 'add' => 'гара'),
		    
			// Каспичан
		    36587 => array('CityID' => 2275, 'add' => ''),
		    36590 => array('CityID' => 2276, 'add' => 'type'),
		    
			// Крайна
		    39311 => array('CityID' => 2531, 'add' => 'municipality'),
		    41277 => array('CityID' => 2532, 'add' => 'municipality'),
		    
			// Лакатник
		    14475 => array('CityID' => 2701, 'add' => ''),
		    43092 => array('CityID' => 2702, 'add' => 'гара'),
		    
			// Орешец
		    14489 => array('CityID' => 3397, 'add' => ''),
		    53758 => array('CityID' => 3398, 'add' => 'гара'),
		    
			// Светлина
		    65574 => array('CityID' => 4179, 'add' => 'municipality'),
		    65588 => array('CityID' => 4180, 'add' => 'municipality'),
		    
			// Слатина
		    67218 => array('CityID' => 4338, 'add' => 'municipality'),
		    67204 => array('CityID' => 5343, 'add' => 'municipality'),
		    
			// Тополовец
		    72775 => array('CityID' => 4734, 'add' => 'municipality'),
		    73924 => array('CityID' => 4735, 'add' => 'municipality'),
		    
			// Чайка
		    80073 => array('CityID' => 5009, 'add' => 'type'),
		    92090 => array('CityID' => 5354, 'add' => 'type'),
		);
	}
?>