<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scTableBased implements IShippingModule, IModuleOptions {
		const TABLE_BASED_SHIPPING_METHOD = 1;
		
		const BASED_ON_WEIGHT = 1;
		const BASED_ON_SUBTOTAL = 2;
		const BASED_ON_QUANTITY = 3;
		
		protected $_moduleId;
		protected $_shippingModuleId;
		
		public function __construct($moduleId, $shippingModuleId)
		{
			$this->_moduleId = $moduleId;
			$this->_shippingModuleId = $shippingModuleId;
		}
		
		public function getShippingMethods()
		{
			return array(self::TABLE_BASED_SHIPPING_METHOD => 'Table-based Shipping'); // _t('Table-based Shipping')
		}
		
		public function getShippingRates(scCart $cart, $shippingMethods)
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			
			switch ($options['ShippingTableBase']) {
				case self::BASED_ON_SUBTOTAL:
					$value = $this->getCartSubtotal($cart);
					break;
				
				case self::BASED_ON_QUANTITY:
					$value = $cart->getTotalQuantity();
					break;
				
				default:
					$value = $cart->getTotalWeight(scSettings::getSettingValue(scSettings::SETTING_LOCALIZATION_WEIGHT_UNIT));
			}
			
			$rate = $this->getShippingRate($value);
			
			if ($rate === false)
				return array();
			return array(self::SHIPPING_RATES => array(self::TABLE_BASED_SHIPPING_METHOD => $rate));
		}
		
		public function getShippingModuleImage()
		{
			return 'skins/default/modules/shipping/table_based/logo_table-based.png';
		}
		
		protected function getCartSubtotal(scCart $cart)
		{
			$orderTotalLines = $cart->getOrderTotal()->getCurrentOrderTotalLines();
			
			$orderTotalLineTypes = scUtil::getActiveOrderTotalLineTypes(IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING);
			if (isset($orderTotalLineTypes[IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING])) {
				unset($orderTotalLineTypes[IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING]);
			}
			
			$totalAmount = 0;
			foreach ($orderTotalLines as $line) {
				if (!in_array($line->getType(), $orderTotalLineTypes)) {
					continue;
				}
				$totalAmount += $line->getAmount()->get();
			}
			return $totalAmount;
		}
		
		protected function getShippingRate($value)
		{
			$query = 'SELECT ShippingTableCost FROM ShippingTable WHERE ShippingModuleID = ? ' .
				' AND ShippingTableUpTo >= ? ORDER BY ShippingTableUpTo ASC LIMIT 1';
			$rows = miStaticDBUtil::execSelect($query, array($this->_shippingModuleId, $value));
			if (count($rows) == 0)
				return false;
			
			return $rows[0]['ShippingTableCost'];
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			list($currencySymbolLeft, $currencySymbolRight) = scCurrencyUtil::getBaseCurrencySymbols();
			
			$fieldsDescriptions = array(
				array(
					'field' => 'miRadioWidget',
					'data' => 'ShippingTableBase',
				),
				array(
					'field' => 'scTableBasedShippingDynRowsWidget',
					'data' => 'ShippingFee',
					'properties' => array(
						'template' => 'modules/shipping/table_based/table_based_rs.tmpl',
						'table' => 'Options',
						'foreignKey' => 'OptionGroupID',
						'fields' => array( 
				       		array(
								'field' => 'miBaseTextWidget',
								'data' => 'ShippingTableID'
				        		),
				       		array(
								'field' => 'miTextWidget',							
								'data' => 'ShippingTableUpTo',
				        		),
				        	array(
								'field' => 'miTextWidget',
								'data' => 'ShippingTableCost'
				        		),
				        	),
						),
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->setTemplateFile('modules/shipping/table_based/options.tmpl');
			$basedOn = array(self::BASED_ON_WEIGHT => _t('Weight') . ' (' . scWeightUnits::getCurrentWeightUnitName() . ')', self::BASED_ON_SUBTOTAL => _t('Subtotal'), self::BASED_ON_QUANTITY => _t('Quantity'));
			$options->getOptionField('ShippingTableBase')->setRadioButtons($basedOn, self::BASED_ON_WEIGHT);
			$options->setModelStrategies(array('ShippingFee' => 'scViewMapperTableBasedShipping'));
			return $options;
		}
	}
	
	/**
	 * Subscalls of scDynRowsWidget. Adds the base currency code
	 */
	class scTableBasedShippingDynRowsWidget extends scDynRowsWidget {
		public function getEditableControl()
		{
			return parent::getEditableControl() + array('BaseCurrencyCode' => scCurrencyUtil::getBaseCurrencyCode());
		}
	}
	
	class scViewMapperTableBasedShipping extends miViewMapper {
		protected function getShippingModuleId()
		{
			$defaultStrategy = $this->getViewMapperController()->getViewMapper();
			return $defaultStrategy->get('ShippingModuleID');
		}
		
		public function read()
		{
			$query = 'SELECT * FROM ShippingTable WHERE ShippingModuleID=? ORDER BY ShippingTableUpTo';
			$rows = miStaticDBUtil::execSelect($query, array($this->getShippingModuleId()));
			
			$viewMapper = $this->getViewMapperController()->getViewMapper('scViewMapperShippingModuleOptions');
			$viewMapper->setModuleOption('ShippingFee', $rows);
		}
		
		public function insert()
		{
			$this->update();
		}
		
		public function update()
		{
			$viewMapper = $this->getViewMapperController()->getViewMapper('scViewMapperShippingModuleOptions');
			$shippingFee = $viewMapper->getModuleOption('ShippingFee');
			$shippingModuleId = $this->getShippingModuleId();
			
			miStaticDBUtil::execDelete('ShippingTable', 'ShippingModuleID', $shippingModuleId);
			
			$record = new miSqlRecord('ShippingTable', 'ShippingTableID');
			foreach ($shippingFee as $rowIndex => $shippingFeeValues) {
				foreach ($shippingFeeValues as $shippingFeeName => $shippingFeeValue)
					$record->set($shippingFeeName, $shippingFeeValue);
				$record->set('ShippingModuleID', $shippingModuleId);
				if ($shippingFeeValues['ShippingTableID'] == '') {
					$record->set('ShippingTableID', null); // Fix for MySQL in STRICT mode
				}
				$record->insert();
			}
		}
		
		public function delete()
		{
			miStaticDBUtil::execDelete('ShippingTable', 'ShippingModuleID', $this->getShippingModuleId());
		}
	}
?>