<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scAdminLastOrdersBox implements ICMSBox {
		protected $_boxId = 'last_orders';
		protected $_boxInfo = array();
		
		const STATE_VISIBLE = 'visible';
		const STATE_VISIBLE_ARROW = 'arrow';
		
		public function __construct($moduleId, $cmsBoxId)
		{
			// Set the default box state
			$this->_boxInfo[self::STATE_VISIBLE] = 'block';
			$this->_boxInfo[self::STATE_VISIBLE_ARROW] = 'none';
		}
		
		public function getBoxContents()
		{
			$stats = $this->getLastOrdersStats();
			
			$today = array(scOrderStatus::ORDER_STATUS_NEW => 0, scOrderStatus::ORDER_STATUS_DELIVERED => 0);
			$yesterday = array(scOrderStatus::ORDER_STATUS_NEW => 0, scOrderStatus::ORDER_STATUS_DELIVERED => 0);
			foreach ($stats as $stat) {
				if ($stat['T'])
					$yesterday[$stat['OrderStatus']] = $stat['N'];
				else
					$today[$stat['OrderStatus']] = $stat['N'];
			}
			
			$orderStatsYesterday = array();
			$orderStatsYesterday[] = $yesterday[scOrderStatus::ORDER_STATUS_NEW];
			$orderStatsYesterday[] = array_sum($yesterday) - $yesterday[scOrderStatus::ORDER_STATUS_NEW] - $yesterday[scOrderStatus::ORDER_STATUS_DELIVERED];
			$orderStatsYesterday[] = $yesterday[scOrderStatus::ORDER_STATUS_DELIVERED];
			
			$orderStatsToday = array();
			$orderStatsToday[] = $today[scOrderStatus::ORDER_STATUS_NEW];
			$orderStatsToday[] = array_sum($today) - $today[scOrderStatus::ORDER_STATUS_NEW] - $today[scOrderStatus::ORDER_STATUS_DELIVERED];
			$orderStatsToday[] = $today[scOrderStatus::ORDER_STATUS_DELIVERED];
			
			$orderStatsNames = array(_t('New'), _t('Processed'), _t('Delivered'));
			
			$section = new miTemplateParserSectionInfo();
			$section->setSectionInfo('rows', 3, array(
				'%%ORDER_STATS_NAME%%' => $orderStatsNames,
				'%%ORDER_STATS_TODAY%%' => $orderStatsToday,
				'%%ORDER_STATS_YESTERDAY%%' => $orderStatsYesterday));
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/cmsboxes/last_orders.tmpl');
			$t->setSectionInfos(array($section));
			
			$t->assign('%%BOXID%%', $this->_boxId);
			$t->assign('%%VISIBLE%%', $this->_boxInfo[self::STATE_VISIBLE]);
			$t->assign('%%VISIBLEARROW%%', $this->_boxInfo[self::STATE_VISIBLE_ARROW]);
			
			return $t->templateParse();
		}
		
		
		/**
		 * Return statistic rows for the order statuses for today and yesterday
		 * Returns rows the following keys: OrderStatus, T and N, where
		 * T is 0 for today, and 1 for yesterday
		 *
		 * @return array rows
		 */
		protected function getLastOrdersStats()
		{
			// Find the statuses of orders placed today and yesterday
			$orderPlaced = intval(time() / 86400) * 86400 - 86400;
			$sql = 'SELECT OrderStatus, FLOOR(UNIX_TIMESTAMP()/86400)-FLOOR((OrderTimestamp)/86400) AS T, COUNT(*) AS N FROM Orders WHERE '.
				'OrderTimestamp >= ' . $orderPlaced . ' GROUP BY OrderStatus, T';
			return miStaticDBUtil::execSelect($sql);
		}
		
		public function setVisible($visible = true)
		{
			if (!$visible) {
				$this->_boxInfo[self::STATE_VISIBLE] = 'none';
				$this->_boxInfo[self::STATE_VISIBLE_ARROW] = 'block';
			}
		}
	}
?>