<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scBestsellersBoxTable extends miTable {
		public function addUiElements(&$rows)
		{
			scFriendlyUrl::initFriendlyUrls($rows, scFriendlyUrl::ROW_TYPE_PRODUCTS);
		}
	}
	
	class scCustomerBestSellersBox implements ICMSBox, IModuleOptions {
		protected $_moduleId;
		protected $_cmsBoxId;
		
		const LIST_BY_NUMBER_OF_ORDERS = 'NumberOfOrders';
		const LIST_BY_NUMBER_OF_ITEMS_SOLD = 'NumberOfItemsSold';
		const LIST_BY_VALUE_OF_ITEMS_SOLD = 'ValueOfItemsSold';
		
		public function __construct($moduleId, $cmsBoxId)
		{
			$this->_moduleId = $moduleId;
			$this->_cmsBoxId = $cmsBoxId;
		}
		
		public function getBoxContents()
		{
			$parser = new scTemplateParser();
			$parser->readTemplate('customer/cmsboxes/bestsellers.tmpl');
			
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0 ' : '';
			
			$products = miStaticDBUtil::execSelect(
				'SELECT *, Products.ProductID as ProductID FROM Bestsellers ' .
				'INNER JOIN Products USING(ProductID) ' .
				'INNER JOIN ProductsML USING(ProductID) ' .
				'LEFT JOIN ProductImages ON (ProductImages.ProductID = Products.ProductID AND ProductImageIsDefault<>0) ' .
				'LEFT JOIN ProductVariants ON (ProductVariants.ProductID = Products.ProductID AND ProductVariantIsDefault<>0) ' .
				'WHERE LanguageID = ? AND ProductIsActive<>0 ' . $hideOutOfStockSql . 'ORDER BY BestsellerSort',
				array(
					scMultiLanguage::getCurrentLanguageId()
				)
			);
			
			$listingItems = scProductService::getListingItemsFromArray($products);
			if (!empty($listingItems)) {
				scProductService::replaceOutOfStockVariants($listingItems);
			}
			
			$options = scProductListingService::OPTION_NO_MODULES;
			$productsSection = scProductListingService::singleton()->getSection('products', $listingItems, scProductService::VIEW_LIST, $options);
			$parser->addSectionInfo($productsSection);
			$parser->assign('%%CMSBOXID%%', $this->_cmsBoxId);
			
			return $parser->templateParse();
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			scModuleOptions::registerOnUpdateCallback($this->_moduleId, array($this, 'onModuleOptionsUpdate'));
			
			$daysField = new miTextWidget($webForm);
			$daysField->setFieldName('BestsellersDays');
			
			$bestsellersCountField = new miTextWidget($webForm);
			$bestsellersCountField->setFieldName('BestsellersCount');
			
			$listByOptions = array(
				self::LIST_BY_NUMBER_OF_ORDERS => _t('Number of orders'),
				self::LIST_BY_NUMBER_OF_ITEMS_SOLD => _t('Quantity of items sold'),
				self::LIST_BY_VALUE_OF_ITEMS_SOLD => _t('Value of items sold')
			);
			$listByField = new miSelectWidget($webForm);
			$listByField->setOptions($listByOptions, self::LIST_BY_VALUE_OF_ITEMS_SOLD);
			$listByField->setFieldName('BestsellersListBy');
			
			$options = new scModuleOptionsInfo();
			$options->addOptionField($daysField);
			$options->addOptionField($bestsellersCountField);
			$options->addOptionField($listByField);
			$options->setTemplateFile('modules/system/blocks/bestsellers_box/options.tmpl');
			return $options;
		}
		
		/**
		 * Called immediately after module options have been updated
		 *
		 * @param array $optionValues
		 */
		public function onModuleOptionsUpdate($optionValues)
		{
			$this->updateBestsellers();
		}
		
		/**
		 * Update the bestsellers
		 */
		public function updateBestsellers()
		{
			$options = scModuleOptions::getCmsBoxOptions($this->_cmsBoxId);
			
			switch ($options['BestsellersListBy']) {
				default:
				case self::LIST_BY_NUMBER_OF_ITEMS_SOLD:
					$listBySql = 'SUM(OrderItemQty)';
					break;
				
				case self::LIST_BY_NUMBER_OF_ORDERS:
					$listBySql = 'COUNT(*)';
					break;
				
				case self::LIST_BY_VALUE_OF_ITEMS_SOLD:
					$listBySql = 'SUM(OrderItemTotal)';
			}
			
			$sql = 'SELECT Products.ProductID, ' . $listBySql . ' AS N' .
				' FROM Orders' .
				' JOIN OrderItems USING(OrderID)' .
				' JOIN Products USING (ProductID)' .
				' WHERE OrderTimestamp > ' . (time() - ($options['BestsellersDays'] * 86400)) .
				' AND OrderStatus!=' . scOrderStatus::ORDER_STATUS_UNFINISHED .
				' AND ProductIsActive<>0' .
				' GROUP BY ProductID ORDER BY N DESC LIMIT ' . intval($options['BestsellersCount']);
			$bestsellerRows = miStaticDBUtil::execSelect($sql);
			
			miStaticDBUtil::startTransaction();
			
			$sql = 'DELETE FROM Bestsellers';
			miStaticDBUtil::execSQL($sql);
			
			$sort = 1;
			foreach ($bestsellerRows as $row) {
				$values = array(
					'ProductID' => $row['ProductID'],
					'BestsellerSort' => $sort++
				);
				miStaticDBUtil::execInsert('Bestsellers', $values);
			}
			
			miStaticDBUtil::commit();
		}
	}
?>