<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scDropdownCategoriesBoxTable extends miTable {
		
		/**
		 * @var string
		 */
		protected $_subcategoriesTemplate;
		
		protected $_categories = array();
		
		protected $_openCategories = null;
		
		public function __construct($recordset)
		{
			parent::__construct($recordset);
			
			$this->addMainPageElements(array('%%HTML_CATEGORIES%%' => ''));
		}
		
		public function setSubcategoriesTemplate($template)
		{
			$this->_subcategoriesTemplate = $template;
		}
		
		protected function addUiElements(&$rows)
		{
			parent::addUiElements($rows);
			scFriendlyUrl::initFriendlyUrls($rows, scFriendlyUrl::ROW_TYPE_CATEGORIES);
			
			foreach ($rows as $row) {
				if (!isset($this->_categories[$row['CategoryParentID']])) {
					$this->_categories[$row['CategoryParentID']] = array();
				}
				$this->_categories[$row['CategoryParentID']][] = $row;
				$this->_openCategories[$row['CategoryID']] = $row['CategoryParentID'];
			}
			
			$this->_openCategories = $this->getOpenCategories($this->getCurrentCategoryId());
			
			$this->addMainPageElements(array(
				'%%HTML_CATEGORIES%%' => (count($rows) ? $this->getParsedTree() : '')
			));
		}
		
		protected function getOpenCategories($categoryId)
		{
			if (!isset($this->_openCategories[$categoryId])) {
				return array($categoryId);
			}
			return array_merge(array($categoryId), $this->getOpenCategories($this->_openCategories[$categoryId]));
		}
		
		protected function getCurrentCategoryId()
		{
			static $categoryId = null;
			
			if ($categoryId === null) {
				$categoryId = (int)miGetParamDefault('CategoryID', 0);
				if (!empty($categoryId)) {
					return $categoryId;
				}
				
				$productId = (int)miGetParamDefault('ProductID', 0);
				if (empty($productId)) {
					$categoryId = '';
				} else {
					$categoryId = scUtil::getDbValue('Products', 'CategoryID', 'ProductID', $productId, '');
				}
			}
			return $categoryId;
		}
		
		protected function getParsedTree($parent = '')
		{
			$categories = &$this->_categories[$parent];
			$categoriesCount = count($categories);
			
			// Initilize the sections so we use only one instance of each
			$hasSubcategoriesSection = new miTemplateParserSectionInfo('hasSubcategories', 1);
			$hasNoSubcategoriesSection = new miTemplateParserSectionInfo('hasSubcategories', 0);
			$isNotCurrentSection = new miTemplateParserSectionInfo('isCurrent', 0);
			// Predefine the sections array
			$hasSubcategoriesSections = array();
			$isOpenSections = array();
			$isCurrentSections = array();
			foreach ($categories as &$category) {
				// Check if the "has" or the "has no" should be added.
				if (isset($this->_categories[$category['CategoryID']])) {
					$category['HTML_SUBCATEGORIES'] = $this->getParsedTree($category['CategoryID']);
					$hasSubcategoriesSections[] = $hasSubcategoriesSection;
				} else {
					$category['HTML_SUBCATEGORIES'] = '';
					$hasSubcategoriesSections[] = $hasNoSubcategoriesSection;
				}
				// Check if this is the current category or not
				if ($this->getCurrentCategoryId() == $category['CategoryID']) {
					$isCurrentSections[] = new miTemplateParserSectionInfo('isCurrent', 1);
				} else {
					$isCurrentSections[] = $isNotCurrentSection;
				}
				
				$isOpenSections[] = new miTemplateParserSectionInfo('isOpen', (int)in_array($category['CategoryID'], $this->_openCategories));
			}
			
			// Initialize the sections array by filling it with non-iterated sections
			$isFirst = array_fill(0, $categoriesCount, new miTemplateParserSectionInfo('isFirst', 0));
			// Set the first section to be iterated once
			// Do not use setIterationsCount because the whole array contains
			// references to the same instance of the section info.
			$isFirst[0] = new miTemplateParserSectionInfo('isFirst', 1);
			
			$isLast = array_fill(0, $categoriesCount, new miTemplateParserSectionInfo('isLast', 0));
			// Set the last section to be iterated once
			$isLast[$categoriesCount-1] = new miTemplateParserSectionInfo('isLast', 1);
			
			$categoriesSection = new miTemplateParserSectionInfo('categories');
			$categoriesSection->setIterationsCount($categoriesCount);
			$categoriesSection->setValues(miUIUtil::templetizeRows(miUIUtil::transposeRows($categories)));
			$categoriesSection->addSubsectionsArray('isFirst', $isFirst); // The first category on each level
			$categoriesSection->addSubsectionsArray('isLast', $isLast); // The last category on each level
			$categoriesSection->addSubsectionsArray('hasSubcategories', $hasSubcategoriesSections);
			$categoriesSection->addSubsectionsArray('isOpen', $isOpenSections);	// The current category and all its parents
			$categoriesSection->addSubsectionsArray('isCurrent', $isCurrentSections);	// The currently selected category
			
			$t = new scTemplateParser();
			$t->readTemplate($this->_subcategoriesTemplate);
			$t->addSectionInfo($categoriesSection);
			return $t->templateParse();
		}
	}
	
	class scDropdownCategoriesBox implements ICMSBox {
		
		const OPTION_JS_ENABLED = 'DropdownCategoriesJsEnabled';
		
		protected $_moduleId;
		protected $_cmsBoxId;
				
		public function __construct($moduleId, $cmsBoxId)
		{
			$this->_moduleId = $moduleId;
			$this->_cmsBoxId = $cmsBoxId;
		}
		
		public function getBoxContents()
		{
			$categories = new scSqlRecordsetML('Categories', 'CategoryID');
			$categories->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$categories->setOrder('CategorySort', 'ASC');
			
			$table = new scDropdownCategoriesBoxTable($categories);
			$table->setSubcategoriesTemplate('customer/cmsboxes/dropdown_categories/dropdown_subcategories.tmpl');
			$table->assign('%%CMSBOXID%%', $this->_cmsBoxId);
			return $table->parse('customer/cmsboxes/dropdown_categories/dropdown_categories.tmpl');
		}
	}
?>