<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scManufacturersBoxTable extends miTable {
		protected $_viewAll = true;
		
		public function addUiElements(&$rows)
		{
			scFriendlyUrl::initFriendlyUrls($rows, scFriendlyUrl::ROW_TYPE_MANUFACTURERS);
		}
		
		public function setViewAll($viewAll)
		{
			$this->_viewAll = $viewAll;
		}
		
		protected function addMainUiElements(&$t)
		{
			if ($this->_viewAll) {
				$viewAllSection = new miTemplateParserSectionInfo('viewAllLink');
				$t->addSectionInfo($viewAllSection);
			}
		}
	}
	
	class scManufacturersRecordset extends scPreparedRecordset {
		public function getAllFields()
		{
			if (isset($this->_rows[0]))
				return array_keys($this->_rows[0]);
			
			return array();
		}
	}
	
	class scCustomerManufacturersBox implements ICMSBox, IModuleOptions {
		const DEFAULT_MANUFACTURERS_COUNT = 5;
		
		protected $_moduleId;
		protected $_cmsBoxId;
		
		public function __construct($moduleId, $cmsBoxId)
		{
			$this->_moduleId = $moduleId;
			$this->_cmsBoxId = $cmsBoxId;
		}
		
		public function getBoxContents()
		{
			// Get the max number of manufacturers for the box to show
			$maxCount = $this->getAllowedManufacturersCount();
			if ($maxCount <= 0)
				$maxCount = self::DEFAULT_MANUFACTURERS_COUNT;
			
			$query = 'SELECT *, Manufacturers.ManufacturerID AS ManufacturerID ' .
				'FROM Manufacturers LEFT JOIN ManufacturersML USING(ManufacturerID) ' . 
				'WHERE LanguageID = ? ORDER BY ManufacturerSort ASC ' .
				'LIMIT 0, ' . ($maxCount + 1);
			$manufacturers = miStaticDBUtil::execSelect($query, array(scMultiLanguage::getCurrentLanguageId()));
			if (count($manufacturers) > $maxCount) {
				$viewAllButton = true;
				array_pop($manufacturers);
			}
			else
				$viewAllButton = false;
				
			$table = new scManufacturersBoxTable(new scManufacturersRecordset($manufacturers));
			$table->setViewAll($viewAllButton);
			$table->assign('%%CMSBOXID%%', $this->_cmsBoxId);
			return $table->parse('customer/cmsboxes/manufacturers.tmpl');
		}
		
		public function getOptionFields(miWebForm &$form)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ManufacturersCount',
					'validator' => 'scValidatorInt'
				)
			);

			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $form);
			$options->setTemplateFile('modules/system/blocks/manufacturers_box/options.tmpl');
			
			return $options;
		}
		
		protected function getAllowedManufacturersCount()
		{
			$options = scModuleOptions::getCmsBoxOptions($this->_cmsBoxId);
			if (!isset($options['ManufacturersCount'])) {
				return 0;
			}
			return $options['ManufacturersCount'];
		}
	}
?>