<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scNewProductsBoxTable extends miTable {
		public function addUiElements(&$rows)
		{
			scFriendlyUrl::initFriendlyUrls($rows, scFriendlyUrl::ROW_TYPE_PRODUCTS);
		}
	}
	
	class scCustomerNewProductsBox implements ICMSBox, IModuleOptions {
		
		const DEFAULT_COUNT = 3;
		
		protected $_moduleId;
		protected $_cmsBoxId;
		
		public function __construct($moduleId, $cmsBoxId)
		{
			$this->_moduleId = $moduleId;
			$this->_cmsBoxId = $cmsBoxId;
		}
		
		public function getBoxContents()
		{
			$parser = new scTemplateParser();
			$parser->readTemplate('customer/cmsboxes/new_products.tmpl');
			
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0 ' : '';
			
			// Display only configured number of products
			$maxCount = $this->getProductsCount();
			if ($maxCount <= 0)
				$maxCount = self::DEFAULT_COUNT;
			
			$products = miStaticDBUtil::execSelect(
				'SELECT *, Products.ProductID as ProductID FROM Products ' .
				'LEFT JOIN ProductsML ON(Products.ProductID=ProductsML.ProductID AND LanguageID = ?) ' .
				'LEFT JOIN ProductImages ON (ProductImages.ProductID = Products.ProductID AND ProductImageIsDefault<>0) ' .
				'LEFT JOIN ProductVariants ON (ProductVariants.ProductID = Products.ProductID AND ProductVariantIsDefault<>0) ' .
				'WHERE ProductIsActive<>0 ' . $hideOutOfStockSql .
				'ORDER BY ProductCreatedTimestamp DESC ' .
				'LIMIT ' . $maxCount,
				array(
					scMultiLanguage::getCurrentLanguageId()
				)
			);
			
			$listingItems = scProductService::getListingItemsFromArray($products);
			if (!empty($listingItems)) {
				scProductService::replaceOutOfStockVariants($listingItems);
			}
			
			$options = scProductListingService::OPTION_NO_MODULES;
			$productsSection = scProductListingService::singleton()->getSection('products', $listingItems, scProductService::VIEW_LIST, $options);
			$parser->addSectionInfo($productsSection);
			
			$parser->assign('%%CMSBOXID%%', $this->_cmsBoxId);
			return $parser->templateParse();
		}
		
		/**
		 * @param miWebForm $webForm
		 * @return scModuleOptionsInfo
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$countField = new miTextWidget($webForm);
			$countField->setFieldName('NewProductsBoxProductsCount');
			
			$options = new scModuleOptionsInfo();
			$options->addOptionField($countField);
			$options->setTemplateFile('modules/system/blocks/new_products_box/options.tmpl');
			return $options;
		}
		
		/**
		 * @return int|false
		 */
		public function getProductsCount()
		{
			$options = scModuleOptions::getCmsBoxOptions($this->_cmsBoxId);
			if (!isset($options['NewProductsBoxProductsCount'])) {
				return 0;
			}
			return (int)$options['NewProductsBoxProductsCount'];
		}
	}
?>