<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductAtributesFilterBox implements ICMSBox {
		
		protected $_moduleId;
		protected $_cmsBoxId;
		
		protected $_usedInPages = array(
			'categories.php',
			'manufacturers.php'
		);
		
		public function __construct($moduleId, $cmsBoxId)
		{
			$this->_moduleId = $moduleId;
			$this->_cmsBoxId = $cmsBoxId;
		}
		
		public function getBoxContents()
		{
			$file = scPaths::getCurrentScriptBaseName();
			if (!in_array($file, $this->_usedInPages)) {
				return '';
			}
			
			$productClasses = scUtil::getProductClassesArray();
			if (count($productClasses) == 0) {
				return ''; // No filters will be available as there are no classes.
			}
			
			$formData = array();
			$formData[scProductAtributesFilter::KEY_CLASS_ID] = miGetParamDefault(scProductAtributesFilter::KEY_CLASS_ID, 0);
			$formData[scProductAtributesFilter::KEY_ATTRIBUTES] = miGetParamDefault(scProductAtributesFilter::KEY_ATTRIBUTES, array());
			
			$form = new scProductAtributesFilterForm($formData);
			
			$widget = new miSelectWidget($form, array());
			$widget->setFieldName(scProductAtributesFilter::KEY_CLASS_ID);
			$widget->setOptions(array(0 =>  _t('Any')) + $productClasses);
			$form->addWidget(scProductAtributesFilter::KEY_CLASS_ID, $widget);
			
			$widget = new scProductAttributesSearchWidget($form, array());
			$widget->setProductClassId($formData[scProductAtributesFilter::KEY_CLASS_ID]);
			$widget->setFieldName(scProductAtributesFilter::KEY_ATTRIBUTES);
			$widget->setTemplate('customer/cmsboxes/product_attributes/filter_attributes.tmpl');
			$widget->setIncludeSystemAttributes(scProductAttributes::GET_SYSTEM_ATTRIBUTES_INCLUDE);
			$form->addWidget(scProductAtributesFilter::KEY_ATTRIBUTES, $widget);
			
			$form->assignTemplateVars(array('%%FORM_ACTION%%' => $this->getFormAction(), '%%CMSBOXID%%' => $this->_cmsBoxId));
			
			return $form->parse('customer/cmsboxes/product_attributes/filter.tmpl', true);
		}
		
		protected function getFormAction()
		{
			// Get the current page
			$currentPage = scUrlUtil::parse($_SERVER['REQUEST_URI']);
			// Make sure the QUERY part is set
			if (!isset($currentPage[scUrlUtil::QUERY])) {
				$currentPage[scUrlUtil::QUERY] = '';
			}
			// Convert its query string to an array
			$currentVars = scUrlUtil::parseStr($currentPage[scUrlUtil::QUERY]);
			// Add any POST data into the query string
			$currentVars = array_merge($currentVars, $_POST);
			// Remove the vars that are going to be set through the inputs in the form
			if (isset($currentVars[scProductAtributesFilter::KEY_CLASS_ID])) {
				unset($currentVars[scProductAtributesFilter::KEY_CLASS_ID]);
			}
			if (isset($currentVars[scProductAtributesFilter::KEY_ATTRIBUTES])) {
				unset($currentVars[scProductAtributesFilter::KEY_ATTRIBUTES]);
			}
			// Convert the query array back to string
			$currentPage[scUrlUtil::QUERY] = http_build_query($currentVars);
			// If not needed, remove the QUERY part
			if (empty($currentPage[scUrlUtil::QUERY])) {
				unset($currentPage[scUrlUtil::QUERY]);
			}
			
			// return the glued URI
			return scUrlUtil::glue($currentPage);
		}
	}
	
	class scProductAtributesFilter implements IProductsListingFilter, IUserModule {
		
		const KEY_CLASS_ID = 'ProductAttributesFilter_ProductClassID';
		const KEY_ATTRIBUTES = 'ProductAttributesFilter';
		
		/**
		 * @return array
		 */
		public function getSqlFilters()
		{
			$productClassId = $this->getFilterProductClassId();
			if ($productClassId <= 0) {
				return array();
			}
			$attributeValues = miGetParamDefault(self::KEY_ATTRIBUTES, array());
			
			$filters[] = 'ProductClassID = ' . (int)$productClassId;
			
			$attributeFilters = scProductAttributes::singleton()->getSqlFilters($productClassId, $attributeValues);
			foreach ($attributeFilters as $filter) {
				$filters[] = 'Products.ProductID IN (' . $filter . ')';
			}
			
			return $filters;
		}
		
		public function getFilterVars()
		{
			return array(
				self::KEY_CLASS_ID,
				self::KEY_ATTRIBUTES,
			);
		}
		
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('', 'scProductAttributesFilterGetAttributesAction');
		}
		
		public function getView()
		{
			return new miView();
		}
		
		public function getFilterProductClassId()
		{
			return (int)miGetParamDefault(scProductAtributesFilter::KEY_CLASS_ID, 0);
		}
	}
	
	class scProductAttributesFilterGetAttributesAction extends miAction {
		
		public function doAction()
		{
			$classId = (int)miGetParamDefault('ProductClassID', 0);
			$attributes = miGetParamDefault(scProductAtributesFilter::KEY_ATTRIBUTES, array());
			$template = 'customer/cmsboxes/product_attributes/filter_attributes.tmpl';
			
			$html = scProductAttributesSearchWidget::getProductClassAttributesHtml($classId, $attributes, scProductAtributesFilter::KEY_ATTRIBUTES, $template);
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			return true;
		}
	}
	
	class scProductAtributesFilterForm extends miWebForm {
		
		protected $_additionalTemplateVars = array();
		
		public function assignTemplateVars($templateVars)
		{
			$this->_additionalTemplateVars = $templateVars;
		}
		
		
		protected function onParse(miTemplateParser &$templateParser)
		{
			$templateParser->assignArray($this->_additionalTemplateVars);
		}
	}
?>