<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scRecentlyViewedProducts
	{
		protected $_products = array();
		protected static $_instance = null;
		
		public function __construct()
		{
		}
		
		public function __destruct()
		{
			$_SESSION['RecentlyViewed'] = serialize($this);
		}

		/**
		 * Returns the one and only instance of the object
		 * 
		 * @return scRecentlyViewedProducts
		 */
		public static function singleton()
		{
			if (!is_null(self::$_instance))
				return self::$_instance;
			
			if (array_key_exists('RecentlyViewed', $_SESSION))
				self::$_instance = unserialize($_SESSION['RecentlyViewed']);
			else
				self::$_instance = new scRecentlyViewedProducts();
			
			return self::$_instance;
		}
				
		public function addProductId($productId, $allowedProductsCount)
		{
			$key = array_search($productId, $this->_products);
			if ($key !== false)
				unset($this->_products[$key]);
				
			if (count($this->_products) >= $allowedProductsCount)
				array_pop($this->_products);
				
			array_unshift($this->_products, $productId);
		}
		
		public function getProducts()
		{
			return $this->_products;
		}
	}
	
	class scCustomerRecentlyViewedBox implements ICMSBox, IModuleOptions {
		
		protected $_moduleId;
		protected $_cmsBoxId;

		const DEFAULT_ALLOWED_COUNT = 3;
		
		public function __construct($moduleId, $cmsBoxId)
		{
			$this->_moduleId = $moduleId;
			$this->_cmsBoxId = $cmsBoxId;
		}
		
		public function getBoxContents()
		{		
			$recentlyViewed = scRecentlyViewedProducts::singleton()->getProducts();
			if (count($recentlyViewed) == 0) {
				return '';
			}
			
			$allowedCount = $this->getAllowedProductsCount();
			if ($allowedCount <= 0)
				$allowedCount = self::DEFAULT_ALLOWED_COUNT;
			
			// Display only configured number of products
			$recentlyViewed = array_slice($recentlyViewed, 0, $allowedCount);
			
			$parser = new scTemplateParser();
			$parser->readTemplate('customer/cmsboxes/recently_viewed.tmpl');
			
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0' : '';
			$variantSql = 'LEFT JOIN ProductVariants ON (ProductVariants.ProductID = Products.ProductID AND ProductVariantIsDefault<>0) ';
			
			$recentlyViewed = array_map('sql_escape_string', $recentlyViewed);
			$products = miStaticDBUtil::execSelect(
				'SELECT *, Products.ProductID as ProductID FROM Products ' .
				'INNER JOIN ProductsML USING(ProductID) ' .
				'LEFT JOIN ProductImages ON (ProductImages.ProductID = Products.ProductID AND ProductImageIsDefault<>0) ' .
				$variantSql .
				'WHERE Products.ProductID IN ("' . implode('", "', $recentlyViewed) . '") AND ProductIsActive<>0 AND LanguageID = ? ' . $hideOutOfStockSql,
				array(
					scMultiLanguage::getCurrentLanguageId()
				)
			);
			
			$recentlyViewed = array_flip($recentlyViewed);
			
			$productsViewed = array();
			foreach ($products as $product) {
				// Order the products in the proper order
				$index = $recentlyViewed[intval($product['ProductID'])];
				$productsViewed[$index] = $product;
			}
			unset($products);
			
			$listingItems = scProductService::getListingItemsFromArray($productsViewed);
			if (!empty($listingItems)) {
				scProductService::replaceOutOfStockVariants($listingItems);
			}
			
			$options = scProductListingService::OPTION_NO_MODULES;
			$productsSection = scProductListingService::singleton()->getSection('products', $listingItems, scProductService::VIEW_LIST, $options);
			$parser->addSectionInfo($productsSection);
			$parser->assign('%%CMSBOXID%%', $this->_cmsBoxId);
			
			return $parser->templateParse();
		}
		
		public function getOptionFields(miWebForm &$form)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'RecentlyViewedProductsCount',
					'validator' => 'scValidatorInt'
				)
			);

			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $form);
			$options->setTemplateFile('modules/system/blocks/recently_viewed_box/options.tmpl');
			
			return $options;
		}
		
		public function addProductId($productId)
		{
			scRecentlyViewedProducts::singleton()->addProductId($productId, $this->getAllowedProductsCount());
		}
		
		public function getAllowedProductsCount()
		{
			$options = scModuleOptions::getCmsBoxOptions($this->_cmsBoxId);
			if (!isset($options['RecentlyViewedProductsCount'])) {
				return 0;
			}
			return $options['RecentlyViewedProductsCount'];
		}
	}
?>