<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scEditOrderAction extends miEditAction {
		
		protected $_warnings = array();
		
		public function doAction()
		{
			try {
				$vmController = $this->_view->getViewMapperController();
				$vmController->read();
				
				$this->_view->callPlugin($this, 'preShowForm');
				
				$form = $this->getWebForm();
				$vmController->setDataToWebForm();
				
				$this->_view->initMainPageElements();
				
				$uiCart = new scEditOrderUICart();
				$uiCart->setCart(scEditOrderCart::getCart());
				$uiCart->getTemplater()->set(scUICartTemplater::ITEMS, $this->_view->getTemplateFilename(scEditOrderView::TEMPLATE_ID_CART_ITEMS));
				$uiCart->getTemplater()->set(scUICartTemplater::BUNDLED_ITEMS, $this->_view->getTemplateFilename(scEditOrderView::TEMPLATE_ID_BUNDLED_ITEMS));
				
				// $form->addTemplateSectionInfo($uiCart->getCheckoutModulesSection());
				$form->addTemplateSectionInfo($uiCart->getCouponsTemplateSection());
				
				$msg = miGetParamDefault(miMessage::PARAM_MESSAGE, '');
				$mainPageElements = array(
					'%%ORDERTOTALLINES%%' => $uiCart->getOrderTotalLinesHtml($this->_view->getTemplateFilename(scEditOrderView::TEMPLATE_ID_ORDER_TOTAL_LINES)),
					'%%CARTITEMS%%' => $uiCart->getCartItemsHtml(),
					'%%INITIAL_WARNINGS%%' => $uiCart->getInitialWarningsHtml(),
				);
				
				$orderInfoVars = scAdminOrdersUI::singleton()->getOrderInfoVars($this->_view->getDomainObject());
				
				$form->addMainPageElements(array_merge(
					$this->_view->getMainPageElements(),
					$mainPageElements,
					miUIUtil::templetizeRows($orderInfoVars)
				));
				
				$html = $form->parse($this->_view->getTemplateFileName(scEditOrderView::TEMPLATE_ID_EDIT), true);
				$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($controllerCommand);
				
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToOrderControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scEditOrderApplyAction extends miAction {
		
		public function doAction()
		{
			try {
				miStaticDBUtil::startTransaction();
				
				$orderId = (int)miGetParam('OrderID');
				$order = $this->_view->getDomainObject();
				$order->read($orderId);
				$customerCurrency = $order->getCustomerCurrency();
				
				scEditOrderService::singleton()->setCurrentOrder($order);
				
				$orderQuantitiesReduced = $order->get('OrderQuantitiesInStockReduced');
				if ($orderQuantitiesReduced) {
					$order->returnInventory();
				}
				
				// Get the cart that is going to be converted to an order.
				$cart = scEditOrderCart::getCart();
				
				// Delete previous data.
				miStaticDBUtil::execDelete('OrderItems', 'OrderID', $orderId);
				miStaticDBUtil::execDelete('OrderTotalLines', 'OrderID', $orderId);
				
				$customerInfo = $cart->getCustomerInfo();
				$couponInfo = $cart->getCoupon();
				
				// Set the basic order data.
				// Since the order is the holder of items and total lines it should be updated after they have been.
				$order->setCustomerInfo($customerInfo);
				$order->set('OrderCurrencyCode', scCurrencyUtil::getBaseCurrencyCode());
				$order->set('OrderCustomerCurrencyCode', $customerCurrency->getCurrencyCode());
				
				$order->set('OrderTotal', $cart->getOrderTotal()->getGrandTotalAmount()->get());
				$order->set('OrderCustomerCurrencyTotal', $cart->getOrderTotal()->getGrandTotalAmount()->get($customerCurrency));
				$order->set('OrderTotalPayAmount', $cart->getOrderTotal()->getGrandTotalAmount()->get());
				
				$order->set('OrderCouponCode', (is_null($customerInfo) || is_null($couponInfo['CouponCode']) ? '' : $couponInfo['CouponCode']));
				
				scEditOrderService::singleton()->saveOrderItems($order, $cart->getItems());
				
				// Save order total lines
				$orderTotalLines = $cart->getOrderTotal()->getOrderTotalLines();
				scEditOrderService::singleton()->saveOrderTotalLines($orderTotalLines, $orderId);
				
				// After the order items have been saved, check if inventory should be reduced.
				if ($orderQuantitiesReduced) {
					$order->reduceInventory();
				}
				
				// Update the order
				$order->update();
			} catch (miException $exception) {
				miStaticDBUtil::rollback();
				
				$msg = sprintf(_t('Could not save the order: %s.'), $exception->getMessage());
				$this->_view->addRedirectToOrderControllerCommand($msg, miMessage::MSG_TYPE_ERROR);
				return false;
			}
			
			miStaticDBUtil::commit();
			
			scEditOrderCart::discardCurrentCart();
			
			$msg = sprintf($this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG'), $orderId);
			$this->_view->addRedirectToOrderControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			
			return true;
		}
	}
	
	class scEditOrderDiscardAction extends miAction {
		
		public function doAction()
		{
			scEditOrderCart::discardCurrentCart();
			$this->_view->addRedirectToOrderControllerCommand('', miMessage::MSG_TYPE_HIDDEN);
			return true;
		}
	}
	
	class scEditOrderListProductsAction extends miAction {
		
		/**
		 * @return miTemplateParser
		 */
		protected function getTemplateParser()
		{
			$filterCategoryId = (int)miGetParamDefault('CategoryID', '');
			$filterSubcategories = (miGetParamDefault('subcategories', '') == 1);
			$filterProductName = miGetParamDefault('ProductName', '');
			
			$view = scProductService::VIEW_LIST;
			$recordsPerPage = scCustomerAreaPager::getProductsPerPage();
			
			$pageNum = miGetParamDefault('page', 1);
			if ($pageNum < 1) {
				$pageNum = 1;
			}
			
			$search = scUserSearch::singleton();
			$search->setVariantsMode(scUserSearch::VARIANTS_MODE_LIST_PRODUCTS);
			$search->addCriteria(scUserSearch::CATEGORY_ID, array($filterCategoryId, $filterSubcategories));
			$search->addCriteria(scUserSearch::PRODUCT_NAME, array($filterProductName, true));
			$search->addOrderBy('ProductName');
			$productsCount = $search->getRecordsCount();
			$products = $search->getRecords(($pageNum - 1) * $recordsPerPage, $recordsPerPage);
			
			$this->modifyProductsData($products);
			
			$productsSection = scEditOrderProductListingService::singleton()->getSection('products', $products, scProductService::VIEW_LIST, scProductListingService::OPTION_INCLUDE_PRODUCT_OPTIONS_SECTION);
			
			$url = 'admin/module.php?ModuleName=com.summercart.editorder&OrderID=' . intval(miGetParam('OrderID')) . '&action=dmListProducts&load=table';
			$url = scUrlUtil::convertUrl($url, scUrlUtil::URL_TYPE_DOCROOT);
			if (!empty($filterCategoryId)) {
				$url = scUrlUtil::addQueryParams($url, 'CategoryID=' . $filterCategoryId);
				if ($filterSubcategories) {
					$url = scUrlUtil::addQueryParams($url, 'subcategories=1');
				}
			}
			if (!empty($filterProductName)) {
				$url = scUrlUtil::addQueryParams($url, 'ProductName=' . $filterProductName);
			}
			
			$paging = scCustomerAreaPager::getPaging($pageNum, ceil($productsCount / $recordsPerPage), $url);
			
			$t = new scTemplateParser();
			$t->readTemplate($this->_view->getTemplateFilename(scEditOrderView::TEMPLATE_ID_LIST_PRODUCTS));
			$t->addSectionInfo($productsSection);
			$t->assign('%%HTML_PRODUCTS_PAGES%%', $paging);
			return $t;
		}
		
		public function doAction()
		{
			try {
				$this->_view->initMainPageElements();
				
				$t = $this->getTemplateParser();
				$t->assignArray($this->_view->getMainPageElements());
				$html = $t->templateParse();
			} catch (miException $e) {
				$this->_view->addRedirectToOrderControllerCommand($e->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;	
			}
			
			$controllerCommand = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($controllerCommand);
			return true;
		}
		
		protected function modifyProductsData(&$products)
		{
			try {
				scEditOrderCart::setCurrentOrderId((int)miGetParam('OrderID'));
				$cartItems = scEditOrderCart::getCart()->getItems();
				if (empty($cartItems)) {
					return;
				}
				
				$quantitiesInUse = array(); // Store quantities that are currently in use by the currently editted order.
				foreach ($cartItems as $cartItem) {
					if (!($cartItem instanceof scCartItemProduct)) {
						continue;
					}
					$quantitiesInUse[$cartItem->getProductId()] = $cartItem->getQuantity();
				}
				
				foreach ($products as $product) {
					if (!($product instanceof scProductSL)) {
						continue;
					}
					$productId = $product->getProductId();
					if (!isset($quantitiesInUse[$productId])) {
						continue;
					}
					$availableQuantity = $product->get('ProductQuantity') - $quantitiesInUse[$productId];
					$product->set('ProductQuantity', $availableQuantity);
				}
			} catch (miException $e) {
				// TODO: There is no current way to let the user of Edit Order that there is an error here other than logging.
				scLog::singleton()->log(scLog::SEVERITY_ERROR, 'EditOrder, modifyProductsData: ' . $e->getMessage());
			}
		}
	}
	
	class scEditOrderSelectProductAction extends scEditOrderListProductsAction {
		
		/**
		 * @return miTemplateParser
		 */
		protected function getTemplateParser()
		{
			$form = new miWebForm(array());
			
			$categorySelectWidget = new scCategorySelect($form, array());
			$categoriesArray = array('' => _t('Select a Category')) + $categorySelectWidget->getCategoriesTree(0, true);
			$categorySelectHtml = scUIUtil::getConstDropdown('Category', '', $categoriesArray, array('id' => 'CategoryID'));
			
			$t = new scTemplateParser();
			$t->assign('%%CATEGORYID%%', $categorySelectHtml);
			$t->readTemplate($this->_view->getTemplateFilename(scEditOrderView::TEMPLATE_ID_SELECT_CATEGORY));
			return $t;
		}
	}
	
	class scEditOrderCreateBlankOrderAction extends miCreateAction {
		
		public function doAction()
		{
			try {
				// Create a cart.
				$cart = new scEditOrderCart();
				
				// Set the order's currency to the default one.
				$currency = scCurrencyUtil::getBaseCurrency();
				$cart->setCurrentCurrency($currency);
				
				$customerInfo = $this->getCustomerInfo();
				$cart->setCustomerInfo($customerInfo);
				
				// The checkout module is required. Create the order as if it was created through Wizard Checkout.
				$moduleId = scModulesFactory::getModuleIdByName('com.summercart.checkout.wizard');
				$checkoutModuleId = scUtil::getDbValue('CheckoutModules', 'CheckoutModuleID', 'ModuleID', $moduleId);
				
				$orderId = scEditOrderOrdersInProgress::singleton()->createOrder(0, $currency->getCurrencyCode(), $cart, $checkoutModuleId);
				
				$loc = 'module.php?ModuleName=com.summercart.editorder&OrderID=' . $orderId;
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $e) {
				$this->_view->addRedirectToOrderControllerCommand($e->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		/**
		 * @return array
		 */
		protected function getCustomerInfo()
		{
			if ((int)miGetParamDefault('CustomerID', 0) == 0) {
				// Use the guest customer to fill customer info.
				$customer = scCustomers::getGuestCustomer();
				$customerInfo = $customer->getBillingAndShippingAddresses();
				$customerInfo['CustomerID'] = ICheckoutModule::GUEST_USER_ID;
			} else {
				$customerId = (int)miGetParam('CustomerID');
				$customer = new scCustomerDomainObject();
				$customer->read($customerId);
				$customerInfo = $customer->getBillingAndShippingAddresses();
				$customerInfo['CustomerID'] = $customerId;
				$customerInfo['OrderCustomerEmail'] = $customer->get('CustomerEmail');
			}
			return $customerInfo;
		}
	}
?>