<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	abstract class scEditOrderAddress extends scAddress implements IObservable {
		
		protected $_customerInfoToAddress = array(
			'Address1' => 'setAddressLine1',
			'Address2' => 'setAddressLine2',
			'City' => 'setCity',
			'CityId' => 'setCityId',
			'Country' => 'setCountryId',
			'State' => 'setStateId',
			'Zip' => 'setPostalCode',
		);
		
		protected $_billingAddressPrefix = 'CustomerBilling';
		protected $_shippingAddressPrefix = 'CustomerShipping';
		
		protected $_observers = array();
		
		public function attachObserver(IObserver $observer)
		{
			$this->_observers[] = $observer;
		}
		
		public function detachAllObservers()
		{
			$this->_observers = array();
		}
		
		public function notifyObservers($updatedField = '')
		{
			if (empty($updatedField)) {
				return;
			}
			
			foreach ($this->_observers as $observer) {
				$observer->notify($this, $updatedField);
			}
		}
		
		protected function isPrefixedField($field, $prefix)
		{
			$prefixLength = strlen($prefix);
			if (strlen($field) <= $prefixLength) {
				return false;
			}
			if (substr($field, 0, $prefixLength) != $prefix) {
				return false;
			}
			return true;
		}
	}
	
	class scEditOrderBillingAddress extends scEditOrderAddress implements IObserver {
		
		/**
		 * @param IObservable $subject
		 * @param string $updatedField
		 */
		public function notify(IObservable $subject, $updatedField = '')
		{
			if (empty($updatedField)) {
				return;
			}
			
			if (!($subject instanceof scEditOrderCustomerInfo)) {
				return;
			}
			
			$customerInfo = scEditOrderCart::getCart()->getCustomerInfo();
			$oldHash = $this->getHash();
			
			// Is $updateField a billing address field
			if (!$this->isPrefixedField($updatedField, $this->_billingAddressPrefix)) {
				return;
			}
			
			$prefixLength = strlen($this->_billingAddressPrefix);
			$addressField = substr($updatedField, $prefixLength);
			if (isset($this->_customerInfoToAddress[$addressField])) {
				$setter = $this->_customerInfoToAddress[$addressField];
				$this->$setter($customerInfo[$updatedField]);
			}
			
			scEditOrderCart::getCart()->setCustomerInfoField($updatedField, $customerInfo[$updatedField]);
			$newHash = $this->getHash();
			if ($newHash != $oldHash) {
				$this->notifyObservers($updatedField);
			}
		}
		
		/**
		 * @return int
		 */
		protected function getHash()
		{
			return crc32($this->_addressLine1 . $this->_addressLine2 . $this->_city .
				$this->_stateId . $this->_postalCode . $this->_countryId . $this->_cityId);
		}
	}
	
	class scEditOrderShippingAddress extends scEditOrderAddress implements IObserver {
		
		/**
		 * @param IObservable $subject
		 * @param string $updatedField
		 */
		public function notify(IObservable $subject, $updatedField = '')
		{
			if (empty($updatedField)) {
				return;
			}
			
			if (!($subject instanceof scEditOrderCustomerInfo)) {
				return;
			}
			
			$customerInfo = scEditOrderCart::getCart()->getCustomerInfo();
			
			$oldHash = $this->getHash();
			
			// Is $updateField a shipping address field
			if ($this->isPrefixedField($updatedField, $this->_shippingAddressPrefix)) {
				$prefixLength = strlen($this->_shippingAddressPrefix);
				$addressField = substr($updatedField, $prefixLength);
				if (!isset($this->_customerInfoToAddress[$addressField])) {
					return;
				}
				$setter = $this->_customerInfoToAddress[$addressField];
				$myUpdateField = $updatedField;
				$this->$setter($customerInfo[$updatedField]);
				
			} else if ($customerInfo['CustomerShipToBillingAddress']) {
				// If $updateField is the one that says "copy all billing data to shipping"
				if ($updatedField == 'CustomerShipToBillingAddress') {
					foreach ($customerInfo as $field => $value) {
						// Check if the field is a billing one.
						if (!$this->isPrefixedField($field, $this->_billingAddressPrefix)) {
							continue;
						}
						// It's a billing field. Have to set it to the shipping.
						$prefixLength = strlen($this->_billingAddressPrefix);
						$addressField = substr($field, $prefixLength);
						$myUpdateField = $this->_shippingAddressPrefix . $addressField;
						scEditOrderCart::getCart()->setCustomerInfoField($myUpdateField, $value);
					}
					
				// Is $updateField a billing address field
				} else if ($this->isPrefixedField($updatedField, $this->_billingAddressPrefix)) {
					$prefixLength = strlen($this->_billingAddressPrefix);
					$addressField = substr($updatedField, $prefixLength);
					if (!isset($this->_customerInfoToAddress[$addressField])) {
						return;
					}
					$myUpdateField = $this->_shippingAddressPrefix . $addressField;
					scEditOrderCart::getCart()->setCustomerInfoField($myUpdateField, $customerInfo[$updatedField]);
					
				}
			} else {
				return;
			}
			
			$newHash = $this->getHash();
			if ($newHash != $oldHash) {
				$this->notifyObservers($myUpdateField);
			}
		}
		
		/**
		 * @return int
		 */
		protected function getHash()
		{
			return crc32($this->_addressLine1 . $this->_addressLine2 . $this->_city .
				$this->_stateId . $this->_postalCode . $this->_countryId . $this->_cityId);
		}
	}
?>