<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	abstract class scEditOrderAjaxAction extends miActionWithWebForm implements IObserver {
		
		/**
		 * @return bool
		 */
		abstract protected function modifySessionData();
		
		const JSON_KEY_SUCCESS = 'success';
		const JSON_KEY_WARNINGS = 'warnings';
		const JSON_KEY_FIELDS = 'fields';
		const JSON_KEY_ALERTS = 'alerts';
		
		protected $_warnings = array();
		
		protected $_alerts = array();
		
		protected $_fields = array();
		
		/**
		 * @var scOrder
		 */
		protected $_order;
		
		public function doAction()
		{
			header('Content-Type: text/plain; charset=utf-8');
			
			$this->initObservation();
			
			try {
				$this->_order = new scOrder((int)miGetParam('OrderID'));
				scEditOrderCartItemService::singleton()->setCurrentOrder($this->_order);
				
				$modificationSuccess = $this->modifySessionData();
			} catch (miException $e) {
				$modificationSuccess = false;
				$this->addWarnings(array($e->getMessage()));
			}
			
			$this->endObservation();
			
			echo scJson::jsonEncode(array(
				self::JSON_KEY_SUCCESS => $modificationSuccess,
				self::JSON_KEY_WARNINGS => $this->_warnings,
				self::JSON_KEY_FIELDS => $this->_fields,
				self::JSON_KEY_ALERTS => $this->_alerts
			));
			exit;
		}
		
		public function notify(IObservable $subject, $updatedField = '')
		{
			if ($subject instanceof scEditOrderCustomerInfo) {
				$customerInfo = scEditOrderCart::getCart()->getCustomerInfo();
				
				$mapping = scEditOrderCart::getCart()->getCartToOrderMapping();
				$orderField = $updatedField;
				if (isset($mapping[$updatedField])) {
					$orderField = $mapping[$updatedField];
				}
				// The ShippingModuleAndMethod are handled by the scEditOrderShippingMethodWidget
				if ($orderField != 'ShippingModuleAndMethod') {
					$this->_fields[$orderField] = $customerInfo[$updatedField];
				}
			} else if ($subject instanceof scEditOrderShippingMethodWidget) {
				$this->_fields = array_merge($this->_fields, $subject->getEditableControl());
			} else if ($subject instanceof scOrderTotal) {
				$scEditOrderUICart = new scEditOrderUICart;
				$scEditOrderUICart->setCart(scEditOrderCart::getCart());
				$this->_fields['OrderTotalLines'] = $scEditOrderUICart->getOrderTotalLinesHtml($this->_view->getTemplateFilename(scEditOrderView::TEMPLATE_ID_ORDER_TOTAL_LINES));
			} else if ($subject instanceof scEditOrderCart) {
				$scEditOrderUICart = new scEditOrderUICart;
				$scEditOrderUICart->setCart(scEditOrderCart::getCart());
				$scEditOrderUICart->getTemplater()->set(scUICartTemplater::ITEMS, $this->_view->getTemplateFilename(scEditOrderView::TEMPLATE_ID_CART_ITEMS));
				$scEditOrderUICart->getTemplater()->set(scUICartTemplater::BUNDLED_ITEMS, $this->_view->getTemplateFileName(scEditOrderView::TEMPLATE_ID_BUNDLED_ITEMS));
				$this->_fields['CartItems'] = $scEditOrderUICart->getCartItemsHtml();
			}
		}
		
		/**
		 * @return scEditOrderWebForm
		 */
		public function getWebForm()
		{
			if ($this->_webForm == null) {
				$this->_webForm = new scEditOrderWebForm(array());
				$this->_view->getViewMapperController()->setWebForm($this->_webForm);
				$this->_view->initWebForm($this->_webForm);
			}
			return $this->_webForm;
		}
		
		protected function initObservation()
		{
			$cart = scEditOrderCart::getCart();
			$customerInfo = $cart->getCustomerInfoSubject();
			$billingAddress = $cart->getBillingAddressObject();
			$shippingAddress = $cart->getShippingAddressObject();
			$orderTotal = $cart->getOrderTotal();
			$shippingWidget = $this->getWebForm()->getWidget('ShippingModuleAndMethod');
			$billingCityIdWidget = $this->getWebForm()->getWidget('OrderBillingCityID');
			$shippingCityIdWidget = $this->getWebForm()->getWidget('OrderShippingCityID');
			
			$cart->attachObserver($shippingWidget);
			$cart->attachObserver($orderTotal);
			$cart->attachObserver($this);
			
			$billingAddress->attachObserver($billingCityIdWidget);
			
			$shippingAddress->attachObserver($shippingCityIdWidget);
			$shippingAddress->attachObserver($shippingWidget);
			
			$customerInfo->attachObserver($billingAddress);
			$customerInfo->attachObserver($shippingAddress);
			$customerInfo->attachObserver($orderTotal);
			$customerInfo->attachObserver($this);
			
			$shippingWidget->attachObserver($orderTotal);
			$shippingWidget->attachObserver($this);
			
			$orderTotal->attachObserver($this);
		}
		
		protected function endObservation()
		{
			$cart = scEditOrderCart::getCart();
			
			$cart->detachAllObservers();
			$cart->getBillingAddressObject()->detachAllObservers();
			$cart->getShippingAddressObject()->detachAllObservers();
			$cart->getCustomerInfoSubject()->detachAllObservers();
			$this->getWebForm()->getWidget('ShippingModuleAndMethod')->detachAllObservers();
			$cart->getOrderTotal()->detachAllObservers();
		}
		
		protected function addWarnings($warnings)
		{
			$this->_warnings = array_merge($this->_warnings, $warnings);
		}
		
		protected function addAlerts($alerts)
		{
			$this->_alerts = array_merge($this->_alerts, $alerts);
		}
	}
	
	class scEditOrderUpdateField extends scEditOrderAjaxAction {
		
		public function modifySessionData()
		{
			$fieldName = miGetParam('fieldName');
			$fieldValue = miGetParam($fieldName);
			
			// We need a specific handler in order to validate fields and get their errors.
			// The default ErrorHandler does not have a getErrors() method.
			$handler = new scEditOrderValidationErrorsHandler();
			
			$form = $this->getWebForm();
			$form->setErrorsHandler($handler);
			$widget = $form->getWidget($fieldName);
			
			if (is_null($widget)) {
				throw new miException(sprintf('Unknown field "%s".', $fieldName));
			}
			
			$form->validateField($fieldName);
			$errors = $handler->getErrors();
			
			if (!empty($errors)) {
				$this->addWarnings($errors);
				return false;// We don't want to modify the cart by setting invalid data.
			}
			
			$cart = scEditOrderCart::getCart();
			$cart->setCustomerInfoField($fieldName, $fieldValue);
			
			if ($fieldName == 'PaymentModuleID') {
				$cart->applyPriceModifications();
				$cart->notifyObservers();
			}
			
			return true;
		}
	}
	
	class scEditOrderAddCartItemAction extends scEditOrderAjaxAction {
		
		public function modifySessionData()
		{
			$productId = (int)miGetParam('ProductID');
			
			// Check for user image options and add warnings if such exist.
			$options = scProductService::getProductsOptions(array($productId));
			if (isset($options[$productId])) {
				$options = array_merge($options[$productId]['LocalOptions'], $options[$productId]['ProductOptions']);
				foreach ($options as $option) {
					if ($option['OptionGroupType'] == scProductOptions::GROUP_TYPE_USER_IMAGE) {
						$this->addWarnings(array(
							_t('"%s" has an option of type "user image" which is not supported by "Edit Order".')
						));
					}
				}
			}
			
			$options = scEditOrderCartItemService::singleton()->getOptionsParam('Options');
			
			// Create the cart item object
			$item = scEditOrderCartItemService::singleton()->createCartItemProduct($productId, (int)miGetParam('Quantity'), $options);
			
			// Add the item in the cart
			scEditOrderCart::getCart()->addItem($item);
			return true;
		}
	}
	
	class scEditOrderRemoveCartItemAction extends scEditOrderAjaxAction {
		
		protected function modifySessionData()
		{
			scEditOrderCart::getCart()->removeItem(miGetParam('CartItemCode'));
			return true;
		}
	}
	
	class scEditOrderValidateAjaxAction extends scEditOrderAjaxAction {
		
		protected function modifySessionData()
		{
			$alerts = scEditOrderCart::getCart()->validate();
			$this->addAlerts($alerts);
			if (count($alerts) > 0) {
				return false;
			}
			return true;
		}
	}
?>