<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Override the original cart.
	 */
	class scEditOrderCart extends scCart implements IObservable {
		
		const WARNING_NO_ESCAPE = true;
		
		/**
		 * @var scEditOrderCart
		 */
		protected static $_currentCart = null;
		
		/**
		 * @var int
		 */
		protected static $_orderId = null;
		
		/**
		 * @var array
		 */
		protected static $_initialWarnings = array();
		
		/**
		 * @var array
		 */
		protected $_observers = array();
		
		protected $_customerInfoSubject = null;
		
		protected $_billingAddressObject = null;
		
		protected $_shippingAddressObject = null;
		
		protected $_mapping = array(
			'ShippingModuleAndMethod' => 'ShippingModuleAndMethod',
			
			'CustomerID' => 'CustomerID',
			'CustomerEmail' => 'OrderCustomerEmail',
			'CustomerShipToBillingAddress' => 'OrderShipToBillingAddress',
			
			'CustomerBillingFirstName' => 'OrderBillingFirstName',
			'CustomerBillingLastName' => 'OrderBillingLastName',
			'CustomerBillingCompany' => 'OrderBillingCompany',
			'CustomerBillingPhone' => 'OrderBillingPhone',
			'CustomerBillingFax' => 'OrderBillingFax',
			'CustomerBillingAddress1' => 'OrderBillingAddress1',
			'CustomerBillingAddress2' => 'OrderBillingAddress2',
			'CustomerBillingCountry' => 'OrderBillingCountry',
			'CustomerBillingState' => 'OrderBillingState',
			'CustomerBillingCity' => 'OrderBillingCity',
			'CustomerBillingCityID' => 'OrderBillingCityID',
			'CustomerBillingZip' => 'OrderBillingZip',
			'CustomerBillingVatNumber' => 'OrderBillingVatNumber',
			
			'CustomerShippingFirstName' => 'OrderShippingFirstName',
			'CustomerShippingLastName' => 'OrderShippingLastName',
			'CustomerShippingCompany' => 'OrderShippingCompany',
			'CustomerShippingPhone' => 'OrderShippingPhone',
			'CustomerShippingFax' => 'OrderShippingFax',
			'CustomerShippingAddress1' => 'OrderShippingAddress1',
			'CustomerShippingAddress2' => 'OrderShippingAddress2',
			'CustomerShippingCountry' => 'OrderShippingCountry',
			'CustomerShippingState' => 'OrderShippingState',
			'CustomerShippingCity' => 'OrderShippingCity',
			'CustomerShippingCityID' => 'OrderShippingCityID',
			'CustomerShippingZip' => 'OrderShippingZip',
			'CustomerShippingVatNumber' => 'OrderShippingVatNumber',
		);
		
		/**
		 * Specifies which is the order that is going to be used as cart.
		 * @param int $orderId
		 */
		public static function setCurrentOrderId($orderId)
		{
			self::$_orderId = intval($orderId);
		}
		
		/**
		 * @return int
		 */
		public static function getCurrentOrderId()
		{
			return self::$_orderId;
		}
		
		/**
		 * @param int $orderId
		 * @return scEditOrderCart
		 */
		public static function getCart()
		{
			if (is_null(self::$_currentCart)) {
				if (is_null(self::$_orderId)) {
					throw new miException(_i('There is no order specified.'));
				}
				
				scCustomerAuth::setCustomerAuth(new scEditOrderCustomerAuth());
				
				if (isset($_SESSION['OrderCarts']) && isset($_SESSION['OrderCarts'][self::$_orderId])) {
					self::$_currentCart = unserialize($_SESSION['OrderCarts'][self::$_orderId]);
				} else {
					// Initialize the cart so if this method is called from inside the
					// getOrderCart() method the object will be returned (although incomplete).
					self::$_currentCart = new scEditOrderCart();
					self::getOrderCart();
				}
			}
			
			return self::$_currentCart;
		}
		
		public static function discardCurrentCart()
		{
			self::$_currentCart = null;
			unset($_SESSION['OrderCarts'][self::$_orderId]);
		}
		
		public static function getInitialWarnings()
		{
			return self::$_initialWarnings;
		}
		
		public function __construct()
		{
			$this->_orderTotal = new scEditOrderOrderTotal($this);
			$this->initCustomerInfo();
		}
		
		public function __destruct()
		{
			$_SESSION['OrderCarts'][self::$_orderId] = serialize($this);
		}
		
		public function applyCoupon($couponCode)
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.ordertotal.coupon');
			$couponsModule = scModulesFactory::getImplementation($moduleId, 'IOrderTotalModule');
			if (!$couponsModule->isActive()) {
				throw new miException(_t('Coupon module is not active.'));
			}
			
			// Returns the coupon info. If error occurs throws exception
			$coupon = $this->getCouponInfo($couponCode);
			
			if (!scCouponsContainer::singleton()->isCouponValid($coupon, $this))
				self::addInitialWarning(sprintf(_t('Not all conditions for coupon "%s" are met.'), $couponCode));
			
			$this->_coupon = $coupon;
		}
		
		public function addItem(scCartItem $cartItem)
		{
			parent::addItem($cartItem);
			$this->notifyObservers();
		}
		
		public function removeItem($cartItemHash)
		{
			// Do not call parent::removeItem() as it may remove the payment and shipping selections.
			if (array_key_exists($cartItemHash, $this->_cartItems))
				unset($this->_cartItems[$cartItemHash]);
			$this->notifyObservers();
		}
		
		public function removeAllItems()
		{
			$this->_cartItems = array();
			$this->notifyObservers();
		}
		
		/**
		 * @return scEditOrderCustomerAuth
		 */
		protected function getCustomerAuth()
		{
			return scEditOrderCustomerAuth::singleton();
		}
		
		/**
		 * @return bool
		 */
		protected function hasCustomerPoints(scCustomerDomainObject $customer, $requiredPoints)
		{
			$inOrder = scEditOrderOrderTotalPoints::getBonusPointsSpentInOrder($customer, scEditOrderCart::getCurrentOrderId());
			$inCustomer = $customer->get('CustomerBonusPoints');
			$toSpend = $inCustomer + $inOrder;
			
			if ($toSpend < $requiredPoints) {
				return false;
			}
			return true;
		}	
		
		public function attachObserver(IObserver $observer)
		{
			$this->_observers[] = $observer;
		}
		
		public function detachAllObservers()
		{
			$this->_observers = array();
		}
		
		public function notifyObservers()
		{
			foreach ($this->_observers as $observer) {
				$observer->notify($this);
			}
		}
		
		/**
		 * @return array
		 */
		public function getCartToOrderMapping()
		{
			return $this->_mapping;
		}
		
		public function setCustomerInfoField($fieldName, $fieldValue)
		{
			// First we check if $fieldName is a _mapping field.
			// If so, we need to set the value of the mapped field.
			// Otherwise we'll just set it as is.
			
			$mappedKey = array_search($fieldName, $this->_mapping);
			if ($mappedKey === false) {
				$mappedKey = $fieldName;
			}
			
			$this->_customerInfo[$mappedKey] = $fieldValue;
			$this->getCustomerInfoSubject()->set($mappedKey, $fieldValue);
		}
		
		/**
		 * @param array $customerInfo
		 */
		public function setCustomerInfo($customerInfo)
		{
			foreach ($customerInfo as $field => $value) {
				$this->setCustomerInfoField($field, $value);
			}
		}
		
		/**
		 * @return scEditOrderBillingAddress
		 */
		public function getBillingAddressObject()
		{
			if (is_null($this->_billingAddressObject)) {
				$this->_billingAddressObject = new scEditOrderBillingAddress();
				$this->attachObserver($this->_billingAddressObject);
			}
			return $this->_billingAddressObject;
		}
		
		/**
		 * @return scEditOrderShippingAddress
		 */
		public function getShippingAddressObject()
		{
			if (is_null($this->_shippingAddressObject)) {
				$this->_shippingAddressObject = new scEditOrderShippingAddress();
				$this->attachObserver($this->_shippingAddressObject);
			}
			return $this->_shippingAddressObject;
		}
		
		/**
		 * @return scEditOrderCustomerInfo
		 */
		public function getCustomerInfoSubject()
		{
			if (is_null($this->_customerInfoSubject)) {
				$this->_customerInfoSubject = new scEditOrderCustomerInfo();
			}
			return $this->_customerInfoSubject;
		}
		
		/**
		 * @return array
		 */
		public function validate()
		{
			// Start a transaction to allow returning inventory.
			// Roll it back once validation is complete.
			miStaticDBUtil::startTransaction();
			$order = new scEditOrderDomainObject(self::$_orderId);
			$order->returnInventory();
			
			$cartItems = scEditOrderCart::getCart()->getItems();
			
			$errors = array();
			foreach ($cartItems as $cartItem) {
				// In current implementation each IProductListingItem is an IStockable.
				$stockable = $cartItem->getProductListingItem(true);
				if ($stockable->isOutOfStock($cartItem->getQuantity())) {
					$errors[] = sprintf(_t('There are only %d "%s" left.'), $stockable->getQuantityInStock(), $stockable->getName());
				}
			}
			
			try {
				scEditOrderCart::getCart()->checkMinimumPoints();
			} catch (miException $e) {
				$errors[] = $e->getMessage();
			}
			
			miStaticDBUtil::rollback();
			return $errors;
		}
		
		protected static function addInitialWarning($warning, $dontEscape = false)
		{
			if ($dontEscape === self::WARNING_NO_ESCAPE) {
				self::$_initialWarnings[] = $warning;
			} else {
				self::$_initialWarnings[] = miI18N::htmlEscape($warning);
			}
		}
		
		/**
		 * Builds the cart from the currently set order.
		 * @return scEditOrderCart
		 */
		protected static function getOrderCart()
		{
			$order = new scEditOrderDomainObject(self::$_orderId);
			
			scOrdersCache::fullLoadOrderData(self::$_orderId);
			
			$orderCurrencyCode = self::getOrderCurrencyCode($order);
			
			$cart = self::getCart();
			$cart->setCurrentCurrency(new scCurrency($orderCurrencyCode));
			$cart->setCustomerInfo($order->getRow());
			
			$warnings = $order->getWarnings();
			foreach ($warnings as $warning) {
				self::addInitialWarning($warning, self::WARNING_NO_ESCAPE);
			}
			
			$orderItems = $order->getOrderItems();
			// Check if the order has any items.
			if (empty($orderItems)) {
				// If it is an empty order (no items), no other processing is required. 
				return $cart;
			}
			
			// Create cart items and add them to the cart
			foreach ($orderItems as $orderItemObject) {
				if (!is_null($orderItemObject->get('ParentOrderItemID'))) {
					continue;
				}
				try {
					$cartItem = scEditOrderCartItemService::singleton()->createCartItemFromOrderItem($orderItemObject);
					if (is_null($cartItem)) {
						continue;
					}
					$cart->addItem($cartItem);
				} catch (miException $e) {
					self::addInitialWarning($e->getMessage());
				}
			}
			$serviceWarnings = scEditOrderCartItemService::getInitialWarnings();
			foreach ($serviceWarnings as $warning) {
				// The scEditOrderCartItemService should return its warnings escaped.
				self::addInitialWarning($warning, self::WARNING_NO_ESCAPE);
			}
			
			// Check if a coupon was applied to this order and try to reapply it.
			// Note: This will only work if the coupon is currently applicable.
			$orderCoupon = self::getOrderCouponCode($order);
			if (!empty($orderCoupon)) {
				$cart->applyCoupon($orderCoupon);
			}
			
			// Check the order total lines for changes and add initialWarnings if needed.
			self::checkInitialOrderTotalLines($order, $cart);
			
			return $cart;
		}
		
		/**
		 * @return string
		 */
		protected static function getOrderCurrencyCode(scOrder $order)
		{
			$code = $order->get('OrderCurrencyCode');
			
			if (empty($code)) {
				self::addInitialWarning(_t('No currency code has been saved in the order.'));
				return self::getCurrentCurrencyCodeForOrder();
			}
			
			try {
				$currency = new scCurrency($code);
			} catch (miDBException $exception) {
				if ($exception->getCode() != miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					// We don't want to handle exceptions other than "RECORD NOT FOUND".
					throw $exception;
				}
				
				self::addInitialWarning(sprintf(_t('The selected currency "%s" could not be found.'), $code));
				return self::getCurrentCurrencyCodeForOrder();
			}
			
			$activeCurrencies = scCurrencyUtil::getActiveCurrencies();
			$currencyId = $currency->getCurrencyId();
			if (!isset($activeCurrencies[$currencyId])) {
				self::addInitialWarning(sprintf(_t('The selected currency "%s" is currently inactive.'), $code));
				return self::getCurrentCurrencyCodeForOrder();
			}
			
			return $code;
		}
		
		/**
		 * @return string
		 */
		protected static function getCurrentCurrencyCodeForOrder()
		{
			$currentCurrency = scCurrencyUtil::getCurrentCurrency();
			self::addInitialWarning(sprintf(_t('The current currency "%s" will be used.'), $currentCurrency->getCurrencyCode()));
			return $currentCurrency->getCurrencyCode();
		}
		
		/**
		 * @return string
		 */
		protected static function getOrderCouponCode(scOrder $order)
		{
			$code = $order->get('OrderCouponCode');
			if (empty($code)) {
				return '';
			}
			
			// Find the coupon
			$sql = 'SELECT * FROM Coupons WHERE CouponCode = ?';
			$couponData = miStaticDBUtil::execSelect($sql, array($code));
			
			if (count($couponData) > 1) {
				self::addInitialWarning(sprintf(_t('Multiple coupons found with code "%s". None will be applied.'), $code));
				return '';
			}
			
			if (count($couponData) == 0) {
				self::addInitialWarning(sprintf(_t('Coupon "%s" not found.'), $code));
				return '';
			}
			
			// Check if the coupon is applicable.
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.ordertotal.coupon');
			$couponsModule = scModulesFactory::getImplementation($moduleId, 'IOrderTotalModule');
			if (!$couponsModule->isActive()) {
				$warning = _t('Coupon module is not active.');
				$warning .= ' ' . sprintf(_t('Coupon "%s" will not be applied.'), $code);
				self::addInitialWarning($warning);
				return '';
			}
			
			return $code;
		}
		
		protected static function checkInitialOrderTotalLines(scEditOrderDomainObject $order, scEditOrderCart $cart)
		{
			// Get the lines from the order.
			// More than one line of a type is possible so sum them by type.
			$lines = $order->getOrderTotalLines();
			$orderTotalLines = array();
			foreach ($lines as $line) {
				$amount = scMultiCurrencyAmount::factory();
				$amount->set($order->getBaseCurrency(), $line->get('OrderTotalLineAmount'));
				$amount->set($order->getCustomerCurrency(), $line->get('OrderTotalLineCustomerCurrencyAmount'));
				
				if (!isset($orderTotalLines[$line->get('OrderTotalLineType')])) {
					$orderTotalLines[$line->get('OrderTotalLineType')] = $amount;
					continue;
				}
				
				$orderTotalLines[$line->get('OrderTotalLineType')] = scMultiCurrencyAmountMath::add($orderTotalLines[$line->get('OrderTotalLineType')], $amount);
			}
			
			// Get the lines from the cart that was just generated.
			$lines = $cart->getOrderTotal()->getOrderTotalLines();
			$cartTotalLines = array();
			foreach ($lines as $line) {
				if (!isset($cartTotalLines[$line->getType()])) {
					$cartTotalLines[$line->getType()] = $line->getAmount();
					continue;
				}
				$cartTotalLines[$line->getType()] = scMultiCurrencyAmountMath::add($cartTotalLines[$line->getType()], $line->getAmount());
			}
			
			// Get possible warnings of the OrderTotal and assign them
			$orderTotalWarnings = $cart->getOrderTotal()->getInitialWarnings();
			foreach ($orderTotalWarnings as $warning) {
				self::addInitialWarning($warning, self::WARNING_NO_ESCAPE);
			}
			
			$lineTypeNames = array(
				IOrderTotalModule::ORDER_TOTAL_TYPE_SUBTOTAL => _t('Subtotal'),
				IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING => _t('Shipping'),
				IOrderTotalModule::ORDER_TOTAL_TYPE_SURCHARGE => _t('Surcharge'),
				IOrderTotalModule::ORDER_TOTAL_TYPE_DISCOUNT => _t('Discount'),
				IOrderTotalModule::ORDER_TOTAL_TYPE_COUPON => _t('Coupon'),
				IOrderTotalModule::ORDER_TOTAL_TYPE_TAX => _t('Tax'),
				IOrderTotalModule::ORDER_TOTAL_TYPE_OTHER => _t('Other'),
				IOrderTotalModule::ORDER_TOTAL_TYPE_POINTS => _t('Points'),
			);
			
			// By type find the ones that existed in the order but do not exist in the cart.
			$deleted = array_diff_key($orderTotalLines, $cartTotalLines);
			if (!empty($deleted)) {
				$warnings = array();
				foreach ($deleted as $type => $lineAmount) {
					$warnings[] = $lineTypeNames[$type] . ': ' . scPrice::factory($lineAmount)->getDisplayPrice();
				}
				$warnings = miI18N::htmlEscape(_t('Some total lines from the original order do not exist now:')) .
					scEditOrderUICart::getImplodedListHtml($warnings);
				self::addInitialWarning($warnings, self::WARNING_NO_ESCAPE);
			}
			// Now find the ones that exist in the cart but did not exist in the order.
			$added = array_diff_key($cartTotalLines, $orderTotalLines);
			if (!empty($added)) {
				$warnings = array();
				foreach ($added as $type => $lineAmount) {
					$warnings[] = $lineTypeNames[$type] . ': ' . scPrice::factory($lineAmount)->getDisplayPrice();
				}
				$warnings = miI18N::htmlEscape(_t('Some total lines did not exist in the original order:')) .
					scEditOrderUICart::getImplodedListHtml($warnings);
				self::addInitialWarning($warnings, self::WARNING_NO_ESCAPE);
			}
			// Remove the ones found above so no more warnings about them are made.
			$orderTotalLines = array_intersect_key($orderTotalLines, $cartTotalLines);
			$cartTotalLines = array_intersect_key($cartTotalLines, $orderTotalLines);
			
			// Check if any of the amounts have changed.
			foreach ($orderTotalLines as $type => $orderLine) {
				$cartLine = $cartTotalLines[$type];
				if ($orderLine->get() == $cartLine->get()) {
					continue;
				}
				self::addInitialWarning(sprintf(_t('The amount of the "%s" total line has changed from %s to %s.'),
					$lineTypeNames[$type], scPrice::factory($orderLine)->getDisplayPrice(), scPrice::factory($cartLine)->getDisplayPrice()));
			}
		}
	}
?>