<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scEditOrderCartItemProduct extends scCartItemProduct {
		
		protected $_order;
		
		protected $_warnings = array();
		
		public function __construct(scEditOrderCartItemInfo $cartItemInfo)
		{
			// Initialize the prices in both base and customer currencies.
			$listingItemPrice = $cartItemInfo->getListingItem()->getPrice()->getAmount();
			$listingItemPrice->get(scCurrencyUtil::getBaseCurrency());
			$listingItemPrice->get($cartItemInfo->getOrder()->getCustomerCurrency());
			
			$this->_order = $cartItemInfo->getOrder();
			parent::__construct($cartItemInfo);
		}
		
		/**
		 * Use setRow(_productInfo) on scProductSL instead of reading it
		 * as the quantity has been modified by scEditOrderCartItemService
		 */
		public function setQuantity($quantity)
		{
			if (!($this->_order instanceof scOrder)) {
				throw new miException(_i('scOrder needed.'));
			}
			
			$product = new scProductSL();
			$product->setRow($this->_productInfo);
			if ($product->isOutOfStock($quantity)) {
				$this->addWarning(sprintf(_t('There are only %d "%s" left.'), $product->getQuantityInStock(), $this->getName()));
			}
			
			$this->checkPointsForNewQuantity($quantity);
			
			scCartItem::setQuantity($quantity);
			
			$this->applyPriceModification();
		}
		
		/**
		 * @return array
		 */
		public function getWarnings()
		{
			return $this->_warnings;
		}
		
		protected function addWarning($warning, $dontEscape = false)
		{
			$this->_warnings[] = $warning;
		}
		
		public function getProductListingItem($forceRead = false)
		{
			$item = parent::getProductListingItem();
			if (!$forceRead) {
				return $item;
			}
			$toRead = clone $item;
			$toRead->read($toRead->getPK());
			return $toRead;
		}
		
		/**
		 * @return scProductPriceModification
		 */
		protected function getCartItemPriceModifications()
		{
			return scEditOrderProductPrice::getCartItemPriceModifications($this);
		}
		
		/**
		 * @return scEditOrderCart
		 */
		protected function getCart()
		{
			return scEditOrderCart::getCart();
		}
	}
?>