<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 * @see scCartItemProductBundle
	 */
	
	class scEditOrderCartItemProductBundle extends scEditOrderCartItemProduct implements ICartItemProductBundle {
		
		/**
		 * @var scEditOrderCartItemInfo
		 */
		protected $_cartItemInfo;
		
		protected $_bundledItems = array();
		
		public function __construct(scEditOrderCartItemInfo $cartItemInfo)
		{
			$bundleProduct = $cartItemInfo->getProduct();
			if (!$bundleProduct->get('ProductIsBundle')) {
				throw new miException(_i('Product is not a bundle.'));
			}
			
			$this->_cartItemInfo = $cartItemInfo;
			
			$this->createBundledItems();
			
			parent::__construct($cartItemInfo);
		}
		
		public function getBundledItems()
		{
			return $this->_bundledItems;
		}
		
		protected function createBundledItems()
		{
			if (is_null($this->_cartItemInfo->getOrderItem())) {
				throw new miException(_i('scOrderItem needed.'));
			}
			$bundledOrderItems = $this->getBundledOrderItems();
			foreach ($bundledOrderItems as $orderItem) {
				$subItem = scEditOrderCartItemService::singleton()->createCartItemFromOrderItem($orderItem);
				if (is_null($subItem)) {
					continue;
				}
				$this->_bundledItems[$orderItem->get('ProductID')] = $subItem;
			}
		}
		
		/**
		 * @return scOrderItem[]
		 */
		protected function getBundledOrderItems()
		{
			$currentOrderItem = $this->_cartItemInfo->getOrderItem();
			$order = $this->_cartItemInfo->getOrder();
			
			$orderItems = $order->getOrderItems();
			$bundledItems = array();
			foreach ($orderItems as $orderItem) {
				if ($orderItem->get('ParentOrderItemID') != $currentOrderItem->get('OrderItemID')) {
					continue;
				}
				$bundledItems[] = $orderItem;
			}
			return $bundledItems;
		}
		
		/**
		 * @see scCartItemProductBundle::setQuantity()
		 */
		public function setQuantity($quantity)
		{
			$previousQuantity = $this->getQuantity();
			if ($previousQuantity == 0) {
				$previousQuantity = $quantity;
			}
			
			parent::setQuantity($quantity);
			
			foreach ($this->_bundledItems as $item) {
				$qtyRate = $item->getQuantity() / $previousQuantity;
				$item->setQuantity($quantity * $qtyRate);
			}
		}
		
		/**
		 * @see scCartItemProductBundle::serialize()
		 */
		protected function serialize()
		{
			$subsHashes = array();
			foreach ($this->_bundledItems as $subItem) {
				$subsHashes[] = $subItem->getHash();
			}
			
			return parent::serialize() . '-' . implode('-', $subsHashes);
		}
	}
?>