<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scEditOrderCartItemProductOptionsRetriever extends scCartItemProductOptionsRetriever {
		
		/**
		 * Uses Edit Order's implementations of the local and global options retrievers
		 * @return scCartItemOptionsInfo
		 */
		public function getCartItemOptions()
		{
			if (is_null($this->_result)) {
				$this->_result = new scCartItemOptionsInfo();
				
				$globalOptions = new scEditOrderCartItemOptionsRetriever($this->_info);
				$globalOptions = $globalOptions->getCartItemOptions();
				$this->_result->setOptions($globalOptions);
				
				$localOptions = new scEditOrderCartItemLocalOptionsRetriever($this->_info);
				$localOptions = $localOptions->getCartItemOptions();
				$this->_result->setLocalOptions($localOptions);
			}
			
			return $this->_result;
		}
	}
	
	class scEditOrderCartItemOptionsRetriever extends scCartItemOptionsRetriever {
		
		public function handleImageUploadOptions()
		{
			if (empty($this->_options)) {
				return;
			}
			$this->_options = self::handleImageOptions('ProductOptions', $this->_options);
		}
		
		protected function handleUserTextOptions()
		{
			if (empty($this->_options)) {
				return;
			}
			$this->_options = self::handleTextOptions('ProductOptions', $this->_options);
		}
		
		protected function getOptionsInfo()
		{
			$options = parent::getOptionsInfo();
			
			foreach ($options as $optionId => $optionData) {
				$options[$optionId]['OptionUploadedFileName'] = self::getSessionTempUserFileName('ProductOptions', $optionData['OptionGroupID'], $optionId);
				$options[$optionId]['OptionUserText'] = self::getSessionTempUserText('ProductOptions', $optionData['OptionGroupID'], $optionId);
			}
			
			return $options;
		}
		
		/**
		 * Remove strictness from the method. It has already been uploaded by the user.
		 */
		public static function handleImageOptions($optionType, $options)
		{
			foreach ($options as $optionGroupId => $optionGroup) {
				foreach ($optionGroup as $optionId => $file) {
					if (!($file instanceof scFile)) {
						continue;
					}
					// Add the option to the list so its ID can be validated.
					$options[$optionGroupId][] = $optionId;
					
					// The parent class uses scProductOptions::setSessionTempUserFileName() to store the user image data
					// while the order is not completed. Replicate that behaiour.
					$currentOrderId = scEditOrderCart::getCurrentOrderId();
					$_SESSION['EditOrderCartItemOptions'][$currentOrderId]['UserImages'][$optionType][$optionGroupId][$optionId] = $file->getTempName();
				}
			}
			return $options;
		}
		
		public static function handleTextOptions($optionType, $options)
		{
			foreach ($options as $optionGroupId => $optionTexts) {
				foreach ($optionTexts as $optionId => $optionText) {
					// validate optionid / option group and get required value
					if ($optionType == 'ProductOptions') {
						$optionSql = 'SELECT OptionRequired AS Required FROM Options ' .
							'INNER JOIN OptionGroups USING(OptionGroupID) ' .
							'WHERE OptionGroupType = ? AND OptionID = ? AND OptionGroups.OptionGroupID = ?';
					} else {
						$optionSql = 'SELECT LocalOptionRequired AS Required FROM LocalOptions ' .
							'INNER JOIN LocalOptionGroups USING(LocalOptionGroupID) ' .
							'WHERE LocalOptionGroupType = ? AND LocalOptionID = ? AND LocalOptionGroups.LocalOptionGroupID = ?';
					}
					
					$option = miStaticDBUtil::execSelect($optionSql, array(scProductOptions::GROUP_TYPE_USER_TEXT, $optionId, $optionGroupId));
					if (count($option) == 0) {
						continue;
					}
					
					$options[$optionGroupId][$optionId] = $optionId;
					
					$currentOrderId = scEditOrderCart::getCurrentOrderId();
					$_SESSION['EditOrderCartItemOptions'][$currentOrderId]['UserText'][$optionType][$optionGroupId][$optionId] = $optionText;
				}
			}
			return $options;
		}
		
		/**
		 * Replicate the behaviour of scProductOptions::getSessionTempUserText()
		 */
		public static function getSessionTempUserText($optionType, $optionGroupId, $optionId)
		{
			$currentOrderId = scEditOrderCart::getCurrentOrderId();
			if (!isset($_SESSION['EditOrderCartItemOptions'][$currentOrderId]['UserText'][$optionType][$optionGroupId][$optionId]))
				return '';
			
			return $_SESSION['EditOrderCartItemOptions'][$currentOrderId]['UserText'][$optionType][$optionGroupId][$optionId];
		}
		
		/**
		 * Replicate the behaviour of scProductOptions::getSessionTempUserFileName()
		 */
		public static function getSessionTempUserFileName($optionType, $optionGroupId, $optionId)
		{
			$currentOrderId = scEditOrderCart::getCurrentOrderId();
			if (!isset($_SESSION['EditOrderCartItemOptions'][$currentOrderId]['UserImages'][$optionType][$optionGroupId][$optionId]))
				return '';
			if (!is_file($_SESSION['EditOrderCartItemOptions'][$currentOrderId]['UserImages'][$optionType][$optionGroupId][$optionId]))
				return '';
			return $_SESSION['EditOrderCartItemOptions'][$currentOrderId]['UserImages'][$optionType][$optionGroupId][$optionId];
		}
	}
	
	class scEditOrderCartItemLocalOptionsRetriever extends scCartItemLocalOptionsRetriever {
		
		public function handleImageUploadOptions()
		{
			if (empty($this->_options)) {
				return;
			}
			$this->_options = scEditOrderCartItemOptionsRetriever::handleImageOptions('LocalOptions', $this->_options);
		}
		
		protected function handleUserTextOptions()
		{
			if (empty($this->_options)) {
				return;
			}
			$this->_options = scEditOrderCartItemOptionsRetriever::handleTextOptions('LocalOptions', $this->_options);
		}
		
		protected function getOptionsInfo()
		{
			$options = parent::getOptionsInfo();
			
			foreach ($options as $optionId => $optionData) {
				$options[$optionId]['OptionUploadedFileName'] = scEditOrderCartItemOptionsRetriever::getSessionTempUserFileName('LocalOptions', $optionData['LocalOptionGroupID'], $optionId);
				$options[$optionId]['OptionUserText'] = scEditOrderCartItemOptionsRetriever::getSessionTempUserText('LocalOptions', $optionData['LocalOptionGroupID'], $optionId);
			}
			
			return $options;
		}
	}
?>