<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 * @see scCartItemProductVariant
	 */
	
	class scEditOrderCartItemProductVariant extends scEditOrderCartItemProduct implements ICartItemProductVariant {
		
		public function __construct(scEditOrderCartItemInfo $cartItemInfo)
		{
			$this->_productVariant = $cartItemInfo->getProductVariant();
			$this->_order = $cartItemInfo->getOrder();
			
			parent::__construct($cartItemInfo);
			
			$this->setWeight($this->_productVariant->getWeight());
		}
		
		/**
		 * @return scProductVariant
		 */
		public function getProductVariant()
		{
			return $this->_productVariant;
		}
		
		/**
		 * @return scProductVariant
		 */
		public function getProductListingItem($forceRead = false)
		{
			$item = $this->getProductVariant();
			if (!$forceRead) {
				return $item;
			}
			$toRead = clone $item;
			$toRead->read($toRead->getPK());
			return $toRead;
		}
		
		public function setQuantity($quantity)
		{
			if (!($this->_order instanceof scOrder)) {
				throw new miException(_i('scOrder needed.'));
			}
			
			if ($this->getProductVariant()->isOutOfStock($quantity)) {
				$this->addWarning(sprintf(_t('There are only %d "%s" left.'), $this->_productVariant->getQuantityInStock(), $this->getFullName()));
			}
			
			$this->checkPointsForNewQuantity($quantity);
			
			// Skip CartItemProduct and use the CartItem's setQuantity() instead.
			scCartItem::setQuantity($quantity);
			
			$this->applyPriceModification();
		}
		
		public function getImageInfo()
		{
			return $this->_productVariant->getImageInfo();
		}
		
		/**
		 * @return string
		 */
		public function getProductCode()
		{
			return $this->_productVariant->get('ProductVariantCode');
		}
		
		protected function getFullName()
		{
			$optionsByType = $this->_productVariant->getSelectedOptionsDetails();
			$nameOptions = array();
			foreach ($optionsByType as $type => $options) {
				foreach ($options as $option) {
					$nameOptions[] = ($type == scProductVariantOption::TYPE_LOCAL ? $option['LocalOptionName'] : $option['OptionName']);
				}
			}
			
			return $this->getName() . ' (' . implode(', ', $nameOptions) . ')';
		}
	}
?>