<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scEditOrderOrderTotal extends scOrderTotal implements IObserver, IObservable {
		
		protected $_observers = array();
		
		protected $_initialWarnings = array();
		
		public function notify(IObservable $subject)
		{
			$this->computeOrderTotalLines();
		}
		
		public function attachObserver(IObserver $observer)
		{
			$this->_observers[] = $observer;
		}
		
		public function detachAllObservers()
		{
			$this->_observers = array();
		}
		
		public function notifyObservers()
		{
			foreach ($this->_observers as $observer) {
				$observer->notify($this);
			}
		}
		
		public function getInitialWarnings()
		{
			return $this->_initialWarnings;
		}
		
		protected function computeOrderTotalLines($type = false)
		{
			$oldHash = crc32(serialize($this->_orderTotalLines));
			parent::computeOrderTotalLines($type);
			$newHash = crc32(serialize($this->_orderTotalLines));
			
			if ($oldHash != $newHash) {
				$this->notifyObservers();
			}
		}
		
		protected function processOrderTotalModules($orderTotalModules)
		{
			foreach ($orderTotalModules as $orderTotalModule) {
				try {
					$result = $orderTotalModule->getOrderTotalLines($this->_cart);
					$this->_orderTotalLines = array_merge($this->_orderTotalLines, $result);
				} catch (Exception $exception) {
					// Log the error and go on with the rest of the lines.
					$warning = sprintf(_t('Order total module exception: "%s".'), $exception->getMessage());
					$this->addInitialWarning($warning);
				}
			}
		}
		
		/**
		 * Returns an array with the active order total modules
		 * Overrides order total modules that need special implementation for Edit Order.
		 */
		protected function getActiveOrderTotalModules($type = false)
		{
			$modules = scModulesFactory::getActiveOrderTotalModules($type);
			
			$editOrderModules = array();
			foreach ($modules as $moduleId => $module) {
				$editOrderModules[$moduleId] = $this->getEditOrderTotalModule($moduleId, $module);
			}
			
			return $editOrderModules;
		}
		
		protected function getEditOrderTotalModule($moduleId, IOrderTotalModule $orderTotalModule)
		{
			if ($orderTotalModule instanceof scOrderTotalPoints) {
				$editOrderTotalModule = new scEditOrderOrderTotalPoints($moduleId);
			} else if ($orderTotalModule instanceof scOrderTotalDiscount) {
				$editOrderTotalModule = new scEditOrderOrderTotalDiscount($moduleId);
			} else if ($orderTotalModule instanceof  scOrderTotalShipping) {
				$editOrderTotalModule = new scEditOrderOrderTotalShipping($moduleId);
			}
			
			if (isset($editOrderTotalModule)) {
				$editOrderTotalModule->setOrderTotalModule($orderTotalModule);
				return $editOrderTotalModule;
			}
			
			return $orderTotalModule;
		}
		
		protected function addInitialWarning($warning)
		{
			$this->_initialWarnings[] = miI18N::htmlEscape($warning);
		}
	}
?>