<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scEditOrderProductPrice extends scProductPrice {
		
		/**
		 * Initializes the price modules array, call it first in every public function in the class.
		 * The modules are ordered the way they are in the parent class.
		 * The Discount and Wholesale implementations are replaced with Edit Order's versions.
		 */
		public static function initPriceModules($forceRead = true)
		{
			if ((self::$_priceModificationModules !== null) && !$forceRead)
				return;
			
			try {
				$priceModificationModules = scModulesFactory::getImplementations('IProductPriceModifier');
				self::$_priceModificationModules = array(0 => null, 1 => null);
				foreach ($priceModificationModules as $module) {
					if ($module instanceof scProductPriceWholesalePrices) {
						$replacement = new scEditOrderWholesalePrices();
						self::$_priceModificationModules[0] = $replacement;
						continue;
					}
					if ($module instanceof scProductPriceDiscount) {
						$replacement = new scEditOrderProductPriceDiscount();
						$replacement->setProductPriceModifier($module);
						self::$_priceModificationModules[1] = $replacement;
						continue;
					}
					self::$_priceModificationModules[] = $module;
				}
			} catch (miException $exception) {
				scLog::singleton()->error($exception->getMessage(), scLog::CATEGORY_NONE);
				self::$_priceModificationModules = array();
			}
		}
	}
	
	// Load IProductPriceModifier implementations making sure the static variable contains the ones Edit Order needs.
	scEditOrderProductPrice::initPriceModules(true);
?>