<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scEditOrderUICart extends scUICart {
		
		public static function getImplodedListHtml($arrayToList)
		{
			if (empty($arrayToList)) {
				return '';
			}
			return '<ul><li>' . implode('</li><li>', $arrayToList) . '</li></ul>';
		}
		
		/**
		 * @param string $template
		 * @return string
		 */
		public function getOrderTotalLinesHtml($template)
		{
			$t = new scTemplateParser();
			$t->readTemplate($template);
			$t->addSectionInfo($this->getOrderTotalLinesTemplateSection());
			
			$grandTotal = scPrice::factory($this->getCart()->getOrderTotal()->getGrandTotalAmount());
			
			$t->assign('%%GRANDTOTAL%%', $grandTotal->getDisplayPrice());
			
			return $t->templateParse();
		}
		
		/**
		 * @param string $template
		 * @return string
		 */
		public function getCartItemsHtml()
		{
			$t = new scTemplateParser();
			$t->readTemplate($this->getTemplater()->get(scUICartTemplater::ITEMS));
			$t->addSectionInfo($this->getCartTemplateSection());
			return $t->templateParse();
		}
		
		public function getInitialWarningsHtml()
		{
			return self::getImplodedListHtml(scEditOrderCart::getInitialWarnings());
		}
		
		/**
		 * Only change the way the paths to image options are generated.
		 */
		protected function getOptionsTemplateSection(scCartItem $item)
		{
			$hasOptionsSection = new miTemplateParserSectionInfo('hasOptions', 0);
			
			if (!($item instanceof scCartItemProduct)) {
				return $hasOptionsSection;
			}
			
			$options = array_merge($item->getOptions(), $item->getLocalOptions());
			if (count($options) == 0)
				return $hasOptionsSection;
			
			$hasOptionsSection->setSectionInfo('hasOptions', 1);
			$price = $item->getPriceObj(false)->getAmount();
			
			// Get options info
			$optionsInfo = array();
			$hasPriceModificationSections = array();
			$hasImageOptionSubsections = array();
			$hasUserTextOptionSections = array();
			foreach ($options as $option) {
				$optionInfo = $option->getOptionInfo($price);
				$subsection = new miTemplateParserSectionInfo('hasPriceModification', 1);
				if (($optionInfo['OptionPriceModType'] == scProductOptions::PRODUCT_OPTION_TYPE_NONE) || ($optionInfo['OptionPriceModValue'] == 0)) {
					$subsection->setIterationsCount(0);
				}
				
				$path = str_replace(scPaths::getStoreRootDir(), '', $optionInfo['OptionUploadedFileName']);
				$optionInfo['OptionUploadedFilePathWeb'] = scPaths::getUrl($path);
				
				$hasImageOptionSubsection = new miTemplateParserSectionInfo('hasImageOption', 0);
				if ($optionInfo['OptionGroupType'] == scProductOptions::GROUP_TYPE_USER_IMAGE) {
					$hasImageOptionSubsection->setIterationsCount(1);
				}
				$hasImageOptionSubsections[] = $hasImageOptionSubsection;
				
				$hasUserTextOptionSection = new miTemplateParserSectionInfo('hasUserTextOption', 0);
				if ($optionInfo['OptionGroupType'] == scProductOptions::GROUP_TYPE_USER_TEXT) {
					$hasUserTextOptionSection->setIterationsCount(1);
				}
				$hasUserTextOptionSections[] = $hasUserTextOptionSection;
				
				$hasPriceModificationSections[] = $subsection;
				$optionsInfo[] = $optionInfo;
			}
			
			// Add the options section
			$optionsSection = new miTemplateParserSectionInfo('options', count($optionsInfo), miUIUtil::templetizeRows(miUIUtil::transposeRows($optionsInfo)));
			$optionsSection->addSubsectionsArray('hasPriceModification', $hasPriceModificationSections);
			$optionsSection->addSubsectionsArray('hasImageOption', $hasImageOptionSubsections);
			$optionsSection->addSubsectionsArray('hasUserTextOption', $hasUserTextOptionSections);
			$hasOptionsSection->addSubsection($optionsSection);
			
			return $hasOptionsSection;
		}
	}
?>