<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Needed for the implementation of the Observer design pattern.
	 * Observers may attach to the observable. When the observable's state changes
	 * it notifies the observer by calling its notify() method by giving a reference to itself.
	 */
	interface IObservable {
		
		/**
		 * Observers call this method to become "listeners" for any state changes of the observable.
		 * @param IObserver $observer
		 */
		public function attachObserver(IObserver $observer);
		
		/**
		 * Detaches all observers of the observable.
		 */
		public function detachAllObservers();
		
		/**
		 * Should call the notify() method of all attached observers.
		 */
		public function notifyObservers();
	}
?>