<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * News module
	 */
	class scAdminNews implements IMenu, IAdminModule {
		
		public function getMenus()
		{
			return array(scAdminMenus::MENU_TOP_CMS => array(
				array('name' => _t('News'), 'url' => 'module.php?ModuleName=com.summercart.news')
			));
		}
		
		public function initActionHandlers(miPage $page)
		{
			scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_NEWS);
			
			$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
			$page->getDispatcher()->setActionHandler('dmPrint', 'scPrintAction');
			$page->getDispatcher()->setActionHandler('dmExport', 'scExportCSVAction');
		}
		
		public function getView()
		{
			// By default sort by news date desc
			if (miGetParamDefault(miTableSorter::PARAM_SORTBY, '') == '') {
				miSetParam(miTableSorter::PARAM_SORTBY, 'NewsDate');
				miSetParam(miTableSorter::PARAM_SORTDIR, 'DESC');
			}
			
			
			$newsRs = new scSqlRecordsetML('News', 'NewsID');
			$table = new scNewsAdminTable($newsRs);
			$moduleParams = new miTableCustomParams($table);
			$moduleParams->setRecordLocatorParams(array('ModuleName' => 'com.summercart.news'));
			
			$view = new scAdminNewsView($newsRs);
			$view->setRecord(new scSqlRecordML('News', 'NewsID', array('NewsTitle', 'NewsContents'), array_keys(scMultiLanguage::getActiveLanguageCodes())));
			$view->setTable($table);
			
			return $view;
		}
	}
	
	class scAdminNewsView extends scView {
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'modules/news/admin/news_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'modules/news/admin/news_edit.tmpl',
			self::TEMPLATE_ID_CREATE => 'modules/news/admin/news_create.tmpl'
		);
		
		protected $_dataFields = array(
       		array(
				'field' => 'miBaseTextWidget',
				'data' => 'NewsID'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'NewsTitle',
				'validator' => 'scValidatorNoEmpty',
				'displayName' => 'Title'
        	),
       		array(
				'field' => 'scWebFormWidgetTextAreaML',
				'data' => 'NewsContents',
				'displayName' => 'Content'
        	),
        	array(
				'field' => 'scDateWidget',
				'data' => 'NewsDate',
				'displayName' => 'Date'
        	),
       		array(
				'field' => 'miCheckboxWidget',
				'data' => 'NewsIsActive',
				'displayName' => 'Active'
        	),
        	array(
				'field' => 'scCustomUrlsWidget',
				'data' => 'CustomUrls',
				'modelStrategy' => 'scViewMapperNewsCustomUrls',
				'properties' => array(
					'template' => 'admin/custom_urls/custom_url.tmpl',
					
					'table' => 'CustomUrls',
					'foreignKey' => 'CustomUrlEntityID',
					'foreignTablePrimaryKey' => 'CustomUrlID',
					
					'fields' => array(
			       		array(
							'field' => 'scWebFormWidgetTextML',
							'data' => 'CustomUrlText',
			       			'properties' => array('minLength' => 1),
			       			'validator' => 'scValidatorUniqueCustomUrl'
			        	),
			       		array(
							'field' => 'miCheckboxWidget',
							'data' => 'CustomUrlIsCurrent'
			        	),
			       		array(
							'field' => 'miBaseTextWidget',
							'data' => 'CustomUrlID'
			        	)
			        ),
			        'mlFields' => array('CustomUrlText')
				),
			)
		);
		
		protected $_domainObjectDisplayName = 'News';	// _t('News');
		
		public function __construct($recordset)
		{
			$this->_persistVars[] = 'ModuleName';
			
			parent::__construct($recordset);
		}
		
		public function initTable($table)
		{
			parent::initTable($table);
			
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
        	$form->addMainPageElements(array(
				'%%CKEDITOR_CONTENTCSS%%' => scSkins::singleton()->getCurrentSkin()->getCssFilesForWysiwygEditor(),
				'%%CKEDITOR_LANGUAGE%%' => scCMS::getCkEditorLanguage()
			));
		}
		
		public function preProcess($actionObj)
		{
			if ($this->getController()->getDispatcher()->getAction() == 'dmChangeIsActive')
				$actionObj->setFieldName('NewsIsActive');
		}
	}
	
	class scViewMapperNewsCustomUrls extends scViewMapperCustomUrls {
		protected $_entityCode = scFriendlyUrl::TYPE_NEWS;
		protected $_tableKeyName = 'NewsID';
	}
	
	class scNewsAdminTable extends scTable {
		public function addUiElements(&$rows)
		{
			foreach ($rows as &$row) {
				$row['HTML_NewsTitle'] = $row['NewsTitle'];
				$row['HTML_NewsContents'] = $row['NewsContents'];
				$row['NewsTitle'] = strip_tags($row['NewsTitle']);
				$row['NewsContents'] = strip_tags($row['NewsContents']);
				$row['NewsDate'] = scDateTime::getFormattedDate($row['NewsDate']);
			}
		}
	}
?>