<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * News module
	 */
	class scUserNews implements IUserModule {
		/**
		 * Initializes the action handlers for this module
		 * 
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			if (miHasParam('NewsID'))
				miSetParam('action', 'dmView');
			
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('', 'miListAction');
			$page->getDispatcher()->setActionHandler('dmView', 'scViewNewsAction');
			
			$page->setSeo(new scNewsSeo());
		}
		
		/**
		 * Returns the view that should be displayed
		 * 
		 * @return miDefaultView
		 */
		public function getView()
		{
			miSetParam(miTableSorter::PARAM_SORTBY, 'NewsDate');
			miSetParam(miTableSorter::PARAM_SORTDIR, 'DESC');
			
			$recordset = new miSqlRecordset('News');
			$recordset->addJoinCondition('LEFT', 'NewsML', 'USING(NewsID)');
			$recordset->setSelectFields(array('*', 'News.NewsID AS NewsID'));
			// The miListAction overwrites the order using sortBy and sortDir params
			//$recordset->setOrder('NewsDate', 'DESC');
			$recordset->addFilters(
				array(
					new miSqlFilterEqual('NewsIsActive', 1),
					new miSqlFilterEqual('LanguageID', scMultiLanguage::getCurrentLanguageId()),
					new miSqlFilterCustom('NewsDate', 'NewsDate <= CURDATE()')
				)
			);
			
			$view = new scUserNewsView($recordset);
			$view->setTable(new scNewsTable($recordset));
			
			return $view;
		}
	}
	
	/**
	 * The news view
	 */
	class scUserNewsView extends scView implements ICMSBlock {
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'modules/news/customer/news.tmpl',
			self::TEMPLATE_ID_VIEW => 'modules/news/customer/news_view.tmpl'
		);
		
		public function __construct($recordset)
		{
			parent::__construct($recordset);
			
			$this->_persistVars = array('ModuleName');
		}
			
		public function addRedirectToListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			$params = $this->getRedirectParams($message, $messageType);
			foreach ($this->_persistVars as $persistVar)
				$params .= '&' . $persistVar . '=' . miGetParamDefault($persistVar, '');
			
			$this->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, '?' . $params));
		}
		
		/**
		 * @param miTable $table
		 */
		public function initTable($table)
		{
			parent::initTable($table);
			
			$this->getTablePagerObj()->setDefaultRecordsPerPage(PHP_INT_MAX);
		}
	}
	
	/**
	 * View single news record action
	 */
	class scViewNewsAction extends miAction {
		public function doAction()
		{
			$data = miStaticDBUtil::execSelect(
				'SELECT * FROM News LEFT JOIN NewsML ON News.NewsID = NewsML.NewsID AND LanguageID = ? ' .
				'WHERE News.NewsID = ? AND NewsIsActive = 1 AND NewsDate <= CURDATE()',
				array(
					scMultiLanguage::getCurrentLanguageId(),
				 	(int) miGetParam('NewsID')
				)
			);
			
			if (count($data) == 0) {
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode(_t('News not found.'));
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
				return true;
			}
			
			$data = reset($data);
			
			$parser = new scTemplateParser();
			$parser->assignArray(miUIUtil::templetizeRows($data));
			
			$parser->readTemplate($this->_view->getTemplateFileName(scUserNewsView::TEMPLATE_ID_VIEW));
			$html = $parser->templateParse();
			
			// Set the page title
			$this->_view->getController()->setSeo(new scNewsSeo());
			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;
		}
	}
	
	/**
	 * The news module CMS box
	 */
	class scNewsBox implements ICMSBox {
		/**
		 * ICMSBox constructor
		 */
		public function __construct($moduleId, $cmsBoxId) {}
		
		/**
		 * Returns the CMS box contents
		 */
		public function getBoxContents() 
		{
			$recordset = new miSqlRecordset('News');
			$recordset->addJoinCondition('LEFT', 'NewsML', 'USING(NewsID)');
			$recordset->setSelectFields(array('*', 'News.NewsID AS NewsID'));
			$recordset->setOrder('NewsDate', 'DESC');
			$recordset->setRecordsLimit(0, 3);			// TODO: V1.1 Get it from the settings or eventually from a constant
			$recordset->addFilters(
				array(
					new miSqlFilterEqual('NewsIsActive', 1),
					new miSqlFilterEqual('LanguageID', scMultiLanguage::getCurrentLanguageId()),
					new miSqlFilterCustom('NewsDate', 'NewsDate <= CURDATE()')
				)
			);
			
			$table = new scNewsTable($recordset);
			
			if (scFriendlyUrl::hasFriendlyUrls()) {
				$channelLink = scPaths::getUrl() . 'rss/news/' . strtolower(scMultiLanguage::singleton()->getCurrentLanguageCode());
			} else {
				$channelLink = scPaths::getUrl() . 'module.php?ModuleName=com.summercart.rss&UILanguage=' .
					strtolower(scMultiLanguage::singleton()->getCurrentLanguageCode()) . '&FeedType=news';
			}
			
			$table->assign('%%NEWS_RSS_URL%%', $channelLink);
			return $table->parse('modules/news/customer/news_box.tmpl');
		}
	}
	
	class scNewsTable extends miTable {
		public function addUiElements(&$rows)
		{
			scFriendlyUrl::initFriendlyUrls($rows, scFriendlyUrl::ROW_TYPE_NEWS);
			
			foreach($rows as &$row) {
				$row['HTML_NewsTitle'] = $row['NewsTitle'];
				$row['HTML_NewsContents'] = $row['NewsContents'];
				$row['NewsTitle'] = strip_tags($row['NewsTitle']);
				$row['NewsContents'] = strip_tags($row['NewsContents']);
				$row['NewsDate'] = scDateTime::getFormattedDate($row['NewsDate']);
			}
		}
	}
	
	class scNewsSeo extends scDynamicPageSeo {
		
		public function __construct()
		{
			parent::__construct('module.php?ModuleName=com.summercart.news');
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			$this->_pageTitle = _t('News');
			
			if (miHasParam('NewsID')) {
				$seo = miStaticDBUtil::execSelect(
					'SELECT NewsTitle AS Title FROM NewsML ' .
					'WHERE NewsID = ? AND LanguageID = ?',
					array(intval(miGetParam('NewsID')), scMultiLanguage::getCurrentLanguageId())
				);
				// Remove any HTML tags from the Title
				foreach ($seo as &$row) {
					$row['Title'] = strip_tags($row['Title']);
				}
				
				$this->addMetaData($seo);
			}
		}
		
		public function isMultilanguage()
		{
			return true;
		}
		
		public function getUrlParams()
		{
			if (miHasParam('NewsID')) {
				return array(
					scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_NEWS,
					scSeoUrlTypes::PAGE_ID => intval(miGetParam('NewsID')),
				);
			}
			return array(
				scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_DYNAMIC_PAGE,
				scSeoUrlTypes::PAGE_ID => scFriendlyUrl::DYN_PAGE_NEWS,
			);
		}
	}
?>