<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scNewsletter extends scDomainObject {
		
		protected $_recordMlFields = array('NewsletterName', 'NewsletterSubject', 'NewsletterXhtmlText');
		protected $_activeLanguageCodes = array();
		
		public function __construct() 
		{
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			parent::__construct(new scSqlRecordML('NLNewsletters', 'NewsletterID', $this->_recordMlFields, $recordLanguages));
		}
		
		protected function onDelete($pkValue)
		{
			parent::onDelete($pkValue);
			
			$this->deactivateCmsBoxes($pkValue);
		}
		
		protected function deactivateCmsBoxes($newsletterIds)
		{
			// Find active boxes
			$recordset = new miSqlRecordset('CMSBoxes');
			$recordset->addFilter(new miSqlFilterEqual('CMSBoxType', scCmsBoxTypes::CMS_BOX_TYPE_NEWSLETTER));
			$recordset->addFilter(new miSqlFilterEqual('CMSBoxIsActive', 1));
			$boxes = $recordset->getAllRecords();
			
			foreach ($boxes as $cmsBox) {
				// Get the box's options
				$options = scModuleOptions::getCmsBoxOptions($cmsBox['CMSBoxID'], scModuleOptions::MODULE_OPTIONS_NO_ML);
				// Check if the box is for one of the newsletters that are going to be deleted.
				if (isset($options['NewsletterID']) and in_array($options['NewsletterID'], $newsletterIds)) {
					// Deactivate the box
					$boxObject = new scCmsBoxDomainObject();
					$boxObject->setRow($cmsBox);
					$boxObject->set('CMSBoxIsActive', 0);
					$boxObject->update();
				}
			}
		}
		
		/**
		 * Update the newsletter.
		 * Checks if the subject or body has been modified.
		 * If so, sets the IsTested flag to FALSE
		 */
		public function update()
		{
			$copy = new scNewsletter();
			$copy->read($this->get('NewsletterID'));
			
			$subjectChanged = ($copy->get('NewsletterSubject') != $this->get('NewsletterSubject'));
			$bodyChanged = ($copy->get('NewsletterXhtmlText') != $this->get('NewsletterXhtmlText'));
			if ($subjectChanged || $bodyChanged) {
				$this->set('NewsletterIsTested', false);
			}
			
			$isActiveChanged = ($copy->get('NewsletterIsActive') != $this->get('NewsletterIsActive'));
			if ($isActiveChanged && ($copy->get('NewsletterIsActive') == 1)) {
				// The newsletter is being deactivated
				$this->deactivateCmsBoxes(array($this->get('NewsletterID')));
			}
			
			return parent::update();
		}
		
		/**
		 * @return scNewsletterBlast|null
		 */
		public function getActiveBlast()
		{
			$query = 'SELECT * FROM NLBlasts WHERE BlastIsActive AND NewsletterID = ?';
			$data = miStaticDBUtil::execSelect($query, array($this->get('NewsletterID')));
			
			if (empty($data) || !isset($data[0])) {
				return null;
			}
			
			foreach ($data as $blastRow) {
				$blast = new scNewsletterBlast();
				$blast->setRow($blastRow);
				if ($blast->getSentEmailsCount() < $blast->getEmailsCount()) {
					return $blast;
				}
				$blast->set('BlastIsActive', 0);
				$blast->update();
			}
			
			return null;
		}
		
		public function hasSubscriber($subscriberEmail)
		{
			$recordset = new miSqlRecordset('NLSubscribers');
			$recordset->addFilter(new miSqlFilterEqual('NewsletterID', $this->get('NewsletterID')));
			$recordset->addFilter(new miSqlFilterEqual('SubscriberEmail', $subscriberEmail));
			return (bool)$recordset->getRecordsCount();
		}
		
		public function subscribe($subscriberEmail, $customerId = null, $languageId = null)
		{
			$query = 'INSERT INTO NLSubscribers SET' .
				' NewsletterID = ' . intval($this->get('NewsletterID')) .
				(!is_null($customerId) ? ', CustomerID = ' . intval($customerId) : '') .
				', LanguageID = ' . (empty($languageId) ? intval(scMultiLanguage::getCurrentLanguageId()) : intval($languageId)) .
				', SubscriberEmail = "' . sql_escape_string($subscriberEmail) . '"' .
				', SubscriberIsActive = 1' .
				' ON DUPLICATE KEY UPDATE SubscriberIsActive = 1' .
				(!is_null($customerId) ? ', CustomerID = ' . intval($customerId) : '') .
				', LanguageID = ' . (empty($languageId) ? intval(scMultiLanguage::getCurrentLanguageId()) : intval($languageId));
			miStaticDBUtil::execSQL($query);
			return true;
		}
		
		public function unsubscribe($email)
		{
			miStaticDBUtil::execUpdate('NLSubscribers', array('SubscriberIsActive' => 0), 'SubscriberEmail', $email);
			return true;
		}
		
		public function getRow($languageId = null)
		{
			$row = parent::getRow();
			
			if (!empty($languageId)) {
				$row['NewsletterName'] = $row['NewsletterName'][$languageId];
				$row['NewsletterSubject'] = $row['NewsletterSubject'][$languageId];
				$row['NewsletterXhtmlText'] = $row['NewsletterXhtmlText'][$languageId];
				$row['LanguageID'] = $row['LanguageID'][$languageId];
			}
			
			return $row;
		}
		
		public function get($name, $languageId = null)
		{
			$value = parent::get($name);
			
			if (!empty($languageId) && in_array($name, array_merge($this->_recordMlFields, array('LanguageID'))))
				return $value[$languageId];
			
			return $value;
		}
		
		/**
		 * Allowed languages for this Newsletter
		 * A language is allowed if the newsletter has configured subject in that particular language
		 * @return array(LanguageID => LanguageCode)
		 */
		public function getAllowedLanguageCodes()
		{
			if (empty($this->_activeLanguageCodes))
				foreach ($this->get('NewsletterSubject') as $languageId => $subject)
					if (!empty($subject))
						$this->_activeLanguageCodes[$languageId] = scMultiLanguage::singleton()->getLanguageCodeById($languageId);
			
			return $this->_activeLanguageCodes;
		}
	}
?>