<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scNewsletterTestAction extends miAction {
		
		public function doAction()
		{
			try {
				$email = scSettings::getSettingValue(scSettings::SETTING_EMAIL_REPLY_TO_NEWSLETTER);
				
				$newsletterId = (int)miGetParam('NewsletterID');
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read($newsletterId);
				
				$form = $this->getWebForm();
				
				$newsletterActiveLanguages = array_intersect_key(scMultiLanguage::getActiveLanguageNames(), $newsletter->getAllowedLanguageCodes());
				$form->addMainPageElements(array(
					'%%NEWSLETTER_LANGUAGES%%' => sprintf(_t('You will receive newsletters in %s.'), implode(', ', $newsletterActiveLanguages))
				));
				
				$form->setFormData('NewsletterID', $newsletterId);
				$form->setFormData('NewsletterTestEmails', $email);
				
				$template = $this->_view->getTemplateFileName(scNewsletterAdminView::TEMPLATE_ID_TEST);
				$html = $form->parse($template, true);
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$msg = $exception->getMessage();
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		/**
		 * @return miWebForm
		 */
		protected function getWebForm()
		{
			$form = new miWebForm(array());
			$form->setErrorsHandler(new scByFieldErrorsHandler());
			
			$widget = new miBaseTextWidget($form);
			$widget->setFieldName('NewsletterID');
			$form->addWidget('NewsletterID', $widget);
			
			$widget = new miBaseTextWidget($form);
			$widget->setFieldName('NewsletterTestEmails');
			$form->addWidget('NewsletterTestEmails', $widget);
			$form->addValidator('NewsletterTestEmails', new scValidatorNoEmpty($form, 'NewsletterTestEmails'));
			
			$this->_view->initMainPageElements();
			$form->addMainPageElements($this->_view->getMainPageElements());
			
			return $form;
		}
	}
	
	class scNewsletterExecTestAction extends scNewsletterTestAction {
		
		public function doAction()
		{
			try {
				$form = $this->getWebForm();
				$isOk = $form->processSubmit();
				
				if (!$isOk) {
					parent::doAction();
					return false;
				}
				
				$data = $form->getWidget('NewsletterID')->getData();
				$newsletterId = (int)$data['NewsletterID'];
				
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read($newsletterId);
				
				$data = $form->getWidget('NewsletterTestEmails')->getData();
				$sendTo = new scEmailAddress($data['NewsletterTestEmails']);
				
				$options = scNewsletterOptions::ADD_STORE_BASE_HREF | scNewsletterOptions::ADD_BASE_TARGET_BLANK;
				
				$email = new scEmail();
				$email->setTo($sendTo);
				
				$newsletterActiveLanguages = array_intersect_key(scMultiLanguage::getActiveLanguageNames(), $newsletter->getAllowedLanguageCodes());
				foreach ($newsletterActiveLanguages as $languageId => $languageCode) {
					$email->setSubject($newsletter->get('NewsletterSubject', $languageId));
					if (!$email->sendHtmlEmail(scNewsletterBlasts::getBlastHtmlText($newsletter, $options, $languageId))) {
						throw new miException(_t('E-mail sending failed.'));
					}
				}
				
				$newsletter->set('NewsletterIsTested', true);
				$newsletter->update();
				
				$msg = _t('Test message successfully sent.');
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
				return true;
			} catch (miException $exception) {
				$msg = $exception->getMessage();
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scNewsletterBlastAction extends miCreateAction {
		
		public function doAction()
		{
			try {
				$form = $this->getWebForm();
				
				$newsletterId = (int)miGetParam('NewsletterID');
				$blastId = (int)miGetParamDefault('BlastID', 0);
				if ($blastId > 0) {
					$blast = new scNewsletterBlast();
					$blast->read($blastId);
				} else {
					$newsletter = $this->_view->getDomainObject();
					$newsletter->read($newsletterId);
					$blast = $newsletter->getActiveBlast();
				}
				
				if ($blast instanceof scNewsletterBlast) {
					$cmd = scNewsletterBlasts::getBlastProgressControllerCommand($blast, $this->_view->getTemplateFileName(scNewsletterAdminView::TEMPLATE_ID_BLAST_PROGRESS), $this->_view->getMainPageElements());
					$this->_view->addControllerCommand($cmd);
					return true;
				}
				
				$form->setFormData('NewsletterID', $newsletterId);
				
				$newsletterActiveLanguageIds = array_keys($newsletter->getAllowedLanguageCodes());
				$newsletterRows = array();
				foreach (scMultiLanguage::getActiveLanguageNames() as $languageId => $languageName) {
					$languageIdWidget = $form->getWidget('LanguageID' . $languageId);
					$languageIdWidget->setDefaultValue(in_array($languageId, $newsletterActiveLanguageIds) ? 1 : 0);
					$languageIdControl = $languageIdWidget->getEditableControl();
					
					$blastToCustomersControl = $form->getWidget('BlastToCustomers' . $languageId)->getEditableControl();
					$blastToSubscribersControl = $form->getWidget('BlastToSubscribers' . $languageId)->getEditableControl();
					$elementsArray = array(
						'LanguageName' => $languageName,
						'HTML_LanguageID' => $languageIdControl['LanguageID' . $languageId],
						'HTML_BlastToCustomers' => $blastToCustomersControl['BlastToCustomers' . $languageId],
						'HTML_BlastToSubscribers' => $blastToSubscribersControl['BlastToSubscribers' . $languageId]
					);
					$newsletterRows[] = $newsletter->getRow($languageId) + $elementsArray;
				}
				
				$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('allLanguages', count($newsletterRows), miUIUtil::templetizeRows(miUIUtil::transposeRows($newsletterRows))));
				
				$this->showForm($form, $this->_view->getTemplateFileName(scNewsletterAdminView::TEMPLATE_ID_BLAST), true);
				return $this->onSuccess();
			} catch (miException $exception) {
				return $this->onFailure($exception);
			}
		}
		
		/**
		 * @return miWebForm
		 */
		public function getWebForm()
		{
			$newsletterId = (int)miGetParam('NewsletterID');
			
			if ($this->_webForm == null) {
				$form = new miWebForm(array());
				$form->setErrorsHandler(new scByFieldErrorsHandler());
				
				$widget = new miBaseTextWidget($form);
				$widget->setFieldName('NewsletterID');
				$form->addWidget('NewsletterID', $widget);
				
				foreach (scMultiLanguage::getActiveLanguageNames() as $languageId => $languageName) {
					$widget = new miCheckboxWidget($form);
					$widget->setFieldName('LanguageID' . $languageId);
					$form->addWidget('LanguageID' . $languageId, $widget);
					
					$widget = new miSelectWidget($form);
					$widget->setFieldName('BlastToCustomers' . $languageId);
					$widget->setOptions(scNewsletterBlasts::getSendToCustomersOptions($newsletterId, $languageId), scNewsletterBlasts::SEND_TO_CUSTOMERS_SUBSCRIBED);
					$form->addWidget('BlastToCustomers' . $languageId, $widget);
					
					$widget = new miSelectWidget($form);
					$widget->setFieldName('BlastToSubscribers' . $languageId);
					$widget->setOptions(scNewsletterBlasts::getSendToSubscribersOptions($newsletterId, $languageId), scNewsletterBlasts::SEND_TO_SUBSCRIBERS_NORMAL);
					$form->addWidget('BlastToSubscribers' . $languageId, $widget);
				}
				
				$this->_view->initMainPageElements();
				$form->addMainPageElements($this->_view->getMainPageElements());
				
				$this->_webForm = $form;
			}
			
			return $this->_webForm;
		}
	}
	
	class scNewsletterExecBlastAction extends scNewsletterBlastAction {
		
		public function doAction()
		{
			try {
				scDateTime::setTimezone();
				
				$form = $this->getWebForm();
				$isOk = $form->processSubmit();
				
				if ($isOk === false) {
					parent::doAction();
					return false;
				}
				
				$data = $form->getWidget('NewsletterID')->getData();
				$newsletterId = (int)$data['NewsletterID'];
				
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read($newsletterId);
				
				$newsletterActiveLanguageIds = array_keys($newsletter->getAllowedLanguageCodes());
				$blastToCustomers = array();
				$blastToSubscribers = array();
				
				foreach ($newsletterActiveLanguageIds as $languageId) {
					$widget = $form->getWidget('LanguageID' . $languageId);
					// This might happen if there are more languages in the newsletter then active languages in the store
					if (!$widget)
						continue;
					
					$languageIsChecked = $widget->getData();
					if (!(int)$languageIsChecked['LanguageID' . $languageId])
						continue;
					
					$data = $form->getWidget('BlastToCustomers' . $languageId)->getData();
					$blastToCustomers[$languageId] = (int)$data['BlastToCustomers' . $languageId];
					
					$data = $form->getWidget('BlastToSubscribers' . $languageId)->getData();
					$blastToSubscribers[$languageId] = (int)$data['BlastToSubscribers' . $languageId];
				}
				
				$options = $newsletter->get('NewsletterOptions');
				$options |= scNewsletterOptions::ADD_STORE_BASE_HREF;
				
				// Create blast
				$blast = scNewsletterBlasts::createBlast($newsletter, $blastToCustomers, $blastToSubscribers, $options);
				
				scNewsletterStartBlastProcessAction::registerProcessStart($blast->get('BlastID'));
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			
			$loc = 'module.php?action=dmBlast&BlastID=' . intval($blast->get('BlastID')) . '&NewsletterID=' . $newsletterId . '&ModuleName=com.summercart.newsletter';
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($cmd);
			return true;
		}
	}
	
	class scNewsletterShowBlastAction extends miCreateAction {
		
		public function doAction()
		{
			try {
				$blast = new scNewsletterBlast();
				$blast->read((int)miGetParam('BlastID'));
				
				$customerOptions = array(
					scNewsletterBlasts::SEND_TO_CUSTOMERS_NONE => _t('None'),
					scNewsletterBlasts::SEND_TO_CUSTOMERS_SUBSCRIBED => _t('Subscribed'),
					scNewsletterBlasts::SEND_TO_CUSTOMERS_ALL => _t('All')
				);
				$subscriberOptions = array(
					scNewsletterBlasts::SEND_TO_SUBSCRIBERS_NONE => _t('None'),
					scNewsletterBlasts::SEND_TO_SUBSCRIBERS_NORMAL => _t('Subscribed'),
					scNewsletterBlasts::SEND_TO_SUBSCRIBERS_INCLUDING_UNSUBSCRIBED => _t('All including unsubscribed')
				);
				
				$blastRows = array();
				foreach (scMultiLanguage::getActiveLanguageNames() as $languageId => $languageName) {
					$blastSubject = $blast->get('BlastSubject', $languageId);
					if (empty($blastSubject))
						continue;
					
					$blastRows[] = array(
						'LanguageName' => $languageName,
						'BlastSubject' => $blastSubject,
						'LanguageID' => $blast->get('LanguageID', $languageId),
						'BlastToCustomers' => $customerOptions[$blast->get('BlastToCustomers', $languageId)],
						'BlastToSubscribers' => $subscriberOptions[$blast->get('BlastToSubscribers', $languageId)]
					);
				}
				
				$t = new scTemplateParser();
				$t->readTemplate($this->_view->getTemplateFileName(scNewsletterAdminView::TEMPLATE_ID_BLAST_VIEW));
				$t->addSectionInfo(new miTemplateParserSectionInfo('allLanguages', count($blastRows), miUIUtil::templetizeRows(miUIUtil::transposeRows($blastRows))));
				$t->assign('%%FMT_BLASTDATE%%', scDateTime::getFormattedDate($blast->get('BlastDate')));
				$t->assign('%%BLASTID%%', $blast->get('BlastID'));
				$t->assign('%%NEWSLETTERID%%', $blast->get('NewsletterID'));
				
				$this->_view->initMainPageElements();
				$t->assignArray($this->_view->getMainPageElements());
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $t->templateParse());
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scNewsletterShowBlastContentsAction extends miAction {
		
		public function doAction()
		{
			try {
				$blast = new scNewsletterBlast();
				$blast->read((int)miGetParam('BlastID'));
				
				// Make sure that the Iframe is in the correct encoding (UTF-8)
				$html =
'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
</head><body>';		
				$html .= $blast->get('BlastXhtmlText', (int)miGetParam('LanguageID'));
				$html .= '</body></html>';
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scNewsletterShowContentsAction extends miAction {
		
		public function doAction()
		{
			try {
				$newsletterId = (int)miGetParam('NewsletterID');
				
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read($newsletterId);
				
				$options = scNewsletterOptions::ADD_STORE_BASE_HREF | scNewsletterOptions::ADD_BASE_TARGET_BLANK;
				
				// Make sure that the Iframe is in the correct encoding (UTF-8)
				$html =
'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
</head><body>';		
				$html .= scNewsletterBlasts::getBlastHtmlText($newsletter, $options, (int)miGetParam('LanguageID'));
				$html .= '</body></html>';
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scNewsletterGetBlastProgressJsonAction extends miAction {
		
		public function doAction()
		{
			$output = array(
				'error' => '',
				'sentCount' => '',
				'sentPercentage' => '',
			);
			
			try {
				$blastId = (int)miGetParam('BlastID');
				
				$blast = new scNewsletterBlast();
				$blast->read($blastId);
				
				$sent = $blast->getSentEmailsCount();
				$total = $blast->getEmailsCount();
				
				$output['sentCount'] = $sent;
				if ($total == 0) {
					$output['sentPercentage'] = 0;
				} else {
					$output['sentPercentage'] = round(($sent * 100) / $total);
				}
				$return = true;
				
			} catch (miException $e) {
				$output['error'] = $e->getMessage();
				$return = false;
			}
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, scJson::jsonEncode($output));
			$this->_view->addControllerCommand($cmd);
			return $return;
		}
	}
	
	class scNewsletterStartBlastProcessAction extends scNewsletterGetBlastProgressJsonAction {
		
		public static function registerProcessStart($blastId)
		{
			// Register shutdown functions to process the blasts
			ignore_user_abort(true);
			set_time_limit(0);
			header('Connection: close');
			ini_set('max_execution_time', 0);
			register_shutdown_function('flush');
			register_shutdown_function(array('scNewsletterBlasts', 'processPendingBlasts'), $blastId);
		}
		
		public function doAction()
		{
			$blastId = (int)miGetParam('BlastID');
			self::registerProcessStart($blastId);
			header('Location: module.php?ModuleName=com.summercart.newsletter&action=dmGetBlastProgress&BlastID=' . $blastId . '&load=table');
			die();
		}
	}
	
	class scNewsletterStopBlastAction extends miAction {
		
		public function doAction()
		{
			try {
				$blastId = (int)miGetParam('BlastID');
				
				$blast = new scNewsletterBlast();
				$blast->read($blastId);
				
				$blast->set('BlastIsActive', 0);
				$blast->update();
				
				$msg = _t('Blast successfully stopped.');
				$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
	}
	
	class scNewsletterShowStatsAction extends miAction {
		
		public function doAction()
		{
			try {
				scDateTime::setTimezone();
				
				$newsletterId = (int)miGetParam('NewsletterID');
				$month = miGetParamDefault('StatsMonth', 0);
				$year = miGetParamDefault('StatsYear', date('Y'));
				$languageId = miGetParamDefault('LanguageID', '');
				
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read($newsletterId);
				
				$rows = $this->getStatsRows($newsletterId, $month, $year, $languageId);
				$filters = $this->getFiltersDropdown($month, $year, $languageId);
				
				$sections = array();
				if (miGetParamDefault('load', '') != 'table') {
					$sections[] = new miTemplateParserSectionInfo('static', 1);
				}
				$sections[] = new miTemplateParserSectionInfo('rows', count($rows), miUIUtil::templetizeRows(miUIUtil::transposeRows($rows)));
				
				$t = new scTemplateParser();
				$t->readTemplate($this->_view->getTemplateFileName(scNewsletterAdminView::TEMPLATE_ID_STATS));
				$t->setSectionInfos($sections);
				$t->assignArray(array(
					'%%NEWSLETTERID%%' => $newsletterId,
					'%%NEWSLETTERTITLE%%' => sprintf(_t('Newsletter "%s" Statistics'), $newsletter->get('NewsletterName', scMultiLanguage::getCurrentLanguageId()))
				));
				$t->assignArray(miUIUtil::templetizeRows($filters));
				
				$this->_view->initMainPageElements();
				$t->assignArray($this->_view->getMainPageElements());
				
				$html = $t->templateParse();
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		protected function getFiltersDropdown($month, $year, $languageId)
		{
			$months = array(
				0 => _t('All months'),
				1 => _t('January'),
				_t('February'),
				_t('March'),
				_t('April'),
				_t('May'),
				_t('June'),
				_t('July'),
				_t('August'),
				_t('September'),
				_t('October'),
				_t('November'),
				_t('December'),
			);
			
			$query = 'SELECT DISTINCT(YEAR(EventDate)) AS Years FROM (' .
					'SELECT DISTINCT(BlastEmailSentDate) AS EventDate FROM NLBlastEmails' .
					' UNION DISTINCT ' .
					'SELECT DISTINCT(BlastEmailOpenDate) AS EventDate FROM NLBlastEmailOpens' .
					' UNION DISTINCT ' .
					'SELECT DISTINCT(BlastEmailClickDate) AS EventDate FROM NLBlastEmailClicks' .
				') AS Years ORDER BY Years';
			$years = miStaticDBUtil::execSelect($query);
			$years = miUtil::getKeyValueArray($years, 'Years', 'Years');
			
			$return = array(
				'StatsMonth' => miUIUtil::getConstDropdown('StatsMonth', $month, $months),
				'StatsYear' => miUIUtil::getConstDropdown('StatsYear', $year, $years),
			);
			
			$languages = array('' => _t('All languages')) + scMultiLanguage::getActiveLanguageNames();
			$return['LanguageID'] = miUIUtil::getConstDropdown('LanguageID', $languageId, $languages);
			
			return $return;
		}
		
		protected function getStatsRows($newsletterId, $month, $year, $languageId)
		{
			$startMonth = $month;
			$endMonth = $month + 1;
			if ($month == 0) {
				$startMonth = 1;
				$endMonth = 13; // Next year's January
			}
			
			$periodStart = mktime(0, 0, 0, $startMonth, 1, $year);
			$periodEnd = mktime(0, 0, 0, $endMonth, 1, $year);
			$languageCondition = '';
			if (!empty($languageId))
				$languageCondition = sprintf(' AND NLBlastEmails.LanguageID = %d', $languageId);
			
			$recordset = new miSqlRecordset('NLBlasts');
			$recordset->addFilter(new miSqlFilterEqual('NewsletterID', $newsletterId));
			$recordset->addFilter(new miSqlFilterBiggerOrEqual('BlastDate', date(MI_MYSQL_DATE_FORMAT, $periodStart)));
			$recordset->addFilter(new miSqlFilterSmallerThan('BlastDate', date(MI_MYSQL_DATE_FORMAT, $periodEnd)));
			$recordset->setSelectFields(array(
				'NLBlasts.*',
				sprintf('(SELECT COUNT(BlastEmailID) FROM NLBlastEmails WHERE NLBlastEmails.BlastID = NLBlasts.BlastID %s) AS StatsEmailsSent', $languageCondition),
				sprintf('(SELECT COUNT(NLBlastEmailOpens.BlastEmailID) FROM NLBlastEmails INNER JOIN NLBlastEmailOpens USING(BlastEmailID) WHERE NLBlastEmails.BlastID = NLBlasts.BlastID %s) AS StatsEmailsOpened', $languageCondition),
				sprintf('(SELECT SUM(NLBlastEmailClicks.BlastEmailClicksCount) FROM NLBlastEmails INNER JOIN NLBlastEmailClicks USING(BlastEmailID) WHERE NLBlastEmails.BlastID = NLBlasts.BlastID %s) AS StatsEmailClicks', $languageCondition),
				sprintf('(SELECT NLBlastsML.BlastSubject FROM NLBlastsML WHERE NLBlastsML.BlastID = NLBlasts.BlastID AND NLBlastsML.LanguageID = %d) AS BlastSubject', scMultiLanguage::getCurrentLanguageId())
			));
			
			$rows = $recordset->getRecords();
			foreach ($rows as &$row) {
				$row['FMT_BlastDate'] = scDateTime::getFormattedDate($row['BlastDate']);
				$row['StatsEmailClicks'] = (is_null($row['StatsEmailClicks']) ? 0 : $row['StatsEmailClicks']);
			}
			
			return $rows;
		}
	}
	
	class scNewsletterShowStatsBlastAction extends scNewsletterShowStatsAction {
		
		public function doAction()
		{
			try {
				scDateTime::setTimezone();
				
				$blastId = (int)miGetParam('BlastID');
				$month = miGetParamDefault('StatsMonth', date('n'));
				$year = miGetParamDefault('StatsYear', date('Y'));
				$languageId = miGetParamDefault('LanguageID', '');
				
				$blast = new scNewsletterBlast();
				$blast->read($blastId);
				$blastDate = scDateTime::getFormattedDate($blast->get('BlastDate'));
				$blastSubject = $blast->get('BlastSubject', scMultiLanguage::getCurrentLanguageId());
				
				$rows = $this->getStatsRows($blastId, $month, $year, $languageId);
				$filters = $this->getFiltersDropdown($month, $year, $languageId);
				
				$sections = array();
				if (miGetParamDefault('load', '') != 'table')
					$sections[] = new miTemplateParserSectionInfo('static', 1);
				
				$sections[] = new miTemplateParserSectionInfo('rows', count($rows), miUIUtil::templetizeRows(miUIUtil::transposeRows($rows)));
				
				$t = new scTemplateParser();
				$t->readTemplate($this->_view->getTemplateFileName(scNewsletterAdminView::TEMPLATE_ID_STATS_BLAST));
				$t->setSectionInfos($sections);
				$t->assignArray(array(
					'%%BLASTID%%' => $blastId,
					'%%NEWSLETTERID%%' => $blast->get('NewsletterID'),
					'%%BLASTTITLE%%' => sprintf(_t('Blast "%s - %s" Statistics'), $blastDate, $blastSubject)
				));
				$t->assignArray(miUIUtil::templetizeRows($filters));
				
				$this->_view->initMainPageElements();
				$t->assignArray($this->_view->getMainPageElements());
				
				$html = $t->templateParse();
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		protected function getStatsRows($blastId, $month, $year, $languageId)
		{
			$startMonth = $month;
			$endMonth = $month + 1;
			if ($month == 0) {
				$startMonth = 1;
				$endMonth = 13; // Next year's January
			}
			
			$periodStart = mktime(0, 0, 0, $startMonth, 1, $year);
			$periodEnd = mktime(0, 0, 0, $endMonth, 1, $year);
			
			$recordsets = array();
			$recordsets['StatsEmailsSent'] = $this->getEmailsSentRecordset($periodStart, $periodEnd);
			$recordsets['StatsEmailsOpened'] = $this->getEmailsOpenedRecordset($periodStart, $periodEnd);
			$recordsets['StatsEmailClicks'] = $this->getEmailClicksRecordset($periodStart, $periodEnd);
			
			$columns = array();
			$dates = array();
			foreach ($recordsets as $columnName => $recordset) {
				$recordset->addFilter(new miSqlFilterEqual('NLBlastEmails.BlastID', $blastId));
				if (!empty($languageId))
					$recordset->addFilter(new miSqlFilterEqual('LanguageID', $languageId));
				$column = miUtil::getKeyValueArray($recordset->getAllRecords(), 'EventDate', 'EventValue');
				$dates = array_merge($dates, array_keys($column));
				$columns[$columnName] = $column;
			}
			$dates = array_unique($dates);
			sort($dates);
			
			$rows = array();
			foreach ($dates as $date) {
				$row = array(
					'StatsDate' => $date,
					'FMT_StatsDate' => scDateTime::getFormattedDate($date),
				);
				foreach ($columns as $columnName => $column) {
					$row[$columnName] = 0;
					if (isset($column[$date])) {
						$row[$columnName] = $column[$date];
					}
				}
				$rows[] = $row;
			}
			return $rows;			
		}
		
		/**
		 * @return miSqlRecordset
		 */
		protected function getEmailsSentRecordset($periodStart, $periodEnd)
		{
			$recordset = new miSqlRecordset('NLBlastEmails');
			$recordset->addJoinCondition('INNER', 'NLBlasts', 'USING(BlastID)');
			$recordset->setGroupBy('BlastEmailSentDate');
			$recordset->setSelectFields(array(
				'BlastEmailSentDate AS EventDate',
				'COUNT(BlastEmailID) AS EventValue'
			));
			$recordset->addFilter(new miSqlFilterBiggerOrEqual('BlastEmailSentDate', date(MI_MYSQL_DATE_FORMAT, $periodStart)));
			$recordset->addFilter(new miSqlFilterSmallerThan('BlastEmailSentDate', date(MI_MYSQL_DATE_FORMAT, $periodEnd)));
			
			return $recordset;
		}
		
		/**
		 * @return miSqlRecordset
		 */
		protected function getEmailsOpenedRecordset($periodStart, $periodEnd)
		{
			$recordset = new miSqlRecordset('NLBlastEmailOpens');
			$recordset->addJoinCondition('INNER', 'NLBlastEmails', 'USING(BlastEmailID)');
			$recordset->addJoinCondition('INNER', 'NLBlasts', 'USING(BlastID)');
			$recordset->setGroupBy('BlastEmailOpenDate');
			$recordset->setSelectFields(array(
				'BlastEmailOpenDate AS EventDate',
				'COUNT(NLBlastEmailOpens.BlastEmailID) AS EventValue'
			));
			$recordset->addFilter(new miSqlFilterBiggerOrEqual('BlastEmailOpenDate', date(MI_MYSQL_DATE_FORMAT, $periodStart)));
			$recordset->addFilter(new miSqlFilterSmallerThan('BlastEmailOpenDate', date(MI_MYSQL_DATE_FORMAT, $periodEnd)));
			
			return $recordset;
		}
		
		/**
		 * @return miSqlRecordset
		 */
		protected function getEmailClicksRecordset($periodStart, $periodEnd)
		{
			$recordset = new miSqlRecordset('NLBlastEmailClicks');
			$recordset->addJoinCondition('INNER', 'NLBlastEmails', 'USING(BlastEmailID)');
			$recordset->addJoinCondition('INNER', 'NLBlasts', 'USING(BlastID)');
			$recordset->setGroupBy('BlastEmailClickDate');
			$recordset->setSelectFields(array(
				'BlastEmailClickDate AS EventDate',
				'SUM(BlastEmailClicksCount) AS EventValue'
			));
			$recordset->addFilter(new miSqlFilterBiggerOrEqual('BlastEmailClickDate', date(MI_MYSQL_DATE_FORMAT, $periodStart)));
			$recordset->addFilter(new miSqlFilterSmallerThan('BlastEmailClickDate', date(MI_MYSQL_DATE_FORMAT, $periodEnd)));
			
			return $recordset;
		}
	}
	
	class scNewsletterShowSubscribersAction extends miAction {
		
		public function doAction()
		{
			try {
				$newsletterId = (int)miGetParam('NewsletterID');
				
				$recordset = new miSqlRecordset('NLSubscribers');
				$recordset->addFilter(new miSqlFilterEqual('NewsletterID', $newsletterId));
				
				$table = new scNewsletterSubscribersTable($recordset);
				$table->setState(miAppFactory::singleton()->getStateObj()->getArray());
				
				$pager = new scTablePager($table);
				$table->addTableFeature($pager);
				$table->addTableFeature(new scTableRecordsPerPage($table, $pager));
				
				$moduleParams = new miTableCustomParams($table);
				$moduleParams->setRecordLocatorParams(array(
					'ModuleName' => 'com.summercart.newsletter',
					'action' => 'dmShowSubscribers',
					'NewsletterID' => $newsletterId
				));
				$table->addTableFeature($moduleParams);
				
				$filters = new miTableFilters($table);
				$filters->setDefaultFilterValues(array(
					'SubscriberIsActive' => 1,
					'SubscriberEmail' => '',
					'LanguageID' => '',
				));
				$table->addTableFeature($filters);
				
				$active = array('' => _t('All statuses'), 1 => _t('Active'), 0 => _t('Inactive'));
				$this->createDropdownFilter($filters, $table, 'SubscriberIsActive', $active, '=', 1);
				
				$languages = array('' => _t('All languages')) + scMultiLanguage::getActiveLanguageNames();
				$this->createDropdownFilter($filters, $table, 'LanguageID', $languages, '=', '');
				
				$recordset->addFilters($filters->getFilterObjs());
				$recordset->setFieldsValidation(true);
				$totalRecords = $recordset->getRecordsCount();
				$page = $pager->getPageParam();
				$recordsPerPage = $pager->getRecordsPerPageParam();
				
				if (($page-1) * $recordsPerPage >= $totalRecords) {
					$page = intval(($totalRecords-1) / $recordsPerPage) + 1;
				}
				
				$pager->setPagerLocation($page, $recordsPerPage, $totalRecords);
				$from = intval($page - 1) * $recordsPerPage;
				
				$recordset->setRecordsLimit($from, $recordsPerPage);
				$table->updateState();
				
				$html = $table->parse('modules/system/newsletter/admin/newsletter_subscribers_mgmt.tmpl');
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		protected function createDropdownFilter($filterObj, $table, $field, $values, $condition, $defaultValue = null)
		{
			$filterValues = $filterObj->getFilterValues();
			$selectedValue = isset($filterValues[$field]) ? $filterValues[$field] : '';
			
			// Make sure that the filter is applied
			if (!isset($_REQUEST[$field . 'Filter']))
				$filterObj->addAdditionalFilter($field, $selectedValue, $condition);
			
			$htmlAttributes = array();
			if (!is_null($defaultValue)) {
				$htmlAttributes['sc:defaultValue'] = $defaultValue;
			}
			$dropdown = scUIUtil::getConstDropdown($field . 'Filter', $selectedValue, $values, $htmlAttributes);
			$condition = '<input type="hidden" name="' . miI18N::htmlEscape($field) . 'Condition" value="' . miI18N::htmlEscape($condition) . '"/>';
			$table->assign('%%HTML_' . strtoupper($field) . '_FILTER%%', $dropdown . $condition);
		}
	}
	
	class scNewsletterAddSubscribersAction extends miAction {
		
		/**
		 * @var miWebForm
		 */
		protected $_webForm = null;
		
		public function doAction()
		{
			try {
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read((int)miGetParam('NewsletterID'));
				
				$form = $this->getWebForm();
				
				$form->setFormData('NewsletterID', $newsletter->get('NewsletterID'));
				
				$form->addMainPageElements(miUIUtil::templetizeRows($newsletter->getRow(scMultiLanguage::getCurrentLanguageId())));
				$form->addMainPageElements($this->_view->getMainPageElements());
				
				$html = $form->parse($this->_view->getTemplateFileName(scNewsletterAdminView::TEMPLATE_ID_ADD_SUBSCRIBERS), true);
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		/**
		 * @return miWebForm
		 */
		public function getWebForm()
		{
			$newsletterId = (int)miGetParam('NewsletterID');
			
			if ($this->_webForm == null) {
				$form = new miWebForm(array());
				$form->setErrorsHandler(new scByFieldErrorsHandler());
				
				$widget = new miBaseTextWidget($form);
				$widget->setFieldName('NewsletterID');
				$form->addWidget('NewsletterID', $widget);
				
				$widget = new miSelectWidget($form);
				$widget->setFieldName('ChoosenLanguageID'); // LanguageID is reserved by the global ML selector
				$widget->setOptions(scMultiLanguage::getActiveLanguageNames());
				$form->addWidget('ChoosenLanguageID', $widget);
				
				$widget = new miBaseTextWidget($form);
				$widget->setFieldName('AddedSubscribers');
				$form->addWidget('AddedSubscribers', $widget);
				
				$this->_view->initMainPageElements();
				$form->addMainPageElements($this->_view->getMainPageElements());
				
				$this->_webForm = $form;
			}
			
			return $this->_webForm;
		}
	}
	
	class scNewsletterExecAddSubscribersAction extends scNewsletterAddSubscribersAction {
		
		public function doAction()
		{
			try {
				scDateTime::setTimezone();
				
				$form = $this->getWebForm();
				$isOk = $form->processSubmit();
				
				if ($isOk === false) {
					parent::doAction();
					return false;
				}
				
				$data = $form->getWidget('NewsletterID')->getData();
				$newsletterId = (int)$data['NewsletterID'];
				
				$data = $form->getWidget('ChoosenLanguageID')->getData();
				$languageId = (int)$data['ChoosenLanguageID'];
				
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read($newsletterId);
				
				$data = $form->getWidget('AddedSubscribers')->getData();
				$addedSubscribers = $data['AddedSubscribers'];
				
				$addedSubscribers = str_replace("\n", ',', $addedSubscribers);
				$addedSubscribers = explode(',', $addedSubscribers);
				$addedSubscribers = array_filter($addedSubscribers);
				
				$addedCount = 0;
				$invalidEmails = array();
				foreach ($addedSubscribers as $email) {
					$email = trim($email);
					if (empty($email)) {
						continue;
					}
					if (!miValidatorEmail::isValid($email)) {
						$invalidEmails[] = $email;
						continue;
					}
					
					$customerId = scUtil::getDbValue('Customers', 'CustomerID', 'CustomerEmail', $email, null);
					$newsletter->subscribe($email, $customerId, $languageId);
					$addedCount += 1;
				}
				
				if (($addedCount == 0) && (count($invalidEmails) == 0)) {
					$msg = _t('No e-mails to add.');
				} else {
					$msg = '';
					if ($addedCount > 0) {
						$msg .= sprintf(_t('%s e-mails successfully subscribed.'), $addedCount);
					}
					if (count($invalidEmails) > 0) {
						$msg .= ' ' . sprintf(_t('These e-mails are invalid and were not subscribed: %s'), implode(', ', $invalidEmails));
					}
				}
				
				$this->redirectToNewsletter($newsletter, $msg);
				return true;
			} catch (miException $exception) {
				$this->_view->addRedirectToListControllerCommand($exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
		}
		
		protected function redirectToNewsletter(scNewsletter $newsletter, $msg)
		{
			$params = array(
				'ModuleName' => 'com.summercart.newsletter',
				'action' => 'dmShowSubscribers',
				'NewsletterID' => $newsletter->get('NewsletterID'),
				miMessage::PARAM_MESSAGE => $msg,
				miMessage::PARAM_MESSAGE_TYPE => miMessage::MSG_TYPE_INFO,
			);
			
			$loc = '?' . http_build_query($params);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($cmd);
		}
	}
	
	class scNewsletterCsvSubscribersAction extends scExportCSVAction {
		protected function createPrintExportUtil()
		{
			return new scNewslettersPrintExportUtil($this->_view);
		}
	}
	
	class scNewslettersPrintExportUtil extends scPrintExportUtil {
		public function getFields()
		{
			return array(
				'SubscriberEmail' => 'SubscriberEmail',
				'SubscriberIsActive' => 'SubscriberIsActive',
				'LanguageID' => 'LanguageID',
			);
		}
		
		public function getRecordset()
		{
			$recordset = new miSqlRecordset('NLSubscribers');
			$recordset->addFilter(new miSqlFilterEqual('NewsletterID', (int)miGetParam('NewsletterID')));
			$recordset->setSelectFields(array('SubscriberEmail', 'SubscriberIsActive', 'LanguageID'));
			return $recordset;
		}
	}
?>