<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scNewsletterBlastEmail extends scDomainObject {
		
		const STATUS_PENDING = 1;
		const STATUS_IN_PROGRESS = 2;
		const STATUS_SENT = 3;
		const STATUS_FAILED = 4;
		
		/**
		 * @var scNewsletterBlast
		 */
		protected $_blast = null;
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('NLBlastEmails', 'BlastEmailID'));
		}
		
		public function setBlast(scNewsletterBlast $blast)
		{
			$this->_blast = $blast;
		}
		
		public function updateStatus($newStatus, $sentDate = false)
		{
			$this->set('BlastEmailStatus', $newStatus);
			$values = array('BlastEmailStatus' => $newStatus);
			if ($sentDate !== false)
				$values['BlastEmailSentDate'] = $sentDate;
			
			miStaticDBUtil::execUpdate('NLBlastEmails', $values, 'BlastEmailID', $this->get('BlastEmailID'));
		}
		
		public function send()
		{
			// Get LanguageID
			$languageId = $this->get('LanguageID');
			
			// Get the blast's subject.
			$subject = $this->_blast->get('BlastSubject', $languageId);
			
			// Get the prepared blast HTML
			$html = $this->_blast->get('BlastXhtmlText', $languageId);
			
			// The blast text may contain template variables. (e.g. %%BLASTEMAILID%%)
			$t = new scTemplateParser();
			$t->setContents($html);
			$t->assignArray(miUIUtil::templetizeRows($this->getRow()));
			$html = $t->templateParse();
			
			$email = new scEmail();
			$fromAddress = scSettings::getSettingValue(scSettings::SETTING_EMAIL_REPLY_TO_NEWSLETTER);
			$fromName = scUtil::getMLDbValue('SettingsML', 'SettingValue', 'SettingName', scSettings::SETTING_COMPANY_NAME, $languageId);
			$email->setFrom(new scEmailAddress($fromAddress, $fromName));
			$email->setTo(new scEmailAddress($this->get('BlastEmail')));
			$email->setSubject($subject);
			if (!$email->sendHtmlEmail($html)) {
				$this->updateStatus(self::STATUS_FAILED, date(MI_MYSQL_DATE_FORMAT));
				return false;
			}
			
			$this->updateStatus(self::STATUS_SENT, date(MI_MYSQL_DATE_FORMAT));
			return true;
		}
	}
?>