<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scNewsletterBlasts {
		
		const SEND_TO_CUSTOMERS_NONE = 0;
		const SEND_TO_CUSTOMERS_SUBSCRIBED = 1;
		const SEND_TO_CUSTOMERS_ALL = 2;
		
		const SEND_TO_SUBSCRIBERS_NONE = 0;
		const SEND_TO_SUBSCRIBERS_NORMAL = 1;
		const SEND_TO_SUBSCRIBERS_INCLUDING_UNSUBSCRIBED = 2;
		
		/**
		 * Blast pending blasts
		 */
		public static function processPendingBlasts($blastId = 0)
		{
			$blastId = (int)$blastId;
			
			try {
				while (true) {
					// Find active blasts. First try to find the specified blast.
					$selectQuery = 'SELECT * FROM NLBlasts' .
						' WHERE BlastIsActive' .
						($blastId != 0 ? ' AND BlastID = ' . $blastId : '') .
						' LIMIT 1';
					$blast = miStaticDBUtil::execSelect($selectQuery);
					
					if (empty($blast) || !isset($blast[0])) {
						if ($blastId != 0) {
							// We were searching for the specified by $blastId blast.
							// Obviously this blast is no longer active (no matter the reason)
							// so we may continue processing other active blasts.
							$blastId = 0;
							continue;
						}
						
						// No active blasts found
						break;
					}
					
					$blastObject = new scNewsletterBlast();
					$blastObject->read($blast[0]['BlastID']);
					self::processBlast($blastObject);
				}
			} catch (Exception $exception) {
				scLog::singleton()->debug(_i('Error: ') . $exception->getMessage(), scLog::CATEGORY_NEWSLETTER);
			}
		}
		
		/**
		 * @param arrat $data Blast Data
		 */
		protected static function processBlast(scNewsletterBlast $blast)
		{
			miStaticDBUtil::startTransaction();
			
			try {
				$selectQuery = 'SELECT *' .
					' FROM NLBlastEmails' .
					' WHERE BlastID = ? AND BlastEmailStatus = ? ORDER BY BlastEmailID' .
					' LIMIT 1 FOR UPDATE';
				$params = array(
					$blast->get('BlastID'),
					scNewsletterBlastEmail::STATUS_PENDING
				);
				$email = miStaticDBUtil::execSelect($selectQuery, $params);
				
				if (empty($email) || !isset($email[0])) {
					// No blast email has been found for processing.
					// Mark the blast as inactive.
					$blast->set('BlastIsActive', 0);
					$blast->update();
					
					miStaticDBUtil::commit();
					return;
				}
				
				$blastEmail = new scNewsletterBlastEmail();
				$blastEmail->read($email[0]['BlastEmailID']);
				$blastEmail->setBlast($blast);
				
				// Set the status to IN PROGRESS so other instances of the blaster
				// don't attempt to send this one.
				$blastEmail->updateStatus(scNewsletterBlastEmail::STATUS_IN_PROGRESS, date(MI_MYSQL_DATE_FORMAT));
			} catch (Exception $exception) {
				miStaticDBUtil::rollback();
				throw $exception;
			}
			
			miStaticDBUtil::commit();
			
			$blastEmail->send();
		}
		
		/**
		 * @param scNewsletter $newsletter
		 * @param int $blastToCustomers
		 * @param int $blastToSubscribers
		 * @return scNewsletterBlast
		 */
		public static function createBlast(scNewsletter $newsletter, $blastToCustomers, $blastToSubscribers, $options = 0)
		{
			$newsletterId = (int)$newsletter->get('NewsletterID');
			
			$blast = new scNewsletterBlast();
			$blast->set('NewsletterID', $newsletter->get('NewsletterID'));
			$blast->set('BlastToCustomers', $blastToCustomers);
			$blast->set('BlastToSubscribers', $blastToSubscribers);
			$blast->set('BlastIsActive', 1);
			$blast->set('BlastDate', date(MI_MYSQL_DATE_FORMAT));
			
			$blastEmails = array();
			$blastSubjects = array();
			$blastXhtmlTexts = array();
			foreach ($blastToCustomers as $languageId => $customers) {
				$html = self::getBlastHtmlText($newsletter, $options, $languageId);
				
				$receivers = array();
				$receivers = array_merge($receivers, self::getNewsletterCustomersRecordset($newsletterId, $blastToCustomers[$languageId], $languageId)->getAllRecords());
				$receivers = array_merge($receivers, self::getNewsletterSubscribersRecordset($newsletterId, $blastToSubscribers[$languageId], $languageId)->getAllRecords());
				
				$blastSubjects[$languageId] = $newsletter->get('NewsletterSubject', $languageId);
				$blastXhtmlTexts[$languageId] = $html;
				
				foreach ($receivers as $receiverEmail) {
					$blastEmails[$receiverEmail['ReceiverEmail']] = array(
						'LanguageID' => $languageId,
						'BlastEmail' => $receiverEmail['ReceiverEmail'],
						'BlastEmailStatus' => scNewsletterBlastEmail::STATUS_PENDING
					);
				}
			}
			$blast->set('BlastSubject', $blastSubjects);
			$blast->set('BlastXhtmlText', $blastXhtmlTexts);
			$blastId = $blast->insert();
			
			// Set BlastID
			foreach ($blastEmails as &$blastEmail) {
				$blastEmail['BlastID'] = $blastId;
			}
			
			miStaticDBUtil::execMultiInsert('NLBlastEmails', array_values($blastEmails));
			
			return $blast;
		}
		
		/**
		 * @param scNewsletterBlast $blast
		 * @param string $template
		 * @param array $mainPageElements
		 * @return miControllerCommand
		 */
		public static function getBlastProgressControllerCommand(scNewsletterBlast $blast, $template, $mainPageElements = array())
		{
			$scheduledEmails = $blast->getEmailsCount();
			if ($scheduledEmails <= 0) {
				$params = array(
					'ModuleName' => 'com.summercart.newsletter',
					miMessage::PARAM_MESSAGE => _t('No scheduled messages.'),
					miMessage::PARAM_MESSAGE_TYPE => miMessage::MSG_TYPE_INFO
				);
				
				$loc = '?' . http_build_query($params);
				return new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			}
			
			$sentEmails = $blast->getSentEmailsCount();
			$percentage = round(($sentEmails * 100) / $scheduledEmails);
			
			$t = new scTemplateParser();
			$t->readTemplate($template);
			$t->assignArray($mainPageElements);
			$t->assign('%%BLASTID%%', $blast->get('BlastID'));
			$t->assign('%%BLASTEMAILSCOUNT%%', $scheduledEmails);
			$t->assign('%%BLASTSENTEMAILSCOUNT%%', $sentEmails);
			$t->assign('%%BLASTPROGRESSPERCENTAGE%%', $percentage);
			$html = $t->templateParse();
			
			return new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
		}
		
		public static function getSendToCustomersOptions($newsletterId, $languageId = null)
		{
			$options = array();
			
			$options[self::SEND_TO_CUSTOMERS_NONE] = _t('None');
			$options[self::SEND_TO_CUSTOMERS_SUBSCRIBED] = sprintf(_t('Subscribed (%s)'), self::getNewsletterCustomersRecordset($newsletterId, self::SEND_TO_CUSTOMERS_SUBSCRIBED, $languageId)->getRecordsCount());
			$options[self::SEND_TO_CUSTOMERS_ALL] = sprintf(_t('All (%s)'), self::getNewsletterCustomersRecordset($newsletterId, self::SEND_TO_CUSTOMERS_ALL, $languageId)->getRecordsCount());
			
			return $options;
		}
		
		public static function getSendToSubscribersOptions($newsletterId, $languageId = null)
		{
			$options = array();
			
			$options[self::SEND_TO_SUBSCRIBERS_NONE] = _t('None');
			$options[self::SEND_TO_SUBSCRIBERS_NORMAL] = sprintf(_t('Subscribed (%s)'), self::getNewsletterSubscribersRecordset($newsletterId, self::SEND_TO_SUBSCRIBERS_NORMAL, $languageId)->getRecordsCount());
			$options[self::SEND_TO_SUBSCRIBERS_INCLUDING_UNSUBSCRIBED] = sprintf(_t('All including unsubscribed (%s)'), self::getNewsletterSubscribersRecordset($newsletterId, self::SEND_TO_SUBSCRIBERS_INCLUDING_UNSUBSCRIBED, $languageId)->getRecordsCount());
			
			return $options;
		}
		
		/**
		 * @param int $newsletterId
		 * @param int $type
		 * @return miSqlRecordset
		 */
		public static function getNewsletterCustomersRecordset($newsletterId, $type, $languageId = null)
		{
			$recordset = new miSqlRecordset('NLSubscribers');
			$recordset->addFilter(new miSqlFilterEqual('NewsletterID', $newsletterId));
			if (!empty($languageId))
				$recordset->addFilter(new miSqlFilterEqual('LanguageID', $languageId));
			$emailField = 'SubscriberEmail';
			
			switch ($type) {
				case self::SEND_TO_CUSTOMERS_NONE:
					return new scPreparedRecordset(array());
				
				case self::SEND_TO_CUSTOMERS_SUBSCRIBED:
					$recordset->addJoinCondition('INNER', 'Customers', 'USING(CustomerID)');
					$recordset->addFilter(new miSqlFilterEqual('SubscriberIsActive', 1));
					break;
				
				case self::SEND_TO_CUSTOMERS_ALL:
					$recordset->addJoinCondition('RIGHT', 'Customers', 'USING(CustomerID)');
					$emailField = 'CustomerEmail';
					break;
				
				default:
					throw new miException(sprintf(_i('Unknown type option "%s" selected for "send to customers".'), $type));
			}
			
			$recordset->setSelectFields(array('DISTINCT(' . $emailField . ') AS ReceiverEmail'));
			return $recordset;
		}
		
		/**
		 * @param int $newsletterId
		 * @param int $type
		 * @return miSqlRecordset
		 */
		public static function getNewsletterSubscribersRecordset($newsletterId, $type, $languageId = null)
		{
			$recordset = new miSqlRecordset('NLSubscribers');
			$recordset->addFilter(new miSqlFilterEqual('NewsletterID', $newsletterId));
			if (!empty($languageId))
				$recordset->addFilter(new miSqlFilterEqual('LanguageID', $languageId));
			
			switch ($type) {
				case self::SEND_TO_SUBSCRIBERS_NONE:
					return new scPreparedRecordset(array());
				
				case self::SEND_TO_SUBSCRIBERS_NORMAL:
					$recordset->addFilter(new scSqlFilterIsNull('CustomerID'));
					$recordset->addFilter(new miSqlFilterEqual('SubscriberIsActive', 1));
					break;
				
				case self::SEND_TO_SUBSCRIBERS_INCLUDING_UNSUBSCRIBED:
					$recordset->addFilter(new scSqlFilterIsNull('CustomerID'));
					break;
				
				default:
					throw new miException(sprintf(_i('Unknown type option "%s" selected for "send to subscribers".'), $type));
			}
			
			$recordset->setSelectFields(array('DISTINCT(SubscriberEmail) AS ReceiverEmail'));
			return $recordset;
		}
		
		public static function getBlastHtmlText(scNewsletter $newsletter, $options = 0, $languageId)
		{
			$html = $newsletter->get('NewsletterXhtmlText', $languageId);
			
			$systemHtml = '';
			if ((bool)($options & scNewsletterOptions::ENABLE_OPEN_TRACKING)) {
				$systemHtml .= self::prepareBlastHtmlOpenTracking($html);
			}
			if ((bool)($options & scNewsletterOptions::ENABLE_CLICK_TRACKING)) {
				$systemHtml .= self::prepareBlastHtmlClickTracking($html);
			}
			if (!(bool)($options & scNewsletterOptions::REMOVE_UNSUBSCRIBE)) {
				$unsubscribeUrl = scPaths::getUrl('module.php?ModuleName=com.summercart.newsletter&action=unsubscribe&NewsletterID=' . $newsletter->get('NewsletterID'));
				
				// These texts will be picked up by the interface texts finder
				// _t('You may unsubscribe by following this link:')
				// _t('Unsubscribe')
				$systemHtml .= '<p>' . scInterfaceTexts::getTranslatedTextInLanguage('You may unsubscribe by following this link:', $languageId) .
					' <a class="noTracking" href="' . miI18N::htmlEscape($unsubscribeUrl) . '">' . scInterfaceTexts::getTranslatedTextInLanguage('Unsubscribe', $languageId) .
					'</a></p>';
			}
			if ((bool)($options & scNewsletterOptions::ADD_BASE_TARGET_BLANK)) {
				$html = '<base target="_blank" />' . $html;
			}
			
			$systemHtml .= self::prepareBlastHtmlAbsoluteUrls($html);
			
			if (!empty($systemHtml)) {
				$systemHtml = '<div id="scNewsletter">' . $systemHtml . '</div>';
				$html = preg_replace('/(<\/body>|$)/i', $systemHtml . '$1', $html, 1);
			}
			
			return $html;
		}
		
		protected static function prepareBlastHtmlOpenTracking(&$html)
		{
			$trackingImageUrl = scPaths::getUrl('module.php?ModuleName=com.summercart.newsletter&action=open&BlastEmailID=%%BLASTEMAILID%%');
			return '<img src="' . miI18N::htmlEscape($trackingImageUrl) . '" alt="" />';
		}
		
		protected static function prepareBlastHtmlClickTracking(&$html)
		{
			if (!preg_match_all('/<a[\s\S]+?<\/a>/', $html, $links)) {
				return;
			}
			
			foreach ($links[0] as $link) {
				list($classes) = self::getAttributeValue('class', $link);
				if ($classes !== false) {
					$classes = explode(' ', $classes);
					if (in_array('noTracking', $classes)) {
						continue;
					}
				}
				
				list($href, $wholeHrefAttribute) = self::getAttributeValue('href', $link);
				if ($href === false) {
					continue;
				}
				
				$trackedAttribute = str_replace($href, miI18N::htmlEscape(self::getClickUrl($href)), $wholeHrefAttribute);
				$trackedLink = str_replace($wholeHrefAttribute, $trackedAttribute, $link);
				$html = str_replace($link, $trackedLink, $html);
			}
			
			return '';
		}
		
		protected static function prepareBlastHtmlAbsoluteUrls(&$html)
		{
			$attributesInTags = scHtmlUtil::getUrlAttributes();
			$attributes = array_keys($attributesInTags);
			
			$attributesRegex = '/' .
					'<([^\s>]+)(?:[^>]+)' . // tag beginning (and matching the tag in [1])
					'(' . // the whole attribute (in [2])
						'(' . implode('|', $attributes) . ')' . // attribute name (in [3])
						'\s*=\s*' . // = sign and spacing
						'(("|\')([^\5]*?)\5|[^\s>]+)' . // attribute value (with quotes in [4], without - in [6])
					')' .
				'/i';
			preg_match_all($attributesRegex, $html, $matches);
			
			foreach ($matches[0] as $key => $wholeMatch) {
				$tagName = $matches[1][$key];
				$attribute = $matches[2][$key];
				$attributeName = $matches[3][$key];
				$attributeValue = $matches[6][$key];
				
				if ($attributeValue == '') {
					continue;
				}
				
				if (!in_array($tagName, $attributesInTags[$attributeName])) {
					// We have found an attribute in a tag where it is not expected.
					// e.g. a "href" attribute in an "img" tag.
					continue;
				}
				
				try {
					$urlType = scUrlUtil::getUrlType($attributeValue);
				} catch (miException $e) {
					// It is obviusly some kind of a URL that we cannot handle.
					continue;
				}
				
				if ($urlType == scUrlUtil::URL_TYPE_ABSOLUTE) {
					// Nothing to do. It is already absolute.
					continue;
				}
				if ($urlType == scUrlUtil::URL_TYPE_HTTPS) {
					// HTTTPS URLs are absolte, so nothing to do again.
					continue;
				}
				
				if ($attributeValue{0} == '#') {
					// It is a fragment link. It is not expected to become absolute.
					continue;
				}
				
				$replacement = $attributeValue; // Keep the original value in $attributeValue
				if ($urlType == scUrlUtil::URL_TYPE_RELATIVE) {
					// Make it relative to the "admin/" folder of the docroot.
					$replacement = scUrlUtil::normalizePath('/admin/' . $replacement);
				}
				
				$replacement = scUrlUtil::convertUrl($replacement, scUrlUtil::URL_TYPE_ABSOLUTE);
				
				$attributeReplaced = str_replace($attributeValue, miI18N::htmlEscape($replacement), $attribute);
				$wholeTagReplaced = str_replace($attribute, $attributeReplaced, $wholeMatch);
				$html = str_replace($wholeMatch, $wholeTagReplaced, $html);
			}
		}
		
		protected static function getClickUrl($clickLocation)
		{
			return scPaths::getUrl('module.php?ModuleName=com.summercart.newsletter&action=click&click=' . urlencode($clickLocation) . '&BlastEmailID=%%BLASTEMAILID%%');
		}
		
		/**
		 * @param string $attribute
		 * @param string $tag
		 * @return array(attributeValue, wholeAttribute)
		 */
		protected static function getAttributeValue($attribute, $tag)
		{
			preg_match('/' . $attribute . '\s*=\s*(("|\')([^\2]*?)\2|[^\s>]+)/i', $tag, $matches);
			if (isset($matches[1])) {
				$value = trim($matches[1],'"\'');
				$wholeAttribute = $matches[0];
				return array($value, $wholeAttribute);
			}
			return array(false, false);
		}
	}
?>