<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scNewsletterModule {
		
		public function __construct()
		{
			$this->setupAutoloading();
		}
		
		protected function setupAutoloading()
		{
			$autoloadClasses = array(
				'scNewsletter' => 'include/Newsletter.php',
				'scNewsletterAdminView' => 'include/NewsletterView.php',
				'scNewsletterAddSubscribersAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterBaseView' => 'include/NewsletterView.php',
				'scNewsletterBlast' => 'include/NewsletterBlast.php',
				'scNewsletterBlastAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterBlastEmail' => 'include/NewsletterBlastEmail.php',
				'scNewsletterBlasts' => 'include/NewsletterBlasts.php',
				'scNewsletterClickAction' => 'include/NewsletterUserActions.php',
				'scNewsletterCsvSubscribersAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterExecAddSubscribersAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterExecBlastAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterExecDeleteSubscriber' => 'include/NewsletterAdminActions.php',
				'scNewsletterExecTestAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterExecUnsubscribeAction' => 'include/NewsletterUserActions.php',
				'scNewsletterGetBlastProgressJsonAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterOpenAction' => 'include/NewsletterUserActions.php',
				'scNewsletterOptions' => 'include/NewsletterOptions.php',
				'scNewslettersTable' => 'include/NewslettersTable.php',
				'scNewsletterShowBlastAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterShowBlastContentsAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterShowContentsAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterShowStatsAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterShowStatsBlastAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterShowSubscribersAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterStartBlastProcessAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterStopBlastAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterSubscribeAction' => 'include/NewsletterUserActions.php',
				'scNewsletterSubscriber' => 'include/NewsletterSubscriber.php',
				'scNewsletterSubscribersTable' => 'include/NewsletterSubscribersTable.php',
				'scNewsletterSubscribersView' => 'include/NewsletterSubscribersView.php',
				'scNewsletterTestAction' => 'include/NewsletterAdminActions.php',
				'scNewsletterUnsubscribeAction' => 'include/NewsletterUserActions.php',
			);
			
			foreach ($autoloadClasses as $className => $fileName) {
				// Relative from the location of miLoader
				miLoader::addClass($className, '../../modules/system/newsletter/' . $fileName);
			}
		}
	}
	
	class scNewsletterAdminModule extends scNewsletterModule implements IAdminModule {
		
		public function getView()
		{
			$action = miGetParamDefault('action', '');
			
			switch ($action) {
				case 'dmExecDeleteSubscriber':
				case 'dmEditSubscriber':
				case 'dmExecEditSubscriber':
					return $this->getSubscribersView();
				
				default:
					return $this->getNewslettersView();
			}
		}
		
		public function initActionHandlers(miPage $page)
		{
			scAdminAuth::singleton()->checkAuthorization(scSections::SECTION_NEWSLETTERS);
			
			$page->getDispatcher()->clearActionHandlers();
			
			// Newsletter
			$page->getDispatcher()->setActionHandler('', 'miListAction');
			$page->getDispatcher()->setActionHandler('dmCreate', 'miCreateAction');
			$page->getDispatcher()->setActionHandler('dmExecCreate', 'miExecCreateAction');
			$page->getDispatcher()->setActionHandler('dmEdit', 'miEditAction');
			$page->getDispatcher()->setActionHandler('dmExecEdit', 'miExecEditAction');
			$page->getDispatcher()->setActionHandler('dmExecDelete', 'miExecDeleteAction');
			$page->getDispatcher()->setActionHandler('dmChangeIsActive', 'scExecChangeIsActiveAction');
			$page->getDispatcher()->setActionHandler('dmShowNewsletter', 'scNewsletterShowContentsAction');
			
			// Test
			$page->getDispatcher()->setActionHandler('dmTest', 'scNewsletterTestAction');
			$page->getDispatcher()->setActionHandler('dmExecTest', 'scNewsletterExecTestAction');
			
			// Blast
			$page->getDispatcher()->setActionHandler('dmBlast', 'scNewsletterBlastAction');
			$page->getDispatcher()->setActionHandler('dmExecBlast', 'scNewsletterExecBlastAction');
			$page->getDispatcher()->setActionHandler('dmGetBlastProgress', 'scNewsletterGetBlastProgressJsonAction');
			$page->getDispatcher()->setActionHandler('dmShowBlast', 'scNewsletterShowBlastAction');
			$page->getDispatcher()->setActionHandler('dmShowBlastHtml', 'scNewsletterShowBlastContentsAction');
			$page->getDispatcher()->setActionHandler('dmStartProcess', 'scNewsletterStartBlastProcessAction');
			$page->getDispatcher()->setActionHandler('dmStopBlast', 'scNewsletterStopBlastAction');
			
			// Stats
			$page->getDispatcher()->setActionHandler('dmShowStats', 'scNewsletterShowStatsAction');
			$page->getDispatcher()->setActionHandler('dmShowStatsBlast', 'scNewsletterShowStatsBlastAction');
			
			// Subscribers
			$page->getDispatcher()->setActionHandler('dmShowSubscribers', 'scNewsletterShowSubscribersAction');
			$page->getDispatcher()->setActionHandler('dmAddSubscribers', 'scNewsletterAddSubscribersAction');
			$page->getDispatcher()->setActionHandler('dmExecAddSubscribers', 'scNewsletterExecAddSubscribersAction');
			$page->getDispatcher()->setActionHandler('dmCsvSubscribers', 'scNewsletterCsvSubscribersAction');
			
			$page->getDispatcher()->setActionHandler('dmExecDeleteSubscriber', 'miExecDeleteAction');
			$page->getDispatcher()->setActionHandler('dmEditSubscriber', 'miEditAction');
			$page->getDispatcher()->setActionHandler('dmExecEditSubscriber', 'miExecEditAction');
		}
		
		protected function getNewslettersView()
		{
			$recordset = new miSqlRecordset('NLNewsletters');
			$recordset->addJoinCondition('LEFT', 'NLNewslettersML',
				sprintf('ON NLNewsletters.NewsletterID = NLNewslettersML.NewsletterID AND NLNewslettersML.LanguageID = %d', scMultiLanguage::getCurrentLanguageId()));
			$recordset->addJoinCondition('LEFT', 'NLSubscribers', 'ON NLNewsletters.NewsletterID = NLSubscribers.NewsletterID');
			$recordset->setSelectFields(array(
				'NLNewsletters.*',
				'NLNewslettersML.NewsletterName',
				'NLNewslettersML.NewsletterSubject',
				'COUNT(SubscriberID) AS NewsletterAllSubscribersCount',
				'(SELECT COUNT(SubscriberID) FROM NLSubscribers' .
					' AS InnerSubscribers WHERE InnerSubscribers.NewsletterID = NLNewsletters.NewsletterID' .
					' AND SubscriberIsActive) AS NewsletterSubscribersCount'
			));
			$recordset->setHavingFilterNames(array(
				'NewsletterSubscribersCount'
			));
			$recordset->setGroupBy('NLNewsletters.NewsletterID');
			
			$table = new scNewslettersTable($recordset);
			
			$moduleParams = new miTableCustomParams($table);
			$moduleParams->setRecordLocatorParams(array('ModuleName' => 'com.summercart.newsletter'));
			
			$view = new scNewsletterAdminView();
			$view->setTable($table);
			
			return $view;
		}
		
		protected function getSubscribersView()
		{
			$view = new scNewsletterSubscribersView();
			
			return $view;
		}
	}
	
	class scNewsletterUserModule extends scNewsletterModule implements IUserModule {
		
		public function getView()
		{
			return new scNewsletterBaseView();
		}
		
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('subscribe', 'scNewsletterSubscribeAction');
			$page->getDispatcher()->setActionHandler('unsubscribe', 'scNewsletterUnsubscribeAction');
			$page->getDispatcher()->setActionHandler('execUnsubscribe', 'scNewsletterExecUnsubscribeAction');
			$page->getDispatcher()->setActionHandler('open', 'scNewsletterOpenAction');
			$page->getDispatcher()->setActionHandler('click', 'scNewsletterClickAction');
			
			$page->setSeo(new scNewsletterSeo());
		}
	}
	
	class scNewsletterCmsBox extends scNewsletterModule implements ICMSBox, IModuleOptions {
		
		protected $_moduleId;
		protected $_boxId;
		
		public function __construct($moduleId, $cmsBoxId)
		{
			parent::__construct();
			$this->_moduleId = $moduleId;
			$this->_boxId = $cmsBoxId;
		}
		
		public function getBoxContents()
		{
			$options = scModuleOptions::getCmsBoxOptions($this->_boxId);
			
			if (!isset($options['NewsletterID']) || empty($options['NewsletterID'])) {
				return '';
			}
			
			$newsletter = new scNewsletter();
			$newsletter->read((int)$options['NewsletterID']);
			
			$parser = new scTemplateParser();
			$parser->readTemplate('modules/system/newsletter/customer/newsletter_box.tmpl');
			$parser->assignArray(miUIUtil::templetizeRows($newsletter->getRow(scMultiLanguage::getCurrentLanguageId())));
			$parser->assign('%%CMSBOXID%%', $this->_boxId);
			
			return $parser->templateParse();
		}
		
		/**
		 * @param miWebForm $webForm
		 * @return scModuleOptionsInfo
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$options = new scModuleOptionsInfo();
			
			$widget = new miSelectWidget($webForm);
			$widget->setFieldName('NewsletterID');
			$widget->setOptions(array(0 => _t('Not selected')) + self::getNewslettersArray());
			$options->addOptionField($widget);
			
			$widget = new miCheckboxWidget($webForm);
			$widget->setFieldName('AdminNotification');
			$options->addOptionField($widget);
			
			$options->setTemplateFile('modules/system/newsletter/admin/newsletter_box_options.tmpl');
			
			return $options;
		}
		
		public static function getNewslettersArray()
		{
			$recordset = new miSqlRecordset('NLNewsletters');
			$recordset->addJoinCondition('LEFT', 'NLNewslettersML',
				sprintf('ON NLNewsletters.NewsletterID = NLNewslettersML.NewsletterID AND NLNewslettersML.LanguageID = %d', scMultiLanguage::getCurrentLanguageId()));
			$recordset->setOrder('NewsletterName');
			$recordset->setSelectFields(array(
				'NLNewsletters.NewsletterID AS NewsletterID',
				'NewsletterName'
			));
			$newsletters = $recordset->getAllRecords();
			return miUtil::getKeyValueArray($newsletters, 'NewsletterID', 'NewsletterName');
		}
	}
	
	class scNewsletterSeo extends scDynamicPageSeo {
		
		public function __construct()
		{
			parent::__construct('module.php?ModuleName=com.summercart.newsletter');
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			$this->_pageTitle = _t('Newsletter');
		}
	}
?>
