<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductComparison {
		
		const SESSION_KEY = 'ProductComparisonProducts';
		const MAX_PRODUCTS = 3;
		
		protected $_productIds = array();
		
		protected function __construct()
		{
		}
		
		public function __destruct()
		{
			$_SESSION[self::SESSION_KEY] = serialize($this);
		}

		/**
		 * @return scProductComparison
		 */
		public static function singleton()
		{
			static $instance = null;
			
			if (is_null($instance)) {
				if (array_key_exists(self::SESSION_KEY, $_SESSION)) {
					$instance = unserialize($_SESSION[self::SESSION_KEY]);
				} else {
					$instance = new scProductComparison();
				}
			}
			
			return $instance;
		}
		
		/**
		 * @return string
		 */
		public static function getCmsBoxContents()
		{
			$productIds = scProductComparison::singleton()->getProductIds();
			$products = array();
			if (!empty($productIds)) {
				$products = miStaticDBUtil::execSelect(
					'SELECT *, Products.ProductID as ProductID FROM Products ' .
					'LEFT JOIN ProductsML ON(Products.ProductID=ProductsML.ProductID AND LanguageID = ?) ' .
					'LEFT JOIN ProductImages ON (ProductImages.ProductID = Products.ProductID AND ProductImageIsDefault<>0) ' .
					'LEFT JOIN ProductVariants ON (ProductVariants.ProductID = Products.ProductID AND ProductVariantIsDefault<>0) ' .
					'WHERE ProductIsActive ' .
					'AND Products.ProductID IN (' . implode(', ', array_map('intval', $productIds)) . ')',
					array(
						scMultiLanguage::getCurrentLanguageId()
					)
				);
			}
			
			$isComparisonAvailable = new miTemplateParserSectionInfo('isComparisonAvailable', 0);
			if (count($products) >= 2) {
				$isComparisonAvailable->setIterationsCount(1);
				
				$products = miUtil::getArrayWithKeys($products, 'ProductID');
				
				// Reorder products in the order they were added to the list.
				$ordered = array();
				foreach ($productIds as $productId) {
					if (isset($products[$productId])) {
						$ordered[] = $products[$productId];
					}
				}
				$products = $ordered;
				unset($ordered);
			}
			
			$listingItems = scProductService::getListingItemsFromArray($products);
			if (!empty($listingItems)) {
				scProductService::replaceOutOfStockVariants($listingItems);
			}
			
			$options = scProductListingService::OPTION_NO_MODULES;
			$productsSection = scProductListingService::singleton()->getSection('products', $listingItems, scProductService::VIEW_LIST, $options);
			$productsSection->addSubsection($isComparisonAvailable);
			
			$t = new scTemplateParser();
			$t->readTemplate('modules/product_comparison/product_comparison_box.tmpl');
			$t->addSectionInfo($productsSection);
			
			if (miGetParamDefault('load', '') != 'table')
				$t->addSectionInfo(new miTemplateParserSectionInfo('static', 1));
			
			return $t->templateParse();
		}
		
		public static function getProductDetailsBoxContents($productId, $productClassId = 0)
		{
			if (is_null($productClassId)) {
				return '';
			}
			
			if (in_array($productId, scProductComparison::singleton()->getProductIds())) {
				$section = new miTemplateParserSectionInfo('productIsInCompareList', 1);
			} else {
				$section = new miTemplateParserSectionInfo('productIsComparable', 1);
			}
			
			$t = new scTemplateParser();
			$t->readTemplate('modules/product_comparison/product_comparison_details_box.tmpl');
			$t->addSectionInfo($section);
			$t->assignArray(array(
				'%%PRODUCTID%%' => $productId,
				'%%PRODUCTCLASSID%%' => $productClassId,
			));
			return $t->templateParse();
		}
				
		public function addProductId($productId)
		{
			$key = array_search($productId, $this->_productIds);
			if ($key !== false) {
				return;
			}
			
			$maxProducts = (int)scSettings::getSettingValue(scSettings::SETTING_PRODUCT_COMPARISON_MAX_PRODUCTS);
			if ($maxProducts == 0) {
				$maxProducts = (int)self::MAX_PRODUCTS;
			}
			
			if (count($this->_productIds) >= $maxProducts) {
				array_shift($this->_productIds);
			}
			
			$this->_productIds[] = $productId;
		}
		
		public function removeProductId($productId)
		{
			$key = array_search($productId, $this->_productIds);
			if ($key !== false)
				unset($this->_productIds[$key]);
		}
		
		/**
		 * @return array
		 */
		public function getProductIds()
		{
			return $this->_productIds;
		}
	}
	
	class scProductComparisonBoxTable extends miTable {
		
		public function addUiElements(&$rows)
		{
			scFriendlyUrl::initFriendlyUrls($rows, scFriendlyUrl::ROW_TYPE_PRODUCTS);
		}
	}
	
	class scProductComparisonBox implements ICMSBox {
		
		protected $_moduleId;
		protected $_cmsBoxId;
		
		public function __construct($moduleId, $cmsBoxId)
		{
			$this->_moduleId = $moduleId;
			$this->_cmsBoxId = $cmsBoxId;
		}
		
		public function getBoxContents()
		{
			return scProductComparison::getCmsBoxContents();
		}
	}
	
	class scProductComparisonProductDetails implements IProductDetails {
		
		public function getProductDetailsHtml(scProductSL $product, miTemplateParserSectionInfo $productSection)
		{
			$productSection->addValues(array(
				'%%PRODUCT_COMPARISON_DETAILS_BOX%%' => scProductComparison::getProductDetailsBoxContents($product->get('ProductID'), $product->get('ProductClassID'))
			));
		}
	}
	
	class scProductComparisonUserModule implements IUserModule {
		
		/**
		 * Initializes the action handlers for the user module
		 *
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->setActionHandler('', 'scProductComparisonCompareAction');
			$page->getDispatcher()->setActionHandler('dmAddProduct', 'scProductComparisonAddProductAction');
			$page->getDispatcher()->setActionHandler('dmRemoveProduct', 'scProductComparisonRemoveProductAction');
			$page->getDispatcher()->setActionHandler('dmAjaxAddProduct', 'scProductComparisonAjaxAddProductAction');
			$page->getDispatcher()->setActionHandler('dmAjaxRemoveProduct', 'scProductComparisonAjaxRemoveProductAction');
			$page->getDispatcher()->setActionHandler('dmAjaxAddListingProduct', 'scProductComparisonAjaxAddListingProductAction');
			$page->getDispatcher()->setActionHandler('dmAjaxRemoveListingProduct', 'scProductComparisonAjaxRemoveListingProductAction');
			$page->getDispatcher()->setActionHandler('dmRemoveAndShow', 'scProductComparisonRemoveAndShowAction');
			
			$page->setSeo(new scProductComparisonSeo());
		}
		
		/**
		 * Returns the current view that should be displayed
		 * 
		 * @return miView
		 */
		public function getView()
		{
			return new miDefaultView();
		}
	}
	
	class scProductComparisonAction extends miAction {
		
		protected function doProductAction($action, $productId)
		{
			switch ($action) {
				case 'add':
					scProductComparison::singleton()->addProductId($productId);
					break;
				case 'remove':
					scProductComparison::singleton()->removeProductId($productId);
					break;
				default:
					throw new miException(sprintf(_i('Unknown action "%s".'), $action));
			}
		}
	}
	
	class scProductComparisonAddProductAction extends scProductComparisonAction {
		
		protected $_doProductAction = 'add';
		
		public function doAction()
		{
			try {
				$productId = (int)miGetParamDefault('ProductID', 0);
				if ($productId <= 0) {
					throw new miException(sprintf(_t('Unable to add product "%s" for comparison.'), $productId));
				}
				
				$this->doProductAction($this->_doProductAction, $productId);
				$redirectUrl = scPaths::getAbsoluteUrl(scFriendlyUrl::getUrl(scFriendlyUrl::ROW_TYPE_PRODUCTS, $productId));
			} catch (miException $e) {
				$redirectUrl = scPaths::getAbsoluteUrl('error.php?msg=' . urlencode($e->getMessage()));
			}
			
			$command = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $redirectUrl);
			$this->_view->addControllerCommand($command);
			return true;
		}
	}
	
	class scProductComparisonRemoveProductAction extends scProductComparisonAddProductAction {
		
		protected $_doProductAction = 'remove';
	}
	
	class scProductComparisonAjaxAddProductAction extends scProductComparisonAction {
		
		protected $_doProductAction = 'add';
		
		public function doAction()
		{
			$response = new scProductComparisonAjaxResponse();
			
			try {
				$productId = (int)miGetParamDefault('ProductID', 0);
				if ($productId <= 0) {
					throw new miException(sprintf(_t('Unable to add product "%s" for comparison.'), $productId));
				}
				
				$this->doProductAction($this->_doProductAction, $productId);
				$response->set(scProductComparisonAjaxResponse::BOX, scProductComparison::getCmsBoxContents());
				$response->set(scProductComparisonAjaxResponse::PRODUCT, scProductComparison::getProductDetailsBoxContents((int)miGetParamDefault('ProductID', 0)));
				$response->set(scProductComparisonAjaxResponse::PRODUCT_ID, $productId);
				$response->set(scProductComparisonAjaxResponse::PRODUCT_ACTION, $this->_doProductAction);
				// Must use array_values(). Otherwise the JSON encoder might encode as hash instead of array
				$response->set(scProductComparisonAjaxResponse::COMPARE_LIST, array_values(scProductComparison::singleton()->getProductIds()));
			} catch (miException $e) {
				$response->set(scProductComparisonAjaxResponse::ERROR, $e->getMessage());
			}
			
			$command = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $response->__toString());
			$this->_view->addControllerCommand($command);
			return true;
		}
	}
	
	
	class scProductComparisonAjaxAddListingProductAction extends scProductComparisonAction {
		
		protected $_doProductAction = 'add';
		
		public function doAction()
		{
			$response = new scProductComparisonAjaxResponse();
			
			try {
				$productId = (int)miGetParamDefault('ProductID', 0);
				if ($productId <= 0) {
					throw new miException(sprintf(_t('Unable to add product "%s" for comparison.'), $productId));
				}
				
				$this->doProductAction($this->_doProductAction, $productId);
				$response->set(scProductComparisonAjaxResponse::BOX, scProductComparison::getCmsBoxContents());
				$response->set(scProductComparisonAjaxResponse::PRODUCT_ID, $productId);
				$response->set(scProductComparisonAjaxResponse::PRODUCT_ACTION, $this->_doProductAction);
				// Must use array_values(). Otherwise the JSON encoder might encode as hash instead of array
				$response->set(scProductComparisonAjaxResponse::COMPARE_LIST, array_values(scProductComparison::singleton()->getProductIds()));
			} catch (miException $e) {
				$response->set(scProductComparisonAjaxResponse::ERROR, $e->getMessage());
			}
			
			$command = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $response->__toString());
			$this->_view->addControllerCommand($command);
			return true;
		}
	}
	
	class scProductComparisonAjaxRemoveProductAction extends scProductComparisonAjaxAddProductAction {
		
		protected $_doProductAction = 'remove';
	}
	
	class scProductComparisonAjaxRemoveListingProductAction extends scProductComparisonAjaxAddListingProductAction {
		
		protected $_doProductAction = 'remove';
	}
	
	class scProductComparisonCompareAction extends miAction {
		
		public function doAction()
		{
			$products = $this->getProducts();
			
			$hasProductsSection = new miTemplateParserSectionInfo('hasProducts');
			$hasNoProductsSection = new miTemplateParserSectionInfo('hasNoProducts');
			
			if (!empty($products)) {
				$sections = $this->getComparisonSections($products);
				$hasProductsSection->setIterationsCount(1);
				foreach ($sections as $section) {
					$hasProductsSection->addSubsection($section);
				}
				
				$hasNoProductsSection->setIterationsCount(0);
			} else {
				$sections = array(new miTemplateParserSectionInfo('nothingToCompare', 1));
				$hasProductsSection->setIterationsCount(0);
				
				$hasNoProductsSection->setIterationsCount(1);
			}
			
			$t = new scTemplateParser();
			$t->readTemplate('modules/product_comparison/product_comparison.tmpl');
			$t->addSectionInfo($hasProductsSection);
			$t->addSectionInfo($hasNoProductsSection);
			$t->assignArray(scDynamicPages::getTemplateVars('module.php?ModuleName=com.summercart.productcomparison'));
			$html = $t->templateParse();
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			return true;
 		}
 		
 		/**
 		 * @return array
 		 */
 		protected function getComparisonSections($listingItems)
 		{
			$hasIncompatibleProductsSection = new miTemplateParserSectionInfo('hasIncompatibleProducts', 0);
			
			// Get all product class ids.
			$productClassIds = array();
			foreach ($listingItems as $listingItem) {
				$productClassIds[] = $listingItem->getProductClassId();
			}
			
			// Count how many products are there in the list for each class.
			$productClassIds = array_count_values($productClassIds);
			if (count($productClassIds) > 1) {
				// Sort them setting the class with the most values as first.
				arsort($productClassIds);
				$hasIncompatibleProductsSection->setIterationsCount(1);
			}
			
			// PHP.net does not say anything about resetting the internal pointer after sorting.
			reset($productClassIds);
			// Get the first class id (the one with the most products)
			$comparisonProductClassId = key($productClassIds);
			
			$productsSection = scProductListingService::singleton()->getSection('products', $listingItems);
			$productsSection->addSubsectionsArray('productIsIncompatible', $this->getIsIncompatibleSectionsArray($listingItems, $comparisonProductClassId));
 			
 			list($attributeNames, $attributeValues) = $this->getAttributes($listingItems, $comparisonProductClassId);
 			
 			$attributesSection = new miTemplateParserSectionInfo('attributes', count($attributeNames), array('%%ATTRIBUTENAME%%' => array_values($attributeNames)));
			
			$subsections = array();
			foreach ($attributeNames as $sort => $attributeName) {
				$productIsIncompatibleSectionArray = array();
				$multiValueSectionArray = array();
				foreach ($listingItems as $listingItem) {
					$productId = $listingItem->getProductId();
					
					$productIsIncompatibleSection = new miTemplateParserSectionInfo('productIsIncompatible');
					$productIsIncompatibleSection->setIterationsCount((int)(bool)($listingItem->getProductClassId() != $comparisonProductClassId));
					
					$multiValueSection = new miTemplateParserSectionInfo('multiValue', 0);
					if (isset($attributeValues[$sort][$productId])) {
						$productAttributeValue = $attributeValues[$sort][$productId];
						
						if (isset($productAttributeValue['AttributeMultiValue']) && !empty($productAttributeValue['AttributeMultiValue'])) {
							$multiValueSection->setIterationsCount(count($productAttributeValue['AttributeMultiValue']));
							$values = array('AttributeMultiValue' => array_values($productAttributeValue['AttributeMultiValue']));
							$multiValueSection->setValues(miUIUtil::templetizeAndEscapeRows($values));
						}
					} else {
						$productAttributeValue = array('AttributeID' => '', 'AttributeName' => '', 'AttributeValue' => '');
					}
					$productAttributeValue['HTML_AttributeValue'] = $productAttributeValue['AttributeValue'];
					
					$productAttributeValue['AttributeMultiValue'] = '';
					$attributeValues[$sort][$productId] = $productAttributeValue;
					$multiValueSectionArray[] = $multiValueSection;
					$productIsIncompatibleSectionArray[] = $productIsIncompatibleSection;
				}
				
				// Ensure that attributes are ordered correctly.
				$reordered = array();
				foreach ($listingItems as $listingItem) {
					$reordered[] = $attributeValues[$sort][$listingItem->getProductId()];
				}
				$attributeValues[$sort] = $reordered;
				
				$subsectionValues = miUIUtil::templetizeRows(miUIUtil::transposeRows($attributeValues[$sort]));
				$subsection = new miTemplateParserSectionInfo('productAttributeValues', count($attributeValues[$sort]), $subsectionValues);
				$subsection->addSubsectionsArray('multiValue', $multiValueSectionArray);
				$subsection->addSubsectionsArray('productIsIncompatible', $productIsIncompatibleSectionArray);
				$subsections[] = $subsection;
			}
			
			$attributesSection->addSubsectionsArray('productAttributeValues', $subsections);
			return array(
				$hasIncompatibleProductsSection,
				$productsSection,
				$attributesSection
			);
 		}
 		
		/**
		 * @return array
		 */
 		protected function getProducts()
		{
			$productIds = scProductComparison::singleton()->getProductIds();
			
			if (count($productIds) == 0) {
				return array();
			}
			
			$recordset = new scSqlRecordsetML('Products', 'ProductID');
			$recordset->addJoinCondition('LEFT', 'ProductImages', 'ON ProductImages.ProductID = Products.ProductID AND ProductImageIsDefault<>0');
			$recordset->addJoinCondition('LEFT', 'ProductVariants', 'ON (ProductVariants.ProductID = Products.ProductID AND ProductVariantIsDefault<>0)');
			$recordset->setSelectFields(array(
				'*',
				'Products.ProductID AS ProductID',
			));
			$recordset->addFilter(new miSqlFilterIn('Products.ProductID', $productIds));
			$recordset->setLanguageId(scMultiLanguage::getCurrentLanguageId());
			$products = $recordset->getAllRecords();
			
			if (count($products) == 0) {
				return array();
			}
			
			$products = miUtil::getArrayWithKeys($products, 'ProductID');
			
			// Reorder products in the order they were added to the list.
			$ordered = array();
			foreach ($productIds as $productId) {
				if (isset($products[$productId])) {
					$ordered[] = $products[$productId];
				}
			}
			unset($products);
			
			return scProductService::getListingItemsFromArray($ordered);
		}
		
		/**
		 * @return array
		 */
		protected function getAttributes($listingItems, $comparisonProductClassId)
		{
			$attributeNames = array();
			$attributeValues = array();
			foreach ($listingItems as $listingItem) {
				$attributes = scProductService::getProductAttributes($listingItem->getProductId(), $comparisonProductClassId, scProductAttributes::GET_SYSTEM_ATTRIBUTES_ALWAYS);
				foreach ($attributes as $attribute) {
					$sort = $this->map($attribute['AttributeID']);
					$attributeNames[$sort] = $attribute['AttributeName'];
					$attributeValues[$sort][$listingItem->getProductId()] = $attribute;
				}
			}
			ksort($attributeNames, SORT_NUMERIC);
			ksort($attributeValues, SORT_NUMERIC);
			
			return array($attributeNames, $attributeValues);
		}
		
		/**
		 * For each given $attributeId return a unique sort number.
		 * In case a non-numeric $attributeId is passed, its value is returned.
		 */
		protected function map($attributeId)
		{
			static $map = array();
			static $num = 1;
			if (!isset($map[$attributeId])) {
				if (is_numeric($attributeId)) {
					$map[$attributeId] = $num;
					$num += 1;
				} else {
					$map[$attributeId] = $attributeId;
				}
			}
			return $map[$attributeId];
		}
		
		/**
		 * @param IProduuctListingItem[] $listingItems
		 * @param int $comparisonProductClassId
		 * @return array
		 */
		protected function getIsIncompatibleSectionsArray($listingItems, $comparisonProductClassId)
		{
			$compatible = new miTemplateParserSectionInfo('productIsIncompatible');
			$compatible->setIterationsCount(0);
			$incompatible = clone $compatible;
			$incompatible->setIterationsCount(1);
 			
			$productIsIncompatible = array();
 			foreach ($listingItems as $listingItem) {
				if ($listingItem->getProductClassId() != $comparisonProductClassId) {
					$productIsIncompatible[] = $incompatible;
				} else {
					$productIsIncompatible[] = $compatible;
				}
			}
			
			return $productIsIncompatible;
		}
	}
	
	class scProductComparisonRemoveAndShowAction extends scProductComparisonCompareAction {
		
		public function doAction()
		{
			$productId = (int)miGetParamDefault('ProductID', 0);
			if ($productId <= 0) {
				throw new miException(sprintf(_t('Unable to add product "%s" for comparison.'), $productId));
			}
			
			scProductComparison::singleton()->removeProductId($productId);
			
			return parent::doAction();
		}
	}
	
	class scProductComparisonAjaxResponse {
		
		const ERROR = 'error';
		const BOX = 'box';
		const PRODUCT = 'product';
		const PRODUCT_ID = 'productId';
		const PRODUCT_ACTION = 'productAction';
		const COMPARE_LIST = 'compareList';
		
		protected $_data = array(
			self::ERROR => '',
			self::BOX => '',
			self::PRODUCT => '',
			self::PRODUCT_ID => '',
			self::PRODUCT_ACTION => '',
			self::COMPARE_LIST => array(),
		);
		
		public function __toString()
		{
			return scJson::jsonEncode($this->_data);
		}
		
		public function set($key, $value)
		{
			if (!isset($this->_data[$key])) {
				throw new miException(sprintf(_i('Unknown field "%s".'), $key));
			}
			
			$this->_data[$key] = $value;
		}
	}
	
	class scProductComparisonSeo extends scDynamicPageSeo {
		
		public function __construct()
		{
			parent::__construct('module.php?ModuleName=com.summercart.productcomparison');
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			$this->_pageTitle = _t('Product Comparison');
		}
		
		public function isMultilanguage()
		{
			return true;
		}
		
		public function getUrlParams()
		{
			return array(
				scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_DYNAMIC_PAGE,
				scSeoUrlTypes::PAGE_ID => scFriendlyUrl::DYN_PAGE_PRODUCT_COMPARISON,
			);
		}
	}
?>