<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductsFeedAdminModule implements IAdminModule {
		
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('', 'miEditAction');
			$page->getDispatcher()->setActionHandler('dmExecEdit', 'scProductsFeedAdminExecEditAction');
		}
		
		public function getView()
		{
			return new scProductsFeedAdminView();
		}
	}
	
	class scProductsFeedAdminView extends scView {
		
		protected $_templates = array(
			self::TEMPLATE_ID_EDIT => 'modules/system/products_feed/products_feed_options.tmpl',
		);
		
		protected $_dataFields = array(
			array(
				'field' => 'miCheckboxWidget',
				'data' => 'RequireKey',
			),
			array(
				'field' => 'miTextWidget',
				'data' => 'SecretKey',
			),
		);
		
		/**
		 * @var miWebForm $form
		 */
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$languages = scMultiLanguage::singleton()->getActiveLanguages();
			$languages = miUtil::getKeyValueArray($languages, 'LanguageCode', 'LanguageName');
			if (count($languages) > 0) {
				$languages = array('' => _t('Select language')) + $languages;
			}
			
			$currencies = array_combine(scCurrencyUtil::getActiveCurrencyCodes(), scCurrencyUtil::getActiveCurrencies());
			if (count($currencies) > 0) {
				$currencies = array('' => _t('Select currency')) + $currencies;
			}
			
			$this->addMainPageElements(array(
				'%%STORE_ROOT%%' => scPaths::getUrl(),
				'%%HTML_LANGUAGECODE_DROPDOWN%%' => scUIUtil::getConstDropdown('LanguageCode', null, $languages),
				'%%HTML_CURRENCYCODE_DROPDOWN%%' => scUIUtil::getConstDropdown('CurrencyCode', null, $currencies),
			));
		}
		
		protected function createDomainObject()
		{
			$record = new scProductsFeedModuleOptions('ModuleOptions', 'ModuleOptionName', 'ModuleOptionValue');
			return new miDefaultDomainObject($record);
		}
	}
	
	class scProductsFeedModuleOptions extends miPropertiesRecord {
		
		public function readPK($value)
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.products_feed');
			$query = 'SELECT * FROM ' . $this->_table . ' WHERE ModuleID = ?';
			$rows = miStaticDBUtil::execSelect($query, array($moduleId));
			
			foreach ($rows as $key => $row) {
				$this->_row[$row[$this->_propertyField]] = $row[$this->_valueField];
			}
		}
		
		public function update()
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.products_feed');
			foreach ($this->_row as $property => $value) {
				$row = array($this->_valueField => $value);
				miStaticDBUtil::execUpdate($this->_table, $row, array('ModuleID', $this->_propertyField), array($moduleId, $property));
			}
		}
	}
	
	class scProductsFeedAdminExecEditAction extends miExecEditAction {
		
		/**
		 * @param array $viewMapperControllerResult
		 * @return true
		 */
		protected function onSuccess($viewMapperControllerResult = null)
		{
			$msg = sprintf($this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG'), _t('Products Feed'));
			$loc = 'plugins.php?' . $this->_view->getRedirectParams($msg, miMessage::MSG_TYPE_INFO);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->_view->addControllerCommand($cmd);
			return true;
		}
	}
?>