<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductsFeedUserModule implements IUserModule {
		
		public function __construct()
		{
			miSetParam('load', 'table');
			// Allow the script to work as long as the "browser" requests
			set_time_limit(0);
			ignore_user_abort(false);
		}
		
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('', 'scProductsFeedAction');
		}
		
		public function getView()
		{
			return new miDefaultView();
		}
	}
	
	class scProductsFeedAction extends miAction {
		
		const CATEGORY_BRANCH_SEPARATOR = '|';
		const PROCESS_RECORDS_AT_ONCE = 500;
		
		/**
		 * @var int
		 */
		protected $_offset = 0;
		
		/**
		 * Limit the number of products to be returned 
		 * @var int|false
		 */
		protected $_numProducts = false;
		
		/**
		 * @var bool
		 */
		protected $_applyTax = false;
		
		/**
		 * @var scCurrency
		 */
		protected $_currency = null;
		
		/**
		 * Mark if this is the last resultset
		 * @var bool
		 */
		protected $_lastResultset = false;
		
		/**
		 * Products data
		 * @see scProductsFeedAction::loadNextProducts()
		 * @var array
		 */
		protected $_options = null;
		protected $_products = array();
		protected $_additionalImagesByProduct = array();
		protected $_relatedProductsByProduct = array();
		protected $_bundleProductsByProduct = array();
		protected $_additionalCategoriesByProduct = array();
		protected $_allCategories;
		protected $_categoryNames = array();
		protected $_categoryBranches = array();
		
		public function doAction()
		{
			if (ob_get_length() !== false) {
				ob_end_clean();
			}
			header('Content-Encoding: none');
			header('Content-type: text/xml; charset=utf-8');
			
			// Make sure scOrderTotalDiscount is loaded. That includes scDiscountCalculator.
			scModulesFactory::getActiveOrderTotalModules(IOrderTotalModule::ORDER_TOTAL_TYPE_DISCOUNT);
			
			echo '<?xml version="1.0" encoding="UTF-8"?>';
			echo '<Response xmlns="http://schemas.summercart.com/products_feed/v2">';
			echo "\n"; 
			try {
				$this->applyModuleOptions();
				
				echo '<Store>';
				echo 	'<StoreName>' . scXml::escape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME)) . '</StoreName>';
				echo	'<StoreUrl>' . scXml::escape(scPaths::getAbsoluteUrl()) . '</StoreUrl>';
				echo '</Store>';
				echo "\n";
				echo '<FeedInfo>';
				echo	'<LanguageCode>' . scXml::escape(scMultiLanguage::getCurrentLanguageCode()) . '</LanguageCode>';
				echo	'<CurrencyCode>' . scXml::escape($this->_currency->getCurrencyCode()) . '</CurrencyCode>';
				echo	'<ContentEncoding>UTF-8</ContentEncoding>';
				echo	'<Generator>Summer Cart</Generator>';
				echo	'<GenerationTimestamp>' . scXml::escape(date('c', time())) . '</GenerationTimestamp>';
				echo	'<CategoriesSeparator>' . scXml::escape(self::CATEGORY_BRANCH_SEPARATOR) . '</CategoriesSeparator>';
				echo'</FeedInfo>';
				
				$this->loadAllCategories();
				$firstLoad = $this->loadNextProducts();
				
				if ($firstLoad) {
					echo '<Products>';
					do {
						$this->showLoadedProducts();
					} while ($this->loadNextProducts());
					echo '</Products>';
				}
				
				$this->printExtraXml();
				
				$success = true;
			} catch (miException $exception) {
				$success = false;
				echo '<error>' .
						'<message>' . scXml::escape($exception->getMessage()) . '</message>' .
					'</error>';
			}
			
			echo '</Response>';
			// return $success;
			//// Cannot return the success state because the UserDisplayController
			//// handles ob incorrectly. Should die instead.
			die;
		}
		
		/**
		 * Applies the module options like 
		 * language, currency and checks SecretKey
		 */
		protected function applyModuleOptions()
		{
			$moduleName = 'com.summercart.products_feed';
			$moduleId = scModulesFactory::getModuleIdByName($moduleName);
			
			// get options
			$this->_options = scModuleOptions::getModuleOptions($moduleId);
			$this->_options['ApplyTax'] = 1;
			
			// check the key 
			$key = miGetParamDefault('ProductsFeedKey', '');
			if ($this->_options['RequireKey'] == 1 && $this->_options['SecretKey'] != $key) {
				throw new miException(_t('Invalid Products Feed Key.'));
			}
			
			// check the tax settings 
			if ($this->_options['ApplyTax'] == 1) {
				$this->_applyTax = true;
			}
			// validate and set the language
			$languageCode = strtoupper(miGetParamDefault('LanguageCode', ''));
			$languageCodes = scMultiLanguage::getActiveLanguageCodes(); 
			if (!in_array($languageCode, $languageCodes)) {
				throw new miException(_i('Invalid language code.'));
			}
			$languageId = scMultiLanguage::singleton()->getLanguageIdByCode($languageCode);
			scMultiLanguage::singleton()->setCurrentLanguageId($languageId);
			
			// validate and set the currency
			$currencyCode = strtoupper(miGetParamDefault('CurrencyCode', ''));
			$currencyCodes = scCurrencyUtil::getActiveCurrencyCodes();
			if (!in_array($currencyCode, $currencyCodes)) {
				throw new miException(_i('Invalid currency code.'));
			}
			$currencyId = array_search($currencyCode, $currencyCodes);
			$this->_currency = new scCurrency($currencyId);
			
			// Limit the number of products to return
			$this->_numProducts = miGetParamDefault('NumProducts', false);
			$this->_offset = miGetParamDefault('ProductsOffset', 0);
		}
		
		protected function showLoadedProducts()
		{
			try {
				if (class_exists('scDiscountCalculator', false))
					$discountCalculator = new scDiscountCalculator(scDiscountCalculator::MODE_FEED);
			} catch (miException $e) {
			}
			
			$toShow = count($this->_products);
			for ($i = 0; $i < $toShow; $i++) {
				$productId = $this->_products[$i]['ProductID'];
				
				// Pass the additional categories to save calculations
				$additionalCategories = isset($this->_additionalCategoriesByProduct[$productId]) ? $this->_additionalCategoriesByProduct[$productId] : array();
				$additionalCategories[] = $this->_products[$i]['CategoryID'];
				$product = new scProductFeedProductSL($additionalCategories);
				$product->setRow($this->_products[$i]);
				
				// Price
				$priceAmount = $product->getPrice()->getAmount();
				$discountedPriceAmount = clone $priceAmount;
				if (isset($discountCalculator)) {
					$discountAmount = $discountCalculator->getProductDiscount($product);
					if ($discountAmount->getAmount()->get() != 0) {
						$discountedPriceAmount = scMultiCurrencyAmountMath::add($priceAmount, $discountAmount);
					}
				}
				if ($this->_applyTax) {
					$productPrice = $this->getTaxedPrice($discountedPriceAmount->get($this->_currency), $product->getProductId());
					$listingPrice = $this->getTaxedPrice($priceAmount->get($this->_currency), $product->getProductId());
				} else {
					$productPrice = $discountedPriceAmount->get($this->_currency);
					$listingPrice = $priceAmount->get($this->_currency);
				}
				
				// Weight
				$weight = $product->getWeight();
				$weightUnits = ($weight->getWeightUnits() == scWeightUnits::WEIGHT_UNIT_KGS) ? 'Kgs' : 'Lbs';
				$imageData = $product->getImageInfo()->getDataArray();
				
				// Manufacturer
				if ($product->get('ManufacturerID')) {
					$manufacturerId = $product->get('ManufacturerID');
					$manufacturerName = scXml::escape($product->get('ManufacturerName'));
				} else {
					$manufacturerId = '';
					$manufacturerName = '';
				}
				
				echo '<Product>';
				
				echo '<ProductID>' . $product->get('ProductID') . '</ProductID>';				echo '<ProductCode>' . scXml::escape($product->get('ProductCode')) . '</ProductCode>';				echo '<ProductName html="' . ($product->get('ProductNameXHTML') ? 'true' : 'false') . '">' . scXml::escape($product->get('ProductName')) . '</ProductName>';				echo '<ProductDescription html="' . ($product->get('ProductDescriptionXHTML') ? 'true' : 'false') . '">' . $this->getCData($product->get('ProductDescription')) . '</ProductDescription>';
				echo '<ProductDetailedDescription html="' . ($product->get('ProductDetailedDescriptionXHTML') ? 'true' : 'false') . '">' . $this->getCData($product->get('ProductDetailedDescription')) . '</ProductDetailedDescription>';
				echo '<ProductQuantity>' . $product->getQuantityInStock() . '</ProductQuantity>';				echo '<ProductPrice currencycode="' . $this->_currency->getCurrencyCode() . '" listingprice="' . $listingPrice . '">' . $productPrice . '</ProductPrice>';				echo '<ProductWeight units="' . $weightUnits . '">' . $weight->getWeight() . '</ProductWeight>';				echo '<ProductIsEgood>' . ($product->get('ProductIsEgood') ? 'true' : 'false') . '</ProductIsEgood>';				echo '<ProductUrl>' . scXml::escape(scUrlUtil::convertUrl($product->getUrl(), scUrlUtil::URL_TYPE_ABSOLUTE)) . '</ProductUrl>';				echo '<ProductCreatedTimestamp>' . scXml::escape(date('c', $product->get('ProductCreatedTimestamp'))) . '</ProductCreatedTimestamp>';				echo '<ProductImage>';				echo 	'<ProductThumbnailImage>' . scXml::escape($imageData['ImageThumb']) . '</ProductThumbnailImage>';				echo 	'<ProductStandardImage>' . scXml::escape($imageData['Image']) . '</ProductStandardImage>';				echo 	'<ProductLargeImage>' . scXml::escape($imageData['ImageLarge']) . '</ProductLargeImage>';				echo '</ProductImage>';				echo '<Category>';				echo 	'<CategoryID>' . $product->get('CategoryID') . '</CategoryID>';				echo 	'<CategoryName>' . scXml::escape($product->get('CategoryName')) . '</CategoryName>';				echo 	'<CategoryBranch>' . scXml::escape($this->getCategoryBranch($product->get('CategoryID'))) . '</CategoryBranch>';				echo '</Category>';				echo '<ManufacturerID>' . $manufacturerId . '</ManufacturerID>';				echo '<ManufacturerName>' . $manufacturerName . '</ManufacturerName>';
				
				$this->showAdditionalImages($product);
				$this->showBundleProducts($product);
				$this->showRelatedProducts($product);
				$this->showAdditionalCategories($product);
				
				echo '</Product>';
				echo "\n";
				if (ob_get_length()) {
					ob_flush();
				}
				flush();
				unset($product);
			}
		}
		
		protected function getCData($string)
		{
			// Escape CDATA that may be contained in $string
			$string = str_replace(']]>', ']]]]><![CDATA[>', $string);
			
			return '<![CDATA[' . $string . ']]>';
		}
		
		protected function showAdditionalImages(scProductSL $product)
		{
			$productId = $product->getProductId();
			if (!isset($this->_additionalImagesByProduct[$productId])) {
				return;
			}
			
			$productImages = $this->_additionalImagesByProduct[$productId];
			if (empty($productImages)) {
				return;
			}
			
			// Get the product images. Remove any non-existent.
			$imagesData = array();
			$storeRootDir = scPaths::getStoreRootDir();
			$baseUrl = scPaths::getAbsoluteUrl();
			foreach ($productImages as $image) {
				if (!file_exists($storeRootDir . $image['ProductImageThumb']) or !file_exists($storeRootDir . $image['ProductImage']) or !file_exists($storeRootDir . $image['ProductImageLarge']))
					continue;
				
				$imagesData[] = array(
					'ImageThumb' => $baseUrl . $image['ProductImageThumb'],
					'Image' => $baseUrl . $image['ProductImage'],
					'ImageLarge' => $baseUrl . $image['ProductImageLarge']
				);
			}
			unset($productImages);
			
			if (empty($imagesData)) {
				return;
			}
			echo '<AlternativeImages>';
			foreach ($imagesData as $imageData) {
				echo '<ProductImage>';
				echo 	'<ProductThumbnailImage>' . scXml::escape($imageData['ImageThumb']) . '</ProductThumbnailImage>';
				echo	'<ProductStandardImage>' . scXml::escape($imageData['Image']) . '</ProductStandardImage>';
				echo	'<ProductLargeImage>' . scXml::escape($imageData['ImageLarge']) . '</ProductLargeImage>';
				echo '</ProductImage>';
			}
			echo '</AlternativeImages>';
		}
		
		protected function showBundleProducts(scProductSL $product)
		{
			$productId = $product->getProductId();
			if (!isset($this->_bundleProductsByProduct[$productId])) {
				return;
			}
			if (empty($this->_bundleProductsByProduct[$productId])) {
				return;
			}
			
			echo '<BundleProducts>';
			foreach ($this->_bundleProductsByProduct[$productId] as $bundleProductId) {
				echo '<ProductID>' . $bundleProductId . '</ProductID>';
			}
			echo '</BundleProducts>';
		}
		
		protected function showRelatedProducts(scProductSL $product)
		{
			$productId = $product->getProductId();
			if (!isset($this->_relatedProductsByProduct[$productId])) {
				return;
			}
			if (empty($this->_relatedProductsByProduct[$productId])) {
				return;
			}
			
			echo '<RelatedProducts>';
			foreach ($this->_relatedProductsByProduct[$productId] as $bundleProductId) {
				echo '<ProductID>' . $bundleProductId . '</ProductID>';
			}
			echo '</RelatedProducts>';
		}
		
		protected function showAdditionalCategories(scProductSL $product)
		{
			$productId = $product->getProductId();
			if (!isset($this->_additionalCategoriesByProduct[$productId])) {
				return;
			}
			$categories = &$this->_additionalCategoriesByProduct[$productId];
			if (empty($categories)) {
				return;
			}
			echo '<AdditionalCategories>';
			foreach ($categories as $category) {
				echo '<Category>' .
						'<CategoryID>' . $category['CategoryID'] . '</CategoryID>' .
						'<CategoryName>' . scXml::escape($category['CategoryName']) . '</CategoryName>' .
						'<CategoryBranch>' . scXml::escape($this->getCategoryBranch($category['CategoryID'])) . '</CategoryBranch>' .
					'</Category>';
			}
			echo '</AdditionalCategories>';
		}
		
		protected function getRecordset()
		{
			$languageId = scMultiLanguage::getCurrentLanguageId();
			
			$recordset = new miSqlRecordset('Products');
			$recordset->addJoinCondition('LEFT', 'ProductsML', 'ON Products.ProductID=ProductsML.ProductID AND ProductsML.LanguageID=' . $languageId);
			$recordset->addJoinCondition('LEFT', 'ProductImages', 'ON ProductImages.ProductID = Products.ProductID AND ProductImages.ProductImageIsDefault');
			$recordset->addJoinCondition('LEFT', 'ManufacturersML', 'ON Products.ManufacturerID = ManufacturersML.ManufacturerID AND ManufacturersML.LanguageID=' . $languageId);
			$recordset->addJoinCondition('LEFT', 'CategoriesML', 'ON Products.CategoryID = CategoriesML.CategoryID AND CategoriesML.LanguageID=' . $languageId);
			$recordset->addJoinCondition('LEFT', 'ProductVariants', 'ON (ProductVariants.ProductID = Products.ProductID AND ProductVariantIsDefault<>0)');
			$recordset->addFilter(new miSqlFilterEqual('ProductIsActive', 1));
			if (scProductService::hideOutOfStock()) {
				$recordset->addFilter(new miSqlFilterBiggerThan('ProductQuantity', 0));
			}
			
			$recordset->setSelectFields(array(
				// Product data
				'Products.ProductID',
				'Products.ProductCode',
				'Products.ProductPrice',
				'Products.ProductQuantity',
				'Products.ProductWeight',
				'Products.ProductIsEgood',
				'Products.ProductCreatedTimestamp',
				'Products.CategoryID',
				'Products.ManufacturerID',
				'CategoriesML.CategoryName',
				'ManufacturersML.ManufacturerName',
				'Products.ProductNameXHTML',
				'ProductsML.ProductName',
				'Products.ProductDescriptionXHTML',
				'ProductsML.ProductDescription',
				'Products.ProductDetailedDescriptionXHTML',
				'ProductsML.ProductDetailedDescription',
				
				// Image
				'ProductImages.ProductImage',
//				'ProductImages.ProductImageWidth',
//				'ProductImages.ProductImageHeight',
//				'ProductImages.ProductImageSize',
				
				// Thumb
				'ProductImages.ProductImageThumb',
//				'ProductImages.ProductImageThumbWidth',
//				'ProductImages.ProductImageThumbHeight',
//				'ProductImages.ProductImageThumbSize',
				
				// Large image
				'ProductImages.ProductImageLarge',
//				'ProductImages.ProductImageLargeWidth',
//				'ProductImages.ProductImageLargeHeight',
//				'ProductImages.ProductImageLargeSize',
				
				// Variant image
				'ProductVariants.ProductVariantImage',
				'ProductVariants.ProductVariantImageThumb',
				'ProductVariants.ProductVariantImageLarge',
			));
			
			return $recordset;
		}
		
		protected function loadNextProducts()
		{
			// Use less memory
			unset($this->_products);
			
			if ($this->_lastResultset) {
				return false;
			}
			
			$recordset = $this->getRecordset();
			
			$numRecords = self::PROCESS_RECORDS_AT_ONCE;
			if ($this->_numProducts !== false) {
				if ($this->_numProducts <= 0)
					return false;
				
				if ($this->_numProducts < $numRecords)
					$numRecords = $this->_numProducts;
				$this->_numProducts -= $numRecords;
			}
			
			$this->_products = $recordset->getRecordsByIndex($this->_offset, $numRecords);
			$this->_offset += $numRecords;
			
			if (empty($this->_products)) {
				return false;
			}
			
			// Check if the last call returned less than the requested number of records
			if (count($this->_products) < self::PROCESS_RECORDS_AT_ONCE)
				$this->_lastResultset = true;
			
			$this->loadProductsDetails();
			
			return true;
		}
		
		protected function loadProductsDetails()
		{
			$productIds = array_values(miUtil::getKeyValueArray($this->_products, 'ProductID', 'ProductID'));
			$this->loadAdditionalImages($productIds);
			$this->loadRelatedProducts($productIds);
			$this->loadBundleProducts($productIds);
			$this->loadAdditionalCategories($productIds);
			
			// FIXME: These caches might grow with lot's of products as it is not cleared between batches
			scCustomUrls::prepareItems(scFriendlyUrl::TYPE_PRODUCT, $productIds);
			
			$manufacturerIds = array_values(miUtil::getKeyValueArray($this->_products, 'ManufacturerID', 'ManufacturerID'));
			scManufacturersCache::prepareItems($manufacturerIds);
		}
		
		protected function loadAdditionalImages($productIds)
		{
			$recordset = new miSqlRecordset('ProductImages');
			$recordset->addFilter(new miSqlFilterIn('ProductID', $productIds));
			$recordset->addFilter(new miSqlFilterEqual('ProductImageIsDefault', 0));
			$recordset->setSelectFields(array(
				'ProductID',
				'ProductImageThumb',
				'ProductImage',
				'ProductImageLarge'
			));
			$recordset->setOrder('ProductImageSort', 'ASC');
			$productImages = $recordset->getAllRecords();
			
			$this->_additionalImagesByProduct = array();
			foreach ($productImages as $productImage) {
				$this->_additionalImagesByProduct[$productImage['ProductID']][] = $productImage;
			}
			unset($productImages);
		}
		
		protected function loadRelatedProducts($productIds)
		{
			$recordset = new miSqlRecordset('RelatedProducts');
			$recordset->addFilter(new miSqlFilterIn('ProductID', $productIds));
			$recordset->setOrder('RelatedProductSort', 'ASC');
			$recordset->setSelectFields(array(
				'ProductID',
				'RelatedProductID'
			));
			$relatedProducts = $recordset->getAllRecords();
			
			$this->_relatedProductsByProduct = array();
			foreach ($relatedProducts as $product) {
				$this->_relatedProductsByProduct[$product['ProductID']][] = $product['RelatedProductID'];
			}
			unset($relatedProducts);
		}
		
		protected function loadBundleProducts($productIds)
		{
			$recordset = new miSqlRecordset('BundleProducts'); 
			$recordset->addFilter(new miSqlFilterIn('ProductID', $productIds)); 
			$recordset->setOrder('BundleProductSort', 'ASC');
			$recordset->setSelectFields(array(
				'ProductID',
				'SubProductID'
			));
			$bundleProducts = $recordset->getAllRecords();
			
			$this->_bundleProductsByProduct = array();
			foreach ($bundleProducts as $product) {
				$this->_bundleProductsByProduct[$product['ProductID']][] = $product['SubProductID'];
			}
			unset($bundleProducts);
		}
		
		protected function loadAdditionalCategories($productIds)
		{
			$languageId = scMultiLanguage::getCurrentLanguageId();
			$recordset = new miSqlRecordset('ProductXCategories'); 
			$recordset->addJoinCondition('LEFT', 'CategoriesML', 'ON ProductXCategories.CategoryID = CategoriesML.CategoryID AND LanguageID = ' . $languageId); 
			$recordset->addFilter(new miSqlFilterIn('ProductID', $productIds)); 
			$recordset->setSelectFields(array(
				'ProductID',
				'ProductXCategories.CategoryID',
				'CategoryName',
			));
			$additionalCategories = $recordset->getAllRecords();
			
			$this->_additionalCategoriesByProduct = array();
			foreach ($additionalCategories as $category) {
				$category['CategoryBranch'] = $this->getCategoryBranch($category['CategoryID']);
				$this->_additionalCategoriesByProduct[$category['ProductID']][] = $category;
			}
			unset($additionalCategories);
		}
		
		protected function loadAllCategories()
		{
			$languageId = scMultiLanguage::getCurrentLanguageId();
			$recordset = new miSqlRecordset('Categories');
			$recordset->addJoinCondition('LEFT', 'CategoriesML', 'ON Categories.CategoryID = CategoriesML.CategoryID AND LanguageID = ' . $languageId); 
			$recordset->setSelectFields(array(
				'Categories.CategoryID',
				'CategoryParentID',
				'CategoryName',
			));
			$this->_allCategories = $recordset->getAllRecords();
			
			$this->_allCategories = miUtil::getArrayWithKeys($this->_allCategories, 'CategoryID');
		}
		
		protected function getCategoryBranch($categoryId)
		{
			if (isset($this->_categoryBranches[$categoryId])) {
				return $this->_categoryBranches[$categoryId];
			}
			
			$category = $this->_allCategories[$categoryId];
			if ($category['CategoryParentID'] == null) {
				$this->_categoryBranches[$categoryId] = $category['CategoryName'];
			} else {
				$this->_categoryBranches[$categoryId] = $this->getCategoryBranch($category['CategoryParentID']) . self::CATEGORY_BRANCH_SEPARATOR . $category['CategoryName'];
			}
			return $this->_categoryBranches[$categoryId];
		}
		
		protected function printExtraXml() {}
		
		protected function getTaxedPrice($productPrice, $productId)
		{
			$taxClasses = $this->getProductTaxClasses($productId);
			if (is_null($taxClasses))
				return $productPrice;
				
			foreach ($taxClasses as $taxClassId) {
				$taxInfo = scTax::getTaxInfoByTaxClassId($taxClassId); // this is cached in scTax class
				$taxId = $taxInfo['TaxID'];
				$taxZoneId = $this->getTaxZoneId($taxId);
				$taxRate = $this->getTaxRate($taxZoneId, $taxClassId);
				$productPrice = $productPrice + $productPrice * $taxRate / 100;
			}
			
			return $productPrice;
		}
		
		protected $_productsTaxClasses = null;
		
		protected function getProductTaxClasses($productId)
		{
			// cache the Product TaxClass data
			if (is_null($this->_productsTaxClasses)) {
				$sql = 'SELECT ProductID, TaxClassID FROM ProductTaxClasses';
				$result = miStaticDBUtil::execSelect($sql);
				$this->_productsTaxClasses = array();
				foreach ($result as $productTax) {
					if (!isset($this->_productsTaxClasses[$productTax['ProductID']])) {
						$this->_productsTaxClasses[$productTax['ProductID']] = array();
					}
					$this->_productsTaxClasses[$productTax['ProductID']][] = $productTax['TaxClassID']; 
				}
			}
			
			if (!isset($this->_productsTaxClasses[$productId]))
				return null;
				
			return $this->_productsTaxClasses[$productId];
		}
		
		protected function getTaxZoneId($taxId)
		{
			if (!isset($this->_taxZones[$taxId])) {
				$this->_taxZones[$taxId] = scZones::getTaxZoneId($taxId, $this->getCustomerAddress());
			}
			return $this->_taxZones[$taxId];
		}
		
		protected function getCustomerAddress()
		{
			if (is_null($this->_customerAdderess)) {
				scCustomerAuth::singleton()->logIn(ICheckoutModule::GUEST_USER_ID);
				$customerShippingAddress = new scCustomerAddress();
				$customerShippingAddress->read(scCustomerAuth::singleton()->getCustomer()->getShippingAddressId());
				$this->_customerAdderess = $customerShippingAddress->getMappedAddressFields(scCustomerAddress::SHIPPING, $customerShippingAddress->getRow());
			}
			return $this->_customerAdderess;
		}
		
		protected function getTaxRate($taxZoneId, $taxClassId)
		{
			if (is_null($this->_taxRates)) {
				$allRates = scTax::getAllTaxRates();
				foreach ($allRates as $rate) {
					$this->_taxRates[$rate['TaxZoneID']][$rate['TaxClassID']] = $rate['TaxRate'];
				}
			}
			if (!isset($this->_taxRates[$taxZoneId][$taxClassId]))
				return 0;
			
			return $this->_taxRates[$taxZoneId][$taxClassId];
		}
	}
	
	class scProductFeedProductSL extends scProductSL {
		protected $_additionalCategories;
		
		public function __construct($additionalCategories)
		{
			parent::__construct();
			$this->_additionalCategories = $additionalCategories;
		}
		
		/**
		 * Use the already calculated additional categories
		 */
		public function getAllCategoryIds()
		{
			return $this->_additionalCategories;
		}
	}
?>
