<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Class for handling different rating actions statuses
	 */
	class scRatingActionsStatuses
	{
		const RATING_ACTIONS_STATUS_WAITING_FOR_APPROVAL = 0;
		const RATING_ACTIONS_STATUS_APPROVED = 1;
		const RATING_ACTIONS_STATUS_DISAPPROVED = 2;
		
		protected static $_ratingIconsNames = array(
			self::RATING_ACTIONS_STATUS_APPROVED => 'approved',
			self::RATING_ACTIONS_STATUS_DISAPPROVED => 'disapproved',
			self::RATING_ACTIONS_STATUS_WAITING_FOR_APPROVAL => 'waiting_for_approval',
		);
		
		/**
		 * Returns array with all the statuses 
		 *
		 * @return array
		 */
		public static function getRatingActionStatuses()
		{
			return array(
				self::RATING_ACTIONS_STATUS_WAITING_FOR_APPROVAL => _t('Waiting for approval'),
				self::RATING_ACTIONS_STATUS_APPROVED => _t('Approved'),
				self::RATING_ACTIONS_STATUS_DISAPPROVED => _t('Disapproved')
			);
		}
		
		/**
		 * @param int $status
		 * @return string
		 */
		public static function getIconNameByStatus($status)
		{
			if (!isset(self::$_ratingIconsNames[$status]))
				throw new miException(_i('Invalid status.'));
			return self::$_ratingIconsNames[$status];
		}
		
		public static function getCombinedRatingActionStatus($productReviewStatus, $productRatingVoteStatus)
		{
			// Mark records not found as waiting for approval
			$combinedStatus = scRatingActionsStatuses::RATING_ACTIONS_STATUS_WAITING_FOR_APPROVAL;
			
			if (!is_null($productReviewStatus) && !is_null($productRatingVoteStatus) && ($productReviewStatus != $productRatingVoteStatus))
				return $combinedStatus;

			if (!is_null($productReviewStatus))
				$combinedStatus = $productReviewStatus;
			
			if (!is_null($productRatingVoteStatus))
				$combinedStatus = $productRatingVoteStatus;
			
			return $combinedStatus;
		}
	}
		
	/**
	 * Class for managing Rating votes
	 */
	class scRatingVotes
	{
		const RATING_MAX_VOTE = 5;
		const RATING_MIN_VOTE = 1;
		
		/**
		 * Returns array with rating votes options ready for miSelectWidget
		 *
		 * @return array
		 */
		public static function getRatingVotesChoices()
		{
			$actionsStatusesComments = array(	
				1 => _t('Poor'),
				2 => _t('Average'),
				3 => _t('Good'),
				4 => _t('Very good'),
				5 => _t('Excellent'),
			);
			
			$options = range(self::RATING_MIN_VOTE, self::RATING_MAX_VOTE);
			$values = array();
			foreach($options as $option) {
				$values[$option] = $actionsStatusesComments[$option];
			}
			
			return $values;
		}
	}
?>