<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scRatingActionsUtils
	{
		public static function getAllowRateSetting()
		{
			return scSettings::getSettingValue(scSettings::SETTING_USERS_ABLE_TO_RATE);
		}
		
		public static function setAllowRateSetting($settingValue)
		{
			if (in_array($settingValue, scRatingActionsSettings::$allowedRateSetting))
				return scSettings::setSettingValue(scSettings::SETTING_USERS_ABLE_TO_RATE, $settingValue);

			return false;
		}
		
		public static function getAllowReviewSetting()
		{
			return scSettings::getSettingValue(scSettings::SETTING_USERS_ABLE_TO_REVIEW);
		}
		
		public static function setAllowReviewSetting($settingValue)
		{
			if (in_array($settingValue, scRatingActionsSettings::$allowedReviewSetting))
				return scSettings::setSettingValue(scSettings::SETTING_USERS_ABLE_TO_REVIEW, $settingValue);
			
			return false;
		}
		
		public static function getAutomaticApprovalSetting()
		{
			return scSettings::getSettingValue(scSettings::SETTING_RATING_ACTIONS_AUTOMATIC_APPROVAL);
		}
		
		public static function setAutomaticApprovalSetting($settingValue)
		{
			if (in_array($settingValue, scRatingActionsSettings::$allowedAutomaticUpdateSetting))
				return scSettings::setSettingValue(scSettings::SETTING_RATING_ACTIONS_AUTOMATIC_APPROVAL);
			
			return false;
		}
		
		public static function getAllowedRatingActionsForCustomer($customerId, $productId)
		{
			$allowedRatingActions = array(
				'rate' => array(
					'allowed' => false,
					'error' => '',
				), 
				'review' => array(
					'allowed' => false,
					'error' => '',
				),
			);
			
			$ratingActionRate = self::getAllowRateSetting();
			$ratingActionReview = self::getAllowReviewSetting();
			
			switch ($ratingActionRate) {
				case scRatingActionsSettings::RATING_ACTION_SETTING_ALLOW_RATE_ALL:
					$allowedRatingActions['rate']['allowed'] = true;
					break;
				case scRatingActionsSettings::RATING_ACTION_SETTING_ALLOW_RATE_REGISTERED:
					if ($customerId) {
						$allowedRatingActions['rate']['allowed'] = true;
					} else {
						$allowedRatingActions['rate']['error'] = _t('Only registered users can rate.');
					}
					break;
				case scRatingActionsSettings::RATING_ACTION_SETTING_ALLOW_RATE_BOUGHT:
					if (scUtil::hasCustomerBoughtProduct($customerId, $productId)) {
						$allowedRatingActions['rate']['allowed'] = true;
					} else {
						$allowedRatingActions['rate']['error'] = _t('Only customers that have bought the product can rate it.');
					}
					break;
				default:
					// Asume self::RATING_ACTION_SETTING_ALLOW_RATE_NONE
					$allowedRatingActions['rate']['error'] = ''; // ('The administrator has disabled rating of products.');
					break;
			}
			
			switch ($ratingActionReview) {
				case scRatingActionsSettings::RATING_ACTION_SETTING_ALLOW_REVIEW_ALL:
					$allowedRatingActions['review']['allowed'] = true;
					break;
				case scRatingActionsSettings::RATING_ACTION_SETTING_ALLOW_REVIEW_REGISTERED:
					if ($customerId) {
						$allowedRatingActions['review']['allowed'] = true;
					} else {
						$allowedRatingActions['review']['error'] = _t('Only registered users can write reviews.');
					}
					break;
				case scRatingActionsSettings::RATING_ACTION_SETTING_ALLOW_REVIEW_BOUGHT:
					if (scUtil::hasCustomerBoughtProduct($customerId, $productId)) {
						$allowedRatingActions['review']['allowed'] = true;
					} else {
						$allowedRatingActions['review']['error'] = _t('Only customers that have bought the product can write review about it.');
					}
					break;
				default:
					// Asume self::RATING_ACTION_SETTING_ALLOW_REVIEW_NONE
					$allowedRatingActions['review']['error'] = ''; // ('The administrator has disabled writing reviews of products.');
					break;
			}
			
			$hasEnabledRatingAction = false;
			foreach($allowedRatingActions as $allowedAction) {
				if ($allowedAction['allowed'])
					$hasEnabledRatingAction = true;
			}
			
			$allowedRatingActions['hasEnabledRatingAction'] = $hasEnabledRatingAction;
			return $allowedRatingActions;
		}
		
		public static function isApprovedAutomatically($customerId)
		{ 
			$approvalSetting = self::getAutomaticApprovalSetting();
			
			if ($approvalSetting == scRatingActionsSettings::RATING_ACTION_SETTING_AUTOMATIC_APPROVAL_ALL)
				return true;
			elseif ($approvalSetting == scRatingActionsSettings::RATING_ACTION_SETTING_AUTOMATIC_APPROVAL_NONE)
				return false;
			
			return (boolean)$customerId;
		}
	}
?>