<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scRssFeedsModule implements IUserModule {
		
		const RSS_RESULTS_COUNT = 20;
		
		const FEED_TYPE_NEWS = 'news';
		
		protected $_rssFeeds = array (
			self::FEED_TYPE_NEWS => 'scRssFeedNews',
		);
		
		protected $_feedType;
		
		public function __construct()
		{
			try {
				// First make sure the sent FeedType is available
				$feedType = strtolower(miGetParamDefault('FeedType', ''));
				if (!array_key_exists($feedType, $this->_rssFeeds)) {
					throw new miException(_t('Unknown RSS feed.'));
				}
				
				$this->_feedType = $feedType;
				
				$this->setupAutoloading();
			}
			catch (miException $exception) {
				$url = scPaths::getStoreWebPath() . 'error.php?' . 
						miMessage::PARAM_MESSAGE .'='. urlencode($exception->getMessage()) . '&'.
						miMessage::PARAM_MESSAGE_TYPE .'='. urlencode(miMessage::MSG_TYPE_ERROR);
				
				scUtil::redirect($url);
			}
		}
		
		/**
		 * Initializes the action handlers for the user module
		 *
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('', 'scRssFeedsListAction');
		}
		
		/**
		 * Returns the view that should be displayed
		 * 
		 * @return miDefaultView
		 */
		public function getView()
		{
			$view = new scRssFeedsView();
			$view->setRssFeedClass(new $this->_rssFeeds[$this->_feedType]());
			$view->setRssFeedType($this->_feedType);
			return $view;
		}
		
		protected function setupAutoloading()
		{
			$autoloadClasses = array(
				// Interface
				'IRssFeed' => 'include/IRssFeed.php',
				
				// Feeds
				'scRssFeedNews' => 'news.php',
			);
		
			foreach ($autoloadClasses as $className => $fileName) {
				// Relative from the location of miLoader
				miLoader::addClass($className, '../../modules/system/rss/' . $fileName);
			}
		}
	}
	
	class scRssFeedsView extends scView {
		
		protected $_rssFeed;
		protected $_rssFeedType;
		
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'modules/system/rss/rss2.0.tmpl',
		);
		
		public function setRssFeedClass(IRssFeed $rssFeed)
		{ 
			$this->_rssFeed = $rssFeed;
		}
		
		public function getRssFeedClass()
		{
			return $this->_rssFeed;
		}
		
		public function setRssFeedType($rssFeedType)
		{
			$this->_rssFeedType = $rssFeedType;
		}
		
		public function getRssFeedType()
		{
			return $this->_rssFeedType;
		}
	}
	
	class scRssFeedsListAction extends miListAction {
		
		public function doAction()
		{
			$parser = new scTemplateParser();
			$parser->readTemplate($this->_view->getTemplateFileName(scRssFeedsView::TEMPLATE_ID_LIST));
			
			$items = $this->_view->getRssFeedClass()->getRssFeedItems(scMultiLanguage::getCurrentLanguageId());
			$parser->addSectionInfo(new miTemplateParserSectionInfo('items', count($items), 
				miUIUtil::templetizeRows(miUIUtil::transposeRows($items))
			));
			
			if (scFriendlyUrl::hasFriendlyUrls()) {
				$channelLink = scPaths::getUrl() . 'rss/' . $this->_view->getRssFeedType() . '/' .
					strtolower(scMultiLanguage::singleton()->getCurrentLanguageCode());
			} else {
				$channelLink = scPaths::getUrl() . 'module.php?ModuleName=com.summercart.rss&amp;UILanguage=' .
					strtolower(scMultiLanguage::singleton()->getCurrentLanguageCode()) . '&amp;FeedType=' . $this->_view->getRssFeedType();
			}
			
			$parser->assignArray(array (
				'%%CHANNELTITLE%%' => miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME)),
				'%%SITELINK%%' => scPaths::getUrl(),
				'%%CHANNELLINK%%' => $channelLink,
				'%%CHANNELDESCRIPTION%%' => miI18N::htmlEscape(scSettings::getSettingValue(scSettings::SETTING_SEO_SITE_DESCRIPTION))
			));
			
			header('Content-type: text/xml');
			echo $parser->templateParse();
			exit;
		}
	}
?>