<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSitemapsProtocolModule implements IUserModule 
	{
		
		/**
		 * Initializes the action handlers for this module
		 * 
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('acGenerateSitemapXml', 'scGenerateSitemapXmlAction');
		}
		
		/**
		 * Returns the view that should be displayed
		 * 
		 * @return miDefaultView
		 */
		public function getView()
		{
			return new scSitemapsProtocolView();
		}
	}
	
	class scSitemapsProtocolView extends scView
	{
		
	}
	
	
	/**
	 * Generate the sitemap XML
	 */
	class scGenerateSitemapXmlAction extends miAction {
		public function doAction()
		{
			if (ob_get_level()) {
				ob_end_clean();
				header('Content-Encoding: none');
			}
//			ob_start('ob_gzhandler');
						
			$urls = $this->getAllUrls();
			$xml = $this->generateXml($urls);
			echo $xml;
			die;
			
//			echo '<pre>';
//			echo htmlspecialchars(var_export($xml, true));
//			die;
		}
		
		protected function getAllUrls()
		{
			$languages = scMultiLanguage::getActiveLanguageCodes();
			
			// FIXME: the BG transliteration
			
			$saveCurrentLanguageId = scMultiLanguage::getCurrentLanguageId();
			
			$urls = array();
			foreach ($languages as $languageId => $languageCode) {
				scMultiLanguage::singleton()->setCurrentLanguageId($languageId);
				
				$urls = array_merge($urls,
					$this->getProductUrls($languageId),
					$this->getCategoryUrls($languageId),
					$this->getRichTextPagesUrls($languageId),
					$this->getFormsUrls($languageId),
					$this->getManufacturersUrls($languageId),
					$this->getMenuItemsUrls($languageId),
					$this->getNewsUrls($languageId));
			}
			
			$urls = array_merge($urls, $this->getMainPagesUrls());
			$urls = array_unique($urls);
			
			scMultiLanguage::singleton()->setCurrentLanguageId($saveCurrentLanguageId);
			
			return $urls;
		}
		
		protected function getMainPagesUrls()
		{
			$scripts = array(
				'index.php',
				'search.php',
				'search.php?action=dmAdvancedSearch',
				'manufacturers.php',
				'module.php?ModuleName=com.summercart.news',
				'login.php',
				'forgot_pass.php'
			);
			
			$urls = array();
			foreach ($scripts as $script)
				$urls[] = scPaths::getUrl($script);
			return $urls;
		}
		
		protected function getProductUrls($languageId)
		{
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0 ' : '';
			$sql = 'SELECT Products.ProductID,ProductName FROM Products JOIN ProductsML USING(ProductID) WHERE LanguageID = ? AND ProductIsActive<>0 ' . $hideOutOfStockSql;
			$products = miStaticDBUtil::execSelect($sql, array($languageId));
			
			$urls = array();
			foreach ($products as $product)
				$urls[] = scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_PRODUCT, $product['ProductID'], $product['ProductName']), scUrlUtil::URL_TYPE_ABSOLUTE);
			return $urls;
		}
		
		protected function getCategoryUrls($languageId)
		{
			$sql = 'SELECT * FROM Categories JOIN CategoriesML USING(CategoryID) WHERE LanguageID=?';
			$categories = miStaticDBUtil::execSelect($sql, array($languageId));
			
			scFriendlyUrl::initFriendlyUrls($categories, scFriendlyUrl::ROW_TYPE_CATEGORIES);
			
			$urls = array();
			foreach ($categories as $category)
				$urls[] = scPaths::getUrl($category['CategoryUrl']);
			return $urls;
		}
		
		protected function getRichTextPagesUrls($languageId)
		{
			$sql =  'SELECT * FROM CMSRichTextPages JOIN CMSRichTextPagesML USING(CMSRichTextPageID) WHERE LanguageID = ?';
			$pages = miStaticDBUtil::execSelect($sql, array($languageId));
			
			$urls = array();
			foreach ($pages as $page)
				$urls[] = scPaths::getUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_RICH_TEXT_PAGE, $page['CMSRichTextPageID'], $page['CMSRichTextPageTitle']));
			return $urls;
		}
		
		protected function getFormsUrls($languageId)
		{
			$sql =  'SELECT CMSForms.CMSFormID,CMSFormTitle FROM CMSForms JOIN CMSFormsML USING(CMSFormID) WHERE LanguageID = ?';
			$forms = miStaticDBUtil::execSelect($sql, array($languageId));
			
			$urls = array();
			foreach ($forms as $form)
				$urls[] = scPaths::getUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_FORM, $form['CMSFormID'], $form['CMSFormTitle']));
			return $urls;
		}
		
		protected function getMenuItemsUrls($languageId)
		{
			$sql =  'SELECT * FROM CMSMenuItems JOIN CMSMenuItemsML USING(CMSMenuItemID) WHERE LanguageID = ?';
			$menuItems = miStaticDBUtil::execSelect($sql, array($languageId));
			
			scFriendlyUrl::initFriendlyUrls($menuItems, scFriendlyUrl::ROW_TYPE_MENU_ITEMS);
			
			$urls = array();
			foreach ($menuItems as $menuItem) {
				// TODO: check full urls to the store
				if ((substr($menuItem['MenuUrl'], 0, 7) == 'http://') or (substr($menuItem['MenuUrl'], 0, 8) == 'https://') or (substr($menuItem['MenuUrl'], 0, 7) == 'callto:'))
					continue;
				$urls[] = scPaths::getUrl($menuItem['MenuUrl']);
			}
			return $urls;
		}
		
		protected function getManufacturersUrls($languageId)
		{
			$sql = 'SELECT * FROM Manufacturers JOIN ManufacturersML USING(ManufacturerID) WHERE LanguageID = ?';
			$manufacturers = miStaticDBUtil::execSelect($sql, array($languageId));
			
			scFriendlyUrl::initFriendlyUrls($manufacturers, scFriendlyUrl::ROW_TYPE_MANUFACTURERS);
			
			$urls = array();
			foreach ($manufacturers as $manufacturer)
				$urls[] = scPaths::getUrl($manufacturer['ManufacturerUrl']);
			return $urls;
		}
		
		protected function getNewsUrls($languageId)
		{
			$sql = 'SELECT News.NewsID FROM News JOIN NewsML USING(NewsID) WHERE LanguageID = ? AND NewsIsActive = 1 AND NewsDate <= CURDATE()';
			$news = miStaticDBUtil::execSelect($sql, array($languageId));
			
			$urls = array();
			foreach ($news as $new)
				$urls[] = scPaths::getUrl('module.php?ModuleName=com.summercart.news&NewsID=' . $new['NewsID']);
			return $urls;
		}
		
		protected function generateXml($urls)
		{
			$xml = '<?xml version="1.0" encoding="UTF-8"?>';
			$xml .= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">';
			
			foreach ($urls as $url) {
				$xml .= '<url>';
				$xml .= '<loc>' . scXml::escape($url) . '</loc>';
				$xml .= '</url>' . "\n";
			}
			
			$xml .= '</urlset>';
			return $xml;
		}
	}
?>