<?php
	/**
	 * Return pricing details - discount by quantity
	 *
	 */
	class scWholesaleProductDetails implements IProductDetails {
		/**
		 * Returns a table showing the quantity discounts 
		 * 
		 * @param scProductSL $product
		 * @param miTemplateParserSectionInfo $productSection
		 * @return string product details info
		 */
		public function getProductDetailsHtml(scProductSL $product, miTemplateParserSectionInfo $productSection)
		{
			// Check that wholesale prices are enabled
			if (!scSettings::getSettingValue(scSettings::SETTING_WHOLESALE_PRICES)) {
				return '';
			}
			
			list($priceRows, $wholesalePriceSectionsArray, $wholesaleTaxedPriceSectionsArray) = $this->getPriceRows($product);
			if (empty($priceRows))
				return '';
			
			$productSection->addSubsection(new miTemplateParserSectionInfo('productHasWholesalePrices', 1));
			
			$parser = new scTemplateParser();
			$parser->readTemplate('modules/system/wholesale/product_prices.tmpl');
			
			$pricesSection = new miTemplateParserSectionInfo('prices', count($priceRows), miUIUtil::templetizeRows(miUIUtil::transposeRows($priceRows)));
			$pricesSection->addSubsectionsArray('wholesalePrice', $wholesalePriceSectionsArray);
			$pricesSection->addSubsectionsArray('wholesaleTaxedPrice', $wholesaleTaxedPriceSectionsArray);
			$parser->addSectionInfo($pricesSection);
			
			$displayPricesTaxMode = scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE);
			if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
				$parser->addSectionInfo(new miTemplateParserSectionInfo('wholesalePrice', 1));
			} else {
				$parser->addSectionInfo(new miTemplateParserSectionInfo('wholesaleTaxedPrice', 1));
			}
			
			return $parser->templateParse();
		}
		
		/**
		 * Returns an array of product wholesale prices 
		 * 
		 * @param scProductSL $product
		 * @return array product prices
		 */
		public function getPriceRows(scProductSL $product)
		{
			$customerGroupId = (int)scCustomerAuth::singleton()->getCustomerGroupId();
			$productId = $product->getProductId();
			
			if ($product->getProductDomainObject()->hasVariants()) {
				$variantId = miGetParamDefault('ProductVariantID', 0);
				if (empty($variantId))
					$variantId = $product->getProductDomainObject()->getDefaultVariant()->get('ProductVariantID');
				
				$sql = 'SELECT ProductWholesalePriceQuantity,ProductWholesalePriceAmount FROM ProductWholesalePrices WHERE CustomerGroupID=?' . 
				' AND ProductID=? AND ProductVariantID=? ORDER BY ProductWholesalePriceQuantity';
				$priceRows = miStaticDBUtil::execSelect($sql, array($customerGroupId, $productId, $variantId));
			} else {
				$sql = 'SELECT ProductWholesalePriceQuantity,ProductWholesalePriceAmount FROM ProductWholesalePrices WHERE CustomerGroupID=?' . 
					' AND ProductID=? AND ProductVariantID IS NULL ORDER BY ProductWholesalePriceQuantity';
				$priceRows = miStaticDBUtil::execSelect($sql, array($customerGroupId, $productId));
			}
			
			// If there are no prices, or just one price don't print the table
			if (count($priceRows) <= 1)
				return array(null, null, null);
			
			$wholesalePriceSectionsArray = array();
			$wholesaleTaxedPriceSectionsArray = array();
			foreach ($priceRows as &$priceRow) {
				$price = scMultiCurrencyAmount::factoryBaseCurrency($priceRow['ProductWholesalePriceAmount']);
				// Check general setting display prices mode and generate the modified price to the section
				$taxedPrice = scProductPrice::getTaxedAmount($price, $productId);
				$wholesalePriceElements = array();
				$displayPricesTaxMode = scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE);
				if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
					$wholesalePriceElements['%%PRODUCTWHOLESALEPRICEAMOUNT%%'] = scPrice::factory($taxedPrice)->getDisplayPrice();
					$wholesalePriceSectionsArray[] = new miTemplateParserSectionInfo('wholesalePrice', 1, $wholesalePriceElements);
					$wholesaleTaxedPriceSectionsArray[] = new miTemplateParserSectionInfo('wholesaleTaxedPrice', 0);
				} else if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_AND_WITHOUT_TAX) {
					$wholesalePriceElements['%%PRODUCTWHOLESALEPRICEAMOUNT%%'] = scPrice::factory($price)->getDisplayPrice();
					$wholesalePriceElements['%%PRODUCTWHOLESALETAXEDPRICEAMOUNT%%'] = scPrice::factory($taxedPrice)->getDisplayPrice();
					$wholesalePriceSectionsArray[] = new miTemplateParserSectionInfo('wholesalePrice', 0);
					$wholesaleTaxedPriceSectionsArray[] = new miTemplateParserSectionInfo('wholesaleTaxedPrice', 1, $wholesalePriceElements);
				}
			}
			
			return array($priceRows, $wholesalePriceSectionsArray, $wholesaleTaxedPriceSectionsArray);
		}
	}
?>