<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scWishlistUserModule implements IUserModule {
		
		public static function getWishlistPageUrl()
		{
			return scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_WISHLIST);
		}
		
		/**
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->setActionHandler('', 'scWishlistListAction');
			$page->getDispatcher()->setActionHandler('addItem', 'scWishlistAddItemAction');
			$page->getDispatcher()->setActionHandler('removeItem', 'scWishlistRemoveItemAction');
			$page->getDispatcher()->setActionHandler('send', 'scWishlistSendAction');
			
			$page->setSeo(new scWishlistSeo());
		}
		
		public function getView()
		{
			return new scWishlistView();
		}
	}
	
	class scWishlistProductsDetails implements IProductDetails {
		
		public function getProductDetailsHtml(scProductSL $product, miTemplateParserSectionInfo $productSection)
		{
			$productSection->addValues(array(
				'%%WISHLIST_DETAILS_BOX%%' => $this->getDetailsBoxHtml($product)
			));
		}
		
		protected function getDetailsBoxHtml(scProductSL $product)
		{
			$hasFriendlyUrlsSection = new miTemplateParserSectionInfo();
			$hasFriendlyUrlsSection->setIterationsCount(1);
			if (scFriendlyUrl::hasFriendlyUrls()) {
				$hasFriendlyUrlsSection->setSectionName('hasFriendlyUrls');
			} else {
				$hasFriendlyUrlsSection->setSectionName('noFriendlyUrls');
			}
			
			$t = new scTemplateParser();
			$t->readTemplate('modules/system/wishlist/wishlist_details_box.tmpl');
			$t->addSectionInfo($hasFriendlyUrlsSection);
			$templateData = array(
				'%%PRODUCTID%%' => $product->get('ProductID'),
				'%%PRODUCTNAME%%' => $product->get('ProductName'),
			);
			$t->assignArray($templateData);
			return $t->templateParse();
		}
	}
	
	class scWishlistView extends miDefaultView {
		
		public function addRedirectToLoginControllerCommand($params = '')
		{
			$loc = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN);
			if (!empty($params)) {
				$ref = scUrlUtil::addQueryParams(scWishlistUserModule::getWishlistPageUrl(), $params);
				$loc = scUrlUtil::addQueryParams($loc, 'Referer=' . urlencode($ref));
			}
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
		
		public function addRedirectToErrorControllerCommand($error)
		{
			$loc = 'error.php?' . miMessage::PARAM_MESSAGE . '=' . urlencode($error);
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
		
		public function addRedirectToMessageControllerCommand($msg)
		{
			$loc = scWishlistUserModule::getWishlistPageUrl();
			$loc = scUrlUtil::addQueryParams($loc, miMessage::PARAM_MESSAGE . '=' . urlencode($msg));
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
	}
	
	class scWishlistAddItemAction extends miAction {
		
		public function doAction()
		{
			try {
				$productId = (int)miGetParam('ProductID');
				$productVariantId = (int)miGetParamDefault('ProductVariantID', 0);
				$quantity = (int)miGetParamDefault('Quantity', 1);
				if ($quantity < 1) {
					$quantity = 1;
				}
				$options = miGetParamDefault('Options', array());
				
				// Check if logged in
				if (!scCustomerAuth::singleton()->isLoggedIn()) {
					$params = array('action' => 'addItem', 'ProductID' => $productId, 'Quantity' => $quantity, 'Options' => $options);
					$this->_view->addRedirectToLoginControllerCommand(http_build_query($params));
					return false;
				}
				
				if (!isset($options['ProductOptions'])) {
					$options['ProductOptions'] = array();
				}
				if (!isset($options['LocalOptions'])) {
					$options['LocalOptions'] = array();
				}
				
				$listingItem = $this->createListingItem($productId, $options, $productVariantId);
				
				$item = new scWishlistItem();
				$item->set('ProductID', $productId);
				if ($listingItem instanceof scProductVariant) {
					$item->set('ProductVariantID', $listingItem->get('ProductVariantID'));
				}
				$item->set('CustomerID', scCustomerAuth::singleton()->getCustomerId());
				$item->set('WishlistItemQuantity', $quantity);
				
				$options = $this->handleUserTextOptions($options);
				$item->setOptions($options);

				if ($item->isAlreadyAdded()) {
					$this->_view->addRedirectToMessageControllerCommand(_t('This item is already on your wishlist.'));
					return false;
				}
				$item->insert();
				
				$this->_view->addRedirectToMessageControllerCommand(_t('Item successfully added to your wishlist.'));
				return true;
			} catch (miException $e) {
				$this->_view->addRedirectToErrorControllerCommand($e->getMessage());
				return false;
			}
		}
		
		/**
		 * Options are passed as array, with empty key and optionId for the value
		 * User text options however are passes as array with optionid as key, and the user text option as the value
		 * In order to save the item in the database, we have to pass the user text options in the same format as the rest of the options
		 */
		protected function handleUserTextOptions($options)
		{
			if (!empty($options['ProductOptions'])) {
				$optionGroupIds = array_map('intval', array_keys($options['ProductOptions']));
				
				$sql = 'SELECT OptionGroupID FROM OptionGroups' .
					' WHERE OptionGroupType = ' . scProductOptions::GROUP_TYPE_USER_TEXT .
					' AND OptionGroupID IN (' . implode(',', $optionGroupIds) . ')';
				$rows = miStaticDBUtil::execSelect($sql);
				
				foreach ($rows as $row) {
					foreach ($options['ProductOptions'][$row['OptionGroupID']] as $key => $val)
						 $options['ProductOptions'][$row['OptionGroupID']][$key] = $key;
				}
			}
			
			if (!empty($options['LocalOptions'])) {
				$localOptionGroupIds = array_map('intval', array_keys($options['LocalOptions']));
				
				$sql = 'SELECT LocalOptionGroupID FROM LocalOptionGroups' .
					' WHERE LocalOptionGroupType = ' . scProductOptions::GROUP_TYPE_USER_TEXT .
					' AND LocalOptionGroupID IN (' . implode(',', $localOptionGroupIds) . ')';
				
				$rows = miStaticDBUtil::execSelect($sql);
				
				foreach ($rows as $row) {
					foreach ($options['LocalOptions'][$row['LocalOptionGroupID']] as $key => $val)
						 $options['LocalOptions'][$row['LocalOptionGroupID']][$key] = $key;
				}
			}
			
			return $options;
		}
		
		/**
		 * @param int $productId
		 * @param array $options
		 * @return IProductListingItem
		 */
		protected function createListingItem($productId, $options, $productVariantId = 0)
		{
			if (!isset($options['ProductOptions'])) {
				$options['ProductOptions'] = array();
			}
			if (!isset($options['LocalOptions'])) {
				$options['LocalOptions'] = array();
			}
			
			if ($productVariantId > 0) {
				$variant = new scProductVariant();
				$variant->read($productVariantId);
				if ($variant->get('ProductID') != $productId) {
					throw new miException(_i('Invalid product variant.'));
				}
				return $variant;
			}
			
			$product = new scProductDomainObject();
			$product->read($productId);
			if (!$product->hasVariants()) {
				return $product;
			}
			
			// Find the variant options
			$variantOptions = array();
			
			if (!empty($options['ProductOptions'])) {
				$optionGroupIds = array_map('intval', array_keys($options['ProductOptions']));
				$sql = 'SELECT OptionGroupID FROM OptionGroups' .
					' WHERE OptionGroupType = ' . scProductOptions::GROUP_TYPE_VARIANT .
					' AND OptionGroupID IN (' . implode(',', $optionGroupIds) . ')';
				$sql = miStaticDBUtil::execSelect($sql);
				foreach ($sql as $option) {
					$groupId = $option['OptionGroupID'];
					$optionId = reset($options['ProductOptions'][$groupId]);
					$variantOptions[] = new scProductVariantOption($optionId, scProductVariantOption::TYPE_GLOBAL);
				}
			}
			if (!empty($options['LocalOptions'])) {
				$optionGroupIds = array_map('intval', array_keys($options['LocalOptions']));
				$sql = 'SELECT LocalOptionGroupID FROM LocalOptionGroups' .
					' WHERE LocalOptionGroupType = ' . scProductOptions::GROUP_TYPE_VARIANT .
					' AND LocalOptionGroupID IN (' . implode(',', $optionGroupIds) . ')';
				$sql = miStaticDBUtil::execSelect($sql);
				foreach ($sql as $option) {
					$groupId = $option['LocalOptionGroupID'];
					$optionId = reset($options['LocalOptions'][$groupId]);
					$variantOptions[] = new scProductVariantOption($optionId, scProductVariantOption::TYPE_LOCAL);
				}
			}
			
			$variant = new scProductVariant();
			$variant->set('ProductID', $productId);
			$variant->setOptions($variantOptions);
			$variantId = $variant->readByOptions();
			
			if (!is_null($variantId)) {
				return $variant;
			}
			
			return $product->getDefaultVariant();
		}
	}
	
	class scWishlistRemoveItemAction extends miAction {
		
		public function doAction()
		{
			try {
				if (!scCustomerAuth::singleton()->isLoggedIn()) {
					$this->_view->addRedirectToLoginControllerCommand();
					return false;
				}
				
				$itemId = (int)miGetParam('WishlistItemID');
				
				$sql = 'DELETE FROM WishlistItems WHERE WishlistItemID = ? AND CustomerID = ?';
				miStaticDBUtil::execSQL($sql, array($itemId, scCustomerAuth::singleton()->getCustomerId()));
				
				$loc = scWishlistUserModule::getWishlistPageUrl();
				$loc = scUrlUtil::addQueryParams($loc, miMessage::PARAM_MESSAGE . '=' . urlencode(_t('Wishlist item removed.')));
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $e) {
				$this->_view->addRedirectToErrorControllerCommand($e->getMessage());
				return false;
			}
		}
	}
	
	class scWishlistListAction extends miAction {
		
		public function doAction()
		{
			try {
				if (!scCustomerAuth::singleton()->isLoggedIn()) {
					$this->_view->addRedirectToLoginControllerCommand();
					return false;
				}
				
				$productsSection = $this->getWishlistItemsSection();
				
				$t = new scTemplateParser();
				$t->readTemplate('modules/system/wishlist/wishlist.tmpl');
				$t->addSectionInfo($productsSection);
				$t->assignArray(scDynamicPages::getTemplateVars('module.php?ModuleName=com.summercart.wishlist'));
				
				$msg = new miMessage();
				$t->assignArray($msg->getMessageTemplateVars());
				$t->addSectionInfo(new miTemplateParserSectionInfo('hasMessage', (int)$msg->hasMessage()));
				
				$html = $t->templateParse();
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $e) {
				$this->_view->addRedirectToErrorControllerCommand($e->getMessage());
				return false;
			}
		}
		
		protected function getWishlistItemsSection()
		{
			$hideOutOfStockSql = scProductService::hideOutOfStock() ? 'AND ProductQuantity > 0 ' : '';
			
			$customerId = scCustomerAuth::singleton()->getCustomerId();
			$languageId = scMultiLanguage::getCurrentLanguageId();
			
			$sql = 'SELECT *, Products.ProductID AS ProductID' .
				' FROM WishlistItems' .
				' INNER JOIN Products USING(ProductID)' .
				' LEFT JOIN ProductsML USING(ProductID)' .
				' LEFT JOIN ProductImages ON(Products.ProductID = ProductImages.ProductID AND ProductImageIsDefault<>0)' .
				' LEFT JOIN ProductVariants ON (' .
					'(WishlistItems.ProductVariantID = ProductVariants.ProductVariantID)' .
					' OR (WishlistItems.ProductVariantID IS NULL AND WishlistItems.ProductID = ProductVariants.ProductID AND ProductVariantIsDefault<>0)) ' .
				' WHERE LanguageID = ' . scMultiLanguage::getCurrentLanguageId() .
				' AND CustomerID = ' . $customerId .
				' AND ProductIsActive <> 0 ' . $hideOutOfStockSql .
				' ORDER BY WishlistItemAddedTimestamp DESC';
			
			$products = miStaticDBUtil::execSelect($sql);
			
			// Build listing items (implementations of IProductListingItem)
			$listingItems = array();
			foreach ($products as $item) {
				if (!empty($item['ProductVariantID'])) {
					$listingItem = new scProductVariant();
				} else {
					$listingItem = new scProductSL();
				}
				$listingItem->setRow($item);
				$listingItems[] = $listingItem;
			}
			
			// Use scProductListingService instead of scProductService so:
			// 1. Out-of-stock variants are not replaced with such that are in stock.
			// 2. Include wishlist items data in the section info.
			// 3. It provides more flexibility with its options parameter
			$section = scProductListingService::singleton()->getSection('products', $listingItems, scProductService::VIEW_LIST);
			
			if (count($products) == 0) {
				return $section;
			}
			
			// Get options
			$sql = 'SELECT WishlistItems.WishlistItemID, Options.OptionID, Options.OptionGroupID, OptionsML.OptionName, OptionGroupsML.OptionGroupName' .
				' FROM WishlistItems' .
				' INNER JOIN WishlistItemOptions USING(WishlistItemID)' .
				' INNER JOIN Options USING(OptionID)' .
				' LEFT JOIN OptionsML ON Options.OptionID = OptionsML.OptionID AND OptionsML.LanguageID = ?' .
				' INNER JOIN OptionGroups ON Options.OptionGroupID = OptionGroups.OptionGroupID' .
				' LEFT JOIN OptionGroupsML ON OptionGroups.OptionGroupID = OptionGroupsML.OptionGroupID AND OptionGroupsML.LanguageID = ? ' .
				' WHERE CustomerID = ?';
			$options = miStaticDBUtil::execSelect($sql, array($languageId, $languageId, $customerId));
			$optionsArray = array();
			foreach ($options as $option) {
				$optionsArray[$option['WishlistItemID']][] = $option;
			}
			
			// Get local options
			$sql = 'SELECT WishlistItems.WishlistItemID, LocalOptions.LocalOptionID, LocalOptions.LocalOptionGroupID, LocalOptionsML.LocalOptionName, LocalOptionGroupsML.LocalOptionGroupName' .
				' FROM WishlistItems' .
				' INNER JOIN WishlistItemLocalOptions USING(WishlistItemID)' .
				' INNER JOIN LocalOptions USING(LocalOptionID)' .
				' LEFT JOIN LocalOptionsML ON LocalOptions.LocalOptionID = LocalOptionsML.LocalOptionID AND LocalOptionsML.LanguageID = ?' .
				' INNER JOIN LocalOptionGroups ON LocalOptions.LocalOptionGroupID = LocalOptionGroups.LocalOptionGroupID' .
				' LEFT JOIN LocalOptionGroupsML ON LocalOptionGroups.LocalOptionGroupID = LocalOptionGroupsML.LocalOptionGroupID AND LocalOptionGroupsML.LanguageID = ? ' .
				' WHERE CustomerID = ?';
			$options = miStaticDBUtil::execSelect($sql, array($languageId, $languageId, $customerId));
			$localOptionsArray = array();
			foreach ($options as $option) {
				$localOptionsArray[$option['WishlistItemID']][] = $option;
			}
			
			// Build options sections
			$optionsSectionArray = array();
			$localOptionsSectionArray = array();
			foreach ($products as $product) {
				$itemId = $product['WishlistItemID'];
				
				$optionsSection = new miTemplateParserSectionInfo('options', 0);
				if (isset($optionsArray[$itemId])) {
					$optionsSection->setIterationsCount(count($optionsArray[$itemId]));
					$optionsSection->setValues(miUIUtil::templetizeRows(miUIUtil::transposeRows($optionsArray[$itemId])));
				}
				
				$localOptionsSection = new miTemplateParserSectionInfo('options', 0);
				if (isset($localOptionsArray[$itemId])) {
					$localOptionsSection->setIterationsCount(count($localOptionsArray[$itemId]));
					$localOptionsSection->setValues(miUIUtil::templetizeRows(miUIUtil::transposeRows($localOptionsArray[$itemId])));
				}
				
				$optionsSectionArray[] = $optionsSection;
				$localOptionsSectionArray[] = $localOptionsSection;
			}
			
			$section->_subsections['products']->addSubsectionsArray('options', $optionsSectionArray);
			$section->_subsections['products']->addSubsectionsArray('localOptions', $localOptionsSectionArray);
			
			return $section;
		}
	}
	
	class scWishlistSendAction extends scWishlistListAction {
		
		public function doAction()
		{
			try {
				if (!scCustomerAuth::singleton()->isLoggedIn()) {
					$this->_view->addRedirectToLoginControllerCommand();
					return false;
				}
				
				$sendTo = miGetParam('email');
				$sendTo = explode(',', $sendTo);
				$sendTo = array_filter($sendTo);
				$to = array();
				foreach ($sendTo as $key => $email) {
					if (miValidatorEmail::isValid($email)) {
						$to[] = new scEmailAddress($email);
					}
				}
				
				if (empty($to)) {
					throw new miException(_t('Invalid email format.'));
				}
				
				scProductListingService::singleton()->setForceAbsoluteUrls(true);
				$productsSection = $this->getWishlistItemsSection();
				scProductListingService::singleton()->setForceAbsoluteUrls(false);
				
				$t = new scTemplateParser();
				$t->readTemplate('modules/system/wishlist/wishlist_email.tmpl');
				$t->addSectionInfo($productsSection);
				$html = $t->templateParse();
				
				$subject = _t('Here is my wishlist');
				
				$email = new scEmail();
				$email->setFrom(new scEmailAddress(scCustomerAuth::singleton()->getCustomer()->get('CustomerEmail')));
				$email->setTo($to);
				$email->setSubject($subject);
				if (!$email->sendHtmlEmail($html)) {
					throw new miException(_t('E-mail sending failed.'));
				}
				
				$loc = scWishlistUserModule::getWishlistPageUrl();
				$loc = scUrlUtil::addQueryParams($loc, miMessage::PARAM_MESSAGE . '=' . urlencode(_t('Wishlist sent.')));
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $e) {
				$this->_view->addRedirectToErrorControllerCommand($e->getMessage());
				return false;
			}
		}
	}
	
	class scWishlistItem extends scDomainObject {
		
		protected $_options = array();
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('WishlistItems', 'WishlistItemID'));
		}
		
		/**
		 * @param array $options
		 */
		public function setOptions($options)
		{
			// Check options validity and filter out in-byndle options.
			$this->_options = array();
			foreach ($options as $optionType => $optionsOfType) {
				$this->_options[$optionType] = array();
				foreach ($optionsOfType as $optionGroupId => $optionIds) {
					if (('' . $optionGroupId) != ('' . intval($optionGroupId))) {
						continue;
					}
					$this->_options[$optionType][$optionGroupId] = $optionIds;
				}
			}
		}
		
		public function insert()
		{
			miStaticDBUtil::startTransaction();
			$this->set('WishlistItemAddedTimestamp', time());
			$itemId = parent::insert();
			
			$multiValues = array();
			foreach ($this->_options['LocalOptions'] as $option) {
				if (!is_array($option) || count($option) <> 1) {
					throw new miException(_i('Options and local options are expected as arrays with one element only.'));
				}
				
				$option = reset($option);
				$multiValues[] = array(
					'WishlistItemID' => $itemId,
					'LocalOptionID' => (int)$option,
				);
			}
			
			if (!empty($multiValues)) {
				miStaticDBUtil::execMultiInsert('WishlistItemLocalOptions', $multiValues, miDBUtilImpl::FLAG_IGNORE);
			}
			
			$multiValues = array();
			foreach ($this->_options['ProductOptions'] as $option) {
				if (!is_array($option) || count($option) <> 1) {
					throw new miException(_i('Options and local options are expected as arrays with one element only.'));
				}
				
				$option = reset($option);
				$multiValues[] = array(
					'WishlistItemID' => $itemId,
					'OptionID' => (int)$option,
				);
			}
			
			if (!empty($multiValues)) {
				miStaticDBUtil::execMultiInsert('WishlistItemOptions', $multiValues, miDBUtilImpl::FLAG_IGNORE);
			}
			
			miStaticDBUtil::commit();
			
			return $itemId;
		}
		
		/**
		 * Returns if this wishlist item has already been added to the wishlist
		 * 
		 * @return int|false the WishlistItemID if it has been added, or false if not
		 */
		public function isAlreadyAdded()
		{
			$items = array();
			
			if ($this->has('ProductVariantID')) {
				// Find items for the same variant.
				$sql = 'SELECT WishlistItemID FROM WishlistItems WHERE ProductVariantID = ? AND CustomerID = ?';
				$items = miStaticDBUtil::execSelect($sql, array($this->get('ProductVariantID'), scCustomerAuth::singleton()->getCustomerId()));
			}
			
			// Find items for the same product
			$sql = 'SELECT WishlistItemID FROM WishlistItems WHERE ProductID = ? AND CustomerID = ?';
			$items = array_merge($items, miStaticDBUtil::execSelect($sql, array($this->get('ProductID'), scCustomerAuth::singleton()->getCustomerId())));
			
			if (empty($items)) {
				return false;
			}
			
			$items = miUtil::getKeyValueArray($items, 'WishlistItemID', 'WishlistItemID');
			
			// Match items by global options
			if (count($this->_options['ProductOptions']) > 0) {
				$checkOptions = array();
				foreach ($this->_options['ProductOptions'] as $optionGroupId => $selectedOptions) {
					$checkOptions = array_merge($checkOptions, $selectedOptions);
				}
				
				if (!empty($checkOptions)) {
					$sql = 'SELECT WishlistItemID, COUNT(OptionID) AS OptionsCount' .
						' FROM WishlistItemOptions' .
						' WHERE WishlistItemID IN (' . implode(', ', $items) . ')' .
						' AND OptionID IN (' . implode(', ', array_map('intval', $checkOptions)) . ')' .
						' GROUP BY WishlistItemID HAVING OptionsCount = ' . count($checkOptions);
					$items = miStaticDBUtil::execSelect($sql);
					$items = miUtil::getKeyValueArray($items, 'WishlistItemID', 'WishlistItemID');
					
					if (empty($items)) {
						return false;
					}
				}
			}
			// Match items by local options
			if (count($this->_options['LocalOptions']) > 0) {
				$checkOptions = array();
				foreach ($this->_options['LocalOptions'] as $optionGroupId => $selectedOptions) {
					$checkOptions = array_merge($checkOptions, $selectedOptions);
				}
				
				if (!empty($checkOptions)) {
					$sql = 'SELECT WishlistItemID, COUNT(LocalOptionID) AS OptionsCount' .
						' FROM WishlistItemLocalOptions' .
						' WHERE WishlistItemID IN (' . implode(', ', $items) . ')' .
						' AND LocalOptionID IN (' . implode(', ', array_map('intval', $checkOptions)) . ')' .
						' GROUP BY WishlistItemID HAVING OptionsCount = ' . count($checkOptions);
					$items = miStaticDBUtil::execSelect($sql);
					$items = miUtil::getKeyValueArray($items, 'WishlistItemID', 'WishlistItemID');
					
					if (empty($items)) {
						return false;
					}
				}
			}
			
			return reset($items);
		}
	}
	
	class scWishlistSeo extends scDynamicPageSeo {
		
		public function __construct()
		{
			parent::__construct('module.php?ModuleName=com.summercart.wishlist');
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			$this->_pageTitle = _t('Wishlist');
		}
	}
?>