<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	scDateTime::setTimezone();
	
	class scUserPageViewOrderEgoods extends miDefaultView {
		
		public function __construct()
		{
			// Create the view without recordset
			parent::__construct(null);
		}
		
		/**
		 * @return array
		 */
		public function getControllerCommands()
		{
			try {
				$orderId = (int)miGetParamDefault('OrderID', 0);
				if ($orderId < 1)
					return $this->displayErrorPage(_t('Access denied.'));
				
				$order = new scOrder($orderId);
				
				if (scCustomerAuth::singleton()->isLoggedIn()) {
					$customer = scCustomerAuth::singleton()->getCustomer();
					if ($order->get('CustomerID') != $customer->get('CustomerID')) {
						return $this->displayErrorPage(_t('Access denied.'));
					}
				} else if (miGetParamDefault('AccessKey', '') !== $order->getAccessKey()) {
					$loginUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN);
					$loginUrl = scUrlUtil::addQueryParams($loginUrl, 'Referer=' . urlencode('order_egoods.php?OrderID=' . $orderId));
					return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loginUrl));
				}
				
				$egoods = $order->getEgoods();
				$currency = $order->getBaseCurrency();
				$orderItemsSection = scCustomerOrdersUI::singleton()->getOrderItemsSection($order, scOrdersUI::ORDER_ITEMS_WITH_IMAGES, $egoods, scOrdersUI::ORDER_ITEMS_EGOODS_REQUIRED);
				$message = new miMessage();
				
				$t = new scTemplateParser();
				$t->setSectionInfos(array($orderItemsSection));
				$t->readTemplate('customer/myaccount/order_egoods.tmpl');
				$this->addStaticSection($t);
				$t->assignArray(miUIUtil::templetizeRows($order->getRow()));
				$t->assignArray(miUIUtil::templetizeRows(array('AccessKey' => $order->getAccessKey())));
				$t->assignArray($message->getMessageTemplateVars());
				
				// Assign Addresses Count and Orders Count
				$t->assignArray(array(
					'%%CUSTOMER_ADDRESSES_COUNT%%' => scCustomerAddress::getAddressesCountByCustomerId($_SESSION['CustomerID']),
					'%%CUSTOMER_ORDERS_COUNT%%' => scCustomerOrdersUI::getOrdersCountByCustomerId($_SESSION['CustomerID'])
				));
				
				$orderDetailsHtml = $t->templateParse();
				
				return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $orderDetailsHtml));
				
			} catch (miException $exception) {
				return $this->displayErrorPage(_t('Access denied.'));
			}
		}
		
		protected function addStaticSection($template)
		{
			if (miGetParamDefault('load', '') != 'table') {
				$template->addSectionInfo(new miTemplateParserSectionInfo('static', 1));
			}
		}
		
		protected function displayErrorPage($msg)
		{
			$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($msg);
			return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
		}
	}
	
	class scOrderEgoodsSeo extends scSeo {
		
		protected function addBodyClasses()
		{
			parent::addBodyClasses();
			$this->_bodyClasses[] = 'dynamic-page-order-egoods';
		}
	}
	
	$view = new scUserPageViewOrderEgoods();
	$page = new scUserDisplayController(array($view));
	$page->setSeo(new scOrderEgoodsSeo(_t(sprintf('E-Goods for order %d ', miGetParamDefault('OrderID', 0)))));
	$page->showPage();
?>