<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
		
	class scUserPageOrders implements ICMSBlock {
		public function getControllerCommands()
		{
			$customer = scCustomerAuth::singleton()->getCustomer();
			
			if (!scCustomerAuth::singleton()->isLoggedIn() or ($customer === false)) {
				$loginUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN);
				$ordersUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_ORDERS);
				$loginUrl = scUrlUtil::addQueryParams($loginUrl, 'Referer=' . urlencode($ordersUrl));
				return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loginUrl));
			}
			
			$ordersRs = new miSqlRecordset('Orders');
			$ordersRs->addFilter(new miSqlFilterEqual('CustomerID', $customer->get('CustomerID')));
			$ordersRs->addFilter(new miSqlFilterNotEqual('OrderStatus', scOrderStatus::ORDER_STATUS_UNFINISHED));
			$ordersRs->setOrder('OrderTimestamp', 'DESC');
			$orders = $ordersRs->getAllRecords();
			
			scDateTime::setTimezone();
			foreach ($orders as &$order) {
				$order['OrderStatusText'] = scOrderStatuses::singleton()->getOrderStatusName($order['OrderStatus']);
				$order['OrderDate'] = scDateTime::getLocalFormattedTimestamp($order['OrderTimestamp']);
				
				// TODO: V1.1 Optimize scCurrency objects creation
				try {
					$currency = new scCurrency($order['OrderCurrencyCode']);
					$amount = scMultiCurrencyAmount::factory();
					$amount->set($currency, $order['OrderTotal']);
					$order['OrderTotal'] = scPrice::factory($amount)->getDisplayPrice($currency);
				} catch (miDBException $e) {
					if ($e->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
						$order['OrderTotal'] = $order['OrderCurrencyCode'] . ' ' . $order['OrderTotal'];
					}
				}
				try {
					$currency = new scCurrency($order['OrderCustomerCurrencyCode']);
					$amount = scMultiCurrencyAmount::factory();
					$amount->set($currency, $order['OrderCustomerCurrencyTotal']);
					$order['OrderCustomerCurrencyTotal'] = scPrice::factory($amount)->getDisplayPrice($currency);
				} catch (miDBException $e) {
					if ($e->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
						$order['OrderCustomerCurrencyTotal'] = $order['OrderCustomerCurrencyCode'] . ' ' . $order['OrderCustomerCurrencyTotal'];
					}
				}
			}
			
            $ordersSection = new miTemplateParserSectionInfo();
            if (count($orders) == 0)
            	$ordersSection->setSectionInfo('noorders', 1);
            else
				$ordersSection->setSectionInfo('orders', count($orders), miUIUtil::templetizeRows(miUIUtil::transposeRows($orders)));
            
			$sections = array($ordersSection);
			$t = new scTemplateParser();
			
			$t->readTemplate('customer/myaccount/orders.tmpl');
			$t->setSectionInfos($sections);

			// Assign dynamic page areas values
			$t->assignArray(scDynamicPages::getTemplateVars('orders.php'));
			
			// Assign Addresses Count and Orders Count
			$t->assignArray(array(
				'%%CUSTOMER_ADDRESSES_COUNT%%' => scCustomerAddress::getAddressesCountByCustomerId($_SESSION['CustomerID']),
				'%%CUSTOMER_ORDERS_COUNT%%' => scCustomerOrdersUI::getOrdersCountByCustomerId($_SESSION['CustomerID'])
			));

			$html = $t->templateParse();
			
			return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
		}
	}
	
	$userPageOrders = new scUserPageOrders();
	$page = new scUserDisplayController(array($userPageOrders));
	$page->setSeo(new scDynamicPageSeo('orders.php', _t('My Orders')));
	$page->showPage();
?>