<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	scSession::singleton()->openSession();
	
	class scUserProductView extends scView {
		public function __construct()
		{
			parent::__construct(null);
		}
	}
	
	class scShowProductAction extends miAction {
		public function doAction()
		{
			try {
				// Define $productVariant
				$productVariant = null;
				if (!miHasParam('ProductVariantID')) {
					// There's no ProductVariant ID so we go to normal ProductID
					$productId = (int)miGetParam('ProductID');
				} else {
					// Try to find the product variant requested by ProductVariantID
					$productVariantId = (int)miGetParam('ProductVariantID');
					$productVariant = new scProductVariant();
					try {
						// The variant may not exist so miDBException is expected.
						$productVariant->read($productVariantId);
						if (!miHasParam('ProductID') || (miGetParam('ProductID') == $productVariant->get('ProductID'))) {
							// There is no ProductID or the product id passed through the request is the same as the variant's product id.
							$productId = (int)$productVariant->get('ProductID');
						} else {
							// There is a ProductID and it defers from the one the found variant is for.
							// Override the variant selection and open the product.
							$productId = (int)miGetParam('ProductID');
							$productVariant = null;
						}
					} catch (miDBException $e) {
						if ($e->getCode() !== miDBException::EXCEPTION_RECORD_NOT_FOUND) {
							throw $e;
						}
						// No variant was found so we go to normal ProductID
						$productId = (int)miGetParam('ProductID');
						$productVariant = null;
					}
				}
				
				$product = new scProductSL();
				$product->read($productId);
				
				// Init breadcrumb section
				$categoriesBreadcrumbSection = scUIUtil::getCategoriesBreadcrumb($product->getCategoryId());
				$productSection = scProductService::getSingleProductSection($product, $productVariant, miGetParamDefault('Options', array()));
				
				// Init the template parser
				$t = new scTemplateParser();
				$t->readTemplate('customer/product.tmpl');
				
				// Add the sections to the parser object
				$productSection->addSubsection($categoriesBreadcrumbSection);
				$t->addSectionInfo($productSection);
				
				// Assign dynamic page areas values
				$t->assignArray(scDynamicPages::getTemplateVars('product.php'));
				
				// Parse the template
				$html = $t->templateParse();
				
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
				
				$this->countProductView($product);
				
			} catch (miException $exception) {
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($exception->getMessage());
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
			}
			
			return true;
		}
		
		protected function countProductView(scProductSL $product)
		{
			$productId = $product->getProductId();
			
			// Add the product to recently viewed box
			$recentlyViewedBox = scModulesFactory::loadCmsBoxClassByName('scCustomerRecentlyViewedBox');
			if ($recentlyViewedBox instanceof scCustomerRecentlyViewedBox) {
				$recentlyViewedBox->addProductId($productId);
			}
			
			if (!isset($_SESSION['productViewsCount']))
				$_SESSION['productViewsCount'] = array();
			
			if (in_array($productId, $_SESSION['productViewsCount'])) {
				return '';
			}
			
			$_SESSION['productViewsCount'][] = $productId;
			
			$q = 'INSERT INTO StatProductViews (ProductID, StatProductViewsDate, StatProductViewsCount) VALUES' .
				' (?, ?, 1) ON DUPLICATE KEY UPDATE StatProductViewsCount = StatProductViewsCount + 1';
			miStaticDBUtil::execSQL($q, array($productId, date(MI_MYSQL_DATE_FORMAT)));
		}
	}
	
	class scProductVariantDetailsJson extends miAction {
		
		public function doAction()
		{
			$output = array(
				'PricesTable' => '',
				'WholesalePricesTab' => '',
				'DefaultImageHtml' => '',
				'OptionTexts' => array(),
				'isOutOfStock' => false,
				'hasAddToCart' => true,
				'error' => '',
			);
			
			try {
				$variantId = (int)miGetParam('ProductVariantID');
				
				$variant = new scProductVariant();
				$variant->read($variantId);
				
				$output['PricesTable'] = scProductService::getProductPriceHtml($variant);
				$output['WholesalePricesTab'] = scProductService::getWholesalePricesHtml($variant);
				$output['DefaultImageHtml'] = scProductService::getProductVariantImage($variant);
				
				if ($variant->isOutOfStock()) {
					$output['isOutOfStock'] = true;
					$output['hasAddToCart'] = false;
				}
				
				$output['OptionTexts'] = $this->getOptionTexts($variant);
				
				$result = true;
			} catch (miException $e) {
				$output['error'] = _i('Unable to obtain product variant prices table.');
				
				$result = false;
			}
			
			$output = scJson::jsonEncode($output);
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $output));
			return $result;
		}
		
		protected function getOptionTexts(scProductVariant $variant)
		{
			$productOptions = $variant->getProductSL()->getOptionsData();
			
			$optionTexts = array();
			$productId = $variant->getProductId();
			$price = scWholesalePrices::getListingItemWholesalePrice($variant)->getAmount();
			foreach ($productOptions as $optionType => $options) {
				foreach ($options as $optionGroupId => $optionGroup) {
					if ($optionGroup['OptionGroupType'] != scProductOptions::GROUP_TYPE_PRICE_MODIFIER)
						continue;
					foreach ($optionGroup['Options'] as $optionId => $option) {
						$optionInfo = $option;
						$optionInfo['OptionID'] = $optionId;
						$optionInfo['OptionGroupID'] = $optionGroupId;
						$optionInfo['OptionGroupName'] = $optionGroup['OptionGroupName'];
						$optionInfo['OptionGroupType'] = $optionGroup['OptionGroupType'];
						$optionObject = new scProductOption($optionId, $optionInfo);
						
						$priceChange = $optionObject->getOptionPriceChange($price);
						$displayPricesTaxMode = scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE);
						if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
							$priceChange = scProductPrice::getTaxedAmount($priceChange, $productId);
						}
						$priceChange = scPrice::factory($priceChange);
						$priceChange = $priceChange->getDisplayPrice();
						// Construct the price variation
						if ($option['OptionPriceModValue'] != 0) {
							if ($option['OptionPriceModValue'] > 0) {
								$priceChange = '+ ' . $priceChange;
							}
							if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_AND_WITHOUT_TAX) {
								$priceChange .= ' ' . _t('without tax');
							}
							$option['OptionName'] .= ' (' . $priceChange . ')';
						}
						
						$optionTexts[$optionType][$optionGroupId][$optionId] = $option['OptionName'];
					}
				}
			}
			return $optionTexts;
		}
	}
	
	$page = new scUserDisplayController(array(new scUserProductView()));
	$page->setSeo(new scProductSeo());
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scShowProductAction');
	$page->getDispatcher()->setActionHandler('variantDetailsJson', 'scProductVariantDetailsJson');
	$page->dispatchAndShow();
?>