<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scUserPageProfile extends miDefaultView implements ICMSBlock {
	    protected $_templates = array(
			self::TEMPLATE_ID_EDIT => 'customer/myaccount/profile.tmpl',
		);
		
		protected $_dataFields = array(
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'CustomerID'
 			),
       		array(
				'field' => 'scProfilePasswordWidget',
				'data' => 'CustomerPassword',
       		),
       		array(
       			'field' => 'scNewslettersWidget',
       			'data' => 'Newsletters',
       			'modelStrategy' => 'scViewMapperNewsletters',
       			'properties' => array(
					'template' => 'customer/widgets/newsletters_widget.tmpl'
				),
       		),
       		array(
       			'field' => 'miBaseTextWidget',
       			'data' => 'CustomerBonusPoints',
       			'properties' => array(
       				'readOnly' => true
       			),
       		),
        );
       
        protected function createDomainObject()
		{
			return new scCustomerDomainObject();
		}
 		
		protected function addTableFeatures($table)
		{
			// Overload parent and do nothing
		}
		
		public function initWebForm(&$form)
		{
			parent::initWebForm($form);
			
			$customer = $this->getDomainObject();
			$customer->read($_SESSION['CustomerID']);
			$this->addMainPageElements(array('%%CUSTOMEREMAIL%%' => miI18N::htmlEscape($customer->get('CustomerEmail'))));
			
			// Assign formatted billing and shipping addresses
			$billingAddress = '';
			$address = $customer->getBillingAddress();
			if (!empty($address)) {
				$billingAddress = scCustomerAddress::getFormattedAddress($address);
			}
			$address = $customer->getShippingAddress();
			$shippingAddress = '';
			if (!empty($address)) {
				$shippingAddress = scCustomerAddress::getFormattedAddress($address);
			}
			
			$this->addMainPageElements(array(
				'%%HTML_CUSTOMERBILLINGADDRESS%%' => $billingAddress,
				'%%HTML_CUSTOMERSHIPPINGADDRESS%%' => $shippingAddress,
				'%%CUSTOMER_ADDRESSES_COUNT%%' => scCustomerAddress::getAddressesCountByCustomerId($_SESSION['CustomerID']),
				'%%CUSTOMER_ORDERS_COUNT%%' => scCustomerOrdersUI::getOrdersCountByCustomerId($_SESSION['CustomerID'])
			));
			
			// Assign dynamic page areas values
			$this->addMainPageElements(scDynamicPages::getTemplateVars('profile.php'));		
			
			$form->setErrorsHandler(new scByFieldErrorsHandler);
			
			$message = new miMessage();
			$form->addMainPageElements($message->getMessageTemplateVars());
		}
		
		public function getControllerCommands()
		{
			if (!scCustomerAuth::singleton()->isLoggedIn()) {
				$loginUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN);
				$profileUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_PROFILE);
				$loginUrl = scUrlUtil::addQueryParams($loginUrl, 'Referer=' . urlencode($profileUrl));
				return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loginUrl));
			}
			
			return parent::getControllerCommands();
		}
		
		public function addRedirectToListControllerCommand($msg, $msgType = miMessage::MSG_TYPE_ERROR)
		{
			if ($msgType == miMessage::MSG_TYPE_ERROR) {
				$loc = 'error.php?' . $this->getRedirectParams($msg, $msgType);
			} else {
				$loc = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_PROFILE);
				$loc = scUrlUtil::addQueryParams($loc, $this->getRedirectParams($msg, $msgType));
			}
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loc);
			$this->addControllerCommand($cmd);
		}
		
		public function getMessage($msgCode)
		{
			if ($msgCode == 'MI_RECORD_UPDATED_SUCCESSFULLY_MSG')
				return _t('Profile updated successfully.');
			
			return parent::getMessage($msgCode);
		}
		
		public function postProcess($actionObj, $actionResult)
		{
			parent::postProcess($actionObj, $actionResult);
			
			if (!$actionResult) {
				return;
			}
			
			if ($actionObj instanceof miExecEditAction) {
				scCart::getCart()->setCustomerInfo(scCustomerAuth::singleton()->getCustomer()->getRow());
			}
		}
	}
	
	class scProfilePasswordWidget extends miBaseTextWidget {
		function getData()
		{
			$password = miGetParamDefault($this->_fieldName, '');
			$passwordConfirm = miGetParamDefault($this->_fieldName . 'Confirm', '');
			if (($password == $passwordConfirm) and ($password != ''))
				return array($this->_fieldName => $password);
			else
				return array();
		}
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			$password = miGetParamDefault($this->_fieldName, '');
			$passwordConfirm = miGetParamDefault($this->_fieldName . 'Confirm', '');
			if ($password != $passwordConfirm)
				$errors->addError($this->_fieldName, _t('Passwords do not match!'));
		}
		
	}
	
	miSetParam('CustomerID', isset($_SESSION['CustomerID']) ? $_SESSION['CustomerID'] : 0);
	
	$view = new scUserPageProfile();
	
	$page = new scUserDisplayController(array($view));
	$page->setSeo(new scDynamicPageSeo('profile.php', _t('My Profile')));
	$page->getDispatcher()->setActionHandler('', 'miEditAction');
	$page->getDispatcher()->setActionHandler('acUpdateProfile', 'miExecEditAction');
	
	$page->dispatchAndShow();
?>