<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();

	class scUserSearchView extends scView {
		public function __construct()
		{
			$this->_persistVars = $this->_persistVars + array(
				scSearchData::KEY_CATEGORY_ID,
				scSearchData::KEY_MANUFACTURER_ID,
				scSearchData::KEY_PHRASE,
				scSearchData::KEY_PRODUCT_ATTRIBUTES,
				scSearchData::KEY_PRODUCT_CLASS_ID,
				scSearchData::KEY_PRODUCT_CODE,
				scSearchData::KEY_PRODUCT_DETAILS,
				scSearchData::KEY_PRODUCT_NAME,
				scSearchData::KEY_PRODUCT_OPTIONS,
				scSearchData::KEY_PRODUCT_PRICE_FROM,
				scSearchData::KEY_PRODUCT_PRICE_TO,
				scSearchData::KEY_SUBCATEGORY_SEARCH,
			);
			
			parent::__construct(null);
		}
	}
	
	class scSearchData {
		
		// Simple search fields
		const KEY_PHRASE = 'phrase';
		
		// Advanced search fields
		const KEY_PRODUCT_NAME = 'phrase';
		const KEY_PRODUCT_DETAILS = 'ProductDetails';
		const KEY_PRODUCT_CODE = 'ProductCode';
		const KEY_CATEGORY_ID = 'CategoryID';
		const KEY_SUBCATEGORY_SEARCH = 'SearchInSubcategory';
		const KEY_MANUFACTURER_ID = 'ManufacturerID';
		const KEY_PRODUCT_PRICE_FROM = 'ProductPriceFrom';
		const KEY_PRODUCT_PRICE_TO = 'ProductPriceTo';
		const KEY_PRODUCT_CLASS_ID = 'ProductClassID';
		const KEY_PRODUCT_ATTRIBUTES = 'ProductAttributes';
		const KEY_PRODUCT_OPTIONS = 'ProductOptions';
		
		public static function getSearchData()
		{
			return array(
				self::KEY_PHRASE => miGetParamDefault('phrase', '')
			);
		}
		
		public static function getAdvancedSearchData()
		{
			// Default value should be zero if the visitor is submitting the advanced search form
			$defaultValue = miHasParam('ProductCode') ? 0 : 1;
			
			$data = array(
				self::KEY_PRODUCT_NAME => miGetParamDefault('phrase', ''),
				self::KEY_PRODUCT_DETAILS => miGetParamDefault('ProductDetails', $defaultValue),
				self::KEY_PRODUCT_CODE => miGetParamDefault('ProductCode', ''),
				self::KEY_CATEGORY_ID => miGetParamDefault('CategoryID', 0),
				self::KEY_SUBCATEGORY_SEARCH => miGetParamDefault('SearchInSubcategory', $defaultValue),
				self::KEY_MANUFACTURER_ID => miGetParamDefault('ManufacturerID', 0),
				self::KEY_PRODUCT_PRICE_FROM => miGetParamDefault('ProductPriceFrom', ''),
				self::KEY_PRODUCT_PRICE_TO => miGetParamDefault('ProductPriceTo', ''),
				self::KEY_PRODUCT_CLASS_ID => miGetParamDefault('ProductClassID', 0),
				self::KEY_PRODUCT_ATTRIBUTES => miGetParamDefault('ProductAttributes', 0),
				self::KEY_PRODUCT_OPTIONS => miGetParamDefault('ProductOptions', 0),
			);
			
			// Normalize checkbox data
			$data[self::KEY_PRODUCT_DETAILS] = ($data[self::KEY_PRODUCT_DETAILS]) ? 1 : 0;
			$data[self::KEY_SUBCATEGORY_SEARCH] = ($data[self::KEY_SUBCATEGORY_SEARCH]) ? 1 : 0;

			return $data;
		}
	}
	
	class scSearchListingProductVariant extends scProductVariant {
		
		public function getUrl()
		{
			return scFriendlyUrl::get(scFriendlyUrl::TYPE_PRODUCT_VARIANT, $this->get('ProductVariantID'), $this->getName());
		}
	}

	class scUserSearchAction extends miAction {
		protected $_page;
		protected $_numPages;
		
		public function doAction()
		{
			$parser = new scTemplateParser();
			
			// Get the search data (the search phrase)
			$searchdata = http_build_query(scSearchData::getAdvancedSearchData());
			
			$parser->assign('%%SEARCHPARAMETERS%%', miI18N::htmlEscape($searchdata));
			$parser->assign('%%SELF_URL%%', scPaths::getUrl('search.php?action=dmExecAdvancedSearch&' . $searchdata));
			$modifyUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_SEARCH);
			$modifyUrl = scUrlUtil::addQueryParams($modifyUrl, $searchdata);
			$modifyUrl = scUrlUtil::addQueryParams($modifyUrl, 'action=dmAdvancedSearch');
			$parser->assign('%%MODIFY_URL%%', $modifyUrl);
			$parser->assign('%%HTML_SORT_PRODUCTS%%', scProductSortService::getSortDropdown());
			
			$viewType = scProductService::getViewType();
			$parser->addSectionInfo($this->getProductsSection($searchdata, $viewType));
			$parser->assignArray($this->getPagerVars());
			
			$parser->assignArray(scDynamicPages::getTemplateVars('search.php'));
			
			if ($viewType == scProductService::VIEW_GRID)
				$parser->readTemplate('customer/search/search_grid.tmpl');
			else
				$parser->readTemplate('customer/search/search.tmpl');
			
			$html = $parser->templateParse();			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			
			return true;
		}
		
		protected function getProductsSection($searchdata, $viewType)
		{
			$search = $this->getSearch();
			$search->addOrderBy(scProductSortService::getSortClause());
			$productsPerPage = scCustomerAreaPager::getProductsPerPage();
			list($this->_page, $this->_numPages) = scCustomerAreaPager::calculatePages('page', $search->getRecordsCount());
			$start = ($this->_page - 1) * $productsPerPage;
			
			// Get found products
			$listingItems = $search->getRecords($start, $productsPerPage);
			
			scProductService::replaceOutOfStockVariants($listingItems, 'scSearchListingProductVariant');
			
			return scProductListingService::singleton()->getSection('products', $listingItems, $viewType);
		}
		
		/**
		 * Returns the found products array
		 *
		 * @return scUserSearch
		 */
		public function getSearch()
		{
			$search = scUserSearch::singleton();
			
			$data = scSearchData::getSearchData();
			$data = reset($data);
			
			$search->addCriteria(scUserSearch::PRODUCT_NAME, array($data, 1));
			
			return $search;
		}
		
		/**
		 * Returns the pager variables
		 * 
		 * @return array
		 */
		public function getPagerVars()
		{
			$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_SEARCH);
			$url = scUrlUtil::addQueryParams($url, 'action=' . miGetParamDefault('action', '') . '&' . http_build_query(scSearchData::getAdvancedSearchData()));
			
			$view = scProductService::getViewType();
			if ($view == scProductService::VIEW_GRID) {
				$altView = scProductService::VIEW_LIST;
				$altViewString = 'list';
			} else {
				$altView = scProductService::VIEW_GRID;
				$altViewString = 'grid';
			}
			$altViewPage = scCustomerAreaPager::calculateAltViewPage($this->_page, $view, $altView);
			$altViewUrl = scUrlUtil::addQueryParams($url, 'page=' . $altViewPage . '&view=' . $altViewString);
			
			return array(
				'%%HTML_PAGER%%' => scCustomerAreaPager::getPaging($this->_page, $this->_numPages, $url),
				'%%PAGE%%' => $this->_page,
				'%%NUMPAGES%%' => $this->_numPages,
				'%%ALT_VIEW_URL%%' => $altViewUrl,
			);
		}
	}
	
	class scUserAdvancedSearchAction extends miAction {
		public function doAction()
		{
			// Init the webform
			$form = new miWebForm(array());
			$this->_view->initWebForm($form);
			
			// Init the form data
			$form->setFormDataRow(scSearchData::getAdvancedSearchData());
			
			// Product name widget
			$widget = new miTextWidget($form);
			$widget->setFieldName('phrase');
			$form->addWidget('phrase', $widget);
			
			// Product description checkbox
			$widget = new miCheckboxWidget($form);
			$widget->setFieldName('ProductDetails');
			$form->addWidget('ProductDetails', $widget);
			
			// Product code widget
			$widget = new miTextWidget($form);
			$widget->setFieldName('ProductCode');
			$form->addWidget('ProductCode', $widget);

			// Category select widget
			$widget = new scCategorySelect($form);
			$widget->setFieldName('CategoryID');
			$categories = $widget->getCategoriesTree(0);
			$categories[0] = _t('-- All --');
			$widget->setOptions($categories, 0);
			$form->addWidget('CategoryID', $widget);
			
			// Search in subcategories widget
			$widget = new miCheckboxWidget($form);
			$widget->setFieldName('SearchInSubcategory');
			$form->addWidget('SearchInSubcategory', $widget);
			
			// Manufacturer select widget
			$widget = new miSelectWidget($form);
			$widget->setFieldName('ManufacturerID');
			$widget->setOptions(array(0 => _t('-- All --')) + scUtil::getManufacturersArray(), miGetParamDefault('ManufacturerID', 0));
			$form->addWidget('ManufacturerID', $widget);
			
			// Product price from widget
			$widget = new miTextWidget($form);
			$widget->setFieldName('ProductPriceFrom');
			$form->addWidget('ProductPriceFrom', $widget);
			
			// Product price to widget
			$widget = new miTextWidget($form);
			$widget->setFieldName('ProductPriceTo');
			$form->addWidget('ProductPriceTo', $widget);
			
			// Product class widget
			$widget = new miSelectWidget($form, array());
			$widget->setFieldName('ProductClassID');
			$productClassId = (int)miGetParamDefault('ProductClassID', 0);
			$widget->setOptions(array(0 =>  _t('Any')) + scUtil::getProductClassesArray(), $productClassId);
			$form->addWidget('ProductClassID', $widget);
			
			// Product attributes widget
			$widget = new scProductAttributesSearchWidget($form, array());
			$widget->setFieldName('ProductAttributes');
			$widget->setProductClassId($productClassId);
			$widget->setTemplate('customer/search/advanced_search_product_attributes.tmpl');
			$form->addWidget('ProductAttributes', $widget);
			
			/*// Product options widget
			$widget = new scProductOptionsSearchWidget($form, array());
			$widget->setFieldName('ProductOptions');
			$widget->setTemplate('customer/search/advanced_search_product_options.tmpl');
			$form->addWidget('ProductOptions', $widget);*/
			
			list($symbolLeft, $symbolRight) = scCurrencyUtil::getCurrencySymbols(scCurrencyUtil::getCurrentCurrencyId());
			$form->addMainPageElements(miUIUtil::templetizeAndEscapeRows(array('CurrencySymbolLeft' => miI18N::htmlEscape($symbolLeft), 'CurrencySymbolRight' => miI18N::htmlEscape($symbolRight))));
			
			$form->addMainPageElements($this->_view->getMainPageElements());
			
			// Assign dynamic page areas values
			$form->addMainPageElements(scDynamicPages::getTemplateVars('search.php'));
			
			$html = $form->parse('customer/search/advanced_search.tmpl', true);
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
		}
	}
	
	class scUserExecAdvancedSearchAction extends scUserSearchAction {
		/**
		 * Returns the found products array
		 *
		 * @return array
		 */
		public function getSearch()
		{
			$search = scUserSearch::singleton();
			
			$data = scSearchData::getAdvancedSearchData();
			
			$search->addCriteria(scUserSearch::MANUFACTURER_ID, $data[scSearchData::KEY_MANUFACTURER_ID]);
			$search->addCriteria(scUserSearch::PRODUCT_CLASS_ID, $data[scSearchData::KEY_PRODUCT_CLASS_ID]);
			$search->addCriteria(scUserSearch::PRODUCT_CODE, $data[scSearchData::KEY_PRODUCT_CODE]);
			$search->addCriteria(scUserSearch::PRODUCT_NAME, array($data[scSearchData::KEY_PRODUCT_NAME], $data[scSearchData::KEY_PRODUCT_DETAILS]));
			$search->addCriteria(scUserSearch::PRODUCT_PRICE, array($data[scSearchData::KEY_PRODUCT_PRICE_FROM], $data[scSearchData::KEY_PRODUCT_PRICE_TO]));
			$search->addCriteria(scUserSearch::CATEGORY_ID, array($data[scSearchData::KEY_CATEGORY_ID], $data[scSearchData::KEY_SUBCATEGORY_SEARCH]));
			$search->addCriteria(scUserSearch::PRODUCT_ATTRIBUTES, $this->getAttributeFilters($data));
			$search->addCriteria(scUserSearch::PRODUCT_OPTIONS, $data[scSearchData::KEY_PRODUCT_OPTIONS]);
			
			return $search;
		}
		
		/**
		 * @param array $searchData
		 * @return array
		 */
		protected function getAttributeFilters($searchData)
		{
			$productClassId = (int)$searchData[scSearchData::KEY_PRODUCT_CLASS_ID];
			$attributeValues = $searchData[scSearchData::KEY_PRODUCT_ATTRIBUTES];
			
			return scProductAttributes::singleton()->getSqlFilters($productClassId, $attributeValues);
		}
	}
	
	class scUserSearchGetAttributesAction extends miAction {
		
		public function doAction()
		{
			$classId = (int)miGetParamDefault('ProductClassID', 0);
			$fieldName = 'ProductAttributes';
			$attributes = miGetParamDefault($fieldName, array());
			$template = 'customer/search/advanced_search_product_attributes.tmpl';
			
			$html = scProductAttributesSearchWidget::getProductClassAttributesHtml($classId, $attributes, $fieldName, $template, scProductAttributes::GET_SYSTEM_ATTRIBUTES_NONE);
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			return true;
		}
	}
	
	$searchView = new scUserSearchView();
	$page = new scUserDisplayController(array($searchView));
	$page->setSeo(new scDynamicPageSeo('search.php', _t('Search')));
	
	$page->getDispatcher()->clearActionHandlers();
	$page->getDispatcher()->setActionHandler('', 'scUserSearchAction');
	$page->getDispatcher()->setActionHandler('dmAdvancedSearch', 'scUserAdvancedSearchAction');
	$page->getDispatcher()->setActionHandler('dmExecAdvancedSearch', 'scUserExecAdvancedSearchAction');
	$page->getDispatcher()->setActionHandler('dmGetAttributes', 'scUserSearchGetAttributesAction');
	
	$page->dispatchAndShow();
?>