<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scSendToFriendView extends miDefaultView {
		public function __construct()
		{
			parent::__construct(null);
			
			$this->setRecord(new miSqlRecord('Customers', 'CustomerID'));
		}
		
		protected $_templates = array(
			self::TEMPLATE_ID_VIEW => 'customer/popup/send_to_friend.tmpl',
		);
		
		protected $_dataFields = array(
			array(
				'field' => 'miTextWidget',
				'data' => 'CustomerEmail',
				'validator' => array('scValidatorNoEmpty', 'scValidatorEmail'),
			),
			array(
				'field' => 'miTextWidget',
				'data' => 'CustomerName',
				'validator' => 'scValidatorNoEmpty'
			),
			array(
				'field' => 'miTextWidget',
				'data' => 'CustomerFriendEmail',
				'validator' => array('scValidatorNoEmpty', 'scValidatorEmail'),
			),
			array(
				'field' => 'scWebFormWidgetTextarea',
				'data' => 'CustomerEmailNotes',
				'properties' => array('id' => 'emailNotes')
			)
		);
	}
	
	class scSendToFriendAction extends miActionWithWebForm {
		/**
		 * @var miWebFormMessageErrorsHandler $_errorsHandler
		 */
		protected $_errorsHandler = null;
		
		public function __construct(miView $view)
		{
			parent::__construct($view);
			
			$this->_errorsHandler = new scByFieldErrorsHandler();
		}
		
		public function doAction()
		{
			$form = $this->getWebForm();
			$form->setErrorsHandler($this->_errorsHandler);
			
			$form->addMainPageElements(array('%%PRODUCTID%%' => (int)miGetParam('ProductID'), '%%STORE_ROOT_URL%%' => miI18N::htmlEscape('http://' . scPaths::getHttpHost() . scPaths::getStoreWebPath())));
			
			$html = $form->parse($this->_view->getTemplateFileName(scSendToFriendView::TEMPLATE_ID_VIEW), true);
			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
			return true;
		}
	}
	
	class scExecSendToFriendAction extends scSendToFriendAction {
		public function doAction()
		{
			try {
				$form = $this->getWebForm();
				$form->setErrorsHandler($this->_errorsHandler);
	
				if ($form->processSubmit()) {
					if ($this->sendEmail()) {
						// Mail sent successfully, close the lightbox
	//					$this->_errorsHandler->addError(_t('Success'), _t('Your email was successfully sent.'));
						$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('closeThickbox'));
					}
					else  
						$this->_errorsHandler->addError(_t('Error'), _t('An error occurred while sending your email.'));
				}
				
				$form->setFormDataRow($form->getSubmittedDataRow());
				
				parent::doAction();
				return false;
			} catch (Exception $exception) {
				header('Location: ' . scPaths::getUrl('error.php') . '?msg=' . urlencode($exception->getMessage()));
			}
		}
		
		/**
		 * Send the email using scEmailNotifications
		 */
		protected function sendEmail()
		{
			$customerEmail = miGetParam('CustomerEmail');
			$customerName = miGetParam('CustomerName');
			$friendEmail = miGetParam('CustomerFriendEmail');
			$emailNotes = miGetParam('CustomerEmailNotes');
			$productId = (int)miGetParam('ProductID');
			
			$notifications = new scEmailNotifications;
			return $notifications->sendToFriend($customerEmail, $customerName, $friendEmail, $emailNotes, $productId);
		}
	}
	
	$view = new scSendToFriendView();
	$page = new miPage(array($view));
	$page->getDispatcher()->setActionHandler('', 'scSendToFriendAction');
	$page->getDispatcher()->setActionHandler('dmExecSendToFriend', 'scExecSendToFriendAction');
	$page->dispatchAndShow();
?>