// Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.

function constructSCRS()
{
	this.separator = "::";
	this.addRowCallbackFunc = null;
	this.removeRowCallbackFunc = null;

	/**
	 * Add row number to element names and ids
	 */
	this.addSectionIds = function(node, RSName, rowNum) {
		if (!node.hasChildNodes())
			return;
		
		var replaceStr = this.separator + rowNum + this.separator;
		var re = new RegExp(this.separator + "([0-9]*)" + this.separator, "gi");
		
		for (var i = 0; i < node.childNodes.length; i++) {
			var childNode = node.childNodes[i];
			
			this.addSectionIds(childNode, RSName, rowNum);
			
			if (childNode.getAttribute == undefined)
				continue;
			
			// To display the row number in HTML, use <span rstype="rownum">%%ROWNUM%%</span>
			if (childNode.getAttribute('rstype') == 'rownum')
				childNode.innerHTML = rowNum;
			
			if (childNode.getAttribute("id")) {
				childNode.setAttribute("id", childNode.getAttribute("id").replace(re, replaceStr));
			}
			if (childNode.getAttribute("name")) {
				childNode.setAttribute("name", childNode.getAttribute("name").replace(re, replaceStr));
			}
			if (childNode.getAttribute("for")) {
				childNode.setAttribute("for", childNode.getAttribute("for").replace(re, replaceStr));
			}
		}
		return;
	}
	
	/**
	 * Renumber element ids and names on rowNum
	 */
	this.renumberSectionIds = function(re, node, rowNum, step) {
		if (!node.hasChildNodes())
			return;

		var replaceStr = this.separator+(rowNum+step)+this.separator;
		for (var i = 0; i < node.childNodes.length; i++) {
			if (typeof node.childNodes[i].id != 'undefined' && node.childNodes[i].id) {
				node.childNodes[i].id = String(node.childNodes[i].id.replace(re, replaceStr))
			}				
			if (typeof node.childNodes[i].name != 'undefined' && node.childNodes[i].name) {
				node.childNodes[i].name = String(node.childNodes[i].name.replace(re, replaceStr))
			}
			// To display the row number in HTML, use <span rstype="rownum">%%ROWNUM%%</span>
			if (node.childNodes[i].getAttribute) {
				if (node.childNodes[i].getAttribute('rstype') == 'rownum') {
					node.childNodes[i].innerHTML = rowNum+step;
				}
			}

			this.renumberSectionIds(re, node.childNodes[i], rowNum, step);
		}
	}
		
	
	/**
	 * Decrement (renumber) element ids and names on rowNum
	 */
	this.decrementSectionIds = function(re, node, rowNum) {
		this.renumberSectionIds(re, node, rowNum, -1);
	}
	
	/**
	 * Increase (renumber) element ids and names on rowNum
	 */
	this.incrementSectionIds = function(re, node, rowNum) {
		this.renumberSectionIds(re, node, rowNum, 1);
	}
	
	// To be called after the section DOM is ready
	this.initRS = function(RSName, addRowCallbackFunc, removeRowCallbackFunc) {
		if (typeof addRowCallbackFunc == 'function') {
			this.addRowCallbackFunc = addRowCallbackFunc;
		}
		if (typeof removeRowCallbackFunc == 'function') {
			this.removeRowCallbackFunc = removeRowCallbackFunc;
		}
		
		var sectionRowsCount = parseInt(document.getElementById(RSName+"__rowCount").value);
		
		this.maintainMinNumOfRows(RSName, sectionRowsCount);
	}

	this.removeRSRow = function(RSName, element) {
		var sectionRowsCount = parseInt(document.getElementById(RSName+'__rowCount').value);

		// Find out the number of the row
		var rowNum = this.getRowNum(RSName+'_removelink', element);

		var row = document.getElementById(RSName + '_row' + this.separator + rowNum + this.separator);
		sectionRowsCount--;
		document.getElementById(RSName+'__rowCount').value = sectionRowsCount;
		row.parentNode.removeChild(row);
		
		// Renumber sections after the removed
		var re = new RegExp(this.separator + "([0-9]+)" + this.separator, "gi");
		for (i = rowNum + 1; i <= sectionRowsCount + 1; i++) {
			var node = document.getElementById(RSName + '_row' + this.separator + i + this.separator);
			var replaceStr = this.separator+(i-1)+this.separator;
			if (typeof node.id != 'undefined' && node.id) {
				node.id = String(node.id.replace(re, replaceStr))
			}
			if (typeof node.name != 'undefined' && node.name) {
				node.name = String(node.name.replace(re, replaceStr))
			}
			this.decrementSectionIds(re, node, i);
		}
		
		if (typeof this.removeRowCallbackFunc == 'function') {
			this.removeRowCallbackFunc.apply(this, [RSName, rowNum]);
		}
		
		this.maintainMinNumOfRows(RSName, sectionRowsCount);
		return false;
	}

	/**
	 * Maintain the minimum number of rows in the section
	 */
	this.maintainMinNumOfRows = function(RSName, sectionRowsCount) {
		var minNumOfRowsElement = document.getElementById(RSName+"__minRows");
		if (typeof minNumOfRowsElement != "undefined") {
			minNumOfRows = minNumOfRowsElement.value;
			if (minNumOfRows > sectionRowsCount) {
				for (i = sectionRowsCount; i < minNumOfRows; i++) {
					this.addRSRow(RSName);
				}
			}
		}
	}

	this.addRSRow = function(RSName) {
		var sectionRowsCount = parseInt(document.getElementById(RSName+'__rowCount').value);
		
		sectionRowsCount++;
		document.getElementById(RSName+'__rowCount').value = sectionRowsCount;

		var addNew = document.getElementById(RSName + '_addnewlink');

		return this.doInsertRow(RSName, sectionRowsCount, sectionRowsCount, addNew);
	}
	
	this.insertRSRow = function(RSName, element) {
		var sectionRowsCount = parseInt(document.getElementById(RSName+'__rowCount').value);
		
		// Find out the number of the row
		var rowNum = this.getRowNum(RSName+'_insertlink', element);
		
		// Renumber sections after the current (including current)
		var re = new RegExp(this.separator + "([0-9]+)" + this.separator, "gi");
		for (i = sectionRowsCount; i >= rowNum; i--) {
			var node = document.getElementById(RSName + '_row' + this.separator + i + this.separator);
			var replaceStr = this.separator+(i+1)+this.separator;
			if (typeof node.id != 'undefined' && node.id)
				node.id = String(node.id.replace(re, replaceStr))
			if (typeof node.name != 'undefined' && node.name)
				node.name = String(node.name.replace(re, replaceStr))

			this.incrementSectionIds(re, node, i);
		}
		
		sectionRowsCount++;
		document.getElementById(RSName+'__rowCount').value = sectionRowsCount;
		
		var currentRow = document.getElementById(RSName + '_row' + this.separator + (rowNum + 1) + this.separator);
		
		return this.doInsertRow(RSName, rowNum, sectionRowsCount, currentRow);
	}
	
	/**
	 * Internal helper that inserts row
	 * 
	 * @param RSName repeating section name
	 * @param rowNum the number that the new row will have
	 * @param sectionRowsCount the new number of rows
	 * @param beforeRow the existing row before which the new row will be inserted 
	 */
	this.doInsertRow = function(RSName, rowNum, sectionRowsCount, beforeRow)
	{
		var holderTable = document.getElementById(RSName + '_table');
		var originalRow = document.getElementById(RSName + '_row' + this.separator + this.separator);
		
		var rowClone = originalRow.cloneNode(true);
		rowClone.id = RSName + '_row' + this.separator + rowNum + this.separator;
		this.addSectionIds(rowClone, RSName, rowNum);
		rowClone.style.display = '';
		
		if (!document.createElement || !document.childNodes ) {
			window.alert('Your browser is not DOM compliant.');
		} else {
			holderTable.insertBefore(rowClone, beforeRow);
		}
		
		if (typeof this.addRowCallbackFunc == 'function') {
			this.addRowCallbackFunc.apply(this, [RSName, sectionRowsCount, rowNum]);
		}
		return false;
	}
	
	/**
	 * Extracts the rownum from a DOM element of the repeating section
	 */
	this.getRowNumByElement = function(element)
	{
		elementId = $(element).attr('id');
		if (typeof elementId == 'undefined') return false;
		var re = new RegExp(this.separator + "([0-9]+)" + this.separator);
		return parseInt(String(elementId).match(re)[1]);
	}
	
	// Extracts the rownum from a DOM element of the repeating section
	this.getRowNum = function(RSName, element)
	{
		// Find out the number of the row
		var sectionRowId = $(element).parents('[id^='+RSName+this.separator+']').attr('id');
		if (typeof sectionRowId == 'undefined')
			return false;
		return parseInt(sectionRowId.substring(String(RSName+this.separator).length));
	}
	
/*	
	this.registerMultiLanguageControls(RSName, mlControls) {
		var sectionRowsCount = parseInt(document.getElementById(RSName+'__rowCount').value);
		
		for (i = 0; i < mlControls.length; i++) {
			SC.ML.registerMultilanguageControl(mlControls[i].);
		}
	}
*/	
}

SC.RS = new constructSCRS();
