/**
 * Tree control
 *
 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
 */
function Tree(treeName, treeId)
{
	this.treeId = treeId;
	this.treeName = treeName;
	this.treeRows = {};
	this.treeSubnodes = [];
	this.treeRowCollapsed = [];
	
	this.buildTree = function() {
		this.treeRows = $("." + this.treeName);
		
		for (i = 0; i < this.treeRows.length; i++) {
			var treeRow = $(this.treeRows[i]);
			
			// Check if this is actually a tree row
			if (treeRow.attr('id').indexOf(this.treeId) != 0)
				continue;
			
			var idAndParent = treeRow.attr('id').substring(treeId.length);
			var dashIndex = idAndParent.indexOf("-");
			if (dashIndex < 1)
				continue;
			
			// Get the node id and the parent id
			var id = idAndParent.substr(0, dashIndex);
			var parentId = idAndParent.substr(dashIndex + 1);
			
			// Save the default state of the nodes
			this.treeRowCollapsed[id] = true;
			
			// If it is root node do not init children array
			if (parentId.length == 0)
			    continue;
			
			// Init children array
			if (this.treeSubnodes[parentId] == undefined)
				this.treeSubnodes[parentId] = [];
			this.treeSubnodes[parentId].push(id);
		}
	};
	
	this.toggleSingleRow = function() {
	}
	
	this.toggle = function(link, id) {
		// Do collapse / expand the nodes
		this.doToggle(id, !this.treeRowCollapsed[id]);
	}
	
	this.doToggle = function(id, isExpand) {
		// If this node has no subnodes there is nothing to toggle
		if (this.treeSubnodes[id] == undefined)
			return;
			
		// Save the current node state
		this.treeRowCollapsed[id] = isExpand;
		
		// Toggle the image that indicates the node state - expanded/collapsed
		if (isExpand) {
		    $("#expand" + id).hide();
		    $("#collapse" + id).show();
		}
		else {
		    $("#expand" + id).show();
		    $("#collapse" + id).hide();
		}
		
		// Walk the sub nodes
		var subnodes = this.treeSubnodes[id];
		for (var i = 0; i < subnodes.length; i++) {
			// Get the current node
			treeRow = $('#' + this.treeId + subnodes[i] + "-" + id);
			// Toggle the current node
			isExpand ? treeRow.hide() : treeRow.show();
			// If needed toggle all the subnodes bellow the current node
			if (isExpand && !this.treeRowCollapsed[subnodes[i]])
				this.doToggle(subnodes[i], isExpand);
			
		}
	}
	
	this.restore = function () {
		for (var id in this.treeRowCollapsed) {
			this.doToggle(id, this.treeRowCollapsed[id]);
		}
	}
	
	this.buildTree();
}
