window.addScCartItem = function (requestParams)
{
	SC.EditOrder.Cart.addItem(requestParams);
};

SC.EditOrder = new function ()
{
	this.Cart = {};
	this.JsonHandler = {};
	
	this.init = function () {
		$('#editOrderForm').submit(function () {return SC.EditOrder.validateFormSubmit(this)});
		
		$('#ShippingInfo :input').change(function () {SC.EditOrder.updateCartField(this);});
		$('#BillingInfo :input').change(function () {SC.EditOrder.updateCartField(this);});
		$('#OrderShipToBillingAddress').change(function () {SC.EditOrder.updateCartField(this);});
		
		SC.EditOrder.initPaymentModuleHandler();
		SC.EditOrder.initShippingModuleHandler();
		
		if ($.trim($('#InitialWarnings span').html()) != '') {
			$('#InitialWarnings').show();
		}
	}
	
	this.validateFormSubmit = function (editOrderForm) {
		if ($(':input.error-field').length > 0) {
			return false;
		}
		
		validationSuccess = false;
		$.ajax({
			async: false,
			url: this.baseLink,
			data: {
				'action': 'dmValidate'
			},
			success: function (jsonData) {
				validationSuccess = jsonData.success;
				SC.EditOrder.JsonHandler.handle(jsonData);
			},
			dataType: 'json'
		});
		return validationSuccess;
	}
	
	this.initShippingModuleHandler = function () {
		$('[name=ShippingModuleAndMethod]').change(function () {
			if (this.checked) {
				SC.EditOrder.updateCartField(this);
			}
		});
		if ($.trim($('#SHIPPINGMESSAGES').html()) == '') {
			$('#SHIPPINGMESSAGES').hide();
		} else {
			$('#SHIPPINGMESSAGES').show();
		}
	}
	
	this.initPaymentModuleHandler = function () {
		$('[name=PaymentModuleID]').change(function () {
			if (this.checked) {
				SC.EditOrder.updateCartField(this);
			}
		});
	}
	
	this.updateCartField = function (modifiedField) {
		if (!this.baseLink) {
			alert('The required base link is not set.');
			return;
		}
		var modifiedFieldTagName = modifiedField.tagName;
		modifiedField = $(modifiedField);// Convert to a jQuery object
		var modifiedFieldName = modifiedField.attr('name');
		var modifiedFieldValue = modifiedField.val();
		if (modifiedFieldTagName.toLowerCase() == 'input' && modifiedField.attr('type').toLowerCase() == 'checkbox') {
			if (modifiedField.attr('checked')) {
				modifiedFieldValue = 1;
			} else {
				modifiedFieldValue = 0;
			}
		}
		if (modifiedField.parent().is('.fieldError')) {
			$('#' + modifiedFieldName + '_fieldError').remove();
			modifiedField.removeClass('error-field');
			modifiedField.parent().replaceWith(modifiedField);
		}
		
		var ajaxParams = {
			'action': 'dmUpdateField',
			'fieldName': modifiedFieldName
		};
		ajaxParams[modifiedFieldName] = modifiedFieldValue;
		
		ajaxParams = this.modifyAjaxParams(modifiedFieldName, ajaxParams);
		
		$.ajax({
			async: false,
			type: 'POST',
			url: this.baseLink,
			data: ajaxParams,
			success: function (jsonData) {
				SC.EditOrder.JsonHandler.handle(jsonData);
			},
			dataType: 'json'
		});
	}
	
	this.modifyAjaxParams = function (modifiedFieldName, ajaxParams) {
		if (modifiedFieldName == 'OrderBillingVatNumber') {
			ajaxParams['OrderBillingCountry'] = $('[name=OrderBillingCountry]').val();
		} else if (modifiedFieldName == 'OrderShippingVatNumber') {
			ajaxParams['OrderShippingCountry'] = $('[name=OrderShippingCountry]').val();
		} else if (modifiedFieldName == 'OrderBillingCity') {
			ajaxParams['OrderBillingCityID'] = $('[name=OrderBillingCityID]').val();
			ajaxParams['OrderBillingState'] = $('[name=OrderBillingState]').val();
		} else if (modifiedFieldName == 'OrderShippingCity') {
			ajaxParams['OrderShippingCityID'] = $('[name=OrderShippingCityID]').val();
			ajaxParams['OrderShippingState'] = $('[name=OrderShippingState]').val();
		}
		return ajaxParams;
	}
	
	this.filterBillingStates = function() 
	{
		var selectControl = $('#OrderBillingState')[0];
		var originalValue = $('#OrderBillingState')[0].value;
		var filterByValue = $('#OrderBillingCountry')[0].value;
		
		SC.Util.filterStates(selectControl, originalValue, filterByValue, states);
	}
	
	this.filterShippingStates = function() 
	{
		var selectControl = $('#OrderShippingState')[0];
		var originalValue = $('#OrderShippingState')[0].value;
		var filterByValue = $('#OrderShippingCountry')[0].value;
		
		SC.Util.filterStates(selectControl, originalValue, filterByValue, states);
	}
};

SC.EditOrder.Cart = new function ()
{
	this.addItem = function (requestParams)
	{
		var link = SC.EditOrder.baseLink + '&action=dmAddCartItem&' + requestParams;
		$.getJSON(link, function (jsonData) {SC.EditOrder.JsonHandler.handle(jsonData);});
	}
	
	this.removeItem = function (cartItemCode)
	{
		var link = SC.EditOrder.baseLink + '&action=dmRemoveCartItem&CartItemCode=' + cartItemCode;
		$.getJSON(link, function (jsonData) {SC.EditOrder.JsonHandler.handle(jsonData);});
	}
}

SC.EditOrder.JsonHandler = new function ()
{
	this.handle = function (jsonData)
	{
		if (jsonData.fields) {
			this.handleFields(jsonData.fields);
		}
		if (jsonData.warnings) {
			this.handleWarnings(jsonData.warnings);
		}
		if (jsonData.alerts) {
			this.handleAlerts(jsonData.alerts);
		}
	}
	
	this.handleFields = function (jsonFields)
	{
		var modifyField, tagName;// Local vars
		for (var fieldName in jsonFields) {
			modifyField = $('#' + fieldName);
			if (modifyField.length == 0) {
				modifyField = $('[name="' + fieldName + '"]');
			}
			if (modifyField.length == 1) {
				tagName = modifyField.get(0).tagName.toLowerCase();
				if ('input' == tagName || 'select' == tagName || 'textarea' == tagName) {
					modifyField.val(jsonFields[fieldName]);
				} else {
					modifyField.html(jsonFields[fieldName]);
				}
				
				if (fieldName == 'ShippingModuleAndMethod') {
					SC.EditOrder.initShippingModuleHandler();
				} else if (fieldName == 'PaymentModuleID') {
					SC.EditOrder.initPaymentModuleHandler();
				}
			}
		}
	}
	
	this.handleWarnings = function (jsonWarnings)
	{
		var tooltipPrefix, tooltipSuffix;// Make them local
		for (var fieldName in jsonWarnings) {
			field = $('#' + fieldName);
			if (field.length == 0) {
				field = $('[name="' + fieldName + '"]');
			}
			field.addClass('error-field');
			field.wrap('<div class="fieldError" />');
			tooltipPrefix = '<div id="' + fieldName + '_fieldError" class="fieldError-tooltip" title="' + (typeof(ML_CLICK_TO_HIDE) != 'undefined' ? ML_CLICK_TO_HIDE : 'Click to hide') + '"><div>';
			tooltipSuffix = '</div></div>';
			field.after(tooltipPrefix + jsonWarnings[fieldName] + tooltipSuffix);
		}
	}
	
	this.handleAlerts = function (jsonAlerts)
	{
		if (jsonAlerts.length == 0) {
			return;
		}
		alert(jsonAlerts.join("\r\n"));
	}
}

$(function() {
	SC.EditOrder.init();
	$('#OrderShipToBillingAddress').change(function () {
		if (this.checked) {
			$('#ShippingInfo').hide();
		} else {
			$('#ShippingInfo').show();
		}
	});
	if ($('#OrderShipToBillingAddress:checked').length > 0) {
		$('#ShippingInfo').hide();
	}
	
	SC.EditOrder.filterBillingStates();
	SC.EditOrder.filterShippingStates();
	
	$("#OrderBillingCountry").change(function(){ SC.EditOrder.filterBillingStates()});
	$("#OrderShippingCountry").change(function(){ SC.EditOrder.filterShippingStates()});
});