<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('include/miphpf/Init.php');
	require_once('include/sc/Init.php');
	
	scSession::singleton()->openSession();
	
	class scUserPageViewOrder extends miDefaultView {
		
		public function __construct()
		{
			parent::__construct(null);
		}
		
		public function getControllerCommands()
		{
			$orderId = (int)miGetParamDefault('OrderID', 0);
			
			if (!scCustomerAuth::singleton()->isLoggedIn()) {
				$loginUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_LOGIN);
				$loginUrl = scUrlUtil::addQueryParams($loginUrl, 'Referer=' . urlencode('view_order.php?OrderID=' . $orderId));
				return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $loginUrl));
			}
			
			$customer = scCustomerAuth::singleton()->getCustomer();
			
			try {
				if ($orderId < 1)
					return $this->displayErrorPage(_t('Access denied.'));
			
				$order = new scOrder($orderId);
				
				if ($order->get('CustomerID') != $customer->get('CustomerID'))
					return $this->displayErrorPage(_t('Access denied.'));
				
				$options = array(scOrdersUI::OPTION_CHECK_FOR_EGOODS);
				$orderDetailsHtml = scCustomerOrdersUI::singleton()->getOrderDetailsHtml($order, $options);
			} catch (miException $exception) {
				return $this->displayErrorPage(_t('Access denied.'));
			}
			
			// TODO: V1.1 Display dynamic page Top and Bottom areas
			$t = new scTemplateParser();
			$t->readTemplate('customer/myaccount/view_order.tmpl');
			$t->assign('%%ORDER_DETAILS%%', $orderDetailsHtml);
			
			// Assign Addresses Count and Orders Count
			$t->assignArray(array(
				'%%CUSTOMER_ADDRESSES_COUNT%%' => scCustomerAddress::getAddressesCountByCustomerId($_SESSION['CustomerID']),
				'%%CUSTOMER_ORDERS_COUNT%%' => scCustomerOrdersUI::getOrdersCountByCustomerId($_SESSION['CustomerID'])
			));
			
			// get payment module implementation from order's PaymentModuleID 
			$paymentModuleDomainObj = new scPaymentModule($order->get('PaymentModuleID'));
			$moduleId = $paymentModuleDomainObj->get('ModuleID');
			$paymentModule = scModulesFactory::getImplementation($moduleId, 'IPaymentModule');
			
			// assign notes from the payment module
			$t->assign('%%PAYMENTDETAILS%%', $paymentModule->getPaymentContent(IPaymentModule::CONTENT_TYPE_VIEW_ORDER, array('OrderID' => $orderId)));
			
			$html = $t->templateParse();
			return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
		}
		
		protected function displayErrorPage($msg)
		{
			$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($msg);
			return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
		}
	}
	
	$userPageViewOrder = new scUserPageViewOrder();
	$page = new scUserDisplayController(array($userPageViewOrder));
	$page->setSeo(new scDynamicPageSeo('view_order.php', _t(sprintf('Order %d Details', miGetParamDefault('OrderID', 0)))));
	$page->showPage();
?>